! -*- coding: utf-8 -*-
!
! Dual licensed under the MIT or GPL Version 3 license.
! See 'LICENSE.MIT' for the definition of the MIT license,
! and 'COPYING' for the definition of the GPL license.
!
!> 基底関数クラス
module basis_class
  implicit none
  private

  public::init_basis
  public::new_basis
  public::free_basis
 !public::read_basis
  public::print_basis
  public::print_brief_basis
  public::normalize_basis

  public::save_basis_f
  public::load_basis_f
  public::bcast_basis
  public::merge_basis

  public::basis_type
  public::get_shell_type
  public::get_shell_comp

  type::basis_type
    integer::num     !< number of basis sets
    integer::mxshell !< maximum number of shells for each basis set
    integer::mxprim  !< maximum numner of primitives for each shell
    integer::natom   !< number of atoms
    integer::nao     !< number of AOs
    integer::nshell  !< number of shells
!!$    integer,allocatable::atom2basis(:)
    integer,allocatable::ntshell(:) !< number of shells on each basis set
    integer,allocatable::ntao(:)    !< number of AOs on each basis set
    integer,allocatable::ntshtype(:,:) !< shell type codes
    integer,allocatable::ntprim(:,:)   !< number of ptimitives for each shell
    real(8),allocatable::z(:,:,:)   !< exponents of primitives
    real(8),allocatable::c(:,:,:)   !< contraction coefficients of primitives
  end type

  integer,parameter::SHELL_TYPE_S  = 1 !< shell type code for s shell
  integer,parameter::SHELL_TYPE_P  = 2
  integer,parameter::SHELL_TYPE_D  = 3
  integer,parameter::SHELL_TYPE_F  = 4
  integer,parameter::SHELL_TYPE_G  = 5
  integer,parameter::SHELL_TYPE_SP = 6
  integer,parameter::N_SHELL_TYPES = 6
  character(len=*),parameter::shell_types(N_SHELL_TYPES) = (/&
      'S ','P ','D ','F ','G ','SP' /)

  character(len=*),parameter::unit = ' '

contains
  !-----------------------------------------------------------------------------
  function get_shell_type(ishell_type) result(cshell_type)
    character(len=1)::cshell_type
    integer::ishell_type

    if (0 <= ishell_type .and. ishell_type <= SHELL_TYPE_G) then
       cshell_type = shell_types(ishell_type)
    else
       cshell_type = '  '
    end if

  end function get_shell_type

  !-----------------------------------------------------------------------------
  function get_shell_comp(ishell_type, icomp) result(comp)
    character(len=4)::comp
    integer::ishell_type, icomp

    character(len=1),parameter::pcomp(3) = (/&
         'X','Y','Z'/)
    character(len=2),parameter::dcomp(6) = (/&
         'XX','YY','ZZ','XY','XZ','YZ'/)
    character(len=3),parameter::fcomp(10) = (/&
         'XXX','YYY','ZZZ','XXY','XXZ','YYX','YYZ','ZZX','ZZY','XYZ'/)
    character(len=4),parameter::gcomp(15) = (/&
         'XXXX','YYYY','ZZZZ','XXXY','XXXZ','YYYX','YYYZ','ZZZX','ZZZY',&
         'XXYY','XXZZ','YYZZ','XXYZ','YYXZ','ZZXY'/)

    comp = ' '
    if (0 <= ishell_type .and. ishell_type <= SHELL_TYPE_G) then
       select case(ishell_type)
       !case(SHELL_TYPE_S)
       !   comp = ' '
       case(SHELL_TYPE_P)
          if (0 <= icomp .and. icomp <= 3) comp = pcomp(icomp)
       case(SHELL_TYPE_D)
          if (0 <= icomp .and. icomp <= 6) comp = dcomp(icomp)
       case(SHELL_TYPE_F)
          if (0 <= icomp .and. icomp <= 10) comp = dcomp(icomp)
       case(SHELL_TYPE_G)
          if (0 <= icomp .and. icomp <= 15) comp = dcomp(icomp)
       end select
    end if

  end function get_shell_comp

  !-----------------------------------------------------------------------------
  !> BMS形式ファイルへの保存
  subroutine save_basis_f(self, url, id, ldebug, istat)
    use pl_bms_class
    type(basis_type),intent(in)::self
    character(len=*),intent(in)::url
    integer,intent(in)::id
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    call pl_bms_write(self%num, url, 'nbasis', id, unit, istat)
    if (istat /= 0) return
    call pl_bms_write(self%mxshell, url, 'mxshell', id, unit, istat)
    if (istat /= 0) return
    call pl_bms_write(self%mxprim, url, 'mxprim', id, unit, istat)
    if (istat /= 0) return
    call pl_bms_write(self%nao, url, 'nao', id, unit, istat)
    if (istat /= 0) return
    call pl_bms_write(self%nshell, url, 'nshell', id, unit, istat)
    if (istat /= 0) return
    call pl_bms_write(self%ntshell, self%num, url, 'ntshell', id, unit, istat)
    if (istat /= 0) return
    call pl_bms_write(self%ntao, self%num, url, 'ntao', id, unit, istat)
    if (istat /= 0) return
    call pl_bms_write(self%ntshtype, self%mxshell, self%mxshell, self%num,&
         url, 'ntshtype', id, unit, istat)
    if (istat /= 0) return
    call pl_bms_write(self%ntprim, self%mxshell, self%mxshell, self%num,&
         url, 'ntprim', id, unit, istat)
    if (istat /= 0) return
    call pl_bms_write(self%z,&
         self%mxprim, self%mxshell,&
         self%mxprim, self%mxshell, self%num, url, 'z', id, unit, istat)
    if (istat /= 0) return
    call pl_bms_write(self%c,&
         self%mxprim, self%mxshell,&
         self%mxprim, self%mxshell, self%num, url, 'c', id, unit, istat)
    if (istat /= 0) return

  end subroutine save_basis_f

  !-----------------------------------------------------------------------------
  !> BMS形式ファイルからの読み込み
  subroutine load_basis_f(self, url, id, natom, ldebug, istat)
    use pl_bms_class
    type(basis_type),intent(inout)::self
    character(len=*),intent(in)::url
    integer,intent(in)::id
    integer,intent(in)::natom
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    integer::nbasis, mxshell, mxprim

    call pl_bms_read(nbasis, url, 'nbasis', id, istat)
    if (istat /= 0) return
    call pl_bms_read(mxshell, url, 'mxshell', id, istat)
    if (istat /= 0) return
    call pl_bms_read(mxprim, url, 'mxprim', id, istat)
    if (istat /= 0) return

    call new_basis(self, natom, nbasis, mxshell, mxprim, istat)
    if (istat /= 0) return

    call pl_bms_read(self%nao, url, 'nao', id, istat)
    if (istat /= 0) return
    call pl_bms_read(self%nshell, url, 'nshell', id, istat)
    if (istat /= 0) return
    call pl_bms_read(self%ntshell, self%num, url, 'ntshell', id, istat)
    if (istat /= 0) return
    call pl_bms_read(self%ntao, self%num, url, 'ntao', id, istat)
    if (istat /= 0) return
    call pl_bms_read(self%ntshtype, self%mxshell, self%mxshell, self%num,&
         url, 'ntshtype', id, istat)
    if (istat /= 0) return
    call pl_bms_read(self%ntprim, self%mxshell, self%mxshell, self%num,&
         url, 'ntprim', id, istat)
    if (istat /= 0) return
    call pl_bms_read(self%z,&
         self%mxprim, self%mxshell,&
         self%mxprim, self%mxshell, self%num, url, 'z', id, istat)
    if (istat /= 0) return
    call pl_bms_read(self%c,&
         self%mxprim, self%mxshell,&
         self%mxprim, self%mxshell, self%num, url, 'c', id, istat)
    if (istat /= 0) return

  end subroutine load_basis_f

  !-----------------------------------------------------------------------------
  !> データBcast
  subroutine bcast_basis(self, pl_mpi, ldebug, istat)
    use pl_mpi_class
    type(basis_type),intent(inout)::self
    type(pl_mpi_type),intent(in)::pl_mpi
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    integer::nbasis, mxshell, mxprim, natom, nao, nshell
    integer::icomm, myrank

    istat = 0

    icomm  = pl_mpi%comm
    myrank = pl_mpi%myrank

    if (myrank == 0) then
      nbasis  = self%num
      mxshell = self%mxshell
      mxprim  = self%mxprim
      natom   = self%natom
    end if
    call mpi_bcast(natom, 1, MPI_INTEGER, 0, icomm, istat)
    if (istat /= 0) return
    call mpi_bcast(nbasis, 1, MPI_INTEGER, 0, icomm, istat)
    if (istat /= 0) return
    call mpi_bcast(mxshell, 1, MPI_INTEGER, 0, icomm, istat)
    if (istat /= 0) return
    call mpi_bcast(mxprim, 1, MPI_INTEGER, 0, icomm, istat)
    if (istat /= 0) return
    if (myrank /= 0) then
      call new_basis(self, natom, nbasis, mxshell, mxprim, istat)
      if (istat /= 0) return
    end if
    call mpi_bcast(self%nao, 1, MPI_INTEGER, 0, icomm, istat)
    if (istat /= 0) return
    call mpi_bcast(self%nshell, 1, MPI_INTEGER, 0, icomm, istat)
    if (istat /= 0) return

!!$    call mpi_bcast(self%atom2basis, natom,  MPI_INTEGER, 0, icomm, istat)
!!$    if (istat /= 0) return
    call mpi_bcast(self%ntao, nbasis, MPI_INTEGER, 0, icomm, istat)
    if (istat /= 0) return
    call mpi_bcast(self%ntshell, nbasis, MPI_INTEGER, 0, icomm, istat)
    if (istat /= 0) return
    call mpi_bcast(self%ntshtype, mxshell*nbasis, MPI_INTEGER, 0, icomm, istat)
    if (istat /= 0) return
    call mpi_bcast(self%ntprim, mxshell*nbasis, MPI_INTEGER, 0, icomm, istat)
    if (istat /= 0) return
    call mpi_bcast(&
        self%z, mxprim*mxshell*nbasis, MPI_DOUBLE_PRECISION, 0, icomm, istat)
    if (istat /= 0) return
    call mpi_bcast(&
        self%c, mxprim*mxshell*nbasis, MPI_DOUBLE_PRECISION, 0, icomm, istat)
    if (istat /= 0) return

  end subroutine bcast_basis

  !-----------------------------------------------------------------------------
  !> 初期化
  subroutine init_basis(self, ldebug, istat)
    type(basis_type),intent(inout)::self
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    istat = 0

!!$    call free_basis(self, ldebug, istat)
    self%num = 0

  end subroutine init_basis

  !-----------------------------------------------------------------------------
  !> 配列確保
  subroutine new_basis(self, natom, nbasis, mxshell, mxprim, istat)
    type(basis_type),intent(inout)::self
    integer,intent(in)::natom, nbasis, mxshell, mxprim
    integer,intent(inout)::istat
    istat = 0

    self%num     = nbasis
    self%mxshell = mxshell
    self%mxprim  = mxprim
    self%natom   = natom
    allocate(&
!!$         self%atom2basis(natom),&
         self%ntao(nbasis), self%ntshell(nbasis),&
         self%ntshtype(mxshell,nbasis), self%ntprim(mxshell,nbasis),&
         self%z(mxprim,mxshell,nbasis), self%c(mxprim,mxshell,nbasis),&
         stat=istat)
    if (istat /= 0) return
    self%ntao(:) = 0
    self%ntshell(:) = 0
    self%ntshtype(:,:) = 0
    self%ntprim(:,:) = 0
    self%z(:,:,:) = 0.0d0
    self%c(:,:,:) = 0.0d0

  end subroutine new_basis

  !-----------------------------------------------------------------------------
  !> 配列開放
  subroutine free_basis(self, ldebug, istat)
    type(basis_type),intent(inout)::self
    integer,intent(in)::ldebug
    integer,intent(inout)::istat
    istat = 0

    self%num     = 0
    self%mxshell = 0
    self%mxprim  = 0
    self%natom   = 0
    deallocate(&
!!$        self%atom2basis,&
        self%ntao, self%ntshell, self%ntshtype, self%ntprim, self%z, self%c,&
        stat=istat)
    if (istat /= 0) return

  end subroutine free_basis

  !-----------------------------------------------------------------------------
  !> データの簡易表示
  subroutine print_brief_basis(self)
    type(basis_type),intent(in)::self
    integer::ibasis, isht, nshl, ishl, ncnt, icnt

    write(6,'(1x)')
    write(6,'(''Basis sets:'')')
    write(6,'(1x,''Number of basis sets         = '',i6)') self%num
    write(6,'(1x,''Maximum number of shells     = '',i6)') self%mxshell
    write(6,'(1x,''Maximum number of primitives = '',i6)') self%mxprim
    write(6,'(1x,''Number of AOs                = '',i6)') self%nao
    write(6,'(1x,''Number of shells             = '',i6)') self%nshell

  end subroutine print_brief_basis
  
  !-----------------------------------------------------------------------------
  !> データの表示
  subroutine print_basis(self)
    type(basis_type),intent(in)::self
    integer::ibasis, isht, nshl, ishl, ncnt, icnt

    call print_brief_basis(self)

    if (self%num > 0) then
      write(6,'(1x)')
      write(6,'(6x,''No.'')')
      do ibasis = 1, self%num
        write(6,'(i8)') ibasis
        do ishl = 1, self%ntshell(ibasis)
          isht = self%ntshtype(ishl,ibasis)
          ncnt = self%ntprim(ishl,ibasis)
          write(6,'(9x,a,i7)') shell_types(isht), ncnt
          do icnt = 1, ncnt
            write(6,'(8x,f20.10,f18.10)')&
                self%z(icnt,ishl,ibasis),&
                self%c(icnt,ishl,ibasis)
          end do
        end do
      end do
    end if

  end subroutine print_basis
  
  !-----------------------------------------------------------------------------
  !> データの表示
  subroutine normalize_basis(self, istat)
    type(basis_type),intent(inout)::self
    integer,intent(out)::istat

    istat = 0

    call p_basis_inpnr(&
         self%c, self%z, self%ntprim, self%ntshtype,&
         self%ntshell, self%mxprim, self%mxshell, self%num)
    call p_basis_inrmo(&
         self%c, self%z, self%ntprim, self%ntshtype,&
         self%ntshell, self%mxprim, self%mxshell, self%num, istat)

  end subroutine normalize_basis

  !=============================================================================
  ! private
  !-----------------------------------------------------------------------------
  !
  !  Multiply contraction coefficient by the normalizing factor
  !
  !    NOR.F(Z,LMN) = (2Z/PAI)**(3/4) * (4Z)**(LMN)
  !                               / [(2(LMN)-1)!!]**(1/2)
  !    LMN = L + M + N
  !
  !       nasp  : i : total count of atomic species
  !       nastbl: i : shell count in atomic species
  !       ntsht : i : shell type table in atomic species
  !       ntprm : i : primitive count in a shell
  !       z     : i : orbital exp. at each primitive
  !       c     : o : contraction coeffients at each primitive
  !-----------------------------------------------------------------------------
  subroutine p_basis_inpnr(c, z, ntprm, ntsht, ntshl, mxprm, mxsa, nasp)
    use physical_constants_class
    integer,intent(in)::mxprm,mxsa,nasp
    integer,intent(in)::ntprm(mxsa,nasp)
    integer,intent(in)::ntsht(mxsa,nasp)
    integer,intent(in)::ntshl(nasp)
    real(8),intent(inout)::c(mxprm,mxsa,nasp)
    real(8),intent(in)::z(mxprm,mxsa,nasp)

    integer::i,j,lmn,lmn2,k,kw
    real(8)::rlmn,rlmn2
    !data pi / 3.14159265358979323846d+00 /

    do i = 1, nasp
       do j = 1, ntshl(i)
          lmn = ntsht(j,i) - 1
          rlmn = dble(lmn)*0.5d0
          if (lmn  >=  2) then
             lmn2 = 1
             do kw = 1, lmn - 1
                lmn2 = lmn2*(2*kw+1)
             end do
             rlmn2 = 1.0d0 / dsqrt(dble(lmn2))
          else
             rlmn2 = 1.0d0
          end if

          do k = 1, ntprm(j,i)
             c(k,j,i) = 1.0d0&
                  *(z(k,j,i)*2.0d0/pi)**0.75d0&
                  * (4.0d0*z(k,j,i))**rlmn &
                                !* rlmn2 * c(k,j,i)
                  * rlmn2 * c(k,j,i)
          end do
       end do
    end do

  end subroutine p_basis_inpnr

  !-----------------------------------------------------------------------------
  !  NORMALIZE A.O.
  !
  !  OVERLAP INTEGRAL :
  !               PAI  **(3/2)    (2NX-1)!!*(2NY-1)!!*(2NZ-1)!!
  !     <I,J> = (-----)        * ------------------------------- .
  !               ETA                 (2*ETA)**(NX+NY+NZ)
  !
  !  IN THIS ROUTINE THE NUMERATOR MODIFIED TO
  !
  !     (2*(NX+NY+NZ)-1)!!
  !
  !  IS USED. THEREFOR, DXY,DYZ AND DZX ORBITALS MUST BE MULTIPLIED BY
  !  3**(1/2). THIS FACTOR IS TAKEN INTO ACCOUNT IN SUBROUTINE 'IADNRM'
  !  AND OUTPUT PROCESS.
  !
  !     nasp   : I : TOTAL COUNT OF ATOMIC SPECIES
  !     nastbl : I : SHELL COUNT IN ATOMIC SPECIES
  !     nthst  : I : SHELL TYPE TABLE IN ATOMIC SPECIES
  !     ntprm  : I : PRIMITIVE COUNT IN A SHELL
  !     z      : I : ORBITAL EXPONENTS AT EACH PRIMITIVE
  !     c      : O : CONTRACTION COEFFICIENTS
  !
  !-----------------------------------------------------------------------------
  subroutine p_basis_inrmo(c, z, ntprm, ntsht, ntshl, mxprm, mxsa, nasp, ierr)
    use physical_constants_class
    integer,intent(in)::nasp,mxsa,mxprm
    integer,intent(in)::ntshl(nasp),ntsht(mxsa,nasp),ntprm(mxsa,nasp)
    real(8),intent(in)::z(mxprm,mxsa,nasp)
    real(8),intent(inout)::c(mxprm,mxsa,nasp)
    integer,intent(out)::ierr

    integer::i,j,ic,k,l,m,nthstp
    real(8)::ovlap,eta,etai,etai3,ovl,ovlap2,pi3

    ierr = 0

    pi3 = pi*dsqrt(pi)
    do i = 1, nasp
       do j = 1, ntshl(i)
          nthstp = ntsht(j,i)
          if (nthstp > 5) then
             print *,' Error : angular moment is over '
             ierr = 1
             return
          end if

          do ic = 1, nthstp
             ovlap = 0.0d0
             do k = 1, ntprm(j,i)
                do l = 1, ntprm(j,i)
                   eta   = z(k,j,i) + z(l,j,i)
                   etai  = 1.0d0/eta
                   etai3 = etai*dsqrt(etai)
                   ovl   = pi3*etai3

                   select case (nthstp)
                   case (2)
                      ovl  =  0.5d0* etai    *ovl
                   case (3)
                      ovl  =  3.0d0*(etai**2)*ovl/ 4.0d0
                   case (4)
                      ovl  = 15.0d0*(etai**3)*ovl/ 8.0d0
                   case (5)
                      ovl  =105.0d0*(etai**4)*ovl/16.0d0
                   end select

                   ovl   = c(k,j,i)*c(l,j,i)*ovl
                   ovlap = ovlap + ovl
                end do
             end do

             ovlap  = dabs(ovlap)
             ovlap2 = 1.0d0/dsqrt(ovlap)
             do m = 1, ntprm(j,i)
                c(m,j,i) = c(m,j,i)*ovlap2
             end do
          end do
       end do
    end do

  end subroutine p_basis_inrmo

  !-----------------------------------------------------------------------------
  subroutine merge_basis(self, basis1, basis2, ldebug, istat)
    type(basis_type),intent(inout)::self
    type(basis_type),intent(in)::basis1
    type(basis_type),intent(in)::basis2
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    integer::natom, nbasis, mxshell, mxprim

    natom   = basis1%natom + basis2%natom
    nbasis  = basis1%num + basis2%num
    mxshell = max(basis1%mxshell, basis2%mxshell)
    mxprim  = max(basis1%mxprim, basis2%mxprim)

    call new_basis(self, natom, nbasis, mxshell, mxprim, istat)
    if (istat /= 0) return

    self%nao    = basis1%nao + basis2%nao
    self%nshell = basis1%nshell + basis2%nshell

    self%ntao(1:basis1%num)           = basis1%ntao(1:basis1%num)
    self%ntao(basis1%num+1:nbasis)    = basis2%ntao(1:basis2%num)
    self%ntshell(1:basis1%num)        = basis1%ntshell(1:basis1%num)
    self%ntshell(basis1%num+1:nbasis) = basis2%ntshell(1:basis2%num)
    self%ntshtype(1:basis1%mxshell,1:basis1%num)&
         = basis1%ntshtype(1:basis1%mxshell,1:basis1%num)
    self%ntshtype(1:basis2%mxshell,basis1%num+1:nbasis)&
         = basis2%ntshtype(1:basis2%mxshell,1:basis2%num)
    self%ntprim(1:basis1%mxshell,1:basis1%num)&
         = basis1%ntprim(1:basis1%mxshell,1:basis1%num)
    self%ntprim(1:basis2%mxshell,basis1%num+1:nbasis)&
         = basis2%ntprim(1:basis2%mxshell,1:basis2%num)
    self%z(1:basis1%mxprim,1:basis1%mxshell,1:basis1%num)&
         = basis1%z(1:basis1%mxprim,1:basis1%mxshell,1:basis1%num)
    self%z(1:basis2%mxprim,1:basis2%mxshell,basis1%num+1:nbasis)&
         = basis2%z(1:basis2%mxprim,1:basis2%mxshell,1:basis2%num)
    self%c(1:basis1%mxprim,1:basis1%mxshell,1:basis1%num)&
         = basis1%c(1:basis1%mxprim,1:basis1%mxshell,1:basis1%num)
    self%c(1:basis2%mxprim,1:basis2%mxshell,basis1%num+1:nbasis)&
         = basis2%c(1:basis2%mxprim,1:basis2%mxshell,1:basis2%num)

  end subroutine merge_basis

  !-----------------------------------------------------------------------------
end module basis_class
