! -*- coding: utf-8 -*-
!
! Dual licensed under the MIT or GPL Version 3 license.
! See 'LICENSE.MIT' for the definition of the MIT license,
! and 'COPYING' for the definition of the GPL license.
!
!> Controlオプションクラス
module control_options_class
  implicit none
  private

  public::options_type
  public::read_options
  public::save_options
  public::load_options
  public::bcast_options
  public::free_options
  public::print_options
  public::QM_PMON_NO
  public::QM_PMON_CLOCK
  public::QM_PMON_HWC

  type::options_type
     integer::lprint
     integer::ldebug
     integer::mem
     integer::pmon
     logical::do_pmon
     logical::do_hwc
  end type

  character(len=*),parameter::group_name = 'CONTROL' !< データグループ名称

  integer,parameter::MAX_OPTION_NAME_LEN = 8
  integer,parameter::nopt = 3                               !< オプション数
  character(len=MAX_OPTION_NAME_LEN)::opt_names(nopt) = (/& !< オプションの名称
       'PRINT',&
       'DEBUG',&
       'PMON '/)

  integer,parameter::QM_PMON_NO    = 0
  integer,parameter::QM_PMON_CLOCK = 1
  integer,parameter::QM_PMON_HWC   = 2
  integer,parameter::QM_PMON_NUM = QM_PMON_HWC + 1
  character(len=*),parameter::QM_PMON(QM_PMON_NUM) = (/&
       'NO   ',&
       'CLOCK',&
       'HWC  '/)

  character(len=*),parameter::url = 'options_control' !< BMSファイル名称
  integer,parameter::id = 1
  character(len=*),parameter::no_unit = ' '

contains
  !-----------------------------------------------------------------------------
  !> オプションの表示
  subroutine print_options(self)
    type(options_type),intent(in)::self

    write(6,'(/a,'' Options:'')') trim(group_name)
    write(6,'(2x,a,'' : '',i6)') 'PRINT',self%lprint
    if (self%ldebug /= 0)&
    write(6,'(2x,a,'' : '',i6)') 'DEBUG',self%ldebug
    if (self%pmon /= QM_PMON_NO)&
    write(6,'(2x,a,'' : '',a)')  'PMON ',trim(QM_PMON(self%pmon + 1))
    !write(6,'(2x,a,'' : '',i6)') 'MEM  ',self%mem

  end subroutine print_options

  !-----------------------------------------------------------------------------
  !> オプションの入力ファイルからの読み込み
  subroutine read_options(self, lines, nline, ldebug, istat)
    use input_class
    type(options_type),intent(inout)::self
    integer,intent(in)::nline
    character(len=*),intent(in)::lines(nline)
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    character(len=MAX_OPTION_VALUE_LEN)::opt_values(nopt)!< オプションの値
    integer::opt_count(nopt)                             !< オプションの出現回数
    character(len=MAX_OPTION_NAME_LEN)::opt_name
    character(len=MAX_OPTION_VALUE_LEN)::opt_value
    integer::iopt, ival, ipos, ival_default, ipos_default
    real(8)::dval, dval_default
    character(len=MAX_OPTION_VALUE_LEN)::str, str_default

    integer::nerror, ierror

    call get_options(&
       lines, nline,&
       group_name, opt_names, opt_values, opt_count, nopt, ldebug, istat)
    if (istat /= 0) return

    nerror = 0
    ierror = 0

    opt_name = 'print'
    ival_default = 0
    call get_option_values_int(&
         opt_name, opt_names, opt_values, nopt, ival_default, ival, istat)
    if (istat /= 0) then
       nerror = nerror + 1; ierror = istat
    end if
    self%lprint = ival

    opt_name = 'debug'
    ival_default = 0
    call get_option_values_int(&
         opt_name, opt_names, opt_values, nopt, ival_default, ival, istat)
    if (istat /= 0) then
       nerror = nerror + 1; ierror = istat
    end if
    self%ldebug = ival

    opt_name = 'pmon'
    ipos_default = pl_str_search_name_ignorecase(&
         'no', QM_PMON, QM_PMON_NUM)
    call get_option_values_strlist(&
         opt_name, opt_names, opt_values, nopt,&
         QM_PMON, QM_PMON_NUM, ipos_default, ipos, istat)
    if (istat /= 0) then
       nerror = nerror + 1; ierror = istat
    end if
    self%pmon = ipos - 1
    self%do_pmon = .false.
    if (self%pmon > QM_PMON_NO) self%do_pmon = .true.
    self%do_hwc = .false.
    if (self%pmon == QM_PMON_HWC) self%do_hwc = .true.

    self%mem = 0

    if (nerror /= 0) then
       istat = ierror; return
    end if

  end subroutine read_options

  !-----------------------------------------------------------------------------
  !> オプションのBMS形式ファイルへの保存
  subroutine save_options(self, ldebug, istat)
    use pl_bms_class
    type(options_type),intent(in)::self
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    call pl_bms_write(self%lprint, url, 'lprint', id, no_unit, istat)
    if (istat /= 0) return
    call pl_bms_write(self%ldebug, url, 'ldebug', id, no_unit, istat)
    if (istat /= 0) return
    call pl_bms_write(self%pmon, url, 'pmon', id, no_unit, istat)
    if (istat /= 0) return
    call pl_bms_write(self%mem, url, 'mem', id, no_unit, istat)
    if (istat /= 0) return

  end subroutine save_options

  !-----------------------------------------------------------------------------
  !> オプションのBMS形式ファイルからの読み込み
  subroutine load_options(self, ldebug, istat)
    use pl_bms_class
    type(options_type),intent(inout)::self
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    call pl_bms_read(self%lprint, url, 'lprint', id, istat)
    if (istat /= 0) return
    call pl_bms_read(self%ldebug, url, 'ldebug', id, istat)
    if (istat /= 0) return
    call pl_bms_read(self%pmon, url, 'pmon', id, istat)
    if (istat /= 0) return
    call pl_bms_read(self%mem, url, 'mem', id, istat)
    if (istat /= 0) return

    self%do_pmon = .false.
    if (self%pmon > QM_PMON_NO) self%do_pmon = .true.
    self%do_hwc = .false.
    if (self%pmon == QM_PMON_HWC) self%do_hwc = .true.

  end subroutine load_options

  !-----------------------------------------------------------------------------
  !> オプションのBcast
  subroutine bcast_options(self, pl_mpi, ldebug, istat)
    use pl_mpi_class
    type(options_type),intent(inout)::self
    type(pl_mpi_type),intent(in)::pl_mpi
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    integer::icomm
    icomm = pl_mpi%comm

    call mpi_bcast(self%lprint, 1, MPI_INTEGER, 0, icomm, istat)
    if (istat /= 0) return
    call mpi_bcast(self%ldebug, 1, MPI_INTEGER, 0, icomm, istat)
    if (istat /= 0) return
    call mpi_bcast(self%pmon, 1, MPI_INTEGER, 0, icomm, istat)
    if (istat /= 0) return
    call mpi_bcast(self%mem, 1, MPI_INTEGER, 0, icomm, istat)
    if (istat /= 0) return

    self%do_pmon = .false.
    if (self%pmon > QM_PMON_NO) self%do_pmon = .true.
    self%do_hwc = .false.
    if (self%pmon == QM_PMON_HWC) self%do_hwc = .true.

  end subroutine bcast_options

  !-----------------------------------------------------------------------------
  !> 配列開放
  subroutine free_options(self, ldebug, istat)
    type(options_type),intent(inout)::self
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    istat = 0
  end subroutine free_options

  !-----------------------------------------------------------------------------
end module control_options_class
