! -*- coding: utf-8 -*-
!
! Dual licensed under the MIT or GPL Version 3 license.
! See 'LICENSE.MIT' for the definition of the MIT license,
! and 'COPYING' for the definition of the GPL license.
!
!> データクラス
!!
module data_class
  use atoms_class, only:atoms_type
  use ptcharges_class, only:ptcharges_type
  use gscharges_class, only:gscharges_type
  use fswolf_class, only:fswolf_type
  private

  public::data_type
  public::init_data
  public::read_inp_data
  public::read_data
  public::load_data
  public::load_data_f
  public::bcast_data
  public::free_data
  public::print_data
  public::num_data
  public::get_data

  type data_type
    type(atoms_type)::atoms
    type(ptcharges_type)::ptcharges
    type(gscharges_type)::gscharges
    type(fswolf_type)::fswolf
  end type data_type

contains
  !-----------------------------------------------------------------------------
  !> 初期化
  subroutine init_data(self, ldebug, istat)
    use atoms_class, only:init_atoms
    use ptcharges_class, only:init_ptcharges
    use gscharges_class, only:init_gscharges
    use fswolf_class, only:init_fswolf
    type(data_type),intent(inout)::self
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    call init_atoms(self%atoms, ldebug, istat)
    if (istat /= 0) return
    call init_ptcharges(self%ptcharges, ldebug, istat)
    if (istat /= 0) return
    call init_gscharges(self%gscharges, ldebug, istat)
    if (istat /= 0) return
    call init_fswolf(self%fswolf, ldebug, istat)
    if (istat /= 0) return

  end subroutine init_data

  !-----------------------------------------------------------------------------
  !> 入力ファイルからの読み込み
  subroutine read_inp_data(self, input_file, ldebug, istat)
    use input_class, only:read_file_n, read_file, MAX_INPUT_LINE_LEN
    type(data_type),intent(inout)::self
    character(len=*),intent(in)::input_file
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    integer::nline
    character(len=MAX_INPUT_LINE_LEN),allocatable::lines(:)

    call read_file_n(input_file, nline, istat)
    if (istat /= 0) return
    allocate(lines(nline), stat=istat)
    if (istat /= 0) return
    call read_file(input_file, lines, nline, istat)
    if (istat /= 0) return

    call read_data(self, lines, nline, ldebug, istat)
    if (istat /= 0) return

    deallocate(lines, stat=istat)
    if (istat /= 0) return

  end subroutine read_inp_data

  !-----------------------------------------------------------------------------
  !> 入力ファイルからの読み込み
  subroutine read_data(self, lines, nline, ldebug, istat)
    use atoms_class, only:read_atoms
    use ptcharges_class, only:read_ptcharges
    use gscharges_class, only:read_gscharges
    use fswolf_class, only:read_fswolf
    use qm_options_class, only: options_type, read_options
    type(data_type),intent(inout)::self
    integer,intent(in)::nline
    character(len=*),intent(in)::lines(nline)
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    type(options_type)::qm_options

!!$    qm_options%pdbinput = QM_NO
    call read_options(qm_options, lines, nline, ldebug, istat)
    if (istat /= 0) return

    call read_atoms(self%atoms, qm_options, lines, nline, ldebug, istat)
    if (istat /= 0) return
    call read_ptcharges(self%ptcharges, qm_options, lines, nline, ldebug, istat)
    if (istat /= 0) return
    call read_gscharges(self%gscharges, lines, nline,  ldebug, istat)
    if (istat /= 0) return
    call read_fswolf(self%fswolf, lines, nline,  ldebug, istat)
    if (istat /= 0) return

  end subroutine read_data

  !-----------------------------------------------------------------------------
  !> BMS形式ファイルからの読み込み
  subroutine load_data(self, ldebug, istat)
    use atoms_class, only:load_atoms
    use ptcharges_class, only:load_ptcharges
    use gscharges_class, only:load_gscharges
    use fswolf_class, only:load_fswolf
    type(data_type),intent(inout)::self
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    call load_atoms(self%atoms, ldebug, istat)
    if (istat /= 0) return
    call load_ptcharges(self%ptcharges, ldebug, istat)
    if (istat /= 0) return
    call load_gscharges(self%gscharges, ldebug, istat)
    if (istat /= 0) return
    call load_fswolf(self%fswolf, ldebug, istat)
    if (istat /= 0) return

  end subroutine load_data

  !-----------------------------------------------------------------------------
  !> BMS形式ファイルからの読み込み
  subroutine load_data_f(self, url, id, ldebug, istat)
    use atoms_class, only:load_atoms_f
    use ptcharges_class, only:load_ptcharges_f
    use gscharges_class, only:load_gscharges_f
    use fswolf_class, only:load_fswolf_f
    type(data_type),intent(inout)::self
    character(len=*),intent(in)::url
    integer,intent(in)::id
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    call load_atoms_f(self%atoms, url, id, ldebug, istat)
    if (istat /= 0) return
    call load_ptcharges_f(self%ptcharges, url, id, ldebug, istat)
    if (istat /= 0) return
    call load_gscharges_f(self%gscharges, url, id, ldebug, istat)
    if (istat /= 0) return
    call load_fswolf_f(self%fswolf, url, id, ldebug, istat)
    if (istat /= 0) return

  end subroutine load_data_f

  !-----------------------------------------------------------------------------
  !> データBcast
  subroutine bcast_data(self, pl_mpi, ldebug, istat)
    use pl_mpi_class
    use atoms_class, only:bcast_atoms
    use ptcharges_class, only:bcast_ptcharges
    use gscharges_class, only:bcast_gscharges
    use fswolf_class, only:bcast_fswolf
    type(data_type),intent(inout)::self
    type(pl_mpi_type),intent(in)::pl_mpi
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    call bcast_atoms(self%atoms, pl_mpi, ldebug, istat)
    if (istat /= 0) return
    call bcast_ptcharges(self%ptcharges, pl_mpi, ldebug, istat)
    if (istat /= 0) return
    call bcast_gscharges(self%gscharges, pl_mpi, ldebug, istat)
    if (istat /= 0) return
    call bcast_fswolf(self%fswolf, pl_mpi, ldebug, istat)
    if (istat /= 0) return

  end subroutine bcast_data

  !-----------------------------------------------------------------------------
  !> 配列開放
  subroutine free_data(self, ldebug, istat)
    use atoms_class, only:free_atoms
    use ptcharges_class, only:free_ptcharges
    use gscharges_class, only:free_gscharges
    use fswolf_class, only:free_fswolf
    type(data_type),intent(inout)::self
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    call free_atoms(self%atoms, ldebug, istat)
    if (istat /= 0) return
    call free_ptcharges(self%ptcharges, ldebug, istat)
    if (istat /= 0) return
    call free_gscharges(self%gscharges, ldebug, istat)
    if (istat /= 0) return
    call free_fswolf(self%fswolf, ldebug, istat)
    if (istat /= 0) return

  end subroutine free_data

  !-----------------------------------------------------------------------------
  !> データの表示
  subroutine print_data(self)
    use atoms_class, only:print_atoms
    use ptcharges_class, only:print_brief_ptcharges
    use gscharges_class, only:print_brief_gscharges
    use fswolf_class, only:print_brief_fswolf
    type(data_type),intent(inout)::self

    call print_atoms(self%atoms)
    call print_brief_ptcharges(self%ptcharges)
    call print_brief_gscharges(self%gscharges)
    call print_brief_fswolf(self%fswolf)

  end subroutine print_data

  !-----------------------------------------------------------------------------
  !> 個数の取得
  subroutine num_data(self, natoms, nptcharges, ngscharges, nfswolf)
    use atoms_class, only:num_atoms
    use ptcharges_class, only:num_ptcharges
    use gscharges_class, only:num_gscharges
    use fswolf_class, only:num_fswolf
    type(data_type),intent(in)::self
    integer,intent(out)::natoms
    integer,intent(out)::nptcharges
    integer,intent(out)::ngscharges
    integer,intent(out)::nfswolf

    natoms = num_atoms(self%atoms)
    nptcharges = num_ptcharges(self%ptcharges)
    ngscharges = num_gscharges(self%gscharges)
    nfswolf = num_fswolf(self%fswolf)

  end subroutine num_data

  !-----------------------------------------------------------------------------
  !> データ取得
  subroutine get_data(self,&
       num_atoms, xyz_atoms, chg_atoms,&
       num_ptcharges, xyz_ptcharges, chg_ptcharges,&
       num_gscharges, xyz_gscharges, chg_gscharges, rc2v_gscharges,&
       num_fswolf, xyz_fswolf, chg_fswolf, rc2v_fswolf,&
       istat)
    use atoms_class, only:get_atoms
    use ptcharges_class, only:get_ptcharges
    use gscharges_class, only:get_gscharges
    use fswolf_class, only:get_fswolf
    type(data_type),intent(in)::self
    integer,intent(in)::num_atoms
    real(8),intent(inout)::xyz_atoms(3,num_atoms)
    real(8),intent(inout)::chg_atoms(num_atoms)
    integer,intent(inout)::istat
    integer,intent(in)::num_ptcharges
    real(8),intent(inout)::xyz_ptcharges(3,num_ptcharges)
    real(8),intent(inout)::chg_ptcharges(num_ptcharges)
    integer,intent(in)::num_gscharges
    real(8),intent(inout)::xyz_gscharges(3,num_gscharges)
    real(8),intent(inout)::chg_gscharges(num_gscharges)
    real(8),intent(inout)::rc2v_gscharges(num_gscharges)
    integer,intent(in)::num_fswolf
    real(8),intent(inout)::xyz_fswolf(3,num_fswolf)
    real(8),intent(inout)::chg_fswolf(num_fswolf)
    real(8),intent(inout)::rc2v_fswolf(num_fswolf)

    call get_atoms(self%atoms,&
         num_atoms, xyz_atoms, chg_atoms, istat)
    if (istat /= 0) return
    call get_ptcharges(self%ptcharges,&
         num_ptcharges, xyz_ptcharges, chg_ptcharges, istat)
    if (istat /= 0) return
    call get_gscharges(self%gscharges,&
         num_gscharges, xyz_gscharges, chg_gscharges, rc2v_gscharges, istat)
    if (istat /= 0) return
    call get_fswolf(self%fswolf,&
         num_fswolf, xyz_fswolf, chg_fswolf, rc2v_fswolf,&
         istat)
    if (istat /= 0) return

  end subroutine get_data

  !-----------------------------------------------------------------------------
end module data_class
