! -*- coding: utf-8 -*-
!
! Dual licensed under the MIT or GPL Version 3 license.
! See 'LICENSE.MIT' for the definition of the MIT license,
! and 'COPYING' for the definition of the GPL license.
!
!> エネルギー計算モジュール
!!
!! \f$E^{QM}       = E^{QM}_{QM\_QM} + E^{QM}_{QM\_MM} + E^{QM}_{MM\_MM}\f$
!! \f$E^{QM}_{QM\_QM} = T_e + V_{ee} + V_{Ne} + V_{NN}\f$
!! \f$E^{QM}_{QM\_MM} =                V_{pe} + V_{pN} + V_{ge} + V_{gN}\f$
!! \f$E^{QM}_{MM\_MM} =                         V_{pp} + V_{gg} + V_{pg}\f$
!!
module energy_class
  use energy_rhf_options_class, only: rhf_options_type => options_type
!!$  use energy_uhf_options_class, only: uhf_options_type => options_type
!!$  use energy_casscf_options_class, only: casscf_options_type => options_type
!!$  use energy_cis_options_class, only: cis_options_type => options_type
  use energy_rhf_class, only: rhf_data_type => data_type
!!$  use energy_uhf_class, only: uhf_data_type => data_type
!!$  use energy_casscf_class, only: casscf_data_type => data_type
!!$  use energy_cis_class, only: cis_data_type => data_type

  implicit none
  private

  public::energy_init
  public::energy_set_wftype
  public::energy_read_options
  public::energy_bcast_options
  public::energy_print_options
  public::energy_new_data
  public::energy_guess
  public::energy_orthogonalize
  public::energy_solve
  public::energy_print_total_energy
  public::energy_print_data
  public::energy_print_data_brief
  public::energy_save_data
  public::energy_free
  public::energy_free_options
  public::energy_free_data
  public::energy_type

  type::energy_type
     real(8)::total !< \f$E^{QM}\f$
!!$    real(8)::qm_qm !< \f$E^{QM}_{QM\_QM}\f$
!!$    real(8)::qm_mm !< \f$E^{QM}_{QM\_MM}\f$
!!$    real(8)::mm_mm !< \f$E^{QM}_{MM\_MM}\f$
     real(8)::T_e
     real(8)::V_ee
     real(8)::V_NN
     real(8)::V_Ne
     real(8)::V_pe
     real(8)::V_ge
     real(8)::V_fe
     real(8)::V_pN
     real(8)::V_gN
     real(8)::V_fN
     real(8)::V_pp
     real(8)::V_gg
     real(8)::V_pg
     real(8)::V_fp
     real(8)::V_fg
     real(8)::V_ff
     integer::wftype
     integer::nao
     integer::nele
     type(rhf_options_type)::rhf_options
     type(rhf_data_type)::rhf_data
!!$     type(uhf_options_type)::uhf_options
!!$     type(uhf_data_type)::uhf_data
!!$     type(casscf_options_type)::casscf_options
!!$     type(casscf_data_type)::casscf_data
!!$     type(cis_options_type)::cis_options
!!$     type(cis_data_type)::cis_data
  end type

contains
  !-----------------------------------------------------------------------------
  !> 初期化
  subroutine energy_init(self, ldebug, istat)
    type(energy_type),intent(inout)::self
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    istat = 0

    self%total = 0.0d0
!!$    self%qm_qm = 0.0d0
!!$    self%qm_mm = 0.0d0
!!$    self%mm_mm = 0.0d0
    self%T_e   = 0.0d0
    self%V_ee  = 0.0d0
    self%V_NN  = 0.0d0
    self%V_Ne  = 0.0d0
    self%V_pe  = 0.0d0
    self%V_ge  = 0.0d0
    self%V_fe  = 0.0d0
    self%V_pN  = 0.0d0
    self%V_gN  = 0.0d0
    self%V_fN  = 0.0d0
    self%V_pp  = 0.0d0
    self%V_gg  = 0.0d0
    self%V_pg  = 0.0d0
    self%V_fp  = 0.0d0
    self%V_fg  = 0.0d0
    self%V_ff  = 0.0d0

  end subroutine energy_init

  !-----------------------------------------------------------------------------
  !> 波動関数タイプのセット
  subroutine energy_set_wftype(self, wftype, istat)
    type(energy_type),intent(inout)::self
    integer,intent(in)::wftype
    integer,intent(inout)::istat

    istat = 0

    self%wftype = wftype

  end subroutine energy_set_wftype

  !-----------------------------------------------------------------------------
  !> \e input_file からの読み込み
  subroutine energy_read_options(self, lines, nline, nao, nele, ldebug, istat)
    use qm_options_class
    use energy_rhf_options_class, only: rhf_read_options => read_options
!!$    use energy_uhf_options_class, only: uhf_read_options => read_options
!!$    use energy_casscf_options_class, only: casscf_read_options => read_options
!!$    use energy_cis_options_class, only: cis_read_options => read_options
    type(energy_type),intent(inout)::self
    integer,intent(in)::nline
    character(len=*),intent(in)::lines(nline)
    integer,intent(in)::nao
    integer,intent(in)::nele
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    istat = 0

    select case (self%wftype)
    case (QM_WFTYPE_RHF)
       call rhf_read_options(&
            self%rhf_options, lines, nline, nao, nele, ldebug, istat)
!!$    case (QM_WFTYPE_UHF)
!!$       call uhf_read_options(&
!!$            self%uhf_options, lines, nline, nao, nele, ldebug, istat)
!!$    case (QM_WFTYPE_CASSCF)
!!$       call rhf_read_options(&
!!$            self%rhf_options, lines, nline, nao, nele, ldebug, istat)
!!$       if (istat .ne. 0) return
!!$       call casscf_read_options(&
!!$            self%casscf_options, lines, nline, nao,nele, ldebug, istat)
!!$    case (QM_WFTYPE_RDFT)
!!$       call rhf_read_options(&
!!$            self%rhf_options, lines, nline, nao, nele, ldebug, istat)
!!$    case (QM_WFTYPE_UDFT)
!!$       call uhf_read_options(&
!!$            self%uhf_options, lines, nline, nao, nele, ldebug, istat)
!!$    case (QM_WFTYPE_CIS)
!!$       call rhf_read_options(&
!!$            self%rhf_options, lines, nline, nao, nele, ldebug, istat)
!!$       call cis_read_options(&
!!$            self%cis_options, lines, nline, nele/2, nao-nele/2, ldebug, istat)
    case default
       istat = 1
    end select

    self%nao  = nao
    self%nele = nele

  end subroutine energy_read_options

  !-----------------------------------------------------------------------------
  !> OptionデータBcast
  subroutine energy_bcast_options(self, pl_mpi, ldebug, istat)
    use qm_options_class
    use energy_rhf_options_class, only: rhf_bcast_options => bcast_options
!!$    use energy_uhf_options_class, only: uhf_bcast_options => bcast_options
!!$    use energy_casscf_options_class, only: casscf_bcast_options => bcast_options
!!$    use energy_cis_options_class, only: cis_bcast_options => bcast_options
    use pl_mpi_class
    type(energy_type),intent(inout)::self
    type(pl_mpi_type),intent(in)::pl_mpi
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    integer::icomm, myrank

    istat = 0

    icomm  = pl_mpi%comm
    myrank = pl_mpi%myrank

    call mpi_bcast(self%wftype, 1, MPI_INTEGER, 0, icomm, istat)
    if (istat .ne. 0) return
    call mpi_bcast(self%nao, 1, MPI_INTEGER, 0, icomm, istat)
    if (istat .ne. 0) return
    call mpi_bcast(self%nele, 1, MPI_INTEGER, 0, icomm, istat)
    if (istat .ne. 0) return

    select case (self%wftype)
    case (QM_WFTYPE_RHF)
       call rhf_bcast_options(self%rhf_options, pl_mpi, ldebug, istat)
!!$    case (QM_WFTYPE_UHF)
!!$       call uhf_bcast_options(self%uhf_options, pl_mpi, ldebug, istat)
!!$    case (QM_WFTYPE_CASSCF)
!!$       call casscf_bcast_options(self%casscf_options, pl_mpi, ldebug, istat)
!!$    case (QM_WFTYPE_RDFT)
!!$       call rhf_bcast_options(self%rhf_options, pl_mpi, ldebug, istat)
!!$    case (QM_WFTYPE_UDFT)
!!$       call uhf_bcast_options(self%uhf_options, pl_mpi, ldebug, istat)
!!$    case (QM_WFTYPE_CIS)
!!$       call rhf_bcast_options(self%rhf_options, pl_mpi, ldebug, istat)
!!$       call cis_bcast_options(self%cis_options, pl_mpi, ldebug, istat)
    case default
       istat = 1
    end select

  end subroutine energy_bcast_options

  !-----------------------------------------------------------------------------
  !> Optionデータの表示
  subroutine energy_print_options(self)
    use qm_options_class
    use energy_rhf_options_class, only: rhf_print_options => print_options
!!$    use energy_uhf_options_class, only: uhf_print_options => print_options
!!$    use energy_casscf_options_class, only: casscf_print_options => print_options
!!$    use energy_cis_options_class, only: cis_print_options => print_options
    type(energy_type),intent(in)::self

    select case (self%wftype)
    case (QM_WFTYPE_RHF)
       call rhf_print_options(self%rhf_options)
!!$    case (QM_WFTYPE_UHF)
!!$       call uhf_print_options(self%uhf_options)
!!$    case (QM_WFTYPE_CASSCF)
!!$       call casscf_print_options(self%casscf_options)
!!$    case (QM_WFTYPE_RDFT)
!!$       call rhf_print_options(self%rhf_options)
!!$    case (QM_WFTYPE_UDFT)
!!$       call uhf_print_options(self%uhf_options)
!!$    case (QM_WFTYPE_CIS)
!!$       call rhf_print_options(self%rhf_options)
!!$       call cis_print_options(self%cis_options)
    case default
    end select

  end subroutine energy_print_options

  !-----------------------------------------------------------------------------
  !> 前処理
  subroutine energy_new_data(self, ldebug, istat)
    use qm_options_class
    use energy_rhf_class,    only: rhf_new_data    => new_data
!!$    use energy_uhf_class,    only: uhf_new_data    => new_data
!!$    use energy_casscf_class, only: casscf_new_data => new_data
!!$    use energy_cis_class,    only: cis_new_data    => new_data
    type(energy_type),intent(inout)::self
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    istat = 0

    select case (self%wftype)
    case (QM_WFTYPE_RHF)
       call rhf_new_data(self%rhf_data, self%nao, ldebug, istat)
!!$    case (QM_WFTYPE_UHF)
!!$       call uhf_new_data(self%uhf_data, self%nao, ldebug, istat)
!!$    case (QM_WFTYPE_CASSCF)
!!$       call casscf_new_data(&
!!$            self%casscf_data, self%casscf_options, self%nao, ldebug, istat)
!!$    case (QM_WFTYPE_RDFT)
!!$       call rhf_new_data(self%rhf_data, self%nao, ldebug, istat)
!!$    case (QM_WFTYPE_UDFT)
!!$       call uhf_new_data(self%uhf_data, self%nao, ldebug, istat)
!!$    case (QM_WFTYPE_CIS)
!!$       call rhf_new_data(self%rhf_data, self%nao, ldebug, istat)
!!$       call cis_new_data(&
!!$            self%cis_data, self%cis_options%ni, self%cis_options%ne,&
!!$            self%cis_options%nsolv, ldebug, istat)
    case default
       istat = 1
    end select

  end subroutine energy_new_data

  !-----------------------------------------------------------------------------
  !> 初期値生成
  subroutine energy_guess(self, integral, pl_mpi, ldebug, istat)
    use qm_options_class
    use energy_rhf_class, only: rhf_guess => guess
!!$    use energy_uhf_class, only: uhf_guess => guess
!!$    use energy_casscf_class, only: casscf_guess => guess
!!$    !use energy_cis_class, only: cis_guess => guess
    use integral_class, only: integral_type
    use pl_mpi_class
    use pl_prof_class
    type(energy_type),intent(inout)::self
    type(integral_type),intent(in)::integral
    type(pl_mpi_type),intent(in)::pl_mpi
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    integer::icomm, myrank, istat2

    istat = 0
    icomm  = pl_mpi%comm
    myrank = pl_mpi%myrank

    select case (self%wftype)
!!$    case (QM_WFTYPE_RHF, QM_WFTYPE_RDFT, QM_WFTYPE_CIS)
    case (QM_WFTYPE_RHF)
       call mpi_barrier(icomm, istat)
       call pl_prof_region_begin('s:Guess')
       if (myrank == 0) then
          call rhf_guess(self%rhf_data, integral, ldebug, istat)
       end if
       call mpi_bcast(istat, 1, MPI_INTEGER, 0, icomm, istat2)
       if (istat /= 0) return
       call mpi_barrier(icomm, istat)
       call pl_prof_region_end('s:Guess')

       call mpi_barrier(icomm, istat)
       call pl_prof_region_begin('m:Guess_bcast')
       call mpi_bcast(&
            self%rhf_data%c, self%rhf_data%nao*self%rhf_data%nmo,&
            MPI_DOUBLE_PRECISION, 0, icomm, istat)
       if (istat /= 0) return
       call mpi_barrier(icomm, istat)
       call pl_prof_region_end('m:Guess_bcast')
!!$    case (QM_WFTYPE_UHF, QM_WFTYPE_UDFT)
!!$       if (myrank == 0) then
!!$          call uhf_guess(self%uhf_data, self%uhf_options, integral,ldebug,istat)
!!$       end if
!!$       call mpi_bcast(istat, 1, MPI_INTEGER, 0, icomm, istat2)
!!$       if (istat /= 0) return
!!$
!!$       call mpi_bcast(&
!!$            self%uhf_data%ca, self%uhf_data%nao*self%uhf_data%nmo,&
!!$            MPI_DOUBLE_PRECISION, 0, icomm, istat)
!!$       if (istat /= 0) return
!!$
!!$       call mpi_bcast(&
!!$            self%uhf_data%cb, self%uhf_data%nao*self%uhf_data%nmo,&
!!$            MPI_DOUBLE_PRECISION, 0, icomm, istat)
!!$       if (istat /= 0) return
!!$    case (QM_WFTYPE_CASSCF)
!!$       call mpi_barrier(icomm, istat)
!!$       call pl_prof_region_begin('s:Guess')
!!$       if (myrank == 0) then
!!$          call casscf_guess(self%casscf_data, integral, ldebug, istat)
!!$       end if
!!$       call mpi_bcast(istat, 1, MPI_INTEGER, 0, icomm, istat2)
!!$       if (istat /= 0) return
!!$       call mpi_barrier(icomm, istat)
!!$       call pl_prof_region_end('s:Guess')
!!$
!!$       call mpi_barrier(icomm, istat)
!!$       call pl_prof_region_begin('m:Guess_bcast')
!!$       call mpi_bcast(&
!!$            self%casscf_data%c, self%casscf_data%nao*self%casscf_data%nmo,&
!!$            MPI_DOUBLE_PRECISION, 0, icomm, istat)
!!$       if (istat /= 0) return
!!$       call mpi_barrier(icomm, istat)
!!$       call pl_prof_region_end('m:Guess_bcast')
!!$    case (QM_WFTYPE_RDFT)
!!$       call mpi_barrier(icomm, istat)
!!$       call pl_prof_region_begin('s:Guess')
!!$       if (myrank == 0) then
!!$          call rhf_guess(self%rhf_data, integral, ldebug, istat)
!!$       end if
!!$       call mpi_bcast(istat, 1, MPI_INTEGER, 0, icomm, istat2)
!!$       if (istat /= 0) return
!!$       call mpi_barrier(icomm, istat)
!!$       call pl_prof_region_end('s:Guess')
!!$
!!$       call mpi_barrier(icomm, istat)
!!$       call pl_prof_region_begin('m:Guess_bcast')
!!$       call mpi_bcast(&
!!$            self%rhf_data%c, self%rhf_data%nao*self%rhf_data%nmo,&
!!$            MPI_DOUBLE_PRECISION, 0, icomm, istat)
!!$       if (istat /= 0) return
!!$       call mpi_barrier(icomm, istat)
!!$       call pl_prof_region_end('m:Guess_bcast')
!!$    case (QM_WFTYPE_UDFT)
!!$       if (myrank == 0) then
!!$          call uhf_guess(self%uhf_data, self%uhf_options, integral,ldebug,istat)
!!$       end if
!!$       call mpi_bcast(istat, 1, MPI_INTEGER, 0, icomm, istat2)
!!$       if (istat /= 0) return
!!$
!!$       call mpi_bcast(&
!!$            self%uhf_data%ca, self%uhf_data%nao*self%uhf_data%nmo,&
!!$            MPI_DOUBLE_PRECISION, 0, icomm, istat)
!!$       if (istat /= 0) return
!!$
!!$       call mpi_bcast(&
!!$            self%uhf_data%cb, self%uhf_data%nao*self%uhf_data%nmo,&
!!$            MPI_DOUBLE_PRECISION, 0, icomm, istat)
!!$       if (istat /= 0) return
!!$    case (QM_WFTYPE_CIS)
    case default
       istat = 1
    end select

  end subroutine energy_guess

  !-----------------------------------------------------------------------------
  !> 直交化
  subroutine energy_orthogonalize(self, integral, ldebug, istat)
    use qm_options_class
    use energy_rhf_class, only: rhf_orthogonalize => orthogonalize
!!$    use energy_uhf_class, only: uhf_orthogonalize => orthogonalize
!!$    use energy_casscf_class, only: casscf_orthogonalize => orthogonalize
!!$    !use energy_cis_class, only: cis_orthogonalize => orthogonalize
    use integral_class, only: integral_type
    type(energy_type),intent(inout)::self
    type(integral_type),intent(in)::integral
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    istat = 0

    select case (self%wftype)
    case (QM_WFTYPE_RHF)
       call rhf_orthogonalize(self%rhf_data, integral, ldebug, istat)
!!$    case (QM_WFTYPE_UHF)
!!$       call uhf_orthogonalize(self%uhf_data, integral, ldebug, istat)
!!$    case (QM_WFTYPE_CASSCF)
!!$       call casscf_orthogonalize(self%casscf_data, integral, ldebug, istat)
!!$    case (QM_WFTYPE_RDFT)
!!$       call rhf_orthogonalize(self%rhf_data, integral, ldebug, istat)
!!$    case (QM_WFTYPE_UDFT)
!!$       call uhf_orthogonalize(self%uhf_data, integral, ldebug, istat)
!!$    case (QM_WFTYPE_CIS)
!!$       call rhf_orthogonalize(self%rhf_data, integral, ldebug, istat)
    case default
       istat = 1
    end select

  end subroutine energy_orthogonalize

  !-----------------------------------------------------------------------------
  !> エネルギー計算
  subroutine energy_solve(&
       self,&
!!$       dft_data, dft_options,&
       atoms, ptcharges, gscharges, fswolf, rcut_, alpha, method_zd,&
       integral, pl_mpi,&
       lprint, ldebug, istat)
    use qm_options_class
    use energy_rhf_class,    only: rhf_solve     => solve
!!$    use energy_rhf_class,    only: rhf_solve_dft => solve_dft
!!$    use energy_uhf_class,    only: uhf_solve     => solve
!!$    use energy_uhf_class,    only: uhf_solve_dft => solve_dft
!!$    use energy_casscf_class, only: casscf_solve  => solve
!!$    use energy_cis_class,    only: cis_solve => solve,&
!!$         cis_data_bcast => bcast_data
!!$    use dft_options_class, only: dft_options_type => options_type
!!$    use dft_class, only: dft_data_type
    use atoms_class,     only: atoms_type
    use ptcharges_class, only: ptcharges_type
    use gscharges_class, only: gscharges_type
    use fswolf_class, only: fswolf_type
    use integral_class,  only: integral_type,integral_calc_swz
    use mo_data_class, only: mo_data_type,&
         mo_data_new => new_data,&
         mo_data_bcast => bcast_data,&
         mo_data_free => free_data
    use pl_mpi_class
    use pl_prof_class
    use physical_constants_class
    !use energy_rhf_class,    only: rhf_read_options  => read_options
    !use energy_rhf_class,    only: rhf_bcast_options => bcast_options
    !use energy_rhf_class,    only: rhf_print_options => print_options
    !use energy_rhf_class,    only: rhf_prep      => prep
    !use energy_rhf_class,    only: rhf_guess     => guess
    !use energy_rhf_class,    only: rhf_free_options  => free_options
    !use energy_rhf_class,    only: rhf_free_data => free_data
    type(energy_type),intent(inout)::self
!!$    type(dft_options_type),intent(in)::dft_options
!!$    type(dft_data_type),intent(in)::dft_data
    type(atoms_type),intent(in)::atoms
    type(ptcharges_type),intent(in)::ptcharges
    type(gscharges_type),intent(in)::gscharges
    type(fswolf_type),intent(in)::fswolf
    type(integral_type),intent(inout)::integral
    type(pl_mpi_type),intent(in)::pl_mpi
    integer,intent(in)::lprint
    integer,intent(in)::ldebug
    integer,intent(inout)::istat
    real(8),intent(in)::rcut_
    real(8),intent(in)::alpha
    integer,intent(in)::method_zd

    type(mo_data_type)::mo_data
    real(8)::e_rep, e_total
    real(8)::V_sum
    integer::icomm, myrank
    real(8)::rcut

    !   integer(8) :: incore_count

    icomm  = pl_mpi%comm
    myrank = pl_mpi%myrank

    !call mpi_barrier(icomm, istat)
    !call pl_prof_region_begin('s:Inter_e')
    !write(6,*) 'integral%options%rcut=',integral%options%rcut
    !write(6,*) 'integral%options%zerodipole=',integral%options%zerodipole
    rcut = rcut_ / Bohr
    e_total = 0.0d0
    call interaction_energies_zd(&
         e_rep,&
         self%V_NN, self%V_gN, self%V_pN, self%V_fN,&
         self%V_gg, self%V_pg, self%V_fg,&
         self%V_pp, self%V_fp,&
         self%V_ff,&
         atoms%xyz, atoms%chg, atoms%num,&
         gscharges%xyz, gscharges%chg, gscharges%rc2v, gscharges%num,&
         ptcharges%xyz, ptcharges%chg, ptcharges%num,&
         fswolf%xyz, fswolf%chg, fswolf%rc2v, fswolf%num,&
         rcut, alpha, method_zd,&
         ldebug, istat)
    if (istat .ne. 0) return
    !call mpi_barrier(icomm, istat)
    !call pl_prof_region_end('s:Inter_e')

!!$    if (self%wftype == QM_WFTYPE_UHF .or.&
!!$        self%wftype == QM_WFTYPE_UDFT .or.&
!!$        self%wftype == QM_WFTYPE_CASSCF .or.&
!!$        self%wftype == QM_WFTYPE_CIS) then
!!$       call integral_calc_eri(integral, atoms, pl_mpi, lprint, ldebug, istat)
!!$       if (istat .ne. 0) return
!!$    end if

    allocate(integral%rsrs(integral%tables%nshelp))
    allocate(integral%eri%ishp(2,integral%tables%nshell*(integral%tables%nshell+1)/2),&
         stat=istat)
    call integral_calc_swz(integral, atoms, pl_mpi, ldebug, istat)

    select case (self%wftype)
    case (QM_WFTYPE_RHF)
       call rhf_solve(&
            self%rhf_data, self%rhf_options, e_total, e_rep,&
            atoms, ptcharges, gscharges, fswolf, integral,&
            pl_mpi, lprint, ldebug,istat)
       if (istat /= 0) return
!!$    case (QM_WFTYPE_UHF)
!!$       call uhf_solve(&
!!$            self%uhf_data, self%uhf_options, e_total, e_rep,&
!!$            atoms, ptcharges, gscharges, fswolf, integral,&
!!$            pl_mpi, lprint, ldebug,istat)
!!$       if (istat /= 0) return
!!$    case (QM_WFTYPE_CASSCF)
!!$       !call rhf_bcast_options(self%rhf_options, pl_mpi, ldebug, istat)
!!$       !if (pl_mpi%myrank == 0)&
!!$       !    call rhf_print_options(self%rhf_options)
!!$       !call rhf_prep(self%rhf_data, self%nao, ldebug, istat)
!!$       !call rhf_guess(self%rhf_data, integral, ldebug, istat)
!!$       !call rhf_solve(&
!!$       !    self%rhf_data, self%rhf_options, e_total, e_rep,&
!!$       !    atoms, ptcharges, gscharges, fswolf, integral,&
!!$       !    pl_mpi, lprint,ldebug,istat)
!!$       !self%casscf_data%c(:,:) = self%rhf_data%c(:,:)
!!$       !call rhf_free_options(self%rhf_options, ldebug, istat)
!!$       !call rhf_free_data(self%rhf_data, ldebug, istat)
!!$       call casscf_solve(&
!!$            self%casscf_data, self%casscf_options, e_total, e_rep,&
!!$            atoms, ptcharges, gscharges, fswolf, integral,&
!!$            pl_mpi, lprint, ldebug,istat)
!!$       if (istat /= 0) return
!!$       if (lprint >= 5) then
!!$          call ummtw('MO Coeffients', self%casscf_data%c,&
!!$              self%casscf_data%nao, self%casscf_data%nao, self%casscf_data%nmo,&
!!$              10, 6, istat)
!!$          if (istat /= 0) return
!!$       end if
!!$    case (QM_WFTYPE_RDFT)
!!$       call rhf_solve_dft(&
!!$            self%rhf_data, self%rhf_options,&
!!$            dft_data, dft_options,&
!!$            e_total, e_rep,&
!!$            atoms, ptcharges, gscharges, fswolf, integral,&
!!$            pl_mpi, lprint, ldebug,istat)
!!$       if (istat /= 0) return
!!$    case (QM_WFTYPE_UDFT)
!!$       call uhf_solve_dft(&
!!$            self%uhf_data, self%uhf_options,&
!!$            dft_data, dft_options,&
!!$            e_total, e_rep,&
!!$            atoms, ptcharges, gscharges, fswolf, integral,&
!!$            pl_mpi, lprint, ldebug,istat)
!!$       if (istat /= 0) return
!!$    case (QM_WFTYPE_CIS)
!!$       call rhf_solve(&
!!$            self%rhf_data, self%rhf_options, e_total, e_rep,&
!!$            atoms, ptcharges, gscharges, fswolf, integral,&
!!$            pl_mpi, lprint, ldebug,istat)
!!$       if (istat /= 0) return
!!$       if (myrank == 0) then
!!$          call mo_data_new(&
!!$               mo_data, self%rhf_data%nao, self%rhf_data%nao,&
!!$               self%rhf_data%nmo, ldebug, istat)
!!$          if (istat /= 0) return
!!$          mo_data%c(:,:) = self%rhf_data%c(:,:)
!!$          mo_data%oe(:) = self%rhf_data%oe(:)
!!$       end if
!!$       call mo_data_bcast(mo_data, pl_mpi, ldebug, istat)
!!$       if (istat /= 0) return
!!$       call cis_solve(&
!!$            self%cis_data, self%cis_options, mo_data,&
!!$            e_total, e_rep,&
!!$            atoms, ptcharges, gscharges, fswolf, integral,&
!!$            pl_mpi, lprint, ldebug,istat)
!!$       if (istat /= 0) return
!!$       call mpi_bcast(self%cis_data%ci_val, self%cis_data%nsolv,&
!!$            MPI_DOUBLE_PRECISION, 0, icomm, istat)
!!$       if (istat /= 0) return
!!$       call mpi_bcast(self%cis_data%ci_vec,&
!!$            self%cis_data%nimo*self%cis_data%nemo*self%cis_data%nsolv,&
!!$            MPI_DOUBLE_PRECISION, 0, icomm, istat)
!!$       if (istat /= 0) return
!!$       if (istat /= 0) return
!!$       call mo_data_free(mo_data, ldebug, istat)
!!$       if (istat /= 0) return
    case default
       istat = 1
    end select

    self%total = e_total
    deallocate(integral%rsrs)
    deallocate(integral%eri%ishp)

    if (lprint >= 2) then
       if (myrank == 0) then
          V_sum =&
                 self%V_NN + self%V_gN + self%V_pN + self%V_fN&
               + self%V_gg + self%V_pg + self%V_fg&
               + self%V_pp + self%V_fp&
               + self%V_ff
          write(6,'(1x)')
          if (atoms%num > 0) then
             write(6,'('' Nuclear repulsion energy       ='',f20.10)') self%V_NN
             if (gscharges%num > 0) then
             write(6,'('' Nuclear-GaussCharge energy     ='',f20.10)') self%V_gN
             end if
             if (ptcharges%num > 0) then
             write(6,'('' Nuclear-PointCharge energy     ='',f20.10)') self%V_pN
             end if
             if (fswolf%num > 0) then
             write(6,'('' Nuclear-FSWolf energy          ='',f20.10)') self%V_fN
             end if
          end if
          if (gscharges%num > 0) then
            write(6,'('' GaussCharge-GaussCharge energy ='',f20.10)') self%V_gg
            if (ptcharges%num > 0) then
            write(6,'('' GaussCharge-PointCharge energy ='',f20.10)') self%V_pg
            end if
          end if
          if (ptcharges%num > 0) then
            write(6,'('' PointCharge-PointCharge energy ='',f20.10)') self%V_pp
          end if
          write(6,'(''                          V_Sum :'',f20.10)') V_sum
       end if
    end if

!!$    if (self%wftype == QM_WFTYPE_UHF .or.&
!!$        self%wftype == QM_WFTYPE_UDFT .or.&
!!$        self%wftype == QM_WFTYPE_CASSCF .or.&
!!$        self%wftype == QM_WFTYPE_CIS) then
!!$       call integral_free_eri(integral%eri, ldebug, istat)
!!$       if (istat .ne. 0) return
!!$    end if

  end subroutine energy_solve

  !-----------------------------------------------------------------------------
  !> エネルギー表示
  subroutine energy_print_total_energy(self)
    type(energy_type),intent(in)::self

    write(6,'(''Total energy ='',f20.10)') self%total
  end subroutine energy_print_total_energy

  !-----------------------------------------------------------------------------
  !> データの簡易表示
  subroutine energy_print_data_brief(self)
    use qm_options_class
    use energy_rhf_class, only: rhf_print_data => print_data
!!$    use energy_uhf_class, only: uhf_print_data => print_data
!!$    use energy_casscf_class, only: casscf_print_data => print_data
!!$    use energy_cis_class, only: cis_print_data => print_data
    type(energy_type),intent(in)::self

    select case (self%wftype)
    case (QM_WFTYPE_RHF)
       call rhf_print_data(self%rhf_data)
!!$    case (QM_WFTYPE_UHF)
!!$       call uhf_print_data(self%uhf_data)
!!$    case (QM_WFTYPE_CASSCF)
!!$       call casscf_print_data(self%casscf_data)
!!$    case (QM_WFTYPE_RDFT)
!!$       call rhf_print_data(self%rhf_data)
!!$    case (QM_WFTYPE_UDFT)
!!$       call uhf_print_data(self%uhf_data)
!!$    case (QM_WFTYPE_CIS)
!!$       call rhf_print_data(self%rhf_data)
!!$       call cis_print_data(self%cis_data, self%cis_options)
    case default
    end select

  end subroutine energy_print_data_brief

  !-----------------------------------------------------------------------------
  !> データの表示
  subroutine energy_print_data(self)
    use qm_options_class
    use energy_rhf_class, only: rhf_print_data => print_data
!!$    use energy_uhf_class, only: uhf_print_data => print_data
!!$    use energy_casscf_class, only: casscf_print_data => print_data
!!$    use energy_cis_class, only: cis_print_data => print_data
    type(energy_type),intent(in)::self

    select case (self%wftype)
    case (QM_WFTYPE_RHF)
       call rhf_print_data(self%rhf_data)
!!$    case (QM_WFTYPE_UHF)
!!$       call uhf_print_data(self%uhf_data)
!!$    case (QM_WFTYPE_CASSCF)
!!$       call casscf_print_data(self%casscf_data)
!!$    case (QM_WFTYPE_RDFT)
!!$       call rhf_print_data(self%rhf_data)
!!$    case (QM_WFTYPE_UDFT)
!!$       call uhf_print_data(self%uhf_data)
!!$    case (QM_WFTYPE_CIS)
!!$       write(6,*) 'print_data'
!!$       call rhf_print_data(self%rhf_data)
!!$       call cis_print_data(self%cis_data, self%cis_options)
    case default
    end select

  end subroutine energy_print_data

  !-----------------------------------------------------------------------------
  !> データの保存
  subroutine energy_save_data(&
       self, atoms, ptcharges, gscharges, fswolf, istat)
    use qm_options_class
    use energy_rhf_class,    only: rhf_save_data    => save_data
!!$    use energy_uhf_class,    only: uhf_save_data    => save_data
!!$    use energy_casscf_class, only: casscf_save_data => save_data
!!$    use energy_cis_class,    only: cis_save_data    => save_data
    use atoms_class, only: atoms_type
    use ptcharges_class,   only: ptcharges_type
    use gscharges_class,   only: gscharges_type
    use fswolf_class, only: fswolf_type
    type(energy_type),intent(in)::self
    type(atoms_type),intent(in)::atoms
    type(ptcharges_type),intent(in)::ptcharges
    type(gscharges_type),intent(in)::gscharges
    type(fswolf_type),intent(in)::fswolf
    integer,intent(inout)::istat

    istat = 0

    select case (self%wftype)
    case (QM_WFTYPE_RHF)
       call rhf_save_data(self%rhf_data, istat)
!!$    case (QM_WFTYPE_UHF)
!!$       call uhf_save_data(self%uhf_data, istat)
!!$    case (QM_WFTYPE_CASSCF)
!!$       call casscf_save_data(self%casscf_data, istat)
!!$    case (QM_WFTYPE_RDFT)
!!$       call rhf_save_data(self%rhf_data, istat)
!!$    case (QM_WFTYPE_UDFT)
!!$       call uhf_save_data(self%uhf_data, istat)
!!$    case (QM_WFTYPE_CIS)
!!$       call rhf_save_data(self%rhf_data, istat)
!!$       call cis_save_data(self%cis_data, istat)
    case default
       istat = 1
    end select

  end subroutine energy_save_data

  !-----------------------------------------------------------------------------
  !> 配列開放
  subroutine energy_free(self, ldebug, istat)
    use qm_options_class
    type(energy_type),intent(inout)::self
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    istat = 0

    call energy_free_options(self, ldebug, istat)
    if (istat .ne. 0) return

    call energy_free_data(self, ldebug, istat)
    if (istat .ne. 0) return

  end subroutine energy_free

  !-----------------------------------------------------------------------------
  !> 配列開放
  subroutine energy_free_options(self, ldebug, istat)
    use qm_options_class
    use energy_rhf_options_class, only: rhf_free_options => free_options
!!$    use energy_uhf_options_class, only: uhf_free_options => free_options
!!$    use energy_casscf_options_class, only: casscf_free_options => free_options
!!$    use energy_cis_options_class, only: cis_free_options => free_options
    type(energy_type),intent(inout)::self
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    istat = 0

    select case (self%wftype)
    case (QM_WFTYPE_RHF)
       call rhf_free_options(self%rhf_options, ldebug, istat)
!!$    case (QM_WFTYPE_UHF)
!!$       call uhf_free_options(self%uhf_options, ldebug, istat)
!!$    case (QM_WFTYPE_CASSCF)
!!$       call casscf_free_options(self%casscf_options, ldebug, istat)
!!$    case (QM_WFTYPE_RDFT)
!!$       call rhf_free_options(self%rhf_options, ldebug, istat)
!!$    case (QM_WFTYPE_UDFT)
!!$       call uhf_free_options(self%uhf_options, ldebug, istat)
!!$    case (QM_WFTYPE_CIS)
!!$       call rhf_free_options(self%rhf_options, ldebug, istat)
!!$       call cis_free_options(self%cis_options, ldebug, istat)
    case default
       istat = 1
    end select

  end subroutine energy_free_options

  !-----------------------------------------------------------------------------
  !> 配列開放
  subroutine energy_free_data(self, ldebug, istat)
    use qm_options_class
    use energy_rhf_class, only: rhf_free_data => free_data
!!$    use energy_uhf_class, only: uhf_free_data => free_data
!!$    use energy_casscf_class, only: casscf_free_data => free_data
!!$    use energy_cis_class, only: cis_free_data => free_data
    type(energy_type),intent(inout)::self
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    istat = 0

    select case (self%wftype)
    case (QM_WFTYPE_RHF)
       call rhf_free_data(self%rhf_data, ldebug, istat)
!!$    case (QM_WFTYPE_UHF)
!!$       call uhf_free_data(self%uhf_data, ldebug, istat)
!!$    case (QM_WFTYPE_CASSCF)
!!$       call casscf_free_data(self%casscf_data, ldebug, istat)
!!$    case (QM_WFTYPE_RDFT)
!!$       call rhf_free_data(self%rhf_data, ldebug, istat)
!!$    case (QM_WFTYPE_UDFT)
!!$       call uhf_free_data(self%uhf_data, ldebug, istat)
!!$    case (QM_WFTYPE_CIS)
!!$       call rhf_free_data(self%rhf_data, ldebug, istat)
!!$       call cis_free_data(self%cis_data, ldebug, istat)
    case default
       istat = 1
    end select

  end subroutine energy_free_data

end module energy_class
