! -*- coding: utf-8 -*-
!
! Dual licensed under the MIT or GPL Version 3 license.
! See 'LICENSE.MIT' for the definition of the MIT license,
! and 'COPYING' for the definition of the GPL license.
!
!> 
!!
module error_class
  implicit none
  private

  public::pl_error_print_message
  public::pl_error_print_file
  public::pl_error_print_code

!                           1234567890123456789012345678901
  integer,public,parameter::PL_ERR_INTERNAL_ERROR          = 999

  integer,public,parameter::PL_ERR_FILE_NOT_FOUND          = 200
  integer,public,parameter::PL_ERR_FILE_OPEN_ERROR         = 201
  integer,public,parameter::PL_ERR_FILE_CLOSE_ERROR        = 202
  integer,public,parameter::PL_ERR_FILE_REWIND_ERROR       = 203
  integer,public,parameter::PL_ERR_FILE_READ_ERROR         = 204
  integer,public,parameter::PL_ERR_FILE_WRITE_ERROR        = 205

  integer,public,parameter::PL_ERR_GROUP_NOT_SPECIFIED     = 300
  integer,public,parameter::PL_ERR_GROUP_BEGIN             = 301
  integer,public,parameter::PL_ERR_GROUP_END               = 302
  integer,public,parameter::PL_ERR_GROUP_EOF               = 303
  integer,public,parameter::PL_ERR_INVALID_GROUP_NAME      = 304
  integer,public,parameter::PL_ERR_DUPLICATE_GROUP         = 305

  integer,public,parameter::PL_ERR_INCORRECT_OPTION_SYNTAX = 311
  integer,public,parameter::PL_ERR_INVALID_OPTION_NAME     = 312
  integer,public,parameter::PL_ERR_DUPLICATE_OPTION        = 313

  integer,public,parameter::PL_ERR_INVALID_VALUE           = 321
  integer,public,parameter::PL_ERR_OUT_OF_RANGE            = 322
  integer,public,parameter::PL_ERR_INCONSISTENT_VALUE      = 323
  integer,public,parameter::PL_ERR_RANGE                   = 324

  integer,public,parameter::PL_ERR_INVALID_FORMAT          = 331

  integer,public,parameter::PL_ERR_INVALID_NAME            = 341

  integer,public,parameter::PL_ERR_DUPLICATE               = 351
  integer,public,parameter::PL_ERR_NO_ATOMS                = 352
  integer,public,parameter::PL_ERR_NO_BASIS_SET            = 355

!!$  integer,public,parameter::PL_ERR_NO_NAME           = 999
!!$  integer,public,parameter::PL_ERR_INVALID_XXX       = 999
!!$  integer,public,parameter::PL_ERR_INCONSISTENT_XXX  = 999
!!$  integer,public,parameter::PL_ERR_UNIMPLEMENTED_XXX = 999

  integer,public,parameter::PL_MAX_ERROR_MESSAGE_LEN = 256

! The specified UNIT number was outside the range of 1 to 255.
!error occurred while writing.
!error occurred while attempting to find the length of a file.
contains
  !-----------------------------------------------------------------------------
  subroutine pl_error_print_message(error_code)
    integer,intent(in)::error_code

    character(len=PL_MAX_ERROR_MESSAGE_LEN)::message

    select case (error_code)
    case (PL_ERR_INTERNAL_ERROR)
       message = 'INTERNAL ERROR'
    case (PL_ERR_FILE_NOT_FOUND)
       message = 'FILE_NOT_FOUND'
    case (PL_ERR_FILE_OPEN_ERROR)
       message = 'FILE_OPEN_ERROR'
    case (PL_ERR_FILE_CLOSE_ERROR)
       message = 'FILE_CLOSE_ERROR'
    case (PL_ERR_FILE_REWIND_ERROR)
       message = 'FILE_REWIND_ERROR'
    case (PL_ERR_FILE_READ_ERROR)
       message = 'FILE_READ_ERROR'
    case (PL_ERR_FILE_WRITE_ERROR)
       message = 'FILE_WRITE_ERROR'
    case (PL_ERR_GROUP_BEGIN)
       message = 'GROUP_BEGIN'
    case (PL_ERR_GROUP_END)
       message = 'GROUP_END'   
    case (PL_ERR_GROUP_EOF)
       message = 'GROUP_EOF'   
    case (PL_ERR_INVALID_GROUP_NAME)
       message = 'INVALID_GROUP_NAME'   
    case (PL_ERR_DUPLICATE_GROUP)
       message = 'DUPLICATE_GROUP'   
    case (PL_ERR_INCORRECT_OPTION_SYNTAX)
       message = 'INCORRECT_OPTION_SYNTAX'   
    case (PL_ERR_INVALID_OPTION_NAME)
       message = 'INVALID_OPTION_NAME'   
    case (PL_ERR_DUPLICATE_OPTION)
       message = 'DUPLICATE_OPTION'   
    case (PL_ERR_INVALID_VALUE)
       message = 'INVALID VALUE'
    case (PL_ERR_OUT_OF_RANGE)
       message = 'OUT_OF_RANGE'   
    case (PL_ERR_INCONSISTENT_VALUE)
       message = 'INCONSISTENT_VALUE'   
    case (PL_ERR_RANGE)
       message = 'RANGE'   
    case (PL_ERR_INVALID_FORMAT)
       message = 'INVALID FORMAT'
    case (PL_ERR_INVALID_NAME)
       message = 'INVALID_NAME'
    case (PL_ERR_DUPLICATE)
       message = 'DUPLICATE'
    case (PL_ERR_NO_ATOMS)
       message = 'NO ATOMS'
    case (PL_ERR_NO_BASIS_SET)
       message = 'NO BASIS SET'
    case default
       message = 'UNKNOWN'
    end select
    !write(6,'(2x,''Error Code : '',i6)') error_code
    write(6,'(2x,''Error :'',i4,'' : '',a)') error_code, trim(message)

  end subroutine pl_error_print_message

  !-----------------------------------------------------------------------------
  subroutine pl_error_print_file(file_name)
    character(len=*),intent(in)::file_name

    write(6,'(2x,''File : '',a)') trim(file_name)
  end subroutine pl_error_print_file

  !-----------------------------------------------------------------------------
  subroutine pl_error_print_code(error_code)
    integer,intent(in)::error_code

    write(6,'(2x,''Error Code : '',i6)') error_code
  end subroutine pl_error_print_code

  !-----------------------------------------------------------------------------
end module error_class
