! -*- coding: utf-8 -*-
!
! Dual licensed under the MIT or GPL Version 3 license.
! See 'LICENSE.MIT' for the definition of the MIT license,
! and 'COPYING' for the definition of the GPL license.
!
!> RHF力の計算クラス
!!
module force_rhf_class
  implicit none
  private

  public::solve
!!$  public::solve_dft

contains
  !-----------------------------------------------------------------------------
  !> 力の計算
  subroutine solve(&
      force_atom, natom,&
      rhf_data, atoms, ptcharges, gscharges, fswolf, integral,&
      pl_mpi, ldebug, istat)
    use energy_rhf_class, only: rhf_data_type => data_type
    use atoms_class, only: atoms_type
    use ptcharges_class, only: ptcharges_type
    use gscharges_class, only: gscharges_type
    use fswolf_class, only: fswolf_type
    use integral_class, only: integral_type, integral_get_force_rhf
    use pl_mpi_class
    use pl_prof_class
    integer,intent(in)::natom
    real(8),intent(inout)::force_atom(3,natom)
    type(rhf_data_type),intent(in)::rhf_data
    type(atoms_type),intent(in)::atoms
    type(ptcharges_type),intent(in)::ptcharges
    type(gscharges_type),intent(in)::gscharges
    type(fswolf_type),intent(in)::fswolf
    type(integral_type),intent(in)::integral
    type(pl_mpi_type),intent(in)::pl_mpi
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    real(8),allocatable::gw(:,:)

    integer::icomm, myrank

    istat = 0

    icomm  = pl_mpi%comm
    myrank = pl_mpi%myrank

    allocate(gw(3,natom), stat=istat)
    if (istat .ne. 0) return
    gw(:,:) = 0.0d0

    call integral_get_force_rhf(&
        integral,&
        gw, natom,&
        rhf_data%d, rhf_data%eij, rhf_data%nao,&
        atoms, ptcharges, gscharges, fswolf,&
        pl_mpi, ldebug, istat)
    if (istat .ne. 0) return

    call mpi_barrier(icomm, istat)
    call pl_prof_region_begin('m:dAtom_reduce')
    call mpi_reduce(&
        gw, force_atom, 3*natom, MPI_DOUBLE_PRECISION, MPI_SUM, 0, icomm, istat)
    if (istat .ne. 0) return
    call mpi_barrier(icomm, istat)
    call pl_prof_region_end('m:dAtom_reduce')

    deallocate(gw, stat=istat)
    if (istat .ne. 0) return

  end subroutine solve

!!$  !-----------------------------------------------------------------------------
!!$  !> 力の計算
!!$  subroutine solve_dft(&
!!$      force_atom, natom,&
!!$      rhf_data, dft_data, dft_options,&
!!$      atoms, ptcharges, gscharges, fswolf, integral,&
!!$      pl_mpi, ldebug, istat)
!!$    use energy_rhf_class, only: rhf_data_type => data_type
!!$    use dft_options_class, only: dft_options_type => options_type
!!$    use dft_class, only: dft_data_type, dft_get_gradient_rdft
!!$    use atoms_class, only: atoms_type
!!$    use ptcharges_class, only: ptcharges_type
!!$    use gscharges_class, only: gscharges_type
!!$    use fswolf_class, only: fswolf_type
!!$    use integral_class, only: integral_type, integral_get_force_rdft
!!$    use pl_mpi_class
!!$    use pl_prof_class
!!$    integer,intent(in)::natom
!!$    real(8),intent(inout)::force_atom(3,natom)
!!$    type(rhf_data_type),intent(in)::rhf_data
!!$    type(dft_options_type),intent(in)::dft_options
!!$    type(dft_data_type),intent(in)::dft_data
!!$    type(atoms_type),intent(in)::atoms
!!$    type(ptcharges_type),intent(in)::ptcharges
!!$    type(gscharges_type),intent(in)::gscharges
!!$    type(fswolf_type),intent(in)::fswolf
!!$    type(integral_type),intent(in)::integral
!!$    type(pl_mpi_type),intent(in)::pl_mpi
!!$    integer,intent(in)::ldebug
!!$    integer,intent(inout)::istat
!!$
!!$    real(8),allocatable::gw(:,:)
!!$    real(8),allocatable::dexcw(:,:)
!!$
!!$    real(8)::scaleh, exc, excw, totpn, totpnw
!!$    integer::icomm, myrank
!!$
!!$    istat = 0
!!$
!!$    icomm  = pl_mpi%comm
!!$    myrank = pl_mpi%myrank
!!$
!!$    scaleh = dft_options%scaleh
!!$
!!$    allocate(gw(3,natom), stat=istat)
!!$    if (istat .ne. 0) return
!!$    gw(:,:) = 0.0d0
!!$
!!$    call integral_get_force_rdft(&
!!$        integral,&
!!$        gw, natom,&
!!$        rhf_data%d, rhf_data%eij, rhf_data%nao,&
!!$        atoms, ptcharges, gscharges, fswolf,&
!!$        scaleh,&
!!$        pl_mpi, ldebug, istat)
!!$    if (istat .ne. 0) return
!!$
!!$    call mpi_barrier(icomm, istat)
!!$!$omp parallel
!!$    call pl_prof_region_begin('p:dExc')
!!$!$omp end parallel
!!$    allocate(dexcw(3,natom), stat=istat)
!!$    if (istat .ne. 0) return
!!$
!!$    dexcw = 0.0d0
!!$    call dft_get_gradient_rdft(&
!!$        dft_data, dft_options, dexcw, excw, totpnw,&
!!$        atoms%xyz, rhf_data%c, rhf_data%d,&
!!$        natom, rhf_data%nao, rhf_data%nclosed,&
!!$        ldebug, istat)
!!$    if (istat .ne. 0) return
!!$
!!$    if (ldebug >= 2) then
!!$      call mpi_reduce(&
!!$          excw, exc, 1, MPI_DOUBLE_PRECISION, MPI_SUM, 0, icomm, istat)
!!$      call mpi_reduce(&
!!$          totpnw, totpn, 1, MPI_DOUBLE_PRECISION, MPI_SUM, 0, icomm, istat)
!!$      if ( myrank == 0 ) then
!!$        write(6,'(''exc    ='',f20.10)') exc
!!$        write(6,'(''totpn  ='',f20.10)') totpn
!!$        !call ummtw('dExc', dexc, 3, 3, natom, 10, 6, istat)
!!$      end if
!!$    end if
!!$
!!$    gw(:,:) = gw(:,:) + dexcw(:,:)
!!$    call mpi_barrier(icomm, istat)
!!$!$omp parallel
!!$    call pl_prof_region_end('p:dExc')
!!$!$omp end parallel
!!$
!!$    call mpi_barrier(icomm, istat)
!!$    call pl_prof_region_begin('m:dAtom_reduce')
!!$    call mpi_reduce(&
!!$        gw, force_atom, 3*natom, MPI_DOUBLE_PRECISION, MPI_SUM, 0, icomm, istat)
!!$    if (istat .ne. 0) return
!!$    call mpi_barrier(icomm, istat)
!!$    call pl_prof_region_end('m:dAtom_reduce')
!!$
!!$    deallocate(gw, dexcw, stat=istat)
!!$    if (istat .ne. 0) return
!!$
!!$  end subroutine solve_dft

end module force_rhf_class
