! -*- coding: utf-8 -*-
!
! Dual licensed under the MIT or GPL Version 3 license.
! See 'LICENSE.MIT' for the definition of the MIT license,
! and 'COPYING' for the definition of the GPL license.
!
!! GRMAIN ===========================================================
! GRADIENT MAIN ROUTINE
! ===================================================================      

      subroutine grmain (
     O     eg      ,
     &     nsh2atm ,  nsh2shty  ,  nsh2ssh,nsh2asp,
     &     natm2asp,  nao2dir   ,  nao2sh ,
     &     fdat     ,  edat     ,
     &     xyz     ,  zz        ,  zz2    , cc     , chrg    , 
     $     xyz_gc, chrg_gc, rc2v, ngc,
     $     xyz_pc, chrg_pc, npc,
     $     xyz_fsw, chrg_fsw, rc2v_fsw, nfsw,
     &     zetpq2  ,  zxi       ,  zp32   , zetpq  ,
     &     ntprm   , ntshty,
     &     wm,dm,
     &     nang2n,
     &     nfsz     ,  nesz      ,
     &     nr_s1   ,  nr_e1     ,  ns_s1  ,  ns_e1  ,
     &     nr_s0   ,  nr_e0     ,  ns_s0  ,  ns_e0  ,
     &     nt_s0   ,  nt_e0     ,  nu_s0  ,  nu_e0  ,
     &     nao     ,  nasp      ,  natom  ,  nshell ,
     &     mxprm   ,  mxsa      ,
     &     erithr  ,
     I     icomm, nproc, myrank,
     &     lprint  ,  ldebug    ,  ierr,
     &     tables, atoms, ptcharges, gscharges, fmt,
     $     rcut, alpha_zd, method_zd)
      use integral_tables_class
      use atoms_class
      use ptcharges_class
      use gscharges_class
      use integral_fmt_class
      use pl_prof_class
      implicit real(8)(a-h,o-z)
      include 'mpif.h'
      dimension eg(3,natom)

      dimension nsh2atm(nshell),nsh2shty(nshell),nsh2ssh(nshell)
      dimension nsh2asp(nshell),natm2asp(natom),nao2dir(nao),nao2sh(nao)
      dimension fdat(nfsz,nesz),edat(nesz)
      dimension xyz(3,natom),zz(mxprm,mxsa,nasp),zz2(mxprm,mxsa,nasp)
      dimension xyz_gc(3,ngc),chrg_gc(ngc),rc2v(ngc)
      dimension xyz_pc(3,npc),chrg_pc(npc)
      dimension xyz_fsw(3,nfsw),chrg_fsw(nfsw),rc2v_fsw(nfsw)
      dimension cc(mxprm,mxsa,nasp),chrg(natom)
      dimension zetpq2(mxprm,mxprm,mxsa,nasp,mxsa,nasp)
      dimension zetpq (mxprm,mxprm,mxsa,nasp,mxsa,nasp)
      dimension zxi(mxprm,mxprm,mxsa,nasp,mxsa,nasp)
      dimension zp32(mxprm,mxprm,mxsa,nasp,mxsa,nasp)
      dimension ntprm(mxsa,nasp),ntshty(mxsa,nasp)
      dimension dm(nao,nao),wm(nao,nao)

      dimension nang2n(5,5,5,5)

      integer,allocatable :: iaoslix(:,:),iaosljx(:,:)
      integer,allocatable :: iaoslkx(:,:),iaosllx(:,:)

      integer,allocatable :: iplset(:,:),iqlset(:,:)
      integer,allocatable :: jdet(:,:)

      real(8),allocatable :: swzrs(:,:),swzdrs(:,:)
 
      real(8),allocatable::eg_w(:,:)

      real(8),allocatable::g(:,:)
      type(atoms_type),intent(in)::atoms
      type(ptcharges_type),intent(in)::ptcharges
      type(gscharges_type),intent(in)::gscharges
      type(tables_type),intent(in)::tables
      type(fmt_type),intent(in)::fmt

      real(8),allocatable::gx1(:,:)
      real(8),allocatable::gx2(:,:)

      if (ldebug > 0) then
         if (myrank == 0) then
            allocate(gx1(3,natom),gx2(3,natom))
         else
            allocate(gx1(1,1),gx2(1,1))
         end if
      end if
! =========================================================
! == OVERLAP ==

      call mpi_barrier(icomm, istat)
      call pl_prof_region_begin('p:dOEI')

      eg(:,:) = 0.0d0

      if (myrank == 0) then
         call pl_prof_region_begin('p:dovi')
         call os_dovi_t1(eg,wm,nao,tables,atoms)
         call pl_prof_region_end('p:dovi')
      end if

      if (myrank == 0) then
         call pl_prof_region_begin('p:dkei')
         call os_dkei_t1(eg,dm,nao,tables,atoms)
         call pl_prof_region_end('p:dkei')
      end if

      call pl_prof_region_begin('p:dnai')
      call os_dnai_v2(eg,dm,nao,tables,atoms,fmt,myrank,nproc)
      call pl_prof_region_end('p:dnai')

      call pl_prof_region_begin('p:hfi')
      call os_hfi_v2(eg,dm,nao,tables,atoms,fmt,myrank,nproc)
      call pl_prof_region_end('p:hfi')

      if (ldebug > 0) then
         call mpi_reduce(
     $        eg,gx1,3*natom,MPI_DOUBLE_PRECISION,MPI_SUM,0,icomm,istat)
         if (myrank == 0) then
            call ummtw( 'eg_hcore', gx1, 3, 3, natom, 10, 6, ierr)
         end if
      end if

      if ( ngc /= 0 ) then
         call pl_prof_region_begin('p:dnai_gc')
         call os_dnai_gc_v2(
     $        eg,dm,nao,tables,atoms,gscharges,fmt,myrank,nproc)
         call pl_prof_region_end('p:dnai_gc')
         if (myrank == 0) then
            if (ldebug > 0) then
               call ummtw( 'eg_dnai_gc', eg, 3, 3, natom, 10, 6, ierr)
            end if
         end if

      end if

      if ( npc /= 0 ) then

         if (ldebug > 0) then
            call ummtw('eg_dnai_pc:before', eg, 3, 3, natom, 10,6,ierr)
         end if

c$$$         !rcut = 0.0d0
c$$$         !rcut = 32.0d0/0.5291772083d0
c$$$         write(6,*) 'rcut@grmain=',rcut,alpha_zd,method_zd
c$$$         if (method_zd == 1 .or. method_zd == 2 .or. method_zd == 3)then
c$$$            call pl_prof_region_begin('p:dnai_zd')
c$$$            call os_dnai_zd_v2(
c$$$     $           eg,dm,nao,tables,atoms,ptcharges,
c$$$     $           rcut,alpha_zd,method_zd,fmt,
c$$$     $           myrank,nproc)
c$$$            call pl_prof_region_end('p:dnai_zd')
c$$$            if (myrank == 0) then
c$$$               call pl_prof_region_begin('p:demq_zd')
c$$$               call os_demq_zd_t1(
c$$$     $              eg,dm,nao,tables,atoms,ptcharges,
c$$$     $              rcut,alpha_zd,method_zd)
c$$$               call pl_prof_region_end('p:demq_zd')
c$$$            end if
c$$$         else if (method_zd == 0) then
            call pl_prof_region_begin('p:dnai_pc')
            call os_dnai_pc_v2(
     $           eg,dm,nao,tables,atoms,ptcharges,fmt,myrank,nproc)
            call pl_prof_region_end('p:dnai_pc')
c$$$         end if
         if (ldebug > 0) then
            call ummtw( 'eg_dnai_pc', eg, 3, 3, natom, 10, 6, ierr)
         end if

      end if

c$$$      if ( nfsw /= 0 ) then
c$$$         call mpi_barrier(icomm, istat)
c$$$         call pl_prof_region_begin('p:dNucAt_fsw')
c$$$!     Force Switching Wolf
c$$$         call jnuc_pc(
c$$$     +        eg,
c$$$     +        dm,
c$$$!     &     val_dhcore,
c$$$     &        nsh2atm, nsh2shty ,   nsh2ssh,
c$$$     &        natm2asp ,  nao2dir,nao2sh ,  
c$$$     &        fdat     ,  edat   ,
c$$$     &        xyz      ,  zz     ,  cc     ,  chrg   ,
c$$$     $        xyz_fsw, chrg_fsw, nfsw,
c$$$     &        zetpq2   ,  zxi    , 
c$$$     &        ntprm    ,
c$$$     &        nfsz     ,  nesz   ,
c$$$     &        nr_s1    ,  nr_e1     ,  ns_s1  ,  ns_e1  ,
c$$$     &        nao,nasp,natom,nshell,
c$$$     &        mxprm,mxsa,
c$$$     &        erithr,
c$$$     +        icomm  ,nproc,myrank,
c$$$     &        lprint,ldebug,ierr  )
c$$$         allocate(eg_w(3,natom))
c$$$         eg_w(:,:) = 0.0d0
c$$$         call jnuc_gc(
c$$$     +        eg_w,
c$$$     +        dm,
c$$$!     &     val_dhcore,
c$$$     &        nsh2atm, nsh2shty ,   nsh2ssh,
c$$$     &        natm2asp ,  nao2dir,nao2sh ,  
c$$$     &        fdat     ,  edat   ,
c$$$     &        xyz      ,  zz     ,  cc     ,  chrg   ,
c$$$     $        xyz_fsw, chrg_fsw, nfsw,
c$$$     &        zetpq2   ,  zxi    , 
c$$$     &        ntprm    ,
c$$$     &        nfsz     ,  nesz   ,
c$$$     &        nr_s1    ,  nr_e1     ,  ns_s1  ,  ns_e1  ,
c$$$     &        nao,nasp,natom,nshell,
c$$$     &        mxprm,mxsa,
c$$$     &        erithr,
c$$$     &        rc2v_fsw  ,
c$$$     +        icomm  ,nproc,myrank,
c$$$     &        lprint,ldebug,ierr  )
c$$$         deallocate(eg_w)
c$$$         call mpi_barrier(icomm, istat)
c$$$         call pl_prof_region_end('p:dNucAt_fsw')
c$$$      end if
c$$$
c$$$!     call gsum_hco( 
c$$$!    &     eg        ,
c$$$!    &     val_dhcore, dm   , natom, nao ,
c$$$!    &     nr_s1     , nr_e1, ns_s1, ns_e1  )
c$$$
c$$$!     deallocate(val_dhcore)
c$$$
c$$$      endif
c$$$
c$$$      if ( npc /= 0 ) then
c$$$         allocate(g(3,natom))
c$$$         g(:,:) = 0.0d0
c$$$!$omp parallel
c$$$         call pl_prof_region_begin('p:dnai_pc')
c$$$!$omp end parallel
c$$$         call os_dnai_pc_v2(
c$$$     $        g,dm,nao,tables,atoms,ptcharges,fmt,myrank,nproc)
c$$$!$omp parallel
c$$$         call pl_prof_region_end('p:dnai_pc')
c$$$!$omp end parallel
c$$$         !call ummtw('dnai_pc', g, 3, 3, natom, 10, 6, ierr)
c$$$         eg(:,:) = eg(:,:) + g(:,:)
c$$$         deallocate(g)
c$$$      end if
c$$$
c$$$      if ( lprint > 0 )
      call mpi_barrier(icomm, istat)
      call pl_prof_region_end('p:dOEI')

! == JTEI ==
      call mpi_barrier(icomm, istat)
      call pl_prof_region_begin('::dERI')

      ir_s = nao2sh(nr_s0) ; ir_e = nao2sh(nr_e0)
      is_s = nao2sh(ns_s0) ; is_e = nao2sh(ns_e0)
      it_s = nao2sh(nt_s0) ; it_e = nao2sh(nt_e0)
      iu_s = nao2sh(nu_s0) ; iu_e = nao2sh(nu_e0)

      allocate (swzrs(nshell,nshell),swzdrs(nshell,nshell))

      call swz_main(
     o     cutthrp , cutthrq ,  cutthdp,   cutthdq ,
     &     swzrs  ,  swzdrs ,
     &     zz     ,  zz2    , cc     , 
     &     xyz    ,
     &     fdat   ,  edat   , 
     &     ntprm  ,  ntshty ,      
     &     nsh2ssh,  nsh2asp, nsh2atm, nsh2shty,
     &     nfsz   ,  nesz   , erithr,
     &     nshell ,  mxsa   ,  nasp ,  mxprm , natom,nao,
     &     ir_s   ,  ir_e   ,  is_s ,  is_e,
     &     it_s   ,  it_e   ,  iu_s ,  iu_e,
     &     icomm, myrank,  nproc,
     &     lprint ,  ldebug , ierr )

      !call ummtw( 'swzrs', swzrs, nshell, nshell, nshell, 10, 6, ierr)
      !call ummtw( 'swzdrs',swzdrs,nshell, nshell, nshell, 10, 6, ierr)

      nr_s = nr_s0 ; nr_e = nr_e0
      ns_s = ns_s0 ; ns_e = ns_e0
      nt_s = nt_s0 ; nt_e = nt_e0
      nu_s = nu_s0 ; nu_e = nu_e0
 
!      ir_s = nao2sh(nr_s) ; ir_e = nao2sh(nr_e)
!      is_s = nao2sh(ns_s) ; is_e = nao2sh(ns_e)
!      it_s = nao2sh(nt_s) ; it_e = nao2sh(nt_e)
!      iu_s = nao2sh(nu_s) ; iu_e = nao2sh(nu_e)

         ips = (ir_e - ir_s + 1) * (is_e - is_s + 1)
         iqs = (it_e - it_s + 1) * (iu_e - iu_s + 1)

         nix = (ir_e - ir_s + 1)*3
         njx = (is_e - is_s + 1)*3
         nkx = (it_e - it_s + 1)*3
         nlx = (iu_e - iu_s + 1)*3

         allocate(iaoslix(ir_s:ir_e,3),iaosljx(is_s:is_e,3))
         allocate(iaoslkx(it_s:it_e,3),iaosllx(iu_s:iu_e,3))

         allocate(iplset(3,ips),iqlset(2,iqs))
         allocate(jdet(4,8))

         call jpasize(
     o        jdet    ,
     o        iaoslix , iaosljx , iaoslkx , iaosllx ,
     o        mwki    , mwkr    , mxxang  , kphky   ,
     o        cutthrp,  cutthrq,  cutthdp,  cutthdq,
     &        nsh2ssh , nsh2asp , nsh2shty, nao2sh  ,
     &        ntshty  , 
     &        swzrs   , swzdrs  ,
     &        iplset  , iqlset  ,
     &        ips     , iqs     ,
     &        nr_s    , nr_e    , ns_s    , ns_e    ,
     &        nt_s    , nt_e    , nu_s    , nu_e    ,
     &        ir_s    , ir_e    , is_s    , is_e    ,
     &        it_s    , it_e    , iu_s    , iu_e    ,
     &        mxsa    , nao     , nshell  , nasp    ,
     &        erithr  ,
     &        lprint  ,ldebug   ,ierr          )

         if ( ldebug > 10 ) then
            write(6,*) 'iplset(1,i)=',(iplset(1,i),i=1,ips)
            write(6,*) 'iplset(2,i)=',(iplset(2,i),i=1,ips)
            write(6,*) 'iplset(3,i)=',(iplset(3,i),i=1,ips)
            write(6,*) 'iqlset(1,i)=',(iqlset(1,i),i=1,iqs)
            write(6,*) 'iqlset(2,i)=',(iqlset(2,i),i=1,iqs)
         endif

         call mpi_barrier(icomm, istat)
!$omp parallel
         call pl_prof_region_begin('p:jeri18')
!$omp end parallel
         call jeri18(
     +        eg,
     +        dm,
!     call jeri(
!    o        index_jtei, val_jtei, njtei2cut,
     &        zz2      , cc     , zetpq   , xyz    ,
     &        fdat     , edat   ,
     &        nsh2ssh  , nsh2atm, nsh2asp , nao2sh , natm2asp,
     &        nsh2shty , ntshty , ntprm   ,
     &        nang2n   ,
     &        iaoslix  , iaosljx, iaoslkx , iaosllx,
     &        iplset   , iqlset , kphky   , ips, iqs, jdet,
!    &        njtei1cut, mxxang ,
     +        mxxang ,
     &        nfsz     , nesz   , 
     &        natom    , mxsa   , mxprm   , nao    , nshell  , nasp,
     &        erithr   , 
     &        ir_s     , ir_e   , is_s    , is_e   ,
     &        it_s     , it_e   , iu_s    , iu_e   ,
     &        mwki     , mwkr   ,
     &        myrank   , nproc  ,
     &        ierr                         )
         if (ldebug > 0) then
            call mpi_reduce(
     $        eg,gx1,3*natom,MPI_DOUBLE_PRECISION,MPI_SUM,0,icomm,istat)
            if (myrank == 0) then
               if (ldebug > 0) then
                  call ummtw( 'eg_2e', gx1, 3, 3, natom, 10, 6, ierr)
               end if
            end if
         end if

         deallocate(iaoslix,iaosljx,iaoslkx,iaosllx)
         deallocate(iplset,iqlset,jdet)

         call mpi_barrier(icomm, istat)
!$omp parallel
         call pl_prof_region_end('p:jeri18')
!$omp end parallel

      deallocate(swzrs,swzdrs)

      call mpi_barrier(icomm, istat)
      call pl_prof_region_end('::dERI')

      end subroutine

! GSUM_GNR===========================================================
! summation nuclear replsion energy  for gradient
!===================================================================

      subroutine gsum_gnr(eg,gnr,natom)
      implicit real(8)(a-h,o-z)
      dimension eg(3,natom)
      dimension gnr(3,natom)

      do iat  = 1,natom
         do ixyz = 1,3
            eg(ixyz,iat) = eg(ixyz,iat) + gnr(ixyz,iat)
         enddo
      enddo

      end
! GSUM_JTEI==========================================================
! summation two electron integrals for gradient
!===================================================================

      subroutine gsum_jtei(
     &     eg        ,
     &     index_jtei,  val_jtei , dm ,
     &     njtei2cut ,  njtei1cut, nao, natom,
     &     ldebug    )

      implicit real(8)(a-h,o-z)

      integer(8) njtei2cut, njtei1cut, i

      dimension eg(3,natom)
c$$$      dimension index_jtei(6,njtei1cut),val_jtei(njtei1cut)
      dimension val_jtei(njtei1cut)
      integer(2) index_jtei(6,njtei1cut)
      dimension dm(nao,nao)

      do i = 1,njtei2cut
         ir = index_jtei(1,i)
         is = index_jtei(2,i)
         it = index_jtei(3,i)
         iu = index_jtei(4,i)
         ia = index_jtei(5,i)
         ix = index_jtei(6,i)
        
         vol = val_jtei(i)
!        write(80,"(6i4,e26.13)")index_Jtei(:,i),vol
         if (ir.ne.is) vol = vol * 2.d0
         if (it.ne.iu) vol = vol * 2.d0
         if ( (ir.eq.it).and.(is.eq.iu) ) then
            else
            vol = vol*2.d0
         endif

         alpha = dm(ir,is)*dm(it,iu)
         beta  = dm(ir,it)*dm(is,iu)
         gama  = dm(ir,iu)*dm(is,it)

         delta = beta + gama

         eg(ix,ia) = eg(ix,ia) + (0.5d0* alpha - 0.125d0*delta ) * vol

      enddo

      end

