! -*- coding: utf-8 -*-
!
! Dual licensed under the MIT or GPL Version 3 license.
! See 'LICENSE.MIT' for the definition of the MIT license,
! and 'COPYING' for the definition of the GPL license.
!
! SWZ_MAIN ==========================================================
! MAIN ROUTINE Schwarz Inequality integral cut
!===================================================================      

      subroutine swz_main(
     o     cutthrp , cutthrq ,  cutthdp,   cutthdq ,
     o     swzrs   , swzdrs , 
     &     zz      ,  zz2  , cc     ,         
     &     xyz     ,
     &     fdat    ,  edat   , 
     &     ntprm  ,  ntshty,
     &     nsh2ssh,  nsh2asp, nsh2atm,nsh2shty,
     &     nfsz   ,  nesz   , erithr,
     &     nshell ,  mxsa   ,  nasp ,  mxprm , natom,nao,
     &     ir_s,ir_e,is_s,is_e,
     &     it_s,it_e,iu_s,iu_e,
     &     icomm, myrank, nproc,
     &     lprint ,  ldebug , ierr )
      use pl_prof_class
c$$$      use mpi
           
      implicit real(8)(a-h,o-z)
      include 'mpif.h'

      dimension nsh2ssh(nshell),nsh2asp(nshell)
      dimension nsh2atm(nshell),nsh2shty(nshell)
      dimension ntprm(mxsa,nasp),ntshty(mxsa,nasp)
      dimension zz(mxprm,mxsa,nasp),zz2(mxprm,mxsa,nasp) 
      dimension cc(mxprm,mxsa,nasp)
      dimension xyz(3,natom)
      dimension fdat(nfsz,nesz),edat(nesz)
      dimension swzrs(nshell,nshell),swzdrs(nshell,nshell)

      real(8),allocatable :: e_rrrr(:),d_rrrr(:)
      real(8),allocatable :: swzrsw(:,:),swzdrsw(:,:)

      call mpi_barrier(icomm, istat)
      call pl_prof_region_begin('p:dSwz')

      allocate (e_rrrr(nshell),d_rrrr(nshell))

      call swz_rrrr(
     &     e_rrrr ,
     &     nsh2ssh,  nsh2asp,  ntprm,  ntshty,
     &     zz     ,  cc     ,         
     &     nshell ,  mxsa   ,  nasp ,  mxprm ,
     &     ierr   )

      call swz_drrrr(
     &     d_rrrr ,
     &     nsh2ssh, nsh2asp, ntprm, ntshty,
     &     zz2    , cc     ,
     &     nshell , mxsa   , nasp , mxprm,
     &     ierr )

      call iget_mxshtya(mxxang0,nshell,nsh2shty)
      call iget_bufsize(mrk , mik , mxxang0, 1)
      call jget_bufsize(mrkj, mikj, mxxang0, 1)

      allocate(swzrsw(nshell,nshell),swzdrsw(nshell,nshell))
      swzrsw = 0.d0 ; swzdrsw = 0.d0
      call swz_drsrs(
     &     swzrsw ,  swzdrsw, 
     &     zz     ,  cc     , 
     &     xyz    ,          
     &     fdat   ,  edat   , 
     &     ntprm  ,  ntshty ,      
     &     nsh2ssh,  nsh2asp, nsh2atm,
     &     mik    ,  mikj   , mrkj   ,
     &     nfsz   ,  nesz   , 
!    &     ir_s   ,  ir_e   , is_s   , is_e  ,
     &     1      ,  nshell , 1      , nshell,     
     &     natom  ,  mxsa   , mxprm  , nao   , nasp, nshell,
     &     myrank ,  nproc  ,
     &     lprint ,  ldebug , ierr )

      call swzj_cutthr(
     o     cutthrp , cutthrq ,  cutthdp,   cutthdq ,
     &     e_rrrr  , d_rrrr  ,
     &     ir_s    , ir_e    ,  is_s   ,   is_e,
     &     it_s    , it_e    ,  iu_s   ,   iu_e,
     &     nshell  , erithr  )

      deallocate(e_rrrr,d_rrrr)

      call mpi_barrier(icomm, istat)
      call pl_prof_region_end('p:dSwz')

      call mpi_barrier(icomm, istat)
      call pl_prof_region_begin('m:dSwz_allreduce')

      call mpi_allreduce(swzrsw,swzrs,nshell*nshell,
     &     mpi_double_precision,mpi_sum,icomm,ierr)

      call mpi_allreduce(swzdrsw,swzdrs,nshell*nshell,
     &     mpi_double_precision,mpi_sum,icomm,ierr)

      call mpi_barrier(icomm, istat)
      call pl_prof_region_end('m:dSwz_allreduce')

      deallocate(swzrsw,swzdrsw)

      end subroutine
CC SWZ_DRRRR ========================================================
C  CALCULATION MAX VALUE OF |(i'i'|i'i')|
C====================================================================

      subroutine swz_drrrr(
     &     d_rrrr ,
     &     nsh2ssh, nsh2asp, ntprm, ntshty,
     &     zz     , cc     ,
     &     nshell , mxsa   , nasp , mxprm , 
     &     ierr    )

      implicit real(8)(a-h,o-z)

      dimension d_rrrr(nshell)
      dimension zz(mxprm,mxsa,nasp),cc(mxprm,mxsa,nasp)
      dimension nsh2ssh(nshell),nsh2asp(nshell)
      dimension ntprm(mxsa,nasp),ntshty(mxsa,nasp)

      do i = 1,nshell

         issh = nsh2ssh(i)
         iasp = nsh2asp(i)
         ipri = ntprm(issh,iasp)
         iang = ntshty(issh,iasp)

         if (iang.ge.4) ierr = 999 
         select case (iang)
         case(1)
         call swz_dssss(zz(1,issh,iasp),cc(1,issh,iasp),ipri,rmax)
          case(2)
         call swz_dpppp(zz(1,issh,iasp),cc(1,issh,iasp),ipri,rmax)
         case(3)
         call swz_ddddd(zz(1,issh,iasp),cc(1,issh,iasp),ipri,rmax)
         end select

         d_rrrr(i) = rmax

      enddo

      if (ierr.eq.999)  then
         write(*,*)"ERROR : SWZ_DRRR"
         stop
      endif

      end
C JCSSMX ============================================================
C (s's's's') CALCULATION AND FIND MAX VALUE
C====================================================================
      subroutine swz_dssss (zij,cij,ncnt,rmax)
      
      implicit real(8)(a-h,o-z)
      dimension zij(ncnt),cij(ncnt)
      
      parameter ( rt = 0.33333333333333333d0 )
      parameter ( rv = 0.2d0 )
      
      parameter (wpi52 = 197.915435609545184401d0)
      
      pppp = 0.0d0
      bssss= 0.0d0
    
      bb = 1.0d0

       
      do i= 1,ncnt
         do j= 1,ncnt
            
            zet =  zij(i) + zij(j) 
            wij = cij(i) * cij(j) * wpi52 / zet
            rzet= 1.0d0 / zet
            
            do k= 1,ncnt
               do l= 1,ncnt
                  
                  eta  = zij(k) + zij(l)
                  reta = 1.0d0 / eta
                  wexp = eta + zet
                  upq  = wij / ( eta * dsqrt(wexp) )

                  s0 = cij(k) * cij(l) * upq
                  s1 = s0 * rt  !1.0d0 / 3.0d0
                  s2 = s0 * rv  !1.0d0 / 5.0d0
                  
                  roo2 = zet *eta / wexp
                  rooz = roo2 *rzet
                  rooe = roo2 *reta
                  zeta = zet / eta

!     ===OS PART===
                  d0 = rzet*(s0-rooz*s1)
                  d1 = rzet*(s1-rooz*s2)

                  g0 = 3.0d0*rzet*(d0-rooz*d1)
                  
!     ===HS PART===
                  pp = -zeta*d0 + 1.0d0*reta*s0
                  fp = -zeta*g0 + 3.0d0*reta*d0
                  
                  dd = -zeta*fp + 2.0d0*reta*pp + reta*d0
                  bb = zij(i)*zij(j)*zij(k)*zij(l) !*16.0d0
                  dd = bb *  dd
                  
                  pppp = pppp + dd  
                  
               enddo
            enddo
            
         enddo
      enddo

      rmax = dabs(pppp)

      end
C JCPPMX ============================================================
C (p'p'p'p') CALCULATION AND FIND MAX VALUE
C====================================================================
      subroutine swz_dpppp (zij,cij,ncnt,rmax)
      
      implicit real(8)(a-h,o-z)
      parameter (wpi52 = 197.915435609545184401d0)
      dimension zij(ncnt),cij(ncnt)
      
      parameter ( r3 = 0.333333333333333333d0 )
      parameter ( r5 = 0.2d0 )
      parameter ( r7 = 0.142857142855714286d0 )
      parameter ( r9 = 0.111111111111111111d0 )
      
      
      ssss = 0.0d0
      dsss = 0.0d0
      gsss = 0.0d0
      dsds = 0.0d0
      gsds = 0.0d0
      gsgs = 0.0d0
      gsg2 = 0.0d0
      
      dsgs = 0.0d0
c      do i=1,ncnt
c        zij(i) = zij(i) * 2.0d0
c      enddo
      
      do i= 1,ncnt
         do j= 1,ncnt
            
            zet = zij(i) + zij(j) 
            wij = cij(i) * cij(j) * wpi52 / zet
            rzet= 1.0d0 / zet
            
            bjij = zij(i) * zij(j)
            
            do k= 1,ncnt
               do l= 1,ncnt
                  
                  bjkl = zij(k) * zij(l)
                  
                  eta  =  zij(k) + zij(l) 
                  reta = 1.0d0 / eta
                  wexp = eta + zet
                  upq  = wij / ( eta * dsqrt(wexp) )
                  
                  s0 = cij(k) * cij(l) * upq
                  s1 = s0 * r3  !1.0d0 / 3.0d0
                  s2 = s0 * r5  !1.0d0 / 5.0d0
                  s3 = s0 * r7  !1.0d0 / 7.0d0
                  s4 = s0 * r9  !1.0d0 / 9.0d0
                  
                  roo2 = zet *eta / wexp
                  rooz = roo2 *rzet
                  rooe = roo2 *reta
                  zeta = zet / eta
                  
!     ===OS PART===
                  d0 = rzet*(s0-rooz*s1)
                  d1 = rzet*(s1-rooz*s2)
                  d2 = rzet*(s2-rooz*s3)
                  d3 = rzet*(s3-rooz*s4)
                 !!
!                 sd  = reta*(s0-rooe*s1)
                 !!
                  g0 = 3.0d0*rzet*(d0-rooz*d1)
                  g1 = 3.0d0*rzet*(d1-rooz*d2)
                  g2 = 3.0d0*rzet*(d2-rooz*d3)
                  bg0=       rzet*(d0-rooz*d1)!yyxx
                  
                  ai0= 5.0d0*rzet*(g0-rooz*g1)
                  ai1= 5.0d0*rzet*(g1-rooz*g2)
                  bi0=       rzet*(g0-rooz*g1)!yyyyxx
                  bi1=       rzet*(g1-rooz*g2)

                  ak0= 7.0d0*rzet*(ai0-rooz*ai1)
                  bk0= 3.0d0*rzet*(bi0-rooz*bi1)

!     ===HS PART===
                  pp = -zeta*d0  + 1.0d0 * reta * s0
                  fp = -zeta*g0  + 3.0d0 * reta * d0
                  hp = -zeta*ai0 + 5.0d0 * reta * g0
                  ajp= -zeta*ak0 + 7.0d0 * reta *ai0
                  
                  bfp= -zeta*bg0  +         reta *  d0
                  bhp= -zeta*bi0  +3.0d0 *  reta * bg0
                  chp= -zeta*bi0  +         reta *  g0
                  bjp= -zeta*bk0  +3.0d0 *  reta * bi0



                  dd = -zeta* fp + 2.0d0 * reta * pp + reta * d0
                  gd = -zeta* hp + 4.0d0 * reta * fp + reta * g0
                  aid= -zeta*ajp + 6.0d0 * reta * hp + reta *ai0

                  bgd= -zeta*bhp + 2.0d0 * reta * bfp +reta *bg0
                  bid= -zeta*bjp + 2.0d0 * reta * chp +reta *bi0
                 !!!
!                 pf = -zeta*dd  + 1.d0  * reta * sd + 2.d0  * reta * pp
                 !!!
                  ff = -zeta* gd + 3.0d0 * reta * dd + 2.0d0 * reta * fp
                  hf = -zeta*aid + 5.0d0 * reta * gd + 2.0d0 * reta * hp

                  bff= -zeta*bgd +         reta * dd
                  bhf= -zeta*bid + 3.0d0 * reta *bgd
                  !!
!                 dg = -zeta*ff  + 2.d0  * reta *pf  + 3.d0  * reta *dd
                  !!
                  gg = -zeta* hf + 4.0d0 * reta * ff + 3.0d0 * reta * gd
                  bgg= -zeta*bhf + 2.0d0 * reta *bff +         reta *bgd

c     dd = bjij * bjkl * 16.0d0 * dd

                  gsgs = gsgs + gg  * bjkl *bjij !*16.0d0
                  gsg2 = gsg2 +bgg  * bjkl *bjij !*16.0d0

                  gsds = gsds + zij(i)*zij(j)*zij(k)*gd
                  dsds = dsds + zij(i)*zij(k)*dd
                  gsss = gsss + bjij  *g0
                  dsss = dsss + zij(i)*d0
                  ssss = ssss + s0

!                 dsgs = gsds + zij(i)*zij(j)*zij(k)*dg

               enddo
            enddo
c            pppp= pppp +  dddd * bjij !*wij! * 16.0d0
c            bppp= bppp +  bddd * bjij !*wij! * 16.0d0

         enddo
      enddo

      pppp1 =        gsgs -4.0d0 * gsds +4.0d0*dsds
     &       + 2.0d0*gsss -4.0d0 * dsss +      ssss
      
      pppp2 = gsg2

      pppp1 = dabs(pppp1)
      pppp2 = dabs(pppp2)

      rmax = max(pppp1,pppp2)

      end
C JCDDMX ============================================================
C (d'd'd'd') CALCULATION AND FIND MAX VALUE
C====================================================================
      subroutine swz_ddddd (zij,cij,ncnt,rmax)
      
      implicit real(8)(a-h,o-z)
      parameter (wpi52 = 197.915435609545184401d0)
      dimension zij(ncnt),cij(ncnt)
      
      parameter ( r3 = 0.333333333333333333d0 )
      parameter ( r5 = 0.2d0 )
      parameter ( r7 = 0.142857142855714286d0 )
      parameter ( r9 = 0.111111111111111111d0 )
      parameter ( r11= 0.90909090909090909d-01)
      parameter ( r13= 0.76923076923076923d-01)
      
      
      aisis = 0.0d0
      aisgs = 0.0d0
      agsgs = 0.0d0
      aisds = 0.0d0
      agsds = 0.0d0
      adsds = 0.0d0

      bisis = 0.0d0
      bisgs = 0.0d0
      bgsgs = 0.0d0
      bisds = 0.0d0
      bgsds = 0.0d0
      bdsds = 0.0d0
      
      disis = 0.0d0

c      do i=1,ncnt
c        zij(i) = zij(i) * 2.0d0
c      enddo
      
      do i= 1,ncnt
         do j= 1,ncnt
            
            zet = zij(i) + zij(j) 
            wij = cij(i) * cij(j) * wpi52 / zet
            rzet= 1.0d0 / zet
            
            bjij = zij(i) * zij(j)
            
            do k= 1,ncnt
               do l= 1,ncnt
                  
                  bjkl = zij(k) * zij(l)
                  
                  eta  =  zij(k) + zij(l) 
                  reta = 1.0d0 / eta
                  wexp = eta + zet
                  upq  = wij / ( eta * dsqrt(wexp) )
                  
                  s0 = cij(k) * cij(l) * upq
                  s1 = s0 * r3  !1.0d0 / 3.0d0
                  s2 = s0 * r5  !1.0d0 / 5.0d0
                  s3 = s0 * r7  !1.0d0 / 7.0d0
                  s4 = s0 * r9  !1.0d0 / 9.0d0
                  s5 = s0 * r11 !1.0d0 / 11.0d0
                  s6 = s0 * r13 !1.0d0 / 13.0d0
                  
                  roo2 = zet *eta / wexp
                  rooz = roo2 *rzet
                  rooe = roo2 *reta
                  zeta = zet / eta

!     FXXX=====OS PART============
                  d0 =rzet*(s0 - rooz*s1)
                  d1 =rzet*(s1 - rooz*s2)
                  d2 =rzet*(s2 - rooz*s3)
                  d3 =rzet*(s3 - rooz*s4)
                  d4 =rzet*(s4 - rooz*s5)
                  d5 =rzet*(s5 - rooz*s6)
                  
                  g0 =  3.0d0* rzet*(d0 - rooz*d1) 
                  g1 =  3.0d0* rzet*(d1 - rooz*d2)
                  g2 =  3.0d0* rzet*(d2 - rooz*d3)
                  g3 =  3.0d0* rzet*(d3 - rooz*d4)
                  g4 =  3.0d0* rzet*(d4 - rooz*d5)
                  bg0=         rzet*(d0 - rooz*d1) !yyxx
                  bg1=         rzet*(d1 - rooz*d2)
                  bg2=         rzet*(d2 - rooz*d3)
                  bg3=         rzet*(d3 - rooz*d4)
                  bg4=         rzet*(d4 - rooz*d5)
                  
                  ai0 = 5.0d0* rzet*(g0 - rooz*g1)
                  ai1 = 5.0d0* rzet*(g1 - rooz*g2)
                  ai2 = 5.0d0* rzet*(g2 - rooz*g3)
                  ai3 = 5.0d0* rzet*(g3 - rooz*g4)
                  bi0 = 3.0d0* rzet*(bg0 - rooz*bg1) !yyxxxx
                  bi1 = 3.0d0* rzet*(bg1 - rooz*bg2)
                  bi2 = 3.0d0* rzet*(bg2 - rooz*bg3)
                  bi3 = 3.0d0* rzet*(bg3 - rooz*bg4)
                  di0 =        rzet*(bg0 - rooz*bg1) !zzyyxx
                  
                  ak0 = 7.0d0* rzet*(ai0 - rooz*ai1)
                  ak1 = 7.0d0* rzet*(ai1 - rooz*ai2)
                  ak2 = 7.0d0* rzet*(ai2 - rooz*ai3)
                  bk0 =        rzet*(ai0 - rooz*ai1)!yyxxxxxx
                  ck0 = 3.0d0* rzet*(bi0 - rooz*bi1)!yyyyxxxx
                  ck1 = 3.0d0* rzet*(bi1 - rooz*bi2)
                  ck2 = 3.0d0* rzet*(bi2 - rooz*bi3)
                  dk0 =        rzet*(bi0 - rooz*bi1)!zzyyxxxx
                   
                  am0 = 9.0d0* rzet*(ak0 - rooz*ak1)
                  am1 = 9.0d0* rzet*(ak1 - rooz*ak2)
                  bm0 =        rzet*(ak0 - rooz*ak1)!yyxxxxxxxx
                  cm0 = 5.0d0* rzet*(ck0 - rooz*ck1)!yyyyxxxxxx
                  cm1 = 5.0d0* rzet*(ck1 - rooz*ck2)
                  dm0 =        rzet*(ck0 - rooz*ck1)!zzyyyyxxxx
                  dm1 =        rzet*(ck1 - rooz*ck2)

                  o0 = 11.0d0* rzet*(am0 - rooz*am1)
                 co0 =  7.0d0* rzet*(cm0 - rooz*cm1)!yyyyxxxxxxxx
                 do0 =  3.0d0* rzet*(dm0 - rooz*dm1)!zzzzyyyyxxxx

!         =====HS PART======
                  
                  pp = -zeta* d0 +         reta* s0
                  fp = -zeta* g0 + 3.0d0*  reta* d0
                  hp = -zeta*ai0 + 5.0d0*  reta* g0
                  ajp= -zeta*ak0 + 7.0d0*  reta*ai0 
                  alp= -zeta*am0 + 9.0d0*  reta*ak0
                  anp= -zeta* o0 +11.0d0*  reta*am0 

                  bfp= -zeta*bg0 +         reta* d0 !yxx_y
                  bhp= -zeta*bi0 +         reta* g0 !yxxxx_y
                  bjp= -zeta*bk0 +         reta*ai0 !yxxxxxx_y
                  blp= -zeta*bm0 +         reta*ak0 !yxxxxxxxx_y

                  chp= -zeta*bi0 + 3.0d0*  reta*bg0 !yyyxx_y
                  cjp= -zeta*ck0 + 3.0d0*  reta*bi0 !yyyxxxx_y
                  clp= -zeta*cm0 + 3.0d0*  reta*bk0 !yyyxxxxxx_y
                  cnp= -zeta*co0 + 3.0d0*  reta*bm0

                  dhp= -zeta*di0 +         reta*bg0
                  djp= -zeta*dk0 + 3.0d0*  reta*di0 
                  dlp= -zeta*dm0 + 3.0d0*  reta*dk0
                  dnp= -zeta*do0 + 3.0d0*  reta*dm0 

                  ejp= -zeta*dk0 +         reta*bi0 
                  elp= -zeta*dm0 +         reta*ck0

                  dd = -zeta* fp + 2.0d0*  reta* pp +      reta* d0
                  gd = -zeta* hp + 4.0d0*  reta* fp +      reta* g0
                  aid= -zeta*ajp + 6.0d0*  reta* hp +      reta*ai0
                  akd= -zeta*alp + 8.0d0*  reta*ajp +      reta*ak0
                  amd= -zeta*anp +10.0d0*  reta*alp +      reta*am0

                  bgd= -zeta*chp + 2.0d0*  reta*bfp +      reta*bg0!yyxx_yy
                  bid= -zeta*cjp + 2.0d0*  reta*bhp +      reta*bi0!yyxxxx
                  bkd= -zeta*clp + 2.0d0*  reta*bjp +      reta*bk0!yyxxxxxx
                  bmd= -zeta*cnp + 2.0d0*  reta*blp +      reta*bm0!yyxxxxxxxx

                  did= -zeta*djp + 2.0d0*  reta*dhp +      reta*di0
                  dkd= -zeta*dlp + 2.0d0*  reta*ejp +      reta*dk0
                  dmd= -zeta*dnp + 2.0d0*  reta*elp +      reta*dm0
                  
                  ff = -zeta* gd + 3.0d0*  reta* dd +2.0d0*reta* fp
                  hf = -zeta*aid + 5.0d0*  reta* gd +2.0d0*reta* hp
                  ajf= -zeta*akd + 7.0d0*  reta*aid +2.0d0*reta*ajp
                  alf= -zeta*amd + 9.0d0*  reta*akd +2.0d0*reta*alp
                  bff= -zeta*bgd +         reta* dd 
                  bhf= -zeta*bid + 3.0d0*  reta*bgd
                  bjf= -zeta*bkd + 5.0d0*  reta*bid
                  blf= -zeta*bmd + 7.0d0*  reta*bkd
                  
                  dhf= -zeta*did +         reta*bgd
                  djf= -zeta*dkd + 3.0d0*  reta*did
                  ejf= -zeta*dkd +         reta*bid
                  dlf= -zeta*dmd + 3.0d0*  reta*dkd
                  
                  gg = -zeta* hf + 4.0d0*  reta* ff +3.0d0*reta* gd
                  aig= -zeta*ajf + 6.0d0*  reta* hf +3.0d0*reta*aid
                  akg= -zeta*alf + 8.0d0*  reta*ajf +3.0d0*reta*akd
                  bgg= -zeta*bhf + 2.0d0*  reta*bff +      reta*bgd
                  big= -zeta*bjf + 4.0d0*  reta*bhf +      reta*bid
                  bkg= -zeta*blf + 6.0d0*  reta*bjf +      reta*bkd
                  dgg= -zeta*bhf + 2.0d0*  reta*bff +      reta*bgd
                  dig= -zeta*djf + 2.0d0*  reta*dhf +      reta*did
                  dkg= -zeta*dlf + 2.0d0*  reta*ejf +      reta*dkd
                  
                  hh = -zeta*aig + 5.0d0*  reta* gg +4.0d0*reta* hf
                  ajh= -zeta*akg + 7.0d0*  reta*aig +4.0d0*reta*ajf

                  bhh= -zeta*big + 3.0d0*  reta*bgg +2.0d0*reta*bhf
                  bjh= -zeta*bkg + 5.0d0*  reta*big +2.0d0*reta*bjf
                  
                  dhh= -zeta*dig +         reta*dgg 
                  djh= -zeta*dkg + 3.0d0*  reta*dig 
                   
                  aii= -zeta*ajh+6.0d0*reta* hh+5.0d0*reta*aig
                  bii= -zeta*bjh+4.0d0*reta*bhh+3.0d0*reta*big
                  dii= -zeta*djh+2.0d0*reta*dhh+      reta*dig
 
                  aisis = aisis + aii *  bjij  * bjkl
                  aisgs = aisgs + aig *  bjij  * zij(k)
                  agsgs = agsgs +  gg * zij(i) * zij(k)
                  aisds = aisds + aid *  bjij
                  agsds = agsds +  gd * zij(i)
                  adsds = adsds +  dd
                  
                  bisis = bisis + bii *  bjij  * bjkl
                  bisgs = bisgs + big *  bjij  * zij(k)
                  bgsgs = bgsgs + bgg * zij(i) * zij(k)
                  bisds = bisds + bid *  bjij
                  bgsds = bgsds + bgd * zij(i)
                  
                  disis = disis + dii *  bjij  * bjkl
               enddo
            enddo
         enddo
      enddo
      
      xaaaa =        aisis - 8.0d0 * aisgs + 1.6d1 * agsgs
     &     +8.0d0*aisds - 3.2d1 * agsds + 1.6d1 * adsds
      yaaaa = bisis
      
      xbbbb =        bisis - 4.0d0 * bisgs + 4.0d0 * bgsgs
     &     +2.0d0*bisds - 4.0d0 * bgsds +         adsds
      xbbbb = 9.0d0 * xbbbb
      zbbbb = 9.0d0 * disis
      
      xaaaa = dabs(xaaaa)
      yaaaa = dabs(yaaaa)
      xbbbb = dabs(xbbbb)
c$$$      zaaaa = dabs(zaaaa)
      
      rmax = max(xaaaa,yaaaa,xbbbb,zbbbb)
      
      end

!## swz_drsrs ========================================================
!   calculate (rs|rs),(r's|r's)shell type eri for schwarz's inequality
!======================================================================*
      subroutine swz_drsrs(
     &     swzrs  , swzdrs ,
     &     zz     , cc     , 
     &     xyz    ,
     &     fdat   , edat   ,
     &     ntprm  , ntshty ,
     &     nsh2ssh, nsh2asp, nsh2atm,
     &     mik    , mikj   , mrkj   ,
     &     nfsz   , nesz   ,
     &     ir_s   , ir_e   , is_s   , is_e,
     &     natom  , mxsa   , mxprm  , nao , nasp, nshell,
     &     myrank , nproc  ,
     &     lprint , ldebug , ierr   )
      
      implicit real(8) (a-h,o-z)
!     include 'mpif.h'
      dimension ntshty(mxsa,nasp),ntprm(mxsa,nasp)
      dimension zz(mxprm,mxsa,nasp),cc(mxprm,mxsa,nasp)
      dimension fdat(nfsz,nesz),edat(nesz)
      dimension nsh2ssh(nshell),nsh2asp(nshell)
      dimension nsh2atm(nshell)

      dimension xyz(3,natom)
      dimension swzrs(nshell,nshell),swzdrs(nshell,nshell)

!     dimension swzrs (ir_s:ir_e,is_s:is_e)
!     dimension swzdrs(ir_s:ir_e,is_s:is_e)

      integer,allocatable :: iswz(:)
      real(8),allocatable :: rswz(:)
! --for parallel
      logical ipart
! --for parallel

! --for parallel
      icount = -1
! --for parallel

      allocate ( iswz(mikj),rswz(mrkj) ,stat = ierr)
      iswz = 0
      rswz = 0.d0

      do i = 1,nshell!ir_s,ir_e
         do j = 1,nshell!is_s,is_e

!         --for parallel --
            icount = icount + 1
            ipart = icount.ne.myrank
            if (icount.eq.nproc-1) icount = -1
            if (ipart) cycle
!         --for parallel --

            xyxymax = 0.0d+00
            
            swzrs(i,j) = 0.0d+00
            swzdrs(i,j) = 0.0d+00

            ish  = nsh2ssh(i)
            jsh  = nsh2ssh(j)
            iasp = nsh2asp(i)
            jasp = nsh2asp(j)
            
            ipri = ntprm(ish,iasp)
            jpri = ntprm(jsh,jasp)
            
            ian = ntshty(ish,iasp)
            jan = ntshty(jsh,jasp)
            iatom = nsh2atm(i)
            jatom = nsh2atm(j)

            if((ian.eq.1).and.(jan.eq.1)) then
               
               call swz_s1s2s1s2(
     &              xyz(1,iatom)  ,   xyz(1,jatom)  ,  
     &              nfsz          ,   nesz          ,   fdat ,
     &              zz(1,ish,iasp),   zz(1,jsh,jasp),  
     &              cc(1,ish,iasp),   cc(1,jsh,jasp),  
     &              ipri          ,   jpri          ,   s1s2max   )
               
               swzrs(i,j) = s1s2max

               call jcutss(
     &              xyz(1,iatom)  , xyz(1,jatom)  ,
     &              nfsz          , nesz          , fdat , edat,
     &              zz(1,ish,iasp), zz(1,jsh,jasp),
     &              cc(1,ish,iasp), cc(1,jsh,jasp),
     &              ipri          , jpri          , ddmax      )
                
               swzdrs(i,j) = ddmax
               cycle
               
            elseif ((ian.eq.1).and.(jan.eq.2)) then
               
               call swz_psps_obara(
     &              xyz(1,jatom)  , xyz(1,iatom)  ,
     &              nfsz          , nesz          , fdat   , edat ,
     &              zz(1,jsh,jasp), zz(1,ish,iasp),
     &              cc(1,jsh,jasp), cc(1,ish,iasp),
     &              jpri          , ipri          , pspsmax )
               
               swzrs(i,j) = pspsmax

               call jcutsp(
     &              xyz(1,iatom)  , xyz(1,jatom)  ,
     &              nfsz          , nesz          , fdat , edat,
     &              zz(1,ish,iasp), zz(1,jsh,jasp),
     &              cc(1,ish,iasp), cc(1,jsh,jasp),
     &              ipri          , jpri          , ddmax      )
                
               swzdrs(i,j) = ddmax
 

               cycle
               
               
            elseif ((ian.eq.2).and.(jan.eq.1)) then
               
               call swz_psps_obara(
     &              xyz(1,iatom)  , xyz(1,jatom)  ,
     &              nfsz          , nesz          , fdat   , edat ,
     &              zz(1,ish,iasp), zz(1,jsh,jasp),
     &              cc(1,ish,iasp), cc(1,jsh,jasp),
     &              ipri          , jpri          , pspsmax )
               
               swzrs(i,j) = pspsmax
               call jcutps(
     &              xyz(1,iatom)  , xyz(1,jatom)  ,
     &              nfsz          , nesz          , fdat , edat,
     &              zz(1,ish,iasp), zz(1,jsh,jasp),
     &              cc(1,ish,iasp), cc(1,jsh,jasp),
     &              ipri          , jpri          , ddmax      )
                
               swzdrs(i,j) = ddmax
               cycle
               
               
            elseif ((ian.eq.2).and.(jan.eq.2)) then
               
               call swz_p1p2p1p2_obara(
     &              xyz(1,iatom)  , xyz(1,jatom)  ,
     &              nfsz          , nesz          , fdat , edat ,
     &              zz(1,ish,iasp), zz(1,jsh,jasp),
     &              cc(1,ish,iasp), cc(1,jsh,jasp),
     &              ipri          , jpri          , p1p2p1p2max )
               
                    swzrs(i,j) = p1p2p1p2max
               call jcutpp(
     &              xyz(1,iatom)  , xyz(1,jatom)  ,
     &              nfsz          , nesz          , fdat , edat,
     &              zz(1,ish,iasp), zz(1,jsh,jasp),
     &              cc(1,ish,iasp), cc(1,jsh,jasp),
     &              ipri          , jpri          , ddmax      )
                
                    swzdrs(i,j) = ddmax
               cycle
               
            else
            endif
            
            call iscdij (
     o           xijij , 
     &           fdat  ,   edat ,  
     &           zz    ,   cc   ,   xyz  ,
     &           iswz  ,   rswz ,
     &           nfsz  ,   nesz ,   mikj ,   mrkj  ,
     &           ian   ,   jan  ,
     &           ipri  ,   jpri ,   iasp ,   jasp  ,
     &           ish   ,   jsh  ,   iatom,   jatom ,
     &           natom ,   mxprm,   mxsa ,   nshell,  mvhgp,nasp)

                 swzrs(i,j) = xijij

!!!!~~~~dijij
            ian = ntshty(ish,iasp)
            jan = ntshty(jsh,jasp)

            call jscdij (
     o           xijij , 
     &           fdat  ,   edat ,  
     &           zz    ,   cc   ,   xyz  ,
     &           iswz  ,   rswz ,   
     &           nfsz  ,   nesz ,    mikj ,   mrkj  ,
     &           ian   ,   jan  ,
     &           ipri  ,   jpri ,   iasp ,   jasp  ,
     &           ish   ,   jsh  ,   iatom,   jatom ,
     &           natom ,   mxprm,   mxsa ,   nshell,  nasp )

                 swzdrs(i,j) = xijij

         enddo
      enddo
      
      if (ldebug.ge.1) then
         write(6,*)"  "
         write(6,'(2a9,2a20)')"r_shell","s_shell",
     &                         "max(rs|rs)","max(r's|r's)"
         do i=1,nshell
            do j=1,nshell
               write(6,'(2i9,2f20.14)')i,j,swzrs(i,j),swzdrs(i,j)
            enddo
         enddo
      endif

      end

