! -*- coding: utf-8 -*-
!
! Dual licensed under the MIT or GPL Version 3 license.
! See 'LICENSE.MIT' for the definition of the MIT license,
! and 'COPYING' for the definition of the GPL license.
!
! ISCDIJ =============================================================
! CALUCULATE (RS|RS)-TYPE INTEGRALS FOR SCHWARTZ INEQUALITY
!                                   OVER D-ANGULAR MOMENTUM 
!                                   WITH OPERATION TABLE
!
! ===================================================================

      subroutine iscdij (
     &     xijij,
     &     fdat , edat  ,
     &     zz   , cc    , xyz  ,
     &     iswz , rswz  ,
     &     nfsz , nesz  , mikj , mrkj  ,
     &     ian  , jan   ,
     &     ipri , jpri  , iasp , jasp  ,
     &     ish  , jsh   , iatom, jatom ,
     &     natom, mxprm , mxsa , nshell, mvhgp , nasp)

      implicit real(8)(a-h,o-z)

      dimension fdat(nfsz,nesz),edat(nesz)
      dimension zz(mxprm,mxsa,nasp),cc(mxprm,mxsa,nasp)
      dimension xyz(3,natom)
      dimension iswz(mikj),rswz(mrkj)
      ijan = ian*(ian+1) * jan*(jan+1)  / 4
      mvhgp =  ijan **2

      kflag = 0
      if (ian.lt.jan) then
         kflag = 1
         call iswap(ian,jan)
      endif

      maxcod = max(ian,jan)
      na = maxcod**4
     
      lflag1 = 0
      lflag2 = 0
     
      call ivrrgr (
     &     ian    , jan     , ian     , jan,
     &     iswz(1), na      , nv      ,
     &     ir     , lflag1  , lflag2            )
 
      nbfsz = 2*na + 4 * maxcod * (maxcod+1)*maxcod/2

      i01 = 1
      i02 = i01 + 10*na
      i03 = i02 + nbfsz
      i04 = i03 + 3*nv

      call ivrrgt (
     &     iswz(i01), na    , iswz(i03), nv   , 
     &     iswz(i02), nbfsz , ir       , 
     &     maxcod   , issss , ninto    , nv2  )

      iv  = nv
      j01 =  1
      j02 =  ninto + 1
      j03 =  j02 + mvhgp
      iang = 2*(ian +jan) - 4

      if (kflag.eq.0) then

      call istos (
     &     fdat          , edat          ,
     &     zz(1,ish,iasp), zz(1,jsh,jasp),
     &     cc(1,ish,iasp), cc(1,jsh,jasp),
     &     xyz(1,iatom)  , xyz(1,jatom)  ,
     &     ipri          , jpri          ,
     &     iswz(i03)     , nv            , iv   , nv2, iang,
     &     issss         , ninto         , mvhgp,
     &     rswz(j01)     , rswz(j02)     )

      else
      call istos (
     &     fdat           , edat          ,
     &     zz(1,jsh,jasp) , zz(1,ish,iasp), 
     &     cc(1,jsh,jasp) , cc(1,ish,iasp), 
     &     xyz(1,jatom)   , xyz(1,iatom)  , 
     &     jpri           , ipri          , 
     &     iswz(i03)      , nv            , iv   , nv2, iang,
     &     issss          , ninto         , mvhgp,
     &     rswz(j01)      , rswz(j02)     )

      endif

      call itsortao( ian,jan,ian,jan,mvhgp,rswz(j02) )
      
      call isijij(rswz(j02),mvhgp,xijij)
      end

! JSCDIJ =============================================================
! CALUCULATE (R'S|R'S)-TYPE INTEGRALS FOR SCHWARTZ INEQUALITY
!                                   OVER D-ANGULAR MOMENTUM 
!                                   WITH OPERATION TABLE
!
! ===================================================================

      subroutine jscdij (
     &     xijij,
     &     fdat , edat  ,
     &     zz   , cc    , xyz  ,
     &     iswz , rswz  ,
     &     nfsz , nesz  , mikj , mrkj  ,
     &     ian  , jan   ,
     &     ipri , jpri  , iasp , jasp  ,
     &     ish  , jsh   , iatom, jatom ,
     &     natom, mxprm , mxsa , nshell, nasp)

      implicit real(8)(a-h,o-z)

      dimension fdat(nfsz,nesz),edat(nesz)
      dimension zz(mxprm,mxsa,nasp),cc(mxprm,mxsa,nasp)
      dimension xyz(3,natom)
      dimension iswz(mikj),rswz(mrkj)
      
      dimension indxd(21,2),dfg(7)
      dimension mm(10)

      data (dfg(i),i=1,7)/1.73205080756887729352d+00 ,!root3
     &                    2.23606797749978969640d+00 ,!root5
     &                    3.87298334620741688517d+00 ,!root15 
     &                    2.64575131106459071617d+00 ,!root7
     &                    3.41565025531986599105d+00 ,!root105/3
     &                    5.91607978309961524133d+00 ,!root35
     &                    1.d+00                     /

      data (mm(i),i=1,10) /1,3,6,10,15,21,28,36,45,55/

      do ii=1,21
         do jj=1,2
            indxd(ii,jj) = 0
         enddo
      enddo

      iam = ian
      jam = jan
      kam = iam
      lam = jam
 
      ian =    iam + 1
      jan =    jam
      kan =    kam + 1
      lan =    lam

! ~~ (r+1 s|t+1 u)
      call iscdijkl (
     &     fdat , edat  ,
     &     zz   , cc    , xyz  ,
     &     natom, mxprm , mxsa , nshell, nasp,
     &     ian  , jan   , kan  , lan   ,
     &     ipri , jpri  , iasp , jasp  ,
     &     ish  , jsh   , iatom, jatom ,
     &     iswz , rswz  , mikj , mrkj  ,
     &     mvhgp, j02   , 1 )

      mmvhgp = mvhgp
      if (iam.eq.1) goto 1919

      ian = iam + 1
      jan = jam
      kan = kam - 1
      lan = lam

      j03 = j02 + mmvhgp

! ~~ (r+1 s|t-1 u)
      call iscdijkl (
     &     fdat , edat  ,
     &     zz   , cc   , xyz  ,
     &     natom, mxprm, mxsa , nshell ,nasp,
     &     ian  , jan   , kan  , lan  ,
     &     ipri, jpri,iasp,jasp,
     &     ish , jsh  , iatom, jatom,
     &     iswz  , rswz   , mikj  , mrkj  ,
     &     mvhgp, j03   ,2)

        call jdelndx (indxd,kan+2)

        ncount = 0
        do i0 = 1,mm(ian)
           do j0 = 1,mm(jan)
              do k0 = 1,mm(kan+2)
                 do l0 = 1,mm(lan)
                   ncount = ncount + 1
                   cofix = indxd(k0,1) 
                   novix = indxd(k0,2) 
                  if ((cofix.eq.0).or.(novix.eq.0)) goto 1111
                   i1 = (i0-1)*mm(jan)*mm(kan)*mm(lan)
                   j1 = (j0-1)        *mm(kan)*mm(lan)
                   k1 = (novix-1)             *mm(lan)
                   
                   key = i1 + j1 + k1 + l0
                   rswz(j02+ncount-1) = 
     &             rswz(j02+ncount-1) - cofix * rswz(j03+key-1)
1111             enddo
              enddo
           enddo
        enddo

      ian = iam - 1
      jan = jam
      kan = kam + 1
      lan = lam

!     j04 = j03 + mvhgp
      j04 = j02 + mmvhgp

! ~~ (r-1 s|t+1 u)
      call iscdijkl (fdat , edat  ,
!    &             i    , j     ,
     &             zz   , cc   , xyz  ,
     &             natom, mxprm, mxsa , nshell ,nasp,
     &             ian  , jan   , kan  , lan  ,
     &             ipri, jpri,iasp,jasp,
     &             ish , jsh  , iatom, jatom,
     &             iswz  , rswz   , mikj  , mrkj  , 
     &             mvhgp, j04   ,3)

        ncount = 0
        do i0 = 1,mm(ian+2)
           do j0 = 1,mm(jan)
              do k0 = 1,mm(kan)
                 do l0 = 1,mm(lan)
                   ncount = ncount + 1
                   cofix = indxd(i0,1) 
                   novix = indxd(i0,2) 
                  if ((cofix.eq.0).or.(novix.eq.0)) goto 1112
                   i1 = (novix-1)*mm(jan)*mm(kan)*mm(lan)
                   j1 = (j0-1)           *mm(kan)*mm(lan)
                   k1 = (k0-1)                   *mm(lan)
                   
                   key = i1 + j1 + k1 + l0
                   rswz(j02+ncount-1) = 
     &             rswz(j02+ncount-1) - cofix * rswz(j04+key-1)

1112             enddo
              enddo
           enddo
        enddo

      ian = iam - 1
      jan = jam
      kan = kam - 1
      lan = lam

!     j05 = j04 + mvhgp
      j05 = j02 + mmvhgp

! ~~ (r-1 s|t-1 u)
      call iscdijkl (fdat , edat  ,
!    &             i    , j     ,
     &             zz   , cc   , xyz  ,
     &             natom, mxprm, mxsa , nshell ,nasp,
     &             ian  , jan   , kan  , lan  ,
     &             ipri, jpri,iasp,jasp,
     &             ish , jsh  , iatom, jatom,
     &             iswz  , rswz   , mikj  , mrkj  , 
     &             mvhgp, j05   ,0)

 
        ncount = 0

        do i0 = 1,mm(ian+2)
           do j0 = 1,mm(jan)
              do k0 = 1,mm(kan+2)
                 do l0 = 1,mm(lan)
                   ncount = ncount + 1
                   cofix = indxd(i0,1)
                   novix = indxd(i0,2)
                   cofjx = indxd(k0,1)
                   novjx = indxd(k0,2)
                   
                  
                  if ((novix.ge.1).and.(novjx.ge.1)) then
                   i1 = (novix-1)*mm(jan)*mm(kan)*mm(lan)
                   j1 = (j0-1)           *mm(kan)*mm(lan)
                   k1 = (novjx-1)                *mm(lan)
                   key = i1 + j1 + k1 + l0
                   rswz(j02+ncount-1) = 
     &             rswz(j02+ncount-1) + cofix *cofjx * rswz(j05+key-1)

                  endif
                 enddo
              enddo
           enddo
        enddo

!     call jtsortao (ian,jan,ian,jan,mm,mmvhgp,rswz(j02) )

1919  ncount = 0
      call jshrnmd (indxd,iam,jam)

        do i0 = 1,mm(iam+1)
           do j0 = 1,mm(jam)
              do k0 = 1,mm(kam+1)
                 do l0 = 1,mm(lam)

                    i1 = indxd(i0,1)
                    j1 = indxd(j0,2)
                    k1 = indxd(k0,1)
                    l1 = indxd(l0,2)
                    ncount = ncount + 1
                    rnmd = dfg(i1)*dfg(j1)*dfg(k1)*dfg(l1)
                    rswz(j02+ncount-1) = rnmd*rswz(j02+ncount-1)
                 enddo
              enddo
           enddo
        enddo

       call jtsortao(iam+1,jam,kam+1,lam,mmvhgp,rswz(j02))

      ncount = 0
        do i0 = 1,mm(iam+1)
           do j0 = 1,mm(jam)
              do k0 = 1,mm(kam+1)
                 do l0 = 1,mm(lam)
                    ncount = ncount + 1
                 enddo
              enddo
           enddo
        enddo

       call isijij (rswz(j02),mmvhgp,xijij)

      end

! ISCDIJkL============================================================
! CALUCULATE (RS|RS)-TYPE INTEGRALS FOR SCHWARTZ INEQUALITY
!                                   OVER D-ANGULAR MOMENTUM 
!                                   WITH OPERATION TABLE
!
! ===================================================================

      subroutine iscdijkl (
     &     fdat , edat ,
!    &     i    , j    ,
     &     zz   , cc   , xyz  ,
     &     natom, mxprm, mxsa , nshell, nasp,
     &     ian  , jan  , kan  , lan   ,
     &     ipri , jpri , iasp , jasp  ,
     &     ish  , jsh  , iatom, jatom ,
     &     iswz , rswz , mikj , mrkj  ,
     &     mvhgp, j02  , jcfsw)

      implicit real(8)(a-h,o-z)

      dimension fdat(31,2001),edat(2001)
      dimension zz(mxprm,mxsa,nasp),cc(mxprm,mxsa,nasp)
      dimension xyz(3,natom)
      dimension iswz(mikj),rswz(mrkj)
      dimension mm(10)

      data (mm(i),i=1,10) /1,3,6,10,15,21,28,36,45,55/
      mvhgp =  mm(ian)*mm(jan)*mm(kan)*mm(lan)

      maxcod = max(ian,jan,kan,lan)
      na = maxcod**4
      iang = ian+jan+kan+lan - 4

      lflag1 = 0
      lflag2 = 0


      call ivrrgr (
     &     ian   , jan, kan, lan,
     &     iswz(1), na , nv ,
     &     ir    , lflag1  , lflag2  )

      nbfsz = 2*na + 4 * maxcod * (maxcod+1)*maxcod/2
      i01 = 1
      i02 = i01 + 10*na
      i03 = i02 + nbfsz
      i04 = i03 + 3*nv

      call ivrrgt (
     &     iswz(i01), na    , iswz(i03), nv   , 
     &     iswz(i02),nbfsz,ir      , 
     &     maxcod, issss   , ninto, nv2        )

      iv  = nv
      j01 =  1
      j02 =  ninto + 1
      j03 =  j02 + mvhgp

      select  case(jcfsw)

      case(0)
      call istos (
     &          fdat, edat ,
     &          zz(1,ish,iasp)        , zz(1,jsh,jasp)         ,
     &          cc(1,ish,iasp) , cc(1,jsh,jasp) ,
     &          xyz(1,iatom) , xyz(1,jatom) ,
     &          ipri, jpri,
     &          iswz(i03),nv,iv,nv2,iang,
     &          issss,ninto,mvhgp,rswz(j01),rswz(j02)           )

      case(1)

      call jstos1(
     &     fdat          , edat          , 
     &     zz(1,ish,iasp), zz(1,jsh,jasp),
     &     cc(1,ish,iasp), cc(1,jsh,jasp),
     &     xyz(1,iatom)  , xyz(1,jatom)  ,
     &     ipri          , jpri          ,
     &     iswz(i03)     , nv            , iv   , nv2, iang,
     &     issss         , ninto         , mvhgp,
     &     rswz(j01)     , rswz(j02)     )


      case(2)
      call jstos2(
     &            fdat, edat ,
     &          zz(1,ish,iasp)         , zz(1,jsh,jasp)         ,
     &          cc(1,ish,iasp) , cc(1,jsh,jasp) ,
     &          xyz(1,iatom) , xyz(1,jatom) ,
     &          ipri, jpri,
     &          iswz(i03),nv,iv,nv2,iang,
     &          issss,ninto,mvhgp,rswz(j01),rswz(j02)        )

      case(3)
      call jstos3(
     &            fdat, edat ,
     &          zz(1,ish,iasp)         , zz(1,jsh,jasp)         ,
     &          cc(1,ish,iasp) , cc(1,jsh,jasp) ,
     &          xyz(1,iatom) , xyz(1,jatom) ,
     &          ipri, jpri,
     &          iswz(i03),nv,iv,nv2,iang,
     &          issss,ninto,mvhgp,rswz(j01),rswz(j02)           )
      end select

!     call itsortao (ian,jan,ian,jan,mm,mvhgp,rswz(j02) )
 
!     call isijij ( rswz(j02),mvhgp,xijij )

!     xyxyind(i,j) = xijij
 1921 end

        subroutine jdelndx (indxd,jkind)

        implicit real(8)(a-h,o-z)
        dimension indxd(21,2)!,mm(20)

        mm = jkind*(jkind-1) /2
        do i=1,mm!mm(jkind-1)
           do j=1,2
              indxd(i,j) = 0
           enddo
        enddo

        select case (jkind)

        case(1)
        case(2)
        case(3)

        indxd( 1,1) = 1
        indxd( 3,1) = 1 
        indxd( 6,1) = 1 

        indxd( 1,2) = 1
        indxd( 3,2) = 1
        indxd( 6,2) = 1

        case(4)

        indxd( 1,1) = 2 !x
        indxd( 4,1) = 2 !y
        indxd(10,1) = 2 !z

        indxd( 1,2) = 1
        indxd( 4,2) = 2 
        indxd(10,2) = 3 

        case(5)

        indxd( 1,1) = 3  !x
        indxd( 3,1) = 1  !y
        indxd( 5,1) = 3  !y
        indxd(10,1) = 1  !z
        indxd(12,1) = 1  !z
        indxd(15,1) = 3  !z

        indxd( 1,2) = 1
        indxd( 3,2) = 1
        indxd( 5,2) = 3
        indxd(10,2) = 1
        indxd(12,2) = 3
        indxd(15,2) = 6

        case(6)
        indxd( 1,1) = 4  !x
        indxd( 3,1) = 1  !y
        indxd( 4,1) = 1  !x
        indxd( 6,1) = 4  !y
        indxd(12,1) = 1  !z
        indxd(15,1) = 1  !z
        indxd(16,1) = 1  !x
        indxd(18,1) = 1  !y
        indxd(21,1) = 4  !z

        indxd( 1,2) = 1
        indxd( 3,2) = 1
        indxd( 4,2) = 4
        indxd( 6,2) = 4
        indxd(12,2) = 1
        indxd(15,2) = 4
        indxd(16,2) =10
        indxd(18,2) =10
        indxd(21,2) =10

        endselect
        end

       subroutine jshrnmd(indxd,iam,jam)

       implicit real(8)(a-h,o-z)
       dimension indxd(21,2)

       do i=1,21
       do j=1,2
         indxd(i,j)=7
       enddo
       enddo

       ian = iam + 1
       jan = jam

       select case (ian)
       case (3)
       case (4)
       indxd( 6,1) = 1
       case (5)
       indxd( 3,1) = 2
       indxd( 7,1) = 2
       indxd( 8,1) = 2
       indxd(10,1) = 2
       indxd(11,1) = 2
       indxd(12,1) = 2
       case (6)
       indxd( 3,1) = 4
       indxd( 4,1) = 4
       indxd( 8,1) = 4
       indxd( 9,1) = 5
       indxd(10,1) = 4
       indxd(12,1) = 4
       indxd(13,1) = 5
       indxd(14,1) = 5
       indxd(15,1) = 4
       indxd(16,1) = 4
       indxd(17,1) = 4
       indxd(18,1) = 4
       end select

       select case (jan)
       case (3)
            indxd( 2,2) = 1
            indxd( 4,2) = 1
            indxd( 5,2) = 1
       case (4)
            indxd( 2,2) = 2
            indxd( 3,2) = 2
            indxd( 5,2) = 2
            indxd( 6,2) = 3
            indxd( 7,2) = 2
            indxd( 8,2) = 2
            indxd( 9,2) = 2
       case (5)
            indxd( 2,2) = 4
            indxd( 3,2) = 5
            indxd( 4,2) = 4
            indxd( 6,2) = 4
            indxd( 7,2) = 6
            indxd( 8,2) = 6
            indxd( 9,2) = 4
            indxd(10,2) = 5
            indxd(11,2) = 6
            indxd(12,2) = 5
            indxd(13,2) = 4
            indxd(14,2) = 4

       end select


       end
CC JTSORTAO==========================================================
C  SORT ERIS-ORDER FOR GAMESS-order
C ===================================================================
      subroutine jtsortao (iangi,iangj,iangk,iangl,mvtl,e)
      implicit real(8)(a-h,o-z)
      dimension e(mvtl)
      dimension mm(10)

      data (mm(i),i=1,10) /1,3,6,10,15,21,28,36,45,55/

      if (iangl.ge.3) then
         mvol = 1
         ivol = mm(iangl)
         nvol = mm(iangi)*mm(iangj)*mm(iangk)
         call jtsort (e,mvtl,iangl,ivol,mvol,nvol)
      endif
      
      if (iangk.ge.3) then
         mvol = mm(iangl)
         ivol = mm(iangk)
         nvol = mm(iangi)*mm(iangj)
         call jtsort (e,mvtl,iangk,ivol,mvol,nvol)
      endif
      
      if (iangj.ge.3) then
         mvol = mm(iangk)*mm(iangl)
         ivol = mm(iangj)
         nvol = mm(iangi)
         call jtsort (e,mvtl,iangj,ivol,mvol,nvol)
      endif

      if (iangi.ge.3) then
         mvol = mm(iangj)*mm(iangk)*mm(iangl)
         ivol = mm(iangi)
         nvol = 1
         call jtsort (e,mvtl,iangi,ivol,mvol,nvol)
      endif

      end
C  JTSORT============================================================ 
C  SORT ERI TO SUITABLE FORM(gamess ,gaussian)
CC =================================================================
      subroutine jtsort (eb,mvhgp,iangi,ivol,mvol,nvol)
      
      implicit real(8)(a-h,o-z)
c      real*16    eb(mvhgp)
      dimension    ec(mvhgp),eb(mvhgp)
      dimension md(6),mf(10),mg(15),mh(21),mi(28)
c
      data (md(i),i=1, 6)/1,3, 6,2,4,5/
      data (mf(i),i=1,10)/1,4,10,2,5,3,7,8,9,6/
      data (mg(i),i=1,15)/1,5,15,2,6,4,9,13,14,3,10,12,7,8,11/
      data (mh(i),i=1,21)/1,6,21,2,7,5,11,19,20,3,12,4,15,16,18,
     &                    8,10,17,9,13,14 /

      data (mi(i),i=1,28)/1,7,28,2,8,6,13,26,27,3,14,5,18,23,25,
     &                    4,19,22,9,12,24,10,15,11,17,20,21,16  /

      select case(ivol)

      case(6)
         do j = 1,nvol
            nof = (j-1) * ivol*mvol
            do i = 1,ivol
               
               nog = (i-1) * mvol + nof
               key = md(i)
               
                  do i1 = 1,mvol
                     ec(i1+nog) = eb((key-1)*mvol+i1+nof)
                  enddo
               
            enddo
         enddo
         
      case(10)
         do j = 1,nvol
            nof = (j-1) * ivol*mvol
            do i = 1,ivol
               nog = (i-1) * mvol + nof
               key = mf(i)
               
                  do i1 = 1,mvol
                     ec(i1+nog) = eb((key-1)*mvol+i1+nof)
                  enddo
               
            enddo
         enddo
      case(15)

         do j = 1,nvol
            nof = (j-1) * ivol*mvol
            do i = 1,ivol
               nog = (i-1) * mvol + nof
               key = mg(i)
               
                  do i1 = 1,mvol
                     ec(i1+nog) = eb((key-1)*mvol+i1+nof)
                  enddo
            enddo
         enddo
         
      case(21)

         do j = 1,nvol
            nof = (j-1) * ivol*mvol
            do i = 1,ivol
               nog = (i-1) * mvol + nof
               key = mh(i)
               
                  do i1 = 1,mvol
                     ec(i1+nog) = eb((key-1)*mvol+i1+nof)
                  enddo
            enddo
         enddo
      case(28)

         do j = 1,nvol
            nof = (j-1) * ivol*mvol
            do i = 1,ivol
               nog = (i-1) * mvol + nof
               key = mi(i)
               
                  do i1 = 1,mvol
                     ec(i1+nog) = eb((key-1)*mvol+i1+nof)
                  enddo
            enddo
         enddo
           
         
      end select
      
      do i=1,mvhgp
         eb(i) = ec(i)
      enddo
      
      return
      end

