! -*- coding: utf-8 -*-
!
! Dual licensed under the MIT or GPL Version 3 license.
! See 'LICENSE.MIT' for the definition of the MIT license,
! and 'COPYING' for the definition of the GPL license.
!
C## IERI =============================================================
C     CALCULATION OF TWO-ELECTRON INTEGRALS
C
C   === OUTPUT ===
C     val_tei   : Integral value   
C     index_tei : Index of integral value (ao number)
!     ntei2cut  : number of integrals after 2st cut
C
C=====================================================================
      SUBROUTINE jeri18 (
     +     eg,
     +     dm,
!    &     index_jtei, val_jtei , njtei2cut    ,
     &     zz       , cc      , zetpq  , xyz     ,! zetpq23, zetpqze,
     &     fdat     , edat    ,
     &     nsh2ssh  , nsh2atm , nsh2asp, nao2sh  , natm2asp,
     &     nsh2shty , ntshty  , ntprm  ,
     &     nang2n ,
     &     iaoslix  , iaosljx , iaoslkx, iaosllx,
     &     iplset   , iqlset  , kphky  , ips    ,iqs       , jdet,
     &     mxxang  ,
     &     nfsz     , nesz    , 
     &     natom    , mxsa    , mxprm  , nao     , nshell  , nasp,
     &     erithr   , 
!    &     is       , ie      , js     , je      ,
!    &     ks       , ke      , ls     , le      ,
     &     ir_s     , ir_e    , is_s   , is_e    ,
     &     it_s     , it_e    , iu_s   , iu_e    ,
     &     mik      , mrk     ,
     &     myrank   , nproc   ,
     &     ierr                                 )
c$$$      use mpi

      implicit real(8) (a-h,o-z)
      include 'mpif.h'

      dimension xyz(3,natom)
      dimension ntshty(mxsa,nasp),ntprm(mxsa,nasp)

!     integer(8) njtei1cut, njtei2cut
!     dimension val_jtei(njtei1cut)
!     integer(2) index_jtei(6,njtei1cut)
      dimension fdat(nfsz,nesz),edat(nesz)
 
!!======================================= 
      dimension zz(mxprm,mxsa,nasp),cc(mxprm,mxsa,nasp)
      dimension zetpq(mxprm,mxprm,mxsa,nasp,mxsa,nasp)
!! ======================================

      dimension iaoslix(ir_s:ir_e,3),iaosljx(is_s:is_e,3)
      dimension iaoslkx(it_s:it_e,3),iaosllx(iu_s:iu_e,3)

      dimension nsh2ssh(nshell),nsh2atm(nshell),nsh2asp(nshell)
      dimension nao2sh(nao),natm2asp(natom),nsh2shty(nshell)

      dimension iplset(3,ips),iqlset(2,iqs)
      dimension jdet(4,8)

      dimension nang2n(5,5,5,5),mm(10)

      real(8),allocatable :: pp(:,:,:),qq(:,:,:)
      real(8),allocatable :: uup(:,:),uuq(:,:)
      real(8),allocatable :: uzp(:,:),uzq(:,:)

      integer,allocatable :: ita(:)
      integer,allocatable :: iwk(:)

      dimension nhk(4,3),ntt(4,2,3)!,jdet(4,8)
      real(8),allocatable :: rwk(:)
! --- for parallel --
      logical ipart
! --- for parallel --

      dimension eg(3,natom),dm(nao,nao)
      integer(2),allocatable :: index_jtei(:,:)
      real(8),allocatable :: val_jtei(:)
      integer(8) :: mtei, njtei2cut

      data (mm(i),i=1,10)/1,3,6,10,15,21,28,36,45,55/

      allocate(rwk(mrk))
!     call jdex(jdet)

      mtei = 12*mm(mxxang)**4
      allocate(index_jtei(6,mtei),val_jtei(mtei))
      index_jtei=0
      val_jtei=0

      njtei2cut = 0

      jtnow = 0
      itnow = 1
      ierisw =1

      if (mxxang.ge.3) then
         mxxi = mxxang + 1      !mxxang+1!mxxang+2
         mxxj = mxxang          !mxxang  !mxxang+1

         itmax = mxxang**2 * mxxi * mxxj * 5
         maxsize = 12 * mxxi**4 + 4*mxxi*mm(mxxi)
         
         allocate(iwk(mik)    ,stat = ierror)
         allocate(ita(itmax)  ,stat = ierror)
         
         if (ierror.ne.0)write(*,*)"not allocate iwk,ita"

         ita = 0
         iwk = 0

      endif
      
      allocate(pp(3,mxprm,mxprm))
      allocate( uup(mxprm,mxprm))
      allocate( uzp(mxprm,mxprm))

      allocate(qq(3,mxprm,mxprm))
      allocate( uuq(mxprm,mxprm))
      allocate( uzq(mxprm,mxprm))

! ========================================================

! --- for parallel --
      icount = -1
! --- for parallel --

c     ##  start main loop
      do ip = 1,kphky
!        iiq = iplset(3,ip)
!        
!        i = iplset(1,ip)
!        j = iplset(2,ip)
!        ian = nsh2shty(i)
!        jan = nsh2shty(j)
         do iq = 1,ip
            
!       --- for parallel --
            icount = icount + 1
            ipart = icount.ne.myrank
            if (icount .eq. nproc - 1 ) icount = -1
            if ( ipart ) cycle             
!       --- for parallel --

            i = iplset(1,ip)
            j = iplset(2,ip)
            k = iplset(1,iq)
            l = iplset(2,iq)
            
            kab = 0 ; kcd =0 ; kac = 0
            if (i.eq.j) kab = 1
            if (k.eq.l) kcd = 1
            if (i.eq.k) kac = 1

            kabcd = 4*kab + 2*kcd + kac
            labcd = kabcd
 
            if (kabcd.eq.1) labcd = 0
            if ((kabcd.eq.1).and.(j.eq.l)) labcd = 1

            ian = nsh2shty(i)
            jan = nsh2shty(j)
            kan = nsh2shty(k)
            lan = nsh2shty(l)
            
            mao3 = 3*mm(ian)*mm(jan)*mm(kan)*mm(lan)
            
            call jnclas(
     &           nhk     , ntt     ,
     &           icod    , ncod    , idatom, 
     &           i       , j       , k     , l    ,
     &           nsh2atm , nsh2shty, nsh2ssh,nshell       )
            
            if (ncod.eq.0) cycle
            
            do i3 = 1,mao3
               rwk(i3) = 0.d0
            enddo

            nangty = nang2n(ian,jan,kan,lan)

            if (ncod.eq.3) then
               if ((nangty.le.8).or.(nangty.eq.11)) then
                  call jerios(
     o                 rwk(1)    ,
     o                 index_jtei, val_jtei, mtei, njtei2cut,
     &                 fdat      , edat    ,
     &                 zz        , cc      , xyz     , zetpq   ,
     &                 pp        , qq      , uup     , uuq     ,
     &                 nfsz      , nesz    ,
     &                 ian       , jan     , kan     , lan     ,
     &                 i         , j       , k       , l       , mao3  ,
     &                 mxprm     , nasp    , natom   , mxsa    , nshell,
     &                 nangty    ,           erithr  ,
     &                 ntprm     , nsh2atm , nsh2ssh , natm2asp,
     &                 nsh2shty  ,
     &                 iaoslix   , iaosljx , iaoslkx , iaosllx ,
     &                 ir_s      , ir_e    , is_s    , is_e    ,
     &                 it_s      , it_e    , iu_s    , iu_e    ,
     &                 ierr          )
                  goto 122 
               endif
            endif
!           if (nangty == 1) cycle

            do iat = 1,ncod !grad atom
               ncoc1 = nhk(1,iat)
               ndod  = nhk(4,iat)

               do iab = 1,ncoc1  !ncoc1 is 1 or 2
                  i0 = i ; j0 = j ; k0 = k ; l0 = l
                  
                  jab = 0 ; jcd = 0 ; jal = 0
                  
                  iangi = ntt(1,iab,iat)
                  iangj = ntt(2,iab,iat)
                  iangk = ntt(3,iab,iat)
                  iangl = ntt(4,iab,iat)
                  
                  jangi = iangi
                  jangj = iangj
                  jangk = iangk
                  jangl = iangl
                  
                  if (iangi.lt.iangj) then
                     call iswap(i0,j0)
                     call iswap(iangi,iangj)
                     jab = 1
                  endif
                  
                  if (iangk.lt.iangl) then
                     call iswap(k0,l0)
                     call iswap(iangk,iangl)
                     jcd = 1
                  endif
                  
                  iangp = iangi + iangj
                  iangq = iangk + iangl
                  ianga = iangp + iangq
                  
                  if ( (iangi.lt.iangk).or.
     &                 ((iangi.eq.iangk).and.(iangp.lt.iangq))) then
                     call iswap(i0,k0)
                     call iswap(j0,l0)
                     call iswap(iangi,iangk)
                     call iswap(iangj,iangl)
                     jal = 1
                  endif
                  
                  ishel = nsh2ssh(i0); jshel = nsh2ssh(j0)
                  kshel = nsh2ssh(k0); lshel = nsh2ssh(l0)
                  
                  iatom = nsh2atm(i0); jatom = nsh2atm(j0)
                  katom = nsh2atm(k0); latom = nsh2atm(l0)
                  
                  iasp = natm2asp(iatom) ; jasp = natm2asp(jatom)
                  kasp = natm2asp(katom) ; lasp = natm2asp(latom)
                  
                  call ipara_iup(
     &                 pp   ,  qq   ,
     &                 uup  ,  uuq  ,
     &                 uzp  ,  uzq  ,
     &                 zz   ,  cc   ,  zetpq,  xyz  ,
     &                 ntprm,
     &                 iasp ,  jasp ,  kasp ,  lasp ,
     &                 iatom,  jatom,  katom,  latom,
     &                 ishel,  jshel,  kshel,  lshel,
     &                 natom,  nasp ,  mxprm,  mxsa ,
     &                 ierr     )
                  
                  mvhgp = mm(iangi)*mm(iangj)*mm(iangk)*mm(iangl)
                  
                  matom = nhk(2,iat)
                  mshel = nhk(3,iat)
                  ida   = nhk(4,iat)

                  jkout = 4*jal + 2*jcd + jab + 1
                  iflg = jdet(ida,jkout)
                  masp = natm2asp(matom)

                  if (iab.eq.1) then

                     call j2zeta(
     &                    uup  ,   uuq  , 
     &                    uzp  ,   uzq  , 
     &                    zz   ,   ntprm, 
     &                    mshel,   iflg , 
     &                    ishel,   jshel,   kshel,   lshel,
     &                    iasp ,   jasp ,   kasp ,   lasp ,
     &                    mxsa ,   mxprm,   nasp ,   masp ) 
                  endif

                  nangty = nang2n(iangi,iangj,iangk,iangl)

                  if (nangty.le.21) then
!!!   prism method  
                     call ieripd (
     &                    rwk(mao3+1),
     &                    fdat   , edat   , 
     &                    zz     , zetpq  , xyz  ,
     &                    pp              , qq   ,
     &                    uup             , uuq  ,
     &                    uzp             , uzq  ,
     &                    ntprm  ,
     &                    iasp   , jasp   , kasp , lasp   ,
     &                    ishel  , jshel  , kshel, lshel  ,
     &                    iatom  , jatom  , katom, latom  ,
     &                    nasp   , mxsa   , mxprm ,natom  ,
     &                    mvhgp  ,
     &                    ierisw , nangty )

                  else   
                     if(ianga.le.9) then
                        call ierifg (
     &                       rwk(mao3+1),
     &                       fdat   , edat   , 
     &                       zz     , zetpq  , xyz  ,
     &                       pp              , qq             ,
     &                       uup             , uuq            ,
     &                       ntprm  ,
     &                       iasp   , jasp   , kasp , lasp   ,
     &                       ishel  , jshel  , kshel, lshel  ,
     &                       iatom  , jatom  , katom, latom  ,
     &                       nasp   , mxsa   , mxprm ,natom  ,
     &                       mvhgp  ,
     &                       ierisw , nangty )
                        
                     else 
                        call itmain (
     &                       rwk(mao3+1),
     &                       fdat   , edat  ,
     &                       zetpq  , xyz   ,
     &                       pp     , qq    ,
     &                       uup    , uuq   ,
     &                       ntprm  , ita   ,
     &                       mxxi   , mxxj  , mxxang  , maxsize,
     &                       iasp   , jasp  , kasp    , lasp   ,
     &                       ishel  , jshel , kshel   , lshel  ,
     &                       iatom  , jatom , katom   , latom  ,
     &                       nasp   , mxsa  , mxprm   , natom  ,
     &                       iangi  , iangj , iangk   , iangl  ,
     &                       ianga  , mvhgp ,
c$$$     &                       iwk    , rwk(mao3+1)  ,
     &                       iwk    , rwk(mao3+1+mvhgp)  ,
c$$$     &                       mik    , mrk   ,
     &                       mik    , mrk-mao3-mvhgp   ,
     &                       itnow  , jtnow )
                        
                        call jtsortao(
     &                       iangi,  iangj      ,iangk,  iangl,
     &                       mvhgp,  rwk(mao3+1)              )
                        
                     endif
                  endif
                  
                  iec = mao3 + 1 + mvhgp
                  call jtransa(
     o                 rwk(iec)   ,
     &                 rwk(mao3+1),
     &                 jangi      ,  jangj,  jangk,  jangl,
     &                 jkout      ,  mvhgp      )
                  
                  if (iab.eq.1) then
                     
                     ie = iec + mvhgp
                     call jtransb(
     &                    rwk(ie) ,
     &                    rwk(iec),
     &                    jangi   ,  jangj,  jangk,  jangl,
     &                    ian     ,  jan  ,  kan  ,  lan  ,
     &                    ida     ,  mvhgp,  mao3 )
                     
                  else
                     
                     call jtransc(
     &                    rwk(ie) ,
     &                    rwk(iec),
     &                    ian     ,  jan  ,  kan  ,lan,
     &                    ida     ,  mvhgp,  mao3     )
                  endif
                  
                  
               enddo            !loop_end iab(1or2)
               
               call jhrnmd( rwk(ie),ian,jan,kan,lan,mao3,3 )
               
               do ix = 1,mao3                 
                  rwk(ix) = rwk(ix) + rwk(ie + ix - 1)
               enddo
               
               iout = ie
               if (icod.eq.1) then 
                  if (iat.eq.ncod) then
                     iout = 1
                  else
                     cycle
                  endif
               endif
               
               call jtout(
     &              index_jtei, val_jtei, mtei, njtei2cut,
     &              iaoslix   , !iaosljx , iaoslkx  , iaosllx  ,
     &              rwk(iout) ,
     &              ian       , jan     , kan      , lan      ,
     &              i         , j       , k        , l        ,
     &              ir_s      , ir_e    , !is_s     , is_e     ,
!    &              it_s      , it_e    , iu_s     , iu_e     ,
     &              mao3      , matom   , erithr   , labcd    )
               
            enddo !loop_end iat (grad ATOM)
            
            do ix = 1,mao3
               rwk(ix) = - 1.d0*rwk(ix)
            enddo
            
            call jtout(
     &           index_jtei, val_jtei, mtei, njtei2cut,
     &           iaoslix   , !iaosljx , iaoslkx  , iaosllx  ,
     &           rwk(1)    ,
     &           ian       , jan     , kan      , lan      ,
     &           i         , j       , k        , l        ,
     &           ir_s      , ir_e    , !is_s     , is_e     ,
!    &           it_s      , it_e    , iu_s     , iu_e     ,
     &           mao3      , idatom  , erithr   , labcd    )

122      call gsum_jtei(
     +        eg,
     +        index_jtei,val_jtei,dm,
     +        njtei2cut,mtei,nao,natom,ldebug )
  
          njtei2cut = 0

         enddo !loop_end iq
      enddo !loop_end ip
      
      if (mxxang.ge.3) then
         deallocate(ita)
         deallocate(iwk)
      endif
      deallocate(index_jtei,val_jtei)
      
      deallocate(pp,qq)
      deallocate(uup,uuq)
      deallocate(uzp,uzq)
      deallocate(rwk)
      
      END
