! -*- coding: utf-8 -*-
!
! Dual licensed under the MIT or GPL Version 3 license.
! See 'LICENSE.MIT' for the definition of the MIT license,
! and 'COPYING' for the definition of the GPL license.
!
C#### SUB. GCONRG #####################################################
C##
C##    1 SUMMATION OF NUCLEAR REPULSION GRADIENT
C##       ( CLOSED AND OPEN SHELL )
C##
C#####################################################################
      subroutine gconrg ( gnr,chg,cord,natom,lprint,ldebug )
 
      implicit real(8) (a-h,o-z)
      dimension gnr(3,natom),chg(natom),cord(3,natom)
c
      data underf / 1.0d-40 /
      thresh = 1.0d-6
c
c
c --- array initialize ....
      do 110 jn = 1 , natom
        do 100 in = 1 , 3
          gnr(in,jn) = 0.d+00
  100   continue
  110 continue

      do 130 iq = 1 , natom
        do 120 ip = 1 , natom
          if (iq .ne. ip) then
            wk1       =     cord(1,iq)-cord(1,ip)
            wk2       =     cord(2,iq)-cord(2,ip)
            wk3       =     cord(3,iq)-cord(3,ip)
            if ( dabs(wk1) .lt. underf ) wk1 = 0.0d+00
            if ( dabs(wk2) .lt. underf ) wk2 = 0.0d+00
            if ( dabs(wk3) .lt. underf ) wk3 = 0.0d+00
            r2        =     wk1**2 + wk2**2 + wk3**2
            r1        = dsqrt(r2)
            if ( r1 > thresh ) then
            r3        = r1 ** 3
            gnr(1,iq) = gnr(1,iq) + chg(iq) * chg(ip) *
     1                  ( cord(1,ip)-cord(1,iq) ) /r3
            gnr(2,iq) = gnr(2,iq) + chg(iq) * chg(ip) *
     1                  ( cord(2,ip)-cord(2,iq) ) /r3
            gnr(3,iq) = gnr(3,iq) + chg(iq) * chg(ip) *
     1                  ( cord(3,ip)-cord(3,iq) ) /r3
            else
            endif
          else
          endif
  120   continue
  130 continue

      if (ldebug.eq.2) then
         write(6,*)" "
         write(6,*)"nuclear repulsion energy"
         do j=1,natom
            write(6,'(i3,3f19.10)')j,(gnr(i,j),i=1,3)
         enddo
      endif

      return
      end
c$$$C#####################################################################
c$$$      subroutine gconrg_pc_zd1 (
c$$$     $     gnr,chg_pc,xyz_pc,npc,xyz_atom,natom,rcut,
c$$$     $     lprint,ldebug )
c$$$ 
c$$$      implicit real(8) (a-h,o-z)
c$$$      dimension gnr(3,npc),chg_pc(npc),xyz_pc(3,npc),xyz_atom(3,natom)
c$$$c
c$$$      data underf / 1.0d-40 /
c$$$
c$$$      logical,allocatable::lskip(:)
c$$$      logical::flag
c$$$
c$$$      allocate(lskip(npc))
c$$$
c$$$      thresh = 1.0d-6
c$$$c
c$$$      rcut2 = rcut**2
c$$$
c$$$c
c$$$c --- array initialize ....
c$$$      do 110 jn = 1 , npc
c$$$        do 100 in = 1 , 3
c$$$          gnr(in,jn) = 0.d+00
c$$$  100   continue
c$$$  110 continue
c$$$
c$$$      do 130 iq = 1 , npc
c$$$        do 120 ip = 1 , npc
c$$$          if (iq .ne. ip) then
c$$$            wk1       =     xyz_pc(1,iq)-xyz_pc(1,ip)
c$$$            wk2       =     xyz_pc(2,iq)-xyz_pc(2,ip)
c$$$            wk3       =     xyz_pc(3,iq)-xyz_pc(3,ip)
c$$$            if ( dabs(wk1) .lt. underf ) wk1 = 0.0d+00
c$$$            if ( dabs(wk2) .lt. underf ) wk2 = 0.0d+00
c$$$            if ( dabs(wk3) .lt. underf ) wk3 = 0.0d+00
c$$$            r2        =     wk1**2 + wk2**2 + wk3**2
c$$$            r1        = dsqrt(r2)
c$$$            if ( r1 > rcut ) cycle !!!!!
c$$$            if ( r1 > thresh ) then
c$$$            r3        = r1 ** 3
c$$$            gnr(1,iq) = gnr(1,iq) + chg_pc(iq) * chg_pc(ip) *
c$$$     1                  ( xyz_pc(1,ip)-xyz_pc(1,iq) ) /r3
c$$$            gnr(2,iq) = gnr(2,iq) + chg_pc(iq) * chg_pc(ip) *
c$$$     1                  ( xyz_pc(2,ip)-xyz_pc(2,iq) ) /r3
c$$$            gnr(3,iq) = gnr(3,iq) + chg_pc(iq) * chg_pc(ip) *
c$$$     1                  ( xyz_pc(3,ip)-xyz_pc(3,iq) ) /r3
c$$$            else
c$$$            endif
c$$$          else
c$$$          endif
c$$$  120   continue
c$$$  130 continue
c$$$
c$$$      if (ldebug.eq.2) then
c$$$         write(6,*)" "
c$$$         write(6,*)"nuclear repulsion energy"
c$$$         do j=1,npc
c$$$            write(6,'(i3,3f19.10)')j,(gnr(i,j),i=1,3)
c$$$         enddo
c$$$      endif
c$$$
c$$$      return
c$$$      end
c$$$C#####################################################################
c$$$      subroutine gconrg_pc_zd3 (
c$$$     $     gnr,chg_pc,xyz_pc,npc,xyz_atom,natom,rcut,
c$$$     $     lprint,ldebug )
c$$$ 
c$$$      implicit real(8) (a-h,o-z)
c$$$      dimension gnr(3,npc),chg_pc(npc),xyz_pc(3,npc),xyz_atom(3,natom)
c$$$c
c$$$      data underf / 1.0d-40 /
c$$$
c$$$      logical,allocatable::lskip(:)
c$$$      logical::flag
c$$$
c$$$      allocate(lskip(npc))
c$$$
c$$$      thresh = 1.0d-6
c$$$c
c$$$      rcut2 = rcut**2
c$$$      lskip(:) = .true.
c$$$      do k = 1, npc
c$$$         Cx = xyz_pc(1,k)
c$$$         Cy = xyz_pc(2,k)
c$$$         Cz = xyz_pc(3,k)
c$$$         flag = .false.
c$$$         do iatom = 1, natom
c$$$            Ax = xyz_atom(1,iatom)
c$$$            Ay = xyz_atom(2,iatom)
c$$$            Az = xyz_atom(3,iatom)
c$$$            ACx = Ax - Cx
c$$$            ACy = Ay - Cy
c$$$            ACz = Az - Cz
c$$$            AC2 = ACx*ACx + ACy*ACy + ACz*ACz
c$$$            if (AC2 < rcut2) then
c$$$               flag = .true.
c$$$               go to 1
c$$$            end if
c$$$         end do
c$$$ 1       if (flag) lskip(k) = .false.
c$$$      end do
c$$$
c$$$c
c$$$c --- array initialize ....
c$$$      do 110 jn = 1 , npc
c$$$        do 100 in = 1 , 3
c$$$          gnr(in,jn) = 0.d+00
c$$$  100   continue
c$$$  110 continue
c$$$
c$$$      do 130 iq = 1 , npc
c$$$        if (lskip(iq)) cycle
c$$$        do 120 ip = 1 , npc
c$$$          if (lskip(ip)) cycle
c$$$          if (iq .ne. ip) then
c$$$            wk1       =     xyz_pc(1,iq)-xyz_pc(1,ip)
c$$$            wk2       =     xyz_pc(2,iq)-xyz_pc(2,ip)
c$$$            wk3       =     xyz_pc(3,iq)-xyz_pc(3,ip)
c$$$            if ( dabs(wk1) .lt. underf ) wk1 = 0.0d+00
c$$$            if ( dabs(wk2) .lt. underf ) wk2 = 0.0d+00
c$$$            if ( dabs(wk3) .lt. underf ) wk3 = 0.0d+00
c$$$            r2        =     wk1**2 + wk2**2 + wk3**2
c$$$            r1        = dsqrt(r2)
c$$$            if ( r1 > thresh ) then
c$$$            r3        = r1 ** 3
c$$$            gnr(1,iq) = gnr(1,iq) + chg_pc(iq) * chg_pc(ip) *
c$$$     1                  ( xyz_pc(1,ip)-xyz_pc(1,iq) ) /r3
c$$$            gnr(2,iq) = gnr(2,iq) + chg_pc(iq) * chg_pc(ip) *
c$$$     1                  ( xyz_pc(2,ip)-xyz_pc(2,iq) ) /r3
c$$$            gnr(3,iq) = gnr(3,iq) + chg_pc(iq) * chg_pc(ip) *
c$$$     1                  ( xyz_pc(3,ip)-xyz_pc(3,iq) ) /r3
c$$$            else
c$$$            endif
c$$$          else
c$$$          endif
c$$$  120   continue
c$$$  130 continue
c$$$
c$$$      if (ldebug.eq.2) then
c$$$         write(6,*)" "
c$$$         write(6,*)"nuclear repulsion energy"
c$$$         do j=1,natom
c$$$            write(6,'(i3,3f19.10)')j,(gnr(i,j),i=1,3)
c$$$         enddo
c$$$      endif
c$$$
c$$$      return
c$$$      end

