! -*- coding: utf-8 -*-
!
! Dual licensed under the MIT or GPL Version 3 license.
! See 'LICENSE.MIT' for the definition of the MIT license,
! and 'COPYING' for the definition of the GPL license.
!
C#### SUB. GCONRG #####################################################
C##
C##    1 SUMMATION OF NUCLEAR REPULSION GRADIENT
C##       ( CLOSED AND OPEN SHELL )
C##
C#####################################################################
      subroutine gconrg_pc (
     $     gnr,chg,cord,natom,
     $     xyz_pc, chrg_pc, npc,
     $     lprint,ldebug )
 
      implicit real(8) (a-h,o-z)
      dimension gnr(3,natom),chg(natom),cord(3,natom)
      dimension xyz_pc(3,npc),chrg_pc(npc)
c
      data underf / 1.0d-40 /
      thresh = 1.0d-6
c
c
c --- array initialize ....
      do 110 jn = 1 , natom
        do 100 in = 1 , 3
          gnr(in,jn) = 0.d+00
  100   continue
  110 continue

      do 130 iq = 1 , natom
        do 120 ip = 1 , npc
            wk1       =     cord(1,iq)-xyz_pc(1,ip)
            wk2       =     cord(2,iq)-xyz_pc(2,ip)
            wk3       =     cord(3,iq)-xyz_pc(3,ip)
            if ( dabs(wk1) .lt. underf ) wk1 = 0.0d+00
            if ( dabs(wk2) .lt. underf ) wk2 = 0.0d+00
            if ( dabs(wk3) .lt. underf ) wk3 = 0.0d+00
            r2        =     wk1**2 + wk2**2 + wk3**2
            r1        = dsqrt(r2)
            if ( r1 > thresh ) then
            r3        = r1 ** 3
            gnr(1,iq) = gnr(1,iq) + chg(iq) * chrg_pc(ip) *
     1                  ( xyz_pc(1,ip)-cord(1,iq) ) /r3
            gnr(2,iq) = gnr(2,iq) + chg(iq) * chrg_pc(ip) *
     1                  ( xyz_pc(2,ip)-cord(2,iq) ) /r3
            gnr(3,iq) = gnr(3,iq) + chg(iq) * chrg_pc(ip) *
     1                  ( xyz_pc(3,ip)-cord(3,iq) ) /r3
            else
            endif
  120   continue
  130 continue

      if (ldebug.eq.2) then
         write(6,*)" "
         write(6,*)"nuclear repulsion energy"
         do j=1,natom
            write(6,'(i3,3f19.10)')j,(gnr(i,j),i=1,3)
         enddo
      endif

      return
      end
CC-------------------------------------------------------------------
CC
CC-------------------------------------------------------------------
      subroutine gconrg_nuc_gc (
     $     gnr,chg,cord,natom,
     $     xyz_gc, chrg_gc, rc2v, ngc,
     $     lprint,ldebug )

      implicit real(8) (a-h,o-z)
      dimension gnr(3,natom),chg(natom),cord(3,natom)  ! Gauss Charge
      dimension xyz_gc(3,ngc),chrg_gc(ngc)             ! QM Nucl.
      dimension rc2v(ngc)
c
      data underf / 1.0d-40 /
      thresh = 1.0d-6
      pi = 3.14159265358979323846d+00
      piroot = sqrt( pi )
c
c
c --- array initialize ....
      do 110 jn = 1 , natom
        do 100 in = 1 , 3
          gnr(in,jn) = 0.d+00
  100   continue
  110 continue

!!!   write(6,'(" natom ngc = ",2i5)') natom, ngc
      do 130 iq = 1 , natom               ! GaussCharge loop
      ! write(6,'(" rc2v = ",f12.6)') rc2v(iq)
         do 120 ip = 1 , ngc    ! QM Nucl. loop

          wk1 = cord(1,iq)-xyz_gc(1,ip)
          wk2 = cord(2,iq)-xyz_gc(2,ip)
          wk3 = cord(3,iq)-xyz_gc(3,ip)

          if ( dabs(wk1) .lt. underf ) wk1 = 0.0d+00
          if ( dabs(wk2) .lt. underf ) wk2 = 0.0d+00
          if ( dabs(wk3) .lt. underf ) wk3 = 0.0d+00

          r2  = wk1**2 + wk2**2 + wk3**2
          r1  = dsqrt(r2)

          if ( r1 > thresh ) then
            if( rc2v(ip) >= thresh ) then
               alpha2 = 1.0d0/rc2v(ip)
               alpha  = sqrt( alpha2 )   
               coeff  = 2.0d0*alpha/piroot
               factor = derf(alpha*r1)/r1 - coeff*exp(-alpha2*r2)
            elseif( rc2v(ip) < thresh .and. 
     1              rc2v(ip) >= 0.0d0         ) then
              factor = 1.0d0 / r1 
            elseif( rc2v(ip) < 0 ) then
              write(6,'(" rc2v is negative value. stop : ", f12.8, i6)')
     $              rc2v(ip), ip
              stop
            endif  
          gnr(1,iq) = gnr(1,iq) - chg(iq) * chrg_gc(ip) *
     1                wk1/r2 * factor                    
          gnr(2,iq) = gnr(2,iq) - chg(iq) * chrg_gc(ip) *
     1                wk2 /r2 * factor 
          gnr(3,iq) = gnr(3,iq) - chg(iq) * chrg_gc(ip) *
     1                wk3 /r2 * factor
          else
          endif

  120   continue
  130 continue

      if (ldebug.eq.2) then
         write(6,*)" "
         write(6,*)"nuclear-gauss repulsion energy"
         do j=1,natom
            write(6,'(i3,3f19.10)')j,(gnr(i,j),i=1,3)
         enddo
      endif

      return
      end
CC-------------------------------------------------------------------
CC
CC-------------------------------------------------------------------
      subroutine gconrg_gc_nuc (
     $     gnr,chrg_gc, xyz_gc, rc2v, ngc,
     $     cord,chg,natom,
     $     lprint,ldebug )

      implicit real(8) (a-h,o-z)
      dimension gnr(3,ngc),chg(natom),cord(3,natom)
      dimension xyz_gc(3,ngc),chrg_gc(ngc)
      dimension rc2v(ngc)
c
      data underf / 1.0d-40 /
      thresh = 1.0d-6
      pi = 3.14159265358979323846d+00
      piroot = sqrt( pi )
c
c
c --- array initialize ....
      do 110 jn = 1 , ngc
        do 100 in = 1 , 3
          gnr(in,jn) = 0.d+00
  100   continue
  110 continue

c$$$      write(6,'(" natom ngc = ",2i5)') natom, ngc
      do 130 iq = 1 , ngc
         do 120 ip = 1 , natom

            wk1 = xyz_gc(1,iq)-cord(1,ip)
            wk2 = xyz_gc(2,iq)-cord(2,ip)
            wk3 = xyz_gc(3,iq)-cord(3,ip)

            if ( dabs(wk1) .lt. underf ) wk1 = 0.0d+00
            if ( dabs(wk2) .lt. underf ) wk2 = 0.0d+00
            if ( dabs(wk3) .lt. underf ) wk3 = 0.0d+00

            r2  = wk1**2 + wk2**2 + wk3**2
            r1  = dsqrt(r2)

            if ( r1 > thresh ) then
               r3  = r1 * r2

               if( rc2v(iq) >= thresh ) then
                  alpha2 = 1.0d0/rc2v(iq)
                  alpha  = sqrt( alpha2 )   
                  coeff  = 2.0d0*alpha/piroot
                  factor = derf(alpha*r1)/r1 - coeff*exp(-alpha2*r2)
               elseif( 0.0d0 <= rc2v(iq) .and. rc2v(iq) < thresh ) then
                  factor = 1.0d0 / r1
               else
                  write(6,'(" rc2v is negative value. stop ")')
                  stop
               endif  
               gnr(1,iq) = gnr(1,iq) - chg(ip) * chrg_gc(iq) *
     1              wk1/r2 * factor
               gnr(2,iq) = gnr(2,iq) - chg(ip) * chrg_gc(iq) *
     1              wk2/r2 * factor
               gnr(3,iq) = gnr(3,iq) - chg(ip) * chrg_gc(iq) *
     1              wk3/r2 * factor
            else
            endif

 120     continue
 130  continue

      if (ldebug.eq.2) then
         write(6,*)" "
         write(6,*)"nuclear-gauss repulsion energy"
         do j=1,ngc
            write(6,'(i3,3f19.10)')j,(gnr(i,j),i=1,3)
         enddo
      endif

      return
      end
CC-------------------------------------------------------------------
CC
CC-------------------------------------------------------------------
      subroutine gconrg_gc_gc (
     $     gnr,chrg_gc, xyz_gc, rc2v, ngc,
     $     lprint,ldebug )
 
      implicit real(8) (a-h,o-z)
      dimension gnr(3,ngc)
      dimension xyz_gc(3,ngc),chrg_gc(ngc)
      dimension rc2v(ngc)
c
      data underf / 1.0d-40 /
      thresh = 1.0d-6
      pi = 3.14159265358979323846d+00
      piroot = sqrt( pi )
c
c
c --- array initialize ....
      do 110 jn = 1 , ngc
         do 100 in = 1 , 3
            gnr(in,jn) = 0.d+00
 100     continue
 110  continue

      do 130 iq = 1 , ngc
         do 120 ip = 1 , ngc
            if (iq .ne. ip) then
               wk1       =     xyz_gc(1,iq)-xyz_gc(1,ip)
               wk2       =     xyz_gc(2,iq)-xyz_gc(2,ip)
               wk3       =     xyz_gc(3,iq)-xyz_gc(3,ip)
               if ( dabs(wk1) .lt. underf ) wk1 = 0.0d+00
               if ( dabs(wk2) .lt. underf ) wk2 = 0.0d+00
               if ( dabs(wk3) .lt. underf ) wk3 = 0.0d+00
               r2        =     wk1**2 + wk2**2 + wk3**2
               r1        = dsqrt(r2)
               if ( r1 > thresh ) then
                  r3        = r1 ** 3

                  if( rc2v(ip) < 0.0d0 .or. rc2v(iq) < 0.0d0 ) then
                     write(6,'(" rc2v must not to be negative. ")') 
                     stop 
                  endif
                  factor_ip = 1.0d0
                  factor_iq = 1.0d0
                  erf_ip = 1.0d0
                  erf_iq = 1.0d0
                  if( rc2v(ip) >= thresh ) then
                     alpha2 = 1.0d0/rc2v(ip)
                     alpha  = sqrt( alpha2 )   
                     coeff  = 2.0d0*alpha/piroot
                     factor_ip = coeff*exp(-alpha2*r2)
                     erf_ip    = derf(alpha*r1)
                  elseif(0.0d0 <= rc2v(ip) .and. rc2v(ip) < thresh)then
                     factor_ip = 0.0d0
                     erf_ip    = 1.0d0
                  endif  
                  if( rc2v(iq) >= thresh ) then
                     alpha2 = 1.0d0/rc2v(iq)
                     alpha  = sqrt( alpha2 )   
                     coeff  = 2.0d0*alpha/piroot
                     factor_iq = coeff*exp(-alpha2*r2)
                     erf_iq    = derf(alpha*r1)
                  elseif(0.0d0 <= rc2v(iq) .and. rc2v(iq) < thresh)then
                     factor_iq = 0.0d0
                     erf_iq    = 1.0d0
                  endif  
                  factor = ( erf_ip*erf_iq/r1
     $                 - erf_iq*factor_iq - erf_ip*factor_ip )
                  gnr(1,iq) = gnr(1,iq) - chrg_gc(iq) * chrg_gc(ip) *
     1                 wk1/r2 * factor
                  gnr(2,iq) = gnr(2,iq) - chrg_gc(iq) * chrg_gc(ip) *
     1                 wk2/r2 * factor
                  gnr(3,iq) = gnr(3,iq) - chrg_gc(iq) * chrg_gc(ip) *
     1                 wk3/r2 * factor
               else
               endif
            else
            endif
 120     continue
 130  continue

      if (ldebug.eq.2) then
         write(6,*)" "
         write(6,*)"nuclear repulsion energy"
         do j=1,ngc
            write(6,'(i3,3f19.10)')j,(gnr(i,j),i=1,3)
         enddo
      endif

      return
      end
CC-------------------------------------------------------------------
CC
CC-------------------------------------------------------------------
      subroutine gconrg_nuc_fsw (
     $     gnr,chg,cord,natom,
     $     xyz_fsw, chrg_fsw, rc2v_fsw, nfsw,
     $     lprint,ldebug )

      implicit real(8) (a-h,o-z)
      dimension gnr(3,natom),chg(natom),cord(3,natom)  ! Gauss Charge
      dimension xyz_fsw(3,nfsw),chrg_fsw(nfsw)             ! QM Nucl.
      dimension rc2v_fsw(nfsw)
c
      data underf / 1.0d-40 /
      thresh = 1.0d-6
      pi = 3.14159265358979323846d+00
      piroot = sqrt( pi )
c
c
c --- array initialize ....
      do 110 jn = 1 , natom
        do 100 in = 1 , 3
          gnr(in,jn) = 0.d+00
  100   continue
  110 continue

!!!   write(6,'(" natom nfsw = ",2i5)') natom, nfsw
      do 130 iq = 1 , natom               ! GaussCharge loop
      ! write(6,'(" rc2v = ",f12.6)') rc2v_fsw(iq)
         do 120 ip = 1 , nfsw    ! QM Nucl. loop

          wk1 = cord(1,iq)-xyz_fsw(1,ip)
          wk2 = cord(2,iq)-xyz_fsw(2,ip)
          wk3 = cord(3,iq)-xyz_fsw(3,ip)

          if ( dabs(wk1) .lt. underf ) wk1 = 0.0d+00
          if ( dabs(wk2) .lt. underf ) wk2 = 0.0d+00
          if ( dabs(wk3) .lt. underf ) wk3 = 0.0d+00

          r2  = wk1**2 + wk2**2 + wk3**2
          r1  = dsqrt(r2)

          if ( r1 > thresh ) then
            if( rc2v_fsw(ip) >= thresh ) then
               alpha2 = 1.0d0/rc2v_fsw(ip)
               alpha  = sqrt( alpha2 )   
               coeff  = 2.0d0*alpha/piroot
               factor = derfc(alpha*r1)/r1 - coeff*exp(-alpha2*r2)
            elseif( rc2v_fsw(ip) < thresh .and. 
     1              rc2v_fsw(ip) >= 0.0d0         ) then
              factor = 1.0d0 / r1 
            elseif( rc2v_fsw(ip) < 0 ) then
              write(6,'(" rc2v is negative value. stop : ", f12.8, i6)')
     $              rc2v_fsw(ip), ip
              stop
            endif  
          gnr(1,iq) = gnr(1,iq) - chg(iq) * chrg_fsw(ip) *
     1                wk1/r2 * factor                     
          gnr(2,iq) = gnr(2,iq) - chg(iq) * chrg_fsw(ip) *
     1                wk2 /r2 * factor 
          gnr(3,iq) = gnr(3,iq) - chg(iq) * chrg_fsw(ip) *
     1                wk3 /r2 * factor
          else
          endif

  120   continue
  130 continue

      if (ldebug.eq.2) then
         write(6,*)" "
         write(6,*)"nuclear-gauss repulsion energy"
         do j=1,natom
            write(6,'(i3,3f19.10)')j,(gnr(i,j),i=1,3)
         enddo
      endif

      return
      end
CC-------------------------------------------------------------------
CC
CC-------------------------------------------------------------------
      subroutine gconrg_fsw_nuc (
     $     gnr,chrg_fsw, xyz_fsw, rc2v_fsw, nfsw,
     $     cord,chg,natom,
     $     lprint,ldebug )

      implicit real(8) (a-h,o-z)
      dimension gnr(3,nfsw),chg(natom),cord(3,natom)
      dimension xyz_fsw(3,nfsw),chrg_fsw(nfsw)
      dimension rc2v_fsw(nfsw)
c
      data underf / 1.0d-40 /
      thresh = 1.0d-6
      pi = 3.14159265358979323846d+00
      piroot = sqrt( pi )
c
c
c --- array initialize ....
      do 110 jn = 1 , nfsw
        do 100 in = 1 , 3
          gnr(in,jn) = 0.d+00
  100   continue
  110 continue

c$$$      write(6,'(" natom nfsw = ",2i5)') natom, nfsw
      do 130 iq = 1 , nfsw
         do 120 ip = 1 , natom

            wk1 = xyz_fsw(1,iq)-cord(1,ip)
            wk2 = xyz_fsw(2,iq)-cord(2,ip)
            wk3 = xyz_fsw(3,iq)-cord(3,ip)

            if ( dabs(wk1) .lt. underf ) wk1 = 0.0d+00
            if ( dabs(wk2) .lt. underf ) wk2 = 0.0d+00
            if ( dabs(wk3) .lt. underf ) wk3 = 0.0d+00

            r2  = wk1**2 + wk2**2 + wk3**2
            r1  = dsqrt(r2)

            if ( r1 > thresh ) then
               r3  = r1 * r2

               if( rc2v_fsw(iq) >= thresh ) then
                  alpha2 = 1.0d0/rc2v_fsw(iq)
                  alpha  = sqrt( alpha2 )   
                  coeff  = 2.0d0*alpha/piroot
                  factor = derfc(alpha*r1)/r1 - coeff*exp(-alpha2*r2)
               elseif( 0.0d0 <= rc2v_fsw(iq)
     $                 .and. rc2v_fsw(iq) < thresh ) then
                  factor = 1.0d0 / r1
               else
                  write(6,'(" rc2v is negative value. stop ")')
                  stop
               endif  
               gnr(1,iq) = gnr(1,iq) - chg(ip) * chrg_fsw(iq) *
     1              wk1/r2 * factor
               gnr(2,iq) = gnr(2,iq) - chg(ip) * chrg_fsw(iq) *
     1              wk2/r2 * factor
               gnr(3,iq) = gnr(3,iq) - chg(ip) * chrg_fsw(iq) *
     1              wk3/r2 * factor
            else
            endif

 120     continue
 130  continue

      if (ldebug.eq.2) then
         write(6,*)" "
         write(6,*)"nuclear-gauss repulsion energy"
         do j=1,nfsw
            write(6,'(i3,3f19.10)')j,(gnr(i,j),i=1,3)
         enddo
      endif

      return
      end
CC-------------------------------------------------------------------
CC
CC-------------------------------------------------------------------
      subroutine gconrg_fsw_fsw (
     $     gnr,chrg_fsw, xyz_fsw, rc2v_fsw, nfsw,
     $     lprint,ldebug )
 
      implicit real(8) (a-h,o-z)
      dimension gnr(3,nfsw)
      dimension xyz_fsw(3,nfsw),chrg_fsw(nfsw)
      dimension rc2v_fsw(nfsw)
c
      data underf / 1.0d-40 /
      thresh = 1.0d-6
      pi = 3.14159265358979323846d+00
      piroot = sqrt( pi )
c
c
c --- array initialize ....
      do 110 jn = 1 , nfsw
         do 100 in = 1 , 3
            gnr(in,jn) = 0.d+00
 100     continue
 110  continue

      do 130 iq = 1 , nfsw
         do 120 ip = 1 , nfsw
            if (iq .ne. ip) then
               wk1       =     xyz_fsw(1,iq)-xyz_fsw(1,ip)
               wk2       =     xyz_fsw(2,iq)-xyz_fsw(2,ip)
               wk3       =     xyz_fsw(3,iq)-xyz_fsw(3,ip)
               if ( dabs(wk1) .lt. underf ) wk1 = 0.0d+00
               if ( dabs(wk2) .lt. underf ) wk2 = 0.0d+00
               if ( dabs(wk3) .lt. underf ) wk3 = 0.0d+00
               r2        =     wk1**2 + wk2**2 + wk3**2
               r1        = dsqrt(r2)
               if ( r1 > thresh ) then
                  r3        = r1 ** 3

                  if( rc2v_fsw(ip) < 0.0d0
     $                 .or. rc2v_fsw(iq) < 0.0d0 ) then
                     write(6,'(" rc2v must not to be negative. ")') 
                     stop 
                  endif
                  factor_ip = 1.0d0
                  factor_iq = 1.0d0
                  erfc_ip = 1.0d0
                  erfc_iq = 1.0d0
                  if( rc2v_fsw(ip) >= thresh ) then
                     alpha2 = 1.0d0/rc2v_fsw(ip)
                     alpha  = sqrt( alpha2 )   
                     coeff  = 2.0d0*alpha/piroot
                     factor_ip = coeff*exp(-alpha2*r2)
                     erfc_ip   = derfc(alpha*r1)
                  elseif(0.0d0 <= rc2v_fsw(ip)
     $                    .and. rc2v_fsw(ip) < thresh)then
                     factor_ip = 0.0d0
                     erfc_ip   = 1.0d0
                  endif  
                  if( rc2v_fsw(iq) >= thresh ) then
                     alpha2 = 1.0d0/rc2v_fsw(iq)
                     alpha  = sqrt( alpha2 )   
                     coeff  = 2.0d0*alpha/piroot
                     factor_iq = coeff*exp(-alpha2*r2)
                     erfc_iq   = derfc(alpha*r1)
                  elseif(0.0d0 <= rc2v_fsw(iq)
     $                    .and. rc2v_fsw(iq) < thresh)then
                     factor_iq = 0.0d0
                     erfc_iq   = 1.0d0
                  endif  
                  factor = ( erfc_ip*erfc_iq/r1
     $                 - erfc_iq*factor_iq - erfc_ip*factor_ip )
                  gnr(1,iq) = gnr(1,iq) - chrg_fsw(iq) * chrg_fsw(ip) *
     1                 wk1/r2 * factor
                  gnr(2,iq) = gnr(2,iq) - chrg_fsw(iq) * chrg_fsw(ip) *
     1                 wk2/r2 * factor
                  gnr(3,iq) = gnr(3,iq) - chrg_fsw(iq) * chrg_fsw(ip) *
     1                 wk3/r2 * factor
               else
               endif
            else
            endif
 120     continue
 130  continue

      if (ldebug.eq.2) then
         write(6,*)" "
         write(6,*)"nuclear repulsion energy"
         do j=1,nfsw
            write(6,'(i3,3f19.10)')j,(gnr(i,j),i=1,3)
         enddo
      endif

      return
      end
c$$$CC-------------------------------------------------------------------
c$$$CC
c$$$CC-------------------------------------------------------------------
c$$$      subroutine gconrg_zd1(
c$$$     $     g, xyz_atom, chg_atom, natom, rcut,
c$$$     $     xyz_pc, chg_pc, npc,
c$$$     $     ldebug)
c$$$ 
c$$$      implicit real(8) (a-h,o-z)
c$$$      dimension g(3,natom),chg_pc(npc),xyz_pc(3,npc)
c$$$      dimension xyz_atom(3,natom),chg_atom(natom)
c$$$c
c$$$      data underf / 1.0d-40 /
c$$$
c$$$      logical,allocatable::lskip(:)
c$$$      logical::flag
c$$$
c$$$      allocate(lskip(npc))
c$$$
c$$$      thresh = 1.0d-6
c$$$c
c$$$c
c$$$c --- array initialize ....
c$$$      do jn = 1 , natom
c$$$        do  in = 1 , 3
c$$$          g(in,jn) = 0.d+00
c$$$       end do
c$$$      end do
c$$$
c$$$      rcut2 = rcut**2
c$$$      rcut3 = rcut**3
c$$$
c$$$      do ip = 1 , natom
c$$$         do iq = 1 , npc
c$$$            x = xyz_pc(1,iq) - xyz_atom(1,ip)
c$$$            y = xyz_pc(2,iq) - xyz_atom(2,ip)
c$$$            z = xyz_pc(3,iq) - xyz_atom(3,ip)
c$$$            if ( dabs(x) .lt. underf ) x = 0.0d+00
c$$$            if ( dabs(y) .lt. underf ) y = 0.0d+00
c$$$            if ( dabs(z) .lt. underf ) z = 0.0d+00
c$$$            r2 = x*x + y*y + z*z
c$$$            r1 = dsqrt(r2)
c$$$            if ( r1 >= rcut ) cycle!!!!!
c$$$            r3 = r1 * r2
c$$$            if ( r1 > thresh ) then
c$$$               if (rcut > 0.0d0) then
c$$$                  v = chg_pc(iq)*chg_atom(ip) * (1.0d0/r3 - 1.0d0/rcut3)
c$$$               else
c$$$                  v = chg_pc(iq)*chg_atom(ip) / r3
c$$$               end if
c$$$               g(1,ip) = g(1,ip) + v * x
c$$$               g(2,ip) = g(2,ip) + v * y
c$$$               g(3,ip) = g(3,ip) + v * z
c$$$            endif
c$$$         end do
c$$$      end do
c$$$
c$$$      if (ldebug.eq.2) then
c$$$         do j = 1, npc
c$$$            write(6,'(i3,3f19.10)') j,(g(i,j),i=1,3)
c$$$         enddo
c$$$      endif
c$$$
c$$$      return
c$$$      end
c$$$CC-------------------------------------------------------------------
c$$$CC
c$$$CC-------------------------------------------------------------------
c$$$      subroutine gconrg_zd3(
c$$$     $     g, xyz_atom, chg_atom, natom, rcut,
c$$$     $     xyz_pc, chg_pc, npc,
c$$$     $     ldebug)
c$$$ 
c$$$      implicit real(8) (a-h,o-z)
c$$$      dimension g(3,natom),chg_pc(npc),xyz_pc(3,npc)
c$$$      dimension xyz_atom(3,natom),chg_atom(natom)
c$$$c
c$$$      data underf / 1.0d-40 /
c$$$
c$$$      logical,allocatable::lskip(:)
c$$$      logical::flag
c$$$
c$$$      allocate(lskip(npc))
c$$$
c$$$      thresh = 1.0d-6
c$$$c
c$$$c
c$$$c --- array initialize ....
c$$$      do jn = 1 , natom
c$$$        do  in = 1 , 3
c$$$          g(in,jn) = 0.d+00
c$$$       end do
c$$$      end do
c$$$
c$$$      rcut2 = rcut**2
c$$$      rcut3 = rcut**3
c$$$
c$$$      lskip(:) = .true.
c$$$      do k = 1, npc
c$$$         Cx = xyz_pc(1,k)
c$$$         Cy = xyz_pc(2,k)
c$$$         Cz = xyz_pc(3,k)
c$$$         flag = .false.
c$$$         do iatom = 1, natom
c$$$            Ax = xyz_atom(1,iatom)
c$$$            Ay = xyz_atom(2,iatom)
c$$$            Az = xyz_atom(3,iatom)
c$$$            ACx = Ax - Cx
c$$$            ACy = Ay - Cy
c$$$            ACz = Az - Cz
c$$$            AC2 = ACx*ACx + ACy*ACy + ACz*ACz
c$$$            if (AC2 < rcut2) then
c$$$               flag = .true.
c$$$               go to 1
c$$$            end if
c$$$         end do
c$$$ 1       if (flag) lskip(k) = .false.
c$$$      end do
c$$$
c$$$      do ip = 1 , natom
c$$$         do iq = 1 , npc
c$$$            if (lskip(iq)) cycle
c$$$            x = xyz_pc(1,iq) - xyz_atom(1,ip)
c$$$            y = xyz_pc(2,iq) - xyz_atom(2,ip)
c$$$            z = xyz_pc(3,iq) - xyz_atom(3,ip)
c$$$            if ( dabs(x) .lt. underf ) x = 0.0d+00
c$$$            if ( dabs(y) .lt. underf ) y = 0.0d+00
c$$$            if ( dabs(z) .lt. underf ) z = 0.0d+00
c$$$            r2 = x*x + y*y + z*z
c$$$            r1 = dsqrt(r2)
c$$$            r3 = r1 * r2
c$$$            if ( r1 > thresh ) then
c$$$               if (rcut > 0.0d0) then
c$$$                  v = chg_pc(iq)*chg_atom(ip) * (1.0d0/r3 - 1.0d0/rcut3)
c$$$               else
c$$$                  v = chg_pc(iq)*chg_atom(ip) / r3
c$$$               end if
c$$$               g(1,ip) = g(1,ip) + v * x
c$$$               g(2,ip) = g(2,ip) + v * y
c$$$               g(3,ip) = g(3,ip) + v * z
c$$$            endif
c$$$         end do
c$$$      end do
c$$$
c$$$      if (ldebug.eq.2) then
c$$$         do j = 1, npc
c$$$            write(6,'(i3,3f19.10)') j,(g(i,j),i=1,3)
c$$$         enddo
c$$$      endif
c$$$
c$$$      return
c$$$      end
c$$$CC-------------------------------------------------------------------
c$$$CC
c$$$CC-------------------------------------------------------------------
c$$$      subroutine gconrg_on_zd1(
c$$$     $     g, xyz_pc, chg_pc, npc, rcut,
c$$$     $     xyz_atom, chg_atom, natom,
c$$$     $     ldebug)
c$$$ 
c$$$      implicit real(8) (a-h,o-z)
c$$$      dimension g(3,npc),chg_pc(npc),xyz_pc(3,npc)
c$$$      dimension xyz_atom(3,natom),chg_atom(natom)
c$$$c
c$$$      data underf / 1.0d-40 /
c$$$
c$$$      logical,allocatable::lskip(:)
c$$$      logical::flag
c$$$
c$$$      allocate(lskip(npc))
c$$$
c$$$      thresh = 1.0d-6
c$$$c
c$$$c
c$$$c --- array initialize ....
c$$$      do jn = 1 , npc
c$$$        do  in = 1 , 3
c$$$          g(in,jn) = 0.d+00
c$$$       end do
c$$$      end do
c$$$
c$$$      rcut2 = rcut**2
c$$$      rcut3 = rcut**3
c$$$
c$$$      do ip = 1 , natom
c$$$         do iq = 1 , npc
c$$$            x = xyz_atom(1,ip) - xyz_pc(1,iq)
c$$$            y = xyz_atom(2,ip) - xyz_pc(2,iq)
c$$$            z = xyz_atom(3,ip) - xyz_pc(3,iq)
c$$$            if ( dabs(x) .lt. underf ) x = 0.0d+00
c$$$            if ( dabs(y) .lt. underf ) y = 0.0d+00
c$$$            if ( dabs(z) .lt. underf ) z = 0.0d+00
c$$$            r2 = x*x + y*y + z*z
c$$$            r1 = dsqrt(r2)
c$$$            r3 = r1 * r2
c$$$            if ( r1 >= rcut ) cycle!!!!!
c$$$            if ( r1 > thresh ) then
c$$$               if (rcut > 0.0d0) then
c$$$                  v = chg_pc(iq)*chg_atom(ip) * (1.0d0/r3 - 1.0d0/rcut3)
c$$$               else
c$$$                  v = chg_pc(iq)*chg_atom(ip) / r3
c$$$               end if
c$$$               g(1,iq) = g(1,iq) + v * x
c$$$               g(2,iq) = g(2,iq) + v * y
c$$$               g(3,iq) = g(3,iq) + v * z
c$$$            endif
c$$$         end do
c$$$      end do
c$$$
c$$$      if (ldebug.eq.2) then
c$$$         do j = 1, npc
c$$$            write(6,'(i3,3f19.10)') j,(g(i,j),i=1,3)
c$$$         enddo
c$$$      endif
c$$$
c$$$      return
c$$$      end
c$$$CC-------------------------------------------------------------------
c$$$CC
c$$$CC-------------------------------------------------------------------
c$$$      subroutine gconrg_on_zd3(
c$$$     $     g, xyz_pc, chg_pc, npc, rcut,
c$$$     $     xyz_atom, chg_atom, natom,
c$$$     $     ldebug)
c$$$ 
c$$$      implicit real(8) (a-h,o-z)
c$$$      dimension g(3,npc),chg_pc(npc),xyz_pc(3,npc)
c$$$      dimension xyz_atom(3,natom),chg_atom(natom)
c$$$c
c$$$      data underf / 1.0d-40 /
c$$$
c$$$      logical,allocatable::lskip(:)
c$$$      logical::flag
c$$$
c$$$      allocate(lskip(npc))
c$$$
c$$$      thresh = 1.0d-6
c$$$c
c$$$c
c$$$c --- array initialize ....
c$$$      do jn = 1 , npc
c$$$        do  in = 1 , 3
c$$$          g(in,jn) = 0.d+00
c$$$       end do
c$$$      end do
c$$$
c$$$      rcut2 = rcut**2
c$$$      rcut3 = rcut**3
c$$$
c$$$      lskip(:) = .true.
c$$$      do k = 1, npc
c$$$         Cx = xyz_pc(1,k)
c$$$         Cy = xyz_pc(2,k)
c$$$         Cz = xyz_pc(3,k)
c$$$         flag = .false.
c$$$         do iatom = 1, natom
c$$$            Ax = xyz_atom(1,iatom)
c$$$            Ay = xyz_atom(2,iatom)
c$$$            Az = xyz_atom(3,iatom)
c$$$            ACx = Ax - Cx
c$$$            ACy = Ay - Cy
c$$$            ACz = Az - Cz
c$$$            AC2 = ACx*ACx + ACy*ACy + ACz*ACz
c$$$            if (AC2 < rcut2) then
c$$$               flag = .true.
c$$$               go to 1
c$$$            end if
c$$$         end do
c$$$ 1       if (flag) lskip(k) = .false.
c$$$      end do
c$$$
c$$$      do ip = 1 , natom
c$$$         do iq = 1 , npc
c$$$            if (lskip(iq)) cycle
c$$$            x = xyz_atom(1,ip) - xyz_pc(1,iq)
c$$$            y = xyz_atom(2,ip) - xyz_pc(2,iq)
c$$$            z = xyz_atom(3,ip) - xyz_pc(3,iq)
c$$$            if ( dabs(x) .lt. underf ) x = 0.0d+00
c$$$            if ( dabs(y) .lt. underf ) y = 0.0d+00
c$$$            if ( dabs(z) .lt. underf ) z = 0.0d+00
c$$$            r2 = x*x + y*y + z*z
c$$$            r1 = dsqrt(r2)
c$$$            r3 = r1 * r2
c$$$            if ( r1 > thresh ) then
c$$$               if (rcut > 0.0d0) then
c$$$                  v = chg_pc(iq)*chg_atom(ip) * (1.0d0/r3 - 1.0d0/rcut3)
c$$$               else
c$$$                  v = chg_pc(iq)*chg_atom(ip) / r3
c$$$               end if
c$$$               g(1,iq) = g(1,iq) + v * x
c$$$               g(2,iq) = g(2,iq) + v * y
c$$$               g(3,iq) = g(3,iq) + v * z
c$$$            endif
c$$$         end do
c$$$      end do
c$$$
c$$$      if (ldebug.eq.2) then
c$$$         do j = 1, npc
c$$$            write(6,'(i3,3f19.10)') j,(g(i,j),i=1,3)
c$$$         enddo
c$$$      endif
c$$$
c$$$      return
c$$$      end
