! -*- coding: utf-8 -*-
!
! Dual licensed under the MIT or GPL Version 3 license.
! See 'LICENSE.MIT' for the definition of the MIT license,
! and 'COPYING' for the definition of the GPL license.
!
!> Initial MO
!!
module initmo_class
  implicit none
  private

  public::initmo_gbe
  public::initmo_hcore
  public::initmo_mix4uhf
  public::initmo_proj

contains
  !-----------------------------------------------------------------------------
  !> GBE
  subroutine initmo_gbe(c, oe, lao, nao, nmo, s, h, lsh, ldebug, istat)
    integer,intent(in)::lao, nao, lsh
    integer,intent(out)::nmo
    real(8),intent(inout)::c(lao,nao)
    real(8),intent(inout)::oe(nao)
    real(8),intent(in)::s(lsh,nao)
    real(8),intent(in)::h(lsh,nao)
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    real(8),allocatable::f(:,:)
    real(8),allocatable::t(:,:)

    nmo = nao

    allocate(f(nao,nao), t(nao,nao), stat=istat)
    if (istat /= 0) go to 9

    call fock_matrix_gbe(f, s, h, lao, nao, ldebug, istat)
    if (istat /= 0) go to 9

    call rhf_t_matrix(t, s, nao, ldebug, istat)
    if (istat /= 0) go to 9

    call rhf_new_mo(c, oe, f, t, nao, nmo, ldebug, istat)
    if (istat /= 0) go to 9

    deallocate(f, t, stat=istat)
    if (istat /= 0) go to 9

9   return
  end subroutine initmo_gbe

  !-----------------------------------------------------------------------------
  !> Hcore
  subroutine initmo_hcore(c, oe, lao, nao, nmo, s, h, lsh, ldebug, istat)
    integer,intent(in)::lao, nao, lsh
    integer,intent(out)::nmo
    real(8),intent(inout)::c(lao,nao)
    real(8),intent(inout)::oe(nao)
    real(8),intent(in)::s(lsh,nao)
    real(8),intent(in)::h(lsh,nao)
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    real(8),allocatable::f(:,:)
    real(8),allocatable::t(:,:)

    nmo = nao

    allocate(f(nao,nao), t(nao,nao), stat=istat)
    if (istat /= 0) go to 9

    f(1:nao,1:nao) = h(1:nao,1:nao)

    call rhf_t_matrix(t, s, nao, ldebug, istat)
    if (istat /= 0) go to 9

    call rhf_new_mo(c, oe, f, t, nao, nmo, ldebug, istat)
    if (istat /= 0) go to 9

    deallocate(f, t, stat=istat)
    if (istat /= 0) go to 9

9   return
  end subroutine initmo_hcore

  !-----------------------------------------------------------------------------
  subroutine fock_matrix_gbe(f, s, h, lao, nao, ldebug, istat)
    integer,intent(in)::lao, nao
    real(8),intent(inout)::f(lao,nao)
    real(8),intent(in)::s(lao,nao)
    real(8),intent(in)::h(lao,nao)
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    real(8),allocatable::wm1(:,:)
    real(8),allocatable::wm2(:,:)
    real(8)::afac, bfac, cfac, fd
    integer::i, j, k

    allocate(wm1(lao,nao), wm2(lao,nao), stat=istat)
    if (istat /= 0) go to 9

    afac = 1.0d0
    f(:,:) = 0.0d0
    do i = 1, nao
!!$       f(i,i) = h(i,i)
       fd = dabs(h(i,i))**afac
       if (h(i,i) < 0.0d0) then
          f(i,i) = -fd
       else
          f(i,i) = fd
       end if
    end do
!!$    call dsymm( 'L', 'U', nao, nao, 1.0d0, s, lao, f, lao, 0.0d0, wm1, lao )
    wm1(:,:) = 0.0d0
    do j = 1, nao
       do i = 1, nao
          wm1(i,j) = wm1(i,j) + s(i,j)*f(j,j)
       end do
    end do
    call dsymm( 'R', 'U', nao, nao, 1.0d0, s, lao, wm1, lao, 0.0d0, f, lao )

    bfac = 1.1d0
    do j = 1, nao - 1
       do i = j + 1, nao
          f(i,j) = f(i,j)*bfac
          f(j,i) = f(j,i)*bfac
       end do
    end do

    cfac = -0.004d0
    do j = 1, nao
       do i = 1, nao
          f(i,j) = f(i,j) + h(i,j)*cfac
       end do
    end do

    if (ldebug >= 2) then
       call ummtw('GBE Fock matrix', f, nao, nao, nao, 10, 6, istat)
    end if

    deallocate(wm1, wm2, stat=istat)
    if (istat /= 0) go to 9

9   return
  end subroutine fock_matrix_gbe

  !-----------------------------------------------------------------------------
  subroutine initmo_mix4uhf(ca, cb, lao, nao, nmo, nclosed)
    integer,intent(in)::lao, nao, nmo, nclosed
    real(8),intent(inout)::ca(lao,nmo)
    real(8),intent(inout)::cb(lao,nmo)

    real(8)::x
    integer::iao

    x = 1.0d0 / dsqrt(2.0d0)

    cb(1:nao,1:nmo) = ca(1:nao,1:nmo)
    do iao = 1, nao
       ca(iao,nclosed) = x * ( ca(iao,nclosed) + ca(iao,nclosed+1) )
       cb(iao,nclosed) = x * ( cb(iao,nclosed) - cb(iao,nclosed+1) )
    end do

  end subroutine initmo_mix4uhf

  !-----------------------------------------------------------------------------
  subroutine initmo_proj(mo2, mo1, s21, s22, nao2, nao1, ldebug, istat)
    use mo_data_class, only: mo_data_type
    type(mo_data_type),intent(inout)::mo2
    type(mo_data_type),intent(in)::mo1
    integer,intent(in)::nao2, nao1
    real(8),intent(in)::s21(nao2,nao1)
    real(8),intent(in)::s22(nao2,nao2)
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    call dgemm('n', 'n', nao2, nao1, nao1,&
         1.0d0, s21, nao2, mo1%c, mo1%lao, 0.0d0, mo2%c, mo2%lao)
    if (ldebug > 2) call ummtw('C1', mo2%c, mo2%lao, nao2, nao1, 8, 6, istat)

    call orthogonalize_mo(mo2%c, s22, nao2, nao1, ldebug, istat)
    if (ldebug > 2) call ummtw('C2', mo2%c, mo2%lao, nao2, nao1, 8, 6, istat)

  end subroutine initmo_proj

  !-----------------------------------------------------------------------------
end module initmo_class
