! -*- coding: utf-8 -*-
!
! Dual licensed under the MIT or GPL Version 3 license.
! See 'LICENSE.MIT' for the definition of the MIT license,
! and 'COPYING' for the definition of the GPL license.
!
!> 入力データクラス
!!
module input_class
  use error_class
  use pl_str_utils_class
  implicit none
!!$  private

  public::get_options
  public::get_data_group_position
  public::parse_options

  character(len=*),private,parameter::c_data_group_prefix = '&'
  character(len=*),private,parameter::c_data_group_end = 'end' ! lower
  character(len=*),private,parameter::c_data_group_close&
                                        = c_data_group_prefix // 'END'
  character(len=*),private,parameter::c_comment = ';#!'
  character(len=*),private,parameter::c_space = ' '
  character(len=*),private,parameter::c_tab = achar(9)
  character(len=*),private,parameter::c_blank = c_space // c_tab
  character(len=*),private,parameter::c_assignment_op = '='
  character(len=*),private,parameter::c_quote = '"'
  character(len=*),private,parameter::c_paren_open = '('
  character(len=*),private,parameter::c_paren_close = ')'
  character(len=*),private,parameter::c_paren = c_paren_open // c_paren_close
  character(len=*),private,parameter::c_lower = 'abcdefghijklmnopqrstuvwxyz'
  character(len=*),private,parameter::c_upper = 'ABCDEFGHIJKLMNOPQRSTUVWXYZ'
  character(len=*),private,parameter::c_alpha = c_lower // c_upper
  character(len=*),private,parameter::c_digit = '1234567890'
  character(len=*),private,parameter::c_alnum = c_alpha // c_digit
  !character(len=*),private,parameter::c_comma = ','
  character(len=*),private,parameter::c_minus = '-'
  !character(len=*),private,parameter::c_plus = '+'
  !character(len=*),private,parameter::c_sign = c_minus // c_plus
  character(len=*),private,parameter::c_underscore = '_'
  character(len=*),private,parameter::c_word = c_alnum // c_underscore

  integer,public,parameter::MAX_INPUT_LINE_LEN = 256
  integer,public,parameter::MAX_GROUP_NAME_LEN = 32
  integer,public,parameter::MAX_OPTION_NAME_LEN = 32
  integer,public,parameter::MAX_OPTION_VALUE_LEN = 256

  character(len=*),parameter::PL_QM_INPUT_FILE_NAME = 'qm.inp'

contains
  !-----------------------------------------------------------------------------
  !> @brief ファイルの行数の取得
  !> 
  subroutine read_file_n(file_name, nline, istat)
    character(len=*),intent(in)::file_name !< ファイルの名称
    integer,intent(out)::nline !< 行数
    integer,intent(out)::istat

    character(len=MAX_INPUT_LINE_LEN)::line

    istat = 0
    open(1, file=file_name, status='old', err=101)
    rewind(1, err=102)
    nline = 0
1   read(1, '(a)', end=2, err=103) line
    nline = nline + 1
    go to 1
2   close(1, err=104)

    return
101 istat = PL_ERR_FILE_OPEN_ERROR
    go to 199
102 istat = PL_ERR_FILE_REWIND_ERROR
    go to 199
103 istat = PL_ERR_FILE_READ_ERROR
    go to 199
104 istat = PL_ERR_FILE_CLOSE_ERROR
    go to 199
199 call pl_error_print_message(istat)
    call pl_error_print_file(file_name)
    return
  end subroutine read_file_n

  !-----------------------------------------------------------------------------
  !> @brief ファイルの内容の取得
  !> 
  subroutine read_file(file_name, lines, mxline, istat)
    character(len=*),intent(in)::file_name !< ファイルの名称
    integer,intent(in)::mxline !< 行数
    character(len=*),intent(inout)::lines(mxline)
    integer,intent(out)::istat

    integer::nline
    character(len=MAX_INPUT_LINE_LEN)::line

    istat = 0

    open(1, file=file_name, status='old', err=101)
    rewind(1, err=102)
    nline = 0
1   read(1, '(a)', end=2, err=103) line
    nline = nline + 1
    lines(nline) = line
    go to 1
2   close(1, err=104)

    return
101 istat = PL_ERR_FILE_OPEN_ERROR
    go to 199
102 istat = PL_ERR_FILE_REWIND_ERROR
    go to 199
103 istat = PL_ERR_FILE_READ_ERROR
    go to 199
104 istat = PL_ERR_FILE_CLOSE_ERROR
    go to 199
199 call pl_error_print_message(istat)
    call pl_error_print_file(file_name)
    return
  end subroutine read_file

  !-----------------------------------------------------------------------------
  !> @brief データグループの記述位置の取得
  !> 
  !> @verbatim
  !>    &Group   !<-- iline_gs
  !>         !^ icol_gs
  !>       &Endp !<-- iline_ge
  !>     !^ icol_ge
  !> @endverbatim
  !>
  subroutine get_data_group_position(&
       lines, nline,&
       group_name,&
       iline_gs, icol_gs, iline_ge, icol_ge, ngroup,&
       ldebug, iline_err, icol_err, istat)
    integer,intent(in)::nline !< 入力データの行数
    character(len=*),intent(in)::lines(nline) !< 入力データ
    character(len=*),intent(in)::group_name !< 対象とするデータグループの名称
    integer,intent(out)::iline_gs!< 対象データグループの開始行
    integer,intent(out)::iline_ge!< 対象データグループの終了行
    integer,intent(out)::icol_gs !< 対象データグループの開始行における開始カラム
    integer,intent(out)::icol_ge !< 対象データグループの終了行における終了カラム
    integer,intent(out)::ngroup  !< 対象データグループの出現回数
    integer,intent(in)::ldebug     !< デバッグプリントレベル
    integer,intent(out)::iline_err !< エラー行       (=0:正常, /=0:エラー)
    integer,intent(out)::icol_err  !< エラーカラム   (=0:正常, /=0:エラー)
    integer,intent(out)::istat     !< 終了ステータス (=0:正常, /=0:エラー)

    integer::iline
    integer::ncol, icol, icol_e, ipos, itoken, iline_dup, icol_dup
    logical::in_group
    logical::in_mygroup
    character(len=MAX_GROUP_NAME_LEN)::group_name_lower
    character(len=MAX_GROUP_NAME_LEN)::group_begin
    character(len=MAX_GROUP_NAME_LEN)::group_end

    istat = -1
    iline_err = 0
    icol_err  = 0

    if (pl_str_check(trim(c_word), trim(group_name)) /= 0) then
       istat = PL_ERR_INTERNAL_ERROR
       !write(6,*) 'internal error in get_data_group_position'
       go to 9
    end if

    call pl_str_tolower(group_name, group_name_lower)

    ngroup = 0
    iline_gs = 0; icol_gs = 0
    iline_ge = 0; icol_ge = 0
    in_group = .false.
    in_mygroup = .false.
    !write(6,'(2x,a,i6)') 'nline =',nline
    do iline = 1, nline
       if (ldebug > 2) write(6,'(i6,a)') iline,':' // trim(lines(iline)) // '$'

       icol   = 1
       icol_e = len_trim(lines(iline))
       do while (icol <= icol_e)

          if (in_group) then
             ! データグループをスキップ
             call skip_data_group(lines(iline), icol, icol_e, ipos, istat)
             if (istat /= 0) go to 9
          else
             ! 空白をスキップ
             call skip_space(lines(iline), icol, icol_e, ipos, istat)
             if (istat /= 0) go to 9
          end if
          !write(6,*) 'ipos=',ipos, icol_e
          if (icol_e < ipos) exit ! 最後のカラムを越えたら次の行へ

          !write(6,*) 'prefix=',lines(iline)(ipos:ipos)
          if (lines(iline)(ipos:ipos) /= c_data_group_prefix) then
             istat = PL_ERR_GROUP_BEGIN ! no prefix
             iline_err = iline; icol_err = ipos
             go to 9
          end if

          call get_token_group_name(lines(iline), ipos, icol_e, itoken, istat)
          if (ldebug > 2) write(6,*) 'group=',lines(iline)(ipos:itoken),'$'
          if (ipos == itoken) then
             istat = PL_ERR_INVALID_GROUP_NAME ! no name
             iline_err = iline; icol_err = ipos
             go to 9
          end if
          if (pl_str_check(trim(c_word), lines(iline)(ipos+1:itoken)) /= 0) then
             istat = PL_ERR_INVALID_GROUP_NAME ! invalid name
             iline_err = iline; icol_err = ipos
             go to 9
          end if
          if (in_group) then
             call pl_str_tolower(lines(iline)(ipos+1:itoken), group_end)
             if (trim(group_end) /= c_data_group_end) then
                istat = PL_ERR_GROUP_END ! no end
                iline_err = iline; icol_err = ipos
                go to 9
             end if
             if (in_mygroup) then
                in_mygroup = .false.
                iline_ge = iline
                icol_ge  = ipos - 1
             end if
             in_group = .false.
          else
             call pl_str_tolower(lines(iline)(ipos+1:itoken), group_begin)
             if (trim(group_begin) == c_data_group_end) then
                istat = PL_ERR_GROUP_BEGIN ! end
                iline_err = iline; icol_err = ipos
                go to 9
             end if
             if (trim(group_begin) == trim(group_name_lower)) then
                in_mygroup = .true.
                ngroup = ngroup + 1
                if (ngroup == 2) then
                   iline_dup = iline
                   icol_dup  = ipos
                end if
                iline_gs = iline
                icol_gs  = itoken + 1
             end if
             in_group = .true.
          end if

          icol = itoken + 1
       end do
    end do
    if (in_group) then
       istat = PL_ERR_GROUP_EOF ! no end
       iline_err = nline; icol_err = len(lines(1))
       go to 9
    end if
    if (ngroup > 1) then
       istat = PL_ERR_DUPLICATE_GROUP ! duplicate
       iline_err = iline_dup; icol_err = icol_dup
       go to 9
    end if

    if (ldebug > 0) then
       write(6,'(a,a)')     'group  = ',trim(group_name)
       write(6,'(a,i6)')    'ngroup = ',ngroup
       write(6,'(a,2i6,a)') 'start  = (',iline_gs, icol_gs,')'
       write(6,'(a,2i6,a)') 'end    = (',iline_ge, icol_ge,')'
    end if

    istat = 0
    return

9   call pl_error_print_message(istat)
    call print_error_lc(iline_err, icol_err)
    call print_error_line(lines(iline_err))
    return
  end subroutine get_data_group_position

  !-----------------------------------------------------------------------------
  !> @brief データグループ内のオプションを取得
  !>
  !>  -# option_name = value
  !>  -# option_name = ( value1, value2, ... )
  !>  -# option_name = "file name"
  !>
  subroutine parse_options(&
       lines, nline, iline_gs, icol_gs, iline_ge, icol_ge,&
       opt_names, opt_values, opt_count, nopt,&
       ldebug, iline_err, icol_err, istat)
    integer,intent(in)::nline !< 入力データの行数
    character(len=*),intent(in)::lines(nline) !< 入力データ
    integer,intent(in)::iline_gs !< 対象データグループの開始行
    integer,intent(in)::iline_ge !< 対象データグループの終了行
    integer,intent(in)::icol_gs  !< 対象データグループの開始行における開始カラム
    integer,intent(in)::icol_ge  !< 対象データグループの終了行における終了カラム
    integer,intent(in)::nopt                         !< オプション数
    character(len=*),intent(in)::opt_names(nopt)     !< オプションの名称
    character(len=*),intent(inout)::opt_values(nopt) !< オプションの値
    integer,intent(inout)::opt_count(nopt) !< オプションの出現回数
    integer,intent(in)::ldebug     !< デバッグプリントレベル
    integer,intent(out)::iline_err !< エラー行       (=0:正常, /=0:エラー)
    integer,intent(out)::icol_err  !< エラーカラム   (=0:正常, /=0:エラー)
    integer,intent(out)::istat     !< 終了ステータス (=0:正常, /=0:エラー)

    character(len=MAX_OPTION_NAME_LEN)::opt_name_lower
    character(len=MAX_OPTION_NAME_LEN),allocatable::opt_names_lower(:)
    integer::iline, iline_name, icol_name
    integer::ncol, icol, icol_s, icol_e, ipos, ipos_c
    integer::itoken, itoken_s, itoken_e, ipos_val_s, ipos_val_e
    logical::in
    integer::i, iopt, nerror

    istat = -1
    iline_err = 0
    icol_err  = 0

    allocate(opt_names_lower(nopt))
    do i = 1, nopt
       call pl_str_tolower(opt_names(i), opt_names_lower(i))
       !opt_values(i) = ' '
       !opt_count(i)  = 0
    end do

    in = .false.
    do iline = iline_gs, iline_ge
       if (ldebug > 2) write(6,'(i6,a)') iline,':' // trim(lines(iline)) // '$'

       if (iline == iline_gs) then
          icol_s = icol_gs
       else
          icol_s = 1
       end if
       if (iline == iline_ge) then
          icol_e = icol_ge
       else
          icol_e = len_trim(lines(iline))
       end if

       icol = icol_s
       !write(6,*) 'icol0=',icol
       do while (icol <= icol_e)

          call skip_space(lines(iline), icol, icol_e, ipos, istat)
          if (istat /= 0) go to 9
          !write(6,*) 'ipos1=',ipos, icol_e
          if (icol_e < ipos) exit ! next line

          call get_token_option_name(lines(iline), ipos, icol_e, itoken, istat)
          if (istat /= 0) go to 9
          !write(6,*) 'itoken1=',itoken, icol_e
          if (itoken < ipos) then
             istat = PL_ERR_INCORRECT_OPTION_SYNTAX ! '= val'
             iline_err = iline; icol_err = ipos
             go to 9
          end if
          if (ldebug > 2) write(6,*) 'opt_name:',lines(iline)(ipos:itoken),'$'
          call pl_str_tolower(lines(iline)(ipos:itoken), opt_name_lower)
          iline_name = iline; icol_name = ipos

          icol = itoken + 1
          !write(6,*) 'icol1=',icol

          call skip_space(lines(iline), icol, icol_e, ipos, istat)
          if (istat /= 0) go to 9
          !write(6,*) 'ipos2=',ipos, icol_e
          if (icol_e < ipos) then
             istat = PL_ERR_INCORRECT_OPTION_SYNTAX ! 'dir'
             iline_err = iline; icol_err = ipos
             go to 9
          end if
          if (ldebug > 2) write(6,*) 'eq:',lines(iline)(ipos:ipos),'$'
          if (lines(iline)(ipos:ipos) /= c_assignment_op) then
             istat = PL_ERR_INCORRECT_OPTION_SYNTAX ! 'dir val'
             iline_err = iline; icol_err = ipos
             go to 9
          end if
          icol = ipos + 1
          !write(6,*) 'icol2=',icol

          call skip_space(lines(iline), icol, icol_e, ipos, istat)
          if (istat /= 0) go to 9
          !write(6,*) 'ipos3=',ipos, icol_e
          if (icol_e < ipos) then
             istat = PL_ERR_INCORRECT_OPTION_SYNTAX ! 'dir ='
             iline_err = iline; icol_err = ipos
             go to 9
          end if

          if (lines(iline)(ipos:ipos) == c_paren_open) then
             call get_token_paren(lines(iline), ipos+1, icol_e, itoken, istat)
             if (istat /= 0) go to 9
             ipos_val_s = ipos+1
             ipos_val_e = itoken-1
          else if (lines(iline)(ipos:ipos) == c_quote) then
             call get_token_quote(lines(iline), ipos+1, icol_e, itoken, istat)
             if (istat /= 0) go to 9
             ipos_val_s = ipos+1
             ipos_val_e = itoken-1
          else
             call get_token_option_value(&
                  lines(iline), ipos, icol_e, itoken, istat)
             if (istat /= 0) go to 9
             ipos_val_s = ipos
             ipos_val_e = itoken
          end if
          !write(6,*) 'itoken3=',itoken, icol_e
          if (icol_e < itoken) then
             istat = PL_ERR_INCORRECT_OPTION_SYNTAX ! 'dir ='
             iline_err = iline; icol_err = ipos
             go to 9
          end if

          ! check option name
          iopt = pl_str_search_name(opt_name_lower, opt_names_lower, nopt)
          if (iopt == 0) then
             istat = PL_ERR_INVALID_OPTION_NAME
             iline_err = iline_name; icol_err = icol_name
             go to 9
          end if

          ! store option value
          if (ldebug > 2) write(6,*) 'value:',lines(iline)(ipos:itoken),'$'
          opt_values(iopt) = adjustl(lines(iline)(ipos_val_s:ipos_val_e))
          opt_count(iopt)  = opt_count(iopt) + 1

          icol = itoken + 1
          !write(6,*) 'icol3=',icol
       end do
    end do

    if (ldebug > 0) then
       do i = 1, nopt
          write(6,'(i3,'':'',a,'':'',a)')&
               opt_count(i),trim(opt_names(i)),trim(opt_values(i))
       end do
    end if

    nerror = 0
    do i = 1, nopt
       if (opt_count(i) > 1) then
          istat = PL_ERR_DUPLICATE_OPTION
          call pl_error_print_message(istat)
          call print_error_invalid_option(opt_names(i))
          nerror = nerror + 1
       end if
    end do
    if (nerror /= 0) return

    deallocate(opt_names_lower)

    istat = 0
    return

9   call pl_error_print_message(istat)
    call print_error_lc(iline_err, icol_err)
    call print_error_line(lines(iline_err))
    return
  end subroutine parse_options

  !-----------------------------------------------------------------------------
  !> @brief オプションの取得
  !>
  subroutine get_options(&
       lines, nline,&
       group_name, opt_names, opt_values, opt_count, nopt,&
       ldebug, istat)
    integer,intent(in)::nline
    character(len=*),intent(in)::lines(nline)
    character(len=*),intent(in)::group_name          !< データグループ名称
    integer,intent(in)::nopt                         !< オプション数
    character(len=*),intent(in)::opt_names(nopt)     !< オプションの名称
    character(len=*),intent(inout)::opt_values(nopt) !< オプションの値
    integer,intent(inout)::opt_count(nopt)           !< オプションの出現回数
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    integer::iline_gs  !< データグループの開始行
    integer::iline_ge  !< データグループの終了行
    integer::icol_gs   !< データグループの開始行における開始カラム
    integer::icol_ge   !< データグループの終了行における終了カラム
    integer::ngroup    !< データグループの出現回数
    integer::iline_err !< エラー行       (=0:正常, /=0:エラー)
    integer::icol_err  !< エラーカラム   (=0:正常, /=0:エラー)

    !do iline = 1, nline
    !   write(6,'(i3,'':'',a)')  iline,trim(lines(iline))
    !end do

    call get_data_group_position(&
         lines, nline, group_name,&
         iline_gs, icol_gs, iline_ge, icol_ge, ngroup,&
         ldebug, iline_err, icol_err, istat)
    if (istat /= 0) return

    opt_values(1:nopt) = ' '
    opt_count(1:nopt)  = 0
    if (ngroup /= 0) then
       call parse_options(&
            lines, nline, iline_gs, icol_gs, iline_ge, icol_ge,&
            opt_names, opt_values, opt_count, nopt,&
            ldebug, iline_err, icol_err, istat)
       if (istat /= 0) return
    end if

  end subroutine get_options

  !-----------------------------------------------------------------------------
  !> @brief オプションの値を取得 (文字列リスト版)
  !>
  subroutine get_option_values_strlist(&
       opt_name, opt_names, opt_values, nopt, val_list, nval,&
       ipos_default, ipos, istat)
    character(len=*),intent(in)::opt_name         !< 対象オプションの名称
    integer,intent(in)::nopt                      !< オプション数
    character(len=*),intent(in)::opt_names(nopt)  !< オプションの名称テーブル
    character(len=*),intent(in)::opt_values(nopt) !< オプションの値テーブル
    integer,intent(in)::nval                      !< 値リスト数
    character(len=*),intent(in)::val_list(nval)   !< 値リストテーブル
    integer,intent(in)::ipos_default !< 対象オプションの値リストテーブルでの既定値
    integer,intent(out)::ipos        !< 対象オプションの値リストテーブルでの位置
    integer,intent(out)::istat       !< 終了ステータス (=0:正常, /=0:エラー)

    integer::iopt
    integer,parameter::mxstr = 1 ! 値は1つの文字列のみを許す
    integer::nstr
    character(len=MAX_INPUT_LINE_LEN)::strs(mxstr)

    !write(6,*) 'opt_name=',trim(opt_name)
    iopt = pl_str_search_name_ignorecase(opt_name, opt_names, nopt)
    if (iopt == 0) then
       istat = PL_ERR_INTERNAL_ERROR
       return
    end if
    !write(6,*) 'iopt=',iopt
    !write(6,*) 'opt_value=',trim(opt_values(iopt))
!!$    call split_str_n(trim(opt_values(iopt)), n, istat)
    call split_str(trim(opt_values(iopt)), strs, mxstr, nstr, istat)
    !write(6,*) 'nstr=',nstr
    if (istat /= 0) go to 9 ! 2つ以上の場合
    if (nstr == 0) then ! 0個の場合
       ipos = ipos_default
    else ! 1つの場合
       ipos = pl_str_search_name_ignorecase(opt_values(iopt), val_list, nval)
       if (ipos == 0) go to 9 ! 値リストテーブルにない場合
    end if
    !write(6,*) 'ipos=',ipos
    !write(6,*) 'ipos_default=',ipos_default
    istat = 0
    return

9   istat = PL_ERR_INVALID_VALUE
    call pl_error_print_message(istat)
    call print_error_invalid_value(opt_names(iopt), opt_values(iopt))
    return
  end subroutine get_option_values_strlist

  !-----------------------------------------------------------------------------
  !> @brief オプションの値を取得 (文字列版)
  !>
  subroutine get_option_values_str(&
       opt_name, opt_names, opt_values, nopt, str_default, str, istat)
    character(len=*),intent(in)::opt_name         !< 対象オプションの名称
    integer,intent(in)::nopt                      !< オプション数
    character(len=*),intent(in)::opt_names(nopt)  !< オプションの名称テーブル
    character(len=*),intent(in)::opt_values(nopt) !< オプションの値テーブル
    character(len=*),intent(in)::str_default      !< 対象オプションの既定値
    character(len=*),intent(inout)::str           !< 対象オプションの値
    integer,intent(out)::istat       !< 終了ステータス (=0:正常, /=0:エラー)

    integer::iopt

    !write(6,*) 'opt_name=',trim(opt_name)
    iopt = pl_str_search_name_ignorecase(opt_name, opt_names, nopt)
    if (iopt == 0) then
       istat = PL_ERR_INTERNAL_ERROR
       str = ' '
       return
    end if
    !write(6,*) 'iopt=',iopt
    !write(6,*) 'opt_value=',trim(opt_values(iopt))
    if (len_trim(opt_values(iopt)) == 0) then
       str = str_default
    else
       str = opt_values(iopt)
    end if
    !write(6,*) 'str=',trim(str)
    !write(6,*) 'str_default=',trim(str_default)

    istat = 0
  end subroutine get_option_values_str

  !-----------------------------------------------------------------------------
  !> @brief オプションの値を取得 (整数版)
  !>
  subroutine get_option_values_int(&
       opt_name, opt_names, opt_values, nopt, ival_default, ival, istat)
    character(len=*),intent(in)::opt_name         !< 対象オプションの名称
    integer,intent(in)::nopt                      !< オプション数
    character(len=*),intent(in)::opt_names(nopt)  !< オプションの名称テーブル
    character(len=*),intent(in)::opt_values(nopt) !< オプションの値テーブル
    integer,intent(in)::ival_default !< 対象オプションの既定値
    integer,intent(out)::ival        !< 対象オプションの値
    integer,intent(out)::istat       !< 終了ステータス (=0:正常, /=0:エラー)

    integer::iopt
    integer,parameter::mxstr = 1 ! 値は1つの文字列のみを許す
    integer::nstr
    character(len=MAX_INPUT_LINE_LEN)::strs(mxstr)

    ival = 0

    iopt = pl_str_search_name_ignorecase(opt_name, opt_names, nopt)
    if (iopt == 0) then
       istat = PL_ERR_INTERNAL_ERROR
       return
    end if
    !write(6,*) 'iopt=',iopt
    !write(6,*) 'opt_value=',trim(opt_values(iopt))
    call split_str(trim(opt_values(iopt)), strs, mxstr, nstr, istat)
    !write(6,*) 'nstr=',nstr
    if (istat /= 0) go to 9 ! 2つ以上の場合
    if (nstr == 0) then ! 0個の場合
       ival = ival_default
    else ! 1つの場合
       call pl_str_to_int(trim(opt_values(iopt)), ival, istat)
       if (istat /= 0) go to 9 ! 整数でない場合
    end if
    !write(6,*) 'ival=',ival
    istat = 0
    return

9   istat = PL_ERR_INVALID_VALUE
    call pl_error_print_message(istat)
    call print_error_invalid_value(opt_names(iopt), opt_values(iopt))
    return
  end subroutine get_option_values_int

  !-----------------------------------------------------------------------------
  !> @brief オプションの値を取得 (実数版)
  !>
  subroutine get_option_values_dbl(&
       opt_name, opt_names, opt_values, nopt, val_default, val, istat)
    character(len=*),intent(in)::opt_name         !< 対象オプションの名称
    integer,intent(in)::nopt                      !< オプション数
    character(len=*),intent(in)::opt_names(nopt)  !< オプションの名称テーブル
    character(len=*),intent(in)::opt_values(nopt) !< オプションの値テーブル
    real(8),intent(in)::val_default !< 対象オプションの既定値
    real(8),intent(out)::val        !< 対象オプションの値
    integer,intent(out)::istat      !< 終了ステータス (=0:正常, /=0:エラー)

    integer::iopt
    integer,parameter::mxstr = 1 ! 値は1つの文字列のみを許す
    integer::nstr
    character(len=MAX_INPUT_LINE_LEN)::strs(mxstr)

    val = 0.0d0

    iopt = pl_str_search_name_ignorecase(opt_name, opt_names, nopt)
    if (iopt == 0) then
       istat = PL_ERR_INTERNAL_ERROR
       return
    end if
    !write(6,*) 'iopt=',iopt
    !write(6,*) 'opt_value=',trim(opt_values(iopt))
    call split_str(trim(opt_values(iopt)), strs, mxstr, nstr, istat)
    !write(6,*) 'nstr=',nstr
    if (istat /= 0) go to 9 ! 2つ以上の場合
    if (nstr == 0) then
       val = val_default
    else
       call pl_str_to_dbl(trim(opt_values(iopt)), val, istat)
       if (istat /= 0) go to 9 ! 実数でない場合
    end if
    !write(6,*) 'val=',val

    istat = 0
    return

9   istat = PL_ERR_INVALID_VALUE
    call pl_error_print_message(istat)
    call print_error_invalid_value(opt_names(iopt), opt_values(iopt))
    return
  end subroutine get_option_values_dbl

  !-----------------------------------------------------------------------------
  !> @brief 空白以外の文字が現れるまでスキップ (コメント文字を考慮)
  !> 
  !> - 空白とコメント以外
  !> @verbatim
  !>          123456
  !> cline = "  abc "
  !>          ^icol_s = 1
  !>            ^ ipos = 3
  !> @endverbatim
  !> - 空白
  !> @verbatim
  !>          123456
  !> cline = "      "
  !>          ^icol_s = 1
  !>                ^ ipos = 7
  !> @endverbatim
  !> - コメント
  !> @verbatim
  !>          123456
  !> cline = "  !   "
  !>          ^icol_s = 1
  !>                ^ ipos = 7
  !> @endverbatim
  !> 
  subroutine skip_space(cline, icol_s, icol_e, ipos, istat)
    character(len=*),intent(in)::cline !< 探索される文字列
    integer,intent(in)::icol_s  !< 探索開始カラム位置
    integer,intent(in)::icol_e  !< 探索終了カラム位置
    integer,intent(out)::ipos   !< 探索されたカラム位置
    integer,intent(out)::istat  !< 終了ステータス (=0:正常, /=0:エラー)

    integer::icol
    character(len=1)::c

    if (icol_s < 0 .or. len(cline) < icol_e) then
       ipos = 0
       istat = PL_ERR_INTERNAL_ERROR
       !write(6,*) 'internal error in skip_space'
       return
    end if

    istat = 0
    ipos = icol_e + 1
    do icol = icol_s, icol_e
       !write(6,*) icol,':',cline(icol:icol)
!!$       if (index(c_blank, cline(icol:icol)) /= 0) cycle
!!$       if (index(c_comment, cline(icol:icol)) /= 0) return
       c = cline(icol:icol)
       if (index(c_blank, c) /= 0) cycle
       if (index(c_comment, c) /= 0) return
       ipos = icol
       return
    end do

  end subroutine skip_space

  !-----------------------------------------------------------------------------
  !> @brief データグループのプレフィックスが現れるまでスキップ (コメント文字を考慮)
  !>
  !> @verbatim
  !>          123456
  !> cline = " a=c &"
  !>          ^icol_s = 1
  !>               ^ ipos = 6
  !> @endverbatim
  !> 
  subroutine skip_data_group(cline, icol_s, icol_e, ipos, istat)
    character(len=*),intent(in)::cline !< 探索される文字列
    integer,intent(in)::icol_s  !< 探索開始カラム位置
    integer,intent(in)::icol_e  !< 探索終了カラム位置
    integer,intent(out)::ipos   !< 探索されたカラム位置
    integer,intent(out)::istat  !< 終了ステータス (=0:正常, /=0:エラー)

    integer::icol
    character(len=1)::c
    
    if (icol_s < 0 .or. len(cline) < icol_e) then
       ipos = 0
       istat = PL_ERR_INTERNAL_ERROR
       !write(6,*) 'internal error in skip_data_group'
       return
    end if

    istat = 0
    ipos = icol_e + 1
    do icol = icol_s, icol_e
       !write(6,*) icol,':',cline(icol:icol)
!!$       if (index(c_blank, cline(icol:icol)) /= 0) cycle
!!$       if (index(c_comment, cline(icol:icol)) /= 0) return
!!$       if (c_data_group_prefix == cline(icol:icol)) then
!!$          ipos = icol
!!$          return
!!$       end if
       c = cline(icol:icol)
       if (index(c_blank, c) /= 0) cycle
       if (index(c_comment, c) /= 0) return
       if (c_data_group_prefix == c) then
          ipos = icol
          return
       end if
    end do

  end subroutine skip_data_group

  !-----------------------------------------------------------------------------
  !> @brief トークンを取得
  !>
  !> - ケース1
  !> @verbatim
  !>          123456
  !> cline = "a=c   "
  !>          ^icol_s = 1
  !>            ^ ipos = 3
  !> @endverbatim
  !> - ケース2
  !> @verbatim
  !>          123456
  !> cline = " a=c  "
  !>          ^icol_s = 1
  !>         ^ ipos = 0
  !> @endverbatim
  !> 
  subroutine get_token(cline, icol_s, icol_e, ipos, istat)
    character(len=*),intent(in)::cline !< 探索される文字列
    integer,intent(in)::icol_s  !< 探索開始カラム位置
    integer,intent(in)::icol_e  !< 探索終了カラム位置
    integer,intent(out)::ipos   !< 探索されたカラム位置
    integer,intent(out)::istat  !< 終了ステータス (=0:正常, /=0:エラー)

    integer::icol

    if (icol_s < 0 .or. len(cline) < icol_e) then
       ipos = 0
       istat = PL_ERR_INTERNAL_ERROR
       !write(6,*) 'internal error in get_token'
       return
    end if

    istat = 0
    ipos = icol_s - 1
    do icol = icol_s, icol_e
       !write(6,*) icol,':',cline(icol:icol)
       if (index(c_blank, cline(icol:icol)) /= 0) then
          ipos = icol - 1
          return
       end if
    end do
    ipos = icol_e

  end subroutine get_token

  !-----------------------------------------------------------------------------
  !> @brief データグループ名を取得 (コメント文字を考慮)
  !>
  !> @verbatim
  !>          123456
  !> cline = "&ABC  "
  !>          ^icol_s = 1
  !>             ^ ipos = 4
  !> @endverbatim
  !>
  subroutine get_token_group_name(cline, icol_s, icol_e, ipos, istat)
    character(len=*),intent(in)::cline !< 探索される文字列
    integer,intent(in)::icol_s  !< 探索開始カラム位置
    integer,intent(in)::icol_e  !< 探索終了カラム位置
    integer,intent(out)::ipos   !< 探索されたカラム位置
    integer,intent(out)::istat  !< 終了ステータス (=0:正常, /=0:エラー)

    integer::icol

    if (icol_s < 0 .or. len(cline) < icol_e) then
       ipos = 0
       istat = PL_ERR_INTERNAL_ERROR
       !write(6,*) 'internal error in get_token_group_name'
       return
    end if

    istat = 0
    ipos = icol_s - 1
    do icol = icol_s, icol_e
       !write(6,*) icol,':',cline(icol:icol)
       if (index(c_blank, cline(icol:icol)) /= 0) then
          ipos = icol - 1
          return
       else
          if (index(c_comment, cline(icol:icol)) /= 0) then
             ipos = icol - 1
             return
          end if
          if (c_data_group_prefix == cline(icol:icol)) then
             if (icol == icol_s) then
                cycle
             else
                ipos = icol - 1
                return
             end if
          end if
       end if
    end do
    ipos = icol_e

  end subroutine get_token_group_name

  !-----------------------------------------------------------------------------
  !> @brief オプション名を取得 (コメント文字を考慮)
  !>
  !> @verbatim
  !>          123456
  !> cline = "ABC=XY"
  !>          ^icol_s = 1
  !>            ^ ipos = 3
  !> @endverbatim
  !>
  subroutine get_token_option_name(cline, icol_s, icol_e, ipos, istat)
    character(len=*),intent(in)::cline !< 探索される文字列
    integer,intent(in)::icol_s  !< 探索開始カラム位置
    integer,intent(in)::icol_e  !< 探索終了カラム位置
    integer,intent(out)::ipos   !< 探索されたカラム位置
    integer,intent(out)::istat  !< 終了ステータス (=0:正常, /=0:エラー)

    integer::icol

    if (icol_s < 0 .or. len(cline) < icol_e) then
       ipos = 0
       istat = PL_ERR_INTERNAL_ERROR
       !write(6,*) 'internal error in get_token_option_name'
       return
    end if

    istat = 0
    ipos = icol_s - 1
    do icol = icol_s, icol_e
       !write(6,*) icol,':',cline(icol:icol)
       if (index(c_blank, cline(icol:icol)) /= 0) then
          ipos = icol - 1
          return
       else
          if (index(c_comment, cline(icol:icol)) /= 0) then
             ipos = icol - 1
             return
          end if
          if (c_assignment_op == cline(icol:icol)) then
             ipos = icol - 1
             return
          end if
       end if
    end do
    ipos = icol_e

  end subroutine get_token_option_name

  !-----------------------------------------------------------------------------
  !> @brief オプション値を取得 (コメント文字を考慮)
  !>
  !> - ケース1
  !> @verbatim
  !>          123456
  !> cline = "ABC=XY"
  !>              ^icol_s = 5
  !>               ^ ipos = 6
  !> @endverbatim
  !> - ケース1
  !> @verbatim
  !>          123456
  !> cline = "ABC=XY="
  !>              ^icol_s = 5
  !>               ^ ipos = 6
  !> @endverbatim
  !>
  subroutine get_token_option_value(cline, icol_s, icol_e, ipos, istat)
    character(len=*),intent(in)::cline !< 探索される文字列
    integer,intent(in)::icol_s  !< 探索開始カラム位置
    integer,intent(in)::icol_e  !< 探索終了カラム位置
    integer,intent(out)::ipos   !< 探索されたカラム位置
    integer,intent(out)::istat  !< 終了ステータス (=0:正常, /=0:エラー)

    integer::icol

    ipos = icol_s - 1

    if (icol_s < 0 .or. len(cline) < icol_e) then
       ipos = 0
       istat = PL_ERR_INTERNAL_ERROR
       !write(6,*) 'internal error in get_token_option_value'
       return
    end if

    istat = 0
    ipos = icol_s - 1
    do icol = icol_s, icol_e
       !write(6,*) icol,':',cline(icol:icol)
       if (index(c_blank, cline(icol:icol)) /= 0) then
          ipos = icol - 1
          return
       else
          if (index(c_comment, cline(icol:icol)) /= 0) then
             ipos = icol - 1
             return
          end if
          if (c_assignment_op == cline(icol:icol)) then
             ipos = icol - 1
             return
          end if
       end if
    end do
    ipos = icol_e

  end subroutine get_token_option_value

  !-----------------------------------------------------------------------------
  !> @brief ( )で囲まれた文字列を取得 (コメント文字を考慮)
  !>
  subroutine get_token_paren(cline, icol_s, icol_e, ipos, istat)
    character(len=*),intent(in)::cline !< 探索される文字列
    integer,intent(in)::icol_s  !< 探索開始カラム位置
    integer,intent(in)::icol_e  !< 探索終了カラム位置
    integer,intent(out)::ipos   !< 探索されたカラム位置
    integer,intent(out)::istat  !< 終了ステータス (=0:正常, /=0:エラー)

    integer::icol
    
    if (icol_s < 0 .or. len(cline) < icol_e) then
       ipos = 0
       istat = PL_ERR_INTERNAL_ERROR
       !write(6,*) 'internal error in get_token_paren'
       return
    end if

    istat = 0
    ipos = icol_e + 1
    do icol = icol_s, icol_e
       !write(6,*) icol,':',cline(icol:icol)
       if (index(c_blank, cline(icol:icol)) /= 0) cycle
       if (index(c_comment, cline(icol:icol)) /= 0) return
       if (c_paren_close == cline(icol:icol)) then
          ipos = icol
          return
       end if
       if (c_paren_open == cline(icol:icol)) then
          ! error
          return
       end if
    end do

  end subroutine get_token_paren

  !-----------------------------------------------------------------------------
  !> @brief " "で囲まれた文字列を取得 (コメント文字を考慮)
  !>
  subroutine get_token_quote(cline, icol_s, icol_e, ipos, istat)
    character(len=*),intent(in)::cline !< 探索される文字列
    integer,intent(in)::icol_s  !< 探索開始カラム位置
    integer,intent(in)::icol_e  !< 探索終了カラム位置
    integer,intent(out)::ipos   !< 探索されたカラム位置
    integer,intent(out)::istat  !< 終了ステータス (=0:正常, /=0:エラー)

    integer::icol
    
    if (icol_s < 0 .or. len(cline) < icol_e) then
       ipos = 0
       istat = PL_ERR_INTERNAL_ERROR
       !write(6,*) 'internal error in get_token_quote'
       return
    end if

    istat = 0
    ipos = icol_e + 1
    do icol = icol_s, icol_e
       !write(6,*) icol,':',cline(icol:icol)
       if (index(c_blank, cline(icol:icol)) /= 0) cycle
       if (index(c_comment, cline(icol:icol)) /= 0) return
       if (c_quote == cline(icol:icol)) then
          ipos = icol
          return
       end if
    end do

  end subroutine get_token_quote

  !-----------------------------------------------------------------------------
  !> @brief 空白文字で区切られた文字列の個数を取得
  !>
  subroutine split_str_n(str, nval, istat)
    character(len=*),intent(in)::str !< 探索される文字列
    integer,intent(out)::nval   !< 空白文字で区切られた文字列の個数
    integer,intent(out)::istat  !< 終了ステータス (=0:正常, /=0:エラー)

    integer::icol_e, icol, itoken_s, itoken_e

    nval = 0
    icol = 1
    icol_e = len_trim(str)
    !write(6,*) 'icol=',icol
    do while (icol <= icol_e)
       call skip_space(str, icol, icol_e, itoken_s, istat)
       if (istat /= 0) return
       !write(6,*) 'itoken_s=',itoken_s
       if (icol_e < itoken_s) exit

       call get_token(str, itoken_s, icol_e, itoken_e, istat)
       if (istat /= 0) return
       !write(6,*) 'itoken_e=',itoken_e
       if (icol_e < itoken_e) exit
       nval = nval + 1
       icol = itoken_e + 1
       !write(6,*) 'icol=',icol
    end do

    !write(6,*) 'nval=',nval
    istat = 0
  end subroutine split_str_n

  !-----------------------------------------------------------------------------
  !> @brief 空白文字で区切られた文字列を取得
  !>
  subroutine split_str(str, vals, mxval, nval, istat)
    character(len=*),intent(in)::str !< 探索される文字列
    integer,intent(in)::mxval   !< 格納配列valsのサイズ
    character(len=*),intent(inout)::vals(mxval) !< 格納配列
    integer,intent(out)::nval   !< 空白文字で区切られた文字列の個数
    integer,intent(out)::istat  !< 終了ステータス (=0:正常, /=0:エラー)

    integer::icol_e, icol, itoken_s, itoken_e

    nval = 0
    icol = 1
    icol_e = len_trim(str)
    !write(6,*) 'icol=',icol
    do while (icol <= icol_e)
       call skip_space(str, icol, icol_e, itoken_s, istat)
       if (istat /= 0) return
       !write(6,*) 'itoken_s=',itoken_s
       if (icol_e < itoken_s) exit
       call get_token(str, itoken_s, icol_e, itoken_e, istat)
       if (istat /= 0) return
       !write(6,*) 'itoken_e=',itoken_e
       if (icol_e < itoken_e) exit
       nval = nval + 1
       if (nval > mxval) then
          istat = -1
          return ! 配列サイズを越えた
       end if
       vals(nval) = str(itoken_s:itoken_e)
       icol = itoken_e + 1
       !write(6,*) 'icol=',icol
    end do

    !write(6,*) 'nval=',nval
    istat = 0
  end subroutine split_str

  !-----------------------------------------------------------------------------
  !> @brief 空白文字で区切られた実数値を取得
  !>
  subroutine split_dbl(str, vals, mxval, nval, istat)
    character(len=*),intent(in)::str !< 探索される文字列
    integer,intent(in)::mxval   !< 格納配列valsのサイズ
    real(8),intent(inout)::vals(mxval) !< 格納配列
    integer,intent(out)::nval   !< 空白文字で区切られた文字列の個数
    integer,intent(out)::istat  !< 終了ステータス (=0:正常, /=0:エラー)

    integer::icol_e, icol, itoken_s, itoken_e
    real(8)::val

    nval = 0
    icol = 1
    icol_e = len_trim(str)
    !write(6,*) 'icol=',icol
    do while (icol <= icol_e)
       call skip_space(str, icol, icol_e, itoken_s, istat)
       if (istat /= 0) return
       !write(6,*) 'itoken_s=',itoken_s
       if (icol_e < itoken_s) exit
       call get_token(str, itoken_s, icol_e, itoken_e, istat)
       if (istat /= 0) return
       !write(6,*) 'itoken_e=',itoken_e
       if (icol_e < itoken_e) exit
       nval = nval + 1
       if (nval > mxval) then
          istat = -1
          return ! 配列サイズを越えた
       end if

       call pl_str_to_dbl(str(itoken_s:itoken_e), val, istat)
       if (istat /= 0) return
       vals(nval) = val

       icol = itoken_e + 1
       !write(6,*) 'icol=',icol
    end do

    !write(6,*) 'nval=',nval
    istat = 0
  end subroutine split_dbl

  !-----------------------------------------------------------------------------
  !> @brief 空白文字で区切られた整数値を取得
  !>
  subroutine split_int(str, vals, mxval, nval, istat)
    character(len=*),intent(in)::str !< 探索される文字列
    integer,intent(in)::mxval   !< 格納配列valsのサイズ
    integer,intent(inout)::vals(mxval) !< 格納配列
    integer,intent(out)::nval   !< 空白文字で区切られた文字列の個数
    integer,intent(out)::istat  !< 終了ステータス (=0:正常, /=0:エラー)

    integer::icol_e, icol, itoken_s, itoken_e
    integer::val

    nval = 0
    icol = 1
    icol_e = len_trim(str)
    !write(6,*) 'icol=',icol
    do while (icol <= icol_e)
       call skip_space(str, icol, icol_e, itoken_s, istat)
       if (istat /= 0) return
       !write(6,*) 'itoken_s=',itoken_s
       if (icol_e < itoken_s) exit
       call get_token(str, itoken_s, icol_e, itoken_e, istat)
       if (istat /= 0) return
       !write(6,*) 'itoken_e=',itoken_e
       if (icol_e < itoken_e) exit
       nval = nval + 1
       if (nval > mxval) then
          istat = -1
          return ! 配列サイズを越えた
       end if

       call pl_str_to_int(str(itoken_s:itoken_e), val, istat)
       if (istat /= 0) return
       vals(nval) = val

       icol = itoken_e + 1
       !write(6,*) 'icol=',icol
    end do

    !write(6,*) 'nval=',nval
    istat = 0
  end subroutine split_int

  !-----------------------------------------------------------------------------
  !> @brief 空白文字あるいは'='で区切られた文字列の個数を取得
  !>
  subroutine split_str_eq_n(str, nval, istat)
    character(len=*),intent(in)::str !< 探索される文字列
    integer,intent(out)::nval   !< 空白文字あるいは'='で区切られた文字列の個数
    integer,intent(out)::istat  !< 終了ステータス (=0:正常, /=0:エラー)

    integer::icol_e, icol, itoken_s, itoken_e

    nval = 0
    icol = 1
    icol_e = len_trim(str)
    !write(6,*) 'icol=',icol
    do while (icol <= icol_e)
       call skip_space(str, icol, icol_e, itoken_s, istat)
       if (istat /= 0) return
       !write(6,*) 'itoken_s=',itoken_s
       if (icol_e < itoken_s) exit
       if (str(itoken_s:itoken_s) == c_assignment_op) then
          !write(6,*) 'token=',str(itoken_s:itoken_s),'$'
          nval = nval + 1
          icol = itoken_s + 1
       else
!!$          call get_token_option_name(str, itoken_s, icol_e, itoken_e, istat)
          call get_token_option_value(str, itoken_s, icol_e, itoken_e, istat)
          if (istat /= 0) return
          !write(6,*) 'itoken_e=',itoken_e
          if (itoken_e < icol) exit
          !write(6,*) 'token=',str(itoken_s:itoken_e),'$'
          nval = nval + 1
          icol = itoken_e + 1
       end if
       !write(6,*) 'icol=',icol
    end do

    !write(6,*) 'nval=',nval
    istat = 0
  end subroutine split_str_eq_n

  !-----------------------------------------------------------------------------
  !> @brief 空白文字あるいは'='で区切られた文字列を取得
  !>
  subroutine split_str_eq(str, vals, mxval, nval, istat)
    character(len=*),intent(in)::str !< 探索される文字列
    integer,intent(in)::mxval   !< 格納配列valsのサイズ
    character(len=*),intent(inout)::vals(mxval) !< 格納配列
    integer,intent(out)::nval   !< 空白文字あるいは'='で区切られた文字列の個数
    integer,intent(out)::istat  !< 終了ステータス (=0:正常, /=0:エラー)

    integer::icol_e, icol, itoken_s, itoken_e

    nval = 0
    icol = 1
    icol_e = len_trim(str)
    !write(6,*) 'icol=',icol
    do while (icol <= icol_e)
       call skip_space(str, icol, icol_e, itoken_s, istat)
       if (istat /= 0) return
       !write(6,*) 'itoken_s=',itoken_s
       if (icol_e < itoken_s) exit
       if (str(itoken_s:itoken_s) == c_assignment_op) then
          !write(6,*) 'token=',str(itoken_s:itoken_s),'$'
          nval = nval + 1
          if (nval > mxval) then
             istat = -1
             return ! 配列サイズを越えた
          end if
          vals(nval) = str(itoken_s:itoken_s)
          icol = itoken_s + 1
       else
!!$          call get_token_option_name(str, itoken_s, icol_e, itoken_e, istat)
          call get_token_option_value(str, itoken_s, icol_e, itoken_e, istat)
          if (istat /= 0) return
          !write(6,*) 'itoken_e=',itoken_e
          if (itoken_e < icol) exit
          !write(6,*) 'token=',str(itoken_s:itoken_e),'$'
          nval = nval + 1
          if (nval > mxval) then
             istat = -1
             return ! 配列サイズを越えた
          end if
          vals(nval) = str(itoken_s:itoken_e)
          icol = itoken_e + 1
       end if
       !write(6,*) 'icol=',icol
    end do

    !write(6,*) 'nval=',nval
    istat = 0
  end subroutine split_str_eq

  !-----------------------------------------------------------------------------
  subroutine print_error_code(error_code)
    integer,intent(in)::error_code

    write(6,'(2x,''Error Code : '',i6)') error_code
  end subroutine print_error_code

  !-----------------------------------------------------------------------------
  subroutine print_error_lc(iline, icol)
    integer,intent(in)::iline, icol

    write(6,'(2x,''Error Position : Line ='',i6,'', Column ='',i6)') iline, icol
  end subroutine print_error_lc

  !-----------------------------------------------------------------------------
  subroutine print_error_line(line)
    character(len=*),intent(in)::line

    write(6,'(2x,''Error Line :'',a)') trim(line)
  end subroutine print_error_line

  !-----------------------------------------------------------------------------
  subroutine print_error_invalid_option(opt_name)
    character(len=*),intent(in)::opt_name

    write(6,'(2x,a,a)') 'Option Name : ', trim(opt_name)
  end subroutine print_error_invalid_option

  !-----------------------------------------------------------------------------
  subroutine print_error_invalid_value(opt_name, opt_value)
    character(len=*),intent(in)::opt_name
    character(len=*),intent(in)::opt_value

    write(6,'(2x,a,a)') 'Option Name  : ', trim(opt_name)
    write(6,'(2x,a,a)') '       Value : ', trim(opt_value)
  end subroutine print_error_invalid_value

  !-----------------------------------------------------------------------------
end module input_class
