! -*- coding: utf-8 -*-
!
! Dual licensed under the MIT or GPL Version 3 license.
! See 'LICENSE.MIT' for the definition of the MIT license,
! and 'COPYING' for the definition of the GPL license.
!
!> 積分Fm(T)テーブルクラス
!!
module integral_fmt_class
  implicit none
  private

  public::fmt_type
  public::fmt_new
  public::fmt_prep
  public::fmt_free

  type::fmt_type
     integer::nfsz
     integer::nesz
     real(8),allocatable::fdat(:,:)
     real(8),allocatable::edat(:)
  end type

  integer,parameter::FMT_NESIZE = 2001
  integer,parameter::FMT_NFSIZE = 31

  real(8),public,parameter::TF(25) = (/&
       33.0d+00, 37.0d+00, 41.0d+00, 43.0d+00, 46.0d+00, 49.0d+00,&
       51.0d+00, 54.0d+00, 56.0d+00, 58.0d+00, 61.0d+00, 63.0d+00,&
       66.0d+00, 68.0d+00, 70.0d+00, 72.0d+00, 74.0d+00, 80.0d+00,&
       85.0d+00, 90.0d+00, 95.0d+00, 95.0d+00, 98.0d+00, 99.0d+00,&
       99.0d+00 /)
  real(8),public,parameter :: ga0 = 1.0d0
  real(8),public,parameter :: ga1 = 1.0d0
  real(8),public,parameter :: ga2 = 0.5d0
  real(8),public,parameter :: ga3 = 1.0d0 /  6.0d0
  real(8),public,parameter :: ga4 = 1.0d0 / 24.0d0
  real(8),public,parameter :: ga5 = 1.0d0 /120.0d0
  real(8),public,parameter :: ga6 = 1.0d0 /720.0d0

  real(8),public,parameter :: rminv(8) = (/&
       1.0d0, &
       0.33333333333333333d0, &
       0.2d0, &
       0.14285714285714286d0, &
       0.11111111111111111d0, &
       0.09090909090909091d0, &
       0.07692307692307693d0 ,&
       0.06666666666666666d0 /)

contains
  !-----------------------------------------------------------------------------
  !> 配列確保
  subroutine fmt_new(self, ldebug, istat)
    type(fmt_type),intent(inout)::self
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    istat = 0

    self%nfsz = FMT_NFSIZE
    self%nesz = FMT_NESIZE

    allocate(&
         self%fdat(self%nfsz,self%nesz), self%edat(self%nesz), stat=istat)
    if (istat /= 0) return

    self%fdat(:,:) = 0.d0
    self%edat(:)   = 0.d0

  end subroutine fmt_new

  !-----------------------------------------------------------------------------
  !> 前処理
  subroutine fmt_prep(self, ldebug, istat)
    type(fmt_type),intent(inout)::self
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    real(8)::t
    integer::i

    call p_ifmtcl(self%fdat, self%nfsz, self%nesz, istat)
    if (istat /= 0) go to 9

    do i = 1, self%nesz
       t = 0.05d0 * (i-1)
       self%edat(i) = dexp(-t)
    end do

    return
9   write(6,'(''error called from sub-ifmegt'')')
    return 
  end subroutine fmt_prep

  !-----------------------------------------------------------------------------
  !> 配列開放
  subroutine fmt_free(self, ldebug, istat)
    type(fmt_type),intent(inout)::self
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    istat = 0

    deallocate(self%fdat, self%edat, stat=istat)
    if (istat /= 0) return

  end subroutine fmt_free

  !-----------------------------------------------------------------------------
  subroutine p_ifmtcl(fdat, nfsz, nesz, istat)
    integer,intent(in)::nfsz, nesz
    real(8),intent(inout)::fdat(nfsz,nesz)
    integer,intent(inout)::istat

    integer::i
    real(8)::t(2010)

    if(nfsz /= FMT_NFSIZE .or. nesz /= FMT_NESIZE) go to 9

    do i = 1, nesz
       t(i) = 0.05d+00*(i-1)
    end do

    call p_ifmtca (t, nesz, 30, fdat, nfsz, nesz, istat)
    if (istat /= 0) return

    return
9   istat = -1030
    write(6,'(/'' >>> error in ifmtcl. incorrect size '',2i6)') nfsz,nesz
    return
  end subroutine p_ifmtcl

  !-----------------------------------------------------------------------------
  subroutine p_ifmtca(ttbl, numt, mmax, fdat, nfsz, nesz, istat)
    integer,intent(in)::numt, mmax
    real(8),intent(in)::ttbl(numt)
    integer,intent(in)::nfsz, nesz
    real(8),intent(inout)::fdat(nfsz,nesz)
    integer,intent(inout)::istat

    integer,parameter::isz = 300
    integer,parameter::jsz = 600
    real(8)::fmttbl(isz),gmttbl(jsz),tt
    integer::i,j,itt

    if (mmax < 0 .or. mmax > 299) then
       istat = -1
       return
    end if

    fmttbl(:) = 0.d0
    gmttbl(:) = 0.d0

    istat = 0
    !write (6,'(/''    number of t values ='',i5)') numt

    do i = 1, numt
       tt = ttbl(i)
       if (tt <= 2.0d+00) then
          call p_ifmtba(fmttbl,isz,50,0,tt,istat)
       else if (tt <= 10.0d+00) then
          call p_ifmtba(fmttbl,isz,70,0,tt,istat)
       else if (tt <= 20.0d+00) then
          call p_ifmtba(fmttbl,isz,120,0,tt,istat)
       else
          itt = tt + 0.5d+00
          call p_ifmtfo(fmttbl,isz,gmttbl,jsz,-itt,mmax,tt,istat)
       end if
       if (istat /= 0) return
       !write (6,'(1h ,''   t='',g30.20,''  max of m='',i5)')  tt,mmax

       do j = 1, mmax+1
          fdat(j,i) = fmttbl(j)
       end do
    end do

  end subroutine p_ifmtca

  !-----------------------------------------------------------------------------
  !    FMT CAL. BY BACKWARD RECURSIVE FORMULA
  !
  !        F(M,T) = (2T*F(M+1,T) + EXP(-T)) / (2M+1)
  !
  !        FMTTBL  : D(ISZ) : CALC. BUFFER
  !        ISZ     : I      : SIZE OF FMTTBL BUFFER
  !        MS      : I      : START M AT WHICH F=0  ,F(NS,T)=0
  !        ME      : I      : END M... AT THIS POINT CALC. IS TERMINATED
  !        T       : D      : T VALUE
  !        ISTAT    : I      : ERROR CODE   0.. NORMAL
  !                                       -1031..MS<ME OR MS=ME
  !                                       -1032..MS+1>ISZ
  !                                       -1033..ME<0
  !  CAUTIONS
  !       MUST BE MS>ME.
  !       F(T); M=0----FMTTBL(1),  F(T);M=MS----FMTTBL(MS+1)
  !
  !**********************************************************************
  subroutine p_ifmtba(fmttbl, isz, ms, me, t, istat)
    integer,intent(in)::isz, ms, me
    real(8),intent(inout)::fmttbl(isz)
    real(8),intent(in)::t
    integer,intent(inout)::istat

    integer::m, mw
    real(8)::t2,d

    if (isz < (ms+1)) go to 902
    if (ms <= me) go to 901
    if (me < 0) go to 903

    fmttbl(ms+1) = 0.0d+00
    t2           = 2.0d+00*t
    d            = 1.0d+00/dexp(t)
    mw           = ms + 1

    do m = 1, ms - me
       mw         = mw - 1
       fmttbl(mw) = (t2*fmttbl(mw+1) + d) / (2.0d+00*mw - 1)
    end do

    return

901 write(6,'(/'' >>> error in ifmtba.   ms<me or ms=me'')')
    istat = -1031
    return

902 write(6,'(/'' >>> error in ifmtba.   ms+1>isz'')')
    istat = -1032
    return

903 write(6,'(/'' >>> error in ifmtba.   me<0           '')')
    istat = -1033
    return
  end subroutine p_ifmtba

  !-----------------------------------------------------------------------------
  !   FMT CAL. BY FORWARD RECURSIVE FORMULA
  !
  !        F(M,T) = GAMMA(M+1/2) / (2T**(M+1/2) - G(M,T)
  !
  !        G(M,T) = ( (2M-1)*G(M-1,T) + EXP(-T) ) / 2T
  !
  !         FMTTBL : O,D(ISZ) : BUFFER OF CAL. DATA
  !         ISZ    : I,I      : SIZE
  !         GMTTBL : O,D(JSZ) : BUFFER OF G(M,T)
  !         JSZ    : I,I      : SIZE
  !         MS     : I,I      : START M AT WHICH G(MS,T) = EXP(-T)/2(T-S)
  !         ME     : I,I      : END M   AT THIS POINT CAL. IS TERMINATED
  !         T      : I,D      : T VAL.
  !         ISTAT   : O,I      : ERROR CODE   0.. NORMAL END
  !                                         -1041.. MS>0 OR MS=0
  !                                         -1042.. ME+1>ISZ,ME-MS+1>JSZ
  !                                         -1043.. ME<0
  !    CAUTIONS
  !       MUST BE MS<0 .
  !       F(T);M=0----FMTTBL(1),  F(T);M=MS----FMTTBL(ME+1)
  !======================================================================
  subroutine p_ifmtfo(fmttbl, isz, gmttbl, jsz, ms, me, t, istat)
    use physical_constants_class
    integer,intent(in)::isz,jsz,ms,me
    real(8),intent(inout)::fmttbl(isz),gmttbl(jsz)
    real(8),intent(in)::t
    integer,intent(inout)::istat

    integer::m, mw, iw
    real(8)::t2, d, gamma

    if (isz < (me+1) .or. jsz < (me-ms+1)) go to 902
    if (ms >= 0) go to 901
    if (me < 0) go to 903

    gmttbl(1) = 1.0d+00/(dexp(t)*2.0d+00*(t-ms))
    t2        = 2.0d+00*t
    d         = 1.0d+00/(dexp(t)*t2)
    mw        = ms + 1

    do m = 2, me - ms + 1
       gmttbl(m) = ((2*mw-1)/t2)*gmttbl(m-1) + d
       mw        = mw + 1
    end do

    gamma     = dsqrt(pi/t)/2.0d+00
    fmttbl(1) = gamma - gmttbl(1-ms)
    iw        = 1

    do m = 2, me + 1
       gamma     = gamma*(dble(iw)/t2)
       fmttbl(m) = gamma - gmttbl(m-ms)
       iw        = iw + 2
    end do

    return

901 write(6,'(/'' >>> error in ifmtfo.   ms>0 or ms=0'')')
    istat   = -1041
    return

902 write(6,'(/'' >>> error in ifmtfo.   me+1>isz,me-ms+1>jsz'')')
    istat   = -1042
    return

903 write(6,'(/'' >>> error in ifmtfo.   me<0'')')
    istat   = -1043
    return
  end subroutine p_ifmtfo

end module integral_fmt_class
