! -*- coding: utf-8 -*-
!
! Dual licensed under the MIT or GPL Version 3 license.
! See 'LICENSE.MIT' for the definition of the MIT license,
! and 'COPYING' for the definition of the GPL license.
!
module integral_hgp_table_class
  implicit none
  private

  public::jatom_type
  public::grad_output0
  public::grad_output1
  public::grad_output2
  public::grad_output3
  public::grad_output4
  public::set_graderi_index
  type,public::grad_table
    integer::index_prod(3)
    integer::index_grad(3)
    integer::index_out(3)
    integer::np_atom,nd_atom
    integer::nprod,ngrad,nout
  end type

  type,public::grad_index
    integer,allocatable :: index_vrr_range(:,:)
    integer,allocatable :: index_vrr0(:,:)
    integer,allocatable :: index_vrr1(:,:)
    integer,allocatable :: index_vrr2(:,:)
    integer,allocatable :: index_set_hgp(:,:,:)
    integer,allocatable :: num_hgp_type(:)
    integer,allocatable :: index_hrr(:,:,:)
    integer,allocatable :: index_geri4(:,:,:)
    integer,allocatable :: index_hrr_range(:,:)

    integer :: max_vrr_eri
    integer :: mx_vrr0,mx_vrr1,mx_vrr2
    integer :: max_hgp_eri,max_nhgp,min_nhgp,mxang3

  end type

contains
  subroutine jatom_type(i_atom,self)
    integer :: i_atom(4)
    integer :: iatom,jatom,katom,latom
    integer :: kij,kik,kil,kjk,kjl,kkl,key
    type(grad_table) :: self

    iatom = i_atom(1)
    jatom = i_atom(2)
    katom = i_atom(3)
    latom = i_atom(4)

    kij = 0
    kik = 0
    kil = 0
    kjk = 0
    kjl = 0
    kkl = 0

    if (iatom == jatom) kij = 1
    if (iatom == katom) kik = 1
    if (iatom == latom) kil = 1
    if (jatom == katom) kjk = 1
    if (jatom == latom) kjl = 1
    if (katom == latom) kkl = 1

    key = kij + 2*kik + 4*kil + 8*kjk + 16*kjl + 32*kkl + 1

    self%index_prod = 0
    self%index_grad = 0
    self%index_out = 0
    self%nprod = 0
    self%ngrad = 0
    self%nout = 0
    self%np_atom = 0
    self%nd_atom = 0

    select case(key)

    case(64)!AAAA 0

    case(57)!ABBB 1
      self%ngrad = 1
      self%index_grad(1) = 1
      self%nd_atom = 2

      self%nout = 1
      self%index_out(1) = 1

    case(39)!BABB 2
      self%ngrad = 1
      self%index_grad(1) = 2
      self%nd_atom = 1

      self%nout = 1
      self%index_out(1) = 2

    case(22)!BBAB 3
      self%ngrad = 1
      self%index_grad(1) = 3
      self%nd_atom = 1

      self%nout = 1
      self%index_out(1) = 3

    case(12)!BBBA 4
      self%nprod = 3
      self%index_prod(1) = 1
      self%index_prod(2) = 2
      self%index_prod(3) = 3
      self%np_atom = 1

      self%ngrad = 3
      self%index_grad(1) = 1
      self%index_grad(2) = 2
      self%index_grad(3) = 3
      self%nd_atom = 4

    case(34)!AABB 5
      self%nprod = 2
      self%index_prod(1) = 1
      self%index_prod(2) = 2
      self%np_atom = 1

      self%ngrad = 2
      self%index_grad(1) = 1
      self%index_grad(2) = 2
      self%nd_atom = 3

    case(19)!ABAB 6
      self%nprod = 2
      self%index_prod(1) = 1
      self%index_prod(2) = 3
      self%np_atom = 1

      self%ngrad = 2
      self%index_grad(1) = 1
      self%index_grad(2) = 3
      self%nd_atom = 2

    case(13)!ABBA 7
      self%nprod = 2
      self%index_prod(1) = 2
      self%index_prod(2) = 3
      self%np_atom = 2

      self%ngrad = 2
      self%index_grad(1) = 2
      self%index_grad(2) = 3
      self%nd_atom = 1

    case(33)!ABCC 8
      self%ngrad = 2
      self%index_grad(1) = 1
      self%index_grad(2) = 2
      self%nd_atom = 3

      self%nout = 2
      self%index_out(1) = 1
      self%index_out(2) = 2

    case(2)!CCAB 9
      self%nprod = 2
      self%index_prod(1) = 1
      self%index_prod(2) = 2
      self%np_atom = 1

      self%ngrad = 3
      self%index_grad(1) = 1
      self%index_grad(2) = 2
      self%index_grad(3) = 3
      self%nd_atom = 4

      self%nout = 1
      self%index_out(1) = 3

    case(17)!ACBC 10
      self%ngrad = 2
      self%index_grad(1) = 1
      self%index_grad(2) = 3
      self%nd_atom = 2

      self%nout = 2
      self%index_out(1) = 1
      self%index_out(2) = 3

    case(9)!ACCB 11
      self%nprod = 2
      self%index_prod(1) = 2
      self%index_prod(2) = 3
      self%np_atom = 2

      self%ngrad = 3
      self%index_grad(1) = 1
      self%index_grad(2) = 2
      self%index_grad(3) = 3
      self%nd_atom = 4

      self%nout = 1
      self%index_out(1) = 1

    case(5)!CABC 12
      self%ngrad = 2
      self%index_grad(1) = 2
      self%index_grad(2) = 3
      self%nd_atom = 1

      self%nout = 2
      self%index_out(1) = 2
      self%index_out(2) = 3

    case(3)!CACB 13
      self%nprod = 2
      self%index_prod(1) = 1
      self%index_prod(2) = 3
      self%np_atom = 1

      self%ngrad = 3
      self%index_grad(1) = 1
      self%index_grad(2) = 2
      self%index_grad(3) = 3
      self%nd_atom = 4

      self%nout = 1
      self%index_out(1) = 2

    case(1)!ABCD 14
      self%ngrad = 3
      self%index_grad(1) = 1
      self%index_grad(2) = 2
      self%index_grad(3) = 3
      self%nd_atom = 4

      self%nout = 3
      self%index_out(1) = 1
      self%index_out(2) = 2
      self%index_out(3) = 3

    end select

  end subroutine jatom_type!jatom_type

  subroutine grad_output0(gval,indexg,ec,mxang3,i_atom,num,self,i_shell,&
       ian,jan,kan,lan,erithr,iaoslix,nshell)

    integer :: nshell,mxang3
    integer(2) :: indexg(6,mxang3*12)
    integer :: iaoslix(nshell,3)
    real(8) :: gval(mxang3*12),ec(mxang3,3)
    integer :: i_shell(4),num

    integer :: i_atom(4),mm(11),m4(4),i4(4)
    real(8) :: ex(mxang3)

    integer :: ian,jan,kan,lan
    real(8) :: erithr

    integer :: i,i0,j0,k0,l0,ii,iam0,iam

    data (mm(i),i=1,11) /1,3,6,10,15,21,28,36,45,55,66/
    type(grad_table) :: self

    i0 = i_shell(1)
    j0 = i_shell(2)
    k0 = i_shell(3)
    l0 = i_shell(4)

    m4(1) = mm(ian)
    m4(2) = mm(jan)
    m4(3) = mm(kan)
    m4(4) = mm(lan)

    i4(1) = iaoslix(i0,2)
    i4(2) = iaoslix(j0,2)
    i4(3) = iaoslix(k0,2)
    i4(4) = iaoslix(l0,2)

    ex = 0.d0

    if (self%nprod == 0) goto 123
    do ii=1,self%nprod
      iam = self%index_prod(ii) 
      ex(:) = ex(:) + ec(:,iam)
    enddo
    
    ! ==== output ====
    iam = self%np_atom
    iam0 = i_atom(iam)

    call grad_output_core(ex,indexg,gval,mxang3,i4,m4,iam0,num,erithr)

    ! ==== output ====
123 if (self%nout == 0) goto 124

    ! ==== output ====
    do i=1,self%nout
      iam = self%index_out(i)
      iam0 = i_atom(iam)
      ex(:) = ec(:,iam)

      call grad_output_core(ex,indexg,gval,mxang3,i4,m4,iam0,num,erithr)

    enddo

    ! ==== output ====
124 ex = 0.d0
    do i=1,self%ngrad
      iam = self%index_grad(i)
      ex(:) = ex(:) - ec(:,iam)
    enddo

    ! ==== output ====
    iam = self%nd_atom
    iam0 = i_atom(iam)
    call grad_output_core(ex,indexg,gval,mxang3,i4,m4,iam0,num,erithr)


  end subroutine grad_output0

  subroutine grad_output4(gval,indexg,ec,mxang3,i_atom,num,self,i_shell,&
       ian,jan,kan,lan,erithr,iaoslix,nshell)

    integer :: nshell,mxang3
    integer :: iaoslix(nshell,3)
    integer(2) :: indexg(6,mxang3*12)
    real(8) :: gval(mxang3*12),ec(mxang3,3)
    integer :: i_shell(4),num

    integer :: i_atom(4),mm(11),m4(4),i4(4)
    real(8) :: ex(mxang3)
    integer :: ian,jan,kan,lan
    real(8) :: erithr

    integer :: i,i0,j0,k0,l0,ii,iam0,iam
    data (mm(i),i=1,11) /1,3,6,10,15,21,28,36,45,55,66/

    type(grad_table) :: self
    i0 = i_shell(1)
    j0 = i_shell(2)
    k0 = i_shell(3)
    l0 = i_shell(4)

    m4(1) = mm(ian)
    m4(2) = mm(jan)
    m4(3) = mm(kan)
    m4(4) = mm(lan)

    i4(1) = iaoslix(i0,2)
    i4(2) = iaoslix(j0,2)
    i4(3) = iaoslix(k0,2)
    i4(4) = iaoslix(l0,2)

    ex = 0.d0

    if (self%nprod == 0) goto 123
    do ii=1,self%nprod
      iam = self%index_prod(ii) 
      ex(:) = ex(:) + ec(:,iam)
    enddo

    ! ==== output ====
    iam = self%np_atom
    iam0 = i_atom(iam)

    call grad_output_core4(ex,indexg,gval,mxang3,i4,m4,iam0,num,erithr)
    ! ==== output ====

123 if (self%nout == 0) goto 124

    ! ==== output ====
    do i=1,self%nout
      iam = self%index_out(i)
      iam0 = i_atom(iam)
      ex(:) = ec(:,iam)
      call grad_output_core4(ex,indexg,gval,mxang3,i4,m4,iam0,num,erithr)

    enddo
    ! ==== output ====
124 ex = 0.d0

    do i=1,self%ngrad
      iam = self%index_grad(i)
      ex(:) = ex(:) - ec(:,iam)
    enddo

    ! ==== output ====
    iam = self%nd_atom
    iam0 = i_atom(iam)

    call grad_output_core4(ex,indexg,gval,mxang3,i4,m4,iam0,num,erithr)

  end subroutine grad_output4

  subroutine grad_output1(gval,indexg,ec,mxang3,i_atom,num,self,i_shell,&
       ian,jan,kan,lan,erithr,iaoslix,nshell)

    integer :: nshell,mxang3
    integer :: iaoslix(nshell,3)
    integer(2) :: indexg(6,mxang3*12)
    real(8) :: gval(mxang3*12),ec(mxang3,3)
    integer :: i_shell(4),num

    integer :: i_atom(4),mm(11),m4(4),i4(4)
    real(8) :: ex(mxang3)
    integer :: ian,jan,kan,lan
    real(8) :: erithr


    integer :: i,i0,j0,k0,l0,ii,iam0,iam
    data (mm(i),i=1,11) /1,3,6,10,15,21,28,36,45,55,66/

    type(grad_table) :: self
    i0 = i_shell(1)
    j0 = i_shell(2)
    k0 = i_shell(3)
    l0 = i_shell(4)

    m4(1) = mm(ian)
    m4(2) = mm(jan)
    m4(3) = mm(kan)
    m4(4) = mm(lan)

    i4(1) = iaoslix(i0,2)
    i4(2) = iaoslix(j0,2)
    i4(3) = iaoslix(k0,2)
    i4(4) = iaoslix(l0,2)

    ex = 0.d0

    if (self%nprod == 0) goto 123
    do ii=1,self%nprod
      iam = self%index_prod(ii) 
      ex(:) = ex(:) + ec(:,iam)
    enddo

    ! ==== output ====
    iam = self%np_atom
    iam0 = i_atom(iam)

    call grad_output_core1(ex,indexg,gval,mxang3,i4,m4,iam0,num,erithr)
    ! ==== output ====

123 if (self%nout == 0) goto 124

    ! ==== output ====
    do i=1,self%nout
      iam = self%index_out(i)
      iam0 = i_atom(iam)
      ex(:) = ec(:,iam)
      call grad_output_core1(ex,indexg,gval,mxang3,i4,m4,iam0,num,erithr)

    enddo
    ! ==== output ====
124 ex = 0.d0

    do i=1,self%ngrad
      iam = self%index_grad(i)
      ex(:) = ex(:) - ec(:,iam)
    enddo

    ! ==== output ====
    iam = self%nd_atom
    iam0 = i_atom(iam)

    call grad_output_core1(ex,indexg,gval,mxang3,i4,m4,iam0,num,erithr)

  end subroutine grad_output1

  subroutine grad_output2(gval,indexg,ec,mxang3,i_atom,num,self,i_shell,&
       ian,jan,kan,lan,erithr,iaoslix,nshell)

    integer :: nshell,mxang3
    integer :: iaoslix(nshell,3)
    integer(2) :: indexg(6,mxang3*12)
    real(8) :: gval(mxang3*12),ec(mxang3,3)
    integer :: i_shell(4),num

    integer :: i_atom(4),mm(11),m4(4),i4(4)
    real(8) :: ex(mxang3)
    integer :: ian,jan,kan,lan
    real(8) :: erithr

    integer :: i,i0,j0,k0,l0,ii,iam0,iam

    data (mm(i),i=1,11) /1,3,6,10,15,21,28,36,45,55,66/
    type(grad_table) :: self

    i0 = i_shell(1)
    j0 = i_shell(2)
    k0 = i_shell(3)
    l0 = i_shell(4)

    m4(1) = mm(ian)
    m4(2) = mm(jan)
    m4(3) = mm(kan)
    m4(4) = mm(lan)

    i4(1) = iaoslix(i0,2)
    i4(2) = iaoslix(j0,2)
    i4(3) = iaoslix(k0,2)
    i4(4) = iaoslix(l0,2)

    ex = 0.d0

    if (self%nprod == 0) goto 123
    do ii=1,self%nprod
      iam = self%index_prod(ii) 
      ex(:) = ex(:) + ec(:,iam)
    enddo

    ! ==== output ====
    iam = self%np_atom
    iam0 = i_atom(iam)
    call grad_output_core2(ex,indexg,gval,mxang3,i4,m4,iam0,num,erithr)
    ! ==== output ====
123 if (self%nout == 0) goto 124

    ! ==== output ====
    do i=1,self%nout
      iam = self%index_out(i)
      iam0 = i_atom(iam)
      ex(:) = ec(:,iam)
      call grad_output_core2(ex,indexg,gval,mxang3,i4,m4,iam0,num,erithr)
    enddo
    ! ==== output ====
124 ex = 0.d0
    do i=1,self%ngrad
      iam = self%index_grad(i)
      ex(:) = ex(:) - ec(:,iam)
    enddo

    ! ==== output ====
    iam = self%nd_atom
    iam0 = i_atom(iam)
    call grad_output_core2(ex,indexg,gval,mxang3,i4,m4,iam0,num,erithr)

  end subroutine grad_output2

  subroutine grad_output3(gval,indexg,ec,mxang3,i_atom,num,self,i_shell,&
       ian,jan,kan,lan,erithr,iaoslix,nshell)

    integer :: nshell,mxang3
    integer :: iaoslix(nshell,3)
    integer(2) :: indexg(6,mxang3*12)
    real(8) :: gval(mxang3*12),ec(mxang3,3)
    integer :: i_shell(4),num

    integer :: i_atom(4),mm(11),m4(4),i4(4)
    real(8) :: ex(mxang3)
    integer :: ian,jan,kan,lan
    real(8) :: erithr


    integer :: i,i0,j0,k0,l0,ii,iam0,iam
    data (mm(i),i=1,11) /1,3,6,10,15,21,28,36,45,55,66/
    type(grad_table) :: self

    i0 = i_shell(1)
    j0 = i_shell(2)
    k0 = i_shell(3)
    l0 = i_shell(4)

    m4(1) = mm(ian)
    m4(2) = mm(jan)
    m4(3) = mm(kan)
    m4(4) = mm(lan)

    i4(1) = iaoslix(i0,2)
    i4(2) = iaoslix(j0,2)
    i4(3) = iaoslix(k0,2)
    i4(4) = iaoslix(l0,2)

    ex = 0.d0

    if (self%nprod == 0) goto 123
    do ii=1,self%nprod
      iam = self%index_prod(ii) 
      ex(:) = ex(:) + ec(:,iam)
    enddo

    ! ==== output ====
    iam = self%np_atom
    iam0 = i_atom(iam)
    call grad_output_core3(ex,indexg,gval,mxang3,i4,m4,iam0,num,erithr)
    ! ==== output ====
123 if (self%nout == 0) goto 124

    ! ==== output ====
    do i=1,self%nout
      iam = self%index_out(i)
      iam0 = i_atom(iam)
      ex(:) = ec(:,iam)
      call grad_output_core3(ex,indexg,gval,mxang3,i4,m4,iam0,num,erithr)

    enddo
    ! ==== output ====
124 ex = 0.d0
    do i=1,self%ngrad
      iam = self%index_grad(i)
      ex(:) = ex(:) - ec(:,iam)
    enddo

    ! ==== output ====
    iam = self%nd_atom
    iam0 = i_atom(iam)
    call grad_output_core3(ex,indexg,gval,mxang3,i4,m4,iam0,num,erithr)

  end subroutine grad_output3

  subroutine set_graderi_index(ii,jj,kk,ll,self,ifn)

    integer,allocatable :: nhgp(:,:,:),index_hgp(:,:,:),nang2n(:,:)
    integer,allocatable :: nang2count(:,:)
    integer :: ii,jj,kk,ll
    !==
    integer ::  index_geri(4,100),mm(7),ifn(6)
    integer ::  idz2dir(15),indir(3,55)!,self%index_hrr_range(2,7)
    integer ::  nindex(6),ind_set_hgp(4,40,6)!dummy

    integer :: init(4)

    integer :: ian,n,ny,nz,nx,i0,lan0,kan0
    integer :: id0,id1,id2,id3,ihe,ihs,ikey,im1,im2,im3
    integer :: imx,imy,imz,ix,iy,j0,jan,jang,iss,icount
    integer :: jd0,jd1,jd2,jhs,jhe,jcount,ijang
    integer :: ic_geri1,ic_geri2,ic_geri
    integer :: key,kang,k0,k,jkey,iang,kan,kcount,key_m,key_vrr
    integer :: key_vrr0,key_vrr1,key_vrr2,klang
    integer :: m,m0,m1,m2,m3,m4,m5,i,j
    integer :: ms_vrr0,ms_vrr1,ms_vrr,mho,maang
    integer :: l,lan,ms_vr10,ms_vrr2,ms_vrr20,ms_vrr10,mx,my,mz

    character :: ang(10)
    data (ang(i),i=1,10)/'s','p','d','f','g','h','i','j','k','l'/
    data mm /1,3,6,10,15,21,28/

    type(grad_index) :: self
!   print *,"ian = ",ii,jj,kk,ll
    init(1) = ii
    init(2) = jj
    init(3) = kk
    init(4) = ll
    
    index_geri = 0
    ind_set_hgp = 0
    nindex = 0

    ijang = init(1) + init(2)
    klang = init(3) + init(4)
    maang = ijang + klang 

    self%mxang3 = mm(init(1)) * mm(init(2)) *mm(init(3)) *mm(init(4)) * 3
    !-----------------------------------------
    !-----------------------------------------
    ifn = 0
    ifn(1) = mm(init(1)+1) * mm(init(2)  ) * mm(init(3)  ) * mm(init(4))
    ifn(2) = mm(init(1)  ) * mm(init(2)+1) * mm(init(3)  ) * mm(init(4))
    ifn(3) = mm(init(1)  ) * mm(init(2)  ) * mm(init(3)+1) * mm(init(4))

    if(init(1)-1>=1) ifn(4)=mm(init(1)-1)*mm(init(2))*mm(init(3))*mm(init(4))
    if(init(2)-1>=1) ifn(5)=mm(init(1))*mm(init(2)-1)*mm(init(3))*mm(init(4))
    if(init(3)-1>=1) ifn(6)=mm(init(1))*mm(init(2))*mm(init(3)-1)*mm(init(4))
    !-----------------------------------------
    !-----------------------------------------

    allocate(index_hgp(ijang,klang,6),nhgp(ijang,klang,0:maang-3))
    allocate(nang2n(ijang,klang),nang2count(ijang,klang))

    allocate(self%num_hgp_type(6))
    allocate(self%index_hrr_range(2,7))

    self%num_hgp_type = 0
    self%index_hrr_range = 0

    nhgp = 0
    index_hgp = 0
    nang2n = 0
    nang2count = 0

    do n=1,6
      ian = init(1)
      jan = init(2)
      kan = init(3)
      lan = init(4)

      select case(n)
      case(1)
        ian = init(1) + 1
      case(2)
        jan = init(2) + 1
      case(3)
        kan = init(3) + 1
      case(4)
        ian = init(1) - 1
        if (ian <= 0) cycle
      case(5)
        jan = init(2) - 1
        if (jan <= 0) cycle
      case(6)
        kan = init(3) - 1
        if (kan <= 0) cycle
      end select

      do i = ian,ian+jan-1
        do j = kan,kan+lan -1
          index_hgp(i,j,n) = 1
          nhgp(i,j,0) = 1
        enddo
      enddo

    enddo

    ian = init(1)
    jan = init(2)
    kan = init(3)
    lan = init(4)

    do i=1,ian+jan
      nhgp(i,1,0:ian+jan+kan+lan-i-2) = 1
    enddo

    ! goto 11

    do i = ian+jan,1,-1
      do j = kan+lan,1,-1
        key = nhgp(i,j,0)
        if (key /= 0) then

          do k = j-1,1,-1
            nhgp(i,k,0:j-k) = 1
          enddo

          do k = 1,min(i,j)-1
            nhgp(i-k,j-k,k) = 1
            i0 = i-k
            j0 = j-k
            do l = j0,1,-1
              nhgp(i0,l,k:k+j0-l) = 1
            enddo

          enddo

          exit
        endif
      enddo
    enddo

    !==========================================
    self%max_vrr_eri = 0
    self%mx_vrr0 = 0
    self%mx_vrr1 = 0
    self%mx_vrr2 = 0

    do j=1,klang
      do i=1,ijang
        do n=1,6
          if (index_hgp(i,j,n) ==1) then
            self%num_hgp_type(n) = self%num_hgp_type(n) + 1
            ind_set_hgp(1,self%num_hgp_type(n),n) = self%mx_vrr0+1
            ind_set_hgp(2,self%num_hgp_type(n),n) = self%mx_vrr0+(i*(i+1)/2)*(j*(j+1)/2)

            ind_set_hgp(3,self%num_hgp_type(n),n) = i
            ind_set_hgp(4,self%num_hgp_type(n),n) = j
          endif
        enddo

        key_vrr0 = 0
        do m =0, maang-3
          if (nhgp(i,j,m) == 1) then
            key_vrr0= 1
            self%mx_vrr0 = self%mx_vrr0 + (i*(i+1)/2) * (j*(j+1)/2)
            if ((j == 1).and.(i > 2)) then
              self%mx_vrr1 = self%mx_vrr1 + (i*(i-1)/2)
              cycle
            endif

            if (j <= 1) cycle
            if (i >= 2) then
              self%mx_vrr2 = self%mx_vrr2 + (i*(i-1)/2) * (j*(j+1)/2)
            endif

            if (j >= 3) then
              self%mx_vrr1 = self%mx_vrr1 + (i*(i+1)/2) * (j*(j-1)/2)
            endif

          endif
        enddo
        if (key_vrr0 == 1) self%max_vrr_eri = self%max_vrr_eri + 1
      enddo
    enddo

    self%max_hgp_eri = maxval(self%num_hgp_type)
    if (self%max_hgp_eri >40 ) print *,"ERROR" 

    ! ====== HGP INDEX SIZE ======
    !in : init,mm,self%num_hgp_type,index_set_hgp
    !out : self%index_hrr_range
    call set_hgp_range(init,self%index_hrr_range,ind_set_hgp,self%num_hgp_type,mm)
    ! ====== HGP INDEX SIZE ======

    ! print *,"size = ",self%max_vrr_eri
    ! print *,"size0= ",self%mx_vrr0
    ! print *,"size1= ",self%mx_vrr1
    ! print *,"size2= ",self%mx_vrr2
    ! print *,"max_hgpTYPE= ",self%max_hgp_eri
    ! ==== END --calc size-- END == 

    !+++++ allocate index table +++++
    self%min_nhgp = self%index_hrr_range(1,7)
    self%max_nhgp = self%index_hrr_range(2,7)

    allocate(self%index_vrr_range(6,self%max_vrr_eri))
    allocate(self%index_vrr0(3,self%mx_vrr0))
    allocate(self%index_vrr1(4,self%mx_vrr1))
    allocate(self%index_vrr2(3,self%mx_vrr2))
    allocate(self%index_set_hgp(4,self%max_hgp_eri,6))
    allocate(self%index_hrr(3,self%min_nhgp:self%max_nhgp,6))
    allocate(self%index_geri4(3,self%mxang3,4))

    self%index_vrr_range(1,:) = 0
    self%index_vrr_range(2,:) =-1
    self%index_vrr_range(3,:) = 0
    self%index_vrr_range(4,:) =-1
    self%index_vrr_range(5,:) = 0
    self%index_vrr_range(6,:) =-1
    self%index_vrr0      = 0
    self%index_vrr1      = 0
    self%index_vrr2      = 0
    self%index_hrr     = 0
    self%index_set_hgp = 0
    self%index_geri4 = 0
    !+++++ allocate index table +++++

    ! +++++++++++++ make index table +++++++++++++
    ! +++++++++++++ make index table +++++++++++++
    icount  = 0
    jcount  = 0
    kcount  = 0
    ms_vrr  = 0
    ms_vrr0 = 0

    ms_vrr10 = 0
    ms_vrr1 = 0
    ms_vrr20 = 0
    ms_vrr2 = 0

    m0 = 99

    ! ngeri = 0
    do j=1,klang
      do i=1,ijang
        do n=1,6
          if (index_hgp(i,j,n) ==1) then
            nindex(n) = nindex(n) + 1
            self%index_set_hgp(1,nindex(n),n) = ms_vrr+1
            self%index_set_hgp(2,nindex(n),n) = ms_vrr+(i*(i+1)/2)*(j*(j+1)/2)

            self%index_set_hgp(3,nindex(n),n) = i
            self%index_set_hgp(4,nindex(n),n) = j
          endif
        enddo

        key_vrr  = 0
        key_vrr1 = 0
        key_vrr2 = 0

        key_m = 0
        do m =0, maang-3
          if (nhgp(i,j,m) == 1) then
            !------

            key_vrr = 1

            ms_vrr = ms_vrr + (i*(i+1)/2) * (j*(j+1)/2)
            m1 = m

            if (key_m == 0) then
              m0 = m
              key_m = 1
            endif

            if ((j == 1).and.(i > 2)) then
              ms_vrr1 = ms_vrr1 + (i*(i-1)/2)
              key_vrr1 = 1
              cycle
            endif

            if (j <= 1) cycle
            !         if (i <= 1) cycle

            if (i >= 2) then
              ms_vrr2 = ms_vrr2 + (i*(i-1)/2) * (j*(j+1)/2)
              key_vrr2 = 1
            endif

            if (j >= 3) then
              ms_vrr1 = ms_vrr1 + (i*(i+1)/2) * (j*(j-1)/2)
              key_vrr1 = 1
            endif

          endif
        enddo

        if (key_vrr == 1) then
          icount = icount + 1
          self%index_vrr_range(1,icount) = ms_vrr0 + 1
          self%index_vrr_range(2,icount) = ms_vrr

          index_geri(1,icount) = i
          index_geri(2,icount) = j
          index_geri(3,icount) = m0
          index_geri(4,icount) = m1

          nang2n(i,j)  = ms_vrr0 + 1
          nang2count(i,j)  = icount

          ms_vrr0 =  ms_vrr
        endif

        if (key_vrr1 == 1) then
          jcount = jcount + 1
          self%index_vrr_range(3,icount) = ms_vrr10 + 1
          self%index_vrr_range(4,icount) = ms_vrr1 
          ms_vrr10 =  ms_vrr1
        endif

        if (key_vrr2 == 1) then
          kcount = kcount + 1
          self%index_vrr_range(5,icount) = ms_vrr20 + 1
          self%index_vrr_range(6,icount) = ms_vrr2 

          ms_vrr20 =  ms_vrr2
        endif

      enddo
    enddo

    if(icount /= self%max_vrr_eri) write(*,*)" -- ERROR -- "

    ! ===== set order calc VRR-eri ===

    ic_geri  = maang - 2
    ic_geri1 = 0
    ic_geri2 = 0

    do ix = 2,ijang!icount
      i0 = index_geri(1,ix)
      k0 = index_geri(2,ix)
      m0 = index_geri(3,ix)
      m1 = index_geri(4,ix)

      iang = i0 * (i0+1)/2
      kang = k0 * (k0+1)/2
      iss = self%index_vrr_range(1,ix-1)

      call set_ind_d2z(idz2dir,i0)

      do m = m0,m1
        do nx =1,1
          ic_geri = ic_geri + 1

          im1 = iss + m*mm(i0-1)
          im2 = im1 + mm(i0-1)
          im3 = 1

          self%index_vrr0(1,ic_geri) = im1
          self%index_vrr0(2,ic_geri) = im2
          self%index_vrr0(3,ic_geri) = im3

          if (i0 < 3)cycle
          ic_geri1 = ic_geri1 + 1

          id0 = ic_geri1
          id1 = i0-2
          id2 = self%index_vrr_range(1,i0-2)+m*mm(i0-2)
          id3 = self%index_vrr_range(1,i0-2)+m*mm(i0-2) + mm(i0-2)

          self%index_vrr1(1,id0) = ic_geri
          self%index_vrr1(2,id0) = id2
          self%index_vrr1(3,id0) = id3
          self%index_vrr1(4,id0) = id1
        enddo

        do ny =2,i0
          ic_geri = ic_geri + 1

          im1 = iss + m*mm(i0-1) + ny - 2
          im2 = im1 + mm(i0-1)
          im3 = 2

          self%index_vrr0(1,ic_geri) = im1
          self%index_vrr0(2,ic_geri) = im2
          self%index_vrr0(3,ic_geri) = im3 

          if (ny > 2) then
            ic_geri1 =  ic_geri1 + 1
            id0 = ic_geri1
            id1 = ny-2
            id2 = self%index_vrr_range(1,i0-2)+m*mm(i0-2) + id1 -1
            id3 = id2 + mm(i0-2)

            self%index_vrr1(1,id0) = ic_geri
            self%index_vrr1(2,id0) = id2
            self%index_vrr1(3,id0) = id3
            self%index_vrr1(4,id0) = id1
          endif

        enddo

        do nz =i0+1,iang
          ic_geri = ic_geri + 1

          im1 = iss + m*mm(i0-1) + nz -i0-1
          im2 = im1 + mm(i0-1)
          im3 = 3

          self%index_vrr0(1,ic_geri) = im1
          self%index_vrr0(2,ic_geri) = im2
          self%index_vrr0(3,ic_geri) = im3

          if (nz > i0+i0-1) then
            ic_geri1 =  ic_geri1 + 1
            id0 = ic_geri1
            id1 = idz2dir(nz-i0*2+1)
            id2 = self%index_vrr_range(1,i0-2)+m*mm(i0-2) + nz-i0*2
            id3 = id2 + mm(i0-2)
            self%index_vrr1(1,id0) = ic_geri
            self%index_vrr1(2,id0) = id2
            self%index_vrr1(3,id0) = id3
            self%index_vrr1(4,id0) = id1

          endif

        enddo
      enddo
      iss = ic_geri

    enddo

    do ix = ijang+1,self%max_vrr_eri
      i0 = index_geri(1,ix)
      k0 = index_geri(2,ix)
      m0 = index_geri(3,ix)
      m1 = index_geri(4,ix)

      iang = i0 * (i0+1)/2
      kang = k0 * (k0+1)/2
      iss = nang2n(i0,k0-1)

      call set_ind_d2z(idz2dir,k0)
      call set_ind_ndir(indir,i0,iang)

      mho = 0
      m4 = 0
      m5 = 0
      if (i0>1.and.k0>1)then
        m3 = index_geri(3,nang2count(i0-1,k0-1))
        mho = m3 - m0
      endif

      m4 = index_geri(3,nang2count(i0,k0-1))
      if (k0-2>1) m5 = index_geri(3,nang2count(i0,k0-2))

      do m = m0,m1
        imx = 0
        imy = 0
        imz = 0
        do iy = 1,iang

          mx = indir(1,iy)
          my = indir(2,iy)
          mz = indir(3,iy)

          do nx =1,1
            ic_geri = ic_geri + 1

            !         im1 = iss + (iy - 1)*mm(k0-1) + m*mm(i0)
            im1 = iss + (iy - 1)*mm(k0-1) + (m-m4)*mm(i0)*mm(k0-1)
            im2 = im1 + mm(k0-1) * mm(i0)
            im3 = 1
            self%index_vrr0(1,ic_geri) = im1
            self%index_vrr0(2,ic_geri) = im2
            self%index_vrr0(3,ic_geri) = im3

            if(mx >=1) then
              ic_geri2 = ic_geri2 + 1
              jd0 = ic_geri2
              jd1 = mx
              jd2=nang2n(i0-1,k0-1)+(m+1-mho)*mm(i0-1)*mm(k0-1)+imx*mm(k0-1)
              imx = imx + 1 

              self%index_vrr2(1,jd0) = ic_geri
              self%index_vrr2(2,jd0) = jd2
              self%index_vrr2(3,jd0) = jd1
            endif

            if (k0 < 3)cycle
            ic_geri1 = ic_geri1 + 1

            id0 = ic_geri1
            id1 = k0-2
            id2 = nang2n(i0,k0-2) + (m-m5)*mm(i0)*mm(k0-2) + (iy-1)*mm(k0-2)
            id3 =  id2 + mm(i0)*mm(k0-2)
            self%index_vrr1(1,id0) = ic_geri
            self%index_vrr1(2,id0) = id2
            self%index_vrr1(3,id0) = id3
            self%index_vrr1(4,id0) = id1

          enddo

          if (my >= 1) imy = imy + 1
          do ny =2,k0
            ic_geri = ic_geri + 1

            im1 = iss + (iy - 1)*mm(k0-1) + (m-m4)*mm(i0)*mm(k0-1) + ny - 2
            im2 = im1 + mm(k0-1) * mm(i0)
            im3 = 2

            self%index_vrr0(1,ic_geri) = im1
            self%index_vrr0(2,ic_geri) = im2
            self%index_vrr0(3,ic_geri) = im3

            if(my >=1) then
              ic_geri2 = ic_geri2 + 1
              jd0 = ic_geri2
              jd1 = my
              jd2=nang2n(i0-1,k0-1)+(m+1-mho)*mm(i0-1)*mm(k0-1)+ny-2+&
                   (imy-1)*mm(k0-1)
              self%index_vrr2(1,jd0) = ic_geri
              self%index_vrr2(2,jd0) = jd2
              self%index_vrr2(3,jd0) = jd1

            endif

            if (ny > 2) then
              ic_geri1 =  ic_geri1 + 1
              id0 = ic_geri1
              id1 = ny-2
              id2 = nang2n(i0,k0-2)+(m-m5)*mm(i0)*mm(k0-2)+(iy-1)*mm(k0-2)+ny-3
              id3 = id2 + mm(i0) * mm(k0-2)
              self%index_vrr1(1,id0) = ic_geri
              self%index_vrr1(2,id0) = id2
              self%index_vrr1(3,id0) = id3
              self%index_vrr1(4,id0) = id1

            endif

          enddo

          if (mz >= 1) imz = imz + 1
          do nz =k0+1,kang
            ic_geri = ic_geri + 1

            im1 = iss + (iy - 1)*mm(k0-1) + (m-m4)*mm(i0)*mm(k0-1) + nz - k0 - 1
            im2 = im1 + mm(k0-1) * mm(i0)
            im3 = 3

            self%index_vrr0(1,ic_geri) = im1
            self%index_vrr0(2,ic_geri) = im2
            self%index_vrr0(3,ic_geri) = im3

            if(mz >=1) then
              ic_geri2 = ic_geri2 + 1
              jd0 = ic_geri2
              jd1 = mz
              jd2=nang2n(i0-1,k0-1)+(m+1-mho)*mm(i0-1)*mm(k0-1)+nz-k0-1+(imz-1)*mm(k0-1)
              self%index_vrr2(1,jd0) = ic_geri
              self%index_vrr2(2,jd0) = jd2
              self%index_vrr2(3,jd0) = jd1

            endif

            if (nz > k0+k0-1) then
              ic_geri1 =  ic_geri1 + 1
              id0 = ic_geri1
              id1 = idz2dir(nz-k0*2+1)
              id2 = nang2n(i0,k0-2)+(m-m5)*mm(i0)*mm(k0-2)+(iy-1)*mm(k0-2)+nz-k0*2
              id3 = id2 + mm(i0) * mm(k0-2)
              self%index_vrr1(1,id0) = ic_geri
              self%index_vrr1(2,id0) = id2
              self%index_vrr1(3,id0) = id3
              self%index_vrr1(4,id0) = id1

            endif
          enddo

        enddo
      enddo
    enddo

    ! ==== HGP ===
    do j=1,6
      icount = self%num_hgp_type(j)
      ihs  = self%index_hrr_range(1,j)
      ihe  = self%index_hrr_range(2,j)

      if (ihe - ihs < 0) cycle

      call jthgp(init,icount,mm,self%index_hrr,self%index_set_hgp, &
           j,self%min_nhgp,self%max_nhgp,self%max_hgp_eri)

    enddo

    !-------  calc gradient -----
    call set_index_geri4(self%index_geri4,self%mxang3,init,mm)
    !-------  calc gradient -----


    do i=1,3
      ihs  = self%index_hrr_range(1,i)
      ihe  = self%index_hrr_range(2,i)
      jhs  = self%index_hrr_range(1,i+3)
      jhe  = self%index_hrr_range(2,i+3)
      ikey = 0
      jkey = 0

      !   if ( ihe-ihs > 1 ) ikey = ihs-1
      !   if ( jhe-jhs > 1 ) jkey = jhs-1

      if ( ihe-ihs > 1 ) ikey = ihe-ifn(i)
      if ( jhe-jhs > 1 ) jkey = jhe-ifn(i+3)

      self%index_geri4(1,:,i) = self%index_geri4(1,:,i) + ikey
      self%index_geri4(2,:,i) = self%index_geri4(2,:,i) + jkey
!     print *,i,self%index_geri4(1,1,i), ihe
!     print *,i,self%index_geri4(2,1,i),jhe
    enddo

!   print *,"---------- OUTPUT ----------"
!   print *,"ba,d1,d1",icount,jcount,kcount!,ngeri
!   print *,"ms_vrr,ms_vrr1,ms_vrr2 = ",ms_vrr,ms_vrr1,ms_vrr2
!   print "(a15,6i4)"," num_hgp_type = ",self%num_hgp_type(:)

    do i = 1,4
      do j = 1,self%mxang3
        if (  self%index_geri4(3,j,i) == 0 ) self%index_geri4(2,j,i) = 1
      enddo
    enddo

    deallocate ( nhgp, index_hgp, nang2n )
    deallocate ( nang2count )

  end subroutine set_graderi_index

  subroutine set_ind_d2z(idz2dir,i0)
    integer :: i0
    integer :: idz2dir((i0-2)*(i0-1)/2)
    integer :: ic,ix,i,j
    idz2dir = 0
    ix = i0-2
    ic = 0 
    do i = ix,1,-1
      do j = 1,i
        ic = ic + 1
        idz2dir(ic) = ix+1 - i
      enddo
    enddo

  end subroutine set_ind_d2z

  subroutine set_ind_ndir(indir,i0,iang)
    integer :: i0,iang
    integer :: indir(3,iang)

    integer :: ic,i,icc,j
    indir = 0
    ic = i0

    do i=1,1
      indir(1,i) = i0-1
    enddo

    do i=2,i0
      indir(2,i) = i-1
      indir(1,i) = i0-i
    enddo

    do i=i0-1,1,-1
      icc = 0
      do j=1,i
        ic = ic + 1
        indir(2,ic) = icc
        indir(3,ic) = i0-1+1 - i
        indir(1,ic) = i0-1 - (icc + (i0-i))
        icc = icc + 1

      enddo
    enddo
  end subroutine set_ind_ndir

  subroutine set_hgp_range(init,index_hrr_range,index_set_hgp,num_hgp_type,mm)
    integer :: init(4)
    integer :: index_hrr_range(2,7)
    integer :: index_set_hgp(4,40,6)
    integer :: num_hgp_type(6),mm(7)

    integer :: j,i,icount,ian0,jan0,kan0,lan0,jcount,imin0,imax0
    integer :: imin,imax,idd

    index_hrr_range = 0
    ian0 = init(1) 
    jan0 = init(2) 
    kan0 = init(3) 
    lan0 = init(4)

    do j=1,6
      icount = 0
      do i=1,num_hgp_type(j)
        icount = icount + index_set_hgp(2,i,j) - index_set_hgp(1,i,j) + 1
      enddo
      index_hrr_range(1,j) = icount + 1
      select case(j)
      case(1)
        init(1) = init(1) + 1
      case(2)
        init(2) = init(2) + 1
      case(3)
        init(3) = init(3) + 1
      case(4)
        init(1) = init(1) - 1
      case(5)
        init(2) = init(2) - 1
      case(6)
        init(3) = init(3) - 1
      endselect
      call jtshgp(init,mm,jcount)

      index_hrr_range(2,j) = icount + jcount

      init(1) = ian0
      init(2) = jan0
      init(3) = kan0
      init(4) = lan0
    enddo

    imin0 = maxval(index_hrr_range(1,1:6))
    imax0 = 0

    do i=1,6
      imin = index_hrr_range(1,i)
      imax = index_hrr_range(2,i)

      idd = imax - imin

      if (idd < 0) cycle
      if (imax > imax0 ) imax0 = imax
      if (imin < imin0 ) imin0 = imin

    enddo

    index_hrr_range(1,7) = imin0
    index_hrr_range(2,7) = imax0

  end subroutine set_hgp_range

end module integral_hgp_table_class
