! -*- coding: utf-8 -*-
!
! Dual licensed under the MIT or GPL Version 3 license.
! See 'LICENSE.MIT' for the definition of the MIT license,
! and 'COPYING' for the definition of the GPL license.
!
!> 積分オプションクラス
!! 
module integral_options_class
  implicit none
  private

  public::options_type
  public::read_options
  public::save_options
  public::load_options
  public::bcast_options
  public::free_options
  public::print_options
  public::QM_ZERODIPOLE_NO
  public::QM_ZERODIPOLE_METHOD1
  public::QM_ZERODIPOLE_METHOD2
  public::QM_ZERODIPOLE_METHOD3
!!$  public::QM_RC_UNIT_ANG
!!$  public::QM_RC_UNIT_AU

  type::options_type
     real(8)::thresh_ovl
     real(8)::thresh_eri
     !logical::l_incore
     integer(8)::incoremem
     integer(8)::mem_limit
     !integer::incore_p
     !integer::incore_q
     integer::zerodipole
     real(8)::rcut !< in Angstrom
     real(8)::alpha
  end type

  character(len=*),parameter::group_name = 'INTEGRAL' !< データグループ名称

  integer,parameter::MAX_OPTION_NAME_LEN = 16
  integer,parameter::nopt = 6                               !< オプション数
  character(len=MAX_OPTION_NAME_LEN)::opt_names(nopt) = (/& !< オプションの名称
       'OVLCUT    ',&
       'ERITHRES  ',&
       'INCOREMEM ',&
       'ZERODIPOLE',&
       'RC        ',&
       'ALPHA     '/)

  integer,parameter::QM_ZERODIPOLE_NO      = 0
  integer,parameter::QM_ZERODIPOLE_METHOD1 = 1 !< only diagonal
  integer,parameter::QM_ZERODIPOLE_METHOD2 = 2 !< and off-diagonal
  integer,parameter::QM_ZERODIPOLE_METHOD3 = 3 !< 
  integer,parameter::QM_ZERODIPOLE_NUM = QM_ZERODIPOLE_METHOD3 + 1
  character(len=*),parameter::QM_ZERODIPOLE(QM_ZERODIPOLE_NUM) = (/&
       'NO     ',&
       'METHOD1',&
       'METHOD2',&
       'METHOD3'/)

  character(len=*),parameter::url = 'options_integral' !< BMSファイル名称
  integer,parameter::id = 1
  character(len=*),parameter::no_unit = ' '

contains
  !-----------------------------------------------------------------------------
  !> オプションの表示
  subroutine print_options(self)
    type(options_type),intent(in)::self

    write(6,'(/a,'' Options'')') trim(group_name)
    write(6,'(2x,a,'' : '',1p,d10.3)') 'ERITHRES ',self%thresh_eri
!!$    write(6,'(2x,a,'' : '',1p,d10.3)') 'OVLCUT   ',self%thresh_ovl
!!$    write(6,'(2x,a,'' : '',i6)')       'INCOREMEM',self%incoremem
    write(6,'(2x,a,'' : '',a)') 'ZERODIPOLE  ',&
         trim(QM_ZERODIPOLE(self%zerodipole+1))
    if (self%zerodipole >= QM_ZERODIPOLE_METHOD1) then
    write(6,'(2x,a,'' : '',f10.3)')    'RC       ',self%rcut
    write(6,'(2x,a,'' : '',f10.3)')    'ALPHA    ',self%alpha
    end if
    !write(6,'('' Threshold of ERIs ='',1p,d10.3)') self%thresh_eri

  end subroutine print_options

  !-----------------------------------------------------------------------------
  !> オプションの入力ファイルからの読み込み
  subroutine read_options(self, lines, nline, ldebug, istat)
    use input_class
    type(options_type),intent(inout)::self
    integer,intent(in)::nline
    character(len=*),intent(in)::lines(nline)
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    character(len=MAX_OPTION_VALUE_LEN)::opt_values(nopt)!< オプションの値
    integer::opt_count(nopt)                             !< オプションの出現回数
    character(len=MAX_OPTION_NAME_LEN)::opt_name
    character(len=MAX_OPTION_VALUE_LEN)::opt_value
    integer::iopt, ival, ipos, ival_default, ipos_default
    real(8)::dval, dval_default
    character(len=MAX_OPTION_VALUE_LEN)::str, str_default

    integer::nerror, ierror

    call get_options(&
       lines, nline,&
       group_name, opt_names, opt_values, opt_count, nopt, ldebug, istat)
    if (istat /= 0) return

    nerror = 0
    ierror = 0

    opt_name = 'ovlcut'
    dval_default = 0.d0
    call get_option_values_dbl(&
         opt_name, opt_names, opt_values, nopt, dval_default, dval, istat)
    if (istat /= 0) then
       nerror = nerror + 1; ierror = istat
    else
       if (dval < 0.0d0) then
          istat = PL_ERR_OUT_OF_RANGE
          call pl_error_print_message(istat)
          iopt = pl_str_search_name_ignorecase(opt_name, opt_names, nopt)
          call print_error_invalid_value(opt_names(iopt), opt_values(iopt))
          nerror = nerror + 1; ierror = istat
       end if
    end if
    self%thresh_ovl = dval

    opt_name = 'erithres'
    dval_default = 1.d-8
    call get_option_values_dbl(&
         opt_name, opt_names, opt_values, nopt, dval_default, dval, istat)
    if (istat /= 0) then
       nerror = nerror + 1; ierror = istat
    else
       if (dval < 0.0d0) then
          istat = PL_ERR_OUT_OF_RANGE
          call pl_error_print_message(istat)
          iopt = pl_str_search_name_ignorecase(opt_name, opt_names, nopt)
          call print_error_invalid_value(opt_names(iopt), opt_values(iopt))
          nerror = nerror + 1; ierror = istat
       end if
    end if
    self%thresh_eri = dval

    opt_name = 'incoremem'
    ival_default = 0 !512 Mbyte
    call get_option_values_int(&
         opt_name, opt_names, opt_values, nopt, ival_default, ival, istat)
    if (istat /= 0) then
       nerror = nerror + 1; ierror = istat
    else
       if (ival < 0) then
          istat = PL_ERR_OUT_OF_RANGE
          call pl_error_print_message(istat)
          iopt = pl_str_search_name_ignorecase(opt_name, opt_names, nopt)
          call print_error_invalid_value(opt_names(iopt), opt_values(iopt))
          nerror = nerror + 1; ierror = istat
       end if
    end if
    self%incoremem = ival
    self%mem_limit = ival * 125000 !* (10**6) / 8

    opt_name = 'zerodipole'
    ipos_default = pl_str_search_name_ignorecase(&
         'NO', QM_ZERODIPOLE, QM_ZERODIPOLE_NUM)
    call get_option_values_strlist(&
         opt_name, opt_names, opt_values, nopt,&
         QM_ZERODIPOLE, QM_ZERODIPOLE_NUM, ipos_default, ipos, istat)
    if (istat /= 0) then
       nerror = nerror + 1; ierror = istat
    end if
    self%zerodipole = ipos - 1
    !write(6,*) 'self%zerodipole=',self%zerodipole

    opt_name = 'rc'
    dval_default = 16.0d0
    call get_option_values_dbl(&
         opt_name, opt_names, opt_values, nopt, dval_default, dval, istat)
    if (istat /= 0) then
       nerror = nerror + 1; ierror = istat
    else
       if (dval < 0.0d0) then
          istat = PL_ERR_OUT_OF_RANGE
          call pl_error_print_message(istat)
          iopt = pl_str_search_name_ignorecase(opt_name, opt_names, nopt)
          call print_error_invalid_value(opt_names(iopt), opt_values(iopt))
          nerror = nerror + 1; ierror = istat
       end if
    end if
    self%rcut = dval

    opt_name = 'alpha'
    dval_default = 0.0d0
    call get_option_values_dbl(&
         opt_name, opt_names, opt_values, nopt, dval_default, dval, istat)
    if (istat /= 0) then
       nerror = nerror + 1; ierror = istat
    else
       if (dval < 0.0d0) then
          istat = PL_ERR_OUT_OF_RANGE
          call pl_error_print_message(istat)
          iopt = pl_str_search_name_ignorecase(opt_name, opt_names, nopt)
          call print_error_invalid_value(opt_names(iopt), opt_values(iopt))
          nerror = nerror + 1; ierror = istat
       end if
    end if
    self%alpha = dval

    if (nerror /= 0) then
       istat = ierror; return
    end if

  end subroutine read_options

  !-----------------------------------------------------------------------------
  !> オプションのBMS形式ファイルへの保存
  subroutine save_options(self, ldebug, istat)
    use pl_bms_class
    type(options_type),intent(in)::self
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    call pl_bms_write(self%thresh_ovl, url, 'ovlcut', id, no_unit, istat)
    if (istat /= 0) return
    call pl_bms_write(self%thresh_eri, url, 'erithres', id, no_unit, istat)
    if (istat /= 0) return
    call pl_bms_write(self%mem_limit, url, 'incoremem', id, no_unit, istat)
    if (istat /= 0) return
    call pl_bms_write(self%zerodipole, url, 'zerodipole', id, no_unit, istat)
    if (istat /= 0) return
    call pl_bms_write(self%rcut, url, 'rc', id, no_unit, istat)
    if (istat /= 0) return
    call pl_bms_write(self%alpha, url, 'alpha', id, no_unit, istat)
    if (istat /= 0) return

  end subroutine save_options

  !-----------------------------------------------------------------------------
  !> オプションのBMS形式ファイルからの読み込み
  subroutine load_options(self, ldebug, istat)
    use pl_bms_class
    type(options_type),intent(inout)::self
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    call pl_bms_read(self%thresh_ovl, url, 'ovlcut', id, istat)
    if (istat /= 0) return
    call pl_bms_read(self%thresh_eri, url, 'erithres', id, istat)
    if (istat /= 0) return
    call pl_bms_read(self%mem_limit, url, 'incoremem', id, istat)
    if (istat /= 0) return
    call pl_bms_read(self%zerodipole, url, 'zerodipole', id, istat)
    if (istat /= 0) return
    call pl_bms_read(self%rcut, url, 'rc', id, istat)
    if (istat /= 0) return
    call pl_bms_read(self%alpha, url, 'alpha', id, istat)
    if (istat /= 0) return

  end subroutine load_options

  !-----------------------------------------------------------------------------
  !> オプションのBcast
  subroutine bcast_options(self, pl_mpi, ldebug, istat)
    use pl_mpi_class
    type(options_type),intent(inout)::self
    type(pl_mpi_type),intent(in)::pl_mpi
    integer,intent(in)::ldebug
    integer,intent(inout)::istat
    integer::icomm

    icomm = pl_mpi%comm

    call mpi_bcast(self%thresh_ovl, 1, MPI_DOUBLE_PRECISION, 0, icomm, istat)
    if (istat .ne. 0) return
    call mpi_bcast(self%thresh_eri, 1, MPI_DOUBLE_PRECISION, 0, icomm, istat)
    if (istat .ne. 0) return
    call mpi_bcast(self%mem_limit, 1, MPI_INTEGER8, 0, icomm, istat)
    if (istat .ne. 0) return
    call mpi_bcast(self%zerodipole, 1, MPI_INTEGER, 0, icomm, istat)
    if (istat .ne. 0) return
    call mpi_bcast(self%rcut, 1, MPI_DOUBLE_PRECISION, 0, icomm, istat)
    if (istat .ne. 0) return
    call mpi_bcast(self%alpha, 1, MPI_DOUBLE_PRECISION, 0, icomm, istat)
    if (istat .ne. 0) return

  end subroutine bcast_options

  !-----------------------------------------------------------------------------
  !> 配列開放
  subroutine free_options(self, ldebug, istat)
    type(options_type),intent(in)::self
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    istat = 0
  end subroutine free_options

  !-----------------------------------------------------------------------------
end module integral_options_class
