! -*- coding: utf-8 -*-
!
! Dual licensed under the MIT or GPL Version 3 license.
! See 'LICENSE.MIT' for the definition of the MIT license,
! and 'COPYING' for the definition of the GPL license.
!
!> 積分テーブルクラス
!!
module integral_tables_class
  implicit none
  private

  public::tables_type
  public::tables_new
  public::tables_prep
  public::tables_prep2
  public::tables_free
  public::tables_swz

  type::tables_type
     integer::natom
     integer::nasp
     integer::nshell
     integer::nao
     integer::mxsa
     integer::mxprm

     integer::nshelp
     integer::nprimp
     integer::mang

     integer,allocatable::natm2asp(:)
     integer,allocatable::nsh2ssh(:)
     integer,allocatable::nsh2atm(:)
     integer,allocatable::nsh2asp(:)
     integer,allocatable::nsh2sht(:)
     integer,allocatable::nsh2ao(:)
     integer,allocatable::nsh2sh(:)
     integer,allocatable::nao2sh(:)
     integer,allocatable::nao2dir(:)
     integer,allocatable::ntsht(:,:)
     integer,allocatable::ntprm(:,:)
     integer,allocatable::nastbl(:,:)

     integer,allocatable::npsh(:,:)
     integer,allocatable::npsh2pr(:)
     integer,allocatable::ishp(:)
     integer,allocatable::idxshp(:)

     real(8),allocatable::rzet(:) !1/za + zb
     real(8),allocatable::rzeth(:) !(1/za + zb)/2
     real(8),allocatable::ovl(:)  !overlap integral
     real(8),allocatable::xsi(:)  !zazb/za+zb
     real(8),allocatable::bdz(:)  !zb/za + zb
     real(8),allocatable::zet(:)  !za + zb
     real(8),allocatable::zdp(:)  !za + zb/pi

     integer::mxprim2
     integer,allocatable::nprims2(:)  ! (1:nshelp)
     real(8),allocatable::AB(:,:)     ! A-B, (1:3,1:nshelp)
     real(8),allocatable::oozeta(:,:) ! 1/zeta(1:mxprim2,1:nshelp)
     real(8),allocatable::Sab(:,:)    ! S_AB(1:mxprim2,1:nshelp)
     real(8),allocatable::P(:,:,:)    ! P(1:3,1:mxprim2,1:nshelp)
     real(8),allocatable::PA(:,:,:)   ! P-A, (1:3,1:mxprim2,1:nshelp)
  end type

contains
  !-----------------------------------------------------------------------------
  !> 配列確保
  subroutine tables_new(self, atoms, ldebug, istat)
    use atoms_class
    type(tables_type),intent(inout)::self
    type(atoms_type),intent(in)::atoms
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    integer::natom, nbasis, nshell, nao, mxshell, mxprim
    integer::nshelp

    istat = 0

    natom   = atoms%num
    nbasis  = atoms%basis%num
    mxshell = atoms%basis%mxshell
    mxprim  = atoms%basis%mxprim
    nshell  = atoms%basis%nshell
    nao     = atoms%basis%nao
    nshelp  = nshell * (nshell + 1) /2

    allocate(&
         self%natm2asp(natom),&
         self%nsh2ssh(nshell),&
         self%nsh2atm(nshell),&
         self%nsh2asp(nshell),&
         self%nsh2sht(nshell),&
         self%nsh2ao(nshell),&
         self%nao2sh(nao),&
         self%nao2dir(nao),&
         self%ntsht(mxshell,nbasis),&
         self%ntprm(mxshell,nbasis),&
         self%nastbl(2,nbasis),&
         stat=istat)
    if (istat /= 0) return
    allocate(&
         self%npsh(2,nshelp),& !not dealloc
         self%npsh2pr(nshelp+1),& !not dealloc
         self%ishp(6),& !not dealloc
         self%idxshp(nshell*(nshell+1)/2),& !not dealloc
         stat=istat)
    if (istat /= 0) return

    self%natom  = natom
    self%nasp   = nbasis
    self%mxsa   = mxshell
    self%mxprm  = mxprim
    self%nshell = nshell
    self%nao    = nao
    self%nshelp = nshelp

  end subroutine tables_new

  !-----------------------------------------------------------------------------
  !> 前処理
  subroutine tables_prep(self, atoms, ldebug, istat)
    use atoms_class
    use pl_prof_class
    type(tables_type),intent(inout)::self
    type(atoms_type),intent(in)::atoms
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    integer::natom, nbasis, nshell, nao, mxshell, mxprim
    integer::nshelp, nprimp

    istat = 0

    natom   = atoms%num
    nbasis  = atoms%basis%num
    mxshell = atoms%basis%mxshell
    mxprim  = atoms%basis%mxprim
    nshell  = atoms%basis%nshell
    nao     = atoms%basis%nao
    nshelp  = self%nshelp

    self%nastbl(1,:) = atoms%basis%ntshell(1:nbasis)
    self%nastbl(2,:) = atoms%basis%ntao(1:nbasis)
    self%ntsht(:,:)  = atoms%basis%ntshtype(1:mxshell,1:nbasis)
    self%ntprm(:,:)  = atoms%basis%ntprim(1:mxshell,1:nbasis)
!!$    self%natm2asp(:) = atoms%basis%atom2basis(1:natom)
    self%natm2asp(:) = atoms%atom2basis(1:natom)

    call p_set_nsh2ssh(&
         self%nsh2ssh, self%nastbl, self%natm2asp, nbasis, natom, nshell ,istat)
    if (istat /= 0) return

    call p_set_nsh2atm(&
         self%nsh2atm, self%nastbl, self%natm2asp, nbasis, natom, nshell ,istat)
    if (istat /= 0) return

    call p_set_nsh2asp(&
         self%nsh2asp, self%nsh2atm, self%natm2asp, natom, nshell ,istat)
    if (istat /= 0) return

    call p_set_nsh2sht(&
         self%nsh2sht, self%nsh2asp, self%nsh2ssh, self%ntsht,&
         nbasis, nshell, mxshell, istat)
    if (istat /= 0) return

    call p_set_nsh2ao(&
         self%nsh2ao, self%nsh2asp, self%nsh2ssh, self%ntsht,&
         nbasis, nshell, mxshell, nao, istat)  
    if (istat /= 0) return

    call p_set_nao2sh(&
         self%nao2sh, self%natm2asp, self%nsh2atm, self%nsh2ssh, self%ntsht,&
         nao, nbasis, natom, nshell, mxshell, istat )
    if (istat /= 0) return

    call p_set_nao2dir(&
         self%nao2dir, self%nsh2asp, self%nsh2ssh, self%ntsht,&
         nao, nbasis, nshell, mxshell, istat)
    if (istat /= 0) return

    call p_set_pairtable(&
         self%ishp, self%npsh, self%npsh2pr, nprimp,&
         self%nsh2sht, self%nsh2ssh,self%nsh2asp, self%ntprm,&
         nshell,mxshell,nbasis,nshelp,istat)
    if (istat /= 0) return

    self%nprimp = nprimp
    if (ldebug > 0) then
       write(6,*) '#nprimp,nshelp=',self%nprimp,nshelp
    end if

    allocate(self%rzet(self%nprimp)) !not dealloc
    allocate(self%ovl(self%nprimp)) !not dealloc
    allocate(self%xsi(self%nprimp)) !not dealloc
    allocate(self%bdz(self%nprimp)) !not dealloc
    allocate(self%zet(self%nprimp)) !not dealloc
    allocate(self%zdp(self%nprimp)) !not dealloc

    call pl_prof_region_begin('s:set_pairtable3')
    call p_set_pairtable3(&
         self%ishp, self%npsh, self%npsh2pr, self%nprimp,&
         self%nsh2sht, self%nsh2ssh,self%nsh2asp, self%ntprm,&
         self%rzet,self%ovl,self%xsi,self%bdz,self%zet,self%zdp,self%idxshp,&
         self%nsh2atm,atoms%xyz,atoms%basis%z,atoms%basis%c,&
         self%mxprm,self%mxsa,self%nasp,self%natom,&
         nshell,mxshell,nbasis,nshelp,ldebug,istat)
    if (istat /= 0) return
    call pl_prof_region_end('s:set_pairtable3')
    allocate(self%rzeth(self%nprimp)) !not dealloc
    self%rzeth(:) = self%rzet(:)/2.0d0

    call set_mang(self%mang,&
         self%nsh2ssh,self%nsh2asp,self%ntsht,nshell,mxshell,nbasis)

    allocate(self%AB(3,nshelp),& !not dealloc
         self%nprims2(nshelp)) !not dealloc
    call p_set_pairtable4(&
         self%AB, self%nprims2, self%mxprim2,&
         self%npsh, self%nsh2atm, self%npsh2pr, self%ovl, atoms%xyz,&
         self%natom, nshell, nshelp, self%nprimp, self%mxprm, ldebug)

    allocate(self%oozeta(self%mxprim2,nshelp),&!not dealloc
         self%Sab(self%mxprim2,nshelp),&!not dealloc
         self%P(3,self%mxprim2,nshelp),&!not dealloc
         self%PA(3,self%mxprim2,nshelp)) !not dealloc

    call p_set_pairtable5(&
         self%oozeta, self%Sab, self%P, self%PA,&
         self%AB, self%npsh, self%nsh2atm, self%npsh2pr,&
         self%ovl, self%bdz, self%zet, atoms%xyz,&
         self%natom, nshell, nshelp, self%nprimp, self%mxprim2, self%mxprm)

  end subroutine tables_prep

  !-----------------------------------------------------------------------------
  !> 前処理
  subroutine tables_prep2(self, atoms, ldebug, istat)
    use atoms_class
    use pl_prof_class
    type(tables_type),intent(inout)::self
    type(atoms_type),intent(in)::atoms
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    integer::natom, nbasis, nshell, nao, mxshell, mxprim
    integer::nshelp, nprimp

    istat = 0

    natom   = atoms%num
    nbasis  = atoms%basis%num
    mxshell = atoms%basis%mxshell
    mxprim  = atoms%basis%mxprim
    nshell  = atoms%basis%nshell
    nao     = atoms%basis%nao
    nshelp  = self%nshelp

    self%nastbl(1,:) = atoms%basis%ntshell(1:nbasis)
    self%nastbl(2,:) = atoms%basis%ntao(1:nbasis)
    self%ntsht(:,:)  = atoms%basis%ntshtype(1:mxshell,1:nbasis)
    self%ntprm(:,:)  = atoms%basis%ntprim(1:mxshell,1:nbasis)
!!$    self%natm2asp(:) = atoms%basis%atom2basis(1:natom)
    self%natm2asp(:) = atoms%atom2basis(1:natom)

    call p_set_nsh2ssh(&
         self%nsh2ssh, self%nastbl, self%natm2asp, nbasis, natom, nshell ,istat)
    if (istat /= 0) return

    call p_set_nsh2atm(&
         self%nsh2atm, self%nastbl, self%natm2asp, nbasis, natom, nshell ,istat)
    if (istat /= 0) return

    call p_set_nsh2asp(&
         self%nsh2asp, self%nsh2atm, self%natm2asp, natom, nshell ,istat)
    if (istat /= 0) return

    call p_set_nsh2sht(&
         self%nsh2sht, self%nsh2asp, self%nsh2ssh, self%ntsht,&
         nbasis, nshell, mxshell, istat)
    if (istat /= 0) return

    call p_set_nsh2ao(&
         self%nsh2ao, self%nsh2asp, self%nsh2ssh, self%ntsht,&
         nbasis, nshell, mxshell, nao, istat)  
    if (istat /= 0) return

    call p_set_nao2sh(&
         self%nao2sh, self%natm2asp, self%nsh2atm, self%nsh2ssh, self%ntsht,&
         nao, nbasis, natom, nshell, mxshell, istat )
    if (istat /= 0) return

    call p_set_nao2dir(&
         self%nao2dir, self%nsh2asp, self%nsh2ssh, self%ntsht,&
         nao, nbasis, nshell, mxshell, istat)
    if (istat /= 0) return

    call p_set_pairtable(&
         self%ishp, self%npsh, self%npsh2pr, nprimp,&
         self%nsh2sht, self%nsh2ssh,self%nsh2asp, self%ntprm,&
         nshell,mxshell,nbasis,nshelp,istat)
    if (istat /= 0) return

    self%nprimp = nprimp
    if (ldebug > 0) then
       write(6,*) '#nprimp,nshelp=',self%nprimp,nshelp
    end if

!!$    allocate(self%rzet(self%nprimp))
!!$    allocate(self%ovl(self%nprimp))
!!$    allocate(self%xsi(self%nprimp))
!!$    allocate(self%bdz(self%nprimp))
!!$    allocate(self%zet(self%nprimp))
!!$    allocate(self%zdp(self%nprimp))

    call pl_prof_region_begin('s:set_pairtable3')
    call p_set_pairtable3(&
         self%ishp, self%npsh, self%npsh2pr, self%nprimp,&
         self%nsh2sht, self%nsh2ssh,self%nsh2asp, self%ntprm,&
         self%rzet,self%ovl,self%xsi,self%bdz,self%zet,self%zdp,self%idxshp,&
         self%nsh2atm,atoms%xyz,atoms%basis%z,atoms%basis%c,&
         self%mxprm,self%mxsa,self%nasp,self%natom,&
         nshell,mxshell,nbasis,nshelp,ldebug,istat)
    if (istat /= 0) return
    call pl_prof_region_end('s:set_pairtable3')
!!$    allocate(self%rzeth(self%nprimp))
    self%rzeth(:) = self%rzet(:)/2.0d0

    call set_mang(self%mang,&
         self%nsh2ssh,self%nsh2asp,self%ntsht,nshell,mxshell,nbasis)

!!$    allocate(self%AB(3,nshelp),self%nprims2(nshelp))
    call p_set_pairtable4(&
         self%AB, self%nprims2, self%mxprim2,&
         self%npsh, self%nsh2atm, self%npsh2pr, self%ovl, atoms%xyz,&
         self%natom, nshell, nshelp, self%nprimp, self%mxprm, ldebug)

!!$   allocate(self%oozeta(self%mxprim2,nshelp), self%Sab(self%mxprim2,nshelp),&
!!$         self%P(3,self%mxprim2,nshelp),self%PA(3,self%mxprim2,nshelp))

    call p_set_pairtable5(&
         self%oozeta, self%Sab, self%P, self%PA,&
         self%AB, self%npsh, self%nsh2atm, self%npsh2pr,&
         self%ovl, self%bdz, self%zet, atoms%xyz,&
         self%natom, nshell, nshelp, self%nprimp, self%mxprim2, self%mxprm)

  end subroutine tables_prep2

  !-----------------------------------------------------------------------------
  !> 配列開放
  subroutine tables_free(self, ldebug, istat)
    type(tables_type),intent(inout)::self
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    istat = 0

    deallocate(&
         self%natm2asp, self%nsh2ssh, self%nsh2atm, self%nsh2asp,&
         self%nsh2sht, self%nsh2ao, self%nao2sh, self%nao2dir,&
         self%ntsht, self%ntprm, self%nastbl, stat=istat)
    if (istat /= 0) return

    !----------------------------------------------------
    deallocate(self%npsh, self%npsh2pr, self%ishp, self%idxshp, stat=istat)

    deallocate(self%rzet)
    deallocate(self%ovl)
    deallocate(self%xsi)
    deallocate(self%bdz)
    deallocate(self%zet)
    deallocate(self%zdp)

    deallocate(self%rzeth)
    deallocate(self%AB, self%nprims2)
    deallocate(self%oozeta, self%Sab, self%P, self%PA)

    self%natom  = 0
    self%nasp   = 0
    self%mxsa   = 0
    self%mxprm  = 0
    self%nshell = 0
    self%nao    = 0

  end subroutine tables_free

  !-----------------------------------------------------------------------------
  !> シュワルツ不等式による積分カットオフテーブルの作成
  subroutine tables_swz(self,  swzij,cutthr,nshell,istat)
    !   use atoms_class
    type(tables_type),intent(inout)::self
    !   type(atoms_type),intent(in)::atoms
    integer,intent(inout)::istat
    integer::nshell
    real(8)::swzij(nshell,nshell),cutthr

    !   integer::natom, nbasis, nao, mxshell, mxprim
    !   integer :: nshelp

    istat = 0

    !   natom   = atoms%num
    !   nbasis  = atoms%basis%num
    !   mxshell = atoms%basis%mxshell
    !   mxprim  = atoms%basis%mxprim
    !   nshell  = atoms%basis%nshell
    !   nao     = atoms%basis%nao

    !   nshelp  = self%nshelp

    !   self%nastbl(1,:) = atoms%basis%ntshell(1:nbasis)
    !   self%nastbl(2,:) = atoms%basis%ntao(1:nbasis)
    !   self%ntsht(:,:)  = atoms%basis%ntshtype(1:mxshell,1:nbasis)
    !   self%ntprm(:,:)  = atoms%basis%ntprim(1:mxshell,1:nbasis)
    !   self%natm2asp(:) = atoms%basis%atom2basis(1:natom)
    call p_set_pairtable2(&
         self%ishp, self%npsh, self%npsh2pr, self%idxshp,&
         self%nprimp,self%nshelp,&
         swzij,cutthr,&
         nshell,istat)
    if (istat /= 0) return

  end subroutine tables_swz

  !-----------------------------------------------------------------------------
  subroutine p_set_nsh2ssh ( &
       nsh2ssh,&
       nastbl ,  natm2asp,&
       nasp   ,  natom   ,  nshell,&
       ierr         )
    integer::nasp,natom,nshell
    integer::ierr
    integer::nsh2ssh(nshell)
    integer::nastbl(2,nasp),natm2asp(natom)
    integer::icount,i,j,iasp

    icount = 0
    do j = 1,natom
       iasp = natm2asp(j)
       do i = 1,nastbl(1,iasp)
          icount = icount + 1
          nsh2ssh(icount) = i
       end do
    end do

    if (icount /= nshell) then
       write(6,'(''Error : set_nsh2ssh'')')
       ierr = 1
       return
    endif

  end subroutine p_set_nsh2ssh

  !-----------------------------------------------------------------------------
  subroutine p_set_nsh2atm (&
       nsh2atm,&
       nastbl  ,  natm2asp,&
       nasp    ,  natom   ,  nshell,  &
       ierr )
    integer::ierr
    integer::nasp,natom,nshell
    integer::nsh2atm(nshell)
    integer::nastbl(2,nasp),natm2asp(natom)
    integer::icount,i,j,iasp

    icount = 0
    do j = 1 , natom
       iasp = natm2asp(j)
       do i = 1 , nastbl(1,iasp)
          icount = icount + 1
          nsh2atm(icount) = j
       end do
    end do

    if (icount /= nshell) then
       write(6,'(''Error : set_nsh2atm'')')
       ierr = 1
       return
    endif

  end subroutine p_set_nsh2atm

  !-----------------------------------------------------------------------------
  subroutine p_set_nsh2asp(nsh2asp,nsh2atm,natm2asp,natom,nshell,ierr)
    integer::ierr
    integer::natom,nshell
    integer::nsh2asp(nshell)
    integer::natm2asp(natom),nsh2atm(nshell)
    integer::i,iatom,iasp

    do i = 1 , nshell
       iatom = nsh2atm(i)
       iasp  = natm2asp(iatom)
       nsh2asp(i) = iasp
    end do

    if (minval(nsh2asp) <= 0) then
       write(6,'(''Error : set_nsh2asp'')')
       ierr = 1
       return
    endif

  end subroutine p_set_nsh2asp

  !-----------------------------------------------------------------------------
  subroutine p_set_nsh2sht(&
       nsh2sht,&
       nsh2asp,  nsh2ssh,  ntsht,&
       nasp, nshell, mxsa, ierr)
    integer::nasp,nshell,mxsa
    integer::nsh2sht(nshell)
    integer::nsh2asp(nshell),nsh2ssh(nshell)
    integer::ntsht(mxsa,nasp)
    integer::i,ishel,iasp,iang
    integer::ierr

    do i = 1 , nshell
       ishel = nsh2ssh(i)
       iasp  = nsh2asp(i)
       iang  = ntsht(ishel,iasp)
       nsh2sht(i) = iang
    enddo

    if (minval(nsh2sht) <= 0) then
       write(6,'(''Error : set_nsh2sht'')')
       ierr = 1
       return
    endif

  end subroutine p_set_nsh2sht

  !-----------------------------------------------------------------------------
  subroutine p_set_nsh2ao(&
       nsh2ao,nsh2asp,nsh2ssh,ntsht,nasp,nshell,mxsa,nao,ierr)
    integer::nasp,nshell,mxsa,nao
    integer::nsh2ao(nshell)
    integer::nsh2asp(nshell)
    integer::nsh2ssh(nshell)
    integer::ntsht(mxsa,nasp)
    integer::ierr
    integer::icount,i,iasp,ishel,iang

    icount = 1
    do i = 1, nshell
       nsh2ao(i) = icount
       iasp  = nsh2asp(i)
       ishel = nsh2ssh(i)
       iang = ntsht(ishel,iasp)
       icount = icount + (iang * (iang +1)/2)
    enddo

    if (nao /= icount-1) then
       write(6,'(''error'')')
       ierr = 1
       return
    end if

  end subroutine p_set_nsh2ao

  !-----------------------------------------------------------------------------
  subroutine p_set_nao2sh(&
       nao2sh,&
       natm2asp, nsh2atm, nsh2ssh, ntsht,&
       nao     , nasp   , natom  , nshell, mxsa,&
       ierr)
    integer::nao, nasp, natom, nshell, mxsa
    integer::nao2sh(nao)
    integer::nsh2ssh(nshell),natm2asp(natom),nsh2atm(nshell)
    integer::ntsht(mxsa,nasp)
    integer::icount,ishel,iatom,iasp,iang,mang 
    integer::i,j
    integer::ierr

    icount = 0
    do i = 1 , nshell
       ishel = nsh2ssh(i)
       iatom = nsh2atm(i)
       iasp  = natm2asp(iatom)
       iang  = ntsht(ishel,iasp)
       mang = iang * (iang+1) /2

       do j = 1 , mang
          icount = icount + 1
          nao2sh(icount) = i
       end do
    end do

    if (icount /= nao) then
       write(6,'(''Error : set_nao2sh'')')
       ierr = 1
       return
    endif

  end subroutine p_set_nao2sh

  !-----------------------------------------------------------------------------
  subroutine p_set_nao2dir(&
       nao2dir,&
       nsh2asp, nsh2ssh, ntsht ,&
       nao    , nasp   , nshell, mxsa,&
       ierr)
    integer::nao, nasp, nshell, mxsa
    integer::nao2dir(nao)
    integer::ntsht(mxsa,nasp)
    integer::nsh2ssh(nshell)
    integer::nsh2asp(nshell)
    integer::m,i,nmax,n,iang
    integer::ierr

    m = 0
    do i = 1 , nshell
       iang = ntsht(nsh2ssh(i),nsh2asp(i))
       nmax = ( iang * (iang + 1) )/ 2

       do n = 1 , nmax
          m = m + 1
          nao2dir(m) = n
       end do
    end do

    if (m /= nao) then
       write(6,'(''Error : set_nao2dir'')')
       ierr = 1
       return
    endif

  end subroutine p_set_nao2dir

  !-----------------------------------------------------------------------------
  subroutine p_set_pairtable(&
       ishp, npsh, npsh2pr, nprimp,&
       nsh2sht, nsh2ssh, nsh2asp, ntprm,&
       nshell, mxshell, nbasis, nshelp, istat)
    integer,intent(in)::nshell,nshelp,mxshell,nbasis
    integer,intent(inout)::nprimp
    integer,intent(inout)::istat
    integer,intent(inout)::ishp(6)
    integer,intent(inout)::npsh(2,nshelp)
    integer,intent(inout)::npsh2pr(nshelp+1)
    integer,intent(in)::nsh2sht(nshell),nsh2asp(nshell)
    integer,intent(in)::nsh2ssh(nshell),ntprm(mxshell,nbasis)
    integer,allocatable :: ijsh(:,:,:)
    integer::i,j,ii,jj,iang,jang
    integer::id,ix,icount,ishel,jshel,iasp,jasp,igp,jgp

    allocate(ijsh(2,6,nshelp))

    ijsh = 0 
    ishp = 0 
    npsh = 0 
    npsh2pr = 0 
    nprimp = 0

    do j = 1 , nshell
       do i = j , nshell 
          ii = i
          jj = j
          iang = nsh2sht(ii)
          jang = nsh2sht(jj)

          if (iang < jang) then
             id = i
             ii = j
             jj = id
             iang = nsh2sht(ii)
             jang = nsh2sht(jj)
          end if

          ix = jang + iang * ( iang - 1 )/2
          ishp(ix) = ishp(ix) + 1
          ijsh(1,ix,ishp(ix)) = ii
          ijsh(2,ix,ishp(ix)) = jj
       end do
    end do

    !== set ishp : total shell pair index table ==
    icount = 0

    do i = 1, 6!maxangp !ss,ps,pp
       do j = 1, ishp(i) !number of shell-pair
          icount = icount + 1
          npsh(1,icount) = ijsh(1,i,j)
          npsh(2,icount) = ijsh(2,i,j)

          ii = ijsh(1,i,j)
          jj = ijsh(2,i,j)

          ishel = nsh2ssh(ii)
          jshel = nsh2ssh(jj)
          iasp = nsh2asp(ii)
          jasp = nsh2asp(jj)

          igp = ntprm(ishel,iasp)
          jgp = ntprm(jshel,jasp)
          !=====8/18====
          !=====8/18====

          npsh2pr(icount) = nprimp + 1

          nprimp = nprimp + igp * jgp !<<<OVERLAP CUTOFF
       end do
    end do
    npsh2pr(icount+1) = nprimp + 1

    if (icount /= nshelp) then
       istat = 1
       return
    end if

    !== set ishp : total shell pair index table ==
    do i = 2, 6!maxangp
       ishp(i) = ishp(i) + ishp(i-1)
    end do

    deallocate(ijsh)
  end subroutine p_set_pairtable

  !-----------------------------------------------------------------------------
  subroutine p_set_pairtable2(&
       ishp, npsh, npsh2pr, idxshp,&
       nprimp,nshelp,&
       swzij,cutthr,&
       nshell,istat)
    integer::nshelp,nprimp
    integer::istat,nshell
    integer::ishp(6)
    integer::npsh(2,nshelp)
    integer::npsh2pr(nshelp+1)
    integer::idxshp(nshell*(nshell+1)/2)
    real(8)::swzij(nshell,nshell),cutthr
    integer::i,ij,ish,jsh,icount,key,ijs
    integer::jshp(6)
    real(8)::dswzij
    integer::ijshp_

!!$    idxshp(:) = 0
    icount = 0
    key = 0
    do i = 1, 6
       if (i==1) then
          ijs =1 
       else
          ijs = ishp(i-1) + 1
       end if
       do ij = ijs,ishp(i)
          ish = npsh(1,ij)
          jsh = npsh(2,ij)

          dswzij = swzij(ish,jsh)
          if (ish < jsh) dswzij = swzij(jsh,ish) 

          if (dswzij < cutthr ) cycle

          icount = icount + 1
          key = key + npsh2pr(ij+1) - npsh2pr(ij)

          npsh2pr(icount+1) = key 
          npsh(:,icount) = npsh(:,ij) 
          !         swzrs(icount) = swzrs(ij) 

          if (ish < jsh) then
             ijshp_ = jsh*(jsh-1)/2 + ish
          else
             ijshp_ = ish*(ish-1)/2 + jsh
          end if
!!$          idxshp(ijshp_) = icount
!!$          write(6,*) 'idxshp(',ish,jsh,ijshp_,') =',icount
       end do
       jshp(i) = icount
    end do

    ishp(:) = jshp(:)
    nshelp = icount
    nprimp = key - 1

  end subroutine p_set_pairtable2

  !-----------------------------------------------------------------------------
  subroutine p_set_pairtable3(&
       ishp, npsh, npsh2pr, nprimp,&
       nsh2sht, nsh2ssh, nsh2asp, ntprm,&
       rzet,ovl,xsi,bdz,zet,zdp,idxshp,&
       nsh2atm,xyz,zz,cc,&
       mxprm,mxsa,nasp,natom,&
       nshell, mxshell, nbasis, nshelp, ldebug, istat)
    use physical_constants_class
    integer,intent(in)::nshell,nshelp,mxshell,nbasis,nprimp
    integer,intent(in)::ishp(6)
    integer,intent(in)::npsh(2,nshelp)
    integer,intent(in)::npsh2pr(nshelp+1)
    integer,intent(in)::nsh2sht(nshell),nsh2asp(nshell)
    integer,intent(in)::nsh2ssh(nshell),ntprm(mxshell,nbasis)
    real(8),intent(inout)::rzet(nprimp)
    real(8),intent(inout)::ovl(nprimp)
    real(8),intent(inout)::xsi(nprimp)
    real(8),intent(inout)::bdz(nprimp)
    real(8),intent(inout)::zet(nprimp)
    real(8),intent(inout)::zdp(nprimp)
    integer,intent(inout)::idxshp(nshell*(nshell+1)/2)
    integer,intent(in)::mxprm,mxsa,nasp,natom
    integer,intent(in)::nsh2atm(nshell)
    real(8),intent(in)::xyz(3,natom)
    real(8),intent(in)::zz(mxprm,mxsa,nasp)
    real(8),intent(in)::cc(mxprm,mxsa,nasp)
    integer,intent(in)::ldebug
    integer,intent(inout)::istat
    integer :: i,j,k,l,m,n,ii,jj,ipcount,kpcount
    integer :: ishel,jshel,iasp,jasp,mpr,mps,iatom,jatom
    integer :: ir,is
    real(8) :: abx,aby,abz,ab2
    real(8) :: alp,bet,c1,c2,rzet0,rzet32,xsi0
    real(8) :: a(3),b(3)
    integer::nrs, jpcount
    integer::ii_,jj_,ij_

    if (ldebug > 0) then
!!$    write(6,*) '---------------------------------'
       print *,"nprimp,nshelp = ",nprimp,nshelp
!!$
!!$    write(6,*) 'npsh=', npsh
!!$    write(6,*) 'nsh2atm=', nsh2atm
!!$    write(6,*) 'nsh2asp=', nsh2asp
!!$    write(6,*) 'ntprm=', ntprm
!!$    write(6,*) 'xyz=', xyz
!!$    write(6,*) 'zz=', zz
!!$    write(6,*) 'cc=', cc
!!$    write(6,*) '---------------------------------'
    end if

    idxshp(:) = 0

    ipcount = 0
    kpcount = 0
    jpcount = 0 ! survival shell-pairs
    do i = 1,nshelp

      ii = npsh(1,i)
      jj = npsh(2,i)

      ii_ = max0(ii,jj)
      jj_ = min0(ii,jj)
      ij_ = ii_*(ii_-1)/2 + jj_
      !write(6,*) 'idxshp(',ii,jj,ij_,') =',i
      idxshp(ij_) = i

      ishel = nsh2ssh(ii)
      jshel = nsh2ssh(jj)

      iatom = nsh2atm(ii)
      jatom = nsh2atm(jj)

      iasp = nsh2asp(ii)
      jasp = nsh2asp(jj)

      mpr = ntprm(ishel,iasp)
      mps = ntprm(jshel,jasp)

      a(:) = xyz(:,iatom)
      b(:) = xyz(:,jatom)

      abx = xyz(1,iatom) - xyz(1,jatom)
      aby = xyz(2,iatom) - xyz(2,jatom)
      abz = xyz(3,iatom) - xyz(3,jatom)

      ab2 = abx*abx + aby*aby + abz*abz   

      nrs = 0
      do ir = 1,mpr
        alp = zz(ir,ishel,iasp)
        c1  = cc(ir,ishel,iasp)
        do is = 1,mps

          bet = zz(is,jshel,jasp)
          c2  = cc(is,jshel,jasp)

          ipcount = ipcount + 1

          !         zet(ipcount)  = zet0(ir,is,ishel,iasp,jshel,jasp)
          !         rzet(ipcount)  = roo(ir,is,ishel,iasp,jshel,jasp)
          !         zdp(ipcount)  = zdp0(ir,is,ishel,iasp,jshel,jasp)
          !         bdz(ipcount) = bet * rzet(ipcount)
          !         xsi(ipcount) = alp * bdz(ipcount)
          !         ovl(ipcount) = dexp( -xsi(ipcount) * ab2 )  &
          !                        * rz32c(ir,is,ishel,iasp,jshel,jasp)

          rzet0 =  1.d0 / ( alp + bet )

          rzet32 = ( pi * rzet0)**1.5

          xsi0 =   rzet0 * (alp * bet)

          ovl(ipcount) = dexp( -xsi0 * ab2 ) *c1 *c2 * rzet32
!         write(3,*)ipcount,ovl(ipcount)
          if (dabs(ovl(ipcount)) > 1.d-20) then
             kpcount = kpcount + 1
             nrs = nrs + 1
          end if
          bdz(ipcount) = bet * rzet0
          rzet(ipcount) = rzet0
          xsi(ipcount)  = xsi0
          zet(ipcount)  = alp + bet
          zdp(ipcount)  = dsqrt(zet(ipcount)/pi)

        enddo
      enddo
      if (nrs > 0) jpcount = jpcount + 1
    enddo

    if (ldebug > 0) then
!!$    write(6,*) 'rzet=', rzet
!!$    write(6,*) 'bdz=', bdz
!!$    write(6,*) 'ovl=', ovl
       print *," overlap_cutoff_count = ",kpcount
       print *,"          total_count = ",ipcount,kpcount*100.d0/ipcount
       print *," survival shell pairs = ",jpcount
    end if

  end subroutine p_set_pairtable3

  !-----------------------------------------------------------------------------
  subroutine p_set_pairtable4(&
       AB, nprims2, mxprim2,&
       npsh, nsh2atm, npsh2pr, ovl, xyz,&
       natom, nshell, nshelp, nprimp, mxprm, ldebug)
    use physical_constants_class
    integer,intent(in)::natom,nshell,nshelp,nprimp,mxprm
    real(8),intent(inout)::AB(3,nshelp)
    integer,intent(inout)::nprims2(nshelp)
    integer,intent(inout)::mxprim2
    integer,intent(in)::npsh(2,nshelp)
    integer,intent(in)::nsh2atm(nshell)
    integer,intent(in)::npsh2pr(nshelp+1)
    real(8),intent(in)::ovl(nprimp)
    real(8),intent(in)::xyz(3,natom)
    integer,intent(in)::ldebug

    integer::ij,iatom,jatom,np2,mx,irs,irsx,n
    integer,allocatable::nshp_ec(:)

    do ij = 1, nshelp
       iatom = nsh2atm(npsh(1,ij))
       jatom = nsh2atm(npsh(2,ij))
       AB(1:3,ij) = xyz(1:3,iatom) - xyz(1:3,jatom)
    end do

    allocate(nshp_ec(0:mxprm*mxprm))
    nshp_ec(:) = 0

    n = 0
    mxprim2 = 0
    do ij = 1, nshelp
!ovl cutoff
!!$       np2 = npsh2pr(ij+1) - npsh2pr(ij)
!!$       nprims2(ij) = np2
!!$       mxprim2 = max(mxprim2, np2)
!!$       nshp_ec(np2) = nshp_ec(np2) + 1
!ovl cutoff
     irsx = 0
     do irs = npsh2pr(ij), npsh2pr(ij+1)-1
        if (dabs(ovl(irs)) < 1.d-20) cycle
        irsx = irsx + 1
     end do
     if (irsx > 0) n = n + 1
     nprims2(ij) = irsx
     mxprim2 = max(mxprim2, irsx)
     nshp_ec(irsx) = nshp_ec(irsx) + 1
!ovl cutoff
    end do

    if (ldebug > 0) then
       write(6,*) 'mxprim2=',mxprim2,'n=',n,'/ nshellp=',nshelp
    end if
!!$    do ij = 0, mxprm*mxprm
!!$       if (nshp_ec(ij) /= 0) then
!!$          write(6,'(''nshp_ec('',i4,'' )='',i8)') ij,nshp_ec(ij)
!!$       end if
!!$    end do
    deallocate(nshp_ec)

  end subroutine p_set_pairtable4

  !-----------------------------------------------------------------------------
  subroutine p_set_pairtable5(&
       oozeta, Sab, P, PA,&
       AB, npsh, nsh2atm, npsh2pr, ovl, bdz, zet, xyz,&
       natom, nshell, nshelp, nprimp, mxprim2, mxprm)
    use physical_constants_class
    integer,intent(in)::natom,nshell,nshelp,nprimp,mxprim2,mxprm
    real(8),intent(inout)::oozeta(mxprim2,nshelp)
    real(8),intent(inout)::Sab(mxprim2,nshelp)
    real(8),intent(inout)::P(3,mxprim2,nshelp)
    real(8),intent(inout)::PA(3,mxprim2,nshelp)
    real(8),intent(in)::AB(3,nshelp)
    integer,intent(in)::npsh(2,nshelp)
    integer,intent(in)::nsh2atm(nshell)
    integer,intent(in)::npsh2pr(nshelp+1)
    real(8),intent(in)::ovl(nprimp)
    real(8),intent(in)::bdz(nprimp)
    real(8),intent(in)::zet(nprimp)
    real(8),intent(in)::xyz(3,natom)

    integer::ij,iatom,irs,irsx

    do ij = 1, nshelp
       iatom = nsh2atm(npsh(1,ij))
       irsx = 0
       do irs = npsh2pr(ij), npsh2pr(ij+1)-1
!ovl cutoff
          if (dabs(ovl(irs)) < 1.d-20) cycle
!ovl cutoff
          irsx = irsx + 1
          Sab(irsx,ij) = ovl(irs)
          PA(1:3,irsx,ij) =-AB(1:3,ij) * bdz(irs)
          P (1:3,irsx,ij) = PA(1:3,irsx,ij) + xyz(1:3,iatom)
          oozeta(irsx,ij) = 1.0d0/zet(irs)
       end do
    end do

  end subroutine p_set_pairtable5
  !-----------------------------------------------------------------------------

  subroutine set_mang(mang,nsh2ssh,nsh2asp,ntsht,nshell,mxshell,nbasis)
    integer,intent(inout)::mang
    integer,intent(in)::nshell,mxshell,nbasis
    integer,intent(in)::nsh2ssh(nshell)
    integer,intent(in)::nsh2asp(nshell)
    integer,intent(in)::ntsht(mxshell,nbasis)

    integer::iang,ishell

    mang = 0
    do ishell = 1, nshell
       iang = ntsht(nsh2ssh(ishell),nsh2asp(ishell))
       mang = max(mang, iang)
    end do

  end subroutine set_mang
  !-----------------------------------------------------------------------------
end module integral_tables_class
