! -*- coding: utf-8 -*-
!
! Dual licensed under the MIT or GPL Version 3 license.
! See 'LICENSE.MIT' for the definition of the MIT license,
! and 'COPYING' for the definition of the GPL license.
!
C** IKSSSS ===========================================================
C          CALUCATE FMT OF SSSS TYPE
C====================================================================

      SUBROUTINE ikssss(
     &           dm0  ,
     &           fdat ,
     &           pp   ,  qq,
     &           uu   ,  uv    , 
     &           zetpq,
     &           nasp ,  mxsa  ,  mxprm,  ntprm, 
!    &           igp ,  jgp  ,  kgp ,  lgp )
     &           iasp ,  jasp  ,  kasp ,  lasp ,
     &           ishel,  jshel ,  kshel,  lshel)

      IMPLICIT DOUBLE PRECISION  (a-h,o-z)
      DIMENSION fdat(31,2001)
      DIMENSION uu(  mxprm,mxprm),uv(  mxprm,mxprm)
      dimension pp(3,mxprm,mxprm),qq(3,mxprm,mxprm)

      dimension zetpq(mxprm,mxprm,mxsa,nasp,mxsa,nasp)
      dimension ntprm(mxsa,nasp)

      real(8),parameter::pi    = 3.14159265358979323846d+00
      real(8),parameter::tf(25) = (/
     &                33.0,37.0,41.0,43.0,46.0,49.0,
     &                51.0,54.0,56.0,58.0,61.0,63.0,
     &                66.0,68.0,70.0,72.0,74.0,80.0,
     &                85.0,90.0,95.0,95.0,98.0,99.0,
     &                99.0                           /)

#ifdef PL_OMP_ERI
!$omp parallel reduction(+:dm0)
!$omp& default(private)
!$omp& shared(fdat,
!$omp&        zetpq,ntprm,pp,qq,uu,uv,
!$omp&        ishel,jshel,kshel,lshel,iasp,jasp,kasp,lasp)
#endif

      igp = ntprm(ishel,iasp)
      jgp = ntprm(jshel,jasp)
      kgp = ntprm(kshel,kasp)
      lgp = ntprm(lshel,lasp)
      
      pi2 = 2.0d0 * pi
      
c      dsq_pi = dsqrt(pi2) * 0.5d0
      
#ifdef PL_OMP_ERI
!$omp do
#endif
      do i = 1, igp
         do j = 1, jgp
            dm1 = 0.0d0
            zet = zetpq(j,i,jshel,jasp,ishel,iasp)

            px = pp(1,j,i)
            py = pp(2,j,i)
            pz = pp(3,j,i)

            up =  uu(j,i)
            do k = 1, kgp
               do l = 1, lgp
                  eta  = zetpq(l,k,lshel,lasp,kshel,kasp)

                  qpx =  qq(1,l,k) - px
                  qpy =  qq(2,l,k) - py
                  qpz =  qq(3,l,k) - pz

                  vq  =  uv  (l,k)

                  roo2 = 1.0d0 / (zet + eta)
                  
                  qp2      = qpx*qpx + qpy*qpy + qpz*qpz
                  t        = 0.5d0 * roo2 * qp2
                  
                  if (t. le .tf(1)) then

                     it = idint((t+0.025d+00)*20.0d+00)
                     dt = 0.05d+00*it - t
                     it = it + 1
                     dt4 = dt*0.25d+00
                     dt5 = dt*0.2d+00
                     dt6 = dt*0.166666666666666667d+00
                     dt2 = dt4 + dt4
                     dt3 = dt6 + dt6
                     
                     epq   = (((((fdat(7,it)*dt6   + fdat(6,it))*dt5
     &                    + fdat(5,it))*dt4  + fdat(4,it))*dt3
     &                    + fdat(3,it))*dt2  + fdat(2,it))*dt
     &                    + fdat(1,it)
                     
                     fpp =  vq*dsqrt(roo2)
                     dm1 =  dm1 + fpp * epq
                     
                  else
                     fpp =  vq * dsqrt( pi2/qp2)*0.5d0
                     dm1 =  dm1 + fpp
                  end if
                  
               enddo
            enddo
            dm0 = dm0 + dm1 * up
         enddo
      enddo
#ifdef PL_OMP_ERI
!$omp end do
!$omp end parallel
#endif

      END
C** IKPSSS ===========================================================
C          CALUCATE FMT OF PSSS TYPE
C====================================================================

      SUBROUTINE ikpsss(
     &           psss ,
     &           fdat ,  edat  ,
     &           pp   ,  qq    , 
     &           uu   ,  uv    ,
     &           z    ,  zetpq ,
     &           nasp ,  mxsa  ,  mxprm,   ntprm,  
     &           iasp ,  jasp  ,  kasp ,  lasp ,
     &           ishel,  jshel ,  kshel,  lshel  )

      IMPLICIT DOUBLE PRECISION  (a-h,o-z)
      DIMENSION fdat(31,2001)
      DIMENSION edat(2001)

      DIMENSION psss(4)

      DIMENSION pp(3,mxprm,mxprm),qq(3,mxprm,mxprm)
      DIMENSION uu(  mxprm,mxprm),uv(  mxprm,mxprm)
      DIMENSION z(mxprm,mxsa,nasp)
      dimension zetpq(mxprm,mxprm,mxsa,nasp,mxsa,nasp)

      DIMENSION ntprm(mxsa,nasp)

      real(8),parameter::pi    = 3.14159265358979323846d+00
      real(8),parameter::tf(25) = (/
     &                33.0,37.0,41.0,43.0,46.0,49.0,
     &                51.0,54.0,56.0,58.0,61.0,63.0,
     &                66.0,68.0,70.0,72.0,74.0,80.0,
     &                85.0,90.0,95.0,95.0,98.0,99.0,
     &                99.0                           /)
 
      real(8),parameter::ga0 = 1.0d+00
      real(8),parameter::ga1 = 1.000 000 000 000 005 298d+00
      real(8),parameter::ga2 = 0.5d+00
      real(8),parameter::ga3 = 0.166 666 666 598 849 826d+00
      real(8),parameter::ga4 = 0.416 666 666 666 666 667d-01
      real(8),parameter::ga5 = 0.833 355 034 722 222 222d-02
      real(8),parameter::ga6 = 0.138 888 888 888 888 889d-02

#ifdef PL_SIMD_ERI
      dimension dmx(2,mxprm), val(1,mxprm)
#endif

      do ijk=1,4
         psss(ijk)=0.0d0
      enddo

#ifdef PL_OMP_ERI
!$omp parallel reduction(+:psss)
!$omp& default(private)
!$omp& shared(fdat,edat,
!$omp&        zetpq,ntprm,z,pp,qq,uu,uv,
!$omp&        ishel,jshel,kshel,lshel,iasp,jasp,kasp,lasp)
#endif

      igp = ntprm(ishel,iasp)
      jgp = ntprm(jshel,jasp)
      kgp = ntprm(kshel,kasp)
      lgp = ntprm(lshel,lasp)

      pi2 = 2.0d0*pi
      
#ifdef PL_OMP_ERI
!$omp do
#endif
      do i = 1, igp
         bet4 = 0.0d0
         
         alp= z(i,ishel,iasp)
         do j = 1, jgp
            del1 = 0.0d0
            del2 = 0.0d0
            
            bet = z(j,jshel,jasp)
            zet = zetpq(j,i,jshel,jasp,ishel,iasp)
            px = pp(1,j,i)
            py = pp(2,j,i)
            pz = pp(3,j,i)
            up  = uu (j,i)
             
            do k = 1, kgp
               del3 = 0.0d0
               
               ram = z(k,kshel,kasp)
               
               do l = 1, lgp
                  del = z(l,lshel,lasp)
                  
                  eta  = zetpq(l,k,lshel,lasp,kshel,kasp)
                  qpx = qq(1,l,k) - px
                  qpy = qq(2,l,k) - py
                  qpz = qq(3,l,k) - pz
                  vq  =  uv (l,k)
 
                  roo2 = 1.0d0 / (zet + eta)
                  qp2  = qpx*qpx + qpy*qpy + qpz*qpz
                  t    = 0.5d0*roo2*qp2
                  
                  if (t. le .tf(2)) then
                     
                     it = idint((t+0.025d+00)*20.0d+00)
                     dt = 0.05d+00*it - t
                     it = it + 1
                     dt4 = dt*0.25d+00
                     dt5 = dt*0.2d+00
                     dt6 = dt*0.166666666666666667d+00
                     dt2 = dt4 + dt4
                     dt3 = dt6 + dt6
                     
                     f1   = (((((fdat(8,it)*dt6   + fdat(7,it))*dt5
     &                    + fdat(6,it))*dt4  + fdat(5,it))*dt3
     &                    + fdat(4,it))*dt2  + fdat(3,it))*dt
     &                    + fdat(2,it)
                     
                     eed  = (((((ga6*dt + ga5)*dt + ga4)*dt
     &                    + ga3)*dt + ga2)*dt + ga1)*dt + ga0
                     ee   = edat(it)*eed
                     
                     f0   = 2.0d+00*t*f1 + ee
                     
c     fpp =  up * vq * dsqrt(roo2)
                     fpp =  vq * dsqrt(roo2)
                     
                     dm0 = fpp*f0
                     dm1 = fpp*f1 * roo2
                     
                  else
                     
                     qinv = 1.0d0 / qp2
                     dm0  = vq*dsqrt(pi2*qinv)*0.5d0
                     dm1  = qinv *  dm0
                     
                  end if

#ifdef PL_SIMD_ERI
                  val(1,l) = eta

                  dmx(1,l) = dm0
                  dmx(2,l) = dm1
               enddo
               
               do l = 1, lgp
                  eta = val(1,l)

                  dm0 = dmx(1,l)
                  dm1 = dmx(2,l)
#endif

! make kernels for psss ======================================= 
!       1:z011000  2:u001000  3:u001101  4:u102000
!============================================================== 

                  del1 = del1 + dm0
                  del2 = del2 + dm1
                  del3 = del3 + dm1 * eta
                  
               enddo
               
               psss(3) = psss(3) + up*del3 * ram
               
            enddo
            
c     updel2 = up*del2
            
            psss(1) = psss(1) + up*del1 * bet
            psss(2) = psss(2) + up*del2 
            bet4    = bet4    + up*del2       * zet
            
         enddo
         psss(4) = psss(4) + bet4 * alp
      enddo
#ifdef PL_OMP_ERI
!$omp end do
!$omp end parallel
#endif
      return
      END
C** IKPSPS ===========================================================
C          CALUCATE FMT OF PSPS TYPE
C====================================================================

      SUBROUTINE ikpsps(
     &           psps ,
     &           fdat ,  edat  , 
     &           pp   ,  qq    ,
     &           uu   ,  uv    ,
     &           z    ,  zetpq , 
     &           nasp ,  mxsa  ,  mxprm ,   ntprm, 
     &           iasp ,  jasp  ,  kasp  ,  lasp ,
     &           ishel,  jshel ,  kshel ,  lshel    )

      IMPLICIT DOUBLE PRECISION  (a-h,o-z)
      DIMENSION fdat(31,2001)
      DIMENSION edat(2001)
      DIMENSION psps(14)
      dimension zetpq(mxprm,mxprm,mxsa,nasp,mxsa,nasp)
      DIMENSION pp(3,mxprm,mxprm),qq(3,mxprm,mxprm)
      DIMENSION uu(mxprm,mxprm),uv(mxprm,mxprm)
      DIMENSION z(mxprm,mxsa,nasp)
      DIMENSION ntprm(mxsa,nasp)

      real(8),parameter::pi    = 3.14159265358979323846d+00
      real(8),parameter::tf(25) = (/
     &                33.0,37.0,41.0,43.0,46.0,49.0,
     &                51.0,54.0,56.0,58.0,61.0,63.0,
     &                66.0,68.0,70.0,72.0,74.0,80.0,
     &                85.0,90.0,95.0,95.0,98.0,99.0,
     &                99.0                           /)
 
      real(8),parameter::ga0 = 1.0d+00
      real(8),parameter::ga1 = 1.000 000 000 000 005 298d+00
      real(8),parameter::ga2 = 0.5d+00
      real(8),parameter::ga3 = 0.166 666 666 598 849 826d+00
      real(8),parameter::ga4 = 0.416 666 666 666 666 667d-01
      real(8),parameter::ga5 = 0.833 355 034 722 222 222d-02
      real(8),parameter::ga6 = 0.138 888 888 888 888 889d-02
c
      real(8),parameter::rminv(5) = (/
     &   1.0d+00,  0.333333333333333333d+00,  0.20d+00,
     &   0.14285714285714286d+00,  0.11111111111111111d+00 /)

#ifdef PL_SIMD_ERI
      dimension dmx(3,mxprm), val(2,mxprm)
#endif

      do ijk=1,14
         psps(ijk)=0.0d0
      enddo
      
#ifdef PL_OMP_ERI
!$omp parallel reduction(+:psps)
!$omp& default(private)
!$omp& shared(fdat,edat,
!$omp&        zetpq,ntprm,z,pp,qq,uu,uv,
!$omp&        ishel,jshel,kshel,lshel,iasp,jasp,kasp,lasp)
#endif

      igp = ntprm(ishel,iasp)
      jgp = ntprm(jshel,jasp)
      kgp = ntprm(kshel,kasp)
      lgp = ntprm(lshel,lasp)

      pi2 = 2.0d0*pi

#ifdef PL_OMP_ERI
!$omp do
#endif
      do i = 1, igp
         bet02 = 0.0d0
         bet03 = 0.0d0
         bet09 = 0.0d0
         bet10 = 0.0d0
         bet11 = 0.0d0
         alp = z(i,ishel,iasp)
         
         do j = 1, jgp
            del01 = 0.0d0
            del02 = 0.0d0
            del03 = 0.0d0
            ram04 = 0.0d0
            del09 = 0.0d0
            ram10 = 0.0d0
            
            bet = z(j,jshel,jasp)
            zet = zetpq(j,i,jshel,jasp,ishel,iasp)
            px  = pp (1,j,i)
            py  = pp (2,j,i)
            pz  = pp (3,j,i)
            up  = uu (j,i)
            
            do k = 1, kgp
               del04 = 0.0d0
               del05 = 0.0d0
               del10 = 0.0d0
               del12 = 0.0d0
               
               ram = z(k,kshel,kasp)
               
               do l = 1, lgp
                  del = z(l,lshel,lasp)
                  eta  = zetpq(l,k,lshel,lasp,kshel,kasp)
                  qpx =  qq(1,l,k) - px
                  qpy =  qq(2,l,k) - py
                  qpz =  qq(3,l,k) - pz
                  vq  = uv (l,k)
                  
                  roo2 = 1.0d0 / (zet + eta)
                  qp2      = qpx*qpx + qpy*qpy + qpz*qpz
                  t        = 0.5d0*roo2*qp2
                  
                  if (t. le .tf(3)) then
                     
                     it = idint((t+0.025d+00)*20.0d+00)
                     dt = 0.05d+00*it - t
                     it = it + 1
                     dt4 = dt*0.25d+00
                     dt5 = dt*0.2d+00
                     dt6 = dt*0.166666666666666667d+00
                     dt2 = dt4 + dt4
                     dt3 = dt6 + dt6
                     f2   = (((((fdat(9,it)*dt6   + fdat(8,it))*dt5
     &                    + fdat(7,it))*dt4  + fdat(6,it))*dt3
     &                    + fdat(5,it))*dt2  + fdat(4,it))*dt
     &                    + fdat(3,it)
                     
                     eed  = (((((ga6*dt + ga5)*dt + ga4)*dt
     &                    + ga3)*dt + ga2)*dt + ga1)*dt + ga0
                     ee   = edat(it)*eed
                     
                     t2   = t*2.0d0
                     f1   = (t2*f2 + ee)*rminv(2)
                     f0   = t2*f1 + ee
                     fpp =  up * vq * dsqrt(roo2)
                     dm0 = fpp*f0
                     dm1 = fpp*f1 *  roo2
                     dm2 = fpp*f2 * (roo2)**2
                  else
                     
                     qinv = 1.0d0 / qp2
                     dm0=  up*vq*dsqrt( pi2*qinv)*0.5d0
                     dm1 =        qinv *  dm0
                     dm2 =  3.0d0*qinv *  dm1
                     
                  end if
                  
#ifdef PL_SIMD_ERI
                  val(1,l) = eta
                  val(2,l) = del

                  dmx(1,l) = dm0
                  dmx(2,l) = dm1
                  dmx(3,l) = dm2
               enddo
               
               do l = 1, lgp
                  eta = val(1,l)
                  del = val(2,l)

                  dm0 = dmx(1,l)
                  dm1 = dmx(2,l)
                  dm2 = dmx(3,l)
#endif

! make kernels for psps ======================================= 
! 1:z011011  4:u011102  7:u001011  10:d102102  13:d001102
! 2:u112001  5:u001112  8:u011001  11:d102001  14:d001001
! 3:u102011  6:u001001  9:d203001  12:d001203
!============================================================== 

                  del01=del01 +dm0*del

                  del02=del02 +dm1
                  del03=del03 +dm1*del
                  del04=del04 +dm1    *eta
                  del05=del05 +dm1*del*eta

                  del09=del09 +dm2
                  del10=del10 +dm2    *eta
                  del12=del12 +dm2    *eta**2

                  psps(6) = psps(6) + dm1
                  psps(7) = psps(7) + dm1 * del
                  psps(14)= psps(14)+ dm2

               enddo

               ram04   = ram04   + del04 * ram
               ram10   = ram10   + del10 * ram
               
               psps(12)= psps(12)+ del12 * ram **2
               psps(13)= psps(13)+ del10 * ram
               psps(5) = psps(5) + del05 * ram
               
            enddo
            bet02   = bet02   + del02 * bet * zet
            bet03   = bet03   + del03       * zet
            bet09   = bet09   + del09       * zet**2
            bet10   = bet10   + ram10       * zet
            bet11   = bet11   + del09       * zet
            
            psps(1) = psps(1) + del01 * bet
            psps(4) = psps(4) + ram04 * bet
            psps(8) = psps(8) + del02 * bet

         enddo
         psps(2) = psps(2) + bet02 * alp
         psps(3) = psps(3) + bet03 * alp
         psps(9) = psps(9) + bet09 * alp **2
         psps(10)= psps(10)+ bet10 * alp
         psps(11)= psps(11)+ bet11 * alp
         
         
      enddo
#ifdef PL_OMP_ERI
!$omp end do
!$omp end parallel
#endif
      
      return
      END
C** IKPPSS ===========================================================
C          CALUCATE FMT OF PPSS TYPE
C====================================================================

      SUBROUTINE ikppss(
     &                 ppss ,
     &                 fdat ,  edat  ,  
     &                 pp   ,  qq    ,  
     &                 uu   ,  uv    ,
     &                 z    ,  zetpq , 
     &                 nasp ,  mxsa  ,  mxprm,  ntprm,  
     &                 iasp ,  jasp  ,  kasp , lasp ,
     &                 ishel,  jshel ,  kshel, lshel)

      IMPLICIT DOUBLE PRECISION  (a-h,o-z)
      DIMENSION fdat(31,2001)
      DIMENSION edat(2001)
      DIMENSION ppss(15)
      dimension zetpq(mxprm,mxprm,mxsa,nasp,mxsa,nasp)
      DIMENSION pp(3,mxprm,mxprm),qq(3,mxprm,mxprm)
      DIMENSION uu(mxprm,mxprm),uv(mxprm,mxprm)
      DIMENSION z(mxprm,mxsa,nasp)
      DIMENSION ntprm(mxsa,nasp)

      real(8),parameter::pi    = 3.14159265358979323846d+00
      real(8),parameter::tf(25) = (/
     &                33.0,37.0,41.0,43.0,46.0,49.0,
     &                51.0,54.0,56.0,58.0,61.0,63.0,
     &                66.0,68.0,70.0,72.0,74.0,80.0,
     &                85.0,90.0,95.0,95.0,98.0,99.0,
     &                99.0                           /)
 
      real(8),parameter::ga0 = 1.0d+00
      real(8),parameter::ga1 = 1.000 000 000 000 005 298d+00
      real(8),parameter::ga2 = 0.5d+00
      real(8),parameter::ga3 = 0.166 666 666 598 849 826d+00
      real(8),parameter::ga4 = 0.416 666 666 666 666 667d-01
      real(8),parameter::ga5 = 0.833 355 034 722 222 222d-02
      real(8),parameter::ga6 = 0.138 888 888 888 888 889d-02
c
      real(8),parameter::rminv(5) = (/
     &   1.0d+00,  0.333333333333333333d+00,  0.20d+00,
     &   0.14285714285714286d+00,  0.11111111111111111d+00 /)

#ifdef PL_SIMD_ERI
      dimension dmx(3,mxprm), val(1,mxprm)
#endif

      do ijk=1,15
        ppss(ijk)=0.0d0
      enddo

#ifdef PL_OMP_ERI
!$omp parallel reduction(+:ppss)
!$omp& default(private)
!$omp& shared(fdat,edat,
!$omp&        zetpq,ntprm,z,pp,qq,uu,uv,
!$omp&        ishel,jshel,kshel,lshel,iasp,jasp,kasp,lasp)
#endif

      igp = ntprm(ishel,iasp)
      jgp = ntprm(jshel,jasp)
      kgp = ntprm(kshel,kasp)
      lgp = ntprm(lshel,lasp)

      pi2 = 2.0d0*pi

#ifdef PL_OMP_ERI
!$omp do
#endif
      do i = 1, igp
         bet01 = 0.0d0   
         bet03 = 0.0d0
         bet04 = 0.0d0
         bet06 = 0.0d0  
         bet08 = 0.0d0  
         bet10 = 0.0d0
         bet11 = 0.0d0
         bet13 = 0.0d0
         alp = z(i,ishel,iasp)
         
         do j = 1, jgp
            del01 = 0.0d0
            del03 = 0.0d0
            ram05 = 0.0d0
            del10 = 0.0d0
            ram11 = 0.0d0
            ram12 = 0.0d0
            
            bet = z(j,jshel,jasp)
            zet = zetpq(j,i,jshel,jasp,ishel,iasp)
            px  = pp (1,j,i)
            py  = pp (2,j,i)
            pz  = pp (3,j,i)
            up  = uu(j,i)
            
            do k = 1, kgp
               del05 = 0.0d0
               del11 = 0.0d0
               del12 = 0.0d0
               
               ram = z(k,kshel,kasp)
               
               do l = 1, lgp
                  eta  = zetpq(l,k,lshel,lasp,kshel,kasp)
                  qpx =  qq(1,l,k) - px
                  qpy =  qq(2,l,k) - py
                  qpz =  qq(3,l,k) - pz
                  vq  =  uv (l,k)
                  
                  roo2 = 1.0d0 / (zet + eta)
                  qp2      = qpx*qpx + qpy*qpy + qpz*qpz
                  t        = 0.5d0*roo2*qp2
                  
                  if (t. le .tf(3)) then
                     
                     it = idint((t+0.025d+00)*20.0d+00)
                     dt = 0.05d+00*it - t
                     it = it + 1
                     dt4 = dt*0.25d+00
                     dt5 = dt*0.2d+00
                     dt6 = dt*0.166666666666666667d+00
                     dt2 = dt4 + dt4
                     dt3 = dt6 + dt6
                     f2   = (((((fdat(9,it)*dt6   + fdat(8,it))*dt5
     &                    + fdat(7,it))*dt4  + fdat(6,it))*dt3
     &                    + fdat(5,it))*dt2  + fdat(4,it))*dt
     &                    + fdat(3,it)
c     
                     eed  = (((((ga6*dt + ga5)*dt + ga4)*dt
     &                    + ga3)*dt + ga2)*dt + ga1)*dt + ga0
                     ee   = edat(it)*eed
c     
                     t2   = t*2.0d0
                     f1   = (t2*f2 + ee)*rminv(2)
                     f0   = t2*f1 + ee
                     
                     fpp =  up * vq * dsqrt(roo2)
                     dm0 = fpp*f0
                     dm1 = fpp*f1 *  roo2
                     dm2 = fpp*f2 * (roo2)**2
                     
                  else
                     qinv = 1.0d0/qp2
                     dm0=  up*vq*dsqrt( pi2*qinv)*0.5d0
                     dm1 =        qinv *  dm0
                     dm2 =  3.0d0*qinv *  dm1
                     
                  end if
                  
#ifdef PL_SIMD_ERI
                  val(1,l) = eta

                  dmx(1,l) = dm0
                  dmx(2,l) = dm1
                  dmx(3,l) = dm2
               enddo
               
               do l = 1, lgp
                  eta = val(1,l)

                  dm0 = dmx(1,l)
                  dm1 = dmx(2,l)
                  dm2 = dmx(3,l)
#endif

! make kernels for ppss ======================================= 
! 1:z112000  4:u113000  7:u002000  10:d204000  13:d103000
! 2:z001000  5:u012101  8:u102000  11:d103101  14:d002101
! 3:u203000  6:u102101  9:u012000  12:d002202  15:d002000
!============================================================== 
                  
                  del01 =    del01+dm0              
                  
                  del03 =    del03+dm1              
                  del05 =    del05+dm1       *eta
                  
                  del10 =    del10+dm2           
                  del11 =    del11+dm2       *eta
                  del12 =    del12+dm2       *eta**2
                  
                  ppss( 2)=ppss( 2)+dm0           
               enddo         
               ram05 = ram05 + del05       *ram
               ram11 = ram11 + del11       *ram
               ram12 = ram12 + del12       *ram**2
               
            enddo         
            
            bet01   = bet01   + del01 * bet * zet
            bet03   = bet03   + del03       * zet**2
            bet04   = bet04   + del03 * bet * zet**2
            bet06   = bet06   + ram05       * zet   
            bet08   = bet08   + del03       * zet   
            bet10   = bet10   + del10       * zet**3
            bet11   = bet11   + ram11       * zet**2
            bet13   = bet13   + del10       * zet**2
            
            ppss(5) = ppss(5) + ram05 * bet * zet   
            ppss(7) = ppss(7) + del03       * zet   
            ppss(9) = ppss(9) + del03 * bet * zet   
            ppss(12)= ppss(12)+ ram12       * zet   
            ppss(14)= ppss(14)+ ram11       * zet   
            ppss(15)= ppss(15)+ del10       * zet   
            
         enddo     
         ppss( 1) = ppss( 1) + bet01 * alp    
         ppss( 3) = ppss( 3) + bet03 * alp**2
         ppss( 4) = ppss( 4) + bet04 * alp
         ppss( 6) = ppss( 6) + bet06 * alp   
         ppss( 8) = ppss( 8) + bet08 * alp   
         ppss(10) = ppss(10) + bet10 * alp**2
         ppss(11) = ppss(11) + bet11 * alp
         ppss(13) = ppss(13) + bet13 * alp
      enddo
#ifdef PL_OMP_ERI
!$omp end do
!$omp end parallel
#endif

      END
C** IKPPPS ===========================================================
C          CALUCATE FMT OF PPPS TYPE
C====================================================================
      SUBROUTINE ikppps(      
     &           ppps ,
     &           fdat ,  edat  ,  
     &           pp   ,  qq    ,
     &           uu   ,  uv    ,
     &           z    ,  zetpq ,
     &           nasp ,  mxsa  ,  mxprm,   ntprm,
     &           iasp ,  jasp  ,  kasp ,  lasp ,
     &           ishel,  jshel ,  kshel,  lshel     )

      IMPLICIT DOUBLE PRECISION  (a-h,o-z)           
      DIMENSION fdat(31,2001)
      DIMENSION edat(2001)

      DIMENSION ppps(47)
      dimension zetpq(mxprm,mxprm,mxsa,nasp,mxsa,nasp)
      DIMENSION pp(3,mxprm,mxprm),qq(3,mxprm,mxprm)
      DIMENSION uu(mxprm,mxprm),uv(mxprm,mxprm)
      DIMENSION z(mxprm,mxsa,nasp)
      DIMENSION ntprm(mxsa,nasp)

      real(8),parameter::pi    = 3.14159265358979323846d+00
      real(8),parameter::tf(25) = (/
     &                33.0,37.0,41.0,43.0,46.0,49.0,
     &                51.0,54.0,56.0,58.0,61.0,63.0,
     &                66.0,68.0,70.0,72.0,74.0,80.0,
     &                85.0,90.0,95.0,95.0,98.0,99.0,
     &                99.0                           /)
 
      real(8),parameter::ga0 = 1.0d+00
      real(8),parameter::ga1 = 1.000 000 000 000 005 298d+00
      real(8),parameter::ga2 = 0.5d+00
      real(8),parameter::ga3 = 0.166 666 666 598 849 826d+00
      real(8),parameter::ga4 = 0.416 666 666 666 666 667d-01
      real(8),parameter::ga5 = 0.833 355 034 722 222 222d-02
      real(8),parameter::ga6 = 0.138 888 888 888 888 889d-02
c
      real(8),parameter::rminv(5) = (/
     &   1.0d+00,  0.333333333333333333d+00,  0.20d+00,
     &   0.14285714285714286d+00,  0.11111111111111111d+00 /)

#ifdef PL_SIMD_ERI
      dimension dmx(4,mxprm), val(2,mxprm)
#endif

      do ijk=1,47
        ppps(ijk)=0.0d0
      enddo

#ifdef PL_OMP_ERI
!$omp parallel reduction(+:ppps)
!$omp& default(private)
!$omp& shared(fdat,edat,
!$omp&        zetpq,ntprm,z,pp,qq,uu,uv,
!$omp&        ishel,jshel,kshel,lshel,iasp,jasp,kasp,lasp)
#endif

      igp = ntprm(ishel,iasp)
      jgp = ntprm(jshel,jasp)
      kgp = ntprm(kshel,kasp)
      lgp = ntprm(lshel,lasp)

      pi2 = 2.0d0*pi

#ifdef PL_OMP_ERI
!$omp do
#endif
      do i = 1, igp
         bet09  = 0.0d0
         bet10  = 0.0d0
         bet11  = 0.0d0
         bet12  = 0.0d0
         bet13  = 0.0d0
         bet14  = 0.0d0
         bet15  = 0.0d0
         bet16  = 0.0d0
         bet18  = 0.0d0
         bet19  = 0.0d0
         bet20  = 0.0d0
         bet21  = 0.0d0
         bet26  = 0.0d0
         bet27  = 0.0d0
         bet28  = 0.0d0
         bet29  = 0.0d0
         bet30  = 0.0d0
         bet31  = 0.0d0
         bet38  = 0.0d0
         bet39  = 0.0d0
         bet40  = 0.0d0
         bet41  = 0.0d0
         bet42  = 0.0d0
         bet43  = 0.0d0
         bet44  = 0.0d0
         bet45  = 0.0d0
         bet47  = 0.0d0
         alp = z(i,ishel,iasp)
         
         do j = 1, jgp
            del01 = 0.0d0
            del02 = 0.0d0
            del22 = 0.0d0
            del32 = 0.0d0
            del34 = 0.0d0
            del46 = 0.0d0
            ram03 = 0.0d0
            ram04 = 0.0d0
            ram05 = 0.0d0
            ram08 = 0.0d0
            ram23 = 0.0d0
            ram24 = 0.0d0
            ram25 = 0.0d0
            ram37 = 0.0d0
            ram42 = 0.0d0
            
            bet = z(j,jshel,jasp)
            zet = zetpq(j,i,jshel,jasp,ishel,iasp)
            px  = pp (1,j,i)
            py  = pp (2,j,i)
            pz  = pp (3,j,i)
            up  = uu  (j,i)
            
            do k = 1, kgp
               del03   =0.0d0 
               del04   =0.0d0 
               del08   =0.0d0 
               del23   =0.0d0 
               del37   =0.0d0 
               del24   =0.0d0 
               del05   =0.0d0 
               del25   =0.0d0 
               del33   =0.0d0 
               
               ram = z(k,kshel,kasp)
               
               do l = 1, lgp
                  del = z(l,lshel,lasp)
                  eta  = zetpq(l,k,lshel,lasp,kshel,kasp)
                  qpx =  qq(1,l,k) - px
                  qpy =  qq(2,l,k) - py
                  qpz =  qq(3,l,k) - pz
                  vq  = uv  (l,k)
                  
                  roo2 = 1.0d0 / (zet + eta)
                  qp2  = qpx*qpx + qpy*qpy + qpz*qpz
                  t    = 0.5d0* roo2*qp2
                  
                  if (t. le .tf(4)) then
                     
                     it = idint((t+0.025d+00)*20.0d+00)
                     dt = 0.05d+00*it - t
                     it = it + 1
                     dt4 = dt*0.25d+00
                     dt5 = dt*0.2d+00
                     dt6 = dt*0.166666666666666667d+00
                     dt2 = dt4 + dt4
                     dt3 = dt6 + dt6
                     f3   = (((((fdat(10,it)*dt6   + fdat(9,it))*dt5
     &                    + fdat(8,it))*dt4  + fdat(7,it))*dt3
     &                    + fdat(6,it))*dt2  + fdat(5,it))*dt
     &                    + fdat(4,it)
                     
                     eed  = (((((ga6*dt + ga5)*dt + ga4)*dt
     &                    + ga3)*dt + ga2)*dt + ga1)*dt + ga0
                     ee   = edat(it)*eed
                     
                     t2   = t*2.0d0
                     
                     f2   = (t2*f3 + ee)*rminv(3)
                     f1   = (t2*f2 + ee)*rminv(2)
                     f0   =  t2*f1 + ee
                     fpp =  up * vq * dsqrt(roo2)
                     dm0 = fpp*f0
                     dm1 = fpp*f1 *  roo2
                     dm2 = fpp*f2 *( roo2)**2
                     dm3 = fpp*f3 *( roo2)**3
                     
                  else
                     qinv = 1.0d0/qp2
                     dm0=  up*vq*dsqrt( pi2*qinv)*0.5d0
                     dm1 =        qinv *  dm0
                     dm2 =  3.0d0*qinv *  dm1
                     dm3 =  5.0d0*qinv *  dm2
                     
                  end if
                  
#ifdef PL_SIMD_ERI
                  val(1,l) = eta
                  val(2,l) = del

                  dmx(1,l) = dm0
                  dmx(2,l) = dm1
                  dmx(3,l) = dm2
                  dmx(4,l) = dm3
               enddo
               
               do l = 1, lgp
                  eta = val(1,l)
                  del = val(2,l)

                  dm0 = dmx(1,l)
                  dm1 = dmx(2,l)
                  dm2 = dmx(3,l)
                  dm3 = dmx(4,l)
#endif

! make kernels for ppps =======================================
! 1:d002001 11:d102203 21:d304001 31:t305001 41:u112001
! 2:d002011 12:d103001 22:t002001 32:u001001 42:u112102
! 3:d002102 13:d103011 23:t002102 33:u001102 43:u113011
! 4:d002112 14:d103112 24:t002203 34:u002011 44:u203011
! 5:d002213 15:d113001 25:t002304 35:u012001 45:u213001
! 6:d012001 16:d113102 26:t103001 36:u012011 46:z001011
! 7:d012102 17:d203001 27:t103102 37:u012112 47:z112011
! 8:d012203 18:d203102 28:t103203 38:u102001
! 9:d102001 19:d204011 29:t204001 39:u102011
!10:d102102 20:d214001 30:t204102 40:u102112
!=================================================================
                  
                  del01 =del01+dm2
                  del02 =del02+dm2*del
                  del03 =del03+dm2       *eta
                  del04 =del04+dm2*del   *eta
                  del08 =del08+dm2       *eta**2
                  del05 =del05+dm2*del   *eta**2
                  
                  del22 =del22+dm3
                  del23 =del23+dm3       *eta
                  del24 =del24+dm3       *eta**2
                  del25 =del25+dm3       *eta**3
                  
                  del32 =del32+dm1
                  del34 =del34+dm1*del
                  del33 =del33+dm1       *eta
                  del37 =del37+dm1*del   *eta
                  
                  del46 =del46+dm0*del
                  
                  ppps(32) =ppps(32)+dm1
                  ppps(46) =ppps(46)+dm0     *del
                  
               enddo
               ram03   = ram03   + del03 * ram   
               ram04   = ram04   + del04 * ram   
               ram08   = ram08   + del08 * ram**2
               ram23   = ram23   + del23 * ram   
               ram37   = ram37   + del37 * ram   
               ram24   = ram24   + del24 * ram**2
               ram05   = ram05   + del05 * ram**2
               ram25   = ram25   + del25 * ram**3
               ram42   = ram42   + del33 * ram
               
               ppps(33)= ppps(33)+ del33 * ram   
            enddo
            bet12 = bet12 + del01       *zet**2
            bet09 = bet09 + del01       *zet
            bet15 = bet15 + del01*bet   *zet**2
            bet20 = bet20 + del01*bet   *zet**3
            bet21 = bet21 + del01       *zet**3
            bet10 = bet10 + ram03       *zet    
            bet16 = bet16 + ram03*bet   *zet**2 
            bet18 = bet18 + ram03       *zet**2 
            bet13 = bet13 + del02       *zet**2 
            bet19 = bet19 + del02       *zet**3 
            bet26 = bet26 + del22       *zet**2
            bet29 = bet29 + del22       *zet**3
            bet31 = bet31 + del22       *zet**4
            bet27 = bet27 + ram23       *zet**2 
            bet30 = bet30 + ram23       *zet**3 
            bet38 = bet38 + del32       *zet
            bet41 = bet41 + del32*bet   *zet
            bet45 = bet45 + del32*bet   *zet**2
            bet39 = bet39 + del34       *zet    
            bet43 = bet43 + del34*bet   *zet**2 
            bet44 = bet44 + del34       *zet**2 
            bet11 = bet11 + ram08       *zet    
            bet14 = bet14 + ram04       *zet**2 
            bet28 = bet28 + ram24       *zet**2 
            bet40 = bet40 + ram37       *zet    
            bet42 = bet42 + ram42*bet   *zet    
            bet47 = bet47 + del46*bet   *zet**1 
            
            ppps( 1) = ppps( 1) + del01              *zet
            ppps( 2) = ppps( 2) + del02              *zet    
            ppps( 3) = ppps( 3) + ram03              *zet    
            ppps( 4) = ppps( 4) + ram04              *zet    
            ppps( 5) = ppps( 5) + ram05              *zet    
            ppps( 6) = ppps( 6) + del01       *bet   *zet
            ppps( 7) = ppps( 7) + ram03       *bet   *zet    
            ppps( 8) = ppps( 8) + ram08       *bet   *zet    
            ppps(22) = ppps(22) + del22              *zet
            ppps(23) = ppps(23) + ram23              *zet    
            ppps(24) = ppps(24) + ram24              *zet    
            ppps(25) = ppps(25) + ram25              *zet    
            ppps(34) = ppps(34) + del34              *zet    
            ppps(35) = ppps(35) + del32       *bet   *zet
            ppps(36) = ppps(36) + del34       *bet   *zet    
            ppps(37) = ppps(37) + ram37       *bet   *zet    
            
         enddo
         ppps( 9) = ppps( 9) + bet09  * alp   
         ppps(10) = ppps(10) + bet10  * alp   
         ppps(11) = ppps(11) + bet11  * alp   
         ppps(12) = ppps(12) + bet12  * alp   
         ppps(13) = ppps(13) + bet13  * alp   
         ppps(14) = ppps(14) + bet14  * alp   
         ppps(15) = ppps(15) + bet15  * alp   
         ppps(16) = ppps(16) + bet16  * alp   
         ppps(17) = ppps(17) + bet12  * alp**2
         ppps(18) = ppps(18) + bet18  * alp**2
         ppps(19) = ppps(19) + bet19  * alp**2
         ppps(20) = ppps(20) + bet20  * alp**2
         ppps(21) = ppps(21) + bet21  * alp**3
         ppps(26) = ppps(26) + bet26  * alp   
         ppps(27) = ppps(27) + bet27  * alp   
         ppps(28) = ppps(28) + bet28  * alp   
         ppps(29) = ppps(29) + bet29  * alp**2
         ppps(30) = ppps(30) + bet30  * alp**2
         ppps(31) = ppps(31) + bet31  * alp**3
         ppps(38) = ppps(38) + bet38  * alp   
         ppps(39) = ppps(39) + bet39  * alp   
         ppps(40) = ppps(40) + bet40  * alp   
         ppps(41) = ppps(41) + bet41  * alp   
         ppps(42) = ppps(42) + bet42  * alp   
         ppps(43) = ppps(43) + bet43  * alp   
         ppps(44) = ppps(44) + bet44  * alp**2
         ppps(45) = ppps(45) + bet45  * alp**2
         ppps(47) = ppps(47) + bet47  * alp 
         
      enddo
#ifdef PL_OMP_ERI
!$omp end do
!$omp end parallel
#endif


      END
C** IKPPPP ===========================================================
C          CALUCATE FMT OF PPPP TYPE
C====================================================================
      SUBROUTINE ikpppp(
     &           pppp ,
     &           fdat ,  edat  ,
     &           pp   ,  qq    ,
     &           uu   ,  uv    ,
     &           z    ,  zetpq , 
     &           nasp ,  mxsa  ,  mxprm,   ntprm, 
     &           iasp ,  jasp  ,  kasp ,  lasp ,
     &           ishel,  jshel ,  kshel,  lshel    )

      IMPLICIT DOUBLE PRECISION  (a-h,o-z)
      DIMENSION fdat(31,2001)
      DIMENSION edat(2001)
      DIMENSION pppp(148)

      dimension zetpq(mxprm,mxprm,mxsa,nasp,mxsa,nasp)
      DIMENSION pp(3,mxprm,mxprm),qq(3,mxprm,mxprm)
      DIMENSION uu(mxprm,mxprm),uv(mxprm,mxprm)
      DIMENSION z(mxprm,mxsa,nasp)
      DIMENSION ntprm(mxsa,nasp)

      real(8),parameter::pi    = 3.14159265358979323846d+00
      real(8),parameter::tf(25) = (/
     &                33.0,37.0,41.0,43.0,46.0,49.0,
     &                51.0,54.0,56.0,58.0,61.0,63.0,
     &                66.0,68.0,70.0,72.0,74.0,80.0,
     &                85.0,90.0,95.0,95.0,98.0,99.0,
     &                99.0                           /)
 
      real(8),parameter::ga0 = 1.0d+00
      real(8),parameter::ga1 = 1.000 000 000 000 005 298d+00
      real(8),parameter::ga2 = 0.5d+00
      real(8),parameter::ga3 = 0.166 666 666 598 849 826d+00
      real(8),parameter::ga4 = 0.416 666 666 666 666 667d-01
      real(8),parameter::ga5 = 0.833 355 034 722 222 222d-02
      real(8),parameter::ga6 = 0.138 888 888 888 888 889d-02
c
      real(8),parameter::rminv(5) = (/
     &   1.0d+00,  0.333333333333333333d+00,  0.20d+00,
     &   0.14285714285714286d+00,  0.11111111111111111d+00 /)

#ifdef PL_SIMD_ERI
      dimension dmx(5,mxprm), val(2,mxprm)
#endif

      do ijk=1,148
        pppp(ijk)=0.0d0
      enddo

#ifdef PL_OMP_ERI
!$omp parallel reduction(+:pppp)
!$omp& default(private)
!$omp& shared(fdat,edat,
!$omp&        zetpq,ntprm,z,pp,qq,uu,uv,
!$omp&        ishel,jshel,kshel,lshel,iasp,jasp,kasp,lasp)
#endif

      igp = ntprm(ishel,iasp)
      jgp = ntprm(jshel,jasp)
      kgp = ntprm(kshel,kasp)
      lgp = ntprm(lshel,lasp)

      pi2 = 2.0d0*pi
      
#ifdef PL_OMP_ERI
!$omp do
#endif
      do i = 1, igp
         bet021=0.0d0
         bet022=0.0d0
         bet023=0.0d0
         bet024=0.0d0
         bet025=0.0d0
         bet026=0.0d0
         bet027=0.0d0
         bet028=0.0d0
         bet029=0.0d0
         bet030=0.0d0
         bet031=0.0d0
         bet032=0.0d0
         bet033=0.0d0
         bet034=0.0d0
         bet035=0.0d0
         bet036=0.0d0
         bet037=0.0d0
         bet038=0.0d0
         bet039=0.0d0
         bet040=0.0d0
         bet041=0.0d0
         bet042=0.0d0
         bet045=0.0d0
         bet046=0.0d0
         bet047=0.0d0
         bet048=0.0d0
         bet049=0.0d0
         bet050=0.0d0
         bet051=0.0d0
         bet052=0.0d0
         bet053=0.0d0
         bet054=0.0d0
         bet055=0.0d0
         bet061=0.0d0
         bet062=0.0d0
         bet063=0.0d0
         bet064=0.0d0
         bet065=0.0d0
         bet066=0.0d0
         bet067=0.0d0
         bet068=0.0d0
         bet069=0.0d0
         bet070=0.0d0
         bet086=0.0d0
         bet087=0.0d0
         bet088=0.0d0
         bet089=0.0d0
         bet090=0.0d0
         bet091=0.0d0
         bet092=0.0d0
         bet093=0.0d0
         bet094=0.0d0
         bet095=0.0d0
         bet096=0.0d0
         bet097=0.0d0
         bet098=0.0d0
         bet099=0.0d0
         bet100=0.0d0
         bet103=0.0d0
         bet104=0.0d0
         bet105=0.0d0
         bet106=0.0d0
         bet107=0.0d0
         bet108=0.0d0
         bet109=0.0d0
         bet110=0.0d0
         bet112=0.0d0
         bet113=0.0d0
         bet114=0.0d0
         bet115=0.0d0
         bet116=0.0d0
         bet129=0.0d0
         bet130=0.0d0
         bet131=0.0d0
         bet132=0.0d0
         bet133=0.0d0
         bet134=0.0d0
         bet135=0.0d0
         bet136=0.0d0
         bet137=0.0d0
         bet138=0.0d0
         bet139=0.0d0
         bet140=0.0d0
         bet141=0.0d0
         bet142=0.0d0
         bet143=0.0d0
         bet144=0.0d0
         bet147=0.0d0
         bet148=0.0d0
         alp = z(i,ishel,iasp)
         
         do j = 1, jgp
            
            del004=0.0d0
            del005=0.0d0
            del006=0.0d0
            del056=0.0d0
            del071=0.0d0
            del072=0.0d0
            del119=0.0d0
            del122=0.0d0
            del125=0.0d0
            del147=0.0d0
            ram007=0.0d0
            ram008=0.0d0
            ram009=0.0d0
            ram010=0.0d0
            ram011=0.0d0
            ram012=0.0d0
            ram016=0.0d0
            ram019=0.0d0
            ram020=0.0d0
            ram036=0.0d0
            ram057=0.0d0
            ram058=0.0d0
            ram059=0.0d0
            ram060=0.0d0
            ram073=0.0d0
            ram074=0.0d0
            ram075=0.0d0
            ram076=0.0d0
            ram077=0.0d0
            ram078=0.0d0
            ram079=0.0d0
            ram080=0.0d0
            ram081=0.0d0
            ram085=0.0d0
            ram123=0.0d0
            ram126=0.0d0
            ram128=0.0d0
            ram137=0.0d0
            ram138=0.0d0
            ram148=0.0d0
            
            bet = z(j,jshel,jasp)
            zet = zetpq(j,i,jshel,jasp,ishel,iasp)
            px  = pp (1,j,i)
            py  = pp (2,j,i)
            pz  = pp (3,j,i)
            up  = uu(j,i)
            
            do k = 1, kgp
               del002 = 0.0d0
               del003 = 0.0d0
               del007 = 0.0d0
               del008 = 0.0d0
               del009 = 0.0d0
               del012 = 0.0d0
               del057 = 0.0d0
               del058 = 0.0d0
               del059 = 0.0d0
               del060 = 0.0d0
               del073 = 0.0d0
               del074 = 0.0d0
               del075 = 0.0d0
               del077 = 0.0d0
               del078 = 0.0d0
               del080 = 0.0d0
               del081 = 0.0d0
               del120 = 0.0d0
               del121 = 0.0d0
               del123 = 0.0d0
               del126 = 0.0d0 
               del146 = 0.0d0     
               ram = z(k,kshel,kasp)
               
               do l = 1, lgp
                  del = z(l,lshel,lasp)
                  eta  = zetpq(l,k,lshel,lasp,kshel,kasp)
                  qpx =  qq(1,l,k) - px
                  qpy =  qq(2,l,k) - py
                  qpz =  qq(3,l,k) - pz
                  vq  = uv(l,k)
                  
                  roo2 = 1.0d0 / (zet + eta)
                  qp2  = qpx*qpx + qpy*qpy + qpz*qpz
                  t    = 0.5d0* roo2*qp2 
                  if (t. le .tf(5)) then
                     
                     it = idint((t+0.025d+00)*20.0d+00)
                     dt = 0.05d+00*it - t
                     it = it + 1
                     dt4 = dt*0.25d+00
                     dt5 = dt*0.2d+00
                     dt6 = dt*0.166666666666666667d+00
                     dt2 = dt4 + dt4
                     dt3 = dt6 + dt6
                     f4   = (((((fdat(11,it)*dt6   + fdat(10,it))*dt5
     &                    + fdat(9,it))*dt4  + fdat(8,it))*dt3
     &                    + fdat(7,it))*dt2  + fdat(6,it))*dt
     &                    + fdat(5,it)
                     
                     eed  = (((((ga6*dt + ga5)*dt + ga4)*dt
     &                    + ga3)*dt + ga2)*dt + ga1)*dt + ga0
                     ee   = edat(it)*eed
                     
                     t2   = t*2.0d0
                     f3   = (t2*f4 + ee)*rminv(4)
                     f2   = (t2*f3 + ee)*rminv(3)
                     f1   = (t2*f2 + ee)*rminv(2)
                     f0   =  t2*f1 + ee
                     fpp =  up * vq * dsqrt(roo2)
                     
                     dm0 = fpp*f0
                     dm1 = fpp*f1 *  roo2
                     dm2 = fpp*f2 *( roo2)**2
                     dm3 = fpp*f3 *( roo2)**3
                     dm4 = fpp*f4 *( roo2)**4

                  else 
                     
                     qinv = 1.0d0/qp2
                     dm0=  up*vq*dsqrt( pi2*qinv)*0.5d0
                     dm1 =        qinv *  dm0
                     dm2 =  3.0d0*qinv *  dm1
                     dm3 =  5.0d0*qinv *  dm2
                     dm4 =  7.0d0*qinv *  dm3
                     
                  end if

#ifdef PL_SIMD_ERI
                  val(1,l) = eta
                  val(2,l) = del

                  dmx(1,l) = dm0
                  dmx(2,l) = dm1
                  dmx(3,l) = dm2
                  dmx(4,l) = dm3
                  dmx(5,l) = dm4
               enddo
               
               do l = 1, lgp
                  eta = val(1,l)
                  del = val(2,l)

                  dm0 = dmx(1,l)
                  dm1 = dmx(2,l)
                  dm2 = dmx(3,l)
                  dm3 = dmx(4,l)
                  dm4 = dmx(5,l)
#endif

! make kernels for pppp ====================================
!

                  
                  
                  
                  del004 = del004 + dm2              
                  del005 = del005 + dm2       *eta   
                  del007 = del007 + dm2       *eta   
                  del002 = del002 + dm2       *eta**2
                  del003 = del003 + dm2       *eta**3
                  del006 = del006 + dm2*del   *eta   
                  del008 = del008 + dm2*del   *eta   
                  del009 = del009 + dm2*del   *eta**2
                  del012 = del012 + dm2*del   *eta**3
c==============================================
                  
                  del056 = del056 + dm4       *eta   
                  del057 = del057 + dm4       *eta**2
                  del058 = del058 + dm4       *eta**3
                  del059 = del059 + dm4       *eta**4
                  del060 = del060 + dm4       *eta**5
c==============================================
                  del071 = del071 + dm3       *eta   
                  del073 = del073 + dm3       *eta   
                  del074 = del074 + dm3       *eta**2
                  del077 = del077 + dm3       *eta**3
                  del081 = del081 + dm3       *eta**4
                  del072 = del072 + dm3*del   *eta   
                  del075 = del075 + dm3*del   *eta**2
                  del078 = del078 + dm3*del   *eta**3
                  del080 = del080 + dm3*del   *eta**4
c==============================================
                  del122 = del122 + dm1       
                  del119 = del119 + dm1       *eta   
                  del126 = del126 + dm1       *eta   
                  del121 = del121 + dm1       *eta**2
                  del123 = del123 + dm1*del   *eta   
                  del125 = del125 + dm1*del   *eta   
                  del120 = del120 + dm1*del   *eta**2
c====================================================
                  del147 = del147 + dm0       
                  del146 = del146 + dm0*del   *eta   
                  
                  pppp(  1)=pppp(  1)+dm2     *eta   
                  pppp(117)=pppp(117)+dm1     *eta   
                  pppp(118)=pppp(118)+dm1     *del   *eta   
                  pppp(145)=pppp(145)+dm0
                  
               enddo
               ram007 = ram007 + del007*ram   
               ram008 = ram008 + del008*ram   
               ram009 = ram009 + del009*ram   
               ram011 = ram011 + del009*ram**2
               ram016 = ram016 + del002*ram   
               ram010 = ram010 + del002*ram**2
               ram019 = ram019 + del012*ram**2
               ram012 = ram012 + del012*ram**3
               ram036 = ram036 + del003*ram**2
               ram020 = ram020 + del003*ram**3
               ram057 = ram057 + del057*ram   
               ram058 = ram058 + del058*ram**2
               ram059 = ram059 + del059*ram**3
               ram060 = ram060 + del060*ram**4
               ram073 = ram073 + del073*ram   
               ram074 = ram074 + del074*ram   
               ram076 = ram076 + del074*ram**2
               ram075 = ram075 + del075*ram   
               ram077 = ram077 + del077*ram**2
               ram079 = ram079 + del077*ram**3
               ram078 = ram078 + del078*ram**2
               ram080 = ram080 + del080*ram**3
               ram081 = ram081 + del081*ram**4
               ram085 = ram085 + del081*ram**3
               ram123 = ram123 + del123*ram   
               ram126 = ram126 + del126*ram   
               ram128 = ram128 + del120*ram**2
               ram137 = ram137 + del120*ram   
               ram138 = ram138 + del121*ram**2
               ram148 = ram148 + del146*ram   
               
               pppp(  2) = pppp(  2) + del002            *ram   
               pppp(  3) = pppp(  3) + del003            *ram**2
               pppp(119) = pppp(119) + del126            *ram   
               pppp(120) = pppp(120) + del120            *ram   
               pppp(121) = pppp(121) + del121            *ram**2
               pppp(146) = pppp(146) + del146            *ram 
      enddo
      
      pppp(  4) = pppp(  4) + del004              *zet
      pppp(  5) = pppp(  5) + del005              *zet
      pppp(  6) = pppp(  6) + del006              *zet
      pppp(  7) = pppp(  7) + ram007              *zet
      pppp(  8) = pppp(  8) + ram008              *zet
      pppp(  9) = pppp(  9) + ram009              *zet
      pppp( 10) = pppp( 10) + ram010              *zet
      pppp( 11) = pppp( 11) + ram011              *zet   
      pppp( 12) = pppp( 12) + ram012              *zet   
      pppp( 13) = pppp( 13) + del005       *bet   *zet   
      pppp( 14) = pppp( 14) + del006       *bet   *zet   
      pppp( 15) = pppp( 15) + ram007       *bet   *zet   
      pppp( 16) = pppp( 16) + ram016       *bet   *zet   
      pppp( 17) = pppp( 17) + ram009       *bet   *zet   
      pppp( 18) = pppp( 18) + ram010       *bet   *zet   
      pppp( 19) = pppp( 19) + ram019       *bet   *zet   
      pppp( 20) = pppp( 20) + ram020       *bet   *zet   
      pppp( 56) = pppp( 56) + del056              *zet   
      pppp( 57) = pppp( 57) + ram057              *zet   
      pppp( 58) = pppp( 58) + ram058              *zet   
      pppp( 59) = pppp( 59) + ram059              *zet   
      pppp( 60) = pppp( 60) + ram060              *zet   
      pppp( 71) = pppp( 71) + del071              *zet   
      pppp( 72) = pppp( 72) + del072              *zet   
      pppp( 73) = pppp( 73) + ram073              *zet   
      pppp( 74) = pppp( 74) + ram074              *zet   
      pppp( 75) = pppp( 75) + ram075              *zet   
      pppp( 76) = pppp( 76) + ram076              *zet   
      pppp( 77) = pppp( 77) + ram077              *zet   
      pppp( 78) = pppp( 78) + ram078              *zet   
      pppp( 79) = pppp( 79) + ram079              *zet   
      pppp( 80) = pppp( 80) + ram080              *zet   
      pppp( 81) = pppp( 81) + ram081              *zet   
      pppp( 82) = pppp( 82) + del071       *bet   *zet   
      pppp( 83) = pppp( 83) + ram074       *bet   *zet   
      pppp( 84) = pppp( 84) + ram077       *bet   *zet   
      pppp( 85) = pppp( 85) + ram085       *bet   *zet   
      pppp(122) = pppp(122) + del122              *zet   
      pppp(123) = pppp(123) + ram123              *zet   
      pppp(124) = pppp(124) + del122       *bet   *zet   
      pppp(125) = pppp(125) + del125       *bet   *zet   
      pppp(126) = pppp(126) + ram126       *bet   *zet   
      pppp(127) = pppp(127) + ram123       *bet   *zet   
      pppp(128) = pppp(128) + ram128       *bet   *zet   

      bet021 = bet021 + del005       *zet   
      bet034 = bet034 + del005*bet   *zet   
      bet037 = bet037 + del005*bet   *zet**2
      bet042 = bet042 + del005       *zet**2
      bet022 = bet022 + del006       *zet   
      bet030 = bet030 + del006       *zet**2
      bet038 = bet038 + del006*bet   *zet**2
      bet051 = bet051 + del006*bet   *zet**3
      bet053 = bet053 + del006       *zet**3
      bet023 = bet023 + ram007       *zet   
      bet031 = bet031 + ram007       *zet**2
      bet039 = bet039 + ram007*bet   *zet**2
      bet052 = bet052 + ram007*bet   *zet**3
      bet054 = bet054 + ram007       *zet**3
      bet024 = bet024 + ram016       *zet   
      bet035 = bet035 + ram016*bet   *zet   
      bet050 = bet050 + ram016*bet   *zet**2
      bet025 = bet025 + ram009       *zet   
      bet040 = bet040 + ram009*bet   *zet**2
      bet045 = bet045 + ram009       *zet**2
      bet026 = bet026 + ram010       *zet   
      bet041 = bet041 + ram010*bet   *zet**2
      bet046 = bet046 + ram010       *zet**2
      bet027 = bet027 + ram019       *zet   
      bet028 = bet028 + ram020       *zet   
      bet029 = bet029 + del004       *zet**2
      bet047 = bet047 + del004       *zet**3
      bet032 = bet032 + ram008       *zet**2
      bet048 = bet048 + ram008       *zet**3
      bet033 = bet033 + ram011       *zet**2
      bet036 = bet036 + ram036*bet   *zet   
      bet049 = bet049 + del005*bet   *zet**2
      bet055 = bet055 + del005*bet   *zet**3
      bet061 = bet061 + del056       *zet**2
      bet065 = bet065 + del056       *zet**3
      bet068 = bet068 + del056       *zet**4
      bet070 = bet070 + del056       *zet**5
      bet062 = bet062 + ram057       *zet**2
      bet066 = bet066 + ram057       *zet**3
      bet069 = bet069 + ram057       *zet**4
      bet063 = bet063 + ram058       *zet**2
      bet067 = bet067 + ram058       *zet**3
      bet064 = bet064 + ram059       *zet**2
      bet086 = bet086 + del071       *zet   
      bet090 = bet090 + del071       *zet**2
      bet098 = bet098 + del071*bet   *zet**2
      bet104 = bet104 + del071       *zet**3
      bet109 = bet109 + del071*bet   *zet**3
      bet115 = bet115 + del071*bet   *zet**4
      bet116 = bet116 + del071       *zet**4
      bet087 = bet087 + ram074       *zet   
      bet093 = bet093 + ram074       *zet**2
      bet099 = bet099 + ram074*bet   *zet**2
      bet110 = bet110 + ram074*bet   *zet**3
      bet112 = bet112 + ram074       *zet**3
      bet088 = bet088 + ram077       *zet   
      bet100 = bet100 + ram077*bet   *zet**2
      bet103 = bet103 + ram077       *zet**2
      bet089 = bet089 + ram085       *zet   
      bet091 = bet091 + del072       *zet**2
      bet105 = bet105 + del072       *zet**3
      bet113 = bet113 + del072       *zet**4
      bet092 = bet092 + ram073       *zet**2
      bet106 = bet106 + ram073       *zet**3
      bet114 = bet114 + ram073       *zet**4
      bet094 = bet094 + ram075       *zet**2
      bet107 = bet107 + ram075       *zet**3
      bet095 = bet095 + ram076       *zet**2
      bet108 = bet108 + ram076       *zet**3
      bet096 = bet096 + ram078       *zet**2
      bet097 = bet097 + ram079       *zet**2
      bet129 = bet129 + del122       *zet   
      bet139 = bet139 + del122*bet   *zet**2
      bet141 = bet141 + del122       *zet**2
      bet130 = bet130 + del125       *zet   
      bet135 = bet135 + del125*bet   *zet   
      bet143 = bet143 + del125*bet   *zet**2
      bet131 = bet131 + ram126       *zet   
      bet136 = bet136 + ram126*bet   *zet   
      bet144 = bet144 + ram126*bet   *zet**2
      bet132 = bet132 + ram123       *zet   
      bet140 = bet140 + ram123*bet   *zet**2
      bet142 = bet142 + ram123       *zet**2
      bet133 = bet133 + ram128       *zet   
      bet134 = bet134 + del119*bet   *zet   
      bet137 = bet137 + ram137*bet   *zet   
      bet138 = bet138 + ram138*bet   *zet   
      bet147 = bet147 + del147*bet   *zet   
      bet148 = bet148 + ram148*bet   *zet   

      enddo
      pppp( 21)=pppp( 21)+bet021*alp   
      pppp( 22)=pppp( 22)+bet022*alp   
      pppp( 23)=pppp( 23)+bet023*alp   
      pppp( 24)=pppp( 24)+bet024*alp   
      pppp( 25)=pppp( 25)+bet025*alp   
      pppp( 26)=pppp( 26)+bet026*alp   
      pppp( 27)=pppp( 27)+bet027*alp   
      pppp( 28)=pppp( 28)+bet028*alp   
      pppp( 29)=pppp( 29)+bet029*alp   
      pppp( 30)=pppp( 30)+bet030*alp   
      pppp( 31)=pppp( 31)+bet031*alp   
      pppp( 32)=pppp( 32)+bet032*alp   
      pppp( 33)=pppp( 33)+bet033*alp   
      pppp( 34)=pppp( 34)+bet034*alp   
      pppp( 35)=pppp( 35)+bet035*alp   
      pppp( 36)=pppp( 36)+bet036*alp   
      pppp( 37)=pppp( 37)+bet037*alp   
      pppp( 38)=pppp( 38)+bet038*alp   
      pppp( 39)=pppp( 39)+bet039*alp   
      pppp( 40)=pppp( 40)+bet040*alp   
      pppp( 41)=pppp( 41)+bet041*alp   
      pppp( 42)=pppp( 42)+bet042*alp**2
      pppp( 43)=pppp( 43)+bet030*alp**2
      pppp( 44)=pppp( 44)+bet031*alp**2
      pppp( 45)=pppp( 45)+bet045*alp**2
      pppp( 46)=pppp( 46)+bet046*alp**2
      pppp( 47)=pppp( 47)+bet047*alp**2
      pppp( 48)=pppp( 48)+bet048*alp**2
      pppp( 49)=pppp( 49)+bet049*alp**2
      pppp( 50)=pppp( 50)+bet050*alp**2
      pppp( 51)=pppp( 51)+bet051*alp**2
      pppp( 52)=pppp( 52)+bet052*alp**2
      pppp( 53)=pppp( 53)+bet053*alp**3
      pppp( 54)=pppp( 54)+bet054*alp**3
      pppp( 55)=pppp( 55)+bet055*alp**3
      pppp( 61)=pppp( 61)+bet061*alp   
      pppp( 62)=pppp( 62)+bet062*alp   
      pppp( 63)=pppp( 63)+bet063*alp   
      pppp( 64)=pppp( 64)+bet064*alp   
      pppp( 65)=pppp( 65)+bet065*alp**2
      pppp( 66)=pppp( 66)+bet066*alp**2
      pppp( 67)=pppp( 67)+bet067*alp**2
      pppp( 68)=pppp( 68)+bet068*alp**3
      pppp( 69)=pppp( 69)+bet069*alp**3
      pppp( 70)=pppp( 70)+bet070*alp**4
      pppp( 86)=pppp( 86)+bet086*alp   
      pppp( 87)=pppp( 87)+bet087*alp   
      pppp( 88)=pppp( 88)+bet088*alp   
      pppp( 89)=pppp( 89)+bet089*alp   
      pppp( 90)=pppp( 90)+bet090*alp   
      pppp( 91)=pppp( 91)+bet091*alp   
      pppp( 92)=pppp( 92)+bet092*alp   
      pppp( 93)=pppp( 93)+bet093*alp   
      pppp( 94)=pppp( 94)+bet094*alp   
      pppp( 95)=pppp( 95)+bet095*alp   
      pppp( 96)=pppp( 96)+bet096*alp   
      pppp( 97)=pppp( 97)+bet097*alp   
      pppp( 98)=pppp( 98)+bet098*alp   
      pppp( 99)=pppp( 99)+bet099*alp   
      pppp(100)=pppp(100)+bet100*alp   
      pppp(101)=pppp(101)+bet090*alp**2
      pppp(102)=pppp(102)+bet093*alp**2
      pppp(103)=pppp(103)+bet103*alp**2
      pppp(104)=pppp(104)+bet104*alp**2
      pppp(105)=pppp(105)+bet105*alp**2
      pppp(106)=pppp(106)+bet106*alp**2
      pppp(107)=pppp(107)+bet107*alp**2
      pppp(108)=pppp(108)+bet108*alp**2
      pppp(109)=pppp(109)+bet109*alp**2
      pppp(110)=pppp(110)+bet110*alp**2
      pppp(111)=pppp(111)+bet104*alp**3
      pppp(112)=pppp(112)+bet112*alp**3
      pppp(113)=pppp(113)+bet113*alp**3
      pppp(114)=pppp(114)+bet114*alp**3
      pppp(115)=pppp(115)+bet115*alp**3
      pppp(116)=pppp(116)+bet116*alp**4
      pppp(129)=pppp(129)+bet129*alp   
      pppp(130)=pppp(130)+bet130*alp   
      pppp(131)=pppp(131)+bet131*alp   
      pppp(132)=pppp(132)+bet132*alp   
      pppp(133)=pppp(133)+bet133*alp   
      pppp(134)=pppp(134)+bet134*alp
      pppp(135)=pppp(135)+bet135*alp   
      pppp(136)=pppp(136)+bet136*alp   
      pppp(137)=pppp(137)+bet137*alp   
      pppp(138)=pppp(138)+bet138*alp   
      pppp(139)=pppp(139)+bet139*alp   
      pppp(140)=pppp(140)+bet140*alp   
      pppp(141)=pppp(141)+bet141*alp**2
      pppp(142)=pppp(142)+bet142*alp**2
      pppp(143)=pppp(143)+bet143*alp**2
      pppp(144)=pppp(144)+bet144*alp**2
      pppp(147)=pppp(147)+bet147*alp   
      pppp(148)=pppp(148)+bet148*alp   
      enddo
#ifdef PL_OMP_ERI
!$omp end do
!$omp end parallel
#endif

      END
