! -*- coding: utf-8 -*-
!
! Dual licensed under the MIT or GPL Version 3 license.
! See 'LICENSE.MIT' for the definition of the MIT license,
! and 'COPYING' for the definition of the GPL license.
!
C======================================================================
C
C======================================================================
      subroutine swz_cutoff(
     O     swzij  ,
     O     iaoslix , iaosljx , iaoslkx, iaosllx ,
     O     mwki    , mwkr    , mxxang , cutthr  ,
     O     ntei0cut,  ntei1cut,
     O     ishp,npswz,
     I     zz     , cc     ,
     I     xyz    ,
     I     fdat   , edat   ,
     I     ntprm  , ntshty , 
     I     nsh2ssh, nsh2asp, nsh2atm,
     I     mik    ,
     I     nfsz   , nesz   ,
     I     natom  , mxsa   , mxprm  , nao , nasp, nshell,
     I     nsh2shty, nao2sh  ,
     I     e_rrrr  ,
     I     nr_s    , nr_e    , ns_s   , ns_e    ,
     I     nt_s    , nt_e    , nu_s   , nu_e    ,
     I     ir_s    , ir_e    , is_s   , is_e    ,
     I     it_s    , it_e    , iu_s   , iu_e    ,
     I     erithr  ,
     I     icomm, nproc, myrank,
     I     lprint, ldebug  ,
     O     ierr    )
      use pl_prof_class
c$$$      use mpi
      implicit real(8) (a-h,o-z)

      include 'mpif.h'

      dimension swzij(nshell,nshell)
      dimension iaoslix(ir_s:ir_e,3), iaosljx(is_s:is_e,3)
      dimension iaoslkx(it_s:it_e,3), iaosllx(iu_s:iu_e,3)

      dimension xyz(3,natom)
      dimension e_rrrr(nshell)
      dimension
     $     fdat(nfsz,nesz), edat(nesz),
     $     ntshty(mxsa,nasp), ntprm(mxsa,nasp),
     $     zz(mxprm,mxsa,nasp), cc(mxprm,mxsa,nasp),
     $     nsh2shty(nshell),
     $     nsh2ssh(nshell), nsh2atm(nshell), nsh2asp(nshell),
     $     nao2sh(nao),ishp(2,nshell*(nshell+1)/2)

      integer(8) ntei0cut, ntei1cut
      real(8),allocatable :: wswzij(:,:)
      allocate (wswzij(nshell,nshell))
      wswzij = 0.d0

      swzij(:,:) = 0.0d0

      call mpi_barrier(icomm, istat)
      call pl_prof_region_begin('p:swz_rsrs12')
      call swz_rsrs12(
!    O     swzij  ,
     O     wswzij ,
     I     zz     , cc     ,
     I     xyz    ,
     I     fdat   , edat   ,
     I     ntprm  , ntshty , 
     I     nsh2ssh, nsh2asp, nsh2atm,
     I     mik    ,
     I     nfsz   , nesz   ,
     I     1      , nshell , 1      , nshell,
     I     natom  , mxsa   , mxprm  , nao , nasp, nshell,
     I     myrank  , nproc   ,
     O     ierr    )
      if ( ierr.ne.0 ) go to 9999
      call mpi_barrier(icomm, istat)
      call pl_prof_region_end('p:swz_rsrs12')

      call mpi_barrier(icomm, istat)
      call pl_prof_region_begin('m:swz_allreduce')
      call mpi_allreduce(wswzij,swzij,nshell*nshell,
     +     mpi_double_precision,mpi_sum,icomm,ierr)
      call mpi_barrier(icomm, istat)
      call pl_prof_region_end('m:swz_allreduce')

      deallocate(wswzij)

      if ( ldebug > 1 ) then
         call ummtw( 'swzij',
     $        swzij, nshell, nshell, nshell, 8, 6, ierr )
      endif

      call ipasize(
     O     iaoslix , iaosljx , iaoslkx, iaosllx ,
     O     mwki    , mwkr    , mxxang , cutthr  ,
     I     nsh2ssh , nsh2asp , nsh2shty, nao2sh  ,
     I     ntshty  , 
     I     e_rrrr  ,
     I     nr_s    , nr_e    , ns_s   , ns_e    ,
     I     nt_s    , nt_e    , nu_s   , nu_e    ,
     I     ir_s    , ir_e    , is_s   , is_e    ,
     I     it_s    , it_e    , iu_s   , iu_e    ,
     I     mxsa    , nao     , nshell , nasp    ,
     I     erithr  ,
     O     ierr    )
      if ( ierr.ne.0 ) go to 9999

      if ( ldebug > 0 ) then
         write(6,*) 'erithr=',erithr
         write(6,*) 'cutthr=',cutthr
      endif

      np = 0
      do i = 1,nshell
        do j = 1,i
          if (swzij(i,j) < cutthr) cycle
          np = np + 1
          ishp(1,np) = i
          ishp(2,np) = j
        enddo
      enddo
      npswz = np

      if ( ldebug > 1 ) then
         write(6,*) 'ntei1cut=',ntei1cut
      endif

      dtei1cut = ntei1cut
      if ( ldebug > 1 ) then
         write(6,'('' Schwarz ('',i5,'') = '',f15.0)') myrank,dtei1cut
      endif

 9999 return
      end
c$$$C======================================================================
c$$$C
c$$$C======================================================================
c$$$      subroutine eri18p(
c$$$     O     index_tei, val_tei, ntei2cut,
c$$$     I     zz2      , cc     , zetpq   , xyz    ,
c$$$     I     fdat     , edat   ,
c$$$     I     nsh2ssh  , nsh2atm, nsh2asp , nao2sh , natm2asp,
c$$$     I     nsh2shty , ntshty , ntprm   ,
c$$$     I     swzij    , nang2n  ,
c$$$     I     iaoslix  , iaosljx, iaoslkx , iaosllx,
c$$$     I     ntei1cut , mxxang ,
c$$$     I     nfsz     , nesz   , 
c$$$     I     natom    , mxsa   , mxprm   , nao    , nshell  , nasp,
c$$$     I     erithr   , cutthr ,
c$$$     I     ishp     , npswz  ,
c$$$     I     mem_limit,l_incore,
c$$$     I     incore_p,incore_q,
c$$$     I     nr_s     , nr_e   , ns_s    , ns_e   ,
c$$$     I     nt_s     , nt_e   , nu_s    , nu_e   ,
c$$$     I     ir_s     , ir_e   , is_s    , is_e   ,
c$$$     I     it_s     , it_e   , iu_s    , iu_e   ,
c$$$     I     mwki     , mwkr   ,
c$$$     I     icomm, nproc, myrank,
c$$$     I     lprint, ldebug,
c$$$     O     ierr)
c$$$c$$$      use mpi
c$$$      implicit real(8) (a-h,o-z)
c$$$
c$$$      include 'mpif.h'
c$$$
c$$$      integer(8) ntei1cut, ntei2cut
c$$$      DIMENSION val_tei(ntei1cut)
c$$$      integer(2) index_tei(4,ntei1cut)
c$$$
c$$$      dimension swzij(nshell,nshell)
c$$$      dimension iaoslix(ir_s:ir_e,3), iaosljx(is_s:is_e,3)
c$$$      dimension iaoslkx(it_s:it_e,3), iaosllx(iu_s:iu_e,3)
c$$$
c$$$      dimension xyz(3,natom)
c$$$      dimension ishp(2,npswz)
c$$$
c$$$      dimension
c$$$     $     zetpq(mxprm,mxprm,mxsa,nasp,mxsa,nasp),
c$$$     $     zz2(mxprm,mxsa,nasp), nang2n(5,5,5,5),
c$$$     $     fdat(nfsz,nesz), edat(nesz),
c$$$     $     ntshty(mxsa,nasp), ntprm(mxsa,nasp),
c$$$     $     cc(mxprm,mxsa,nasp),
c$$$     $     natm2asp(natom), nsh2shty(nshell),
c$$$     $     nsh2ssh(nshell), nsh2atm(nshell), nsh2asp(nshell),
c$$$     $     nao2sh(nao)
c$$$
c$$$      integer(8) mem_limit
c$$$
c$$$      logical :: l_incore
c$$$
c$$$      call ieri18p(
c$$$     O     index_tei, val_tei, ntei2cut,
c$$$     I     zz2      , cc     , zetpq   , xyz    ,
c$$$     I     fdat     , edat   ,
c$$$     I     nsh2ssh  , nsh2atm, nsh2asp , nao2sh , natm2asp,
c$$$     I     nsh2shty , ntshty , ntprm   ,
c$$$     I     swzij    , nang2n  ,
c$$$     I     iaoslix  , iaosljx, iaoslkx , iaosllx,
c$$$     I     ntei1cut , mxxang ,
c$$$     I     nfsz     , nesz   , 
c$$$     I     natom    , mxsa   , mxprm   , nao    , nshell  , nasp,
c$$$     I     erithr   , cutthr , 
c$$$     I     ishp, npswz,
c$$$     I     mem_limit,l_incore,
c$$$     I     incore_p, incore_q,
c$$$     I     nr_s     , nr_e   , ns_s    , ns_e   ,
c$$$     I     nt_s     , nt_e   , nu_s    , nu_e   ,
c$$$     I     ir_s     , ir_e   , is_s    , is_e   ,
c$$$     I     it_s     , it_e   , iu_s    , iu_e   ,
c$$$     I     mwki     , mwkr   ,
c$$$     I     nproc, myrank,
c$$$     $     ierr     , ldebug )
c$$$
c$$$      if ( lprint >= 0 ) then
c$$$         if ( myrank == 0 ) then
c$$$            write(6,*)" "
c$$$            write(6,*)"=== Integral incore-direct ==="
c$$$            print *,"Incore flag          = ", l_incore
c$$$            print *,"Incore memory size   = ", 8*mem_limit/1000000,"MB"
c$$$            print *,"Incore word   size   = ", mem_limit
c$$$            print *,"Limit of TEI size    = ", ntei2cut
c$$$         endif
c$$$      endif
c$$$
c$$$      if ( ierr.ne.0 ) go to 9999
c$$$
c$$$      dtei2cut = ntei2cut
c$$$      if ( ldebug > 1 ) then
c$$$         write(6,'('' Integral('',i5,'') = '',f15.0)') myrank,dtei2cut
c$$$      endif
c$$$
c$$$      if ( lprint >= 2 ) then
c$$$         call mpi_reduce(
c$$$     $        dtei2cut, dtei2cut_t, 1,
c$$$     $        MPI_DOUBLE_PRECISION, MPI_SUM, 0, icomm, ierr )
c$$$         if ( ierr.ne.0 ) go to 9999
c$$$
c$$$         call mpi_reduce(
c$$$     $        dtei2cut, dtei2cut_max, 1,
c$$$     $        MPI_DOUBLE_PRECISION, MPI_MAX, 0, icomm, ierr )
c$$$         if ( ierr.ne.0 ) go to 9999
c$$$
c$$$         call mpi_reduce(
c$$$     $        dtei2cut, dtei2cut_min, 1,
c$$$     $        MPI_DOUBLE_PRECISION, MPI_MIN, 0, icomm, ierr )
c$$$         if ( ierr.ne.0 ) go to 9999
c$$$
c$$$         if ( myrank == 0 ) then
c$$$            dtei2cut_GB     = dtei2cut_t  /1024/1024/1024*(8+2*4)
c$$$            dtei2cut_min_GB = dtei2cut_min/1024/1024/1024*(8+2*4)
c$$$            dtei2cut_max_GB = dtei2cut_max/1024/1024/1024*(8+2*4)
c$$$            if ( dtei2cut_t > huge(i) ) then
c$$$               write(6,
c$$$     $              '(/'' Number of nonzero two-electron integrals = '',
c$$$     $                  d15.8,f8.2,'' GB'',
c$$$     $                  '' (min '',f5.2,'' GB, max'',f5.2,'' GB)'')')
c$$$     $              dtei2cut_t, dtei2cut_GB,
c$$$     $              dtei2cut_min_GB, dtei2cut_max_GB
c$$$            else
c$$$               write(6,
c$$$     $              '(/'' Number of nonzero two-electron integrals = '',
c$$$     $                  f15.0,f8.2,'' GB'',
c$$$     $                  '' (min '',f5.2,'' GB, max'',f5.2,'' GB)'')')
c$$$     $              dtei2cut_t, dtei2cut_GB,
c$$$     $              dtei2cut_min_GB, dtei2cut_max_GB
c$$$            endif
c$$$         endif
c$$$      endif
c$$$
c$$$ 9999 return
c$$$      end
