! -*- coding: utf-8 -*-
!
! Dual licensed under the MIT or GPL Version 3 license.
! See 'LICENSE.MIT' for the definition of the MIT license,
! and 'COPYING' for the definition of the GPL license.
!
!OPARA_INDEX ========================================================
! SET INDEX TABLE OF SHELL2AO
!
! -- OUTPUT --
! NSH2AO     : INDEX TABLE OF SHELL-->FIRST NUMBER OF AO
! IAOINDEX   : 1: satart DIR number  2: end dir number
!              ordinary start = 1, end = mm(iang)
! JAOINDEX
!=================-     =============================================

      subroutine opara_index(
     &     nsh2ao,iaoindex,jaoindex,
     &     nsh2shty,nao2dir,nao2sh,
     &     nr_s,nr_e,ns_s,ns_e,
     &     ir_s,ir_e,is_s,is_e,
     &     nshell, nao,
     &     lprint,ierr )

      implicit real(8)(a-h,o-z)

      dimension nsh2ao(nshell)
      dimension iaoindex(2,ir_s:ir_e),jaoindex(2,is_s:is_e)

      dimension nsh2shty(nshell),nao2dir(nao) ,nao2sh(nao)

      ishelo = 0

      do i = 1,nao
         isheln = nao2sh(i)
         if (isheln.ne.ishelo) then
            nsh2ao(isheln) = i
            ishelo = isheln
         endif
      enddo

      ilength = ir_e - ir_s
      jlength = is_e - is_s

c$$$      write(6,*) 'ierr=',ierr
c$$$      write(6,*) 'ir_e, ir_s =', ir_e, ir_s
c$$$      write(6,*) 'is_e, is_s =', is_e, is_s

      if (ilength.lt.0) ierr = 1
      if (jlength.lt.0) ierr = 1
      if (ierr.ge.1) write(*,*)"error ao-range"

      if (ilength.ge.2) then

         do i = ir_s+1,ir_e-1
            iang = nsh2shty(i)
            iaoindex(1,i) = 1
            iaoindex(2,i) = (iang+1) * iang /2
         enddo
         
         iang = nsh2shty(ir_s)
         iaoindex(1,ir_s) = nao2dir(nr_s)
         iaoindex(2,ir_s) = (iang+1) * iang/2

         iang = nsh2shty(ir_e)
         iaoindex(1,ir_e) = 1
         iaoindex(2,ir_e) = nao2dir(nr_e)

      endif

      if (jlength.ge.2) then
         
         do i = is_s+1,is_e-1
            iang = nsh2shty(i)
            jaoindex(1,i) = 1
            jaoindex(2,i) = (iang+1) * iang /2
         enddo
         
         iang = nsh2shty(is_s)
         jaoindex(1,is_s) = nao2dir(ns_s)
         jaoindex(2,is_s) = (iang+1) * iang /2

         iang = nsh2shty(is_e)
         jaoindex(1,is_e) = 1
         jaoindex(2,is_e) = nao2dir(ns_e)

      endif

      if (ilength.eq.0) then
         iaoindex(1,ir_s) = nao2dir(nr_s)
         iaoindex(2,ir_e) = nao2dir(nr_e)
      endif

      if (jlength.eq.0) then
         jaoindex(1,is_s) = nao2dir(ns_s)
         jaoindex(2,is_e) = nao2dir(ns_e)
      endif

      if (ilength.eq.1) then
         iang = nsh2shty(ir_s)
         iaoindex(1,ir_s) = nao2dir(nr_s)
         iaoindex(2,ir_s) = (iang+1)*iang/2

         iaoindex(1,ir_e) = 1
         iaoindex(2,ir_e) = nao2dir(nr_e)
      endif

      if (jlength.eq.1) then
         iang = nsh2shty(is_s)
         jaoindex(1,is_s) = nao2dir(ns_s)
         jaoindex(2,is_s) = (iang+1)*iang/2

         jaoindex(1,is_e) = 1
         jaoindex(2,is_e) = nao2dir(ns_e)
      endif

      if (lprint.eq.1) then
         write(*,*) "IAOINDEX"
         do i=ir_s,ir_e
            write(*,*)i,iaoindex(1,i),iaoindex(2,i)
         enddo

         write(*,*) "jaoindex"
         do i=is_s,is_e
            write(*,*)i,jaoindex(1,i),jaoindex(2,i)
         enddo

      endif

      end

!!IOPARA_ZET = ======================================================
! pre-calc
! zetpq2 = 1 / (alp + bet)
! zxi    = alp * bet / (alp + bet)
! zp32   = (pi*zet) **1.5
!                                           
!! -----------------------------------------------------------------

      subroutine iopara_zet(zetpq2, zxi, zp32, zz, nasp, mxprm, mxsa )

      implicit real(8)(a-h,o-z)

      dimension zetpq2(mxprm,mxprm,mxsa,nasp,mxsa,nasp)
      dimension zxi   (mxprm,mxprm,mxsa,nasp,mxsa,nasp)
      dimension zp32  (mxprm,mxprm,mxsa,nasp,mxsa,nasp)

      dimension zz(mxprm,mxsa,nasp)

      pi = 3.14159265358979323846D+00 

      do n = 1,nasp
         do l = 1,mxsa
            do m = 1,nasp
               do k = 1,mxsa
                  do j = 1,mxprm
                     do i = 1,mxprm
!                       ??
c$$$                        zetpq2(i,j,k,m,l,n)=1.d0/(zz(i,k,m) + zz(j,l,n))
                        if ( zz(i,k,m) + zz(j,l,n) .gt. 0.0d0 ) then
                        zetpq2(i,j,k,m,l,n)=1.d0/(zz(i,k,m) + zz(j,l,n))
                        else
                        zetpq2(i,j,k,m,l,n)=0.d0
                        endif
!                       ??

                        zxi(i,j,k,m,l,n)  = zz(i,k,m)*zz(j,l,n)
     &                                    * zetpq2(i,j,k,m,l,n)

                       zp32(i,j,k,m,l,n)=(pi*zetpq2(i,j,k,m,l,n))**1.5d0

                     enddo
                  enddo
               enddo
            enddo
         enddo
      enddo

      end subroutine
