! -*- coding: utf-8 -*-
!
! Dual licensed under the MIT or GPL Version 3 license.
! See 'LICENSE.MIT' for the definition of the MIT license,
! and 'COPYING' for the definition of the GPL license.
!
!> MO クラス
!!
module mo_data_class
  implicit none
  private

  public::mo_data_type
  public::new_data
  public::free_data
  public::save_data
  public::save_data_f
  public::load_data
  public::load_data_f
  public::bcast_data
  public::print_data
  public::print_data_w_ao_labels

  public::load_nmo_data

  type mo_data_type
     integer::lao
     integer::nao
     integer::nmo
     real(8),allocatable::c(:,:)
     real(8),allocatable::oe(:)
     real(8),allocatable::occ(:)
  end type mo_data_type

  character(len=*),private,parameter::url = 'data_mo'
  integer,private,parameter::id = 1
  character(len=*),private,parameter::unit_au = 'a.u.'
  character(len=*),private,parameter::unit = ' '

contains
  !-----------------------------------------------------------------------------
  !> 領域確保
  subroutine new_data(self, lao, nao, nmo, ldebug, istat)
    type(mo_data_type),intent(inout)::self
    integer,intent(in)::lao, nao, nmo
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    allocate(self%c(lao,nmo), self%oe(nmo), self%occ(nmo), stat=istat)
    if (istat /= 0) return

    self%c(:,:) = 0.0d0
    self%oe(:)  = 0.0d0
    self%occ(:) = 0.0d0
    self%lao = lao
    self%nao = nao
    self%nmo = nmo

  end subroutine new_data

  !-----------------------------------------------------------------------------
  !> 領域解放
  subroutine free_data(self, ldebug, istat)
    type(mo_data_type),intent(inout)::self
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    deallocate(self%c, self%oe, self%occ, stat=istat)
    if (istat /= 0) return

  end subroutine free_data

  !-----------------------------------------------------------------------------
  !> BMS形式ファイルへの保存
  subroutine save_data(self, ldebug, istat)
    use pl_bms_class
    type(mo_data_type),intent(in)::self
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    call pl_bms_write(self%nao, url, 'nao', id, unit, istat)
    if (istat /= 0) return

    call pl_bms_write(self%nmo, url, 'nmo', id, unit, istat)
    if (istat /= 0) return

    call pl_bms_write(self%c, self%lao, self%nao, self%nmo,&
         url, 'mo_coefficients', id, unit, istat)
    if (istat /= 0) return

    call pl_bms_write(self%oe, self%nmo,&
         url,'orbital_energies', id, unit, istat)
    if (istat /= 0) return

    call pl_bms_write(self%occ, self%nmo,&
         url,'occupation_numbers', id, unit, istat)
    if (istat /= 0) return

  end subroutine save_data

  !-----------------------------------------------------------------------------
  !> BMS形式ファイルへの保存
  subroutine save_data_f(self, url2, ldebug, istat)
    use pl_bms_class
    type(mo_data_type),intent(in)::self
    character(len=*),intent(in)::url2
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    call pl_bms_write(self%nao, url2, 'nao', id, unit, istat)
    if (istat /= 0) return

    call pl_bms_write(self%nmo, url2, 'nmo', id, unit, istat)
    if (istat /= 0) return

    call pl_bms_write(self%c, self%lao, self%nao, self%nmo,&
         url2, 'mo_coefficients', id, unit, istat)
    if (istat /= 0) return

    call pl_bms_write(self%oe, self%nmo,&
         url2,'orbital_energies', id, unit, istat)
    if (istat /= 0) return

    call pl_bms_write(self%occ, self%nmo,&
         url2,'occupation_numbers', id, unit, istat)
    if (istat /= 0) return

  end subroutine save_data_f

  !-----------------------------------------------------------------------------
  !> BMS形式ファイルからの読み込み
  subroutine load_data(self, ldebug, istat)
    use pl_bms_class
    type(mo_data_type),intent(inout)::self
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    integer::lao, nao, nmo

    call pl_bms_read(nao, url, 'nao', id, istat)
    if (istat /= 0) return

    call pl_bms_read(nmo, url, 'nmo', id, istat)
    if (istat /= 0) return

    lao = nao
    call new_data(self, lao, nao, nmo, ldebug, istat)
    if (istat /= 0) return

    call pl_bms_read(self%c, lao, nao, nmo,&
         url, 'mo_coefficients', id, istat)
    if (istat /= 0) return

    call pl_bms_read(self%oe, nmo,&
         url, 'orbital_energies', id, istat)
    if (istat /= 0) return

    call pl_bms_read(self%occ, nmo,&
         url,'occupation_numbers', id, istat)
    if (istat /= 0) return

  end subroutine load_data

  !-----------------------------------------------------------------------------
  !> BMS形式ファイルからの読み込み
  subroutine load_data_f(self, url2, ldebug, istat)
    use pl_bms_class
    type(mo_data_type),intent(inout)::self
    character(len=*),intent(in)::url2
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    integer::lao, nao, nmo

    call pl_bms_read(nao, url2, 'nao', id, istat)
    if (istat /= 0) return

    call pl_bms_read(nmo, url2, 'nmo', id, istat)
    if (istat /= 0) return

    lao = nao
    call new_data(self, lao, nao, nmo, ldebug, istat)
    if (istat /= 0) return

    call pl_bms_read(self%c, lao, nao, nmo,&
         url2, 'mo_coefficients', id, istat)
    if (istat /= 0) return

    call pl_bms_read(self%oe, nmo,&
         url2, 'orbital_energies', id, istat)
    if (istat /= 0) return

    call pl_bms_read(self%occ, nmo,&
         url2,'occupation_numbers', id, istat)
    if (istat /= 0) return

  end subroutine load_data_f

  !-----------------------------------------------------------------------------
  !> BMS形式ファイルからの読み込み
  subroutine load_nmo_data(nmo, ldebug, istat)
    use pl_bms_class
    integer,intent(out)::nmo
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    call pl_bms_read(nmo, url, 'nmo', id, istat)
    if (istat /= 0) return

  end subroutine load_nmo_data

  !-----------------------------------------------------------------------------
  !> データBcast
  subroutine bcast_data(self, pl_mpi, ldebug, istat)
    use pl_mpi_class
    type(mo_data_type),intent(inout)::self
    type(pl_mpi_type),intent(in)::pl_mpi
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    integer::lao, nao, nmo
    integer::icomm, myrank

    istat = 0

    icomm  = pl_mpi%comm
    myrank = pl_mpi%myrank

    if (myrank == 0) then
       nao = self%nao
       nmo = self%nmo
    end if
    call mpi_bcast(nao, 1, MPI_INTEGER, 0, icomm, istat)
    if (istat /= 0) return
    call mpi_bcast(nmo, 1, MPI_INTEGER, 0, icomm, istat)
    if (istat /= 0) return
    if (myrank /= 0) then
       lao = nao
       call new_data(self, lao, nao, nmo, ldebug, istat)
       if (istat /= 0) return
    end if

    call mpi_bcast(self%c, nao*nmo, MPI_DOUBLE_PRECISION, 0, icomm, istat)
    if (istat /= 0) return
    call mpi_bcast(self%oe, nmo, MPI_DOUBLE_PRECISION, 0, icomm, istat)
    if (istat /= 0) return

  end subroutine bcast_data
  
  !-----------------------------------------------------------------------------
  !> データの表示
  subroutine print_data(self)
    type(mo_data_type),intent(in)::self
    integer::istat

    if (self%nao > 0 .and. self%nmo > 0) then
       call ummtw2('MO Coefficients',&
            self%c, self%oe, self%lao, self%nao, self%nmo, 8, 6, istat)
    end if

  end subroutine print_data

  !-----------------------------------------------------------------------------
  !> データの表示
  subroutine print_data_w_ao_labels(self, atoms, ldebug, istat)
    use atoms_class
    type(mo_data_type),intent(in)::self
    type(atoms_type),intent(in)::atoms
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    character(len=8),allocatable::labels(:)
    integer::natom, iao, nao

    if (self%nao > 0 .and. self%nmo > 0) then

       natom = atoms%num
       nao = atoms%basis%nao
       allocate(labels(nao), stat=istat)
       if (istat /= 0) return
       call get_ao_labels(atoms, labels, nao, ldebug, istat)

       if (ldebug > 0) then
          do iao = 1, self%nao
             write(6,*) labels(iao)
          end do
       end if
       call ummtwl('MO Coefficients',&
            labels, self%c, self%lao, self%nao, self%nmo, 8, 6, istat)

       deallocate(labels, stat=istat)
       if (istat /= 0) return
    end if

  end subroutine print_data_w_ao_labels

  !-----------------------------------------------------------------------------
end module mo_data_class
