! -*- coding: utf-8 -*-
!
! Dual licensed under the MIT or GPL Version 3 license.
! See 'LICENSE.MIT' for the definition of the MIT license,
! and 'COPYING' for the definition of the GPL license.
!
!> (dd|dd) integrals
!
! Target class:
!    (dd|dd)
! Applying the horizontal recurrence relation:
!    (dd|dd) = HRR{ (fp|dd) (dp|dd) }
!    (fp|dd) = HRR{ (gs|dd) (fs|dd) }
!    (gs|dd) = HRR{ (gs|fp) (gs|dp) }
!    (gs|fp) = HRR{ (gs|gs) (gs|fs) }
!    (gs|dp) = HRR{ (gs|fs) (gs|ds) }
!    (dp|dd) = HRR{ (fs|dd) (ds|dd) }
!    (fs|dd) = HRR{ (fs|fp) (fs|dp) }
!    (fs|fp) = HRR{ (fs|gs) (fs|fs) }
!    (fs|dp) = HRR{ (fs|fs) (fs|ds) }
!    (ds|dd) = HRR{ (ds|fp) (ds|dp) }
!    (ds|fp) = HRR{ (ds|gs) (ds|fs) }
!    (ds|dp) = HRR{ (ds|fs) (ds|ds) }
! Classes to be formes by contraction of primitive integrals:
!    (ds|ds) (ds|fs) (ds|gs) (fs|ds) (fs|fs) (fs|gs) (gs|ds) (gs|fs) (gs|gs)
! Applying the vertical recurrence relation:
!    [gs|gs](0)    = VRR{ [gs|fs](0..1) [gs|ds](0..1) [fs|fs](1) }
!    [gs|fs](0..1) = VRR{ [gs|ds](0..2) [gs|ps](0..2) [fs|ds](1..2) }
!    [gs|ds](0..2) = VRR{ [gs|ps](0..3) [gs|ss](0..3) [fs|ps](1..3) }
!    [gs|ps](0..3) = VRR{ [gs|ss](0..4) [fs|ss](1..4) }
!    [gs|ss](0..4) = VRR{ [fs|ss](0..5) [ds|ss](0..5) }
!    [fs|gs](0)    = VRR{ [fs|fs](0..1) [fs|ds](0..1) [ds|fs](1) }
!    [fs|fs](0..1) = VRR{ [fs|ds](0..2) [fs|ps](0..2) [ds|ds](1..2) }
!    [fs|ds](0..2) = VRR{ [fs|ps](0..3) [fs|ss](0..3) [ds|ps](1..3) }
!    [fs|ps](0..3) = VRR{ [fs|ss](0..4) [ds|ss](1..4) }
!    [fs|ss](0..5) = VRR{ [ds|ss](0..6) [ps|ss](0..6) }
!    [ds|gs](0)    = VRR{ [ds|fs](0..1) [ds|ds](0..1) [ps|fs](1) }
!    [ds|fs](0..1) = VRR{ [ds|ds](0..2) [ds|ps](0..2) [ps|ds](1..2) }
!    [ds|ds](0..2) = VRR{ [ds|ps](0..3) [ds|ss](0..3) [ps|ps](1..3) }
!    [ds|ps](0..3) = VRR{ [ds|ss](0..4) [ps|ss](1..4) }
!    [ds|ss](0..6) = VRR{ [ps|ss](0..7) [ss|ss](0..7) }
!    [ps|fs](1)    = VRR{ [ps|ds](1..2) [ps|ps](1..2) [ss|ds](2) }
!    [ps|ds](1..2) = VRR{ [ps|ps](1..3) [ps|ss](1..3) [ss|ps](2..3) }
!    [ps|ps](1..3) = VRR{ [ps|ss](1..4) [ss|ss](2..4) }
!    [ps|ss](0..7) = VRR{ [ss|ss](0..8) }
!    [ss|ds](2)    = VRR{ [ss|ps](2..3) [ss|ss](2..3) }
!    [ss|ps](2..3) = VRR{ [ss|ss](2..4) }
!    [ss|ss](2..8)
!
! References:
!  [1] S.Obara and A.Saika, J.Chem.Phys. 84, 3963 (1986).
!  [2] M.Head-Gordon and J.A.Pople, J.Chem.Phys. 89, 5777 (1988).
!
subroutine os_dddd_ov5(&
     fj,fk,d,nao,&
     swz_ijij,oozeta,nprims2_ij,sab,ab,p,pa,npsh_ij,nij,&
     swz_klkl,ooeta,nprims2_kl,scd,cd,q,qc,npsh_kl,nkl,&
     mxprim2,nshell,nsh2ao,fmt,&
     nteiv2,erithr,myrank,nproc)
  use physical_constants_class
  use integral_fmt_class
  use omp_lib

  implicit real(8) (a-h,o-z)

  integer,intent(in)::myrank,nproc
  integer,intent(in)::nao
  real(8),intent(in)::d(nao,nao)
  real(8),intent(inout)::fj(nao,nao),fk(nao,nao)

  integer(8),intent(inout)::nteiv2
  real(8),intent(in)::erithr

  integer,intent(in)::nshell
  integer,intent(in)::nsh2ao(nshell)
  type(fmt_type),intent(in)::fmt

  integer,intent(in)::nij
  integer,intent(in)::nkl
  integer,intent(in)::mxprim2
  real(8),intent(in)::ab(3,nij),P(3,mxprim2,nij),PA(3,mxprim2,nij)
  real(8),intent(in)::cd(3,nkl),Q(3,mxprim2,nkl),QC(3,mxprim2,nkl)
  real(8),intent(in)::swz_ijij(nij),oozeta(mxprim2,nij),sab(mxprim2,nij)
  real(8),intent(in)::swz_klkl(nkl),ooeta (mxprim2,nkl),scd(mxprim2,nkl)
  integer,intent(in)::nprims2_ij(nij),npsh_ij(2,nij)
  integer,intent(in)::nprims2_kl(nkl),npsh_kl(2,nkl)

  real(8)::ec(1296)
  integer::ntask

  real(8),allocatable::QPs(:,:),rhos(:),ssssx(:,:)
  real(8),allocatable::ts(:),QP2s(:)

  nteiv2 = 0

  allocate(QPs(3,mxprim2),rhos(mxprim2),ssssx(0:8,mxprim2))
  allocate(ts(mxprim2),QP2s(mxprim2))

  erithr2 = erithr*erithr
  tosqrtpi = 2.d0/dsqrt(pi)
!!$  ntask = -1

!$omp do schedule(dynamic)
!!$!$omp do
  do ij = myrank+1, nij, nproc
!!$  do ij = 1, nij
     swz_ijij_ = swz_ijij(ij)
     nrs = nprims2_ij(ij)
     ABx = ab(1,ij)
     ABy = ab(2,ij)
     ABz = ab(3,ij)

     do kl = 1, ij
        if (swz_ijij_*swz_klkl(kl) < erithr2) cycle
!!$        ntask = mod(ntask+1,nproc); if (ntask /= myrank) cycle
        ntu = nprims2_kl(kl)
        CDx = cd(1,kl)
        CDy = cd(2,kl)
        CDz = cd(3,kl)

        include "os_dddd_ini.inc"

        do irs = 1, nrs
           Px = P(1,irs,ij)
           Py = P(2,irs,ij)
           Pz = P(3,irs,ij)
           PAx = PA(1,irs,ij)
           PAy = PA(2,irs,ij)
           PAz = PA(3,irs,ij)
           Sp = sab(irs,ij)
           ooz = oozeta(irs,ij)
           oo2zeta = 0.5d0 * ooz

           do itu = 1, ntu
              Qx = Q(1,itu,kl)
              Qy = Q(2,itu,kl)
              Qz = Q(3,itu,kl)
              ooe = ooeta(itu,kl)
              QPx = Qx - Px
              QPy = Qy - Py
              QPz = Qz - Pz
              QP2 = QPx*QPx + QPy*QPy + QPz*QPz
              rho = 1.0d0/(ooz + ooe)
              t = rho * QP2
              ts(itu) = t
              QPs(1,itu) = QPx
              QPs(2,itu) = QPy
              QPs(3,itu) = QPz
              rhos(itu)  = rho
           end do

           do itu = 1, ntu
              t = ts(itu)
              Sq = scd(itu,kl)
              rho = rhos(itu)

              if (t <= tf(9)) then
                 it = idint((t + 0.025d0)*20.0d0)
                 dt = 0.05d0*it - t
                 it = it + 1
                 dt4 = dt * 0.25d0
                 dt5 = dt * 0.2d0
                 dt6 = dt * 0.1666666666666666666d0
                 dt2 = dt4 + dt4
                 dt3 = dt6 + dt6
                 ft  = (((((fmt%fdat(15,it)*dt6 + fmt%fdat(14,it))*dt5&
                     + fmt%fdat(13,it))*dt4 + fmt%fdat(12,it))*dt3&
                     + fmt%fdat(11,it))*dt2 + fmt%fdat(10,it))*dt + fmt%fdat(9,it)
                 et  = ((((((ga6*dt + ga5)*dt + ga4)*dt&
                     + ga3)*dt + ga2)*dt + ga1)*dt + ga0) * fmt%edat(it)
                 t2 = 2.d0 * t
                 cn = dsqrt(rho) * Sq * tosqrtpi * Sp
                 etcn = et * cn
                 ssss8 = ft * cn
                 ssss7 = (ssss8 * t2 + etcn) * rminv(8)
                 ssss6 = (ssss7 * t2 + etcn) * rminv(7)
                 ssss5 = (ssss6 * t2 + etcn) * rminv(6)
                 ssss4 = (ssss5 * t2 + etcn) * rminv(5)
                 ssss3 = (ssss4 * t2 + etcn) * rminv(4)
                 ssss2 = (ssss3 * t2 + etcn) * rminv(3)
                 ssss1 = (ssss2 * t2 + etcn) * rminv(2)
                 ssss0 =  ssss1 * t2 + etcn
              else
                 tinv = 1.d0/t
                 ssss0 = dsqrt(rho*tinv) * Sq * Sp
                 ssss1 = ssss0 * tinv * 0.5d0
                 ssss2 = ssss1 * tinv * 1.5d0
                 ssss3 = ssss2 * tinv * 2.5d0
                 ssss4 = ssss3 * tinv * 3.5d0
                 ssss5 = ssss4 * tinv * 4.5d0
                 ssss6 = ssss5 * tinv * 5.5d0
                 ssss7 = ssss6 * tinv * 6.5d0
                 ssss8 = ssss7 * tinv * 7.5d0
              end if

              ssssx(0,itu) = ssss0
              ssssx(1,itu) = ssss1
              ssssx(2,itu) = ssss2
              ssssx(3,itu) = ssss3
              ssssx(4,itu) = ssss4
              ssssx(5,itu) = ssss5
              ssssx(6,itu) = ssss6
              ssssx(7,itu) = ssss7
              ssssx(8,itu) = ssss8
           end do

           do itu = 1, ntu
              ssss0 = ssssx(0,itu)
              ssss1 = ssssx(1,itu)
              ssss2 = ssssx(2,itu)
              ssss3 = ssssx(3,itu)
              ssss4 = ssssx(4,itu)
              ssss5 = ssssx(5,itu)
              ssss6 = ssssx(6,itu)
              ssss7 = ssssx(7,itu)
              ssss8 = ssssx(8,itu)
              QPx = QPs(1,itu)
              QPy = QPs(2,itu)
              QPz = QPs(3,itu)
              QCx = QC(1,itu,kl)
              QCy = QC(2,itu,kl)
              QCz = QC(3,itu,kl)
              rho = rhos(itu)
              rhoozeta = rho*ooz
              ooe = ooeta(itu,kl)
              rhooeta = rho*ooe
              oo2eta = 0.5d0 * ooe
              rhoo2ze = rhoozeta*oo2eta
              WPx = QPx * rhoozeta
              WPy = QPy * rhoozeta
              WPz = QPz * rhoozeta
              WQx = WPx - QPx
              WQy = WPy - QPy
              WQz = WPz - QPz

              include "os_dddd_vrr.inc"
           end do
        end do

        include "os_dddd_hrr.inc"
        include "os_dddd_str.inc"

        i = npsh_ij(1,ij)
        j = npsh_ij(2,ij)
        k = npsh_kl(1,kl)
        l = npsh_kl(2,kl)

        ir = nsh2ao(i)
        is = nsh2ao(j)
        it = nsh2ao(k)
        iu = nsh2ao(l)

        ikey = 0                     !               ...  1
        if(ij==kl) then
           ikey = 4                  !         ij=kl ... 1/2
           if (i==j) ikey=5          ! i=j,k=l,ij=kl ... 1/8
        else
           if (i==j) ikey=1          ! i=j           ... 1/2
           if (k==l) ikey=2          !     k=l       ... 1/2
           if (i==j.and.k==l) ikey=3 ! i=j,k=l       ... 1/4
        end if
        select case(ikey)
        case(0)
           icount = 0
           do ix=1,6
              do jx=1,6
                 do kx=1,6
                    do lx=1,6
                       icount = icount + 1
                       if (dabs(ec(icount)) <= erithr) cycle
                       nteiv2 = nteiv2 + 1
!!$                       index_teiv(1,nteiv2) = ir + ix - 1
!!$                       index_teiv(2,nteiv2) = is + jx - 1
!!$                       index_teiv(3,nteiv2) = it + kx - 1
!!$                       index_teiv(4,nteiv2) = iu + lx - 1
!!$                       teiv(nteiv2) = ec(icount)
                       irx = ir + ix - 1
                       isx = is + jx - 1
                       itx = it + kx - 1
                       iux = iu + lx - 1
                       eri = ec(icount)
                       if (irx == isx) eri = 0.5d0*eri
                       if (itx == iux) eri = 0.5d0*eri
                       if (((irx == itx) .and. (isx == iux)) .or.&
                            ((irx == iux) .and. (isx == itx))) eri = 0.5d0*eri
                       fj(irx,isx) = fj(irx,isx) + d(itx,iux)*eri
                       fj(itx,iux) = fj(itx,iux) + d(irx,isx)*eri
                       fk(irx,itx) = fk(irx,itx) + d(isx,iux)*eri
                       fk(irx,iux) = fk(irx,iux) + d(isx,itx)*eri
                       fk(isx,itx) = fk(isx,itx) + d(irx,iux)*eri
                       fk(isx,iux) = fk(isx,iux) + d(irx,itx)*eri
                    end do
                 end do
              end do
           end do
        case(1)
           do ix=1,6
              do jx=ix,6
                 do kx=1,6
                    do lx=1,6
                       icount = 216*(ix-1) + 36*(jx-1) + 6*(kx-1) + lx
                       if (dabs(ec(icount)) <= erithr) cycle
                       nteiv2 = nteiv2 + 1
!!$                       index_teiv(1,nteiv2) = ir + ix - 1
!!$                       index_teiv(2,nteiv2) = is + jx - 1
!!$                       index_teiv(3,nteiv2) = it + kx - 1
!!$                       index_teiv(4,nteiv2) = iu + lx - 1
!!$                       teiv(nteiv2) = ec(icount)
                       irx = ir + ix - 1
                       isx = is + jx - 1
                       itx = it + kx - 1
                       iux = iu + lx - 1
                       eri = ec(icount)!*0.5d0
                       if (irx == isx) eri = 0.5d0*eri
                       if (itx == iux) eri = 0.5d0*eri
                       if (((irx == itx) .and. (isx == iux)) .or.&
                            ((irx == iux) .and. (isx == itx))) eri = 0.5d0*eri
                       fj(irx,isx) = fj(irx,isx) + d(itx,iux)*eri
                       fj(itx,iux) = fj(itx,iux) + d(irx,isx)*eri
                       fk(irx,itx) = fk(irx,itx) + d(isx,iux)*eri
                       fk(irx,iux) = fk(irx,iux) + d(isx,itx)*eri
                       fk(isx,itx) = fk(isx,itx) + d(irx,iux)*eri
                       fk(isx,iux) = fk(isx,iux) + d(irx,itx)*eri
                    end do
                 end do
              end do
           end do
        case(2)
           do ix=1,6
              do jx=1,6
                 do kx=1,6
                    do lx=kx,6
                       icount = 216*(ix-1) + 36*(jx-1) + 6*(kx-1) + lx
                       if (dabs(ec(icount)) <= erithr) cycle
                       nteiv2 = nteiv2 + 1
!!$                       index_teiv(1,nteiv2) = ir + ix - 1
!!$                       index_teiv(2,nteiv2) = is + jx - 1
!!$                       index_teiv(3,nteiv2) = it + kx - 1
!!$                       index_teiv(4,nteiv2) = iu + lx - 1
!!$                       teiv(nteiv2) = ec(icount)
                       irx = ir + ix - 1
                       isx = is + jx - 1
                       itx = it + kx - 1
                       iux = iu + lx - 1
                       eri = ec(icount)!*0.5d0
                       if (irx == isx) eri = 0.5d0*eri
                       if (itx == iux) eri = 0.5d0*eri
                       if (((irx == itx) .and. (isx == iux)) .or.&
                            ((irx == iux) .and. (isx == itx))) eri = 0.5d0*eri
                       fj(irx,isx) = fj(irx,isx) + d(itx,iux)*eri
                       fj(itx,iux) = fj(itx,iux) + d(irx,isx)*eri
                       fk(irx,itx) = fk(irx,itx) + d(isx,iux)*eri
                       fk(irx,iux) = fk(irx,iux) + d(isx,itx)*eri
                       fk(isx,itx) = fk(isx,itx) + d(irx,iux)*eri
                       fk(isx,iux) = fk(isx,iux) + d(irx,itx)*eri
                    end do
                 end do
              end do
           end do
        case(3)
           do ix=1,6
              do jx=ix,6
                 do kx=1,6
                    do lx=kx,6
                       icount = 216*(ix-1) + 36*(jx-1) + 6*(kx-1) + lx
                       if (dabs(ec(icount)) <= erithr) cycle
                       nteiv2 = nteiv2 + 1
!!$                       index_teiv(1,nteiv2) = ir + ix - 1
!!$                       index_teiv(2,nteiv2) = is + jx - 1
!!$                       index_teiv(3,nteiv2) = it + kx - 1
!!$                       index_teiv(4,nteiv2) = iu + lx - 1
!!$                       teiv(nteiv2) = ec(icount)
                       irx = ir + ix - 1
                       isx = is + jx - 1
                       itx = it + kx - 1
                       iux = iu + lx - 1
                       eri = ec(icount)!*0.25d0
                       if (irx == isx) eri = 0.5d0*eri
                       if (itx == iux) eri = 0.5d0*eri
                       if (((irx == itx) .and. (isx == iux)) .or.&
                            ((irx == iux) .and. (isx == itx))) eri = 0.5d0*eri
                       fj(irx,isx) = fj(irx,isx) + d(itx,iux)*eri
                       fj(itx,iux) = fj(itx,iux) + d(irx,isx)*eri
                       fk(irx,itx) = fk(irx,itx) + d(isx,iux)*eri
                       fk(irx,iux) = fk(irx,iux) + d(isx,itx)*eri
                       fk(isx,itx) = fk(isx,itx) + d(irx,iux)*eri
                       fk(isx,iux) = fk(isx,iux) + d(irx,itx)*eri
                    end do
                 end do
              end do
           end do
        case(4)
           do ix=1,6
              do jx=1,6
                 do kx=ix,6
                    do lx=1,6
                       if((ix==kx).and.(jx<lx)) cycle
                       icount = 216*(ix-1) + 36*(jx-1) + 6*(kx-1) + lx
                       if (dabs(ec(icount)) <= erithr) cycle
                       nteiv2 = nteiv2 + 1
!!$                       index_teiv(1,nteiv2) = ir + ix - 1
!!$                       index_teiv(2,nteiv2) = is + jx - 1
!!$                       index_teiv(3,nteiv2) = it + kx - 1
!!$                       index_teiv(4,nteiv2) = iu + lx - 1
!!$                       teiv(nteiv2) = ec(icount)
                       irx = ir + ix - 1
                       isx = is + jx - 1
                       itx = it + kx - 1
                       iux = iu + lx - 1
                       eri = ec(icount)!*0.5d0
                       if (irx == isx) eri = 0.5d0*eri
                       if (itx == iux) eri = 0.5d0*eri
                       if (((irx == itx) .and. (isx == iux)) .or.&
                            ((irx == iux) .and. (isx == itx))) eri = 0.5d0*eri
                       fj(irx,isx) = fj(irx,isx) + d(itx,iux)*eri
                       fj(itx,iux) = fj(itx,iux) + d(irx,isx)*eri
                       fk(irx,itx) = fk(irx,itx) + d(isx,iux)*eri
                       fk(irx,iux) = fk(irx,iux) + d(isx,itx)*eri
                       fk(isx,itx) = fk(isx,itx) + d(irx,iux)*eri
                       fk(isx,iux) = fk(isx,iux) + d(irx,itx)*eri
                    end do
                 end do
              end do
           end do
        case(5)
           do ix=1,6
              do jx=ix,6
                 do kx=ix,6
                    do lx=kx,6
                       if((ix==kx).and.(jx<lx)) cycle
                       icount = 216*(ix-1) + 36*(jx-1) + 6*(kx-1) + lx
                       if (dabs(ec(icount)) <= erithr) cycle
                       nteiv2 = nteiv2 + 1
!!$                       index_teiv(1,nteiv2) = ir + ix - 1
!!$                       index_teiv(2,nteiv2) = is + jx - 1
!!$                       index_teiv(3,nteiv2) = it + kx - 1
!!$                       index_teiv(4,nteiv2) = iu + lx - 1
!!$                       teiv(nteiv2) = ec(icount)
                       irx = ir + ix - 1
                       isx = is + jx - 1
                       itx = it + kx - 1
                       iux = iu + lx - 1
                       eri = ec(icount)!*0.125d0
                       if (irx == isx) eri = 0.5d0*eri
                       if (itx == iux) eri = 0.5d0*eri
                       if (((irx == itx) .and. (isx == iux)) .or.&
                            ((irx == iux) .and. (isx == itx))) eri = 0.5d0*eri
                       fj(irx,isx) = fj(irx,isx) + d(itx,iux)*eri
                       fj(itx,iux) = fj(itx,iux) + d(irx,isx)*eri
                       fk(irx,itx) = fk(irx,itx) + d(isx,iux)*eri
                       fk(irx,iux) = fk(irx,iux) + d(isx,itx)*eri
                       fk(isx,itx) = fk(isx,itx) + d(irx,iux)*eri
                       fk(isx,iux) = fk(isx,iux) + d(irx,itx)*eri
                    end do
                 end do
              end do
           end do
        end select
     end do
  end do
!$omp end do nowait
end subroutine os_dddd_ov5
