! -*- coding: utf-8 -*-
!
! Dual licensed under the MIT or GPL Version 3 license.
! See 'LICENSE.MIT' for the definition of the MIT license,
! and 'COPYING' for the definition of the GPL license.
!
!------------------------------------------------------------------------------
!> kinetic energy integrals
!
! References:
!  [1] S.Obara and A.Saika, J.Chem.Phys. 84, 3963 (1986).
!
subroutine os_dkei_t1(g_,d,nao,tables,atoms)
  use integral_tables_class
  use atoms_class
  use pl_prof_class
  integer,intent(in)::nao
  type(atoms_type),intent(in)::atoms
  real(8),intent(inout)::g_(3,atoms%natom)
  real(8),intent(in)::d(nao,nao)
  type(tables_type),intent(in)::tables

  real(8),allocatable::g(:,:)

  allocate(g(3,atoms%natom))
  g(:,:) = 0.0d0

!$omp parallel&
!$omp& default(private)&
!$omp& shared(d,nao,tables,atoms)&
!$omp& reduction(+:g)
  call pl_prof_region_begin('os_dkei_ss_t1')
  call os_dkei_ss_t1(&
       g,d,nao,&
       tables%ishp,tables%npsh,tables%npsh2pr,tables%nsh2atm,tables%nsh2ao,&
       atoms%xyz,tables%ovl,tables%bdz,tables%xsi,tables%zet,&
       tables%nshell,tables%nshelp,tables%nprimp,atoms%natom)
  call pl_prof_region_end('os_dkei_ss_t1')
  if (tables%mang == 1) go to 9

  call pl_prof_region_begin('os_dkei_ps_t1')
  call os_dkei_ps_t1(&
       g,d,nao,&
       tables%ishp,tables%npsh,tables%npsh2pr,tables%nsh2atm,tables%nsh2ao,&
       atoms%xyz,tables%ovl,tables%bdz,tables%xsi,tables%zet,&
       tables%nshell,tables%nshelp,tables%nprimp,atoms%natom)
  call pl_prof_region_end('os_dkei_ps_t1')

  call pl_prof_region_begin('os_dkei_pp_t1')
  call os_dkei_pp_t1(&
       g,d,nao,&
       tables%ishp,tables%npsh,tables%npsh2pr,tables%nsh2atm,tables%nsh2ao,&
       atoms%xyz,tables%ovl,tables%bdz,tables%xsi,tables%zet,&
       tables%nshell,tables%nshelp,tables%nprimp,atoms%natom)
  call pl_prof_region_end('os_dkei_pp_t1')
  if (tables%mang == 2) go to 9

  call pl_prof_region_begin('os_dkei_ds_t1')
  call os_dkei_ds_t1(&
       g,d,nao,&
       tables%ishp,tables%npsh,tables%npsh2pr,tables%nsh2atm,tables%nsh2ao,&
       atoms%xyz,tables%ovl,tables%bdz,tables%xsi,tables%zet,&
       tables%nshell,tables%nshelp,tables%nprimp,atoms%natom)
  call pl_prof_region_end('os_dkei_ds_t1')

  call pl_prof_region_begin('os_dkei_dp_t1')
  call os_dkei_dp_t1(&
       g,d,nao,&
       tables%ishp,tables%npsh,tables%npsh2pr,tables%nsh2atm,tables%nsh2ao,&
       atoms%xyz,tables%ovl,tables%bdz,tables%xsi,tables%zet,&
       tables%nshell,tables%nshelp,tables%nprimp,atoms%natom)
  call pl_prof_region_end('os_dkei_dp_t1')

  call pl_prof_region_begin('os_dkei_dd_t1')
  call os_dkei_dd_t1(&
       g,d,nao,&
       tables%ishp,tables%npsh,tables%npsh2pr,tables%nsh2atm,tables%nsh2ao,&
       atoms%xyz,tables%ovl,tables%bdz,tables%xsi,tables%zet,&
       tables%nshell,tables%nshelp,tables%nprimp,atoms%natom)
  call pl_prof_region_end('os_dkei_dd_t1')
9 continue
!$omp end parallel

  g_(:,:) = g_(:,:) + g(:,:)*2.0d0

end subroutine os_dkei_t1

!------------------------------------------------------------------------------
!> ss kinetic energy integrals
!
subroutine os_dkei_ss_t1(&
     g,dm,nao,&
     ishp,npsh,npsh2pr,nsh2atm,nsh2ao,xyz,ss,xiozetaa,xi,zeta,&
     nshell,nshellp,nprimp,natom)
  use physical_constants_class

  implicit real(8) (a-h,o-z)
  integer,intent(in)::nao

  integer,intent(in)::nshell,nshellp,nprimp,natom
  integer,intent(in)::ishp(6),npsh(2,nshellp),npsh2pr(nshellp+1)
  integer,intent(in)::nsh2atm(nshell),nsh2ao(nshell)
  real(8),intent(in)::ss(nprimp),xiozetaa(nprimp),xi(nprimp),zeta(nprimp)
  real(8),intent(in)::xyz(3,natom)

  real(8),intent(inout)::g(3,natom)
  real(8),intent(in)::dm(nao,nao)

  real(8)::v(1,3)

!$omp do schedule(dynamic)
  do ij = 1, ishp(1)
     i = npsh(1,ij)
     j = npsh(2,ij)
     iatom = nsh2atm(i)
     jatom = nsh2atm(j)
     Ax = xyz(1,iatom)
     Ay = xyz(2,iatom)
     Az = xyz(3,iatom)
     Bx = xyz(1,jatom)
     By = xyz(2,jatom)
     Bz = xyz(3,jatom)
     ABx = Ax - Bx
     ABy = Ay - By
     ABz = Az - Bz
     AB2 = ABx*ABx + ABy*ABy + ABz*ABz

t_ss_dAx = 0.0d0
t_ss_dAy = 0.0d0
t_ss_dAz = 0.0d0
do irs = npsh2pr(ij), npsh2pr(ij+1) - 1
o_ss = ss(irs)
zeta_ = zeta(irs)
xi_ = xi(irs)
xi2 = 2.0d0*xi_
t_ss = o_ss*xi_*( 3.0d0 - xi2 * AB2 )
xioza = xiozetaa(irs)
PAx = -ABx*xioza
PAy = -ABy*xioza
PAz = -ABz*xioza
o_pxs = PAx*o_ss
o_pys = PAy*o_ss
o_pzs = PAz*o_ss
t_pxs = PAx*t_ss + xi2*o_pxs
t_pys = PAy*t_ss + xi2*o_pys
t_pzs = PAz*t_ss + xi2*o_pzs
beta = xioza*zeta_
alpha = zeta_ - beta
alpha2 = 2.0d0*alpha
t_ss_dAx = t_ss_dAx + alpha2*t_pxs
t_ss_dAy = t_ss_dAy + alpha2*t_pys
t_ss_dAz = t_ss_dAz + alpha2*t_pzs
end do
v(1,1) = t_ss_dAx
v(1,2) = t_ss_dAy
v(1,3) = t_ss_dAz

     i = npsh(1,ij)
     j = npsh(2,ij)
     iao = nsh2ao(i)
     jao = nsh2ao(j)

!!$     write(6,'(''ec(ss)='',3f12.7)') ((v(i,j),i=1,1),j=1,3)
     do ixyz = 1, 3
        g(ixyz,iatom) = g(ixyz,iatom) + dm(iao,jao)*v(1,ixyz)
     end do
     do ixyz = 1, 3
        g(ixyz,jatom) = g(ixyz,jatom) - dm(iao,jao)*v(1,ixyz)
     end do
  end do
!$omp end do nowait
end subroutine os_dkei_ss_t1

!------------------------------------------------------------------------------
!> ps kinetic energy integrals
!
subroutine os_dkei_ps_t1(&
     g,dm,nao,&
     ishp,npsh,npsh2pr,nsh2atm,nsh2ao,xyz,ss,xiozetaa,xi,zeta,&
     nshell,nshellp,nprimp,natom)
  use physical_constants_class

  implicit real(8) (a-h,o-z)
  integer,intent(in)::nao

  integer,intent(in)::nshell,nshellp,nprimp,natom
  integer,intent(in)::ishp(6),npsh(2,nshellp),npsh2pr(nshellp+1)
  integer,intent(in)::nsh2atm(nshell),nsh2ao(nshell)
  real(8),intent(in)::ss(nprimp),xiozetaa(nprimp),xi(nprimp),zeta(nprimp)
  real(8),intent(in)::xyz(3,natom)

  real(8),intent(inout)::g(3,natom)
  real(8),intent(in)::dm(nao,nao)

  real(8)::v(3,3)

!$omp do schedule(dynamic)
  do ij = ishp(1)+1, ishp(2)
     i = npsh(1,ij)
     j = npsh(2,ij)
     iatom = nsh2atm(i)
     jatom = nsh2atm(j)
     Ax = xyz(1,iatom)
     Ay = xyz(2,iatom)
     Az = xyz(3,iatom)
     Bx = xyz(1,jatom)
     By = xyz(2,jatom)
     Bz = xyz(3,jatom)
     ABx = Ax - Bx
     ABy = Ay - By
     ABz = Az - Bz
     AB2 = ABx*ABx + ABy*ABy + ABz*ABz

t_pxs_dAx = 0.0d0
t_pys_dAx = 0.0d0
t_pzs_dAx = 0.0d0
t_pxs_dAy = 0.0d0
t_pys_dAy = 0.0d0
t_pzs_dAy = 0.0d0
t_pxs_dAz = 0.0d0
t_pys_dAz = 0.0d0
t_pzs_dAz = 0.0d0
do irs = npsh2pr(ij), npsh2pr(ij+1) - 1
o_ss = ss(irs)
zeta_ = zeta(irs)
xi_ = xi(irs)
xi2 = 2.0d0*xi_
t_ss = o_ss*xi_*( 3.0d0 - xi2 * AB2 )
xioza = xiozetaa(irs)
PAx = -ABx*xioza
PAy = -ABy*xioza
PAz = -ABz*xioza
o_pxs = PAx*o_ss
o_pys = PAy*o_ss
o_pzs = PAz*o_ss
oo2zeta = 0.5d0/zeta_
ai_o_ss = oo2zeta*o_ss
o_dxxs = PAx*o_pxs + ai_o_ss
o_dyys = PAy*o_pys + ai_o_ss
o_dzzs = PAz*o_pzs + ai_o_ss
o_dxys = PAy*o_pxs
o_dxzs = PAz*o_pxs
o_dyzs = PAz*o_pys
t_pxs = PAx*t_ss + xi2*o_pxs
t_pys = PAy*t_ss + xi2*o_pys
t_pzs = PAz*t_ss + xi2*o_pzs
!oo2zeta = 0.5d0*oozeta(irs)
ai_t_ss = oo2zeta*t_ss
xiai_o_ss = xioza*o_ss
t_dxxs = PAx*t_pxs + ai_t_ss + xi2*o_dxxs - xiai_o_ss
t_dyys = PAy*t_pys + ai_t_ss + xi2*o_dyys - xiai_o_ss
t_dzzs = PAz*t_pzs + ai_t_ss + xi2*o_dzzs - xiai_o_ss
t_dxys = PAy*t_pxs + xi2*o_dxys
t_dxzs = PAz*t_pxs + xi2*o_dxzs
t_dyzs = PAz*t_pys + xi2*o_dyzs
beta = xioza*zeta_
alpha = zeta_ - beta
alpha2 = 2.0d0*alpha
t_pxs_dAx = t_pxs_dAx + alpha2*t_dxxs - t_ss
t_pys_dAx = t_pys_dAx + alpha2*t_dxys
t_pzs_dAx = t_pzs_dAx + alpha2*t_dxzs
t_pxs_dAy = t_pxs_dAy + alpha2*t_dxys
t_pys_dAy = t_pys_dAy + alpha2*t_dyys - t_ss
t_pzs_dAy = t_pzs_dAy + alpha2*t_dyzs
t_pxs_dAz = t_pxs_dAz + alpha2*t_dxzs
t_pys_dAz = t_pys_dAz + alpha2*t_dyzs
t_pzs_dAz = t_pzs_dAz + alpha2*t_dzzs - t_ss
end do
v(1,1) = t_pxs_dAx
v(2,1) = t_pys_dAx
v(3,1) = t_pzs_dAx
v(1,2) = t_pxs_dAy
v(2,2) = t_pys_dAy
v(3,2) = t_pzs_dAy
v(1,3) = t_pxs_dAz
v(2,3) = t_pys_dAz
v(3,3) = t_pzs_dAz

     i = npsh(1,ij)
     j = npsh(2,ij)
     iao = nsh2ao(i)
     jao = nsh2ao(j)

     if (i < j) then
        do ixyz = 1, 3
           do ix = iao,iao + 3-1
              key = ix-iao + 1
              g(ixyz,iatom) = g(ixyz,iatom) + dm(jao,ix)*v(key,ixyz)
           end do
        end do
        do ixyz = 1, 3
           do ix = iao,iao + 3-1
              key = ix-iao + 1
              g(ixyz,jatom) = g(ixyz,jatom) - dm(jao,ix)*v(key,ixyz)
           end do
        end do
     else
        do ixyz = 1, 3
           do ix = iao,iao + 3-1
              key = ix-iao + 1
              g(ixyz,iatom) = g(ixyz,iatom) + dm(ix,jao)*v(key,ixyz)
           end do
        end do
        do ixyz = 1, 3
           do ix = iao,iao + 3-1
              key = ix-iao + 1
              g(ixyz,jatom) = g(ixyz,jatom) - dm(ix,jao)*v(key,ixyz)
           end do
        end do
     end if
  end do
!$omp end do nowait
end subroutine os_dkei_ps_t1

!------------------------------------------------------------------------------
!> pp kinetic energy integrals
!
subroutine os_dkei_pp_t1(&
     g,dm,nao,&
     ishp,npsh,npsh2pr,nsh2atm,nsh2ao,xyz,ss,xiozetaa,xi,zeta,&
     nshell,nshellp,nprimp,natom)
  use physical_constants_class

  implicit real(8) (a-h,o-z)
  integer,intent(in)::nao

  integer,intent(in)::nshell,nshellp,nprimp,natom
  integer,intent(in)::ishp(6),npsh(2,nshellp),npsh2pr(nshellp+1)
  integer,intent(in)::nsh2atm(nshell),nsh2ao(nshell)
  real(8),intent(in)::ss(nprimp),xiozetaa(nprimp),xi(nprimp),zeta(nprimp)
  real(8),intent(in)::xyz(3,natom)

  real(8),intent(inout)::g(3,natom)
  real(8),intent(in)::dm(nao,nao)

  real(8)::v(9,3)

!$omp do schedule(dynamic)
  do ij = ishp(2)+1, ishp(3)
     i = npsh(1,ij)
     j = npsh(2,ij)
     iatom = nsh2atm(i)
     jatom = nsh2atm(j)
     Ax = xyz(1,iatom)
     Ay = xyz(2,iatom)
     Az = xyz(3,iatom)
     Bx = xyz(1,jatom)
     By = xyz(2,jatom)
     Bz = xyz(3,jatom)
     ABx = Ax - Bx
     ABy = Ay - By
     ABz = Az - Bz
     AB2 = ABx*ABx + ABy*ABy + ABz*ABz

t_pxpx_dAx = 0.0d0
t_pxpy_dAx = 0.0d0
t_pxpz_dAx = 0.0d0
t_pypx_dAx = 0.0d0
t_pypy_dAx = 0.0d0
t_pypz_dAx = 0.0d0
t_pzpx_dAx = 0.0d0
t_pzpy_dAx = 0.0d0
t_pzpz_dAx = 0.0d0
t_pxpx_dAy = 0.0d0
t_pxpy_dAy = 0.0d0
t_pxpz_dAy = 0.0d0
t_pypx_dAy = 0.0d0
t_pypy_dAy = 0.0d0
t_pypz_dAy = 0.0d0
t_pzpx_dAy = 0.0d0
t_pzpy_dAy = 0.0d0
t_pzpz_dAy = 0.0d0
t_pxpx_dAz = 0.0d0
t_pxpy_dAz = 0.0d0
t_pxpz_dAz = 0.0d0
t_pypx_dAz = 0.0d0
t_pypy_dAz = 0.0d0
t_pypz_dAz = 0.0d0
t_pzpx_dAz = 0.0d0
t_pzpy_dAz = 0.0d0
t_pzpz_dAz = 0.0d0
do irs = npsh2pr(ij), npsh2pr(ij+1) - 1
o_ss = ss(irs)
zeta_ = zeta(irs)
xi_ = xi(irs)
xi2 = 2.0d0*xi_
t_ss = o_ss*xi_*( 3.0d0 - xi2 * AB2 )
xioza = xiozetaa(irs)
PAx = -ABx*xioza
PAy = -ABy*xioza
PAz = -ABz*xioza
o_pxs = PAx*o_ss
o_pys = PAy*o_ss
o_pzs = PAz*o_ss
oo2zeta = 0.5d0/zeta_
ai_o_ss = oo2zeta*o_ss
o_dxxs = PAx*o_pxs + ai_o_ss
o_dyys = PAy*o_pys + ai_o_ss
o_dzzs = PAz*o_pzs + ai_o_ss
o_dxys = PAy*o_pxs
o_dxzs = PAz*o_pxs
o_dyzs = PAz*o_pys
PBx = PAx + ABx
PBy = PAy + ABy
PBz = PAz + ABz
ai_o_pxs = oo2zeta*o_pxs
o_dxxpx = PBx*o_dxxs + 2.0d0*ai_o_pxs
o_dxxpy = PBy*o_dxxs
o_dxxpz = PBz*o_dxxs
o_dyypx = PBx*o_dyys
ai_o_pys = oo2zeta*o_pys
o_dyypy = PBy*o_dyys + 2.0d0*ai_o_pys
o_dyypz = PBz*o_dyys
o_dzzpx = PBx*o_dzzs
o_dzzpy = PBy*o_dzzs
ai_o_pzs = oo2zeta*o_pzs
o_dzzpz = PBz*o_dzzs + 2.0d0*ai_o_pzs
o_dxypx = PBx*o_dxys + ai_o_pys
o_dxypy = PBy*o_dxys + ai_o_pxs
o_dxypz = PBz*o_dxys
o_dxzpx = PBx*o_dxzs + ai_o_pzs
o_dxzpy = PBy*o_dxzs
o_dxzpz = PBz*o_dxzs + ai_o_pxs
o_dyzpx = PBx*o_dyzs
o_dyzpy = PBy*o_dyzs + ai_o_pzs
o_dyzpz = PBz*o_dyzs + ai_o_pys
t_pxs = PAx*t_ss + xi2*o_pxs
t_pys = PAy*t_ss + xi2*o_pys
t_pzs = PAz*t_ss + xi2*o_pzs
!oo2zeta = 0.5d0*oozeta(irs)
ai_t_ss = oo2zeta*t_ss
xiai_o_ss = xioza*o_ss
t_dxxs = PAx*t_pxs + ai_t_ss + xi2*o_dxxs - xiai_o_ss
t_dyys = PAy*t_pys + ai_t_ss + xi2*o_dyys - xiai_o_ss
t_dzzs = PAz*t_pzs + ai_t_ss + xi2*o_dzzs - xiai_o_ss
t_dxys = PAy*t_pxs + xi2*o_dxys
t_dxzs = PAz*t_pxs + xi2*o_dxzs
t_dyzs = PAz*t_pys + xi2*o_dyzs
ai_t_pxs = oo2zeta*t_pxs
t_dxxpx = PBx*t_dxxs + 2.0d0*ai_t_pxs + xi2*o_dxxpx
t_dxxpy = PBy*t_dxxs + xi2*o_dxxpy
t_dxxpz = PBz*t_dxxs + xi2*o_dxxpz
t_dyypx = PBx*t_dyys + xi2*o_dyypx
ai_t_pys = oo2zeta*t_pys
t_dyypy = PBy*t_dyys + 2.0d0*ai_t_pys + xi2*o_dyypy
t_dyypz = PBz*t_dyys + xi2*o_dyypz
t_dzzpx = PBx*t_dzzs + xi2*o_dzzpx
t_dzzpy = PBy*t_dzzs + xi2*o_dzzpy
ai_t_pzs = oo2zeta*t_pzs
t_dzzpz = PBz*t_dzzs + 2.0d0*ai_t_pzs + xi2*o_dzzpz
t_dxypx = PBx*t_dxys + ai_t_pys + xi2*o_dxypx
t_dxypy = PBy*t_dxys + ai_t_pxs + xi2*o_dxypy
t_dxypz = PBz*t_dxys + xi2*o_dxypz
t_dxzpx = PBx*t_dxzs + ai_t_pzs + xi2*o_dxzpx
t_dxzpy = PBy*t_dxzs + xi2*o_dxzpy
t_dxzpz = PBz*t_dxzs + ai_t_pxs + xi2*o_dxzpz
t_dyzpx = PBx*t_dyzs + xi2*o_dyzpx
t_dyzpy = PBy*t_dyzs + ai_t_pzs + xi2*o_dyzpy
t_dyzpz = PBz*t_dyzs + ai_t_pys + xi2*o_dyzpz
o_spx = PBx*o_ss
o_spy = PBy*o_ss
o_spz = PBz*o_ss
t_spx = PBx*t_ss + xi2*o_spx
t_spy = PBy*t_ss + xi2*o_spy
t_spz = PBz*t_ss + xi2*o_spz
beta = xioza*zeta_
alpha = zeta_ - beta
alpha2 = 2.0d0*alpha
t_pxpx_dAx = t_pxpx_dAx + alpha2*t_dxxpx - t_spx
t_pxpy_dAx = t_pxpy_dAx + alpha2*t_dxxpy - t_spy
t_pxpz_dAx = t_pxpz_dAx + alpha2*t_dxxpz - t_spz
t_pypx_dAx = t_pypx_dAx + alpha2*t_dxypx
t_pypy_dAx = t_pypy_dAx + alpha2*t_dxypy
t_pypz_dAx = t_pypz_dAx + alpha2*t_dxypz
t_pzpx_dAx = t_pzpx_dAx + alpha2*t_dxzpx
t_pzpy_dAx = t_pzpy_dAx + alpha2*t_dxzpy
t_pzpz_dAx = t_pzpz_dAx + alpha2*t_dxzpz
t_pxpx_dAy = t_pxpx_dAy + alpha2*t_dxypx
t_pxpy_dAy = t_pxpy_dAy + alpha2*t_dxypy
t_pxpz_dAy = t_pxpz_dAy + alpha2*t_dxypz
t_pypx_dAy = t_pypx_dAy + alpha2*t_dyypx - t_spx
t_pypy_dAy = t_pypy_dAy + alpha2*t_dyypy - t_spy
t_pypz_dAy = t_pypz_dAy + alpha2*t_dyypz - t_spz
t_pzpx_dAy = t_pzpx_dAy + alpha2*t_dyzpx
t_pzpy_dAy = t_pzpy_dAy + alpha2*t_dyzpy
t_pzpz_dAy = t_pzpz_dAy + alpha2*t_dyzpz
t_pxpx_dAz = t_pxpx_dAz + alpha2*t_dxzpx
t_pxpy_dAz = t_pxpy_dAz + alpha2*t_dxzpy
t_pxpz_dAz = t_pxpz_dAz + alpha2*t_dxzpz
t_pypx_dAz = t_pypx_dAz + alpha2*t_dyzpx
t_pypy_dAz = t_pypy_dAz + alpha2*t_dyzpy
t_pypz_dAz = t_pypz_dAz + alpha2*t_dyzpz
t_pzpx_dAz = t_pzpx_dAz + alpha2*t_dzzpx - t_spx
t_pzpy_dAz = t_pzpy_dAz + alpha2*t_dzzpy - t_spy
t_pzpz_dAz = t_pzpz_dAz + alpha2*t_dzzpz - t_spz
end do
v(1,1) = t_pxpx_dAx
v(2,1) = t_pxpy_dAx
v(3,1) = t_pxpz_dAx
v(4,1) = t_pypx_dAx
v(5,1) = t_pypy_dAx
v(6,1) = t_pypz_dAx
v(7,1) = t_pzpx_dAx
v(8,1) = t_pzpy_dAx
v(9,1) = t_pzpz_dAx
v(1,2) = t_pxpx_dAy
v(2,2) = t_pxpy_dAy
v(3,2) = t_pxpz_dAy
v(4,2) = t_pypx_dAy
v(5,2) = t_pypy_dAy
v(6,2) = t_pypz_dAy
v(7,2) = t_pzpx_dAy
v(8,2) = t_pzpy_dAy
v(9,2) = t_pzpz_dAy
v(1,3) = t_pxpx_dAz
v(2,3) = t_pxpy_dAz
v(3,3) = t_pxpz_dAz
v(4,3) = t_pypx_dAz
v(5,3) = t_pypy_dAz
v(6,3) = t_pypz_dAz
v(7,3) = t_pzpx_dAz
v(8,3) = t_pzpy_dAz
v(9,3) = t_pzpz_dAz

     i = npsh(1,ij)
     j = npsh(2,ij)
     iao = nsh2ao(i)
     jao = nsh2ao(j)

     if (iao == jao) then
        do ixyz = 1, 3
           do jx = jao,jao + 3-1
              do ix = jx,iao + 3-1
                 key = (ix-iao)*3 + jx-jao + 1
                 g(ixyz,iatom) = g(ixyz,iatom) + dm(ix,jx)*v(key,ixyz)
              end do
           end do
        end do
        do ixyz = 1, 3
           do jx = jao,jao + 3-1
              do ix = jx,iao + 3-1
                 key = (ix-iao)*3 + jx-jao + 1
                 g(ixyz,jatom) = g(ixyz,jatom) - dm(ix,jx)*v(key,ixyz)
              end do
           end do
        end do
     else
        do ixyz = 1, 3
           do jx = jao,jao + 3-1
              do ix = iao,iao + 3-1
                 key = (ix-iao)*3 + jx-jao + 1
                 g(ixyz,iatom) = g(ixyz,iatom) + dm(ix,jx)*v(key,ixyz)
              end do
           end do
        end do
        do ixyz = 1, 3
           do jx = jao,jao + 3-1
              do ix = iao,iao + 3-1
                 key = (ix-iao)*3 + jx-jao + 1
                 g(ixyz,jatom) = g(ixyz,jatom) - dm(ix,jx)*v(key,ixyz)
              end do
           end do
        end do
     end if
  end do
!$omp end do nowait
end subroutine os_dkei_pp_t1

!------------------------------------------------------------------------------
!> ds kinetic energy integrals
!
subroutine os_dkei_ds_t1(&
     g,dm,nao,&
     ishp,npsh,npsh2pr,nsh2atm,nsh2ao,xyz,ss,xiozetaa,xi,zeta,&
     nshell,nshellp,nprimp,natom)
  use physical_constants_class

  implicit real(8) (a-h,o-z)
  integer,intent(in)::nao

  integer,intent(in)::nshell,nshellp,nprimp,natom
  integer,intent(in)::ishp(6),npsh(2,nshellp),npsh2pr(nshellp+1)
  integer,intent(in)::nsh2atm(nshell),nsh2ao(nshell)
  real(8),intent(in)::ss(nprimp),xiozetaa(nprimp),xi(nprimp),zeta(nprimp)
  real(8),intent(in)::xyz(3,natom)

  real(8),intent(inout)::g(3,natom)
  real(8),intent(in)::dm(nao,nao)

  real(8)::v(6,3)

!$omp do schedule(dynamic)
  do ij = ishp(3)+1, ishp(4)
     i = npsh(1,ij)
     j = npsh(2,ij)
     iatom = nsh2atm(i)
     jatom = nsh2atm(j)
     Ax = xyz(1,iatom)
     Ay = xyz(2,iatom)
     Az = xyz(3,iatom)
     Bx = xyz(1,jatom)
     By = xyz(2,jatom)
     Bz = xyz(3,jatom)
     ABx = Ax - Bx
     ABy = Ay - By
     ABz = Az - Bz
     AB2 = ABx*ABx + ABy*ABy + ABz*ABz

t_dxxs_dAx = 0.0d0
t_dyys_dAx = 0.0d0
t_dzzs_dAx = 0.0d0
t_dxys_dAx = 0.0d0
t_dxzs_dAx = 0.0d0
t_dyzs_dAx = 0.0d0
t_dxxs_dAy = 0.0d0
t_dyys_dAy = 0.0d0
t_dzzs_dAy = 0.0d0
t_dxys_dAy = 0.0d0
t_dxzs_dAy = 0.0d0
t_dyzs_dAy = 0.0d0
t_dxxs_dAz = 0.0d0
t_dyys_dAz = 0.0d0
t_dzzs_dAz = 0.0d0
t_dxys_dAz = 0.0d0
t_dxzs_dAz = 0.0d0
t_dyzs_dAz = 0.0d0
do irs = npsh2pr(ij), npsh2pr(ij+1) - 1
o_ss = ss(irs)
zeta_ = zeta(irs)
xi_ = xi(irs)
xi2 = 2.0d0*xi_
t_ss = o_ss*xi_*( 3.0d0 - xi2 * AB2 )
xioza = xiozetaa(irs)
PAx = -ABx*xioza
PAy = -ABy*xioza
PAz = -ABz*xioza
o_pxs = PAx*o_ss
o_pys = PAy*o_ss
o_pzs = PAz*o_ss
oo2zeta = 0.5d0/zeta_
ai_o_ss = oo2zeta*o_ss
o_dxxs = PAx*o_pxs + ai_o_ss
o_dyys = PAy*o_pys + ai_o_ss
o_dzzs = PAz*o_pzs + ai_o_ss
o_dxys = PAy*o_pxs
o_dxzs = PAz*o_pxs
o_dyzs = PAz*o_pys
ai_o_pxs = oo2zeta*o_pxs
o_fxxxs = PAx*o_dxxs + 2.0d0*ai_o_pxs
ai_o_pys = oo2zeta*o_pys
o_fyyys = PAy*o_dyys + 2.0d0*ai_o_pys
ai_o_pzs = oo2zeta*o_pzs
o_fzzzs = PAz*o_dzzs + 2.0d0*ai_o_pzs
o_fxxys = PAy*o_dxxs
o_fxxzs = PAz*o_dxxs
o_fyyxs = PAx*o_dyys
o_fyyzs = PAz*o_dyys
o_fzzxs = PAx*o_dzzs
o_fzzys = PAy*o_dzzs
o_fxyzs = PAz*o_dxys
t_pxs = PAx*t_ss + xi2*o_pxs
t_pys = PAy*t_ss + xi2*o_pys
t_pzs = PAz*t_ss + xi2*o_pzs
!oo2zeta = 0.5d0*oozeta(irs)
ai_t_ss = oo2zeta*t_ss
xiai_o_ss = xioza*o_ss
t_dxxs = PAx*t_pxs + ai_t_ss + xi2*o_dxxs - xiai_o_ss
t_dyys = PAy*t_pys + ai_t_ss + xi2*o_dyys - xiai_o_ss
t_dzzs = PAz*t_pzs + ai_t_ss + xi2*o_dzzs - xiai_o_ss
t_dxys = PAy*t_pxs + xi2*o_dxys
t_dxzs = PAz*t_pxs + xi2*o_dxzs
t_dyzs = PAz*t_pys + xi2*o_dyzs
ai_t_pxs = oo2zeta*t_pxs
xiai_o_pxs = xioza*o_pxs
t_fxxxs = PAx*t_dxxs + 2.0d0*ai_t_pxs + xi2*o_fxxxs - 2.0d0*xiai_o_pxs
ai_t_pys = oo2zeta*t_pys
xiai_o_pys = xioza*o_pys
t_fyyys = PAy*t_dyys + 2.0d0*ai_t_pys + xi2*o_fyyys - 2.0d0*xiai_o_pys
ai_t_pzs = oo2zeta*t_pzs
xiai_o_pzs = xioza*o_pzs
t_fzzzs = PAz*t_dzzs + 2.0d0*ai_t_pzs + xi2*o_fzzzs - 2.0d0*xiai_o_pzs
t_fxxys = PAy*t_dxxs + xi2*o_fxxys
t_fxxzs = PAz*t_dxxs + xi2*o_fxxzs
t_fyyxs = PAx*t_dyys + xi2*o_fyyxs
t_fyyzs = PAz*t_dyys + xi2*o_fyyzs
t_fzzxs = PAx*t_dzzs + xi2*o_fzzxs
t_fzzys = PAy*t_dzzs + xi2*o_fzzys
t_fxyzs = PAz*t_dxys + xi2*o_fxyzs
beta = xioza*zeta_
alpha = zeta_ - beta
alpha2 = 2.0d0*alpha
t_dxxs_dAx = t_dxxs_dAx + alpha2*t_fxxxs - 2.0d0*t_pxs
t_dyys_dAx = t_dyys_dAx + alpha2*t_fyyxs
t_dzzs_dAx = t_dzzs_dAx + alpha2*t_fzzxs
t_dxys_dAx = t_dxys_dAx + alpha2*t_fxxys - t_pys
t_dxzs_dAx = t_dxzs_dAx + alpha2*t_fxxzs - t_pzs
t_dyzs_dAx = t_dyzs_dAx + alpha2*t_fxyzs
t_dxxs_dAy = t_dxxs_dAy + alpha2*t_fxxys
t_dyys_dAy = t_dyys_dAy + alpha2*t_fyyys - 2.0d0*t_pys
t_dzzs_dAy = t_dzzs_dAy + alpha2*t_fzzys
t_dxys_dAy = t_dxys_dAy + alpha2*t_fyyxs - t_pxs
t_dxzs_dAy = t_dxzs_dAy + alpha2*t_fxyzs
t_dyzs_dAy = t_dyzs_dAy + alpha2*t_fyyzs - t_pzs
t_dxxs_dAz = t_dxxs_dAz + alpha2*t_fxxzs
t_dyys_dAz = t_dyys_dAz + alpha2*t_fyyzs
t_dzzs_dAz = t_dzzs_dAz + alpha2*t_fzzzs - 2.0d0*t_pzs
t_dxys_dAz = t_dxys_dAz + alpha2*t_fxyzs
t_dxzs_dAz = t_dxzs_dAz + alpha2*t_fzzxs - t_pxs
t_dyzs_dAz = t_dyzs_dAz + alpha2*t_fzzys - t_pys
end do
v(1,1) = t_dxxs_dAx
v(2,1) = t_dyys_dAx
v(3,1) = t_dzzs_dAx
v(4,1) = t_dxys_dAx*root3
v(5,1) = t_dxzs_dAx*root3
v(6,1) = t_dyzs_dAx*root3
v(1,2) = t_dxxs_dAy
v(2,2) = t_dyys_dAy
v(3,2) = t_dzzs_dAy
v(4,2) = t_dxys_dAy*root3
v(5,2) = t_dxzs_dAy*root3
v(6,2) = t_dyzs_dAy*root3
v(1,3) = t_dxxs_dAz
v(2,3) = t_dyys_dAz
v(3,3) = t_dzzs_dAz
v(4,3) = t_dxys_dAz*root3
v(5,3) = t_dxzs_dAz*root3
v(6,3) = t_dyzs_dAz*root3

     i = npsh(1,ij)
     j = npsh(2,ij)
     iao = nsh2ao(i)
     jao = nsh2ao(j)

     if (i < j) then
        do ixyz = 1, 3
           do ix = iao,iao + 6-1
              key = ix-iao + 1
              g(ixyz,iatom) = g(ixyz,iatom) + dm(jao,ix)*v(key,ixyz)
           end do
        end do
        do ixyz = 1, 3
           do ix = iao,iao + 6-1
              key = ix-iao + 1
              g(ixyz,jatom) = g(ixyz,jatom) - dm(jao,ix)*v(key,ixyz)
           end do
        end do
     else
        do ixyz = 1, 3
           do ix = iao,iao + 6-1
              key = ix-iao + 1
              g(ixyz,iatom) = g(ixyz,iatom) + dm(ix,jao)*v(key,ixyz)
           end do
        end do
        do ixyz = 1, 3
           do ix = iao,iao + 6-1
              key = ix-iao + 1
              g(ixyz,jatom) = g(ixyz,jatom) - dm(ix,jao)*v(key,ixyz)
           end do
        end do
     end if
  end do
!$omp end do nowait
end subroutine os_dkei_ds_t1

!------------------------------------------------------------------------------
!> dp kinetic energy integrals
!
subroutine os_dkei_dp_t1(&
     g,dm,nao,&
     ishp,npsh,npsh2pr,nsh2atm,nsh2ao,xyz,ss,xiozetaa,xi,zeta,&
     nshell,nshellp,nprimp,natom)
  use physical_constants_class

  implicit real(8) (a-h,o-z)
  integer,intent(in)::nao

  integer,intent(in)::nshell,nshellp,nprimp,natom
  integer,intent(in)::ishp(6),npsh(2,nshellp),npsh2pr(nshellp+1)
  integer,intent(in)::nsh2atm(nshell),nsh2ao(nshell)
  real(8),intent(in)::ss(nprimp),xiozetaa(nprimp),xi(nprimp),zeta(nprimp)
  real(8),intent(in)::xyz(3,natom)

  real(8),intent(inout)::g(3,natom)
  real(8),intent(in)::dm(nao,nao)

  real(8)::v(18,3)

!$omp do schedule(dynamic)
  do ij = ishp(4)+1, ishp(5)
     i = npsh(1,ij)
     j = npsh(2,ij)
     iatom = nsh2atm(i)
     jatom = nsh2atm(j)
     Ax = xyz(1,iatom)
     Ay = xyz(2,iatom)
     Az = xyz(3,iatom)
     Bx = xyz(1,jatom)
     By = xyz(2,jatom)
     Bz = xyz(3,jatom)
     ABx = Ax - Bx
     ABy = Ay - By
     ABz = Az - Bz
     AB2 = ABx*ABx + ABy*ABy + ABz*ABz

t_dxxpx_dAx = 0.0d0
t_dxxpy_dAx = 0.0d0
t_dxxpz_dAx = 0.0d0
t_dyypx_dAx = 0.0d0
t_dyypy_dAx = 0.0d0
t_dyypz_dAx = 0.0d0
t_dzzpx_dAx = 0.0d0
t_dzzpy_dAx = 0.0d0
t_dzzpz_dAx = 0.0d0
t_dxypx_dAx = 0.0d0
t_dxypy_dAx = 0.0d0
t_dxypz_dAx = 0.0d0
t_dxzpx_dAx = 0.0d0
t_dxzpy_dAx = 0.0d0
t_dxzpz_dAx = 0.0d0
t_dyzpx_dAx = 0.0d0
t_dyzpy_dAx = 0.0d0
t_dyzpz_dAx = 0.0d0
t_dxxpx_dAy = 0.0d0
t_dxxpy_dAy = 0.0d0
t_dxxpz_dAy = 0.0d0
t_dyypx_dAy = 0.0d0
t_dyypy_dAy = 0.0d0
t_dyypz_dAy = 0.0d0
t_dzzpx_dAy = 0.0d0
t_dzzpy_dAy = 0.0d0
t_dzzpz_dAy = 0.0d0
t_dxypx_dAy = 0.0d0
t_dxypy_dAy = 0.0d0
t_dxypz_dAy = 0.0d0
t_dxzpx_dAy = 0.0d0
t_dxzpy_dAy = 0.0d0
t_dxzpz_dAy = 0.0d0
t_dyzpx_dAy = 0.0d0
t_dyzpy_dAy = 0.0d0
t_dyzpz_dAy = 0.0d0
t_dxxpx_dAz = 0.0d0
t_dxxpy_dAz = 0.0d0
t_dxxpz_dAz = 0.0d0
t_dyypx_dAz = 0.0d0
t_dyypy_dAz = 0.0d0
t_dyypz_dAz = 0.0d0
t_dzzpx_dAz = 0.0d0
t_dzzpy_dAz = 0.0d0
t_dzzpz_dAz = 0.0d0
t_dxypx_dAz = 0.0d0
t_dxypy_dAz = 0.0d0
t_dxypz_dAz = 0.0d0
t_dxzpx_dAz = 0.0d0
t_dxzpy_dAz = 0.0d0
t_dxzpz_dAz = 0.0d0
t_dyzpx_dAz = 0.0d0
t_dyzpy_dAz = 0.0d0
t_dyzpz_dAz = 0.0d0
do irs = npsh2pr(ij), npsh2pr(ij+1) - 1
o_ss = ss(irs)
zeta_ = zeta(irs)
xi_ = xi(irs)
xi2 = 2.0d0*xi_
t_ss = o_ss*xi_*( 3.0d0 - xi2 * AB2 )
xioza = xiozetaa(irs)
PAx = -ABx*xioza
PAy = -ABy*xioza
PAz = -ABz*xioza
o_pxs = PAx*o_ss
o_pys = PAy*o_ss
o_pzs = PAz*o_ss
oo2zeta = 0.5d0/zeta_
ai_o_ss = oo2zeta*o_ss
o_dxxs = PAx*o_pxs + ai_o_ss
o_dyys = PAy*o_pys + ai_o_ss
o_dzzs = PAz*o_pzs + ai_o_ss
o_dxys = PAy*o_pxs
o_dxzs = PAz*o_pxs
o_dyzs = PAz*o_pys
ai_o_pxs = oo2zeta*o_pxs
o_fxxxs = PAx*o_dxxs + 2.0d0*ai_o_pxs
ai_o_pys = oo2zeta*o_pys
o_fyyys = PAy*o_dyys + 2.0d0*ai_o_pys
ai_o_pzs = oo2zeta*o_pzs
o_fzzzs = PAz*o_dzzs + 2.0d0*ai_o_pzs
o_fxxys = PAy*o_dxxs
o_fxxzs = PAz*o_dxxs
o_fyyxs = PAx*o_dyys
o_fyyzs = PAz*o_dyys
o_fzzxs = PAx*o_dzzs
o_fzzys = PAy*o_dzzs
o_fxyzs = PAz*o_dxys
PBx = PAx + ABx
PBy = PAy + ABy
PBz = PAz + ABz
ai_o_dxxs = oo2zeta*o_dxxs
o_fxxxpx = PBx*o_fxxxs + 3.0d0*ai_o_dxxs
o_fxxxpy = PBy*o_fxxxs
o_fxxxpz = PBz*o_fxxxs
o_fyyypx = PBx*o_fyyys
ai_o_dyys = oo2zeta*o_dyys
o_fyyypy = PBy*o_fyyys + 3.0d0*ai_o_dyys
o_fyyypz = PBz*o_fyyys
o_fzzzpx = PBx*o_fzzzs
o_fzzzpy = PBy*o_fzzzs
ai_o_dzzs = oo2zeta*o_dzzs
o_fzzzpz = PBz*o_fzzzs + 3.0d0*ai_o_dzzs
ai_o_dxys = oo2zeta*o_dxys
o_fxxypx = PBx*o_fxxys + 2.0d0*ai_o_dxys
o_fxxypy = PBy*o_fxxys + ai_o_dxxs
o_fxxypz = PBz*o_fxxys
ai_o_dxzs = oo2zeta*o_dxzs
o_fxxzpx = PBx*o_fxxzs + 2.0d0*ai_o_dxzs
o_fxxzpy = PBy*o_fxxzs
o_fxxzpz = PBz*o_fxxzs + ai_o_dxxs
o_fyyxpx = PBx*o_fyyxs + ai_o_dyys
o_fyyxpy = PBy*o_fyyxs + 2.0d0*ai_o_dxys
o_fyyxpz = PBz*o_fyyxs
o_fyyzpx = PBx*o_fyyzs
ai_o_dyzs = oo2zeta*o_dyzs
o_fyyzpy = PBy*o_fyyzs + 2.0d0*ai_o_dyzs
o_fyyzpz = PBz*o_fyyzs + ai_o_dyys
o_fzzxpx = PBx*o_fzzxs + ai_o_dzzs
o_fzzxpy = PBy*o_fzzxs
o_fzzxpz = PBz*o_fzzxs + 2.0d0*ai_o_dxzs
o_fzzypx = PBx*o_fzzys
o_fzzypy = PBy*o_fzzys + ai_o_dzzs
o_fzzypz = PBz*o_fzzys + 2.0d0*ai_o_dyzs
o_fxyzpx = PBx*o_fxyzs + ai_o_dyzs
o_fxyzpy = PBy*o_fxyzs + ai_o_dxzs
o_fxyzpz = PBz*o_fxyzs + ai_o_dxys
t_pxs = PAx*t_ss + xi2*o_pxs
t_pys = PAy*t_ss + xi2*o_pys
t_pzs = PAz*t_ss + xi2*o_pzs
!oo2zeta = 0.5d0*oozeta(irs)
ai_t_ss = oo2zeta*t_ss
xiai_o_ss = xioza*o_ss
t_dxxs = PAx*t_pxs + ai_t_ss + xi2*o_dxxs - xiai_o_ss
t_dyys = PAy*t_pys + ai_t_ss + xi2*o_dyys - xiai_o_ss
t_dzzs = PAz*t_pzs + ai_t_ss + xi2*o_dzzs - xiai_o_ss
t_dxys = PAy*t_pxs + xi2*o_dxys
t_dxzs = PAz*t_pxs + xi2*o_dxzs
t_dyzs = PAz*t_pys + xi2*o_dyzs
ai_t_pxs = oo2zeta*t_pxs
xiai_o_pxs = xioza*o_pxs
t_fxxxs = PAx*t_dxxs + 2.0d0*ai_t_pxs + xi2*o_fxxxs - 2.0d0*xiai_o_pxs
ai_t_pys = oo2zeta*t_pys
xiai_o_pys = xioza*o_pys
t_fyyys = PAy*t_dyys + 2.0d0*ai_t_pys + xi2*o_fyyys - 2.0d0*xiai_o_pys
ai_t_pzs = oo2zeta*t_pzs
xiai_o_pzs = xioza*o_pzs
t_fzzzs = PAz*t_dzzs + 2.0d0*ai_t_pzs + xi2*o_fzzzs - 2.0d0*xiai_o_pzs
t_fxxys = PAy*t_dxxs + xi2*o_fxxys
t_fxxzs = PAz*t_dxxs + xi2*o_fxxzs
t_fyyxs = PAx*t_dyys + xi2*o_fyyxs
t_fyyzs = PAz*t_dyys + xi2*o_fyyzs
t_fzzxs = PAx*t_dzzs + xi2*o_fzzxs
t_fzzys = PAy*t_dzzs + xi2*o_fzzys
t_fxyzs = PAz*t_dxys + xi2*o_fxyzs
ai_t_dxxs = oo2zeta*t_dxxs
t_fxxxpx = PBx*t_fxxxs + 3.0d0*ai_t_dxxs + xi2*o_fxxxpx
t_fxxxpy = PBy*t_fxxxs + xi2*o_fxxxpy
t_fxxxpz = PBz*t_fxxxs + xi2*o_fxxxpz
t_fyyypx = PBx*t_fyyys + xi2*o_fyyypx
ai_t_dyys = oo2zeta*t_dyys
t_fyyypy = PBy*t_fyyys + 3.0d0*ai_t_dyys + xi2*o_fyyypy
t_fyyypz = PBz*t_fyyys + xi2*o_fyyypz
t_fzzzpx = PBx*t_fzzzs + xi2*o_fzzzpx
t_fzzzpy = PBy*t_fzzzs + xi2*o_fzzzpy
ai_t_dzzs = oo2zeta*t_dzzs
t_fzzzpz = PBz*t_fzzzs + 3.0d0*ai_t_dzzs + xi2*o_fzzzpz
ai_t_dxys = oo2zeta*t_dxys
t_fxxypx = PBx*t_fxxys + 2.0d0*ai_t_dxys + xi2*o_fxxypx
t_fxxypy = PBy*t_fxxys + ai_t_dxxs + xi2*o_fxxypy
t_fxxypz = PBz*t_fxxys + xi2*o_fxxypz
ai_t_dxzs = oo2zeta*t_dxzs
t_fxxzpx = PBx*t_fxxzs + 2.0d0*ai_t_dxzs + xi2*o_fxxzpx
t_fxxzpy = PBy*t_fxxzs + xi2*o_fxxzpy
t_fxxzpz = PBz*t_fxxzs + ai_t_dxxs + xi2*o_fxxzpz
t_fyyxpx = PBx*t_fyyxs + ai_t_dyys + xi2*o_fyyxpx
t_fyyxpy = PBy*t_fyyxs + 2.0d0*ai_t_dxys + xi2*o_fyyxpy
t_fyyxpz = PBz*t_fyyxs + xi2*o_fyyxpz
t_fyyzpx = PBx*t_fyyzs + xi2*o_fyyzpx
ai_t_dyzs = oo2zeta*t_dyzs
t_fyyzpy = PBy*t_fyyzs + 2.0d0*ai_t_dyzs + xi2*o_fyyzpy
t_fyyzpz = PBz*t_fyyzs + ai_t_dyys + xi2*o_fyyzpz
t_fzzxpx = PBx*t_fzzxs + ai_t_dzzs + xi2*o_fzzxpx
t_fzzxpy = PBy*t_fzzxs + xi2*o_fzzxpy
t_fzzxpz = PBz*t_fzzxs + 2.0d0*ai_t_dxzs + xi2*o_fzzxpz
t_fzzypx = PBx*t_fzzys + xi2*o_fzzypx
t_fzzypy = PBy*t_fzzys + ai_t_dzzs + xi2*o_fzzypy
t_fzzypz = PBz*t_fzzys + 2.0d0*ai_t_dyzs + xi2*o_fzzypz
t_fxyzpx = PBx*t_fxyzs + ai_t_dyzs + xi2*o_fxyzpx
t_fxyzpy = PBy*t_fxyzs + ai_t_dxzs + xi2*o_fxyzpy
t_fxyzpz = PBz*t_fxyzs + ai_t_dxys + xi2*o_fxyzpz
o_pxpx = PBx*o_pxs + ai_o_ss
o_pxpy = PBy*o_pxs
o_pxpz = PBz*o_pxs
o_pypx = PBx*o_pys
o_pypy = PBy*o_pys + ai_o_ss
o_pypz = PBz*o_pys
o_pzpx = PBx*o_pzs
o_pzpy = PBy*o_pzs
o_pzpz = PBz*o_pzs + ai_o_ss
t_pxpx = PBx*t_pxs + ai_t_ss + xi2*o_pxpx
t_pxpy = PBy*t_pxs + xi2*o_pxpy
t_pxpz = PBz*t_pxs + xi2*o_pxpz
t_pypx = PBx*t_pys + xi2*o_pypx
t_pypy = PBy*t_pys + ai_t_ss + xi2*o_pypy
t_pypz = PBz*t_pys + xi2*o_pypz
t_pzpx = PBx*t_pzs + xi2*o_pzpx
t_pzpy = PBy*t_pzs + xi2*o_pzpy
t_pzpz = PBz*t_pzs + ai_t_ss + xi2*o_pzpz
beta = xioza*zeta_
alpha = zeta_ - beta
alpha2 = 2.0d0*alpha
t_dxxpx_dAx = t_dxxpx_dAx + alpha2*t_fxxxpx - 2.0d0*t_pxpx
t_dxxpy_dAx = t_dxxpy_dAx + alpha2*t_fxxxpy - 2.0d0*t_pxpy
t_dxxpz_dAx = t_dxxpz_dAx + alpha2*t_fxxxpz - 2.0d0*t_pxpz
t_dyypx_dAx = t_dyypx_dAx + alpha2*t_fyyxpx
t_dyypy_dAx = t_dyypy_dAx + alpha2*t_fyyxpy
t_dyypz_dAx = t_dyypz_dAx + alpha2*t_fyyxpz
t_dzzpx_dAx = t_dzzpx_dAx + alpha2*t_fzzxpx
t_dzzpy_dAx = t_dzzpy_dAx + alpha2*t_fzzxpy
t_dzzpz_dAx = t_dzzpz_dAx + alpha2*t_fzzxpz
t_dxypx_dAx = t_dxypx_dAx + alpha2*t_fxxypx - t_pypx
t_dxypy_dAx = t_dxypy_dAx + alpha2*t_fxxypy - t_pypy
t_dxypz_dAx = t_dxypz_dAx + alpha2*t_fxxypz - t_pypz
t_dxzpx_dAx = t_dxzpx_dAx + alpha2*t_fxxzpx - t_pzpx
t_dxzpy_dAx = t_dxzpy_dAx + alpha2*t_fxxzpy - t_pzpy
t_dxzpz_dAx = t_dxzpz_dAx + alpha2*t_fxxzpz - t_pzpz
t_dyzpx_dAx = t_dyzpx_dAx + alpha2*t_fxyzpx
t_dyzpy_dAx = t_dyzpy_dAx + alpha2*t_fxyzpy
t_dyzpz_dAx = t_dyzpz_dAx + alpha2*t_fxyzpz
t_dxxpx_dAy = t_dxxpx_dAy + alpha2*t_fxxypx
t_dxxpy_dAy = t_dxxpy_dAy + alpha2*t_fxxypy
t_dxxpz_dAy = t_dxxpz_dAy + alpha2*t_fxxypz
t_dyypx_dAy = t_dyypx_dAy + alpha2*t_fyyypx - 2.0d0*t_pypx
t_dyypy_dAy = t_dyypy_dAy + alpha2*t_fyyypy - 2.0d0*t_pypy
t_dyypz_dAy = t_dyypz_dAy + alpha2*t_fyyypz - 2.0d0*t_pypz
t_dzzpx_dAy = t_dzzpx_dAy + alpha2*t_fzzypx
t_dzzpy_dAy = t_dzzpy_dAy + alpha2*t_fzzypy
t_dzzpz_dAy = t_dzzpz_dAy + alpha2*t_fzzypz
t_dxypx_dAy = t_dxypx_dAy + alpha2*t_fyyxpx - t_pxpx
t_dxypy_dAy = t_dxypy_dAy + alpha2*t_fyyxpy - t_pxpy
t_dxypz_dAy = t_dxypz_dAy + alpha2*t_fyyxpz - t_pxpz
t_dxzpx_dAy = t_dxzpx_dAy + alpha2*t_fxyzpx
t_dxzpy_dAy = t_dxzpy_dAy + alpha2*t_fxyzpy
t_dxzpz_dAy = t_dxzpz_dAy + alpha2*t_fxyzpz
t_dyzpx_dAy = t_dyzpx_dAy + alpha2*t_fyyzpx - t_pzpx
t_dyzpy_dAy = t_dyzpy_dAy + alpha2*t_fyyzpy - t_pzpy
t_dyzpz_dAy = t_dyzpz_dAy + alpha2*t_fyyzpz - t_pzpz
t_dxxpx_dAz = t_dxxpx_dAz + alpha2*t_fxxzpx
t_dxxpy_dAz = t_dxxpy_dAz + alpha2*t_fxxzpy
t_dxxpz_dAz = t_dxxpz_dAz + alpha2*t_fxxzpz
t_dyypx_dAz = t_dyypx_dAz + alpha2*t_fyyzpx
t_dyypy_dAz = t_dyypy_dAz + alpha2*t_fyyzpy
t_dyypz_dAz = t_dyypz_dAz + alpha2*t_fyyzpz
t_dzzpx_dAz = t_dzzpx_dAz + alpha2*t_fzzzpx - 2.0d0*t_pzpx
t_dzzpy_dAz = t_dzzpy_dAz + alpha2*t_fzzzpy - 2.0d0*t_pzpy
t_dzzpz_dAz = t_dzzpz_dAz + alpha2*t_fzzzpz - 2.0d0*t_pzpz
t_dxypx_dAz = t_dxypx_dAz + alpha2*t_fxyzpx
t_dxypy_dAz = t_dxypy_dAz + alpha2*t_fxyzpy
t_dxypz_dAz = t_dxypz_dAz + alpha2*t_fxyzpz
t_dxzpx_dAz = t_dxzpx_dAz + alpha2*t_fzzxpx - t_pxpx
t_dxzpy_dAz = t_dxzpy_dAz + alpha2*t_fzzxpy - t_pxpy
t_dxzpz_dAz = t_dxzpz_dAz + alpha2*t_fzzxpz - t_pxpz
t_dyzpx_dAz = t_dyzpx_dAz + alpha2*t_fzzypx - t_pypx
t_dyzpy_dAz = t_dyzpy_dAz + alpha2*t_fzzypy - t_pypy
t_dyzpz_dAz = t_dyzpz_dAz + alpha2*t_fzzypz - t_pypz
end do
v(1,1) = t_dxxpx_dAx
v(2,1) = t_dxxpy_dAx
v(3,1) = t_dxxpz_dAx
v(4,1) = t_dyypx_dAx
v(5,1) = t_dyypy_dAx
v(6,1) = t_dyypz_dAx
v(7,1) = t_dzzpx_dAx
v(8,1) = t_dzzpy_dAx
v(9,1) = t_dzzpz_dAx
v(10,1) = t_dxypx_dAx*root3
v(11,1) = t_dxypy_dAx*root3
v(12,1) = t_dxypz_dAx*root3
v(13,1) = t_dxzpx_dAx*root3
v(14,1) = t_dxzpy_dAx*root3
v(15,1) = t_dxzpz_dAx*root3
v(16,1) = t_dyzpx_dAx*root3
v(17,1) = t_dyzpy_dAx*root3
v(18,1) = t_dyzpz_dAx*root3
v(1,2) = t_dxxpx_dAy
v(2,2) = t_dxxpy_dAy
v(3,2) = t_dxxpz_dAy
v(4,2) = t_dyypx_dAy
v(5,2) = t_dyypy_dAy
v(6,2) = t_dyypz_dAy
v(7,2) = t_dzzpx_dAy
v(8,2) = t_dzzpy_dAy
v(9,2) = t_dzzpz_dAy
v(10,2) = t_dxypx_dAy*root3
v(11,2) = t_dxypy_dAy*root3
v(12,2) = t_dxypz_dAy*root3
v(13,2) = t_dxzpx_dAy*root3
v(14,2) = t_dxzpy_dAy*root3
v(15,2) = t_dxzpz_dAy*root3
v(16,2) = t_dyzpx_dAy*root3
v(17,2) = t_dyzpy_dAy*root3
v(18,2) = t_dyzpz_dAy*root3
v(1,3) = t_dxxpx_dAz
v(2,3) = t_dxxpy_dAz
v(3,3) = t_dxxpz_dAz
v(4,3) = t_dyypx_dAz
v(5,3) = t_dyypy_dAz
v(6,3) = t_dyypz_dAz
v(7,3) = t_dzzpx_dAz
v(8,3) = t_dzzpy_dAz
v(9,3) = t_dzzpz_dAz
v(10,3) = t_dxypx_dAz*root3
v(11,3) = t_dxypy_dAz*root3
v(12,3) = t_dxypz_dAz*root3
v(13,3) = t_dxzpx_dAz*root3
v(14,3) = t_dxzpy_dAz*root3
v(15,3) = t_dxzpz_dAz*root3
v(16,3) = t_dyzpx_dAz*root3
v(17,3) = t_dyzpy_dAz*root3
v(18,3) = t_dyzpz_dAz*root3

     i = npsh(1,ij)
     j = npsh(2,ij)
     iao = nsh2ao(i)
     jao = nsh2ao(j)

     if (i < j) then
        do ixyz = 1, 3
           do jx = jao,jao + 3-1
              do ix = iao,iao + 6-1
                 key = ( ix-iao ) * 3 + jx-jao +1
                 g(ixyz,iatom) = g(ixyz,iatom) + dm(jx,ix)*v(key,ixyz)
              end do
           end do
        end do
        do ixyz = 1, 3
           do jx = jao,jao + 3-1
              do ix = iao,iao + 6-1
                 key = ( ix-iao ) * 3 + jx-jao +1
                 g(ixyz,jatom) = g(ixyz,jatom) - dm(jx,ix)*v(key,ixyz)
              end do
           end do
        end do
     else
        do ixyz = 1, 3
           do jx = jao,jao + 3-1
              do ix = iao,iao + 6-1
                 key = ( ix-iao ) * 3 + jx-jao +1
                 g(ixyz,iatom) = g(ixyz,iatom) + dm(ix,jx)*v(key,ixyz)
              end do
           end do
        end do
        do ixyz = 1, 3
           do jx = jao,jao + 3-1
              do ix = iao,iao + 6-1
                 key = ( ix-iao ) * 3 + jx-jao +1
                 g(ixyz,jatom) = g(ixyz,jatom) - dm(ix,jx)*v(key,ixyz)
              end do
           end do
        end do
     end if
  end do
!$omp end do nowait
end subroutine os_dkei_dp_t1

!------------------------------------------------------------------------------
!> dd kinetic energy integrals
!
subroutine os_dkei_dd_t1(&
     g,dm,nao,&
     ishp,npsh,npsh2pr,nsh2atm,nsh2ao,xyz,ss,xiozetaa,xi,zeta,&
     nshell,nshellp,nprimp,natom)
  use physical_constants_class

  implicit real(8) (a-h,o-z)
  integer,intent(in)::nao

  integer,intent(in)::nshell,nshellp,nprimp,natom
  integer,intent(in)::ishp(6),npsh(2,nshellp),npsh2pr(nshellp+1)
  integer,intent(in)::nsh2atm(nshell),nsh2ao(nshell)
  real(8),intent(in)::ss(nprimp),xiozetaa(nprimp),xi(nprimp),zeta(nprimp)
  real(8),intent(in)::xyz(3,natom)

  real(8),intent(inout)::g(3,natom)
  real(8),intent(in)::dm(nao,nao)

  real(8)::v(36,3)

!$omp do schedule(dynamic)
  do ij = ishp(5)+1, ishp(6)
     i = npsh(1,ij)
     j = npsh(2,ij)
     iatom = nsh2atm(i)
     jatom = nsh2atm(j)
     Ax = xyz(1,iatom)
     Ay = xyz(2,iatom)
     Az = xyz(3,iatom)
     Bx = xyz(1,jatom)
     By = xyz(2,jatom)
     Bz = xyz(3,jatom)
     ABx = Ax - Bx
     ABy = Ay - By
     ABz = Az - Bz
     AB2 = ABx*ABx + ABy*ABy + ABz*ABz

t_dxxdxx_dAx = 0.0d0
t_dxxdyy_dAx = 0.0d0
t_dxxdzz_dAx = 0.0d0
t_dxxdxy_dAx = 0.0d0
t_dxxdxz_dAx = 0.0d0
t_dxxdyz_dAx = 0.0d0
t_dyydxx_dAx = 0.0d0
t_dyydyy_dAx = 0.0d0
t_dyydzz_dAx = 0.0d0
t_dyydxy_dAx = 0.0d0
t_dyydxz_dAx = 0.0d0
t_dyydyz_dAx = 0.0d0
t_dzzdxx_dAx = 0.0d0
t_dzzdyy_dAx = 0.0d0
t_dzzdzz_dAx = 0.0d0
t_dzzdxy_dAx = 0.0d0
t_dzzdxz_dAx = 0.0d0
t_dzzdyz_dAx = 0.0d0
t_dxydxx_dAx = 0.0d0
t_dxydyy_dAx = 0.0d0
t_dxydzz_dAx = 0.0d0
t_dxydxy_dAx = 0.0d0
t_dxydxz_dAx = 0.0d0
t_dxydyz_dAx = 0.0d0
t_dxzdxx_dAx = 0.0d0
t_dxzdyy_dAx = 0.0d0
t_dxzdzz_dAx = 0.0d0
t_dxzdxy_dAx = 0.0d0
t_dxzdxz_dAx = 0.0d0
t_dxzdyz_dAx = 0.0d0
t_dyzdxx_dAx = 0.0d0
t_dyzdyy_dAx = 0.0d0
t_dyzdzz_dAx = 0.0d0
t_dyzdxy_dAx = 0.0d0
t_dyzdxz_dAx = 0.0d0
t_dyzdyz_dAx = 0.0d0
t_dxxdxx_dAy = 0.0d0
t_dxxdyy_dAy = 0.0d0
t_dxxdzz_dAy = 0.0d0
t_dxxdxy_dAy = 0.0d0
t_dxxdxz_dAy = 0.0d0
t_dxxdyz_dAy = 0.0d0
t_dyydxx_dAy = 0.0d0
t_dyydyy_dAy = 0.0d0
t_dyydzz_dAy = 0.0d0
t_dyydxy_dAy = 0.0d0
t_dyydxz_dAy = 0.0d0
t_dyydyz_dAy = 0.0d0
t_dzzdxx_dAy = 0.0d0
t_dzzdyy_dAy = 0.0d0
t_dzzdzz_dAy = 0.0d0
t_dzzdxy_dAy = 0.0d0
t_dzzdxz_dAy = 0.0d0
t_dzzdyz_dAy = 0.0d0
t_dxydxx_dAy = 0.0d0
t_dxydyy_dAy = 0.0d0
t_dxydzz_dAy = 0.0d0
t_dxydxy_dAy = 0.0d0
t_dxydxz_dAy = 0.0d0
t_dxydyz_dAy = 0.0d0
t_dxzdxx_dAy = 0.0d0
t_dxzdyy_dAy = 0.0d0
t_dxzdzz_dAy = 0.0d0
t_dxzdxy_dAy = 0.0d0
t_dxzdxz_dAy = 0.0d0
t_dxzdyz_dAy = 0.0d0
t_dyzdxx_dAy = 0.0d0
t_dyzdyy_dAy = 0.0d0
t_dyzdzz_dAy = 0.0d0
t_dyzdxy_dAy = 0.0d0
t_dyzdxz_dAy = 0.0d0
t_dyzdyz_dAy = 0.0d0
t_dxxdxx_dAz = 0.0d0
t_dxxdyy_dAz = 0.0d0
t_dxxdzz_dAz = 0.0d0
t_dxxdxy_dAz = 0.0d0
t_dxxdxz_dAz = 0.0d0
t_dxxdyz_dAz = 0.0d0
t_dyydxx_dAz = 0.0d0
t_dyydyy_dAz = 0.0d0
t_dyydzz_dAz = 0.0d0
t_dyydxy_dAz = 0.0d0
t_dyydxz_dAz = 0.0d0
t_dyydyz_dAz = 0.0d0
t_dzzdxx_dAz = 0.0d0
t_dzzdyy_dAz = 0.0d0
t_dzzdzz_dAz = 0.0d0
t_dzzdxy_dAz = 0.0d0
t_dzzdxz_dAz = 0.0d0
t_dzzdyz_dAz = 0.0d0
t_dxydxx_dAz = 0.0d0
t_dxydyy_dAz = 0.0d0
t_dxydzz_dAz = 0.0d0
t_dxydxy_dAz = 0.0d0
t_dxydxz_dAz = 0.0d0
t_dxydyz_dAz = 0.0d0
t_dxzdxx_dAz = 0.0d0
t_dxzdyy_dAz = 0.0d0
t_dxzdzz_dAz = 0.0d0
t_dxzdxy_dAz = 0.0d0
t_dxzdxz_dAz = 0.0d0
t_dxzdyz_dAz = 0.0d0
t_dyzdxx_dAz = 0.0d0
t_dyzdyy_dAz = 0.0d0
t_dyzdzz_dAz = 0.0d0
t_dyzdxy_dAz = 0.0d0
t_dyzdxz_dAz = 0.0d0
t_dyzdyz_dAz = 0.0d0
do irs = npsh2pr(ij), npsh2pr(ij+1) - 1
o_ss = ss(irs)
zeta_ = zeta(irs)
xi_ = xi(irs)
xi2 = 2.0d0*xi_
t_ss = o_ss*xi_*( 3.0d0 - xi2 * AB2 )
xioza = xiozetaa(irs)
PAx = -ABx*xioza
PAy = -ABy*xioza
PAz = -ABz*xioza
o_pxs = PAx*o_ss
o_pys = PAy*o_ss
o_pzs = PAz*o_ss
oo2zeta = 0.5d0/zeta_
ai_o_ss = oo2zeta*o_ss
o_dxxs = PAx*o_pxs + ai_o_ss
o_dyys = PAy*o_pys + ai_o_ss
o_dzzs = PAz*o_pzs + ai_o_ss
o_dxys = PAy*o_pxs
o_dxzs = PAz*o_pxs
o_dyzs = PAz*o_pys
ai_o_pxs = oo2zeta*o_pxs
o_fxxxs = PAx*o_dxxs + 2.0d0*ai_o_pxs
ai_o_pys = oo2zeta*o_pys
o_fyyys = PAy*o_dyys + 2.0d0*ai_o_pys
ai_o_pzs = oo2zeta*o_pzs
o_fzzzs = PAz*o_dzzs + 2.0d0*ai_o_pzs
o_fxxys = PAy*o_dxxs
o_fxxzs = PAz*o_dxxs
o_fyyxs = PAx*o_dyys
o_fyyzs = PAz*o_dyys
o_fzzxs = PAx*o_dzzs
o_fzzys = PAy*o_dzzs
o_fxyzs = PAz*o_dxys
PBx = PAx + ABx
PBy = PAy + ABy
PBz = PAz + ABz
ai_o_dxxs = oo2zeta*o_dxxs
o_fxxxpx = PBx*o_fxxxs + 3.0d0*ai_o_dxxs
o_fxxxpy = PBy*o_fxxxs
o_fxxxpz = PBz*o_fxxxs
o_fyyypx = PBx*o_fyyys
ai_o_dyys = oo2zeta*o_dyys
o_fyyypy = PBy*o_fyyys + 3.0d0*ai_o_dyys
o_fyyypz = PBz*o_fyyys
o_fzzzpx = PBx*o_fzzzs
o_fzzzpy = PBy*o_fzzzs
ai_o_dzzs = oo2zeta*o_dzzs
o_fzzzpz = PBz*o_fzzzs + 3.0d0*ai_o_dzzs
ai_o_dxys = oo2zeta*o_dxys
o_fxxypx = PBx*o_fxxys + 2.0d0*ai_o_dxys
o_fxxypy = PBy*o_fxxys + ai_o_dxxs
o_fxxypz = PBz*o_fxxys
ai_o_dxzs = oo2zeta*o_dxzs
o_fxxzpx = PBx*o_fxxzs + 2.0d0*ai_o_dxzs
o_fxxzpy = PBy*o_fxxzs
o_fxxzpz = PBz*o_fxxzs + ai_o_dxxs
o_fyyxpx = PBx*o_fyyxs + ai_o_dyys
o_fyyxpy = PBy*o_fyyxs + 2.0d0*ai_o_dxys
o_fyyxpz = PBz*o_fyyxs
o_fyyzpx = PBx*o_fyyzs
ai_o_dyzs = oo2zeta*o_dyzs
o_fyyzpy = PBy*o_fyyzs + 2.0d0*ai_o_dyzs
o_fyyzpz = PBz*o_fyyzs + ai_o_dyys
o_fzzxpx = PBx*o_fzzxs + ai_o_dzzs
o_fzzxpy = PBy*o_fzzxs
o_fzzxpz = PBz*o_fzzxs + 2.0d0*ai_o_dxzs
o_fzzypx = PBx*o_fzzys
o_fzzypy = PBy*o_fzzys + ai_o_dzzs
o_fzzypz = PBz*o_fzzys + 2.0d0*ai_o_dyzs
o_fxyzpx = PBx*o_fxyzs + ai_o_dyzs
o_fxyzpy = PBy*o_fxyzs + ai_o_dxzs
o_fxyzpz = PBz*o_fxyzs + ai_o_dxys
o_dxxpx = PBx*o_dxxs + 2.0d0*ai_o_pxs
o_dxxpy = PBy*o_dxxs
o_dxxpz = PBz*o_dxxs
o_dyypx = PBx*o_dyys
o_dyypy = PBy*o_dyys + 2.0d0*ai_o_pys
o_dyypz = PBz*o_dyys
o_dzzpx = PBx*o_dzzs
o_dzzpy = PBy*o_dzzs
o_dzzpz = PBz*o_dzzs + 2.0d0*ai_o_pzs
o_dxypx = PBx*o_dxys + ai_o_pys
o_dxypy = PBy*o_dxys + ai_o_pxs
o_dxypz = PBz*o_dxys
o_dxzpx = PBx*o_dxzs + ai_o_pzs
o_dxzpy = PBy*o_dxzs
o_dxzpz = PBz*o_dxzs + ai_o_pxs
o_dyzpx = PBx*o_dyzs
o_dyzpy = PBy*o_dyzs + ai_o_pzs
o_dyzpz = PBz*o_dyzs + ai_o_pys
ai_o_dxxpx = oo2zeta*o_dxxpx
bi_o_fxxxs = oo2zeta*o_fxxxs
o_fxxxdxx = PBx*o_fxxxpx + 3.0d0*ai_o_dxxpx + bi_o_fxxxs
o_fxxxdyy = PBy*o_fxxxpy + bi_o_fxxxs
o_fxxxdzz = PBz*o_fxxxpz + bi_o_fxxxs
o_fxxxdxy = PBy*o_fxxxpx
o_fxxxdxz = PBz*o_fxxxpx
o_fxxxdyz = PBz*o_fxxxpy
bi_o_fyyys = oo2zeta*o_fyyys
o_fyyydxx = PBx*o_fyyypx + bi_o_fyyys
ai_o_dyypy = oo2zeta*o_dyypy
o_fyyydyy = PBy*o_fyyypy + 3.0d0*ai_o_dyypy + bi_o_fyyys
o_fyyydzz = PBz*o_fyyypz + bi_o_fyyys
ai_o_dyypx = oo2zeta*o_dyypx
o_fyyydxy = PBy*o_fyyypx + 3.0d0*ai_o_dyypx
o_fyyydxz = PBz*o_fyyypx
o_fyyydyz = PBz*o_fyyypy
bi_o_fzzzs = oo2zeta*o_fzzzs
o_fzzzdxx = PBx*o_fzzzpx + bi_o_fzzzs
o_fzzzdyy = PBy*o_fzzzpy + bi_o_fzzzs
ai_o_dzzpz = oo2zeta*o_dzzpz
o_fzzzdzz = PBz*o_fzzzpz + 3.0d0*ai_o_dzzpz + bi_o_fzzzs
o_fzzzdxy = PBy*o_fzzzpx
ai_o_dzzpx = oo2zeta*o_dzzpx
o_fzzzdxz = PBz*o_fzzzpx + 3.0d0*ai_o_dzzpx
ai_o_dzzpy = oo2zeta*o_dzzpy
o_fzzzdyz = PBz*o_fzzzpy + 3.0d0*ai_o_dzzpy
ai_o_dxypx = oo2zeta*o_dxypx
bi_o_fxxys = oo2zeta*o_fxxys
o_fxxydxx = PBx*o_fxxypx + 2.0d0*ai_o_dxypx + bi_o_fxxys
ai_o_dxxpy = oo2zeta*o_dxxpy
o_fxxydyy = PBy*o_fxxypy + ai_o_dxxpy + bi_o_fxxys
o_fxxydzz = PBz*o_fxxypz + bi_o_fxxys
o_fxxydxy = PBy*o_fxxypx + ai_o_dxxpx
o_fxxydxz = PBz*o_fxxypx
o_fxxydyz = PBz*o_fxxypy
ai_o_dxzpx = oo2zeta*o_dxzpx
bi_o_fxxzs = oo2zeta*o_fxxzs
o_fxxzdxx = PBx*o_fxxzpx + 2.0d0*ai_o_dxzpx + bi_o_fxxzs
o_fxxzdyy = PBy*o_fxxzpy + bi_o_fxxzs
ai_o_dxxpz = oo2zeta*o_dxxpz
o_fxxzdzz = PBz*o_fxxzpz + ai_o_dxxpz + bi_o_fxxzs
o_fxxzdxy = PBy*o_fxxzpx
o_fxxzdxz = PBz*o_fxxzpx + ai_o_dxxpx
o_fxxzdyz = PBz*o_fxxzpy + ai_o_dxxpy
bi_o_fyyxs = oo2zeta*o_fyyxs
o_fyyxdxx = PBx*o_fyyxpx + ai_o_dyypx + bi_o_fyyxs
ai_o_dxypy = oo2zeta*o_dxypy
o_fyyxdyy = PBy*o_fyyxpy + 2.0d0*ai_o_dxypy + bi_o_fyyxs
o_fyyxdzz = PBz*o_fyyxpz + bi_o_fyyxs
o_fyyxdxy = PBy*o_fyyxpx + 2.0d0*ai_o_dxypx
o_fyyxdxz = PBz*o_fyyxpx
o_fyyxdyz = PBz*o_fyyxpy
bi_o_fyyzs = oo2zeta*o_fyyzs
o_fyyzdxx = PBx*o_fyyzpx + bi_o_fyyzs
ai_o_dyzpy = oo2zeta*o_dyzpy
o_fyyzdyy = PBy*o_fyyzpy + 2.0d0*ai_o_dyzpy + bi_o_fyyzs
ai_o_dyypz = oo2zeta*o_dyypz
o_fyyzdzz = PBz*o_fyyzpz + ai_o_dyypz + bi_o_fyyzs
ai_o_dyzpx = oo2zeta*o_dyzpx
o_fyyzdxy = PBy*o_fyyzpx + 2.0d0*ai_o_dyzpx
o_fyyzdxz = PBz*o_fyyzpx + ai_o_dyypx
o_fyyzdyz = PBz*o_fyyzpy + ai_o_dyypy
bi_o_fzzxs = oo2zeta*o_fzzxs
o_fzzxdxx = PBx*o_fzzxpx + ai_o_dzzpx + bi_o_fzzxs
o_fzzxdyy = PBy*o_fzzxpy + bi_o_fzzxs
ai_o_dxzpz = oo2zeta*o_dxzpz
o_fzzxdzz = PBz*o_fzzxpz + 2.0d0*ai_o_dxzpz + bi_o_fzzxs
o_fzzxdxy = PBy*o_fzzxpx
o_fzzxdxz = PBz*o_fzzxpx + 2.0d0*ai_o_dxzpx
ai_o_dxzpy = oo2zeta*o_dxzpy
o_fzzxdyz = PBz*o_fzzxpy + 2.0d0*ai_o_dxzpy
bi_o_fzzys = oo2zeta*o_fzzys
o_fzzydxx = PBx*o_fzzypx + bi_o_fzzys
o_fzzydyy = PBy*o_fzzypy + ai_o_dzzpy + bi_o_fzzys
ai_o_dyzpz = oo2zeta*o_dyzpz
o_fzzydzz = PBz*o_fzzypz + 2.0d0*ai_o_dyzpz + bi_o_fzzys
o_fzzydxy = PBy*o_fzzypx + ai_o_dzzpx
o_fzzydxz = PBz*o_fzzypx + 2.0d0*ai_o_dyzpx
o_fzzydyz = PBz*o_fzzypy + 2.0d0*ai_o_dyzpy
bi_o_fxyzs = oo2zeta*o_fxyzs
o_fxyzdxx = PBx*o_fxyzpx + ai_o_dyzpx + bi_o_fxyzs
o_fxyzdyy = PBy*o_fxyzpy + ai_o_dxzpy + bi_o_fxyzs
ai_o_dxypz = oo2zeta*o_dxypz
o_fxyzdzz = PBz*o_fxyzpz + ai_o_dxypz + bi_o_fxyzs
o_fxyzdxy = PBy*o_fxyzpx + ai_o_dxzpx
o_fxyzdxz = PBz*o_fxyzpx + ai_o_dxypx
o_fxyzdyz = PBz*o_fxyzpy + ai_o_dxypy
t_pxs = PAx*t_ss + xi2*o_pxs
t_pys = PAy*t_ss + xi2*o_pys
t_pzs = PAz*t_ss + xi2*o_pzs
!oo2zeta = 0.5d0*oozeta(irs)
ai_t_ss = oo2zeta*t_ss
xiai_o_ss = xioza*o_ss
t_dxxs = PAx*t_pxs + ai_t_ss + xi2*o_dxxs - xiai_o_ss
t_dyys = PAy*t_pys + ai_t_ss + xi2*o_dyys - xiai_o_ss
t_dzzs = PAz*t_pzs + ai_t_ss + xi2*o_dzzs - xiai_o_ss
t_dxys = PAy*t_pxs + xi2*o_dxys
t_dxzs = PAz*t_pxs + xi2*o_dxzs
t_dyzs = PAz*t_pys + xi2*o_dyzs
ai_t_pxs = oo2zeta*t_pxs
xiai_o_pxs = xioza*o_pxs
t_fxxxs = PAx*t_dxxs + 2.0d0*ai_t_pxs + xi2*o_fxxxs - 2.0d0*xiai_o_pxs
ai_t_pys = oo2zeta*t_pys
xiai_o_pys = xioza*o_pys
t_fyyys = PAy*t_dyys + 2.0d0*ai_t_pys + xi2*o_fyyys - 2.0d0*xiai_o_pys
ai_t_pzs = oo2zeta*t_pzs
xiai_o_pzs = xioza*o_pzs
t_fzzzs = PAz*t_dzzs + 2.0d0*ai_t_pzs + xi2*o_fzzzs - 2.0d0*xiai_o_pzs
t_fxxys = PAy*t_dxxs + xi2*o_fxxys
t_fxxzs = PAz*t_dxxs + xi2*o_fxxzs
t_fyyxs = PAx*t_dyys + xi2*o_fyyxs
t_fyyzs = PAz*t_dyys + xi2*o_fyyzs
t_fzzxs = PAx*t_dzzs + xi2*o_fzzxs
t_fzzys = PAy*t_dzzs + xi2*o_fzzys
t_fxyzs = PAz*t_dxys + xi2*o_fxyzs
ai_t_dxxs = oo2zeta*t_dxxs
t_fxxxpx = PBx*t_fxxxs + 3.0d0*ai_t_dxxs + xi2*o_fxxxpx
t_fxxxpy = PBy*t_fxxxs + xi2*o_fxxxpy
t_fxxxpz = PBz*t_fxxxs + xi2*o_fxxxpz
t_fyyypx = PBx*t_fyyys + xi2*o_fyyypx
ai_t_dyys = oo2zeta*t_dyys
t_fyyypy = PBy*t_fyyys + 3.0d0*ai_t_dyys + xi2*o_fyyypy
t_fyyypz = PBz*t_fyyys + xi2*o_fyyypz
t_fzzzpx = PBx*t_fzzzs + xi2*o_fzzzpx
t_fzzzpy = PBy*t_fzzzs + xi2*o_fzzzpy
ai_t_dzzs = oo2zeta*t_dzzs
t_fzzzpz = PBz*t_fzzzs + 3.0d0*ai_t_dzzs + xi2*o_fzzzpz
ai_t_dxys = oo2zeta*t_dxys
t_fxxypx = PBx*t_fxxys + 2.0d0*ai_t_dxys + xi2*o_fxxypx
t_fxxypy = PBy*t_fxxys + ai_t_dxxs + xi2*o_fxxypy
t_fxxypz = PBz*t_fxxys + xi2*o_fxxypz
ai_t_dxzs = oo2zeta*t_dxzs
t_fxxzpx = PBx*t_fxxzs + 2.0d0*ai_t_dxzs + xi2*o_fxxzpx
t_fxxzpy = PBy*t_fxxzs + xi2*o_fxxzpy
t_fxxzpz = PBz*t_fxxzs + ai_t_dxxs + xi2*o_fxxzpz
t_fyyxpx = PBx*t_fyyxs + ai_t_dyys + xi2*o_fyyxpx
t_fyyxpy = PBy*t_fyyxs + 2.0d0*ai_t_dxys + xi2*o_fyyxpy
t_fyyxpz = PBz*t_fyyxs + xi2*o_fyyxpz
t_fyyzpx = PBx*t_fyyzs + xi2*o_fyyzpx
ai_t_dyzs = oo2zeta*t_dyzs
t_fyyzpy = PBy*t_fyyzs + 2.0d0*ai_t_dyzs + xi2*o_fyyzpy
t_fyyzpz = PBz*t_fyyzs + ai_t_dyys + xi2*o_fyyzpz
t_fzzxpx = PBx*t_fzzxs + ai_t_dzzs + xi2*o_fzzxpx
t_fzzxpy = PBy*t_fzzxs + xi2*o_fzzxpy
t_fzzxpz = PBz*t_fzzxs + 2.0d0*ai_t_dxzs + xi2*o_fzzxpz
t_fzzypx = PBx*t_fzzys + xi2*o_fzzypx
t_fzzypy = PBy*t_fzzys + ai_t_dzzs + xi2*o_fzzypy
t_fzzypz = PBz*t_fzzys + 2.0d0*ai_t_dyzs + xi2*o_fzzypz
t_fxyzpx = PBx*t_fxyzs + ai_t_dyzs + xi2*o_fxyzpx
t_fxyzpy = PBy*t_fxyzs + ai_t_dxzs + xi2*o_fxyzpy
t_fxyzpz = PBz*t_fxyzs + ai_t_dxys + xi2*o_fxyzpz
t_dxxpx = PBx*t_dxxs + 2.0d0*ai_t_pxs + xi2*o_dxxpx
t_dxxpy = PBy*t_dxxs + xi2*o_dxxpy
t_dxxpz = PBz*t_dxxs + xi2*o_dxxpz
t_dyypx = PBx*t_dyys + xi2*o_dyypx
t_dyypy = PBy*t_dyys + 2.0d0*ai_t_pys + xi2*o_dyypy
t_dyypz = PBz*t_dyys + xi2*o_dyypz
t_dzzpx = PBx*t_dzzs + xi2*o_dzzpx
t_dzzpy = PBy*t_dzzs + xi2*o_dzzpy
t_dzzpz = PBz*t_dzzs + 2.0d0*ai_t_pzs + xi2*o_dzzpz
t_dxypx = PBx*t_dxys + ai_t_pys + xi2*o_dxypx
t_dxypy = PBy*t_dxys + ai_t_pxs + xi2*o_dxypy
t_dxypz = PBz*t_dxys + xi2*o_dxypz
t_dxzpx = PBx*t_dxzs + ai_t_pzs + xi2*o_dxzpx
t_dxzpy = PBy*t_dxzs + xi2*o_dxzpy
t_dxzpz = PBz*t_dxzs + ai_t_pxs + xi2*o_dxzpz
t_dyzpx = PBx*t_dyzs + xi2*o_dyzpx
t_dyzpy = PBy*t_dyzs + ai_t_pzs + xi2*o_dyzpy
t_dyzpz = PBz*t_dyzs + ai_t_pys + xi2*o_dyzpz
ai_t_dxxpx = oo2zeta*t_dxxpx
bi_t_fxxxs = oo2zeta*t_fxxxs
xiozb = 1.0d0 - xioza
xibi_o_fxxxs = xiozb*o_fxxxs
t_fxxxdxx = PBx*t_fxxxpx + 3.0d0*ai_t_dxxpx + bi_t_fxxxs + xi2*o_fxxxdxx - xibi_o_fxxxs
t_fxxxdyy = PBy*t_fxxxpy + bi_t_fxxxs + xi2*o_fxxxdyy - xibi_o_fxxxs
t_fxxxdzz = PBz*t_fxxxpz + bi_t_fxxxs + xi2*o_fxxxdzz - xibi_o_fxxxs
t_fxxxdxy = PBy*t_fxxxpx + xi2*o_fxxxdxy
t_fxxxdxz = PBz*t_fxxxpx + xi2*o_fxxxdxz
t_fxxxdyz = PBz*t_fxxxpy + xi2*o_fxxxdyz
bi_t_fyyys = oo2zeta*t_fyyys
xibi_o_fyyys = xiozb*o_fyyys
t_fyyydxx = PBx*t_fyyypx + bi_t_fyyys + xi2*o_fyyydxx - xibi_o_fyyys
ai_t_dyypy = oo2zeta*t_dyypy
t_fyyydyy = PBy*t_fyyypy + 3.0d0*ai_t_dyypy + bi_t_fyyys + xi2*o_fyyydyy - xibi_o_fyyys
t_fyyydzz = PBz*t_fyyypz + bi_t_fyyys + xi2*o_fyyydzz - xibi_o_fyyys
ai_t_dyypx = oo2zeta*t_dyypx
t_fyyydxy = PBy*t_fyyypx + 3.0d0*ai_t_dyypx + xi2*o_fyyydxy
t_fyyydxz = PBz*t_fyyypx + xi2*o_fyyydxz
t_fyyydyz = PBz*t_fyyypy + xi2*o_fyyydyz
bi_t_fzzzs = oo2zeta*t_fzzzs
xibi_o_fzzzs = xiozb*o_fzzzs
t_fzzzdxx = PBx*t_fzzzpx + bi_t_fzzzs + xi2*o_fzzzdxx - xibi_o_fzzzs
t_fzzzdyy = PBy*t_fzzzpy + bi_t_fzzzs + xi2*o_fzzzdyy - xibi_o_fzzzs
ai_t_dzzpz = oo2zeta*t_dzzpz
t_fzzzdzz = PBz*t_fzzzpz + 3.0d0*ai_t_dzzpz + bi_t_fzzzs + xi2*o_fzzzdzz - xibi_o_fzzzs
t_fzzzdxy = PBy*t_fzzzpx + xi2*o_fzzzdxy
ai_t_dzzpx = oo2zeta*t_dzzpx
t_fzzzdxz = PBz*t_fzzzpx + 3.0d0*ai_t_dzzpx + xi2*o_fzzzdxz
ai_t_dzzpy = oo2zeta*t_dzzpy
t_fzzzdyz = PBz*t_fzzzpy + 3.0d0*ai_t_dzzpy + xi2*o_fzzzdyz
ai_t_dxypx = oo2zeta*t_dxypx
bi_t_fxxys = oo2zeta*t_fxxys
xibi_o_fxxys = xiozb*o_fxxys
t_fxxydxx = PBx*t_fxxypx + 2.0d0*ai_t_dxypx + bi_t_fxxys + xi2*o_fxxydxx - xibi_o_fxxys
ai_t_dxxpy = oo2zeta*t_dxxpy
t_fxxydyy = PBy*t_fxxypy + ai_t_dxxpy + bi_t_fxxys + xi2*o_fxxydyy - xibi_o_fxxys
t_fxxydzz = PBz*t_fxxypz + bi_t_fxxys + xi2*o_fxxydzz - xibi_o_fxxys
t_fxxydxy = PBy*t_fxxypx + ai_t_dxxpx + xi2*o_fxxydxy
t_fxxydxz = PBz*t_fxxypx + xi2*o_fxxydxz
t_fxxydyz = PBz*t_fxxypy + xi2*o_fxxydyz
ai_t_dxzpx = oo2zeta*t_dxzpx
bi_t_fxxzs = oo2zeta*t_fxxzs
xibi_o_fxxzs = xiozb*o_fxxzs
t_fxxzdxx = PBx*t_fxxzpx + 2.0d0*ai_t_dxzpx + bi_t_fxxzs + xi2*o_fxxzdxx - xibi_o_fxxzs
t_fxxzdyy = PBy*t_fxxzpy + bi_t_fxxzs + xi2*o_fxxzdyy - xibi_o_fxxzs
ai_t_dxxpz = oo2zeta*t_dxxpz
t_fxxzdzz = PBz*t_fxxzpz + ai_t_dxxpz + bi_t_fxxzs + xi2*o_fxxzdzz - xibi_o_fxxzs
t_fxxzdxy = PBy*t_fxxzpx + xi2*o_fxxzdxy
t_fxxzdxz = PBz*t_fxxzpx + ai_t_dxxpx + xi2*o_fxxzdxz
t_fxxzdyz = PBz*t_fxxzpy + ai_t_dxxpy + xi2*o_fxxzdyz
bi_t_fyyxs = oo2zeta*t_fyyxs
xibi_o_fyyxs = xiozb*o_fyyxs
t_fyyxdxx = PBx*t_fyyxpx + ai_t_dyypx + bi_t_fyyxs + xi2*o_fyyxdxx - xibi_o_fyyxs
ai_t_dxypy = oo2zeta*t_dxypy
t_fyyxdyy = PBy*t_fyyxpy + 2.0d0*ai_t_dxypy + bi_t_fyyxs + xi2*o_fyyxdyy - xibi_o_fyyxs
t_fyyxdzz = PBz*t_fyyxpz + bi_t_fyyxs + xi2*o_fyyxdzz - xibi_o_fyyxs
t_fyyxdxy = PBy*t_fyyxpx + 2.0d0*ai_t_dxypx + xi2*o_fyyxdxy
t_fyyxdxz = PBz*t_fyyxpx + xi2*o_fyyxdxz
t_fyyxdyz = PBz*t_fyyxpy + xi2*o_fyyxdyz
bi_t_fyyzs = oo2zeta*t_fyyzs
xibi_o_fyyzs = xiozb*o_fyyzs
t_fyyzdxx = PBx*t_fyyzpx + bi_t_fyyzs + xi2*o_fyyzdxx - xibi_o_fyyzs
ai_t_dyzpy = oo2zeta*t_dyzpy
t_fyyzdyy = PBy*t_fyyzpy + 2.0d0*ai_t_dyzpy + bi_t_fyyzs + xi2*o_fyyzdyy - xibi_o_fyyzs
ai_t_dyypz = oo2zeta*t_dyypz
t_fyyzdzz = PBz*t_fyyzpz + ai_t_dyypz + bi_t_fyyzs + xi2*o_fyyzdzz - xibi_o_fyyzs
ai_t_dyzpx = oo2zeta*t_dyzpx
t_fyyzdxy = PBy*t_fyyzpx + 2.0d0*ai_t_dyzpx + xi2*o_fyyzdxy
t_fyyzdxz = PBz*t_fyyzpx + ai_t_dyypx + xi2*o_fyyzdxz
t_fyyzdyz = PBz*t_fyyzpy + ai_t_dyypy + xi2*o_fyyzdyz
bi_t_fzzxs = oo2zeta*t_fzzxs
xibi_o_fzzxs = xiozb*o_fzzxs
t_fzzxdxx = PBx*t_fzzxpx + ai_t_dzzpx + bi_t_fzzxs + xi2*o_fzzxdxx - xibi_o_fzzxs
t_fzzxdyy = PBy*t_fzzxpy + bi_t_fzzxs + xi2*o_fzzxdyy - xibi_o_fzzxs
ai_t_dxzpz = oo2zeta*t_dxzpz
t_fzzxdzz = PBz*t_fzzxpz + 2.0d0*ai_t_dxzpz + bi_t_fzzxs + xi2*o_fzzxdzz - xibi_o_fzzxs
t_fzzxdxy = PBy*t_fzzxpx + xi2*o_fzzxdxy
t_fzzxdxz = PBz*t_fzzxpx + 2.0d0*ai_t_dxzpx + xi2*o_fzzxdxz
ai_t_dxzpy = oo2zeta*t_dxzpy
t_fzzxdyz = PBz*t_fzzxpy + 2.0d0*ai_t_dxzpy + xi2*o_fzzxdyz
bi_t_fzzys = oo2zeta*t_fzzys
xibi_o_fzzys = xiozb*o_fzzys
t_fzzydxx = PBx*t_fzzypx + bi_t_fzzys + xi2*o_fzzydxx - xibi_o_fzzys
t_fzzydyy = PBy*t_fzzypy + ai_t_dzzpy + bi_t_fzzys + xi2*o_fzzydyy - xibi_o_fzzys
ai_t_dyzpz = oo2zeta*t_dyzpz
t_fzzydzz = PBz*t_fzzypz + 2.0d0*ai_t_dyzpz + bi_t_fzzys + xi2*o_fzzydzz - xibi_o_fzzys
t_fzzydxy = PBy*t_fzzypx + ai_t_dzzpx + xi2*o_fzzydxy
t_fzzydxz = PBz*t_fzzypx + 2.0d0*ai_t_dyzpx + xi2*o_fzzydxz
t_fzzydyz = PBz*t_fzzypy + 2.0d0*ai_t_dyzpy + xi2*o_fzzydyz
bi_t_fxyzs = oo2zeta*t_fxyzs
xibi_o_fxyzs = xiozb*o_fxyzs
t_fxyzdxx = PBx*t_fxyzpx + ai_t_dyzpx + bi_t_fxyzs + xi2*o_fxyzdxx - xibi_o_fxyzs
t_fxyzdyy = PBy*t_fxyzpy + ai_t_dxzpy + bi_t_fxyzs + xi2*o_fxyzdyy - xibi_o_fxyzs
ai_t_dxypz = oo2zeta*t_dxypz
t_fxyzdzz = PBz*t_fxyzpz + ai_t_dxypz + bi_t_fxyzs + xi2*o_fxyzdzz - xibi_o_fxyzs
t_fxyzdxy = PBy*t_fxyzpx + ai_t_dxzpx + xi2*o_fxyzdxy
t_fxyzdxz = PBz*t_fxyzpx + ai_t_dxypx + xi2*o_fxyzdxz
t_fxyzdyz = PBz*t_fxyzpy + ai_t_dxypy + xi2*o_fxyzdyz
o_pxpx = PBx*o_pxs + ai_o_ss
o_pxpy = PBy*o_pxs
o_pxpz = PBz*o_pxs
o_pypx = PBx*o_pys
o_pypy = PBy*o_pys + ai_o_ss
o_pypz = PBz*o_pys
o_pzpx = PBx*o_pzs
o_pzpy = PBy*o_pzs
o_pzpz = PBz*o_pzs + ai_o_ss
o_spx = PBx*o_ss
o_spy = PBy*o_ss
o_spz = PBz*o_ss
ai_o_spx = oo2zeta*o_spx
bi_o_pxs = oo2zeta*o_pxs
o_pxdxx = PBx*o_pxpx + ai_o_spx + bi_o_pxs
o_pxdyy = PBy*o_pxpy + bi_o_pxs
o_pxdzz = PBz*o_pxpz + bi_o_pxs
o_pxdxy = PBy*o_pxpx
o_pxdxz = PBz*o_pxpx
o_pxdyz = PBz*o_pxpy
bi_o_pys = oo2zeta*o_pys
o_pydxx = PBx*o_pypx + bi_o_pys
ai_o_spy = oo2zeta*o_spy
o_pydyy = PBy*o_pypy + ai_o_spy + bi_o_pys
o_pydzz = PBz*o_pypz + bi_o_pys
o_pydxy = PBy*o_pypx + ai_o_spx
o_pydxz = PBz*o_pypx
o_pydyz = PBz*o_pypy
bi_o_pzs = oo2zeta*o_pzs
o_pzdxx = PBx*o_pzpx + bi_o_pzs
o_pzdyy = PBy*o_pzpy + bi_o_pzs
ai_o_spz = oo2zeta*o_spz
o_pzdzz = PBz*o_pzpz + ai_o_spz + bi_o_pzs
o_pzdxy = PBy*o_pzpx
o_pzdxz = PBz*o_pzpx + ai_o_spx
o_pzdyz = PBz*o_pzpy + ai_o_spy
t_pxpx = PBx*t_pxs + ai_t_ss + xi2*o_pxpx
t_pxpy = PBy*t_pxs + xi2*o_pxpy
t_pxpz = PBz*t_pxs + xi2*o_pxpz
t_pypx = PBx*t_pys + xi2*o_pypx
t_pypy = PBy*t_pys + ai_t_ss + xi2*o_pypy
t_pypz = PBz*t_pys + xi2*o_pypz
t_pzpx = PBx*t_pzs + xi2*o_pzpx
t_pzpy = PBy*t_pzs + xi2*o_pzpy
t_pzpz = PBz*t_pzs + ai_t_ss + xi2*o_pzpz
t_spx = PBx*t_ss + xi2*o_spx
t_spy = PBy*t_ss + xi2*o_spy
t_spz = PBz*t_ss + xi2*o_spz
ai_t_spx = oo2zeta*t_spx
bi_t_pxs = oo2zeta*t_pxs
xibi_o_pxs = xiozb*o_pxs
t_pxdxx = PBx*t_pxpx + ai_t_spx + bi_t_pxs + xi2*o_pxdxx - xibi_o_pxs
t_pxdyy = PBy*t_pxpy + bi_t_pxs + xi2*o_pxdyy - xibi_o_pxs
t_pxdzz = PBz*t_pxpz + bi_t_pxs + xi2*o_pxdzz - xibi_o_pxs
t_pxdxy = PBy*t_pxpx + xi2*o_pxdxy
t_pxdxz = PBz*t_pxpx + xi2*o_pxdxz
t_pxdyz = PBz*t_pxpy + xi2*o_pxdyz
bi_t_pys = oo2zeta*t_pys
xibi_o_pys = xiozb*o_pys
t_pydxx = PBx*t_pypx + bi_t_pys + xi2*o_pydxx - xibi_o_pys
ai_t_spy = oo2zeta*t_spy
t_pydyy = PBy*t_pypy + ai_t_spy + bi_t_pys + xi2*o_pydyy - xibi_o_pys
t_pydzz = PBz*t_pypz + bi_t_pys + xi2*o_pydzz - xibi_o_pys
t_pydxy = PBy*t_pypx + ai_t_spx + xi2*o_pydxy
t_pydxz = PBz*t_pypx + xi2*o_pydxz
t_pydyz = PBz*t_pypy + xi2*o_pydyz
bi_t_pzs = oo2zeta*t_pzs
xibi_o_pzs = xiozb*o_pzs
t_pzdxx = PBx*t_pzpx + bi_t_pzs + xi2*o_pzdxx - xibi_o_pzs
t_pzdyy = PBy*t_pzpy + bi_t_pzs + xi2*o_pzdyy - xibi_o_pzs
ai_t_spz = oo2zeta*t_spz
t_pzdzz = PBz*t_pzpz + ai_t_spz + bi_t_pzs + xi2*o_pzdzz - xibi_o_pzs
t_pzdxy = PBy*t_pzpx + xi2*o_pzdxy
t_pzdxz = PBz*t_pzpx + ai_t_spx + xi2*o_pzdxz
t_pzdyz = PBz*t_pzpy + ai_t_spy + xi2*o_pzdyz
beta = xioza*zeta_
alpha = zeta_ - beta
alpha2 = 2.0d0*alpha
t_dxxdxx_dAx = t_dxxdxx_dAx + alpha2*t_fxxxdxx - 2.0d0*t_pxdxx
t_dxxdyy_dAx = t_dxxdyy_dAx + alpha2*t_fxxxdyy - 2.0d0*t_pxdyy
t_dxxdzz_dAx = t_dxxdzz_dAx + alpha2*t_fxxxdzz - 2.0d0*t_pxdzz
t_dxxdxy_dAx = t_dxxdxy_dAx + alpha2*t_fxxxdxy - 2.0d0*t_pxdxy
t_dxxdxz_dAx = t_dxxdxz_dAx + alpha2*t_fxxxdxz - 2.0d0*t_pxdxz
t_dxxdyz_dAx = t_dxxdyz_dAx + alpha2*t_fxxxdyz - 2.0d0*t_pxdyz
t_dyydxx_dAx = t_dyydxx_dAx + alpha2*t_fyyxdxx
t_dyydyy_dAx = t_dyydyy_dAx + alpha2*t_fyyxdyy
t_dyydzz_dAx = t_dyydzz_dAx + alpha2*t_fyyxdzz
t_dyydxy_dAx = t_dyydxy_dAx + alpha2*t_fyyxdxy
t_dyydxz_dAx = t_dyydxz_dAx + alpha2*t_fyyxdxz
t_dyydyz_dAx = t_dyydyz_dAx + alpha2*t_fyyxdyz
t_dzzdxx_dAx = t_dzzdxx_dAx + alpha2*t_fzzxdxx
t_dzzdyy_dAx = t_dzzdyy_dAx + alpha2*t_fzzxdyy
t_dzzdzz_dAx = t_dzzdzz_dAx + alpha2*t_fzzxdzz
t_dzzdxy_dAx = t_dzzdxy_dAx + alpha2*t_fzzxdxy
t_dzzdxz_dAx = t_dzzdxz_dAx + alpha2*t_fzzxdxz
t_dzzdyz_dAx = t_dzzdyz_dAx + alpha2*t_fzzxdyz
t_dxydxx_dAx = t_dxydxx_dAx + alpha2*t_fxxydxx - t_pydxx
t_dxydyy_dAx = t_dxydyy_dAx + alpha2*t_fxxydyy - t_pydyy
t_dxydzz_dAx = t_dxydzz_dAx + alpha2*t_fxxydzz - t_pydzz
t_dxydxy_dAx = t_dxydxy_dAx + alpha2*t_fxxydxy - t_pydxy
t_dxydxz_dAx = t_dxydxz_dAx + alpha2*t_fxxydxz - t_pydxz
t_dxydyz_dAx = t_dxydyz_dAx + alpha2*t_fxxydyz - t_pydyz
t_dxzdxx_dAx = t_dxzdxx_dAx + alpha2*t_fxxzdxx - t_pzdxx
t_dxzdyy_dAx = t_dxzdyy_dAx + alpha2*t_fxxzdyy - t_pzdyy
t_dxzdzz_dAx = t_dxzdzz_dAx + alpha2*t_fxxzdzz - t_pzdzz
t_dxzdxy_dAx = t_dxzdxy_dAx + alpha2*t_fxxzdxy - t_pzdxy
t_dxzdxz_dAx = t_dxzdxz_dAx + alpha2*t_fxxzdxz - t_pzdxz
t_dxzdyz_dAx = t_dxzdyz_dAx + alpha2*t_fxxzdyz - t_pzdyz
t_dyzdxx_dAx = t_dyzdxx_dAx + alpha2*t_fxyzdxx
t_dyzdyy_dAx = t_dyzdyy_dAx + alpha2*t_fxyzdyy
t_dyzdzz_dAx = t_dyzdzz_dAx + alpha2*t_fxyzdzz
t_dyzdxy_dAx = t_dyzdxy_dAx + alpha2*t_fxyzdxy
t_dyzdxz_dAx = t_dyzdxz_dAx + alpha2*t_fxyzdxz
t_dyzdyz_dAx = t_dyzdyz_dAx + alpha2*t_fxyzdyz
t_dxxdxx_dAy = t_dxxdxx_dAy + alpha2*t_fxxydxx
t_dxxdyy_dAy = t_dxxdyy_dAy + alpha2*t_fxxydyy
t_dxxdzz_dAy = t_dxxdzz_dAy + alpha2*t_fxxydzz
t_dxxdxy_dAy = t_dxxdxy_dAy + alpha2*t_fxxydxy
t_dxxdxz_dAy = t_dxxdxz_dAy + alpha2*t_fxxydxz
t_dxxdyz_dAy = t_dxxdyz_dAy + alpha2*t_fxxydyz
t_dyydxx_dAy = t_dyydxx_dAy + alpha2*t_fyyydxx - 2.0d0*t_pydxx
t_dyydyy_dAy = t_dyydyy_dAy + alpha2*t_fyyydyy - 2.0d0*t_pydyy
t_dyydzz_dAy = t_dyydzz_dAy + alpha2*t_fyyydzz - 2.0d0*t_pydzz
t_dyydxy_dAy = t_dyydxy_dAy + alpha2*t_fyyydxy - 2.0d0*t_pydxy
t_dyydxz_dAy = t_dyydxz_dAy + alpha2*t_fyyydxz - 2.0d0*t_pydxz
t_dyydyz_dAy = t_dyydyz_dAy + alpha2*t_fyyydyz - 2.0d0*t_pydyz
t_dzzdxx_dAy = t_dzzdxx_dAy + alpha2*t_fzzydxx
t_dzzdyy_dAy = t_dzzdyy_dAy + alpha2*t_fzzydyy
t_dzzdzz_dAy = t_dzzdzz_dAy + alpha2*t_fzzydzz
t_dzzdxy_dAy = t_dzzdxy_dAy + alpha2*t_fzzydxy
t_dzzdxz_dAy = t_dzzdxz_dAy + alpha2*t_fzzydxz
t_dzzdyz_dAy = t_dzzdyz_dAy + alpha2*t_fzzydyz
t_dxydxx_dAy = t_dxydxx_dAy + alpha2*t_fyyxdxx - t_pxdxx
t_dxydyy_dAy = t_dxydyy_dAy + alpha2*t_fyyxdyy - t_pxdyy
t_dxydzz_dAy = t_dxydzz_dAy + alpha2*t_fyyxdzz - t_pxdzz
t_dxydxy_dAy = t_dxydxy_dAy + alpha2*t_fyyxdxy - t_pxdxy
t_dxydxz_dAy = t_dxydxz_dAy + alpha2*t_fyyxdxz - t_pxdxz
t_dxydyz_dAy = t_dxydyz_dAy + alpha2*t_fyyxdyz - t_pxdyz
t_dxzdxx_dAy = t_dxzdxx_dAy + alpha2*t_fxyzdxx
t_dxzdyy_dAy = t_dxzdyy_dAy + alpha2*t_fxyzdyy
t_dxzdzz_dAy = t_dxzdzz_dAy + alpha2*t_fxyzdzz
t_dxzdxy_dAy = t_dxzdxy_dAy + alpha2*t_fxyzdxy
t_dxzdxz_dAy = t_dxzdxz_dAy + alpha2*t_fxyzdxz
t_dxzdyz_dAy = t_dxzdyz_dAy + alpha2*t_fxyzdyz
t_dyzdxx_dAy = t_dyzdxx_dAy + alpha2*t_fyyzdxx - t_pzdxx
t_dyzdyy_dAy = t_dyzdyy_dAy + alpha2*t_fyyzdyy - t_pzdyy
t_dyzdzz_dAy = t_dyzdzz_dAy + alpha2*t_fyyzdzz - t_pzdzz
t_dyzdxy_dAy = t_dyzdxy_dAy + alpha2*t_fyyzdxy - t_pzdxy
t_dyzdxz_dAy = t_dyzdxz_dAy + alpha2*t_fyyzdxz - t_pzdxz
t_dyzdyz_dAy = t_dyzdyz_dAy + alpha2*t_fyyzdyz - t_pzdyz
t_dxxdxx_dAz = t_dxxdxx_dAz + alpha2*t_fxxzdxx
t_dxxdyy_dAz = t_dxxdyy_dAz + alpha2*t_fxxzdyy
t_dxxdzz_dAz = t_dxxdzz_dAz + alpha2*t_fxxzdzz
t_dxxdxy_dAz = t_dxxdxy_dAz + alpha2*t_fxxzdxy
t_dxxdxz_dAz = t_dxxdxz_dAz + alpha2*t_fxxzdxz
t_dxxdyz_dAz = t_dxxdyz_dAz + alpha2*t_fxxzdyz
t_dyydxx_dAz = t_dyydxx_dAz + alpha2*t_fyyzdxx
t_dyydyy_dAz = t_dyydyy_dAz + alpha2*t_fyyzdyy
t_dyydzz_dAz = t_dyydzz_dAz + alpha2*t_fyyzdzz
t_dyydxy_dAz = t_dyydxy_dAz + alpha2*t_fyyzdxy
t_dyydxz_dAz = t_dyydxz_dAz + alpha2*t_fyyzdxz
t_dyydyz_dAz = t_dyydyz_dAz + alpha2*t_fyyzdyz
t_dzzdxx_dAz = t_dzzdxx_dAz + alpha2*t_fzzzdxx - 2.0d0*t_pzdxx
t_dzzdyy_dAz = t_dzzdyy_dAz + alpha2*t_fzzzdyy - 2.0d0*t_pzdyy
t_dzzdzz_dAz = t_dzzdzz_dAz + alpha2*t_fzzzdzz - 2.0d0*t_pzdzz
t_dzzdxy_dAz = t_dzzdxy_dAz + alpha2*t_fzzzdxy - 2.0d0*t_pzdxy
t_dzzdxz_dAz = t_dzzdxz_dAz + alpha2*t_fzzzdxz - 2.0d0*t_pzdxz
t_dzzdyz_dAz = t_dzzdyz_dAz + alpha2*t_fzzzdyz - 2.0d0*t_pzdyz
t_dxydxx_dAz = t_dxydxx_dAz + alpha2*t_fxyzdxx
t_dxydyy_dAz = t_dxydyy_dAz + alpha2*t_fxyzdyy
t_dxydzz_dAz = t_dxydzz_dAz + alpha2*t_fxyzdzz
t_dxydxy_dAz = t_dxydxy_dAz + alpha2*t_fxyzdxy
t_dxydxz_dAz = t_dxydxz_dAz + alpha2*t_fxyzdxz
t_dxydyz_dAz = t_dxydyz_dAz + alpha2*t_fxyzdyz
t_dxzdxx_dAz = t_dxzdxx_dAz + alpha2*t_fzzxdxx - t_pxdxx
t_dxzdyy_dAz = t_dxzdyy_dAz + alpha2*t_fzzxdyy - t_pxdyy
t_dxzdzz_dAz = t_dxzdzz_dAz + alpha2*t_fzzxdzz - t_pxdzz
t_dxzdxy_dAz = t_dxzdxy_dAz + alpha2*t_fzzxdxy - t_pxdxy
t_dxzdxz_dAz = t_dxzdxz_dAz + alpha2*t_fzzxdxz - t_pxdxz
t_dxzdyz_dAz = t_dxzdyz_dAz + alpha2*t_fzzxdyz - t_pxdyz
t_dyzdxx_dAz = t_dyzdxx_dAz + alpha2*t_fzzydxx - t_pydxx
t_dyzdyy_dAz = t_dyzdyy_dAz + alpha2*t_fzzydyy - t_pydyy
t_dyzdzz_dAz = t_dyzdzz_dAz + alpha2*t_fzzydzz - t_pydzz
t_dyzdxy_dAz = t_dyzdxy_dAz + alpha2*t_fzzydxy - t_pydxy
t_dyzdxz_dAz = t_dyzdxz_dAz + alpha2*t_fzzydxz - t_pydxz
t_dyzdyz_dAz = t_dyzdyz_dAz + alpha2*t_fzzydyz - t_pydyz
end do
v(1,1) = t_dxxdxx_dAx
v(2,1) = t_dxxdyy_dAx
v(3,1) = t_dxxdzz_dAx
v(4,1) = t_dxxdxy_dAx*root3
v(5,1) = t_dxxdxz_dAx*root3
v(6,1) = t_dxxdyz_dAx*root3
v(7,1) = t_dyydxx_dAx
v(8,1) = t_dyydyy_dAx
v(9,1) = t_dyydzz_dAx
v(10,1) = t_dyydxy_dAx*root3
v(11,1) = t_dyydxz_dAx*root3
v(12,1) = t_dyydyz_dAx*root3
v(13,1) = t_dzzdxx_dAx
v(14,1) = t_dzzdyy_dAx
v(15,1) = t_dzzdzz_dAx
v(16,1) = t_dzzdxy_dAx*root3
v(17,1) = t_dzzdxz_dAx*root3
v(18,1) = t_dzzdyz_dAx*root3
v(19,1) = t_dxydxx_dAx*root3
v(20,1) = t_dxydyy_dAx*root3
v(21,1) = t_dxydzz_dAx*root3
v(22,1) = t_dxydxy_dAx*root3x2
v(23,1) = t_dxydxz_dAx*root3x2
v(24,1) = t_dxydyz_dAx*root3x2
v(25,1) = t_dxzdxx_dAx*root3
v(26,1) = t_dxzdyy_dAx*root3
v(27,1) = t_dxzdzz_dAx*root3
v(28,1) = t_dxzdxy_dAx*root3x2
v(29,1) = t_dxzdxz_dAx*root3x2
v(30,1) = t_dxzdyz_dAx*root3x2
v(31,1) = t_dyzdxx_dAx*root3
v(32,1) = t_dyzdyy_dAx*root3
v(33,1) = t_dyzdzz_dAx*root3
v(34,1) = t_dyzdxy_dAx*root3x2
v(35,1) = t_dyzdxz_dAx*root3x2
v(36,1) = t_dyzdyz_dAx*root3x2
v(1,2) = t_dxxdxx_dAy
v(2,2) = t_dxxdyy_dAy
v(3,2) = t_dxxdzz_dAy
v(4,2) = t_dxxdxy_dAy*root3
v(5,2) = t_dxxdxz_dAy*root3
v(6,2) = t_dxxdyz_dAy*root3
v(7,2) = t_dyydxx_dAy
v(8,2) = t_dyydyy_dAy
v(9,2) = t_dyydzz_dAy
v(10,2) = t_dyydxy_dAy*root3
v(11,2) = t_dyydxz_dAy*root3
v(12,2) = t_dyydyz_dAy*root3
v(13,2) = t_dzzdxx_dAy
v(14,2) = t_dzzdyy_dAy
v(15,2) = t_dzzdzz_dAy
v(16,2) = t_dzzdxy_dAy*root3
v(17,2) = t_dzzdxz_dAy*root3
v(18,2) = t_dzzdyz_dAy*root3
v(19,2) = t_dxydxx_dAy*root3
v(20,2) = t_dxydyy_dAy*root3
v(21,2) = t_dxydzz_dAy*root3
v(22,2) = t_dxydxy_dAy*root3x2
v(23,2) = t_dxydxz_dAy*root3x2
v(24,2) = t_dxydyz_dAy*root3x2
v(25,2) = t_dxzdxx_dAy*root3
v(26,2) = t_dxzdyy_dAy*root3
v(27,2) = t_dxzdzz_dAy*root3
v(28,2) = t_dxzdxy_dAy*root3x2
v(29,2) = t_dxzdxz_dAy*root3x2
v(30,2) = t_dxzdyz_dAy*root3x2
v(31,2) = t_dyzdxx_dAy*root3
v(32,2) = t_dyzdyy_dAy*root3
v(33,2) = t_dyzdzz_dAy*root3
v(34,2) = t_dyzdxy_dAy*root3x2
v(35,2) = t_dyzdxz_dAy*root3x2
v(36,2) = t_dyzdyz_dAy*root3x2
v(1,3) = t_dxxdxx_dAz
v(2,3) = t_dxxdyy_dAz
v(3,3) = t_dxxdzz_dAz
v(4,3) = t_dxxdxy_dAz*root3
v(5,3) = t_dxxdxz_dAz*root3
v(6,3) = t_dxxdyz_dAz*root3
v(7,3) = t_dyydxx_dAz
v(8,3) = t_dyydyy_dAz
v(9,3) = t_dyydzz_dAz
v(10,3) = t_dyydxy_dAz*root3
v(11,3) = t_dyydxz_dAz*root3
v(12,3) = t_dyydyz_dAz*root3
v(13,3) = t_dzzdxx_dAz
v(14,3) = t_dzzdyy_dAz
v(15,3) = t_dzzdzz_dAz
v(16,3) = t_dzzdxy_dAz*root3
v(17,3) = t_dzzdxz_dAz*root3
v(18,3) = t_dzzdyz_dAz*root3
v(19,3) = t_dxydxx_dAz*root3
v(20,3) = t_dxydyy_dAz*root3
v(21,3) = t_dxydzz_dAz*root3
v(22,3) = t_dxydxy_dAz*root3x2
v(23,3) = t_dxydxz_dAz*root3x2
v(24,3) = t_dxydyz_dAz*root3x2
v(25,3) = t_dxzdxx_dAz*root3
v(26,3) = t_dxzdyy_dAz*root3
v(27,3) = t_dxzdzz_dAz*root3
v(28,3) = t_dxzdxy_dAz*root3x2
v(29,3) = t_dxzdxz_dAz*root3x2
v(30,3) = t_dxzdyz_dAz*root3x2
v(31,3) = t_dyzdxx_dAz*root3
v(32,3) = t_dyzdyy_dAz*root3
v(33,3) = t_dyzdzz_dAz*root3
v(34,3) = t_dyzdxy_dAz*root3x2
v(35,3) = t_dyzdxz_dAz*root3x2
v(36,3) = t_dyzdyz_dAz*root3x2

     i = npsh(1,ij)
     j = npsh(2,ij)
     iao = nsh2ao(i)
     jao = nsh2ao(j)

     if (iao == jao) then
        do ixyz = 1, 3
           do jx = jao,jao + 6-1
              do ix = jx,iao + 6-1
                 key = ( ix-iao ) * 6 + jx-jao +1
                 g(ixyz,iatom) = g(ixyz,iatom) + dm(ix,jx)*v(key,ixyz)
              end do
           end do
        end do
        do ixyz = 1, 3
           do jx = jao,jao + 6-1
              do ix = jx,iao + 6-1
                 key = ( ix-iao ) * 6 + jx-jao +1
                 g(ixyz,jatom) = g(ixyz,jatom) - dm(ix,jx)*v(key,ixyz)
              end do
           end do
        end do
     else
        do ixyz = 1, 3
           do jx = jao,jao + 6-1
              do ix = iao,iao + 6-1
                 key = ( ix-iao ) * 6 + jx-jao +1
                 g(ixyz,iatom) = g(ixyz,iatom) + dm(ix,jx)*v(key,ixyz)
              end do
           end do
        end do
        do ixyz = 1, 3
           do jx = jao,jao + 6-1
              do ix = iao,iao + 6-1
                 key = ( ix-iao ) * 6 + jx-jao +1
                 g(ixyz,jatom) = g(ixyz,jatom) - dm(ix,jx)*v(key,ixyz)
              end do
           end do
        end do
     end if
  end do
!$omp end do nowait
end subroutine os_dkei_dd_t1
