! -*- coding: utf-8 -*-
!
! Dual licensed under the MIT or GPL Version 3 license.
! See 'LICENSE.MIT' for the definition of the MIT license,
! and 'COPYING' for the definition of the GPL license.
!
!------------------------------------------------------------------------------
!> overlap integrals
!
! References:
!  [1] S.Obara and A.Saika, J.Chem.Phys. 84, 3963 (1986).
!
subroutine os_dovi_t1(g_,d,nao,tables,atoms)
  use integral_tables_class
  use atoms_class
  use pl_prof_class
  integer,intent(in)::nao
  type(atoms_type),intent(in)::atoms
  real(8),intent(inout)::g_(3,atoms%natom)
  real(8),intent(in)::d(nao,nao)
  type(tables_type),intent(in)::tables

  real(8),allocatable::g(:,:)

  allocate(g(3,atoms%natom))
  g(:,:) = 0.0d0

!$omp parallel&
!$omp& default(private)&
!$omp& shared(d,nao,tables,atoms)&
!$omp& reduction(+:g)
  call pl_prof_region_begin('os_dovi_ss_t1')
  call os_dovi_ss_t1(&
       g,d,nao,&
       tables%ishp,tables%npsh,tables%npsh2pr,tables%nsh2atm,tables%nsh2ao,&
       atoms%xyz,tables%ovl,tables%bdz,tables%zet,&
       tables%nshell,tables%nshelp,tables%nprimp,atoms%natom)
  call pl_prof_region_end('os_dovi_ss_t1')
  if (tables%mang == 1) go to 9

  call pl_prof_region_begin('os_dovi_ps_t1')
  call os_dovi_ps_t1(&
       g,d,nao,&
       tables%ishp,tables%npsh,tables%npsh2pr,tables%nsh2atm,tables%nsh2ao,&
       atoms%xyz,tables%ovl,tables%bdz,tables%zet,&
       tables%nshell,tables%nshelp,tables%nprimp,atoms%natom)
  call pl_prof_region_end('os_dovi_ps_t1')

  call pl_prof_region_begin('os_dovi_pp_t1')
  call os_dovi_pp_t1(&
       g,d,nao,&
       tables%ishp,tables%npsh,tables%npsh2pr,tables%nsh2atm,tables%nsh2ao,&
       atoms%xyz,tables%ovl,tables%bdz,tables%zet,&
       tables%nshell,tables%nshelp,tables%nprimp,atoms%natom)
  call pl_prof_region_end('os_dovi_pp_t1')
  if (tables%mang == 2) go to 9

  call pl_prof_region_begin('os_dovi_ds_t1')
  call os_dovi_ds_t1(&
       g,d,nao,&
       tables%ishp,tables%npsh,tables%npsh2pr,tables%nsh2atm,tables%nsh2ao,&
       atoms%xyz,tables%ovl,tables%bdz,tables%zet,&
       tables%nshell,tables%nshelp,tables%nprimp,atoms%natom)
  call pl_prof_region_end('os_dovi_ds_t1')

  call pl_prof_region_begin('os_dovi_dp_t1')
  call os_dovi_dp_t1(&
       g,d,nao,&
       tables%ishp,tables%npsh,tables%npsh2pr,tables%nsh2atm,tables%nsh2ao,&
       atoms%xyz,tables%ovl,tables%bdz,tables%zet,&
       tables%nshell,tables%nshelp,tables%nprimp,atoms%natom)
  call pl_prof_region_end('os_dovi_dp_t1')

  call pl_prof_region_begin('os_dovi_dd_t1')
  call os_dovi_dd_t1(&
       g,d,nao,&
       tables%ishp,tables%npsh,tables%npsh2pr,tables%nsh2atm,tables%nsh2ao,&
       atoms%xyz,tables%ovl,tables%bdz,tables%zet,&
       tables%nshell,tables%nshelp,tables%nprimp,atoms%natom)
  call pl_prof_region_end('os_dovi_dd_t1')
9 continue
!$omp end parallel

  g_(:,:) = g_(:,:) + g(:,:)

end subroutine os_dovi_t1

!------------------------------------------------------------------------------
!> ss overlap integrals
!
subroutine os_dovi_ss_t1(&
     g,dm,nao,&
     ishp,npsh,npsh2pr,nsh2atm,nsh2ao,xyz,ss,xiozetaa,zeta,&
     nshell,nshellp,nprimp,natom)
  use physical_constants_class

  implicit real(8) (a-h,o-z)
  integer,intent(in)::nao

  integer,intent(in)::nshell,nshellp,nprimp,natom
  integer,intent(in)::ishp(6),npsh(2,nshellp),npsh2pr(nshellp+1)
  integer,intent(in)::nsh2atm(nshell),nsh2ao(nshell)
  real(8),intent(in)::ss(nprimp),xiozetaa(nprimp),zeta(nprimp)
  real(8),intent(in)::xyz(3,natom)

  real(8),intent(inout)::g(3,natom)
  real(8),intent(in)::dm(nao,nao)

  real(8)::v(1,3)

!$omp do schedule(dynamic)
  do ij = 1, ishp(1)
     i = npsh(1,ij)
     j = npsh(2,ij)
     iatom = nsh2atm(i)
     jatom = nsh2atm(j)
     Ax = xyz(1,iatom)
     Ay = xyz(2,iatom)
     Az = xyz(3,iatom)
     Bx = xyz(1,jatom)
     By = xyz(2,jatom)
     Bz = xyz(3,jatom)
     ABx = Ax - Bx
     ABy = Ay - By
     ABz = Az - Bz

o_ss_dAx = 0.0d0
o_ss_dAy = 0.0d0
o_ss_dAz = 0.0d0
do irs = npsh2pr(ij), npsh2pr(ij+1) - 1
o_ss = ss(irs)
zeta_ = zeta(irs)
xioza = xiozetaa(irs)
PAx = -ABx*xioza
PAy = -ABy*xioza
PAz = -ABz*xioza
o_pxs = PAx*o_ss
o_pys = PAy*o_ss
o_pzs = PAz*o_ss
beta = xioza*zeta_
alpha = zeta_ - beta
alpha2 = 2.0d0*alpha
o_ss_dAx = o_ss_dAx + alpha2*o_pxs
o_ss_dAy = o_ss_dAy + alpha2*o_pys
o_ss_dAz = o_ss_dAz + alpha2*o_pzs
end do
v(1,1) = o_ss_dAx
v(1,2) = o_ss_dAy
v(1,3) = o_ss_dAz

     i = npsh(1,ij)
     j = npsh(2,ij)
     iao = nsh2ao(i)
     jao = nsh2ao(j)

     do ixyz = 1, 3
        g(ixyz,iatom) = g(ixyz,iatom) + dm(iao,jao)*v(1,ixyz)
     end do
     do ixyz = 1, 3
        g(ixyz,jatom) = g(ixyz,jatom) - dm(iao,jao)*v(1,ixyz)
     end do
  end do
!$omp end do nowait
end subroutine os_dovi_ss_t1

!------------------------------------------------------------------------------
!> ps overlap integrals
!
subroutine os_dovi_ps_t1(&
     g,dm,nao,&
     ishp,npsh,npsh2pr,nsh2atm,nsh2ao,xyz,ss,xiozetaa,zeta,&
     nshell,nshellp,nprimp,natom)
  use physical_constants_class

  implicit real(8) (a-h,o-z)
  integer,intent(in)::nao

  integer,intent(in)::nshell,nshellp,nprimp,natom
  integer,intent(in)::ishp(6),npsh(2,nshellp),npsh2pr(nshellp+1)
  integer,intent(in)::nsh2atm(nshell),nsh2ao(nshell)
  real(8),intent(in)::ss(nprimp),xiozetaa(nprimp),zeta(nprimp)
  real(8),intent(in)::xyz(3,natom)

  real(8),intent(inout)::g(3,natom)
  real(8),intent(in)::dm(nao,nao)

  real(8)::v(3,3)

!$omp do schedule(dynamic)
  do ij = ishp(1)+1, ishp(2)
     i = npsh(1,ij)
     j = npsh(2,ij)
     iatom = nsh2atm(i)
     jatom = nsh2atm(j)
     Ax = xyz(1,iatom)
     Ay = xyz(2,iatom)
     Az = xyz(3,iatom)
     Bx = xyz(1,jatom)
     By = xyz(2,jatom)
     Bz = xyz(3,jatom)
     ABx = Ax - Bx
     ABy = Ay - By
     ABz = Az - Bz

o_pxs_dAx = 0.0d0
o_pys_dAx = 0.0d0
o_pzs_dAx = 0.0d0
o_pxs_dAy = 0.0d0
o_pys_dAy = 0.0d0
o_pzs_dAy = 0.0d0
o_pxs_dAz = 0.0d0
o_pys_dAz = 0.0d0
o_pzs_dAz = 0.0d0
do irs = npsh2pr(ij), npsh2pr(ij+1) - 1
o_ss = ss(irs)
zeta_ = zeta(irs)
xioza = xiozetaa(irs)
PAx = -ABx*xioza
PAy = -ABy*xioza
PAz = -ABz*xioza
o_pxs = PAx*o_ss
o_pys = PAy*o_ss
o_pzs = PAz*o_ss
oo2zeta = 0.5d0/zeta_
ai_o_ss = oo2zeta*o_ss
o_dxxs = PAx*o_pxs + ai_o_ss
o_dyys = PAy*o_pys + ai_o_ss
o_dzzs = PAz*o_pzs + ai_o_ss
o_dxys = PAy*o_pxs
o_dxzs = PAz*o_pxs
o_dyzs = PAz*o_pys
beta = xioza*zeta_
alpha = zeta_ - beta
alpha2 = 2.0d0*alpha
o_pxs_dAx = o_pxs_dAx + alpha2*o_dxxs - o_ss
o_pys_dAx = o_pys_dAx + alpha2*o_dxys
o_pzs_dAx = o_pzs_dAx + alpha2*o_dxzs
o_pxs_dAy = o_pxs_dAy + alpha2*o_dxys
o_pys_dAy = o_pys_dAy + alpha2*o_dyys - o_ss
o_pzs_dAy = o_pzs_dAy + alpha2*o_dyzs
o_pxs_dAz = o_pxs_dAz + alpha2*o_dxzs
o_pys_dAz = o_pys_dAz + alpha2*o_dyzs
o_pzs_dAz = o_pzs_dAz + alpha2*o_dzzs - o_ss
end do
v(1,1) = o_pxs_dAx
v(2,1) = o_pys_dAx
v(3,1) = o_pzs_dAx
v(1,2) = o_pxs_dAy
v(2,2) = o_pys_dAy
v(3,2) = o_pzs_dAy
v(1,3) = o_pxs_dAz
v(2,3) = o_pys_dAz
v(3,3) = o_pzs_dAz

     i = npsh(1,ij)
     j = npsh(2,ij)
     iao = nsh2ao(i)
     jao = nsh2ao(j)

     if (i<j) then
        do ixyz = 1, 3
           do ix = iao,iao + 3-1
              key = ix-iao + 1
              g(ixyz,iatom) = g(ixyz,iatom) + dm(jao,ix)*v(key,ixyz)
           end do
        end do
        do ixyz = 1, 3
           do ix = iao,iao + 3-1
              key = ix-iao + 1
              g(ixyz,jatom) = g(ixyz,jatom) - dm(jao,ix)*v(key,ixyz)
           end do
        end do
     else
        do ixyz = 1, 3
           do ix = iao,iao + 3-1
              key = ix-iao + 1
              g(ixyz,iatom) = g(ixyz,iatom) + dm(ix,jao)*v(key,ixyz)
           end do
        end do
        do ixyz = 1, 3
           do ix = iao,iao + 3-1
              key = ix-iao + 1
              g(ixyz,jatom) = g(ixyz,jatom) - dm(ix,jao)*v(key,ixyz)
           end do
        end do
     end if
  end do
!$omp end do nowait
end subroutine os_dovi_ps_t1

!------------------------------------------------------------------------------
!> pp overlap integrals
!
subroutine os_dovi_pp_t1(&
     g,dm,nao,&
     ishp,npsh,npsh2pr,nsh2atm,nsh2ao,xyz,ss,xiozetaa,zeta,&
     nshell,nshellp,nprimp,natom)
  use physical_constants_class

  implicit real(8) (a-h,o-z)
  integer,intent(in)::nao

  integer,intent(in)::nshell,nshellp,nprimp,natom
  integer,intent(in)::ishp(6),npsh(2,nshellp),npsh2pr(nshellp+1)
  integer,intent(in)::nsh2atm(nshell),nsh2ao(nshell)
  real(8),intent(in)::ss(nprimp),xiozetaa(nprimp),zeta(nprimp)
  real(8),intent(in)::xyz(3,natom)

  real(8),intent(inout)::g(3,natom)
  real(8),intent(in)::dm(nao,nao)

  real(8)::v(9,3)

!$omp do schedule(dynamic)
  do ij = ishp(2)+1, ishp(3)
     i = npsh(1,ij)
     j = npsh(2,ij)
     iatom = nsh2atm(i)
     jatom = nsh2atm(j)
     Ax = xyz(1,iatom)
     Ay = xyz(2,iatom)
     Az = xyz(3,iatom)
     Bx = xyz(1,jatom)
     By = xyz(2,jatom)
     Bz = xyz(3,jatom)
     ABx = Ax - Bx
     ABy = Ay - By
     ABz = Az - Bz

o_pxpx_dAx = 0.0d0
o_pxpy_dAx = 0.0d0
o_pxpz_dAx = 0.0d0
o_pypx_dAx = 0.0d0
o_pypy_dAx = 0.0d0
o_pypz_dAx = 0.0d0
o_pzpx_dAx = 0.0d0
o_pzpy_dAx = 0.0d0
o_pzpz_dAx = 0.0d0
o_pxpx_dAy = 0.0d0
o_pxpy_dAy = 0.0d0
o_pxpz_dAy = 0.0d0
o_pypx_dAy = 0.0d0
o_pypy_dAy = 0.0d0
o_pypz_dAy = 0.0d0
o_pzpx_dAy = 0.0d0
o_pzpy_dAy = 0.0d0
o_pzpz_dAy = 0.0d0
o_pxpx_dAz = 0.0d0
o_pxpy_dAz = 0.0d0
o_pxpz_dAz = 0.0d0
o_pypx_dAz = 0.0d0
o_pypy_dAz = 0.0d0
o_pypz_dAz = 0.0d0
o_pzpx_dAz = 0.0d0
o_pzpy_dAz = 0.0d0
o_pzpz_dAz = 0.0d0
do irs = npsh2pr(ij), npsh2pr(ij+1) - 1
o_ss = ss(irs)
zeta_ = zeta(irs)
xioza = xiozetaa(irs)
PAx = -ABx*xioza
PAy = -ABy*xioza
PAz = -ABz*xioza
o_pxs = PAx*o_ss
o_pys = PAy*o_ss
o_pzs = PAz*o_ss
oo2zeta = 0.5d0/zeta_
ai_o_ss = oo2zeta*o_ss
o_dxxs = PAx*o_pxs + ai_o_ss
o_dyys = PAy*o_pys + ai_o_ss
o_dzzs = PAz*o_pzs + ai_o_ss
o_dxys = PAy*o_pxs
o_dxzs = PAz*o_pxs
o_dyzs = PAz*o_pys
PBx = PAx + ABx
PBy = PAy + ABy
PBz = PAz + ABz
ai_o_pxs = oo2zeta*o_pxs
o_dxxpx = PBx*o_dxxs + 2.0d0*ai_o_pxs
o_dxxpy = PBy*o_dxxs
o_dxxpz = PBz*o_dxxs
o_dyypx = PBx*o_dyys
ai_o_pys = oo2zeta*o_pys
o_dyypy = PBy*o_dyys + 2.0d0*ai_o_pys
o_dyypz = PBz*o_dyys
o_dzzpx = PBx*o_dzzs
o_dzzpy = PBy*o_dzzs
ai_o_pzs = oo2zeta*o_pzs
o_dzzpz = PBz*o_dzzs + 2.0d0*ai_o_pzs
o_dxypx = PBx*o_dxys + ai_o_pys
o_dxypy = PBy*o_dxys + ai_o_pxs
o_dxypz = PBz*o_dxys
o_dxzpx = PBx*o_dxzs + ai_o_pzs
o_dxzpy = PBy*o_dxzs
o_dxzpz = PBz*o_dxzs + ai_o_pxs
o_dyzpx = PBx*o_dyzs
o_dyzpy = PBy*o_dyzs + ai_o_pzs
o_dyzpz = PBz*o_dyzs + ai_o_pys
o_spx = PBx*o_ss
o_spy = PBy*o_ss
o_spz = PBz*o_ss
beta = xioza*zeta_
alpha = zeta_ - beta
alpha2 = 2.0d0*alpha
o_pxpx_dAx = o_pxpx_dAx + alpha2*o_dxxpx - o_spx
o_pxpy_dAx = o_pxpy_dAx + alpha2*o_dxxpy - o_spy
o_pxpz_dAx = o_pxpz_dAx + alpha2*o_dxxpz - o_spz
o_pypx_dAx = o_pypx_dAx + alpha2*o_dxypx
o_pypy_dAx = o_pypy_dAx + alpha2*o_dxypy
o_pypz_dAx = o_pypz_dAx + alpha2*o_dxypz
o_pzpx_dAx = o_pzpx_dAx + alpha2*o_dxzpx
o_pzpy_dAx = o_pzpy_dAx + alpha2*o_dxzpy
o_pzpz_dAx = o_pzpz_dAx + alpha2*o_dxzpz
o_pxpx_dAy = o_pxpx_dAy + alpha2*o_dxypx
o_pxpy_dAy = o_pxpy_dAy + alpha2*o_dxypy
o_pxpz_dAy = o_pxpz_dAy + alpha2*o_dxypz
o_pypx_dAy = o_pypx_dAy + alpha2*o_dyypx - o_spx
o_pypy_dAy = o_pypy_dAy + alpha2*o_dyypy - o_spy
o_pypz_dAy = o_pypz_dAy + alpha2*o_dyypz - o_spz
o_pzpx_dAy = o_pzpx_dAy + alpha2*o_dyzpx
o_pzpy_dAy = o_pzpy_dAy + alpha2*o_dyzpy
o_pzpz_dAy = o_pzpz_dAy + alpha2*o_dyzpz
o_pxpx_dAz = o_pxpx_dAz + alpha2*o_dxzpx
o_pxpy_dAz = o_pxpy_dAz + alpha2*o_dxzpy
o_pxpz_dAz = o_pxpz_dAz + alpha2*o_dxzpz
o_pypx_dAz = o_pypx_dAz + alpha2*o_dyzpx
o_pypy_dAz = o_pypy_dAz + alpha2*o_dyzpy
o_pypz_dAz = o_pypz_dAz + alpha2*o_dyzpz
o_pzpx_dAz = o_pzpx_dAz + alpha2*o_dzzpx - o_spx
o_pzpy_dAz = o_pzpy_dAz + alpha2*o_dzzpy - o_spy
o_pzpz_dAz = o_pzpz_dAz + alpha2*o_dzzpz - o_spz
end do
v(1,1) = o_pxpx_dAx
v(2,1) = o_pxpy_dAx
v(3,1) = o_pxpz_dAx
v(4,1) = o_pypx_dAx
v(5,1) = o_pypy_dAx
v(6,1) = o_pypz_dAx
v(7,1) = o_pzpx_dAx
v(8,1) = o_pzpy_dAx
v(9,1) = o_pzpz_dAx
v(1,2) = o_pxpx_dAy
v(2,2) = o_pxpy_dAy
v(3,2) = o_pxpz_dAy
v(4,2) = o_pypx_dAy
v(5,2) = o_pypy_dAy
v(6,2) = o_pypz_dAy
v(7,2) = o_pzpx_dAy
v(8,2) = o_pzpy_dAy
v(9,2) = o_pzpz_dAy
v(1,3) = o_pxpx_dAz
v(2,3) = o_pxpy_dAz
v(3,3) = o_pxpz_dAz
v(4,3) = o_pypx_dAz
v(5,3) = o_pypy_dAz
v(6,3) = o_pypz_dAz
v(7,3) = o_pzpx_dAz
v(8,3) = o_pzpy_dAz
v(9,3) = o_pzpz_dAz

     i = npsh(1,ij)
     j = npsh(2,ij)
     iao = nsh2ao(i)
     jao = nsh2ao(j)

     if (iao == jao) then
        do ixyz = 1, 3
           do jx = jao,jao + 3-1
              do ix = jx,iao + 3-1
                 key = (ix-iao)*3 + jx-jao + 1
                 g(ixyz,iatom) = g(ixyz,iatom) + dm(ix,jx)*v(key,ixyz)
              end do
           end do
        end do
        do ixyz = 1, 3
           do jx = jao,jao + 3-1
              do ix = jx,iao + 3-1
                 key = (ix-iao)*3 + jx-jao + 1
                 g(ixyz,jatom) = g(ixyz,jatom) - dm(ix,jx)*v(key,ixyz)
              end do
           end do
        end do
     else
        do ixyz = 1, 3
           do jx = jao,jao + 3-1
              do ix = iao,iao + 3-1
                 key = (ix-iao)*3 + jx-jao + 1
                 g(ixyz,iatom) = g(ixyz,iatom) + dm(ix,jx)*v(key,ixyz)
              end do
           end do
        end do
        do ixyz = 1, 3
           do jx = jao,jao + 3-1
              do ix = iao,iao + 3-1
                 key = (ix-iao)*3 + jx-jao + 1
                 g(ixyz,jatom) = g(ixyz,jatom) - dm(ix,jx)*v(key,ixyz)
              end do
           end do
        end do
     end if
  end do
!$omp end do nowait
end subroutine os_dovi_pp_t1

!------------------------------------------------------------------------------
!> ds overlap integrals
!
subroutine os_dovi_ds_t1(&
     g,dm,nao,&
     ishp,npsh,npsh2pr,nsh2atm,nsh2ao,xyz,ss,xiozetaa,zeta,&
     nshell,nshellp,nprimp,natom)
  use physical_constants_class

  implicit real(8) (a-h,o-z)
  integer,intent(in)::nao

  integer,intent(in)::nshell,nshellp,nprimp,natom
  integer,intent(in)::ishp(6),npsh(2,nshellp),npsh2pr(nshellp+1)
  integer,intent(in)::nsh2atm(nshell),nsh2ao(nshell)
  real(8),intent(in)::ss(nprimp),xiozetaa(nprimp),zeta(nprimp)
  real(8),intent(in)::xyz(3,natom)

  real(8),intent(inout)::g(3,natom)
  real(8),intent(in)::dm(nao,nao)

  real(8)::v(6,3)

!$omp do schedule(dynamic)
  do ij = ishp(3)+1, ishp(4)
     i = npsh(1,ij)
     j = npsh(2,ij)
     iatom = nsh2atm(i)
     jatom = nsh2atm(j)
     Ax = xyz(1,iatom)
     Ay = xyz(2,iatom)
     Az = xyz(3,iatom)
     Bx = xyz(1,jatom)
     By = xyz(2,jatom)
     Bz = xyz(3,jatom)
     ABx = Ax - Bx
     ABy = Ay - By
     ABz = Az - Bz

o_dxxs_dAx = 0.0d0
o_dyys_dAx = 0.0d0
o_dzzs_dAx = 0.0d0
o_dxys_dAx = 0.0d0
o_dxzs_dAx = 0.0d0
o_dyzs_dAx = 0.0d0
o_dxxs_dAy = 0.0d0
o_dyys_dAy = 0.0d0
o_dzzs_dAy = 0.0d0
o_dxys_dAy = 0.0d0
o_dxzs_dAy = 0.0d0
o_dyzs_dAy = 0.0d0
o_dxxs_dAz = 0.0d0
o_dyys_dAz = 0.0d0
o_dzzs_dAz = 0.0d0
o_dxys_dAz = 0.0d0
o_dxzs_dAz = 0.0d0
o_dyzs_dAz = 0.0d0
do irs = npsh2pr(ij), npsh2pr(ij+1) - 1
o_ss = ss(irs)
zeta_ = zeta(irs)
xioza = xiozetaa(irs)
PAx = -ABx*xioza
PAy = -ABy*xioza
PAz = -ABz*xioza
o_pxs = PAx*o_ss
o_pys = PAy*o_ss
o_pzs = PAz*o_ss
oo2zeta = 0.5d0/zeta_
ai_o_ss = oo2zeta*o_ss
o_dxxs = PAx*o_pxs + ai_o_ss
o_dyys = PAy*o_pys + ai_o_ss
o_dzzs = PAz*o_pzs + ai_o_ss
o_dxys = PAy*o_pxs
o_dxzs = PAz*o_pxs
o_dyzs = PAz*o_pys
ai_o_pxs = oo2zeta*o_pxs
o_fxxxs = PAx*o_dxxs + 2.0d0*ai_o_pxs
ai_o_pys = oo2zeta*o_pys
o_fyyys = PAy*o_dyys + 2.0d0*ai_o_pys
ai_o_pzs = oo2zeta*o_pzs
o_fzzzs = PAz*o_dzzs + 2.0d0*ai_o_pzs
o_fxxys = PAy*o_dxxs
o_fxxzs = PAz*o_dxxs
o_fyyxs = PAx*o_dyys
o_fyyzs = PAz*o_dyys
o_fzzxs = PAx*o_dzzs
o_fzzys = PAy*o_dzzs
o_fxyzs = PAz*o_dxys
beta = xioza*zeta_
alpha = zeta_ - beta
alpha2 = 2.0d0*alpha
o_dxxs_dAx = o_dxxs_dAx + alpha2*o_fxxxs - 2.0d0*o_pxs
o_dyys_dAx = o_dyys_dAx + alpha2*o_fyyxs
o_dzzs_dAx = o_dzzs_dAx + alpha2*o_fzzxs
o_dxys_dAx = o_dxys_dAx + alpha2*o_fxxys - o_pys
o_dxzs_dAx = o_dxzs_dAx + alpha2*o_fxxzs - o_pzs
o_dyzs_dAx = o_dyzs_dAx + alpha2*o_fxyzs
o_dxxs_dAy = o_dxxs_dAy + alpha2*o_fxxys
o_dyys_dAy = o_dyys_dAy + alpha2*o_fyyys - 2.0d0*o_pys
o_dzzs_dAy = o_dzzs_dAy + alpha2*o_fzzys
o_dxys_dAy = o_dxys_dAy + alpha2*o_fyyxs - o_pxs
o_dxzs_dAy = o_dxzs_dAy + alpha2*o_fxyzs
o_dyzs_dAy = o_dyzs_dAy + alpha2*o_fyyzs - o_pzs
o_dxxs_dAz = o_dxxs_dAz + alpha2*o_fxxzs
o_dyys_dAz = o_dyys_dAz + alpha2*o_fyyzs
o_dzzs_dAz = o_dzzs_dAz + alpha2*o_fzzzs - 2.0d0*o_pzs
o_dxys_dAz = o_dxys_dAz + alpha2*o_fxyzs
o_dxzs_dAz = o_dxzs_dAz + alpha2*o_fzzxs - o_pxs
o_dyzs_dAz = o_dyzs_dAz + alpha2*o_fzzys - o_pys
end do
v(1,1) = o_dxxs_dAx
v(2,1) = o_dyys_dAx
v(3,1) = o_dzzs_dAx
v(4,1) = o_dxys_dAx*root3
v(5,1) = o_dxzs_dAx*root3
v(6,1) = o_dyzs_dAx*root3
v(1,2) = o_dxxs_dAy
v(2,2) = o_dyys_dAy
v(3,2) = o_dzzs_dAy
v(4,2) = o_dxys_dAy*root3
v(5,2) = o_dxzs_dAy*root3
v(6,2) = o_dyzs_dAy*root3
v(1,3) = o_dxxs_dAz
v(2,3) = o_dyys_dAz
v(3,3) = o_dzzs_dAz
v(4,3) = o_dxys_dAz*root3
v(5,3) = o_dxzs_dAz*root3
v(6,3) = o_dyzs_dAz*root3

     i = npsh(1,ij)
     j = npsh(2,ij)
     iao = nsh2ao(i)
     jao = nsh2ao(j)

     if (i<j) then
        do ixyz = 1, 3
           do ix = iao,iao + 6-1
              key = ix-iao + 1
              g(ixyz,iatom) = g(ixyz,iatom) + dm(jao,ix)*v(key,ixyz)
           end do
        end do
        do ixyz = 1, 3
           do ix = iao,iao + 6-1
              key = ix-iao + 1
              g(ixyz,jatom) = g(ixyz,jatom) - dm(jao,ix)*v(key,ixyz)
           end do
        end do
     else
        do ixyz = 1, 3
           do ix = iao,iao + 6-1
              key = ix-iao + 1
              g(ixyz,iatom) = g(ixyz,iatom) + dm(ix,jao)*v(key,ixyz)
           end do
        end do
        do ixyz = 1, 3
           do ix = iao,iao + 6-1
              key = ix-iao + 1
              g(ixyz,jatom) = g(ixyz,jatom) - dm(ix,jao)*v(key,ixyz)
           end do
        end do
     end if
  end do
!$omp end do nowait
end subroutine os_dovi_ds_t1

!------------------------------------------------------------------------------
!> dp overlap integrals
!
subroutine os_dovi_dp_t1(&
     g,dm,nao,&
     ishp,npsh,npsh2pr,nsh2atm,nsh2ao,xyz,ss,xiozetaa,zeta,&
     nshell,nshellp,nprimp,natom)
  use physical_constants_class

  implicit real(8) (a-h,o-z)
  integer,intent(in)::nao

  integer,intent(in)::nshell,nshellp,nprimp,natom
  integer,intent(in)::ishp(6),npsh(2,nshellp),npsh2pr(nshellp+1)
  integer,intent(in)::nsh2atm(nshell),nsh2ao(nshell)
  real(8),intent(in)::ss(nprimp),xiozetaa(nprimp),zeta(nprimp)
  real(8),intent(in)::xyz(3,natom)

  real(8),intent(inout)::g(3,natom)
  real(8),intent(in)::dm(nao,nao)

  real(8)::v(18,3)

!$omp do schedule(dynamic)
  do ij = ishp(4)+1, ishp(5)
     i = npsh(1,ij)
     j = npsh(2,ij)
     iatom = nsh2atm(i)
     jatom = nsh2atm(j)
     Ax = xyz(1,iatom)
     Ay = xyz(2,iatom)
     Az = xyz(3,iatom)
     Bx = xyz(1,jatom)
     By = xyz(2,jatom)
     Bz = xyz(3,jatom)
     ABx = Ax - Bx
     ABy = Ay - By
     ABz = Az - Bz

o_dxxpx_dAx = 0.0d0
o_dxxpy_dAx = 0.0d0
o_dxxpz_dAx = 0.0d0
o_dyypx_dAx = 0.0d0
o_dyypy_dAx = 0.0d0
o_dyypz_dAx = 0.0d0
o_dzzpx_dAx = 0.0d0
o_dzzpy_dAx = 0.0d0
o_dzzpz_dAx = 0.0d0
o_dxypx_dAx = 0.0d0
o_dxypy_dAx = 0.0d0
o_dxypz_dAx = 0.0d0
o_dxzpx_dAx = 0.0d0
o_dxzpy_dAx = 0.0d0
o_dxzpz_dAx = 0.0d0
o_dyzpx_dAx = 0.0d0
o_dyzpy_dAx = 0.0d0
o_dyzpz_dAx = 0.0d0
o_dxxpx_dAy = 0.0d0
o_dxxpy_dAy = 0.0d0
o_dxxpz_dAy = 0.0d0
o_dyypx_dAy = 0.0d0
o_dyypy_dAy = 0.0d0
o_dyypz_dAy = 0.0d0
o_dzzpx_dAy = 0.0d0
o_dzzpy_dAy = 0.0d0
o_dzzpz_dAy = 0.0d0
o_dxypx_dAy = 0.0d0
o_dxypy_dAy = 0.0d0
o_dxypz_dAy = 0.0d0
o_dxzpx_dAy = 0.0d0
o_dxzpy_dAy = 0.0d0
o_dxzpz_dAy = 0.0d0
o_dyzpx_dAy = 0.0d0
o_dyzpy_dAy = 0.0d0
o_dyzpz_dAy = 0.0d0
o_dxxpx_dAz = 0.0d0
o_dxxpy_dAz = 0.0d0
o_dxxpz_dAz = 0.0d0
o_dyypx_dAz = 0.0d0
o_dyypy_dAz = 0.0d0
o_dyypz_dAz = 0.0d0
o_dzzpx_dAz = 0.0d0
o_dzzpy_dAz = 0.0d0
o_dzzpz_dAz = 0.0d0
o_dxypx_dAz = 0.0d0
o_dxypy_dAz = 0.0d0
o_dxypz_dAz = 0.0d0
o_dxzpx_dAz = 0.0d0
o_dxzpy_dAz = 0.0d0
o_dxzpz_dAz = 0.0d0
o_dyzpx_dAz = 0.0d0
o_dyzpy_dAz = 0.0d0
o_dyzpz_dAz = 0.0d0
do irs = npsh2pr(ij), npsh2pr(ij+1) - 1
o_ss = ss(irs)
zeta_ = zeta(irs)
xioza = xiozetaa(irs)
PAx = -ABx*xioza
PAy = -ABy*xioza
PAz = -ABz*xioza
o_pxs = PAx*o_ss
o_pys = PAy*o_ss
o_pzs = PAz*o_ss
oo2zeta = 0.5d0/zeta_
ai_o_ss = oo2zeta*o_ss
o_dxxs = PAx*o_pxs + ai_o_ss
o_dyys = PAy*o_pys + ai_o_ss
o_dzzs = PAz*o_pzs + ai_o_ss
o_dxys = PAy*o_pxs
o_dxzs = PAz*o_pxs
o_dyzs = PAz*o_pys
ai_o_pxs = oo2zeta*o_pxs
o_fxxxs = PAx*o_dxxs + 2.0d0*ai_o_pxs
ai_o_pys = oo2zeta*o_pys
o_fyyys = PAy*o_dyys + 2.0d0*ai_o_pys
ai_o_pzs = oo2zeta*o_pzs
o_fzzzs = PAz*o_dzzs + 2.0d0*ai_o_pzs
o_fxxys = PAy*o_dxxs
o_fxxzs = PAz*o_dxxs
o_fyyxs = PAx*o_dyys
o_fyyzs = PAz*o_dyys
o_fzzxs = PAx*o_dzzs
o_fzzys = PAy*o_dzzs
o_fxyzs = PAz*o_dxys
PBx = PAx + ABx
PBy = PAy + ABy
PBz = PAz + ABz
ai_o_dxxs = oo2zeta*o_dxxs
o_fxxxpx = PBx*o_fxxxs + 3.0d0*ai_o_dxxs
o_fxxxpy = PBy*o_fxxxs
o_fxxxpz = PBz*o_fxxxs
o_fyyypx = PBx*o_fyyys
ai_o_dyys = oo2zeta*o_dyys
o_fyyypy = PBy*o_fyyys + 3.0d0*ai_o_dyys
o_fyyypz = PBz*o_fyyys
o_fzzzpx = PBx*o_fzzzs
o_fzzzpy = PBy*o_fzzzs
ai_o_dzzs = oo2zeta*o_dzzs
o_fzzzpz = PBz*o_fzzzs + 3.0d0*ai_o_dzzs
ai_o_dxys = oo2zeta*o_dxys
o_fxxypx = PBx*o_fxxys + 2.0d0*ai_o_dxys
o_fxxypy = PBy*o_fxxys + ai_o_dxxs
o_fxxypz = PBz*o_fxxys
ai_o_dxzs = oo2zeta*o_dxzs
o_fxxzpx = PBx*o_fxxzs + 2.0d0*ai_o_dxzs
o_fxxzpy = PBy*o_fxxzs
o_fxxzpz = PBz*o_fxxzs + ai_o_dxxs
o_fyyxpx = PBx*o_fyyxs + ai_o_dyys
o_fyyxpy = PBy*o_fyyxs + 2.0d0*ai_o_dxys
o_fyyxpz = PBz*o_fyyxs
o_fyyzpx = PBx*o_fyyzs
ai_o_dyzs = oo2zeta*o_dyzs
o_fyyzpy = PBy*o_fyyzs + 2.0d0*ai_o_dyzs
o_fyyzpz = PBz*o_fyyzs + ai_o_dyys
o_fzzxpx = PBx*o_fzzxs + ai_o_dzzs
o_fzzxpy = PBy*o_fzzxs
o_fzzxpz = PBz*o_fzzxs + 2.0d0*ai_o_dxzs
o_fzzypx = PBx*o_fzzys
o_fzzypy = PBy*o_fzzys + ai_o_dzzs
o_fzzypz = PBz*o_fzzys + 2.0d0*ai_o_dyzs
o_fxyzpx = PBx*o_fxyzs + ai_o_dyzs
o_fxyzpy = PBy*o_fxyzs + ai_o_dxzs
o_fxyzpz = PBz*o_fxyzs + ai_o_dxys
o_pxpx = PBx*o_pxs + ai_o_ss
o_pxpy = PBy*o_pxs
o_pxpz = PBz*o_pxs
o_pypx = PBx*o_pys
o_pypy = PBy*o_pys + ai_o_ss
o_pypz = PBz*o_pys
o_pzpx = PBx*o_pzs
o_pzpy = PBy*o_pzs
o_pzpz = PBz*o_pzs + ai_o_ss
beta = xioza*zeta_
alpha = zeta_ - beta
alpha2 = 2.0d0*alpha
o_dxxpx_dAx = o_dxxpx_dAx + alpha2*o_fxxxpx - 2.0d0*o_pxpx
o_dxxpy_dAx = o_dxxpy_dAx + alpha2*o_fxxxpy - 2.0d0*o_pxpy
o_dxxpz_dAx = o_dxxpz_dAx + alpha2*o_fxxxpz - 2.0d0*o_pxpz
o_dyypx_dAx = o_dyypx_dAx + alpha2*o_fyyxpx
o_dyypy_dAx = o_dyypy_dAx + alpha2*o_fyyxpy
o_dyypz_dAx = o_dyypz_dAx + alpha2*o_fyyxpz
o_dzzpx_dAx = o_dzzpx_dAx + alpha2*o_fzzxpx
o_dzzpy_dAx = o_dzzpy_dAx + alpha2*o_fzzxpy
o_dzzpz_dAx = o_dzzpz_dAx + alpha2*o_fzzxpz
o_dxypx_dAx = o_dxypx_dAx + alpha2*o_fxxypx - o_pypx
o_dxypy_dAx = o_dxypy_dAx + alpha2*o_fxxypy - o_pypy
o_dxypz_dAx = o_dxypz_dAx + alpha2*o_fxxypz - o_pypz
o_dxzpx_dAx = o_dxzpx_dAx + alpha2*o_fxxzpx - o_pzpx
o_dxzpy_dAx = o_dxzpy_dAx + alpha2*o_fxxzpy - o_pzpy
o_dxzpz_dAx = o_dxzpz_dAx + alpha2*o_fxxzpz - o_pzpz
o_dyzpx_dAx = o_dyzpx_dAx + alpha2*o_fxyzpx
o_dyzpy_dAx = o_dyzpy_dAx + alpha2*o_fxyzpy
o_dyzpz_dAx = o_dyzpz_dAx + alpha2*o_fxyzpz
o_dxxpx_dAy = o_dxxpx_dAy + alpha2*o_fxxypx
o_dxxpy_dAy = o_dxxpy_dAy + alpha2*o_fxxypy
o_dxxpz_dAy = o_dxxpz_dAy + alpha2*o_fxxypz
o_dyypx_dAy = o_dyypx_dAy + alpha2*o_fyyypx - 2.0d0*o_pypx
o_dyypy_dAy = o_dyypy_dAy + alpha2*o_fyyypy - 2.0d0*o_pypy
o_dyypz_dAy = o_dyypz_dAy + alpha2*o_fyyypz - 2.0d0*o_pypz
o_dzzpx_dAy = o_dzzpx_dAy + alpha2*o_fzzypx
o_dzzpy_dAy = o_dzzpy_dAy + alpha2*o_fzzypy
o_dzzpz_dAy = o_dzzpz_dAy + alpha2*o_fzzypz
o_dxypx_dAy = o_dxypx_dAy + alpha2*o_fyyxpx - o_pxpx
o_dxypy_dAy = o_dxypy_dAy + alpha2*o_fyyxpy - o_pxpy
o_dxypz_dAy = o_dxypz_dAy + alpha2*o_fyyxpz - o_pxpz
o_dxzpx_dAy = o_dxzpx_dAy + alpha2*o_fxyzpx
o_dxzpy_dAy = o_dxzpy_dAy + alpha2*o_fxyzpy
o_dxzpz_dAy = o_dxzpz_dAy + alpha2*o_fxyzpz
o_dyzpx_dAy = o_dyzpx_dAy + alpha2*o_fyyzpx - o_pzpx
o_dyzpy_dAy = o_dyzpy_dAy + alpha2*o_fyyzpy - o_pzpy
o_dyzpz_dAy = o_dyzpz_dAy + alpha2*o_fyyzpz - o_pzpz
o_dxxpx_dAz = o_dxxpx_dAz + alpha2*o_fxxzpx
o_dxxpy_dAz = o_dxxpy_dAz + alpha2*o_fxxzpy
o_dxxpz_dAz = o_dxxpz_dAz + alpha2*o_fxxzpz
o_dyypx_dAz = o_dyypx_dAz + alpha2*o_fyyzpx
o_dyypy_dAz = o_dyypy_dAz + alpha2*o_fyyzpy
o_dyypz_dAz = o_dyypz_dAz + alpha2*o_fyyzpz
o_dzzpx_dAz = o_dzzpx_dAz + alpha2*o_fzzzpx - 2.0d0*o_pzpx
o_dzzpy_dAz = o_dzzpy_dAz + alpha2*o_fzzzpy - 2.0d0*o_pzpy
o_dzzpz_dAz = o_dzzpz_dAz + alpha2*o_fzzzpz - 2.0d0*o_pzpz
o_dxypx_dAz = o_dxypx_dAz + alpha2*o_fxyzpx
o_dxypy_dAz = o_dxypy_dAz + alpha2*o_fxyzpy
o_dxypz_dAz = o_dxypz_dAz + alpha2*o_fxyzpz
o_dxzpx_dAz = o_dxzpx_dAz + alpha2*o_fzzxpx - o_pxpx
o_dxzpy_dAz = o_dxzpy_dAz + alpha2*o_fzzxpy - o_pxpy
o_dxzpz_dAz = o_dxzpz_dAz + alpha2*o_fzzxpz - o_pxpz
o_dyzpx_dAz = o_dyzpx_dAz + alpha2*o_fzzypx - o_pypx
o_dyzpy_dAz = o_dyzpy_dAz + alpha2*o_fzzypy - o_pypy
o_dyzpz_dAz = o_dyzpz_dAz + alpha2*o_fzzypz - o_pypz
end do
v(1,1) = o_dxxpx_dAx
v(2,1) = o_dxxpy_dAx
v(3,1) = o_dxxpz_dAx
v(4,1) = o_dyypx_dAx
v(5,1) = o_dyypy_dAx
v(6,1) = o_dyypz_dAx
v(7,1) = o_dzzpx_dAx
v(8,1) = o_dzzpy_dAx
v(9,1) = o_dzzpz_dAx
v(10,1) = o_dxypx_dAx*root3
v(11,1) = o_dxypy_dAx*root3
v(12,1) = o_dxypz_dAx*root3
v(13,1) = o_dxzpx_dAx*root3
v(14,1) = o_dxzpy_dAx*root3
v(15,1) = o_dxzpz_dAx*root3
v(16,1) = o_dyzpx_dAx*root3
v(17,1) = o_dyzpy_dAx*root3
v(18,1) = o_dyzpz_dAx*root3
v(1,2) = o_dxxpx_dAy
v(2,2) = o_dxxpy_dAy
v(3,2) = o_dxxpz_dAy
v(4,2) = o_dyypx_dAy
v(5,2) = o_dyypy_dAy
v(6,2) = o_dyypz_dAy
v(7,2) = o_dzzpx_dAy
v(8,2) = o_dzzpy_dAy
v(9,2) = o_dzzpz_dAy
v(10,2) = o_dxypx_dAy*root3
v(11,2) = o_dxypy_dAy*root3
v(12,2) = o_dxypz_dAy*root3
v(13,2) = o_dxzpx_dAy*root3
v(14,2) = o_dxzpy_dAy*root3
v(15,2) = o_dxzpz_dAy*root3
v(16,2) = o_dyzpx_dAy*root3
v(17,2) = o_dyzpy_dAy*root3
v(18,2) = o_dyzpz_dAy*root3
v(1,3) = o_dxxpx_dAz
v(2,3) = o_dxxpy_dAz
v(3,3) = o_dxxpz_dAz
v(4,3) = o_dyypx_dAz
v(5,3) = o_dyypy_dAz
v(6,3) = o_dyypz_dAz
v(7,3) = o_dzzpx_dAz
v(8,3) = o_dzzpy_dAz
v(9,3) = o_dzzpz_dAz
v(10,3) = o_dxypx_dAz*root3
v(11,3) = o_dxypy_dAz*root3
v(12,3) = o_dxypz_dAz*root3
v(13,3) = o_dxzpx_dAz*root3
v(14,3) = o_dxzpy_dAz*root3
v(15,3) = o_dxzpz_dAz*root3
v(16,3) = o_dyzpx_dAz*root3
v(17,3) = o_dyzpy_dAz*root3
v(18,3) = o_dyzpz_dAz*root3

     i = npsh(1,ij)
     j = npsh(2,ij)
     iao = nsh2ao(i)
     jao = nsh2ao(j)

     if (i < j) then
        do ixyz = 1, 3
           do jx = jao,jao + 3-1
              do ix = iao,iao + 6-1
                 key = ( ix-iao ) * 3 + jx-jao +1
                 g(ixyz,iatom) = g(ixyz,iatom) + dm(jx,ix)*v(key,ixyz)
              end do
           end do
        end do
        do ixyz = 1, 3
           do jx = jao,jao + 3-1
              do ix = iao,iao + 6-1
                 key = ( ix-iao ) * 3 + jx-jao +1
                 g(ixyz,jatom) = g(ixyz,jatom) - dm(jx,ix)*v(key,ixyz)
              end do
           end do
        end do
     else
        do ixyz = 1, 3
           do jx = jao,jao + 3-1
              do ix = iao,iao + 6-1
                 key = ( ix-iao ) * 3 + jx-jao +1
                 g(ixyz,iatom) = g(ixyz,iatom) + dm(ix,jx)*v(key,ixyz)
              end do
           end do
        end do
        do ixyz = 1, 3
           do jx = jao,jao + 3-1
              do ix = iao,iao + 6-1
                 key = ( ix-iao ) * 3 + jx-jao +1
                 g(ixyz,jatom) = g(ixyz,jatom) - dm(ix,jx)*v(key,ixyz)
              end do
           end do
        end do
     end if
  end do
!$omp end do nowait
end subroutine os_dovi_dp_t1

!------------------------------------------------------------------------------
!> dd overlap integrals
!
subroutine os_dovi_dd_t1(&
     g,dm,nao,&
     ishp,npsh,npsh2pr,nsh2atm,nsh2ao,xyz,ss,xiozetaa,zeta,&
     nshell,nshellp,nprimp,natom)
  use physical_constants_class

  implicit real(8) (a-h,o-z)
  integer,intent(in)::nao

  integer,intent(in)::nshell,nshellp,nprimp,natom
  integer,intent(in)::ishp(6),npsh(2,nshellp),npsh2pr(nshellp+1)
  integer,intent(in)::nsh2atm(nshell),nsh2ao(nshell)
  real(8),intent(in)::ss(nprimp),xiozetaa(nprimp),zeta(nprimp)
  real(8),intent(in)::xyz(3,natom)

  real(8),intent(inout)::g(3,natom)
  real(8),intent(in)::dm(nao,nao)

  real(8)::v(36,3)

!$omp do schedule(dynamic)
  do ij = ishp(5)+1, ishp(6)
     i = npsh(1,ij)
     j = npsh(2,ij)
     iatom = nsh2atm(i)
     jatom = nsh2atm(j)
     Ax = xyz(1,iatom)
     Ay = xyz(2,iatom)
     Az = xyz(3,iatom)
     Bx = xyz(1,jatom)
     By = xyz(2,jatom)
     Bz = xyz(3,jatom)
     ABx = Ax - Bx
     ABy = Ay - By
     ABz = Az - Bz

o_dxxdxx_dAx = 0.0d0
o_dxxdyy_dAx = 0.0d0
o_dxxdzz_dAx = 0.0d0
o_dxxdxy_dAx = 0.0d0
o_dxxdxz_dAx = 0.0d0
o_dxxdyz_dAx = 0.0d0
o_dyydxx_dAx = 0.0d0
o_dyydyy_dAx = 0.0d0
o_dyydzz_dAx = 0.0d0
o_dyydxy_dAx = 0.0d0
o_dyydxz_dAx = 0.0d0
o_dyydyz_dAx = 0.0d0
o_dzzdxx_dAx = 0.0d0
o_dzzdyy_dAx = 0.0d0
o_dzzdzz_dAx = 0.0d0
o_dzzdxy_dAx = 0.0d0
o_dzzdxz_dAx = 0.0d0
o_dzzdyz_dAx = 0.0d0
o_dxydxx_dAx = 0.0d0
o_dxydyy_dAx = 0.0d0
o_dxydzz_dAx = 0.0d0
o_dxydxy_dAx = 0.0d0
o_dxydxz_dAx = 0.0d0
o_dxydyz_dAx = 0.0d0
o_dxzdxx_dAx = 0.0d0
o_dxzdyy_dAx = 0.0d0
o_dxzdzz_dAx = 0.0d0
o_dxzdxy_dAx = 0.0d0
o_dxzdxz_dAx = 0.0d0
o_dxzdyz_dAx = 0.0d0
o_dyzdxx_dAx = 0.0d0
o_dyzdyy_dAx = 0.0d0
o_dyzdzz_dAx = 0.0d0
o_dyzdxy_dAx = 0.0d0
o_dyzdxz_dAx = 0.0d0
o_dyzdyz_dAx = 0.0d0
o_dxxdxx_dAy = 0.0d0
o_dxxdyy_dAy = 0.0d0
o_dxxdzz_dAy = 0.0d0
o_dxxdxy_dAy = 0.0d0
o_dxxdxz_dAy = 0.0d0
o_dxxdyz_dAy = 0.0d0
o_dyydxx_dAy = 0.0d0
o_dyydyy_dAy = 0.0d0
o_dyydzz_dAy = 0.0d0
o_dyydxy_dAy = 0.0d0
o_dyydxz_dAy = 0.0d0
o_dyydyz_dAy = 0.0d0
o_dzzdxx_dAy = 0.0d0
o_dzzdyy_dAy = 0.0d0
o_dzzdzz_dAy = 0.0d0
o_dzzdxy_dAy = 0.0d0
o_dzzdxz_dAy = 0.0d0
o_dzzdyz_dAy = 0.0d0
o_dxydxx_dAy = 0.0d0
o_dxydyy_dAy = 0.0d0
o_dxydzz_dAy = 0.0d0
o_dxydxy_dAy = 0.0d0
o_dxydxz_dAy = 0.0d0
o_dxydyz_dAy = 0.0d0
o_dxzdxx_dAy = 0.0d0
o_dxzdyy_dAy = 0.0d0
o_dxzdzz_dAy = 0.0d0
o_dxzdxy_dAy = 0.0d0
o_dxzdxz_dAy = 0.0d0
o_dxzdyz_dAy = 0.0d0
o_dyzdxx_dAy = 0.0d0
o_dyzdyy_dAy = 0.0d0
o_dyzdzz_dAy = 0.0d0
o_dyzdxy_dAy = 0.0d0
o_dyzdxz_dAy = 0.0d0
o_dyzdyz_dAy = 0.0d0
o_dxxdxx_dAz = 0.0d0
o_dxxdyy_dAz = 0.0d0
o_dxxdzz_dAz = 0.0d0
o_dxxdxy_dAz = 0.0d0
o_dxxdxz_dAz = 0.0d0
o_dxxdyz_dAz = 0.0d0
o_dyydxx_dAz = 0.0d0
o_dyydyy_dAz = 0.0d0
o_dyydzz_dAz = 0.0d0
o_dyydxy_dAz = 0.0d0
o_dyydxz_dAz = 0.0d0
o_dyydyz_dAz = 0.0d0
o_dzzdxx_dAz = 0.0d0
o_dzzdyy_dAz = 0.0d0
o_dzzdzz_dAz = 0.0d0
o_dzzdxy_dAz = 0.0d0
o_dzzdxz_dAz = 0.0d0
o_dzzdyz_dAz = 0.0d0
o_dxydxx_dAz = 0.0d0
o_dxydyy_dAz = 0.0d0
o_dxydzz_dAz = 0.0d0
o_dxydxy_dAz = 0.0d0
o_dxydxz_dAz = 0.0d0
o_dxydyz_dAz = 0.0d0
o_dxzdxx_dAz = 0.0d0
o_dxzdyy_dAz = 0.0d0
o_dxzdzz_dAz = 0.0d0
o_dxzdxy_dAz = 0.0d0
o_dxzdxz_dAz = 0.0d0
o_dxzdyz_dAz = 0.0d0
o_dyzdxx_dAz = 0.0d0
o_dyzdyy_dAz = 0.0d0
o_dyzdzz_dAz = 0.0d0
o_dyzdxy_dAz = 0.0d0
o_dyzdxz_dAz = 0.0d0
o_dyzdyz_dAz = 0.0d0
do irs = npsh2pr(ij), npsh2pr(ij+1) - 1
o_ss = ss(irs)
zeta_ = zeta(irs)
xioza = xiozetaa(irs)
PAx = -ABx*xioza
PAy = -ABy*xioza
PAz = -ABz*xioza
o_pxs = PAx*o_ss
o_pys = PAy*o_ss
o_pzs = PAz*o_ss
oo2zeta = 0.5d0/zeta_
ai_o_ss = oo2zeta*o_ss
o_dxxs = PAx*o_pxs + ai_o_ss
o_dyys = PAy*o_pys + ai_o_ss
o_dzzs = PAz*o_pzs + ai_o_ss
o_dxys = PAy*o_pxs
o_dxzs = PAz*o_pxs
o_dyzs = PAz*o_pys
ai_o_pxs = oo2zeta*o_pxs
o_fxxxs = PAx*o_dxxs + 2.0d0*ai_o_pxs
ai_o_pys = oo2zeta*o_pys
o_fyyys = PAy*o_dyys + 2.0d0*ai_o_pys
ai_o_pzs = oo2zeta*o_pzs
o_fzzzs = PAz*o_dzzs + 2.0d0*ai_o_pzs
o_fxxys = PAy*o_dxxs
o_fxxzs = PAz*o_dxxs
o_fyyxs = PAx*o_dyys
o_fyyzs = PAz*o_dyys
o_fzzxs = PAx*o_dzzs
o_fzzys = PAy*o_dzzs
o_fxyzs = PAz*o_dxys
PBx = PAx + ABx
PBy = PAy + ABy
PBz = PAz + ABz
ai_o_dxxs = oo2zeta*o_dxxs
o_fxxxpx = PBx*o_fxxxs + 3.0d0*ai_o_dxxs
o_fxxxpy = PBy*o_fxxxs
o_fxxxpz = PBz*o_fxxxs
o_fyyypx = PBx*o_fyyys
ai_o_dyys = oo2zeta*o_dyys
o_fyyypy = PBy*o_fyyys + 3.0d0*ai_o_dyys
o_fyyypz = PBz*o_fyyys
o_fzzzpx = PBx*o_fzzzs
o_fzzzpy = PBy*o_fzzzs
ai_o_dzzs = oo2zeta*o_dzzs
o_fzzzpz = PBz*o_fzzzs + 3.0d0*ai_o_dzzs
ai_o_dxys = oo2zeta*o_dxys
o_fxxypx = PBx*o_fxxys + 2.0d0*ai_o_dxys
o_fxxypy = PBy*o_fxxys + ai_o_dxxs
o_fxxypz = PBz*o_fxxys
ai_o_dxzs = oo2zeta*o_dxzs
o_fxxzpx = PBx*o_fxxzs + 2.0d0*ai_o_dxzs
o_fxxzpy = PBy*o_fxxzs
o_fxxzpz = PBz*o_fxxzs + ai_o_dxxs
o_fyyxpx = PBx*o_fyyxs + ai_o_dyys
o_fyyxpy = PBy*o_fyyxs + 2.0d0*ai_o_dxys
o_fyyxpz = PBz*o_fyyxs
o_fyyzpx = PBx*o_fyyzs
ai_o_dyzs = oo2zeta*o_dyzs
o_fyyzpy = PBy*o_fyyzs + 2.0d0*ai_o_dyzs
o_fyyzpz = PBz*o_fyyzs + ai_o_dyys
o_fzzxpx = PBx*o_fzzxs + ai_o_dzzs
o_fzzxpy = PBy*o_fzzxs
o_fzzxpz = PBz*o_fzzxs + 2.0d0*ai_o_dxzs
o_fzzypx = PBx*o_fzzys
o_fzzypy = PBy*o_fzzys + ai_o_dzzs
o_fzzypz = PBz*o_fzzys + 2.0d0*ai_o_dyzs
o_fxyzpx = PBx*o_fxyzs + ai_o_dyzs
o_fxyzpy = PBy*o_fxyzs + ai_o_dxzs
o_fxyzpz = PBz*o_fxyzs + ai_o_dxys
o_dxxpx = PBx*o_dxxs + 2.0d0*ai_o_pxs
o_dxxpy = PBy*o_dxxs
o_dxxpz = PBz*o_dxxs
o_dyypx = PBx*o_dyys
o_dyypy = PBy*o_dyys + 2.0d0*ai_o_pys
o_dyypz = PBz*o_dyys
o_dzzpx = PBx*o_dzzs
o_dzzpy = PBy*o_dzzs
o_dzzpz = PBz*o_dzzs + 2.0d0*ai_o_pzs
o_dxypx = PBx*o_dxys + ai_o_pys
o_dxypy = PBy*o_dxys + ai_o_pxs
o_dxypz = PBz*o_dxys
o_dxzpx = PBx*o_dxzs + ai_o_pzs
o_dxzpy = PBy*o_dxzs
o_dxzpz = PBz*o_dxzs + ai_o_pxs
o_dyzpx = PBx*o_dyzs
o_dyzpy = PBy*o_dyzs + ai_o_pzs
o_dyzpz = PBz*o_dyzs + ai_o_pys
ai_o_dxxpx = oo2zeta*o_dxxpx
bi_o_fxxxs = oo2zeta*o_fxxxs
o_fxxxdxx = PBx*o_fxxxpx + 3.0d0*ai_o_dxxpx + bi_o_fxxxs
o_fxxxdyy = PBy*o_fxxxpy + bi_o_fxxxs
o_fxxxdzz = PBz*o_fxxxpz + bi_o_fxxxs
o_fxxxdxy = PBy*o_fxxxpx
o_fxxxdxz = PBz*o_fxxxpx
o_fxxxdyz = PBz*o_fxxxpy
bi_o_fyyys = oo2zeta*o_fyyys
o_fyyydxx = PBx*o_fyyypx + bi_o_fyyys
ai_o_dyypy = oo2zeta*o_dyypy
o_fyyydyy = PBy*o_fyyypy + 3.0d0*ai_o_dyypy + bi_o_fyyys
o_fyyydzz = PBz*o_fyyypz + bi_o_fyyys
ai_o_dyypx = oo2zeta*o_dyypx
o_fyyydxy = PBy*o_fyyypx + 3.0d0*ai_o_dyypx
o_fyyydxz = PBz*o_fyyypx
o_fyyydyz = PBz*o_fyyypy
bi_o_fzzzs = oo2zeta*o_fzzzs
o_fzzzdxx = PBx*o_fzzzpx + bi_o_fzzzs
o_fzzzdyy = PBy*o_fzzzpy + bi_o_fzzzs
ai_o_dzzpz = oo2zeta*o_dzzpz
o_fzzzdzz = PBz*o_fzzzpz + 3.0d0*ai_o_dzzpz + bi_o_fzzzs
o_fzzzdxy = PBy*o_fzzzpx
ai_o_dzzpx = oo2zeta*o_dzzpx
o_fzzzdxz = PBz*o_fzzzpx + 3.0d0*ai_o_dzzpx
ai_o_dzzpy = oo2zeta*o_dzzpy
o_fzzzdyz = PBz*o_fzzzpy + 3.0d0*ai_o_dzzpy
ai_o_dxypx = oo2zeta*o_dxypx
bi_o_fxxys = oo2zeta*o_fxxys
o_fxxydxx = PBx*o_fxxypx + 2.0d0*ai_o_dxypx + bi_o_fxxys
ai_o_dxxpy = oo2zeta*o_dxxpy
o_fxxydyy = PBy*o_fxxypy + ai_o_dxxpy + bi_o_fxxys
o_fxxydzz = PBz*o_fxxypz + bi_o_fxxys
o_fxxydxy = PBy*o_fxxypx + ai_o_dxxpx
o_fxxydxz = PBz*o_fxxypx
o_fxxydyz = PBz*o_fxxypy
ai_o_dxzpx = oo2zeta*o_dxzpx
bi_o_fxxzs = oo2zeta*o_fxxzs
o_fxxzdxx = PBx*o_fxxzpx + 2.0d0*ai_o_dxzpx + bi_o_fxxzs
o_fxxzdyy = PBy*o_fxxzpy + bi_o_fxxzs
ai_o_dxxpz = oo2zeta*o_dxxpz
o_fxxzdzz = PBz*o_fxxzpz + ai_o_dxxpz + bi_o_fxxzs
o_fxxzdxy = PBy*o_fxxzpx
o_fxxzdxz = PBz*o_fxxzpx + ai_o_dxxpx
o_fxxzdyz = PBz*o_fxxzpy + ai_o_dxxpy
bi_o_fyyxs = oo2zeta*o_fyyxs
o_fyyxdxx = PBx*o_fyyxpx + ai_o_dyypx + bi_o_fyyxs
ai_o_dxypy = oo2zeta*o_dxypy
o_fyyxdyy = PBy*o_fyyxpy + 2.0d0*ai_o_dxypy + bi_o_fyyxs
o_fyyxdzz = PBz*o_fyyxpz + bi_o_fyyxs
o_fyyxdxy = PBy*o_fyyxpx + 2.0d0*ai_o_dxypx
o_fyyxdxz = PBz*o_fyyxpx
o_fyyxdyz = PBz*o_fyyxpy
bi_o_fyyzs = oo2zeta*o_fyyzs
o_fyyzdxx = PBx*o_fyyzpx + bi_o_fyyzs
ai_o_dyzpy = oo2zeta*o_dyzpy
o_fyyzdyy = PBy*o_fyyzpy + 2.0d0*ai_o_dyzpy + bi_o_fyyzs
ai_o_dyypz = oo2zeta*o_dyypz
o_fyyzdzz = PBz*o_fyyzpz + ai_o_dyypz + bi_o_fyyzs
ai_o_dyzpx = oo2zeta*o_dyzpx
o_fyyzdxy = PBy*o_fyyzpx + 2.0d0*ai_o_dyzpx
o_fyyzdxz = PBz*o_fyyzpx + ai_o_dyypx
o_fyyzdyz = PBz*o_fyyzpy + ai_o_dyypy
bi_o_fzzxs = oo2zeta*o_fzzxs
o_fzzxdxx = PBx*o_fzzxpx + ai_o_dzzpx + bi_o_fzzxs
o_fzzxdyy = PBy*o_fzzxpy + bi_o_fzzxs
ai_o_dxzpz = oo2zeta*o_dxzpz
o_fzzxdzz = PBz*o_fzzxpz + 2.0d0*ai_o_dxzpz + bi_o_fzzxs
o_fzzxdxy = PBy*o_fzzxpx
o_fzzxdxz = PBz*o_fzzxpx + 2.0d0*ai_o_dxzpx
ai_o_dxzpy = oo2zeta*o_dxzpy
o_fzzxdyz = PBz*o_fzzxpy + 2.0d0*ai_o_dxzpy
bi_o_fzzys = oo2zeta*o_fzzys
o_fzzydxx = PBx*o_fzzypx + bi_o_fzzys
o_fzzydyy = PBy*o_fzzypy + ai_o_dzzpy + bi_o_fzzys
ai_o_dyzpz = oo2zeta*o_dyzpz
o_fzzydzz = PBz*o_fzzypz + 2.0d0*ai_o_dyzpz + bi_o_fzzys
o_fzzydxy = PBy*o_fzzypx + ai_o_dzzpx
o_fzzydxz = PBz*o_fzzypx + 2.0d0*ai_o_dyzpx
o_fzzydyz = PBz*o_fzzypy + 2.0d0*ai_o_dyzpy
bi_o_fxyzs = oo2zeta*o_fxyzs
o_fxyzdxx = PBx*o_fxyzpx + ai_o_dyzpx + bi_o_fxyzs
o_fxyzdyy = PBy*o_fxyzpy + ai_o_dxzpy + bi_o_fxyzs
ai_o_dxypz = oo2zeta*o_dxypz
o_fxyzdzz = PBz*o_fxyzpz + ai_o_dxypz + bi_o_fxyzs
o_fxyzdxy = PBy*o_fxyzpx + ai_o_dxzpx
o_fxyzdxz = PBz*o_fxyzpx + ai_o_dxypx
o_fxyzdyz = PBz*o_fxyzpy + ai_o_dxypy
o_pxpx = PBx*o_pxs + ai_o_ss
o_pxpy = PBy*o_pxs
o_pxpz = PBz*o_pxs
o_pypx = PBx*o_pys
o_pypy = PBy*o_pys + ai_o_ss
o_pypz = PBz*o_pys
o_pzpx = PBx*o_pzs
o_pzpy = PBy*o_pzs
o_pzpz = PBz*o_pzs + ai_o_ss
o_spx = PBx*o_ss
o_spy = PBy*o_ss
o_spz = PBz*o_ss
ai_o_spx = oo2zeta*o_spx
bi_o_pxs = oo2zeta*o_pxs
o_pxdxx = PBx*o_pxpx + ai_o_spx + bi_o_pxs
o_pxdyy = PBy*o_pxpy + bi_o_pxs
o_pxdzz = PBz*o_pxpz + bi_o_pxs
o_pxdxy = PBy*o_pxpx
o_pxdxz = PBz*o_pxpx
o_pxdyz = PBz*o_pxpy
bi_o_pys = oo2zeta*o_pys
o_pydxx = PBx*o_pypx + bi_o_pys
ai_o_spy = oo2zeta*o_spy
o_pydyy = PBy*o_pypy + ai_o_spy + bi_o_pys
o_pydzz = PBz*o_pypz + bi_o_pys
o_pydxy = PBy*o_pypx + ai_o_spx
o_pydxz = PBz*o_pypx
o_pydyz = PBz*o_pypy
bi_o_pzs = oo2zeta*o_pzs
o_pzdxx = PBx*o_pzpx + bi_o_pzs
o_pzdyy = PBy*o_pzpy + bi_o_pzs
ai_o_spz = oo2zeta*o_spz
o_pzdzz = PBz*o_pzpz + ai_o_spz + bi_o_pzs
o_pzdxy = PBy*o_pzpx
o_pzdxz = PBz*o_pzpx + ai_o_spx
o_pzdyz = PBz*o_pzpy + ai_o_spy
beta = xioza*zeta_
alpha = zeta_ - beta
alpha2 = 2.0d0*alpha
o_dxxdxx_dAx = o_dxxdxx_dAx + alpha2*o_fxxxdxx - 2.0d0*o_pxdxx
o_dxxdyy_dAx = o_dxxdyy_dAx + alpha2*o_fxxxdyy - 2.0d0*o_pxdyy
o_dxxdzz_dAx = o_dxxdzz_dAx + alpha2*o_fxxxdzz - 2.0d0*o_pxdzz
o_dxxdxy_dAx = o_dxxdxy_dAx + alpha2*o_fxxxdxy - 2.0d0*o_pxdxy
o_dxxdxz_dAx = o_dxxdxz_dAx + alpha2*o_fxxxdxz - 2.0d0*o_pxdxz
o_dxxdyz_dAx = o_dxxdyz_dAx + alpha2*o_fxxxdyz - 2.0d0*o_pxdyz
o_dyydxx_dAx = o_dyydxx_dAx + alpha2*o_fyyxdxx
o_dyydyy_dAx = o_dyydyy_dAx + alpha2*o_fyyxdyy
o_dyydzz_dAx = o_dyydzz_dAx + alpha2*o_fyyxdzz
o_dyydxy_dAx = o_dyydxy_dAx + alpha2*o_fyyxdxy
o_dyydxz_dAx = o_dyydxz_dAx + alpha2*o_fyyxdxz
o_dyydyz_dAx = o_dyydyz_dAx + alpha2*o_fyyxdyz
o_dzzdxx_dAx = o_dzzdxx_dAx + alpha2*o_fzzxdxx
o_dzzdyy_dAx = o_dzzdyy_dAx + alpha2*o_fzzxdyy
o_dzzdzz_dAx = o_dzzdzz_dAx + alpha2*o_fzzxdzz
o_dzzdxy_dAx = o_dzzdxy_dAx + alpha2*o_fzzxdxy
o_dzzdxz_dAx = o_dzzdxz_dAx + alpha2*o_fzzxdxz
o_dzzdyz_dAx = o_dzzdyz_dAx + alpha2*o_fzzxdyz
o_dxydxx_dAx = o_dxydxx_dAx + alpha2*o_fxxydxx - o_pydxx
o_dxydyy_dAx = o_dxydyy_dAx + alpha2*o_fxxydyy - o_pydyy
o_dxydzz_dAx = o_dxydzz_dAx + alpha2*o_fxxydzz - o_pydzz
o_dxydxy_dAx = o_dxydxy_dAx + alpha2*o_fxxydxy - o_pydxy
o_dxydxz_dAx = o_dxydxz_dAx + alpha2*o_fxxydxz - o_pydxz
o_dxydyz_dAx = o_dxydyz_dAx + alpha2*o_fxxydyz - o_pydyz
o_dxzdxx_dAx = o_dxzdxx_dAx + alpha2*o_fxxzdxx - o_pzdxx
o_dxzdyy_dAx = o_dxzdyy_dAx + alpha2*o_fxxzdyy - o_pzdyy
o_dxzdzz_dAx = o_dxzdzz_dAx + alpha2*o_fxxzdzz - o_pzdzz
o_dxzdxy_dAx = o_dxzdxy_dAx + alpha2*o_fxxzdxy - o_pzdxy
o_dxzdxz_dAx = o_dxzdxz_dAx + alpha2*o_fxxzdxz - o_pzdxz
o_dxzdyz_dAx = o_dxzdyz_dAx + alpha2*o_fxxzdyz - o_pzdyz
o_dyzdxx_dAx = o_dyzdxx_dAx + alpha2*o_fxyzdxx
o_dyzdyy_dAx = o_dyzdyy_dAx + alpha2*o_fxyzdyy
o_dyzdzz_dAx = o_dyzdzz_dAx + alpha2*o_fxyzdzz
o_dyzdxy_dAx = o_dyzdxy_dAx + alpha2*o_fxyzdxy
o_dyzdxz_dAx = o_dyzdxz_dAx + alpha2*o_fxyzdxz
o_dyzdyz_dAx = o_dyzdyz_dAx + alpha2*o_fxyzdyz
o_dxxdxx_dAy = o_dxxdxx_dAy + alpha2*o_fxxydxx
o_dxxdyy_dAy = o_dxxdyy_dAy + alpha2*o_fxxydyy
o_dxxdzz_dAy = o_dxxdzz_dAy + alpha2*o_fxxydzz
o_dxxdxy_dAy = o_dxxdxy_dAy + alpha2*o_fxxydxy
o_dxxdxz_dAy = o_dxxdxz_dAy + alpha2*o_fxxydxz
o_dxxdyz_dAy = o_dxxdyz_dAy + alpha2*o_fxxydyz
o_dyydxx_dAy = o_dyydxx_dAy + alpha2*o_fyyydxx - 2.0d0*o_pydxx
o_dyydyy_dAy = o_dyydyy_dAy + alpha2*o_fyyydyy - 2.0d0*o_pydyy
o_dyydzz_dAy = o_dyydzz_dAy + alpha2*o_fyyydzz - 2.0d0*o_pydzz
o_dyydxy_dAy = o_dyydxy_dAy + alpha2*o_fyyydxy - 2.0d0*o_pydxy
o_dyydxz_dAy = o_dyydxz_dAy + alpha2*o_fyyydxz - 2.0d0*o_pydxz
o_dyydyz_dAy = o_dyydyz_dAy + alpha2*o_fyyydyz - 2.0d0*o_pydyz
o_dzzdxx_dAy = o_dzzdxx_dAy + alpha2*o_fzzydxx
o_dzzdyy_dAy = o_dzzdyy_dAy + alpha2*o_fzzydyy
o_dzzdzz_dAy = o_dzzdzz_dAy + alpha2*o_fzzydzz
o_dzzdxy_dAy = o_dzzdxy_dAy + alpha2*o_fzzydxy
o_dzzdxz_dAy = o_dzzdxz_dAy + alpha2*o_fzzydxz
o_dzzdyz_dAy = o_dzzdyz_dAy + alpha2*o_fzzydyz
o_dxydxx_dAy = o_dxydxx_dAy + alpha2*o_fyyxdxx - o_pxdxx
o_dxydyy_dAy = o_dxydyy_dAy + alpha2*o_fyyxdyy - o_pxdyy
o_dxydzz_dAy = o_dxydzz_dAy + alpha2*o_fyyxdzz - o_pxdzz
o_dxydxy_dAy = o_dxydxy_dAy + alpha2*o_fyyxdxy - o_pxdxy
o_dxydxz_dAy = o_dxydxz_dAy + alpha2*o_fyyxdxz - o_pxdxz
o_dxydyz_dAy = o_dxydyz_dAy + alpha2*o_fyyxdyz - o_pxdyz
o_dxzdxx_dAy = o_dxzdxx_dAy + alpha2*o_fxyzdxx
o_dxzdyy_dAy = o_dxzdyy_dAy + alpha2*o_fxyzdyy
o_dxzdzz_dAy = o_dxzdzz_dAy + alpha2*o_fxyzdzz
o_dxzdxy_dAy = o_dxzdxy_dAy + alpha2*o_fxyzdxy
o_dxzdxz_dAy = o_dxzdxz_dAy + alpha2*o_fxyzdxz
o_dxzdyz_dAy = o_dxzdyz_dAy + alpha2*o_fxyzdyz
o_dyzdxx_dAy = o_dyzdxx_dAy + alpha2*o_fyyzdxx - o_pzdxx
o_dyzdyy_dAy = o_dyzdyy_dAy + alpha2*o_fyyzdyy - o_pzdyy
o_dyzdzz_dAy = o_dyzdzz_dAy + alpha2*o_fyyzdzz - o_pzdzz
o_dyzdxy_dAy = o_dyzdxy_dAy + alpha2*o_fyyzdxy - o_pzdxy
o_dyzdxz_dAy = o_dyzdxz_dAy + alpha2*o_fyyzdxz - o_pzdxz
o_dyzdyz_dAy = o_dyzdyz_dAy + alpha2*o_fyyzdyz - o_pzdyz
o_dxxdxx_dAz = o_dxxdxx_dAz + alpha2*o_fxxzdxx
o_dxxdyy_dAz = o_dxxdyy_dAz + alpha2*o_fxxzdyy
o_dxxdzz_dAz = o_dxxdzz_dAz + alpha2*o_fxxzdzz
o_dxxdxy_dAz = o_dxxdxy_dAz + alpha2*o_fxxzdxy
o_dxxdxz_dAz = o_dxxdxz_dAz + alpha2*o_fxxzdxz
o_dxxdyz_dAz = o_dxxdyz_dAz + alpha2*o_fxxzdyz
o_dyydxx_dAz = o_dyydxx_dAz + alpha2*o_fyyzdxx
o_dyydyy_dAz = o_dyydyy_dAz + alpha2*o_fyyzdyy
o_dyydzz_dAz = o_dyydzz_dAz + alpha2*o_fyyzdzz
o_dyydxy_dAz = o_dyydxy_dAz + alpha2*o_fyyzdxy
o_dyydxz_dAz = o_dyydxz_dAz + alpha2*o_fyyzdxz
o_dyydyz_dAz = o_dyydyz_dAz + alpha2*o_fyyzdyz
o_dzzdxx_dAz = o_dzzdxx_dAz + alpha2*o_fzzzdxx - 2.0d0*o_pzdxx
o_dzzdyy_dAz = o_dzzdyy_dAz + alpha2*o_fzzzdyy - 2.0d0*o_pzdyy
o_dzzdzz_dAz = o_dzzdzz_dAz + alpha2*o_fzzzdzz - 2.0d0*o_pzdzz
o_dzzdxy_dAz = o_dzzdxy_dAz + alpha2*o_fzzzdxy - 2.0d0*o_pzdxy
o_dzzdxz_dAz = o_dzzdxz_dAz + alpha2*o_fzzzdxz - 2.0d0*o_pzdxz
o_dzzdyz_dAz = o_dzzdyz_dAz + alpha2*o_fzzzdyz - 2.0d0*o_pzdyz
o_dxydxx_dAz = o_dxydxx_dAz + alpha2*o_fxyzdxx
o_dxydyy_dAz = o_dxydyy_dAz + alpha2*o_fxyzdyy
o_dxydzz_dAz = o_dxydzz_dAz + alpha2*o_fxyzdzz
o_dxydxy_dAz = o_dxydxy_dAz + alpha2*o_fxyzdxy
o_dxydxz_dAz = o_dxydxz_dAz + alpha2*o_fxyzdxz
o_dxydyz_dAz = o_dxydyz_dAz + alpha2*o_fxyzdyz
o_dxzdxx_dAz = o_dxzdxx_dAz + alpha2*o_fzzxdxx - o_pxdxx
o_dxzdyy_dAz = o_dxzdyy_dAz + alpha2*o_fzzxdyy - o_pxdyy
o_dxzdzz_dAz = o_dxzdzz_dAz + alpha2*o_fzzxdzz - o_pxdzz
o_dxzdxy_dAz = o_dxzdxy_dAz + alpha2*o_fzzxdxy - o_pxdxy
o_dxzdxz_dAz = o_dxzdxz_dAz + alpha2*o_fzzxdxz - o_pxdxz
o_dxzdyz_dAz = o_dxzdyz_dAz + alpha2*o_fzzxdyz - o_pxdyz
o_dyzdxx_dAz = o_dyzdxx_dAz + alpha2*o_fzzydxx - o_pydxx
o_dyzdyy_dAz = o_dyzdyy_dAz + alpha2*o_fzzydyy - o_pydyy
o_dyzdzz_dAz = o_dyzdzz_dAz + alpha2*o_fzzydzz - o_pydzz
o_dyzdxy_dAz = o_dyzdxy_dAz + alpha2*o_fzzydxy - o_pydxy
o_dyzdxz_dAz = o_dyzdxz_dAz + alpha2*o_fzzydxz - o_pydxz
o_dyzdyz_dAz = o_dyzdyz_dAz + alpha2*o_fzzydyz - o_pydyz
end do
v(1,1) = o_dxxdxx_dAx
v(2,1) = o_dxxdyy_dAx
v(3,1) = o_dxxdzz_dAx
v(4,1) = o_dxxdxy_dAx*root3
v(5,1) = o_dxxdxz_dAx*root3
v(6,1) = o_dxxdyz_dAx*root3
v(7,1) = o_dyydxx_dAx
v(8,1) = o_dyydyy_dAx
v(9,1) = o_dyydzz_dAx
v(10,1) = o_dyydxy_dAx*root3
v(11,1) = o_dyydxz_dAx*root3
v(12,1) = o_dyydyz_dAx*root3
v(13,1) = o_dzzdxx_dAx
v(14,1) = o_dzzdyy_dAx
v(15,1) = o_dzzdzz_dAx
v(16,1) = o_dzzdxy_dAx*root3
v(17,1) = o_dzzdxz_dAx*root3
v(18,1) = o_dzzdyz_dAx*root3
v(19,1) = o_dxydxx_dAx*root3
v(20,1) = o_dxydyy_dAx*root3
v(21,1) = o_dxydzz_dAx*root3
v(22,1) = o_dxydxy_dAx*root3x2
v(23,1) = o_dxydxz_dAx*root3x2
v(24,1) = o_dxydyz_dAx*root3x2
v(25,1) = o_dxzdxx_dAx*root3
v(26,1) = o_dxzdyy_dAx*root3
v(27,1) = o_dxzdzz_dAx*root3
v(28,1) = o_dxzdxy_dAx*root3x2
v(29,1) = o_dxzdxz_dAx*root3x2
v(30,1) = o_dxzdyz_dAx*root3x2
v(31,1) = o_dyzdxx_dAx*root3
v(32,1) = o_dyzdyy_dAx*root3
v(33,1) = o_dyzdzz_dAx*root3
v(34,1) = o_dyzdxy_dAx*root3x2
v(35,1) = o_dyzdxz_dAx*root3x2
v(36,1) = o_dyzdyz_dAx*root3x2
v(1,2) = o_dxxdxx_dAy
v(2,2) = o_dxxdyy_dAy
v(3,2) = o_dxxdzz_dAy
v(4,2) = o_dxxdxy_dAy*root3
v(5,2) = o_dxxdxz_dAy*root3
v(6,2) = o_dxxdyz_dAy*root3
v(7,2) = o_dyydxx_dAy
v(8,2) = o_dyydyy_dAy
v(9,2) = o_dyydzz_dAy
v(10,2) = o_dyydxy_dAy*root3
v(11,2) = o_dyydxz_dAy*root3
v(12,2) = o_dyydyz_dAy*root3
v(13,2) = o_dzzdxx_dAy
v(14,2) = o_dzzdyy_dAy
v(15,2) = o_dzzdzz_dAy
v(16,2) = o_dzzdxy_dAy*root3
v(17,2) = o_dzzdxz_dAy*root3
v(18,2) = o_dzzdyz_dAy*root3
v(19,2) = o_dxydxx_dAy*root3
v(20,2) = o_dxydyy_dAy*root3
v(21,2) = o_dxydzz_dAy*root3
v(22,2) = o_dxydxy_dAy*root3x2
v(23,2) = o_dxydxz_dAy*root3x2
v(24,2) = o_dxydyz_dAy*root3x2
v(25,2) = o_dxzdxx_dAy*root3
v(26,2) = o_dxzdyy_dAy*root3
v(27,2) = o_dxzdzz_dAy*root3
v(28,2) = o_dxzdxy_dAy*root3x2
v(29,2) = o_dxzdxz_dAy*root3x2
v(30,2) = o_dxzdyz_dAy*root3x2
v(31,2) = o_dyzdxx_dAy*root3
v(32,2) = o_dyzdyy_dAy*root3
v(33,2) = o_dyzdzz_dAy*root3
v(34,2) = o_dyzdxy_dAy*root3x2
v(35,2) = o_dyzdxz_dAy*root3x2
v(36,2) = o_dyzdyz_dAy*root3x2
v(1,3) = o_dxxdxx_dAz
v(2,3) = o_dxxdyy_dAz
v(3,3) = o_dxxdzz_dAz
v(4,3) = o_dxxdxy_dAz*root3
v(5,3) = o_dxxdxz_dAz*root3
v(6,3) = o_dxxdyz_dAz*root3
v(7,3) = o_dyydxx_dAz
v(8,3) = o_dyydyy_dAz
v(9,3) = o_dyydzz_dAz
v(10,3) = o_dyydxy_dAz*root3
v(11,3) = o_dyydxz_dAz*root3
v(12,3) = o_dyydyz_dAz*root3
v(13,3) = o_dzzdxx_dAz
v(14,3) = o_dzzdyy_dAz
v(15,3) = o_dzzdzz_dAz
v(16,3) = o_dzzdxy_dAz*root3
v(17,3) = o_dzzdxz_dAz*root3
v(18,3) = o_dzzdyz_dAz*root3
v(19,3) = o_dxydxx_dAz*root3
v(20,3) = o_dxydyy_dAz*root3
v(21,3) = o_dxydzz_dAz*root3
v(22,3) = o_dxydxy_dAz*root3x2
v(23,3) = o_dxydxz_dAz*root3x2
v(24,3) = o_dxydyz_dAz*root3x2
v(25,3) = o_dxzdxx_dAz*root3
v(26,3) = o_dxzdyy_dAz*root3
v(27,3) = o_dxzdzz_dAz*root3
v(28,3) = o_dxzdxy_dAz*root3x2
v(29,3) = o_dxzdxz_dAz*root3x2
v(30,3) = o_dxzdyz_dAz*root3x2
v(31,3) = o_dyzdxx_dAz*root3
v(32,3) = o_dyzdyy_dAz*root3
v(33,3) = o_dyzdzz_dAz*root3
v(34,3) = o_dyzdxy_dAz*root3x2
v(35,3) = o_dyzdxz_dAz*root3x2
v(36,3) = o_dyzdyz_dAz*root3x2

     i = npsh(1,ij)
     j = npsh(2,ij)
     iao = nsh2ao(i)
     jao = nsh2ao(j)

     if (iao == jao) then
        do ixyz = 1, 3
           do jx = jao,jao + 6-1
              do ix = jx,iao + 6-1
                 key = ( ix-iao ) * 6 + jx-jao +1
                 g(ixyz,iatom) = g(ixyz,iatom) + dm(ix,jx)*v(key,ixyz)
              end do
           end do
        end do
        do ixyz = 1, 3
           do jx = jao,jao + 6-1
              do ix = jx,iao + 6-1
                 key = ( ix-iao ) * 6 + jx-jao +1
                 g(ixyz,jatom) = g(ixyz,jatom) - dm(ix,jx)*v(key,ixyz)
              end do
           end do
        end do
     else
        do ixyz = 1, 3
           do jx = jao,jao + 6-1
              do ix = iao,iao + 6-1
                 key = ( ix-iao ) * 6 + jx-jao +1
                 g(ixyz,iatom) = g(ixyz,iatom) + dm(ix,jx)*v(key,ixyz)
              end do
           end do
        end do
        do ixyz = 1, 3
           do jx = jao,jao + 6-1
              do ix = iao,iao + 6-1
                 key = ( ix-iao ) * 6 + jx-jao +1
                 g(ixyz,jatom) = g(ixyz,jatom) - dm(ix,jx)*v(key,ixyz)
              end do
           end do
        end do
     end if
  end do
!$omp end do nowait
end subroutine os_dovi_dd_t1
