! -*- coding: utf-8 -*-
!
! Dual licensed under the MIT or GPL Version 3 license.
! See 'LICENSE.MIT' for the definition of the MIT license,
! and 'COPYING' for the definition of the GPL license.
!
subroutine os_fock_ov5(fj,fk,d,nao,rsrs,tables,atoms,fmt,erithr,myrank,nproc)
  use integral_tables_class
  use atoms_class
  use integral_fmt_class
  use pl_prof_class
  use omp_lib
  use mpi
  implicit real(8) (a-h,o-z)
!!$  include 'mpif.h'
  integer,intent(in)::nao
  real(8),intent(in)::d(nao,nao)
  real(8),intent(inout)::fj(nao,nao)
  real(8),intent(inout)::fk(nao,nao)
  type(tables_type),intent(in)::tables
  type(atoms_type),intent(in)::atoms
  type(fmt_type),intent(in)::fmt
  real(8),intent(in)::rsrs(tables%nshelp)
  real(8),intent(in)::erithr
  integer,intent(in)::myrank,nproc

  integer(8),allocatable::nteiv2s(:)
  integer,allocatable::ntasks(:)

  mang = tables%mang
  mangp = mang*(mang+1)/2
  !if (myrank == 0) write(6,*) 'mang,mangp=',mang,mangp
  nshell = tables%nshell

  fj(:,:) = 0.0d0
  fk(:,:) = 0.0d0

  allocate(nteiv2s(21))
  nteiv2s(:) = 0

!$omp parallel&
!$omp& default(private)&
!$omp& shared(d,rsrs,&
!$omp&  tables,fmt,&
!$omp&  nao,&
!$omp&  erithr,myrank,nproc)&
!$omp&  firstprivate(nshell,mangp)&
!$omp& reduction(+:fj,fk,nteiv2s)
  do k = 1, mangp
     if (k == 1)then 
        kls = 1
     else
        kls = tables%ishp(k-1) + 1
     end if
     kle = tables%ishp(k)

     do i = k, mangp
        if (i == 1)then 
           ijs = 1
        else
           ijs = tables%ishp(i-1) + 1
        end if
        ije = tables%ishp(i)

        mangt = (i*(i+1)/2) - i + k

        nij = ije - ijs + 1
        nkl = kle - kls + 1
        select case(mangt)
        case(1)
           call pl_prof_region_begin('ov5_ssss')
           call os_ssss_ov5(&
                fj,fk,d,nao,&
                rsrs(ijs),tables%oozeta(1,ijs),tables%nprims2(ijs),&
                tables%sab(1,ijs),tables%ab(1,ijs),&
                tables%p(1,1,ijs),tables%pa(1,1,ijs),&
                tables%npsh(1,ijs),nij,&
                rsrs(kls),tables%oozeta(1,kls),tables%nprims2(kls),&
                tables%sab(1,kls),tables%ab(1,kls),&
                tables%p(1,1,kls),tables%pa(1,1,kls),&
                tables%npsh(1,kls),nkl,&
                tables%mxprim2,nshell,tables%nsh2ao,fmt,&
                nteiv2s(mangt),erithr,myrank,nproc)
           call pl_prof_region_end('ov5_ssss')
        case(2)
           call pl_prof_region_begin('ov5_psss')
           call os_psss_ov5(&
                fj,fk,d,nao,&
                rsrs(ijs),tables%oozeta(1,ijs),tables%nprims2(ijs),&
                tables%sab(1,ijs),tables%ab(1,ijs),&
                tables%p(1,1,ijs),tables%pa(1,1,ijs),&
                tables%npsh(1,ijs),nij,&
                rsrs(kls),tables%oozeta(1,kls),tables%nprims2(kls),&
                tables%sab(1,kls),tables%ab(1,kls),&
                tables%p(1,1,kls),tables%pa(1,1,kls),&
                tables%npsh(1,kls),nkl,&
                tables%mxprim2,nshell,tables%nsh2ao,fmt,&
                nteiv2s(mangt),erithr,myrank,nproc)
           call pl_prof_region_end('ov5_psss')
        case(3)
           call pl_prof_region_begin('ov5_psps')
           call os_psps_ov5(&
                fj,fk,d,nao,&
                rsrs(ijs),tables%oozeta(1,ijs),tables%nprims2(ijs),&
                tables%sab(1,ijs),tables%ab(1,ijs),&
                tables%p(1,1,ijs),tables%pa(1,1,ijs),&
                tables%npsh(1,ijs),nij,&
                rsrs(kls),tables%oozeta(1,kls),tables%nprims2(kls),&
                tables%sab(1,kls),tables%ab(1,kls),&
                tables%p(1,1,kls),tables%pa(1,1,kls),&
                tables%npsh(1,kls),nkl,&
                tables%mxprim2,nshell,tables%nsh2ao,fmt,&
                nteiv2s(mangt),erithr,myrank,nproc)
           call pl_prof_region_end('ov5_psps')
        case(4)
           call pl_prof_region_begin('ov5_ppss')
           call os_ppss_ov5(&
                fj,fk,d,nao,&
                rsrs(ijs),tables%oozeta(1,ijs),tables%nprims2(ijs),&
                tables%sab(1,ijs),tables%ab(1,ijs),&
                tables%p(1,1,ijs),tables%pa(1,1,ijs),&
                tables%npsh(1,ijs),nij,&
                rsrs(kls),tables%oozeta(1,kls),tables%nprims2(kls),&
                tables%sab(1,kls),tables%ab(1,kls),&
                tables%p(1,1,kls),tables%pa(1,1,kls),&
                tables%npsh(1,kls),nkl,&
                tables%mxprim2,nshell,tables%nsh2ao,fmt,&
                nteiv2s(mangt),erithr,myrank,nproc)
           call pl_prof_region_end('ov5_ppss')
        case(5)
           call pl_prof_region_begin('ov5_ppps')
           call os_ppps_ov5(&
                fj,fk,d,nao,&
                rsrs(ijs),tables%oozeta(1,ijs),tables%nprims2(ijs),&
                tables%sab(1,ijs),tables%ab(1,ijs),&
                tables%p(1,1,ijs),tables%pa(1,1,ijs),&
                tables%npsh(1,ijs),nij,&
                rsrs(kls),tables%oozeta(1,kls),tables%nprims2(kls),&
                tables%sab(1,kls),tables%ab(1,kls),&
                tables%p(1,1,kls),tables%pa(1,1,kls),&
                tables%npsh(1,kls),nkl,&
                tables%mxprim2,nshell,tables%nsh2ao,fmt,&
                nteiv2s(mangt),erithr,myrank,nproc)
           call pl_prof_region_end('ov5_ppps')
        case(6)
           call pl_prof_region_begin('ov5_pppp')
           call os_pppp_ov5(&
                fj,fk,d,nao,&
                rsrs(ijs),tables%oozeta(1,ijs),tables%nprims2(ijs),&
                tables%sab(1,ijs),tables%ab(1,ijs),&
                tables%p(1,1,ijs),tables%pa(1,1,ijs),&
                tables%npsh(1,ijs),nij,&
                rsrs(kls),tables%oozeta(1,kls),tables%nprims2(kls),&
                tables%sab(1,kls),tables%ab(1,kls),&
                tables%p(1,1,kls),tables%pa(1,1,kls),&
                tables%npsh(1,kls),nkl,&
                tables%mxprim2,nshell,tables%nsh2ao,fmt,&
                nteiv2s(mangt),erithr,myrank,nproc)
           call pl_prof_region_end('ov5_pppp')
        case(7)
           call pl_prof_region_begin('ov5_dsss')
           call os_dsss_ov5(&
                fj,fk,d,nao,&
                rsrs(ijs),tables%oozeta(1,ijs),tables%nprims2(ijs),&
                tables%sab(1,ijs),tables%ab(1,ijs),&
                tables%p(1,1,ijs),tables%pa(1,1,ijs),&
                tables%npsh(1,ijs),nij,&
                rsrs(kls),tables%oozeta(1,kls),tables%nprims2(kls),&
                tables%sab(1,kls),tables%ab(1,kls),&
                tables%p(1,1,kls),tables%pa(1,1,kls),&
                tables%npsh(1,kls),nkl,&
                tables%mxprim2,nshell,tables%nsh2ao,fmt,&
                nteiv2s(mangt),erithr,myrank,nproc)
           call pl_prof_region_end('ov5_dsss')
        case(8)
           call pl_prof_region_begin('ov5_dsps')
           call os_dsps_ov5(&
                fj,fk,d,nao,&
                rsrs(ijs),tables%oozeta(1,ijs),tables%nprims2(ijs),&
                tables%sab(1,ijs),tables%ab(1,ijs),&
                tables%p(1,1,ijs),tables%pa(1,1,ijs),&
                tables%npsh(1,ijs),nij,&
                rsrs(kls),tables%oozeta(1,kls),tables%nprims2(kls),&
                tables%sab(1,kls),tables%ab(1,kls),&
                tables%p(1,1,kls),tables%pa(1,1,kls),&
                tables%npsh(1,kls),nkl,&
                tables%mxprim2,nshell,tables%nsh2ao,fmt,&
                nteiv2s(mangt),erithr,myrank,nproc)
           call pl_prof_region_end('ov5_dsps')
        case(9)
           call pl_prof_region_begin('ov5_dspp')
           call os_dspp_ov5(&
                fj,fk,d,nao,&
                rsrs(ijs),tables%oozeta(1,ijs),tables%nprims2(ijs),&
                tables%sab(1,ijs),tables%ab(1,ijs),&
                tables%p(1,1,ijs),tables%pa(1,1,ijs),&
                tables%npsh(1,ijs),nij,&
                rsrs(kls),tables%oozeta(1,kls),tables%nprims2(kls),&
                tables%sab(1,kls),tables%ab(1,kls),&
                tables%p(1,1,kls),tables%pa(1,1,kls),&
                tables%npsh(1,kls),nkl,&
                tables%mxprim2,nshell,tables%nsh2ao,fmt,&
                nteiv2s(mangt),erithr,myrank,nproc)
           call pl_prof_region_end('ov5_dspp')
        case(10)
           call pl_prof_region_begin('ov5_dsds')
           call os_dsds_ov5(&
                fj,fk,d,nao,&
                rsrs(ijs),tables%oozeta(1,ijs),tables%nprims2(ijs),&
                tables%sab(1,ijs),tables%ab(1,ijs),&
                tables%p(1,1,ijs),tables%pa(1,1,ijs),&
                tables%npsh(1,ijs),nij,&
                rsrs(kls),tables%oozeta(1,kls),tables%nprims2(kls),&
                tables%sab(1,kls),tables%ab(1,kls),&
                tables%p(1,1,kls),tables%pa(1,1,kls),&
                tables%npsh(1,kls),nkl,&
                tables%mxprim2,nshell,tables%nsh2ao,fmt,&
                nteiv2s(mangt),erithr,myrank,nproc)
           call pl_prof_region_end('ov5_dsds')
        case(11)
           call pl_prof_region_begin('ov5_dpss')
           call os_dpss_ov5(&
                fj,fk,d,nao,&
                rsrs(ijs),tables%oozeta(1,ijs),tables%nprims2(ijs),&
                tables%sab(1,ijs),tables%ab(1,ijs),&
                tables%p(1,1,ijs),tables%pa(1,1,ijs),&
                tables%npsh(1,ijs),nij,&
                rsrs(kls),tables%oozeta(1,kls),tables%nprims2(kls),&
                tables%sab(1,kls),tables%ab(1,kls),&
                tables%p(1,1,kls),tables%pa(1,1,kls),&
                tables%npsh(1,kls),nkl,&
                tables%mxprim2,nshell,tables%nsh2ao,fmt,&
                nteiv2s(mangt),erithr,myrank,nproc)
           call pl_prof_region_end('ov5_dpss')
        case(12)
           call pl_prof_region_begin('ov5_dpps')
           call os_dpps_ov5(&
                fj,fk,d,nao,&
                rsrs(ijs),tables%oozeta(1,ijs),tables%nprims2(ijs),&
                tables%sab(1,ijs),tables%ab(1,ijs),&
                tables%p(1,1,ijs),tables%pa(1,1,ijs),&
                tables%npsh(1,ijs),nij,&
                rsrs(kls),tables%oozeta(1,kls),tables%nprims2(kls),&
                tables%sab(1,kls),tables%ab(1,kls),&
                tables%p(1,1,kls),tables%pa(1,1,kls),&
                tables%npsh(1,kls),nkl,&
                tables%mxprim2,nshell,tables%nsh2ao,fmt,&
                nteiv2s(mangt),erithr,myrank,nproc)
           call pl_prof_region_end('ov5_dpps')
        case(13)
           call pl_prof_region_begin('ov5_dppp')
           call os_dppp_ov5(&
                fj,fk,d,nao,&
                rsrs(ijs),tables%oozeta(1,ijs),tables%nprims2(ijs),&
                tables%sab(1,ijs),tables%ab(1,ijs),&
                tables%p(1,1,ijs),tables%pa(1,1,ijs),&
                tables%npsh(1,ijs),nij,&
                rsrs(kls),tables%oozeta(1,kls),tables%nprims2(kls),&
                tables%sab(1,kls),tables%ab(1,kls),&
                tables%p(1,1,kls),tables%pa(1,1,kls),&
                tables%npsh(1,kls),nkl,&
                tables%mxprim2,nshell,tables%nsh2ao,fmt,&
                nteiv2s(mangt),erithr,myrank,nproc)
           call pl_prof_region_end('ov5_dppp')
        case(14)
           call pl_prof_region_begin('ov5_dpds')
           call os_dpds_ov5(&
                fj,fk,d,nao,&
                rsrs(ijs),tables%oozeta(1,ijs),tables%nprims2(ijs),&
                tables%sab(1,ijs),tables%ab(1,ijs),&
                tables%p(1,1,ijs),tables%pa(1,1,ijs),&
                tables%npsh(1,ijs),nij,&
                rsrs(kls),tables%oozeta(1,kls),tables%nprims2(kls),&
                tables%sab(1,kls),tables%ab(1,kls),&
                tables%p(1,1,kls),tables%pa(1,1,kls),&
                tables%npsh(1,kls),nkl,&
                tables%mxprim2,nshell,tables%nsh2ao,fmt,&
                nteiv2s(mangt),erithr,myrank,nproc)
           call pl_prof_region_end('ov5_dpds')
        case(15)
           call pl_prof_region_begin('ov5_dpdp')
           call os_dpdp_ov5(&
                fj,fk,d,nao,&
                rsrs(ijs),tables%oozeta(1,ijs),tables%nprims2(ijs),&
                tables%sab(1,ijs),tables%ab(1,ijs),&
                tables%p(1,1,ijs),tables%pa(1,1,ijs),&
                tables%npsh(1,ijs),nij,&
                rsrs(kls),tables%oozeta(1,kls),tables%nprims2(kls),&
                tables%sab(1,kls),tables%ab(1,kls),&
                tables%p(1,1,kls),tables%pa(1,1,kls),&
                tables%npsh(1,kls),nkl,&
                tables%mxprim2,nshell,tables%nsh2ao,fmt,&
                nteiv2s(mangt),erithr,myrank,nproc)
           call pl_prof_region_end('ov5_dpdp')
        case(16)
           call pl_prof_region_begin('ov5_ddss')
           call os_ddss_ov5(&
                fj,fk,d,nao,&
                rsrs(ijs),tables%oozeta(1,ijs),tables%nprims2(ijs),&
                tables%sab(1,ijs),tables%ab(1,ijs),&
                tables%p(1,1,ijs),tables%pa(1,1,ijs),&
                tables%npsh(1,ijs),nij,&
                rsrs(kls),tables%oozeta(1,kls),tables%nprims2(kls),&
                tables%sab(1,kls),tables%ab(1,kls),&
                tables%p(1,1,kls),tables%pa(1,1,kls),&
                tables%npsh(1,kls),nkl,&
                tables%mxprim2,nshell,tables%nsh2ao,fmt,&
                nteiv2s(mangt),erithr,myrank,nproc)
           call pl_prof_region_end('ov5_ddss')
        case(17)
           call pl_prof_region_begin('ov5_ddps')
           call os_ddps_ov5(&
                fj,fk,d,nao,&
                rsrs(ijs),tables%oozeta(1,ijs),tables%nprims2(ijs),&
                tables%sab(1,ijs),tables%ab(1,ijs),&
                tables%p(1,1,ijs),tables%pa(1,1,ijs),&
                tables%npsh(1,ijs),nij,&
                rsrs(kls),tables%oozeta(1,kls),tables%nprims2(kls),&
                tables%sab(1,kls),tables%ab(1,kls),&
                tables%p(1,1,kls),tables%pa(1,1,kls),&
                tables%npsh(1,kls),nkl,&
                tables%mxprim2,nshell,tables%nsh2ao,fmt,&
                nteiv2s(mangt),erithr,myrank,nproc)
           call pl_prof_region_end('ov5_ddps')
        case(18)
           call pl_prof_region_begin('ov5_ddpp')
           call os_ddpp_ov5(&
                fj,fk,d,nao,&
                rsrs(ijs),tables%oozeta(1,ijs),tables%nprims2(ijs),&
                tables%sab(1,ijs),tables%ab(1,ijs),&
                tables%p(1,1,ijs),tables%pa(1,1,ijs),&
                tables%npsh(1,ijs),nij,&
                rsrs(kls),tables%oozeta(1,kls),tables%nprims2(kls),&
                tables%sab(1,kls),tables%ab(1,kls),&
                tables%p(1,1,kls),tables%pa(1,1,kls),&
                tables%npsh(1,kls),nkl,&
                tables%mxprim2,nshell,tables%nsh2ao,fmt,&
                nteiv2s(mangt),erithr,myrank,nproc)
           call pl_prof_region_end('ov5_ddpp')
        case(19)
           call pl_prof_region_begin('ov5_ddds')
           call os_ddds_ov5(&
                fj,fk,d,nao,&
                rsrs(ijs),tables%oozeta(1,ijs),tables%nprims2(ijs),&
                tables%sab(1,ijs),tables%ab(1,ijs),&
                tables%p(1,1,ijs),tables%pa(1,1,ijs),&
                tables%npsh(1,ijs),nij,&
                rsrs(kls),tables%oozeta(1,kls),tables%nprims2(kls),&
                tables%sab(1,kls),tables%ab(1,kls),&
                tables%p(1,1,kls),tables%pa(1,1,kls),&
                tables%npsh(1,kls),nkl,&
                tables%mxprim2,nshell,tables%nsh2ao,fmt,&
                nteiv2s(mangt),erithr,myrank,nproc)
           call pl_prof_region_end('ov5_ddds')
        case(20)
           call pl_prof_region_begin('ov5_dddp')
           call os_dddp_ov5(&
                fj,fk,d,nao,&
                rsrs(ijs),tables%oozeta(1,ijs),tables%nprims2(ijs),&
                tables%sab(1,ijs),tables%ab(1,ijs),&
                tables%p(1,1,ijs),tables%pa(1,1,ijs),&
                tables%npsh(1,ijs),nij,&
                rsrs(kls),tables%oozeta(1,kls),tables%nprims2(kls),&
                tables%sab(1,kls),tables%ab(1,kls),&
                tables%p(1,1,kls),tables%pa(1,1,kls),&
                tables%npsh(1,kls),nkl,&
                tables%mxprim2,nshell,tables%nsh2ao,fmt,&
                nteiv2s(mangt),erithr,myrank,nproc)
           call pl_prof_region_end('ov5_dddp')
        case(21)
           call pl_prof_region_begin('ov5_dddd')
           call os_dddd_ov5(&
                fj,fk,d,nao,&
                rsrs(ijs),tables%oozeta(1,ijs),tables%nprims2(ijs),&
                tables%sab(1,ijs),tables%ab(1,ijs),&
                tables%p(1,1,ijs),tables%pa(1,1,ijs),&
                tables%npsh(1,ijs),nij,&
                rsrs(kls),tables%oozeta(1,kls),tables%nprims2(kls),&
                tables%sab(1,kls),tables%ab(1,kls),&
                tables%p(1,1,kls),tables%pa(1,1,kls),&
                tables%npsh(1,kls),nkl,&
                tables%mxprim2,nshell,tables%nsh2ao,fmt,&
                nteiv2s(mangt),erithr,myrank,nproc)
           call pl_prof_region_end('ov5_dddd')
        end select

     end do
  end do
!$omp end parallel

9 continue
  !write(6,'(''nteiv2s('',i2,'')='',i16)') (i,nteiv2s(i),i=1,21)
  do jao = 1, nao
     do iao = jao, nao
        fj(iao,jao) = fj(iao,jao) + fj(jao,iao)
        fj(jao,iao) = fj(iao,jao)
     end do
  end do
  do jao = 1, nao
     do iao = jao, nao
        fk(iao,jao) = fk(iao,jao) + fk(jao,iao)
        fk(jao,iao) = fk(iao,jao)
     end do
  end do
end subroutine os_fock_ov5
