! -*- coding: utf-8 -*-
!
! Dual licensed under the MIT or GPL Version 3 license.
! See 'LICENSE.MIT' for the definition of the MIT license,
! and 'COPYING' for the definition of the GPL license.
!
!------------------------------------------------------------------------------
!> Hellmann-Feynman integrals
!
! References:
!  [1] S.Obara and A.Saika, J.Chem.Phys. 84, 3963 (1986).
!
subroutine os_hfi_on_pc_v2(g_,d,nao,tables,atoms,ptcharges,fmt,myrank,nproc)
  use integral_tables_class
  use atoms_class
  use ptcharges_class
  use integral_fmt_class
  use pl_prof_class
  integer,intent(in)::nao
  real(8),intent(in)::d(nao,nao)
  type(tables_type),intent(in)::tables
  type(atoms_type),intent(in)::atoms
  type(ptcharges_type),intent(in)::ptcharges
  real(8),intent(inout)::g_(3,ptcharges%num)
  type(fmt_type),intent(in)::fmt
  integer,intent(in)::myrank,nproc

  real(8),allocatable::g(:,:)

  allocate(g(3,ptcharges%num))
  g(:,:) = 0.0d0

!$omp parallel&
!$omp& default(private)&
!$omp& shared(d,nao,tables,atoms,ptcharges,fmt,myrank,nproc)&
!$omp& reduction(+:g)
  call pl_prof_region_begin('os_hfi_ss_on_pc_v2')
  call os_hfi_ss_v2(&
       g,d,nao,&
       tables%ishp,tables%npsh,tables%npsh2pr,tables%nsh2atm,tables%nsh2ao,&
       atoms%xyz,tables%ovl,tables%bdz,tables%zet,&
       ptcharges%xyz,ptcharges%chg,ptcharges%num,&
       tables%nshell,tables%nshelp,tables%nprimp,atoms%natom,fmt,myrank,nproc)
  call pl_prof_region_end('os_hfi_ss_on_pc_v2')
  if (tables%mang == 1) go to 9

  call pl_prof_region_begin('os_hfi_ps_on_pc_v2')
  call os_hfi_ps_v2(&
       g,d,nao,&
       tables%ishp,tables%npsh,tables%npsh2pr,tables%nsh2atm,tables%nsh2ao,&
       atoms%xyz,tables%ovl,tables%bdz,tables%zet,&
       ptcharges%xyz,ptcharges%chg,ptcharges%num,&
       tables%nshell,tables%nshelp,tables%nprimp,atoms%natom,fmt,myrank,nproc)
  call pl_prof_region_end('os_hfi_ps_on_pc_v2')

  call pl_prof_region_begin('os_hfi_pp_on_pc_v2')
  call os_hfi_pp_v2(&
       g,d,nao,&
       tables%ishp,tables%npsh,tables%npsh2pr,tables%nsh2atm,tables%nsh2ao,&
       atoms%xyz,tables%ovl,tables%bdz,tables%zet,&
       ptcharges%xyz,ptcharges%chg,ptcharges%num,&
       tables%nshell,tables%nshelp,tables%nprimp,atoms%natom,fmt,myrank,nproc)
  call pl_prof_region_end('os_hfi_pp_on_pc_v2')
  if (tables%mang == 2) go to 9

  call pl_prof_region_begin('os_hfi_ds_on_pc_v2')
  call os_hfi_ds_v2(&
       g,d,nao,&
       tables%ishp,tables%npsh,tables%npsh2pr,tables%nsh2atm,tables%nsh2ao,&
       atoms%xyz,tables%ovl,tables%bdz,tables%zet,&
       ptcharges%xyz,ptcharges%chg,ptcharges%num,&
       tables%nshell,tables%nshelp,tables%nprimp,atoms%natom,fmt,myrank,nproc)
  call pl_prof_region_end('os_hfi_ds_on_pc_v2')

  call pl_prof_region_begin('os_hfi_dp_on_pc_v2')
  call os_hfi_dp_v2(&
       g,d,nao,&
       tables%ishp,tables%npsh,tables%npsh2pr,tables%nsh2atm,tables%nsh2ao,&
       atoms%xyz,tables%ovl,tables%bdz,tables%zet,&
       ptcharges%xyz,ptcharges%chg,ptcharges%num,&
       tables%nshell,tables%nshelp,tables%nprimp,atoms%natom,fmt,myrank,nproc)
  call pl_prof_region_end('os_hfi_dp_on_pc_v2')

  call pl_prof_region_begin('os_hfi_dd_on_pc_v2')
  call os_hfi_dd_v2(&
       g,d,nao,&
       tables%ishp,tables%npsh,tables%npsh2pr,tables%nsh2atm,tables%nsh2ao,&
       atoms%xyz,tables%ovl,tables%bdz,tables%zet,&
       ptcharges%xyz,ptcharges%chg,ptcharges%num,&
       tables%nshell,tables%nshelp,tables%nprimp,atoms%natom,fmt,myrank,nproc)
  call pl_prof_region_end('os_hfi_dd_on_pc_v2')
9 continue
!$omp end parallel

  g_(:,:) = g_(:,:) + g(:,:)

end subroutine os_hfi_on_pc_v2
