! -*- coding: utf-8 -*-
!
! Dual licensed under the MIT or GPL Version 3 license.
! See 'LICENSE.MIT' for the definition of the MIT license,
! and 'COPYING' for the definition of the GPL license.
!
!------------------------------------------------------------------------------
!> nuclear attraction integrals
!
! References:
!  [1] S.Obara and A.Saika, J.Chem.Phys. 84, 3963 (1986).
!
subroutine os_nai_gc_v1(v,nao,tables,atoms,gscharges,fmt,myrank,nproc)
  use integral_tables_class
  use atoms_class
  use gscharges_class
  use integral_fmt_class
  use pl_prof_class
  integer,intent(in)::nao
  real(8),intent(inout)::v(nao,nao)
  type(tables_type),intent(in)::tables
  type(atoms_type),intent(in)::atoms
  type(gscharges_type),intent(in)::gscharges
  type(fmt_type),intent(in)::fmt
  integer,intent(in)::myrank,nproc

!$omp parallel&
!$omp& default(private)&
!$omp& shared(v,nao,tables,atoms,gscharges,fmt,myrank,nproc)
  call pl_prof_region_begin('os_nai_ss_gc_v1')
  call os_nai_ss_gc_v1(&
       v,nao,&
       tables%ishp,tables%npsh,tables%npsh2pr,tables%nsh2atm,tables%nsh2ao,&
       atoms%xyz,tables%ovl,tables%bdz,tables%zet,&
       gscharges%xyz,gscharges%chg,gscharges%rc2v,gscharges%num,&
       tables%nshell,tables%nshelp,tables%nprimp,atoms%natom,fmt,myrank,nproc)
  call pl_prof_region_end('os_nai_ss_gc_v1')

  call pl_prof_region_begin('os_nai_ps_gc_v1')
  call os_nai_ps_gc_v1(&
       v,nao,&
       tables%ishp,tables%npsh,tables%npsh2pr,tables%nsh2atm,tables%nsh2ao,&
       atoms%xyz,tables%ovl,tables%bdz,tables%zet,&
       gscharges%xyz,gscharges%chg,gscharges%rc2v,gscharges%num,&
       tables%nshell,tables%nshelp,tables%nprimp,atoms%natom,fmt,myrank,nproc)
  call pl_prof_region_end('os_nai_ps_gc_v1')

  call pl_prof_region_begin('os_nai_pp_gc_v1')
  call os_nai_pp_gc_v1(&
       v,nao,&
       tables%ishp,tables%npsh,tables%npsh2pr,tables%nsh2atm,tables%nsh2ao,&
       atoms%xyz,tables%ovl,tables%bdz,tables%zet,&
       gscharges%xyz,gscharges%chg,gscharges%rc2v,gscharges%num,&
       tables%nshell,tables%nshelp,tables%nprimp,atoms%natom,fmt,myrank,nproc)
  call pl_prof_region_end('os_nai_pp_gc_v1')

  call pl_prof_region_begin('os_nai_ds_gc_v1')
  call os_nai_ds_gc_v1(&
       v,nao,&
       tables%ishp,tables%npsh,tables%npsh2pr,tables%nsh2atm,tables%nsh2ao,&
       atoms%xyz,tables%ovl,tables%bdz,tables%zet,&
       gscharges%xyz,gscharges%chg,gscharges%rc2v,gscharges%num,&
       tables%nshell,tables%nshelp,tables%nprimp,atoms%natom,fmt,myrank,nproc)
  call pl_prof_region_end('os_nai_ds_gc_v1')

  call pl_prof_region_begin('os_nai_dp_gc_v1')
  call os_nai_dp_gc_v1(&
       v,nao,&
       tables%ishp,tables%npsh,tables%npsh2pr,tables%nsh2atm,tables%nsh2ao,&
       atoms%xyz,tables%ovl,tables%bdz,tables%zet,&
       gscharges%xyz,gscharges%chg,gscharges%rc2v,gscharges%num,&
       tables%nshell,tables%nshelp,tables%nprimp,atoms%natom,fmt,myrank,nproc)
  call pl_prof_region_end('os_nai_dp_gc_v1')

  call pl_prof_region_begin('os_nai_dd_gc_v1')
  call os_nai_dd_gc_v1(&
       v,nao,&
       tables%ishp,tables%npsh,tables%npsh2pr,tables%nsh2atm,tables%nsh2ao,&
       atoms%xyz,tables%ovl,tables%bdz,tables%zet,&
       gscharges%xyz,gscharges%chg,gscharges%rc2v,gscharges%num,&
       tables%nshell,tables%nshelp,tables%nprimp,atoms%natom,fmt,myrank,nproc)
  call pl_prof_region_end('os_nai_dd_gc_v1')
!$omp end parallel

end subroutine os_nai_gc_v1

!------------------------------------------------------------------------------
!> ss nuclear attraction integrals
!
subroutine os_nai_ss_gc_v1(&
     val,nao,&
     ishp,npsh,npsh2pr,nsh2atm,nsh2ao,xyz,ss,xiozetaa,zeta,&
     C,chg_c,rc2v,nc,&
     nshell,nshellp,nprimp,natom,fmt,myrank,nproc)
  use physical_constants_class
  use integral_fmt_class

  implicit real(8) (a-h,o-z)
  integer,intent(in)::nao
  real(8),intent(inout)::val(nao,nao)

  integer,intent(in)::nshell,nshellp,nprimp,natom
  integer,intent(in)::ishp(6),npsh(2,nshellp),npsh2pr(nshellp+1)
  integer,intent(in)::nsh2atm(nshell),nsh2ao(nshell)
  real(8),intent(in)::ss(nprimp),xiozetaa(nprimp),zeta(nprimp)
  real(8),intent(in)::xyz(3,natom)

  integer,intent(in)::nc
  real(8),intent(in)::C(3,nc),chg_c(nc),rc2v(nc)
  type(fmt_type),intent(in)::fmt
  integer,intent(in)::myrank,nproc

  real(8)::v(1)

  real(8),allocatable::ssssx(:,:)

  tosqrtpi = 2.d0/dsqrt(pi)
  allocate(ssssx(0:0,nc))

!$omp do schedule(dynamic)
  do ij = myrank+1, ishp(1), nproc
     i = npsh(1,ij)
     j = npsh(2,ij)
     iatom = nsh2atm(i)
     jatom = nsh2atm(j)
     Ax = xyz(1,iatom)
     Ay = xyz(2,iatom)
     Az = xyz(3,iatom)
     Bx = xyz(1,jatom)
     By = xyz(2,jatom)
     Bz = xyz(3,jatom)
     ABx = Ax - Bx
     ABy = Ay - By
     ABz = Az - Bz

     a0ss0 = 0.0d0

     do irs = npsh2pr(ij), npsh2pr(ij+1) - 1
        oss = ss(irs)
        xioza = xiozetaa(irs)
        zeta_ = zeta(irs)
        PAx = -xioza*ABx
        PAy = -xioza*ABy
        PAz = -xioza*ABz
        Px = PAx + Ax
        Py = PAy + Ay
        Pz = PAz + Az
        osstwosqrtzopi = oss * tosqrtpi * dsqrt(zeta_)

        do k = 1, nc
           PCx = Px - C(1,k)
           PCy = Py - C(2,k)
           PCz = Pz - C(3,k)
           ossx = -chg_c(k) * osstwosqrtzopi
           t = zeta_ * (PCx*PCx + PCy*PCy + PCz*PCz)
           yerf = rc2v(k) * zeta_ + 1.d0
           t = t / yerf

           if (t <= tf(1)) then
              it = idint((t + 0.025d0)*20.0d0)
              dt = 0.05d0*it - t
              it = it + 1
              dt4 = dt * 0.25d0
              dt5 = dt * 0.2d0
              dt6 = dt * 0.1666666666666666666d0
              dt2 = dt4 + dt4
              dt3 = dt6 + dt6
              ft  = (((((fmt%fdat(7,it)*dt6 + fmt%fdat(6,it))*dt5&
                  + fmt%fdat(5,it))*dt4 + fmt%fdat(4,it))*dt3&
                  + fmt%fdat(3,it))*dt2 + fmt%fdat(2,it))*dt + fmt%fdat(1,it)
              ssss0 = ft * ossx
           else
              tinv = 1.d0/t
              ssss0 = dsqrt(pi*tinv) * ossx * 0.5d0
           end if

           ssssx(0,k) = ssss0 * dsqrt(1.d0/yerf)
        end do

        do k = 1, nc
           a0ss0 = a0ss0 + ssssx(0,k)
        end do

     end do

!!$     v(1) = a0ss0

     i = npsh(1,ij)
     j = npsh(2,ij)
     iao = nsh2ao(i)
     jao = nsh2ao(j)

!!$     val(iao,jao) = v(1)
     val(iao,jao) = a0ss0
  end do
!$omp end do nowait
end subroutine os_nai_ss_gc_v1

!------------------------------------------------------------------------------
!> ps nuclear attraction integrals
!
subroutine os_nai_ps_gc_v1(&
     val,nao,&
     ishp,npsh,npsh2pr,nsh2atm,nsh2ao,xyz,ss,xiozetaa,zeta,&
     C,chg_c,rc2v,nc,&
     nshell,nshellp,nprimp,natom,fmt,myrank,nproc)
  use physical_constants_class
  use integral_fmt_class

  implicit real(8) (a-h,o-z)
  integer,intent(in)::nao
  real(8),intent(inout)::val(nao,nao)

  integer,intent(in)::nshell,nshellp,nprimp,natom
  integer,intent(in)::ishp(6),npsh(2,nshellp),npsh2pr(nshellp+1)
  integer,intent(in)::nsh2atm(nshell),nsh2ao(nshell)
  real(8),intent(in)::ss(nprimp),xiozetaa(nprimp),zeta(nprimp)
  real(8),intent(in)::xyz(3,natom)

  integer,intent(in)::nc
  real(8),intent(in)::C(3,nc),chg_c(nc),rc2v(nc)
  type(fmt_type),intent(in)::fmt
  integer,intent(in)::myrank,nproc

  real(8)::v(3)

  real(8),allocatable::ssssx(:,:)

  tosqrtpi = 2.d0/dsqrt(pi)
  allocate(ssssx(0:1,nc))

!$omp do schedule(dynamic)
  do ij = ishp(1)+myrank+1, ishp(2), nproc
     i = npsh(1,ij)
     j = npsh(2,ij)
     iatom = nsh2atm(i)
     jatom = nsh2atm(j)
     Ax = xyz(1,iatom)
     Ay = xyz(2,iatom)
     Az = xyz(3,iatom)
     Bx = xyz(1,jatom)
     By = xyz(2,jatom)
     Bz = xyz(3,jatom)
     ABx = Ax - Bx
     ABy = Ay - By
     ABz = Az - Bz

     a0pxs0 = 0.0d0
     a0pys0 = 0.0d0
     a0pzs0 = 0.0d0

     do irs = npsh2pr(ij), npsh2pr(ij+1) - 1
        oss = ss(irs)
        xioza = xiozetaa(irs)
        zeta_ = zeta(irs)
        PAx = -xioza*ABx
        PAy = -xioza*ABy
        PAz = -xioza*ABz
        Px = PAx + Ax
        Py = PAy + Ay
        Pz = PAz + Az
        osstwosqrtzopi = oss * tosqrtpi * dsqrt(zeta_)

        do k = 1, nc
           PCx = Px - C(1,k)
           PCy = Py - C(2,k)
           PCz = Pz - C(3,k)
           ossx = -chg_c(k) * osstwosqrtzopi
           t = zeta_ * (PCx*PCx + PCy*PCy + PCz*PCz)
           yerf = rc2v(k) * zeta_ + 1.d0
           t = t / yerf

           if (t <= tf(2)) then
              it = idint((t + 0.025d0)*20.0d0)
              dt = 0.05d0*it - t
              it = it + 1
              dt4 = dt * 0.25d0
              dt5 = dt * 0.2d0
              dt6 = dt * 0.1666666666666666666d0
              dt2 = dt4 + dt4
              dt3 = dt6 + dt6
              ft  = (((((fmt%fdat(8,it)*dt6 + fmt%fdat(7,it))*dt5&
                  + fmt%fdat(6,it))*dt4 + fmt%fdat(5,it))*dt3&
                  + fmt%fdat(4,it))*dt2 + fmt%fdat(3,it))*dt + fmt%fdat(2,it)
              et  = ((((((ga6*dt + ga5)*dt + ga4)*dt&
                  + ga3)*dt + ga2)*dt + ga1)*dt + ga0) * fmt%edat(it)
              t2 = 2.d0 * t
              etcn = et * ossx
              ssss1 = ft * ossx
              ssss0 =  ssss1 * t2 + etcn
           else
              tinv = 1.d0/t
              ssss0 = dsqrt(pi*tinv) * ossx * 0.5d0
              ssss1 = ssss0 * tinv * 0.5d0
           end if

           ryerf = 1.d0/yerf
           y0 = dsqrt(ryerf)
           y1 = y0 * ryerf

           ssssx(0,k) = ssss0 * y0
           ssssx(1,k) = ssss1 * y1
        end do

        do k = 1, nc
           a0ss0 = ssssx(0,k)
           a0ss1 = ssssx(1,k)
           PCx = Px - C(1,k)
           PCy = Py - C(2,k)
           PCz = Pz - C(3,k)
           a0pzs0 = a0pzs0 + a0ss0*PAz - a0ss1*PCz
           a0pys0 = a0pys0 + a0ss0*PAy - a0ss1*PCy
           a0pxs0 = a0pxs0 + a0ss0*PAx - a0ss1*PCx
        end do

     end do

     !v(1) = a0pxs0
     !v(2) = a0pys0
     !v(3) = a0pzs0

     i = npsh(1,ij)
     j = npsh(2,ij)
     iao = nsh2ao(i)
     jao = nsh2ao(j)

     if (i < j) then
        !do ix = iao,iao + 3-1
        !   key = ix-iao + 1
        !   val(jao,ix) =  v(key)
        !end do
        val(jao,iao  ) = a0pxs0
        val(jao,iao+1) = a0pys0
        val(jao,iao+2) = a0pzs0
     else
        !do ix = iao,iao + 3-1
        !   key = ix-iao + 1
        !   val(ix,jao) =  v(key)
        !end do
        val(iao  ,jao) = a0pxs0
        val(iao+1,jao) = a0pys0
        val(iao+2,jao) = a0pzs0
     end if
  end do
!$omp end do
end subroutine os_nai_ps_gc_v1

!------------------------------------------------------------------------------
!> pp nuclear attraction integrals
!
subroutine os_nai_pp_gc_v1(&
     val,nao,&
     ishp,npsh,npsh2pr,nsh2atm,nsh2ao,xyz,ss,xiozetaa,zeta,&
     C,chg_c,rc2v,nc,&
     nshell,nshellp,nprimp,natom,fmt,myrank,nproc)
  use physical_constants_class
  use integral_fmt_class

  implicit real(8) (a-h,o-z)
  integer,intent(in)::nao
  real(8),intent(inout)::val(nao,nao)

  integer,intent(in)::nshell,nshellp,nprimp,natom
  integer,intent(in)::ishp(6),npsh(2,nshellp),npsh2pr(nshellp+1)
  integer,intent(in)::nsh2atm(nshell),nsh2ao(nshell)
  real(8),intent(in)::ss(nprimp),xiozetaa(nprimp),zeta(nprimp)
  real(8),intent(in)::xyz(3,natom)

  integer,intent(in)::nc
  real(8),intent(in)::C(3,nc),chg_c(nc),rc2v(nc)
  type(fmt_type),intent(in)::fmt
  integer,intent(in)::myrank,nproc

  real(8)::v(9)

  real(8),allocatable::ssssx(:,:)

  tosqrtpi = 2.d0/dsqrt(pi)
  allocate(ssssx(0:2,nc))

!$omp do schedule(dynamic)
  do ij = ishp(2)+myrank+1, ishp(3), nproc
     i = npsh(1,ij)
     j = npsh(2,ij)
     iatom = nsh2atm(i)
     jatom = nsh2atm(j)
     Ax = xyz(1,iatom)
     Ay = xyz(2,iatom)
     Az = xyz(3,iatom)
     Bx = xyz(1,jatom)
     By = xyz(2,jatom)
     Bz = xyz(3,jatom)
     ABx = Ax - Bx
     ABy = Ay - By
     ABz = Az - Bz

     a0pxpx0 = 0.0d0
     a0pxpy0 = 0.0d0
     a0pxpz0 = 0.0d0
     a0pypx0 = 0.0d0
     a0pypy0 = 0.0d0
     a0pypz0 = 0.0d0
     a0pzpx0 = 0.0d0
     a0pzpy0 = 0.0d0
     a0pzpz0 = 0.0d0

     do irs = npsh2pr(ij), npsh2pr(ij+1) - 1
        oss = ss(irs)
        xioza = xiozetaa(irs)
        zeta_ = zeta(irs)
        oo2zeta = 0.5d0 / zeta_
        PAx = -xioza*ABx
        PAy = -xioza*ABy
        PAz = -xioza*ABz
        PBx = PAx + ABx
        PBy = PAy + ABy
        PBz = PAz + ABz
        Px = PAx + Ax
        Py = PAy + Ay
        Pz = PAz + Az
        osstwosqrtzopi = oss * tosqrtpi * dsqrt(zeta_)

        do k = 1, nc
           PCx = Px - C(1,k)
           PCy = Py - C(2,k)
           PCz = Pz - C(3,k)
           ossx = -chg_c(k) * osstwosqrtzopi
           t = zeta_ * (PCx*PCx + PCy*PCy + PCz*PCz)
           yerf = rc2v(k) * zeta_ + 1.d0
           t = t / yerf

           if (t <= tf(3)) then
              it = idint((t + 0.025d0)*20.0d0)
              dt = 0.05d0*it - t
              it = it + 1
              dt4 = dt * 0.25d0
              dt5 = dt * 0.2d0
              dt6 = dt * 0.1666666666666666666d0
              dt2 = dt4 + dt4
              dt3 = dt6 + dt6
              ft  = (((((fmt%fdat(9,it)*dt6 + fmt%fdat(8,it))*dt5&
                  + fmt%fdat(7,it))*dt4 + fmt%fdat(6,it))*dt3&
                  + fmt%fdat(5,it))*dt2 + fmt%fdat(4,it))*dt + fmt%fdat(3,it)
              et  = ((((((ga6*dt + ga5)*dt + ga4)*dt&
                  + ga3)*dt + ga2)*dt + ga1)*dt + ga0) * fmt%edat(it)
              t2 = 2.d0 * t
              etcn = et * ossx
              ssss2 = ft * ossx
              ssss1 = (ssss2 * t2 + etcn) * rminv(2)
              ssss0 =  ssss1 * t2 + etcn
           else
              tinv = 1.d0/t
              ssss0 = dsqrt(pi*tinv) * ossx * 0.5d0
              ssss1 = ssss0 * tinv * 0.5d0
              ssss2 = ssss1 * tinv * 1.5d0
           end if

           ryerf = 1.d0/yerf
           y0 = dsqrt(ryerf)
           y1 = y0 * ryerf
           y2 = y1 * ryerf

           ssssx(0,k) = ssss0 * y0
           ssssx(1,k) = ssss1 * y1
           ssssx(2,k) = ssss2 * y2
        end do

        do k = 1, nc
           a0ss0 = ssssx(0,k)
           a0ss1 = ssssx(1,k)
           a0ss2 = ssssx(2,k)
           PCx = Px - C(1,k)
           PCy = Py - C(2,k)
           PCz = Pz - C(3,k)
           a0pzs1 = a0ss1*PAz - a0ss2*PCz
           a0pys1 = a0ss1*PAy - a0ss2*PCy
           a0pxs1 = a0ss1*PAx - a0ss2*PCx
           ai_a0ss01 = oo2zeta*( a0ss0 - a0ss1 )
           a0pzs0 = a0ss0*PAz - a0ss1*PCz
           a0pzpz0 = a0pzpz0 + a0pzs0*PBz - a0pzs1*PCz + ai_a0ss01
           a0pzpy0 = a0pzpy0 + a0pzs0*PBy - a0pzs1*PCy
           a0pzpx0 = a0pzpx0 + a0pzs0*PBx - a0pzs1*PCx
           a0pys0 = a0ss0*PAy - a0ss1*PCy
           a0pypz0 = a0pypz0 + a0pys0*PBz - a0pys1*PCz
           a0pypy0 = a0pypy0 + a0pys0*PBy - a0pys1*PCy + ai_a0ss01
           a0pypx0 = a0pypx0 + a0pys0*PBx - a0pys1*PCx
           a0pxs0 = a0ss0*PAx - a0ss1*PCx
           a0pxpz0 = a0pxpz0 + a0pxs0*PBz - a0pxs1*PCz
           a0pxpy0 = a0pxpy0 + a0pxs0*PBy - a0pxs1*PCy
           a0pxpx0 = a0pxpx0 + a0pxs0*PBx - a0pxs1*PCx + ai_a0ss01
        end do

     end do

     v(1) = a0pxpx0
     v(2) = a0pxpy0
     v(3) = a0pxpz0
     v(4) = a0pypx0
     v(5) = a0pypy0
     v(6) = a0pypz0
     v(7) = a0pzpx0
     v(8) = a0pzpy0
     v(9) = a0pzpz0

     i = npsh(1,ij)
     j = npsh(2,ij)
     iao = nsh2ao(i)
     jao = nsh2ao(j)

    if (iao == jao) then
      do jx = jao,jao + 3-1
        do ix = jx,iao + 3-1
          key = (ix-iao)*3 + jx-jao + 1
          val(ix,jx) =  v(key)
        end do
      end do
    else
      do jx = jao,jao + 3-1
        do ix = iao,iao + 3-1
          key = (ix-iao)*3 + jx-jao + 1
          val(ix,jx) =  v(key)
        end do
      end do
    end if
  end do
!$omp end do
end subroutine os_nai_pp_gc_v1

!------------------------------------------------------------------------------
!> ds nuclear attraction integrals
!
subroutine os_nai_ds_gc_v1(&
     val,nao,&
     ishp,npsh,npsh2pr,nsh2atm,nsh2ao,xyz,ss,xiozetaa,zeta,&
     C,chg_c,rc2v,nc,&
     nshell,nshellp,nprimp,natom,fmt,myrank,nproc)
  use physical_constants_class
  use integral_fmt_class

  implicit real(8) (a-h,o-z)
  integer,intent(in)::nao
  real(8),intent(inout)::val(nao,nao)

  integer,intent(in)::nshell,nshellp,nprimp,natom
  integer,intent(in)::ishp(6),npsh(2,nshellp),npsh2pr(nshellp+1)
  integer,intent(in)::nsh2atm(nshell),nsh2ao(nshell)
  real(8),intent(in)::ss(nprimp),xiozetaa(nprimp),zeta(nprimp)
  real(8),intent(in)::xyz(3,natom)
  integer,intent(in)::myrank,nproc

  integer,intent(in)::nc
  real(8),intent(in)::C(3,nc),chg_c(nc),rc2v(nc)
  type(fmt_type),intent(in)::fmt

  real(8)::v(6)

  real(8),allocatable::ssssx(:,:)

  tosqrtpi = 2.d0/dsqrt(pi)
  allocate(ssssx(0:2,nc))

!$omp do schedule(dynamic)
  do ij = ishp(3)+myrank+1, ishp(4), nproc
     i = npsh(1,ij)
     j = npsh(2,ij)
     iatom = nsh2atm(i)
     jatom = nsh2atm(j)
     Ax = xyz(1,iatom)
     Ay = xyz(2,iatom)
     Az = xyz(3,iatom)
     Bx = xyz(1,jatom)
     By = xyz(2,jatom)
     Bz = xyz(3,jatom)
     ABx = Ax - Bx
     ABy = Ay - By
     ABz = Az - Bz

     a0dxxs0 = 0.0d0
     a0dyys0 = 0.0d0
     a0dzzs0 = 0.0d0
     a0dxys0 = 0.0d0
     a0dxzs0 = 0.0d0
     a0dyzs0 = 0.0d0

     do irs = npsh2pr(ij), npsh2pr(ij+1) - 1
        oss = ss(irs)
        xioza = xiozetaa(irs)
        zeta_ = zeta(irs)
        oo2zeta = 0.5d0 / zeta_
        PAx = -xioza*ABx
        PAy = -xioza*ABy
        PAz = -xioza*ABz
        Px = PAx + Ax
        Py = PAy + Ay
        Pz = PAz + Az
        osstwosqrtzopi = oss * tosqrtpi * dsqrt(zeta_)

        do k = 1, nc
           PCx = Px - C(1,k)
           PCy = Py - C(2,k)
           PCz = Pz - C(3,k)
           ossx = -chg_c(k) * osstwosqrtzopi
           t = zeta_ * (PCx*PCx + PCy*PCy + PCz*PCz)
           yerf = rc2v(k) * zeta_ + 1.d0
           t = t / yerf

           if (t <= tf(3)) then
              it = idint((t + 0.025d0)*20.0d0)
              dt = 0.05d0*it - t
              it = it + 1
              dt4 = dt * 0.25d0
              dt5 = dt * 0.2d0
              dt6 = dt * 0.1666666666666666666d0
              dt2 = dt4 + dt4
              dt3 = dt6 + dt6
              ft  = (((((fmt%fdat(9,it)*dt6 + fmt%fdat(8,it))*dt5&
                  + fmt%fdat(7,it))*dt4 + fmt%fdat(6,it))*dt3&
                  + fmt%fdat(5,it))*dt2 + fmt%fdat(4,it))*dt + fmt%fdat(3,it)
              et  = ((((((ga6*dt + ga5)*dt + ga4)*dt&
                  + ga3)*dt + ga2)*dt + ga1)*dt + ga0) * fmt%edat(it)
              t2 = 2.d0 * t
              etcn = et * ossx
              ssss2 = ft * ossx
              ssss1 = (ssss2 * t2 + etcn) * rminv(2)
              ssss0 =  ssss1 * t2 + etcn
           else
              tinv = 1.d0/t
              ssss0 = dsqrt(pi*tinv) * ossx * 0.5d0
              ssss1 = ssss0 * tinv * 0.5d0
              ssss2 = ssss1 * tinv * 1.5d0
           end if

           ryerf = 1.d0/yerf
           y0 = dsqrt(ryerf)
           y1 = y0 * ryerf
           y2 = y1 * ryerf

           ssssx(0,k) = ssss0 * y0
           ssssx(1,k) = ssss1 * y1
           ssssx(2,k) = ssss2 * y2
        end do

        do k = 1, nc
           a0ss0 = ssssx(0,k)
           a0ss1 = ssssx(1,k)
           a0ss2 = ssssx(2,k)
           PCx = Px - C(1,k)
           PCy = Py - C(2,k)
           PCz = Pz - C(3,k)
           a0pzs1 = a0ss1*PAz - a0ss2*PCz
           a0pys1 = a0ss1*PAy - a0ss2*PCy
           a0pxs1 = a0ss1*PAx - a0ss2*PCx
           ai_a0ss01 = oo2zeta*( a0ss0 - a0ss1 )
           a0pzs0 = a0ss0*PAz - a0ss1*PCz
           a0dzzs0 = a0dzzs0 + a0pzs0*PAz - a0pzs1*PCz + ai_a0ss01
           a0pys0 = a0ss0*PAy - a0ss1*PCy
           a0dyzs0 = a0dyzs0 + a0pys0*PAz - a0pys1*PCz
           a0dyys0 = a0dyys0 + a0pys0*PAy - a0pys1*PCy + ai_a0ss01
           a0pxs0 = a0ss0*PAx - a0ss1*PCx
           a0dxzs0 = a0dxzs0 + a0pxs0*PAz - a0pxs1*PCz
           a0dxys0 = a0dxys0 + a0pxs0*PAy - a0pxs1*PCy
           a0dxxs0 = a0dxxs0 + a0pxs0*PAx - a0pxs1*PCx + ai_a0ss01
        end do

     end do

     v(1) = a0dxxs0
     v(2) = a0dyys0
     v(3) = a0dzzs0
     v(4) = a0dxys0*root3
     v(5) = a0dxzs0*root3
     v(6) = a0dyzs0*root3

     i = npsh(1,ij)
     j = npsh(2,ij)
     iao = nsh2ao(i)
     jao = nsh2ao(j)

    if (i < j) then
      do ix = iao,iao + 6-1
        key = ix-iao + 1
        val(jao,ix) =  v(key)
      end do
    else
      do ix = iao,iao + 6-1
        key = ix-iao + 1
        val(ix,jao) =  v(key)
      end do
    end if
  end do
!$omp end do
end subroutine os_nai_ds_gc_v1

!------------------------------------------------------------------------------
!> dp nuclear attraction integrals
!
subroutine os_nai_dp_gc_v1(&
     val,nao,&
     ishp,npsh,npsh2pr,nsh2atm,nsh2ao,xyz,ss,xiozetaa,zeta,&
     C,chg_c,rc2v,nc,&
     nshell,nshellp,nprimp,natom,fmt,myrank,nproc)
  use physical_constants_class
  use integral_fmt_class

  implicit real(8) (a-h,o-z)
  integer,intent(in)::nao
  real(8),intent(inout)::val(nao,nao)

  integer,intent(in)::nshell,nshellp,nprimp,natom
  integer,intent(in)::ishp(6),npsh(2,nshellp),npsh2pr(nshellp+1)
  integer,intent(in)::nsh2atm(nshell),nsh2ao(nshell)
  real(8),intent(in)::ss(nprimp),xiozetaa(nprimp),zeta(nprimp)
  real(8),intent(in)::xyz(3,natom)

  integer,intent(in)::nc
  real(8),intent(in)::C(3,nc),chg_c(nc),rc2v(nc)
  type(fmt_type),intent(in)::fmt
  integer,intent(in)::myrank,nproc

  real(8)::v(18)

  real(8),allocatable::ssssx(:,:)

  tosqrtpi = 2.d0/dsqrt(pi)
  allocate(ssssx(0:3,nc))

!$omp do schedule(dynamic)
  do ij = ishp(4)+myrank+1, ishp(5), nproc
     i = npsh(1,ij)
     j = npsh(2,ij)
     iatom = nsh2atm(i)
     jatom = nsh2atm(j)
     Ax = xyz(1,iatom)
     Ay = xyz(2,iatom)
     Az = xyz(3,iatom)
     Bx = xyz(1,jatom)
     By = xyz(2,jatom)
     Bz = xyz(3,jatom)
     ABx = Ax - Bx
     ABy = Ay - By
     ABz = Az - Bz

     a0dxxpx0 = 0.0d0
     a0dxxpy0 = 0.0d0
     a0dxxpz0 = 0.0d0
     a0dyypx0 = 0.0d0
     a0dyypy0 = 0.0d0
     a0dyypz0 = 0.0d0
     a0dzzpx0 = 0.0d0
     a0dzzpy0 = 0.0d0
     a0dzzpz0 = 0.0d0
     a0dxypx0 = 0.0d0
     a0dxypy0 = 0.0d0
     a0dxypz0 = 0.0d0
     a0dxzpx0 = 0.0d0
     a0dxzpy0 = 0.0d0
     a0dxzpz0 = 0.0d0
     a0dyzpx0 = 0.0d0
     a0dyzpy0 = 0.0d0
     a0dyzpz0 = 0.0d0

     do irs = npsh2pr(ij), npsh2pr(ij+1) - 1
        oss = ss(irs)
        xioza = xiozetaa(irs)
        zeta_ = zeta(irs)
        oo2zeta = 0.5d0 / zeta_
        PAx = -xioza*ABx
        PAy = -xioza*ABy
        PAz = -xioza*ABz
        PBx = PAx + ABx
        PBy = PAy + ABy
        PBz = PAz + ABz
        Px = PAx + Ax
        Py = PAy + Ay
        Pz = PAz + Az
        osstwosqrtzopi = oss * tosqrtpi * dsqrt(zeta_)

        do k = 1, nc
           PCx = Px - C(1,k)
           PCy = Py - C(2,k)
           PCz = Pz - C(3,k)
           ossx = -chg_c(k) * osstwosqrtzopi
           t = zeta_ * (PCx*PCx + PCy*PCy + PCz*PCz)
           yerf = rc2v(k) * zeta_ + 1.d0
           t = t / yerf

           if (t <= tf(4)) then
              it = idint((t + 0.025d0)*20.0d0)
              dt = 0.05d0*it - t
              it = it + 1
              dt4 = dt * 0.25d0
              dt5 = dt * 0.2d0
              dt6 = dt * 0.1666666666666666666d0
              dt2 = dt4 + dt4
              dt3 = dt6 + dt6
              ft  = (((((fmt%fdat(10,it)*dt6 + fmt%fdat(9,it))*dt5&
                  + fmt%fdat(8,it))*dt4 + fmt%fdat(7,it))*dt3&
                  + fmt%fdat(6,it))*dt2 + fmt%fdat(5,it))*dt + fmt%fdat(4,it)
              et  = ((((((ga6*dt + ga5)*dt + ga4)*dt&
                  + ga3)*dt + ga2)*dt + ga1)*dt + ga0) * fmt%edat(it)
              t2 = 2.d0 * t
              etcn = et * ossx
              ssss3 = ft * ossx
              ssss2 = (ssss3 * t2 + etcn) * rminv(3)
              ssss1 = (ssss2 * t2 + etcn) * rminv(2)
              ssss0 =  ssss1 * t2 + etcn
           else
              tinv = 1.d0/t
              ssss0 = dsqrt(pi*tinv) * ossx * 0.5d0
              ssss1 = ssss0 * tinv * 0.5d0
              ssss2 = ssss1 * tinv * 1.5d0
              ssss3 = ssss2 * tinv * 2.5d0
           end if

           ryerf = 1.d0/yerf
           y0 = dsqrt(ryerf)
           y1 = y0 * ryerf
           y2 = y1 * ryerf
           y3 = y2 * ryerf

           ssssx(0,k) = ssss0 * y0
           ssssx(1,k) = ssss1 * y1
           ssssx(2,k) = ssss2 * y2
           ssssx(3,k) = ssss3 * y3
        end do

        do k = 1, nc
           a0ss0 = ssssx(0,k)
           a0ss1 = ssssx(1,k)
           a0ss2 = ssssx(2,k)
           a0ss3 = ssssx(3,k)
           PCx = Px - C(1,k)
           PCy = Py - C(2,k)
           PCz = Pz - C(3,k)
           a0pzs2 = a0ss2*PAz - a0ss3*PCz
           a0pys2 = a0ss2*PAy - a0ss3*PCy
           a0pxs2 = a0ss2*PAx - a0ss3*PCx
           ai_a0ss12 = oo2zeta*( a0ss1 - a0ss2 )
           a0pzs1 = a0ss1*PAz - a0ss2*PCz
           a0dzzs1 = a0pzs1*PAz - a0pzs2*PCz + ai_a0ss12
           a0pys1 = a0ss1*PAy - a0ss2*PCy
           a0dyzs1 = a0pys1*PAz - a0pys2*PCz
           a0dyys1 = a0pys1*PAy - a0pys2*PCy + ai_a0ss12
           a0pxs1 = a0ss1*PAx - a0ss2*PCx
           a0dxzs1 = a0pxs1*PAz - a0pxs2*PCz
           a0dxys1 = a0pxs1*PAy - a0pxs2*PCy
           a0dxxs1 = a0pxs1*PAx - a0pxs2*PCx + ai_a0ss12
           ai_a0ss01 = oo2zeta*( a0ss0 - a0ss1 )
           a0pzs0 = a0ss0*PAz - a0ss1*PCz
           ai_a0pzs01 = oo2zeta*( a0pzs0 - a0pzs1 )
           a0dzzs0 = a0pzs0*PAz - a0pzs1*PCz + ai_a0ss01
           a0dzzpz0 = a0dzzpz0 + a0dzzs0*PBz - a0dzzs1*PCz + ai_a0pzs01*2.0d0
           a0dzzpy0 = a0dzzpy0 + a0dzzs0*PBy - a0dzzs1*PCy
           a0dzzpx0 = a0dzzpx0 + a0dzzs0*PBx - a0dzzs1*PCx
           a0pys0 = a0ss0*PAy - a0ss1*PCy
           ai_a0pys01 = oo2zeta*( a0pys0 - a0pys1 )
           a0dyzs0 = a0pys0*PAz - a0pys1*PCz
           a0dyzpz0 = a0dyzpz0 + a0dyzs0*PBz - a0dyzs1*PCz + ai_a0pys01
           a0dyzpy0 = a0dyzpy0 + a0dyzs0*PBy - a0dyzs1*PCy + ai_a0pzs01
           a0dyzpx0 = a0dyzpx0 + a0dyzs0*PBx - a0dyzs1*PCx
           a0dyys0 = a0pys0*PAy - a0pys1*PCy + ai_a0ss01
           a0dyypz0 = a0dyypz0 + a0dyys0*PBz - a0dyys1*PCz
           a0dyypy0 = a0dyypy0 + a0dyys0*PBy - a0dyys1*PCy + ai_a0pys01*2.0d0
           a0dyypx0 = a0dyypx0 + a0dyys0*PBx - a0dyys1*PCx
           a0pxs0 = a0ss0*PAx - a0ss1*PCx
           ai_a0pxs01 = oo2zeta*( a0pxs0 - a0pxs1 )
           a0dxzs0 = a0pxs0*PAz - a0pxs1*PCz
           a0dxzpz0 = a0dxzpz0 + a0dxzs0*PBz - a0dxzs1*PCz + ai_a0pxs01
           a0dxzpy0 = a0dxzpy0 + a0dxzs0*PBy - a0dxzs1*PCy
           a0dxzpx0 = a0dxzpx0 + a0dxzs0*PBx - a0dxzs1*PCx + ai_a0pzs01
           a0dxys0 = a0pxs0*PAy - a0pxs1*PCy
           a0dxypz0 = a0dxypz0 + a0dxys0*PBz - a0dxys1*PCz
           a0dxypy0 = a0dxypy0 + a0dxys0*PBy - a0dxys1*PCy + ai_a0pxs01
           a0dxypx0 = a0dxypx0 + a0dxys0*PBx - a0dxys1*PCx + ai_a0pys01
           a0dxxs0 = a0pxs0*PAx - a0pxs1*PCx + ai_a0ss01
           a0dxxpz0 = a0dxxpz0 + a0dxxs0*PBz - a0dxxs1*PCz
           a0dxxpy0 = a0dxxpy0 + a0dxxs0*PBy - a0dxxs1*PCy
           a0dxxpx0 = a0dxxpx0 + a0dxxs0*PBx - a0dxxs1*PCx + ai_a0pxs01*2.0d0
        end do

     end do

     v(1) = a0dxxpx0
     v(2) = a0dxxpy0
     v(3) = a0dxxpz0
     v(4) = a0dyypx0
     v(5) = a0dyypy0
     v(6) = a0dyypz0
     v(7) = a0dzzpx0
     v(8) = a0dzzpy0
     v(9) = a0dzzpz0
     v(10) = a0dxypx0*root3
     v(11) = a0dxypy0*root3
     v(12) = a0dxypz0*root3
     v(13) = a0dxzpx0*root3
     v(14) = a0dxzpy0*root3
     v(15) = a0dxzpz0*root3
     v(16) = a0dyzpx0*root3
     v(17) = a0dyzpy0*root3
     v(18) = a0dyzpz0*root3

     i = npsh(1,ij)
     j = npsh(2,ij)
     iao = nsh2ao(i)
     jao = nsh2ao(j)

    if (i < j) then
      do jx = jao,jao + 3-1
        do ix = iao,iao + 6-1
          key = ( ix-iao ) * 3 + jx-jao +1
          val(jx,ix) =  v(key)
        end do
      end do
    else
      do jx = jao,jao + 3-1
        do ix = iao,iao + 6-1
          key = ( ix-iao ) * 3 + jx-jao +1
          val(ix,jx) =  v(key)
        end do
      end do
    end if
  end do
!$omp end do
end subroutine os_nai_dp_gc_v1

!------------------------------------------------------------------------------
!> dd nuclear attraction integrals
!
subroutine os_nai_dd_gc_v1(&
     val,nao,&
     ishp,npsh,npsh2pr,nsh2atm,nsh2ao,xyz,ss,xiozetaa,zeta,&
     C,chg_c,rc2v,nc,&
     nshell,nshellp,nprimp,natom,fmt,myrank,nproc)
  use physical_constants_class
  use integral_fmt_class

  implicit real(8) (a-h,o-z)
  integer,intent(in)::nao
  real(8),intent(inout)::val(nao,nao)

  integer,intent(in)::nshell,nshellp,nprimp,natom
  integer,intent(in)::ishp(6),npsh(2,nshellp),npsh2pr(nshellp+1)
  integer,intent(in)::nsh2atm(nshell),nsh2ao(nshell)
  real(8),intent(in)::ss(nprimp),xiozetaa(nprimp),zeta(nprimp)
  real(8),intent(in)::xyz(3,natom)

  integer,intent(in)::nc
  real(8),intent(in)::C(3,nc),chg_c(nc),rc2v(nc)
  type(fmt_type),intent(in)::fmt
  integer,intent(in)::myrank,nproc

  real(8)::v(36)

  real(8),allocatable::ssssx(:,:)

  tosqrtpi = 2.d0/dsqrt(pi)
  allocate(ssssx(0:4,nc))

!$omp do schedule(dynamic)
  do ij = ishp(5)+myrank+1, ishp(6), nproc
     i = npsh(1,ij)
     j = npsh(2,ij)
     iatom = nsh2atm(i)
     jatom = nsh2atm(j)
     Ax = xyz(1,iatom)
     Ay = xyz(2,iatom)
     Az = xyz(3,iatom)
     Bx = xyz(1,jatom)
     By = xyz(2,jatom)
     Bz = xyz(3,jatom)
     ABx = Ax - Bx
     ABy = Ay - By
     ABz = Az - Bz

     a0dxxdxx0 = 0.0d0
     a0dxxdyy0 = 0.0d0
     a0dxxdzz0 = 0.0d0
     a0dxxdxy0 = 0.0d0
     a0dxxdxz0 = 0.0d0
     a0dxxdyz0 = 0.0d0
     a0dyydxx0 = 0.0d0
     a0dyydyy0 = 0.0d0
     a0dyydzz0 = 0.0d0
     a0dyydxy0 = 0.0d0
     a0dyydxz0 = 0.0d0
     a0dyydyz0 = 0.0d0
     a0dzzdxx0 = 0.0d0
     a0dzzdyy0 = 0.0d0
     a0dzzdzz0 = 0.0d0
     a0dzzdxy0 = 0.0d0
     a0dzzdxz0 = 0.0d0
     a0dzzdyz0 = 0.0d0
     a0dxydxx0 = 0.0d0
     a0dxydyy0 = 0.0d0
     a0dxydzz0 = 0.0d0
     a0dxydxy0 = 0.0d0
     a0dxydxz0 = 0.0d0
     a0dxydyz0 = 0.0d0
     a0dxzdxx0 = 0.0d0
     a0dxzdyy0 = 0.0d0
     a0dxzdzz0 = 0.0d0
     a0dxzdxy0 = 0.0d0
     a0dxzdxz0 = 0.0d0
     a0dxzdyz0 = 0.0d0
     a0dyzdxx0 = 0.0d0
     a0dyzdyy0 = 0.0d0
     a0dyzdzz0 = 0.0d0
     a0dyzdxy0 = 0.0d0
     a0dyzdxz0 = 0.0d0
     a0dyzdyz0 = 0.0d0

     do irs = npsh2pr(ij), npsh2pr(ij+1) - 1
        oss = ss(irs)
        xioza = xiozetaa(irs)
        zeta_ = zeta(irs)
        oo2zeta = 0.5d0 / zeta_
        PAx = -xioza*ABx
        PAy = -xioza*ABy
        PAz = -xioza*ABz
        PBx = PAx + ABx
        PBy = PAy + ABy
        PBz = PAz + ABz
        Px = PAx + Ax
        Py = PAy + Ay
        Pz = PAz + Az
        osstwosqrtzopi = oss * tosqrtpi * dsqrt(zeta_)

        do k = 1, nc
           PCx = Px - C(1,k)
           PCy = Py - C(2,k)
           PCz = Pz - C(3,k)
           ossx = -chg_c(k) * osstwosqrtzopi
           t = zeta_ * (PCx*PCx + PCy*PCy + PCz*PCz)
           yerf = rc2v(k) * zeta_ + 1.d0
           t = t / yerf

           if (t <= tf(5)) then
              it = idint((t + 0.025d0)*20.0d0)
              dt = 0.05d0*it - t
              it = it + 1
              dt4 = dt * 0.25d0
              dt5 = dt * 0.2d0
              dt6 = dt * 0.1666666666666666666d0
              dt2 = dt4 + dt4
              dt3 = dt6 + dt6
              ft  = (((((fmt%fdat(11,it)*dt6 + fmt%fdat(10,it))*dt5&
                  + fmt%fdat(9,it))*dt4 + fmt%fdat(8,it))*dt3&
                  + fmt%fdat(7,it))*dt2 + fmt%fdat(6,it))*dt + fmt%fdat(5,it)
              et  = ((((((ga6*dt + ga5)*dt + ga4)*dt&
                  + ga3)*dt + ga2)*dt + ga1)*dt + ga0) * fmt%edat(it)
              t2 = 2.d0 * t
              etcn = et * ossx
              ssss4 = ft * ossx
              ssss3 = (ssss4 * t2 + etcn) * rminv(4)
              ssss2 = (ssss3 * t2 + etcn) * rminv(3)
              ssss1 = (ssss2 * t2 + etcn) * rminv(2)
              ssss0 =  ssss1 * t2 + etcn
           else
              tinv = 1.d0/t
              ssss0 = dsqrt(pi*tinv) * ossx * 0.5d0
              ssss1 = ssss0 * tinv * 0.5d0
              ssss2 = ssss1 * tinv * 1.5d0
              ssss3 = ssss2 * tinv * 2.5d0
              ssss4 = ssss3 * tinv * 3.5d0
           end if

           ryerf = 1.d0/yerf
           y0 = dsqrt(ryerf)
           y1 = y0 * ryerf
           y2 = y1 * ryerf
           y3 = y2 * ryerf
           y4 = y3 * ryerf

           ssssx(0,k) = ssss0 * y0
           ssssx(1,k) = ssss1 * y1
           ssssx(2,k) = ssss2 * y2
           ssssx(3,k) = ssss3 * y3
           ssssx(4,k) = ssss4 * y4
        end do

        do k = 1, nc
           a0ss0 = ssssx(0,k)
           a0ss1 = ssssx(1,k)
           a0ss2 = ssssx(2,k)
           a0ss3 = ssssx(3,k)
           a0ss4 = ssssx(4,k)
           PCx = Px - C(1,k)
           PCy = Py - C(2,k)
           PCz = Pz - C(3,k)
           a0pzs3 = a0ss3*PAz - a0ss4*PCz
           a0pys3 = a0ss3*PAy - a0ss4*PCy
           a0pxs3 = a0ss3*PAx - a0ss4*PCx
           ai_a0ss23 = oo2zeta*( a0ss2 - a0ss3 )
           a0pzs2 = a0ss2*PAz - a0ss3*PCz
           a0dzzs2 = a0pzs2*PAz - a0pzs3*PCz + ai_a0ss23
           a0pys2 = a0ss2*PAy - a0ss3*PCy
           a0dyzs2 = a0pys2*PAz - a0pys3*PCz
           a0dyys2 = a0pys2*PAy - a0pys3*PCy + ai_a0ss23
           a0pxs2 = a0ss2*PAx - a0ss3*PCx
           a0dxzs2 = a0pxs2*PAz - a0pxs3*PCz
           a0dxys2 = a0pxs2*PAy - a0pxs3*PCy
           a0dxxs2 = a0pxs2*PAx - a0pxs3*PCx + ai_a0ss23
           ai_a0ss12 = oo2zeta*( a0ss1 - a0ss2 )
           a0pzs1 = a0ss1*PAz - a0ss2*PCz
           ai_a0pzs12 = oo2zeta*( a0pzs1 - a0pzs2 )
           a0dzzs1 = a0pzs1*PAz - a0pzs2*PCz + ai_a0ss12
           a0dzzpz1 = a0dzzs1*PBz - a0dzzs2*PCz + ai_a0pzs12*2.0d0
           a0dzzpy1 = a0dzzs1*PBy - a0dzzs2*PCy
           a0dzzpx1 = a0dzzs1*PBx - a0dzzs2*PCx
           a0pzpz1 = a0pzs1*PBz - a0pzs2*PCz + ai_a0ss12
           a0pzpy1 = a0pzs1*PBy - a0pzs2*PCy
           a0pzpx1 = a0pzs1*PBx - a0pzs2*PCx
           a0pys1 = a0ss1*PAy - a0ss2*PCy
           ai_a0pys12 = oo2zeta*( a0pys1 - a0pys2 )
           a0dyzs1 = a0pys1*PAz - a0pys2*PCz
           a0dyzpz1 = a0dyzs1*PBz - a0dyzs2*PCz + ai_a0pys12
           a0dyzpy1 = a0dyzs1*PBy - a0dyzs2*PCy + ai_a0pzs12
           a0dyzpx1 = a0dyzs1*PBx - a0dyzs2*PCx
           a0dyys1 = a0pys1*PAy - a0pys2*PCy + ai_a0ss12
           a0dyypz1 = a0dyys1*PBz - a0dyys2*PCz
           a0dyypy1 = a0dyys1*PBy - a0dyys2*PCy + ai_a0pys12*2.0d0
           a0dyypx1 = a0dyys1*PBx - a0dyys2*PCx
           a0pypz1 = a0pys1*PBz - a0pys2*PCz
           a0pypy1 = a0pys1*PBy - a0pys2*PCy + ai_a0ss12
           a0pypx1 = a0pys1*PBx - a0pys2*PCx
           a0pxs1 = a0ss1*PAx - a0ss2*PCx
           ai_a0pxs12 = oo2zeta*( a0pxs1 - a0pxs2 )
           a0dxzs1 = a0pxs1*PAz - a0pxs2*PCz
           a0dxzpz1 = a0dxzs1*PBz - a0dxzs2*PCz + ai_a0pxs12
           a0dxzpy1 = a0dxzs1*PBy - a0dxzs2*PCy
           a0dxzpx1 = a0dxzs1*PBx - a0dxzs2*PCx + ai_a0pzs12
           a0dxys1 = a0pxs1*PAy - a0pxs2*PCy
           a0dxypz1 = a0dxys1*PBz - a0dxys2*PCz
           a0dxypy1 = a0dxys1*PBy - a0dxys2*PCy + ai_a0pxs12
           a0dxypx1 = a0dxys1*PBx - a0dxys2*PCx + ai_a0pys12
           a0dxxs1 = a0pxs1*PAx - a0pxs2*PCx + ai_a0ss12
           a0dxxpz1 = a0dxxs1*PBz - a0dxxs2*PCz
           a0dxxpy1 = a0dxxs1*PBy - a0dxxs2*PCy
           a0dxxpx1 = a0dxxs1*PBx - a0dxxs2*PCx + ai_a0pxs12*2.0d0
           a0pxpz1 = a0pxs1*PBz - a0pxs2*PCz
           a0pxpy1 = a0pxs1*PBy - a0pxs2*PCy
           a0pxpx1 = a0pxs1*PBx - a0pxs2*PCx + ai_a0ss12
           ai_a0ss01 = oo2zeta*( a0ss0 - a0ss1 )
           a0pzs0 = a0ss0*PAz - a0ss1*PCz
           ai_a0pzs01 = oo2zeta*( a0pzs0 - a0pzs1 )
           a0dzzs0 = a0pzs0*PAz - a0pzs1*PCz + ai_a0ss01
           bj_a0dzzs01 = oo2zeta*( a0dzzs0 - a0dzzs1 )
           a0dzzpz0 = a0dzzs0*PBz - a0dzzs1*PCz + ai_a0pzs01*2.0d0
           a0dzzpy0 = a0dzzs0*PBy - a0dzzs1*PCy
           a0dzzdyy0 = a0dzzdyy0 + a0dzzpy0*PBy - a0dzzpy1*PCy + bj_a0dzzs01
           a0dzzpx0 = a0dzzs0*PBx - a0dzzs1*PCx
           a0dzzdxy0 = a0dzzdxy0 + a0dzzpx0*PBy - a0dzzpx1*PCy
           a0dzzdxx0 = a0dzzdxx0 + a0dzzpx0*PBx - a0dzzpx1*PCx + bj_a0dzzs01
           a0pzpz0 = a0pzs0*PBz - a0pzs1*PCz + ai_a0ss01
           ai_a0pzpz01 = oo2zeta*( a0pzpz0 - a0pzpz1 )
           a0dzzdzz0 = a0dzzdzz0 + a0dzzpz0*PBz - a0dzzpz1*PCz + bj_a0dzzs01 + ai_a0pzpz01*2.0d0
           a0pzpy0 = a0pzs0*PBy - a0pzs1*PCy
           ai_a0pzpy01 = oo2zeta*( a0pzpy0 - a0pzpy1 )
           a0dzzdyz0 = a0dzzdyz0 + a0dzzpy0*PBz - a0dzzpy1*PCz + ai_a0pzpy01*2.0d0
           a0pzpx0 = a0pzs0*PBx - a0pzs1*PCx
           ai_a0pzpx01 = oo2zeta*( a0pzpx0 - a0pzpx1 )
           a0dzzdxz0 = a0dzzdxz0 + a0dzzpx0*PBz - a0dzzpx1*PCz + ai_a0pzpx01*2.0d0
           a0pys0 = a0ss0*PAy - a0ss1*PCy
           ai_a0pys01 = oo2zeta*( a0pys0 - a0pys1 )
           a0dyzs0 = a0pys0*PAz - a0pys1*PCz
           bj_a0dyzs01 = oo2zeta*( a0dyzs0 - a0dyzs1 )
           a0dyzpz0 = a0dyzs0*PBz - a0dyzs1*PCz + ai_a0pys01
           a0dyzpy0 = a0dyzs0*PBy - a0dyzs1*PCy + ai_a0pzs01
           a0dyzdyy0 = a0dyzdyy0 + a0dyzpy0*PBy - a0dyzpy1*PCy + bj_a0dyzs01 + ai_a0pzpy01
           a0dyzpx0 = a0dyzs0*PBx - a0dyzs1*PCx
           a0dyzdxy0 = a0dyzdxy0 + a0dyzpx0*PBy - a0dyzpx1*PCy + ai_a0pzpx01
           a0dyzdxx0 = a0dyzdxx0 + a0dyzpx0*PBx - a0dyzpx1*PCx + bj_a0dyzs01
           a0dyys0 = a0pys0*PAy - a0pys1*PCy + ai_a0ss01
           bj_a0dyys01 = oo2zeta*( a0dyys0 - a0dyys1 )
           a0dyypz0 = a0dyys0*PBz - a0dyys1*PCz
           a0dyydzz0 = a0dyydzz0 + a0dyypz0*PBz - a0dyypz1*PCz + bj_a0dyys01
           a0dyypy0 = a0dyys0*PBy - a0dyys1*PCy + ai_a0pys01*2.0d0
           a0dyydyz0 = a0dyydyz0 + a0dyypy0*PBz - a0dyypy1*PCz
           a0dyypx0 = a0dyys0*PBx - a0dyys1*PCx
           a0dyydxz0 = a0dyydxz0 + a0dyypx0*PBz - a0dyypx1*PCz
           a0dyydxx0 = a0dyydxx0 + a0dyypx0*PBx - a0dyypx1*PCx + bj_a0dyys01
           a0pypz0 = a0pys0*PBz - a0pys1*PCz
           ai_a0pypz01 = oo2zeta*( a0pypz0 - a0pypz1 )
           a0dyzdzz0 = a0dyzdzz0 + a0dyzpz0*PBz - a0dyzpz1*PCz + bj_a0dyzs01 + ai_a0pypz01
           a0pypy0 = a0pys0*PBy - a0pys1*PCy + ai_a0ss01
           ai_a0pypy01 = oo2zeta*( a0pypy0 - a0pypy1 )
           a0dyzdyz0 = a0dyzdyz0 + a0dyzpy0*PBz - a0dyzpy1*PCz + ai_a0pypy01
           a0dyydyy0 = a0dyydyy0 + a0dyypy0*PBy - a0dyypy1*PCy + bj_a0dyys01 + ai_a0pypy01*2.0d0
           a0pypx0 = a0pys0*PBx - a0pys1*PCx
           ai_a0pypx01 = oo2zeta*( a0pypx0 - a0pypx1 )
           a0dyzdxz0 = a0dyzdxz0 + a0dyzpx0*PBz - a0dyzpx1*PCz + ai_a0pypx01
           a0dyydxy0 = a0dyydxy0 + a0dyypx0*PBy - a0dyypx1*PCy + ai_a0pypx01*2.0d0
           a0pxs0 = a0ss0*PAx - a0ss1*PCx
           ai_a0pxs01 = oo2zeta*( a0pxs0 - a0pxs1 )
           a0dxzs0 = a0pxs0*PAz - a0pxs1*PCz
           bj_a0dxzs01 = oo2zeta*( a0dxzs0 - a0dxzs1 )
           a0dxzpz0 = a0dxzs0*PBz - a0dxzs1*PCz + ai_a0pxs01
           a0dxzpy0 = a0dxzs0*PBy - a0dxzs1*PCy
           a0dxzdyy0 = a0dxzdyy0 + a0dxzpy0*PBy - a0dxzpy1*PCy + bj_a0dxzs01
           a0dxzpx0 = a0dxzs0*PBx - a0dxzs1*PCx + ai_a0pzs01
           a0dxzdxy0 = a0dxzdxy0 + a0dxzpx0*PBy - a0dxzpx1*PCy
           a0dxzdxx0 = a0dxzdxx0 + a0dxzpx0*PBx - a0dxzpx1*PCx + bj_a0dxzs01 + ai_a0pzpx01
           a0dxys0 = a0pxs0*PAy - a0pxs1*PCy
           bj_a0dxys01 = oo2zeta*( a0dxys0 - a0dxys1 )
           a0dxypz0 = a0dxys0*PBz - a0dxys1*PCz
           a0dxydzz0 = a0dxydzz0 + a0dxypz0*PBz - a0dxypz1*PCz + bj_a0dxys01
           a0dxypy0 = a0dxys0*PBy - a0dxys1*PCy + ai_a0pxs01
           a0dxydyz0 = a0dxydyz0 + a0dxypy0*PBz - a0dxypy1*PCz
           a0dxypx0 = a0dxys0*PBx - a0dxys1*PCx + ai_a0pys01
           a0dxydxz0 = a0dxydxz0 + a0dxypx0*PBz - a0dxypx1*PCz
           a0dxydxx0 = a0dxydxx0 + a0dxypx0*PBx - a0dxypx1*PCx + bj_a0dxys01 + ai_a0pypx01
           a0dxxs0 = a0pxs0*PAx - a0pxs1*PCx + ai_a0ss01
           bj_a0dxxs01 = oo2zeta*( a0dxxs0 - a0dxxs1 )
           a0dxxpz0 = a0dxxs0*PBz - a0dxxs1*PCz
           a0dxxdzz0 = a0dxxdzz0 + a0dxxpz0*PBz - a0dxxpz1*PCz + bj_a0dxxs01
           a0dxxpy0 = a0dxxs0*PBy - a0dxxs1*PCy
           a0dxxdyz0 = a0dxxdyz0 + a0dxxpy0*PBz - a0dxxpy1*PCz
           a0dxxdyy0 = a0dxxdyy0 + a0dxxpy0*PBy - a0dxxpy1*PCy + bj_a0dxxs01
           a0dxxpx0 = a0dxxs0*PBx - a0dxxs1*PCx + ai_a0pxs01*2.0d0
           a0dxxdxz0 = a0dxxdxz0 + a0dxxpx0*PBz - a0dxxpx1*PCz
           a0dxxdxy0 = a0dxxdxy0 + a0dxxpx0*PBy - a0dxxpx1*PCy
           a0pxpz0 = a0pxs0*PBz - a0pxs1*PCz
           ai_a0pxpz01 = oo2zeta*( a0pxpz0 - a0pxpz1 )
           a0dxzdzz0 = a0dxzdzz0 + a0dxzpz0*PBz - a0dxzpz1*PCz + bj_a0dxzs01 + ai_a0pxpz01
           a0pxpy0 = a0pxs0*PBy - a0pxs1*PCy
           ai_a0pxpy01 = oo2zeta*( a0pxpy0 - a0pxpy1 )
           a0dxzdyz0 = a0dxzdyz0 + a0dxzpy0*PBz - a0dxzpy1*PCz + ai_a0pxpy01
           a0dxydyy0 = a0dxydyy0 + a0dxypy0*PBy - a0dxypy1*PCy + bj_a0dxys01 + ai_a0pxpy01
           a0pxpx0 = a0pxs0*PBx - a0pxs1*PCx + ai_a0ss01
           ai_a0pxpx01 = oo2zeta*( a0pxpx0 - a0pxpx1 )
           a0dxzdxz0 = a0dxzdxz0 + a0dxzpx0*PBz - a0dxzpx1*PCz + ai_a0pxpx01
           a0dxydxy0 = a0dxydxy0 + a0dxypx0*PBy - a0dxypx1*PCy + ai_a0pxpx01
           a0dxxdxx0 = a0dxxdxx0 + a0dxxpx0*PBx - a0dxxpx1*PCx + bj_a0dxxs01 + ai_a0pxpx01*2.0d0
        end do

     end do

     v(1) = a0dxxdxx0
     v(2) = a0dxxdyy0
     v(3) = a0dxxdzz0
     v(4) = a0dxxdxy0*root3
     v(5) = a0dxxdxz0*root3
     v(6) = a0dxxdyz0*root3
     v(7) = a0dyydxx0
     v(8) = a0dyydyy0
     v(9) = a0dyydzz0
     v(10) = a0dyydxy0*root3
     v(11) = a0dyydxz0*root3
     v(12) = a0dyydyz0*root3
     v(13) = a0dzzdxx0
     v(14) = a0dzzdyy0
     v(15) = a0dzzdzz0
     v(16) = a0dzzdxy0*root3
     v(17) = a0dzzdxz0*root3
     v(18) = a0dzzdyz0*root3
     v(19) = a0dxydxx0*root3
     v(20) = a0dxydyy0*root3
     v(21) = a0dxydzz0*root3
     v(22) = a0dxydxy0*root3x2
     v(23) = a0dxydxz0*root3x2
     v(24) = a0dxydyz0*root3x2
     v(25) = a0dxzdxx0*root3
     v(26) = a0dxzdyy0*root3
     v(27) = a0dxzdzz0*root3
     v(28) = a0dxzdxy0*root3x2
     v(29) = a0dxzdxz0*root3x2
     v(30) = a0dxzdyz0*root3x2
     v(31) = a0dyzdxx0*root3
     v(32) = a0dyzdyy0*root3
     v(33) = a0dyzdzz0*root3
     v(34) = a0dyzdxy0*root3x2
     v(35) = a0dyzdxz0*root3x2
     v(36) = a0dyzdyz0*root3x2

     i = npsh(1,ij)
     j = npsh(2,ij)
     iao = nsh2ao(i)
     jao = nsh2ao(j)

     if (iao == jao) then
        do jx = jao,jao + 6-1
           do ix = jx,iao + 6-1
              key = ( ix-iao ) * 6 + jx-jao +1
              val(ix,jx) =  v(key)
           end do
        end do
     else
        do jx = jao,jao + 6-1
           do ix = iao,iao + 6-1
              key = ( ix-iao ) * 6 + jx-jao +1
              val(ix,jx) =  v(key)
           end do
        end do
     end if
  end do
!$omp end do
end subroutine os_nai_dd_gc_v1
