! -*- coding: utf-8 -*-
!
! Dual licensed under the MIT or GPL Version 3 license.
! See 'LICENSE.MIT' for the definition of the MIT license,
! and 'COPYING' for the definition of the GPL license.
!
!------------------------------------------------------------------------------
!> nuclear attraction integrals
!
! References:
!  [1] S.Obara and A.Saika, J.Chem.Phys. 84, 3963 (1986).
!
subroutine os_nai_pc_v1(v,nao,tables,atoms,ptcharges,fmt,myrank,nproc)
  use integral_tables_class
  use atoms_class
  use ptcharges_class
  use integral_fmt_class
  use pl_prof_class
  integer,intent(in)::nao
  real(8),intent(inout)::v(nao,nao)
  type(tables_type),intent(in)::tables
  type(atoms_type),intent(in)::atoms
  type(ptcharges_type),intent(in)::ptcharges
  type(fmt_type),intent(in)::fmt
  integer,intent(in)::myrank,nproc

!$omp parallel&
!$omp& default(private)&
!$omp& shared(v,nao,tables,atoms,ptcharges,fmt,myrank,nproc)
  call pl_prof_region_begin('os_nai_pc_ss_v1')
  call os_nai_ss_v1(&
       v,nao,&
       tables%ishp,tables%npsh,tables%npsh2pr,tables%nsh2atm,tables%nsh2ao,&
       atoms%xyz,tables%ovl,tables%bdz,tables%zet,&
       ptcharges%xyz,ptcharges%chg,ptcharges%num,&
       tables%nshell,tables%nshelp,tables%nprimp,atoms%natom,fmt,myrank,nproc)
  call pl_prof_region_end('os_nai_pc_ss_v1')

  call pl_prof_region_begin('os_nai_pc_ps_v1')
  call os_nai_ps_v1(&
       v,nao,&
       tables%ishp,tables%npsh,tables%npsh2pr,tables%nsh2atm,tables%nsh2ao,&
       atoms%xyz,tables%ovl,tables%bdz,tables%zet,&
       ptcharges%xyz,ptcharges%chg,ptcharges%num,&
       tables%nshell,tables%nshelp,tables%nprimp,atoms%natom,fmt,myrank,nproc)
  call pl_prof_region_end('os_nai_pc_ps_v1')

  call pl_prof_region_begin('os_nai_pc_pp_v1')
  call os_nai_pp_v1(&
       v,nao,&
       tables%ishp,tables%npsh,tables%npsh2pr,tables%nsh2atm,tables%nsh2ao,&
       atoms%xyz,tables%ovl,tables%bdz,tables%zet,&
       ptcharges%xyz,ptcharges%chg,ptcharges%num,&
       tables%nshell,tables%nshelp,tables%nprimp,atoms%natom,fmt,myrank,nproc)
  call pl_prof_region_end('os_nai_pc_pp_v1')

  call pl_prof_region_begin('os_nai_pc_ds_v1')
  call os_nai_ds_v1(&
       v,nao,&
       tables%ishp,tables%npsh,tables%npsh2pr,tables%nsh2atm,tables%nsh2ao,&
       atoms%xyz,tables%ovl,tables%bdz,tables%zet,&
       ptcharges%xyz,ptcharges%chg,ptcharges%num,&
       tables%nshell,tables%nshelp,tables%nprimp,atoms%natom,fmt,myrank,nproc)
  call pl_prof_region_end('os_nai_pc_ds_v1')

  call pl_prof_region_begin('os_nai_pc_dp_v1')
  call os_nai_dp_v1(&
       v,nao,&
       tables%ishp,tables%npsh,tables%npsh2pr,tables%nsh2atm,tables%nsh2ao,&
       atoms%xyz,tables%ovl,tables%bdz,tables%zet,&
       ptcharges%xyz,ptcharges%chg,ptcharges%num,&
       tables%nshell,tables%nshelp,tables%nprimp,atoms%natom,fmt,myrank,nproc)
  call pl_prof_region_end('os_nai_pc_dp_v1')

  call pl_prof_region_begin('os_nai_pc_dd_v1')
  call os_nai_dd_v1(&
       v,nao,&
       tables%ishp,tables%npsh,tables%npsh2pr,tables%nsh2atm,tables%nsh2ao,&
       atoms%xyz,tables%ovl,tables%bdz,tables%zet,&
       ptcharges%xyz,ptcharges%chg,ptcharges%num,&
       tables%nshell,tables%nshelp,tables%nprimp,atoms%natom,fmt,myrank,nproc)
  call pl_prof_region_end('os_nai_pc_dd_v1')
!$omp end parallel

end subroutine os_nai_pc_v1
