! -*- coding: utf-8 -*-
!
! Dual licensed under the MIT or GPL Version 3 license.
! See 'LICENSE.MIT' for the definition of the MIT license,
! and 'COPYING' for the definition of the GPL license.
!
!------------------------------------------------------------------------------
!> overlap integrals
!
! References:
!  [1] S.Obara and A.Saika, J.Chem.Phys. 84, 3963 (1986).
!
subroutine os_ovi_s1(s,nao,tables,atoms)
  use integral_tables_class
  use atoms_class
  use pl_prof_class
  integer,intent(in)::nao
  real(8),intent(inout)::s(nao,nao)
  type(tables_type),intent(in)::tables
  type(atoms_type),intent(in)::atoms

!!$  s(:,:) = 0.0d0

!$omp parallel&
!$omp& default(private)&
!$omp& shared(s,nao,tables,atoms)
  call pl_prof_region_begin('os_ovi_ss_s1')
  call os_ovi_ss_s1(&
       s,nao,&
       tables%ishp,tables%npsh,tables%npsh2pr,tables%nsh2atm,tables%nsh2ao,&
       atoms%xyz,tables%ovl,tables%bdz,tables%rzet,&
       tables%nshell,tables%nshelp,tables%nprimp,atoms%natom)
  call pl_prof_region_end('os_ovi_ss_s1')

  call pl_prof_region_begin('os_ovi_ps_s1')
  call os_ovi_ps_s1(&
       s,nao,&
       tables%ishp,tables%npsh,tables%npsh2pr,tables%nsh2atm,tables%nsh2ao,&
       atoms%xyz,tables%ovl,tables%bdz,tables%rzet,&
       tables%nshell,tables%nshelp,tables%nprimp,atoms%natom)
  call pl_prof_region_end('os_ovi_ps_s1')

  call pl_prof_region_begin('os_ovi_pp_s1')
  call os_ovi_pp_s1(&
       s,nao,&
       tables%ishp,tables%npsh,tables%npsh2pr,tables%nsh2atm,tables%nsh2ao,&
       atoms%xyz,tables%ovl,tables%bdz,tables%rzet,&
       tables%nshell,tables%nshelp,tables%nprimp,atoms%natom)
  call pl_prof_region_end('os_ovi_pp_s1')

  call pl_prof_region_begin('os_ovi_ds_s1')
  call os_ovi_ds_s1(&
       s,nao,&
       tables%ishp,tables%npsh,tables%npsh2pr,tables%nsh2atm,tables%nsh2ao,&
       atoms%xyz,tables%ovl,tables%bdz,tables%rzet,&
       tables%nshell,tables%nshelp,tables%nprimp,atoms%natom)
  call pl_prof_region_end('os_ovi_ds_s1')

  call pl_prof_region_begin('os_ovi_dp_s1')
  call os_ovi_dp_s1(&
       s,nao,&
       tables%ishp,tables%npsh,tables%npsh2pr,tables%nsh2atm,tables%nsh2ao,&
       atoms%xyz,tables%ovl,tables%bdz,tables%rzet,&
       tables%nshell,tables%nshelp,tables%nprimp,atoms%natom)
  call pl_prof_region_end('os_ovi_dp_s1')

  call pl_prof_region_begin('os_ovi_dd_s1')
  call os_ovi_dd_s1(&
       s,nao,&
       tables%ishp,tables%npsh,tables%npsh2pr,tables%nsh2atm,tables%nsh2ao,&
       atoms%xyz,tables%ovl,tables%bdz,tables%rzet,&
       tables%nshell,tables%nshelp,tables%nprimp,atoms%natom)
  call pl_prof_region_end('os_ovi_dd_s1')
!$omp end parallel

end subroutine os_ovi_s1

!------------------------------------------------------------------------------
!> ss overlap integrals
!
subroutine os_ovi_ss_s1(&
     val,nao,&
     ishp,npsh,npsh2pr,nsh2atm,nsh2ao,xyz,ss,xiozetaa,oozeta,&
     nshell,nshellp,nprimp,natom)
  use physical_constants_class

  implicit real(8) (a-h,o-z)
  integer,intent(in)::nao
  real(8),intent(inout)::val(nao,nao)

  integer,intent(in)::nshell,nshellp,nprimp,natom
  integer,intent(in)::ishp(6),npsh(2,nshellp),npsh2pr(nshellp+1)
  integer,intent(in)::nsh2atm(nshell),nsh2ao(nshell)
  real(8),intent(in)::ss(nprimp),xiozetaa(nprimp),oozeta(nprimp)
  real(8),intent(in)::xyz(3,natom)

  real(8)::v(1)

!$omp do schedule(dynamic)
  do ij = 1, ishp(1)
     i = npsh(1,ij)
     j = npsh(2,ij)

     oss = 0.0d0

     do irs = npsh2pr(ij), npsh2pr(ij+1) - 1
        oss = oss + ss(irs)

     end do

     !v(1) = oss

     iao = nsh2ao(i)
     jao = nsh2ao(j)

     !val(iao,jao) = v(1)
     val(iao,jao) = oss
  end do
!$omp end do nowait
end subroutine os_ovi_ss_s1

!------------------------------------------------------------------------------
!> ps overlap integrals
!
subroutine os_ovi_ps_s1(&
     val,nao,&
     ishp,npsh,npsh2pr,nsh2atm,nsh2ao,xyz,ss,xiozetaa,oozeta,&
     nshell,nshellp,nprimp,natom)
  use physical_constants_class

  implicit real(8) (a-h,o-z)
  integer,intent(in)::nao
  real(8),intent(inout)::val(nao,nao)

  integer,intent(in)::nshell,nshellp,nprimp,natom
  integer,intent(in)::ishp(6),npsh(2,nshellp),npsh2pr(nshellp+1)
  integer,intent(in)::nsh2atm(nshell),nsh2ao(nshell)
  real(8),intent(in)::ss(nprimp),xiozetaa(nprimp),oozeta(nprimp)
  real(8),intent(in)::xyz(3,natom)

  real(8)::v(3)

!$omp do schedule(dynamic)
  do ij = ishp(1)+1, ishp(2)
     i = npsh(1,ij)
     j = npsh(2,ij)
     iatom = nsh2atm(i)
     jatom = nsh2atm(j)
     ABx = xyz(1,iatom) - xyz(1,jatom)
     ABy = xyz(2,iatom) - xyz(2,jatom)
     ABz = xyz(3,iatom) - xyz(3,jatom)

     opxs = 0.0d0
     opys = 0.0d0
     opzs = 0.0d0

     do irs = npsh2pr(ij), npsh2pr(ij+1) - 1
        oss = ss(irs)
        xioza = xiozetaa(irs)
        PAx = -xioza*ABx
        PAy = -xioza*ABy
        PAz = -xioza*ABz

        opzs = opzs + oss*PAz
        opys = opys + oss*PAy
        opxs = opxs + oss*PAx
     end do

     !v(1) = opxs
     !v(2) = opys
     !v(3) = opzs

     iao = nsh2ao(i)
     jao = nsh2ao(j)

     if (i < j) then
        val(jao,iao  ) = opxs
        val(jao,iao+1) = opys
        val(jao,iao+2) = opzs
     else
        val(iao  ,jao) = opxs
        val(iao+1,jao) = opys
        val(iao+2,jao) = opzs
     end if
  end do
!$omp end do nowait
end subroutine os_ovi_ps_s1

!------------------------------------------------------------------------------
!> pp overlap integrals
!
subroutine os_ovi_pp_s1(&
     val,nao,&
     ishp,npsh,npsh2pr,nsh2atm,nsh2ao,xyz,ss,xiozetaa,oozeta,&
     nshell,nshellp,nprimp,natom)
  use physical_constants_class

  implicit real(8) (a-h,o-z)
  integer,intent(in)::nao
  real(8),intent(inout)::val(nao,nao)

  integer,intent(in)::nshell,nshellp,nprimp,natom
  integer,intent(in)::ishp(6),npsh(2,nshellp),npsh2pr(nshellp+1)
  integer,intent(in)::nsh2atm(nshell),nsh2ao(nshell)
  real(8),intent(in)::ss(nprimp),xiozetaa(nprimp),oozeta(nprimp)
  real(8),intent(in)::xyz(3,natom)

  real(8)::v(9)

!$omp do schedule(dynamic)
  do ij = ishp(2)+1, ishp(3)
     i = npsh(1,ij)
     j = npsh(2,ij)
     iatom = nsh2atm(i)
     jatom = nsh2atm(j)
     ABx = xyz(1,iatom) - xyz(1,jatom)
     ABy = xyz(2,iatom) - xyz(2,jatom)
     ABz = xyz(3,iatom) - xyz(3,jatom)

     opxpx = 0.0d0
     opxpy = 0.0d0
     opxpz = 0.0d0
     opypx = 0.0d0
     opypy = 0.0d0
     opypz = 0.0d0
     opzpx = 0.0d0
     opzpy = 0.0d0
     opzpz = 0.0d0

     do irs = npsh2pr(ij), npsh2pr(ij+1) - 1
        oss = ss(irs)
        oo2zeta = 0.5d0*oozeta(irs)
        xioza = xiozetaa(irs)
        PAx = -xioza*ABx
        PAy = -xioza*ABy
        PAz = -xioza*ABz
        PBx = PAx + ABx
        PBy = PAy + ABy
        PBz = PAz + ABz

        ai_oss = oo2zeta*oss
        opzs = oss*PAz
        opzpz = opzpz + opzs*PBz + ai_oss
        opzpy = opzpy + opzs*PBy
        opzpx = opzpx + opzs*PBx
        opys = oss*PAy
        opypz = opypz + opys*PBz
        opypy = opypy + opys*PBy + ai_oss
        opypx = opypx + opys*PBx
        opxs = oss*PAx
        opxpz = opxpz + opxs*PBz
        opxpy = opxpy + opxs*PBy
        opxpx = opxpx + opxs*PBx + ai_oss
     end do

     v(1) = opxpx
     v(2) = opxpy
     v(3) = opxpz
     v(4) = opypx
     v(5) = opypy
     v(6) = opypz
     v(7) = opzpx
     v(8) = opzpy
     v(9) = opzpz

     iao = nsh2ao(i)
     jao = nsh2ao(j)

     if (iao == jao) then
        do jx = jao,jao + 3-1
           do ix = jx,iao + 3-1
              key = (ix-iao)*3 + jx-jao + 1
              val(ix,jx) =  v(key)
           end do
        end do
     else
        do jx = jao,jao + 3-1
           do ix = iao,iao + 3-1
              key = (ix-iao)*3 + jx-jao + 1
              val(ix,jx) =  v(key)
           end do
        end do
     end if
  end do
!$omp end do nowait
end subroutine os_ovi_pp_s1

!------------------------------------------------------------------------------
!> ds overlap integrals
!
subroutine os_ovi_ds_s1(&
     val,nao,&
     ishp,npsh,npsh2pr,nsh2atm,nsh2ao,xyz,ss,xiozetaa,oozeta,&
     nshell,nshellp,nprimp,natom)
  use physical_constants_class

  implicit real(8) (a-h,o-z)
  integer,intent(in)::nao
  real(8),intent(inout)::val(nao,nao)

  integer,intent(in)::nshell,nshellp,nprimp,natom
  integer,intent(in)::ishp(6),npsh(2,nshellp),npsh2pr(nshellp+1)
  integer,intent(in)::nsh2atm(nshell),nsh2ao(nshell)
  real(8),intent(in)::ss(nprimp),xiozetaa(nprimp),oozeta(nprimp)
  real(8),intent(in)::xyz(3,natom)

  real(8)::v(6)

!$omp do schedule(dynamic)
  do ij = ishp(3)+1, ishp(4)
     i = npsh(1,ij)
     j = npsh(2,ij)
     iatom = nsh2atm(i)
     jatom = nsh2atm(j)
     ABx = xyz(1,iatom) - xyz(1,jatom)
     ABy = xyz(2,iatom) - xyz(2,jatom)
     ABz = xyz(3,iatom) - xyz(3,jatom)

     odxxs = 0.0d0
     odyys = 0.0d0
     odzzs = 0.0d0
     odxys = 0.0d0
     odxzs = 0.0d0
     odyzs = 0.0d0

     do irs = npsh2pr(ij), npsh2pr(ij+1) - 1
        oss = ss(irs)
        oo2zeta = 0.5d0*oozeta(irs)
        xioza = xiozetaa(irs)
        PAx = -xioza*ABx
        PAy = -xioza*ABy
        PAz = -xioza*ABz

        ai_oss = oo2zeta*oss
        opzs = oss*PAz
        odzzs = odzzs + opzs*PAz + ai_oss
        opys = oss*PAy
        odyzs = odyzs + opys*PAz
        odyys = odyys + opys*PAy + ai_oss
        opxs = oss*PAx
        odxzs = odxzs + opxs*PAz
        odxys = odxys + opxs*PAy
        odxxs = odxxs + opxs*PAx + ai_oss
     end do

     v(1) = odxxs
     v(2) = odyys
     v(3) = odzzs
     v(4) = odxys*root3
     v(5) = odxzs*root3
     v(6) = odyzs*root3

     iao = nsh2ao(i)
     jao = nsh2ao(j)

     if (i < j) then
        do ix = iao,iao + 6-1
           key = ix-iao + 1
           val(jao,ix) =  v(key)
        end do
     else
        do ix = iao,iao + 6-1
           key = ix-iao + 1
           val(ix,jao) =  v(key)
        end do
     end if
  end do
!$omp end do nowait
end subroutine os_ovi_ds_s1

!------------------------------------------------------------------------------
!> dp overlap integrals
!
subroutine os_ovi_dp_s1(&
     val,nao,&
     ishp,npsh,npsh2pr,nsh2atm,nsh2ao,xyz,ss,xiozetaa,oozeta,&
     nshell,nshellp,nprimp,natom)
  use physical_constants_class

  implicit real(8) (a-h,o-z)
  integer,intent(in)::nao
  real(8),intent(inout)::val(nao,nao)

  integer,intent(in)::nshell,nshellp,nprimp,natom
  integer,intent(in)::ishp(6),npsh(2,nshellp),npsh2pr(nshellp+1)
  integer,intent(in)::nsh2atm(nshell),nsh2ao(nshell)
  real(8),intent(in)::ss(nprimp),xiozetaa(nprimp),oozeta(nprimp)
  real(8),intent(in)::xyz(3,natom)

  real(8)::v(18)

!$omp do schedule(dynamic)
  do ij = ishp(4)+1, ishp(5)
     i = npsh(1,ij)
     j = npsh(2,ij)
     iatom = nsh2atm(i)
     jatom = nsh2atm(j)
     ABx = xyz(1,iatom) - xyz(1,jatom)
     ABy = xyz(2,iatom) - xyz(2,jatom)
     ABz = xyz(3,iatom) - xyz(3,jatom)

     odxxpx = 0.0d0
     odxxpy = 0.0d0
     odxxpz = 0.0d0
     odyypx = 0.0d0
     odyypy = 0.0d0
     odyypz = 0.0d0
     odzzpx = 0.0d0
     odzzpy = 0.0d0
     odzzpz = 0.0d0
     odxypx = 0.0d0
     odxypy = 0.0d0
     odxypz = 0.0d0
     odxzpx = 0.0d0
     odxzpy = 0.0d0
     odxzpz = 0.0d0
     odyzpx = 0.0d0
     odyzpy = 0.0d0
     odyzpz = 0.0d0

     do irs = npsh2pr(ij), npsh2pr(ij+1) - 1
        oss = ss(irs)
        oo2zeta = 0.5d0*oozeta(irs)
        xioza = xiozetaa(irs)
        PAx = -xioza*ABx
        PAy = -xioza*ABy
        PAz = -xioza*ABz
        PBx = PAx + ABx
        PBy = PAy + ABy
        PBz = PAz + ABz

        ai_oss = oo2zeta*oss
        opzs = oss*PAz
        ai_opzs = oo2zeta*opzs
        odzzs = opzs*PAz + ai_oss
        odzzpz = odzzpz + odzzs*PBz + ai_opzs*2.0d0
        odzzpy = odzzpy + odzzs*PBy
        odzzpx = odzzpx + odzzs*PBx
        opys = oss*PAy
        ai_opys = oo2zeta*opys
        odyzs = opys*PAz
        odyzpz = odyzpz + odyzs*PBz + ai_opys
        odyzpy = odyzpy + odyzs*PBy + ai_opzs
        odyzpx = odyzpx + odyzs*PBx
        odyys = opys*PAy + ai_oss
        odyypz = odyypz + odyys*PBz
        odyypy = odyypy + odyys*PBy + ai_opys*2.0d0
        odyypx = odyypx + odyys*PBx
        opxs = oss*PAx
        ai_opxs = oo2zeta*opxs
        odxzs = opxs*PAz
        odxzpz = odxzpz + odxzs*PBz + ai_opxs
        odxzpy = odxzpy + odxzs*PBy
        odxzpx = odxzpx + odxzs*PBx + ai_opzs
        odxys = opxs*PAy
        odxypz = odxypz + odxys*PBz
        odxypy = odxypy + odxys*PBy + ai_opxs
        odxypx = odxypx + odxys*PBx + ai_opys
        odxxs = opxs*PAx + ai_oss
        odxxpz = odxxpz + odxxs*PBz
        odxxpy = odxxpy + odxxs*PBy
        odxxpx = odxxpx + odxxs*PBx + ai_opxs*2.0d0
     end do

     v(1) = odxxpx
     v(2) = odxxpy
     v(3) = odxxpz
     v(4) = odyypx
     v(5) = odyypy
     v(6) = odyypz
     v(7) = odzzpx
     v(8) = odzzpy
     v(9) = odzzpz
     v(10) = odxypx*root3
     v(11) = odxypy*root3
     v(12) = odxypz*root3
     v(13) = odxzpx*root3
     v(14) = odxzpy*root3
     v(15) = odxzpz*root3
     v(16) = odyzpx*root3
     v(17) = odyzpy*root3
     v(18) = odyzpz*root3

     iao = nsh2ao(i)
     jao = nsh2ao(j)

     if (i < j) then
        do jx = jao,jao + 3-1
           do ix = iao,iao + 6-1
              key = ( ix-iao ) * 3 + jx-jao +1
              val(jx,ix) =  v(key)
           end do
        end do
     else
        do jx = jao,jao + 3-1
           do ix = iao,iao + 6-1
              key = ( ix-iao ) * 3 + jx-jao +1
              val(ix,jx) =  v(key)
           end do
        end do
     end if
  end do
!$omp end do nowait
end subroutine os_ovi_dp_s1

!------------------------------------------------------------------------------
!> dd overlap integrals
!
subroutine os_ovi_dd_s1(&
     val,nao,&
     ishp,npsh,npsh2pr,nsh2atm,nsh2ao,xyz,ss,xiozetaa,oozeta,&
     nshell,nshellp,nprimp,natom)
  use physical_constants_class

  implicit real(8) (a-h,o-z)
  integer,intent(in)::nao
  real(8),intent(inout)::val(nao,nao)

  integer,intent(in)::nshell,nshellp,nprimp,natom
  integer,intent(in)::ishp(6),npsh(2,nshellp),npsh2pr(nshellp+1)
  integer,intent(in)::nsh2atm(nshell),nsh2ao(nshell)
  real(8),intent(in)::ss(nprimp),xiozetaa(nprimp),oozeta(nprimp)
  real(8),intent(in)::xyz(3,natom)

  real(8)::v(36)

!$omp do schedule(dynamic)
  do ij = ishp(5)+1, ishp(6)
     i = npsh(1,ij)
     j = npsh(2,ij)
     iatom = nsh2atm(i)
     jatom = nsh2atm(j)
     ABx = xyz(1,iatom) - xyz(1,jatom)
     ABy = xyz(2,iatom) - xyz(2,jatom)
     ABz = xyz(3,iatom) - xyz(3,jatom)

     odxxdxx = 0.0d0
     odxxdyy = 0.0d0
     odxxdzz = 0.0d0
     odxxdxy = 0.0d0
     odxxdxz = 0.0d0
     odxxdyz = 0.0d0
     odyydxx = 0.0d0
     odyydyy = 0.0d0
     odyydzz = 0.0d0
     odyydxy = 0.0d0
     odyydxz = 0.0d0
     odyydyz = 0.0d0
     odzzdxx = 0.0d0
     odzzdyy = 0.0d0
     odzzdzz = 0.0d0
     odzzdxy = 0.0d0
     odzzdxz = 0.0d0
     odzzdyz = 0.0d0
     odxydxx = 0.0d0
     odxydyy = 0.0d0
     odxydzz = 0.0d0
     odxydxy = 0.0d0
     odxydxz = 0.0d0
     odxydyz = 0.0d0
     odxzdxx = 0.0d0
     odxzdyy = 0.0d0
     odxzdzz = 0.0d0
     odxzdxy = 0.0d0
     odxzdxz = 0.0d0
     odxzdyz = 0.0d0
     odyzdxx = 0.0d0
     odyzdyy = 0.0d0
     odyzdzz = 0.0d0
     odyzdxy = 0.0d0
     odyzdxz = 0.0d0
     odyzdyz = 0.0d0

     do irs = npsh2pr(ij), npsh2pr(ij+1) - 1
        oss = ss(irs)
        oo2zeta = 0.5d0*oozeta(irs)
        xioza = xiozetaa(irs)
        PAx = -xioza*ABx
        PAy = -xioza*ABy
        PAz = -xioza*ABz
        PBx = PAx + ABx
        PBy = PAy + ABy
        PBz = PAz + ABz

        ai_oss = oo2zeta*oss
        opzs = oss*PAz
        ai_opzs = oo2zeta*opzs
        odzzs = opzs*PAz + ai_oss
        bj_odzzs = oo2zeta*odzzs
        odzzpz = odzzs*PBz + ai_opzs*2.0d0
        odzzpy = odzzs*PBy
        odzzdyy = odzzdyy + odzzpy*PBy + bj_odzzs
        odzzpx = odzzs*PBx
        odzzdxy = odzzdxy + odzzpx*PBy
        odzzdxx = odzzdxx + odzzpx*PBx + bj_odzzs
        opzpz = opzs*PBz + ai_oss
        ai_opzpz = oo2zeta*opzpz
        odzzdzz = odzzdzz + odzzpz*PBz + bj_odzzs + ai_opzpz*2.0d0
        opzpy = opzs*PBy
        ai_opzpy = oo2zeta*opzpy
        odzzdyz = odzzdyz + odzzpy*PBz + ai_opzpy*2.0d0
        opzpx = opzs*PBx
        ai_opzpx = oo2zeta*opzpx
        odzzdxz = odzzdxz + odzzpx*PBz + ai_opzpx*2.0d0
        opys = oss*PAy
        ai_opys = oo2zeta*opys
        odyzs = opys*PAz
        bj_odyzs = oo2zeta*odyzs
        odyzpz = odyzs*PBz + ai_opys
        odyzpy = odyzs*PBy + ai_opzs
        odyzdyy = odyzdyy + odyzpy*PBy + bj_odyzs + ai_opzpy
        odyzpx = odyzs*PBx
        odyzdxy = odyzdxy + odyzpx*PBy + ai_opzpx
        odyzdxx = odyzdxx + odyzpx*PBx + bj_odyzs
        odyys = opys*PAy + ai_oss
        bj_odyys = oo2zeta*odyys
        odyypz = odyys*PBz
        odyydzz = odyydzz + odyypz*PBz + bj_odyys
        odyypy = odyys*PBy + ai_opys*2.0d0
        odyydyz = odyydyz + odyypy*PBz
        odyypx = odyys*PBx
        odyydxz = odyydxz + odyypx*PBz
        odyydxx = odyydxx + odyypx*PBx + bj_odyys
        opypz = opys*PBz
        ai_opypz = oo2zeta*opypz
        odyzdzz = odyzdzz + odyzpz*PBz + bj_odyzs + ai_opypz
        opypy = opys*PBy + ai_oss
        ai_opypy = oo2zeta*opypy
        odyzdyz = odyzdyz + odyzpy*PBz + ai_opypy
        odyydyy = odyydyy + odyypy*PBy + bj_odyys + ai_opypy*2.0d0
        opypx = opys*PBx
        ai_opypx = oo2zeta*opypx
        odyzdxz = odyzdxz + odyzpx*PBz + ai_opypx
        odyydxy = odyydxy + odyypx*PBy + ai_opypx*2.0d0
        opxs = oss*PAx
        ai_opxs = oo2zeta*opxs
        odxzs = opxs*PAz
        bj_odxzs = oo2zeta*odxzs
        odxzpz = odxzs*PBz + ai_opxs
        odxzpy = odxzs*PBy
        odxzdyy = odxzdyy + odxzpy*PBy + bj_odxzs
        odxzpx = odxzs*PBx + ai_opzs
        odxzdxy = odxzdxy + odxzpx*PBy
        odxzdxx = odxzdxx + odxzpx*PBx + bj_odxzs + ai_opzpx
        odxys = opxs*PAy
        bj_odxys = oo2zeta*odxys
        odxypz = odxys*PBz
        odxydzz = odxydzz + odxypz*PBz + bj_odxys
        odxypy = odxys*PBy + ai_opxs
        odxydyz = odxydyz + odxypy*PBz
        odxypx = odxys*PBx + ai_opys
        odxydxz = odxydxz + odxypx*PBz
        odxydxx = odxydxx + odxypx*PBx + bj_odxys + ai_opypx
        odxxs = opxs*PAx + ai_oss
        bj_odxxs = oo2zeta*odxxs
        odxxpz = odxxs*PBz
        odxxdzz = odxxdzz + odxxpz*PBz + bj_odxxs
        odxxpy = odxxs*PBy
        odxxdyz = odxxdyz + odxxpy*PBz
        odxxdyy = odxxdyy + odxxpy*PBy + bj_odxxs
        odxxpx = odxxs*PBx + ai_opxs*2.0d0
        odxxdxz = odxxdxz + odxxpx*PBz
        odxxdxy = odxxdxy + odxxpx*PBy
        opxpz = opxs*PBz
        ai_opxpz = oo2zeta*opxpz
        odxzdzz = odxzdzz + odxzpz*PBz + bj_odxzs + ai_opxpz
        opxpy = opxs*PBy
        ai_opxpy = oo2zeta*opxpy
        odxzdyz = odxzdyz + odxzpy*PBz + ai_opxpy
        odxydyy = odxydyy + odxypy*PBy + bj_odxys + ai_opxpy
        opxpx = opxs*PBx + ai_oss
        ai_opxpx = oo2zeta*opxpx
        odxzdxz = odxzdxz + odxzpx*PBz + ai_opxpx
        odxydxy = odxydxy + odxypx*PBy + ai_opxpx
        odxxdxx = odxxdxx + odxxpx*PBx + bj_odxxs + ai_opxpx*2.0d0
     end do

     v(1) = odxxdxx
     v(2) = odxxdyy
     v(3) = odxxdzz
     v(4) = odxxdxy*root3
     v(5) = odxxdxz*root3
     v(6) = odxxdyz*root3
     v(7) = odyydxx
     v(8) = odyydyy
     v(9) = odyydzz
     v(10) = odyydxy*root3
     v(11) = odyydxz*root3
     v(12) = odyydyz*root3
     v(13) = odzzdxx
     v(14) = odzzdyy
     v(15) = odzzdzz
     v(16) = odzzdxy*root3
     v(17) = odzzdxz*root3
     v(18) = odzzdyz*root3
     v(19) = odxydxx*root3
     v(20) = odxydyy*root3
     v(21) = odxydzz*root3
     v(22) = odxydxy*root3x2
     v(23) = odxydxz*root3x2
     v(24) = odxydyz*root3x2
     v(25) = odxzdxx*root3
     v(26) = odxzdyy*root3
     v(27) = odxzdzz*root3
     v(28) = odxzdxy*root3x2
     v(29) = odxzdxz*root3x2
     v(30) = odxzdyz*root3x2
     v(31) = odyzdxx*root3
     v(32) = odyzdyy*root3
     v(33) = odyzdzz*root3
     v(34) = odyzdxy*root3x2
     v(35) = odyzdxz*root3x2
     v(36) = odyzdyz*root3x2

     iao = nsh2ao(i)
     jao = nsh2ao(j)

     if (iao == jao) then
        do jx = jao,jao + 6-1
           do ix = jx,iao + 6-1
              key = ( ix-iao ) * 6 + jx-jao +1
              val(ix,jx) =  v(key)
           end do
        end do
     else
        do jx = jao,jao + 6-1
           do ix = iao,iao + 6-1
              key = ( ix-iao ) * 6 + jx-jao +1
              val(ix,jx) =  v(key)
           end do
        end do
     end if
  end do
!$omp end do nowait
end subroutine os_ovi_dd_s1
