! -*- coding: utf-8 -*-
!
! Dual licensed under the MIT or GPL Version 3 license.
! See 'LICENSE.MIT' for the definition of the MIT license,
! and 'COPYING' for the definition of the GPL license.
!
!> (ss|ss) integrals
!
! Target class:
!    (ss|ss)
! Applying the horizontal recurrence relation:
! Classes to be formes by contraction of primitive integrals:
!    (ss|ss)
! Applying the vertical recurrence relation:
!    [ss|ss]
!
! References:
!  [1] S.Obara and A.Saika, J.Chem.Phys. 84, 3963 (1986).
!  [2] M.Head-Gordon and J.A.Pople, J.Chem.Phys. 89, 5777 (1988).
!
subroutine os_ssss_ov5(&
     fj,fk,d,nao,&
     swz_ijij,oozeta,nprims2_ij,sab,ab,p,pa,npsh_ij,nij,&
     swz_klkl,ooeta,nprims2_kl,scd,cd,q,qc,npsh_kl,nkl,&
     mxprim2,nshell,nsh2ao,fmt,&
     nteiv2,erithr,myrank,nproc)
  use physical_constants_class
  use integral_fmt_class
  use omp_lib

  implicit real(8) (a-h,o-z)

  integer,intent(in)::myrank,nproc
  integer,intent(in)::nao
  real(8),intent(in)::d(nao,nao)
  real(8),intent(inout)::fj(nao,nao),fk(nao,nao)

  integer(8),intent(inout)::nteiv2
  real(8),intent(in)::erithr

  integer,intent(in)::nshell
  integer,intent(in)::nsh2ao(nshell)
  type(fmt_type),intent(in)::fmt

  integer,intent(in)::nij
  integer,intent(in)::nkl
  integer,intent(in)::mxprim2
  real(8),intent(in)::ab(3,nij),P(3,mxprim2,nij),PA(3,mxprim2,nij)
  real(8),intent(in)::cd(3,nkl),Q(3,mxprim2,nkl),QC(3,mxprim2,nkl)
  real(8),intent(in)::swz_ijij(nij),oozeta(mxprim2,nij),sab(mxprim2,nij)
  real(8),intent(in)::swz_klkl(nkl),ooeta (mxprim2,nkl),scd(mxprim2,nkl)
  integer,intent(in)::nprims2_ij(nij),npsh_ij(2,nij)
  integer,intent(in)::nprims2_kl(nkl),npsh_kl(2,nkl)

  real(8)::ec(1)
  integer::ntask

  real(8),allocatable::QPs(:,:),rhos(:),ssssx(:,:)
  real(8),allocatable::ts(:),QP2s(:)

  nteiv2 = 0

  allocate(QPs(3,mxprim2),rhos(mxprim2),ssssx(0:0,mxprim2))
  allocate(ts(mxprim2),QP2s(mxprim2))

  erithr2 = erithr*erithr
  tosqrtpi = 2.d0/dsqrt(pi)
!!$  ntask = -1

!$omp do schedule(dynamic)
!!$!$omp do
  do ij = myrank+1, nij, nproc
!!$  do ij = 1, nij
     swz_ijij_ = swz_ijij(ij)
     nrs = nprims2_ij(ij)

     do kl = 1, ij
        if (swz_ijij_*swz_klkl(kl) < erithr2) cycle
!!$        ntask = mod(ntask+1,nproc); if (ntask /= myrank) cycle
        ntu = nprims2_kl(kl)

        ! Initializing VRR variables
        ssss = 0.0d0

        do irs = 1, nrs
           Px = P(1,irs,ij)
           Py = P(2,irs,ij)
           Pz = P(3,irs,ij)
           Sp = sab(irs,ij)
           ooz = oozeta(irs,ij)

           do itu = 1, ntu
              Qx = Q(1,itu,kl)
              Qy = Q(2,itu,kl)
              Qz = Q(3,itu,kl)
              ooe = ooeta(itu,kl)
              QPx = Qx - Px
              QPy = Qy - Py
              QPz = Qz - Pz
              QP2 = QPx*QPx + QPy*QPy + QPz*QPz
              rho = 1.0d0/(ooz + ooe)
              t = rho * QP2
              ts(itu) = t
              QP2s(itu)  = QP2
              rhos(itu)  = rho
           end do

           do itu = 1, ntu
              t = ts(itu)
              Sq = scd(itu,kl)
              rho = rhos(itu)
              QP2 = QP2s(itu)

              if (t <= tf(1)) then
                 it = idint((t + 0.025d0)*20.0d0)
                 dt = 0.05d0*it - t
                 it = it + 1
                 dt4 = dt * 0.25d0
                 dt5 = dt * 0.2d0
                 dt6 = dt * 0.1666666666666666666d0
                 dt2 = dt4 + dt4
                 dt3 = dt6 + dt6
                 ft  = (((((fmt%fdat(7,it)*dt6 + fmt%fdat(6,it))*dt5&
                     + fmt%fdat(5,it))*dt4 + fmt%fdat(4,it))*dt3&
                     + fmt%fdat(3,it))*dt2 + fmt%fdat(2,it))*dt + fmt%fdat(1,it)
                 ssss0 = ft * dsqrt(rho) * Sq * tosqrtpi * Sp
              else
                 ssss0 = dsqrt(1.0d0/QP2) * Sq * Sp
              end if

              ssssx(0,itu) = ssss0
           end do

           do itu = 1, ntu
              !ssss0 = ssssx(0,itu)
              !ssss = ssss + ssss0
              ssss = ssss + ssssx(0,itu)
           end do
        end do

        !ec(1) = ssss

        i = npsh_ij(1,ij)
        j = npsh_ij(2,ij)
        k = npsh_kl(1,kl)
        l = npsh_kl(2,kl)

        ir = nsh2ao(i)
        is = nsh2ao(j)
        it = nsh2ao(k)
        iu = nsh2ao(l)

        !if (dabs(ec(1)) <= erithr) cycle
        if (dabs(ssss) <= erithr) cycle
        nteiv2 = nteiv2 + 1
!!$        index_teiv(1,nteiv2) = ir
!!$        index_teiv(2,nteiv2) = is
!!$        index_teiv(3,nteiv2) = it
!!$        index_teiv(4,nteiv2) = iu
!!$        !teiv(nteiv2) = ec(1)
!!$        teiv(nteiv2) = ssss
        irx = ir
        isx = is
        itx = it
        iux = iu
        eri = ssss
        if (irx == isx) eri = 0.5d0*eri
        if (itx == iux) eri = 0.5d0*eri
        if (((irx == itx) .and. (isx == iux)) .or.&
             ((irx == iux) .and. (isx == itx))) eri = 0.5d0*eri
        !write(6,'(''ov5_eri:'',i8,'':'',4i6,f18.10)')nteiv2,irx,isx,itx,iux,eri
        fj(irx,isx) = fj(irx,isx) + d(itx,iux)*eri
        fj(itx,iux) = fj(itx,iux) + d(irx,isx)*eri
        fk(irx,itx) = fk(irx,itx) + d(isx,iux)*eri
        fk(irx,iux) = fk(irx,iux) + d(isx,itx)*eri
        fk(isx,itx) = fk(isx,itx) + d(irx,iux)*eri
        fk(isx,iux) = fk(isx,iux) + d(irx,itx)*eri
     end do
  end do
!$omp end do nowait
end subroutine os_ssss_ov5
