! -*- coding: utf-8 -*-
!
! Dual licensed under the MIT or GPL Version 3 license.
! See 'LICENSE.MIT' for the definition of the MIT license,
! and 'COPYING' for the definition of the GPL license.
!
!> BMSデータ標準化ライブラリクラス
!!
module pl_bms_class
#ifdef PL_BMS_HDF
  use pl_bms_impl_hdf_class
#else
#ifdef PL_BMS_XML
  use pl_bms_impl_xml_class
#else
  use pl_bms_impl_none_class
#endif
#endif
  implicit none
!!$  private

!!$  integer,public,parameter::PL_BMS_NAME_MAX_STR = 16
!!$
!!$  type::pl_bms_type
!!$    character(len=PL_BMS_NAME_MAX_STR)::name
!!$    integer::id
!!$  end type

  !> データの保存
  interface pl_bms_write
    module procedure pl_bms_write_int4_val
    module procedure pl_bms_write_int4_array_1d
    module procedure pl_bms_write_int4_array_2d
    module procedure pl_bms_write_int4_array_3d
    module procedure pl_bms_write_int8_val
    module procedure pl_bms_write_real8_val
    module procedure pl_bms_write_real8_array_1d
    module procedure pl_bms_write_real8_array_2d
    module procedure pl_bms_write_real8_array_3d
    module procedure pl_bms_write_real8_array_4d
    module procedure pl_bms_write_str_val
    module procedure pl_bms_write_str_array_1d
  end interface

  !> データの取得
  interface pl_bms_read
    module procedure pl_bms_read_int4_val
    module procedure pl_bms_read_int4_array_1d
    module procedure pl_bms_read_int4_array_2d
    module procedure pl_bms_read_int4_array_3d
    module procedure pl_bms_read_int8_val
    module procedure pl_bms_read_real8_val
    module procedure pl_bms_read_real8_array_1d
    module procedure pl_bms_read_real8_array_2d
    module procedure pl_bms_read_real8_array_3d
    module procedure pl_bms_read_real8_array_4d
    module procedure pl_bms_read_str_val
    module procedure pl_bms_read_str_array_1d
  end interface

contains
  !-----------------------------------------------------------------------------
  !> データの存在問い合わせ
  function pl_bms_is_exists(url, name, id) result(is_exists)
    character(len=*),intent(in)::url
    character(len=*),intent(in)::name
    integer,intent(in)::id
    logical::is_exists

    is_exists = pl_bms_is_exists_impl(url, name, id)

  end function pl_bms_is_exists
  
  !-----------------------------------------------------------------------------
  !> データ次元数の取得
  function pl_bms_get_ndim(url, name, id) result(ndim)
    character(len=*),intent(in)::url
    character(len=*),intent(in)::name
    integer,intent(in)::id
    integer::ndim

    ndim = pl_bms_get_ndim_impl(url, name, id)

  end function pl_bms_get_ndim

  !-----------------------------------------------------------------------------
  !> データシェイプの取得
  subroutine pl_bms_get_shape(lshape, ndim, url, name, id, istat)
    integer,intent(in)::ndim
    integer,intent(inout)::lshape(ndim)
    character(len=*),intent(in)::url
    character(len=*),intent(in)::name
    integer,intent(in)::id
    integer,intent(inout)::istat

    call pl_bms_get_shape_impl(lshape, ndim, url, name, id, istat)

  end subroutine pl_bms_get_shape
  
  !-----------------------------------------------------------------------------
  !> データの保存
  subroutine pl_bms_write_int4_val(val, url, name, id, unit, istat)
    integer,intent(in)::val
    character(len=*),intent(in)::url
    character(len=*),intent(in)::name
    integer,intent(in)::id
    character(len=*),intent(in)::unit
    integer,intent(inout)::istat

    call pl_bms_write_int4_val_impl(val, url, name, id, unit, istat)

  end subroutine pl_bms_write_int4_val

  !-----------------------------------------------------------------------------
  !> データの保存
  subroutine pl_bms_write_int4_array_1d(&
       data, n1, url, name, id, unit, istat)
    integer,intent(in)::n1
    integer,intent(in)::data(n1)
    character(len=*),intent(in)::url
    character(len=*),intent(in)::name
    integer,intent(in)::id
    character(len=*),intent(in)::unit
    integer,intent(inout)::istat

    call pl_bms_write_int4_array_1d_impl(&
         data, n1, url, name, id, unit, istat)

  end subroutine pl_bms_write_int4_array_1d

  !-----------------------------------------------------------------------------
  !> データの保存
  subroutine pl_bms_write_int4_array_2d(&
       data, ld1, n1, n2, url, name, id, unit, istat)
    integer,intent(in)::ld1
    integer,intent(in)::n1, n2
    integer,intent(in)::data(ld1,n2)
    character(len=*),intent(in)::url
    character(len=*),intent(in)::name
    integer,intent(in)::id
    character(len=*),intent(in)::unit
    integer,intent(inout)::istat

    call pl_bms_write_int4_array_2d_impl(&
         data, ld1, n1, n2, url, name, id, unit, istat)

  end subroutine pl_bms_write_int4_array_2d

  !-----------------------------------------------------------------------------
  !> データの保存
  subroutine pl_bms_write_int4_array_3d(&
       data, ld1, ld2, n1, n2, n3, url, name, id, unit, istat)
    integer,intent(in)::ld1, ld2
    integer,intent(in)::n1, n2, n3
    integer,intent(in)::data(ld1,ld2,n3)
    character(len=*),intent(in)::url
    character(len=*),intent(in)::name
    integer,intent(in)::id
    character(len=*),intent(in)::unit
    integer,intent(inout)::istat

    call pl_bms_write_int4_array_3d_impl(&
         data, ld1, ld2, n1, n2, n3, url, name, id, unit, istat)

  end subroutine pl_bms_write_int4_array_3d

  !-----------------------------------------------------------------------------
  !> データの保存
  subroutine pl_bms_write_int8_val(val, url, name, id, unit, istat)
    integer(8),intent(in)::val
    character(len=*),intent(in)::url
    character(len=*),intent(in)::name
    integer,intent(in)::id
    character(len=*),intent(in)::unit
    integer,intent(inout)::istat

    call pl_bms_write_int8_val_impl(val, url, name, id, unit, istat)

  end subroutine pl_bms_write_int8_val

  !-----------------------------------------------------------------------------
  !> データの保存
  subroutine pl_bms_write_real8_val(val, url, name, id, unit, istat)
    real(8),intent(in)::val
    character(len=*),intent(in)::url
    character(len=*),intent(in)::name
    integer,intent(in)::id
    character(len=*),intent(in)::unit
    integer,intent(inout)::istat

    call pl_bms_write_real8_val_impl(val, url, name, id, unit, istat)

  end subroutine pl_bms_write_real8_val

  !-----------------------------------------------------------------------------
  !> データの保存
  subroutine pl_bms_write_real8_array_1d(&
       data, n1, url, name, id, unit, istat)
    integer,intent(in)::n1
    real(8),intent(in)::data(n1)
    character(len=*),intent(in)::url
    character(len=*),intent(in)::name
    integer,intent(in)::id
    character(len=*),intent(in)::unit
    integer,intent(inout)::istat

    call pl_bms_write_real8_array_1d_impl(&
         data, n1, url, name, id, unit, istat)

  end subroutine pl_bms_write_real8_array_1d

  !-----------------------------------------------------------------------------
  !> データの保存
  subroutine pl_bms_write_real8_array_2d(&
       data, ld1, n1, n2, url, name, id, unit, istat)
    integer,intent(in)::ld1
    integer,intent(in)::n1, n2
    real(8),intent(in)::data(ld1,n2)
    character(len=*),intent(in)::url
    character(len=*),intent(in)::name
    integer,intent(in)::id
    character(len=*),intent(in)::unit
    integer,intent(inout)::istat

    call pl_bms_write_real8_array_2d_impl(&
         data, ld1, n1, n2, url, name, id, unit, istat)

  end subroutine pl_bms_write_real8_array_2d

  !-----------------------------------------------------------------------------
  !> データの保存
  subroutine pl_bms_write_real8_array_3d(&
       data, ld1, ld2, n1, n2, n3, url, name, id, unit, istat)
    integer,intent(in)::ld1, ld2
    integer,intent(in)::n1, n2, n3
    real(8),intent(in)::data(ld1,ld2,n3)
    character(len=*),intent(in)::url
    character(len=*),intent(in)::name
    integer,intent(in)::id
    character(len=*),intent(in)::unit
    integer,intent(inout)::istat

    call pl_bms_write_real8_array_3d_impl(&
         data, ld1, ld2, n1, n2, n3, url, name, id, unit, istat)

  end subroutine pl_bms_write_real8_array_3d

  !-----------------------------------------------------------------------------
  !> データの保存
  subroutine pl_bms_write_real8_array_4d(&
       data, ld1, ld2, ld3, n1, n2, n3, n4, url, name, id, unit, istat)
    integer,intent(in)::ld1, ld2, ld3
    integer,intent(in)::n1, n2, n3, n4
    real(8),intent(in)::data(ld1,ld2,ld3,n4)
    character(len=*),intent(in)::url
    character(len=*),intent(in)::name
    integer,intent(in)::id
    character(len=*),intent(in)::unit
    integer,intent(inout)::istat

    call pl_bms_write_real8_array_4d_impl(&
         data, ld1, ld2, ld3, n1, n2, n3, n4, url, name, id, unit, istat)

  end subroutine pl_bms_write_real8_array_4d

  !-----------------------------------------------------------------------------
  !> データの保存
  subroutine pl_bms_write_str_val(data, url, name, id, unit, istat)
    character(len=*),intent(in)::data
    character(len=*),intent(in)::url
    character(len=*),intent(in)::name
    integer,intent(in)::id
    character(len=*),intent(in)::unit
    integer,intent(inout)::istat

    call pl_bms_write_str_val_impl(data, url, name, id, unit, istat)

  end subroutine pl_bms_write_str_val

  !-----------------------------------------------------------------------------
  !> データの保存
  subroutine pl_bms_write_str_array_1d(&
       data, n1, url, name, id, unit, istat)
    integer,intent(in)::n1
    character(len=*),intent(in)::data(n1)
    character(len=*),intent(in)::url
    character(len=*),intent(in)::name
    integer,intent(in)::id
    character(len=*),intent(in)::unit
    integer,intent(inout)::istat

    call pl_bms_write_str_array_1d_impl(&
         data, n1, url, name, id, unit, istat)

  end subroutine pl_bms_write_str_array_1d

  !-----------------------------------------------------------------------------
  !> データの取得
  subroutine pl_bms_read_int4_val(val, url, name, id, istat)
    integer,intent(inout)::val
    character(len=*),intent(in)::url
    character(len=*),intent(in)::name
    integer,intent(in)::id
    integer,intent(inout)::istat

    call pl_bms_read_int4_val_impl(val, url, name, id, istat)

  end subroutine pl_bms_read_int4_val

  !-----------------------------------------------------------------------------
  !> データの取得
  subroutine pl_bms_read_int4_array_1d(&
       data, n1, url, name, id, istat)
    integer,intent(in)::n1
    integer,intent(inout)::data(n1)
    character(len=*),intent(in)::url
    character(len=*),intent(in)::name
    integer,intent(in)::id
    integer,intent(inout)::istat

    call pl_bms_read_int4_array_1d_impl(data, n1, url, name, id, istat)

  end subroutine pl_bms_read_int4_array_1d

  !-----------------------------------------------------------------------------
  !> データの取得
  subroutine pl_bms_read_int4_array_2d(&
       data, ld1, n1, n2, url, name, id, istat)
    integer,intent(in)::ld1
    integer,intent(in)::n1, n2
    integer,intent(inout)::data(ld1,n2)
    character(len=*),intent(in)::url
    character(len=*),intent(in)::name
    integer,intent(in)::id
    integer,intent(inout)::istat

    call pl_bms_read_int4_array_2d_impl(&
         data, ld1, n1, n2, url, name, id, istat)

  end subroutine pl_bms_read_int4_array_2d

  !-----------------------------------------------------------------------------
  !> データの取得
  subroutine pl_bms_read_int4_array_3d(&
       data, ld1, ld2, n1, n2, n3, url, name, id, istat)
    integer,intent(in)::ld1, ld2
    integer,intent(in)::n1, n2, n3
    integer,intent(inout)::data(ld1,ld2,n3)
    character(len=*),intent(in)::url
    character(len=*),intent(in)::name
    integer,intent(in)::id
    integer,intent(inout)::istat

    call pl_bms_read_int4_array_3d_impl(&
         data, ld1, ld2, n1, n2, n3, url, name, id, istat)

  end subroutine pl_bms_read_int4_array_3d

  !-----------------------------------------------------------------------------
  !> データの取得
  subroutine pl_bms_read_int8_val(val, url, name, id, istat)
    integer(8),intent(inout)::val
    character(len=*),intent(in)::url
    character(len=*),intent(in)::name
    integer,intent(in)::id
    integer,intent(inout)::istat

    call pl_bms_read_int8_val_impl(val, url, name, id, istat)

  end subroutine pl_bms_read_int8_val

  !-----------------------------------------------------------------------------
  !> データの取得
  subroutine pl_bms_read_real8_val(val, url, name, id, istat)
    real(8),intent(inout)::val
    character(len=*),intent(in)::url
    character(len=*),intent(in)::name
    integer,intent(in)::id
    integer,intent(inout)::istat

    call pl_bms_read_real8_val_impl(val, url, name, id, istat)

  end subroutine pl_bms_read_real8_val

  !-----------------------------------------------------------------------------
  !> データの取得
  subroutine pl_bms_read_real8_array_1d(&
       data, n1, url, name, id, istat)
    integer,intent(in)::n1
    real(8),intent(inout)::data(n1)
    character(len=*),intent(in)::url
    character(len=*),intent(in)::name
    integer,intent(in)::id
    integer,intent(inout)::istat

    call pl_bms_read_real8_array_1d_impl(data, n1, url, name, id, istat)

  end subroutine pl_bms_read_real8_array_1d

  !-----------------------------------------------------------------------------
  !> データの取得
  subroutine pl_bms_read_real8_array_2d(&
       data, ld1, n1, n2, url, name, id, istat)
    integer,intent(in)::ld1
    integer,intent(in)::n1, n2
    real(8),intent(inout)::data(ld1,n2)
    character(len=*),intent(in)::url
    character(len=*),intent(in)::name
    integer,intent(in)::id
    integer,intent(inout)::istat

    call pl_bms_read_real8_array_2d_impl(&
         data, ld1, n1, n2, url, name, id, istat)

  end subroutine pl_bms_read_real8_array_2d

  !-----------------------------------------------------------------------------
  !> データの取得
  subroutine pl_bms_read_real8_array_3d(&
       data, ld1, ld2, n1, n2, n3, url, name, id, istat)
    integer,intent(in)::ld1, ld2
    integer,intent(in)::n1, n2, n3
    real(8),intent(inout)::data(ld1,ld2,n3)
    character(len=*),intent(in)::url
    character(len=*),intent(in)::name
    integer,intent(in)::id
    integer,intent(inout)::istat

    call pl_bms_read_real8_array_3d_impl(&
         data, ld1, ld2, n1, n2, n3, url, name, id, istat)

  end subroutine pl_bms_read_real8_array_3d

  !-----------------------------------------------------------------------------
  !> データの取得
  subroutine pl_bms_read_real8_array_4d(&
       data, ld1, ld2, ld3, n1, n2, n3, n4, url, name, id, istat)
    integer,intent(in)::ld1, ld2, ld3
    integer,intent(in)::n1, n2, n3, n4
    real(8),intent(inout)::data(ld1,ld2,ld3,n4)
    character(len=*),intent(in)::url
    character(len=*),intent(in)::name
    integer,intent(in)::id
    integer,intent(inout)::istat

    call pl_bms_read_real8_array_4d_impl(&
         data, ld1, ld2, ld3, n1, n2, n3, n4, url,name,id, istat)

  end subroutine pl_bms_read_real8_array_4d

  !-----------------------------------------------------------------------------
  !> データの取得
  subroutine pl_bms_read_str_val(data, url, name, id, istat)
    character(len=*),intent(inout)::data
    character(len=*),intent(in)::url
    character(len=*),intent(in)::name
    integer,intent(in)::id
    integer,intent(inout)::istat

    call pl_bms_read_str_val_impl(data, url, name, id, istat)

  end subroutine pl_bms_read_str_val

  !-----------------------------------------------------------------------------
  !> データの取得
  subroutine pl_bms_read_str_array_1d(&
       data, n1, url, name, id, istat)
    integer,intent(in)::n1
    character(len=*),intent(inout)::data(n1)
    character(len=*),intent(in)::url
    character(len=*),intent(in)::name
    integer,intent(in)::id
    integer,intent(inout)::istat

    call pl_bms_read_str_array_1d_impl(data, n1, url, name, id, istat)

  end subroutine pl_bms_read_str_array_1d

  !-----------------------------------------------------------------------------
end module pl_bms_class
