! -*- coding: utf-8 -*-
!
! Dual licensed under the MIT or GPL Version 3 license.
! See 'LICENSE.MIT' for the definition of the MIT license,
! and 'COPYING' for the definition of the GPL license.
!
!> BMSデータ標準化ライブラリクラスのHDF5実装
!!
module pl_bms_impl_hdf_class
#ifdef PL_BMS_HDF
  use bms_hdf5 !< BMS HDF5
  use hdf5
#endif
  use pl_bms_status_class
  use pl_bms_helper_class
  implicit none
!!$  private

!!$  public::pl_bms_is_exists_file_impl
!!$  public::pl_bms_is_exists_impl
!!$  public::pl_bms_get_ndim_impl
!!$  public::pl_bms_get_shape_impl
!!$  public::pl_bms_get_size_impl

!!$  !> データの保存
!!$  interface pl_bms_write_impl
!!$    module procedure pl_bms_write_int4_val_impl
!!$    module procedure pl_bms_write_int4_array_1d_impl
!!$    module procedure pl_bms_write_int4_array_2d_impl
!!$    module procedure pl_bms_write_int4_array_3d_impl
!!$    module procedure pl_bms_write_int8_val_impl
!!$    module procedure pl_bms_write_real8_val_impl
!!$    module procedure pl_bms_write_real8_array_1d_impl
!!$    module procedure pl_bms_write_real8_array_2d_impl
!!$    module procedure pl_bms_write_real8_array_3d_impl
!!$    module procedure pl_bms_write_real8_array_4d_impl
!!$    module procedure pl_bms_write_str_val_impl
!!$    module procedure pl_bms_write_str_array_1d_impl
!!$  end interface
!!$
!!$  !> データの取得
!!$  interface pl_bms_read_impl
!!$    module procedure pl_bms_read_int4_val_impl
!!$    module procedure pl_bms_read_int4_array_1d_impl
!!$    module procedure pl_bms_read_int4_array_2d_impl
!!$    module procedure pl_bms_read_int4_array_3d_impl
!!$    module procedure pl_bms_read_int8_val_impl
!!$    module procedure pl_bms_read_real8_val_impl
!!$    module procedure pl_bms_read_real8_array_1d_impl
!!$    module procedure pl_bms_read_real8_array_2d_impl
!!$    module procedure pl_bms_read_real8_array_3d_impl
!!$    module procedure pl_bms_read_real8_array_4d_impl
!!$    module procedure pl_bms_read_str_val_impl
!!$    module procedure pl_bms_read_str_array_1d_impl
!!$  end interface

  character(len=*),parameter::PL_BMS_FILE_EXT     = '.h5'

  character(len=*),parameter::PL_BMS_PROGRAM_NAME = ''
  character(len=*),parameter::PL_BMS_TITLE        = ''

contains
  !-----------------------------------------------------------------------------
  !> ファイル存在問い合わせ
  function pl_bms_is_exists_file_impl(url) result(is_exists)
    character(len=*),intent(in)::url
    logical::is_exists

#ifdef PL_BMS_HDF
    character(len=len(url)+len(PL_BMS_FILE_EXT))::file_name
    integer(HID_T)::file_id
    integer::info

    is_exists = .false.
    file_name = pl_bms_helper_get_file_name(url, PL_BMS_FILE_EXT)
    call h5open_f(info)
    call h5eset_auto_f(0, info)! Turn error print off
    call h5fopen_f(file_name, H5F_ACC_RDWR_F, file_id, info)
    write(6,*) 'info=',info
    if (info == 0) then
       is_exists = .true.
       call h5fclose_f(file_id, info)
    end if
    call h5eset_auto_f(1, info)! Turn error print on
    call h5close_f(info)
#else
    is_exists = .true.
#endif
  end function pl_bms_is_exists_file_impl

  !-----------------------------------------------------------------------------
  !> データ存在問い合わせ
  function pl_bms_is_exists_impl(url, name, id) result(is_exists)
    character(len=*),intent(in)::url
    character(len=*),intent(in)::name
    integer,intent(in)::id
    logical::is_exists

#ifdef PL_BMS_HDF
    character(len=len(url)+len(PL_BMS_FILE_EXT))::file_name
    integer(BH_FILE_T)::file_id
    integer(BH_DATA_T)::data_id
    integer(BH_ERROR_T)::info

    is_exists = .false.
    file_name = pl_bms_helper_get_file_name(url, PL_BMS_FILE_EXT)
    call h5open_f(info)
    call h5eset_auto_f(0, info)! Turn error print off
    call open_file(file_name, 'r', file_id); if (file_id < 0) return
    call open_data(file_id, name, data_id)
    if (data_id > 0) then
       call close_data(data_id, info); if (info /= 0) return
       is_exists = .true.
    end if
    call close_file(file_id, info); if (info /= 0) return
    call h5eset_auto_f(1, info)! Turn error print on
    call h5close_f(info)
#else
    is_exists = .true.
#endif
  end function pl_bms_is_exists_impl

  !-----------------------------------------------------------------------------
  !> データ次元数の取得
  function pl_bms_get_ndim_impl(url, name, id) result(ndim)
    character(len=*),intent(in)::url
    character(len=*),intent(in)::name
    integer,intent(in)::id
    integer::ndim

#ifdef PL_BMS_HDF
    character(len=len(url)+len(PL_BMS_FILE_EXT))::file_name
    integer(BH_FILE_T)::file_id
    integer(BH_DATA_T)::data_id
    integer(BH_ERROR_T)::info

    ndim = PL_BMS_DATA_UNKNOWN
    file_name = pl_bms_helper_get_file_name(url, PL_BMS_FILE_EXT)
    call h5open_f(info)
    !call h5eset_auto_f(1, info)! Turn error print on
    call open_file(file_name, 'r', file_id); if (file_id < 0) return
    call open_data(file_id, name, data_id); if (data_id < 0) return
    call bms_hdf5_get_shape(data_id, ndim, 0, info)
    call close_data(data_id, info); if (info /= 0) return
    call close_file(file_id, info); if (info /= 0) return
    call h5close_f(info)
#endif
  end function pl_bms_get_ndim_impl

  !-----------------------------------------------------------------------------
  !> データシェイプの取得
  subroutine pl_bms_get_shape_impl(dims, ndim, url, name, id, istat)
#ifdef PL_BMS_HDF
    use h5lt
#endif
    integer,intent(in)::ndim
    integer,intent(inout)::dims(ndim)
    character(len=*),intent(in)::url
    character(len=*),intent(in)::name
    integer,intent(in)::id
    integer,intent(inout)::istat

#ifdef PL_BMS_HDF
    character(len=len(url)+len(PL_BMS_FILE_EXT))::file_name
    integer(BH_FILE_T)::file_id
    integer(BH_DATA_T)::data_id
    integer(HID_T)::loc_id
    integer(BH_ERROR_T)::info
    type(bms_shape)::dshape
    integer::ndim_tmp
    integer,allocatable::dims_tmp(:)
    integer::i

    integer(hsize_t)::dimsr(ndim)
    integer::type_class
    integer(size_t)::type_size

    istat = -1
    file_name = pl_bms_helper_get_file_name(url, PL_BMS_FILE_EXT)
    call h5open_f(info)
    !call h5eset_auto_f(1, info)! Turn error print on
    call open_file(file_name, 'r', file_id); if (file_id < 0) return
    call open_data(file_id, name, data_id); if (data_id < 0) return

    loc_id = file_id
    call h5ltget_dataset_info_f(&
         loc_id, name, dimsr, type_class, type_size, info)
    if (info /= 0) return
    do i = 1, ndim
       dims(i) = dimsr(ndim - i + 1)
    end do

!!$    call bms_hdf5_get_shape(data_id, ndim, 0, info)
!!$    allocate(dims_tmp(ndim))
!!$    call bms_hdf5_get_shape(data_id, ndim, dims_tmp, info)
!!$    do i = 1, ndim
!!$       dims(i) = dims_tmp(i)
!!$    end do
!!$    deallocate(dims_tmp)

!!$    dshape = BMS_SHAPE_NULL
!!$    call get_shape(data_id, dshape, info); if (info /= 0) return
!!$    if (ndim /= dshape%ndim) return
!!$    allocate(dshape%dims(dshape%ndim))
!!$    call get_shape(data_id, dshape, info); if (info /= 0) return
!!$    do i = 1, ndim
!!$       dims(i) = dshape%dims(i)
!!$    end do
!!$    if(associated(dshape%dims)) deallocate(dshape%dims)

    call close_data(data_id, info); if (info /= 0) return
    call close_file(file_id, info); if (info /= 0) return
    call h5close_f(info)
#endif
    istat = 0
  end subroutine pl_bms_get_shape_impl

  !-----------------------------------------------------------------------------
  !> データの保存
  subroutine pl_bms_write_int4_val_impl(data, url, name, id, unit, istat)
    integer,intent(in)::data
    character(len=*),intent(in)::url
    character(len=*),intent(in)::name
    integer,intent(in)::id
    character(len=*),intent(in)::unit
    integer,intent(inout)::istat

#ifdef PL_BMS_HDF
    integer::a(1)

    a(1) = data
    call pl_bms_write_int4_array_1d_impl(a, 1, url, name, id, unit, istat)

#endif
    istat = 0
  end subroutine pl_bms_write_int4_val_impl
  
  !-----------------------------------------------------------------------------
  !> データの保存
  subroutine pl_bms_write_int4_array_1d_impl(&
       data, n1, url, name, id, unit, istat)
    integer,intent(in)::n1
    integer,intent(in)::data(n1)
    character(len=*),intent(in)::url
    character(len=*),intent(in)::name
    integer,intent(in)::id
    character(len=*),intent(in)::unit
    integer,intent(inout)::istat

#ifdef PL_BMS_HDF
    character(len=len(url)+len(PL_BMS_FILE_EXT))::file_name
    integer(BH_FILE_T)::file_id
    integer(BH_DATA_T)::data_id
    integer(HID_T)::file_idd
    integer(HID_T)::data_idd
    integer(BH_ERROR_T)::info, info2
    integer,parameter::ndim = 1
    type(bms_shape)::dshape
    logical::is_exist_file
    logical::is_exist_data

    istat = -1
    if (n1 < 0) return

    file_name = pl_bms_helper_get_file_name(url, PL_BMS_FILE_EXT)
    call h5open_f(info)
    call h5eset_auto_f(0, info)! Turn error print off
    is_exist_file = .false.
    is_exist_data = .false.
    call h5fopen_f(file_name, H5F_ACC_RDWR_F, file_idd, info)
    !write(6,*) 'info=',info
    if (info == 0) then
       is_exist_file = .true.
       call h5dopen_f(file_idd, name, data_idd, info2)
       if (info2 == 0) then
          is_exist_data = .true.
          call h5dclose_f(data_idd, info2)
       end if
       call h5fclose_f(file_idd, info2)
    end if
    call h5eset_auto_f(1, info)! Turn error print on

    if (is_exist_file) then
       call open_file(file_name, 'rw', file_id); if (file_id < 0) return
    else
       call create_file(file_name, file_id); if (file_id < 0) return
    end if
    dshape%ndim = ndim
    allocate(dshape%dims(dshape%ndim))
    dshape%dims(1) = n1
    if (is_exist_data) then
       call open_data(file_id, name, data_id)
       !if (info /= 0) return
    else
       call create_data_fixed(file_id, name, BMS_TYPE_INT4, dshape, data_id)
       !if (info /= 0) return
    end if
    call write_value(data_id, data, BMS_SHAPE_NULL, info); if (info /= 0) return
    if(associated(dshape%dims)) deallocate(dshape%dims)
    call close_data(data_id, info); if (info /= 0) return
    call close_file(file_id, info); if (info /= 0) return
    call h5close_f(info)
#endif
    istat = 0
  end subroutine pl_bms_write_int4_array_1d_impl
  
  !-----------------------------------------------------------------------------
  !> データの保存
  subroutine pl_bms_write_int4_array_2d_impl(&
       data, ld1, n1, n2, url, name, id, unit, istat)
    integer,intent(in)::ld1
    integer,intent(in)::n1, n2
    integer,intent(in)::data(ld1,n2)
    character(len=*),intent(in)::url
    character(len=*),intent(in)::name
    integer,intent(in)::id
    character(len=*),intent(in)::unit
    integer,intent(inout)::istat

#ifdef PL_BMS_HDF
    character(len=len(url)+len(PL_BMS_FILE_EXT))::file_name
    integer(BH_FILE_T)::file_id
    integer(BH_DATA_T)::data_id
    integer(HID_T)::file_idd
    integer(HID_T)::data_idd
    integer(BH_ERROR_T)::info, info2
    integer,parameter::ndim = 2
    type(bms_shape)::dshape
    logical::is_exist_file
    logical::is_exist_data
    integer::size
    integer,allocatable::d(:)
    integer::k, i1, i2

    istat = -1
    if (ld1 < 0 .or. n1 < 0 .or. ld1 < n1) return
    if (n2 < 0) return

    file_name = pl_bms_helper_get_file_name(url, PL_BMS_FILE_EXT)
    call h5open_f(info)
    call h5eset_auto_f(0, info)! Turn error print off
    is_exist_file = .false.
    is_exist_data = .false.
    call h5fopen_f(file_name, H5F_ACC_RDWR_F, file_idd, info)
    !write(6,*) 'info=',info
    if (info == 0) then
       is_exist_file = .true.
       call h5dopen_f(file_idd, name, data_idd, info2)
       if (info2 == 0) then
          is_exist_data = .true.
          call h5dclose_f(data_idd, info2)
       end if
       call h5fclose_f(file_idd, info2)
    end if
    call h5eset_auto_f(1, info)! Turn error print on

    if (is_exist_file) then
       call open_file(file_name, 'rw', file_id); if (file_id < 0) return
    else
       call create_file(file_name, file_id); if (file_id < 0) return
    end if
    dshape%ndim = ndim
    allocate(dshape%dims(dshape%ndim))
    dshape%dims(1) = n1
    dshape%dims(2) = n2
    !write(6,*) 'dims=',dshape%dims
    if (is_exist_data) then
       call open_data(file_id, name, data_id)
       !if (info /= 0) return
    else
       call create_data_fixed(file_id, name, BMS_TYPE_INT4, dshape, data_id)
       !if (info /= 0) return
    end if
    size = pl_bms_helper_get_size(dshape%dims)
    allocate(d(size), stat=istat); if (istat /= 0) return
    k = 0
    do i2 = 1, n2
       do i1 = 1, n1
          k = k + 1
          d(k) = data(i1,i2)
       end do
    end do
    call write_value(data_id, d, BMS_SHAPE_NULL, info); if (info /= 0) return
    deallocate(d, stat=istat); if (istat /= 0) return
    if(associated(dshape%dims)) deallocate(dshape%dims)
    call close_data(data_id, info); if (info /= 0) return
    call close_file(file_id, info); if (info /= 0) return
    call h5close_f(info)
#endif
    istat = 0
  end subroutine pl_bms_write_int4_array_2d_impl
  
  !-----------------------------------------------------------------------------
  !> データの保存
  subroutine pl_bms_write_int4_array_3d_impl(&
       data, ld1, ld2, n1, n2, n3, url, name, id, unit, istat)
    integer,intent(in)::ld1, ld2
    integer,intent(in)::n1, n2, n3
    integer,intent(in)::data(ld1,ld2,n3)
    character(len=*),intent(in)::url
    character(len=*),intent(in)::name
    integer,intent(in)::id
    character(len=*),intent(in)::unit
    integer,intent(inout)::istat

#ifdef PL_BMS_HDF
    character(len=len(url)+len(PL_BMS_FILE_EXT))::file_name
    integer(BH_FILE_T)::file_id
    integer(BH_DATA_T)::data_id
    integer(HID_T)::file_idd
    integer(HID_T)::data_idd
    integer(BH_ERROR_T)::info, info2
    integer,parameter::ndim = 3
    type(bms_shape)::dshape
    logical::is_exist_file
    logical::is_exist_data
    integer::size
    integer,allocatable::d(:)
    integer::i1, i2, i3, k

    istat = -1
    if (ld1 < 0 .or. n1 < 0 .or. ld1 < n1) return
    if (ld2 < 0 .or. n2 < 0 .or. ld2 < n2) return
    if (n3 < 0) return

    file_name = pl_bms_helper_get_file_name(url, PL_BMS_FILE_EXT)
    call h5open_f(info)
    call h5eset_auto_f(0, info)! Turn error print off
    is_exist_file = .false.
    is_exist_data = .false.
    call h5fopen_f(file_name, H5F_ACC_RDWR_F, file_idd, info)
    !write(6,*) 'info=',info
    if (info == 0) then
       is_exist_file = .true.
       call h5dopen_f(file_idd, name, data_idd, info2)
       if (info2 == 0) then
          is_exist_data = .true.
          call h5dclose_f(data_idd, info2)
       end if
       call h5fclose_f(file_idd, info2)
    end if
    call h5eset_auto_f(1, info)! Turn error print on

    if (is_exist_file) then
       call open_file(file_name, 'rw', file_id); if (file_id < 0) return
    else
       call bms_hdf5_create_file(file_name, file_id); if (file_id < 0) return
    end if
    dshape%ndim = ndim
    allocate(dshape%dims(dshape%ndim))
    dshape%dims(1) = n1
    dshape%dims(2) = n2
    dshape%dims(3) = n3
    if (is_exist_data) then
       call open_data(file_id, name, data_id)
       !if (info /= 0) return
    else
       call create_data_fixed(file_id, name, BMS_TYPE_INT4, dshape, data_id)
       !if (info /= 0) return
    end if
    size = pl_bms_helper_get_size(dshape%dims)
    allocate(d(size), stat=istat); if (istat /= 0) return
    k = 0
    do i3 = 1, n3
       do i2 = 1, n2
          do i1 = 1, n1
             k = k + 1
             d(k) = data(i1,i2,i3)
          end do
       end do
    end do
    call write_value(data_id, d, BMS_SHAPE_NULL, info); if (info /= 0) return
    deallocate(d, stat=istat); if (istat /= 0) return
    if(associated(dshape%dims)) deallocate(dshape%dims)
    call close_data(data_id, info); if (info /= 0) return
    call close_file(file_id, info); if (info /= 0) return
    call h5close_f(info)
#endif
    istat = 0
  end subroutine pl_bms_write_int4_array_3d_impl
  
  !-----------------------------------------------------------------------------
  !> データの保存
  subroutine pl_bms_write_int8_val_impl(data, url, name, id, unit, istat)
    integer(8),intent(in)::data
    character(len=*),intent(in)::url
    character(len=*),intent(in)::name
    integer,intent(in)::id
    character(len=*),intent(in)::unit
    integer,intent(inout)::istat

#ifdef PL_BMS_HDF
    integer(8)::a(1)

    a(1) = data
    call pl_bms_write_int8_array_1d_impl(a, 1, url, name, id, unit, istat)

#endif
    istat = 0
  end subroutine pl_bms_write_int8_val_impl
  
  !-----------------------------------------------------------------------------
  !> データの保存
  subroutine pl_bms_write_int8_array_1d_impl(&
       data, n1, url, name, id, unit, istat)
    integer,intent(in)::n1
    integer(8),intent(in)::data(n1)
    character(len=*),intent(in)::url
    character(len=*),intent(in)::name
    integer,intent(in)::id
    character(len=*),intent(in)::unit
    integer,intent(inout)::istat

#ifdef PL_BMS_HDF
    character(len=len(url)+len(PL_BMS_FILE_EXT))::file_name
    integer(BH_FILE_T)::file_id
    integer(BH_DATA_T)::data_id
    integer(HID_T)::file_idd
    integer(HID_T)::data_idd
    integer(BH_ERROR_T)::info, info2
    integer,parameter::ndim = 1
    type(bms_shape)::dshape
    logical::is_exist_file
    logical::is_exist_data

    istat = -1
    if (n1 < 0) return

    file_name = pl_bms_helper_get_file_name(url, PL_BMS_FILE_EXT)
    call h5open_f(info)
    call h5eset_auto_f(0, info)! Turn error print off
    is_exist_file = .false.
    is_exist_data = .false.
    call h5fopen_f(file_name, H5F_ACC_RDWR_F, file_idd, info)
    !write(6,*) 'info=',info
    if (info == 0) then
       is_exist_file = .true.
       call h5dopen_f(file_idd, name, data_idd, info2)
       if (info2 == 0) then
          is_exist_data = .true.
          call h5dclose_f(data_idd, info2)
       end if
       call h5fclose_f(file_idd, info2)
    end if
    call h5eset_auto_f(1, info)! Turn error print on

    if (is_exist_file) then
       call open_file(file_name, 'rw', file_id); if (file_id < 0) return
    else
       call create_file(file_name, file_id); if (file_id < 0) return
    end if
    dshape%ndim = ndim
    allocate(dshape%dims(dshape%ndim))
    dshape%dims(1) = n1
    if (is_exist_data) then
       call open_data(file_id, name, data_id)
       !if (info /= 0) return
    else
       call create_data_fixed(file_id, name, BMS_TYPE_INT8, dshape, data_id)
       !if (info /= 0) return
    end if
    call write_value(data_id, data, BMS_SHAPE_NULL, info); if (info /= 0) return
    if(associated(dshape%dims)) deallocate(dshape%dims)
    call close_data(data_id, info); if (info /= 0) return
    call close_file(file_id, info); if (info /= 0) return
    call h5close_f(info)
#endif
    istat = 0
  end subroutine pl_bms_write_int8_array_1d_impl
  
  !-----------------------------------------------------------------------------
  !> データの保存
  subroutine pl_bms_write_real8_val_impl(data, url, name, id, unit, istat)
    real(8),intent(in)::data
    character(len=*),intent(in)::url
    character(len=*),intent(in)::name
    integer,intent(in)::id
    character(len=*),intent(in)::unit
    integer,intent(inout)::istat

#ifdef PL_BMS_HDF
    real(8)::a(1)

    a(1) = data
    call pl_bms_write_real8_array_1d_impl(a, 1, url, name, id, unit, istat)

#endif
    istat = 0
  end subroutine pl_bms_write_real8_val_impl
  
  !-----------------------------------------------------------------------------
  !> データの保存
  subroutine pl_bms_write_real8_array_1d_impl(&
       data, n1, url, name, id, unit, istat)
    integer,intent(in)::n1
    real(8),intent(in)::data(n1)
    character(len=*),intent(in)::url
    character(len=*),intent(in)::name
    integer,intent(in)::id
    character(len=*),intent(in)::unit
    integer,intent(inout)::istat

#ifdef PL_BMS_HDF
    character(len=len(url)+len(PL_BMS_FILE_EXT))::file_name
    integer(BH_FILE_T)::file_id
    integer(BH_DATA_T)::data_id
    integer(HID_T)::file_idd
    integer(HID_T)::data_idd
    integer(BH_ERROR_T)::info, info2
    integer,parameter::ndim = 1
    type(bms_shape)::dshape
    logical::is_exist_file
    logical::is_exist_data

    istat = -1
    if (n1 < 0) return

    file_name = pl_bms_helper_get_file_name(url, PL_BMS_FILE_EXT)
    call h5open_f(info)
    call h5eset_auto_f(0, info)! Turn error print off
    is_exist_file = .false.
    is_exist_data = .false.
    call h5fopen_f(file_name, H5F_ACC_RDWR_F, file_idd, info)
    !write(6,*) 'info=',info
    if (info == 0) then
       is_exist_file = .true.
       call h5dopen_f(file_idd, name, data_idd, info2)
       if (info2 == 0) then
          is_exist_data = .true.
          call h5dclose_f(data_idd, info2)
       end if
       call h5fclose_f(file_idd, info2)
    end if
    call h5eset_auto_f(1, info)! Turn error print on

    if (is_exist_file) then
       call open_file(file_name, 'rw', file_id)
    else
       call create_file(file_name, file_id); if (file_id < 0) return
    end if
    dshape%ndim = ndim
    allocate(dshape%dims(dshape%ndim))
    dshape%dims(1) = n1
    if (is_exist_data) then
       call open_data(file_id, name, data_id)
       !if (info /= 0) return
    else
       call create_data_fixed(file_id, name, BMS_TYPE_REAL8, dshape, data_id)
       !if (info /= 0) return
    end if
    call write_value(data_id, data, BMS_SHAPE_NULL, info); if (info /= 0) return
    if(associated(dshape%dims)) deallocate(dshape%dims)
    call close_data(data_id, info); if (info /= 0) return
    call close_file(file_id, info); if (info /= 0) return
    call h5close_f(info)
#endif
    istat = 0
  end subroutine pl_bms_write_real8_array_1d_impl
  
  !-----------------------------------------------------------------------------
  !> データの保存
  subroutine pl_bms_write_real8_array_2d_impl(&
       data, ld1, n1, n2, url, name, id, unit, istat)
    integer,intent(in)::ld1
    integer,intent(in)::n1, n2
    real(8),intent(in)::data(ld1,n2)
    character(len=*),intent(in)::url
    character(len=*),intent(in)::name
    integer,intent(in)::id
    character(len=*),intent(in)::unit
    integer,intent(inout)::istat

#ifdef PL_BMS_HDF
    character(len=len(url)+len(PL_BMS_FILE_EXT))::file_name
    integer(BH_FILE_T)::file_id
    integer(BH_DATA_T)::data_id
    integer(BH_SIZE_T)::data_size
    integer(HID_T)::file_idd
    integer(HID_T)::data_idd
    integer(BH_ERROR_T)::info, info2
    integer,parameter::ndim = 2
    type(bms_shape)::dshape
    logical::is_exist_file
    logical::is_exist_data
    integer::size
    real(8),allocatable::d(:)
    integer::k, i1, i2

    istat = -1
    if (ld1 < 0 .or. n1 < 0 .or. ld1 < n1) return
    if (n2 < 0) return

    file_name = pl_bms_helper_get_file_name(url, PL_BMS_FILE_EXT)
    call h5open_f(info)
    call h5eset_auto_f(0, info)! Turn error print off
    is_exist_file = .false.
    is_exist_data = .false.
    call h5fopen_f(file_name, H5F_ACC_RDWR_F, file_idd, info)
    !write(6,*) 'info=',info
    if (info == 0) then
       is_exist_file = .true.
       call h5dopen_f(file_idd, name, data_idd, info2)
       if (info2 == 0) then
          is_exist_data = .true.
          call h5dclose_f(data_idd, info2)
       end if
       call h5fclose_f(file_idd, info2)
    end if
    call h5eset_auto_f(1, info)! Turn error print on

    if (is_exist_file) then
       call open_file(file_name, 'rw', file_id)
    else
       call create_file(file_name, file_id); if (file_id < 0) return
    end if
    dshape%ndim = ndim
    allocate(dshape%dims(dshape%ndim))
    dshape%dims(1) = n1
    dshape%dims(2) = n2
    if (is_exist_data) then
       call open_data(file_id, name, data_id)
       !if (info /= 0) return
    else
       call create_data_fixed(file_id, name, BMS_TYPE_REAL8, dshape, data_id)
       !if (info /= 0) return
    end if
    size = pl_bms_helper_get_size(dshape%dims)
    allocate(d(size), stat=istat); if (istat /= 0) return
    k = 0
    do i2 = 1, n2
       do i1 = 1, n1
          k = k + 1
          d(k) = data(i1,i2)
       end do
    end do
    call write_value(data_id, d, BMS_SHAPE_NULL, info); if (info /= 0) return
    deallocate(d, stat=istat); if (istat /= 0) return
    if(associated(dshape%dims)) deallocate(dshape%dims)
    call close_data(data_id, info); if (info /= 0) return
    call close_file(file_id, info); if (info /= 0) return
    call h5close_f(info)
#endif
    istat = 0
  end subroutine pl_bms_write_real8_array_2d_impl
  
  !-----------------------------------------------------------------------------
  !> データの保存
  subroutine pl_bms_write_real8_array_3d_impl(&
       data, ld1, ld2, n1, n2, n3, url, name, id, unit, istat)
    integer,intent(in)::ld1, ld2
    integer,intent(in)::n1, n2, n3
    real(8),intent(in)::data(ld1,ld2,n3)
    character(len=*),intent(in)::url
    character(len=*),intent(in)::name
    integer,intent(in)::id
    character(len=*),intent(in)::unit
    integer,intent(inout)::istat

#ifdef PL_BMS_HDF
    character(len=len(url)+len(PL_BMS_FILE_EXT))::file_name
    integer(BH_FILE_T)::file_id
    integer(BH_DATA_T)::data_id
    integer(HID_T)::file_idd
    integer(HID_T)::data_idd
    integer(BH_ERROR_T)::info, info2
    integer,parameter::ndim = 3
    type(bms_shape)::dshape
    logical::is_exist_file
    logical::is_exist_data
    integer::size
    real(8),allocatable::d(:)
    integer::i1, i2, i3, k

    istat = -1
    if (ld1 < 0 .or. n1 < 0 .or. ld1 < n1) return
    if (ld2 < 0 .or. n2 < 0 .or. ld2 < n2) return
    if (n3 < 0) return

    file_name = pl_bms_helper_get_file_name(url, PL_BMS_FILE_EXT)
    call h5open_f(info)
    call h5eset_auto_f(0, info)! Turn error print off
    is_exist_file = .false.
    is_exist_data = .false.
    call h5fopen_f(file_name, H5F_ACC_RDWR_F, file_idd, info)
    !write(6,*) 'info=',info
    if (info == 0) then
       is_exist_file = .true.
       call h5dopen_f(file_idd, name, data_idd, info2)
       if (info2 == 0) then
          is_exist_data = .true.
          call h5dclose_f(data_idd, info2)
       end if
       call h5fclose_f(file_idd, info2)
    end if
    call h5eset_auto_f(1, info)! Turn error print on

    if (is_exist_file) then
       call open_file(file_name, 'rw', file_id)
    else
       call create_file(file_name, file_id); if (file_id < 0) return
    end if
    dshape%ndim = ndim
    allocate(dshape%dims(dshape%ndim))
    dshape%dims(1) = n1
    dshape%dims(2) = n2
    dshape%dims(3) = n3
    if (is_exist_data) then
       call open_data(file_id, name, data_id)
       !if (info /= 0) return
    else
       call create_data_fixed(file_id, name, BMS_TYPE_REAL8, dshape, data_id)
       !if (info /= 0) return
    end if
    size = pl_bms_helper_get_size(dshape%dims)
    allocate(d(size), stat=istat); if (istat /= 0) return
    k = 0
    do i3 = 1, n3
       do i2 = 1, n2
          do i1 = 1, n1
             k = k + 1
             d(k) = data(i1,i2,i3)
          end do
       end do
    end do
    call write_value(data_id, d, BMS_SHAPE_NULL, info); if (info /= 0) return
    deallocate(d, stat=istat); if (istat /= 0) return
    if(associated(dshape%dims)) deallocate(dshape%dims)
    call close_data(data_id, info); if (info /= 0) return
    call close_file(file_id, info); if (info /= 0) return
    call h5close_f(info)
#endif
    istat = 0
  end subroutine pl_bms_write_real8_array_3d_impl
  
  !-----------------------------------------------------------------------------
  !> データの保存
  subroutine pl_bms_write_real8_array_4d_impl(&
       data, ld1, ld2, ld3, n1, n2, n3, n4, url, name, id, unit, istat)
    integer,intent(in)::ld1, ld2, ld3
    integer,intent(in)::n1, n2, n3, n4
    real(8),intent(in)::data(ld1,ld2,ld3,n4)
    character(len=*),intent(in)::url
    character(len=*),intent(in)::name
    integer,intent(in)::id
    character(len=*),intent(in)::unit
    integer,intent(inout)::istat

#ifdef PL_BMS_HDF
    character(len=len(url)+len(PL_BMS_FILE_EXT))::file_name
    integer(BH_FILE_T)::file_id
    integer(BH_DATA_T)::data_id
    integer(HID_T)::file_idd
    integer(HID_T)::data_idd
    integer(BH_ERROR_T)::info, info2
    integer,parameter::ndim = 4
    type(bms_shape)::dshape
    logical::is_exist_file
    logical::is_exist_data
    integer::size
    real(8),allocatable::d(:)
    integer::i1, i2, i3, i4, k

    istat = -1
    if (ld1 < 0 .or. n1 < 0 .or. ld1 < n1) return
    if (ld2 < 0 .or. n2 < 0 .or. ld2 < n2) return
    if (ld3 < 0 .or. n3 < 0 .or. ld3 < n3) return
    if (n4 < 0) return

    file_name = pl_bms_helper_get_file_name(url, PL_BMS_FILE_EXT)
    call h5open_f(info)
    call h5eset_auto_f(0, info)! Turn error print off
    is_exist_file = .false.
    is_exist_data = .false.
    call h5fopen_f(file_name, H5F_ACC_RDWR_F, file_idd, info)
    !write(6,*) 'info=',info
    if (info == 0) then
       is_exist_file = .true.
       call h5dopen_f(file_idd, name, data_idd, info2)
       if (info2 == 0) then
          is_exist_data = .true.
          call h5dclose_f(data_idd, info2)
       end if
       call h5fclose_f(file_idd, info2)
    end if
    call h5eset_auto_f(1, info)! Turn error print on

    if (is_exist_file) then
       call open_file(file_name, 'rw', file_id)
    else
       call create_file(file_name, file_id); if (file_id < 0) return
    end if
    dshape%ndim = ndim
    allocate(dshape%dims(dshape%ndim))
    dshape%dims(1) = n1
    dshape%dims(2) = n2
    dshape%dims(3) = n3
    dshape%dims(4) = n4
    if (is_exist_data) then
       call open_data(file_id, name, data_id)
       !if (info /= 0) return
    else
       call create_data_fixed(file_id, name, BMS_TYPE_REAL8, dshape, data_id)
       !if (info /= 0) return
    end if
    size = pl_bms_helper_get_size(dshape%dims)
    allocate(d(size), stat=istat); if (istat /= 0) return
    k = 0
    do i4 = 1, n4
       do i3 = 1, n3
          do i2 = 1, n2
             do i1 = 1, n1
                k = k + 1
                d(k) = data(i1,i2,i3,i4)
             end do
          end do
       end do
    end do
    call write_value(data_id, d, BMS_SHAPE_NULL, info); if (info /= 0) return
    deallocate(d, stat=istat); if (istat /= 0) return
    if(associated(dshape%dims)) deallocate(dshape%dims)
    call close_data(data_id, info); if (info /= 0) return
    call close_file(file_id, info); if (info /= 0) return
    call h5close_f(info)
#endif
    istat = 0
  end subroutine pl_bms_write_real8_array_4d_impl
  
  !-----------------------------------------------------------------------------
  !> データの保存
  subroutine pl_bms_write_str_val_impl(data, url, name, id, unit, istat)
    character(len=*),intent(in)::data
    character(len=*),intent(in)::url
    character(len=*),intent(in)::name
    integer,intent(in)::id
    character(len=*),intent(in)::unit
    integer,intent(inout)::istat

#ifdef PL_BMS_HDF
    character(len=512)::a(1)

    a(1) = trim(data)
    call pl_bms_write_str_array_1d_impl(a, 1, url, name, id, unit, istat)

#endif
    istat = 0
  end subroutine pl_bms_write_str_val_impl
  
  !-----------------------------------------------------------------------------
  !> データの保存
  subroutine pl_bms_write_str_array_1d_impl(&
       data, n1, url, name, id, unit, istat)
    integer,intent(in)::n1
    character(len=*),intent(in)::data(n1)
    character(len=*),intent(in)::url
    character(len=*),intent(in)::name
    integer,intent(in)::id
    character(len=*),intent(in)::unit
    integer,intent(inout)::istat

#ifdef PL_BMS_HDF
    character(len=len(url)+len(PL_BMS_FILE_EXT))::file_name
    integer(BH_FILE_T)::file_id
    integer(BH_DATA_T)::data_id
    integer(HID_T)::file_idd
    integer(HID_T)::data_idd
    integer(BH_ERROR_T)::info, info2
    integer,parameter::ndim = 1
    type(bms_shape)::dshape
    logical::is_exist_file
    logical::is_exist_data

    istat = -1
    if (n1 < 0) return

    file_name = pl_bms_helper_get_file_name(url, PL_BMS_FILE_EXT)
    call h5open_f(info)
    call h5eset_auto_f(0, info)! Turn error print off
    is_exist_file = .false.
    is_exist_data = .false.
    call h5fopen_f(file_name, H5F_ACC_RDWR_F, file_idd, info)
    !write(6,*) 'info=',info
    if (info == 0) then
       is_exist_file = .true.
       call h5dopen_f(file_idd, name, data_idd, info2)
       if (info2 == 0) then
          is_exist_data = .true.
          call h5dclose_f(data_idd, info2)
       end if
       call h5fclose_f(file_idd, info2)
    end if
    call h5eset_auto_f(1, info)! Turn error print on

    if (is_exist_file) then
       call open_file(file_name, 'rw', file_id)
    else
       call create_file(file_name, file_id); if (file_id < 0) return
    end if
    dshape%ndim = ndim
    allocate(dshape%dims(dshape%ndim))
    dshape%dims(1) = n1
    if (is_exist_data) then
       call open_data(file_id, name, data_id)
       !if (info /= 0) return
    else
       call create_data_fixed(file_id, name, BMS_TYPE_STRING, dshape, data_id)
       !if (info /= 0) return
    end if
    call write_value(data_id, data, BMS_SHAPE_NULL, info); if (info /= 0) return
    if(associated(dshape%dims)) deallocate(dshape%dims)
    call close_data(data_id, info); if (info /= 0) return
    call close_file(file_id, info); if (info /= 0) return
    call h5close_f(info)
#endif
    istat = 0
  end subroutine pl_bms_write_str_array_1d_impl
  
  !-----------------------------------------------------------------------------
  !> データの取得
  subroutine pl_bms_read_int4_val_impl(data, url, name, id, istat)
    integer,intent(inout)::data
    character(len=*),intent(in)::url
    character(len=*),intent(in)::name
    integer,intent(in)::id
    integer,intent(out)::istat

#ifdef PL_BMS_HDF
    integer::a(1)

    call pl_bms_read_int4_array_1d_impl(a, 1, url, name, id, istat)
    if (istat /= 0) return
    data = a(1)

#endif
    istat = 0
  end subroutine pl_bms_read_int4_val_impl

  !-----------------------------------------------------------------------------
  !> データの取得
  subroutine pl_bms_read_int4_array_1d_impl(&
       data, n1, url, name, id, istat)
    integer,intent(in)::n1
    integer,intent(inout)::data(n1)
    character(len=*),intent(in)::url
    character(len=*),intent(in)::name
    integer,intent(in)::id
    integer,intent(out)::istat

#ifdef PL_BMS_HDF
    character(len=len(url)+len(PL_BMS_FILE_EXT))::file_name
    integer(BH_FILE_T)::file_id
    integer(BH_DATA_T)::data_id
    integer(BH_ERROR_T)::info
    type(bms_shape)::dshape
    type(bms_type)::data_type 
    integer,parameter::ndim = 1

    istat = -1
    if (n1 < 0) return

    file_name = pl_bms_helper_get_file_name(url, PL_BMS_FILE_EXT)
    call h5open_f(info)
    call open_file(file_name, 'r', file_id); if (file_id < 0) return
    call open_data(file_id, name, data_id); if (data_id < 0) return

    ! get num of dimensions
    dshape = BMS_SHAPE_NULL
    call get_shape(data_id, dshape, info); if (info /= 0) return
    if (dshape%ndim /= ndim) return
    !write(6,*) 'dshape%ndim=',dshape%ndim

    ! get dimensions
    allocate(dshape%dims(dshape%ndim))
    call get_shape(data_id, dshape, info); if (info /= 0) return
    if (dshape%dims(1) /= n1) return
    !write(6,*) 'dshape%dims=',dshape%dims

    call get_type(data_id, data_type, info); if (info /= 0) return
    !write(6,*) 'data_type=',data_type

    call read_value(data_id, data, BMS_SHAPE_NULL, info); if (info /= 0) return
    if(associated(dshape%dims)) deallocate(dshape%dims)

    call close_data(data_id, info); if (info /= 0) return
    call close_file(file_id, info); if (info /= 0) return
    call h5close_f(info)
#endif
    istat = 0
  end subroutine pl_bms_read_int4_array_1d_impl

  !-----------------------------------------------------------------------------
  !> データの取得
  subroutine pl_bms_read_int4_array_2d_impl(&
       data, ld1, n1, n2, url, name, id, istat)
    integer,intent(in)::ld1
    integer,intent(in)::n1, n2
    integer,intent(inout)::data(ld1,n2)
    character(len=*),intent(in)::url
    character(len=*),intent(in)::name
    integer,intent(in)::id
    integer,intent(out)::istat

#ifdef PL_BMS_HDF
    character(len=len(url)+len(PL_BMS_FILE_EXT))::file_name
    integer(BH_FILE_T)::file_id
    integer(BH_DATA_T)::data_id
    integer(BH_ERROR_T)::info
    type(bms_shape)::dshape
    type(bms_type)::data_type 
    integer,parameter::ndim = 2
    integer::size
    integer,allocatable::d(:)
    integer::i1, i2, k

    istat = -1
    if (ld1 < 0 .or. n1 < 0 .or. ld1 < n1) return
    if (n2 < 0) return

    file_name = pl_bms_helper_get_file_name(url, PL_BMS_FILE_EXT)
    call h5open_f(info)
    call open_file(file_name, 'r', file_id); if (file_id < 0) return
    call open_data(file_id, name, data_id); if (data_id < 0) return

    ! get num of dimensions
    dshape = BMS_SHAPE_NULL
    call get_shape(data_id, dshape, info); if (info /= 0) return
    if (dshape%ndim /= ndim) return
    !write(6,*) 'dshape%ndim=',dshape%ndim

    ! get dimensions
    allocate(dshape%dims(dshape%ndim))
    call get_shape(data_id, dshape, info); if (info /= 0) return
    !write(6,*) 'dims=',dshape%dims
    if (dshape%dims(1) /= n1) return
    if (dshape%dims(2) /= n2) return
    !write(6,*) 'dshape%dims=',dshape%dims

    call get_type(data_id, data_type, info); if (info /= 0) return
    !write(6,*) 'data_type=',data_type
    
    size = pl_bms_helper_get_size(dshape%dims)
    allocate(d(size), stat=istat); if (istat /= 0) return
    call read_value(data_id, d, BMS_SHAPE_NULL, info); if (info /= 0) return
    k = 0
    do i2 = 1, n2
       do i1 = 1, n1
          k = k + 1
          data(i1,i2) = d(k)
       end do
    end do
    deallocate(d, stat=istat); if (istat /= 0) return

    if(associated(dshape%dims)) deallocate(dshape%dims)
    call close_data(data_id, istat); if (istat < 0) return
    call close_file(file_id, istat); if (istat < 0) return
    call h5close_f(info)
#endif
    istat = 0
  end subroutine pl_bms_read_int4_array_2d_impl

  !-----------------------------------------------------------------------------
  !> データの取得
  subroutine pl_bms_read_int4_array_3d_impl(&
       data, ld1, ld2, n1, n2, n3, url, name, id, istat)
    integer,intent(in)::ld1, ld2
    integer,intent(in)::n1, n2, n3
    integer,intent(inout)::data(ld1,ld2,n3)
    character(len=*),intent(in)::url
    character(len=*),intent(in)::name
    integer,intent(in)::id
    integer,intent(inout)::istat

#ifdef PL_BMS_HDF
    character(len=len(url)+len(PL_BMS_FILE_EXT))::file_name
    integer(BH_FILE_T)::file_id
    integer(BH_DATA_T)::data_id
    integer(BH_ERROR_T)::info
    type(bms_shape)::dshape
    type(bms_type)::data_type 
    integer,parameter::ndim = 3
    integer::size
    integer,allocatable::d(:)
    integer::i1, i2, i3, k

    istat = -1
    if (ld1 < 0 .or. n1 < 0 .or. ld1 < n1) return
    if (ld2 < 0 .or. n2 < 0 .or. ld2 < n2) return
    if (n3 < 0) return

    file_name = pl_bms_helper_get_file_name(url, PL_BMS_FILE_EXT)
    call h5open_f(info)
    call open_file(file_name, 'r', file_id); if (file_id < 0) return
    call open_data(file_id, name, data_id); if (data_id < 0) return

    ! get num of dimensions
    dshape = BMS_SHAPE_NULL
    call get_shape(data_id, dshape, info); if (info /= 0) return
    if (dshape%ndim /= ndim) return
    !write(6,*) 'dshape%ndim=',dshape%ndim

    ! get dimensions
    allocate(dshape%dims(dshape%ndim))
    call get_shape(data_id, dshape, info); if (info /= 0) return
    if (dshape%dims(1) /= n1) return
    if (dshape%dims(2) /= n2) return
    if (dshape%dims(3) /= n3) return
    !write(6,*) 'dshape%dims=',dshape%dims

    call get_type(data_id, data_type, info); if (info /= 0) return
    !write(6,*) 'data_type=',data_type
    
    size = pl_bms_helper_get_size(dshape%dims)
    allocate(d(size), stat=istat); if (istat /= 0) return
    call read_value(data_id, d, BMS_SHAPE_NULL, info); if (info /= 0) return

    k = 0
    do i3 = 1, n3
       do i2 = 1, n2
          do i1 = 1, n1
             k = k + 1
             data(i1,i2,i3) = d(k)
          end do
       end do
    end do
    deallocate(d, stat=istat); if (istat /= 0) return

    if(associated(dshape%dims)) deallocate(dshape%dims)
    call close_data(data_id, istat); if (istat < 0) return
    call close_file(file_id, istat); if (istat < 0) return
    call h5close_f(info)
#endif
    istat = 0
  end subroutine pl_bms_read_int4_array_3d_impl
  
  !-----------------------------------------------------------------------------
  !> データの取得
  subroutine pl_bms_read_int8_val_impl(data, url, name, id, istat)
    integer(8),intent(inout)::data
    character(len=*),intent(in)::url
    character(len=*),intent(in)::name
    integer,intent(in)::id
    integer,intent(out)::istat

#ifdef PL_BMS_HDF
    integer(8)::a(1)

    call pl_bms_read_int8_array_1d_impl(a, 1, url, name, id, istat)
    if (istat /= 0) return
    data = a(1)

#endif
    istat = 0
  end subroutine pl_bms_read_int8_val_impl

  !-----------------------------------------------------------------------------
  !> データの取得
  subroutine pl_bms_read_int8_array_1d_impl(&
       data, n1, url, name, id, istat)
    integer,intent(in)::n1
    integer(8),intent(inout)::data(n1)
    character(len=*),intent(in)::url
    character(len=*),intent(in)::name
    integer,intent(in)::id
    integer,intent(out)::istat

#ifdef PL_BMS_HDF
    character(len=len(url)+len(PL_BMS_FILE_EXT))::file_name
    integer(BH_FILE_T)::file_id
    integer(BH_DATA_T)::data_id
    integer(BH_ERROR_T)::info
    type(bms_shape)::dshape
    type(bms_type)::data_type 
    integer,parameter::ndim = 1

    istat = -1
    if (n1 < 0) return

    file_name = pl_bms_helper_get_file_name(url, PL_BMS_FILE_EXT)
    call h5open_f(info)
    call open_file(file_name, 'r', file_id); if (file_id < 0) return
    call open_data(file_id, name, data_id); if (data_id < 0) return

    ! get num of dimensions
    dshape = BMS_SHAPE_NULL
    call get_shape(data_id, dshape, info); if (info /= 0) return
    if (dshape%ndim /= ndim) return
    !write(6,*) 'dshape%ndim=',dshape%ndim

    ! get dimensions
    allocate(dshape%dims(dshape%ndim))
    call get_shape(data_id, dshape, info); if (info /= 0) return
    if (dshape%dims(1) /= n1) return
    !write(6,*) 'dshape%dims=',dshape%dims

    call get_type(data_id, data_type, info); if (info /= 0) return
    !write(6,*) 'data_type=',data_type

    call read_value(data_id, data, BMS_SHAPE_NULL, info); if (info /= 0) return
    if(associated(dshape%dims)) deallocate(dshape%dims)

    call close_data(data_id, info); if (info /= 0) return
    call close_file(file_id, info); if (info /= 0) return
    call h5close_f(info)
#endif
    istat = 0
  end subroutine pl_bms_read_int8_array_1d_impl

  !-----------------------------------------------------------------------------
  !> データの取得
  subroutine pl_bms_read_real8_val_impl(data, url, name, id, istat)
    real(8),intent(inout)::data
    character(len=*),intent(in)::url
    character(len=*),intent(in)::name
    integer,intent(in)::id
    integer,intent(inout)::istat

#ifdef PL_BMS_HDF
    real(8)::a(1)

    call pl_bms_read_real8_array_1d_impl(a, 1, url, name, id, istat)
    if (istat /= 0) return
    data = a(1)

#endif
    istat = 0
  end subroutine pl_bms_read_real8_val_impl

  !-----------------------------------------------------------------------------
  !> データの取得
  subroutine pl_bms_read_real8_array_1d_impl(&
       data, n1, url, name, id, istat)
    integer,intent(in)::n1
    real(8),intent(inout)::data(n1)
    character(len=*),intent(in)::url
    character(len=*),intent(in)::name
    integer,intent(in)::id
    integer,intent(inout)::istat

#ifdef PL_BMS_HDF
    character(len=len(url)+len(PL_BMS_FILE_EXT))::file_name
    integer(BH_FILE_T)::file_id
    integer(BH_DATA_T)::data_id
    integer(BH_ERROR_T)::info
    type(bms_shape)::dshape
    type(bms_type)::data_type 
    integer,parameter::ndim = 1

    istat = -1
    if (n1 < 0) return

    file_name = pl_bms_helper_get_file_name(url, PL_BMS_FILE_EXT)
    call h5open_f(info)
    call open_file(file_name, 'r', file_id); if (file_id < 0) return
    call open_data(file_id, name, data_id); if (data_id < 0) return

    ! get num of dimensions
    dshape = BMS_SHAPE_NULL
    call get_shape(data_id, dshape, info); if (info /= 0) return
    if (dshape%ndim /= ndim) return
    !write(6,*) 'dshape%ndim=',dshape%ndim

    ! get dimensions
    allocate(dshape%dims(dshape%ndim))
    call get_shape(data_id, dshape, info); if (info /= 0) return
    if (dshape%dims(1) /= n1) return
    !write(6,*) 'dshape%dims=',dshape%dims

    call get_type(data_id, data_type, info); if (info /= 0) return
    !write(6,*) 'data_type=',data_type

    call read_value(data_id, data, BMS_SHAPE_NULL, info); if (info /= 0) return
    if(associated(dshape%dims)) deallocate(dshape%dims)

    call close_data(data_id, info); if (info /= 0) return
    call close_file(file_id, info); if (info /= 0) return
    call h5close_f(info)
#endif
    istat = 0
  end subroutine pl_bms_read_real8_array_1d_impl

  !-----------------------------------------------------------------------------
  !> データの取得
  subroutine pl_bms_read_real8_array_2d_impl(&
       data, ld1, n1, n2, url, name, id, istat)
    integer,intent(in)::ld1
    integer,intent(in)::n1, n2
    real(8),intent(inout)::data(ld1,n2)
    character(len=*),intent(in)::url
    character(len=*),intent(in)::name
    integer,intent(in)::id
    integer,intent(inout)::istat

#ifdef PL_BMS_HDF
    character(len=len(url)+len(PL_BMS_FILE_EXT))::file_name
    integer(BH_FILE_T)::file_id
    integer(BH_DATA_T)::data_id
    integer(BH_ERROR_T)::info
    type(bms_shape)::dshape
    type(bms_type)::data_type 
    integer,parameter::ndim = 2
    integer::size
    real(8),allocatable::d(:)
    integer::i1, i2, k

    istat = -1
    if (ld1 < 0 .or. n1 < 0 .or. ld1 < n1) return
    if (n2 < 0) return

    file_name = pl_bms_helper_get_file_name(url, PL_BMS_FILE_EXT)
    call h5open_f(info)
    call open_file(file_name, 'r', file_id); if (file_id < 0) return
    call open_data(file_id, name, data_id); if (data_id < 0) return

    ! get num of dimensions
    dshape = BMS_SHAPE_NULL
    call get_shape(data_id, dshape, info); if (info /= 0) return
    if (dshape%ndim /= ndim) return
    !write(6,*) 'dshape%ndim=',dshape%ndim

    ! get dimensions
    allocate(dshape%dims(dshape%ndim))
    call get_shape(data_id, dshape, info); if (info /= 0) return
    if (dshape%dims(1) /= n1) return
    if (dshape%dims(2) /= n2) return
    !write(6,*) 'dshape%dims=',dshape%dims

    call get_type(data_id, data_type, info); if (info /= 0) return
    !write(6,*) 'data_type=',data_type
    
    size = pl_bms_helper_get_size(dshape%dims)
    allocate(d(size), stat=istat); if (istat /= 0) return
    call read_value(data_id, d, BMS_SHAPE_NULL, info); if (info /= 0) return
    k = 0
    do i2 = 1, n2
       do i1 = 1, n1
          k = k + 1
          data(i1,i2) = d(k)
       end do
    end do
    deallocate(d, stat=istat); if (istat /= 0) return

    if(associated(dshape%dims)) deallocate(dshape%dims)
    call close_data(data_id, istat); if (istat < 0) return
    call close_file(file_id, istat); if (istat < 0) return
    call h5close_f(info)
#endif
    istat = 0
  end subroutine pl_bms_read_real8_array_2d_impl

  !-----------------------------------------------------------------------------
  !> データの取得
  subroutine pl_bms_read_real8_array_3d_impl(&
       data, ld1, ld2, n1, n2, n3, url, name, id, istat)
    integer,intent(in)::ld1, ld2
    integer,intent(in)::n1, n2, n3
    real(8),intent(inout)::data(ld1,ld2,n3)
    character(len=*),intent(in)::url
    character(len=*),intent(in)::name
    integer,intent(in)::id
    integer,intent(inout)::istat

#ifdef PL_BMS_HDF
    character(len=len(url)+len(PL_BMS_FILE_EXT))::file_name
    integer(BH_FILE_T)::file_id
    integer(BH_DATA_T)::data_id
    integer(BH_ERROR_T)::info
    type(bms_shape)::dshape
    type(bms_type)::data_type 
    integer,parameter::ndim = 3
    integer::size
    real(8),allocatable::d(:)
    integer::i1, i2, i3, k

    istat = -1
    if (ld1 < 0 .or. n1 < 0 .or. ld1 < n1) return
    if (ld2 < 0 .or. n2 < 0 .or. ld2 < n2) return
    if (n3 < 0) return

    file_name = pl_bms_helper_get_file_name(url, PL_BMS_FILE_EXT)
    call h5open_f(info)
    call open_file(file_name, 'r', file_id); if (file_id < 0) return
    call open_data(file_id, name, data_id); if (data_id < 0) return

    ! get num of dimensions
    dshape = BMS_SHAPE_NULL
    call get_shape(data_id, dshape, info); if (info /= 0) return
    if (dshape%ndim /= ndim) return
    !write(6,*) 'dshape%ndim=',dshape%ndim

    ! get dimensions
    allocate(dshape%dims(dshape%ndim))
    call get_shape(data_id, dshape, info); if (info /= 0) return
    if (dshape%dims(1) /= n1) return
    if (dshape%dims(2) /= n2) return
    if (dshape%dims(3) /= n3) return
    !write(6,*) 'dshape%dims=',dshape%dims

    call get_type(data_id, data_type, info); if (info /= 0) return
    !write(6,*) 'data_type=',data_type
    
    size = pl_bms_helper_get_size(dshape%dims)
    allocate(d(size), stat=istat); if (istat /= 0) return
    call read_value(data_id, d, BMS_SHAPE_NULL, info); if (info /= 0) return

    k = 0
    do i3 = 1, n3
       do i2 = 1, n2
          do i1 = 1, n1
             k = k + 1
             data(i1,i2,i3) = d(k)
          end do
       end do
    end do
    deallocate(d, stat=istat); if (istat /= 0) return

    if(associated(dshape%dims)) deallocate(dshape%dims)
    call close_data(data_id, istat); if (istat < 0) return
    call close_file(file_id, istat); if (istat < 0) return
    call h5close_f(info)
#endif
    istat = 0
  end subroutine pl_bms_read_real8_array_3d_impl

  !-----------------------------------------------------------------------------
  !> データの取得
  subroutine pl_bms_read_real8_array_4d_impl(&
       data, ld1, ld2, ld3, n1, n2, n3, n4, url, name, id, istat)
    integer,intent(in)::ld1, ld2, ld3
    integer,intent(in)::n1, n2, n3, n4
    real(8),intent(inout)::data(ld1,ld2,ld3,n4)
    character(len=*),intent(in)::url
    character(len=*),intent(in)::name
    integer,intent(in)::id
    integer,intent(inout)::istat

#ifdef PL_BMS_HDF
    character(len=len(url)+len(PL_BMS_FILE_EXT))::file_name
    integer(BH_FILE_T)::file_id
    integer(BH_DATA_T)::data_id
    integer(BH_ERROR_T)::info
    type(bms_shape)::dshape
    type(bms_type)::data_type 
    integer,parameter::ndim = 4
    integer::size
    real(8),allocatable::d(:)
    integer::i1, i2, i3, i4, k

    istat = -1
    if (ld1 < 0 .or. n1 < 0 .or. ld1 < n1) return
    if (ld2 < 0 .or. n2 < 0 .or. ld2 < n2) return
    if (ld3 < 0 .or. n3 < 0 .or. ld3 < n3) return
    if (n4 < 0) return

    file_name = pl_bms_helper_get_file_name(url, PL_BMS_FILE_EXT)
    call h5open_f(info)
    call open_file(file_name, 'r', file_id); if (file_id < 0) return
    call open_data(file_id, name, data_id); if (data_id < 0) return

    ! get num of dimensions
    dshape = BMS_SHAPE_NULL
    call get_shape(data_id, dshape, info); if (info /= 0) return
    if (dshape%ndim /= ndim) return
    !write(6,*) 'dshape%ndim=',dshape%ndim

    ! get dimensions
    allocate(dshape%dims(dshape%ndim))
    call get_shape(data_id, dshape, info); if (info /= 0) return
    if (dshape%dims(1) /= n1) return
    if (dshape%dims(2) /= n2) return
    if (dshape%dims(3) /= n3) return
    if (dshape%dims(4) /= n4) return
    !write(6,*) 'dshape%dims=',dshape%dims

    call get_type(data_id, data_type, info); if (info /= 0) return
    !write(6,*) 'data_type=',data_type
    
    size = pl_bms_helper_get_size(dshape%dims)
    allocate(d(size), stat=istat); if (istat /= 0) return
    call read_value(data_id, d, BMS_SHAPE_NULL, info); if (info /= 0) return

    k = 0
    do i4 = 1, n4
       do i3 = 1, n3
          do i2 = 1, n2
             do i1 = 1, n1
                k = k + 1
                data(i1,i2,i3,i4) = d(k)
             end do
          end do
       end do
    end do
    deallocate(d, stat=istat); if (istat /= 0) return

    if(associated(dshape%dims)) deallocate(dshape%dims)
    call close_data(data_id, istat); if (istat < 0) return
    call close_file(file_id, istat); if (istat < 0) return
    call h5close_f(info)
#endif
    istat = 0
  end subroutine pl_bms_read_real8_array_4d_impl

  !-----------------------------------------------------------------------------
  !> データの取得
  subroutine pl_bms_read_str_val_impl(data, url, name, id, istat)
    character(len=*),intent(inout)::data
    character(len=*),intent(in)::url
    character(len=*),intent(in)::name
    integer,intent(in)::id
    integer,intent(out)::istat

#ifdef PL_BMS_HDF
    character(len=512)::a(1)

    call pl_bms_read_str_array_1d_impl(a, 1, url, name, id, istat)
    if (istat /= 0) return
    data = trim(a(1))

#endif
    istat = 0
  end subroutine pl_bms_read_str_val_impl

  !-----------------------------------------------------------------------------
  !> データの取得
  subroutine pl_bms_read_str_array_1d_impl(&
       data, n1, url, name, id, istat)
    integer,intent(in)::n1
    character(len=*),intent(inout)::data(n1)
    character(len=*),intent(in)::url
    character(len=*),intent(in)::name
    integer,intent(in)::id
    integer,intent(inout)::istat

#ifdef PL_BMS_HDF
    character(len=len(url)+len(PL_BMS_FILE_EXT))::file_name
    integer(BH_FILE_T)::file_id
    integer(BH_DATA_T)::data_id
    integer(BH_ERROR_T)::info
    type(bms_shape)::dshape
    type(bms_type)::data_type 
    integer,parameter::ndim = 1

    istat = -1
    if (n1 < 0) return

    file_name = pl_bms_helper_get_file_name(url, PL_BMS_FILE_EXT)
    call h5open_f(info)
    call open_file(file_name, 'r', file_id); if (file_id < 0) return
    call open_data(file_id, name, data_id); if (data_id < 0) return

    ! get num of dimensions
    dshape = BMS_SHAPE_NULL
    call get_shape(data_id, dshape, info); if (info /= 0) return
    if (dshape%ndim /= ndim) return
    !write(6,*) 'dshape%ndim=',dshape%ndim

    ! get dimensions
    allocate(dshape%dims(dshape%ndim))
    call get_shape(data_id, dshape, info); if (info /= 0) return
    if (dshape%dims(1) /= n1) return
    !write(6,*) 'dshape%dims=',dshape%dims

    call get_type(data_id, data_type, info); if (info /= 0) return
    !write(6,*) 'data_type=',data_type

    call read_value(data_id, data, BMS_SHAPE_NULL, info); if (info /= 0) return
    if(associated(dshape%dims)) deallocate(dshape%dims)

    call close_data(data_id, info); if (info /= 0) return
    call close_file(file_id, info); if (info /= 0) return
    call h5close_f(info)
#endif
    istat = 0
  end subroutine pl_bms_read_str_array_1d_impl

  !-----------------------------------------------------------------------------
end module pl_bms_impl_hdf_class
