! -*- coding: utf-8 -*-
!
! Dual licensed under the MIT or GPL Version 3 license.
! See 'LICENSE.MIT' for the definition of the MIT license,
! and 'COPYING' for the definition of the GPL license.
!
!> BMSデータ標準化ライブラリクラスのXML実装
!!
module pl_bms_impl_xml_class
  use pl_bms_status_class
  use pl_bms_helper_class
  implicit none
!!$  private

!!$  public::pl_bms_is_exists_impl
!!$  public::pl_bms_get_ndim_impl
!!$  public::pl_bms_get_shape_impl
!!$  public::pl_bms_get_size_impl

!!$  !> データの保存
!!$  interface pl_bms_write_impl
!!$    module procedure pl_bms_write_int4_val_impl
!!$    module procedure pl_bms_write_int4_array_1d_impl
!!$    module procedure pl_bms_write_int4_array_2d_impl
!!$    module procedure pl_bms_write_int4_array_3d_impl
!!$    module procedure pl_bms_write_real8_val_impl
!!$    module procedure pl_bms_write_real8_array_1d_impl
!!$    module procedure pl_bms_write_real8_array_2d_impl
!!$    module procedure pl_bms_write_real8_array_3d_impl
!!$    module procedure pl_bms_write_real8_array_4d_impl
!!$    !module procedure pl_bms_write_str_array_1d_impl
!!$  end interface
!!$
!!$  !> データの取得
!!$  interface pl_bms_read_impl
!!$    module procedure pl_bms_read_int4_val_impl
!!$    module procedure pl_bms_read_int4_array_1d_impl
!!$    module procedure pl_bms_read_int4_array_2d_impl
!!$    module procedure pl_bms_read_int4_array_3d_impl
!!$    module procedure pl_bms_read_real8_val_impl
!!$    module procedure pl_bms_read_real8_array_1d_impl
!!$    module procedure pl_bms_read_real8_array_2d_impl
!!$    module procedure pl_bms_read_real8_array_3d_impl
!!$    module procedure pl_bms_read_real8_array_4d_impl
!!$    !module procedure pl_bms_read_str_array_1d_impl
!!$  end interface

  character(len=*),parameter::PL_BMS_FILE_EXT      = '.xml'

  character(len=*),parameter::PL_BMS_TYPENAME_INT  = 'int'
  character(len=*),parameter::PL_BMS_TYPENAME_REAL = 'float'
  integer,         parameter::PL_BMS_BYTE_4        = 4
  integer,         parameter::PL_BMS_BYTE_8        = 8
  character(len=*),parameter::PL_BMS_PROGRAM_NAME  = ''
  character(len=*),parameter::PL_BMS_TITLE         = ''

  character(len=*),parameter::PL_BMS_ARRAY_ORDER   = 'fortran'
  character(len=*),parameter::PL_BMS_DELIMITER     = ' '
  character(len=*),parameter::PL_BMS_ENDIAN        = ''
  !character(len=*),parameter::PL_BMS_DATA_FORMAT_BASE64 = 'base64'
  !character(len=*),parameter::PL_BMS_DATA_FORMAT_BIN    = 'bin'
  character(len=*),parameter::PL_BMS_BIN_FILE_EXT  = '.bin'
  integer,         parameter::PL_BMS_CID_MAX_LEN   = 8

contains
  !-----------------------------------------------------------------------------
  !> データ存在の問い合わせ
  function pl_bms_is_exists_impl(url, name, id) result(is_exists)
    character(len=*),intent(in)::url
    character(len=*),intent(in)::name
    integer,intent(in)::id
    logical::is_exists

#ifdef PL_BMS_XML
    integer::ndim
    integer::istat

    is_exists = .false.
    ndim = pl_bms_get_ndim_impl(url, name, id)
    if (ndim == PL_BMS_DATA_UNKNOWN) return
#endif
    is_exists = .true.
  end function pl_bms_is_exists_impl

  !-----------------------------------------------------------------------------
  !> データ次元数の取得
  function pl_bms_get_ndim_impl(url, name, id) result(ndim)
    character(len=*),intent(in)::url
    character(len=*),intent(in)::name
    integer,intent(in)::id
    integer::ndim

#ifdef PL_BMS_XML
    character(len=len(url)+len(PL_BMS_FILE_EXT))::file_name
    integer::istat
#endif
    integer::ndim_tmp = 0

#ifdef PL_BMS_XML
    ndim = PL_BMS_DATA_UNKNOWN
    call bms_data_initialize(istat); if (istat /= 0) return
    file_name = pl_bms_helper_get_file_name(url, PL_BMS_FILE_EXT)
    call bms_data_load(file_name, istat); if (istat /= 0) return
    call bms_data_getnumdims(&
         ndim_tmp, file_name, name, id, istat); if (istat /= 0) return
    call bms_data_finalize(istat); if (istat /= 0) return
#endif
    ndim = ndim_tmp
  end function pl_bms_get_ndim_impl

  !-----------------------------------------------------------------------------
  !> データシェイプの取得
  subroutine pl_bms_get_shape_impl(lshape, ndim, url, name, id, istat)
    integer,intent(in)::ndim
    integer,intent(inout)::lshape(ndim)
    character(len=*),intent(in)::url
    character(len=*),intent(in)::name
    integer,intent(in)::id
    integer,intent(inout)::istat

#ifdef PL_BMS_XML
    character(len=len(url)+len(PL_BMS_FILE_EXT))::file_name

    istat = -1
    call bms_data_initialize(istat); if (istat /= 0) return
    file_name = pl_bms_helper_get_file_name(url, PL_BMS_FILE_EXT)
    call bms_data_load(file_name, istat); if (istat /= 0) return
    call bms_data_getsize(&
         lshape, ndim, file_name, name, id, istat); if (istat /= 0) return
    call bms_data_finalize(istat); if (istat /= 0) return
#endif
    istat = 0
  end subroutine pl_bms_get_shape_impl

  !-----------------------------------------------------------------------------
  !> データの保存
  subroutine pl_bms_write_int4_val_impl(data, url, name, id, unit, istat)
    integer,intent(in)::data
    character(len=*),intent(in)::url
    character(len=*),intent(in)::name
    integer,intent(in)::id
    character(len=*),intent(in)::unit
    integer,intent(inout)::istat

#ifdef PL_BMS_XML
    character(len=len(url)+len(PL_BMS_FILE_EXT))::file_name
    character(len=len(url)+len(name)+PL_BMS_CID_MAX_LEN+2+len(PL_BMS_BIN_FILE_EXT))::ext_ref
    character(len=PL_BMS_CID_MAX_LEN)::cid
    character(len=6)::data_format !:= {'text' | 'base64' | 'hex' | 'bin'}
    integer,parameter::ndim = 1
    integer::dims(ndim)

    istat = -1

    call bms_data_initialize(istat); if (istat /= 0) return
    file_name = pl_bms_helper_get_file_name(url, PL_BMS_FILE_EXT)
    call bms_data_loadcreate(file_name, istat); if (istat /= 0) return

    dims(1) = 1
    call bms_data_setDataProperties(&
         file_name, name, id,&
         PL_BMS_TYPENAME_INT, PL_BMS_BYTE_4, dims, ndim, unit,&
         PL_BMS_PROGRAM_NAME, PL_BMS_TITLE, istat)
    if (istat /= 0) return

    data_format = 'base64'
    ext_ref = ''
    call bms_data_setData(&
         data, file_name, name, id,&
         data_format, PL_BMS_ARRAY_ORDER, PL_BMS_DELIMITER, PL_BMS_ENDIAN,&
         ext_ref, istat)
    if (istat /= 0) return

    call bms_data_save(file_name, istat); if (istat /= 0) return
    call bms_data_finalize(istat); if (istat /= 0) return
#endif
    istat = 0
  end subroutine pl_bms_write_int4_val_impl
  
  !-----------------------------------------------------------------------------
  !> データの保存
  subroutine pl_bms_write_int4_array_1d_impl(&
       data, n1, url, name, id, unit, istat)
    integer,intent(in)::n1
    integer,intent(in)::data(n1)
    character(len=*),intent(in)::url
    character(len=*),intent(in)::name
    integer,intent(in)::id
    character(len=*),intent(in)::unit
    integer,intent(inout)::istat

#ifdef PL_BMS_XML
    character(len=len(url)+len(PL_BMS_FILE_EXT))::file_name
    character(len=len(url)+len(name)+PL_BMS_CID_MAX_LEN+2+len(PL_BMS_BIN_FILE_EXT))::ext_ref
    character(len=PL_BMS_CID_MAX_LEN)::cid
    character(len=6)::data_format !:= {'text' | 'base64' | 'hex' | 'bin'}
    integer,parameter::ndim = 1
    integer::dims(ndim)

    istat = -1
    if (n1 < 0) return

    call bms_data_initialize(istat); if (istat /= 0) return
    file_name = pl_bms_helper_get_file_name(url, PL_BMS_FILE_EXT)
    call bms_data_loadcreate(file_name, istat); if (istat /= 0) return

    dims(1) = n1
    call bms_data_setDataProperties(&
         file_name, name, id,&
         PL_BMS_TYPENAME_INT, PL_BMS_BYTE_4, dims, ndim, unit,&
         PL_BMS_PROGRAM_NAME, PL_BMS_TITLE, istat)
    if (istat /= 0) return

    data_format = 'base64'
    ext_ref = ''
    call bms_data_setData(&
         data, file_name, name, id,&
         data_format, PL_BMS_ARRAY_ORDER, PL_BMS_DELIMITER, PL_BMS_ENDIAN,&
         ext_ref, istat)
    if (istat /= 0) return

    call bms_data_save(file_name, istat); if (istat /= 0) return
    call bms_data_finalize(istat); if (istat /= 0) return
#endif
    istat = 0
  end subroutine pl_bms_write_int4_array_1d_impl
  
  !-----------------------------------------------------------------------------
  !> データの保存
  subroutine pl_bms_write_int4_array_2d_impl(&
       data, ld1, n1, n2, url, name, id, unit, istat)
    integer,intent(in)::ld1
    integer,intent(in)::n1, n2
    integer,intent(in)::data(ld1,n2)
    character(len=*),intent(in)::url
    character(len=*),intent(in)::name
    integer,intent(in)::id
    character(len=*),intent(in)::unit
    integer,intent(inout)::istat

#ifdef PL_BMS_XML
    character(len=len(url)+len(PL_BMS_FILE_EXT))::file_name
    character(len=len(url)+len(name)+PL_BMS_CID_MAX_LEN+2+len(PL_BMS_BIN_FILE_EXT))::ext_ref
    character(len=PL_BMS_CID_MAX_LEN)::cid
    character(len=6)::data_format !:= {'text' | 'base64' | 'hex' | 'bin'}
    integer::size
    integer,parameter::ndim = 2
    integer::dims(ndim)
    integer,allocatable::d(:)
    integer::i1, i2, k

    istat = -1
    if (ld1 < 0 .or. n1 < 0 .or. ld1 < n1) return
    if (n2 < 0) return

    call bms_data_initialize(istat); if (istat /= 0) return
    file_name = pl_bms_helper_get_file_name(url, PL_BMS_FILE_EXT)
    call bms_data_loadcreate(file_name, istat); if (istat /= 0) return

    dims(1) = n1
    dims(2) = n2
    call bms_data_setDataProperties(&
         file_name, name, id,&
         PL_BMS_TYPENAME_INT, PL_BMS_BYTE_4, dims, ndim, unit,&
         PL_BMS_PROGRAM_NAME, PL_BMS_TITLE, istat)
    if (istat /= 0) return

    data_format = 'base64'
    ext_ref = ''
    size = pl_bms_helper_get_size(dims)
    allocate(d(size), stat=istat); if (istat /= 0) return
    k = 0
    do i2 = 1, n2
       do i1 = 1, n1
          k = k + 1
          d(k) = data(i1,i2)
       end do
    end do
    call bms_data_setData(&
         d, file_name, name, id,&
         data_format, PL_BMS_ARRAY_ORDER, PL_BMS_DELIMITER, PL_BMS_ENDIAN,&
         ext_ref, istat)
    if (istat /= 0) return
    deallocate(d, stat=istat); if (istat /= 0) return

    call bms_data_save(file_name, istat); if (istat /= 0) return
    call bms_data_finalize(istat); if (istat /= 0) return
#endif
    istat = 0
  end subroutine pl_bms_write_int4_array_2d_impl
  
  !-----------------------------------------------------------------------------
  !> データの保存
  subroutine pl_bms_write_int4_array_3d_impl(&
       data, ld1, ld2, n1, n2, n3, url, name, id, unit, istat)
    integer,intent(in)::ld1, ld2
    integer,intent(in)::n1, n2, n3
    integer,intent(in)::data(ld1,ld2,n3)
    character(len=*),intent(in)::url
    character(len=*),intent(in)::name
    integer,intent(in)::id
    character(len=*),intent(in)::unit
    integer,intent(inout)::istat

#ifdef PL_BMS_XML
    character(len=len(url)+len(PL_BMS_FILE_EXT))::file_name
    character(len=len(url)+len(name)+PL_BMS_CID_MAX_LEN+2+len(PL_BMS_BIN_FILE_EXT))::ext_ref
    character(len=PL_BMS_CID_MAX_LEN)::cid
    character(len=6)::data_format !:= {'text' | 'base64' | 'hex' | 'bin'}
    integer::size
    integer,parameter::ndim = 3
    integer::dims(ndim)
    integer,allocatable::d(:)
    integer::i1, i2, i3, k

    istat = -1
    if (ld1 < 0 .or. n1 < 0 .or. ld1 < n1) return
    if (ld2 < 0 .or. n2 < 0 .or. ld2 < n2) return
    if (n3 < 0) return

    call bms_data_initialize(istat); if (istat /= 0) return
    file_name = pl_bms_helper_get_file_name(url, PL_BMS_FILE_EXT)
    call bms_data_loadcreate(file_name, istat); if (istat /= 0) return

    dims(1) = n1
    dims(2) = n2
    dims(3) = n3
    call bms_data_setDataProperties(&
         file_name, name, id,&
         PL_BMS_TYPENAME_INT, PL_BMS_BYTE_4, dims, ndim, unit,&
         PL_BMS_PROGRAM_NAME, PL_BMS_TITLE, istat)
    if (istat /= 0) return

    data_format = 'base64'
    ext_ref = ''
    size = pl_bms_helper_get_size(dims)
    allocate(d(size), stat=istat); if (istat /= 0) return
    k = 0
    do i3 = 1, n3
       do i2 = 1, n2
          do i1 = 1, n1
             k = k + 1
             d(k) = data(i1,i2,i3)
          end do
       end do
    end do
    call bms_data_setData(&
         d, file_name, name, id,&
         data_format, PL_BMS_ARRAY_ORDER, PL_BMS_DELIMITER, PL_BMS_ENDIAN,&
         ext_ref, istat)
    if (istat /= 0) return
    deallocate(d, stat=istat); if (istat /= 0) return

    call bms_data_save(file_name, istat); if (istat /= 0) return
    call bms_data_finalize(istat); if (istat /= 0) return
#endif
    istat = 0
  end subroutine pl_bms_write_int4_array_3d_impl
  
  !-----------------------------------------------------------------------------
  !> データの保存
  subroutine pl_bms_write_real8_val_impl(data, url, name, id, unit, istat)
    real(8),intent(in)::data
    character(len=*),intent(in)::url
    character(len=*),intent(in)::name
    integer,intent(in)::id
    character(len=*),intent(in)::unit
    integer,intent(inout)::istat

#ifdef PL_BMS_XML
    character(len=len(url)+len(PL_BMS_FILE_EXT))::file_name
    character(len=len(url)+len(name)+PL_BMS_CID_MAX_LEN+2+len(PL_BMS_BIN_FILE_EXT))::ext_ref
    character(len=PL_BMS_CID_MAX_LEN)::cid
    character(len=6)::data_format !:= {'text' | 'base64' | 'hex' | 'bin'}
    integer,parameter::ndim = 1
    integer::dims(ndim)

    istat = -1

    call bms_data_initialize(istat); if (istat /= 0) return
    file_name = pl_bms_helper_get_file_name(url, PL_BMS_FILE_EXT)
    call bms_data_loadcreate(file_name, istat); if (istat /= 0) return

    dims(1) = 1
    call bms_data_setDataProperties(&
         file_name, name, id,&
         PL_BMS_TYPENAME_REAL, PL_BMS_BYTE_8, dims, ndim, unit,&
         PL_BMS_PROGRAM_NAME, PL_BMS_TITLE, istat)
    if (istat /= 0) return

    data_format = 'base64'
    ext_ref = ''
    call bms_data_setData(&
         data, file_name, name, id,&
         data_format, PL_BMS_ARRAY_ORDER, PL_BMS_DELIMITER, PL_BMS_ENDIAN,&
         ext_ref, istat)
    if (istat /= 0) return

    call bms_data_save(file_name, istat); if (istat /= 0) return
    call bms_data_finalize(istat); if (istat /= 0) return
#endif
    istat = 0
  end subroutine pl_bms_write_real8_val_impl

  !-----------------------------------------------------------------------------
  !> データの保存
  subroutine pl_bms_write_real8_array_1d_impl(&
       data, n1, url, name, id, unit, istat)
    integer,intent(in)::n1
    real(8),intent(in)::data(n1)
    character(len=*),intent(in)::url
    character(len=*),intent(in)::name
    integer,intent(in)::id
    character(len=*),intent(in)::unit
    integer,intent(inout)::istat

#ifdef PL_BMS_XML
    character(len=len(url)+len(PL_BMS_FILE_EXT))::file_name
    character(len=len(url)+len(name)+PL_BMS_CID_MAX_LEN+2+len(PL_BMS_BIN_FILE_EXT))::ext_ref
    character(len=PL_BMS_CID_MAX_LEN)::cid
    character(len=6)::data_format !:= {'text' | 'base64' | 'hex' | 'bin'}
    integer,parameter::ndim = 1
    integer::dims(ndim)

    istat = -1
    if (n1 < 0) return

    call bms_data_initialize(istat); if (istat /= 0) return
    file_name = pl_bms_helper_get_file_name(url, PL_BMS_FILE_EXT)
    call bms_data_loadcreate(file_name, istat); if (istat /= 0) return

    dims(1) = n1
    call bms_data_setDataProperties(&
         file_name, name, id,&
         PL_BMS_TYPENAME_REAL, PL_BMS_BYTE_8, dims, ndim, unit,&
         PL_BMS_PROGRAM_NAME, PL_BMS_TITLE, istat)
    if (istat /= 0) return

    if (ndim == 1 .and. dims(1) == 1) then
      data_format = 'base64'
      ext_ref = ''
    else
      data_format = 'bin'
      write(cid,'(i8)') id
      ext_ref =  trim(adjustl(url))  // '_'&
              // trim(adjustl(name)) // '_'&
              // trim(adjustl(cid))  // PL_BMS_BIN_FILE_EXT
    end if
    call bms_data_setData(&
         data, file_name, name, id,&
         data_format, PL_BMS_ARRAY_ORDER, PL_BMS_DELIMITER, PL_BMS_ENDIAN,&
         ext_ref, istat)
    if (istat /= 0) return

    call bms_data_save(file_name, istat); if (istat /= 0) return
    call bms_data_finalize(istat); if (istat /= 0) return
#endif
    istat = 0
  end subroutine pl_bms_write_real8_array_1d_impl

  !-----------------------------------------------------------------------------
  !> データの保存
  subroutine pl_bms_write_real8_array_2d_impl(&
       data, ld1, n1, n2, url, name, id, unit, istat)
    integer,intent(in)::ld1
    integer,intent(in)::n1, n2
    real(8),intent(in)::data(ld1,n2)
    character(len=*),intent(in)::url
    character(len=*),intent(in)::name
    integer,intent(in)::id
    character(len=*),intent(in)::unit
    integer,intent(inout)::istat

#ifdef PL_BMS_XML
    character(len=len(url)+len(PL_BMS_FILE_EXT))::file_name
    character(len=len(url)+len(name)+PL_BMS_CID_MAX_LEN+2+len(PL_BMS_BIN_FILE_EXT))::ext_ref
    character(len=PL_BMS_CID_MAX_LEN)::cid
    character(len=6)::data_format !:= {'text' | 'base64' | 'hex' | 'bin'}
    integer::size
    integer,parameter::ndim = 2
    integer::dims(ndim)
    integer,allocatable::d(:)
    integer::i1, i2, k

    istat = -1
    if (ld1 < 0 .or. n1 < 0 .or. ld1 < n1) return
    if (n2 < 0) return

    call bms_data_initialize(istat); if (istat /= 0) return
    file_name = pl_bms_helper_get_file_name(url, PL_BMS_FILE_EXT)
    call bms_data_loadcreate(file_name, istat); if (istat /= 0) return

    dims(1) = n1
    dims(2) = n2
    call bms_data_setDataProperties(&
         file_name, name, id,&
         PL_BMS_TYPENAME_REAL, PL_BMS_BYTE_8, dims, ndim, unit,&
         PL_BMS_PROGRAM_NAME, PL_BMS_TITLE, istat)
    if (istat /= 0) return

    if (ndim == 1 .and. dims(1) == 1) then
      data_format = 'base64'
      ext_ref = ''
    else
      data_format = 'bin'
      write(cid,'(i8)') id
      ext_ref =  trim(adjustl(url))  // '_'&
              // trim(adjustl(name)) // '_'&
              // trim(adjustl(cid))  // PL_BMS_BIN_FILE_EXT
    end if
    size = pl_bms_helper_get_size(dims)
    allocate(d(size), stat=istat); if (istat /= 0) return
    k = 0
    do i2 = 1, n2
       do i1 = 1, n1
          k = k + 1
          d(k) = data(i1,i2)
       end do
    end do
    call bms_data_setData(&
         data, file_name, name, id,&
         data_format, PL_BMS_ARRAY_ORDER, PL_BMS_DELIMITER, PL_BMS_ENDIAN,&
         ext_ref, istat)
    if (istat /= 0) return
    deallocate(d, stat=istat); if (istat /= 0) return

    call bms_data_save(file_name, istat); if (istat /= 0) return
    call bms_data_finalize(istat); if (istat /= 0) return
#endif
    istat = 0
  end subroutine pl_bms_write_real8_array_2d_impl

  !-----------------------------------------------------------------------------
  !> データの保存
  subroutine pl_bms_write_real8_array_3d_impl(&
       data, ld1, ld2, n1, n2, n3, url, name, id, unit, istat)
    integer,intent(in)::ld1, ld2
    integer,intent(in)::n1, n2, n3
    real(8),intent(in)::data(ld1,ld2,n3)
    character(len=*),intent(in)::url
    character(len=*),intent(in)::name
    integer,intent(in)::id
    character(len=*),intent(in)::unit
    integer,intent(inout)::istat

#ifdef PL_BMS_XML
    character(len=len(url)+len(PL_BMS_FILE_EXT))::file_name
    character(len=len(url)+len(name)+PL_BMS_CID_MAX_LEN+2+len(PL_BMS_BIN_FILE_EXT))::ext_ref
    character(len=PL_BMS_CID_MAX_LEN)::cid
    character(len=6)::data_format !:= {'text' | 'base64' | 'hex' | 'bin'}
    integer,parameter::ndim = 3
    integer::size
    integer::dims(ndim)
    integer,allocatable::d(:)
    integer::i1, i2, i3, k

    istat = -1
    if (ld1 < 0 .or. n1 < 0 .or. ld1 < n1) return
    if (ld2 < 0 .or. n2 < 0 .or. ld2 < n2) return
    if (n3 < 0) return

    call bms_data_initialize(istat); if (istat /= 0) return
    file_name = pl_bms_helper_get_file_name(url, PL_BMS_FILE_EXT)
    call bms_data_loadcreate(file_name, istat); if (istat /= 0) return

    dims(1) = n1
    dims(2) = n2
    dims(3) = n3
    call bms_data_setDataProperties(&
         file_name, name, id,&
         PL_BMS_TYPENAME_REAL, PL_BMS_BYTE_8, dims, ndim, unit,&
         PL_BMS_PROGRAM_NAME, PL_BMS_TITLE, istat)
    if (istat /= 0) return

    if (ndim == 1 .and. dims(1) == 1) then
      data_format = 'base64'
      ext_ref = ''
    else
      data_format = 'bin'
      write(cid,'(i8)') id
      ext_ref =  trim(adjustl(url))  // '_'&
              // trim(adjustl(name)) // '_'&
              // trim(adjustl(cid))  // PL_BMS_BIN_FILE_EXT
    end if
    size = pl_bms_helper_get_size(dims)
    allocate(d(size), stat=istat); if (istat /= 0) return
    k = 0
    do i3 = 1, n3
       do i2 = 1, n2
          do i1 = 1, n1
             k = k + 1
             d(k) = data(i1,i2,i3)
          end do
       end do
    end do
    call bms_data_setData(&
         data, file_name, name, id,&
         data_format, PL_BMS_ARRAY_ORDER, PL_BMS_DELIMITER, PL_BMS_ENDIAN,&
         ext_ref, istat)
    if (istat /= 0) return
    deallocate(d, stat=istat); if (istat /= 0) return

    call bms_data_save(file_name, istat); if (istat /= 0) return
    call bms_data_finalize(istat); if (istat /= 0) return
#endif
    istat = 0
  end subroutine pl_bms_write_real8_array_3d_impl

  !-----------------------------------------------------------------------------
  !> データの保存
  subroutine pl_bms_write_real8_array_4d_impl(&
       data, ld1, ld2, ld3, n1, n2, n3, n4, url, name, id, unit, istat)
    integer,intent(in)::ld1, ld2, ld3
    integer,intent(in)::n1, n2, n3, n4
    real(8),intent(in)::data(ld1,ld2,ld3,n4)
    character(len=*),intent(in)::url
    character(len=*),intent(in)::name
    integer,intent(in)::id
    character(len=*),intent(in)::unit
    integer,intent(inout)::istat

#ifdef PL_BMS_XML
    character(len=len(url)+len(PL_BMS_FILE_EXT))::file_name
    character(len=len(url)+len(name)+PL_BMS_CID_MAX_LEN+2+len(PL_BMS_BIN_FILE_EXT))::ext_ref
    character(len=PL_BMS_CID_MAX_LEN)::cid
    character(len=6)::data_format !:= {'text' | 'base64' | 'hex' | 'bin'}
    integer::size
    integer,parameter::ndim = 4
    integer::dims(ndim)
    integer,allocatable::d(:)
    integer::i1, i2, i3, i4, k

    istat = -1
    if (ld1 < 0 .or. n1 < 0 .or. ld1 < n1) return
    if (ld2 < 0 .or. n2 < 0 .or. ld2 < n2) return
    if (ld3 < 0 .or. n3 < 0 .or. ld3 < n3) return
    if (n4 < 0) return

    call bms_data_initialize(istat); if (istat /= 0) return
    file_name = pl_bms_helper_get_file_name(url, PL_BMS_FILE_EXT)
    call bms_data_loadcreate(file_name, istat); if (istat /= 0) return

    dims(1) = n1
    dims(2) = n2
    dims(3) = n3
    dims(4) = n4
    call bms_data_setDataProperties(&
         file_name, name, id,&
         PL_BMS_TYPENAME_REAL, PL_BMS_BYTE_8, dims, ndim, unit,&
         PL_BMS_PROGRAM_NAME, PL_BMS_TITLE, istat)
    if (istat /= 0) return

    if (ndim == 1 .and. dims(1) == 1) then
      data_format = 'base64'
      ext_ref = ''
    else
      data_format = 'bin'
      write(cid,'(i8)') id
      ext_ref =  trim(adjustl(url))  // '_'&
              // trim(adjustl(name)) // '_'&
              // trim(adjustl(cid))  // PL_BMS_BIN_FILE_EXT
    end if
    size = pl_bms_helper_get_size(dims)
    allocate(d(size), stat=istat); if (istat /= 0) return
    k = 0
    do i4 = 1, n4
       do i3 = 1, n3
          do i2 = 1, n2
             do i1 = 1, n1
                k = k + 1
                d(k) = data(i1,i2,i3,i4)
             end do
          end do
       end do
    end do
    call bms_data_setData(&
         data, file_name, name, id,&
         data_format, PL_BMS_ARRAY_ORDER, PL_BMS_DELIMITER, PL_BMS_ENDIAN,&
         ext_ref, istat)
    if (istat /= 0) return
    deallocate(d, stat=istat); if (istat /= 0) return

    call bms_data_save(file_name, istat); if (istat /= 0) return
    call bms_data_finalize(istat); if (istat /= 0) return
#endif
    istat = 0
  end subroutine pl_bms_write_real8_array_4d_impl

  !-----------------------------------------------------------------------------
  !> データの保存
  subroutine pl_bms_write_str_val_impl(&
       data, url, name, id, unit, istat)
    character(len=*),intent(in)::data
    character(len=*),intent(in)::url
    character(len=*),intent(in)::name
    integer,intent(in)::id
    character(len=*),intent(in)::unit
    integer,intent(inout)::istat

#ifdef PL_BMS_XML
    character(len=len(url)+len(PL_BMS_FILE_EXT))::file_name
    character(len=len(url)+len(name)+PL_BMS_CID_MAX_LEN+2+len(PL_BMS_BIN_FILE_EXT))::ext_ref
    character(len=PL_BMS_CID_MAX_LEN)::cid
    character(len=6)::data_format !:= {'text' | 'base64' | 'hex' | 'bin'}
    integer,parameter::ndim = 1
    integer::dims(ndim)

    istat = -1
    if (n1 < 0) return

#endif
    istat = 0
  end subroutine pl_bms_write_str_val_impl

  !-----------------------------------------------------------------------------
  !> データの保存
  subroutine pl_bms_write_str_array_1d_impl(&
       data, n1, url, name, id, unit, istat)
    integer,intent(in)::n1
    character(len=*),intent(in)::data(n1)
    character(len=*),intent(in)::url
    character(len=*),intent(in)::name
    integer,intent(in)::id
    character(len=*),intent(in)::unit
    integer,intent(inout)::istat

#ifdef PL_BMS_XML
    character(len=len(url)+len(PL_BMS_FILE_EXT))::file_name
    character(len=len(url)+len(name)+PL_BMS_CID_MAX_LEN+2+len(PL_BMS_BIN_FILE_EXT))::ext_ref
    character(len=PL_BMS_CID_MAX_LEN)::cid
    character(len=6)::data_format !:= {'text' | 'base64' | 'hex' | 'bin'}
    integer,parameter::ndim = 1
    integer::dims(ndim)

    istat = -1
    if (n1 < 0) return

#endif
    istat = 0
  end subroutine pl_bms_write_str_array_1d_impl

  !-----------------------------------------------------------------------------
  !> データの取得
  subroutine pl_bms_read_int4_val_impl(data, url, name, id, istat)
    integer,intent(inout)::data
    character(len=*),intent(in)::url
    character(len=*),intent(in)::name
    integer,intent(in)::id
    integer,intent(inout)::istat

#ifdef PL_BMS_XML
    character(len=len(url)+len(PL_BMS_FILE_EXT))::file_name
    integer::size

    size = 1
    call bms_data_initialize(istat); if (istat /= 0) return
    file_name = pl_bms_helper_get_file_name(url, PL_BMS_FILE_EXT)
    call bms_data_load(file_name, istat); if (istat /= 0) return
    call bms_data_getData(&
         data, size, file_name, name, id,&
         PL_BMS_TYPENAME_INT, PL_BMS_BYTE_4, PL_BMS_ARRAY_ORDER, istat)
    if (istat /= 0) return
    call bms_data_finalize(istat); if (istat /= 0) return
#endif
    istat = 0
  end subroutine pl_bms_read_int4_val_impl

  !-----------------------------------------------------------------------------
  !> データの取得
  subroutine pl_bms_read_int4_array_1d_impl(&
       data, n1, url, name, id, istat)
    integer,intent(in)::n1
    integer,intent(inout)::data(n1)
    character(len=*),intent(in)::url
    character(len=*),intent(in)::name
    integer,intent(in)::id
    integer,intent(inout)::istat

#ifdef PL_BMS_XML
    character(len=len(url)+len(PL_BMS_FILE_EXT))::file_name
    integer::size

    size = n1
    call bms_data_initialize(istat); if (istat /= 0) return
    file_name = pl_bms_helper_get_file_name(url, PL_BMS_FILE_EXT)
    call bms_data_load(file_name, istat); if (istat /= 0) return
    call bms_data_getData(&
         data, size, file_name, name, id,&
         PL_BMS_TYPENAME_INT, PL_BMS_BYTE_4, PL_BMS_ARRAY_ORDER, istat)
    if (istat /= 0) return
    call bms_data_finalize(istat); if (istat /= 0) return
#endif
    istat = 0
  end subroutine pl_bms_read_int4_array_1d_impl

  !-----------------------------------------------------------------------------
  !> データの取得
  subroutine pl_bms_read_int4_array_2d_impl(&
       data, ld1, n1, n2, url, name, id, istat)
    integer,intent(in)::ld1
    integer,intent(in)::n1, n2
    integer,intent(inout)::data(ld1,n2)
    character(len=*),intent(in)::url
    character(len=*),intent(in)::name
    integer,intent(in)::id
    integer,intent(inout)::istat

#ifdef PL_BMS_XML
    character(len=len(url)+len(PL_BMS_FILE_EXT))::file_name
    integer::size

    size = n1*n2
    call bms_data_initialize(istat); if (istat /= 0) return
    file_name = pl_bms_helper_get_file_name(url, PL_BMS_FILE_EXT)
    call bms_data_load(file_name, istat); if (istat /= 0) return
    call bms_data_getData(&
         data, size, file_name, name, id,&
         PL_BMS_TYPENAME_INT, PL_BMS_BYTE_4, PL_BMS_ARRAY_ORDER, istat)
    if (istat /= 0) return
    call bms_data_finalize(istat); if (istat /= 0) return
#endif
    istat = 0
  end subroutine pl_bms_read_int4_array_2d_impl

  !-----------------------------------------------------------------------------
  !> データの取得
  subroutine pl_bms_read_int4_array_3d_impl(&
       data, ld1, ld2, n1, n2, n3, url, name, id, istat)
    integer,intent(in)::ld1, ld2
    integer,intent(in)::n1, n2, n3
    integer,intent(inout)::data(ld1,ld2,n3)
    character(len=*),intent(in)::url
    character(len=*),intent(in)::name
    integer,intent(in)::id
    integer,intent(inout)::istat

#ifdef PL_BMS_XML
    character(len=len(url)+len(PL_BMS_FILE_EXT))::file_name
    integer::size

    size = n1*n2*n3
    call bms_data_initialize(istat); if (istat /= 0) return
    file_name = pl_bms_helper_get_file_name(url, PL_BMS_FILE_EXT)
    call bms_data_load(file_name, istat); if (istat /= 0) return
    call bms_data_getData(&
         data, size, file_name, name, id,&
         PL_BMS_TYPENAME_INT, PL_BMS_BYTE_4, PL_BMS_ARRAY_ORDER, istat)
    if (istat /= 0) return
    call bms_data_finalize(istat); if (istat /= 0) return
#endif
    istat = 0
  end subroutine pl_bms_read_int4_array_3d_impl

  !-----------------------------------------------------------------------------
  !> データの取得
  subroutine pl_bms_read_real8_val_impl(data, url, name, id, istat)
    real(8),intent(inout)::data
    character(len=*),intent(in)::url
    character(len=*),intent(in)::name
    integer,intent(in)::id
    integer,intent(inout)::istat

#ifdef PL_BMS_XML
    character(len=len(url)+len(PL_BMS_FILE_EXT))::file_name
    integer::size

    size = 1
    call bms_data_initialize(istat); if (istat /= 0) return
    file_name = pl_bms_helper_get_file_name(url, PL_BMS_FILE_EXT)
    call bms_data_load(file_name, istat); if (istat /= 0) return
    call bms_data_getData(&
         data, size, file_name, name, id,&
         PL_BMS_TYPENAME_REAL, PL_BMS_BYTE_8, PL_BMS_ARRAY_ORDER, istat)
    if (istat /= 0) return
    call bms_data_finalize(istat); if (istat /= 0) return
#endif
    istat = 0
  end subroutine pl_bms_read_real8_val_impl

  !-----------------------------------------------------------------------------
  !> データの取得
  subroutine pl_bms_read_real8_array_1d_impl(&
       data, n1, url, name, id, istat)
    integer,intent(in)::n1
    real(8),intent(inout)::data(n1)
    character(len=*),intent(in)::url
    character(len=*),intent(in)::name
    integer,intent(in)::id
    integer,intent(inout)::istat

#ifdef PL_BMS_XML
    character(len=len(url)+len(PL_BMS_FILE_EXT))::file_name
    integer::size

    size = n1
    call bms_data_initialize(istat); if (istat /= 0) return
    file_name = pl_bms_helper_get_file_name(url, PL_BMS_FILE_EXT)
    call bms_data_load(file_name, istat); if (istat /= 0) return
    call bms_data_getData(&
         data, size, file_name, name, id,&
         PL_BMS_TYPENAME_REAL, PL_BMS_BYTE_8, PL_BMS_ARRAY_ORDER, istat)
    if (istat /= 0) return
    call bms_data_finalize(istat); if (istat /= 0) return
#endif
    istat = 0
  end subroutine pl_bms_read_real8_array_1d_impl

  !-----------------------------------------------------------------------------
  !> データの取得
  subroutine pl_bms_read_real8_array_2d_impl(&
       data, ld1, n1, n2, url, name, id, istat)
    integer,intent(in)::ld1
    integer,intent(in)::n1, n2
    real(8),intent(inout)::data(ld1,n2)
    character(len=*),intent(in)::url
    character(len=*),intent(in)::name
    integer,intent(in)::id
    integer,intent(inout)::istat

#ifdef PL_BMS_XML
    character(len=len(url)+len(PL_BMS_FILE_EXT))::file_name
    integer::size

    size = n1*n2
    call bms_data_initialize(istat); if (istat /= 0) return
    file_name = pl_bms_helper_get_file_name(url, PL_BMS_FILE_EXT)
    call bms_data_load(file_name, istat); if (istat /= 0) return
    call bms_data_getData(&
         data, size, file_name, name, id,&
         PL_BMS_TYPENAME_REAL, PL_BMS_BYTE_8, PL_BMS_ARRAY_ORDER, istat)
    if (istat /= 0) return
    call bms_data_finalize(istat); if (istat /= 0) return
#endif
    istat = 0
  end subroutine pl_bms_read_real8_array_2d_impl

  !-----------------------------------------------------------------------------
  !> データの取得
  subroutine pl_bms_read_real8_array_3d_impl(&
       data, ld1, ld2, n1, n2, n3, url, name, id, istat)
    integer,intent(in)::ld1, ld2
    integer,intent(in)::n1, n2, n3
    real(8),intent(inout)::data(ld1,ld2,n3)
    character(len=*),intent(in)::url
    character(len=*),intent(in)::name
    integer,intent(in)::id
    integer,intent(inout)::istat

#ifdef PL_BMS_XML
    character(len=len(url)+len(PL_BMS_FILE_EXT))::file_name
    integer::size

    size = n1*n2*n3
    call bms_data_initialize(istat); if (istat /= 0) return
    file_name = pl_bms_helper_get_file_name(url, PL_BMS_FILE_EXT)
    call bms_data_load(file_name, istat); if (istat /= 0) return
    call bms_data_getData(&
         data, size, file_name, name, id,&
         PL_BMS_TYPENAME_REAL, PL_BMS_BYTE_8, PL_BMS_ARRAY_ORDER, istat)
    if (istat /= 0) return
    call bms_data_finalize(istat); if (istat /= 0) return
#endif
    istat = 0
  end subroutine pl_bms_read_real8_array_3d_impl

  !-----------------------------------------------------------------------------
  !> データの取得
  subroutine pl_bms_read_real8_array_4d_impl(&
       data, ld1, ld2, ld3, n1, n2, n3, n4, url, name, id, istat)
    integer,intent(in)::ld1, ld2, ld3
    integer,intent(in)::n1, n2, n3, n4
    real(8),intent(inout)::data(ld1,ld2,ld3,n4)
    character(len=*),intent(in)::url
    character(len=*),intent(in)::name
    integer,intent(in)::id
    integer,intent(inout)::istat

#ifdef PL_BMS_XML
    character(len=len(url)+len(PL_BMS_FILE_EXT))::file_name
    integer::size

    size = n1*n2*n3*n4
    call bms_data_initialize(istat); if (istat /= 0) return
    file_name = pl_bms_helper_get_file_name(url, PL_BMS_FILE_EXT)
    call bms_data_load(file_name, istat); if (istat /= 0) return
    call bms_data_getData(&
         data, size, file_name, name, id,&
         PL_BMS_TYPENAME_REAL, PL_BMS_BYTE_8, PL_BMS_ARRAY_ORDER, istat)
    if (istat /= 0) return
    call bms_data_finalize(istat); if (istat /= 0) return
#endif
    istat = 0
  end subroutine pl_bms_read_real8_array_4d_impl

  !-----------------------------------------------------------------------------
  !> データの取得
  subroutine pl_bms_read_str_val_impl(&
       data, url, name, id, istat)
    character(len=*),intent(inout)::data
    character(len=*),intent(in)::url
    character(len=*),intent(in)::name
    integer,intent(in)::id
    integer,intent(inout)::istat

#ifdef PL_BMS_XML
    character(len=len(url)+len(PL_BMS_FILE_EXT))::file_name
    integer::size

#endif
    istat = 0
  end subroutine pl_bms_read_str_val_impl

  !-----------------------------------------------------------------------------
  !> データの取得
  subroutine pl_bms_read_str_array_1d_impl(&
       data, n1, url, name, id, istat)
    integer,intent(in)::n1
    character(len=*),intent(inout)::data(n1)
    character(len=*),intent(in)::url
    character(len=*),intent(in)::name
    integer,intent(in)::id
    integer,intent(inout)::istat

#ifdef PL_BMS_XML
    character(len=len(url)+len(PL_BMS_FILE_EXT))::file_name
    integer::size

#endif
    istat = 0
  end subroutine pl_bms_read_str_array_1d_impl

  !-----------------------------------------------------------------------------
end module pl_bms_impl_xml_class
