! -*- coding: utf-8 -*-
!
! Dual licensed under the MIT or GPL Version 3 license.
! See 'LICENSE.MIT' for the definition of the MIT license,
! and 'COPYING' for the definition of the GPL license.
!
!> MPIクラス
!!
module pl_mpi_class
!!$  use mpi
  implicit none
  include 'mpif.h'
!!$  private

!!$  public::pl_mpi_init
!!$  public::pl_mpi_finalize

  type::pl_mpi_type
    integer::myrank !< ランク
    integer::nprocs !< プロセス数
    integer::comm   !< コミュニケータ
    integer::root
  end type

contains
  !-----------------------------------------------------------------------------
  !> 初期化
  subroutine pl_mpi_init(self, istat)
    type(pl_mpi_type),intent(inout)::self
    integer,intent(inout)::istat

    istat = 0
    self%comm = MPI_COMM_WORLD
    self%root = 0

    call MPI_Init(istat)
    if (istat /= 0) return

    call pl_mpi_prep(self, istat)

  end subroutine pl_mpi_init

  !-----------------------------------------------------------------------------
  !> コミュニケーター設定
  subroutine pl_mpi_set_comm(self, icomm, istat)
    type(pl_mpi_type),intent(inout)::self
    integer,intent(in)::icomm
    integer,intent(inout)::istat

    istat = 0
    self%comm = icomm
    self%root = 0

    call pl_mpi_prep(self, istat)

  end subroutine pl_mpi_set_comm

  !-----------------------------------------------------------------------------
  !> 前処理
  subroutine pl_mpi_prep(self, istat)
    type(pl_mpi_type),intent(inout)::self
    integer,intent(inout)::istat

    istat = 0
    call MPI_Comm_rank(self%comm, self%myrank, istat)
    if (istat /= 0) return
    call MPI_Comm_size(self%comm, self%nprocs, istat)
    if (istat /= 0) return

  end subroutine pl_mpi_prep

  !-----------------------------------------------------------------------------
  !> 終了処理
  subroutine pl_mpi_finalize(self, istat)
    type(pl_mpi_type),intent(inout)::self
    integer,intent(inout)::istat

    istat = 0
    call MPI_Finalize(istat)

  end subroutine pl_mpi_finalize
  
  !-----------------------------------------------------------------------------
  !> 情報の表示
  subroutine pl_mpi_print_info(self)
    type(pl_mpi_type),intent(in)::self

    write(6,'(''size ='',i5,''   rank ='',i5)') self%nprocs, self%myrank

  end subroutine pl_mpi_print_info
  
  !-----------------------------------------------------------------------------
  !> 情報の表示
  subroutine pl_mpi_print_nprocs(self)
    type(pl_mpi_type),intent(in)::self

    if (self%myrank == 0) then
      write(6,'(''Number of procs = '',i5)') self%nprocs
    end if

  end subroutine pl_mpi_print_nprocs
  
  !-----------------------------------------------------------------------------
  !> 経過時間の取得
  function pl_mpi_wtime(self) result(wtime)
    type(pl_mpi_type),intent(in)::self
    real(8)::wtime
    integer::istat

    call MPI_BARRIER(self%comm, istat)
    wtime = MPI_Wtime()

  end function pl_mpi_wtime
  
  !-----------------------------------------------------------------------------
end module pl_mpi_class
