! -*- coding: utf-8 -*-
!
! Dual licensed under the MIT or GPL Version 3 license.
! See 'LICENSE.MIT' for the definition of the MIT license,
! and 'COPYING' for the definition of the GPL license.
!
!> 文字列操作
module pl_str_utils_class
  implicit none

  private::skip_space, get_token

  integer,public,parameter::PL_STR_MAX_LEN = 128

!!$  interface pl_to_s
!!$    module procedure pl_to_s_int
!!$  end interface

  interface pl_str_to_int
     module procedure pl_str_to_int4
     module procedure pl_str_to_int8
  end interface

  character(len=*),private,parameter::c_lower = 'abcdefghijklmnopqrstuvwxyz'
  character(len=*),private,parameter::c_upper = 'ABCDEFGHIJKLMNOPQRSTUVWXYZ'
  character(len=*),private,parameter::c_digit = '1234567890'
  character(len=*),private,parameter::c_minus = '-'
  character(len=*),private,parameter::c_plus = '+'
  character(len=*),private,parameter::c_sign = c_minus // c_plus

  character(len=*),private,parameter::c_space = ' '
  character(len=*),private,parameter::c_tab = achar(9)
  character(len=*),private,parameter::c_blank = c_space // c_tab

contains
  !-----------------------------------------------------------------------------
  function pl_str_int2str(val) result(res)
    integer,intent(in)::val
    character(len=PL_STR_MAX_LEN)::res

    write(res,*) val

  end function pl_str_int2str

  !-----------------------------------------------------------------------------
  subroutine pl_str_to_int4(str, val, istat)
    character(len=*),intent(in)::str
    integer,intent(out)::val
    integer,intent(out)::istat

    character(len=32)::form
    integer::ios

    form = ' '
    write(form, '( ''(i'', i0,'')'')') len_trim(str)
    !write(6,*) trim(str),' ',trim(form)
!!$    if (len_trim(str) /= 0) then
!!$       if (index(c_digit, str(1:1)) == 0) go to 9
      !read(str,'(i12)',iostat=ios,err=9) val
       read(str, form, iostat=ios, err=9) val
       !write(6,*) 'ios=',ios
!!$    else
!!$       val = 0
!!$    end if
    !write(6,*) 'val=',val

    istat = 0
    return
9   istat = -1
    val = 0
  end subroutine pl_str_to_int4

  !-----------------------------------------------------------------------------
  subroutine pl_str_to_int8(str, val, istat)
    character(len=*),intent(in)::str
    integer(8),intent(out)::val
    integer,intent(out)::istat

    character(len=32)::form
    integer::ios

    write(form, '( ''(i'', i0,'')'')') len_trim(str)
    !write(6,*) trim(str),' ',trim(form)
!!$    if (len_trim(str) /= 0) then
!!$       if (index(c_digit, str(1:1)) == 0) go to 9
      !read(str,'(i12)',iostat=ios,err=9) val
       read(str, form, iostat=ios, err=9) val
       !write(6,*) 'ios=',ios
!!$    else
!!$       val = 0
!!$    end if
    !write(6,*) 'val=',val

    istat = 0
    return
9   istat = -1
    val = 0
  end subroutine pl_str_to_int8

  !-----------------------------------------------------------------------------
  subroutine pl_str_to_dbl(str, val, istat)
    character(len=*),intent(in)::str
    real(8),intent(out)::val
    integer,intent(out)::istat

    integer::ios

    if (len_trim(str) /= 0) then
       if (index(c_digit // c_sign, str(1:1)) == 0) go to 9
       read(str,*,iostat=ios,err=9) val
       !write(6,*) 'ios=',ios
    else
       val = 0.0d0
    end if
    !write(6,*) 'val=',val

    istat = 0
    return
9   istat = -1
  end subroutine pl_str_to_dbl

  !-----------------------------------------------------------------------------
  function pl_str_check(req, str) result(res)
    character(len=*),intent(in)::req
    character(len=*),intent(in)::str
    integer::res

    integer::ln_req, ln_str, i

    ln_req = len_trim(req)
    ln_str = len_trim(str)

    res = 0
    do i = 1, ln_str
       if (index(trim(req), str(i:i)) == 0) then
          res = i
          return
       end if
    end do

  end function pl_str_check

  !-----------------------------------------------------------------------------
  function pl_str_cmp_ignorecase(a, b) result(res)
    character(len=*),intent(in)::a
    character(len=*),intent(in)::b
    logical::res

    integer::ln_a, ln_b, i
    character(len=len_trim(a))::al
    character(len=len_trim(b))::bl

    res = .false.

    ln_a = len_trim(a)
    ln_b = len_trim(b)
    if (ln_a /= ln_b) return

    call pl_str_tolower(a, al)
    call pl_str_tolower(b, bl)
    if (trim(al) /= trim(bl)) return
    res = .true.
    return

  end function pl_str_cmp_ignorecase

  !-----------------------------------------------------------------------------
  subroutine pl_str_toupper(str, res)
    character(len=*),intent(in)::str
    character(len=*),intent(out)::res

    integer::ln, i, pos

    ln = len_trim(str)
    res = str
    do i = 1, ln
       pos = index(c_lower, str(i:i))
       if (pos > 0) then
          res(i:i) = c_upper(pos:pos)
       end if
    end do

  end subroutine pl_str_toupper

  !-----------------------------------------------------------------------------
  subroutine pl_str_tolower(str, res)
    character(len=*),intent(in)::str
    character(len=*),intent(out)::res

    integer::ln, i, pos

    ln = len_trim(str)
    res = str
    do i = 1, ln
       pos = index(c_upper, str(i:i))
       if (pos > 0) then
          res(i:i) = c_lower(pos:pos)
       end if
    end do

  end subroutine pl_str_tolower

  !-----------------------------------------------------------------------------
  !> @brief nameと同じ文字列のnamesの位置を取得
  !>
  function pl_str_search_name(name, names, n) result(ipos)
    integer,intent(in)::n
    character(len=*),intent(in)::name
    character(len=*),intent(in)::names(n)

    integer::ipos
    integer::i

    ipos = 0
    do i = 1, n
       if (trim(name) == trim(names(i))) then
          ipos = i
          return
       end if
    end do

  end function pl_str_search_name

  !-----------------------------------------------------------------------------
  !> @brief nameと同じ文字列のnamesの位置を取得 (大文字小文字を区別しない)
  !>
  function pl_str_search_name_ignorecase(name, names, n) result(ipos)
    integer,intent(in)::n
    character(len=*),intent(in)::name
    character(len=*),intent(in)::names(n)

    character(len=len(name))::name_lower
    character(len=len(names(1)))::names_lower
    integer::ipos
    integer::i

    call pl_str_tolower(name, name_lower)
    !write(6,*) 'name_lower=',name_lower
    ipos = 0
    do i = 1, n
       call pl_str_tolower(names(i), names_lower)
       !write(6,*) 'names_lower=',names_lower
       if (trim(name_lower) == trim(names_lower)) then
          ipos = i
          return
       end if
    end do

  end function pl_str_search_name_ignorecase

  !-----------------------------------------------------------------------------
  !> @brief strを分割
  !>
  subroutine pl_str_split(str, carray, ld, n, istat)
    character(len=*),intent(in)::str
    integer,intent(in)::ld
    integer,intent(out)::n
    character(len=*),intent(inout)::carray(ld)
    integer,intent(inout)::istat

    integer::ln, i, ipos, icol_s, icol_e

    istat = 0
    ln = len_trim(str)
    icol_s = 1
    !write(6,*) 'str=',str
    !write(6,*) 'ln =',ln
    i = 0
    do while(icol_s <= ln)
       call skip_space(str, icol_s, ln, ipos, istat)
       icol_s = ipos
       !write(6,*) 'icol_s =', icol_s
       call get_token(str, icol_s, ln, ipos, istat)
       !write(6,*) 'ipos =', ipos
       i = i + 1
       if (i > ld) then
          istat = -1
          return
       end if
       !write(6,*) 'token=',str(icol_s:ipos)
       carray(i) = str(icol_s:ipos)
       icol_s = ipos + 1
    end do
    n = i

  end subroutine pl_str_split

  !-----------------------------------------------------------------------------
  subroutine skip_space(cline, icol_s, icol_e, ipos, istat)
    character(len=*),intent(in)::cline !< 探索される文字列
    integer,intent(in)::icol_s  !< 探索開始カラム位置
    integer,intent(in)::icol_e  !< 探索終了カラム位置
    integer,intent(out)::ipos   !< 探索されたカラム位置
    integer,intent(out)::istat  !< 終了ステータス (=0:正常, /=0:エラー)

    integer::icol

    if (icol_s < 0 .or. len(cline) < icol_e) then
       ipos = 0
       istat = -1
       !write(6,*) 'internal error in skip_space'
       return
    end if

    istat = 0
    ipos = icol_e + 1
    do icol = icol_s, icol_e
       !write(6,*) icol,':',cline(icol:icol)
       if (index(c_blank, cline(icol:icol)) /= 0) cycle
       ipos = icol
       return
    end do

  end subroutine skip_space

  !-----------------------------------------------------------------------------
  subroutine get_token(cline, icol_s, icol_e, ipos, istat)
    character(len=*),intent(in)::cline !< 探索される文字列
    integer,intent(in)::icol_s  !< 探索開始カラム位置
    integer,intent(in)::icol_e  !< 探索終了カラム位置
    integer,intent(out)::ipos   !< 探索されたカラム位置
    integer,intent(out)::istat  !< 終了ステータス (=0:正常, /=0:エラー)

    integer::icol

    if (icol_s < 0 .or. len(cline) < icol_e) then
       ipos = 0
       istat = -1
       !write(6,*) 'internal error in get_token'
       return
    end if

    istat = 0
    ipos = icol_s - 1
    do icol = icol_s, icol_e
       !write(6,*) icol,':',cline(icol:icol)
       if (index(c_blank, cline(icol:icol)) /= 0) then
          ipos = icol - 1
          return
       end if
    end do
    ipos = icol_e

  end subroutine get_token

  !-----------------------------------------------------------------------------
end module pl_str_utils_class
