! -*- coding: utf-8 -*-
!
! Dual licensed under the MIT or GPL Version 3 license.
! See 'LICENSE.MIT' for the definition of the MIT license,
! and 'COPYING' for the definition of the GPL license.
!
!> 点電荷クラス
!!
module ptcharges_class
  use error_class
  implicit none
!!$  private

  public::init_ptcharges
  public::new_ptcharges
  public::free_ptcharges
  public::read_inp_ptcharges
  public::read_ptcharges
  public::set_ptcharges
  public::get_ptcharges
  public::num_ptcharges
  public::print_ptcharges
  public::print_brief_ptcharges

  public::save_ptcharges
  public::save_ptcharges_f
  public::load_ptcharges
  public::load_ptcharges_f
  public::bcast_ptcharges

  public::ptcharges_type

  type::ptcharges_type
    integer::num                  !< the number of point charges
    real(8),allocatable::chg(:)   !< charges in a.u.
    real(8),allocatable::xyz(:,:) !< cartesian coordinates in a.u.
  end type

  character(len=*),private,parameter::url = 'data_pointcharges'
  integer,private,parameter::id = 1
  character(len=*),private,parameter::unit_au = 'a.u.'
  character(len=*),private,parameter::unit = ' '

contains
  !-----------------------------------------------------------------------------
  !> BMS形式ファイルへの保存
  subroutine save_ptcharges(self, ldebug, istat)
    use pl_bms_class
    type(ptcharges_type),intent(in)::self
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    call save_ptcharges_f(self, url, id, ldebug, istat)

  end subroutine save_ptcharges
  !-----------------------------------------------------------------------------
  !> BMS形式ファイルへの保存
  subroutine save_ptcharges_f(self, url, id, ldebug, istat)
    use pl_bms_class
    type(ptcharges_type),intent(in)::self
    character(len=*),intent(in)::url
    integer,intent(in)::id
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    call pl_bms_write(self%num, url, 'nxyz', id, unit, istat)
    if (istat /= 0) return

    if (self%num <= 0) return

    call pl_bms_write(self%chg, self%num, url, 'chg', id, unit_au, istat)
    if (istat /= 0) return
    call pl_bms_write(self%xyz, 3, 3, self%num, url, 'xyz', id, unit_au, istat)
    if (istat /= 0) return

  end subroutine save_ptcharges_f

  !-----------------------------------------------------------------------------
  !> BMS形式ファイルからの読み込み
  subroutine load_ptcharges(self, ldebug, istat)
    use pl_bms_class
    type(ptcharges_type),intent(inout)::self
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    call load_ptcharges_f(self, url, id, ldebug, istat)

  end subroutine load_ptcharges

  !-----------------------------------------------------------------------------
  !> BMS形式ファイルからの読み込み
  subroutine load_ptcharges_f(self, url, id, ldebug, istat)
    use pl_bms_class
    type(ptcharges_type),intent(inout)::self
    character(len=*),intent(in)::url
    integer,intent(in)::id
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    integer::nxyz

    call pl_bms_read(nxyz, url, 'nxyz', id, istat)
    if (istat /= 0) return

    call new_ptcharges(self, nxyz, istat)
    if (istat /= 0) return

    if (self%num <= 0) return

    call pl_bms_read(self%chg, self%num, url, 'chg', id, istat)
    if (istat /= 0) return
    call pl_bms_read(self%xyz, 3, 3, self%num, url, 'xyz', id, istat)
    if (istat /= 0) return

  end subroutine load_ptcharges_f

  !-----------------------------------------------------------------------------
  !> データBcast
  subroutine bcast_ptcharges(self, pl_mpi, ldebug, istat)
    use pl_mpi_class
    type(ptcharges_type),intent(inout)::self
    type(pl_mpi_type),intent(in)::pl_mpi
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    integer::nxyz
    integer::icomm, myrank

    istat = 0

    icomm  = pl_mpi%comm
    myrank = pl_mpi%myrank

    if (myrank == 0) then
      nxyz = self%num
    end if
    call mpi_bcast(nxyz, 1, MPI_INTEGER, 0, icomm, istat)
    if (istat /= 0) return
    if (myrank /= 0) then
      call new_ptcharges(self, nxyz, istat)
      if (istat /= 0) return
    end if

    if (self%num <= 0) return

    call mpi_bcast(self%chg, nxyz, MPI_DOUBLE_PRECISION, 0, icomm, istat)
    if (istat /= 0) return
    call mpi_bcast(self%xyz, 3*nxyz, MPI_DOUBLE_PRECISION, 0, icomm, istat)
    if (istat /= 0) return

  end subroutine bcast_ptcharges
  
  !-----------------------------------------------------------------------------
  !> 初期化
  subroutine init_ptcharges(self, ldebug, istat)
    type(ptcharges_type),intent(inout)::self
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    istat = 0

!!$    call free_ptcharges(self, ldebug, istat)
    self%num = 0

  end subroutine init_ptcharges

  !-----------------------------------------------------------------------------
  !> 配列確保
  subroutine new_ptcharges(self, num, istat)
    type(ptcharges_type),intent(inout)::self
    integer,intent(in)::num
    integer,intent(inout)::istat

    istat = 0

    self%num = num
    allocate(self%chg(num), self%xyz(3,num), stat=istat)
    if (istat /= 0) return

  end subroutine new_ptcharges

  !-----------------------------------------------------------------------------
  !> 配列開放
  subroutine free_ptcharges(self, ldebug, istat)
    type(ptcharges_type),intent(inout)::self
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    istat = 0

    if (allocated(self%chg)) deallocate(self%chg, stat=istat)
    if (istat /= 0) return
    if (allocated(self%xyz)) deallocate(self%xyz, stat=istat)
    if (istat /= 0) return

    self%num = 0
  end subroutine free_ptcharges

  !-----------------------------------------------------------------------------
  !> \e input_file からの読み込み
  subroutine read_inp_ptcharges(self, input_file, ldebug, istat)
    use input_class
    use qm_options_class, only: options_type, QM_NO
    type(ptcharges_type),intent(inout)::self
    character(len=*),intent(in)::input_file
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    type(options_type)::qm_options
    integer::nline, iline
    character(len=MAX_INPUT_LINE_LEN),allocatable::lines(:)
    character(len=MAX_INPUT_LINE_LEN)::line

    istat = 0

    open(1, file=input_file, status='old', err=101)
    rewind(1, err=102)
    iline = 0
1   read(1, '(a)', end=2, err=103) line
    iline = iline + 1
    go to 1
2   nline = iline
    allocate(lines(nline))

    rewind(1, err=102)
    iline = 0
3   read(1, '(a)', end=4, err=103) line
    iline = iline + 1
    lines(iline) = line
    go to 3
4   close(1, err=104)

    qm_options%pdbinput = QM_NO
!!$    call p_read_ptcharges(self,input_file,ldebug,istat)
    call read_ptcharges(self, qm_options, lines, nline, ldebug, istat)
    if (istat /= 0) return

!   self%num = 0

    if (ldebug > 100)&
         call print_ptcharges(self)

    deallocate(lines)
    return
101 write(6,*) 'open error'
    return
102 write(6,*) 'rewind error'
    return
103 write(6,*) 'read error'
    return
104 write(6,*) 'close error'
    return
  end subroutine read_inp_ptcharges

  !-----------------------------------------------------------------------------
  !> xyz, chg 配列の置き換え
  subroutine set_ptcharges(self, num, xyz, chg, istat)
    type(ptcharges_type),intent(inout)::self
    integer,intent(in)::num
    real(8),intent(in)::xyz(3,num)
    real(8),intent(in)::chg(num)
    integer,intent(inout)::istat

    istat = 0

    call new_ptcharges(self, num, istat)
    if (istat /= 0) return

    if (num > 0) then
      self%xyz(:,:) = xyz(1:3,1:num)
      self%chg(:)   = chg(1:num)
    end if

  end subroutine set_ptcharges

  !-----------------------------------------------------------------------------
  !> データ取得
  subroutine get_ptcharges(self, num, xyz, chg, istat)
    type(ptcharges_type),intent(in)::self
    integer,intent(in)::num
    real(8),intent(inout)::xyz(3,num)
    real(8),intent(inout)::chg(num)
    integer,intent(inout)::istat

    istat = 0

    if (self%num /= num) then
      istat = 1
      return
    end if

    if (num > 0) then
      xyz(:,:) = self%xyz(1:3,1:num)
      chg(:)   = self%chg(1:num)
    end if

  end subroutine get_ptcharges

  !-----------------------------------------------------------------------------
  !> 点電荷数の取得
  integer function num_ptcharges(self)
    type(ptcharges_type),intent(in)::self
    num_ptcharges = self%num
  end function num_ptcharges

  !-----------------------------------------------------------------------------
  !> データの簡易表示
  subroutine print_brief_ptcharges(self)
    type(ptcharges_type),intent(in)::self

    write(6,'(/''Point charges:'')')
    !write(6,'(1x,''Number of point charges ='',i6)') self%num
    write(6,'(1x,''num ='',i6)') self%num

  end subroutine print_brief_ptcharges

  !-----------------------------------------------------------------------------
  !> データの表示
  subroutine print_ptcharges(self)
    type(ptcharges_type),intent(in)::self
    integer::i,j

    call print_brief_ptcharges(self)

    if (self%num > 0) then
      write(6,'(/6x,''No.         X             Y             Z'',&
           &''       Charges'')')
      do i = 1, self%num
        write(6,'(i8,3f14.8,f11.5)') i, (self%xyz(j,i),j=1,3), self%chg(i)
      end do
    end if

  end subroutine print_ptcharges

  !-----------------------------------------------------------------------------
  subroutine p_read_pdb_n(nxyz, qm_options, ldebug, istat)
    use qm_options_class, only: options_type, QM_NO
    integer,intent(out)::nxyz
    type(options_type),intent(in)::qm_options
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    !character(len=MAX_INPUT_LINE_LEN)::line
    character(len=256)::line
    integer::lunit, natom

    istat = 0

    lunit = 1

    open(lunit, file=qm_options%qmatomlist, status='old', err=101)
    rewind(lunit, err=102)
    natom = 0
1   read(lunit,'(a)',end=2, err=103 ) line
    natom = natom + 1
    go to 1
2   close(lunit, err=104)

    open(lunit, file=qm_options%pdbfile, status='old', err=101)
    rewind(lunit, err=102)
    nxyz = 0
3   read(lunit,'(a)',end=4, err=103 ) line
    if ((line(1:4) == 'ATOM' ) .or. (line(1:4) == 'HETA')) then
       nxyz = nxyz + 1
    end if
    go to 3
4   close(lunit, err=104)

    write(6,*) 'nxyz,natom=',nxyz,natom
    nxyz = nxyz - natom
    return

101 istat = PL_ERR_FILE_OPEN_ERROR
    go to 199
102 istat = PL_ERR_FILE_REWIND_ERROR
    go to 199
103 istat = PL_ERR_FILE_READ_ERROR
    go to 199
104 istat = PL_ERR_FILE_CLOSE_ERROR
    go to 199
199 call pl_error_print_message(istat)
    call pl_error_print_file(qm_options%pdbfile)
    return
  end subroutine p_read_pdb_n

  !-----------------------------------------------------------------------------
  subroutine p_read_pdb(self, qm_options, ldebug, istat)
    use physical_constants_class
    use qm_options_class, only: options_type, QM_NO
    type(ptcharges_type),intent(inout)::self
    type(options_type),intent(in)::qm_options
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    character(len=256),allocatable::lines(:)
    !character(len=MAX_INPUT_LINE_LEN)::line
    character(len=256)::line
    integer::lunit, nxyz, ixyz, npc, ipc, natom, inum, i
    logical,allocatable::list(:)
    integer::resnum
    real(8)::x,y,z,occ,bfact

    istat = 0

    lunit = 1
    !write(6,*) 'nxyz@pc_pdb=',self%num

    open(lunit, file=qm_options%pdbfile, status='old', err=101)
    rewind(lunit, err=102)
    nxyz = 0
3   read(lunit,'(a)',end=4, err=103 ) line
    if ((line(1:4) == 'ATOM' ) .or. (line(1:4) == 'HETA')) then
       nxyz = nxyz + 1
    end if
    go to 3
4   close(lunit, err=104)
    !write(6,*) 'trap1',nxyz

    allocate(list(nxyz))
    list(:) = .true.

    open(lunit, file=qm_options%qmatomlist, status='old', err=101)
    rewind(lunit, err=102)
1   read(lunit,*,end=2, err=103 ) inum
    !write(6,*) 'inum=',inum
    list(inum) = .false.
    go to 1
2   close(lunit, err=104)
    !write(6,*) 'trap2',list(:)

!!$    allocate(lines(nxyz))

    open(lunit, file=qm_options%pdbfile, status='old', err=101)
    rewind(lunit, err=102)
    ixyz = 0
    ipc = 0
5   read(lunit,'(a)',end=6, err=103 ) line
    if ((line(1:4) == 'ATOM' ) .or. (line(1:4) == 'HETA')) then
       ixyz = ixyz + 1
       if (list(ixyz)) then
          read(line,'(22x,i4,4x,3f8.3,2f6.3)') resnum,x,y,z,occ,bfact
          !write(6,'(i4,4x,3f8.3,2f8.3)') resnum,x,y,z,occ,bfact
          ipc = ipc + 1
          self%xyz(1,ipc) = x
          self%xyz(2,ipc) = y
          self%xyz(3,ipc) = z
          self%chg(  ipc) = bfact
       end if
    end if
    go to 5
6   close(lunit, err=104)

    self%xyz(:,:) = self%xyz(1:3,1:self%num) / bohr

    return

101 istat = PL_ERR_FILE_OPEN_ERROR
    go to 199
102 istat = PL_ERR_FILE_REWIND_ERROR
    go to 199
103 istat = PL_ERR_FILE_READ_ERROR
    go to 199
104 istat = PL_ERR_FILE_CLOSE_ERROR
    go to 199
199 call pl_error_print_message(istat)
    call pl_error_print_file(qm_options%pdbfile)
    return
  end subroutine p_read_pdb

  !-----------------------------------------------------------------------------
  subroutine read_ptcharges(self, qm_options, lines, nline, ldebug, istat)
    use physical_constants_class
    use input_class
    use qm_options_class, only: options_type, QM_NO
    type(ptcharges_type),intent(inout)::self
    type(options_type),intent(in)::qm_options
    integer,intent(in)::nline
    character(len=*),intent(in)::lines(nline)
    integer,intent(in)::ldebug
    integer,intent(out)::istat

    character(len=*),parameter::group_name = 'POINTCHARGES'
    integer,parameter::nopt = 1                          !< オプション数
    character(len=MAX_OPTION_NAME_LEN)::opt_names(nopt)& !< オプションの名称
         =(/'UNITS'/)
    character(len=MAX_OPTION_VALUE_LEN)::opt_values(nopt)!< オプションの値
    character(len=MAX_OPTION_NAME_LEN)::opt_name
    character(len=MAX_OPTION_VALUE_LEN)::opt_value
    integer::opt_count(nopt)
    integer::iopt, ipos, ipos_default

    integer,parameter::QM_UNITS_NUM = 2
    character(len=*),parameter::QM_UNITS(QM_UNITS_NUM) = (/ 'ANG', 'AU '/)
    integer::iunits

    integer::iline_gs  !< 対象データグループの開始行
    integer::iline_ge  !< 対象データグループの終了行
    integer::icol_gs   !< 対象データグループの開始行における開始カラム
    integer::icol_ge   !< 対象データグループの終了行における終了カラム
    integer::iline_err !< エラー行       (=0:正常, /=0:エラー)
    integer::icol_err  !< エラーカラム   (=0:正常, /=0:エラー)
    integer::ngroup

    integer,parameter::mxval = 4
    character(len=MAX_INPUT_LINE_LEN)::vals(mxval)
    integer::nval, ival, iline, icol_s, icol_e, nxyz, ixyz
    logical::is_option

    character(len=*),parameter::c_assignment_op = '='

    istat = 0

    if (qm_options%pdbinput == QM_NO) then

    call get_data_group_position(&
         lines, nline, group_name,&
         iline_gs, icol_gs, iline_ge, icol_ge, ngroup,&
         ldebug, iline_err, icol_err, istat)
    if (istat /= 0) return
    if (ngroup == 0) return

    nxyz = 0
    opt_count(:) = 0
    opt_values(:) = ' '
    do iline = iline_gs, iline_ge
       !write(6,'(i6,a)') iline,':' // trim(lines(iline)) // '$'
       if (iline == iline_gs) then
          icol_s = icol_gs
       else
          icol_s = 1
       end if
       if (iline == iline_ge) then
          icol_e = icol_ge
       else
          icol_e = len_trim(lines(iline))
       end if
       !write(6,*) 'str=',lines(iline)(icol_s:icol_e),'$'

       is_option = .false.
       call split_str_eq(lines(iline)(icol_s:icol_e), vals, mxval, nval, istat)
       if (istat /= 0) then
          istat = PL_ERR_INVALID_FORMAT
          iline_err = iline; icol_err = icol_s
          go to 9
       end if
       !write(6,*) 'nval=',nval
       if (nval == 0) cycle
       do ival = 1, nval
          if (trim(vals(ival)) == c_assignment_op) then
             is_option = .true.
             exit
          end if
       end do
       if (is_option) then
          call parse_options(&
               lines(iline), 1, 1, icol_s, 1, icol_e,&
               opt_names, opt_values, opt_count, nopt,&
               ldebug, iline_err, icol_err, istat)
          if (istat /= 0) return
       else
          if (nval == 4) then
             nxyz = nxyz + 1
          else
             istat = PL_ERR_INVALID_FORMAT
             iline_err = iline; icol_err = icol_s
             go to 9
          end if
       endif
    end do
    !write(6,*) 'nxyz=',nxyz

    do iopt = 1, nopt
       if (opt_count(iopt) > 1) then
          istat = PL_ERR_DUPLICATE_OPTION
          call pl_error_print_message(istat)
          call print_error_invalid_option(opt_names(iopt))
          return
       end if
    end do
    
    call new_ptcharges(self, nxyz, istat)
    if (istat /= 0) return

    ixyz = 0
    do iline = iline_gs, iline_ge
       !write(6,'(i6,a)') iline,':' // trim(lines(iline)) // '$'
       if (iline == iline_gs) then
          icol_s = icol_gs
       else
          icol_s = 1
       end if
       if (iline == iline_ge) then
          icol_e = icol_ge
       else
          icol_e = len_trim(lines(iline))
       end if
       !write(6,*) 'str=',lines(iline)(icol_s:icol_e),'$'

       call split_str_eq(lines(iline)(icol_s:icol_e), vals, mxval, nval, istat)
       if (nval == 0) cycle
       if (nval == 4) then
          ixyz = ixyz + 1
          call pl_str_to_dbl(vals(1), self%xyz(1,ixyz), istat)
          if (istat /= 0) then
             istat = PL_ERR_INVALID_FORMAT
             iline_err = iline; icol_err = icol_s
             go to 9
          end if
          call pl_str_to_dbl(vals(2), self%xyz(2,ixyz), istat)
          if (istat /= 0) then
             istat = PL_ERR_INVALID_FORMAT
             iline_err = iline; icol_err = icol_s
             go to 9
          end if
          call pl_str_to_dbl(vals(3), self%xyz(3,ixyz), istat)
          if (istat /= 0) then
             istat = PL_ERR_INVALID_FORMAT
             iline_err = iline; icol_err = icol_s
             go to 9
          end if
          call pl_str_to_dbl(vals(4), self%chg(ixyz), istat)
          if (istat /= 0) then
             istat = PL_ERR_INVALID_FORMAT
             iline_err = iline; icol_err = icol_s
             go to 9
          end if
       end if
    end do

    opt_name = 'units'
    ipos_default = pl_str_search_name_ignorecase('ang', QM_UNITS, QM_UNITS_NUM)
    call get_option_values_strlist(&
         opt_name, opt_names, opt_values, nopt,&
         QM_UNITS, QM_UNITS_NUM, ipos_default, ipos, istat)
    if (istat /= 0) return
    iunits = ipos
    !write(6,*) 'units:',QM_UNITS(iunits)

    if (self%num /= 0 .and. iunits == 1)&
         self%xyz(:,:) = self%xyz(1:3,1:nxyz) / bohr

    istat = 0

    else
       
       call p_read_pdb_n(nxyz, qm_options, ldebug, istat)
       if (istat /= 0) return
       !write(6,*) 'nxyz@pc=',nxyz

       call new_ptcharges(self, nxyz, istat)
       if (istat /= 0) return

       call p_read_pdb(self, qm_options, ldebug, istat)
       if (istat /= 0) return

    end if

    !call print_ptcharges(self)
    !stop

    return

9   call pl_error_print_message(istat)
    call print_error_lc(iline_err, icol_err)
    call print_error_line(lines(iline_err))
    return
  end subroutine read_ptcharges

  !-----------------------------------------------------------------------------
end module ptcharges_class
