! -*- coding: utf-8 -*-
!
! Dual licensed under the MIT or GPL Version 3 license.
! See 'LICENSE.MIT' for the definition of the MIT license,
! and 'COPYING.GPL' for the definition of the GPL license.
!
!> QM/MM用のQMモジュール
module qm_class
  use control_options_class, only:&
       control_options_type => options_type,&
       control_read_options => read_options,&
       control_bcast_options => bcast_options,&
       control_free_options => free_options,&
       control_print_options => print_options
  use qm_options_class, only:&
       qmmm_options_type => options_type,&
       qmmm_read_options => read_options,&
       qmmm_bcast_options => bcast_options,&
       qmmm_free_options => free_options,&
       qmmm_print_options => print_options,&
       QM_RUNMODE_ENERGY, QM_RUNMODE_OPT,&
       QM_WFTYPE_RHF, QM_WFTYPE_RDFT, QM_WFTYPE_UHF,&
       QM_WFTYPE_UDFT, QM_WFTYPE_CASSCF, QM_WFTYPE_CIS,&
       QM_CHECK_NO, QM_CHECK_DATA, QM_CHECK_ENERGY, QM_CHECK_FORCE, QM_CHECK_ALL
  use energy_class
  use force_class
!!$  use optimize_options_class, only:&
!!$       optimize_options_type => options_type,&
!!$       optimize_read_options => read_options,&
!!$       optimize_bcast_options => bcast_options,&
!!$       optimize_free_options => free_options,&
!!$       optimize_print_options => print_options
  use integral_options_class, only:&
       integral_options_type => options_type,&
       integral_read_options => read_options,&
       integral_bcast_options => bcast_options,&
       integral_free_options => free_options,&
       integral_print_options => print_options
  use integral_class
!!$  use dft_options_class, only:&
!!$       dft_options_type => options_type,&
!!$       dft_read_options => read_options,&
!!$       dft_bcast_options => bcast_options,&
!!$       dft_free_options => free_options,&
!!$       dft_print_options => print_options
!!$  use dft_class
  use data_class, only:data_type, init_data, free_data
  use atoms_class, only:read_atoms, bcast_atoms, nao_atoms, nele_atoms,&
       replace_atoms, free_atoms, print_brief_atoms, print_atoms
  use basis_class, only:print_brief_basis, print_basis
  use ptcharges_class, only:set_ptcharges, print_brief_ptcharges,&
       print_ptcharges, free_ptcharges
  use gscharges_class, only:set_gscharges, print_brief_gscharges,&
       print_gscharges, free_gscharges
  use fswolf_class, only:set_fswolf, print_brief_fswolf,&
       print_fswolf, free_fswolf
  use pl_mpi_class

  implicit none
  private

  public::qm_init
  public::qm_read
  public::qm_print_options
  public::qm_prep
  public::qm_solve
  public::qm_free
  public::qm_finalize
  public::qm_get_lprint
  public::qm_get_ldebug
  public::qm_print_inputs !for test
  public::qm_print_results!for test
  public::qm_type

  type::qm_type
     type(pl_mpi_type)::pl_mpi
     type(control_options_type)::control_options
     type(qmmm_options_type)::qmmm_options
!!$     type(optimize_options_type)::optimize_options
     type(integral_type)::integral
     type(energy_type)::energy
     type(force_type)::force
!!$     type(dft_options_type)::dft_options
!!$     type(dft_data_type)::dft_data
     type(data_type)::data
!!$     type(atoms_type)::atoms
!!$     type(gscharges_type)::gscharges
!!$     type(ptcharges_type)::ptcharges
  end type qm_type

  integer,public,parameter::QM_GUESS_FLAG_OFF = 0
  integer,public,parameter::QM_GUESS_FLAG_ON  = 1

  integer,public,parameter::QM_SAVE_FLAG_OFF  = 0
  integer,public,parameter::QM_SAVE_FLAG_ON   = 1

  integer,public,parameter::QM_PRINT_NONE     = 0
  integer,public,parameter::QM_PRINT_LEVEL_1  = 1
  integer,public,parameter::QM_PRINT_LEVEL_2  = 2
  integer,public,parameter::QM_PRINT_LEVEL_3  = 3
  integer,public,parameter::QM_PRINT_LEVEL_4  = 4
  integer,public,parameter::QM_PRINT_LEVEL_5  = 5

  character(len=*),parameter::QM_INPUT_FILE = 'qm.inp'

contains
  !-----------------------------------------------------------------------------
  !> 初期処理ルーチン
  subroutine qm_init(self, icomm, istat)
    use pl_prof_class
!!$    use mpi
    include 'mpif.h'
    type(qm_type),intent(inout)::self !< QMオブジェクト
    integer,intent(in)::icomm !< コミュニケータ
    integer,intent(inout)::istat !< ステータス

    integer::myrank, istat2
    integer::ldebug
    logical::do_pmon, do_hwc

    istat = 0
    ldebug = 0
    self%control_options%lprint = 0
    self%control_options%ldebug = 0

    !call pl_mpi_init(self%pl_mpi, istat)
    !if (istat /= 0) return
    call pl_mpi_set_comm(self%pl_mpi, icomm, istat)
    if (istat /= 0) return
    myrank = self%pl_mpi%myrank
    !call pl_mpi_print_nprocs(self%pl_mpi)

    call init_data(self%data, ldebug, istat)
    if (istat /= 0) return

    if (myrank == 0) then
       call qm_read(self, QM_INPUT_FILE, istat)
    end if
    call mpi_bcast(istat, 1, MPI_INTEGER, 0, icomm, istat2)
    if (istat /= 0) return
    ldebug = self%control_options%ldebug

    call qm_bcast(self, ldebug, istat)
    if (istat /= 0) return

    do_pmon = self%control_options%do_pmon
    do_hwc = self%control_options%do_hwc
    call pl_prof_initialize(do_pmon, do_hwc)

    call qm_prep(self, ldebug, istat)
    if (istat /= 0) return

  end subroutine qm_init

  !-----------------------------------------------------------------------------
  !> 終了処理ルーチン
  subroutine qm_finalize(self, istat)
    use pl_prof_class
    type(qm_type),intent(inout)::self !< QMオブジェクト
    integer,intent(inout)::istat !< ステータス

    integer::myrank, nprocs
    integer::lprint
    integer,external::omp_get_max_threads

    istat = 0

    call qm_free(self, istat)
    if (istat /= 0) return

    myrank = self%pl_mpi%myrank
    if (myrank == 0) then
       lprint = self%control_options%lprint
       !if (lprint >= QM_PRINT_LEVEL_1) then
       write(6,'(/''# Number of processes = '',i5)') self%pl_mpi%nprocs
       write(6,'( ''# Number of threads = '',i3)') omp_get_max_threads()
       call pl_prof_print()
       !end if
    end if
    call pl_prof_finalize()

    !call pl_mpi_finalize(self%pl_mpi, istat)
    !if (istat /= 0) return

  end subroutine qm_finalize

  !-----------------------------------------------------------------------------
  subroutine qm_read(self, input_file, istat)
!!$    use pl_prof_class
    use input_class
    type(qm_type),intent(inout)::self
    character(len=*),intent(in)::input_file
    integer,intent(inout)::istat

    integer::nline
    character(len=MAX_INPUT_LINE_LEN),allocatable::lines(:)

    integer::nao, nele, wftype, ldebug
    integer::icomm, myrank
    integer::istat2

    istat = 0

!!$    call mpi_barrier(icomm, istat)
!!$    call pl_prof_region_begin('s:Input')

    call read_file_n(input_file, nline, istat)
    if (istat /= 0) return
    allocate(lines(nline), stat=istat)
    if (istat /= 0) return
    call read_file(input_file, lines, nline, istat)
    if (istat /= 0) return

    ldebug = 0
    call control_read_options(self%control_options, lines, nline, ldebug, istat)
    if (istat /= 0) return

    ldebug = self%control_options%ldebug
    if (ldebug > 0) then
       call control_print_options(self%control_options)
    end if

    call qmmm_read_options(self%qmmm_options, lines, nline, ldebug, istat)
    if (istat /= 0) return

    if (ldebug > 0) then
       call qmmm_print_options(self%qmmm_options)
    end if

    call integral_read_options(&
         self%integral%options, lines, nline, ldebug, istat)
    if (istat /= 0) return

    if (ldebug > 0) then
       call integral_print_options(self%integral%options)
    end if

    wftype = self%qmmm_options%wftype
    call energy_set_wftype(self%energy, wftype, istat)
    if (istat /= 0) return

    call force_set_wftype(self%force, wftype, istat)
    if (istat /= 0) return

!!$    if (wftype == QM_WFTYPE_RDFT .or. wftype == QM_WFTYPE_UDFT) then
!!$       call dft_read_options(self%dft_options, lines, nline, ldebug, istat)
!!$       if (istat /= 0) return
!!$
!!$       if (ldebug > 0) then
!!$          call dft_print_options(self%dft_options)
!!$       end if
!!$    end if

    call read_atoms(&
         self%data%atoms, self%qmmm_options, lines, nline, ldebug, istat)
    if (istat /= 0) return

    if (ldebug > 0) then
       call print_atoms(self%data%atoms)
       call print_basis(self%data%atoms%basis)
    end if

    nao  = nao_atoms(self%data%atoms)
    nele = nele_atoms(self%data%atoms)
    call energy_read_options(&
         self%energy, lines, nline, nao, nele, ldebug, istat)
    if (istat /= 0) return

    if (ldebug > 0) then
       call energy_print_options(self%energy)
    end if

!!$    if (self%qmmm_options%runmode == QM_RUNMODE_OPT) then
!!$       call optimize_read_options(&
!!$            self%optimize_options, lines, nline, nao, nele, ldebug, istat)
!!$       if (ldebug > 0) then
!!$          call optimize_print_options(self%optimize_options)
!!$       end if
!!$    end if

!!$    call mpi_barrier(icomm, istat)
!!$    call pl_prof_region_end('s:Input')

    deallocate(lines, stat=istat)
    if (istat /= 0) return
  end subroutine qm_read

  !-----------------------------------------------------------------------------
  subroutine qm_bcast(self, ldebug, istat)
    type(qm_type),intent(inout)::self
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    integer::wftype

    istat = 0

    call control_bcast_options(self%control_options, self%pl_mpi, ldebug, istat)
    if (istat /= 0) return

    call qmmm_bcast_options(self%qmmm_options, self%pl_mpi, ldebug, istat)
    if (istat /= 0) return

!!$    if (self%qmmm_options%runmode == QM_RUNMODE_OPT) then
!!$       call optimize_bcast_options(&
!!$            self%optimize_options, self%pl_mpi, ldebug, istat)
!!$    end if

    call integral_bcast_options(self%integral%options, self%pl_mpi,ldebug,istat)
    if (istat /= 0) return

    call energy_bcast_options(self%energy, self%pl_mpi, ldebug, istat)
    if (istat /= 0) return

    call force_bcast_options(self%force, self%pl_mpi, ldebug, istat)
    if (istat /= 0) return

    wftype = self%qmmm_options%wftype
!!$    if (wftype == QM_WFTYPE_RDFT .or. wftype == QM_WFTYPE_UDFT) then
!!$       call dft_bcast_options(self%dft_options, self%pl_mpi, ldebug, istat)
!!$       if (istat /= 0) return
!!$    end if

    call bcast_atoms(self%data%atoms, self%pl_mpi, ldebug, istat)
    if (istat /= 0) return

  end subroutine qm_bcast

  !-----------------------------------------------------------------------------
  subroutine qm_print_options(self)
    type(qm_type),intent(in)::self

    integer::wftype
    wftype = self%qmmm_options%wftype

    if (self%pl_mpi%myrank == 0) then
       write(6,&
            '(/''== QM options ======================'',&
            &''==========================='')')
       call control_print_options(self%control_options)
       call qmmm_print_options(self%qmmm_options)
!!$       if (self%qmmm_options%runmode == QM_RUNMODE_OPT) then
!!$          call optimize_print_options(self%optimize_options)
!!$       end if
       call integral_print_options(self%integral%options)
       call energy_print_options(self%energy)
!!$       if (wftype == QM_WFTYPE_RDFT .or. wftype == QM_WFTYPE_UDFT) then
!!$          call dft_print_options(self%dft_options)
!!$       end if
       write(6,&
            '(''====================================='',&
            &''==========================='')')
    end if

  end subroutine qm_print_options

  !-----------------------------------------------------------------------------
  subroutine qm_prep(self, ldebug, istat)
    type(qm_type),intent(inout)::self
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    integer::wftype

    istat = 0

    call integral_prep(self%integral, self%data%atoms, ldebug, istat)
    if (istat /= 0) return

!!$    wftype = self%qmmm_options%wftype
!!$    if (wftype == QM_WFTYPE_RDFT .or. wftype == QM_WFTYPE_UDFT) then
!!$       call dft_prep(self%dft_data, self%data%atoms, ldebug, istat)
!!$       if (istat /= 0) return
!!$    end if

    call energy_new_data(self%energy, ldebug, istat)
    if (istat /= 0) return

    call force_new(self%force, ldebug, istat)
    if (istat /= 0) return

  end subroutine qm_prep

  !-----------------------------------------------------------------------------
  subroutine qm_free(self, istat)
    type(qm_type),intent(inout)::self
    integer,intent(out)::istat

    integer::wftype
    integer::ldebug

    istat = 0
    ldebug = self%control_options%ldebug
    wftype = self%qmmm_options%wftype

!!$    if (self%qmmm_options%runmode == QM_RUNMODE_OPT) then
!!$       call optimize_free_options(self%optimize_options, ldebug, istat)
!!$    end if

    call control_free_options(self%control_options, ldebug, istat)
    if (istat /= 0) return

    call qmmm_free_options(self%qmmm_options, ldebug, istat)
    if (istat /= 0) return

    call integral_free_options(self%integral%options, ldebug,istat)
    if (istat /= 0) return

    wftype = self%qmmm_options%wftype
!!$    if (wftype == QM_WFTYPE_RDFT .or. wftype == QM_WFTYPE_UDFT) then
!!$       call dft_free_options(self%dft_options, ldebug, istat)
!!$       if (istat /= 0) return
!!$    end if

    call integral_free(self%integral, ldebug, istat)
    if (istat /= 0) return

    call free_atoms(self%data%atoms, ldebug, istat)
    if (istat /= 0) return

    call energy_free_options(self%energy, ldebug, istat)
    if (istat /= 0) return

    call energy_free_data(self%energy, ldebug, istat)
    if (istat /= 0) return

  end subroutine qm_free

  !-----------------------------------------------------------------------------
  subroutine qm_solve(&
       self,&                           !< [in]  qm_obj
       natom, atom_xyz, atom_chg,&      !< [in]  qm_atom
       npc, pc_xyz, pc_chg,&            !< [in]  mm_pc
       ngc, gc_xyz, gc_chg, gc_rc2,&    !< [in]  mm_gc
       nfsw, fsw_xyz, fsw_chg, fsw_rc2,&!< [in]  mm_fsw
       energy_total,&                   !< [out] E
       atom_force,&                     !< [out] dE_atom
       pc_force,&                       !< [out] dE_pc
       gc_force,&                       !< [out] dE_gc
       fsw_force,&                      !< [out] dE_fsw
       pop,&                            !< [out] pop
       iguess,&                         !< [in]  option
       isave,&                          !< [in]  option
       lprint_in,&                      !< [in]  option
       istat)                           !< [out] status
!!$    use population_class
    use pl_prof_class
!!$    use mpi
    include 'mpif.h'
    type(qm_type),intent(inout)::self
    integer,intent(in)::natom, npc, ngc, nfsw
    real(8),intent(in)::atom_xyz(3,natom), atom_chg(natom)
    real(8),intent(in)::pc_xyz(3,npc), pc_chg(npc)
    real(8),intent(in)::gc_xyz(3,ngc), gc_chg(ngc), gc_rc2(ngc)
    real(8),intent(in)::fsw_xyz(3,nfsw), fsw_chg(nfsw), fsw_rc2(nfsw)
    real(8),intent(inout)::atom_force(3,natom), pc_force(3,npc), gc_force(3,ngc)
    real(8),intent(inout)::fsw_force(3,nfsw)
    real(8),intent(inout)::pop(natom)
    real(8),intent(inout)::energy_total
    integer,intent(in)::iguess, isave, lprint_in
    integer,intent(inout)::istat

    integer::wftype
    integer::icomm, myrank, nprocs
    integer::ldebug
    integer::lprint
    integer::nao
    real(8),allocatable::rsrs(:)
    real(8),allocatable::f(:,:)
    real(8),allocatable::d(:,:)

    real(8),allocatable::center(:),emd(:,:,:),emq(:,:,:),emo(:,:,:)

    istat = 0

    icomm  = self%pl_mpi%comm
    myrank = self%pl_mpi%myrank
    nprocs = self%pl_mpi%nprocs
    call mpi_barrier(icomm, istat)
    call pl_prof_region_begin('QM')

    if (self%qmmm_options%check > QM_CHECK_NO) then
       lprint = self%control_options%lprint
    else
       lprint = lprint_in
    end if
    ldebug = self%control_options%ldebug

    wftype = self%qmmm_options%wftype

    if (lprint >= QM_PRINT_LEVEL_1) then
       if (myrank == 0) then
          call p_print_header()
       end if
    end if

    if (ldebug > 0) then
       call energy_print_options(self%energy)
       call integral_print_options(self%integral%options)
!!$       if (wftype == QM_WFTYPE_RDFT .or. wftype == QM_WFTYPE_UDFT) then
!!$          call dft_print_options(self%dft_options)
!!$       end if
    end if

    call replace_atoms(self%data%atoms, natom, atom_xyz, atom_chg, istat)
    if (istat /= 0) return
    call set_ptcharges(self%data%ptcharges, npc, pc_xyz, pc_chg, istat)
    if (istat /= 0) return
    call set_gscharges(self%data%gscharges, ngc, gc_xyz, gc_chg, gc_rc2, istat)
    if (istat /= 0) return
    call set_fswolf(&
         self%data%fswolf, nfsw, fsw_xyz, fsw_chg, fsw_rc2, istat)
    if (istat /= 0) return

!!$ call zero_dipole(self%data%atoms,self%data%ptcharges,16.0d0/0.529177249d0)
!!$ stop

    if (myrank == 0) then
       !if (lprint >= QM_PRINT_LEVEL_5) then
       if (lprint >= QM_PRINT_LEVEL_2) then
          call print_atoms(self%data%atoms)
          !call print_basis(self%data%atoms%basis)
          call print_brief_gscharges(self%data%gscharges)
          call print_brief_ptcharges(self%data%ptcharges)
          call print_brief_fswolf(self%data%fswolf)
!!$       else if (lprint >= QM_PRINT_LEVEL_2) then
       else
          call print_brief_atoms(self%data%atoms)
          call print_brief_basis(self%data%atoms%basis)
          call print_brief_gscharges(self%data%gscharges)
          call print_brief_ptcharges(self%data%ptcharges)
          call print_brief_fswolf(self%data%fswolf)
       end if
    end if

    call mpi_barrier(icomm, istat)
    call pl_prof_region_begin('s:Int_prep')
    call integral_set_params(self%integral, self%data%atoms, ldebug, istat)
    if (istat /= 0) return
    call mpi_barrier(icomm, istat)
    call pl_prof_region_end('s:Int_prep')

!!$    if (wftype == QM_WFTYPE_RDFT .or. wftype == QM_WFTYPE_UDFT) then
!!$       call dft_prep(self%dft_data, self%data%atoms, ldebug, istat)
!!$       if (istat /= 0) return
!!$       call dft_replace_xyz(&
!!$            self%dft_data, self%data%atoms%xyz, self%data%atoms%atom2basis,&
!!$            self%data%atoms%basis%ntshell, self%data%atoms%num,&
!!$            self%data%atoms%basis%num,&
!!$            ldebug, istat)
!!$       if (istat /= 0) return
!!$
!!$       call mpi_barrier(icomm, istat)
!!$       call pl_prof_region_begin('p:DFT_prep')
!!$       call dft_get_grids(&
!!$            self%dft_data, self%dft_options,&
!!$            self%data%atoms%xyz, self%data%atoms%chg, self%data%atoms%num,&
!!$            myrank, nprocs,&
!!$            ldebug, istat)
!!$       if (istat /= 0) return
!!$       call mpi_barrier(icomm, istat)
!!$       call pl_prof_region_end('p:DFT_prep')
!!$    end if

    !!!!!!
    call tables_prep2(self%integral%tables, self%data%atoms, ldebug, istat)
    if (istat /= 0) return
    !!!!!!

    call mpi_barrier(icomm, istat)
    call pl_prof_region_begin('::Energy')

    call mpi_barrier(icomm, istat)
    call pl_prof_region_begin('::OEI')
    call integral_calc_overlap(self%integral, self%data%atoms,&
         self%pl_mpi, ldebug, istat)
    if (istat /= 0) return

    nao = self%energy%rhf_data%nao
    !call ummtw('s',self%integral%s,nao,nao,nao,10,6,istat)

    call integral_calc_hcore(&
         self%integral,&
         self%data%atoms, self%data%ptcharges, self%data%gscharges,&
         self%data%fswolf,&
         self%pl_mpi, ldebug, istat)
    if (istat /= 0) return
    call mpi_barrier(icomm, istat)
    call pl_prof_region_end('::OEI')
    !call ummtw('h',self%integral%h,nao,nao,nao,10,6,istat)

!!$    call energy_new_data(self%energy, ldebug, istat)
!!$    if (istat /= 0) return

    call energy_init(self%energy, ldebug, istat)
    if (istat /= 0) return

    if (iguess == QM_GUESS_FLAG_ON) then
       call energy_guess(self%energy, self%integral, self%pl_mpi, ldebug, istat)
       if (istat /= 0) return
    else
       call energy_orthogonalize(self%energy, self%integral, ldebug, istat)
       if (istat /= 0) return
    end if

    call energy_solve(&
         self%energy,&
!!$         self%dft_data, self%dft_options,&
         self%data%atoms, self%data%ptcharges, self%data%gscharges,&
         self%data%fswolf,&
         self%integral%options%rcut,&
         self%integral%options%alpha,&
         self%integral%options%zerodipole,&
         self%integral, self%pl_mpi, lprint, ldebug, istat)
    if (istat /= 0) return
    call mpi_barrier(icomm, istat)
6   call pl_prof_region_end('::Energy')

    if (myrank == 0) then
       if (lprint >= QM_PRINT_LEVEL_5) then
          call energy_print_data(self%energy)
       else if (lprint >= QM_PRINT_LEVEL_1) then
          call energy_print_data_brief(self%energy)
       end if
    end if

    !if (lprint >= QM_PRINT_LEVEL_1) then
    if (myrank == 0) then
       call energy_print_total_energy(self%energy)
    end if
    !end if

!!$    if (wftype == QM_WFTYPE_RHF .or. wftype == QM_WFTYPE_RDFT) then
!!$       !call mulliken_population(&
!!$       !     pop, self%integral%s, self%energy%rhf_data%d, self%data%atoms,&
!!$       !     natom, self%energy%rhf_data%nao, ldebug, istat)
!!$       !if (istat /= 0) return
!!$       !if (lprint > 0) then
!!$       !   if (myrank == 0)&
!!$       !        call mulliken_population_print(pop, self%data%atoms, natom)
!!$       !end if
!!$    else
       pop(:) = 0.0d0
!!$    end if

    !if (isave == QM_SAVE_FLAG_ON) then
    !  call energy_save_data(&
    !      self%energy,&
    !      self%data%atoms, self%data%ptcharges, self%data%gscharges,
    !      self%data%fswolf, istat)
    !  if (istat /= 0) return
    !end if

!!$    if (wftype /= QM_WFTYPE_CIS) then
!!$       call integral_free_eri(self%integral%eri, ldebug, istat)
!!$       if (istat .ne. 0) return
!!$    end if

    call integral_free_overlap(self%integral, ldebug, istat)
    if (istat /= 0) return
    call integral_free_hcore(self%integral, ldebug, istat)
    if (istat /= 0) return

    if (self%qmmm_options%check == QM_CHECK_ENERGY) go to 7

    call mpi_barrier(icomm, istat)
    call pl_prof_region_begin('::Force')
    call force_new_data(&
         self%force,&
         self%data%atoms%num, self%data%ptcharges%num, self%data%gscharges%num,&
         self%data%fswolf%num,&
         istat)
    if (istat /= 0) return

    call force_init(self%force, ldebug, istat)
    if (istat /= 0) return

    call force_solve(&
         self%force, self%energy,&
!!$         self%dft_data, self%dft_options,&
         self%data%atoms, self%data%ptcharges, self%data%gscharges,&
         self%data%fswolf,&
         self%integral, self%pl_mpi, lprint, ldebug, istat)
    if (istat /= 0) return
    call mpi_barrier(icomm, istat)
    call pl_prof_region_end('::Force')

!!$    if (wftype == QM_WFTYPE_CIS) then
!!$       call integral_free_eri(self%integral%eri, ldebug, istat)
!!$       if (istat .ne. 0) return
!!$    end if

    !if (isave == QM_SAVE_FLAG_ON) then
    !  call force_save_data(self%force, istat)
    !  if (istat /= 0) return
    !end if

    if (myrank == 0) then
       !if (lprint >= QM_PRINT_LEVEL_5) then
       !!if (lprint >= 1) then
          !!call energy_print_data(self%energy)
          !call force_print_data(self%force)
          call force_print_data_brief(self%force)
       !else if (lprint >= QM_PRINT_LEVEL_1) then
       !   !!call energy_print_data_brief(self%energy)
       !   call force_print_data_brief(self%force)
       !end if
    end if
    energy_total = self%energy%total
    atom_force(:,:) = self%force%force_atom(:,:)
    pc_force(:,:)   = self%force%force_pc(:,:)
    gc_force(:,:)   = self%force%force_gc(:,:)

    call force_free_data(self%force, ldebug, istat)
    if (istat /= 0) return

7   call integral_free_params(self%integral, ldebug, istat)
    if (istat /= 0) return

!!$    if (wftype == QM_WFTYPE_RDFT .or. wftype == QM_WFTYPE_UDFT) then
!!$       call dft_free_data(self%dft_data, ldebug, istat)
!!$       if (istat /= 0) return
!!$    end if

    call free_ptcharges(self%data%ptcharges, ldebug, istat)
    if (istat /= 0) return

    call free_gscharges(self%data%gscharges, ldebug, istat)
    if (istat /= 0) return

    call free_fswolf(self%data%fswolf, ldebug, istat)
    if (istat /= 0) return

!!$    call energy_free_data(self%energy, ldebug, istat)
!!$    if (istat /= 0) return

9   if (lprint >= QM_PRINT_LEVEL_1) then
       if (myrank == 0) then
          call p_print_status(istat)
          call p_print_footer()
       end if
    end if

    call mpi_barrier(icomm, istat)
    call pl_prof_region_end('QM')

  end subroutine qm_solve

  !-----------------------------------------------------------------------------
  integer function qm_get_lprint(self)
    type(qm_type),intent(inout)::self
    qm_get_lprint = self%control_options%lprint
  end function qm_get_lprint

  !-----------------------------------------------------------------------------
  integer function qm_get_ldebug(self)
    type(qm_type),intent(inout)::self
    qm_get_ldebug = self%control_options%ldebug
  end function qm_get_ldebug

  !-----------------------------------------------------------------------------
  subroutine qm_print_inputs(&
       qm_natom, qm_atom_xyz, qm_atom_chg,&
       qm_npc, qm_pc_xyz, qm_pc_chg,&
       qm_ngc, qm_gc_xyz, qm_gc_chg, qm_gc_rc2)
    integer,intent(in)::qm_natom, qm_npc, qm_ngc
    real(8),intent(in)::qm_atom_xyz(:,:), qm_atom_chg(:)
    real(8),intent(in)::qm_pc_xyz(:,:), qm_pc_chg(:)
    real(8),intent(in)::qm_gc_xyz(:,:), qm_gc_chg(:), qm_gc_rc2(:)

    integer::i, j

    write(6,'(1x)')
    write(6,'(''Atoms:'')')
    write(6,'(1x,''Number of atoms ='',i6)') qm_natom
!!$    if (qm_natom > 0) then
!!$       write(6,'(1x)')
!!$       write(6,'(6x,''No.         X             Y             Z'',&
!!$            &''       Charges'')')
!!$       do i = 1, qm_natom
!!$          write(6,'(i8,3f14.8,f11.5)')&
!!$               i, (qm_atom_xyz(j,i),j=1,3), qm_atom_chg(i)
!!$       end do
!!$    end if

    write(6,'(1x)')
    write(6,'(''Gaussian charges:'')')
    write(6,'(1x,''Number of gaussian charges ='',i6)') qm_ngc
!!$    if (qm_ngc > 0) then
!!$       write(6,'(1x)')
!!$       write(6,'(6x,''No.         X             Y             Z'',&
!!$            &''       Charges       Alpha'')')
!!$       do i = 1, qm_ngc
!!$          write(6,'(i8,3f14.8,f11.5,f14.8)')&
!!$               i, (qm_gc_xyz(j,i),j=1,3), qm_gc_chg(i), qm_gc_rc2(i)
!!$       end do
!!$    end if

    write(6,'(1x)')
    write(6,'(''Point charges:'')')
    write(6,'(1x,''Number of point charges ='',i6)') qm_npc
!!$    if (qm_npc > 0) then
!!$       write(6,'(1x)')
!!$       write(6,'(6x,''No.         X             Y             Z'',&
!!$            &''       Charges'')')
!!$       do i = 1, qm_npc
!!$          write(6,'(i8,3f14.8,f11.5)') i, (qm_pc_xyz(j,i),j=1,3), qm_pc_chg(i)
!!$       end do
!!$    end if

  end subroutine qm_print_inputs

  !-----------------------------------------------------------------------------
  subroutine qm_print_results(&
       qm_natom, qm_npc, qm_ngc, qm_energy,&
       qm_atom_force, qm_pc_force, qm_gc_force)
    integer,intent(in)::qm_natom, qm_npc, qm_ngc
    real(8),intent(in)::qm_atom_force(:,:), qm_pc_force(:,:), qm_gc_force(:,:)
    real(8),intent(in)::qm_energy

    real(8)::force_x_tot, force_y_tot, force_z_tot
    real(8)::force_x, force_y, force_z
    integer::i, j

    write(6,'(''Total energy ='',f20.10)') qm_energy
    force_x_tot = 0.0d0
    force_y_tot = 0.0d0
    force_z_tot = 0.0d0

    if (qm_natom > 0) then
       force_x = 0.0d0
       force_y = 0.0d0
       force_z = 0.0d0
       write(6,'('' Atoms:'')')
       write(6,'(20x,''X'',17x,''Y'',17x,''Z'')')
       do i = 1, qm_natom
          !write(6,'(i8,3f18.10)') i, (qm_atom_force(j,i), j = 1, 3)
          force_x = force_x + qm_atom_force(1,i)
          force_y = force_y + qm_atom_force(2,i)
          force_z = force_z + qm_atom_force(3,i)
       end do
       write(6,'(8x,3(6x,''------------''))')
       write(6,'(4x,''sum:'',3f18.10)') force_x, force_y, force_z
       force_x_tot = force_x_tot + force_x
       force_y_tot = force_y_tot + force_y
       force_z_tot = force_z_tot + force_z
    end if

    if (qm_ngc > 0) then
       force_x = 0.0d0
       force_y = 0.0d0
       force_z = 0.0d0
       write(6,'(/'' Gaussian charges:'')')
       write(6,'(20x,''X'',17x,''Y'',17x,''Z'')')
       do i = 1, qm_ngc
          !write(6,'(i8,3f18.10)') i, (qm_gc_force(j,i), j = 1, 3)
          force_x = force_x + qm_gc_force(1,i)
          force_y = force_y + qm_gc_force(2,i)
          force_z = force_z + qm_gc_force(3,i)
       end do
       write(6,'(8x,3(6x,''------------''))')
       write(6,'(4x,''sum:'',3f18.10)') force_x, force_y, force_z
       force_x_tot = force_x_tot + force_x
       force_y_tot = force_y_tot + force_y
       force_z_tot = force_z_tot + force_z
    end if

    if (qm_npc > 0) then
       force_x = 0.0d0
       force_y = 0.0d0
       force_z = 0.0d0
       write(6,'(/'' Point charges:'')')
       write(6,'(20x,''X'',17x,''Y'',17x,''Z'')')
       do i = 1, qm_npc
          !write(6,'(i8,3f18.10)') i, (qm_pc_force(j,i), j = 1, 3)
          force_x = force_x + qm_pc_force(1,i)
          force_y = force_y + qm_pc_force(2,i)
          force_z = force_z + qm_pc_force(3,i)
       end do
       write(6,'(8x,3(6x,''------------''))')
       write(6,'(4x,''sum:'',3f18.10)') force_x, force_y, force_z
       force_x_tot = force_x_tot + force_x
       force_y_tot = force_y_tot + force_y
       force_z_tot = force_z_tot + force_z
    end if

    write(6,'(/'' Total:'')')
    write(6,'(20x,''X'',17x,''Y'',17x,''Z'')')
    write(6,'(8x,3(6x,''------------''))')
    write(6,'(4x,''sum:'',3f18.10)') force_x_tot, force_y_tot, force_z_tot

  end subroutine qm_print_results

  !=============================================================================
  ! private
  !-----------------------------------------------------------------------------
  subroutine p_print_header()
    write(6,&
         '(/''== QM (a.u.) ========================'',&
         &''==========================='')')
  end subroutine p_print_header

  !-----------------------------------------------------------------------------
  subroutine p_print_status(istat)
    integer,intent(in)::istat

    write(6,'(''QM status ='',i5)') istat

  end subroutine p_print_status

  !-----------------------------------------------------------------------------
  subroutine p_print_footer()
    write(6,&
         '(''====================================='',&
         &''==========================='')')
  end subroutine p_print_footer

  !-----------------------------------------------------------------------------
end module qm_class
