! -*- coding: utf-8 -*-
!
! Dual licensed under the MIT or GPL Version 3 license.
! See 'LICENSE.MIT' for the definition of the MIT license,
! and 'COPYING' for the definition of the GPL license.
!
!> QMオプションクラス
!! 
module qm_options_class
  implicit none
  private

  public::options_type
  public::read_options
  public::save_options
  public::load_options
  public::bcast_options
  public::free_options
  public::print_options
  public::QM_RUNMODE_ENERGY
  public::QM_RUNMODE_OPT
  public::QM_WFTYPE_RHF
  public::QM_WFTYPE_RDFT
  public::QM_WFTYPE_UHF
  public::QM_WFTYPE_UDFT
  public::QM_WFTYPE_CASSCF
  public::QM_WFTYPE_CIS
  public::QM_CHECK_NO
  public::QM_CHECK_DATA
  public::QM_CHECK_ENERGY
  public::QM_CHECK_FORCE
  public::QM_CHECK_ALL
  public::QM_NO
  public::QM_YES

  !integer,public,parameter::MAX_INPUT_LINE_LEN = 256

  type::options_type
     integer::runmode
     integer::wftype
     integer::check
     integer::pdbinput
     character(len=256)::pdbfile
     character(len=256)::qmatomlist
     character(len=256)::chargelist
  end type

  character(len=*),parameter::group_name = 'QM' !< データグループ名称

  integer,parameter::MAX_OPTION_NAME_LEN = 10
  integer,parameter::nopt = 7                               !< オプション数
  character(len=MAX_OPTION_NAME_LEN)::opt_names(nopt) = (/& !< オプションの名称
       'WFTYPE    ',&
       'RUNMODE   ',&
       'CHECK     ',&
       'PDBINPUT  ',&
       'PDBFILE   ',&
       'QMATOMLIST',&
       'CHARGELIST'&
       /)

  integer,parameter::QM_RUNMODE_ENERGY = 1
  integer,parameter::QM_RUNMODE_OPT    = 2
  integer,parameter::QM_RUNMODE_NUM = QM_RUNMODE_OPT
  character(len=*),parameter::QM_RUNMODE(QM_RUNMODE_NUM) = (/&
       'ENERGY',&
       'OPT   '/)

  integer,parameter::QM_WFTYPE_RHF    = 1
  integer,parameter::QM_WFTYPE_RDFT   = 2
  integer,parameter::QM_WFTYPE_UHF    = 3
  integer,parameter::QM_WFTYPE_UDFT   = 4
  integer,parameter::QM_WFTYPE_CASSCF = 5
  integer,parameter::QM_WFTYPE_CIS    = 6
  integer,parameter::QM_WFTYPE_NUM = QM_WFTYPE_CIS
  character(len=*),parameter::QM_WFTYPES(QM_WFTYPE_NUM) = (/&
       'RHF   ',&
       'RDFT  ',&
       'UHF   ',&
       'UDFT  ',&
       'CASSCF',&
       'CIS   '/)

  integer,parameter::QM_CHECK_NO     = 0
  integer,parameter::QM_CHECK_DATA   = 1
  integer,parameter::QM_CHECK_ENERGY = 2
  integer,parameter::QM_CHECK_FORCE  = 3
  integer,parameter::QM_CHECK_ALL    = 4
  integer,parameter::QM_CHECK_NUM = QM_CHECK_ALL + 1
  character(len=*),parameter::QM_CHECK(QM_CHECK_NUM) = (/&
       'NO    ',&
       'DATA  ',&
       'ENERGY',&
       'FORCE ',&
       'ALL   '/)

  integer,parameter::QM_NO  = 0
  integer,parameter::QM_YES = 1
  integer,parameter::QM_YESNO_NUM = QM_YES + 1
  character(len=*),parameter::QM_YESNO(QM_YESNO_NUM) = (/&
       'NO ',&
       'YES'/)

  character(len=*),private,parameter::DEFAULT_PDB_FILE_NAME = 'start.pdb'
  character(len=*),private,parameter::DEFAULT_QMATOMLIST_NAME = 'qmatom.list'
  character(len=*),private,parameter::DEFAULT_CHARGELIST_NAME = 'charge.list'

  character(len=*),parameter::url = 'options_qmmm' !< BMSファイル名称
  integer,parameter::id = 1
  character(len=*),parameter::no_unit = ' '

contains
  !-----------------------------------------------------------------------------
  !> オプションの表示
  subroutine print_options(self)
    type(options_type),intent(in)::self

    write(6,'(/a,'' Options'')') trim(group_name)
    write(6,'(2x,a,'' : '',a)') 'RUNMODE   ',trim(QM_RUNMODE(self%runmode))
    write(6,'(2x,a,'' : '',a)') 'WFTYPE    ',trim(QM_WFTYPES(self%wftype))
    if (self%check > QM_CHECK_NO)&
    write(6,'(2x,a,'' : '',a)') 'CHECK     ',trim(QM_CHECK(self%check+1))
    write(6,'(2x,a,'' : '',a)') 'PDBINPUT  ',trim(QM_YESNO(self%pdbinput+1))
    if (self%pdbinput == QM_YES) then
    write(6,'(2x,a,'' : '',a)') 'PDBFILE   ',trim(self%pdbfile)
    write(6,'(2x,a,'' : '',a)') 'QMATOMLIST',trim(self%qmatomlist)
    write(6,'(2x,a,'' : '',a)') 'CHARGELIST',trim(self%chargelist)
    end if

  end subroutine print_options

  !-----------------------------------------------------------------------------
  !> オプションの入力ファイルからの読み込み
  subroutine read_options(self, lines, nline, ldebug, istat)
    use input_class
    type(options_type),intent(inout)::self
    integer,intent(in)::nline
    character(len=*),intent(in)::lines(nline)
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    character(len=MAX_OPTION_VALUE_LEN)::opt_values(nopt)!< オプションの値
    integer::opt_count(nopt)                             !< オプションの出現回数
    character(len=MAX_OPTION_NAME_LEN)::opt_name
    character(len=MAX_OPTION_VALUE_LEN)::opt_value
    integer::iopt, ival, ipos, ival_default, ipos_default
    real(8)::dval, dval_default
    character(len=MAX_INPUT_LINE_LEN)::str, str_default

    integer::nerror, ierror

    call get_options(&
       lines, nline,&
       group_name, opt_names, opt_values, opt_count, nopt, ldebug, istat)
    if (istat /= 0) return

    nerror = 0
    ierror = 0

    opt_name = 'runmode'
    ipos_default = pl_str_search_name_ignorecase(&
         'ENERGY', QM_RUNMODE, QM_RUNMODE_NUM)
    call get_option_values_strlist(&
         opt_name, opt_names, opt_values, nopt,&
         QM_RUNMODE, QM_RUNMODE_NUM, ipos_default, ipos, istat)
    if (istat /= 0) then
       nerror = nerror + 1; ierror = istat
    end if
    self%runmode = ipos

    opt_name = 'wftype'
    ipos_default = pl_str_search_name_ignorecase(&
         'RHF', QM_WFTYPES, QM_WFTYPE_NUM)
    call get_option_values_strlist(&
         opt_name, opt_names, opt_values, nopt,&
         QM_WFTYPES, QM_WFTYPE_NUM, ipos_default, ipos, istat)
    if (istat /= 0) then
       nerror = nerror + 1; ierror = istat
    end if
    self%wftype = ipos

    opt_name = 'check'
    ipos_default = pl_str_search_name_ignorecase(&
         'NO', QM_CHECK, QM_CHECK_NUM)
    call get_option_values_strlist(&
         opt_name, opt_names, opt_values, nopt,&
         QM_CHECK, QM_CHECK_NUM, ipos_default, ipos, istat)
    if (istat /= 0) then
       nerror = nerror + 1; ierror = istat
    end if
    self%check = ipos - 1

    opt_name = 'pdbinput'
    ipos_default = pl_str_search_name_ignorecase(&
         'NO', QM_YESNO, QM_YESNO_NUM)
    call get_option_values_strlist(&
         opt_name, opt_names, opt_values, nopt,&
         QM_YESNO, QM_YESNO_NUM, ipos_default, ipos, istat)
    if (istat /= 0) then
       nerror = nerror + 1; ierror = istat
    end if
    self%pdbinput = ipos - 1

    opt_name = 'pdbfile'
    str_default = DEFAULT_PDB_FILE_NAME
    call get_option_values_str(&
         opt_name, opt_names, opt_values, nopt, str_default, str, istat)
    if (istat /= 0) return
    self%pdbfile = str

    opt_name = 'qmatomlist'
    str_default = DEFAULT_QMATOMLIST_NAME
    call get_option_values_str(&
         opt_name, opt_names, opt_values, nopt, str_default, str, istat)
    if (istat /= 0) return
    self%qmatomlist = str

    opt_name = 'chargelist'
    str_default = DEFAULT_CHARGELIST_NAME
    call get_option_values_str(&
         opt_name, opt_names, opt_values, nopt, str_default, str, istat)
    if (istat /= 0) return
    self%chargelist = str

    if (nerror /= 0) then
       istat = ierror; return
    end if

  end subroutine read_options

  !-----------------------------------------------------------------------------
  !> オプションのBMS形式ファイルへの保存
  subroutine save_options(self, ldebug, istat)
    use pl_bms_class
    type(options_type),intent(in)::self
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    call pl_bms_write(self%wftype, url, 'wftype', id, no_unit, istat)
    if (istat /= 0) return
    call pl_bms_write(self%runmode, url, 'runmode', id, no_unit, istat)
    if (istat /= 0) return
    call pl_bms_write(self%check, url, 'check', id, no_unit, istat)
    if (istat /= 0) return
    call pl_bms_write(self%pdbinput, url, 'pdbinput', id, no_unit, istat)
    if (istat /= 0) return
    call pl_bms_write(self%pdbfile, url, 'pdbfile', id, no_unit, istat)
    if (istat /= 0) return
    call pl_bms_write(self%qmatomlist, url, 'qmatomlist', id, no_unit, istat)
    if (istat /= 0) return
    call pl_bms_write(self%chargelist, url, 'chargelist', id, no_unit, istat)
    if (istat /= 0) return

  end subroutine save_options

  !-----------------------------------------------------------------------------
  !> オプションのBMS形式ファイルからの読み込み
  subroutine load_options(self, ldebug, istat)
    use pl_bms_class
    type(options_type),intent(inout)::self
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    call pl_bms_read(self%wftype, url, 'wftype', id, istat)
    if (istat /= 0) return
    call pl_bms_read(self%runmode, url, 'runmode', id, istat)
    if (istat /= 0) return
    call pl_bms_read(self%check, url, 'check', id, istat)
    if (istat /= 0) return
    call pl_bms_read(self%pdbinput, url, 'pdbinput', id, istat)
    if (istat /= 0) return
    call pl_bms_read(self%pdbfile, url, 'pdbfile', id, istat)
    if (istat /= 0) return
    call pl_bms_read(self%qmatomlist, url, 'qmatomlist', id, istat)
    if (istat /= 0) return
    call pl_bms_read(self%chargelist, url, 'chargelist', id, istat)
    if (istat /= 0) return

  end subroutine load_options

  !-----------------------------------------------------------------------------
  !> オプションのBcast
  subroutine bcast_options(self, pl_mpi, ldebug, istat)
    use pl_mpi_class
    type(options_type),intent(inout)::self
    type(pl_mpi_type),intent(in)::pl_mpi
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    integer::icomm
    icomm  = pl_mpi%comm

    call mpi_bcast(self%wftype, 1, MPI_INTEGER, 0, icomm, istat)
    if (istat .ne. 0) return
    call mpi_bcast(self%runmode, 1, MPI_INTEGER, 0, icomm, istat)
    if (istat .ne. 0) return
    call mpi_bcast(self%check, 1, MPI_INTEGER, 0, icomm, istat)
    if (istat .ne. 0) return
    call mpi_bcast(self%pdbinput, 1, MPI_INTEGER, 0, icomm, istat)
    if (istat .ne. 0) return
    call mpi_bcast(&
         self%pdbfile, len(self%pdbfile), MPI_CHARACTER, 0, icomm, istat)
    if (istat .ne. 0) return
    call mpi_bcast(&
         self%qmatomlist, len(self%qmatomlist),MPI_CHARACTER, 0, icomm, istat)
    if (istat .ne. 0) return
    call mpi_bcast(&
         self%chargelist, len(self%chargelist), MPI_CHARACTER, 0, icomm, istat)
    if (istat .ne. 0) return

  end subroutine bcast_options

  !-----------------------------------------------------------------------------
  !> 配列開放
  subroutine free_options(self, ldebug, istat)
    type(options_type),intent(in)::self
    integer,intent(in)::ldebug
    integer,intent(inout)::istat

    istat = 0
  end subroutine free_options

  !-----------------------------------------------------------------------------
end module qm_options_class
