! -*- coding: utf-8 -*-
!
! Dual licensed under the MIT or GPL Version 3 license.
! See 'LICENSE.MIT' for the definition of the MIT license,
! and 'COPYING' for the definition of the GPL license.
!
C======================================================================
C
C======================================================================
      subroutine peigen(
     U     a,
     O     e,
     I     n,
     I     icomm, myrank, nproc,
     I     ldebug,
     O     ierr )
      implicit real(8) (a-h,o-z)

      dimension a(n,n), e(n)

#ifdef PL_SCALAPACK
      dimension dummy(1)
      real(8),allocatable :: W(:,:)
      real(8),allocatable :: Z(:,:)
      real(8),allocatable :: work(:)
      integer,allocatable :: iwork(:)

      PARAMETER ( NB  = 64 )
      PARAMETER ( NXB =  5 )

      INTEGER  CONTEXT, NB,
     $         NPROW, NPCOL, MYROW, MYCOL

      INTEGER     BLOCK_CYCLIC_2D, DLEN_, DT_,
     $            CTXT_, M_, N_, MB_, NB_,
     $            RSRC_, CSRC_, LLD_
      PARAMETER ( BLOCK_CYCLIC_2D = 1, DLEN_ = 9, DT_ = 1,
     $            CTXT_ = 2, M_ = 3, N_ = 4, MB_ = 5, NB_ = 6,
     $            RSRC_ = 7, CSRC_ = 8, LLD_ = 9 )
      INTEGER  DESCW(DLEN_), DESCZ(DLEN_)

      if ( ldebug > 0 ) then
         write(6,*) 'MYRANK=',MYRANK
         write(6,*) 'NPROC=',NPROC
      endif
      NBLOCKS = MAX(1, N/(NB*NXB))
      MINPROC = MIN(NPROC, NBLOCKS)
      if ( ldebug > 0 ) then
         write(6,*) 'NBLOCKS =',NBLOCKS
         write(6,*) 'MINPROC =',MINPROC
      endif
      NPROW = int(sqrt(float(MINPROC))) 
      NPCOL = MINPROC/NPROW 

      CALL BLACS_GET( -1, 0, CONTEXT )
      CALL BLACS_GRIDINIT( CONTEXT, 'R', NPROW, NPCOL )
      CALL BLACS_GRIDINFO( CONTEXT, NPROW, NPCOL, MYROW, MYCOL )

      if ( ldebug > 0 ) then
         write(6,*) 'NPROW =',NPROW
         write(6,*) 'NPCOL =',NPCOL
         write(6,*) 'MYROW =',MYROW
         write(6,*) 'MYCOL =',MYCOL
      endif

      if ( MYROW < 0 ) go to 9901

      LM = max(1, NUMROC( N, NB, MYROW, 0, NPROW ))
      LN = max(1, NUMROC( N, NB, MYCOL, 0, NPCOL ))
      if ( ldebug > 0 ) then
         write(6,*) 'LM  =',LM
         write(6,*) 'LN  =',LN
      endif
      
      LD = LM
      CALL DESCINIT( DESCW, N, N, NB, NB, 0, 0, CONTEXT, LD, ierr )
      if ( ierr.ne.0 ) go to 9901
      CALL DESCINIT( DESCZ, N, N, NB, NB, 0, 0, CONTEXT, LD, ierr )
      if ( ierr.ne.0 ) go to 9901

      if ( ldebug > 0 ) then
         write(6,*) 'W_M =',DESCW(3)
         write(6,*) 'W_N =',DESCW(4)
         write(6,*) 'W_MB=',DESCW(5)
         write(6,*) 'W_NB=',DESCW(6)
         write(6,*) 'W_R =',DESCW(7)
         write(6,*) 'W_C =',DESCW(8)
         write(6,*) 'W_LD=',DESCW(9)
      endif

      allocate(W(LD,LN),Z(LD,LN))

      if ( myrank == 0 ) then
         if ( ldebug > 0 ) then
            call ummtw( 'A', a, n, n, n, 8, 6, ierr )
            if ( ierr.ne.0 ) go to 9901
         endif
      endif

      ICTXT = DESCW( CTXT_ )
      NBCOL = DESCW( NB_ )
      NBROW = DESCW( MB_ )
      ICURCOL = DESCW( CSRC_ )
      DO J = 1, N, NBCOL
         JB = MIN(  NBCOL, N-J+1 )
         ICURROW = DESCW( RSRC_ )
         DO I = 1, N, NBROW
            IB = MIN( NBROW, N-I+1 )
            IF( MYROW.EQ.0 .AND. MYCOL.EQ.0 ) THEN
               if ( ldebug > 0 ) then
                  call ummtw( 'A(:,:)', A(I,J), N, IB, JB, 8, 6, ierr)
               endif
               CALL DGESD2D( ICTXT, IB, JB, A(I,J),N,ICURROW,ICURCOL)
            ENDIF
            IF( MYROW.EQ.ICURROW .AND. MYCOL.EQ.ICURCOL ) THEN
               if ( ldebug > 0 ) then
                  write(6,*) 'ICURROW=',ICURROW,' ICURCOL=',ICURCOL
               endif
               call INFOG2L(
     $              I, J, DESCW, NPROW, NPCOL, MYROW, MYCOL,
     $              II, JJ, IRSRC, ICSRC )
               if ( ldebug > 0 ) then
                  write(6,
     $           '(''I='',i2,''  J='',i2,''   IB='',i2,''  JB='',i2,
     $           ''  MYRANK='',i2,''  MYROW='',i2,''  MYCOL='',i2)')
     $           I,J,IB,JB,MYRANK,MYROW,MYCOL
                  write(6,*) 'II=',II,' JJ=',JJ
               endif
               CALL DGERV2D( ICTXT, IB, JB, W(II,JJ), LD, 0, 0 )
               if ( ldebug > 0 ) then
                  call ummtw( 'W(:,:)', W(II,JJ), LD, IB, JB, 8,6,ierr)
               endif
            ENDIF
            ICURROW = MOD( ICURROW+1, NPROW )
         ENDDO
         ICURCOL = MOD( ICURCOL+1, NPCOL )
      ENDDO

      if ( ldebug > 0 ) then
         write(6,*) 'rank=',myrank
         call ummtw( 'W', W, LD, LM, LN, 8, 6, ierr )
         if ( ierr.ne.0 ) go to 9901
      endif

      CALL PDSYEVD( 'V', 'U', N, W, 1, 1, DESCW, e, Z, 1, 1,
     $             DESCZ, dummy, -1, idummy, -1, ierr )
      if ( ierr.ne.0 ) go to 9901

      lwork = int(dummy(1))
      liwork = idummy

      CALL PDSYTRD( 'U', N, W, 1, 1, DESCW, dm1, dm2, dm3, dummy,
     $     -1, ierr )
      if ( ierr.ne.0 ) go to 9901

      lwork_PDSYTRD = int(dummy(1))+N*4

      CALL PDORMTR( 'L', 'U', 'N', N, N, W, 1, 1, DESCW,
     $     dm1, Z, 1, 1, DESCZ, dummy, -1, ierr )
      if ( ierr.ne.0 ) go to 9901

      lwork_PDORMTR = int(dummy(1))+N*2

      CALL PDSTEDC( 'I', N, e, dm1, Z, 1, 1, DESCZ,
     $              dummy, -1, idummy, -1, ierr )
      if ( ierr.ne.0 ) go to 9901

      lwork_PDSTEDC = int(dummy(1))+N*2
      liwork_PDSTEDC = idummy

      lwork = max(lwork, lwork_PDSYTRD, lwork_PDORMTR, lwork_PDSTEDC)
      allocate(work(lwork))

      liwork = max(liwork, liwork_PDSTEDC, 7*N + 8*NPCOL + 2)
      allocate(iwork(liwork))

      CALL PDSYEVD( 'V', 'U', N, W, 1, 1, DESCW, e, Z, 1, 1,
     $             DESCZ, WORK, LWORK, IWORK, LIWORK, ierr )
      if ( ierr.ne.0 ) go to 9900

      if ( ldebug > 0 ) then
         write(6,*) 'rank=',myrank
         call ummtw( 'Z(-)', Z, LD, LM, LN, 8,6,ierr)
         if ( ierr.ne.0 ) go to 9901
      endif

      if ( myrank == 0 ) then
         if ( ldebug > 0 ) then
            call ummtw( 'e', e, n, n, 1, 8, 6, ierr)
            if ( ierr.ne.0 ) go to 9901
         endif
      endif

      ICTXT = DESCZ( CTXT_ )
      NBCOL = DESCZ( NB_ )
      NBROW = DESCZ( MB_ )
      ICURCOL = DESCZ( CSRC_ )
      DO J = 1, N, NBCOL
         JB = MIN(  NBCOL, N-J+1 )
         ICURROW = DESCZ( RSRC_ )
         DO I = 1, N, NBROW
            IB = MIN( NBROW, N-I+1 )
            IF( MYROW.EQ.ICURROW .AND. MYCOL.EQ.ICURCOL ) THEN
               if ( ldebug > 0 ) then
                  write(6,*) 'ICURROW=',ICURROW,' ICURCOL=',ICURCOL
               endif
               call INFOG2L(
     $              I, J, DESCZ, NPROW, NPCOL, MYROW, MYCOL,
     $              II, JJ, IRSRC, ICSRC )
               if ( ldebug > 0 ) then
                  write(6,
     $           '(''I='',i2,''  J='',i2,''   IB='',i2,''  JB='',i2,
     $           ''  MYRANK='',i2,''  MYROW='',i2,''  MYCOL='',i2)')
     $           I,J,IB,JB,MYRANK,MYROW,MYCOL
                  write(6,*) 'II=',II,' JJ=',JJ
                  call ummtw( 'Z(:,:)', Z(II,JJ), LD, IB, JB, 8,6,ierr)
               endif
               CALL DGESD2D( ICTXT, IB, JB, Z(II,JJ), LD, 0, 0 )
            ENDIF
            IF( MYROW.EQ.0 .AND. MYCOL.EQ.0 ) THEN
               CALL DGERV2D( ICTXT, IB, JB, A(I,J),N,ICURROW,ICURCOL)
               if ( ldebug > 0 ) then
                  call ummtw( 'A(:,:)', A(I,J), N, IB, JB, 8, 6, ierr)
               endif
            ENDIF
            ICURROW = MOD( ICURROW+1, NPROW )
         ENDDO
         ICURCOL = MOD( ICURCOL+1, NPCOL )
      ENDDO


      CALL BLACS_GRIDEXIT( CONTEXT )
c$$$      CALL BLACS_EXIT( 0 )

      if ( myrank == 0 ) then
         if ( ldebug > 0 ) then
            call ummtw( 'U', a, n, n, n, 8, 6, ierr)
            call ummtw( 'e', e, n, n, 1, 8, 6, ierr)
            if ( ierr.ne.0 ) go to 9901
         endif
      endif

 9900 deallocate(work)
      deallocate(iwork)
      deallocate(W,Z)

#endif
 9901 return
      end
