! -*- coding: utf-8 -*-
!
! Dual licensed under the MIT or GPL Version 3 license.
! See 'LICENSE.MIT' for the definition of the MIT license,
! and 'COPYING' for the definition of the GPL license.
!
C## CSVDX1 #############################################################
C
C   FUNCTION: Z <-- X / ( Y - ALPHA )
C
C   ARGUMENTS:
C     OUTPUT:
C       Z      : THE VECTOR 'Z' OF LENGTH N
C       ICODE  : RETURN CODE
C                  0 =  NORMAL
C                 -1 =  N < 1
C     INPUT:
C       ALPHA  : THE SCALAR 'ALPHA'
C       X      : THE VECTOR 'X' OF LENGTH N
C       Y      : THE VECTOR 'Y' OF LENGTH N
C       N      : THE NUMBER OF ELEMENTS IN  VECTORS 'X' AND 'Y'
C
C#######################################################################
      SUBROUTINE  CSVDX1 ( Z, X, Y, ALPHA,N ,ICODE )
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION  Z(N),X(N),Y(N)
C
      IF(N.LT.1) GO TO 90
C
      DO 1 I = 1 , N
        Z(I) = X(I)/(Y(I)-ALPHA)
 1    CONTINUE
C
      ICODE = 0
      GO TO 99
C
 90   ICODE = -1
 99   RETURN
      END
C## UMUNIT  #############################################################
C
C   FUNCTION: SET UNIT MATRIX
C
C   ARGUMENTS:
C     OUTPUT:
C       A      : N1 BY N2 MATRIX 'A'
C       ICODE  : RETURN CODE
C                  0 = NORMAL
C                 -1 =    MZA< 1
C                      OR N  < 1
C                      OR MZA< N
C     INPUT:
C       A      : N BY N MATRIX 'A'
C       MZA    : THE SIZE OF LEADING DIMENSION OF ARRAY 'A'
C       N      : THE NUMBER OF ROWS/COLUMNS IN MATRIX 'A'
C
C#######################################################################
      SUBROUTINE  UMUNIT ( A, MZA, N, ICODE )
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION A(MZA,N)
C
c$$$      IF(MZA.LT.1 .OR. N.LT.1 .OR. MZA.LT.N) GO TO 90
C
      CALL  UVZERO ( A, MZA*N, ICODE )
      IF ( ICODE.NE.0 ) GO TO 90
C
      DO 1 I = 1, N
         A(I,I) = 1.0D0
 1    CONTINUE
C
      ICODE  = 0
      GO TO 99
C
 90   ICODE = -1
 99   RETURN
      END
C## UMTRNS #############################################################
C
C    FUNCTION: TRANSPOSE A MATRIX 'A' AND STORE IN A MATRIX 'B'
C              ( B <-- A^T )
C
C   ARGUMENTS:
C     OUTPUT:
C       B      : N1 BY N2 MATRIX 'B'
C       ICODE  : RETURN CODE
C                  0 = NORMAL
C                 -1 =    MZB< 1 OR MZA< 1
C                      OR N1 < 1 OR N2 < 1
C                      OR MZB<N1 OR MZA<N2
C     INPUT:
C       A      : N2 BY N1 MATRIX 'A'
C       MZB    : THE SIZE OF LEADING DIMENSION OF ARRAY 'B'
C       MZA    : THE SIZE OF LEADING DIMENSION OF ARRAY 'A'
C       N1     : THE NUMBER OF ROWS IN MATRIX 'B'
C                AND THE NUMBER OF COLUMS IN MATRIX 'A'
C       N2     : THE NUMBER OF COLUMNS IN MATRIX 'B'
C                AND THE NUMBER OF ROWS IN MATRIX 'A'
C
C#######################################################################
      SUBROUTINE  UMTRNS ( B, A, MZB, MZA, N1, N2, ICODE )
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION B(MZB,N2),A(MZA,N1)
C
c$$$      IF(MZB.LT.1 .OR. MZA.LT.1 .OR. N1.LT.1 .OR. N2.LT.1
c$$$     $   .OR. MZB.LT.N1 .OR. MZA.LT.N2) GO TO 90
C
      DO 1 J = 1 , N2
         DO 2 I = 1 , N1
            B(I,J)=A(J,I)
 2       CONTINUE
 1    CONTINUE
C
      ICODE  = 0
      GO TO 99
C
 90   ICODE = -1
 99   RETURN
      END
C#######################################################################
      DOUBLE PRECISION FUNCTION  UVDOT( X,Y, N, ICODE )
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION X(N),Y(N)
C
      UVDOT = 0.0D0
C
c$$$      IF(N.LT.1) GO TO 90
C
      DO 1 I = 1 , N
        UVDOT = UVDOT + X(I)*Y(I)
 1    CONTINUE
C
      ICODE = 0
      GO TO 99
C
 90   ICODE = -1
 99   RETURN
      END
C#######################################################################
      SUBROUTINE  UVZERO ( X, N, ICODE )
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION  X(N)
C
c$$$      IF(N.LT.1) GO TO 90
C
      DO 1 I = 1 , N
        X(I) = 0.0D0
 1    CONTINUE
C
      ICODE = 0
      GO TO 99
C
 90   ICODE = -1
 99   RETURN
      END
C#######################################################################
      SUBROUTINE  UVZERI ( L, N, ICODE )
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION  L(N)
C
c$$$      IF(N.LT.1) GO TO 90
C
      DO 1 I = 1 , N
        L(I) = 0
 1    CONTINUE
C
      ICODE = 0
      GO TO 99
C
 90   ICODE = -1
 99   RETURN
      END
