! -*- coding: utf-8 -*-
!
! Dual licensed under the MIT or GPL Version 3 license.
! See 'LICENSE.MIT' for the definition of the MIT license,
! and 'COPYING' for the definition of the GPL license.
!
C## UMMTW  #############################################################
C
C   FUNCTION: PRINT OUT A MATRIX 'A' WITH A TITLE 'CTTL'
C
C   ARGUMENTS:
C     OUTPUT:
C       ICODE  : RETURN CODE
C                  0 = NORMAL
C                 -1 = MZA<1 OR N1<1 OR N2<1 OR MZA<N1
C                      OR NCOL<>8,10,12
C     INPUT:
C       CTTL   : THE TITLE 'CTTL'
C       A      : N1 BY N2 MATRIX 'A'
C       MZA    : THE SIZE OF LEADING DIMENSION OF ARRAY 'A'
C       N1     : THE NUMBER OF ROWS OF MATRIX 'A'
C       N2     : THE NUMBER OF COLUMNS OF MATRIX 'A'
C       NCOL   : FORMAT INDICATOR
C                  8 =  8 COLUMNS PER LINE
C                 10 = 10 COLUMNS PER LINE
C                 12 = 12 COLUMNS PER LINE
C       LNOUT  : LOGICAL UNIT NUMBER OF OUTPUT FILE
C
C#######################################################################
      SUBROUTINE  UMMTW(
     I     CTTL, A, MZA, N1, N2, NCOL , LNOUT,
     O     ICODE )
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      CHARACTER  CTTL*(*)
      DIMENSION  A(MZA,N2)
C
c$$$      IF(MZA.LT.1 .OR. N1.LT.1 .OR. N2.LT.1 .OR. MZA.LT.N1) GO TO 90
C
 30   FORMAT(1X)
 10   FORMAT(/1X,A)
      IF    (NCOL.EQ.8)THEN
         WRITE(LNOUT,10) CTTL
         DO 1  I = 1 , N2 , 8
            JE = MIN0(I+7,N2)
            WRITE(LNOUT,11)  (J,J=I,JE)
            DO 2 K = 1 , N1
               WRITE(LNOUT,12)  K,(A(K,J),J=I,JE)
 2          CONTINUE
            WRITE(LNOUT,30)
 1       CONTINUE
 11      FORMAT( /1X,5X,    8( 7X,I8))
 12      FORMAT(  1X,I4,1X, 8(F15.10))
      ELSEIF(NCOL.EQ.10)THEN
         WRITE(LNOUT,10) CTTL
         DO 3  I = 1 , N2 , 10
            JE = MIN0(I+9,N2)
            WRITE(LNOUT,13)  (J,J=I,JE)
            DO 4 K = 1 , N1
               WRITE(LNOUT,14)  K,(A(K,J),J=I,JE)
 4          CONTINUE
            WRITE(LNOUT,30)
 3       CONTINUE
 13      FORMAT( /1X,5X,   10(4X,I8))
 14      FORMAT(  1X,I4,1X,10(F12.7))
      ELSEIF(NCOL.EQ.12)THEN
         WRITE(LNOUT,10) CTTL
         DO 5  I = 1 , N2 , 12
            JE = MIN0(I+11,N2)
            WRITE(LNOUT,15)  (J,J=I,JE)
            DO 6 K = 1 , N1
               WRITE(LNOUT,16)  K,(A(K,J),J=I,JE)
 6          CONTINUE
            WRITE(LNOUT,30)
 5       CONTINUE
 15      FORMAT( /1X,5X,   12(2X,I8))
 16      FORMAT(  1X,I4,1X,12(F10.5))
      ELSE
         GO TO 90
      ENDIF
C
      ICODE  = 0
      GO TO 99
C
 90   ICODE = -1
 99   RETURN
      END
C## UMMTWL #############################################################
C
C   FUNCTION: PRINT OUT A MATRIX 'A' WITH A TITLE 'CTTL'
C
C   ARGUMENTS:
C     OUTPUT:
C       ICODE  : RETURN CODE
C                  0 = NORMAL
C                 -1 = MZA<1 OR N1<1 OR N2<1 OR MZA<N1
C                      OR NCOL<>8,10,12
C     INPUT:
C       CTTL   : THE TITLE 'CTTL'
C       A      : N1 BY N2 MATRIX 'A'
C       MZA    : THE SIZE OF LEADING DIMENSION OF ARRAY 'A'
C       N1     : THE NUMBER OF ROWS OF MATRIX 'A'
C       N2     : THE NUMBER OF COLUMNS OF MATRIX 'A'
C       NCOL   : FORMAT INDICATOR
C                  8 =  8 COLUMNS PER LINE
C                 10 = 10 COLUMNS PER LINE
C                 12 = 12 COLUMNS PER LINE
C       LNOUT  : LOGICAL UNIT NUMBER OF OUTPUT FILE
C
C#######################################################################
      SUBROUTINE  UMMTWL(
     I     CTTL, LABEL, A, MZA, N1, N2, NCOL , LNOUT,
     O     ICODE )
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      CHARACTER  CTTL*(*)
      CHARACTER*(*)  LABEL(N1)
      DIMENSION  A(MZA,N2)
C
c$$$      IF(MZA.LT.1 .OR. N1.LT.1 .OR. N2.LT.1 .OR. MZA.LT.N1) GO TO 90
C
 30   FORMAT(1X)
 10   FORMAT(/1X,A)
      IF    (NCOL.EQ.8)THEN
         WRITE(LNOUT,10) CTTL
         DO 1  I = 1 , N2 , 8
            JE = MIN0(I+7,N2)
            WRITE(LNOUT,11)  (J,J=I,JE)
            DO 2 K = 1 , N1
               WRITE(LNOUT,12)  K,LABEL(K),(A(K,J),J=I,JE)
 2          CONTINUE
            WRITE(LNOUT,30)
 1       CONTINUE
 11      FORMAT( /1X,5X,   1X,8X, 8( 7X,I8))
 12      FORMAT(  1X,I4,1X,1X,A8, 8(F15.10))
      ELSEIF(NCOL.EQ.10)THEN
         WRITE(LNOUT,10) CTTL
         DO 3  I = 1 , N2 , 10
            JE = MIN0(I+9,N2)
            WRITE(LNOUT,13)  (J,J=I,JE)
            DO 4 K = 1 , N1
               WRITE(LNOUT,14)  K,LABEL(K),(A(K,J),J=I,JE)
 4          CONTINUE
            WRITE(LNOUT,30)
 3       CONTINUE
 13      FORMAT( /1X,5X,   1X,8X,10(4X,I8))
 14      FORMAT(  1X,I4,1X,1X,A8,10(F12.7))
      ELSEIF(NCOL.EQ.12)THEN
         WRITE(LNOUT,10) CTTL
         DO 5  I = 1 , N2 , 12
            JE = MIN0(I+11,N2)
            WRITE(LNOUT,15)  (J,J=I,JE)
            DO 6 K = 1 , N1
               WRITE(LNOUT,16)  K,LABEL(K),(A(K,J),J=I,JE)
 6          CONTINUE
            WRITE(LNOUT,30)
 5       CONTINUE
 15      FORMAT( /1X,5X,   1X,8X,12(2X,I8))
 16      FORMAT(  1X,I4,1X,1X,A8,12(F10.5))
      ELSE
         GO TO 90
      ENDIF
C
      ICODE  = 0
      GO TO 99
C
 90   ICODE = -1
 99   RETURN
      END
C## UMMTW2 #############################################################
C
C   FUNCTION: PRINT OUT A MATRIX 'A' WITH A TITLE 'CTTL'
C
C   ARGUMENTS:
C     OUTPUT:
C       ICODE  : RETURN CODE
C                  0 = NORMAL
C                 -1 = MZA<1 OR N1<1 OR N2<1 OR MZA<N1
C                      OR NCOL<>8,10,12
C     INPUT:
C       CTTL   : THE TITLE 'CTTL'
C       A      : N1 BY N2 MATRIX 'A'
C       B      : VECTOR 'B' OF LENGTH N2
C       MZA    : THE SIZE OF LEADING DIMENSION OF ARRAY 'A'
C       N1     : THE NUMBER OF ROWS OF MATRIX 'A'
C       N2     : THE NUMBER OF COLUMNS OF MATRIX 'A'
C       NCOL   : FORMAT INDICATOR
C                  8 =  8 COLUMNS PER LINE
C                 10 = 10 COLUMNS PER LINE
C                 12 = 12 COLUMNS PER LINE
C       LNOUT  : LOGICAL UNIT NUMBER OF OUTPUT FILE
C
C#######################################################################
      SUBROUTINE  UMMTW2(
     I     CTTL, A, B, MZA, N1, N2, NCOL , LNOUT,
     O     ICODE )
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      CHARACTER  CTTL*(*)
C      CHARACTER  CLINE8(8)*15
C      DATA  (CLINE8(I),I=1,8) /8*'---------------'/
      DIMENSION  A(MZA,N2), B(N2)
C
c$$$      IF(MZA.LT.1 .OR. N1.LT.1 .OR. N2.LT.1 .OR. MZA.LT.N1) GO TO 90
C
 30   FORMAT(1X)
 10   FORMAT(/1X,A)
      IF    (NCOL.EQ.8)THEN
         WRITE(LNOUT,10) CTTL
         DO 1  I = 1 , N2 , 8
            JE = MIN0(I+7,N2)
            WRITE(LNOUT,11)  (J,J=I,JE)
            WRITE(LNOUT,21)  (B(J),J=I,JE)
            WRITE(LNOUT,'(1X)')
C
            DO 2 K = 1 , N1
               WRITE(LNOUT,12)  K,(A(K,J),J=I,JE)
 2          CONTINUE
            WRITE(LNOUT,30)
 1       CONTINUE
 11      FORMAT(  1X,5X,    8( 7X,I8))
 21      FORMAT(  1X,5X,    8(F15.10))
 12      FORMAT(  1X,I4,1X, 8(F15.10))
      ELSEIF(NCOL.EQ.10)THEN
         WRITE(LNOUT,10) CTTL
         DO 3  I = 1 , N2 , 10
            JE = MIN0(I+9,N2)
            WRITE(LNOUT,13)  (J,J=I,JE)
            WRITE(LNOUT,23)  (B(J),J=I,JE)
            WRITE(LNOUT,'(1X)')
C
            DO 4 K = 1 , N1
               WRITE(LNOUT,14)  K,(A(K,J),J=I,JE)
 4          CONTINUE
            WRITE(LNOUT,30)
 3       CONTINUE
 13      FORMAT(  1X,5X,   10(4X,I8))
 23      FORMAT(  1X,5X,   10(F12.7))
 14      FORMAT(  1X,I4,1X,10(F12.7))
      ELSEIF(NCOL.EQ.12)THEN
         WRITE(LNOUT,10) CTTL
         DO 5  I = 1 , N2 , 12
            JE = MIN0(I+11,N2)
            WRITE(LNOUT,15)  (J,J=I,JE)
            WRITE(LNOUT,25)  (B(J),J=I,JE)
            WRITE(LNOUT,'(1X)')
C
            DO 6 K = 1 , N1
               WRITE(LNOUT,16)  K,(A(K,J),J=I,JE)
 6          CONTINUE
            WRITE(LNOUT,30)
 5       CONTINUE
 15      FORMAT(  1X,5X,   12(2X,I8))
 25      FORMAT(  1X,5X,   12(F10.5))
 16      FORMAT(  1X,I4,1X,12(F10.5))
      ELSE
         GO TO 90
      ENDIF
C
      ICODE  = 0
      GO TO 99
C
 90   ICODE = -1
 99   RETURN
      END
C## UMMTW3 #############################################################
C
C   FUNCTION: PRINT OUT A MATRIX 'A' WITH A TITLE 'CTTL'
C             ( D FORMAT VERSION )
C
C   ARGUMENTS:
C     OUTPUT:
C       ICODE  : RETURN CODE
C                  0 = NORMAL
C                 -1 = MZA<1 OR N1<1 OR N2<1 OR MZA<N1
C                      OR NCOL<>8,10,12
C     INPUT:
C       CTTL   : THE TITLE 'CTTL'
C       A      : N1 BY N2 MATRIX 'A'
C       MZA    : THE SIZE OF LEADING DIMENSION OF ARRAY 'A'
C       N1     : THE NUMBER OF ROWS OF MATRIX 'A'
C       N2     : THE NUMBER OF COLUMNS OF MATRIX 'A'
C       NCOL   : FORMAT INDICATOR
C                  8 =  8 COLUMNS PER LINE
C                 10 = 10 COLUMNS PER LINE
C                 12 = 12 COLUMNS PER LINE
C       LNOUT  : LOGICAL UNIT NUMBER OF OUTPUT FILE
C
C#######################################################################
      SUBROUTINE  UMMTW3(
     I     CTTL, A, MZA, N1, N2, NCOL , LNOUT,
     O     ICODE )
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      CHARACTER  CTTL*(*)
      DIMENSION  A(MZA,N2)
C
c$$$      IF(MZA.LT.1 .OR. N1.LT.1 .OR. N2.LT.1 .OR. MZA.LT.N1) GO TO 90
C
 30   FORMAT(1X)
 10   FORMAT(/1X,A)
      IF    (NCOL.EQ.8)THEN
         WRITE(LNOUT,10) CTTL
         DO 1  I = 1 , N2 , 8
            JE = MIN0(I+7,N2)
            WRITE(LNOUT,11)  (J,J=I,JE)
            DO 2 K = 1 , N1
               WRITE(LNOUT,12)  K,(A(K,J),J=I,JE)
 2          CONTINUE
            WRITE(LNOUT,30)
 1       CONTINUE
 11      FORMAT( /1X,5X,    8( 7X,I8))
 12      FORMAT(  1X,I4,1X, 8(1PD15.7))
      ELSEIF(NCOL.EQ.10)THEN
         WRITE(LNOUT,10) CTTL
         DO 3  I = 1 , N2 , 10
            JE = MIN0(I+9,N2)
            WRITE(LNOUT,13)  (J,J=I,JE)
            DO 4 K = 1 , N1
               WRITE(LNOUT,14)  K,(A(K,J),J=I,JE)
 4          CONTINUE
            WRITE(LNOUT,30)
 3       CONTINUE
 13      FORMAT( /1X,5X,   10(4X,I8))
 14      FORMAT(  1X,I4,1X,10(1PD12.4))
      ELSEIF(NCOL.EQ.12)THEN
         WRITE(LNOUT,10) CTTL
         DO 5  I = 1 , N2 , 12
            JE = MIN0(I+11,N2)
            WRITE(LNOUT,15)  (J,J=I,JE)
            DO 6 K = 1 , N1
               WRITE(LNOUT,16)  K,(A(K,J),J=I,JE)
 6          CONTINUE
            WRITE(LNOUT,30)
 5       CONTINUE
 15      FORMAT( /1X,5X,   12(2X,I8))
 16      FORMAT(  1X,I4,1X,12(1PD10.2))
      ELSE
         GO TO 90
      ENDIF
C
      ICODE  = 0
      GO TO 99
C
 90   ICODE = -1
 99   RETURN
      END
C## UMMTW4 #############################################################
C
C   FUNCTION: PRINT OUT A MATRIX 'A' WITH A TITLE 'CTTL'
C             ( D FORMAT VERSION )
C
C   ARGUMENTS:
C     OUTPUT:
C       ICODE  : RETURN CODE
C                  0 = NORMAL
C                 -1 = MZA<1 OR N1<1 OR N2<1 OR MZA<N1
C                      OR NCOL<>8,10,12
C     INPUT:
C       CTTL   : THE TITLE 'CTTL'
C       A      : N1 BY N2 MATRIX 'A'
C       B      : VECTOR 'B' OF LENGTH N2
C       MZA    : THE SIZE OF LEADING DIMENSION OF ARRAY 'A'
C       N1     : THE NUMBER OF ROWS OF MATRIX 'A'
C       N2     : THE NUMBER OF COLUMNS OF MATRIX 'A'
C       NCOL   : FORMAT INDICATOR
C                  8 =  8 COLUMNS PER LINE
C                 10 = 10 COLUMNS PER LINE
C                 12 = 12 COLUMNS PER LINE
C       LNOUT  : LOGICAL UNIT NUMBER OF OUTPUT FILE
C
C#######################################################################
      SUBROUTINE  UMMTW4(
     I     CTTL, A, B, MZA, N1, N2, NCOL , LNOUT,
     O     ICODE )
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      CHARACTER  CTTL*(*)
C      CHARACTER  CLINE8(8)*15
C      DATA  (CLINE8(I),I=1,8) /8*'---------------'/
      DIMENSION  A(MZA,N2), B(N2)
C
c$$$      IF(MZA.LT.1 .OR. N1.LT.1 .OR. N2.LT.1 .OR. MZA.LT.N1) GO TO 90
C
 30   FORMAT(1X)
 10   FORMAT(/1X,A)
      IF    (NCOL.EQ.8)THEN
         WRITE(LNOUT,10) CTTL
         DO 1  I = 1 , N2 , 8
            JE = MIN0(I+7,N2)
            WRITE(LNOUT,11)  (J,J=I,JE)
            WRITE(LNOUT,21)  (B(J),J=I,JE)
            WRITE(LNOUT,'(1X)')
C
            DO 2 K = 1 , N1
               WRITE(LNOUT,12)  K,(A(K,J),J=I,JE)
 2          CONTINUE
            WRITE(LNOUT,30)
 1       CONTINUE
 11      FORMAT(  1X,5X,    8( 7X,I8))
 21      FORMAT(  1X,5X,    8(1PD15.7))
 12      FORMAT(  1X,I4,1X, 8(1PD15.7))
      ELSEIF(NCOL.EQ.10)THEN
         WRITE(LNOUT,10) CTTL
         DO 3  I = 1 , N2 , 10
            JE = MIN0(I+9,N2)
            WRITE(LNOUT,13)  (J,J=I,JE)
            WRITE(LNOUT,23)  (B(J),J=I,JE)
            WRITE(LNOUT,'(1X)')
C
            DO 4 K = 1 , N1
               WRITE(LNOUT,14)  K,(A(K,J),J=I,JE)
 4          CONTINUE
            WRITE(LNOUT,30)
 3       CONTINUE
 13      FORMAT(  1X,5X,   10(4X,I8))
 23      FORMAT(  1X,5X,   10(1PD12.4))
 14      FORMAT(  1X,I4,1X,10(1PD12.4))
      ELSEIF(NCOL.EQ.12)THEN
         WRITE(LNOUT,10) CTTL
         DO 5  I = 1 , N2 , 12
            JE = MIN0(I+11,N2)
            WRITE(LNOUT,15)  (J,J=I,JE)
            WRITE(LNOUT,25)  (B(J),J=I,JE)
            WRITE(LNOUT,'(1X)')
C
            DO 6 K = 1 , N1
               WRITE(LNOUT,16)  K,(A(K,J),J=I,JE)
 6          CONTINUE
            WRITE(LNOUT,30)
 5       CONTINUE
 15      FORMAT(  1X,5X,   12(2X,I8))
 25      FORMAT(  1X,5X,   12(1PD10.2))
 16      FORMAT(  1X,I4,1X,12(1PD10.2))
      ELSE
         GO TO 90
      ENDIF
C
      ICODE  = 0
      GO TO 99
C
 90   ICODE = -1
 99   RETURN
      END
C## UMMTW5 #############################################################
C
C   FUNCTION: PRINT OUT A MATRIX 'A' WITH A TITLE 'CTTL'
C
C   ARGUMENTS:
C     OUTPUT:
C       ICODE  : RETURN CODE
C                  0 = NORMAL
C                 -1 = MZA<1 OR N1<1 OR N2<1 OR MZA<N1
C                      OR NCOL<>8,10,12
C     INPUT:
C       CTTL   : THE TITLE 'CTTL'
C       A      : N1 BY N2 MATRIX 'A'
C       B      : VECTOR 'B' OF LENGTH N2
C       C      : VECTOR 'C' OF LENGTH N2
C       MZA    : THE SIZE OF LEADING DIMENSION OF ARRAY 'A'
C       N1     : THE NUMBER OF ROWS OF MATRIX 'A'
C       N2     : THE NUMBER OF COLUMNS OF MATRIX 'A'
C       NCOL   : FORMAT INDICATOR
C                  8 =  8 COLUMNS PER LINE
C                 10 = 10 COLUMNS PER LINE
C                 12 = 12 COLUMNS PER LINE
C       LNOUT  : LOGICAL UNIT NUMBER OF OUTPUT FILE
C
C#######################################################################
      SUBROUTINE  UMMTW5(
     I     CTTL, A,B,C, MZA, N1, N2, NCOL, LNOUT,ICODE
     O     )
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      CHARACTER  CTTL*(*)
C      CHARACTER  CLINE8(8)*15
C      DATA  (CLINE8(I),I=1,8) /8*'---------------'/
      DIMENSION  A(MZA,N2), B(N2), C(N2)
C
c$$$      IF(MZA.LT.1 .OR. N1.LT.1 .OR. N2.LT.1 .OR. MZA.LT.N1) GO TO 90
C
 30   FORMAT(1X)
 10   FORMAT(/1X,A)
      IF    (NCOL.EQ.8)THEN
         WRITE(LNOUT,10) CTTL
         DO 1  I = 1 , N2 , 8
            JE = MIN0(I+7,N2)
            WRITE(LNOUT,11)  (J,J=I,JE)
            WRITE(LNOUT,21)  (B(J),J=I,JE)
            WRITE(LNOUT,21)  (C(J),J=I,JE)
            WRITE(LNOUT,'(1X)')
C
            DO 2 K = 1 , N1
               WRITE(LNOUT,12)  K,(A(K,J),J=I,JE)
 2          CONTINUE
            WRITE(LNOUT,30)
 1       CONTINUE
 11      FORMAT(  1X,5X,    8( 7X,I8))
 21      FORMAT(  1X,5X,    8(F15.10))
 12      FORMAT(  1X,I4,1X, 8(F15.10))
      ELSEIF(NCOL.EQ.10)THEN
         WRITE(LNOUT,10) CTTL
         DO 3  I = 1 , N2 , 10
            JE = MIN0(I+9,N2)
            WRITE(LNOUT,13)  (J,J=I,JE)
            WRITE(LNOUT,23)  (B(J),J=I,JE)
            WRITE(LNOUT,23)  (C(J),J=I,JE)
            WRITE(LNOUT,'(1X)')
C
            DO 4 K = 1 , N1
               WRITE(LNOUT,14)  K,(A(K,J),J=I,JE)
 4          CONTINUE
            WRITE(LNOUT,30)
 3       CONTINUE
 13      FORMAT(  1X,5X,   10(4X,I8))
 23      FORMAT(  1X,5X,   10(F12.7))
 14      FORMAT(  1X,I4,1X,10(F12.7))
      ELSEIF(NCOL.EQ.12)THEN
         WRITE(LNOUT,10) CTTL
         DO 5  I = 1 , N2 , 12
            JE = MIN0(I+11,N2)
            WRITE(LNOUT,15)  (J,J=I,JE)
            WRITE(LNOUT,25)  (B(J),J=I,JE)
            WRITE(LNOUT,25)  (C(J),J=I,JE)
            WRITE(LNOUT,'(1X)')
C
            DO 6 K = 1 , N1
               WRITE(LNOUT,16)  K,(A(K,J),J=I,JE)
 6          CONTINUE
            WRITE(LNOUT,30)
 5       CONTINUE
 15      FORMAT(  1X,5X,   12(2X,I8))
 25      FORMAT(  1X,5X,   12(F10.5))
 16      FORMAT(  1X,I4,1X,12(F10.5))
      ELSE
         GO TO 90
      ENDIF
C
      ICODE  = 0
      GO TO 99
C
 90   ICODE = -1
 99   RETURN
      END
C## UMTRW  #############################################################
C
C   FUNCTION: PRINT OUT A LOWER-TRIAGULAR-MATRIX 'A' WITH A TITLE 'CTTL'
C     A(MZA) (<-- A'(N,N),ONE DIMENSIONAL TREATMENT,MZA=N*(N+1)/2)
C
C   ARGUMENTS:
C     OUTPUT:
C       ICODE  : RETURN CODE
C                  0 = NORMAL
C                 -1 = MZA<1 OR N<1 OR MZA<N*(N+1)/2
C                      OR NCOL<>8,10,12
C     INPUT:
C       CTTL   : THE TITLE 'CTTL'
C       A      : THE VECTOR 'A' OF LENGTH N*N(+1)/2
C       MZA    : THE NUMBER OF ELEMENTS IN VECTOR 'A'
C       N      : THE ORDER OF MATRIX 'A''
C       NCOL   : FORMAT INDICATOR
C                  8 =  8 COLUMNS PER LINE
C                 10 = 10 COLUMNS PER LINE
C                 12 = 12 COLUMNS PER LINE
C       LNOUT  : LOGICAL UNIT NUMBER OF OUTPUT FILE
C
C#######################################################################
      SUBROUTINE  UMTRW(
     I     CTTL, A, MZA, N, NCOL, LNOUT,
     O     ICODE )
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      CHARACTER  CTTL*(*)
      DIMENSION  A(N*(N+1)/2)
C
c$$$      IF(MZA.LT.1 .OR. N.LT.1 .OR. MZA.LT.N*(N+1)/2) GO TO 90
C
 10   FORMAT(/1X,A)
      IF    (NCOL.EQ.8)THEN
         WRITE(LNOUT,10) CTTL
         DO 1  I = 1 , N , 8
            JE = MIN0(I+7,N)
            WRITE(LNOUT,11)  (J,J=I,JE)
            DO 2  K = I , N
               IKS = K*(K-1)/2 + I
               IKE = MIN0(IKS+K-I,IKS+7)
               WRITE(LNOUT,12)  K,(A(IK),IK=IKS,IKE)
 2          CONTINUE
 1       CONTINUE
 11      FORMAT( /1X,5X,    8( 7X,I8))
 12      FORMAT(  1X,I4,1X, 8(F15.10))
      ELSEIF(NCOL.EQ.10)THEN
         WRITE(LNOUT,10) CTTL
         DO 3  I = 1 , N , 10
            JE = MIN0(I+9,N)
            WRITE(LNOUT,13)  (J,J=I,JE)
            DO 4  K = I , N
               IKS = K*(K-1)/2 + I
               IKE = MIN0(IKS+K-I,IKS+9)
               WRITE(LNOUT,14)  K,(A(IK),IK=IKS,IKE)
 4          CONTINUE
 3       CONTINUE
 13      FORMAT( /1X,5X,   10(4X,I8))
 14      FORMAT(  1X,I4,1X,10(F12.7))
      ELSEIF(NCOL.EQ.12)THEN
         WRITE(LNOUT,10) CTTL
         DO 5  I = 1 , N , 12
            JE = MIN0(I+11,N)
            WRITE(LNOUT,15)  (J,J=I,JE)
            DO 6  K = I , N
               IKS = K*(K-1)/2 + I
               IKE = MIN0(IKS+K-I,IKS+11)
               WRITE(LNOUT,16)  K,(A(IK),IK=IKS,IKE)
 6          CONTINUE
 5       CONTINUE
 15      FORMAT( /1X,5X,   12(2X,I8))
 16      FORMAT(  1X,I4,1X,12(F10.5))
      ELSE
         GO TO 90
      ENDIF
C
      ICODE  = 0
      GO TO 99
C
 90   ICODE = -1
 99   RETURN
      END
C## UMTRW3 #############################################################
C
C   FUNCTION: PRINT OUT A LOWER-TRIAGULAR-MATRIX 'A' WITH A TITLE 'CTTL'
C     A(MZA) (<-- A'(N,N),ONE DIMENSIONAL TREATMENT,MZA=N*(N+1)/2)
C             ( D FORMAT VERSION )
C
C   ARGUMENTS:
C     OUTPUT:
C       ICODE  : RETURN CODE
C                  0 = NORMAL
C                 -1 = MZA<1 OR N<1 OR MZA<N*(N+1)/2
C                      OR NCOL<>8,10,12
C     INPUT:
C       CTTL   : THE TITLE 'CTTL'
C       A      : THE VECTOR 'A' OF LENGTH N*N(+1)/2
C       MZA    : THE NUMBER OF ELEMENTS IN VECTOR 'A'
C       N      : THE ORDER OF MATRIX 'A''
C       NCOL   : FORMAT INDICATOR
C                  8 =  8 COLUMNS PER LINE
C                 10 = 10 COLUMNS PER LINE
C                 12 = 12 COLUMNS PER LINE
C       LNOUT  : LOGICAL UNIT NUMBER OF OUTPUT FILE
C
C#######################################################################
      SUBROUTINE  UMTRW3(
     I     CTTL, A, MZA, N, NCOL, LNOUT,
     O     ICODE )
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      CHARACTER  CTTL*(*)
      DIMENSION  A(MZA)
C
c$$$      IF(MZA.LT.1 .OR. N.LT.1 .OR. MZA.LT.N*(N+1)/2) GO TO 90
C
 10   FORMAT(/1X,A)
      IF    (NCOL.EQ.8)THEN
         WRITE(LNOUT,10) CTTL
         DO 1  I = 1 , N , 8
            JE = MIN0(I+7,N)
            WRITE(LNOUT,11)  (J,J=I,JE)
            DO 2  K = 1 , N
               IKS = K*(K-1)/2 + I
               IKE = MIN0(IKS+K-I,IKS+7)
               WRITE(LNOUT,12)  K,(A(IK),IK=IKS,IKE)
 2          CONTINUE
 1       CONTINUE
 11      FORMAT( /1X,5X,    8( 7X,I8))
 12      FORMAT(  1X,I4,1X, 8(1PD15.7))
      ELSEIF(NCOL.EQ.10)THEN
         WRITE(LNOUT,10) CTTL
         DO 3  I = 1 , N , 10
            JE = MIN0(I+9,N)
            WRITE(LNOUT,13)  (J,J=I,JE)
            DO 4  K = 1 , N
               IKS = K*(K-1)/2 + I
               IKE = MIN0(IKS+K-I,IKS+9)
               WRITE(LNOUT,14)  K,(A(IK),IK=IKS,IKE)
 4          CONTINUE
 3       CONTINUE
 13      FORMAT( /1X,5X,   10(4X,I8))
 14      FORMAT(  1X,I4,1X,10(1PD12.4))
      ELSEIF(NCOL.EQ.12)THEN
         WRITE(LNOUT,10) CTTL
         DO 5  I = 1 , N , 12
            JE = MIN0(I+11,N)
            WRITE(LNOUT,15)  (J,J=I,JE)
            DO 6  K = 1 , N
               IKS = K*(K-1)/2 + I
               IKE = MIN0(IKS+K-I,IKS+11)
               WRITE(LNOUT,16)  K,(A(IK),IK=IKS,IKE)
 6          CONTINUE
 5       CONTINUE
 15      FORMAT( /1X,5X,   12(2X,I8))
 16      FORMAT(  1X,I4,1X,12(1PD10.2))
      ELSE
         GO TO 90
      ENDIF
C
      ICODE  = 0
      GO TO 99
C
 90   ICODE = -1
 99   RETURN
      END
