! -*- coding: utf-8 -*-
!
! Dual licensed under the MIT or GPL Version 3 license.
! See 'LICENSE.MIT' for the definition of the MIT license,
! and 'COPYING.GPL' for the definition of the GPL license.
!
!> QM main
!!
program main
  use qm_class
  !!==begin test==
  use data_class, only:&
       qm_data_type => data_type,&
       qm_init_data => init_data,&
       qm_read_inp_data => read_inp_data,&
       qm_bcast_data => bcast_data,&
       qm_num_data => num_data,&
       qm_get_data => get_data,&
       qm_free_data => free_data
  !!==end test==
  implicit none
  include 'mpif.h'

  type(qm_type)::qm
  real(8),allocatable::qm_atom_xyz(:,:), qm_atom_chg(:)
  real(8),allocatable::qm_pc_xyz(:,:), qm_pc_chg(:)
  real(8),allocatable::qm_gc_xyz(:,:), qm_gc_chg(:), qm_gc_rc2(:)
  real(8),allocatable::qm_fsw_xyz(:,:), qm_fsw_chg(:), qm_fsw_rc2(:)
  real(8),allocatable::qm_atom_force(:,:), qm_pc_force(:,:), qm_gc_force(:,:)
  real(8),allocatable::qm_fsw_force(:,:)
  real(8),allocatable::qm_pop(:)
  real(8)::qm_energy
  integer::qm_natom, qm_npc, qm_ngc, qm_nfsw
  integer::qm_iguess, qm_isave, qm_lprint, qm_istat
  integer::md_max_iter, md_iter
  integer::icomm, myrank, nproc
  integer::istat

  !!==begin test==
  character(len=*),parameter::QM_INPUT_FILE = 'qm.inp'
  type(qm_data_type)::test_qm_data
  integer::qm_ldebug
  integer::i
  !!==end test==

  istat = 0
  md_max_iter = 1

  icomm = MPI_COMM_WORLD
  call MPI_Init(istat)
  if (istat /= 0) go to 9
  call MPI_Comm_rank(icomm, myrank, istat)
  if (istat /= 0) go to 9
  call MPI_Comm_size(icomm, nproc, istat)
  if (istat /= 0) go to 9
  if (myrank == 0) write(6,'(''Number of processes = '',i5)') nproc

  call qm_init(qm, icomm, qm_istat)
  if (qm_istat /= 0) go to 9
  if (myrank == 0) call qm_print_options(qm)

  !!==begin test==
  qm_ldebug = 0
  call qm_init_data(test_qm_data, qm_ldebug, qm_istat)
  if (qm_istat /= 0) go to 9
  qm_ldebug = qm_get_ldebug(qm)
  if (myrank == 0) then
     call qm_read_inp_data(test_qm_data, QM_INPUT_FILE, qm_ldebug, qm_istat)
  end if
  call mpi_bcast(qm_istat, 1, MPI_INTEGER, 0, icomm, istat)
  if (qm_istat /= 0) go to 9
  call qm_bcast_data(test_qm_data, qm%pl_mpi, qm_ldebug, qm_istat)
  if (qm_istat /= 0) go to 9
  call qm_num_data(test_qm_data, qm_natom, qm_npc, qm_ngc, qm_nfsw)
  !!==end test==

  allocate(&
       qm_atom_xyz(3,qm_natom), qm_atom_chg(qm_natom),&
       qm_atom_force(3,qm_natom),&
       qm_pc_xyz(3,qm_npc), qm_pc_chg(qm_npc),&
       qm_pc_force(3,qm_npc),&
       qm_gc_xyz(3,qm_ngc), qm_gc_chg(qm_ngc), qm_gc_rc2(qm_ngc),&
       qm_gc_force(3,qm_ngc),&
       qm_fsw_xyz(3,qm_nfsw), qm_fsw_chg(qm_nfsw), qm_fsw_rc2(qm_nfsw),&
       qm_fsw_force(3,qm_ngc),&
       qm_pop(qm_natom), stat=qm_istat)
  if (qm_istat /= 0) go to 9

  !!==begin test==
  call qm_get_data(test_qm_data,&
       qm_natom, qm_atom_xyz, qm_atom_chg,&
       qm_npc, qm_pc_xyz, qm_pc_chg,&
       qm_ngc, qm_gc_xyz, qm_gc_chg, qm_gc_rc2,&
       qm_nfsw, qm_fsw_xyz, qm_fsw_chg, qm_fsw_rc2,&
       qm_istat)
  if (qm_istat /= 0) go to 9
  !!==end test==

  do md_iter = 1, md_max_iter
     if (myrank == 0) write(6,'(/''MD_Iter='',i8)') md_iter

     qm_lprint = QM_PRINT_LEVEL_1
     qm_isave  = QM_SAVE_FLAG_OFF
     qm_iguess = QM_GUESS_FLAG_OFF
     if (mod(md_iter,10000) == 1) qm_lprint = QM_PRINT_LEVEL_3
     if (mod(md_iter,  100) == 1) qm_lprint = QM_PRINT_LEVEL_2
     if (mod(md_iter,  100) == 1) qm_isave  = QM_SAVE_FLAG_ON
     if (    md_iter        == 1) qm_iguess = QM_GUESS_FLAG_ON

     !!==begin test==
     !qm_iguess = QM_GUESS_FLAG_ON
     !qm_isave  = QM_SAVE_FLAG_OFF
     !qm_lprint = QM_PRINT_LEVEL_2

     if (md_iter > 1) then
        if (qm_natom > 0) then
           do i = 1, qm_natom, 2
              qm_atom_xyz(1,i) = qm_atom_xyz(1,i) + 0.1d0
              qm_atom_xyz(2,i) = qm_atom_xyz(2,i) + 0.2d0
              qm_atom_xyz(3,i) = qm_atom_xyz(3,i) + 0.3d0
           end do
        end if
        if (qm_npc > 0) then
           do i = 1, qm_npc, 2
              qm_pc_xyz(1,i) = qm_pc_xyz(1,i) + 0.1d0
              qm_pc_xyz(2,i) = qm_pc_xyz(2,i) + 0.2d0
              qm_pc_xyz(3,i) = qm_pc_xyz(3,i) + 0.3d0
           end do
        end if
        if (qm_ngc > 0) then
           do i = 1, qm_ngc, 2
              qm_gc_xyz(1,i) = qm_gc_xyz(1,i) + 0.1d0
              qm_gc_xyz(2,i) = qm_gc_xyz(2,i) + 0.2d0
              qm_gc_xyz(3,i) = qm_gc_xyz(3,i) + 0.3d0
           end do
        end if
        if (qm_nfsw > 0) then
           do i = 1, qm_nfsw, 2
              qm_fsw_xyz(1,i) = qm_fsw_xyz(1,i) + 0.1d0
              qm_fsw_xyz(2,i) = qm_fsw_xyz(2,i) + 0.2d0
              qm_fsw_xyz(3,i) = qm_fsw_xyz(3,i) + 0.3d0
           end do
        end if
     end if

     !call qm_print_inputs(&
     !     qm_natom, qm_atom_xyz, qm_atom_chg,&
     !     qm_npc, qm_pc_xyz, qm_pc_chg,&
     !     qm_ngc, qm_gc_xyz, qm_gc_chg, qm_gc_rc2)
     !!==end test==

     call qm_solve(&
          qm,&                                      !< [inout] qm object
          qm_natom, qm_atom_xyz, qm_atom_chg,&      !< [in]    atoms
          qm_npc, qm_pc_xyz, qm_pc_chg,&            !< [in]    point charges
          qm_ngc, qm_gc_xyz, qm_gc_chg, qm_gc_rc2,& !< [in]    gaussian charges
          qm_nfsw, qm_fsw_xyz, qm_fsw_chg, qm_fsw_rc2,&
                                                    !< [in] Force Switching Wolf
          qm_energy,&                               !< [out]   total energy
          qm_atom_force,&                           !< [out]   force_atom
          qm_pc_force,&                             !< [out]   force_pc
          qm_gc_force,&                             !< [out]   force_gc
          qm_fsw_force,&                            !< [out]   force_fsw
          qm_pop,&                                  !< [out]   mulliken pop
          qm_iguess,&                               !< [in]    guess option
          qm_isave,&                                !< [in]    save option
          qm_lprint,&                               !< [in]    print option
          qm_istat)                                 !< [out]   status
     if (qm_istat /= 0) go to 9

     !!==begin test==
     !call qm_print_results(&
     !     qm_natom, qm_npc, qm_ngc, qm_energy,&
     !     qm_atom_force, qm_pc_force, qm_gc_force)
     !!==end test==
  enddo

  deallocate(&
       qm_atom_xyz, qm_atom_chg, qm_atom_force,&
       qm_pc_xyz, qm_pc_chg, qm_pc_force,&
       qm_gc_xyz, qm_gc_chg, qm_gc_rc2, qm_gc_force,&
       qm_fsw_xyz, qm_fsw_chg, qm_fsw_rc2, qm_fsw_force,&
       qm_pop, stat=qm_istat)
  if (qm_istat /= 0) go to 9

  !!==begin test==
  call qm_free_data(test_qm_data, qm_ldebug, qm_istat)
  if (qm_istat /= 0) go to 9
  !!==end test==

  call qm_finalize(qm, qm_istat)
  if (qm_istat /= 0) go to 9

9 if (myrank == 0) then
     if (qm_istat /= 0) istat = qm_istat
     write(6,'(/''Status ='',i5)') istat
  end if

  if (istat /= 0) call MPI_Abort(icomm, qm_istat, istat)
  call MPI_Finalize(istat)
end program main
