! -*- coding: utf-8 -*-
!
! Dual licensed under the MIT or GPL Version 3 license.
! See 'LICENSE.MIT' for the definition of the MIT license,
! and 'COPYING' for the definition of the GPL license.
!
! JCUTSS ============================================================
! CALUCULATE (R'S|R'S)-TYPE INTEGRALS FOR SCHWARTZ INEQUALITY
! 
! (s's|s's),(ss'|ss')
! ===================================================================
 
      subroutine jcutss(
     &   a   , b    ,
     &   nfsz, nesz , fdat , edat,
     &   za  , zb   ,
     &   ca  , cb   ,
     &   ncnta, ncntb, ddmax )

      implicit real(8)(a-h,o-z)
      
      parameter (pi52 = 17.4934183276248628462d0)
      dimension fdat(nfsz,nesz),edat(nesz)
      dimension rminv(10),tf(25)
      dimension za(ncnta),zb(ncntb)
      dimension ca(ncnta),cb(ncntb)
      dimension a(3),b(3),psps(9)

      data pi/3.14159265358979323846d+00 /
      data      (tf(i),i=1,25) /
     &                33.0,37.0,41.0,43.0,46.0,49.0,
     &                51.0,54.0,56.0,58.0,61.0,63.0,
     &                66.0,68.0,70.0,72.0,74.0,80.0,
     &                85.0,90.0,95.0,95.0,98.0,99.0,
     &                99.0                           /
c
      data      (rminv(i),i=1,10) /
     &   1.0d+00,  0.333333333333333333d+00,  0.20d+00,
     &   0.14285714285714286d+00,  0.11111111111111111d+00 ,
     &   0.90909090909090909d-01,  0.76923076923076923d-01 ,
     &   0.66666666666666666d-01,  0.58823529411764705d-01 ,
     &   0.52631578947368421d-01 /
c
      data  ga0   / 1.0d+00 /
      data  ga1   / 1.000 000 000 000 005 298d+00 /
      data  ga2   / 0.5d+00 /
      data  ga3   / 0.166 666 666 598 849 826d+00 /
      data  ga4   / 0.416 666 666 666 666 667d-01 /
      data  ga5   / 0.833 355 034 722 222 222d-02 /
      data  ga6   / 0.138 888 888 888 888 889d-02 /

      ax=a(1)
      ay=a(2)
      az=a(3)
      bx=b(1)
      by=b(2)
      bz=b(3)

      do i=1,9
         psps(i) = 0.d0
      enddo

      bax = bx - ax
      bay = by - ay
      baz = bz - az

      abxyz = bax*bax + bay*bay + baz*baz
      pi52_2 = 2.d0 * pi52

      do i=1,ncnta
         do j=1,ncntb

            zet   = za(i) + zb(j)
            roop  = za(i)*zb(j) / zet
            gp    = dexp(-roop*abxyz)
            rambdap = pi52_2 /zet

            contp = ca(i)*cb(j)

            px = (za(i)*ax + zb(j)*bx)/zet
            py = (za(i)*ay + zb(j)*by)/zet
            pz = (za(i)*az + zb(j)*bz)/zet

            do k=1,ncnta
               do l=1,ncntb
            eta   = za(k) + zb(l)
            zpq   = zet + eta
            rooq  = za(k)*zb(l) / eta
            gq    = dexp(-rooq*abxyz)
            rambda= rambdap/(eta*dsqrt(zpq))
            contpq = ca(k)*cb(l) * contp

            qx = (za(k)*ax + zb(l)*bx)/eta
            qy = (za(k)*ay + zb(l)*by)/eta
            qz = (za(k)*az + zb(l)*bz)/eta

            wx = (zet*px + eta*qx)/zpq
            wy = (zet*py + eta*qy)/zpq
            wz = (zet*pz + eta*qz)/zpq

            roo = zet*eta /zpq
            t   = roo*((px-qx)**2+(py-qy)**2+(pz-qz)**2)
  
                  if (t. le .tf(3)) then
     
                     it = idint((t+0.0251d+00)*20.0d+00)
                     dt = 0.05d+00*it - t
                     it = it + 1
                     dt4 = dt*0.25d+00
                     dt5 = dt*0.2d+00
                     dt6 = dt*0.166666666666666667d+00
                     dt2 = dt4 + dt4
                     dt3 = dt6 + dt6
                     f2  = (((((fdat(9,it)*dt6   + fdat(8,it))*dt5
     &                    + fdat(7,it))*dt4  + fdat(6,it))*dt3
     &                    + fdat(5,it))*dt2  + fdat(4,it))*dt
     &                    + fdat(3,it)
c     
                     eed  = (((((ga6*dt + ga5)*dt + ga4)*dt
     &                    + ga3)*dt + ga2)*dt + ga1)*dt + ga0
                     ee   = edat(it)*eed
                     t2   = 2.0d+00*t

                     f1   = (t2*f2 + ee)*rminv(2)
                     f0   = t2*f1 + ee
                  else
                     tinv = 1.0d+00/t
                     f0   = dsqrt(pi*tinv)*0.5d+00
                     f1   = 0.5d+00*f0*tinv
                     f2   = 1.5d+00*f1*tinv
                  end if
 
                  s0 = contpq*gp*gq*rambda*f0
                  s1 = contpq*gp*gq*rambda*f1
                  s2 = contpq*gp*gq*rambda*f2
!==
                  wpx = wx - px
                  wpy = wy - py
                  wpz = wz - pz
                  pax = px -ax
                  pay = py -ay
                  paz = pz -az

                  wqx = wx - qx
                  wqy = wy - qy
                  wqz = wz - qz
                  qcx = qx -ax
                  qcy = qy -ay
                  qcz = qz -az

                  rkp = 1.d0/(2.d0*zpq)

                  px0 = pax*s0 + wpx*s1
                  py0 = pay*s0 + wpy*s1
                  pz0 = paz*s0 + wpz*s1

                  px1 = pax*s1 + wpx*s2
                  py1 = pay*s1 + wpy*s2
                  pz1 = paz*s1 + wpz*s2

                  bb = za(i)*za(k)*4.d0

                  psps(1) = psps(1)+bb*(qcx*px0 + wqx*px1 + rkp*s1)
                  psps(2) = psps(2)+bb*(qcx*py0 + wqx*py1)
                  psps(3) = psps(3)+bb*(qcx*pz0 + wqx*pz1)
                  psps(4) = psps(4)+bb*(qcy*px0 + wqy*px1)
                  psps(5) = psps(5)+bb*(qcy*py0 + wqy*py1 + rkp*s1)
                  psps(6) = psps(6)+bb*(qcy*pz0 + wqy*pz1)
                  psps(7) = psps(7)+bb*(qcz*px0 + wqz*px1)
                  psps(8) = psps(8)+bb*(qcz*py0 + wqz*py1)
                  psps(9) = psps(9)+bb*(qcz*pz0 + wqz*pz1 + rkp*s1)


               enddo
            enddo
         enddo
      enddo

      do i=1,9
         psps(i) = dabs(psps(i))
      enddo

      ddmax = maxval(psps)



      end
! JCUTPS ============================================================
! CALUCULATE (R'S|R'S)-TYPE INTEGRALS FOR SCHWARTZ INEQUALITY
! 
! (p's|p's)
! ===================================================================
 
      subroutine jcutps(
     &   a,b,
     &   nfsz,nesz,fdat,edat,
     &   za,zb,
     &   ca,cb,
     &   ncnta,ncntb,ddmax    )

      implicit real(8)(a-h,o-z)
      
      parameter (pi52 = 17.4934183276248628462d0)
      dimension fdat(nfsz,nesz),edat(nesz)
      dimension rminv(10),tf(25)
      dimension za(ncnta),zb(ncntb)
      dimension ca(ncnta),cb(ncntb)
      dimension a(3),b(3),dsds(36),s(1),d1(6),d2(6),psps(81)

      data pi/3.14159265358979323846d+00 /
      data      (tf(i),i=1,25) /
     &                33.0,37.0,41.0,43.0,46.0,49.0,
     &                51.0,54.0,56.0,58.0,61.0,63.0,
     &                66.0,68.0,70.0,72.0,74.0,80.0,
     &                85.0,90.0,95.0,95.0,98.0,99.0,
     &                99.0                           /
c
      data      (rminv(i),i=1,10) /
     &   1.0d+00,  0.333333333333333333d+00,  0.20d+00,
     &   0.14285714285714286d+00,  0.11111111111111111d+00 ,
     &   0.90909090909090909d-01,  0.76923076923076923d-01 ,
     &   0.66666666666666666d-01,  0.58823529411764705d-01 ,
     &   0.52631578947368421d-01 /
c
      data  ga0   / 1.0d+00 /
      data  ga1   / 1.000 000 000 000 005 298d+00 /
      data  ga2   / 0.5d+00 /
      data  ga3   / 0.166 666 666 598 849 826d+00 /
      data  ga4   / 0.416 666 666 666 666 667d-01 /
      data  ga5   / 0.833 355 034 722 222 222d-02 /
      data  ga6   / 0.138 888 888 888 888 889d-02 /
      ax = a(1)
      ay = a(2)
      az = a(3)
      bx = b(1)
      by = b(2)
      bz = b(3)

      do i=1,6
         d1(i) = 0.d0
         d2(i) = 0.d0
      enddo
      do i=1,36
         dsds(i) = 0.d0
      enddo
       do i=1,81
         psps(i) = 0.d0
      enddo
      s(1) = 0.d0

      bax = bx - ax
      bay = by - ay
      baz = bz - az

      abxyz = bax*bax + bay*bay + baz*baz
      pi52_2 = 2.d0 * pi52

      do i=1,ncnta
         do j=1,ncntb

            zet   = za(i) + zb(j)
            roop  = za(i)*zb(j) / zet
            gp    = dexp(-roop*abxyz)
            rambdap = pi52_2 /zet
            contp = ca(i)*cb(j)

            px = (za(i)*ax + zb(j)*bx)/zet
            py = (za(i)*ay + zb(j)*by)/zet
            pz = (za(i)*az + zb(j)*bz)/zet

            do k=1,ncnta
               do l=1,ncntb
            eta   = za(k) + zb(l)
            zpq   = zet + eta
            rooq  = za(k)*zb(l) / eta
            gq    = dexp(-rooq*abxyz)
            rambda= rambdap/(eta*dsqrt(zpq))
            contpq = ca(k)*cb(l) * contp

            qx = (za(k)*ax + zb(l)*bx)/eta
            qy = (za(k)*ay + zb(l)*by)/eta
            qz = (za(k)*az + zb(l)*bz)/eta

            wx = (zet*px + eta*qx)/zpq
            wy = (zet*py + eta*qy)/zpq
            wz = (zet*pz + eta*qz)/zpq

            roo = zet*eta /zpq
            t   = roo*((px-qx)**2+(py-qy)**2+(pz-qz)**2)
  
                  if (t. le .tf(5)) then
     
                     it = idint((t+0.0251d+00)*20.0d+00)
                     dt = 0.05d+00*it - t
                     it = it + 1
                     dt4 = dt*0.25d+00
                     dt5 = dt*0.2d+00
                     dt6 = dt*0.166666666666666667d+00
                     dt2 = dt4 + dt4
                     dt3 = dt6 + dt6
                     f4  = (((((fdat(11,it)*dt6   + fdat(10,it))*dt5
     &                    + fdat(9,it))*dt4  + fdat(8,it))*dt3
     &                    + fdat(7,it))*dt2  + fdat(6,it))*dt
     &                    + fdat(5,it)
c     
                     eed  = (((((ga6*dt + ga5)*dt + ga4)*dt
     &                    + ga3)*dt + ga2)*dt + ga1)*dt + ga0
                     ee   = edat(it)*eed
                     t2   = 2.0d+00*t

                     f3   = (t2*f4 + ee)*rminv(4)
                     f2   = (t2*f3 + ee)*rminv(3)
                     f1   = (t2*f2 + ee)*rminv(2)
                     f0   = t2*f1 + ee
                  else
                     tinv = 1.0d+00/t
                     f0   = dsqrt(pi*tinv)*0.5d+00
                     f1   = 0.5d+00*f0*tinv
                     f2   = 1.5d+00*f1*tinv
                     f3   = 2.5d+00*f2*tinv
                     f4   = 3.5d+00*f3*tinv
                  end if
 
                  s0 = contpq*gp*gq*rambda*f0
                  s1 = contpq*gp*gq*rambda*f1
                  s2 = contpq*gp*gq*rambda*f2
                  s3 = contpq*gp*gq*rambda*f3
                  s4 = contpq*gp*gq*rambda*f4
!==
                  s(1) = s(1) + s0

                  wpx = wx - px
                  wpy = wy - py
                  wpz = wz - pz
                  pax = px -ax
                  pay = py -ay
                  paz = pz -az

                  wqx = wx - qx
                  wqy = wy - qy
                  wqz = wz - qz
                  qcx = qx -ax
                  qcy = qy -ay
                  qcz = qz -az

                  rkp = 1.d0/(2.d0*zpq)

                  px0 = pax*s0 + wpx*s1
                  py0 = pay*s0 + wpy*s1
                  pz0 = paz*s0 + wpz*s1

                  px1 = pax*s1 + wpx*s2
                  py1 = pay*s1 + wpy*s2
                  pz1 = paz*s1 + wpz*s2

                  px2 = pax*s2 + wpx*s3
                  py2 = pay*s2 + wpy*s3
                  pz2 = paz*s2 + wpz*s3

                  px3 = pax*s3 + wpx*s4
                  py3 = pay*s3 + wpy*s4
                  pz3 = paz*s3 + wpz*s4

                  rooz = roo/zet
                  rooe = roo/eta
                  rzet = 0.5d0/zet
                  reta = 0.5d0/eta

                  dxx0 = pax*px0 + wpx*px1 + rzet*(s0 - rooz*s1)
                  dyy0 = pay*py0 + wpy*py1 + rzet*(s0 - rooz*s1)
                  dzz0 = paz*pz0 + wpz*pz1 + rzet*(s0 - rooz*s1)
                  dyx0 = pay*px0 + wpy*px1
                  dzx0 = paz*px0 + wpz*px1
                  dzy0 = paz*py0 + wpz*py1

                  sspsx0=qcx*s0 + wqx*s1
                  sspsy0=qcy*s0 + wqy*s1
                  sspsz0=qcz*s0 + wqz*s1

                  sspsx1=qcx*s1 + wqx*s2
                  sspsy1=qcy*s1 + wqy*s2
                  sspsz1=qcz*s1 + wqz*s2

                  ssdxxs = qcx*sspsx0+wqx*sspsx1+reta*(s0-rooe*s1)
                  ssdyys = qcy*sspsy0+wqy*sspsy1+reta*(s0-rooe*s1)
                  ssdzzs = qcz*sspsz0+wqz*sspsz1+reta*(s0-rooe*s1)
                  ssdyxs = qcy*sspsx0+wqy*sspsx1     
                  ssdzxs = qcz*sspsx0+wqz*sspsx1     
                  ssdzys = qcz*sspsy0+wqz*sspsy1     


                  d1(1) = d1(1) + 2.d0*za(i)*dxx0
                  d1(2) = d1(2) + 2.d0*za(i)*dyy0
                  d1(3) = d1(3) + 2.d0*za(i)*dzz0
                  d1(4) = d1(4) + 2.d0*za(i)*dyx0
                  d1(5) = d1(5) + 2.d0*za(i)*dzx0
                  d1(6) = d1(6) + 2.d0*za(i)*dzy0

                  d2(1) = d2(1) + 2.d0*za(k)*ssdxxs
                  d2(2) = d2(2) + 2.d0*za(k)*ssdyys
                  d2(3) = d2(3) + 2.d0*za(k)*ssdzzs
                  d2(4) = d2(4) + 2.d0*za(k)*ssdyxs
                  d2(5) = d2(5) + 2.d0*za(k)*ssdzxs
                  d2(6) = d2(6) + 2.d0*za(k)*ssdzys

                  dxx1 = pax*px1 + wpx*px2 + rzet*(s1 - rooz*s2)
                  dyy1 = pay*py1 + wpy*py2 + rzet*(s1 - rooz*s2)
                  dzz1 = paz*pz1 + wpz*pz2 + rzet*(s1 - rooz*s2)
                  dyx1 = pay*px1 + wpy*px2
                  dzx1 = paz*px1 + wpz*px2
                  dzy1 = paz*py1 + wpz*py2

                  dxx2 = pax*px2 + wpx*px3 + rzet*(s2 - rooz*s3)
                  dyy2 = pay*py2 + wpy*py3 + rzet*(s2 - rooz*s3)
                  dzz2 = paz*pz2 + wpz*pz3 + rzet*(s2 - rooz*s3)
                  dyx2 = pay*px2 + wpy*px3
                  dzx2 = paz*px2 + wpz*px3
                  dzy2 = paz*py2 + wpz*py3

                  dxxpx0 = qcx*dxx0 + wqx*dxx1 +2.d0*rkp*px1
                  dxxpy0 = qcy*dxx0 + wqy*dxx1
                  dxxpz0 = qcz*dxx0 + wqz*dxx1
                  dyypx0 = qcx*dyy0 + wqx*dyy1
                  dyypy0 = qcy*dyy0 + wqy*dyy1 +2.d0*rkp*py1
                  dyypz0 = qcz*dyy0 + wqz*dyy1
                  dzzpx0 = qcx*dzz0 + wqx*dzz1
                  dzzpy0 = qcy*dzz0 + wqy*dzz1
                  dzzpz0 = qcz*dzz0 + wqz*dzz1 +2.d0*rkp*pz1
                  dyxpx0 = qcx*dyx0 + wqx*dyx1 +     rkp*py1
                  dyxpy0 = qcy*dyx0 + wqy*dyx1 +     rkp*px1
                  dyxpz0 = qcz*dyx0 + wqz*dyx1
                  dzxpx0 = qcx*dzx0 + wqx*dzx1 +     rkp*pz1
                  dzxpy0 = qcy*dzx0 + wqy*dzx1
                  dzxpz0 = qcz*dzx0 + wqz*dzx1 +     rkp*px1
                  dzypx0 = qcx*dzy0 + wqx*dzy1
                  dzypy0 = qcy*dzy0 + wqy*dzy1 +     rkp*pz1
                  dzypz0 = qcz*dzy0 + wqz*dzy1 +     rkp*py1

                  dxxpx1 = qcx*dxx1 + wqx*dxx2 +2.d0*rkp*px2
                  dxxpy1 = qcy*dxx1 + wqy*dxx2
                  dxxpz1 = qcz*dxx1 + wqz*dxx2
                  dyypx1 = qcx*dyy1 + wqx*dyy2
                  dyypy1 = qcy*dyy1 + wqy*dyy2 +2.d0*rkp*py2
                  dyypz1 = qcz*dyy1 + wqz*dyy2
                  dzzpx1 = qcx*dzz1 + wqx*dzz2
                  dzzpy1 = qcy*dzz1 + wqy*dzz2
                  dzzpz1 = qcz*dzz1 + wqz*dzz2 +2.d0*rkp*pz2
                  dyxpx1 = qcx*dyx1 + wqx*dyx2 +     rkp*py2
                  dyxpy1 = qcy*dyx1 + wqy*dyx2 +     rkp*px2
                  dyxpz1 = qcz*dyx1 + wqz*dyx2
                  dzxpx1 = qcx*dzx1 + wqx*dzx2 +     rkp*pz2
                  dzxpy1 = qcy*dzx1 + wqy*dzx2
                  dzxpz1 = qcz*dzx1 + wqz*dzx2 +     rkp*px2
                  dzypx1 = qcx*dzy1 + wqx*dzy2
                  dzypy1 = qcy*dzy1 + wqy*dzy2 +     rkp*pz2
                  dzypz1 = qcz*dzy1 + wqz*dzy2 +     rkp*py2

                  rooe = roo/eta
                  reta = 0.5d0/eta
                 
                  xx1=qcx*px1 + wqx*px2 + rkp*s2
                  xy1=qcy*px1 + wqy*px2
                  xz1=qcz*px1 + wqz*px2
                  yx1=qcx*py1 + wqx*py2
                  yy1=qcy*py1 + wqy*py2 + rkp*s2
                  yz1=qcz*py1 + wqz*py2
                  zx1=qcx*pz1 + wqx*pz2
                  zy1=qcy*pz1 + wqy*pz2
                  zz1=qcz*pz1 + wqz*pz2 + rkp*s2

                  dxx01 = reta*(dxx0 - rooe*dxx1)
                  dyy01 = reta*(dyy0 - rooe*dyy1)
                  dzz01 = reta*(dzz0 - rooe*dzz1)
                  dyx01 = reta*(dyx0 - rooe*dyx1)
                  dzx01 = reta*(dzx0 - rooe*dzx1)
                  dzy01 = reta*(dzy0 - rooe*dzy1)

                  dxxdxx=qcx*dxxpx0+wqx*dxxpx1+2.d0*rkp*xx1 + dxx01
                  dxxdyy=qcy*dxxpy0+wqy*dxxpy1              + dxx01
                  dxxdzz=qcz*dxxpz0+wqz*dxxpz1              + dxx01
                  dxxdyx=qcy*dxxpx0+wqy*dxxpx1
                  dxxdzx=qcz*dxxpx0+wqz*dxxpx1
                  dxxdzy=qcz*dxxpy0+wqz*dxxpy1

                  dyydxx=qcx*dyypx0+wqx*dyypx1              + dyy01
                  dyydyy=qcy*dyypy0+wqy*dyypy1+2.d0*rkp*yy1 + dyy01
                  dyydzz=qcz*dyypz0+wqz*dyypz1              + dyy01
                  dyydyx=qcx*dyypy0+wqx*dyypy1
                  dyydzx=qcz*dyypx0+wqz*dyypx1
                  dyydzy=qcz*dyypy0+wqz*dyypy1

                  dzzdxx=qcx*dzzpx0+wqx*dzzpx1              + dzz01
                  dzzdyy=qcy*dzzpy0+wqy*dzzpy1              + dzz01
                  dzzdzz=qcz*dzzpz0+wqz*dzzpz1+2.d0*rkp*zz1 + dzz01
                  dzzdyx=qcx*dzzpy0+wqx*dzzpy1
                  dzzdzx=qcx*dzzpz0+wqx*dzzpz1
                  dzzdzy=qcy*dzzpz0+wqy*dzzpz1

                  dyxdxx=qcx*dyxpx0+wqx*dyxpx1+     rkp*yx1 + dyx01
                  dyxdyy=qcy*dyxpy0+wqy*dyxpy1+     rkp*xy1 + dyx01
                  dyxdzz=qcz*dyxpz0+wqz*dyxpz1              + dyx01
                  dyxdyx=qcy*dyxpx0+wqy*dyxpx1+     rkp*xx1
                  dyxdzx=qcz*dyxpx0+wqz*dyxpx1
                  dyxdzy=qcz*dyxpy0+wqz*dyxpy1

                  dzxdxx=qcx*dzxpx0+wqx*dzxpx1+     rkp*zx1 + dzx01
                  dzxdyy=qcy*dzxpy0+wqy*dzxpy1              + dzx01
                  dzxdzz=qcz*dzxpz0+wqz*dzxpz1+     rkp*xz1 + dzx01
                  dzxdyx=qcy*dzxpx0+wqy*dzxpx1
                  dzxdzx=qcz*dzxpx0+wqz*dzxpx1+     rkp*xx1
                  dzxdzy=qcy*dzxpz0+wqy*dzxpz1

                  dzydxx=qcx*dzypx0+wqx*dzypx1              + dzy01
                  dzydyy=qcy*dzypy0+wqy*dzypy1+     rkp*zy1 + dzy01
                  dzydzz=qcz*dzypz0+wqz*dzypz1+     rkp*yz1 + dzy01
                  dzydyx=qcx*dzypy0+wqx*dzypy1
                  dzydzx=qcx*dzypz0+wqx*dzypz1
                  dzydzy=qcy*dzypz0+wqy*dzypz1+     rkp*zz1


                  bb = za(i)*za(k)*4.d0
                  dsds( 1) = dsds( 1)+ bb*dxxdxx
                  dsds( 2) = dsds( 2)+ bb*dxxdyy
                  dsds( 3) = dsds( 3)+ bb*dxxdzz
                  dsds( 4) = dsds( 4)+ bb*dxxdyx
                  dsds( 5) = dsds( 5)+ bb*dxxdzx
                  dsds( 6) = dsds( 6)+ bb*dxxdzy
                  dsds( 7) = dsds( 7)+ bb*dyydxx
                  dsds( 8) = dsds( 8)+ bb*dyydyy
                  dsds( 9) = dsds( 9)+ bb*dyydzz
                  dsds(10) = dsds(10)+ bb*dyydyx
                  dsds(11) = dsds(11)+ bb*dyydzx
                  dsds(12) = dsds(12)+ bb*dyydzy
                  dsds(13) = dsds(13)+ bb*dzzdxx
                  dsds(14) = dsds(14)+ bb*dzzdyy
                  dsds(15) = dsds(15)+ bb*dzzdzz
                  dsds(16) = dsds(16)+ bb*dzzdyx
                  dsds(17) = dsds(17)+ bb*dzzdzx
                  dsds(18) = dsds(18)+ bb*dzzdzy
                  dsds(19) = dsds(19)+ bb*dyxdxx
                  dsds(20) = dsds(20)+ bb*dyxdyy
                  dsds(21) = dsds(21)+ bb*dyxdzz
                  dsds(22) = dsds(22)+ bb*dyxdyx
                  dsds(23) = dsds(23)+ bb*dyxdzx
                  dsds(24) = dsds(24)+ bb*dyxdzy
                  dsds(25) = dsds(25)+ bb*dzxdxx
                  dsds(26) = dsds(26)+ bb*dzxdyy
                  dsds(27) = dsds(27)+ bb*dzxdzz
                  dsds(28) = dsds(28)+ bb*dzxdyx
                  dsds(29) = dsds(29)+ bb*dzxdzx
                  dsds(30) = dsds(30)+ bb*dzxdzy
                  dsds(31) = dsds(31)+ bb*dzydxx
                  dsds(32) = dsds(32)+ bb*dzydyy
                  dsds(33) = dsds(33)+ bb*dzydzz
                  dsds(34) = dsds(34)+ bb*dzydyx
                  dsds(35) = dsds(35)+ bb*dzydzx
                  dsds(36) = dsds(36)+ bb*dzydzy

               enddo
            enddo
         enddo
      enddo


      psps( 1) = dsds( 1) - d1(1) - d2(1) + s(1)
      psps( 2) = dsds( 4)         - d2(4)
      psps( 3) = dsds( 5)         - d2(5)
      psps( 4) = dsds(19) - d1(4)        
      psps( 5) = dsds(22)
      psps( 6) = dsds(23)        
      psps( 7) = dsds(25) - d1(5)        
      psps( 8) = dsds(28)        
      psps( 9) = dsds(29)

      psps(10) = dsds( 4)         - d2(4)
      psps(11) = dsds( 2) - d1(1) - d2(2) + s(1)
      psps(12) = dsds( 6)         - d2(6)
      psps(13) = dsds(22)
      psps(14) = dsds(20) - d1(4)
      psps(15) = dsds(24)
      psps(16) = dsds(28)
      psps(17) = dsds(26) - d1(5)
      psps(18) = dsds(30)

      psps(19) = dsds( 5)         - d2(5)
      psps(20) = dsds( 6)         - d2(6)
      psps(21) = dsds( 3) - d1(1) - d2(3) + s(1)
      psps(22) = dsds(23)
      psps(23) = dsds(24)
      psps(24) = dsds(21) - d1(4)
      psps(25) = dsds(29)
      psps(26) = dsds(30)
      psps(27) = dsds(27) - d1(5)
!
      psps(28) = dsds(19) - d1(4)
      psps(29) = dsds(22)        
      psps(30) = dsds(24)        
      psps(31) = dsds( 7) - d1(2) - d2(1) + s(1)
      psps(32) = dsds(10)         - d2(4)
      psps(33) = dsds(11)         - d2(5)
      psps(34) = dsds(31) - d1(6)        
      psps(35) = dsds(34)                
      psps(36) = dsds(35)        
                                         
      psps(37) = dsds(22)        
      psps(38) = dsds(20) - d1(4)        
      psps(39) = dsds(24)                
      psps(40) = dsds(10)         - d2(4)
      psps(41) = dsds( 8) - d1(2) - d2(2) + s(1)
      psps(42) = dsds(12)         - d2(6)
      psps(43) = dsds(34)                
      psps(44) = dsds(32) - d1(6)
      psps(45) = dsds(36)                
                                         
      psps(46) = dsds(23)        
      psps(47) = dsds(24)                
      psps(48) = dsds(21) - d1(4)        
      psps(49) = dsds(11)         - d2(5)
      psps(50) = dsds(12)         - d2(6)
      psps(51) = dsds( 9) - d1(2) - d2(3) + s(1)
      psps(52) = dsds(35)
      psps(53) = dsds(36)
      psps(54) = dsds(33) - d1(6)
!
      psps(55) = dsds(25) - d1(5)
      psps(56) = dsds(28)        
      psps(57) = dsds(29)        
      psps(58) = dsds(31) - d1(6)
      psps(59) = dsds(34)
      psps(60) = dsds(35)        
      psps(61) = dsds(13) - d1(3) - d2(1) + s(1)
      psps(62) = dsds(16)         - d2(4)
      psps(63) = dsds(17)         - d2(5)
                                         
      psps(64) = dsds(28)                
      psps(65) = dsds(26) - d1(5)
      psps(66) = dsds(30)                
      psps(67) = dsds(34)        
      psps(68) = dsds(32) - d1(6)        
      psps(69) = dsds(36)                
      psps(70) = dsds(16)         - d2(4)
      psps(71) = dsds(14) - d1(3) - d2(2) + s(1)
      psps(72) = dsds(18)         - d2(6)
                                         
      psps(73) = dsds(29)        
      psps(74) = dsds(30)                
      psps(75) = dsds(27) - d1(5)        
      psps(76) = dsds(35)        
      psps(77) = dsds(36)                
      psps(78) = dsds(33) - d1(6)        
      psps(79) = dsds(17)         - d2(5)
      psps(80) = dsds(18)         - d2(6)
      psps(81) = dsds(15) - d1(3) - d2(3) + s(1)


       dsds( 1)  = dsds( 1)- d1(1) - d2(1)+ s(1)
       dsds( 2)  = dsds( 2)- d1(1) - d2(2)+ s(1)
       dsds( 3)  = dsds( 3)- d1(1) - d2(3)+ s(1)
       dsds( 4)  = dsds( 4)        - d2(4)
       dsds( 5)  = dsds( 5)        - d2(5)
       dsds( 6)  = dsds( 6)        - d2(6)
       dsds( 7)  = dsds( 7)- d1(2) - d2(1)+ s(1)
       dsds( 8)  = dsds( 8)- d1(2) - d2(2)+ s(1)
       dsds( 9)  = dsds( 9)- d1(2) - d2(3)+ s(1)
       dsds(10)  = dsds(10)        - d2(4)
       dsds(11)  = dsds(11)        - d2(5)
       dsds(12)  = dsds(12)        - d2(6)
       dsds(13)  = dsds(13)- d1(3) - d2(1)+ s(1)
       dsds(14)  = dsds(14)- d1(3) - d2(2)+ s(1)
       dsds(15)  = dsds(15)- d1(3) - d2(3)+ s(1)
       dsds(16)  = dsds(16)        - d2(4)
       dsds(17)  = dsds(17)        - d2(5)
       dsds(18)  = dsds(18)        - d2(6)
       dsds(19)  = dsds(19)- d1(4)
       dsds(20)  = dsds(20)- d1(4)
       dsds(21)  = dsds(21)- d1(4)
       dsds(22)  = dsds(22)
       dsds(23)  = dsds(23)
       dsds(24)  = dsds(24)
       dsds(25)  = dsds(25)- d1(5)
       dsds(26)  = dsds(26)- d1(5)
       dsds(27)  = dsds(27)- d1(5)
       dsds(28)  = dsds(28)
       dsds(29)  = dsds(29)
       dsds(30)  = dsds(30)
       dsds(31)  = dsds(31)- d1(6)
       dsds(32)  = dsds(32)- d1(6)
       dsds(33)  = dsds(33)- d1(6)
       dsds(34)  = dsds(34)
       dsds(35)  = dsds(35)
       dsds(36)  = dsds(36)


       do  i=1,36
           dsds(i) = dabs(dsds(i))
       enddo


       ddmax = maxval(dsds)
       ddmax0= maxval(psps)

      end
! JCUTSP ============================================================
! CALUCULATE (R'S|R'S)-TYPE INTEGRALS FOR SCHWARTZ INEQUALITY
! 
! (sp'|sp')
! ===================================================================
       subroutine jcutsp(
     &   a,b,
     &   nfsz,nesz,fdat,edat,
     &   za,zb,
     &   ca,cb,
     &   ncnta,ncntb,ddmax    )

      implicit real(8)(a-h,o-z)
      
      parameter (pi52 = 17.4934183276248628462d0)
      dimension fdat(nfsz,nesz),edat(nesz)
      dimension rminv(10),tf(25)
      dimension za(ncnta),zb(ncntb)
      dimension ca(ncnta),cb(ncntb)
      dimension a(3),b(3),pppp(81)

      data pi/3.14159265358979323846d+00 /
      data      (tf(i),i=1,25) /
     &                33.0,37.0,41.0,43.0,46.0,49.0,
     &                51.0,54.0,56.0,58.0,61.0,63.0,
     &                66.0,68.0,70.0,72.0,74.0,80.0,
     &                85.0,90.0,95.0,95.0,98.0,99.0,
     &                99.0                           /
c
      data      (rminv(i),i=1,10) /
     &   1.0d+00,  0.333333333333333333d+00,  0.20d+00,
     &   0.14285714285714286d+00,  0.11111111111111111d+00 ,
     &   0.90909090909090909d-01,  0.76923076923076923d-01 ,
     &   0.66666666666666666d-01,  0.58823529411764705d-01 ,
     &   0.52631578947368421d-01 /
c
      data  ga0   / 1.0d+00 /
      data  ga1   / 1.000 000 000 000 005 298d+00 /
      data  ga2   / 0.5d+00 /
      data  ga3   / 0.166 666 666 598 849 826d+00 /
      data  ga4   / 0.416 666 666 666 666 667d-01 /
      data  ga5   / 0.833 355 034 722 222 222d-02 /
      data  ga6   / 0.138 888 888 888 888 889d-02 /

      ax = a(1)
      ay = a(2)
      az = a(3)
      bx = b(1)
      by = b(2)
      bz = b(3)

      bax = bx - ax
      bay = by - ay
      baz = bz - az

      do i=1,81
        pppp(i)=0.d0
      enddo

      abxyz = bax*bax + bay*bay + baz*baz
      pi52_2 = 2.d0 * pi52

      do i=1,ncnta
         do j=1,ncntb

            zet   = za(i) + zb(j)
            roop  = za(i)*zb(j) / zet
            gp    = dexp(-roop*abxyz)
            rambdap = pi52_2 /zet
            contp = ca(i)*cb(j)

            px = (za(i)*ax + zb(j)*bx)/zet
            py = (za(i)*ay + zb(j)*by)/zet
            pz = (za(i)*az + zb(j)*bz)/zet

            pbx= px-bx
            pby= py-by
            pbz= pz-bz

            do k=1,ncnta
               do l=1,ncntb
            eta   = za(k) + zb(l)
            zpq   = zet + eta
            rooq  = za(k)*zb(l) / eta
            gq    = dexp(-rooq*abxyz)
            rambda= rambdap/(eta*dsqrt(zpq))
            contpq = ca(k)*cb(l) * contp

            qx = (za(k)*ax + zb(l)*bx)/eta
            qy = (za(k)*ay + zb(l)*by)/eta
            qz = (za(k)*az + zb(l)*bz)/eta
            qdx= qx-bx
            qdy= qy-by
            qdz= qz-bz


            wx = (zet*px + eta*qx)/zpq
            wy = (zet*py + eta*qy)/zpq
            wz = (zet*pz + eta*qz)/zpq

            roo = zet*eta /zpq
            t   = roo*((px-qx)**2+(py-qy)**2+(pz-qz)**2)
  
                  if (t. le .tf(5)) then
     
                     it = idint((t+0.0251d+00)*20.0d+00)
                     dt = 0.05d+00*it - t
                     it = it + 1
                     dt4 = dt*0.25d+00
                     dt5 = dt*0.2d+00
                     dt6 = dt*0.166666666666666667d+00
                     dt2 = dt4 + dt4
                     dt3 = dt6 + dt6
                     f4  = (((((fdat(11,it)*dt6   + fdat(10,it))*dt5
     &                    + fdat(9,it))*dt4  + fdat(8,it))*dt3
     &                    + fdat(7,it))*dt2  + fdat(6,it))*dt
     &                    + fdat(5,it)
c     
                     eed  = (((((ga6*dt + ga5)*dt + ga4)*dt
     &                    + ga3)*dt + ga2)*dt + ga1)*dt + ga0
                     ee   = edat(it)*eed
                     t2   = 2.0d+00*t

                     f3   = (t2*f4 + ee)*rminv(4)
                     f2   = (t2*f3 + ee)*rminv(3)
                     f1   = (t2*f2 + ee)*rminv(2)
                     f0   = t2*f1 + ee
                  else
                     tinv = 1.0d+00/t
                     f0   = dsqrt(pi*tinv)*0.5d+00
                     f1   = 0.5d+00*f0*tinv
                     f2   = 1.5d+00*f1*tinv
                     f3   = 2.5d+00*f2*tinv
                     f4   = 3.5d+00*f3*tinv
                  end if
 
                  s0 = contpq*gp*gq*rambda*f0
                  s1 = contpq*gp*gq*rambda*f1
                  s2 = contpq*gp*gq*rambda*f2
                  s3 = contpq*gp*gq*rambda*f3
                  s4 = contpq*gp*gq*rambda*f4
!==
                  wpx = wx - px
                  wpy = wy - py
                  wpz = wz - pz
                  pax = px -ax
                  pay = py -ay
                  paz = pz -az

                  wqx = wx - qx
                  wqy = wy - qy
                  wqz = wz - qz
                  qcx = qx -ax
                  qcy = qy -ay
                  qcz = qz -az

                  rkp = 1.d0/(2.d0*zpq)

                  px0 = pax*s0 + wpx*s1
                  py0 = pay*s0 + wpy*s1
                  pz0 = paz*s0 + wpz*s1

                  px1 = pax*s1 + wpx*s2
                  py1 = pay*s1 + wpy*s2
                  pz1 = paz*s1 + wpz*s2

                  px2 = pax*s2 + wpx*s3
                  py2 = pay*s2 + wpy*s3
                  pz2 = paz*s2 + wpz*s3

                  px3 = pax*s3 + wpx*s4
                  py3 = pay*s3 + wpy*s4
                  pz3 = paz*s3 + wpz*s4

                  rooz = roo/zet
                  rooe = roo/eta
                  rzet = 0.5d0/zet
                  reta = 0.5d0/eta

                  ppxx0 = pbx*px0 + wpx*px1 + rzet*(s0 - rooz*s1)
                  ppxy0 = pby*px0 + wpy*px1
                  ppxz0 = pbz*px0 + wpz*px1
                  ppyx0 = pbx*py0 + wpx*py1
                  ppyy0 = pby*py0 + wpy*py1 + rzet*(s0 - rooz*s1)
                  ppyz0 = pbz*py0 + wpz*py1
                  ppzx0 = pbx*pz0 + wpx*pz1
                  ppzy0 = pby*pz0 + wpy*pz1
                  ppzz0 = pbz*pz0 + wpz*pz1 + rzet*(s0 - rooz*s1)
                  ppxx1 = pbx*px1 + wpx*px2 + rzet*(s1 - rooz*s2)
                  ppxy1 = pby*px1 + wpy*px2
                  ppxz1 = pbz*px1 + wpz*px2
                  ppyx1 = pbx*py1 + wpx*py2
                  ppyy1 = pby*py1 + wpy*py2 + rzet*(s1 - rooz*s2)
                  ppyz1 = pbz*py1 + wpz*py2
                  ppzx1 = pbx*pz1 + wpx*pz2
                  ppzy1 = pby*pz1 + wpy*pz2
                  ppzz1 = pbz*pz1 + wpz*pz2 + rzet*(s1 - rooz*s2)
                  ppxx2 = pbx*px2 + wpx*px3 + rzet*(s2 - rooz*s3)
                  ppxy2 = pby*px2 + wpy*px3
                  ppxz2 = pbz*px2 + wpz*px3
                  ppyx2 = pbx*py2 + wpx*py3
                  ppyy2 = pby*py2 + wpy*py3 + rzet*(s2 - rooz*s3)
                  ppyz2 = pbz*py2 + wpz*py3
                  ppzx2 = pbx*pz2 + wpx*pz3
                  ppzy2 = pby*pz2 + wpy*pz3
                  ppzz2 = pbz*pz2 + wpz*pz3 + rzet*(s2 - rooz*s3)
                  
!                 rkp = rooz * eta
C     SPSS
                  xp1 = pbx*s1 +wpx*s2
                  yp1 = pby*s1 +wpy*s2
                  zp1 = pbz*s1 +wpz*s2
                  xp2 = pbx*s2 +wpx*s3
                  yp2 = pby*s2 +wpy*s3
                  zp2 = pbz*s2 +wpz*s3
                  
                  xxx0= qcx*ppxx0 + wqx*ppxx1 +rkp*(px1+xp1)
                  xxy0= qcy*ppxx0 + wqy*ppxx1
                  xxz0= qcz*ppxx0 + wqz*ppxx1
                  xyx0= qcx*ppxy0 + wqx*ppxy1 +rkp*yp1
                  xyy0= qcy*ppxy0 + wqy*ppxy1 +rkp*px1
                  xyz0= qcz*ppxy0 + wqz*ppxy1
                  xzx0= qcx*ppxz0 + wqx*ppxz1 +rkp*zp1
                  xzy0= qcy*ppxz0 + wqy*ppxz1
                  xzz0= qcz*ppxz0 + wqz*ppxz1 +rkp*px1
                  yxx0= qcx*ppyx0 + wqx*ppyx1 +rkp*py1
                  yxy0= qcy*ppyx0 + wqy*ppyx1 +rkp*xp1
                  yxz0= qcz*ppyx0 + wqz*ppyx1
                  yyx0= qcx*ppyy0 + wqx*ppyy1
                  yyy0= qcy*ppyy0 + wqy*ppyy1 +rkp*(py1+yp1)
                  yyz0= qcz*ppyy0 + wqz*ppyy1 
                  yzx0= qcx*ppyz0 + wqx*ppyz1
                  yzy0= qcy*ppyz0 + wqy*ppyz1 +rkp*zp1
                  yzz0= qcz*ppyz0 + wqz*ppyz1 +rkp*py1
                  zxx0= qcx*ppzx0 + wqx*ppzx1 +rkp*pz1
                  zxy0= qcy*ppzx0 + wqy*ppzx1
                  zxz0= qcz*ppzx0 + wqz*ppzx1 +rkp*xp1
                  zyx0= qcx*ppzy0 + wqx*ppzy1
                  zyy0= qcy*ppzy0 + wqy*ppzy1 +rkp*pz1
                  zyz0= qcz*ppzy0 + wqz*ppzy1 +rkp*yp1
                  zzx0= qcx*ppzz0 + wqx*ppzz1
                  zzy0= qcy*ppzz0 + wqy*ppzz1
                  zzz0= qcz*ppzz0 + wqz*ppzz1 +rkp*(pz1+zp1)
                  
                  xxx1= qcx*ppxx1 + wqx*ppxx2 +rkp*(px2+xp2)
                  xxy1= qcy*ppxx1 + wqy*ppxx2
                  xxz1= qcz*ppxx1 + wqz*ppxx2
                  xyx1= qcx*ppxy1 + wqx*ppxy2 +rkp*yp2
                  xyy1= qcy*ppxy1 + wqy*ppxy2 +rkp*px2
                  xyz1= qcz*ppxy1 + wqz*ppxy2
                  xzx1= qcx*ppxz1 + wqx*ppxz2 +rkp*zp2
                  xzy1= qcy*ppxz1 + wqy*ppxz2
                  xzz1= qcz*ppxz1 + wqz*ppxz2 +rkp*px2
                  yxx1= qcx*ppyx1 + wqx*ppyx2 +rkp*py2
                  yxy1= qcy*ppyx1 + wqy*ppyx2 +rkp*xp2
                  yxz1= qcz*ppyx1 + wqz*ppyx2
                  yyx1= qcx*ppyy1 + wqx*ppyy2
                  yyy1= qcy*ppyy1 + wqy*ppyy2 +rkp*(py2+yp2)
                  yyz1= qcz*ppyy1 + wqz*ppyy2 
                  yzx1= qcx*ppyz1 + wqx*ppyz2
                  yzy1= qcy*ppyz1 + wqy*ppyz2 +rkp*zp2
                  yzz1= qcz*ppyz1 + wqz*ppyz2 +rkp*py2
                  zxx1= qcx*ppzx1 + wqx*ppzx2 +rkp*pz2
                  zxy1= qcy*ppzx1 + wqy*ppzx2
                  zxz1= qcz*ppzx1 + wqz*ppzx2 +rkp*xp2
                  zyx1= qcx*ppzy1 + wqx*ppzy2
                  zyy1= qcy*ppzy1 + wqy*ppzy2 +rkp*pz2
                  zyz1= qcz*ppzy1 + wqz*ppzy2 +rkp*yp2
                  zzx1= qcx*ppzz1 + wqx*ppzz2
                  zzy1= qcy*ppzz1 + wqy*ppzz2
                  zzz1= qcz*ppzz1 + wqz*ppzz2 +rkp*(pz2+zp2)
                  
!PP   SS0-PPSS1
                  xx01=reta*(ppxx0 - rooe*ppxx1)
                  xy01=reta*(ppxy0 - rooe*ppxy1)
                  xz01=reta*(ppxz0 - rooe*ppxz1)
                  yx01=reta*(ppyx0 - rooe*ppyx1)
                  yy01=reta*(ppyy0 - rooe*ppyy1)
                  yz01=reta*(ppyz0 - rooe*ppyz1)
                  zx01=reta*(ppzx0 - rooe*ppzx1)
                  zy01=reta*(ppzy0 - rooe*ppzy1)
                  zz01=reta*(ppzz0 - rooe*ppzz1)
!     PSPS
                  pxpx =rkp*(qcx*px1 + wqx*px2 +rkp*s2)
                  pxpy =rkp*(qcy*px1 + wqy*px2        )
                  pxpz =rkp*(qcz*px1 + wqz*px2        )
                  pypx =rkp*(qcx*py1 + wqx*py2        )
                  pypy =rkp*(qcy*py1 + wqy*py2 +rkp*s2)
                  pypz =rkp*(qcz*py1 + wqz*py2        )
                  pzpx =rkp*(qcx*pz1 + wqx*pz2        )
                  pzpy =rkp*(qcy*pz1 + wqy*pz2        )
                  pzpz =rkp*(qcz*pz1 + wqz*pz2 +rkp*s2)
!     SPPS
                  pxxp =rkp*( qcx*xp1 + wqx*xp2 +rkp*s2)
                  pxyp =rkp*( qcy*xp1 + wqy*xp2 )
                  pxzp =rkp*( qcz*xp1 + wqz*xp2 )
                  pyxp =rkp*( qcx*yp1 + wqx*yp2 )
                  pyyp =rkp*( qcy*yp1 + wqy*yp2 +rkp*s2)
                  pyzp =rkp*( qcz*yp1 + wqz*yp2 )
                  pzxp =rkp*( qcx*zp1 + wqx*zp2)
                  pzyp =rkp*( qcy*zp1 + wqy*zp2)
                  pzzp =rkp*( qcz*zp1 + wqz*zp2 +rkp*s2)

                  xxxx = qdx*xxx0+wqx*xxx1+xx01+pxpx+pxxp
                  xxxy = qdy*xxx0+wqy*xxx1         
                  xxxz = qdz*xxx0+wqz*xxx1         
                  xxyx = qdx*xxy0+wqx*xxy1     +pxpy+pxyp
                  xxyy = qdy*xxy0+wqy*xxy1+xx01    
                  xxyz = qdz*xxy0+wqz*xxy1         
                  xxzx = qdx*xxz0+wqx*xxz1     +pxpz+pxzp
                  xxzy = qdy*xxz0+wqy*xxz1         
                  xxzz = qdz*xxz0+wqz*xxz1+xx01    
                  xyxx = qdx*xyx0+wqx*xyx1+xy01     +pyxp
                  xyxy = qdy*xyx0+wqy*xyx1     +pxpx
                  xyxz = qdz*xyx0+wqz*xyx1         
                  xyyx = qdx*xyy0+wqx*xyy1          +pyyp
                  xyyy = qdy*xyy0+wqy*xyy1+xy01+pxpy
                  xyyz = qdz*xyy0+wqz*xyy1   
                  xyzx = qdx*xyz0+wqx*xyz1          +pyzp
                  xyzy = qdy*xyz0+wqy*xyz1     +pxpz
                  xyzz = qdz*xyz0+wqz*xyz1+xy01    
                  xzxx = qdx*xzx0+wqx*xzx1+xz01     +pzxp
                  xzxy = qdy*xzx0+wqy*xzx1         
                  xzxz = qdz*xzx0+wqz*xzx1     +pxpx
                  xzyx = qdx*xzy0+wqx*xzy1          +pzyp
                  xzyy = qdy*xzy0+wqy*xzy1+xz01    
                  xzyz = qdz*xzy0+wqz*xzy1     +pxpy
                  xzzx = qdx*xzz0+wqx*xzz1          +pzzp
                  xzzy = qdy*xzz0+wqy*xzz1         
                  xzzz = qdz*xzz0+wqz*xzz1+xz01+pxpz

                  yxxx = qdx*yxx0+wqx*yxx1+yx01+pypx
                  yxxy = qdy*yxx0+wqy*yxx1          +pxxp
                  yxxz = qdz*yxx0+wqz*yxx1         
                  yxyx = qdx*yxy0+wqx*yxy1     +pypy
                  yxyy = qdy*yxy0+wqy*yxy1+yx01     +pxyp
                  yxyz = qdz*yxy0+wqz*yxy1         
                  yxzx = qdx*yxz0+wqx*yxz1     +pypz
                  yxzy = qdy*yxz0+wqy*yxz1          +pxzp
                  yxzz = qdz*yxz0+wqz*yxz1+yx01    
                  yyxx = qdx*yyx0+wqx*yyx1+yy01    
                  yyxy = qdy*yyx0+wqy*yyx1     +pypx+pyxp
                  yyxz = qdz*yyx0+wqz*yyx1         
                  yyyx = qdx*yyy0+wqx*yyy1         
                  yyyy = qdy*yyy0+wqy*yyy1+yy01+pypy+pyyp
                  yyyz = qdz*yyy0+wqz*yyy1         
                  yyzx = qdx*yyz0+wqx*yyz1         
                  yyzy = qdy*yyz0+wqy*yyz1     +pypz+pyzp
                  yyzz = qdz*yyz0+wqz*yyz1+yy01    
                  yzxx = qdx*yzx0+wqx*yzx1+yz01    
                  yzxy = qdy*yzx0+wqy*yzx1          +pzxp
                  yzxz = qdz*yzx0+wqz*yzx1     +pypx
                  yzyx = qdx*yzy0+wqx*yzy1         
                  yzyy = qdy*yzy0+wqy*yzy1+yz01     +pzyp
                  yzyz = qdz*yzy0+wqz*yzy1     +pypy
                  yzzx = qdx*yzz0+wqx*yzz1
                  yzzy = qdy*yzz0+wqy*yzz1          +pzzp
                  yzzz = qdz*yzz0+wqz*yzz1+yz01+pypz
                  
                  zxxx = qdx*zxx0+wqx*zxx1+zx01+pzpx
                  zxxy = qdy*zxx0+wqy*zxx1         
                  zxxz = qdz*zxx0+wqz*zxx1          +pxxp
                  zxyx = qdx*zxy0+wqx*zxy1     +pzpy
                  zxyy = qdy*zxy0+wqy*zxy1+zx01    
                  zxyz = qdz*zxy0+wqz*zxy1          +pxyp
                  zxzx = qdx*zxz0+wqx*zxz1     +pzpz
                  zxzy = qdy*zxz0+wqy*zxz1         
                  zxzz = qdz*zxz0+wqz*zxz1+zx01     +pxzp
                  zyxx = qdx*zyx0+wqx*zyx1+zy01    
                  zyxy = qdy*zyx0+wqy*zyx1     +pzpx
                  zyxz = qdz*zyx0+wqz*zyx1          +pyxp
                  zyyx = qdx*zyy0+wqx*zyy1         
                  zyyy = qdy*zyy0+wqy*zyy1+zy01+pzpy
                  zyyz = qdz*zyy0+wqz*zyy1          +pyyp
                  zyzx = qdx*zyz0+wqx*zyz1         
                  zyzy = qdy*zyz0+wqy*zyz1     +pzpz
                  zyzz = qdz*zyz0+wqz*zyz1+zy01     +pyzp
                  zzxx = qdx*zzx0+wqx*zzx1+zz01    
                  zzxy = qdy*zzx0+wqy*zzx1         
                  zzxz = qdz*zzx0+wqz*zzx1     +pzpx+pzxp
                  zzyx = qdx*zzy0+wqx*zzy1         
                  zzyy = qdy*zzy0+wqy*zzy1+zz01    
                  zzyz = qdz*zzy0+wqz*zzy1     +pzpy+pzyp
                  zzzx = qdx*zzz0+wqx*zzz1         
                  zzzy = qdy*zzz0+wqy*zzz1         
                  zzzz = qdz*zzz0+wqz*zzz1+zz01+pzpz+pzzp

                  bb = za(i)*za(k)*4.d0
                  PPPP(  1)=PPPP(  1)+bb*xxxx
                  PPPP(  2)=PPPP(  2)+bb*xxxy
                  PPPP(  3)=PPPP(  3)+bb*xxxz
                  PPPP(  4)=PPPP(  4)+bb*xxyx
                  PPPP(  5)=PPPP(  5)+bb*xxyy
                  PPPP(  6)=PPPP(  6)+bb*xxyz
                  PPPP(  7)=PPPP(  7)+bb*xxzx
                  PPPP(  8)=PPPP(  8)+bb*xxzy
                  PPPP(  9)=PPPP(  9)+bb*xxzz
                  PPPP( 10)=PPPP( 10)+bb*xyxx
                  PPPP( 11)=PPPP( 11)+bb*xyxy
                  PPPP( 12)=PPPP( 12)+bb*xyxz
                  PPPP( 13)=PPPP( 13)+bb*xyyx
                  PPPP( 14)=PPPP( 14)+bb*xyyy
                  PPPP( 15)=PPPP( 15)+bb*xyyz
                  PPPP( 16)=PPPP( 16)+bb*xyzx
                  PPPP( 17)=PPPP( 17)+bb*xyzy
                  PPPP( 18)=PPPP( 18)+bb*xyzz
                  PPPP( 19)=PPPP( 19)+bb*xzxx
                  PPPP( 20)=PPPP( 20)+bb*xzxy
                  PPPP( 21)=PPPP( 21)+bb*xzxz
                  PPPP( 22)=PPPP( 22)+bb*xzyx
                  PPPP( 23)=PPPP( 23)+bb*xzyy
                  PPPP( 24)=PPPP( 24)+bb*xzyz
                  PPPP( 25)=PPPP( 25)+bb*xzzx
                  PPPP( 26)=PPPP( 26)+bb*xzzy
                  PPPP( 27)=PPPP( 27)+bb*xzzz
                                             
                  PPPP( 28)=PPPP( 28)+bb*yxxx
                  PPPP( 29)=PPPP( 29)+bb*yxxy
                  PPPP( 30)=PPPP( 30)+bb*yxxz
                  PPPP( 31)=PPPP( 31)+bb*yxyx
                  PPPP( 32)=PPPP( 32)+bb*yxyy
                  PPPP( 33)=PPPP( 33)+bb*yxyz
                  PPPP( 34)=PPPP( 34)+bb*yxzx
                  PPPP( 35)=PPPP( 35)+bb*yxzy
                  PPPP( 36)=PPPP( 36)+bb*yxzz
                  PPPP( 37)=PPPP( 37)+bb*yyxx
                  PPPP( 38)=PPPP( 38)+bb*yyxy
                  PPPP( 39)=PPPP( 39)+bb*yyxz
                  PPPP( 40)=PPPP( 40)+bb*yyyx
                  PPPP( 41)=PPPP( 41)+bb*yyyy
                  PPPP( 42)=PPPP( 42)+bb*yyyz
                  PPPP( 43)=PPPP( 43)+bb*yyzx
                  PPPP( 44)=PPPP( 44)+bb*yyzy
                  PPPP( 45)=PPPP( 45)+bb*yyzz
                  PPPP( 46)=PPPP( 46)+bb*yzxx
                  PPPP( 47)=PPPP( 47)+bb*yzxy
                  PPPP( 48)=PPPP( 48)+bb*yzxz
                  PPPP( 49)=PPPP( 49)+bb*yzyx
                  PPPP( 50)=PPPP( 50)+bb*yzyy
                  PPPP( 51)=PPPP( 51)+bb*yzyz
                  PPPP( 52)=PPPP( 52)+bb*yzzx
                  PPPP( 53)=PPPP( 53)+bb*yzzy
                  PPPP( 54)=PPPP( 54)+bb*yzzz
                                         
                  PPPP( 55)=PPPP( 55)+bb*zxxx
                  PPPP( 56)=PPPP( 56)+bb*zxxy
                  PPPP( 57)=PPPP( 57)+bb*zxxz
                  PPPP( 58)=PPPP( 58)+bb*zxyx
                  PPPP( 59)=PPPP( 59)+bb*zxyy
                  PPPP( 60)=PPPP( 60)+bb*zxyz
                  PPPP( 61)=PPPP( 61)+bb*zxzx
                  PPPP( 62)=PPPP( 62)+bb*zxzy
                  PPPP( 63)=PPPP( 63)+bb*zxzz
                  PPPP( 64)=PPPP( 64)+bb*zyxx
                  PPPP( 65)=PPPP( 65)+bb*zyxy
                  PPPP( 66)=PPPP( 66)+bb*zyxz
                  PPPP( 67)=PPPP( 67)+bb*zyyx
                  PPPP( 68)=PPPP( 68)+bb*zyyy
                  PPPP( 69)=PPPP( 69)+bb*zyyz
                  PPPP( 70)=PPPP( 70)+bb*zyzx
                  PPPP( 71)=PPPP( 71)+bb*zyzy
                  PPPP( 72)=PPPP( 72)+bb*zyzz
                  PPPP( 73)=PPPP( 73)+bb*zzxx
                  PPPP( 74)=PPPP( 74)+bb*zzxy
                  PPPP( 75)=PPPP( 75)+bb*zzxz
                  PPPP( 76)=PPPP( 76)+bb*zzyx
                  PPPP( 77)=PPPP( 77)+bb*zzyy
                  PPPP( 78)=PPPP( 78)+bb*zzyz
                  PPPP( 79)=PPPP( 79)+bb*zzzx
                  PPPP( 80)=PPPP( 80)+bb*zzzy
                  PPPP( 81)=PPPP( 81)+bb*zzzz
!                 do ii=1,81
!                   pppp(ii) = bb*pppp(ii)
!                 enddo

 1111          enddo
            enddo
         enddo
      enddo

      do i=1,81
         pppp(i) = dabs(pppp(i))
      enddo

       ddmax = maxval(pppp)

      end
! JCUTPP ============================================================
! CALUCULATE (R'S|R'S)-TYPE INTEGRALS FOR SCHWARTZ INEQUALITY
! 
! (p'p|p'p),(pp'|pp')
! ===================================================================
 
      subroutine jcutpp(
     &   a,b,
     &   nfsz,nesz,fdat,edat,
     &   za,zb,
     &   ca,cb,
     &   ncnta,ncntb,ddmax    )

      implicit real(8)(a-h,o-z)
      
      parameter (pi52 = 17.4934183276248628462d0)
      dimension fdat(nfsz,nesz),edat(nesz)
      dimension rminv(10),tf(25)
      dimension za(ncnta),zb(ncntb)
      dimension ca(ncnta),cb(ncntb)
      dimension a(3),b(3),d(54),e(54),ec(324),spsp(9)

      data pi/3.14159265358979323846d+00 /
      data      (tf(i),i=1,25) /
     &                33.0,37.0,41.0,43.0,46.0,49.0,
     &                51.0,54.0,56.0,58.0,61.0,63.0,
     &                66.0,68.0,70.0,72.0,74.0,80.0,
     &                85.0,90.0,95.0,95.0,98.0,99.0,
     &                99.0                           /
c
      data      (rminv(i),i=1,10) /
     &   1.0d+00,  0.333333333333333333d+00,  0.20d+00,
     &   0.14285714285714286d+00,  0.11111111111111111d+00 ,
     &   0.90909090909090909d-01,  0.76923076923076923d-01 ,
     &   0.66666666666666666d-01,  0.58823529411764705d-01 ,
     &   0.52631578947368421d-01 /
c
      data  ga0   / 1.0d+00 /
      data  ga1   / 1.000 000 000 000 005 298d+00 /
      data  ga2   / 0.5d+00 /
      data  ga3   / 0.166 666 666 598 849 826d+00 /
      data  ga4   / 0.416 666 666 666 666 667d-01 /
      data  ga5   / 0.833 355 034 722 222 222d-02 /
      data  ga6   / 0.138 888 888 888 888 889d-02 /

      ax = a(1)
      ay = a(2)
      az = a(3)
      bx = b(1)
      by = b(2)
      bz = b(3)

      do i=1,324
         ec(i) = 0.d0
      enddo
      do i=1,54
         d(i) = 0.d0
         e(i) = 0.d0
      enddo

      do i=1,9
            spsp(i) = 0.d0
      enddo

      bax = bx - ax
      bay = by - ay
      baz = bz - az

      abxyz = bax*bax + bay*bay + baz*baz
      pi52_2 = 2.d0 * pi52

      do i=1,ncnta
         do j=1,ncntb

            zet   = za(i) + zb(j)
            roop  = za(i)*zb(j) / zet
            gp    = dexp(-roop*abxyz)
            rambdap = pi52_2 /zet
            contp = ca(i)*cb(j)

            px = (za(i)*ax + zb(j)*bx)/zet
            py = (za(i)*ay + zb(j)*by)/zet
            pz = (za(i)*az + zb(j)*bz)/zet

            pax=px-ax
            pay=py-ay
            paz=pz-az

            pbx=px-bx
            pby=py-by
            pbz=pz-bz


            do k=1,ncnta
               do l=1,ncntb
            eta   = za(k) + zb(l)
            zpq   = zet + eta
            rooq  = za(k)*zb(l) / eta
            gq    = dexp(-rooq*abxyz)
            rambda= rambdap/(eta*dsqrt(zpq))
            contpq = ca(k)*cb(l) * contp

            qx = (za(k)*ax + zb(l)*bx)/eta
            qy = (za(k)*ay + zb(l)*by)/eta
            qz = (za(k)*az + zb(l)*bz)/eta

            qcx=qx-ax
            qcy=qy-ay
            qcz=qz-az

            qdx=qx-bx
            qdy=qy-by
            qdz=qz-bz

            wx = (zet*px + eta*qx)/zpq
            wy = (zet*py + eta*qy)/zpq
            wz = (zet*pz + eta*qz)/zpq

            roo = zet*eta /zpq
            t   = roo*((px-qx)**2+(py-qy)**2+(pz-qz)**2)
  
                  if (t. le .tf(7)) then
     
                     it = idint((t+0.0251d+00)*20.0d+00)
                     dt = 0.05d+00*it - t
                     it = it + 1
                     dt4 = dt*0.25d+00
                     dt5 = dt*0.2d+00
                     dt6 = dt*0.166666666666666667d+00
                     dt2 = dt4 + dt4
                     dt3 = dt6 + dt6
                     f6  = (((((fdat(13,it)*dt6   + fdat(12,it))*dt5
     &                    + fdat(11,it))*dt4  + fdat(10,it))*dt3
     &                    + fdat(9,it))*dt2  + fdat(8,it))*dt
     &                    + fdat(7,it)
c     
                     eed  = (((((ga6*dt + ga5)*dt + ga4)*dt
     &                    + ga3)*dt + ga2)*dt + ga1)*dt + ga0
                     ee   = edat(it)*eed
                     t2   = 2.0d+00*t

                     f5   = (t2*f6 + ee)*rminv(6)
                     f4   = (t2*f5 + ee)*rminv(5)
                     f3   = (t2*f4 + ee)*rminv(4)
                     f2   = (t2*f3 + ee)*rminv(3)
                     f1   = (t2*f2 + ee)*rminv(2)
                     f0   = t2*f1 + ee
                  else
                     tinv = 1.0d+00/t
                     f0   = dsqrt(pi*tinv)*0.5d+00
                     f1   = 0.5d+00*f0*tinv
                     f2   = 1.5d+00*f1*tinv
                     f3   = 2.5d+00*f2*tinv
                     f4   = 3.5d+00*f3*tinv
                     f5   = 4.5d+00*f4*tinv
                     f6   = 5.5d+00*f5*tinv
                  end if
 
                  s0 = contpq*gp*gq*rambda*f0
                  s1 = contpq*gp*gq*rambda*f1
                  s2 = contpq*gp*gq*rambda*f2
                  s3 = contpq*gp*gq*rambda*f3
                  s4 = contpq*gp*gq*rambda*f4
                  s5 = contpq*gp*gq*rambda*f5
                  s6 = contpq*gp*gq*rambda*f6


                  rooz = roo/zet
                  rooe = roo/eta

                  wpx = wx - px
                  wpy = wy - py
                  wpz = wz - pz
                  rkp = 1.d0/(2.d0*zpq)

                  rzet = 0.5d0 / zet
                  reta = 0.5d0 / eta

                  wqx = wx - qx
                  wqy = wy - qy
                  wqz = wz - qz
!psss
                  px0 = pax*s0 + wpx*s1
                  py0 = pay*s0 + wpy*s1
                  pz0 = paz*s0 + wpz*s1
                  px1 = pax*s1 + wpx*s2
                  py1 = pay*s1 + wpy*s2
                  pz1 = paz*s1 + wpz*s2
                  px2 = pax*s2 + wpx*s3
                  py2 = pay*s2 + wpy*s3
                  pz2 = paz*s2 + wpz*s3
                  px3 = pax*s3 + wpx*s4
                  py3 = pay*s3 + wpy*s4
                  pz3 = paz*s3 + wpz*s4
                  px4 = pax*s4 + wpx*s5
                  py4 = pay*s4 + wpy*s5
                  pz4 = paz*s4 + wpz*s5
                  px5 = pax*s5 + wpx*s6
                  py5 = pay*s5 + wpy*s6
                  pz5 = paz*s5 + wpz*s6

!spss
                  xp0 = pbx*s0 + wpx*s1
                  yp0 = pby*s0 + wpy*s1
                  zp0 = pbz*s0 + wpz*s1
                  xp1 = pbx*s1 + wpx*s2
                  yp1 = pby*s1 + wpy*s2
                  zp1 = pbz*s1 + wpz*s2
                  xp2 = pbx*s2 + wpx*s3
                  yp2 = pby*s2 + wpy*s3
                  zp2 = pbz*s2 + wpz*s3
                  xp3 = pbx*s3 + wpx*s4
                  yp3 = pby*s3 + wpy*s4
                  zp3 = pbz*s3 + wpz*s4

!spsp
!     zb2 = zb(k)*2.d0
      spsp(1) = spsp(1)+qdx*xp0+wqx*xp1 +rkp*s1
      spsp(2) = spsp(2)+qdy*xp0+wqy*xp1
      spsp(3) = spsp(3)+qdz*xp0+wqz*xp1
      spsp(4) = spsp(4)+qdx*yp0+wqx*yp1
      spsp(5) = spsp(5)+qdy*yp0+wqy*yp1 +rkp*s1
      spsp(6) = spsp(6)+qdz*yp0+wqz*yp1
      spsp(7) = spsp(7)+qdx*zp0+wqx*zp1
      spsp(8) = spsp(8)+qdy*zp0+wqy*zp1
      spsp(9) = spsp(9)+qdz*zp0+wqz*zp1 +rkp*s1

                  s01 = rzet*(s0-rooz*s1)
                  s12 = rzet*(s1-rooz*s2)
                  s23 = rzet*(s2-rooz*s3)
                  s34 = rzet*(s3-rooz*s4)
                  s45 = rzet*(s4-rooz*s5)
! dsss 
                  dxx0 = pax*px0 + wpx*px1+s01
                  dyy0 = pay*py0 + wpy*py1+s01
                  dzz0 = paz*pz0 + wpz*pz1+s01
                  dyx0 = pay*px0 + wpy*px1
                  dzx0 = paz*px0 + wpz*px1
                  dzy0 = paz*py0 + wpz*py1

                  dxx1 = pax*px1 + wpx*px2+s12
                  dyy1 = pay*py1 + wpy*py2+s12
                  dzz1 = paz*pz1 + wpz*pz2+s12
                  dyx1 = pay*px1 + wpy*px2
                  dzx1 = paz*px1 + wpz*px2
                  dzy1 = paz*py1 + wpz*py2
                  
                  dxx2 = pax*px2 + wpx*px3+s23
                  dyy2 = pay*py2 + wpy*py3+s23
                  dzz2 = paz*pz2 + wpz*pz3+s23
                  dyx2 = pay*px2 + wpy*px3
                  dzx2 = paz*px2 + wpz*px3
                  dzy2 = paz*py2 + wpz*py3
 
                  dxx3 = pax*px3 + wpx*px4+s34
                  dyy3 = pay*py3 + wpy*py4+s34
                  dzz3 = paz*pz3 + wpz*pz4+s34
                  dyx3 = pay*px3 + wpy*px4
                  dzx3 = paz*px3 + wpz*px4
                  dzy3 = paz*py3 + wpz*py4

                  dxx4 = pax*px4 + wpx*px5+s45
                  dyy4 = pay*py4 + wpy*py5+s45
                  dzz4 = paz*pz4 + wpz*pz5+s45
                  dyx4 = pay*px4 + wpy*px5
                  dzx4 = paz*px4 + wpz*px5
                  dzy4 = paz*py4 + wpz*py5
!ppss
                  ppxx1 = pbx * px1 + wpx*px2 + s12
                  ppyx1 = pbx * py1 + wpx*py2
                  ppzx1 = pbx * pz1 + wpx*pz2
                  ppxy1 = pby * px1 + wpy*px2
                  ppyy1 = pby * py1 + wpy*py2 + s12
                  ppzy1 = pby * pz1 + wpy*pz2
                  ppxz1 = pbz * px1 + wpz*px2
                  ppyz1 = pbz * py1 + wpz*py2
                  ppzz1 = pbz * pz1 + wpz*pz2 + s12
                             
                  ppxx2 = pbx * px2 + wpx*px3 + s23
                  ppyx2 = pbx * py2 + wpx*py3
                  ppzx2 = pbx * pz2 + wpx*pz3
                  ppxy2 = pby * px2 + wpy*px3
                  ppyy2 = pby * py2 + wpy*py3 + s23
                  ppzy2 = pby * pz2 + wpy*pz3
                  ppxz2 = pbz * px2 + wpz*px3
                  ppyz2 = pbz * py2 + wpz*py3
                  ppzz2 = pbz * pz2 + wpz*pz3 + s23
           
                  ppxx3 = pbx * px3 + wpx*px4 + s34
                  ppyx3 = pbx * py3 + wpx*py4
                  ppzx3 = pbx * pz3 + wpx*pz4
                  ppxy3 = pby * px3 + wpy*px4
                  ppyy3 = pby * py3 + wpy*py4 + s34
                  ppzy3 = pby * pz3 + wpy*pz4
                  ppxz3 = pbz * px3 + wpz*px4
                  ppyz3 = pbz * py3 + wpz*py4
                  ppzz3 = pbz * pz3 + wpz*pz4 + s34
!spps
!==
      spxpx0 = qcx * xp0 + wqx*xp1 + rkp*s1
      spypx0 = qcx * yp0 + wqx*yp1
      spzpx0 = qcx * zp0 + wqx*zp1
      spxpy0 = qcy * xp0 + wqy*xp1
      spypy0 = qcy * yp0 + wqy*yp1 + rkp*s1
      spzpy0 = qcy * zp0 + wqy*zp1
      spxpz0 = qcz * xp0 + wqz*xp1
      spypz0 = qcz * yp0 + wqz*yp1
      spzpz0 = qcz * zp0 + wqz*zp1 + rkp*s1

      spxpx1 = qcx * xp1 + wqx*xp2 + rkp*s2
      spypx1 = qcx * yp1 + wqx*yp2
      spzpx1 = qcx * zp1 + wqx*zp2
      spxpy1 = qcy * xp1 + wqy*xp2
      spypy1 = qcy * yp1 + wqy*yp2 + rkp*s2
      spzpy1 = qcy * zp1 + wqy*zp2
      spxpz1 = qcz * xp1 + wqz*xp2
      spypz1 = qcz * yp1 + wqz*yp2
      spzpz1 = qcz * zp1 + wqz*zp2 + rkp*s2
!==
                  spxpx2 = qcx * xp2 + wqx*xp3 + rkp*s3
                  spypx2 = qcx * yp2 + wqx*yp3
                  spzpx2 = qcx * zp2 + wqx*zp3
                  spxpy2 = qcy * xp2 + wqy*xp3
                  spypy2 = qcy * yp2 + wqy*yp3 + rkp*s3
                  spzpy2 = qcy * zp2 + wqy*zp3
                  spxpz2 = qcz * xp2 + wqz*xp3
                  spypz2 = qcz * yp2 + wqz*yp3
                  spzpz2 = qcz * zp2 + wqz*zp3 + rkp*s3
!====spdp ====
! spss
      sxss0= pbx*s0+wpx*s1
      syss0= pby*s0+wpy*s1
      szss0= pbz*s0+wpz*s1

      sxss1= pbx*s1+wpx*s2
      syss1= pby*s1+wpy*s2
      szss1= pbz*s1+wpz*s2

      sxss2= pbx*s2+wpx*s3
      syss2= pby*s2+wpy*s3
      szss2= pbz*s2+wpz*s3
!= ssps
      ssxs1= qcx*s1+wqx*s2
      ssys1= qcy*s1+wqy*s2
      sszs1= qcz*s1+wqz*s2

      ssxs2= qcx*s2+wqx*s3
      ssys2= qcy*s2+wqy*s3
      sszs2= qcz*s2+wqz*s3
!spds
      xdxx0=qcx*spxpx0+wqx*spxpx1+rkp*ssxs1+reta*(sxss0-rooe*sxss1)
      xdyy0=qcy*spxpy0+wqy*spxpy1          +reta*(sxss0-rooe*sxss1)
      xdzz0=qcz*spxpz0+wqz*spxpz1          +reta*(sxss0-rooe*sxss1)
      xdyx0=qcy*spxpx0+wqy*spxpx1
      xdzx0=qcz*spxpx0+wqz*spxpx1
      xdzy0=qcz*spxpy0+wqz*spxpy1
      ydxx0=qcx*spypx0+wqx*spypx1          +reta*(syss0-rooe*syss1)
      ydyy0=qcy*spypy0+wqy*spypy1+rkp*ssys1+reta*(syss0-rooe*syss1)
      ydzz0=qcz*spypz0+wqz*spypz1          +reta*(syss0-rooe*syss1)
      ydyx0=qcx*spypy0+wqx*spypy1
      ydzx0=qcz*spypx0+wqz*spypx1
      ydzy0=qcz*spypy0+wqz*spypy1
      zdxx0=qcx*spzpx0+wqx*spzpx1          +reta*(szss0-rooe*szss1)
      zdyy0=qcy*spzpy0+wqy*spzpy1          +reta*(szss0-rooe*szss1)
      zdzz0=qcz*spzpz0+wqz*spzpz1+rkp*sszs1+reta*(szss0-rooe*szss1)
      zdyx0=qcy*spzpx0+wqy*spzpx1
      zdzx0=qcx*spzpz0+wqx*spzpz1
      zdzy0=qcy*spzpz0+wqy*spzpz1

      xdxx1=qcx*spxpx1+wqx*spxpx2+rkp*ssxs2+reta*(sxss1-rooe*sxss2)
      xdyy1=qcy*spxpy1+wqy*spxpy2          +reta*(sxss1-rooe*sxss2)
      xdzz1=qcz*spxpz1+wqz*spxpz2          +reta*(sxss1-rooe*sxss2)
      xdyx1=qcy*spxpx1+wqy*spxpx2
      xdzx1=qcz*spxpx1+wqz*spxpx2
      xdzy1=qcz*spxpy1+wqz*spxpy2
      ydxx1=qcx*spypx1+wqx*spypx2          +reta*(syss1-rooe*syss2)
      ydyy1=qcy*spypy1+wqy*spypy2+rkp*ssys2+reta*(syss1-rooe*syss2)
      ydzz1=qcz*spypz1+wqz*spypz2          +reta*(syss1-rooe*syss2)
      ydyx1=qcx*spypy1+wqx*spypy2
      ydzx1=qcz*spypx1+wqz*spypx2
      ydzy1=qcz*spypy1+wqz*spypy2
      zdxx1=qcx*spzpx1+wqx*spzpx2          +reta*(szss1-rooe*szss2)
      zdyy1=qcy*spzpy1+wqy*spzpy2          +reta*(szss1-rooe*szss2)
      zdzz1=qcz*spzpz1+wqz*spzpz2+rkp*sszs2+reta*(szss1-rooe*szss2)
      zdyx1=qcy*spzpx1+wqy*spzpx2
      zdzx1=qcx*spzpz1+wqx*spzpz2
      zdzy1=qcy*spzpz1+wqy*spzpz2

!spdp
      sxxs01 =reta*(spxpx0-rooe*spxpx1)
      sxys01 =reta*(spypx0-rooe*spypx1)
      sxzs01 =reta*(spzpx0-rooe*spzpx1)
      syxs01 =reta*(spxpy0-rooe*spxpy1)
      syys01 =reta*(spypy0-rooe*spypy1)
      syzs01 =reta*(spzpy0-rooe*spzpy1)
      szxs01 =reta*(spxpz0-rooe*spxpz1)
      szys01 =reta*(spypz0-rooe*spypz1)
      szzs01 =reta*(spzpz0-rooe*spzpz1)

      ssxs1= qcx*s1+wqx*s2
      ssys1= qcy*s1+wqy*s2
      sszs1= qcz*s1+wqz*s2

      ssxs2= qcx*s2+wqx*s3
      ssys2= qcy*s2+wqy*s3
      sszs2= qcz*s2+wqz*s3
!
      ssdxxs= qcx*ssxs1+wqx*ssxs2  +reta*(s1-rooe*s2)
      ssdyys= qcy*ssys1+wqy*ssys2  +reta*(s1-rooe*s2)
      ssdzzs= qcz*sszs1+wqz*sszs2  +reta*(s1-rooe*s2)
      ssdyxs= qcy*ssxs1+wqy*ssxs2
      ssdzxs= qcz*ssxs1+wqz*ssxs2
      ssdzys= qcz*ssys1+wqz*ssys2

      rxx1 = rkp * ssdxxs
      ryy1 = rkp * ssdyys
      rzz1 = rkp * ssdzzs
      ryx1 = rkp * ssdyxs
      rzx1 = rkp * ssdzxs
      rzy1 = rkp * ssdzys
!spdp
       zk2 = 2.d0*za(k)
      e( 1)=e( 1)+zk2*(qdx*xdxx0 + wqx*xdxx1 + rxx1 +2.d0*sxxs01)
      e( 2)=e( 2)+zk2*(qdy*xdxx0 + wqy*xdxx1                    )
      e( 3)=e( 3)+zk2*(qdz*xdxx0 + wqz*xdxx1                    )
      e( 4)=e( 4)+zk2*(qdx*xdyy0 + wqx*xdyy1 + ryy1             )
      e( 5)=e( 5)+zk2*(qdy*xdyy0 + wqy*xdyy1        +2.d0*sxys01)
      e( 6)=e( 6)+zk2*(qdz*xdyy0 + wqz*xdyy1                    )
      e( 7)=e( 7)+zk2*(qdx*xdzz0 + wqx*xdzz1 + rzz1             )
      e( 8)=e( 8)+zk2*(qdy*xdzz0 + wqy*xdzz1                    )
      e( 9)=e( 9)+zk2*(qdz*xdzz0 + wqz*xdzz1        +2.d0*sxzs01)
      e(10)=e(10)+zk2*(qdx*xdyx0 + wqx*xdyx1 + ryx1 +     sxys01)
      e(11)=e(11)+zk2*(qdy*xdyx0 + wqy*xdyx1        +     sxxs01)
      e(12)=e(12)+zk2*(qdz*xdyx0 + wqz*xdyx1                    )
      e(13)=e(13)+zk2*(qdx*xdzx0 + wqx*xdzx1 + rzx1 +     sxzs01)
      e(14)=e(14)+zk2*(qdy*xdzx0 + wqy*xdzx1                    )
      e(15)=e(15)+zk2*(qdz*xdzx0 + wqz*xdzx1        +     sxxs01)
      e(16)=e(16)+zk2*(qdx*xdzy0 + wqx*xdzy1 + rzy1             )
      e(17)=e(17)+zk2*(qdy*xdzy0 + wqy*xdzy1        +     sxzs01)
      e(18)=e(18)+zk2*(qdz*xdzy0 + wqz*xdzy1        +     sxys01)

      e(19)=e(19)+zk2*(qdx*ydxx0 + wqx*ydxx1        +2.d0*syxs01)
      e(20)=e(20)+zk2*(qdy*ydxx0 + wqy*ydxx1 + rxx1             )
      e(21)=e(21)+zk2*(qdz*ydxx0 + wqz*ydxx1                    )
      e(22)=e(22)+zk2*(qdx*ydyy0 + wqx*ydyy1                    )
      e(23)=e(23)+zk2*(qdy*ydyy0 + wqy*ydyy1 + ryy1 +2.d0*syys01)
      e(24)=e(24)+zk2*(qdz*ydyy0 + wqz*ydyy1                    )
      e(25)=e(25)+zk2*(qdx*ydzz0 + wqx*ydzz1                    )
      e(26)=e(26)+zk2*(qdy*ydzz0 + wqy*ydzz1 + rzz1             )
      e(27)=e(27)+zk2*(qdz*ydzz0 + wqz*ydzz1        +2.d0*syzs01)
      e(28)=e(28)+zk2*(qdx*ydyx0 + wqx*ydyx1        +     syys01)
      e(29)=e(29)+zk2*(qdy*ydyx0 + wqy*ydyx1 + ryx1 +     syxs01)
      e(30)=e(30)+zk2*(qdz*ydyx0 + wqz*ydyx1                    )
      e(31)=e(31)+zk2*(qdx*ydzx0 + wqx*ydzx1        +     syzs01)
      e(32)=e(32)+zk2*(qdy*ydzx0 + wqy*ydzx1 + rzx1             )
      e(33)=e(33)+zk2*(qdz*ydzx0 + wqz*ydzx1        +     syxs01)
      e(34)=e(34)+zk2*(qdx*ydzy0 + wqx*ydzy1                    )
      e(35)=e(35)+zk2*(qdy*ydzy0 + wqy*ydzy1 + rzy1 +     syzs01)
      e(36)=e(36)+zk2*(qdz*ydzy0 + wqz*ydzy1        +     syys01)

      e(37)=e(37)+zk2*(qdx*zdxx0 + wqx*zdxx1        +2.d0*szxs01)
      e(38)=e(38)+zk2*(qdy*zdxx0 + wqy*zdxx1                    )
      e(39)=e(39)+zk2*(qdz*zdxx0 + wqz*zdxx1 + rxx1             )
      e(40)=e(40)+zk2*(qdx*zdyy0 + wqx*zdyy1                    )
      e(41)=e(41)+zk2*(qdy*zdyy0 + wqy*zdyy1        +2.d0*szys01)
      e(42)=e(42)+zk2*(qdz*zdyy0 + wqz*zdyy1 + ryy1             )
      e(43)=e(43)+zk2*(qdx*zdzz0 + wqx*zdzz1                    )
      e(44)=e(44)+zk2*(qdy*zdzz0 + wqy*zdzz1                    )
      e(45)=e(45)+zk2*(qdz*zdzz0 + wqz*zdzz1 + rzz1 +2.d0*szzs01)
      e(46)=e(46)+zk2*(qdx*zdyx0 + wqx*zdyx1        +     szys01)
      e(47)=e(47)+zk2*(qdy*zdyx0 + wqy*zdyx1        +     szxs01)
      e(48)=e(48)+zk2*(qdz*zdyx0 + wqz*zdyx1 + ryx1             )
      e(49)=e(49)+zk2*(qdx*zdzx0 + wqx*zdzx1        +     szzs01)
      e(50)=e(50)+zk2*(qdy*zdzx0 + wqy*zdzx1                    )
      e(51)=e(51)+zk2*(qdz*zdzx0 + wqz*zdzx1 + rzx1 +     szxs01)
      e(52)=e(52)+zk2*(qdx*zdzy0 + wqx*zdzy1                    )
      e(53)=e(53)+zk2*(qdy*zdzy0 + wqy*zdzy1        +     szzs01)
      e(54)=e(54)+zk2*(qdz*zdzy0 + wqz*zdzy1 + rzy1 +     szys01)

!psps
                  pxpx2 = qcx * px2 + wqx*px3 + rkp*s3
                  pypx2 = qcx * py2 + wqx*py3
                  pzpx2 = qcx * pz2 + wqx*pz3
                  pxpy2 = qcy * px2 + wqy*px3
                  pypy2 = qcy * py2 + wqy*py3 + rkp*s3
                  pzpy2 = qcy * pz2 + wqy*pz3
                  pxpz2 = qcz * px2 + wqz*px3
                  pypz2 = qcz * py2 + wqz*py3
                  pzpz2 = qcz * pz2 + wqz*pz3 + rkp*s3
                     
                  pxpx3 = qcx * px3 + wqx*px4 + rkp*s4
                  pypx3 = qcx * py3 + wqx*py4
                  pzpx3 = qcx * pz3 + wqx*pz4
                  pxpy3 = qcy * px3 + wqy*px4
                  pypy3 = qcy * py3 + wqy*py4 + rkp*s4
                  pzpy3 = qcy * pz3 + wqy*pz4
                  pxpz3 = qcz * px3 + wqz*px4
                  pypz3 = qcz * py3 + wqz*py4
                  pzpz3 = qcz * pz3 + wqz*pz4 + rkp*s4

                  px01 =rzet*(px0-rooz*px1)
                  py01 =rzet*(py0-rooz*py1)
                  pz01 =rzet*(pz0-rooz*pz1)
                  px12 =rzet*(px1-rooz*px2)
                  py12 =rzet*(py1-rooz*py2)
                  pz12 =rzet*(pz1-rooz*pz2)
                  px23 =rzet*(px2-rooz*px3)
                  py23 =rzet*(py2-rooz*py3)
                  pz23 =rzet*(pz2-rooz*pz3)
                  px34 =rzet*(px3-rooz*px4)
                  py34 =rzet*(py3-rooz*py4)
                  pz34 =rzet*(pz3-rooz*pz4)

!dpss 
                  dpxxx0 =  pbx*dxx0 + wpx*dxx1 + 2.0d0*px01
                  dpxxy0 =  pby*dxx0 + wpy*dxx1             
                  dpxxz0 =  pbz*dxx0 + wpz*dxx1             
                  dpyyx0 =  pbx*dyy0 + wpx*dyy1             
                  dpyyy0 =  pby*dyy0 + wpy*dyy1 + 2.0d0*py01
                  dpyyz0 =  pbz*dyy0 + wpz*dyy1             
                  dpzzx0 =  pbx*dzz0 + wpx*dzz1             
                  dpzzy0 =  pby*dzz0 + wpy*dzz1             
                  dpzzz0 =  pbz*dzz0 + wpz*dzz1 + 2.0d0*pz01
                  dpyxx0 =  pbx*dyx0 + wpx*dyx1 +       py01
                  dpyxy0 =  pby*dyx0 + wpy*dyx1 +       px01
                  dpyxz0 =  pbz*dyx0 + wpz*dyx1             
                  dpzxx0 =  pbx*dzx0 + wpx*dzx1 +       pz01
                  dpzxy0 =  pby*dzx0 + wpy*dzx1             
                  dpzxz0 =  pbz*dzx0 + wpz*dzx1 +       px01
                  dpzyx0 =  pbx*dzy0 + wpx*dzy1             
                  dpzyy0 =  pby*dzy0 + wpy*dzy1 +       pz01
                  dpzyz0 =  pbz*dzy0 + wpz*dzy1 +       py01
           
                  dpxxx1 =  pbx*dxx1 + wpx*dxx2 + 2.0d0*px12
                  dpxxy1 =  pby*dxx1 + wpy*dxx2             
                  dpxxz1 =  pbz*dxx1 + wpz*dxx2             
                  dpyyx1 =  pbx*dyy1 + wpx*dyy2             
                  dpyyy1 =  pby*dyy1 + wpy*dyy2 + 2.0d0*py12
                  dpyyz1 =  pbz*dyy1 + wpz*dyy2             
                  dpzzx1 =  pbx*dzz1 + wpx*dzz2             
                  dpzzy1 =  pby*dzz1 + wpy*dzz2             
                  dpzzz1 =  pbz*dzz1 + wpz*dzz2 + 2.0d0*pz12
                  dpyxx1 =  pbx*dyx1 + wpx*dyx2 +       py12
                  dpyxy1 =  pby*dyx1 + wpy*dyx2 +       px12
                  dpyxz1 =  pbz*dyx1 + wpz*dyx2             
                  dpzxx1 =  pbx*dzx1 + wpx*dzx2 +       pz12
                  dpzxy1 =  pby*dzx1 + wpy*dzx2             
                  dpzxz1 =  pbz*dzx1 + wpz*dzx2 +       px12
                  dpzyx1 =  pbx*dzy1 + wpx*dzy2             
                  dpzyy1 =  pby*dzy1 + wpy*dzy2 +       pz12
                  dpzyz1 =  pbz*dzy1 + wpz*dzy2 +       py12
           
                  dpxxx2 =  pbx*dxx2 + wpx*dxx3 + 2.0d0*px23
                  dpxxy2 =  pby*dxx2 + wpy*dxx3
                  dpxxz2 =  pbz*dxx2 + wpz*dxx3
                  dpyyx2 =  pbx*dyy2 + wpx*dyy3
                  dpyyy2 =  pby*dyy2 + wpy*dyy3 + 2.0d0*py23
                  dpyyz2 =  pbz*dyy2 + wpz*dyy3             
                  dpzzx2 =  pbx*dzz2 + wpx*dzz3             
                  dpzzy2 =  pby*dzz2 + wpy*dzz3             
                  dpzzz2 =  pbz*dzz2 + wpz*dzz3 + 2.0d0*pz23
                  dpyxx2 =  pbx*dyx2 + wpx*dyx3 +       py23
                  dpyxy2 =  pby*dyx2 + wpy*dyx3 +       px23
                  dpyxz2 =  pbz*dyx2 + wpz*dyx3             
                  dpzxx2 =  pbx*dzx2 + wpx*dzx3 +       pz23
                  dpzxy2 =  pby*dzx2 + wpy*dzx3             
                  dpzxz2 =  pbz*dzx2 + wpz*dzx3 +       px23
                  dpzyx2 =  pbx*dzy2 + wpx*dzy3             
                  dpzyy2 =  pby*dzy2 + wpy*dzy3 +       pz23
                  dpzyz2 =  pbz*dzy2 + wpz*dzy3 +       py23
           
                  dpxxx3 =  pbx*dxx3 + wpx*dxx4 + 2.0d0*px34
                  dpxxy3 =  pby*dxx3 + wpy*dxx4             
                  dpxxz3 =  pbz*dxx3 + wpz*dxx4             
                  dpyyx3 =  pbx*dyy3 + wpx*dyy4             
                  dpyyy3 =  pby*dyy3 + wpy*dyy4 + 2.0d0*py34
                  dpyyz3 =  pbz*dyy3 + wpz*dyy4             
                  dpzzx3 =  pbx*dzz3 + wpx*dzz4             
                  dpzzy3 =  pby*dzz3 + wpy*dzz4             
                  dpzzz3 =  pbz*dzz3 + wpz*dzz4 + 2.0d0*pz34
                  dpyxx3 =  pbx*dyx3 + wpx*dyx4 +       py34
                  dpyxy3 =  pby*dyx3 + wpy*dyx4 +       px34
                  dpyxz3 =  pbz*dyx3 + wpz*dyx4             
                  dpzxx3 =  pbx*dzx3 + wpx*dzx4 +       pz34
                  dpzxy3 =  pby*dzx3 + wpy*dzx4             
                  dpzxz3 =  pbz*dzx3 + wpz*dzx4 +       px34
                  dpzyx3 =  pbx*dzy3 + wpx*dzy4             
                  dpzyy3 =  pby*dzy3 + wpy*dzy4 +       pz34
                  dpzyz3 =  pbz*dzy3 + wpz*dzy4 +       py34

                  rpx2 = rkp * px2
                  rpy2 = rkp * py2
                  rpz2 = rkp * pz2
                  rpx3 = rkp * px3
                  rpy3 = rkp * py3
                  rpz3 = rkp * pz3

                  rxp2 = rkp * xp2
                  ryp2 = rkp * yp2
                  rzp2 = rkp * zp2
                  rxp3 = rkp * xp3
                  ryp3 = rkp * yp3
                  rzp3 = rkp * zp3

!dsps
                  dxxpx1 =  qcx*dxx1 + wqx*dxx2 + 2.0d0*rpx2
                  dxxpy1 =  qcy*dxx1 + wqy*dxx2            
                  dxxpz1 =  qcz*dxx1 + wqz*dxx2            
                  dyypx1 =  qcx*dyy1 + wqx*dyy2            
                  dyypy1 =  qcy*dyy1 + wqy*dyy2 + 2.0d0*rpy2
                  dyypz1 =  qcz*dyy1 + wqz*dyy2            
                  dzzpx1 =  qcx*dzz1 + wqx*dzz2            
                  dzzpy1 =  qcy*dzz1 + wqy*dzz2            
                  dzzpz1 =  qcz*dzz1 + wqz*dzz2 + 2.0d0*rpz2
                  dyxpx1 =  qcx*dyx1 + wqx*dyx2 +       rpy2
                  dyxpy1 =  qcy*dyx1 + wqy*dyx2 +       rpx2
                  dyxpz1 =  qcz*dyx1 + wqz*dyx2            
                  dzxpx1 =  qcx*dzx1 + wqx*dzx2 +       rpz2
                  dzxpy1 =  qcy*dzx1 + wqy*dzx2            
                  dzxpz1 =  qcz*dzx1 + wqz*dzx2 +       rpx2
                  dzypx1 =  qcx*dzy1 + wqx*dzy2            
                  dzypy1 =  qcy*dzy1 + wqy*dzy2 +       rpz2
                  dzypz1 =  qcz*dzy1 + wqz*dzy2 +       rpy2
                     
                  dxxpx2 =  qcx*dxx2 + wqx*dxx3 + 2.0d0*rpx3
                  dxxpy2 =  qcy*dxx2 + wqy*dxx3
                  dxxpz2 =  qcz*dxx2 + wqz*dxx3
                  dyypx2 =  qcx*dyy2 + wqx*dyy3
                  dyypy2 =  qcy*dyy2 + wqy*dyy3 + 2.0d0*rpy3
                  dyypz2 =  qcz*dyy2 + wqz*dyy3            
                  dzzpx2 =  qcx*dzz2 + wqx*dzz3            
                  dzzpy2 =  qcy*dzz2 + wqy*dzz3            
                  dzzpz2 =  qcz*dzz2 + wqz*dzz3 + 2.0d0*rpz3
                  dyxpx2 =  qcx*dyx2 + wqx*dyx3 +       rpy3
                  dyxpy2 =  qcy*dyx2 + wqy*dyx3 +       rpx3
                  dyxpz2 =  qcz*dyx2 + wqz*dyx3            
                  dzxpx2 =  qcx*dzx2 + wqx*dzx3 +       rpz3
                  dzxpy2 =  qcy*dzx2 + wqy*dzx3            
                  dzxpz2 =  qcz*dzx2 + wqz*dzx3 +       rpx3
                  dzypx2 =  qcx*dzy2 + wqx*dzy3            
                  dzypy2 =  qcy*dzy2 + wqy*dzy3 +       rpz3
                  dzypz2 =  qcz*dzy2 + wqz*dzy3 +       rpy3

                  pppxxx1 = qcx*ppxx1 + wqx*ppxx2 + rxp2 + rpx2
                  pppxxy1 = qcy*ppxx1 + wqy*ppxx2        
                  pppxxz1 = qcz*ppxx1 + wqz*ppxx2        
                  pppxyx1 = qcx*ppxy1 + wqx*ppxy2 + ryp2
                  pppxyy1 = qcy*ppxy1 + wqy*ppxy2        + rpx2
                  pppxyz1 = qcz*ppxy1 + wqz*ppxy2              
                  pppxzx1 = qcx*ppxz1 + wqx*ppxz2 + rzp2       
                  pppxzy1 = qcy*ppxz1 + wqy*ppxz2              
                  pppxzz1 = qcz*ppxz1 + wqz*ppxz2        + rpx2
                  pppyxx1 = qcx*ppyx1 + wqx*ppyx2        + rpy2
                  pppyxy1 = qcy*ppyx1 + wqy*ppyx2 + rxp2       
                  pppyxz1 = qcz*ppyx1 + wqz*ppyx2              
                  pppyyx1 = qcx*ppyy1 + wqx*ppyy2              
                  pppyyy1 = qcy*ppyy1 + wqy*ppyy2 + ryp2 + rpy2
                  pppyyz1 = qcz*ppyy1 + wqz*ppyy2              
                  pppyzx1 = qcx*ppyz1 + wqx*ppyz2              
                  pppyzy1 = qcy*ppyz1 + wqy*ppyz2 + rzp2       
                  pppyzz1 = qcz*ppyz1 + wqz*ppyz2        + rpy2
                  pppzxx1 = qcx*ppzx1 + wqx*ppzx2        + rpz2
                  pppzxy1 = qcy*ppzx1 + wqy*ppzx2              
                  pppzxz1 = qcz*ppzx1 + wqz*ppzx2 + rxp2       
                  pppzyx1 = qcx*ppzy1 + wqx*ppzy2              
                  pppzyy1 = qcy*ppzy1 + wqy*ppzy2        + rpz2
                  pppzyz1 = qcz*ppzy1 + wqz*ppzy2 + ryp2       
                  pppzzx1 = qcx*ppzz1 + wqx*ppzz2
                  pppzzy1 = qcy*ppzz1 + wqy*ppzz2
                  pppzzz1 = qcz*ppzz1 + wqz*ppzz2 + rzp2 + rpz2

                  pppxxx2 = qcx*ppxx2 + wqx*ppxx3 + rxp3 + rpx3
                  pppxxy2 = qcy*ppxx2 + wqy*ppxx3        
                  pppxxz2 = qcz*ppxx2 + wqz*ppxx3        
                  pppxyx2 = qcx*ppxy2 + wqx*ppxy3 + ryp3
                  pppxyy2 = qcy*ppxy2 + wqy*ppxy3        + rpx3
                  pppxyz2 = qcz*ppxy2 + wqz*ppxy3              
                  pppxzx2 = qcx*ppxz2 + wqx*ppxz3 + rzp3       
                  pppxzy2 = qcy*ppxz2 + wqy*ppxz3              
                  pppxzz2 = qcz*ppxz2 + wqz*ppxz3        + rpx3
                  pppyxx2 = qcx*ppyx2 + wqx*ppyx3        + rpy3
                  pppyxy2 = qcy*ppyx2 + wqy*ppyx3 + rxp3       
                  pppyxz2 = qcz*ppyx2 + wqz*ppyx3              
                  pppyyx2 = qcx*ppyy2 + wqx*ppyy3              
                  pppyyy2 = qcy*ppyy2 + wqy*ppyy3 + ryp3 + rpy3
                  pppyyz2 = qcz*ppyy2 + wqz*ppyy3              
                  pppyzx2 = qcx*ppyz2 + wqx*ppyz3              
                  pppyzy2 = qcy*ppyz2 + wqy*ppyz3 + rzp3       
                  pppyzz2 = qcz*ppyz2 + wqz*ppyz3        + rpy3
                  pppzxx2 = qcx*ppzx2 + wqx*ppzx3        + rpz3
                  pppzxy2 = qcy*ppzx2 + wqy*ppzx3              
                  pppzxz2 = qcz*ppzx2 + wqz*ppzx3 + rxp3       
                  pppzyx2 = qcx*ppzy2 + wqx*ppzy3              
                  pppzyy2 = qcy*ppzy2 + wqy*ppzy3        + rpz3
                  pppzyz2 = qcz*ppzy2 + wqz*ppzy3 + ryp3       
                  pppzzx2 = qcx*ppzz2 + wqx*ppzz3
                  pppzzy2 = qcy*ppzz2 + wqy*ppzz3              
                  pppzzz2 = qcz*ppzz2 + wqz*ppzz3 + rzp3 + rpz3
!RANK = 4
       ppxx12 =reta*(ppxx1 - rooe*ppxx2)
       ppxy12 =reta*(ppxy1 - rooe*ppxy2)
       ppxz12 =reta*(ppxz1 - rooe*ppxz2)
       ppyx12 =reta*(ppyx1 - rooe*ppyx2)
       ppyy12 =reta*(ppyy1 - rooe*ppyy2)
       ppyz12 =reta*(ppyz1 - rooe*ppyz2)
       ppzx12 =reta*(ppzx1 - rooe*ppzx2)
       ppzy12 =reta*(ppzy1 - rooe*ppzy2)
       ppzz12 =reta*(ppzz1 - rooe*ppzz2)
       
      rpxpx2 = rkp*pxpx2
      rpxpy2 = rkp*pxpy2
      rpxpz2 = rkp*pxpz2
      rpypx2 = rkp*pypx2
      rpypy2 = rkp*pypy2
      rpypz2 = rkp*pypz2
      rpzpx2 = rkp*pzpx2
      rpzpy2 = rkp*pzpy2
      rpzpz2 = rkp*pzpz2
      rspxpx2 = rkp*spxpx2
      rspypx2 = rkp*spypx2
      rspzpx2 = rkp*spzpx2
      rspxpy2 = rkp*spxpy2
      rspypy2 = rkp*spypy2
      rspzpy2 = rkp*spzpy2
      rspxpz2 = rkp*spxpz2
      rspypz2 = rkp*spypz2
      rspzpz2 = rkp*spzpz2

!ppds
      xxdxx1 = qcx*pppxxx1 + wqx*pppxxx2 +rspxpx2 + rpxpx2 +ppxx12
      xxdyy1 = qcy*pppxxy1 + wqy*pppxxy2                   +ppxx12
      xxdzz1 = qcz*pppxxz1 + wqz*pppxxz2                   +ppxx12
      xxdyx1 = qcy*pppxxx1 + wqy*pppxxx2
      xxdzx1 = qcz*pppxxx1 + wqz*pppxxx2
      xxdzy1 = qcy*pppxxz1 + wqy*pppxxz2
      xydxx1 = qcx*pppxyx1 + wqx*pppxyx2 +rspypx2          +ppxy12
      xydyy1 = qcy*pppxyy1 + wqy*pppxyy2          +rpxpy2  +ppxy12
      xydzz1 = qcz*pppxyz1 + wqz*pppxyz2                   +ppxy12
      xydyx1 = qcy*pppxyx1 + wqy*pppxyx2          +rpxpx2
      xydzx1 = qcz*pppxyx1 + wqz*pppxyx2
      xydzy1 = qcz*pppxyy1 + wqz*pppxyy2
      xzdxx1 = qcx*pppxzx1 + wqx*pppxzx2 +rspzpx2          +ppxz12
      xzdyy1 = qcy*pppxzy1 + wqy*pppxzy2                   +ppxz12
      xzdzz1 = qcz*pppxzz1 + wqz*pppxzz2          +rpxpz2  +ppxz12
      xzdyx1 = qcy*pppxzx1 + wqy*pppxzx2
      xzdzx1 = qcx*pppxzz1 + wqx*pppxzz2 +rspzpz2
      xzdzy1 = qcy*pppxzz1 + wqy*pppxzz2
      yxdxx1 = qcx*pppyxx1 + wqx*pppyxx2          + rpypx2 +ppyx12
      yxdyy1 = qcy*pppyxy1 + wqy*pppyxy2 +rspxpy2          +ppyx12
      yxdzz1 = qcz*pppyxz1 + wqz*pppyxz2                   +ppyx12
      yxdyx1 = qcx*pppyxy1 + wqx*pppyxy2          + rpypy2
      yxdzx1 = qcz*pppyxx1 + wqz*pppyxx2
      yxdzy1 = qcz*pppyxy1 + wqz*pppyxy2
      yydxx1 = qcx*pppyyx1 + wqx*pppyyx2                   +ppyy12
      yydyy1 = qcy*pppyyy1 + wqy*pppyyy2 +rspypy2 +rpypy2  +ppyy12
      yydzz1 = qcz*pppyyz1 + wqz*pppyyz2                   +ppyy12
      yydyx1 = qcx*pppyyy1 + wqx*pppyyy2
      yydzx1 = qcz*pppyyx1 + wqz*pppyyx2 
      yydzy1 = qcz*pppyyy1 + wqz*pppyyy2 
      yzdxx1 = qcx*pppyzx1 + wqx*pppyzx2                   +ppyz12
      yzdyy1 = qcy*pppyzy1 + wqy*pppyzy2 +rspzpy2          +ppyz12
      yzdzz1 = qcz*pppyzz1 + wqz*pppyzz2          +rpypz2  +ppyz12
      yzdyx1 = qcx*pppyzy1 + wqx*pppyzy2
      yzdzx1 = qcx*pppyzz1 + wqx*pppyzz2
      yzdzy1 = qcy*pppyzz1 + wqy*pppyzz2 +rspzpz2
      zxdxx1 = qcx*pppzxx1 + wqx*pppzxx2          + rpzpx2 +ppzx12
      zxdyy1 = qcy*pppzxy1 + wqy*pppzxy2                   +ppzx12
      zxdzz1 = qcz*pppzxz1 + wqz*pppzxz2 +rspxpz2          +ppzx12
      zxdyx1 = qcy*pppzxx1 + wqy*pppzxx2
      zxdzx1 = qcz*pppzxx1 + wqz*pppzxx2 +rspxpx2
      zxdzy1 = qcy*pppzxz1 + wqy*pppzxz2 
      zydxx1 = qcx*pppzyx1 + wqx*pppzyx2                   +ppzy12
      zydyy1 = qcy*pppzyy1 + wqy*pppzyy2          +rpzpy2  +ppzy12
      zydzz1 = qcz*pppzyz1 + wqz*pppzyz2 +rspypz2          +ppzy12
      zydyx1 = qcx*pppzyy1 + wqx*pppzyy2
      zydzx1 = qcx*pppzyz1 + wqx*pppzyz2
      zydzy1 = qcz*pppzyy1 + wqz*pppzyy2 +rspypy2 
      zzdxx1 = qcx*pppzzx1 + wqx*pppzzx2                   +ppzz12
      zzdyy1 = qcy*pppzzy1 + wqy*pppzzy2                   +ppzz12
      zzdzz1 = qcz*pppzzz1 + wqz*pppzzz2 +rspzpz2 +rpzpz2  +ppzz12
      zzdyx1 = qcx*pppzzy1 + wqx*pppzzy2
      zzdzx1 = qcx*pppzzz1 + wqx*pppzzz2
      zzdzy1 = qcy*pppzzz1 + wqy*pppzzz2

      rdxx1 = rkp*dxx1
      rdyy1 = rkp*dyy1
      rdzz1 = rkp*dzz1
      rdyx1 = rkp*dyx1
      rdzx1 = rkp*dzx1
      rdzy1 = rkp*dzy1
      rdxx2 = rkp*dxx2
      rdyy2 = rkp*dyy2
      rdzz2 = rkp*dzz2
      rdyx2 = rkp*dyx2
      rdzx2 = rkp*dzx2
      rdzy2 = rkp*dzy2
      rdxx3 = rkp*dxx3
      rdyy3 = rkp*dyy3
      rdzz3 = rkp*dzz3
      rdyx3 = rkp*dyx3
      rdzx3 = rkp*dzx3
      rdzy3 = rkp*dzy3

      dxx12 =reta*(dxx1 -rooe*dxx2)
      dyy12 =reta*(dyy1 -rooe*dyy2)
      dzz12 =reta*(dzz1 -rooe*dzz2)
      dyx12 =reta*(dyx1 -rooe*dyx2)
      dzx12 =reta*(dzx1 -rooe*dzx2)
      dzy12 =reta*(dzy1 -rooe*dzy2)

      rppxx1 = rkp*ppxx1
      rppxy1 = rkp*ppxy1
      rppxz1 = rkp*ppxz1
      rppyx1 = rkp*ppyx1
      rppyy1 = rkp*ppyy1
      rppyz1 = rkp*ppyz1
      rppzx1 = rkp*ppzx1
      rppzy1 = rkp*ppzy1
      rppzz1 = rkp*ppzz1
      rppxx2 = rkp*ppxx2
      rppxy2 = rkp*ppxy2
      rppxz2 = rkp*ppxz2
      rppyx2 = rkp*ppyx2
      rppyy2 = rkp*ppyy2
      rppyz2 = rkp*ppyz2
      rppzx2 = rkp*ppzx2
      rppzy2 = rkp*ppzy2
      rppzz2 = rkp*ppzz2
      rppxx3 = rkp*ppxx3
      rppxy3 = rkp*ppxy3
      rppxz3 = rkp*ppxz3
      rppyx3 = rkp*ppyx3
      rppyy3 = rkp*ppyy3
      rppyz3 = rkp*ppyz3
      rppzx3 = rkp*ppzx3
      rppzy3 = rkp*ppzy3
      rppzz3 = rkp*ppzz3

!dsds
      dxxdxx1 = qcx*dxxpx1 + wqx*dxxpx2 +2.0d0*rpxpx2 + dxx12
      dxxdyy1 = qcy*dxxpy1 + wqy*dxxpy2               + dxx12
      dxxdzz1 = qcz*dxxpz1 + wqz*dxxpz2               + dxx12
      dxxdyx1 = qcy*dxxpx1 + wqy*dxxpx2            
      dxxdzx1 = qcz*dxxpx1 + wqz*dxxpx2            
      dxxdzy1 = qcy*dxxpz1 + wqy*dxxpz2            
      dyydxx1 = qcx*dyypx1 + wqx*dyypx2               + dyy12
      dyydyy1 = qcy*dyypy1 + wqy*dyypy2 +2.0d0*rpypy2 + dyy12
      dyydzz1 = qcz*dyypz1 + wqz*dyypz2               + dyy12
      dyydyx1 = qcy*dyypx1 + wqy*dyypx2 +2.0d0*rpypx2
      dyydzx1 = qcz*dyypx1 + wqz*dyypx2            
      dyydzy1 = qcz*dyypy1 + wqz*dyypy2            
      dzzdxx1 = qcx*dzzpx1 + wqx*dzzpx2               + dzz12
      dzzdyy1 = qcy*dzzpy1 + wqy*dzzpy2               + dzz12
      dzzdzz1 = qcz*dzzpz1 + wqz*dzzpz2 +2.0d0*rpzpz2 + dzz12
      dzzdyx1 = qcy*dzzpx1 + wqy*dzzpx2            
      dzzdzx1 = qcz*dzzpx1 + wqz*dzzpx2 +2.0d0*rpzpx2
      dzzdzy1 = qcy*dzzpz1 + wqy*dzzpz2            
      dyxdxx1 = qcx*dyxpx1 + wqx*dyxpx2 +      rpypx2 + dyx12
      dyxdyy1 = qcy*dyxpy1 + wqy*dyxpy2 +      rpxpy2 + dyx12
      dyxdzz1 = qcz*dyxpz1 + wqz*dyxpz2               + dyx12
      dyxdyx1 = qcy*dyxpx1 + wqy*dyxpx2 +      rpxpx2
      dyxdzx1 = qcz*dyxpx1 + wqz*dyxpx2            
      dyxdzy1 = qcz*dyxpy1 + wqz*dyxpy2            
      dzxdxx1 = qcx*dzxpx1 + wqx*dzxpx2 +      rpzpx2 + dzx12
      dzxdyy1 = qcy*dzxpy1 + wqy*dzxpy2               + dzx12
      dzxdzz1 = qcz*dzxpz1 + wqz*dzxpz2 +      rpxpz2 + dzx12
      dzxdyx1 = qcy*dzxpx1 + wqy*dzxpx2            
      dzxdzx1 = qcz*dzxpx1 + wqz*dzxpx2 +      rpxpx2
      dzxdzy1 = qcy*dzxpz1 + wqy*dzxpz2            
      dzydxx1 = qcx*dzypx1 + wqx*dzypx2               + dzy12
      dzydyy1 = qcy*dzypy1 + wqy*dzypy2 +      rpzpy2 + dzy12
      dzydzz1 = qcz*dzypz1 + wqz*dzypz2 +      rpypz2 + dzy12
      dzydyx1 = qcx*dzypy1 + wqx*dzypy2
      dzydzx1 = qcx*dzypz1 + wqx*dzypz2
      dzydzy1 = qcy*dzypz1 + wqy*dzypz2 +      rpzpz2

!dpps
      dxxpxpx0= qcx*dpxxx0 + wqx*dpxxx1 + 2.0d0*rppxx1 + rdxx1
      dxxpxpy0= qcy*dpxxx0 + wqy*dpxxx1 
      dxxpxpz0= qcz*dpxxx0 + wqz*dpxxx1 
      dxxpypx0= qcx*dpxxy0 + wqx*dpxxy1 + 2.0d0*rppxy1       
      dxxpypy0= qcy*dpxxy0 + wqy*dpxxy1                + rdxx1
      dxxpypz0= qcz*dpxxy0 + wqz*dpxxy1                      
      dxxpzpx0= qcx*dpxxz0 + wqx*dpxxz1 + 2.0d0*rppxz1       
      dxxpzpy0= qcy*dpxxz0 + wqy*dpxxz1                      
      dxxpzpz0= qcz*dpxxz0 + wqz*dpxxz1                + rdxx1
      dyypxpx0= qcx*dpyyx0 + wqx*dpyyx1                + rdyy1
      dyypxpy0= qcy*dpyyx0 + wqy*dpyyx1 + 2.0d0*rppyx1       
      dyypxpz0= qcz*dpyyx0 + wqz*dpyyx1                      
      dyypypx0= qcx*dpyyy0 + wqx*dpyyy1                      
      dyypypy0= qcy*dpyyy0 + wqy*dpyyy1 + 2.0d0*rppyy1 + rdyy1
      dyypypz0= qcz*dpyyy0 + wqz*dpyyy1                      
      dyypzpx0= qcx*dpyyz0 + wqx*dpyyz1                      
      dyypzpy0= qcy*dpyyz0 + wqy*dpyyz1 + 2.0d0*rppyz1       
      dyypzpz0= qcz*dpyyz0 + wqz*dpyyz1                + rdyy1
      dzzpxpx0= qcx*dpzzx0 + wqx*dpzzx1                + rdzz1
      dzzpxpy0= qcy*dpzzx0 + wqy*dpzzx1                      
      dzzpxpz0= qcz*dpzzx0 + wqz*dpzzx1 + 2.0d0*rppzx1       
      dzzpypx0= qcx*dpzzy0 + wqx*dpzzy1                      
      dzzpypy0= qcy*dpzzy0 + wqy*dpzzy1                + rdzz1
      dzzpypz0= qcz*dpzzy0 + wqz*dpzzy1 + 2.0d0*rppzy1       
      dzzpzpx0= qcx*dpzzz0 + wqx*dpzzz1                      
      dzzpzpy0= qcy*dpzzz0 + wqy*dpzzz1                      
      dzzpzpz0= qcz*dpzzz0 + wqz*dpzzz1 + 2.0d0*rppzz1 + rdzz1
      dyxpxpx0= qcx*dpyxx0 + wqx*dpyxx1 +       rppyx1 + rdyx1
      dyxpxpy0= qcy*dpyxx0 + wqy*dpyxx1 +       rppxx1       
      dyxpxpz0= qcz*dpyxx0 + wqz*dpyxx1                      
      dyxpypx0= qcx*dpyxy0 + wqx*dpyxy1 +       rppyy1       
      dyxpypy0= qcy*dpyxy0 + wqy*dpyxy1 +       rppxy1 + rdyx1
      dyxpypz0= qcz*dpyxy0 + wqz*dpyxy1      
      dyxpzpx0= qcx*dpyxz0 + wqx*dpyxz1 +       rppyz1       
      dyxpzpy0= qcy*dpyxz0 + wqy*dpyxz1 +       rppxz1       
      dyxpzpz0= qcz*dpyxz0 + wqz*dpyxz1                + rdyx1
      dzxpxpx0= qcx*dpzxx0 + wqx*dpzxx1 +       rppzx1 + rdzx1
      dzxpxpy0= qcy*dpzxx0 + wqy*dpzxx1                      
      dzxpxpz0= qcz*dpzxx0 + wqz*dpzxx1 +       rppxx1       
      dzxpypx0= qcx*dpzxy0 + wqx*dpzxy1 +       rppzy1       
      dzxpypy0= qcy*dpzxy0 + wqy*dpzxy1                + rdzx1
      dzxpypz0= qcz*dpzxy0 + wqz*dpzxy1 +       rppxy1       
      dzxpzpx0= qcx*dpzxz0 + wqx*dpzxz1 +       rppzz1       
      dzxpzpy0= qcy*dpzxz0 + wqy*dpzxz1                      
      dzxpzpz0= qcz*dpzxz0 + wqz*dpzxz1 +       rppxz1 + rdzx1
      dzypxpx0= qcx*dpzyx0 + wqx*dpzyx1                + rdzy1
      dzypxpy0= qcy*dpzyx0 + wqy*dpzyx1 +       rppzx1       
      dzypxpz0= qcz*dpzyx0 + wqz*dpzyx1 +       rppyx1       
      dzypypx0= qcx*dpzyy0 + wqx*dpzyy1                      
      dzypypy0= qcy*dpzyy0 + wqy*dpzyy1 +       rppzy1 + rdzy1
      dzypypz0= qcz*dpzyy0 + wqz*dpzyy1 +       rppyy1       
      dzypzpx0= qcx*dpzyz0 + wqx*dpzyz1                      
      dzypzpy0= qcy*dpzyz0 + wqy*dpzyz1 +       rppzz1       
      dzypzpz0= qcz*dpzyz0 + wqz*dpzyz1 +       rppyz1 + rdzy1
 
!dpsp
      za2=za(i)*2.d0
      d( 1)=d( 1)+za2*(qdx*dpxxx0+wqx*dpxxx1+2.0d0*rppxx1+rdxx1)
      d( 2)=d( 2)+za2*(qdy*dpxxx0+wqy*dpxxx1                   )
      d( 3)=d( 3)+za2*(qdz*dpxxx0+wqz*dpxxx1                   )
      d( 4)=d( 4)+za2*(qdx*dpxxy0+wqx*dpxxy1+2.0d0*rppxy1      )
      d( 5)=d( 5)+za2*(qdy*dpxxy0+wqy*dpxxy1             +rdxx1)
      d( 6)=d( 6)+za2*(qdz*dpxxy0+wqz*dpxxy1                   )
      d( 7)=d( 7)+za2*(qdx*dpxxz0+wqx*dpxxz1+2.0d0*rppxz1      )
      d( 8)=d( 8)+za2*(qdy*dpxxz0+wqy*dpxxz1                   )
      d( 9)=d( 9)+za2*(qdz*dpxxz0+wqz*dpxxz1             +rdxx1)
      d(10)=d(10)+za2*(qdx*dpyyx0+wqx*dpyyx1             +rdyy1)
      d(11)=d(11)+za2*(qdy*dpyyx0+wqy*dpyyx1+2.0d0*rppyx1      )
      d(12)=d(12)+za2*(qdz*dpyyx0+wqz*dpyyx1                   )
      d(13)=d(13)+za2*(qdx*dpyyy0+wqx*dpyyy1                   )
      d(14)=d(14)+za2*(qdy*dpyyy0+wqy*dpyyy1+2.0d0*rppyy1+rdyy1)
      d(15)=d(15)+za2*(qdz*dpyyy0+wqz*dpyyy1                   )
      d(16)=d(16)+za2*(qdx*dpyyz0+wqx*dpyyz1                   )
      d(17)=d(17)+za2*(qdy*dpyyz0+wqy*dpyyz1+2.0d0*rppyz1      )
      d(18)=d(18)+za2*(qdz*dpyyz0+wqz*dpyyz1             +rdyy1)
      d(19)=d(19)+za2*(qdx*dpzzx0+wqx*dpzzx1             +rdzz1)
      d(20)=d(20)+za2*(qdy*dpzzx0+wqy*dpzzx1                   )
      d(21)=d(21)+za2*(qdz*dpzzx0+wqz*dpzzx1+2.0d0*rppzx1      )
      d(22)=d(22)+za2*(qdx*dpzzy0+wqx*dpzzy1                   )
      d(23)=d(23)+za2*(qdy*dpzzy0+wqy*dpzzy1             +rdzz1)
      d(24)=d(24)+za2*(qdz*dpzzy0+wqz*dpzzy1+2.0d0*rppzy1      ) 
      d(25)=d(25)+za2*(qdx*dpzzz0+wqx*dpzzz1                   ) 
      d(26)=d(26)+za2*(qdy*dpzzz0+wqy*dpzzz1                   ) 
      d(27)=d(27)+za2*(qdz*dpzzz0+wqz*dpzzz1+2.0d0*rppzz1+rdzz1)
      d(28)=d(28)+za2*(qdx*dpyxx0+wqx*dpyxx1+      rppyx1+rdyx1)
      d(29)=d(29)+za2*(qdy*dpyxx0+wqy*dpyxx1+      rppxx1      )
      d(30)=d(30)+za2*(qdz*dpyxx0+wqz*dpyxx1                   )
      d(31)=d(31)+za2*(qdx*dpyxy0+wqx*dpyxy1+      rppyy1      )
      d(32)=d(32)+za2*(qdy*dpyxy0+wqy*dpyxy1+      rppxy1+rdyx1)
      d(33)=d(33)+za2*(qdz*dpyxy0+wqz*dpyxy1                   )
      d(34)=d(34)+za2*(qdx*dpyxz0+wqx*dpyxz1+      rppyz1      )
      d(35)=d(35)+za2*(qdy*dpyxz0+wqy*dpyxz1+      rppxz1      )
      d(36)=d(36)+za2*(qdz*dpyxz0+wqz*dpyxz1             +rdyx1)
      d(37)=d(37)+za2*(qdx*dpzxx0+wqx*dpzxx1+      rppzx1+rdzx1)
      d(38)=d(38)+za2*(qdy*dpzxx0+wqy*dpzxx1                   )
      d(39)=d(39)+za2*(qdz*dpzxx0+wqz*dpzxx1+      rppxx1      )
      d(40)=d(40)+za2*(qdx*dpzxy0+wqx*dpzxy1+      rppzy1      )
      d(41)=d(41)+za2*(qdy*dpzxy0+wqy*dpzxy1             +rdzx1)
      d(42)=d(42)+za2*(qdz*dpzxy0+wqz*dpzxy1+      rppxy1      )
      d(43)=d(43)+za2*(qdx*dpzxz0+wqx*dpzxz1+      rppzz1      )
      d(44)=d(44)+za2*(qdy*dpzxz0+wqy*dpzxz1                   )
      d(45)=d(45)+za2*(qdz*dpzxz0+wqz*dpzxz1+      rppxz1+rdzx1)
      d(46)=d(46)+za2*(qdx*dpzyx0+wqx*dpzyx1             +rdzy1)
      d(47)=d(47)+za2*(qdy*dpzyx0+wqy*dpzyx1+      rppzx1      )
      d(48)=d(48)+za2*(qdz*dpzyx0+wqz*dpzyx1+      rppyx1      )
      d(49)=d(49)+za2*(qdx*dpzyy0+wqx*dpzyy1                   )
      d(50)=d(50)+za2*(qdy*dpzyy0+wqy*dpzyy1+      rppzy1+rdzy1)
      d(51)=d(51)+za2*(qdz*dpzyy0+wqz*dpzyy1+      rppyy1      )
      d(52)=d(52)+za2*(qdx*dpzyz0+wqx*dpzyz1                   )
      d(53)=d(53)+za2*(qdy*dpzyz0+wqy*dpzyz1+      rppzz1      )
      d(54)=d(54)+za2*(qdz*dpzyz0+wqz*dpzyz1+      rppyz1+rdzy1)


      dxxpxpx1= qcx*dpxxx1 + wqx*dpxxx2 + 2.0d0*rppxx2 + rdxx2
      dxxpxpy1= qcy*dpxxx1 + wqy*dpxxx2
      dxxpxpz1= qcz*dpxxx1 + wqz*dpxxx2
      dxxpypx1= qcx*dpxxy1 + wqx*dpxxy2 + 2.0d0*rppxy2
      dxxpypy1= qcy*dpxxy1 + wqy*dpxxy2                + rdxx2
      dxxpypz1= qcz*dpxxy1 + wqz*dpxxy2
      dxxpzpx1= qcx*dpxxz1 + wqx*dpxxz2 + 2.0d0*rppxz2        
      dxxpzpy1= qcy*dpxxz1 + wqy*dpxxz2                       
      dxxpzpz1= qcz*dpxxz1 + wqz*dpxxz2                + rdxx2
      dyypxpx1= qcx*dpyyx1 + wqx*dpyyx2                + rdyy2
      dyypxpy1= qcy*dpyyx1 + wqy*dpyyx2 + 2.0d0*rppyx2        
      dyypxpz1= qcz*dpyyx1 + wqz*dpyyx2                       
      dyypypx1= qcx*dpyyy1 + wqx*dpyyy2                       
      dyypypy1= qcy*dpyyy1 + wqy*dpyyy2 + 2.0d0*rppyy2 + rdyy2
      dyypypz1= qcz*dpyyy1 + wqz*dpyyy2                       
      dyypzpx1= qcx*dpyyz1 + wqx*dpyyz2                       
      dyypzpy1= qcy*dpyyz1 + wqy*dpyyz2 + 2.0d0*rppyz2        
      dyypzpz1= qcz*dpyyz1 + wqz*dpyyz2                + rdyy2
      dzzpxpx1= qcx*dpzzx1 + wqx*dpzzx2                + rdzz2
      dzzpxpy1= qcy*dpzzx1 + wqy*dpzzx2                       
      dzzpxpz1= qcz*dpzzx1 + wqz*dpzzx2 + 2.0d0*rppzx2        
      dzzpypx1= qcx*dpzzy1 + wqx*dpzzy2                       
      dzzpypy1= qcy*dpzzy1 + wqy*dpzzy2                + rdzz2
      dzzpypz1= qcz*dpzzy1 + wqz*dpzzy2 + 2.0d0*rppzy2        
      dzzpzpx1= qcx*dpzzz1 + wqx*dpzzz2                       
      dzzpzpy1= qcy*dpzzz1 + wqy*dpzzz2                       
      dzzpzpz1= qcz*dpzzz1 + wqz*dpzzz2 + 2.0d0*rppzz2 + rdzz2
      dyxpxpx1= qcx*dpyxx1 + wqx*dpyxx2 +       rppyx2 + rdyx2
      dyxpxpy1= qcy*dpyxx1 + wqy*dpyxx2 +       rppxx2        
      dyxpxpz1= qcz*dpyxx1 + wqz*dpyxx2                       
      dyxpypx1= qcx*dpyxy1 + wqx*dpyxy2 +       rppyy2        
      dyxpypy1= qcy*dpyxy1 + wqy*dpyxy2 +       rppxy2 + rdyx2
      dyxpypz1= qcz*dpyxy1 + wqz*dpyxy2                       
      dyxpzpx1= qcx*dpyxz1 + wqx*dpyxz2 +       rppyz2        
      dyxpzpy1= qcy*dpyxz1 + wqy*dpyxz2 +       rppxz2        
      dyxpzpz1= qcz*dpyxz1 + wqz*dpyxz2                + rdyx2
      dzxpxpx1= qcx*dpzxx1 + wqx*dpzxx2 +       rppzx2 + rdzx2
      dzxpxpy1= qcy*dpzxx1 + wqy*dpzxx2                       
      dzxpxpz1= qcz*dpzxx1 + wqz*dpzxx2 +       rppxx2        
      dzxpypx1= qcx*dpzxy1 + wqx*dpzxy2 +       rppzy2        
      dzxpypy1= qcy*dpzxy1 + wqy*dpzxy2                + rdzx2
      dzxpypz1= qcz*dpzxy1 + wqz*dpzxy2 +       rppxy2        
      dzxpzpx1= qcx*dpzxz1 + wqx*dpzxz2 +       rppzz2        
      dzxpzpy1= qcy*dpzxz1 + wqy*dpzxz2                       
      dzxpzpz1= qcz*dpzxz1 + wqz*dpzxz2 +       rppxz2 + rdzx2
      dzypxpx1= qcx*dpzyx1 + wqx*dpzyx2                + rdzy2
      dzypxpy1= qcy*dpzyx1 + wqy*dpzyx2 +       rppzx2        
      dzypxpz1= qcz*dpzyx1 + wqz*dpzyx2 +       rppyx2        
      dzypypx1= qcx*dpzyy1 + wqx*dpzyy2                       
      dzypypy1= qcy*dpzyy1 + wqy*dpzyy2 +       rppzy2 + rdzy2
      dzypypz1= qcz*dpzyy1 + wqz*dpzyy2 +       rppyy2        
      dzypzpx1= qcx*dpzyz1 + wqx*dpzyz2                       
      dzypzpy1= qcy*dpzyz1 + wqy*dpzyz2 +       rppzz2        
      dzypzpz1= qcz*dpzyz1 + wqz*dpzyz2 +       rppyz2 + rdzy2

      dxxpxpx2= qcx*dpxxx2 + wqx*dpxxx3 + 2.0d0*rppxx3 + rdxx3
      dxxpxpy2= qcy*dpxxx2 + wqy*dpxxx3                       
      dxxpxpz2= qcz*dpxxx2 + wqz*dpxxx3                       
      dxxpypx2= qcx*dpxxy2 + wqx*dpxxy3 + 2.0d0*rppxy3        
      dxxpypy2= qcy*dpxxy2 + wqy*dpxxy3                + rdxx3
      dxxpypz2= qcz*dpxxy2 + wqz*dpxxy3                       
      dxxpzpx2= qcx*dpxxz2 + wqx*dpxxz3 + 2.0d0*rppxz3        
      dxxpzpy2= qcy*dpxxz2 + wqy*dpxxz3                       
      dxxpzpz2= qcz*dpxxz2 + wqz*dpxxz3                + rdxx3
      dyypxpx2= qcx*dpyyx2 + wqx*dpyyx3                + rdyy3
      dyypxpy2= qcy*dpyyx2 + wqy*dpyyx3 + 2.0d0*rppyx3        
      dyypxpz2= qcz*dpyyx2 + wqz*dpyyx3                       
      dyypypx2= qcx*dpyyy2 + wqx*dpyyy3                       
      dyypypy2= qcy*dpyyy2 + wqy*dpyyy3 + 2.0d0*rppyy3 + rdyy3
      dyypypz2= qcz*dpyyy2 + wqz*dpyyy3                       
      dyypzpx2= qcx*dpyyz2 + wqx*dpyyz3                       
      dyypzpy2= qcy*dpyyz2 + wqy*dpyyz3 + 2.0d0*rppyz3        
      dyypzpz2= qcz*dpyyz2 + wqz*dpyyz3                + rdyy3
      dzzpxpx2= qcx*dpzzx2 + wqx*dpzzx3                + rdzz3
      dzzpxpy2= qcy*dpzzx2 + wqy*dpzzx3                       
      dzzpxpz2= qcz*dpzzx2 + wqz*dpzzx3 + 2.0d0*rppzx3        
      dzzpypx2= qcx*dpzzy2 + wqx*dpzzy3                       
      dzzpypy2= qcy*dpzzy2 + wqy*dpzzy3                + rdzz3
      dzzpypz2= qcz*dpzzy2 + wqz*dpzzy3 + 2.0d0*rppzy3        
      dzzpzpx2= qcx*dpzzz2 + wqx*dpzzz3                       
      dzzpzpy2= qcy*dpzzz2 + wqy*dpzzz3                       
      dzzpzpz2= qcz*dpzzz2 + wqz*dpzzz3 + 2.0d0*rppzz3 + rdzz3
      dyxpxpx2= qcx*dpyxx2 + wqx*dpyxx3 +       rppyx3 + rdyx3
      dyxpxpy2= qcy*dpyxx2 + wqy*dpyxx3 +       rppxx3        
      dyxpxpz2= qcz*dpyxx2 + wqz*dpyxx3                       
      dyxpypx2= qcx*dpyxy2 + wqx*dpyxy3 +       rppyy3        
      dyxpypy2= qcy*dpyxy2 + wqy*dpyxy3 +       rppxy3 + rdyx3
      dyxpypz2= qcz*dpyxy2 + wqz*dpyxy3                       
      dyxpzpx2= qcx*dpyxz2 + wqx*dpyxz3 +       rppyz3        
      dyxpzpy2= qcy*dpyxz2 + wqy*dpyxz3 +       rppxz3        
      dyxpzpz2= qcz*dpyxz2 + wqz*dpyxz3                + rdyx3
      dzxpxpx2= qcx*dpzxx2 + wqx*dpzxx3 +       rppzx3 + rdzx3
      dzxpxpy2= qcy*dpzxx2 + wqy*dpzxx3                       
      dzxpxpz2= qcz*dpzxx2 + wqz*dpzxx3 +       rppxx3        
      dzxpypx2= qcx*dpzxy2 + wqx*dpzxy3 +       rppzy3        
      dzxpypy2= qcy*dpzxy2 + wqy*dpzxy3                + rdzx3
      dzxpypz2= qcz*dpzxy2 + wqz*dpzxy3 +       rppxy3        
      dzxpzpx2= qcx*dpzxz2 + wqx*dpzxz3 +       rppzz3 
      dzxpzpy2= qcy*dpzxz2 + wqy*dpzxz3                       
      dzxpzpz2= qcz*dpzxz2 + wqz*dpzxz3 +       rppxz3 + rdzx3
      dzypxpx2= qcx*dpzyx2 + wqx*dpzyx3                + rdzy3
      dzypxpy2= qcy*dpzyx2 + wqy*dpzyx3 +       rppzx3        
      dzypxpz2= qcz*dpzyx2 + wqz*dpzyx3 +       rppyx3        
      dzypypx2= qcx*dpzyy2 + wqx*dpzyy3                       
      dzypypy2= qcy*dpzyy2 + wqy*dpzyy3 +       rppzy3 + rdzy3
      dzypypz2= qcz*dpzyy2 + wqz*dpzyy3 +       rppyy3        
      dzypzpx2= qcx*dpzyz2 + wqx*dpzyz3                       
      dzypzpy2= qcy*dpzyz2 + wqy*dpzyz3 +       rppzz3        
      dzypzpz2= qcz*dpzyz2 + wqz*dpzyz3 +       rppyz3 + rdzy3

      rpppxxx1 = rkp*pppxxx1
      rpppxxy1 = rkp*pppxxy1
      rpppxxz1 = rkp*pppxxz1
      rpppxyx1 = rkp*pppxyx1
      rpppxyy1 = rkp*pppxyy1
      rpppxyz1 = rkp*pppxyz1
      rpppxzx1 = rkp*pppxzx1
      rpppxzy1 = rkp*pppxzy1
      rpppxzz1 = rkp*pppxzz1
      rpppyxx1 = rkp*pppyxx1
      rpppyxy1 = rkp*pppyxy1
      rpppyxz1 = rkp*pppyxz1
      rpppyyx1 = rkp*pppyyx1
      rpppyyy1 = rkp*pppyyy1
      rpppyyz1 = rkp*pppyyz1
      rpppyzx1 = rkp*pppyzx1
      rpppyzy1 = rkp*pppyzy1
      rpppyzz1 = rkp*pppyzz1
      rpppzxx1 = rkp*pppzxx1
      rpppzxy1 = rkp*pppzxy1
      rpppzxz1 = rkp*pppzxz1
      rpppzyx1 = rkp*pppzyx1
      rpppzyy1 = rkp*pppzyy1
      rpppzyz1 = rkp*pppzyz1
      rpppzzx1 = rkp*pppzzx1
      rpppzzy1 = rkp*pppzzy1
      rpppzzz1 = rkp*pppzzz1

      rpppxxx2 = rkp*pppxxx2
      rpppxxy2 = rkp*pppxxy2
      rpppxxz2 = rkp*pppxxz2
      rpppxyx2 = rkp*pppxyx2
      rpppxyy2 = rkp*pppxyy2
      rpppxyz2 = rkp*pppxyz2
      rpppxzx2 = rkp*pppxzx2
      rpppxzy2 = rkp*pppxzy2
      rpppxzz2 = rkp*pppxzz2
      rpppyxx2 = rkp*pppyxx2
      rpppyxy2 = rkp*pppyxy2
      rpppyxz2 = rkp*pppyxz2
      rpppyyx2 = rkp*pppyyx2
      rpppyyy2 = rkp*pppyyy2
      rpppyyz2 = rkp*pppyyz2
      rpppyzx2 = rkp*pppyzx2
      rpppyzy2 = rkp*pppyzy2
      rpppyzz2 = rkp*pppyzz2
      rpppzxx2 = rkp*pppzxx2
      rpppzxy2 = rkp*pppzxy2
      rpppzxz2 = rkp*pppzxz2
      rpppzyx2 = rkp*pppzyx2
      rpppzyy2 = rkp*pppzyy2
      rpppzyz2 = rkp*pppzyz2
      rpppzzx2 = rkp*pppzzx2
      rpppzzy2 = rkp*pppzzy2
      rpppzzz2 = rkp*pppzzz2
 
      rdxxpx1 =  rkp*dxxpx1
      rdxxpy1 =  rkp*dxxpy1
      rdxxpz1 =  rkp*dxxpz1
      rdyypx1 =  rkp*dyypx1
      rdyypy1 =  rkp*dyypy1
      rdyypz1 =  rkp*dyypz1
      rdzzpx1 =  rkp*dzzpx1
      rdzzpy1 =  rkp*dzzpy1
      rdzzpz1 =  rkp*dzzpz1
      rdyxpx1 =  rkp*dyxpx1
      rdyxpy1 =  rkp*dyxpy1
      rdyxpz1 =  rkp*dyxpz1
      rdzxpx1 =  rkp*dzxpx1
      rdzxpy1 =  rkp*dzxpy1
      rdzxpz1 =  rkp*dzxpz1
      rdzypx1 =  rkp*dzypx1
      rdzypy1 =  rkp*dzypy1
      rdzypz1 =  rkp*dzypz1
                        
      rdxxpx2 =  rkp*dxxpx2
      rdxxpy2 =  rkp*dxxpy2
      rdxxpz2 =  rkp*dxxpz2
      rdyypx2 =  rkp*dyypx2
      rdyypy2 =  rkp*dyypy2
      rdyypz2 =  rkp*dyypz2
      rdzzpx2 =  rkp*dzzpx2
      rdzzpy2 =  rkp*dzzpy2
      rdzzpz2 =  rkp*dzzpz2
      rdyxpx2 =  rkp*dyxpx2
      rdyxpy2 =  rkp*dyxpy2
      rdyxpz2 =  rkp*dyxpz2
      rdzxpx2 =  rkp*dzxpx2
      rdzxpy2 =  rkp*dzxpy2
      rdzxpz2 =  rkp*dzxpz2
      rdzypx2 =  rkp*dzypx2
      rdzypy2 =  rkp*dzypy2
      rdzypz2 =  rkp*dzypz2

      dpxxx01= reta*(dpxxx0 -rooe*dpxxx1)
      dpxxy01= reta*(dpxxy0 -rooe*dpxxy1)
      dpxxz01= reta*(dpxxz0 -rooe*dpxxz1)
      dpyyx01= reta*(dpyyx0 -rooe*dpyyx1)
      dpyyy01= reta*(dpyyy0 -rooe*dpyyy1)
      dpyyz01= reta*(dpyyz0 -rooe*dpyyz1)
      dpzzx01= reta*(dpzzx0 -rooe*dpzzx1)
      dpzzy01= reta*(dpzzy0 -rooe*dpzzy1)
      dpzzz01= reta*(dpzzz0 -rooe*dpzzz1)
      dpyxx01= reta*(dpyxx0 -rooe*dpyxx1)
      dpyxy01= reta*(dpyxy0 -rooe*dpyxy1)
      dpyxz01= reta*(dpyxz0 -rooe*dpyxz1)
      dpzxx01= reta*(dpzxx0 -rooe*dpzxx1)
      dpzxy01= reta*(dpzxy0 -rooe*dpzxy1)
      dpzxz01= reta*(dpzxz0 -rooe*dpzxz1)
      dpzyx01= reta*(dpzyx0 -rooe*dpzyx1)
      dpzyy01= reta*(dpzyy0 -rooe*dpzyy1)
      dpzyz01= reta*(dpzyz0 -rooe*dpzyz1)

      dpxxx12= reta*(dpxxx1 -rooe*dpxxx2)
      dpxxy12= reta*(dpxxy1 -rooe*dpxxy2)
      dpxxz12= reta*(dpxxz1 -rooe*dpxxz2)
      dpyyx12= reta*(dpyyx1 -rooe*dpyyx2)
      dpyyy12= reta*(dpyyy1 -rooe*dpyyy2)
      dpyyz12= reta*(dpyyz1 -rooe*dpyyz2)
      dpzzx12= reta*(dpzzx1 -rooe*dpzzx2)
      dpzzy12= reta*(dpzzy1 -rooe*dpzzy2)
      dpzzz12= reta*(dpzzz1 -rooe*dpzzz2)
      dpyxx12= reta*(dpyxx1 -rooe*dpyxx2)
      dpyxy12= reta*(dpyxy1 -rooe*dpyxy2)
      dpyxz12= reta*(dpyxz1 -rooe*dpyxz2)
      dpzxx12= reta*(dpzxx1 -rooe*dpzxx2)
      dpzxy12= reta*(dpzxy1 -rooe*dpzxy2)
      dpzxz12= reta*(dpzxz1 -rooe*dpzxz2)
      dpzyx12= reta*(dpzyx1 -rooe*dpzyx2)
      dpzyy12= reta*(dpzyy1 -rooe*dpzyy2)
      dpzyz12= reta*(dpzyz1 -rooe*dpzyz2)
  
!dpds 
 
      axa0= qcx*dxxpxpx0+wqx*dxxpxpx1+2.d0*rpppxxx1+rdxxpx1+dpxxx01
      axb0= qcy*dxxpxpy0+wqy*dxxpxpy1                      +dpxxx01
      axc0= qcz*dxxpxpz0+wqz*dxxpxpz1                      +dpxxx01
      axd0= qcy*dxxpxpx0+wqy*dxxpxpx1
      axe0= qcz*dxxpxpx0+wqz*dxxpxpx1
      axf0= qcy*dxxpxpz0+wqy*dxxpxpz1
      aya0= qcx*dxxpypx0+wqx*dxxpypx1+2.d0*rpppxyx1        +dpxxy01
      ayb0= qcy*dxxpypy0+wqy*dxxpypy1              +rdxxpy1+dpxxy01
      ayc0= qcz*dxxpypz0+wqz*dxxpypz1                      +dpxxy01
      ayd0= qcy*dxxpypx0+wqy*dxxpypx1              +rdxxpx1
      aye0= qcz*dxxpypx0+wqz*dxxpypx1
      ayf0= qcz*dxxpypy0+wqz*dxxpypy1
      aza0= qcx*dxxpzpx0+wqx*dxxpzpx1+2.d0*rpppxzx1        +dpxxz01
      azb0= qcy*dxxpzpy0+wqy*dxxpzpy1                      +dpxxz01
      azc0= qcz*dxxpzpz0+wqz*dxxpzpz1              +rdxxpz1+dpxxz01
      azd0= qcy*dxxpzpx0+wqy*dxxpzpx1                      
      aze0= qcz*dxxpzpx0+wqz*dxxpzpx1              +rdxxpx1
      azf0= qcy*dxxpzpz0+wqy*dxxpzpz1      
      bxa0= qcx*dyypxpx0+wqx*dyypxpx1              +rdyypx1+dpyyx01
      bxb0= qcy*dyypxpy0+wqy*dyypxpy1+2.d0*rpppyxy1        +dpyyx01
      bxc0= qcz*dyypxpz0+wqz*dyypxpz1                      +dpyyx01
      bxd0= qcx*dyypxpy0+wqx*dyypxpy1              +rdyypy1
      bxe0= qcx*dyypxpz0+wqx*dyypxpz1              +rdyypz1
      bxf0= qcz*dyypxpy0+wqz*dyypxpy1                      
      bya0= qcx*dyypypx0+wqx*dyypypx1                      +dpyyy01
      byb0= qcy*dyypypy0+wqy*dyypypy1+2.d0*rpppyyy1+rdyypy1+dpyyy01
      byc0= qcz*dyypypz0+wqz*dyypypz1                      +dpyyy01
      byd0= qcx*dyypypy0+wqx*dyypypy1
      bye0= qcx*dyypypz0+wqx*dyypypz1
      byf0= qcz*dyypypy0+wqz*dyypypy1
      bza0= qcx*dyypzpx0+wqx*dyypzpx1                      +dpyyz01
      bzb0= qcy*dyypzpy0+wqy*dyypzpy1+2.d0*rpppyzy1        +dpyyz01
      bzc0= qcz*dyypzpz0+wqz*dyypzpz1              +rdyypz1+dpyyz01
      bzd0= qcx*dyypzpy0+wqx*dyypzpy1
      bze0= qcx*dyypzpz0+wqx*dyypzpz1
      bzf0= qcz*dyypzpy0+wqz*dyypzpy1              +rdyypy1
      cxa0= qcx*dzzpxpx0+wqx*dzzpxpx1              +rdzzpx1+dpzzx01
      cxb0= qcy*dzzpxpy0+wqy*dzzpxpy1                      +dpzzx01
      cxc0= qcz*dzzpxpz0+wqz*dzzpxpz1+2.d0*rpppzxz1        +dpzzx01
      cxd0= qcy*dzzpxpx0+wqy*dzzpxpx1                            
      cxe0= qcx*dzzpxpz0+wqx*dzzpxpz1              +rdzzpz1      
      cxf0= qcy*dzzpxpz0+wqy*dzzpxpz1                            
      cya0= qcx*dzzpypx0+wqx*dzzpypx1                      +dpzzy01
      cyb0= qcy*dzzpypy0+wqy*dzzpypy1              +rdzzpy1+dpzzy01
      cyc0= qcz*dzzpypz0+wqz*dzzpypz1+2.d0*rpppzyz1        +dpzzy01
      cyd0= qcx*dzzpypy0+wqx*dzzpypy1                            
      cye0= qcx*dzzpypz0+wqx*dzzpypz1                            
      cyf0= qcy*dzzpypz0+wqy*dzzpypz1              +rdzzpz1      
      cza0= qcx*dzzpzpx0+wqx*dzzpzpx1                      +dpzzz01
      czb0= qcy*dzzpzpy0+wqy*dzzpzpy1                      +dpzzz01
      czc0= qcz*dzzpzpz0+wqz*dzzpzpz1+2.d0*rpppzzz1+rdzzpz1+dpzzz01
      czd0= qcx*dzzpzpy0+wqx*dzzpzpy1                      
      cze0= qcx*dzzpzpz0+wqx*dzzpzpz1                      
      czf0= qcy*dzzpzpz0+wqy*dzzpzpz1                      
      dxa0= qcx*dyxpxpx0+wqx*dyxpxpx1+     rpppyxx1+rdyxpx1+dpyxx01
      dxb0= qcy*dyxpxpy0+wqy*dyxpxpy1+     rpppxxy1        +dpyxx01
      dxc0= qcz*dyxpxpz0+wqz*dyxpxpz1                      +dpyxx01
      dxd0= qcx*dyxpxpy0+wqx*dyxpxpy1+     rpppyxy1+rdyxpy1
      dxe0= qcx*dyxpxpz0+wqx*dyxpxpz1+     rpppyxz1+rdyxpz1
      dxf0= qcz*dyxpxpy0+wqz*dyxpxpy1                      
      dya0= qcx*dyxpypx0+wqx*dyxpypx1+     rpppyyx1        +dpyxy01
      dyb0= qcy*dyxpypy0+wqy*dyxpypy1+     rpppxyy1+rdyxpy1+dpyxy01
      dyc0= qcz*dyxpypz0+wqz*dyxpypz1                      +dpyxy01
      dyd0= qcx*dyxpypy0+wqx*dyxpypy1+     rpppyyy1
      dye0= qcz*dyxpypx0+wqz*dyxpypx1              
      dyf0= qcz*dyxpypy0+wqz*dyxpypy1              
      dza0= qcx*dyxpzpx0+wqx*dyxpzpx1+     rpppyzx1        +dpyxz01
      dzb0= qcy*dyxpzpy0+wqy*dyxpzpy1+     rpppxzy1        +dpyxz01
      dzc0= qcz*dyxpzpz0+wqz*dyxpzpz1              +rdyxpz1+dpyxz01
      dzd0= qcx*dyxpzpy0+wqx*dyxpzpy1+     rpppyzy1        
      dze0= qcx*dyxpzpz0+wqx*dyxpzpz1+     rpppyzz1        
      dzf0= qcy*dyxpzpz0+wqy*dyxpzpz1+     rpppxzz1        
      exa0= qcx*dzxpxpx0+wqx*dzxpxpx1+     rpppzxx1+rdzxpx1+dpzxx01
      exb0= qcy*dzxpxpy0+wqy*dzxpxpy1                      +dpzxx01
      exc0= qcz*dzxpxpz0+wqz*dzxpxpz1+     rpppxxz1        +dpzxx01
      exd0= qcy*dzxpxpx0+wqy*dzxpxpx1              
      exe0= qcz*dzxpxpx0+wqz*dzxpxpx1+     rpppxxx1
      exf0= qcy*dzxpxpz0+wqy*dzxpxpz1              
      eya0= qcx*dzxpypx0+wqx*dzxpypx1+     rpppzyx1        +dpzxy01
      eyb0= qcy*dzxpypy0+wqy*dzxpypy1              +rdzxpy1+dpzxy01
      eyc0= qcz*dzxpypz0+wqz*dzxpypz1+     rpppxyz1        +dpzxy01
      eyd0= qcx*dzxpypy0+wqx*dzxpypy1+     rpppzyy1        
      eye0= qcz*dzxpypx0+wqz*dzxpypx1+     rpppxyx1        
      eyf0= qcz*dzxpypy0+wqz*dzxpypy1+     rpppxyy1        
      eza0= qcx*dzxpzpx0+wqx*dzxpzpx1+     rpppzzx1        +dpzxz01
      ezb0= qcy*dzxpzpy0+wqy*dzxpzpy1                      +dpzxz01
      ezc0= qcz*dzxpzpz0+wqz*dzxpzpz1+     rpppxzz1+rdzxpz1+dpzxz01
      ezd0= qcy*dzxpzpx0+wqy*dzxpzpx1                      
      eze0= qcx*dzxpzpz0+wqx*dzxpzpz1+     rpppzzz1        
      ezf0= qcy*dzxpzpz0+wqy*dzxpzpz1                      
      fxa0= qcx*dzypxpx0+wqx*dzypxpx1              +rdzypx1+dpzyx01
      fxb0= qcy*dzypxpy0+wqy*dzypxpy1+     rpppzxy1        +dpzyx01
      fxc0= qcz*dzypxpz0+wqz*dzypxpz1+     rpppyxz1        +dpzyx01
      fxd0= qcy*dzypxpx0+wqy*dzypxpx1+     rpppzxx1        
      fxe0= qcz*dzypxpx0+wqz*dzypxpx1+     rpppyxx1        
      fxf0= qcz*dzypxpy0+wqz*dzypxpy1+     rpppyxy1        
      fya0= qcx*dzypypx0+wqx*dzypypx1                      +dpzyy01
      fyb0= qcy*dzypypy0+wqy*dzypypy1+     rpppzyy1+rdzypy1+dpzyy01
      fyc0= qcz*dzypypz0+wqz*dzypypz1+     rpppyyz1        +dpzyy01
      fyd0= qcx*dzypypy0+wqx*dzypypy1                      
      fye0= qcx*dzypypz0+wqx*dzypypz1                      
      fyf0= qcz*dzypypy0+wqz*dzypypy1+     rpppyyy1        
      fza0= qcx*dzypzpx0+wqx*dzypzpx1                      +dpzyz01
      fzb0= qcy*dzypzpy0+wqy*dzypzpy1+     rpppzzy1        +dpzyz01
      fzc0= qcz*dzypzpz0+wqz*dzypzpz1+     rpppyzz1+rdzypz1+dpzyz01
      fzd0= qcx*dzypzpy0+wqx*dzypzpy1                      
      fze0= qcx*dzypzpz0+wqx*dzypzpz1                      
      fzf0= qcy*dzypzpz0+wqy*dzypzpz1+     rpppzzz1    

      axa1= qcx*dxxpxpx1+wqx*dxxpxpx2+2.d0*rpppxxx2+rdxxpx2+dpxxx12
      axb1= qcy*dxxpxpy1+wqy*dxxpxpy2                      +dpxxx12
      axc1= qcz*dxxpxpz1+wqz*dxxpxpz2                      +dpxxx12
      axd1= qcy*dxxpxpx1+wqy*dxxpxpx2                              
      axe1= qcz*dxxpxpx1+wqz*dxxpxpx2                              
      axf1= qcy*dxxpxpz1+wqy*dxxpxpz2                              
      aya1= qcx*dxxpypx1+wqx*dxxpypx2+2.d0*rpppxyx2        +dpxxy12
      ayb1= qcy*dxxpypy1+wqy*dxxpypy2              +rdxxpy2+dpxxy12
      ayc1= qcz*dxxpypz1+wqz*dxxpypz2                      +dpxxy12
      ayd1= qcy*dxxpypx1+wqy*dxxpypx2              +rdxxpx2        
      aye1= qcz*dxxpypx1+wqz*dxxpypx2                              
      ayf1= qcz*dxxpypy1+wqz*dxxpypy2                              
      aza1= qcx*dxxpzpx1+wqx*dxxpzpx2+2.d0*rpppxzx2        +dpxxz12
      azb1= qcy*dxxpzpy1+wqy*dxxpzpy2                      +dpxxz12
      azc1= qcz*dxxpzpz1+wqz*dxxpzpz2              +rdxxpz2+dpxxz12
      azd1= qcy*dxxpzpx1+wqy*dxxpzpx2                              
      aze1= qcz*dxxpzpx1+wqz*dxxpzpx2              +rdxxpx2        
      azf1= qcy*dxxpzpz1+wqy*dxxpzpz2
      bxa1= qcx*dyypxpx1+wqx*dyypxpx2              +rdyypx2+dpyyx12
      bxb1= qcy*dyypxpy1+wqy*dyypxpy2+2.d0*rpppyxy2        +dpyyx12
      bxc1= qcz*dyypxpz1+wqz*dyypxpz2                      +dpyyx12
      bxd1= qcx*dyypxpy1+wqx*dyypxpy2              +rdyypy2        
      bxe1= qcx*dyypxpz1+wqx*dyypxpz2              +rdyypz2        
      bxf1= qcz*dyypxpy1+wqz*dyypxpy2                              
      bya1= qcx*dyypypx1+wqx*dyypypx2                      +dpyyy12
      byb1= qcy*dyypypy1+wqy*dyypypy2+2.d0*rpppyyy2+rdyypy2+dpyyy12
      byc1= qcz*dyypypz1+wqz*dyypypz2                      +dpyyy12
      byd1= qcx*dyypypy1+wqx*dyypypy2                              
      bye1= qcx*dyypypz1+wqx*dyypypz2                              
      byf1= qcz*dyypypy1+wqz*dyypypy2                              
      bza1= qcx*dyypzpx1+wqx*dyypzpx2                      +dpyyz12
      bzb1= qcy*dyypzpy1+wqy*dyypzpy2+2.d0*rpppyzy2        +dpyyz12
      bzc1= qcz*dyypzpz1+wqz*dyypzpz2              +rdyypz2+dpyyz12
      bzd1= qcx*dyypzpy1+wqx*dyypzpy2                              
      bze1= qcx*dyypzpz1+wqx*dyypzpz2                              
      bzf1= qcz*dyypzpy1+wqz*dyypzpy2              +rdyypy2        
      cxa1= qcx*dzzpxpx1+wqx*dzzpxpx2              +rdzzpx2+dpzzx12
      cxb1= qcy*dzzpxpy1+wqy*dzzpxpy2                      +dpzzx12
      cxc1= qcz*dzzpxpz1+wqz*dzzpxpz2+2.d0*rpppzxz2        +dpzzx12
      cxd1= qcy*dzzpxpx1+wqy*dzzpxpx2                              
      cxe1= qcx*dzzpxpz1+wqx*dzzpxpz2              +rdzzpz2        
      cxf1= qcy*dzzpxpz1+wqy*dzzpxpz2                              
      cya1= qcx*dzzpypx1+wqx*dzzpypx2                      +dpzzy12
      cyb1= qcy*dzzpypy1+wqy*dzzpypy2              +rdzzpy2+dpzzy12
      cyc1= qcz*dzzpypz1+wqz*dzzpypz2+2.d0*rpppzyz2        +dpzzy12
      cyd1= qcx*dzzpypy1+wqx*dzzpypy2                              
      cye1= qcx*dzzpypz1+wqx*dzzpypz2                              
      cyf1= qcy*dzzpypz1+wqy*dzzpypz2              +rdzzpz2        
      cza1= qcx*dzzpzpx1+wqx*dzzpzpx2                      +dpzzz12
      czb1= qcy*dzzpzpy1+wqy*dzzpzpy2                      +dpzzz12
      czc1= qcz*dzzpzpz1+wqz*dzzpzpz2+2.d0*rpppzzz2+rdzzpz2+dpzzz12
      czd1= qcx*dzzpzpy1+wqx*dzzpzpy2                              
      cze1= qcx*dzzpzpz1+wqx*dzzpzpz2                              
      czf1= qcy*dzzpzpz1+wqy*dzzpzpz2                              
      dxa1= qcx*dyxpxpx1+wqx*dyxpxpx2+     rpppyxx2+rdyxpx2+dpyxx12
      dxb1= qcy*dyxpxpy1+wqy*dyxpxpy2+     rpppxxy2        +dpyxx12
      dxc1= qcz*dyxpxpz1+wqz*dyxpxpz2                      +dpyxx12
      dxd1= qcx*dyxpxpy1+wqx*dyxpxpy2+     rpppyxy2+rdyxpy2        
      dxe1= qcx*dyxpxpz1+wqx*dyxpxpz2+     rpppyxz2+rdyxpz2        
      dxf1= qcz*dyxpxpy1+wqz*dyxpxpy2                              
      dya1= qcx*dyxpypx1+wqx*dyxpypx2+     rpppyyx2        +dpyxy12
      dyb1= qcy*dyxpypy1+wqy*dyxpypy2+     rpppxyy2+rdyxpy2+dpyxy12
      dyc1= qcz*dyxpypz1+wqz*dyxpypz2                      +dpyxy12
      dyd1= qcx*dyxpypy1+wqx*dyxpypy2+     rpppyyy2                
      dye1= qcz*dyxpypx1+wqz*dyxpypx2                              
      dyf1= qcz*dyxpypy1+wqz*dyxpypy2                              
      dza1= qcx*dyxpzpx1+wqx*dyxpzpx2+     rpppyzx2        +dpyxz12
      dzb1= qcy*dyxpzpy1+wqy*dyxpzpy2+     rpppxzy2        +dpyxz12
      dzc1= qcz*dyxpzpz1+wqz*dyxpzpz2              +rdyxpz2+dpyxz12
      dzd1= qcx*dyxpzpy1+wqx*dyxpzpy2+     rpppyzy2               
      dze1= qcx*dyxpzpz1+wqx*dyxpzpz2+     rpppyzz2                
      dzf1= qcy*dyxpzpz1+wqy*dyxpzpz2+     rpppxzz2                
      exa1= qcx*dzxpxpx1+wqx*dzxpxpx2+     rpppzxx2+rdzxpx2+dpzxx12
      exb1= qcy*dzxpxpy1+wqy*dzxpxpy2                      +dpzxx12
      exc1= qcz*dzxpxpz1+wqz*dzxpxpz2+     rpppxxz2        +dpzxx12
      exd1= qcy*dzxpxpx1+wqy*dzxpxpx2                              
      exe1= qcz*dzxpxpx1+wqz*dzxpxpx2+     rpppxxx2                
      exf1= qcy*dzxpxpz1+wqy*dzxpxpz2                              
      eya1= qcx*dzxpypx1+wqx*dzxpypx2+     rpppzyx2        +dpzxy12
      eyb1= qcy*dzxpypy1+wqy*dzxpypy2              +rdzxpy2+dpzxy12
      eyc1= qcz*dzxpypz1+wqz*dzxpypz2+     rpppxyz2        +dpzxy12
      eyd1= qcx*dzxpypy1+wqx*dzxpypy2+     rpppzyy2                
      eye1= qcz*dzxpypx1+wqz*dzxpypx2+     rpppxyx2                
      eyf1= qcz*dzxpypy1+wqz*dzxpypy2+     rpppxyy2                
      eza1= qcx*dzxpzpx1+wqx*dzxpzpx2+     rpppzzx2        +dpzxz12
      ezb1= qcy*dzxpzpy1+wqy*dzxpzpy2                      +dpzxz12
      ezc1= qcz*dzxpzpz1+wqz*dzxpzpz2+     rpppxzz2+rdzxpz2+dpzxz12
      ezd1= qcy*dzxpzpx1+wqy*dzxpzpx2
      eze1= qcx*dzxpzpz1+wqx*dzxpzpz2+     rpppzzz2                
      ezf1= qcy*dzxpzpz1+wqy*dzxpzpz2                              
      fxa1= qcx*dzypxpx1+wqx*dzypxpx2              +rdzypx2+dpzyx12
      fxb1= qcy*dzypxpy1+wqy*dzypxpy2+     rpppzxy2        +dpzyx12
      fxc1= qcz*dzypxpz1+wqz*dzypxpz2+     rpppyxz2        +dpzyx12
      fxd1= qcy*dzypxpx1+wqy*dzypxpx2+     rpppzxx2                
      fxe1= qcz*dzypxpx1+wqz*dzypxpx2+     rpppyxx2                
      fxf1= qcz*dzypxpy1+wqz*dzypxpy2+     rpppyxy2                
      fya1= qcx*dzypypx1+wqx*dzypypx2                      +dpzyy12
      fyb1= qcy*dzypypy1+wqy*dzypypy2+     rpppzyy2+rdzypy2+dpzyy12
      fyc1= qcz*dzypypz1+wqz*dzypypz2+     rpppyyz2        +dpzyy12
      fyd1= qcx*dzypypy1+wqx*dzypypy2                              
      fye1= qcx*dzypypz1+wqx*dzypypz2                              
      fyf1= qcz*dzypypy1+wqz*dzypypy2+     rpppyyy2                
      fza1= qcx*dzypzpx1+wqx*dzypzpx2                      +dpzyz12
      fzb1= qcy*dzypzpy1+wqy*dzypzpy2+     rpppzzy2        +dpzyz12
      fzc1= qcz*dzypzpz1+wqz*dzypzpz2+     rpppyzz2+rdzypz2+dpzyz12
      fzd1= qcx*dzypzpy1+wqx*dzypzpy2                      
      fze1= qcx*dzypzpz1+wqx*dzypzpz2                      
      fzf1= qcy*dzypzpz1+wqy*dzypzpz2+     rpppzzz2        
  

!RANK=6
      pxpxdxx1 = rkp * xxdxx1
      pxpxdyy1 = rkp * xxdyy1
      pxpxdzz1 = rkp * xxdzz1
      pxpxdyx1 = rkp * xxdyx1
      pxpxdzx1 = rkp * xxdzx1
      pxpxdzy1 = rkp * xxdzy1
      pxpydxx1 = rkp * xydxx1
      pxpydyy1 = rkp * xydyy1
      pxpydzz1 = rkp * xydzz1 
      pxpydyx1 = rkp * xydyx1 
      pxpydzx1 = rkp * xydzx1 
      pxpydzy1 = rkp * xydzy1 
      pxpzdxx1 = rkp * xzdxx1 
      pxpzdyy1 = rkp * xzdyy1 
      pxpzdzz1 = rkp * xzdzz1 
      pxpzdyx1 = rkp * xzdyx1 
      pxpzdzx1 = rkp * xzdzx1
      pxpzdzy1 = rkp * xzdzy1 
      pypxdxx1 = rkp * yxdxx1 
      pypxdyy1 = rkp * yxdyy1 
      pypxdzz1 = rkp * yxdzz1 
      pypxdyx1 = rkp * yxdyx1 
      pypxdzx1 = rkp * yxdzx1 
      pypxdzy1 = rkp * yxdzy1 
      pypydxx1 = rkp * yydxx1 
      pypydyy1 = rkp * yydyy1 
      pypydzz1 = rkp * yydzz1 
      pypydyx1 = rkp * yydyx1 
      pypydzx1 = rkp * yydzx1 
      pypydzy1 = rkp * yydzy1 
      pypzdxx1 = rkp * yzdxx1 
      pypzdyy1 = rkp * yzdyy1 
      pypzdzz1 = rkp * yzdzz1 
      pypzdyx1 = rkp * yzdyx1 
      pypzdzx1 = rkp * yzdzx1 
      pypzdzy1 = rkp * yzdzy1 
      pzpxdxx1 = rkp * zxdxx1 
      pzpxdyy1 = rkp * zxdyy1 
      pzpxdzz1 = rkp * zxdzz1 
      pzpxdyx1 = rkp * zxdyx1 
      pzpxdzx1 = rkp * zxdzx1 
      pzpxdzy1 = rkp * zxdzy1 
      pzpydxx1 = rkp * zydxx1 
      pzpydyy1 = rkp * zydyy1 
      pzpydzz1 = rkp * zydzz1
      pzpydyx1 = rkp * zydyx1 
      pzpydzx1 = rkp * zydzx1
      pzpydzy1 = rkp * zydzy1 
      pzpzdxx1 = rkp * zzdxx1
      pzpzdyy1 = rkp * zzdyy1
      pzpzdzz1 = rkp * zzdzz1
      pzpzdyx1 = rkp * zzdyx1
      pzpzdzx1 = rkp * zzdzx1
      pzpzdzy1 = rkp * zzdzy1

      aa1 = rkp * dxxdxx1
      ab1 = rkp * dxxdyy1
      ac1 = rkp * dxxdzz1
      ad1 = rkp * dxxdyx1
      ae1 = rkp * dxxdzx1
      af1 = rkp * dxxdzy1
      ba1 = rkp * dyydxx1
      bb1 = rkp * dyydyy1
      bc1 = rkp * dyydzz1
      bd1 = rkp * dyydyx1
      be1 = rkp * dyydzx1
      bf1 = rkp * dyydzy1
      ca1 = rkp * dzzdxx1
      cb1 = rkp * dzzdyy1
      cc1 = rkp * dzzdzz1
      cd1 = rkp * dzzdyx1
      ce1 = rkp * dzzdzx1
      cf1 = rkp * dzzdzy1
      da1 = rkp * dyxdxx1
      db1 = rkp * dyxdyy1
      dc1 = rkp * dyxdzz1
      dd1 = rkp * dyxdyx1
      de1 = rkp * dyxdzx1
      df1 = rkp * dyxdzy1
      ea1 = rkp * dzxdxx1
      eb1 = rkp * dzxdyy1
      ec1 = rkp * dzxdzz1
      ed1 = rkp * dzxdyx1
      ee1 = rkp * dzxdzx1
      ef1 = rkp * dzxdzy1
      fa1 = rkp * dzydxx1
      fb1 = rkp * dzydyy1
      fc1 = rkp * dzydzz1
      fd1 = rkp * dzydyx1
      fe1 = rkp * dzydzx1
      ff1 = rkp * dzydzy1

      xxxx= reta*( dxxpxpx0 -rooe*   dxxpxpx1)
      xxxy= reta*( dxxpxpy0 -rooe*   dxxpxpy1)
      xxxz= reta*( dxxpxpz0 -rooe*   dxxpxpz1)
      xxyx= reta*( dxxpypx0 -rooe*   dxxpypx1)
      xxyy= reta*( dxxpypy0 -rooe*   dxxpypy1)
      xxyz= reta*( dxxpypz0 -rooe*   dxxpypz1)
      xxzx= reta*( dxxpzpx0 -rooe*   dxxpzpx1)
      xxzy= reta*( dxxpzpy0 -rooe*   dxxpzpy1)
      xxzz= reta*( dxxpzpz0 -rooe*   dxxpzpz1)
      yyxx= reta*( dyypxpx0 -rooe*   dyypxpx1)
      yyxy= reta*( dyypxpy0 -rooe*   dyypxpy1)
      yyxz= reta*( dyypxpz0 -rooe*   dyypxpz1)
      yyyx= reta*( dyypypx0 -rooe*   dyypypx1)
      yyyy= reta*( dyypypy0 -rooe*   dyypypy1)
      yyyz= reta*( dyypypz0 -rooe*   dyypypz1)
      yyzx= reta*( dyypzpx0 -rooe*   dyypzpx1)
      yyzy= reta*( dyypzpy0 -rooe*   dyypzpy1)
      yyzz= reta*( dyypzpz0 -rooe*   dyypzpz1)
      zzxx= reta*( dzzpxpx0 -rooe*   dzzpxpx1)
      zzxy= reta*( dzzpxpy0 -rooe*   dzzpxpy1)
      zzxz= reta*( dzzpxpz0 -rooe*   dzzpxpz1)
      zzyx= reta*( dzzpypx0 -rooe*   dzzpypx1)
      zzyy= reta*( dzzpypy0 -rooe*   dzzpypy1)
      zzyz= reta*( dzzpypz0 -rooe*   dzzpypz1)
      zzzx= reta*( dzzpzpx0 -rooe*   dzzpzpx1)
      zzzy= reta*( dzzpzpy0 -rooe*   dzzpzpy1)
      zzzz= reta*( dzzpzpz0 -rooe*   dzzpzpz1)
      yxxx= reta*( dyxpxpx0 -rooe*   dyxpxpx1)
      yxxy= reta*( dyxpxpy0 -rooe*   dyxpxpy1)
      yxxz= reta*( dyxpxpz0 -rooe*   dyxpxpz1)
      yxyx= reta*( dyxpypx0 -rooe*   dyxpypx1)
      yxyy= reta*( dyxpypy0 -rooe*   dyxpypy1)
      yxyz= reta*( dyxpypz0 -rooe*   dyxpypz1)
      yxzx= reta*( dyxpzpx0 -rooe*   dyxpzpx1)
      yxzy= reta*( dyxpzpy0 -rooe*   dyxpzpy1)
      yxzz= reta*( dyxpzpz0 -rooe*   dyxpzpz1)
      zxxx= reta*( dzxpxpx0 -rooe*   dzxpxpx1)
      zxxy= reta*( dzxpxpy0 -rooe*   dzxpxpy1)
      zxxz= reta*( dzxpxpz0 -rooe*   dzxpxpz1)
      zxyx= reta*( dzxpypx0 -rooe*   dzxpypx1)
      zxyy= reta*( dzxpypy0 -rooe*   dzxpypy1)
      zxyz= reta*( dzxpypz0 -rooe*   dzxpypz1)
      zxzx= reta*( dzxpzpx0 -rooe*   dzxpzpx1)
      zxzy= reta*( dzxpzpy0 -rooe*   dzxpzpy1)
      zxzz= reta*( dzxpzpz0 -rooe*   dzxpzpz1)
      zyxx= reta*( dzypxpx0 -rooe*   dzypxpx1)
      zyxy= reta*( dzypxpy0 -rooe*   dzypxpy1)
      zyxz= reta*( dzypxpz0 -rooe*   dzypxpz1)
      zyyx= reta*( dzypypx0 -rooe*   dzypypx1)
      zyyy= reta*( dzypypy0 -rooe*   dzypypy1)
      zyyz= reta*( dzypypz0 -rooe*   dzypypz1)
      zyzx= reta*( dzypzpx0 -rooe*   dzypzpx1)
      zyzy= reta*( dzypzpy0 -rooe*   dzypzpy1)
      zyzz= reta*( dzypzpz0 -rooe*   dzypzpz1)

      bb = 4.d0*za(i)*za(k)
!     bb = 1.d0
      ec(  1)=ec(  1)+bb*(qdx*axa0+wqx*axa1+2.d0*pxpxdxx1+aa1+2.d0*xxxx)
      ec(  2)=ec(  2)+bb*(qdy*axa0+wqy*axa1)
      ec(  3)=ec(  3)+bb*(qdz*axa0+wqz*axa1)
      ec(  4)=ec(  4)+bb*(qdx*axb0+wqx*axb1+2.d0*pxpxdyy1+ab1)
      ec(  5)=ec(  5)+bb*(qdy*axb0+wqy*axb1                  +2.d0*xxxy)
      ec(  6)=ec(  6)+bb*(qdz*axb0+wqz*axb1                      )
      ec(  7)=ec(  7)+bb*(qdx*axc0+wqx*axc1+2.d0*pxpxdzz1+ac1)
      ec(  8)=ec(  8)+bb*(qdy*axc0+wqy*axc1                   )   
      ec(  9)=ec(  9)+bb*(qdz*axc0+wqz*axc1                  +2.d0*xxxz)
      ec( 10)=ec( 10)+bb*(qdx*axd0+wqx*axd1+2.d0*pxpxdyx1+ad1+     xxxy)
      ec( 11)=ec( 11)+bb*(qdy*axd0+wqy*axd1                  +     xxxx)
      ec( 12)=ec( 12)+bb*(qdz*axd0+wqz*axd1                      )
      ec( 13)=ec( 13)+bb*(qdx*axe0+wqx*axe1+2.d0*pxpxdzx1+ae1+     xxxz)
      ec( 14)=ec( 14)+bb*(qdy*axe0+wqy*axe1                      )
      ec( 15)=ec( 15)+bb*(qdz*axe0+wqz*axe1                  +     xxxx)
      ec( 16)=ec( 16)+bb*(qdx*axf0+wqx*axf1+2.d0*pxpxdzy1+af1)
      ec( 17)=ec( 17)+bb*(qdy*axf0+wqy*axf1                  +     xxxz)
      ec( 18)=ec( 18)+bb*(qdz*axf0+wqz*axf1                  +     xxxy)
      ec( 19)=ec( 19)+bb*(qdx*aya0+wqx*aya1+2.d0*pxpydxx1    +2.d0*xxyx)
      ec( 20)=ec( 20)+bb*(qdy*aya0+wqy*aya1              +aa1)
      ec( 21)=ec( 21)+bb*(qdz*aya0+wqz*aya1)
      ec( 22)=ec( 22)+bb*(qdx*ayb0+wqx*ayb1+2.d0*pxpydyy1)
      ec( 23)=ec( 23)+bb*(qdy*ayb0+wqy*ayb1              +ab1+2.d0*xxyy)
      ec( 24)=ec( 24)+bb*(qdz*ayb0+wqz*ayb1                      )
      ec( 25)=ec( 25)+bb*(qdx*ayc0+wqx*ayc1+2.d0*pxpydzz1        )
      ec( 26)=ec( 26)+bb*(qdy*ayc0+wqy*ayc1              +ac1)
      ec( 27)=ec( 27)+bb*(qdz*ayc0+wqz*ayc1                  +2.d0*xxyz)
      ec( 28)=ec( 28)+bb*(qdx*ayd0+wqx*ayd1+2.d0*pxpydyx1    +     xxyy)
      ec( 29)=ec( 29)+bb*(qdy*ayd0+wqy*ayd1              +ad1+     xxyx)
      ec( 30)=ec( 30)+bb*(qdz*ayd0+wqz*ayd1                      )
      ec( 31)=ec( 31)+bb*(qdx*aye0+wqx*aye1+2.d0*pxpydzx1    +     xxyz)
      ec( 32)=ec( 32)+bb*(qdy*aye0+wqy*aye1              +ae1)
      ec( 33)=ec( 33)+bb*(qdz*aye0+wqz*aye1                  +     xxyx)
      ec( 34)=ec( 34)+bb*(qdx*ayf0+wqx*ayf1+2.d0*pxpydzy1        )
      ec( 35)=ec( 35)+bb*(qdy*ayf0+wqy*ayf1              +af1+     xxyz)
      ec( 36)=ec( 36)+bb*(qdz*ayf0+wqz*ayf1                  +     xxyy)
      ec( 37)=ec( 37)+bb*(qdx*aza0+wqx*aza1+2.d0*pxpzdxx1    +2.d0*xxzx)
      ec( 38)=ec( 38)+bb*(qdy*aza0+wqy*aza1                      )
      ec( 39)=ec( 39)+bb*(qdz*aza0+wqz*aza1              +aa1)
      ec( 40)=ec( 40)+bb*(qdx*azb0+wqx*azb1+2.d0*pxpzdyy1     )   
      ec( 41)=ec( 41)+bb*(qdy*azb0+wqy*azb1                  +2.d0*xxzy)
      ec( 42)=ec( 42)+bb*(qdz*azb0+wqz*azb1              +ab1)
      ec( 43)=ec( 43)+bb*(qdx*azc0+wqx*azc1+2.d0*pxpzdzz1     )   
      ec( 44)=ec( 44)+bb*(qdy*azc0+wqy*azc1                    )  
      ec( 45)=ec( 45)+bb*(qdz*azc0+wqz*azc1              +ac1+2.d0*xxzz)
      ec( 46)=ec( 46)+bb*(qdx*azd0+wqx*azd1+2.d0*pxpzdyx1    +     xxzy)
      ec( 47)=ec( 47)+bb*(qdy*azd0+wqy*azd1                  +     xxzx)
      ec( 48)=ec( 48)+bb*(qdz*azd0+wqz*azd1              +ad1)
      ec( 49)=ec( 49)+bb*(qdx*aze0+wqx*aze1+2.d0*pxpzdzx1    +     xxzz)
      ec( 50)=ec( 50)+bb*(qdy*aze0+wqy*aze1                 ) 
      ec( 51)=ec( 51)+bb*(qdz*aze0+wqz*aze1              +ae1+     xxzx)
      ec( 52)=ec( 52)+bb*(qdx*azf0+wqx*azf1+2.d0*pxpzdzy1   )     
      ec( 53)=ec( 53)+bb*(qdy*azf0+wqy*azf1                  +     xxzz)
      ec( 54)=ec( 54)+bb*(qdz*azf0+wqz*azf1              +af1+     xxzy)
      ec( 55)=ec( 55)+bb*(qdx*bxa0+wqx*bxa1              +ba1+2.d0*yyxx)
      ec( 56)=ec( 56)+bb*(qdy*bxa0+wqy*bxa1+2.d0*pypxdxx1        )
      ec( 57)=ec( 57)+bb*(qdz*bxa0+wqz*bxa1                      )
      ec( 58)=ec( 58)+bb*(qdx*bxb0+wqx*bxb1              +bb1)
      ec( 59)=ec( 59)+bb*(qdy*bxb0+wqy*bxb1+2.d0*pypxdyy1    +2.d0*yyxy)
      ec( 60)=ec( 60)+bb*(qdz*bxb0+wqz*bxb1                   )   
      ec( 61)=ec( 61)+bb*(qdx*bxc0+wqx*bxc1              +bc1)
      ec( 62)=ec( 62)+bb*(qdy*bxc0+wqy*bxc1+2.d0*pypxdzz1     )   
      ec( 63)=ec( 63)+bb*(qdz*bxc0+wqz*bxc1                  +2.d0*yyxz)
      ec( 64)=ec( 64)+bb*(qdx*bxd0+wqx*bxd1              +bd1+     yyxy)
      ec( 65)=ec( 65)+bb*(qdy*bxd0+wqy*bxd1+2.d0*pypxdyx1    +     yyxx)
      ec( 66)=ec( 66)+bb*(qdz*bxd0+wqz*bxd1                 )     
      ec( 67)=ec( 67)+bb*(qdx*bxe0+wqx*bxe1              +be1+     yyxz)
      ec( 68)=ec( 68)+bb*(qdy*bxe0+wqy*bxe1+2.d0*pypxdzx1   )     
      ec( 69)=ec( 69)+bb*(qdz*bxe0+wqz*bxe1                  +     yyxx)
      ec( 70)=ec( 70)+bb*(qdx*bxf0+wqx*bxf1              +bf1)
      ec( 71)=ec( 71)+bb*(qdy*bxf0+wqy*bxf1+2.d0*pypxdzy1    +     yyxz)
      ec( 72)=ec( 72)+bb*(qdz*bxf0+wqz*bxf1                  +     yyxy)
      ec( 73)=ec( 73)+bb*(qdx*bya0+wqx*bya1                  +2.d0*yyyx)
      ec( 74)=ec( 74)+bb*(qdy*bya0+wqy*bya1+2.d0*pypydxx1+ba1)
      ec( 75)=ec( 75)+bb*(qdz*bya0+wqz*bya1                   )   
      ec( 76)=ec( 76)+bb*(qdx*byb0+wqx*byb1                    )  
      ec( 77)=ec( 77)+bb*(qdy*byb0+wqy*byb1+2.d0*pypydyy1+bb1+2.d0*yyyy)
      ec( 78)=ec( 78)+bb*(qdz*byb0+wqz*byb1)
      ec( 79)=ec( 79)+bb*(qdx*byc0+wqx*byc1 )                     
      ec( 80)=ec( 80)+bb*(qdy*byc0+wqy*byc1+2.d0*pypydzz1+bc1)
      ec( 81)=ec( 81)+bb*(qdz*byc0+wqz*byc1                  +2.d0*yyyz)
      ec( 82)=ec( 82)+bb*(qdx*byd0+wqx*byd1                  +     yyyy)
      ec( 83)=ec( 83)+bb*(qdy*byd0+wqy*byd1+2.d0*pypydyx1+bd1+     yyyx)
      ec( 84)=ec( 84)+bb*(qdz*byd0+wqz*byd1                  )
      ec( 85)=ec( 85)+bb*(qdx*bye0+wqx*bye1                  +     yyyz)
      ec( 86)=ec( 86)+bb*(qdy*bye0+wqy*bye1+2.d0*pypydzx1+be1)
      ec( 87)=ec( 87)+bb*(qdz*bye0+wqz*bye1                  +     yyyx)
      ec( 88)=ec( 88)+bb*(qdx*byf0+wqx*byf1                  )
      ec( 89)=ec( 89)+bb*(qdy*byf0+wqy*byf1+2.d0*pypydzy1+bf1+     yyyz)
      ec( 90)=ec( 90)+bb*(qdz*byf0+wqz*byf1                  +     yyyy)
      ec( 91)=ec( 91)+bb*(qdx*bza0+wqx*bza1                  +2.d0*yyzx)
      ec( 92)=ec( 92)+bb*(qdy*bza0+wqy*bza1+2.d0*pypzdxx1        )
      ec( 93)=ec( 93)+bb*(qdz*bza0+wqz*bza1              +ba1)
      ec( 94)=ec( 94)+bb*(qdx*bzb0+wqx*bzb1                   )   
      ec( 95)=ec( 95)+bb*(qdy*bzb0+wqy*bzb1+2.d0*pypzdyy1    +2.d0*yyzy)
      ec( 96)=ec( 96)+bb*(qdz*bzb0+wqz*bzb1              +bb1)
      ec( 97)=ec( 97)+bb*(qdx*bzc0+wqx*bzc1                   )   
      ec( 98)=ec( 98)+bb*(qdy*bzc0+wqy*bzc1+2.d0*pypzdzz1      )  
      ec( 99)=ec( 99)+bb*(qdz*bzc0+wqz*bzc1              +bc1+2.d0*yyzz)
      ec(100)=ec(100)+bb*(qdx*bzd0+wqx*bzd1                  +     yyzy)
      ec(101)=ec(101)+bb*(qdy*bzd0+wqy*bzd1+2.d0*pypzdyx1    +     yyzx)
      ec(102)=ec(102)+bb*(qdz*bzd0+wqz*bzd1              +bd1)
      ec(103)=ec(103)+bb*(qdx*bze0+wqx*bze1                  +     yyzz)
      ec(104)=ec(104)+bb*(qdy*bze0+wqy*bze1+2.d0*pypzdzx1    )
      ec(105)=ec(105)+bb*(qdz*bze0+wqz*bze1              +be1+     yyzx)
      ec(106)=ec(106)+bb*(qdx*bzf0+wqx*bzf1                      )
      ec(107)=ec(107)+bb*(qdy*bzf0+wqy*bzf1+2.d0*pypzdzy1    +     yyzz)
      ec(108)=ec(108)+bb*(qdz*bzf0+wqz*bzf1              +bf1+     yyzy)
      ec(109)=ec(109)+bb*(qdx*cxa0+wqx*cxa1              +ca1+2.d0*zzxx)
      ec(110)=ec(110)+bb*(qdy*cxa0+wqy*cxa1                      )
      ec(111)=ec(111)+bb*(qdz*cxa0+wqz*cxa1+2.d0*pzpxdxx1        )
      ec(112)=ec(112)+bb*(qdx*cxb0+wqx*cxb1              +cb1)
      ec(113)=ec(113)+bb*(qdy*cxb0+wqy*cxb1                  +2.d0*zzxy)
      ec(114)=ec(114)+bb*(qdz*cxb0+wqz*cxb1+2.d0*pzpxdyy1        )
      ec(115)=ec(115)+bb*(qdx*cxc0+wqx*cxc1              +cc1)
      ec(116)=ec(116)+bb*(qdy*cxc0+wqy*cxc1                   )   
      ec(117)=ec(117)+bb*(qdz*cxc0+wqz*cxc1+2.d0*pzpxdzz1    +2.d0*zzxz)
      ec(118)=ec(118)+bb*(qdx*cxd0+wqx*cxd1              +cd1+     zzxy)
      ec(119)=ec(119)+bb*(qdy*cxd0+wqy*cxd1                  +     zzxx)
      ec(120)=ec(120)+bb*(qdz*cxd0+wqz*cxd1+2.d0*pzpxdyx1        )
      ec(121)=ec(121)+bb*(qdx*cxe0+wqx*cxe1              +ce1+     zzxz)
      ec(122)=ec(122)+bb*(qdy*cxe0+wqy*cxe1                     )
      ec(123)=ec(123)+bb*(qdz*cxe0+wqz*cxe1+2.d0*pzpxdzx1    +     zzxx)
      ec(124)=ec(124)+bb*(qdx*cxf0+wqx*cxf1              +cf1)
      ec(125)=ec(125)+bb*(qdy*cxf0+wqy*cxf1                  +     zzxz)
      ec(126)=ec(126)+bb*(qdz*cxf0+wqz*cxf1+2.d0*pzpxdzy1    +     zzxy)
      ec(127)=ec(127)+bb*(qdx*cya0+wqx*cya1                  +2.d0*zzyx)
      ec(128)=ec(128)+bb*(qdy*cya0+wqy*cya1              +ca1)
      ec(129)=ec(129)+bb*(qdz*cya0+wqz*cya1+2.d0*pzpydxx1     )   
      ec(130)=ec(130)+bb*(qdx*cyb0+wqx*cyb1                    )  
      ec(131)=ec(131)+bb*(qdy*cyb0+wqy*cyb1              +cb1+2.d0*zzyy)
      ec(132)=ec(132)+bb*(qdz*cyb0+wqz*cyb1+2.d0*pzpydyy1        )
      ec(133)=ec(133)+bb*(qdx*cyc0+wqx*cyc1                      )
      ec(134)=ec(134)+bb*(qdy*cyc0+wqy*cyc1              +cc1)
      ec(135)=ec(135)+bb*(qdz*cyc0+wqz*cyc1+2.d0*pzpydzz1    +2.d0*zzyz)
      ec(136)=ec(136)+bb*(qdx*cyd0+wqx*cyd1                  +     zzyy)
      ec(137)=ec(137)+bb*(qdy*cyd0+wqy*cyd1              +cd1+     zzyx)
      ec(138)=ec(138)+bb*(qdz*cyd0+wqz*cyd1+2.d0*pzpydyx1        )
      ec(139)=ec(139)+bb*(qdx*cye0+wqx*cye1                  +     zzyz)
      ec(140)=ec(140)+bb*(qdy*cye0+wqy*cye1              +ce1 )
      ec(141)=ec(141)+bb*(qdz*cye0+wqz*cye1+2.d0*pzpydzx1    +     zzyx)
      ec(142)=ec(142)+bb*(qdx*cyf0+wqx*cyf1                      )
      ec(143)=ec(143)+bb*(qdy*cyf0+wqy*cyf1              +cf1+     zzyz)
      ec(144)=ec(144)+bb*(qdz*cyf0+wqz*cyf1+2.d0*pzpydzy1    +     zzyy)
      ec(145)=ec(145)+bb*(qdx*cza0+wqx*cza1                  +2.d0*zzzx)
      ec(146)=ec(146)+bb*(qdy*cza0+wqy*cza1                      )
      ec(147)=ec(147)+bb*(qdz*cza0+wqz*cza1+2.d0*pzpzdxx1+ca1)
      ec(148)=ec(148)+bb*(qdx*czb0+wqx*czb1                   )   
      ec(149)=ec(149)+bb*(qdy*czb0+wqy*czb1                  +2.d0*zzzy)
      ec(150)=ec(150)+bb*(qdz*czb0+wqz*czb1+2.d0*pzpzdyy1+cb1)
      ec(151)=ec(151)+bb*(qdx*czc0+wqx*czc1                   )   
      ec(152)=ec(152)+bb*(qdy*czc0+wqy*czc1                    )  
      ec(153)=ec(153)+bb*(qdz*czc0+wqz*czc1+2.d0*pzpzdzz1+cc1+2.d0*zzzz)
      ec(154)=ec(154)+bb*(qdx*czd0+wqx*czd1                  +     zzzy)
      ec(155)=ec(155)+bb*(qdy*czd0+wqy*czd1                  +     zzzx)
      ec(156)=ec(156)+bb*(qdz*czd0+wqz*czd1+2.d0*pzpzdyx1+cd1)
      ec(157)=ec(157)+bb*(qdx*cze0+wqx*cze1                  +     zzzz)
      ec(158)=ec(158)+bb*(qdy*cze0+wqy*cze1                   )
      ec(159)=ec(159)+bb*(qdz*cze0+wqz*cze1+2.d0*pzpzdzx1+ce1+     zzzx)
      ec(160)=ec(160)+bb*(qdx*czf0+wqx*czf1                  )
      ec(161)=ec(161)+bb*(qdy*czf0+wqy*czf1                  +     zzzz)
      ec(162)=ec(162)+bb*(qdz*czf0+wqz*czf1+2.d0*pzpzdzy1+cf1+     zzzy)
      ec(163)=ec(163)+bb*(qdx*dxa0+wqx*dxa1+     pypxdxx1+da1+2.d0*yxxx)
      ec(164)=ec(164)+bb*(qdy*dxa0+wqy*dxa1+     pxpxdxx1)
      ec(165)=ec(165)+bb*(qdz*dxa0+wqz*dxa1              )
      ec(166)=ec(166)+bb*(qdx*dxb0+wqx*dxb1+     pypxdyy1+db1)
      ec(167)=ec(167)+bb*(qdy*dxb0+wqy*dxb1+     pxpxdyy1    +2.d0*yxxy)
      ec(168)=ec(168)+bb*(qdz*dxb0+wqz*dxb1                      )
      ec(169)=ec(169)+bb*(qdx*dxc0+wqx*dxc1+     pypxdzz1+dc1)
      ec(170)=ec(170)+bb*(qdy*dxc0+wqy*dxc1+     pxpxdzz1     )   
      ec(171)=ec(171)+bb*(qdz*dxc0+wqz*dxc1                  +2.d0*yxxz)
      ec(172)=ec(172)+bb*(qdx*dxd0+wqx*dxd1+     pypxdyx1+dd1+     yxxy)
      ec(173)=ec(173)+bb*(qdy*dxd0+wqy*dxd1+     pxpxdyx1    +     yxxx)
      ec(174)=ec(174)+bb*(qdz*dxd0+wqz*dxd1                      )
      ec(175)=ec(175)+bb*(qdx*dxe0+wqx*dxe1+     pypxdzx1+de1+     yxxz)
      ec(176)=ec(176)+bb*(qdy*dxe0+wqy*dxe1+     pxpxdzx1          )
      ec(177)=ec(177)+bb*(qdz*dxe0+wqz*dxe1                  +     yxxx)
      ec(178)=ec(178)+bb*(qdx*dxf0+wqx*dxf1+     pypxdzy1+df1)
      ec(179)=ec(179)+bb*(qdy*dxf0+wqy*dxf1+     pxpxdzy1    +     yxxz)
      ec(180)=ec(180)+bb*(qdz*dxf0+wqz*dxf1                  +     yxxy)
      ec(181)=ec(181)+bb*(qdx*dya0+wqx*dya1+     pypydxx1    +2.d0*yxyx)
      ec(182)=ec(182)+bb*(qdy*dya0+wqy*dya1+     pxpydxx1+da1)
      ec(183)=ec(183)+bb*(qdz*dya0+wqz*dya1                   )   
      ec(184)=ec(184)+bb*(qdx*dyb0+wqx*dyb1+     pypydyy1      )  
      ec(185)=ec(185)+bb*(qdy*dyb0+wqy*dyb1+     pxpydyy1+db1+2.d0*yxyy)
      ec(186)=ec(186)+bb*(qdz*dyb0+wqz*dyb1                      )
      ec(187)=ec(187)+bb*(qdx*dyc0+wqx*dyc1+     pypydzz1        )
      ec(188)=ec(188)+bb*(qdy*dyc0+wqy*dyc1+     pxpydzz1+dc1)
      ec(189)=ec(189)+bb*(qdz*dyc0+wqz*dyc1                  +2.d0*yxyz)
      ec(190)=ec(190)+bb*(qdx*dyd0+wqx*dyd1+     pypydyx1    +     yxyy)
      ec(191)=ec(191)+bb*(qdy*dyd0+wqy*dyd1+     pxpydyx1+dd1+     yxyx)
      ec(192)=ec(192)+bb*(qdz*dyd0+wqz*dyd1                      )
      ec(193)=ec(193)+bb*(qdx*dye0+wqx*dye1+     pypydzx1    +     yxyz)
      ec(194)=ec(194)+bb*(qdy*dye0+wqy*dye1+     pxpydzx1+de1)
      ec(195)=ec(195)+bb*(qdz*dye0+wqz*dye1                  +     yxyx)
      ec(196)=ec(196)+bb*(qdx*dyf0+wqx*dyf1+     pypydzy1        )
      ec(197)=ec(197)+bb*(qdy*dyf0+wqy*dyf1+     pxpydzy1+df1+     yxyz)
      ec(198)=ec(198)+bb*(qdz*dyf0+wqz*dyf1                  +     yxyy)
      ec(199)=ec(199)+bb*(qdx*dza0+wqx*dza1+     pypzdxx1    +2.d0*yxzx)
      ec(200)=ec(200)+bb*(qdy*dza0+wqy*dza1+     pxpzdxx1        )
      ec(201)=ec(201)+bb*(qdz*dza0+wqz*dza1              +da1)
      ec(202)=ec(202)+bb*(qdx*dzb0+wqx*dzb1+     pypzdyy1     )   
      ec(203)=ec(203)+bb*(qdy*dzb0+wqy*dzb1+     pxpzdyy1    +2.d0*yxzy)
      ec(204)=ec(204)+bb*(qdz*dzb0+wqz*dzb1              +db1)
      ec(205)=ec(205)+bb*(qdx*dzc0+wqx*dzc1+     pypzdzz1        )
      ec(206)=ec(206)+bb*(qdy*dzc0+wqy*dzc1+     pxpzdzz1        )
      ec(207)=ec(207)+bb*(qdz*dzc0+wqz*dzc1              +dc1+2.d0*yxzz)
      ec(208)=ec(208)+bb*(qdx*dzd0+wqx*dzd1+     pypzdyx1    +     yxzy)
      ec(209)=ec(209)+bb*(qdy*dzd0+wqy*dzd1+     pxpzdyx1    +     yxzx)
      ec(210)=ec(210)+bb*(qdz*dzd0+wqz*dzd1              +dd1)
      ec(211)=ec(211)+bb*(qdx*dze0+wqx*dze1+     pypzdzx1    +     yxzz)
      ec(212)=ec(212)+bb*(qdy*dze0+wqy*dze1+     pxpzdzx1)
      ec(213)=ec(213)+bb*(qdz*dze0+wqz*dze1              +de1+     yxzx)
      ec(214)=ec(214)+bb*(qdx*dzf0+wqx*dzf1+     pypzdzy1)
      ec(215)=ec(215)+bb*(qdy*dzf0+wqy*dzf1+     pxpzdzy1    +     yxzz)
      ec(216)=ec(216)+bb*(qdz*dzf0+wqz*dzf1              +df1+     yxzy)
      ec(217)=ec(217)+bb*(qdx*exa0+wqx*exa1+     pzpxdxx1+ea1+2.d0*zxxx)
      ec(218)=ec(218)+bb*(qdy*exa0+wqy*exa1                      )
      ec(219)=ec(219)+bb*(qdz*exa0+wqz*exa1+     pxpxdxx1        )
      ec(220)=ec(220)+bb*(qdx*exb0+wqx*exb1+     pzpxdyy1+eb1)
      ec(221)=ec(221)+bb*(qdy*exb0+wqy*exb1                  +2.d0*zxxy)
      ec(222)=ec(222)+bb*(qdz*exb0+wqz*exb1+     pxpxdyy1        )
      ec(223)=ec(223)+bb*(qdx*exc0+wqx*exc1+     pzpxdzz1+ec1)
      ec(224)=ec(224)+bb*(qdy*exc0+wqy*exc1                   )   
      ec(225)=ec(225)+bb*(qdz*exc0+wqz*exc1+     pxpxdzz1    +2.d0*zxxz)
      ec(226)=ec(226)+bb*(qdx*exd0+wqx*exd1+     pzpxdyx1+ed1+     zxxy)
      ec(227)=ec(227)+bb*(qdy*exd0+wqy*exd1                  +     zxxx)
      ec(228)=ec(228)+bb*(qdz*exd0+wqz*exd1+     pxpxdyx1        )
      ec(229)=ec(229)+bb*(qdx*exe0+wqx*exe1+     pzpxdzx1+ee1+     zxxz)
      ec(230)=ec(230)+bb*(qdy*exe0+wqy*exe1                       )
      ec(231)=ec(231)+bb*(qdz*exe0+wqz*exe1+     pxpxdzx1    +     zxxx)
      ec(232)=ec(232)+bb*(qdx*exf0+wqx*exf1+     pzpxdzy1+ef1)
      ec(233)=ec(233)+bb*(qdy*exf0+wqy*exf1                  +     zxxz)
      ec(234)=ec(234)+bb*(qdz*exf0+wqz*exf1+     pxpxdzy1    +     zxxy)
      ec(235)=ec(235)+bb*(qdx*eya0+wqx*eya1+     pzpydxx1    +2.d0*zxyx)
      ec(236)=ec(236)+bb*(qdy*eya0+wqy*eya1              +ea1)
      ec(237)=ec(237)+bb*(qdz*eya0+wqz*eya1+     pxpydxx1     )   
      ec(238)=ec(238)+bb*(qdx*eyb0+wqx*eyb1+     pzpydyy1      )  
      ec(239)=ec(239)+bb*(qdy*eyb0+wqy*eyb1              +eb1+2.d0*zxyy)
      ec(240)=ec(240)+bb*(qdz*eyb0+wqz*eyb1+     pxpydyy1        )
      ec(241)=ec(241)+bb*(qdx*eyc0+wqx*eyc1+     pzpydzz1        )
      ec(242)=ec(242)+bb*(qdy*eyc0+wqy*eyc1              +ec1)
      ec(243)=ec(243)+bb*(qdz*eyc0+wqz*eyc1+     pxpydzz1    +2.d0*zxyz)
      ec(244)=ec(244)+bb*(qdx*eyd0+wqx*eyd1+     pzpydyx1    +     zxyy)
      ec(245)=ec(245)+bb*(qdy*eyd0+wqy*eyd1              +ed1+     zxyx)
      ec(246)=ec(246)+bb*(qdz*eyd0+wqz*eyd1+     pxpydyx1    )
      ec(247)=ec(247)+bb*(qdx*eye0+wqx*eye1+     pzpydzx1    +     zxyz)
      ec(248)=ec(248)+bb*(qdy*eye0+wqy*eye1              +ee1 )
      ec(249)=ec(249)+bb*(qdz*eye0+wqz*eye1+     pxpydzx1    +     zxyx)
      ec(250)=ec(250)+bb*(qdx*eyf0+wqx*eyf1+     pzpydzy1        )
      ec(251)=ec(251)+bb*(qdy*eyf0+wqy*eyf1              +ef1+     zxyz)
      ec(252)=ec(252)+bb*(qdz*eyf0+wqz*eyf1+     pxpydzy1    +     zxyy)
      ec(253)=ec(253)+bb*(qdx*eza0+wqx*eza1+     pzpzdxx1    +2.d0*zxzx)
      ec(254)=ec(254)+bb*(qdy*eza0+wqy*eza1                     ) 
      ec(255)=ec(255)+bb*(qdz*eza0+wqz*eza1+     pxpzdxx1+ea1)
      ec(256)=ec(256)+bb*(qdx*ezb0+wqx*ezb1+     pzpzdyy1     )   
      ec(257)=ec(257)+bb*(qdy*ezb0+wqy*ezb1                  +2.d0*zxzy)
      ec(258)=ec(258)+bb*(qdz*ezb0+wqz*ezb1+     pxpzdyy1+eb1)
      ec(259)=ec(259)+bb*(qdx*ezc0+wqx*ezc1+     pzpzdzz1     )   
      ec(260)=ec(260)+bb*(qdy*ezc0+wqy*ezc1                    )  
      ec(261)=ec(261)+bb*(qdz*ezc0+wqz*ezc1+     pxpzdzz1+ec1+2.d0*zxzz)
      ec(262)=ec(262)+bb*(qdx*ezd0+wqx*ezd1+     pzpzdyx1    +     zxzy)
      ec(263)=ec(263)+bb*(qdy*ezd0+wqy*ezd1                  +     zxzx)
      ec(264)=ec(264)+bb*(qdz*ezd0+wqz*ezd1+     pxpzdyx1+ed1)
      ec(265)=ec(265)+bb*(qdx*eze0+wqx*eze1+     pzpzdzx1    +     zxzz)
      ec(266)=ec(266)+bb*(qdy*eze0+wqy*eze1                       )
      ec(267)=ec(267)+bb*(qdz*eze0+wqz*eze1+     pxpzdzx1+ee1+     zxzx)
      ec(268)=ec(268)+bb*(qdx*ezf0+wqx*ezf1+     pzpzdzy1     )
      ec(269)=ec(269)+bb*(qdy*ezf0+wqy*ezf1                  +     zxzz)
      ec(270)=ec(270)+bb*(qdz*ezf0+wqz*ezf1+     pxpzdzy1+ef1+     zxzy)
      ec(271)=ec(271)+bb*(qdx*fxa0+wqx*fxa1              +fa1+2.d0*zyxx)
      ec(272)=ec(272)+bb*(qdy*fxa0+wqy*fxa1+     pzpxdxx1        )
      ec(273)=ec(273)+bb*(qdz*fxa0+wqz*fxa1+     pypxdxx1        )
      ec(274)=ec(274)+bb*(qdx*fxb0+wqx*fxb1              +fb1)
      ec(275)=ec(275)+bb*(qdy*fxb0+wqy*fxb1+     pzpxdyy1    +2.d0*zyxy)
      ec(276)=ec(276)+bb*(qdz*fxb0+wqz*fxb1+     pypxdyy1        )
      ec(277)=ec(277)+bb*(qdx*fxc0+wqx*fxc1              +fc1)
      ec(278)=ec(278)+bb*(qdy*fxc0+wqy*fxc1+     pzpxdzz1     )   
      ec(279)=ec(279)+bb*(qdz*fxc0+wqz*fxc1+     pypxdzz1    +2.d0*zyxz)
      ec(280)=ec(280)+bb*(qdx*fxd0+wqx*fxd1              +fd1+     zyxy)
      ec(281)=ec(281)+bb*(qdy*fxd0+wqy*fxd1+     pzpxdyx1    +     zyxx)
      ec(282)=ec(282)+bb*(qdz*fxd0+wqz*fxd1+     pypxdyx1        )
      ec(283)=ec(283)+bb*(qdx*fxe0+wqx*fxe1              +fe1+     zyxz)
      ec(284)=ec(284)+bb*(qdy*fxe0+wqy*fxe1+     pzpxdzx1          )
      ec(285)=ec(285)+bb*(qdz*fxe0+wqz*fxe1+     pypxdzx1    +     zyxx)
      ec(286)=ec(286)+bb*(qdx*fxf0+wqx*fxf1              +ff1)
      ec(287)=ec(287)+bb*(qdy*fxf0+wqy*fxf1+     pzpxdzy1    +     zyxz)
      ec(288)=ec(288)+bb*(qdz*fxf0+wqz*fxf1+     pypxdzy1    +     zyxy)
      ec(289)=ec(289)+bb*(qdx*fya0+wqx*fya1                  +2.d0*zyyx)
      ec(290)=ec(290)+bb*(qdy*fya0+wqy*fya1+     pzpydxx1+fa1)
      ec(291)=ec(291)+bb*(qdz*fya0+wqz*fya1+     pypydxx1     )   
      ec(292)=ec(292)+bb*(qdx*fyb0+wqx*fyb1)
      ec(293)=ec(293)+bb*(qdy*fyb0+wqy*fyb1+     pzpydyy1+fb1+2.d0*zyyy)
      ec(294)=ec(294)+bb*(qdz*fyb0+wqz*fyb1+     pypydyy1        )
      ec(295)=ec(295)+bb*(qdx*fyc0+wqx*fyc1                      )
      ec(296)=ec(296)+bb*(qdy*fyc0+wqy*fyc1+     pzpydzz1+fc1)
      ec(297)=ec(297)+bb*(qdz*fyc0+wqz*fyc1+     pypydzz1    +2.d0*zyyz)
      ec(298)=ec(298)+bb*(qdx*fyd0+wqx*fyd1                  +     zyyy)
      ec(299)=ec(299)+bb*(qdy*fyd0+wqy*fyd1+     pzpydyx1+fd1+     zyyx)
      ec(300)=ec(300)+bb*(qdz*fyd0+wqz*fyd1+     pypydyx1    )
      ec(301)=ec(301)+bb*(qdx*fye0+wqx*fye1                  +     zyyz)
      ec(302)=ec(302)+bb*(qdy*fye0+wqy*fye1+     pzpydzx1+fe1 )
      ec(303)=ec(303)+bb*(qdz*fye0+wqz*fye1+     pypydzx1    +     zyyx)
      ec(304)=ec(304)+bb*(qdx*fyf0+wqx*fyf1                      )
      ec(305)=ec(305)+bb*(qdy*fyf0+wqy*fyf1+     pzpydzy1+ff1+     zyyz)
      ec(306)=ec(306)+bb*(qdz*fyf0+wqz*fyf1+     pypydzy1    +     zyyy)
      ec(307)=ec(307)+bb*(qdx*fza0+wqx*fza1                  +2.d0*zyzx)
      ec(308)=ec(308)+bb*(qdy*fza0+wqy*fza1+     pzpzdxx1     )   
      ec(309)=ec(309)+bb*(qdz*fza0+wqz*fza1+     pypzdxx1+fa1)
      ec(310)=ec(310)+bb*(qdx*fzb0+wqx*fzb1                      )
      ec(311)=ec(311)+bb*(qdy*fzb0+wqy*fzb1+     pzpzdyy1    +2.d0*zyzy)
      ec(312)=ec(312)+bb*(qdz*fzb0+wqz*fzb1+     pypzdyy1+fb1)
      ec(313)=ec(313)+bb*(qdx*fzc0+wqx*fzc1)
      ec(314)=ec(314)+bb*(qdy*fzc0+wqy*fzc1+     pzpzdzz1        )
      ec(315)=ec(315)+bb*(qdz*fzc0+wqz*fzc1+     pypzdzz1+fc1+2.d0*zyzz)
      ec(316)=ec(316)+bb*(qdx*fzd0+wqx*fzd1                  +     zyzy)
      ec(317)=ec(317)+bb*(qdy*fzd0+wqy*fzd1+     pzpzdyx1    +     zyzx)
      ec(318)=ec(318)+bb*(qdz*fzd0+wqz*fzd1+     pypzdyx1+fd1)
      ec(319)=ec(319)+bb*(qdx*fze0+wqx*fze1                  +     zyzz)
      ec(320)=ec(320)+bb*(qdy*fze0+wqy*fze1+     pzpzdzx1         )
      ec(321)=ec(321)+bb*(qdz*fze0+wqz*fze1+     pypzdzx1+fe1+     zyzx)
      ec(322)=ec(322)+bb*(qdx*fzf0+wqx*fzf1                      )
      ec(323)=ec(323)+bb*(qdy*fzf0+wqy*fzf1+     pzpzdzy1    +     zyzz)
      ec(324)=ec(324)+bb*(qdz*fzf0+wqz*fzf1+     pypzdzy1+ff1+     zyzy)

1111            enddo
            enddo
         enddo
      enddo


!     goto 2222
      ec(  1)=ec(  1) - d( 1)    -e(  1) + spsp(1)
      ec(  2)=ec(  2) - d( 2)    -e(  2) + spsp(2)
      ec(  3)=ec(  3) - d( 3)    -e(  3) + spsp(3)
      ec(  4)=ec(  4) - d( 1)    -e(  4) + spsp(1)
      ec(  5)=ec(  5) - d( 2)    -e(  5) + spsp(2)
      ec(  6)=ec(  6) - d( 3)    -e(  6) + spsp(3)
      ec(  7)=ec(  7) - d( 1)    -e(  7) + spsp(1)
      ec(  8)=ec(  8) - d( 2)    -e(  8) + spsp(2)
      ec(  9)=ec(  9) - d( 3)    -e(  9) + spsp(3)
      ec( 10)=ec( 10)            -e( 10)
      ec( 11)=ec( 11)            -e( 11)
      ec( 12)=ec( 12)            -e( 12)
      ec( 13)=ec( 13)            -e( 13)
      ec( 14)=ec( 14)            -e( 14)
      ec( 15)=ec( 15)            -e( 15)
      ec( 16)=ec( 16)            -e( 16)
      ec( 17)=ec( 17)            -e( 17)
      ec( 18)=ec( 18)            -e( 18)
      ec( 19)=ec( 19) - d( 4)    -e( 19) + spsp(4)
      ec( 20)=ec( 20) - d( 5)    -e( 20) + spsp(5)
      ec( 21)=ec( 21) - d( 6)    -e( 21) + spsp(6)
      ec( 22)=ec( 22) - d( 4)    -e( 22) + spsp(4)
      ec( 23)=ec( 23) - d( 5)    -e( 23) + spsp(5)
      ec( 24)=ec( 24) - d( 6)    -e( 24) + spsp(6)
      ec( 25)=ec( 25) - d( 4)    -e( 25) + spsp(4)
      ec( 26)=ec( 26) - d( 5)    -e( 26) + spsp(5)
      ec( 27)=ec( 27) - d( 6)    -e( 27) + spsp(6)
      ec( 28)=ec( 28)            -e( 28)
      ec( 29)=ec( 29)            -e( 29)
      ec( 30)=ec( 30)            -e( 30)
      ec( 31)=ec( 31)            -e( 31)
      ec( 32)=ec( 32)            -e( 32)
      ec( 33)=ec( 33)            -e( 33)
      ec( 34)=ec( 34)            -e( 34)
      ec( 35)=ec( 35)            -e( 35)
      ec( 36)=ec( 36)            -e( 36)
      ec( 37)=ec( 37) - d( 7)    -e( 37) + spsp(7)
      ec( 38)=ec( 38) - d( 8)    -e( 38) + spsp(8)
      ec( 39)=ec( 39) - d( 9)    -e( 39) + spsp(9)
      ec( 40)=ec( 40) - d( 7)    -e( 40) + spsp(7)
      ec( 41)=ec( 41) - d( 8)    -e( 41) + spsp(8)
      ec( 42)=ec( 42) - d( 9)    -e( 42) + spsp(9)
      ec( 43)=ec( 43) - d( 7)    -e( 43) + spsp(7)
      ec( 44)=ec( 44) - d( 8)    -e( 44) + spsp(8)
      ec( 45)=ec( 45) - d( 9)    -e( 45) + spsp(9)
      ec( 46)=ec( 46)            -e( 46)
      ec( 47)=ec( 47)            -e( 47)
      ec( 48)=ec( 48)            -e( 48)
      ec( 49)=ec( 49)            -e( 49)
      ec( 50)=ec( 50)            -e( 50)
      ec( 51)=ec( 51)            -e( 51)
      ec( 52)=ec( 52)            -e( 52)
      ec( 53)=ec( 53)            -e( 53)
      ec( 54)=ec( 54)            -e( 54)
      ec( 55)=ec( 55) - d(10)    -e(  1) + spsp(1)
      ec( 56)=ec( 56) - d(11)    -e(  2) + spsp(2)
      ec( 57)=ec( 57) - d(12)    -e(  3) + spsp(3)
      ec( 58)=ec( 58) - d(10)    -e(  4) + spsp(1)
      ec( 59)=ec( 59) - d(11)    -e(  5) + spsp(2)
      ec( 60)=ec( 60) - d(12)    -e(  6) + spsp(3)
      ec( 61)=ec( 61) - d(10)    -e(  7) + spsp(1)
      ec( 62)=ec( 62) - d(11)    -e(  8) + spsp(2)
      ec( 63)=ec( 63) - d(12)    -e(  9) + spsp(3)
      ec( 64)=ec( 64)            -e( 10)         
      ec( 65)=ec( 65)            -e( 11)         
      ec( 66)=ec( 66)            -e( 12)         
      ec( 67)=ec( 67)            -e( 13)         
      ec( 68)=ec( 68)            -e( 14)         
      ec( 69)=ec( 69)            -e( 15)         
      ec( 70)=ec( 70)            -e( 16)         
      ec( 71)=ec( 71)            -e( 17)         
      ec( 72)=ec( 72)            -e( 18)         
      ec( 73)=ec( 73) - d(13)    -e( 19) + spsp(4)
      ec( 74)=ec( 74) - d(14)    -e( 20) + spsp(5)
      ec( 75)=ec( 75) - d(15)    -e( 21) + spsp(6)
      ec( 76)=ec( 76) - d(13)    -e( 22) + spsp(4)
      ec( 77)=ec( 77) - d(14)    -e( 23) + spsp(5)
      ec( 78)=ec( 78) - d(15)    -e( 24) + spsp(6)
      ec( 79)=ec( 79) - d(13)    -e( 25) + spsp(4)
      ec( 80)=ec( 80) - d(14)    -e( 26) + spsp(5)
      ec( 81)=ec( 81) - d(15)    -e( 27) + spsp(6)
      ec( 82)=ec( 82)            -e( 28)         
      ec( 83)=ec( 83)            -e( 29)         
      ec( 84)=ec( 84)            -e( 30)         
      ec( 85)=ec( 85)            -e( 31)         
      ec( 86)=ec( 86)            -e( 32)         
      ec( 87)=ec( 87)            -e( 33)         
      ec( 88)=ec( 88)            -e( 34)         
      ec( 89)=ec( 89)            -e( 35)         
      ec( 90)=ec( 90)            -e( 36)         
      ec( 91)=ec( 91) - d(16)    -e( 37) + spsp(7)
      ec( 92)=ec( 92) - d(17)    -e( 38) + spsp(8)
      ec( 93)=ec( 93) - d(18)    -e( 39) + spsp(9)
      ec( 94)=ec( 94) - d(16)    -e( 40) + spsp(7)
      ec( 95)=ec( 95) - d(17)    -e( 41) + spsp(8)
      ec( 96)=ec( 96) - d(18)    -e( 42) + spsp(9)
      ec( 97)=ec( 97) - d(16)    -e( 43) + spsp(7)
      ec( 98)=ec( 98) - d(17)    -e( 44) + spsp(8)
      ec( 99)=ec( 99) - d(18)    -e( 45) + spsp(9)
      ec(100)=ec(100)            -e( 46)
      ec(101)=ec(101)            -e( 47)
      ec(102)=ec(102)            -e( 48)
      ec(103)=ec(103)            -e( 49)
      ec(104)=ec(104)            -e( 50)
      ec(105)=ec(105)            -e( 51)
      ec(106)=ec(106)            -e( 52)
      ec(107)=ec(107)            -e( 53)
      ec(108)=ec(108)            -e( 54)
      ec(109)=ec(109) - d(19)    -e(  1) + spsp(1)
      ec(110)=ec(110) - d(20)    -e(  2) + spsp(2)
      ec(111)=ec(111) - d(21)    -e(  3) + spsp(3)
      ec(112)=ec(112) - d(19)    -e(  4) + spsp(1)
      ec(113)=ec(113) - d(20)    -e(  5) + spsp(2)
      ec(114)=ec(114) - d(21)    -e(  6) + spsp(3)
      ec(115)=ec(115) - d(19)    -e(  7) + spsp(1)
      ec(116)=ec(116) - d(20)    -e(  8) + spsp(2)
      ec(117)=ec(117) - d(21)    -e(  9) + spsp(3)
      ec(118)=ec(118)            -e( 10)         
      ec(119)=ec(119)            -e( 11)         
      ec(120)=ec(120)            -e( 12)         
      ec(121)=ec(121)            -e( 13)         
      ec(122)=ec(122)            -e( 14)         
      ec(123)=ec(123)            -e( 15)         
      ec(124)=ec(124)            -e( 16)         
      ec(125)=ec(125)            -e( 17)         
      ec(126)=ec(126)            -e( 18)         
      ec(127)=ec(127) - d(22)    -e( 19) + spsp(4)
      ec(128)=ec(128) - d(23)    -e( 20) + spsp(5)
      ec(129)=ec(129) - d(24)    -e( 21) + spsp(6)
      ec(130)=ec(130) - d(22)    -e( 22) + spsp(4)
      ec(131)=ec(131) - d(23)    -e( 23) + spsp(5)
      ec(132)=ec(132) - d(24)    -e( 24) + spsp(6)
      ec(133)=ec(133) - d(22)    -e( 25) + spsp(4)
      ec(134)=ec(134) - d(23)    -e( 26) + spsp(5)
      ec(135)=ec(135) - d(24)    -e( 27) + spsp(6)
      ec(136)=ec(136)            -e( 28)         
      ec(137)=ec(137)            -e( 29)         
      ec(138)=ec(138)            -e( 30)         
      ec(139)=ec(139)            -e( 31)         
      ec(140)=ec(140)            -e( 32)         
      ec(141)=ec(141)            -e( 33)         
      ec(142)=ec(142)            -e( 34)         
      ec(143)=ec(143)            -e( 35)         
      ec(144)=ec(144)            -e( 36)         
      ec(145)=ec(145) - d(25)    -e( 37) + spsp(7)
      ec(146)=ec(146) - d(26)    -e( 38) + spsp(8)
      ec(147)=ec(147) - d(27)    -e( 39) + spsp(9)
      ec(148)=ec(148) - d(25)    -e( 40) + spsp(7)
      ec(149)=ec(149) - d(26)    -e( 41) + spsp(8)
      ec(150)=ec(150) - d(27)    -e( 42) + spsp(9)
      ec(151)=ec(151) - d(25)    -e( 43) + spsp(7)
      ec(152)=ec(152) - d(26)    -e( 44) + spsp(8)
      ec(153)=ec(153) - d(27)    -e( 45) + spsp(9)
      ec(154)=ec(154)            -e( 46)
      ec(155)=ec(155)            -e( 47)
      ec(156)=ec(156)            -e( 48)
      ec(157)=ec(157)            -e( 49)
      ec(158)=ec(158)            -e( 50)
      ec(159)=ec(159)            -e( 51)
      ec(160)=ec(160)            -e( 52)
      ec(161)=ec(161)            -e( 53)
      ec(162)=ec(162)            -e( 54)
      ec(163)=ec(163) - d(28)
      ec(164)=ec(164) - d(29)
      ec(165)=ec(165) - d(30)
      ec(166)=ec(166) - d(28)
      ec(167)=ec(167) - d(29)
      ec(168)=ec(168) - d(30)
      ec(169)=ec(169) - d(28)
      ec(170)=ec(170) - d(29)
      ec(171)=ec(171) - d(30)
!     ec(172)=ec(172)            
!     ec(173)=ec(173)            
!     ec(174)=ec(174)            
!     ec(175)=ec(175)            
!     ec(176)=ec(176)            
!     ec(177)=ec(177)            
!     ec(178)=ec(178)            
!     ec(179)=ec(179)            
!     ec(180)=ec(180)            
      ec(181)=ec(181) - d(31)
      ec(182)=ec(182) - d(32)
      ec(183)=ec(183) - d(33)
      ec(184)=ec(184) - d(31)
      ec(185)=ec(185) - d(32)
      ec(186)=ec(186) - d(33)
      ec(187)=ec(187) - d(31)
      ec(188)=ec(188) - d(32)
      ec(189)=ec(189) - d(33)
!     ec(190)=ec(190)            
!     ec(191)=ec(191)            
!     ec(192)=ec(192)            
!     ec(193)=ec(193)            
!     ec(194)=ec(194)            
!     ec(195)=ec(195)            
!     ec(196)=ec(196)            
!     ec(197)=ec(197)            
!     ec(198)=ec(198)            
      ec(199)=ec(199) - d(34)
      ec(200)=ec(200) - d(35)
      ec(201)=ec(201) - d(36)
      ec(202)=ec(202) - d(34)
      ec(203)=ec(203) - d(35)
      ec(204)=ec(204) - d(36)
      ec(205)=ec(205) - d(34)
      ec(206)=ec(206) - d(35)
      ec(207)=ec(207) - d(36)
!     ec(208)=ec(208)            
!     ec(209)=ec(209)            
!     ec(210)=ec(210)            
!     ec(211)=ec(211)            
!     ec(212)=ec(212)            
!     ec(213)=ec(213)            
!     ec(214)=ec(214)            
!     ec(215)=ec(215)            
!     ec(216)=ec(216)            
      ec(217)=ec(217) - d(37)
      ec(218)=ec(218) - d(38)
      ec(219)=ec(219) - d(39)
      ec(220)=ec(220) - d(37)
      ec(221)=ec(221) - d(38)
      ec(222)=ec(222) - d(39)
      ec(223)=ec(223) - d(37)
      ec(224)=ec(224) - d(38)
      ec(225)=ec(225) - d(39)
!     ec(226)=ec(226)            
!     ec(227)=ec(227)            
!     ec(228)=ec(228)            
!     ec(229)=ec(229)            
!     ec(230)=ec(230)            
!     ec(231)=ec(231)            
!     ec(232)=ec(232)            
!     ec(233)=ec(233)            
!     ec(234)=ec(234)            
      ec(235)=ec(235) - d(40)
      ec(236)=ec(236) - d(41)
      ec(237)=ec(237) - d(42)
      ec(238)=ec(238) - d(40)
      ec(239)=ec(239) - d(41)
      ec(240)=ec(240) - d(42)
      ec(241)=ec(241) - d(40)
      ec(242)=ec(242) - d(41)
      ec(243)=ec(243) - d(42)
!     ec(244)=ec(244)            
!     ec(245)=ec(245)            
!     ec(246)=ec(246)            
!     ec(247)=ec(247)            
!     ec(248)=ec(248)            
!     ec(249)=ec(249)            
!     ec(250)=ec(250)            
!     ec(251)=ec(251)            
!     ec(252)=ec(252)            
      ec(253)=ec(253) - d(43)
      ec(254)=ec(254) - d(44)
      ec(255)=ec(255) - d(45)
      ec(256)=ec(256) - d(43)
      ec(257)=ec(257) - d(44)
      ec(258)=ec(258) - d(45)
      ec(259)=ec(259) - d(43)
      ec(260)=ec(260) - d(44)
      ec(261)=ec(261) - d(45)
!     ec(262)=ec(262)            
!     ec(263)=ec(263)            
!     ec(264)=ec(264)            
!     ec(265)=ec(265)            
!     ec(266)=ec(266)            
!     ec(267)=ec(267)            
!     ec(268)=ec(268)            
!     ec(269)=ec(269)            
!     ec(270)=ec(270)            
      ec(271)=ec(271) - d(46)
      ec(272)=ec(272) - d(47)
      ec(273)=ec(273) - d(48)
      ec(274)=ec(274) - d(46)
      ec(275)=ec(275) - d(47)
      ec(276)=ec(276) - d(48)
      ec(277)=ec(277) - d(46)
      ec(278)=ec(278) - d(47)
      ec(279)=ec(279) - d(48)
!     ec(280)=ec(280)
!     ec(281)=ec(281)
!     ec(282)=ec(282)
!     ec(283)=ec(283)
!     ec(284)=ec(284)
!     ec(285)=ec(285)
!     ec(286)=ec(286)
!     ec(287)=ec(287)
!     ec(288)=ec(288)
      ec(289)=ec(289) - d(49)
      ec(290)=ec(290) - d(50)
      ec(291)=ec(291) - d(51)
      ec(292)=ec(292) - d(49)
      ec(293)=ec(293) - d(50)
      ec(294)=ec(294) - d(51)
      ec(295)=ec(295) - d(49)
      ec(296)=ec(296) - d(50)
      ec(297)=ec(297) - d(51)
!     ec(298)=ec(298)
!     ec(299)=ec(299)
!     ec(300)=ec(300)
!     ec(301)=ec(301)
!     ec(302)=ec(302)
!     ec(303)=ec(303)
!     ec(304)=ec(304)
!     ec(305)=ec(305)
!     ec(306)=ec(306)
      ec(307)=ec(307) - d(52)
      ec(308)=ec(308) - d(53)
      ec(309)=ec(309) - d(54)
      ec(310)=ec(310) - d(52)
      ec(311)=ec(311) - d(53)
      ec(312)=ec(312) - d(54)
      ec(313)=ec(313) - d(52)
      ec(314)=ec(314) - d(53)
      ec(315)=ec(315) - d(54)
!     ec(316)=ec(316)
!     ec(317)=ec(317)
!     ec(318)=ec(318)
!     ec(319)=ec(319)
!     ec(320)=ec(320)
!     ec(321)=ec(321)
!     ec(322)=ec(322)
!     ec(323)=ec(323)
!     ec(324)=ec(324)
      do i=1,324
         ec(i) = dabs(ec(i))
      enddo

      ddmax = maxval(ec)


 2222 end
