! -*- coding: utf-8 -*-
!
! Dual licensed under the MIT or GPL Version 3 license.
! See 'LICENSE.MIT' for the definition of the MIT license,
! and 'COPYING' for the definition of the GPL license.
!
C** IKDSSS ===========================================================
C          CALUCATE KERNELS FOR DSSS TYPE
C====================================================================

      subroutine ikdsss(
     &           dsss ,
     &           fdat ,  edat  ,
     &           yp   ,  yq    ,
     &           uu   ,  uv    ,
     &           z    ,  zetpq , 
     &           nasp ,  mxsa  ,  mxprm,  ntprm,  
     &           iasp ,  jasp  ,  kasp ,  lasp ,
     &           ishel,  jshel ,  kshel,  lshel     )

      implicit double precision  (a-h,o-z)
      dimension fdat(31,2001)
      dimension edat(2001)
      dimension zetpq(mxprm,mxprm,mxsa,nasp,mxsa,nasp)
      dimension yp(3,mxprm,mxprm),yq(3,mxprm,mxprm)
      dimension uu(mxprm,mxprm),uv(mxprm,mxprm)
      dimension z(mxprm,mxsa,nasp)
      dimension ntprm(mxsa,nasp)
      dimension dsss(12)

      real(8),parameter::pi    = 3.14159265358979323846d+00
      real(8),parameter::tf(25) = (/
     &                33.0,37.0,41.0,43.0,46.0,49.0,
     &                51.0,54.0,56.0,58.0,61.0,63.0,
     &                66.0,68.0,70.0,72.0,74.0,80.0,
     &                85.0,90.0,95.0,95.0,98.0,99.0,
     &                99.0                           /)

      real(8),parameter::ga0 = 1.0d+00
      real(8),parameter::ga1 = 1.000 000 000 000 005 298d+00
      real(8),parameter::ga2 = 0.5d+00
      real(8),parameter::ga3 = 0.166 666 666 598 849 826d+00
      real(8),parameter::ga4 = 0.416 666 666 666 666 667d-01
      real(8),parameter::ga5 = 0.833 355 034 722 222 222d-02
      real(8),parameter::ga6 = 0.138 888 888 888 888 889d-02

      real(8),parameter::rminv(5) = (/
     &   1.0d+00,  0.333333333333333333d+00,  0.20d+00,
     &   0.14285714285714286d+00,  0.11111111111111111d+00 /)

#ifdef PL_SIMD_ERI
      dimension dmx(3,mxprm), val(1,mxprm)
#endif

      do ijk=1,12
         dsss(ijk)=0.0d0
      enddo
      
#ifdef PL_OMP_ERI
!$omp parallel reduction(+:dsss)
!$omp& default(private)
!$omp& shared(fdat,edat,
!$omp&        zetpq,ntprm,z,yp,yq,uu,uv,
!$omp&        ishel,jshel,kshel,lshel,iasp,jasp,kasp,lasp)
#endif

      igp = ntprm(ishel,iasp)
      jgp = ntprm(jshel,jasp)
      kgp = ntprm(kshel,kasp)
      lgp = ntprm(lshel,lasp)
      
      pi2 = 2.0d0*pi
      
#ifdef PL_OMP_ERI
!$omp do
#endif
      do i = 1, igp
         
         bet06 = 0.0d0  
         bet10 = 0.0d0  
         bet11 = 0.0d0  
         bet12 = 0.0d0
         
         alp = z(i,ishel,iasp)
         do j = 1, jgp

            del02 = 0.0d0
            del03 = 0.0d0
            del07 = 0.0d0
            ram05 = 0.0d0
            ram08 = 0.0d0
            ram09 = 0.0d0
            
            bet = z(j,jshel,jasp)
            zet = zetpq(j,i,jshel,jasp,ishel,iasp)
            px  = yp (1,j,i)
            py  = yp (2,j,i)
            pz  = yp (3,j,i)
            up  = uu (j,i)
            
            do k = 1, kgp
               del05 = 0.0d0        
               del08 = 0.0d0
               del09 = 0.0d0       

               ram = z(k,kshel,kasp)
               
               do l = 1, lgp
                  eta = zetpq(l,k,lshel,lasp,kshel,kasp)
                  qpx =  yq(1,l,k) - px
                  qpy =  yq(2,l,k) - py
                  qpz =  yq(3,l,k) - pz
                  vq  = uv (l,k)
                  
                  roo2 = 1.0d0 / (zet + eta)
                  qp2      = qpx*qpx + qpy*qpy + qpz*qpz
                  t        = 0.5d0*roo2*qp2
                  
                  if (t. le .tf(3)) then
                     
                     it = idint((t+0.025d+00)*20.0d+00)
                     dt = 0.05d+00*it - t
                     it = it + 1
                     dt4 = dt*0.25d+00
                     dt5 = dt*0.2d+00
                     dt6 = dt*0.166666666666666667d+00
                     dt2 = dt4 + dt4
                     dt3 = dt6 + dt6
                     f2   = (((((fdat(9,it)*dt6   + fdat(8,it))*dt5
     &                    + fdat(7,it))*dt4  + fdat(6,it))*dt3
     &                    + fdat(5,it))*dt2  + fdat(4,it))*dt
     &                    + fdat(3,it)
                     
                     eed  = (((((ga6*dt + ga5)*dt + ga4)*dt
     &                    + ga3)*dt + ga2)*dt + ga1)*dt + ga0
                     
                     ee   = edat(it)*eed
                     
                     t2   = t*2.0d0
                     f1   = (t2*f2 + ee)*rminv(2)
                     f0   = t2*f1 + ee
                     
                     fpp =  up * vq * dsqrt(roo2)
                     dm0 = fpp*f0
                     dm1 = fpp*f1 *  roo2
                     dm2 = fpp*f2 * (roo2)**2
                  else
                     
                     qinv = 1.0d0 / qp2
                     dm0=  up*vq*dsqrt( pi2*qinv)*0.5d0
                     dm1 =        qinv *  dm0
                     dm2 =  3.0d0*qinv *  dm1
                     
                  end if

#ifdef PL_SIMD_ERI
                  val(1,l) = eta

                  dmx(1,l) = dm0
                  dmx(2,l) = dm1
                  dmx(3,l) = dm2
               enddo
               
               do l = 1, lgp
                  eta = val(1,l)

                  dm0 = dmx(1,l)
                  dm1 = dmx(2,l)
                  dm2 = dmx(3,l)
#endif

!** kernel dsss =====================================================
!           calculate & contract kernels for dsss type
!  1:Z001000  2:Z022000  3:U002000  4:U012000  5:U012101
!  6:U113000  7:D002000  8:D002101  9:D002202 
! 10:D103000 11:D103101 12:D204000
c====================================================================

                  dsss( 1)= dsss( 1) +dm0
                  
                  del02 = del02 +dm0
                  del03 = del03 +dm1
                  del05 = del05 +dm1      *eta
                  del07 = del07 +dm2
                  del08 = del08 +dm2      *eta
                  del09 = del09 +dm2      *eta**2
                  
               enddo
               ram05 = ram05 +del05 * RAM          
               ram08 = ram08 +del08 * RAM 
               ram09 = ram09 +del09 * RAM**2       
               
            enddo
            
            dsss( 2) = dsss( 2) + del02*BET**2*ZET   
            dsss( 3) = dsss( 3) + del03       *ZET   
            dsss( 4) = dsss( 4) + del03*BET   *ZET
            dsss( 5) = dsss( 5) + ram05*BET   *ZET   
            dsss( 7) = dsss( 7) + del07       *ZET   
            dsss( 8) = dsss( 8) + ram08       *ZET   
            dsss( 9) = dsss( 9) + ram09       *ZET   
            
            bet06 = bet06 + del03*BET   *ZET**2
            bet10 = bet10 + del07       *ZET**2
            bet11 = bet11 + ram08       *ZET**2
            bet12 = bet12 + del07       *ZET**3 
            
         enddo
         dsss( 6)= dsss( 6) + bet06 * alp   
         dsss(10)= dsss(10) + bet10 * alp   
         dsss(11)= dsss(11) + bet11 * alp   
         dsss(12)= dsss(12) + bet12 * alp**2
      enddo
#ifdef PL_OMP_ERI
!$omp end do
!$omp end parallel
#endif
      
      end
C** IKDSPS ===========================================================
C          CALUCATE KERNELS FOR DSPS TYPE
C====================================================================

      subroutine ikdsps(
     &           dsps ,
     &           fdat ,  edat  ,
     &           yp   ,  yq    ,
     &           uu   ,  uv    ,
     &           z    ,  zetpq , 
     &           nasp ,  mxsa  ,  mxprm,  ntprm,  
     &           iasp ,  jasp  ,  kasp ,  lasp ,
     &           ishel,  jshel ,  kshel,  lshel     )


      implicit double precision  (a-h,o-z)
      dimension fdat(31,2001)
      dimension edat(2001)
      dimension zetpq(mxprm,mxprm,mxsa,nasp,mxsa,nasp)
      dimension yp(3,mxprm,mxprm),yq(3,mxprm,mxprm)
      dimension uu(mxprm,mxprm),uv(mxprm,mxprm)
      dimension z(mxprm,mxsa,nasp)
      dimension ntprm(mxsa,nasp)
      dimension dsps(39)

      real(8),parameter::pi    = 3.14159265358979323846d+00
      real(8),parameter::tf(25) = (/
     &                33.0,37.0,41.0,43.0,46.0,49.0,
     &                51.0,54.0,56.0,58.0,61.0,63.0,
     &                66.0,68.0,70.0,72.0,74.0,80.0,
     &                85.0,90.0,95.0,95.0,98.0,99.0,
     &                99.0                           /)

      real(8),parameter::ga0 = 1.0d+00
      real(8),parameter::ga1 = 1.000 000 000 000 005 298d+00
      real(8),parameter::ga2 = 0.5d+00
      real(8),parameter::ga3 = 0.166 666 666 598 849 826d+00
      real(8),parameter::ga4 = 0.416 666 666 666 666 667d-01
      real(8),parameter::ga5 = 0.833 355 034 722 222 222d-02
      real(8),parameter::ga6 = 0.138 888 888 888 888 889d-02

      real(8),parameter::rminv(5) = (/
     &   1.0d+00,  0.333333333333333333d+00,  0.20d+00,
     &   0.14285714285714286d+00,  0.11111111111111111d+00 /)

#ifdef PL_SIMD_ERI
      dimension dmx(4,mxprm), val(2,mxprm)
#endif

      do ijk=1,39
         dsps(ijk)=0.0d0
      enddo
      
#ifdef PL_OMP_ERI
!$omp parallel reduction(+:dsps)
!$omp& default(private)
!$omp& shared(fdat,edat,
!$omp&        zetpq,ntprm,z,yp,yq,uu,uv,
!$omp&        ishel,jshel,kshel,lshel,iasp,jasp,kasp,lasp)
#endif

      igp = ntprm(ishel,iasp)
      jgp = ntprm(jshel,jasp)
      kgp = ntprm(kshel,kasp)
      lgp = ntprm(lshel,lasp)
      
      pi2 = 2.0d0*pi
      
#ifdef PL_OMP_ERI
!$omp do
#endif
      DO I = 1, IGP
         bet05 = 0.0d0
         bet06 = 0.0d0
         bet07 = 0.0d0
         bet08 = 0.0d0
         bet09 = 0.0d0
         bet10 = 0.0d0
         bet20 = 0.0d0
         bet21 = 0.0d0
         bet22 = 0.0d0
         bet23 = 0.0d0
         bet24 = 0.0d0
         bet25 = 0.0d0
         bet26 = 0.0d0
         bet35 = 0.0d0
         bet36 = 0.0d0
         bet37 = 0.0d0

         alp = z(i,ishel,iasp)
         
         DO J = 1, JGP
            del01 =  0.0d0
            del11 =  0.0d0
            del12 =  0.0d0
            del13 =  0.0d0
            del29 =  0.0d0
            del30 =  0.0d0
            del39 =  0.0d0
            ram02 =  0.0d0
            ram03 =  0.0d0
            ram04 =  0.0d0
            ram14 =  0.0d0
            ram15 =  0.0d0
            ram16 =  0.0d0
            ram19 =  0.0d0
            ram32 =  0.0d0
            ram34 =  0.0d0
            
            bet = z(j,jshel,jasp)
            zet = zetpq(j,i,jshel,jasp,ishel,iasp)
            px  = yp (1,j,i)
            py  = yp (2,j,i)
            pz  = yp (3,j,i)
            up  = uu  (j,i)
            
            DO K = 1, KGP
               del02 = 0.0d0
               del03 = 0.0d0
               del04 = 0.0d0
               del14 = 0.0d0
               del15 = 0.0d0
               del16 = 0.0d0
               del19 = 0.0d0
               del32 = 0.0d0
               del28 = 0.0d0
               
               ram = z(k,kshel,kasp)
               
               DO L = 1, LGP
                  del = z(l,lshel,lasp)
                  eta = zetpq(l,k,lshel,lasp,kshel,kasp)
                  reta = 1.0d0 / eta
                  qpx =  yq(1,l,k) - px
                  qpy =  yq(2,l,k) - py
                  qpz =  yq(3,l,k) - pz
                  vq  = uv  (l,k)
                  
                  ROO2 = 1.0d0 / (ZET + ETA)
                  QP2  = QPX*QPX + QPY*QPY + QPZ*QPZ
                  T    = 0.5d0* ROO2*QP2
                  
                  IF (T. LE .TF(4)) THEN
                     
                     IT = IDINT((T+0.025D+00)*20.0D+00)
                     DT = 0.05D+00*IT - T
                     IT = IT + 1
                     DT4 = DT*0.25D+00
                     DT5 = DT*0.2D+00
                     DT6 = DT*0.166666666666666667D+00
                     DT2 = DT4 + DT4
                     DT3 = DT6 + DT6
                     F3   = (((((FDAT(10,IT)*DT6   + FDAT(9,IT))*DT5
     &                    + FDAT(8,IT))*DT4  + FDAT(7,IT))*DT3
     &                    + FDAT(6,IT))*DT2  + FDAT(5,IT))*DT
     &                    + FDAT(4,IT)
                     
                     EED  = (((((GA6*DT + GA5)*DT + GA4)*DT
     &                    + GA3)*DT + GA2)*DT + GA1)*DT + GA0
                     EE   = EDAT(IT)*EED
                     
                     T2   = T*2.0d0
                     
                     F2   = (T2*F3 + EE)*RMINV(3)
                     F1   = (T2*F2 + EE)*RMINV(2)
                     F0   =  T2*F1 + EE
                     FPP =  UP * VQ * DSQRT(ROO2)
                     DM0 = FPP*F0
                     DM1 = FPP*F1 *  ROO2
                     DM2 = FPP*F2 *( ROO2)**2
                     DM3 = FPP*F3 *( ROO2)**3
                     
                  ELSE
                     qinv = 1.0d0/qp2
                     dm0=  up*vq*dsqrt( pi2*qinv)*0.5d0
                     dm1 =        qinv *  dm0
                     dm2 =  3.0d0*qinv *  dm1
                     dm3 =  5.0d0*qinv *  dm2
                     
                  END IF

#ifdef PL_SIMD_ERI
                  val(1,l) = eta
                  val(2,l) = del

                  dmx(1,l) = dm0
                  dmx(2,l) = dm1
                  dmx(3,l) = dm2
                  dmx(4,l) = dm3
               enddo
               
               do l = 1, lgp
                  eta = val(1,l)
                  del = val(2,l)
                  reta = 1.0d0 / eta

                  dm0 = dmx(1,l)
                  dm1 = dmx(2,l)
                  dm2 = dmx(3,l)
                  dm3 = dmx(4,l)
#endif

C** KERNEL DSPS =====================================================
C           CALCULATE & CONTRACT KERNELS FOR DSPS TYPE
C 1:T002001  8:T204001 15:D002112 22:D103112 29:U002011 36:U113011
C 2:T002102  9:T204102 16:D002213 23:D113001 30:U012001 37:U123001
C 3:T002203 10:T305001 17:D012001 24:D113102 31:U012011 38:Z001011
C 4:T002304 11:D002000 18:D012102 25:D204011 32:U012112 39:Z022011
C 5:T103001 12:D002001 19:D012203 26:D214001 33:U022001
C 6:T103102 13:D002011 20:D103001 27:U001001 34:U022102
C 7:T103203 14:D002102 21:D103011 28:U001102 35:U102001
C===================================================================

                  del01  = del01  +dm3                 
                  del02  = del02  +dm3         * eta   
                  del03  = del03  +dm3         * eta**2
                  del04  = del04  +dm3         * eta**3
!===============================================
                  del11  = del11  +dm2         * reta
                  del12  = del12  +dm2                 
                  del14  = del14  +dm2         * eta   
                  del19  = del19  +dm2         * eta**2
                  del13  = del13  +dm2 * del           
                  del15  = del15  +dm2 * del   * eta   
                  del16  = del16  +dm2 * del   * eta**2
!===============================================
                  del30  = del30  +dm1                 
                  del28  = del28  +dm1         * eta   
                  del29  = del29  +dm1 * del           
                  del32  = del32  +dm1 * del   * eta   
!===============================================
                  del39  = del39  +dm0 * del           
                  
                  dsps(27) =dsps(27)+dm1                              
                  dsps(38) =dsps(38)+dm0 * del        
               enddo
               ram02 = ram02 + del02 * ram   
               ram03 = ram03 + del03 * ram**2
               ram04 = ram04 + del04 * ram**3
               ram14 = ram14 + del14 * ram   
               ram15 = ram15 + del15 * ram   
               ram16 = ram16 + del16 * ram**2
               ram19 = ram19 + del19 * ram**2
               ram32 = ram32 + del32 * ram   
               ram34 = ram34 + del28 * ram   
               
               dsps(28) =dsps(28) + del28    *ram         
            enddo
            
            bet05 = bet05 + del01          * ZET**2
            bet08 = bet08 + del01          * ZET**3
            bet10 = bet10 + del01          * ZET**4
            bet20 = bet20 + del12          * ZET**2
            bet23 = bet23 + del12 * BET    * ZET**2
            bet26 = bet26 + del12 * BET    * ZET**3
            bet21 = bet21 + del13          * ZET**2
            bet25 = bet25 + del13          * ZET**3
            bet36 = bet36 + del29 * BET    * ZET**2
            bet35 = bet35 + del30          * ZET   
            bet37 = bet37 + del30 * BET**2 * ZET**2
            bet06 = bet06 + ram02          * ZET**2
            bet09 = bet09 + ram02          * ZET**3
            bet07 = bet07 + ram03          * ZET**2
            bet24 = bet24 + ram14 * BET    * ZET**2
            bet22 = bet22 + ram15          * ZET**2
            
            dsps( 1) = dsps( 1) + del01          * ZET   
            dsps(11) = dsps(11) + del11          * ZET   
            dsps(12) = dsps(12) + del12          * ZET   
            dsps(13) = dsps(13) + del13          * ZET   
            dsps(17) = dsps(17) + del12 * BET    * ZET   
            dsps(29) = dsps(29) + del29          * ZET   
            dsps(31) = dsps(31) + del29 * BET    * ZET   
            dsps(30) = dsps(30) + del30 * BET    * ZET   
            dsps(33) = dsps(33) + del30 * BET**2 * ZET   
            dsps(39) = dsps(39) + del39 * BET**2 * ZET 
            dsps( 2) = dsps( 2) + ram02          * ZET   
            dsps( 3) = dsps( 3) + ram03          * ZET   
            dsps( 4) = dsps( 4) + ram04          * ZET   
            dsps(14) = dsps(14) + ram14          * ZET   
            dsps(15) = dsps(15) + ram15          * ZET   
            dsps(16) = dsps(16) + ram16          * ZET   
            dsps(18) = dsps(18) + ram14 * BET    * ZET   
            dsps(19) = dsps(19) + ram19 * BET    * ZET   
            dsps(32) = dsps(32) + ram32 * BET    * ZET   
            dsps(34) = dsps(34) + ram34 * BET**2 * ZET 
            
         enddo
         DSPS( 5) = DSPS( 5) + bet05 * ALP   
         DSPS( 6) = DSPS( 6) + bet06 * ALP   
         DSPS( 7) = DSPS( 7) + bet07 * ALP   
         DSPS( 8) = DSPS( 8) + bet08 * ALP**2
         DSPS( 9) = DSPS( 9) + bet09 * ALP**2
         DSPS(10) = DSPS(10) + bet10 * ALP**3
         DSPS(20) = DSPS(20) + bet20 * ALP   
         DSPS(21) = DSPS(21) + bet21 * ALP   
         DSPS(22) = DSPS(22) + bet22 * ALP   
         DSPS(23) = DSPS(23) + bet23 * ALP   
         DSPS(24) = DSPS(24) + bet24 * ALP   
         DSPS(25) = DSPS(25) + bet25 * ALP**2
         DSPS(26) = DSPS(26) + bet26 * ALP**2
         DSPS(35) = DSPS(35) + bet35 * ALP   
         DSPS(36) = DSPS(36) + bet36 * ALP   
         DSPS(37) = DSPS(37) + bet37 * ALP   
         
      enddo
#ifdef PL_OMP_ERI
!$omp end do
!$omp end parallel
#endif
      end
C** IKDPSS ===========================================================
C          CALUCATE KERNELS FOR DPSS TYPE
C====================================================================

      subroutine ikdpss(
     &           dpss ,
     &           fdat ,  edat  ,
     &           yp   ,  yq    ,
     &           uu   ,  uv    ,
     &           z    ,  zetpq , 
     &           nasp ,  mxsa  ,  mxprm,  ntprm,  
     &           iasp ,  jasp  ,  kasp ,  lasp ,
     &           ishel,  jshel ,  kshel,  lshel     )


      implicit double precision  (a-h,o-z)
      dimension fdat(31,2001)
      dimension edat(2001)
      dimension zetpq(mxprm,mxprm,mxsa,nasp,mxsa,nasp)
      dimension yp(3,mxprm,mxprm),yq(3,mxprm,mxprm)
      dimension uu(mxprm,mxprm),uv(mxprm,mxprm)
      dimension z(mxprm,mxsa,nasp)
      dimension ntprm(mxsa,nasp)
      dimension dpss(38)
 
      real(8),parameter::pi    = 3.14159265358979323846d+00
      real(8),parameter::tf(25) = (/
     &                33.0,37.0,41.0,43.0,46.0,49.0,
     &                51.0,54.0,56.0,58.0,61.0,63.0,
     &                66.0,68.0,70.0,72.0,74.0,80.0,
     &                85.0,90.0,95.0,95.0,98.0,99.0,
     &                99.0                           /)

      real(8),parameter::ga0 = 1.0d+00
      real(8),parameter::ga1 = 1.000 000 000 000 005 298d+00
      real(8),parameter::ga2 = 0.5d+00
      real(8),parameter::ga3 = 0.166 666 666 598 849 826d+00
      real(8),parameter::ga4 = 0.416 666 666 666 666 667d-01
      real(8),parameter::ga5 = 0.833 355 034 722 222 222d-02
      real(8),parameter::ga6 = 0.138 888 888 888 888 889d-02

      real(8),parameter::rminv(5) = (/
     &   1.0d+00,  0.333333333333333333d+00,  0.20d+00,
     &   0.14285714285714286d+00,  0.11111111111111111d+00 /)

#ifdef PL_SIMD_ERI
      dimension dmx(4,mxprm), val(2,mxprm)
#endif

      do ijk=1,38
         dpss(ijk)=0.0d0
      enddo
      
#ifdef PL_OMP_ERI
!$omp parallel reduction(+:dpss)
!$omp& default(private)
!$omp& shared(fdat,edat,
!$omp&        zetpq,ntprm,z,yp,yq,uu,uv,
!$omp&        ishel,jshel,kshel,lshel,iasp,jasp,kasp,lasp)
#endif
      igp = ntprm(ishel,iasp)
      jgp = ntprm(jshel,jasp)
      kgp = ntprm(kshel,kasp)
      lgp = ntprm(lshel,lasp)

      pi2 = 2.0d0*pi
      
#ifdef PL_OMP_ERI
!$omp do
#endif
      DO I = 1, IGP
         alp = z(i,ishel,iasp)
         bet05 = 0.0d0
         bet06 = 0.0d0
         bet07 = 0.0d0
         bet08 = 0.0d0
         bet09 = 0.0d0
         bet10 = 0.0d0
         bet14 = 0.0d0
         bet15 = 0.0d0
         bet16 = 0.0d0
         bet17 = 0.0d0
         bet18 = 0.0d0
         bet19 = 0.0d0
         bet20 = 0.0d0
         bet21 = 0.0d0
         bet22 = 0.0d0
         bet30 = 0.0d0
         bet32 = 0.0d0
         bet33 = 0.0d0
         bet34 = 0.0d0
         bet35 = 0.0d0
         bet37 = 0.0d0
         bet38 = 0.0d0
         
         DO J = 1, JGP
            bet = z(j,jshel,jasp)
            zet = zetpq(j,i,jshel,jasp,ishel,iasp)
            px  = yp (1,j,i)
            py  = yp (2,j,i)
            pz  = yp (3,j,i)
            up  = uu  (j,i)
            del01  = 0.0d0
            del11  = 0.0d0
            del23  = 0.0d0
            del36  = 0.0d0
            ram02  = 0.0d0
            ram03  = 0.0d0
            ram04  = 0.0d0
            ram12  = 0.0d0
            ram13  = 0.0d0
            ram24  = 0.0d0
            
            DO K = 1, KGP
               ram = z(k,kshel,kasp)
               del02 = 0.0d0 
               del03 = 0.0d0
               del04 = 0.0d0
               del12 = 0.0d0 
               del13 = 0.0d0
               del24 = 0.0d0 
               
               DO L = 1, LGP
                  del = z(l,lshel,lasp)
                  eta = zetpq(l,k,lshel,lasp,kshel,kasp)
                  reta = 1.0d0 / eta
                  qpx =  yq(1,l,k) - px
                  qpy =  yq(2,l,k) - py
                  qpz =  yq(3,l,k) - pz
                  vq  = uv  (l,k)
                  
                  ROO2 = 1.0d0 / (ZET + ETA)
                  QP2  = QPX*QPX + QPY*QPY + QPZ*QPZ
                  T    = 0.5d0* ROO2*QP2
                  
                  IF (T. LE .TF(4)) THEN
                     
                     IT = IDINT((T+0.025D+00)*20.0D+00)
                     DT = 0.05D+00*IT - T
                     IT = IT + 1
                     DT4 = DT*0.25D+00
                     DT5 = DT*0.2D+00
                     DT6 = DT*0.166666666666666667D+00
                     DT2 = DT4 + DT4
                     DT3 = DT6 + DT6
                     F3   = (((((FDAT(10,IT)*DT6   + FDAT(9,IT))*DT5
     &                    + FDAT(8,IT))*DT4  + FDAT(7,IT))*DT3
     &                    + FDAT(6,IT))*DT2  + FDAT(5,IT))*DT
     &                    + FDAT(4,IT)
                     
                     EED  = (((((GA6*DT + GA5)*DT + GA4)*DT
     &                    + GA3)*DT + GA2)*DT + GA1)*DT + GA0
                     EE   = EDAT(IT)*EED
                     
                     T2   = T*2.0d0
                     
                     F2   = (T2*F3 + EE)*RMINV(3)
                     F1   = (T2*F2 + EE)*RMINV(2)
                     F0   =  T2*F1 + EE
                     FPP =  UP * VQ * DSQRT(ROO2)
                     DM0 = FPP*F0
                     DM1 = FPP*F1 *  ROO2
                     DM2 = FPP*F2 *( ROO2)**2
                     DM3 = FPP*F3 *( ROO2)**3
                     
                  ELSE
                     qinv = 1.0d0/qp2
                     dm0=  up*vq*dsqrt( pi2*qinv)*0.5d0
                     dm1 =        qinv *  dm0
                     dm2 =  3.0d0*qinv *  dm1
                     dm3 =  5.0d0*qinv *  dm2
                     
                  END IF

#ifdef PL_SIMD_ERI
                  val(1,l) = eta
                  val(2,l) = del

                  dmx(1,l) = dm0
                  dmx(2,l) = dm1
                  dmx(3,l) = dm2
                  dmx(4,l) = dm3
               enddo
               
               do l = 1, lgp
                  eta = val(1,l)
                  del = val(2,l)
                  reta = 1.0d0 / eta

                  dm0 = dmx(1,l)
                  dm1 = dmx(2,l)
                  dm2 = dmx(3,l)
                  dm3 = dmx(4,l)
#endif

C**   KERNEL DPSS =====================================================
C     CALCULATE & CONTRACT KERNELS FOR DPSS TYPE
C 1:T003000  8:T205000 15:D103101 22:D305000 29:U023101 36:Z012000
C 2:T003101  9:T205101 16:D103202 23:U002000 30:U103000 37:Z102000
C 3:T003202 10:T306000 17:D114000 24:U002101 31:D104000 38:Z123000
C 4:T003303 11:D013000 18:D114101 25:D003000 32:U113000
C 5:T104000 12:D013101 19:D204000 26:D003101 33:U113101
C 6:T104101 13:D013202 20:D204101 27:U013000 34:U124000
C 7:T104202 14:D103000 21:D215000 28:U023000 35:U214000
c===================================================================
                  
                  del01 = del01 + DM3 
                  del02 = del02 + DM3     *ETA
                  del03 = del03 + DM3     *ETA**2
                  del04 = del04 + DM3     *ETA**3
!==========================================--
                  del11 = del11 + DM2            
                  del12 = del12 + DM2     *ETA   
                  del13 = del13 + DM2     *ETA**2
!==========================================--
                  del23 = del23 + DM1            
                  del24 = del24 + DM1     *ETA   
!==========================================--
                  del36 = del36 + DM0            
                  
               enddo
               ram02 = ram02 + del02 * RAM   
               ram12 = ram12 + del12 * RAM   
               ram24 = ram24 + del24 * RAM   
               ram03 = ram03 + del03 * RAM**2
               ram13 = ram13 + del13 * RAM**2
               ram04 = ram04 + del04 * RAM**3
               
            enddo
            dpss( 1) =dpss( 1)+del01              *ZET**2                     
            dpss(11) =dpss(11)+del11       *BET   *ZET**2                     
            dpss(23) =dpss(23)+del23              *ZET                        
            dpss(25) =dpss(25)+del11              *ZET**2                     
            dpss(27) =dpss(27)+del23       *BET   *ZET**2                     
            dpss(28) =dpss(28)+del23       *BET**2*ZET**2                     
            dpss(36) =dpss(36)+del36       *BET   *ZET                        
            
            dpss( 2) =dpss( 2)+ram02              *ZET**2
            dpss( 3) =dpss( 3)+ram03              *ZET**2
            dpss( 4) =dpss( 4)+ram04              *ZET**2
            dpss(12) =dpss(12)+ram12       *BET   *ZET**2
            dpss(13) =dpss(13)+ram13       *BET   *ZET**2
            dpss(24) =dpss(24)+ram24              *ZET   
            dpss(26) =dpss(26)+ram12              *ZET**2
            dpss(29) =dpss(29)+ram24       *BET**2*ZET**2
            
            bet05 = bet05 + del01                  *ZET**3
            bet08 = bet08 + del01                  *ZET**4
            bet10 = bet10 + del01                  *ZET**5
            bet14 = bet14 + del11                  *ZET**2
            bet19 = bet19 + del11                  *ZET**3
            bet22 = bet22 + del11                  *ZET**4
            bet17 = bet17 + del11          *BET    *ZET**3
            bet21 = bet21 + del11          *BET    *ZET**4
            bet30 = bet30 + del23                  *ZET**2
            bet32 = bet32 + del23          *BET    *ZET**2
            bet35 = bet35 + del23          *BET    *ZET**3
            bet34 = bet34 + del23          *BET**2 *ZET**3
            bet37 = bet37 + del36                  *ZET   
            bet38 = bet38 + del36          *BET**2 *ZET**2
            
            bet06 = bet06 + ram02                  *ZET**3
            bet09 = bet09 + ram02                  *ZET**4
            bet16 = bet16 + ram13                  *ZET**2
            bet07 = bet07 + ram03                  *ZET**3
            bet15 = bet15 + ram12                  *ZET**2
            bet18 = bet18 + ram12          *BET    *ZET**3
            bet20 = bet20 + ram12                  *ZET**3
            bet33 = bet33 + ram24          *BET    *ZET**2
            
         enddo
         dpss( 5) = dpss( 5) + bet05 * ALP   
         dpss( 6) = dpss( 6) + bet06 * ALP   
         dpss( 7) = dpss( 7) + bet07 * ALP   
         dpss( 8) = dpss( 8) + bet08 * ALP**2
         dpss( 9) = dpss( 9) + bet09 * ALP**2
         dpss(10) = dpss(10) + bet10 * ALP**3
         dpss(14) = dpss(14) + bet14 * ALP   
         dpss(15) = dpss(15) + bet15 * ALP   
         dpss(16) = dpss(16) + bet16 * ALP   
         dpss(17) = dpss(17) + bet17 * ALP   
         dpss(18) = dpss(18) + bet18 * ALP   
         dpss(19) = dpss(19) + bet19 * ALP**2
         dpss(20) = dpss(20) + bet20 * ALP**2
         dpss(21) = dpss(21) + bet21 * ALP**2
         dpss(22) = dpss(22) + bet22 * ALP**3
         dpss(30) = dpss(30) + bet30 * ALP   
         dpss(31) = dpss(31) + bet19 * ALP   
         dpss(32) = dpss(32) + bet32 * ALP   
         dpss(33) = dpss(33) + bet33 * ALP   
         dpss(34) = dpss(34) + bet34 * ALP   
         dpss(35) = dpss(35) + bet35 * ALP**2
         dpss(37) = dpss(37) + bet37 * ALP   
         dpss(38) = dpss(38) + bet38 * ALP   
         
      enddo
#ifdef PL_OMP_ERI
!$omp end do
!$omp end parallel
#endif
      
      end
C** IKDSPP ===========================================================
C          CALUCATE KERNELS FOR DSPP TYPE
C====================================================================

      subroutine ikdspp(
     &           dspp ,
     &           fdat ,  edat  ,
     &           yp   ,  yq    ,
     &           uu   ,  uv    ,
     &           z    ,  zetpq , 
     &           nasp ,  mxsa  ,  mxprm,  ntprm,  
     &           iasp ,  jasp  ,  kasp ,  lasp ,
     &           ishel,  jshel ,  kshel,  lshel     )


      implicit double precision  (a-h,o-z)
      dimension fdat(31,2001)
      dimension edat(2001)
      dimension zetpq(mxprm,mxprm,mxsa,nasp,mxsa,nasp)
      dimension yp(3,mxprm,mxprm),yq(3,mxprm,mxprm)
      dimension uu(mxprm,mxprm),uv(mxprm,mxprm)
      dimension z(mxprm,mxsa,nasp)
      dimension ntprm(mxsa,nasp)
      dimension dspp(121)
 
      real(8),parameter::pi    = 3.14159265358979323846d+00
      real(8),parameter::tf(25) = (/
     &                33.0,37.0,41.0,43.0,46.0,49.0,
     &                51.0,54.0,56.0,58.0,61.0,63.0,
     &                66.0,68.0,70.0,72.0,74.0,80.0,
     &                85.0,90.0,95.0,95.0,98.0,99.0,
     &                99.0                           /)

      real(8),parameter::ga0 = 1.0d+00
      real(8),parameter::ga1 = 1.000 000 000 000 005 298d+00
      real(8),parameter::ga2 = 0.5d+00
      real(8),parameter::ga3 = 0.166 666 666 598 849 826d+00
      real(8),parameter::ga4 = 0.416 666 666 666 666 667d-01
      real(8),parameter::ga5 = 0.833 355 034 722 222 222d-02
      real(8),parameter::ga6 = 0.138 888 888 888 888 889d-02

      real(8),parameter::rminv(5) = (/
     &   1.0d+00,  0.333333333333333333d+00,  0.20d+00,
     &   0.14285714285714286d+00,  0.11111111111111111d+00 /)

#ifdef PL_SIMD_ERI
      dimension dmx(5,mxprm), val(2,mxprm)
#endif

      DO IJK=1,121
         DSPP(IJK)=0.0d0
      ENDDO
      
#ifdef PL_OMP_ERI
!$omp parallel reduction(+:dspp)
!$omp& default(private)
!$omp& shared(fdat,edat,
!$omp&        zetpq,ntprm,z,yp,yq,uu,uv,
!$omp&        ishel,jshel,kshel,lshel,iasp,jasp,kasp,lasp)
#endif
      igp = ntprm(ishel,iasp)
      jgp = ntprm(jshel,jasp)
      kgp = ntprm(kshel,kasp)
      lgp = ntprm(lshel,lasp)

      pi2 = 2.0d0*pi

#ifdef PL_OMP_ERI
!$omp do
#endif
      DO I = 1, IGP
         alp = z(i,ishel,iasp)
         bet006 =0.0d0
         bet007 =0.0d0
         bet008 =0.0d0
         bet009 =0.0d0
         bet010 =0.0d0
         bet011 =0.0d0
         bet012 =0.0d0
         bet013 =0.0d0
         bet014 =0.0d0
         bet015 =0.0d0
         bet031 =0.0d0
         bet032 =0.0d0
         bet033 =0.0d0
         bet034 =0.0d0
         bet035 =0.0d0
         bet036 =0.0d0
         bet037 =0.0d0
         bet038 =0.0d0
         bet039 =0.0d0
         bet040 =0.0d0
         bet041 =0.0d0
         bet042 =0.0d0
         bet043 =0.0d0
         bet044 =0.0d0
         bet045 =0.0d0
         bet046 =0.0d0
         bet047 =0.0d0
         bet048 =0.0d0
         bet049 =0.0d0
         bet050 =0.0d0
         bet051 =0.0d0
         bet075 =0.0d0
         bet076 =0.0d0
         bet077 =0.0d0
         bet078 =0.0d0
         bet079 =0.0d0
         bet080 =0.0d0
         bet081 =0.0d0
         bet082 =0.0d0
         bet083 =0.0d0
         bet084 =0.0d0
         bet085 =0.0d0
         bet086 =0.0d0
         bet087 =0.0d0
         bet088 =0.0d0
         bet089 =0.0d0
         bet090 =0.0d0
         bet091 =0.0d0
         bet092 =0.0d0
         bet093 =0.0d0
         bet094 =0.0d0
         bet112 =0.0d0
         bet113 =0.0d0
         bet114 =0.0d0
         bet115 =0.0d0
         bet116 =0.0d0
         bet117 =0.0d0
         DO J = 1, JGP
            del001 =0.0d0
            del016 =0.0d0
            del017 =0.0d0
            del055 =0.0d0
            del056 =0.0d0
            del057 =0.0d0
            del100 =0.0d0
            del103 =0.0d0
            del107 =0.0d0
            del120 =0.0d0
            ram002 =0.0d0
            ram003 =0.0d0
            ram004 =0.0d0
            ram005 =0.0d0
            ram018 =0.0d0
            ram019 =0.0d0
            ram020 =0.0d0
            ram021 =0.0d0
            ram022 =0.0d0
            ram023 =0.0d0
            ram024 =0.0d0
            ram025 =0.0d0
            ram026 =0.0d0
            ram030 =0.0d0
            ram058 =0.0d0
            ram059 =0.0d0
            ram060 =0.0d0
            ram061 =0.0d0
            ram062 =0.0d0
            ram063 =0.0d0
            ram067 =0.0d0
            ram070 =0.0d0
            ram071 =0.0d0
            ram074 =0.0d0
            ram101 =0.0d0
            ram104 =0.0d0
            ram106 =0.0d0
            ram110 =0.0d0
            ram111 =0.0d0
            ram121 =0.0d0          
            bet = z(j,jshel,jasp)
            zet = zetpq(j,i,jshel,jasp,ishel,iasp)
            px  = yp (1,j,i)
            py  = yp (2,j,i)
            pz  = yp (3,j,i)
            up  = uu(j,i)
            
            DO K = 1, KGP
               ram = z(k,kshel,kasp)
               del002 = 0.0d0
               del003 = 0.0d0
               del004 = 0.0d0
               del005 = 0.0d0
               del018 = 0.0d0
               del019 = 0.0d0
               del020 = 0.0d0
               del022 = 0.0d0
               del023 = 0.0d0
               del025 = 0.0d0
               del026 = 0.0d0
               del053 = 0.0d0
               del054 = 0.0d0
               del058 = 0.0d0
               del059 = 0.0d0
               del060 = 0.0d0
               del063 = 0.0d0
               del097 = 0.0d0
               del098 = 0.0d0
               del099 = 0.0d0 
               del101 = 0.0d0
               del119 = 0.0d0
               DO L = 1, LGP
                  del = z(l,lshel,lasp)
                  eta = zetpq(l,k,lshel,lasp,kshel,kasp)
                  qpx =  yq(1,l,k) - px
                  qpy =  yq(2,l,k) - py
                  qpz =  yq(3,l,k) - pz
                  vq  = uv(l,k)
                  
                  ROO2 = 1.0d0 / (ZET + ETA)
                  qp2  = qpx*qpx + qpy*qpy + qpz*qpz
                  T    = 0.5d0* ROO2*qp2
                  
                  IF (T. LE .TF(5)) THEN
                     
                     IT = IDINT((T+0.025D+00)*20.0D+00)
                     DT = 0.05D+00*IT - T
                     IT = IT + 1
                     DT4 = DT*0.25D+00
                     DT5 = DT*0.2D+00
                     DT6 = DT*0.166666666666666667D+00
                     DT2 = DT4 + DT4
                     DT3 = DT6 + DT6
                     F4   = (((((FDAT(11,IT)*DT6   + FDAT(10,IT))*DT5
     &                    + FDAT(9,IT))*DT4  + FDAT(8,IT))*DT3
     &                    + FDAT(7,IT))*DT2  + FDAT(6,IT))*DT
     &                    + FDAT(5,IT)
                     
                     EED  = (((((GA6*DT + GA5)*DT + GA4)*DT
     &                    + GA3)*DT + GA2)*DT + GA1)*DT + GA0
                     EE   = EDAT(IT)*EED
                     
                     T2   = T*2.0d0
                     F3   = (T2*F4 + EE)*RMINV(4)
                     F2   = (T2*F3 + EE)*RMINV(3)
                     F1   = (T2*F2 + EE)*RMINV(2)
                     F0   =  T2*F1 + EE
                     FPP =  UP * VQ * DSQRT(ROO2)
                     
                     DM0 = FPP*F0
                     DM1 = FPP*F1 *  ROO2
                     DM2 = FPP*F2 *( ROO2)**2
                     DM3 = FPP*F3 *( ROO2)**3
                     DM4 = FPP*F4 *( ROO2)**4
                     
                  ELSE 
                     
                     qinv = 1.0d0/qp2
                     dm0=  up*vq*dsqrt( pi2*qinv)*0.5d0
                     dm1 =        qinv *  dm0
                     dm2 =  3.0d0*qinv *  dm1
                     dm3 =  5.0d0*qinv *  dm2
                     dm4 =  7.0d0*qinv *  dm3
                  ENDIF

#ifdef PL_SIMD_ERI
                  val(1,l) = eta
                  val(2,l) = del

                  dmx(1,l) = dm0
                  dmx(2,l) = dm1
                  dmx(3,l) = dm2
                  dmx(4,l) = dm3
                  dmx(5,l) = dm4
               enddo
               
               do l = 1, lgp
                  eta = val(1,l)
                  del = val(2,l)

                  dm0 = dmx(1,l)
                  dm1 = dmx(2,l)
                  dm2 = dmx(3,l)
                  dm3 = dmx(4,l)
                  dm4 = dmx(5,l)
#endif

C** KERNEL DSPP =====================================================
C           CALCULATE & CONTRACT KERNELS FOR DSPP TYPE
C====================================================================
C 18:T002102 38:T103304 58:D002102 78:D103012  98:U001113 118:Z001001
C 19:T002103 39:T113002 59:D002112 79:D103102  99:U001203 119:Z001112
C 20:T002113 40:T113103 60:D002113 80:D103112 100:U002001 120:Z022001
                  del001 = del001 + dm4       *eta   
                  del002 = del002 + dm4       *eta**2
                  del003 = del003 + dm4       *eta**3
                  del004 = del004 + dm4       *eta**4
                  del005 = del005 + dm4       *eta**5
!================================================
                  del016 = del016 + dm3       *eta   
                  del018 = del018 + dm3       *eta   
                  del019 = del019 + dm3       *eta**2
                  del022 = del022 + dm3       *eta**3
                  del026 = del026 + dm3       *eta**4
                  del017 = del017 + dm3*del   *eta   
                  del020 = del020 + dm3*del   *eta**2
                  del023 = del023 + dm3*del   *eta**3
                  del025 = del025 + dm3*del   *eta**4
!================================================
                  del055 = del055 + dm2              
                  del056 = del056 + dm2       *eta   
                  del058 = del058 + dm2       *eta   
                  del053 = del053 + dm2       *eta**2
                  del054 = del054 + dm2       *eta**3
                  del057 = del057 + dm2*del   *eta   
                  del059 = del059 + dm2*del   *eta   
                  del060 = del060 + dm2*del   *eta**2
                  del063 = del063 + dm2*del   *eta**3
!============================================
                  del100 = del100 + dm1           
                  del097 = del097 + dm1       *eta   
                  del107 = del107 + dm1       *eta   
                  del099 = del099 + dm1       *eta**2
                  del101 = del101 + dm1*del   *eta   
                  del103 = del103 + dm1*del   *eta   
                  del098 = del098 + dm1*del   *eta**2
!============================================
                  del120 = del120 + dm0       
                  del119 = del119 + dm0*del   *eta   
                  
                  dspp( 52) = dspp( 52)+dm2           *eta
                  dspp( 95) = dspp( 95)+dm1           *eta   
                  dspp( 96) = dspp( 96)+dm1      *del *eta   
                  dspp(118) = dspp(118)+dm0   
                  
               enddo
               ram002 = ram002 + del002 * ram
               ram018 = ram018 + del018 * ram
               ram019 = ram019 + del019 * ram
               ram020 = ram020 + del020 * ram
               ram058 = ram058 + del058 * ram
               ram059 = ram059 + del059 * ram
               ram060 = ram060 + del060 * ram
               ram067 = ram067 + del053 * ram
               ram101 = ram101 + del101 * ram
               ram104 = ram104 + del097 * ram
               ram110 = ram110 + del098 * ram
               ram121 = ram121 + del119 * ram   
               
               ram003 = ram003 + del003 * ram**2
               ram021 = ram021 + del019 * ram**2
               ram022 = ram022 + del022 * ram**2
               ram023 = ram023 + del023 * ram**2
               ram061 = ram061 + del053 * ram**2
               ram062 = ram062 + del060 * ram**2
               ram070 = ram070 + del063 * ram**2
               ram074 = ram074 + del054 * ram**2
               ram106 = ram106 + del098 * ram**2
               ram111 = ram111 + del099 * ram**2
               
               ram004 = ram004 + del004 * ram**3
               ram024 = ram024 + del022 * ram**3
               ram025 = ram025 + del025 * ram**3
               ram030 = ram030 + del026 * ram**3
               ram063 = ram063 + del063 * ram**3
               ram071 = ram071 + del054 * ram**3
               
               ram005 = ram005 + del005 * ram**4
               ram026 = ram026 + del026 * ram**4
               
               dspp( 53) = dspp( 53) + del053     *ram   
               dspp( 54) = dspp( 54) + del054     *ram**2
               dspp( 97) = dspp( 97) + del097     *ram   
               dspp( 98) = dspp( 98) + del098     *ram   
               dspp( 99) = dspp( 99) + del099     *ram**2
               dspp(119) = dspp(119) + del119     *ram   
               
            enddo
            dspp(  1)=dspp(  1)+del001              *zet
            dspp( 16)=dspp( 16)+del016              *zet
            dspp( 17)=dspp( 17)+del017              *zet
            dspp( 27)=dspp( 27)+del016       *bet   *zet
            dspp( 55)=dspp( 55)+del055              *zet
            dspp( 56)=dspp( 56)+del056              *zet
            dspp( 57)=dspp( 57)+del057              *zet
            dspp( 64)=dspp( 64)+del056       *bet   *zet
            dspp( 65)=dspp( 65)+del057       *bet   *zet
            dspp( 72)=dspp( 72)+del056       *bet**2*zet
            dspp(100)=dspp(100)+del100              *zet
            dspp(102)=dspp(102)+del100       *bet   *zet
            dspp(103)=dspp(103)+del103       *bet   *zet
            dspp(107)=dspp(107)+del107       *bet**2*zet
            dspp(108)=dspp(108)+del103       *bet**2*zet
            dspp(120)=dspp(120)+del120       *bet**2*zet
            
            bet006 = bet006 + del001          * zet**2
            bet010 = bet010 + del001          * zet**3
            bet013 = bet013 + del001          * zet**4
            bet015 = bet015 + del001          * zet**5
            bet031 = bet031 + del016          * zet**2
            bet039 = bet039 + del016 * bet    * zet**2
            bet042 = bet042 + del016          * zet**3
            bet047 = bet047 + del016 * bet    * zet**3
            bet051 = bet051 + del016 * bet    * zet**4
            bet032 = bet032 + del017          * zet**2
            bet043 = bet043 + del017          * zet**3
            bet049 = bet049 + del017          * zet**4
            bet077 = bet077 + del055          * zet**2
            bet090 = bet090 + del055          * zet**3
            bet075 = bet075 + del056          * zet   
            bet089 = bet089 + del056          * zet**2
            bet082 = bet082 + del056 * bet    * zet**2
            bet087 = bet087 + del056 * bet**2 * zet**2
            bet094 = bet094 + del056 * bet**2 * zet**3
            bet078 = bet078 + del057          * zet**2
            bet083 = bet083 + del057 * bet    * zet**2
            bet092 = bet092 + del057 * bet    * zet**3
            bet114 = bet114 + del100 * bet    * zet**2
            bet112 = bet112 + del103          * zet   
            bet116 = bet116 + del103 * bet**2 * zet**2
            
            dspp(  2)=dspp(  2)+ram002               *zet
            dspp(  3)=dspp(  3)+ram003               *zet
            dspp(  4)=dspp(  4)+ram004               *zet
            dspp(  5)=dspp(  5)+ram005               *zet
            dspp( 18)=dspp( 18)+ram018               *zet    
            dspp( 19)=dspp( 19)+ram019               *zet    
            dspp( 20)=dspp( 20)+ram020               *zet    
            dspp( 21)=dspp( 21)+ram021               *zet    
            dspp( 22)=dspp( 22)+ram022               *zet    
            dspp( 23)=dspp( 23)+ram023               *zet    
            dspp( 24)=dspp( 24)+ram024               *zet    
            dspp( 25)=dspp( 25)+ram025               *zet    
            dspp( 26)=dspp( 26)+ram026               *zet    
            dspp( 28)=dspp( 28)+ram019        *bet   *zet    
            dspp( 29)=dspp( 29)+ram022        *bet   *zet    
            dspp( 30)=dspp( 30)+ram030        *bet   *zet    
            dspp( 58)=dspp( 58)+ram058               *zet    
            dspp( 59)=dspp( 59)+ram059               *zet    
            dspp( 60)=dspp( 60)+ram060               *zet    
            dspp( 61)=dspp( 61)+ram061               *zet    
            dspp( 62)=dspp( 62)+ram062               *zet    
            dspp( 63)=dspp( 63)+ram063               *zet    
            dspp( 66)=dspp( 66)+ram058        *bet   *zet    
            dspp( 67)=dspp( 67)+ram067        *bet   *zet    
            dspp( 68)=dspp( 68)+ram060        *bet   *zet    
            dspp( 69)=dspp( 69)+ram061        *bet   *zet    
            dspp( 70)=dspp( 70)+ram070        *bet   *zet    
            dspp( 71)=dspp( 71)+ram071        *bet   *zet    
            dspp( 73)=dspp( 73)+ram067        *bet**2*zet    
            dspp( 74)=dspp( 74)+ram074        *bet**2*zet    
            dspp(101)=dspp(101)+ram101               *zet    
            dspp(104)=dspp(104)+ram104        *bet   *zet    
            dspp(105)=dspp(105)+ram101        *bet   *zet    
            dspp(106)=dspp(106)+ram106        *bet   *zet    
            dspp(109)=dspp(109)+ram104        *bet**2*zet    
            dspp(110)=dspp(110)+ram110        *bet**2*zet    
            dspp(111)=dspp(111)+ram111        *bet**2*zet    
            dspp(121)=dspp(121)+ram121        *bet**2*zet    
            
            bet007 = bet007 + ram002          * zet**2 
            bet011 = bet011 + ram002          * zet**3 
            bet014 = bet014 + ram002          * zet**4 
            bet008 = bet008 + ram003          * zet**2 
            bet012 = bet012 + ram003          * zet**3 
            bet009 = bet009 + ram004          * zet**2 
            bet033 = bet033 + ram018          * zet**2 
            bet044 = bet044 + ram018          * zet**3 
            bet050 = bet050 + ram018          * zet**4 
            bet034 = bet034 + ram019          * zet**2 
            bet040 = bet040 + ram019 * bet    * zet**2 
            bet048 = bet048 + ram019 * bet    * zet**3 
            bet035 = bet035 + ram020          * zet**2 
            bet045 = bet045 + ram020          * zet**3 
            bet036 = bet036 + ram021          * zet**2 
            bet046 = bet046 + ram021          * zet**3 
            bet041 = bet041 + ram022 * bet    * zet**2 
            bet037 = bet037 + ram023          * zet**2 
            bet038 = bet038 + ram024          * zet**2 
            bet076 = bet076 + ram067          * zet    
            bet088 = bet088 + ram067 * bet**2 * zet**2 
            bet079 = bet079 + ram058          * zet**2 
            bet084 = bet084 + ram058 * bet    * zet**2 
            bet093 = bet093 + ram058 * bet    * zet**3 
            bet080 = bet080 + ram059          * zet**2 
            bet091 = bet091 + ram059          * zet**3 
            bet085 = bet085 + ram060 * bet    * zet**2 
            bet086 = bet086 + ram061 * bet    * zet**2 
            bet081 = bet081 + ram062          * zet**2 
            bet115 = bet115 + ram101 * bet    * zet**2 
            bet113 = bet113 + ram104          * zet    
            bet117 = bet117 + ram104 * bet**2 * zet**2 
         enddo
         dspp(  6) = dspp(  6) + bet006 * alp   
         dspp(  7) = dspp(  7) + bet007 * alp   
         dspp(  8) = dspp(  8) + bet008 * alp   
         dspp(  9) = dspp(  9) + bet009 * alp   
         dspp( 10) = dspp( 10) + bet010 * alp**2
         dspp( 11) = dspp( 11) + bet011 * alp**2
         dspp( 12) = dspp( 12) + bet012 * alp**2
         dspp( 13) = dspp( 13) + bet013 * alp**3
         dspp( 14) = dspp( 14) + bet014 * alp**3
         dspp( 15) = dspp( 15) + bet015 * alp**4
         dspp( 31) = dspp( 31) + bet031 * alp   
         dspp( 32) = dspp( 32) + bet032 * alp   
         dspp( 33) = dspp( 33) + bet033 * alp   
         dspp( 34) = dspp( 34) + bet034 * alp   
         dspp( 35) = dspp( 35) + bet035 * alp   
         dspp( 36) = dspp( 36) + bet036 * alp   
         dspp( 37) = dspp( 37) + bet037 * alp   
         dspp( 38) = dspp( 38) + bet038 * alp   
         dspp( 39) = dspp( 39) + bet039 * alp   
         dspp( 40) = dspp( 40) + bet040 * alp   
         dspp( 41) = dspp( 41) + bet041 * alp   
         dspp( 42) = dspp( 42) + bet042 * alp**2
         dspp( 43) = dspp( 43) + bet043 * alp**2
         dspp( 44) = dspp( 44) + bet044 * alp**2
         dspp( 45) = dspp( 45) + bet045 * alp**2
         dspp( 46) = dspp( 46) + bet046 * alp**2
         dspp( 47) = dspp( 47) + bet047 * alp**2
         dspp( 48) = dspp( 48) + bet048 * alp**2
         dspp( 49) = dspp( 49) + bet049 * alp**3
         dspp( 50) = dspp( 50) + bet050 * alp**3
         dspp( 51) = dspp( 51) + bet051 * alp**3
         dspp( 75) = dspp( 75) + bet075 * alp   
         dspp( 76) = dspp( 76) + bet076 * alp   
         dspp( 77) = dspp( 77) + bet077 * alp   
         dspp( 78) = dspp( 78) + bet078 * alp   
         dspp( 79) = dspp( 79) + bet079 * alp   
         dspp( 80) = dspp( 80) + bet080 * alp   
         dspp( 81) = dspp( 81) + bet081 * alp   
         dspp( 82) = dspp( 82) + bet082 * alp   
         dspp( 83) = dspp( 83) + bet083 * alp   
         dspp( 84) = dspp( 84) + bet084 * alp   
         dspp( 85) = dspp( 85) + bet085 * alp   
         dspp( 86) = dspp( 86) + bet086 * alp   
         dspp( 87) = dspp( 87) + bet087 * alp   
         dspp( 88) = dspp( 88) + bet088 * alp   
         dspp( 89) = dspp( 89) + bet089 * alp**2
         dspp( 90) = dspp( 90) + bet090 * alp**2
         dspp( 91) = dspp( 91) + bet091 * alp**2
         dspp( 92) = dspp( 92) + bet092 * alp**2
         dspp( 93) = dspp( 93) + bet093 * alp**2
         dspp( 94) = dspp( 94) + bet094 * alp**2
         dspp(112) = dspp(112) + bet112 * alp   
         dspp(113) = dspp(113) + bet113 * alp   
         dspp(114) = dspp(114) + bet114 * alp   
         dspp(115) = dspp(115) + bet115 * alp   
         dspp(116) = dspp(116) + bet116 * alp   
         dspp(117) = dspp(117) + bet117 * alp  
         
      enddo
#ifdef PL_OMP_ERI
!$omp end do
!$omp end parallel
#endif
      end
C** IKDSDS ===========================================================
C          CALUCATE KERNELS FOR DSDS TYPE
C====================================================================

      subroutine ikdsds(
     &           dsds ,
     &           fdat ,  edat  ,
     &           yp   ,  yq    ,
     &           uu   ,  uv    ,
     &           z    ,  zetpq , 
     &           nasp ,  mxsa  ,  mxprm,  ntprm,  
     &           iasp ,  jasp  ,  kasp ,  lasp ,
     &           ishel,  jshel ,  kshel,  lshel     )

      implicit double precision  (a-h,o-z)
      dimension fdat(31,2001)
      dimension edat(2001)
      dimension zetpq(mxprm,mxprm,mxsa,nasp,mxsa,nasp)
      dimension yp(3,mxprm,mxprm),yq(3,mxprm,mxprm)
      dimension uu(mxprm,mxprm),uv(mxprm,mxprm)
      dimension z(mxprm,mxsa,nasp)
      dimension ntprm(mxsa,nasp)
      dimension dsds(99)
 
      real(8),parameter::pi    = 3.14159265358979323846d+00
      real(8),parameter::tf(25) = (/
     &                33.0,37.0,41.0,43.0,46.0,49.0,
     &                51.0,54.0,56.0,58.0,61.0,63.0,
     &                66.0,68.0,70.0,72.0,74.0,80.0,
     &                85.0,90.0,95.0,95.0,98.0,99.0,
     &                99.0                           /)

      real(8),parameter::ga0 = 1.0d+00
      real(8),parameter::ga1 = 1.000 000 000 000 005 298d+00
      real(8),parameter::ga2 = 0.5d+00
      real(8),parameter::ga3 = 0.166 666 666 598 849 826d+00
      real(8),parameter::ga4 = 0.416 666 666 666 666 667d-01
      real(8),parameter::ga5 = 0.833 355 034 722 222 222d-02
      real(8),parameter::ga6 = 0.138 888 888 888 888 889d-02

      real(8),parameter::rminv(5) = (/
     &   1.0d+00,  0.333333333333333333d+00,  0.20d+00,
     &   0.14285714285714286d+00,  0.11111111111111111d+00 /)

#ifdef PL_SIMD_ERI
      dimension dmx(5,mxprm), val(2,mxprm)
#endif

      DO IJK=1,99
         DSDS(IJK)=0.0d0
      ENDDO
      
#ifdef PL_OMP_ERI
!$omp parallel reduction(+:dsds)
!$omp& default(private)
!$omp& shared(fdat,edat,
!$omp&        zetpq,ntprm,z,yp,yq,uu,uv,
!$omp&        ishel,jshel,kshel,lshel,iasp,jasp,kasp,lasp)
#endif
      igp = ntprm(ishel,iasp)
      jgp = ntprm(jshel,jasp)
      kgp = ntprm(kshel,kasp)
      lgp = ntprm(lshel,lasp)

      pi2 = 2.0d0*pi

#ifdef PL_OMP_ERI
!$omp do
#endif
      DO I = 1, IGP
         alp = z(i,ishel,iasp)
         bet06 = 0.0d0
         bet07 = 0.0d0
         bet08 = 0.0d0
         bet09 = 0.0d0
         bet10 = 0.0d0
         bet11 = 0.0d0
         bet12 = 0.0d0
         bet13 = 0.0d0
         bet14 = 0.0d0
         bet15 = 0.0d0
         bet27 = 0.0d0
         bet28 = 0.0d0
         bet29 = 0.0d0
         bet30 = 0.0d0
         bet31 = 0.0d0
         bet32 = 0.0d0
         bet33 = 0.0d0
         bet34 = 0.0d0
         bet35 = 0.0d0
         bet36 = 0.0d0
         bet37 = 0.0d0
         bet38 = 0.0d0
         bet39 = 0.0d0
         bet40 = 0.0d0
         bet41 = 0.0d0
         bet62 = 0.0d0
         bet63 = 0.0d0
         bet64 = 0.0d0
         bet65 = 0.0d0
         bet66 = 0.0d0
         bet67 = 0.0d0
         bet68 = 0.0d0
         bet69 = 0.0d0
         bet70 = 0.0d0
         bet71 = 0.0d0
         bet72 = 0.0d0
         bet73 = 0.0d0
         bet74 = 0.0d0
         bet75 = 0.0d0
         bet76 = 0.0d0
         bet77 = 0.0d0
         bet78 = 0.0d0
         bet92 = 0.0d0
         bet93 = 0.0d0
         bet94 = 0.0d0
         bet95 = 0.0d0
         
         DO J = 1, JGP
            bet = z(j,jshel,jasp)
            zet = zetpq(j,i,jshel,jasp,ishel,iasp)
            
            px  = yp (1,j,i)
            py  = yp (2,j,i)
            pz  = yp (3,j,i)
            up  = uu(j,i)
            del01 =0.0d0
            del16 =0.0d0
            del17 =0.0d0
            del45 =0.0d0
            del46 =0.0d0
            del47 =0.0d0
            del48 =0.0d0
            del82 =0.0d0
            del83 =0.0d0
            del85 =0.0d0
            del89 =0.0d0
            del98 =0.0d0
            del99 =0.0d0
            ram02 =0.0d0
            ram03 =0.0d0
            ram04 =0.0d0
            ram05 =0.0d0
            ram18 =0.0d0
            ram19 =0.0d0
            ram20 =0.0d0
            ram21 =0.0d0
            ram22 =0.0d0
            ram26 =0.0d0
            ram49 =0.0d0
            ram50 =0.0d0
            ram58 =0.0d0
            ram51 =0.0d0
            ram52 =0.0d0
            ram53 =0.0d0
            ram56 =0.0d0
            ram61 =0.0d0
            ram87 =0.0d0
            ram88 =0.0d0
            ram91 =0.0d0
            ram09 =0.0d0
            DO K = 1, KGP
               del02 =0.0d0 
               del18 =0.0d0 
               del19 =0.0d0 
               del49 =0.0d0 
               del50 =0.0d0 
               del51 =0.0d0 
               del43 =0.0d0 
               del53 =0.0d0 
               del58 =0.0d0 
               del87 =0.0d0 
               del88 =0.0d0 
               del03 =0.0d0
               del21 =0.0d0
               del20 =0.0d0
               del04 =0.0d0
               del22 =0.0d0
               del26 =0.0d0
               del05 =0.0d0
               del44 = 0.0d0 
               del81 = 0.0d0 
               
               ram = z(k,kshel,kasp)
               
               DO L = 1, LGP
                  del = z(l,lshel,lasp)
                  eta = zetpq(l,k,lshel,lasp,kshel,kasp)
                  qpx =  yq(1,l,k) - px
                  qpy =  yq(2,l,k) - py
                  qpz =  yq(3,l,k) - pz
                  vq  = uv(l,k)
                  
                  ROO2 = 1.0d0 / (ZET + ETA)
                  qp2  = qpx*qpx + qpy*qpy + qpz*qpz
                  T    = 0.5d0* ROO2*qp2
                  
                  IF (T. LE .TF(5)) THEN
                     
                     IT = IDINT((T+0.025D+00)*20.0D+00)
                     DT = 0.05D+00*IT - T
                     IT = IT + 1
                     DT4 = DT*0.25D+00
                     DT5 = DT*0.2D+00
                     DT6 = DT*0.166666666666666667D+00
                     DT2 = DT4 + DT4
                     DT3 = DT6 + DT6
                     F4   = (((((FDAT(11,IT)*DT6   + FDAT(10,IT))*DT5
     &                    + FDAT(9,IT))*DT4  + FDAT(8,IT))*DT3
     &                    + FDAT(7,IT))*DT2  + FDAT(6,IT))*DT
     &                    + FDAT(5,IT)
                     
                     EED  = (((((GA6*DT + GA5)*DT + GA4)*DT
     &                    + GA3)*DT + GA2)*DT + GA1)*DT + GA0
                     EE   = EDAT(IT)*EED
                     
                     T2   = T*2.0d0
                     F3   = (T2*F4 + EE)*RMINV(4)
                     F2   = (T2*F3 + EE)*RMINV(3)
                     F1   = (T2*F2 + EE)*RMINV(2)
                     F0   =  T2*F1 + EE
                     FPP =  UP * VQ * DSQRT(ROO2)
                     
                     DM0 = FPP*F0
                     DM1 = FPP*F1 *  ROO2
                     DM2 = FPP*F2 *( ROO2)**2
                     DM3 = FPP*F3 *( ROO2)**3
                     DM4 = FPP*F4 *( ROO2)**4
                     
                  ELSE 
                     
                     qinv = 1.0d0/qp2
                     dm0=  up*vq*dsqrt( pi2*qinv)*0.5d0
                     dm1 =        qinv *  dm0
                     dm2 =  3.0d0*qinv *  dm1
                     dm3 =  5.0d0*qinv *  dm2
                     dm4 =  7.0d0*qinv *  dm3
                  ENDIF

#ifdef PL_SIMD_ERI
                  val(1,l) = eta
                  val(2,l) = del

                  dmx(1,l) = dm0
                  dmx(2,l) = dm1
                  dmx(3,l) = dm2
                  dmx(4,l) = dm3
                  dmx(5,l) = dm4
               enddo
               
               do l = 1, lgp
                  eta = val(1,l)
                  del = val(2,l)

                  dm0 = dmx(1,l)
                  dm1 = dmx(2,l)
                  dm2 = dmx(3,l)
                  dm3 = dmx(4,l)
                  dm4 = dmx(5,l)
#endif

C** KERNEL DSDS =====================================================
C           CALCULATE & CONTRACT KERNELS FOR DSDS TYPE
C====================================================================
C  1:F002002 21:T002214 41:T315002 61:D022204  81:U001113 
C  2:F002103 22:T002315 42:D001002 62:D102002  82:U002001 
C  3:F002204 23:T012002 43:D001103 63:D102103  83:U002022 
C  4:F002305 24:T012103 44:D001204 64:D103001  84:U012001 
C  5:F002406 25:T012204 45:D002001 65:D103012  85:U012012 
C  6:F103002 26:T012305 46:D002002 66:D103022  86:U012022 
C  7:F103103 27:T103002 47:D002012 67:D103102  87:U012102 
C  8:F103204 28:T103012 48:D002022 68:D103123  88:U012123 
C  9:F103305 29:T103103 49:D002102 69:D113002  89:U022002 
C 10:F204002 30:T103113 50:D002113 70:D113012  90:U022012 
C 11:F204103 31:T103214 51:D002123 71:D113113  91:U022113  
C 12:F204204 32:T113002 52:D002203 72:D123002  92:U102012  
C 13:F305002 33:T113103 53:D002224 73:D123103  93:U113001  
C 14:F305103 34:T113204 54:D012002 74:D203002  94:U113022  
C 15:F406002 35:T204002 55:D012012 75:D204001  95:U123012  
C 16:T002002 36:T204012 56:D012103 76:D204022  96:Z001001  
C 17:T002012 37:T204113 57:D012113 77:D214012  97:Z001022 
C 18:T002103 38:T214002 58:D012214 78:D224002  98:Z022001 
C 19:T002113 39:T214103 59:D022002 79:U001002  99:Z022022 
C 20:T002204 40:T305012 60:D022103 80:U001012 
C
                  del01=del01+dm4       *eta   
                  del02=del02+dm4       *eta**2
                  del03=del03+dm4       *eta**3
                  del04=del04+dm4       *eta**4
                  del05=del05+dm4       *eta**5
! ====================================
                  del16=del16+dm3       *eta   
                  del18=del18+dm3       *eta**2
                  del20=del20+dm3       *eta**3
                  del26=del26+dm3       *eta**4
                  del17=del17+dm3*del   *eta   
                  del19=del19+dm3*del   *eta**2
                  del21=del21+dm3*del   *eta**3
                  del22=del22+dm3*del   *eta**4
! ====================================

                  del45=del45+dm2              
                  del46=del46+dm2       *eta   
                  del49=del49+dm2       *eta   
                  del43=del43+dm2       *eta**2
                  del44=del44+dm2       *eta**3
                  del47=del47+dm2*del   *eta   
                  del50=del50+dm2*del   *eta**2
                  del58=del58+dm2*del   *eta**3
                  del48=del48+dm2*del**2*eta   
                  del51=del51+dm2*del**2*eta**2
                  del53=del53+dm2*del**2*eta**3
! ====================================

                  del82=del82+dm1              
                  del87=del87+dm1       *eta   
                  del89=del89+dm1       *eta
                  del85=del85+dm1*del   *eta   
                  del83=del83+dm1*del**2*eta   
                  
                  del81=del81+dm1*del   *eta**2
                  
                  del88=del88+dm1*del**2*eta**2
                  
                  del98=del98+dm0              
                  del99=del99+dm0*del**2*eta   
                  
! =================================================================
                  dsds(42)=dsds(42)+dm2              *eta
                  dsds(79)=dsds(79)+dm1              *eta   
                  dsds(80)=dsds(80)+dm1       *del   *eta   
                  dsds(96)=dsds(96)+dm0                 
                  dsds(97)=dsds(97)+dm0       *del**2*eta   
!     =================================================================
               enddo
               ram02 = ram02  + del02 *ram   
               ram18 = ram18  + del18 *ram   
               ram19 = ram19  + del19 *ram   
               ram49 = ram49  + del49 *ram   
               ram50 = ram50  + del50 *ram   
               ram51 = ram51  + del51 *ram   
               ram56 = ram56  + del43 *ram   
               ram87 = ram87  + del87 *ram   
               ram88 = ram88  + del88 *ram   
               ram91 = ram91  + del81 *ram   
               ram03 = ram03  + del03 *ram**2
               ram21 = ram21  + del21 *ram**2
               ram20 = ram20  + del20 *ram**2
               ram61 = ram61  + del44 *ram**2
               ram52 = ram52  + del43 *ram**2
               ram53 = ram53  + del53 *ram**2
               ram58 = ram58  + del58 *ram**2
               ram04 = ram04  + del04 *ram**3
               ram09 = ram09  + del04 *ram**3
               ram22 = ram22  + del22 *ram**3
               ram26 = ram26  + del26 *ram**3
               ram05 = ram05  + del05 *ram**4
               
               dsds(43)=dsds(43)+del43               *ram   
               dsds(44)=dsds(44)+del44               *ram**2
               dsds(81)=dsds(81)+del81               *ram   
            enddo
            dsds( 1)=dsds( 1)+del01            *zet
            dsds(16)=dsds(16)+del16            *zet
            dsds(17)=dsds(17)+del17            *zet
            dsds(23)=dsds(23)+del16     *bet   *zet
            dsds(45)=dsds(45)+del45            *zet
            dsds(46)=dsds(46)+del46            *zet
            dsds(47)=dsds(47)+del47            *zet
            dsds(48)=dsds(48)+del48            *zet
            dsds(54)=dsds(54)+del46     *bet   *zet
            dsds(55)=dsds(55)+del47     *bet   *zet
            dsds(59)=dsds(59)+del46     *bet**2*zet
            dsds(82)=dsds(82)+del82            *zet
            dsds(83)=dsds(83)+del83            *zet
            dsds(84)=dsds(84)+del82     *bet   *zet
            dsds(85)=dsds(85)+del85     *bet   *zet
            dsds(86)=dsds(86)+del83     *bet   *zet
            dsds(89)=dsds(89)+del89     *bet**2*zet
            dsds(90)=dsds(90)+del85     *bet**2*zet
            dsds(98)=dsds(98)+del98     *bet**2*zet
            dsds(99)=dsds(99)+del99     *bet**2*zet
            
            dsds( 2)=dsds( 2)+ram02            *zet 
            dsds( 3)=dsds( 3)+ram03            *zet 
            dsds( 4)=dsds( 4)+ram04            *zet 
            dsds( 5)=dsds( 5)+ram05            *zet 
            dsds(18)=dsds(18)+ram18            *zet 
            dsds(19)=dsds(19)+ram19            *zet 
            dsds(20)=dsds(20)+ram20            *zet 
            dsds(21)=dsds(21)+ram21            *zet 
            dsds(22)=dsds(22)+ram22            *zet 
            dsds(24)=dsds(24)+ram18     *bet   *zet 
            dsds(25)=dsds(25)+ram20     *bet   *zet 
            dsds(26)=dsds(26)+ram26     *bet   *zet 
            dsds(49)=dsds(49)+ram49            *zet 
            dsds(50)=dsds(50)+ram50            *zet 
            dsds(51)=dsds(51)+ram51            *zet 
            dsds(52)=dsds(52)+ram52            *zet 
            dsds(53)=dsds(53)+ram53            *zet 
            dsds(56)=dsds(56)+ram56     *bet   *zet 
            dsds(57)=dsds(57)+ram50     *bet   *zet 
            dsds(58)=dsds(58)+ram58     *bet   *zet 
            dsds(60)=dsds(60)+ram56     *bet**2*zet 
            dsds(61)=dsds(61)+ram61     *bet**2*zet 
            dsds(87)=dsds(87)+ram87     *bet   *zet 
            dsds(88)=dsds(88)+ram88     *bet   *zet 
            dsds(91)=dsds(91)+ram91     *bet**2*zet 
            
            bet07 = bet07 +ram02       *zet**2 
            bet08 = bet08 +ram03       *zet**2 
            bet09 = bet09 +ram04       *zet**2 
            bet11 = bet11 +ram02       *zet**3 
            bet12 = bet12 +ram03       *zet**3 
            bet14 = bet14 +ram02       *zet**4 
            bet29 = bet29 +ram18       *zet**2 
            bet30 = bet30 +ram19       *zet**2 
            bet31 = bet31 +ram21       *zet**2 
            bet33 = bet33 +ram18*bet   *zet**2 
            bet34 = bet34 +ram20*bet   *zet**2 
            bet37 = bet37 +ram19       *zet**3 
            bet39 = bet39 +ram18*bet   *zet**3 
            bet63 = bet63 +ram56       *zet    
            bet67 = bet67 +ram49       *zet**2 
            bet68 = bet68 +ram51       *zet**2 
            bet71 = bet71 +ram50*bet   *zet**2 
            bet73 = bet73 +ram56*bet**2*zet**2 
            
            bet06 =bet06 + del01         *zet**2
            bet10 =bet10 + del01         *zet**3
            bet13 =bet13 + del01         *zet**4
            bet15 =bet15 + del01         *zet**5
            bet27 =bet27 + del16         *zet**2
            bet28 =bet28 + del17         *zet**2
            bet32 =bet32 + del16  *bet   *zet**2
            bet35 =bet35 + del16         *zet**3
            bet36 =bet36 + del17         *zet**3
            bet38 =bet38 + del16  *bet   *zet**3
            bet40 =bet40 + del17         *zet**4
            bet41 =bet41 + del16  *bet   *zet**4
            bet62 =bet62 + del46         *zet   
            bet64 =bet64 + del45         *zet**2
            bet65 =bet65 + del47         *zet**2
            bet66 =bet66 + del48         *zet**2
            bet69 =bet69 + del46  *bet   *zet**2
            bet70 =bet70 + del47  *bet   *zet**2
            bet72 =bet72 + del46  *bet**2*zet**2
            bet74 =bet74 + del46         *zet**2
            bet75 =bet75 + del45         *zet**3
            bet76 =bet76 + del48         *zet**3
            bet77 =bet77 + del47  *bet   *zet**3
            bet78 =bet78 + del46  *bet**2*zet**3
            bet92 =bet92 + del85         *zet   
            bet93 =bet93 + del82  *bet   *zet**2
            bet94 =bet94 + del83  *bet   *zet**2
            bet95 =bet95 + del85  *bet**2*zet**2
      enddo
      dsds( 6) = dsds( 6) + bet06 * alp   
      dsds( 7) = dsds( 7) + bet07 * alp   
      dsds( 8) = dsds( 8) + bet08 * alp   
      dsds( 9) = dsds( 9) + bet09 * alp   
      dsds(10) = dsds(10) + bet10 * alp**2
      dsds(11) = dsds(11) + bet11 * alp**2
      dsds(12) = dsds(12) + bet12 * alp**2
      dsds(13) = dsds(13) + bet13 * alp**3
      dsds(14) = dsds(14) + bet14 * alp**3
      dsds(15) = dsds(15) + bet15 * alp**4
      dsds(27) = dsds(27) + bet27 * alp   
      dsds(28) = dsds(28) + bet28 * alp   
      dsds(29) = dsds(29) + bet29 * alp   
      dsds(30) = dsds(30) + bet30 * alp   
      dsds(31) = dsds(31) + bet31 * alp   
      dsds(32) = dsds(32) + bet32 * alp   
      dsds(33) = dsds(33) + bet33 * alp   
      dsds(34) = dsds(34) + bet34 * alp   
      dsds(35) = dsds(35) + bet35 * alp**2
      dsds(36) = dsds(36) + bet36 * alp**2
      dsds(37) = dsds(37) + bet37 * alp**2
      dsds(38) = dsds(38) + bet38 * alp**2
      dsds(39) = dsds(39) + bet39 * alp**2
      dsds(40) = dsds(40) + bet40 * alp**3
      dsds(41) = dsds(41) + bet41 * alp**3
      dsds(62) = dsds(62) + bet62 * alp   
      dsds(63) = dsds(63) + bet63 * alp   
      dsds(64) = dsds(64) + bet64 * alp   
      dsds(65) = dsds(65) + bet65 * alp   
      dsds(66) = dsds(66) + bet66 * alp   
      dsds(67) = dsds(67) + bet67 * alp   
      dsds(68) = dsds(68) + bet68 * alp   
      dsds(69) = dsds(69) + bet69 * alp   
      dsds(70) = dsds(70) + bet70 * alp   
      dsds(71) = dsds(71) + bet71 * alp   
      dsds(72) = dsds(72) + bet72 * alp   
      dsds(73) = dsds(73) + bet73 * alp   
      dsds(74) = dsds(74) + bet74 * alp**2
      dsds(75) = dsds(75) + bet75 * alp**2
      dsds(76) = dsds(76) + bet76 * alp**2
      dsds(77) = dsds(77) + bet77 * alp**2
      dsds(78) = dsds(78) + bet78 * alp**2
      dsds(92) = dsds(92) + bet92 * alp   
      dsds(93) = dsds(93) + bet93 * alp   
      dsds(94) = dsds(94) + bet94 * alp   
      dsds(95) = dsds(95) + bet95 * alp   
      enddo
#ifdef PL_OMP_ERI
!$omp end do
!$omp end parallel
#endif

      end
C** IKDPPS ===========================================================
C          CALUCATE KERNELS FOR DPPS TYPE
C====================================================================

      subroutine ikdpps(
     &           dpps ,
     &           fdat ,  edat  ,
     &           yp   ,  yq    ,
     &           uu   ,  uv    ,
     &           z    ,  zetpq , 
     &           nasp ,  mxsa  ,  mxprm,  ntprm,  
     &           iasp ,  jasp  ,  kasp ,  lasp ,
     &           ishel,  jshel ,  kshel,  lshel     )


      implicit double precision  (a-h,o-z)
      dimension fdat(31,2001)
      dimension edat(2001)
      dimension zetpq(mxprm,mxprm,mxsa,nasp,mxsa,nasp)
      dimension yp(3,mxprm,mxprm),yq(3,mxprm,mxprm)
      dimension uu(mxprm,mxprm),uv(mxprm,mxprm)
      dimension z(mxprm,mxsa,nasp)
      dimension ntprm(mxsa,nasp)
      dimension dpps(112)
 
      real(8),parameter::pi    = 3.14159265358979323846d+00
      real(8),parameter::tf(25) = (/
     &                33.0,37.0,41.0,43.0,46.0,49.0,
     &                51.0,54.0,56.0,58.0,61.0,63.0,
     &                66.0,68.0,70.0,72.0,74.0,80.0,
     &                85.0,90.0,95.0,95.0,98.0,99.0,
     &                99.0                           /)

      real(8),parameter::ga0 = 1.0d+00
      real(8),parameter::ga1 = 1.000 000 000 000 005 298d+00
      real(8),parameter::ga2 = 0.5d+00
      real(8),parameter::ga3 = 0.166 666 666 598 849 826d+00
      real(8),parameter::ga4 = 0.416 666 666 666 666 667d-01
      real(8),parameter::ga5 = 0.833 355 034 722 222 222d-02
      real(8),parameter::ga6 = 0.138 888 888 888 888 889d-02

      real(8),parameter::rminv(5) = (/
     &   1.0d+00,  0.333333333333333333d+00,  0.20d+00,
     &   0.14285714285714286d+00,  0.11111111111111111d+00 /)

#ifdef PL_SIMD_ERI
      dimension dmx(5,mxprm), val(2,mxprm)
#endif

      DO IJK=1,112
        DPPS(IJK)=0.0d0
      ENDDO

#ifdef PL_OMP_ERI
!$omp parallel reduction(+:dpps)
!$omp& default(private)
!$omp& shared(fdat,edat,
!$omp&        zetpq,ntprm,z,yp,yq,uu,uv,
!$omp&        ishel,jshel,kshel,lshel,iasp,jasp,kasp,lasp)
#endif
      igp = ntprm(ishel,iasp)
      jgp = ntprm(jshel,jasp)
      kgp = ntprm(kshel,kasp)
      lgp = ntprm(lshel,lasp)

      pi2 = 2.0d0*pi

#ifdef PL_OMP_ERI
!$omp do
#endif
      DO I = 1, IGP
         alp = z(i,ishel,iasp)
         bet015 = 0.0d0
         bet016 = 0.0d0
         bet017 = 0.0d0
         bet018 = 0.0d0
         bet019 = 0.0d0
         bet020 = 0.0d0
         bet021 = 0.0d0
         bet022 = 0.0d0
         bet023 = 0.0d0
         bet024 = 0.0d0
         bet025 = 0.0d0
         bet026 = 0.0d0
         bet027 = 0.0d0
         bet028 = 0.0d0
         bet029 = 0.0d0
         bet031 = 0.0d0
         bet033 = 0.0d0
         bet034 = 0.0d0
         bet035 = 0.0d0
         bet036 = 0.0d0
         bet037 = 0.0d0
         bet043 = 0.0d0
         bet044 = 0.0d0
         bet045 = 0.0d0
         bet046 = 0.0d0
         bet047 = 0.0d0
         bet048 = 0.0d0
         bet049 = 0.0d0
         bet050 = 0.0d0
         bet051 = 0.0d0
         bet052 = 0.0d0
         bet064 = 0.0d0
         bet065 = 0.0d0
         bet066 = 0.0d0
         bet067 = 0.0d0
         bet068 = 0.0d0
         bet069 = 0.0d0
         bet070 = 0.0d0
         bet071 = 0.0d0
         bet072 = 0.0d0
         bet073 = 0.0d0
         bet074 = 0.0d0
         bet075 = 0.0d0
         bet078 = 0.0d0
         bet079 = 0.0d0
         bet080 = 0.0d0
         bet081 = 0.0d0
         bet082 = 0.0d0
         bet083 = 0.0d0
         bet085 = 0.0d0
         bet086 = 0.0d0
         bet087 = 0.0d0
         bet088 = 0.0d0
         bet098 = 0.0d0
         bet099 = 0.0d0
         bet100 = 0.0d0
         bet101 = 0.0d0
         bet102 = 0.0d0
         bet103 = 0.0d0
         bet104 = 0.0d0
         bet105 = 0.0d0
         bet106 = 0.0d0
         bet107 = 0.0d0
         bet108 = 0.0d0
         bet109 = 0.0d0
         bet111 = 0.0d0
         bet112 = 0.0d0

         DO J = 1, JGP
            bet = z(j,jshel,jasp)
            zet = zetpq(j,i,jshel,jasp,ishel,iasp)
            px  = yp (1,j,i)
            py  = yp (2,j,i)
            pz  = yp (3,j,i)
            up  = uu(j,i)
            del001 = 0.0d0
            del005 = 0.0d0
            del038 = 0.0d0
            del053 = 0.0d0
            del054 = 0.0d0
            del089 = 0.0d0
            del090 = 0.0d0
            del110 = 0.0d0
            ram002 = 0.0d0
            ram003 = 0.0d0
            ram006 = 0.0d0
            ram011 = 0.0d0
            ram039 = 0.0d0
            ram040 = 0.0d0
            ram041 = 0.0d0
            ram042 = 0.0d0
            ram055 = 0.0d0
            ram056 = 0.0d0
            ram057 = 0.0d0
            ram058 = 0.0d0
            ram059 = 0.0d0
            ram063 = 0.0d0
            ram091 = 0.0d0
            ram093 = 0.0d0
            DO K = 1, KGP
               ram = z(k,kshel,kasp)
               del002 = 0.0d0 
               del006 = 0.0d0 
               del039 = 0.0d0 
               del055 = 0.0d0 
               del056 = 0.0d0 
               del091 = 0.0d0 
               del093 = 0.0d0 
               del003 = 0.0d0
               del011 = 0.0d0
               del040 = 0.0d0
               del057 = 0.0d0
               del058 = 0.0d0
               del041 = 0.0d0
               del059 = 0.0d0
               del063 = 0.0d0
               del042 = 0.0d0

               DO L = 1, LGP
                  del = z(l,lshel,lasp)
                  eta = zetpq(l,k,lshel,lasp,kshel,kasp)
                  qpx =  yq(1,l,k) - px
                  qpy =  yq(2,l,k) - py
                  qpz =  yq(3,l,k) - pz

                  vq  = uv(l,k)
      
                 ROO2 = 1.0d0 / (ZET + ETA)
                  qp2  = qpx*qpx + qpy*qpy + qpz*qpz
                  T    = 0.5d0* ROO2*qp2

                 IF (T. LE .TF(5)) THEN
                  
                    IT = IDINT((T+0.025D+00)*20.0D+00)
                    DT = 0.05D+00*IT - T
                    IT = IT + 1
                    DT4 = DT*0.25D+00
                    DT5 = DT*0.2D+00
                    DT6 = DT*0.166666666666666667D+00
                    DT2 = DT4 + DT4
                    DT3 = DT6 + DT6
                    F4   = (((((FDAT(11,IT)*DT6   + FDAT(10,IT))*DT5
     &                + FDAT(9,IT))*DT4  + FDAT(8,IT))*DT3
     &                + FDAT(7,IT))*DT2  + FDAT(6,IT))*DT
     &                + FDAT(5,IT)
         
                    EED  = (((((GA6*DT + GA5)*DT + GA4)*DT
     &                + GA3)*DT + GA2)*DT + GA1)*DT + GA0
                    EE   = EDAT(IT)*EED
 
                    T2   = T*2.0d0
                    F3   = (T2*F4 + EE)*RMINV(4)
                    F2   = (T2*F3 + EE)*RMINV(3)
                    F1   = (T2*F2 + EE)*RMINV(2)
                    F0   =  T2*F1 + EE
                    FPP =  UP * VQ * DSQRT(ROO2)
                
                    DM0 = FPP*F0
                    DM1 = FPP*F1 *  ROO2
                    DM2 = FPP*F2 *( ROO2)**2
                    DM3 = FPP*F3 *( ROO2)**3
                    DM4 = FPP*F4 *( ROO2)**4

                 ELSE 

                    qinv = 1.0d0/qp2
                    dm0=  up*vq*dsqrt( pi2*qinv)*0.5d0
                    dm1 =        qinv *  dm0
                    dm2 =  3.0d0*qinv *  dm1
                    dm3 =  5.0d0*qinv *  dm2
                    dm4 =  7.0d0*qinv *  dm3
                 ENDIF

#ifdef PL_SIMD_ERI
                  val(1,l) = eta
                  val(2,l) = del

                  dmx(1,l) = dm0
                  dmx(2,l) = dm1
                  dmx(3,l) = dm2
                  dmx(4,l) = dm3
                  dmx(5,l) = dm4
               enddo
               
               do l = 1, lgp
                  eta = val(1,l)
                  del = val(2,l)

                  dm0 = dmx(1,l)
                  dm1 = dmx(2,l)
                  dm2 = dmx(3,l)
                  dm3 = dmx(4,l)
                  dm4 = dmx(5,l)
#endif

!** KERNEL DPPS =====================================================
!           CALCULATE & CONTRACT KERNELS FOR DPPS TYPE
!====================================================================
            
                 del001 = del001 + dm2                 
                 del002 = del002 + dm2          * eta   
                 del003 = del003 + dm2          * eta**2
                 del005 = del005 + dm2 * del           
                 del006 = del006 + dm2 * del    * eta   
                 del011 = del011 + dm2 * del    * eta**2
!================================================
                 del038 = del038 + dm4                 
                 del039 = del039 + dm4          * eta   
                 del040 = del040 + dm4          * eta**2
                 del041 = del041 + dm4          * eta**3
                 del042 = del042 + dm4          * eta**4
!================================================
                 del053 = del053 + dm3                 
                 del055 = del055 + dm3          * eta   
                 del057 = del057 + dm3          * eta**2
                 del063 = del063 + dm3          * eta**3
                 del054 = del054 + dm3 * del           
                 del056 = del056 + dm3 * del    * eta   
                 del058 = del058 + dm3 * del    * eta**2
                 del059 = del059 + dm3 * del    * eta**3
!================================================
                 del089 = del089 + dm1                 
                 del093 = del093 + dm1          * eta   
                 del090 = del090 + dm1 * del           
                 del091 = del091 + dm1 * del    * eta   
!================================================
                 del110 = del110 + dm0 * del           
                 
              enddo
              ram002 = ram002 + del002 * ram   
              ram006 = ram006 + del006 * ram   
              ram039 = ram039 + del039 * ram   
              ram055 = ram055 + del055 * ram   
              ram056 = ram056 + del056 * ram   
              ram091 = ram091 + del091 * ram   
              ram093 = ram093 + del093 * ram   
              ram003 = ram003 + del003 * ram**2
              ram011 = ram011 + del011 * ram**2
              ram040 = ram040 + del040 * ram**2
              ram057 = ram057 + del057 * ram**2
              ram058 = ram058 + del058 * ram**2
              ram041 = ram041 + del041 * ram**3
              ram059 = ram059 + del059 * ram**3
              ram063 = ram063 + del063 * ram**3
              ram042 = ram042 + del042 * ram**4
              
           enddo
           dpps(  2) = dpps(  2) + ram002               * zet   
           dpps(  3) = dpps(  3) + ram003               * zet   
           dpps(  6) = dpps(  6) + ram006               * zet**2
           dpps(  9) = dpps(  9) + ram002      * bet    * zet**2
           dpps( 10) = dpps( 10) + ram006      * bet    * zet**2
           dpps( 11) = dpps( 11) + ram011      * bet    * zet**2
           dpps( 13) = dpps( 13) + ram002      * bet**2 * zet**2
           dpps( 14) = dpps( 14) + ram003      * bet**2 * zet**2
           dpps( 39) = dpps( 39) + ram039               * zet**2
           dpps( 40) = dpps( 40) + ram040               * zet**2
           dpps( 41) = dpps( 41) + ram041               * zet**2
           dpps( 42) = dpps( 42) + ram042               * zet**2
           dpps( 55) = dpps( 55) + ram055               * zet**2
           dpps( 56) = dpps( 56) + ram056               * zet**2
           dpps( 57) = dpps( 57) + ram057               * zet**2
           dpps( 58) = dpps( 58) + ram058               * zet**2
           dpps( 59) = dpps( 59) + ram059               * zet**2
           dpps( 61) = dpps( 61) + ram055      * bet    * zet**2
           dpps( 62) = dpps( 62) + ram057      * bet    * zet**2
           dpps( 63) = dpps( 63) + ram063      * bet    * zet**2
           dpps( 91) = dpps( 91) + ram091               * zet   
           dpps( 93) = dpps( 93) + ram093      * bet    * zet   
           dpps( 97) = dpps( 97) + ram091      * bet**2 * zet**2
           
           bet017 = bet017 + ram002         * zet**2
           bet022 = bet022 + ram002 *bet    * zet**2
           bet028 = bet028 + ram002 *bet**2 * zet**3
           bet033 = bet033 + ram002 *bet    * zet**3
           bet018 = bet018 + ram006         * zet**2
           bet026 = bet026 + ram006 *bet    * zet**3
           bet031 = bet031 + ram006         * zet**3
           bet019 = bet019 + ram011         * zet**2
           bet023 = bet023 + ram003 *bet    * zet**2
           bet044 = bet044 + ram039         * zet**3
           bet048 = bet048 + ram039         * zet**4
           bet051 = bet051 + ram039         * zet**5
           bet045 = bet045 + ram040         * zet**3
           bet049 = bet049 + ram040         * zet**4
           bet046 = bet046 + ram041         * zet**3
           bet065 = bet065 + ram055         * zet**2
           bet070 = bet070 + ram055         * zet**3
           bet074 = bet074 + ram055 *bet    * zet**3
           bet083 = bet083 + ram055 *bet    * zet**4
           bet085 = bet085 + ram055         * zet**4
           bet066 = bet066 + ram057         * zet**2
           bet075 = bet075 + ram057 *bet    * zet**3
           bet078 = bet078 + ram057         * zet**3
           bet067 = bet067 + ram063         * zet**2
           bet071 = bet071 + ram056         * zet**3
           bet081 = bet081 + ram056         * zet**4
           bet072 = bet072 + ram058         * zet**3
           bet099 = bet099 + ram093         * zet   
           bet103 = bet103 + ram091 *bet    * zet**2
           bet105 = bet105 + ram093 *bet**2 * zet**2
           
           dpps(  1)= dpps(  1)+del001          *zet                       
           dpps(  4)= dpps(  4)+del001          *zet**2                    
           dpps(  5)= dpps(  5)+del005          *zet**2
           dpps(  7)= dpps(  7)+del001   *bet   *zet**2
           dpps(  8)= dpps(  8)+del005   *bet   *zet**2
           dpps( 12)= dpps( 12)+del001   *bet**2*zet**2
           dpps( 38)= dpps( 38)+del038          *zet**2
           dpps( 53)= dpps( 53)+del053          *zet**2
           dpps( 54)= dpps( 54)+del054          *zet**2
           dpps( 60)= dpps( 60)+del053   *bet   *zet**2
           dpps( 89)= dpps( 89)+del089          *zet   
           dpps( 90)= dpps( 90)+del090          *zet   
           dpps( 92)= dpps( 92)+del089   *bet   *zet   
           dpps( 94)= dpps( 94)+del090   *bet   *zet**2
           dpps( 95)= dpps( 95)+del089   *bet**2*zet**2
           dpps( 96)= dpps( 96)+del090   *bet**2*zet**2
           dpps(110)= dpps(110)+del110   *bet   *zet   
           
           bet015 = bet015 + del001          * zet**2
           bet021 = bet021 + del001 * bet    * zet**2
           bet024 = bet024 + del001 * bet    * zet**3
           bet027 = bet027 + del001 * bet**2 * zet**3
           bet029 = bet029 + del001          * zet**3
           bet035 = bet035 + del001 * bet**2 * zet**4
           bet037 = bet037 + del001 * bet    * zet**4
           bet016 = bet016 + del005          * zet**2
           bet020 = bet020 + del005          * zet**3
           bet025 = bet025 + del005 * bet    * zet**3
           bet034 = bet034 + del005 * bet    * zet**4
           bet036 = bet036 + del005          * zet**4
           bet043 = bet043 + del038          * zet**3
           bet047 = bet047 + del038          * zet**4
           bet050 = bet050 + del038          * zet**5
           bet052 = bet052 + del038          * zet**6
           bet064 = bet064 + del053          * zet**2
           bet068 = bet068 + del053          * zet**3
           bet073 = bet073 + del053 * bet    * zet**3
           bet079 = bet079 + del053          * zet**4
           bet082 = bet082 + del053 * bet    * zet**4
           bet087 = bet087 + del053 * bet    * zet**5
           bet088 = bet088 + del053          * zet**5
           bet069 = bet069 + del054          * zet**3
           bet080 = bet080 + del054          * zet**4
           bet086 = bet086 + del054          * zet**5
           bet098 = bet098 + del089          * zet   
           bet101 = bet101 + del089 * bet    * zet**2
           bet104 = bet104 + del089 * bet**2 * zet**2
           bet107 = bet107 + del089          * zet**2
           bet109 = bet109 + del089 * bet**2 * zet**3
           bet100 = bet100 + del090          * zet**2
           bet102 = bet102 + del090 * bet    * zet**2
           bet106 = bet106 + del090 * bet**2 * zet**3
           bet108 = bet108 + del090 * bet    * zet**3
           bet111 = bet111 + del110          * zet   
           bet112 = bet112 + del110 * bet**2 * zet**2
        enddo
        dpps( 15) = dpps( 15) + bet015 * alp   
        dpps( 16) = dpps( 16) + bet016 * alp   
        dpps( 17) = dpps( 17) + bet017 * alp   
        dpps( 18) = dpps( 18) + bet018 * alp   
        dpps( 19) = dpps( 19) + bet019 * alp   
        dpps( 20) = dpps( 20) + bet020 * alp   
        dpps( 21) = dpps( 21) + bet021 * alp   
        dpps( 22) = dpps( 22) + bet022 * alp   
        dpps( 23) = dpps( 23) + bet023 * alp   
        dpps( 24) = dpps( 24) + bet024 * alp   
        dpps( 25) = dpps( 25) + bet025 * alp   
        dpps( 26) = dpps( 26) + bet026 * alp   
        dpps( 27) = dpps( 27) + bet027 * alp   
        dpps( 28) = dpps( 28) + bet028 * alp   
        dpps( 29) = dpps( 29) + bet029 * alp**2
        dpps( 30) = dpps( 30) + bet020 * alp**2
        dpps( 31) = dpps( 31) + bet031 * alp**2
        dpps( 32) = dpps( 32) + bet024 * alp**2
        dpps( 33) = dpps( 33) + bet033 * alp**2
        dpps( 34) = dpps( 34) + bet034 * alp**2
        dpps( 35) = dpps( 35) + bet035 * alp**2
        dpps( 36) = dpps( 36) + bet036 * alp**3
        dpps( 37) = dpps( 37) + bet037 * alp**3
        dpps( 43) = dpps( 43) + bet043 * alp   
        dpps( 44) = dpps( 44) + bet044 * alp   
        dpps( 45) = dpps( 45) + bet045 * alp   
        dpps( 46) = dpps( 46) + bet046 * alp   
        dpps( 47) = dpps( 47) + bet047 * alp**2
        dpps( 48) = dpps( 48) + bet048 * alp**2
        dpps( 49) = dpps( 49) + bet049 * alp**2
        dpps( 50) = dpps( 50) + bet050 * alp**3
        dpps( 51) = dpps( 51) + bet051 * alp**3
        dpps( 52) = dpps( 52) + bet052 * alp**4
        dpps( 64) = dpps( 64) + bet064 * alp   
        dpps( 65) = dpps( 65) + bet065 * alp   
        dpps( 66) = dpps( 66) + bet066 * alp   
        dpps( 67) = dpps( 67) + bet067 * alp   
        dpps( 68) = dpps( 68) + bet068 * alp   
        dpps( 69) = dpps( 69) + bet069 * alp   
        dpps( 70) = dpps( 70) + bet070 * alp   
        dpps( 71) = dpps( 71) + bet071 * alp   
        dpps( 72) = dpps( 72) + bet072 * alp   
        dpps( 73) = dpps( 73) + bet073 * alp   
        dpps( 74) = dpps( 74) + bet074 * alp   
        dpps( 75) = dpps( 75) + bet075 * alp   
        dpps( 76) = dpps( 76) + bet068 * alp**2
        dpps( 77) = dpps( 77) + bet070 * alp**2
        dpps( 78) = dpps( 78) + bet078 * alp**2
        dpps( 79) = dpps( 79) + bet079 * alp**2
        dpps( 80) = dpps( 80) + bet080 * alp**2
        dpps( 81) = dpps( 81) + bet081 * alp**2
        dpps( 82) = dpps( 82) + bet082 * alp**2
        dpps( 83) = dpps( 83) + bet083 * alp**2
        dpps( 84) = dpps( 84) + bet079 * alp**3
        dpps( 85) = dpps( 85) + bet085 * alp**3
        dpps( 86) = dpps( 86) + bet086 * alp**3
        dpps( 87) = dpps( 87) + bet087 * alp**3
        dpps( 88) = dpps( 88) + bet088 * alp**4
        dpps( 98) = dpps( 98) + bet098 * alp   
        dpps( 99) = dpps( 99) + bet099 * alp   
        dpps(100) = dpps(100) + bet100 * alp   
        dpps(101) = dpps(101) + bet101 * alp   
        dpps(102) = dpps(102) + bet102 * alp   
        dpps(103) = dpps(103) + bet103 * alp   
        dpps(104) = dpps(104) + bet104 * alp   
        dpps(105) = dpps(105) + bet105 * alp   
        dpps(106) = dpps(106) + bet106 * alp   
        dpps(107) = dpps(107) + bet107 * alp**2
        dpps(108) = dpps(108) + bet108 * alp**2
        dpps(109) = dpps(109) + bet109 * alp**2
        dpps(111) = dpps(111) + bet111 * alp   
        dpps(112) = dpps(112) + bet112 * alp   
      enddo
#ifdef PL_OMP_ERI
!$omp end do
!$omp end parallel
#endif
      
      end
C** IKDDSS ===========================================================
C          CALUCATE KERNELS FOR DDSS TYPE
C====================================================================

      subroutine ikddss(
     &           ddss ,
     &           fdat ,  edat  ,
     &           yp   ,  yq    ,
     &           uu   ,  uv    ,
     &           z    ,  zetpq , 
     &           nasp ,  mxsa  ,  mxprm,  ntprm,  
     &           iasp ,  jasp  ,  kasp ,  lasp ,
     &           ishel,  jshel ,  kshel,  lshel     )

      implicit double precision  (a-h,o-z)
      dimension fdat(31,2001)
      dimension edat(2001)
      dimension zetpq(mxprm,mxprm,mxsa,nasp,mxsa,nasp)
      dimension yp(3,mxprm,mxprm),yq(3,mxprm,mxprm)
      dimension uu(mxprm,mxprm),uv(mxprm,mxprm)
      dimension z(mxprm,mxsa,nasp)
      dimension ntprm(mxsa,nasp)
      dimension ddss(88)

      real(8),parameter::pi    = 3.14159265358979323846d+00
      real(8),parameter::tf(25) = (/
     &                33.0,37.0,41.0,43.0,46.0,49.0,
     &                51.0,54.0,56.0,58.0,61.0,63.0,
     &                66.0,68.0,70.0,72.0,74.0,80.0,
     &                85.0,90.0,95.0,95.0,98.0,99.0,
     &                99.0                           /)

      real(8),parameter::ga0 = 1.0d+00
      real(8),parameter::ga1 = 1.000 000 000 000 005 298d+00
      real(8),parameter::ga2 = 0.5d+00
      real(8),parameter::ga3 = 0.166 666 666 598 849 826d+00
      real(8),parameter::ga4 = 0.416 666 666 666 666 667d-01
      real(8),parameter::ga5 = 0.833 355 034 722 222 222d-02
      real(8),parameter::ga6 = 0.138 888 888 888 888 889d-02

      real(8),parameter::rminv(5) = (/
     &   1.0d+00,  0.333333333333333333d+00,  0.20d+00,
     &   0.14285714285714286d+00,  0.11111111111111111d+00 /)

#ifdef PL_SIMD_ERI
      dimension dmx(5,mxprm), val(1,mxprm)
#endif

      DO IJK=1,88
         DDSS(IJK)=0.0d0
      ENDDO
      
#ifdef PL_OMP_ERI
!$omp parallel reduction(+:ddss)
!$omp& default(private)
!$omp& shared(fdat,edat,
!$omp&        zetpq,ntprm,z,yp,yq,uu,uv,
!$omp&        ishel,jshel,kshel,lshel,iasp,jasp,kasp,lasp)
#endif
      igp = ntprm(ishel,iasp)
      jgp = ntprm(jshel,jasp)
      kgp = ntprm(kshel,kasp)
      lgp = ntprm(lshel,lasp)

      pi2 = 2.0d0*pi
      
#ifdef PL_OMP_ERI
!$omp do
#endif
      DO I = 1, IGP
         alp = z(i,ishel,iasp)
         bet06 =0.0d0
         bet07 =0.0d0
         bet08 =0.0d0
         bet09 =0.0d0
         bet10 =0.0d0
         bet11 =0.0d0
         bet12 =0.0d0
         bet13 =0.0d0
         bet14 =0.0d0
         bet15 =0.0d0
         bet23 =0.0d0
         bet24 =0.0d0
         bet25 =0.0d0
         bet26 =0.0d0
         bet27 =0.0d0
         bet28 =0.0d0
         bet29 =0.0d0
         bet30 =0.0d0
         bet31 =0.0d0
         bet34 =0.0d0
         bet35 =0.0d0
         bet36 =0.0d0
         bet37 =0.0d0
         bet39 =0.0d0
         bet40 =0.0d0
         bet41 =0.0d0
         bet51 =0.0d0
         bet52 =0.0d0
         bet53 =0.0d0
         bet54 =0.0d0
         bet55 =0.0d0
         bet56 =0.0d0
         bet56 =0.0d0
         bet57 =0.0d0
         bet58 =0.0d0
         bet61 =0.0d0
         bet62 =0.0d0
         bet64 =0.0d0
         bet65 =0.0d0
         bet67 =0.0d0
         bet68 =0.0d0
         bet69 =0.0d0
         bet74 =0.0d0
         bet75 =0.0d0
         bet76 =0.0d0
         bet77 =0.0d0
         bet78 =0.0d0
         bet79 =0.0d0
         bet81 =0.0d0
         bet82 =0.0d0
         bet83 =0.0d0
         bet86 =0.0d0
         bet87 =0.0d0
         bet88 =0.0d0
!init error 08/10/29 ddss
         bet60 =0.d0
!init error 08/10/29 ddss

         DO J = 1, JGP
            bet = z(j,jshel,jasp)
            zet = zetpq(j,i,jshel,jasp,ishel,iasp)
            px  = yp (1,j,i)
            py  = yp (2,j,i)
            pz  = yp (3,j,i)
            up  = uu(j,i)
            del01 = 0.0d0
            del16 = 0.0d0
            del42 = 0.0d0
            del70 = 0.0d0
            del84 = 0.0d0
            ram02 = 0.0d0
            ram03 = 0.0d0
            ram04 = 0.0d0
            ram05 = 0.0d0
            ram17 = 0.0d0
            ram18 = 0.0d0
            ram22 = 0.0d0
            ram43 = 0.0d0
            ram44 = 0.0d0
            ram72 = 0.0d0
            DO K = 1, KGP
               ram = z(k,kshel,kasp)
               del02 =0.0d0 
               del17 =0.0d0 
               del43 =0.0d0 
               del72 =0.0d0 
               del03 =0.0d0
               del18 =0.0d0
               del44 =0.0d0
               del04 =0.0d0
               del05 =0.0d0
               del22 =0.0d0
               
               DO L = 1, LGP
                  eta = zetpq(l,k,lshel,lasp,kshel,kasp)
                  qpx =  yq(1,l,k) - px
                  qpy =  yq(2,l,k) - py
                  qpz =  yq(3,l,k) - pz
                  vq  = uv(l,k)
                  
                  ROO2 = 1.0d0 / (ZET + ETA)
                  qp2  = qpx*qpx + qpy*qpy + qpz*qpz
                  T    = 0.5d0* ROO2*qp2
                  
                  IF (T. LE .TF(5)) THEN
                     
                     IT = IDINT((T+0.025D+00)*20.0D+00)
                     DT = 0.05D+00*IT - T
                     IT = IT + 1
                     DT4 = DT*0.25D+00
                     DT5 = DT*0.2D+00
                     DT6 = DT*0.166666666666666667D+00
                     DT2 = DT4 + DT4
                     DT3 = DT6 + DT6
                     F4   = (((((FDAT(11,IT)*DT6   + FDAT(10,IT))*DT5
     &                    + FDAT(9,IT))*DT4  + FDAT(8,IT))*DT3
     &                    + FDAT(7,IT))*DT2  + FDAT(6,IT))*DT
     &                    + FDAT(5,IT)
                     
                     EED  = (((((GA6*DT + GA5)*DT + GA4)*DT
     &                    + GA3)*DT + GA2)*DT + GA1)*DT + GA0
                     EE   = EDAT(IT)*EED
                     
                     T2   = T*2.0d0
                     F3   = (T2*F4 + EE)*RMINV(4)
                     F2   = (T2*F3 + EE)*RMINV(3)
                     F1   = (T2*F2 + EE)*RMINV(2)
                     F0   =  T2*F1 + EE
                     FPP =  UP * VQ * DSQRT(ROO2)
                     
                     DM0 = FPP*F0
                     DM1 = FPP*F1 *  ROO2
                     DM2 = FPP*F2 *( ROO2)**2
                     DM3 = FPP*F3 *( ROO2)**3
                     DM4 = FPP*F4 *( ROO2)**4
                     
                  ELSE 
                     
                     qinv = 1.0d0/qp2
                     dm0=  up*vq*dsqrt( pi2*qinv)*0.5d0
                     dm1 =        qinv *  dm0
                     dm2 =  3.0d0*qinv *  dm1
                     dm3 =  5.0d0*qinv *  dm2
                     dm4 =  7.0d0*qinv *  dm3
                  ENDIF

#ifdef PL_SIMD_ERI
                  val(1,l) = eta

                  dmx(1,l) = dm0
                  dmx(2,l) = dm1
                  dmx(3,l) = dm2
                  dmx(4,l) = dm3
                  dmx(5,l) = dm4
               enddo
               
               do l = 1, lgp
                  eta = val(1,l)

                  dm0 = dmx(1,l)
                  dm1 = dmx(2,l)
                  dm2 = dmx(3,l)
                  dm3 = dmx(4,l)
                  dm4 = dmx(5,l)
#endif

C** KERNEL DDSS =====================================================
C           CALCULATE & CONTRACT KERNELS FOR DDSS TYPE
C====================================================================
C  1:F004000 21:T014202 41:T407000 61:D204202  81:U214101
C  2:F004101 22:T014303 42:D003000 62:D205000  82:U225000
C  3:F004202 23:T104000 43:D003101 63:D215000  83:U315000
C  4:F004303 24:T104101 44:D003202 64:D215101  84:Z002000
C  5:F004404 25:T104202 45:D004000 65:D226000  85:Z023000
C  6:F105000 26:T104303 46:D014000 66:D305000  86:Z113000
C  7:F105101 27:T105000 47:D014101 67:D305101  87:Z203000
C  8:F105202 28:T105101 48:D024000 68:D316000  88:Z224000
C  9:F105303 29:T115000 49:D024101 69:D406000 
C 10:F206000 30:T115101 50:D024202 70:U003000 
C 11:F206101 31:T115202 51:D104000 71:U013000 
C 12:F206202 32:T205000 52:D104101 72:U013101 
C 13:F307000 33:T205101 53:D114000 73:U024000 
C 14:F307101 34:T205202 54:D114101 74:U103000 
C 15:F408000 35:T206000 55:D114202 75:U103101 
C 16:T004000 36:T216000 56:D115000 76:U114000 
C 17:T004101 37:T216101 57:D125000 77:U124000 
C 18:T004202 38:T306000 58:D125101 78:U124101 
C 19:T014000 39:T306101 59:D204000 79:U204000 
C 20:T014101 40:T317000 60:D204101 80:U214000 
C
C     
                  del01 = del01 + dm4
                  del02 = del02 + dm4 * eta   
                  del03 = del03 + dm4 * eta**2
                  del04 = del04 + dm4 * eta**3
                  del05 = del05 + dm4 * eta**4
!===============================
                  del16 =del16 +dm3
                  del17 =del17 +dm3 *eta   
                  del18 =del18 +dm3 *eta**2
                  del22 =del22 +dm3 *eta**3
!===============================
                  del42 =del42 +dm2
                  del43 =del43 +dm2 *eta
                  del44 =del44 +dm2 *eta**2
!===============================
                  del70 = del70 + dm1
                  del72 = del72 + dm1 *eta   
!===============================
                  del84 =del84+dm0
                  
               enddo          
               ram02 =ram02 + del02 *ram   
               ram17 =ram17 + del17 *ram   
               ram43 =ram43 + del43 *ram   
               ram72 =ram72 + del72 *ram   
               ram03 =ram03 + del03 *ram**2
               ram18 =ram18 + del18 *ram**2
               ram44 =ram44 + del44 *ram**2
               ram04 =ram04 + del04 *ram**3
               ram22 =ram22 + del22 *ram**3
               ram05 =ram05 + del05 *ram**4
            enddo
            ddss( 2) =ddss( 2)+ram02         *zet**3
            ddss( 3) =ddss( 3)+ram03         *zet**3
            ddss( 4) =ddss( 4)+ram04         *zet**3
            ddss( 5) =ddss( 5)+ram05         *zet**3
            ddss(17) =ddss(17)+ram17         *zet**3
            ddss(18) =ddss(18)+ram18         *zet**3
            ddss(20) =ddss(20)+ram17  *bet   *zet**3
            ddss(21) =ddss(21)+ram18  *bet   *zet**3
            ddss(22) =ddss(22)+ram22  *bet   *zet**3
            ddss(43) =ddss(43)+ram43         *zet**2
            ddss(44) =ddss(44)+ram44         *zet**2
            ddss(47) =ddss(47)+ram43  *bet   *zet**3
            ddss(49) =ddss(49)+ram43  *bet**2*zet**3
            ddss(50) =ddss(50)+ram44  *bet**2*zet**3
            ddss(72) =ddss(72)+ram72  *bet   *zet**2
            
            ddss( 1) =ddss( 1)+del01              *zet**3
            ddss(16) =ddss(16)+del16              *zet**3
            ddss(19) =ddss(19)+del16       *bet   *zet**3
            ddss(42) =ddss(42)+del42              *zet**2
            ddss(45) =ddss(45)+del42              *zet**3
            ddss(46) =ddss(46)+del42       *bet   *zet**3
            ddss(48) =ddss(48)+del42       *bet**2*zet**3
            ddss(70) =ddss(70)+del70              *zet**2
            ddss(71) =ddss(71)+del70       *bet   *zet**2
            ddss(73) =ddss(73)+del70       *bet**2*zet**3
            ddss(84) =ddss(84)+del84              *zet
            ddss(85) =ddss(85)+del84       *bet**2*zet**2
            
            bet75 =bet75+ram72       *zet**2
            
            bet26 =bet26+ram22       *zet**3
            bet24 =bet24+ram17       *zet**3
            bet25 =bet25+ram18       *zet**3
            bet52 =bet52+ram43       *zet**3
            bet60 =bet60+ram43       *zet**3
            bet61 =bet61+ram44       *zet**3
            bet54 =bet54+ram43*bet   *zet**3
            bet55 =bet55+ram44*bet   *zet**3
            bet81 =bet81+ram72*bet   *zet**3
            bet78 =bet78+ram72*bet**2*zet**3
            
            bet07 =bet07+ram02       *zet**4
            bet08 =bet08+ram03       *zet**4
            bet09 =bet09+ram04       *zet**4
            bet28 =bet28+ram17       *zet**4
            bet34 =bet34+ram18       *zet**4
            bet67 =bet67+ram43       *zet**4
            bet30 =bet30+ram17*bet   *zet**4
            bet31 =bet31+ram18*bet   *zet**4
            bet64 =bet64+ram43*bet   *zet**4
            bet58 =bet58+ram43*bet**2*zet**4
            
            bet11 =bet11+ram02       *zet**5
            bet12 =bet12+ram03       *zet**5
            bet39 =bet39+ram17       *zet**5
            bet37 =bet37+ram17*bet   *zet**5
            
            bet14 =bet14+ram02       *zet**6
            
            bet74 =bet74 +del70       *zet**2
            bet87 =bet87 +del84       *zet**2
            bet86 =bet86 +del84*bet   *zet**2
            
            bet23 =bet23 +del16       *zet**3                      
            bet51 =bet51 +del42       *zet**3                      
            bet79 =bet79 +del70       *zet**3
            bet53 =bet53 +del42*bet   *zet**3                      
            bet76 =bet76 +del70*bet   *zet**3
            bet77 =bet77 +del70*bet**2*zet**3
            bet88 =bet88 +del84*bet**2*zet**3
            
            bet06 =bet06 +del01       *zet**4
            bet27 =bet27 +del16       *zet**4                      
            bet62 =bet62 +del42       *zet**4
            bet29 =bet29 +del16*bet   *zet**4                      
            bet56 =bet56 +del42*bet   *zet**4                      
            bet83 =bet83 +del70*bet   *zet**4
            bet57 =bet57 +del42*bet**2*zet**4                      
            bet82 =bet82 +del70*bet**2*zet**4
            
            bet10 =bet10 +del01       *zet**5
            bet35 =bet35 +del16       *zet**5
            bet69 =bet69 +del42       *zet**5
            bet36 =bet36 +del16*bet   *zet**5
            bet68 =bet68 +del42*bet   *zet**5
            bet65 =bet65 +del42*bet**2*zet**5
            
            bet13 =bet13 +del01       *zet**6
            bet41 =bet41 +del16       *zet**6
            bet40 =bet40 +del16*bet   *zet**6
            
            bet15 =bet15 +del01       *zet**7
         enddo
         ddss( 6) =ddss( 6)+bet06*alp   
         ddss( 7) =ddss( 7)+bet07*alp   
         ddss( 8) =ddss( 8)+bet08*alp   
         ddss( 9) =ddss( 9)+bet09*alp   
         ddss(10) =ddss(10)+bet10*alp**2
         ddss(11) =ddss(11)+bet11*alp**2
         ddss(12) =ddss(12)+bet12*alp**2
         ddss(13) =ddss(13)+bet13*alp**3
         ddss(14) =ddss(14)+bet14*alp**3
         ddss(15) =ddss(15)+bet15*alp**4
         ddss(23) =ddss(23)+bet23*alp   
         ddss(24) =ddss(24)+bet24*alp   
         ddss(25) =ddss(25)+bet25*alp   
         ddss(26) =ddss(26)+bet26*alp   
         ddss(27) =ddss(27)+bet27*alp   
         ddss(28) =ddss(28)+bet28*alp   
         ddss(29) =ddss(29)+bet29*alp   
         ddss(30) =ddss(30)+bet30*alp   
         ddss(31) =ddss(31)+bet31*alp   
         ddss(32) =ddss(32)+bet27*alp**2
         ddss(33) =ddss(33)+bet28*alp**2
         ddss(34) =ddss(34)+bet34*alp**2
         ddss(35) =ddss(35)+bet35*alp**2
         ddss(36) =ddss(36)+bet36*alp**2
         ddss(37) =ddss(37)+bet37*alp**2
         ddss(38) =ddss(38)+bet35*alp**3
         ddss(39) =ddss(39)+bet39*alp**3
         ddss(40) =ddss(40)+bet40*alp**3
         ddss(41) =ddss(41)+bet41*alp**4
         ddss(51) =ddss(51)+bet51*alp   
         ddss(52) =ddss(52)+bet52*alp   
         ddss(53) =ddss(53)+bet53*alp   
         ddss(54) =ddss(54)+bet54*alp   
         ddss(55) =ddss(55)+bet55*alp   
         ddss(56) =ddss(56)+bet56*alp   
         ddss(57) =ddss(57)+bet57*alp   
         ddss(58) =ddss(58)+bet58*alp   
         ddss(59) =ddss(59)+bet51*alp**2
         ddss(60) =ddss(60)+bet52*alp**2
         ddss(61) =ddss(61)+bet61*alp**2
         ddss(62) =ddss(62)+bet62*alp**2
         ddss(63) =ddss(63)+bet56*alp**2
         ddss(64) =ddss(64)+bet64*alp**2
         ddss(65) =ddss(65)+bet65*alp**2
         ddss(66) =ddss(66)+bet62*alp**3
         ddss(67) =ddss(67)+bet67*alp**3
         ddss(68) =ddss(68)+bet68*alp**3
         ddss(69) =ddss(69)+bet69*alp**4
         ddss(74) =ddss(74)+bet74*alp   
         ddss(75) =ddss(75)+bet75*alp   
         ddss(76) =ddss(76)+bet76*alp   
         ddss(77) =ddss(77)+bet77*alp   
         ddss(78) =ddss(78)+bet78*alp   
         ddss(79) =ddss(79)+bet79*alp**2
         ddss(80) =ddss(80)+bet76*alp**2
         ddss(81) =ddss(81)+bet81*alp**2
         ddss(82) =ddss(82)+bet82*alp**2
         ddss(83) =ddss(83)+bet83*alp**3
         ddss(86) =ddss(86)+bet86*alp   
         ddss(87) =ddss(87)+bet87*alp**2
         ddss(88) =ddss(88)+bet88*alp**2
      enddo
#ifdef PL_OMP_ERI
!$omp end do
!$omp end parallel
#endif
      
      end
C**IKDPPP ===========================================================
C          CALUCATE KERNELS FOR DPPP TYPE
C====================================================================

      subroutine ikdppp(
     &           dppp ,
     &           fdat ,  edat  ,
     &           yp   ,  yq    ,
     &           uu   ,  uv    ,
     &           z    ,  zetpq , 
     &           nasp ,  mxsa  ,  mxprm,  ntprm,  
     &           iasp ,  jasp  ,  kasp ,  lasp ,
     &           ishel,  jshel ,  kshel,  lshel     )


      implicit double precision  (a-h,o-z)
      dimension fdat(31,2001)
      dimension edat(2001)
      dimension zetpq(mxprm,mxprm,mxsa,nasp,mxsa,nasp)
      dimension yp(3,mxprm,mxprm),yq(3,mxprm,mxprm)
      dimension uu(mxprm,mxprm),uv(mxprm,mxprm)
      dimension z(mxprm,mxsa,nasp)
      dimension ntprm(mxsa,nasp)
      dimension dppp(336)

      real(8),parameter::pi    = 3.14159265358979323846d+00
      real(8),parameter::tf(25) = (/
     &                33.0,37.0,41.0,43.0,46.0,49.0,
     &                51.0,54.0,56.0,58.0,61.0,63.0,
     &                66.0,68.0,70.0,72.0,74.0,80.0,
     &                85.0,90.0,95.0,95.0,98.0,99.0,
     &                99.0                           /)

      real(8),parameter::ga0 = 1.0d+00
      real(8),parameter::ga1 = 1.000 000 000 000 005 298d+00
      real(8),parameter::ga2 = 0.5d+00
      real(8),parameter::ga3 = 0.166 666 666 598 849 826d+00
      real(8),parameter::ga4 = 0.416 666 666 666 666 667d-01
      real(8),parameter::ga5 = 0.833 355 034 722 222 222d-02
      real(8),parameter::ga6 = 0.138 888 888 888 888 889d-02

      real(8),parameter::rminv(5) = (/
     &   1.0d+00,  0.333333333333333333d+00,  0.20d+00,
     &   0.14285714285714286d+00,  0.11111111111111111d+00 /)
      
#ifdef PL_SIMD_ERI
      dimension dmx(6,mxprm), val(2,mxprm)
#endif

      DO IJK=1,336
         DPPP(IJK)=0.0d0
      ENDDO
      
#ifdef PL_OMP_ERI
!$omp parallel reduction(+:dppp)
!$omp& default(private)
!$omp& shared(fdat,edat,
!$omp&        zetpq,ntprm,z,yp,yq,uu,uv,
!$omp&        ishel,jshel,kshel,lshel,iasp,jasp,kasp,lasp)
#endif
      igp = ntprm(ishel,iasp)
      jgp = ntprm(jshel,jasp)
      kgp = ntprm(kshel,kasp)
      lgp = ntprm(lshel,lasp)

      pi2 = 2.0d0*pi

#ifdef PL_OMP_ERI
!$omp do
#endif
      DO I = 1, IGP
         bet034=0.0d0
         bet035=0.0d0
         bet036=0.0d0
         bet037=0.0d0
         bet038=0.0d0
         bet038=0.0d0
         bet039=0.0d0
         bet040=0.0d0
         bet041=0.0d0
         bet042=0.0d0
         bet043=0.0d0
         bet044=0.0d0
         bet045=0.0d0
         bet046=0.0d0
         bet047=0.0d0
         bet048=0.0d0
         bet049=0.0d0
         bet050=0.0d0
         bet051=0.0d0
         bet052=0.0d0
         bet053=0.0d0
         bet054=0.0d0
         bet055=0.0d0
         bet056=0.0d0
         bet057=0.0d0
         bet058=0.0d0
         bet059=0.0d0
         bet060=0.0d0
         bet061=0.0d0
         bet062=0.0d0
         bet063=0.0d0
         bet064=0.0d0
         bet065=0.0d0
         bet066=0.0d0
         bet067=0.0d0
         bet068=0.0d0
         bet070=0.0d0
         bet072=0.0d0
         bet073=0.0d0
         bet074=0.0d0
         bet075=0.0d0
         bet076=0.0d0
         bet078=0.0d0
         bet079=0.0d0
         bet080=0.0d0
         bet081=0.0d0
         bet082=0.0d0
         bet084=0.0d0
         bet085=0.0d0
         bet086=0.0d0
         bet087=0.0d0
         bet088=0.0d0
         bet089=0.0d0
         bet090=0.0d0
         bet091=0.0d0
         bet092=0.0d0
         bet112=0.0d0
         bet113=0.0d0
         bet114=0.0d0
         bet115=0.0d0
         bet116=0.0d0
         bet117=0.0d0
         bet118=0.0d0
         bet119=0.0d0
         bet120=0.0d0
         bet121=0.0d0
         bet122=0.0d0
         bet123=0.0d0
         bet124=0.0d0
         bet125=0.0d0
         bet126=0.0d0
         bet127=0.0d0
         bet128=0.0d0
         bet129=0.0d0
         bet130=0.0d0
         bet131=0.0d0
         bet133=0.0d0
         bet134=0.0d0
         bet135=0.0d0
         bet136=0.0d0
         bet137=0.0d0
         bet138=0.0d0
         bet139=0.0d0
         bet140=0.0d0
         bet141=0.0d0
         bet142=0.0d0
         bet143=0.0d0
         bet144=0.0d0
         bet145=0.0d0
         bet146=0.0d0
         bet148=0.0d0
         bet149=0.0d0
         bet150=0.0d0
         bet151=0.0d0
         bet152=0.0d0
         bet153=0.0d0
         bet154=0.0d0
         bet155=0.0d0
         bet156=0.0d0
         bet158=0.0d0
         bet159=0.0d0
         bet160=0.0d0
         bet161=0.0d0
         bet162=0.0d0
         bet195=0.0d0
         bet196=0.0d0
         bet197=0.0d0
         bet198=0.0d0
         bet199=0.0d0
         bet200=0.0d0
         bet201=0.0d0
         bet202=0.0d0
         bet203=0.0d0
         bet204=0.0d0
         bet205=0.0d0
         bet206=0.0d0
         bet207=0.0d0
         bet208=0.0d0
         bet209=0.0d0
         bet210=0.0d0
         bet211=0.0d0
         bet212=0.0d0
         bet213=0.0d0
         bet214=0.0d0
         bet215=0.0d0
         bet216=0.0d0
         bet217=0.0d0
         bet218=0.0d0
         bet219=0.0d0
         bet220=0.0d0
         bet221=0.0d0
         bet222=0.0d0
         bet223=0.0d0
         bet224=0.0d0
         bet225=0.0d0
         bet226=0.0d0
         bet227=0.0d0
         bet228=0.0d0
         bet229=0.0d0
         bet232=0.0d0
         bet233=0.0d0
         bet234=0.0d0
         bet235=0.0d0
         bet236=0.0d0
         bet237=0.0d0
         bet238=0.0d0
         bet239=0.0d0
         bet240=0.0d0
         bet241=0.0d0
         bet242=0.0d0
         bet244=0.0d0
         bet245=0.0d0
         bet246=0.0d0
         bet247=0.0d0
         bet248=0.0d0
         bet249=0.0d0
         bet250=0.0d0
         bet251=0.0d0
         bet252=0.0d0
         bet253=0.0d0
         bet255=0.0d0
         bet256=0.0d0
         bet257=0.0d0
         bet258=0.0d0
         bet259=0.0d0
         bet261=0.0d0
         bet262=0.0d0
         bet263=0.0d0
         bet264=0.0d0
         bet265=0.0d0
         bet266=0.0d0
         bet267=0.0d0
         bet285=0.0d0
         bet286=0.0d0
         bet287=0.0d0
         bet288=0.0d0
         bet289=0.0d0
         bet290=0.0d0
         bet291=0.0d0
         bet292=0.0d0
         bet293=0.0d0
         bet294=0.0d0
         bet295=0.0d0
         bet296=0.0d0
         bet297=0.0d0
         bet298=0.0d0
         bet299=0.0d0
         bet300=0.0d0
         bet301=0.0d0
         bet302=0.0d0
         bet303=0.0d0
         bet304=0.0d0
         bet305=0.0d0
         bet306=0.0d0
         bet307=0.0d0
         bet308=0.0d0
         bet309=0.0d0
         bet316=0.0d0
         bet317=0.0d0
         bet318=0.0d0
         bet319=0.0d0
         bet320=0.0d0
         bet321=0.0d0
         bet322=0.0d0
         bet323=0.0d0
         bet324=0.0d0
         bet325=0.0d0
         bet326=0.0d0
         bet327=0.0d0
         bet328=0.0d0
         bet329=0.0d0
         bet330=0.0d0
         bet333=0.0d0
         bet334=0.0d0
         bet335=0.0d0
         bet336=0.0d0          
!init error 08/10/29 dppp
         bet069=0.d0
!init error 08/10/29 dppp
         alp = z(i,ishel,iasp)
         
         DO J = 1, JGP
            del001 = 0.0d0
            del002 = 0.0d0
            del009 = 0.0d0
            del093 = 0.0d0
            del094 = 0.0d0
            del163 = 0.0d0
            del167 = 0.0d0
            del169 = 0.0d0
            del206 = 0.0d0
            del238 = 0.0d0
            del258 = 0.0d0
            del268 = 0.0d0
            del269 = 0.0d0
            del273 = 0.0d0
            del310 = 0.0d0
            del331 = 0.0d0
            ram003 = 0.0d0
            ram004 = 0.0d0
            ram005 = 0.0d0
            ram006 = 0.0d0
            ram007 = 0.0d0
            ram008 = 0.0d0
            ram012 = 0.0d0
            ram013 = 0.0d0
            ram016 = 0.0d0
            ram025 = 0.0d0
            ram095 = 0.0d0
            ram096 = 0.0d0
            ram097 = 0.0d0
            ram098 = 0.0d0
            ram099 = 0.0d0
            ram100 = 0.0d0
            ram101 = 0.0d0
            ram102 = 0.0d0
            ram103 = 0.0d0
            ram104 = 0.0d0
            ram105 = 0.0d0
            ram106 = 0.0d0
            ram111 = 0.0d0
            ram164 = 0.0d0
            ram165 = 0.0d0
            ram166 = 0.0d0
            ram170 = 0.0d0
            ram172 = 0.0d0
            ram173 = 0.0d0
            ram174 = 0.0d0
            ram175 = 0.0d0
            ram176 = 0.0d0
            ram177 = 0.0d0
            ram178 = 0.0d0
            ram179 = 0.0d0
            ram189 = 0.0d0
            ram190 = 0.0d0
            ram270 = 0.0d0
            ram271 = 0.0d0
            ram272 = 0.0d0
            ram276 = 0.0d0
            ram277 = 0.0d0
            ram311 = 0.0d0
            ram312 = 0.0d0
            ram313 = 0.0d0
            ram314 = 0.0d0
            ram315 = 0.0d0
            ram332 = 0.0d0          
            bet = z(j,jshel,jasp)
            zet = zetpq(j,i,jshel,jasp,ishel,iasp)
           px  = yp (1,j,i)
           py  = yp (2,j,i)
           pz  = yp (3,j,i)
           up  = uu(j,i)

           DO K = 1, KGP
              del003 = 0.0d0 
              del004 = 0.0d0 
              del005 = 0.0d0 
              del012 = 0.0d0 
              del095 = 0.0d0 
              del096 = 0.0d0 
              del097 = 0.0d0 
              del164 = 0.0d0 
              del170 = 0.0d0 
              del172 = 0.0d0 
              del173 = 0.0d0 
              del270 = 0.0d0 
              del271 = 0.0d0 
              del272 = 0.0d0 
              del311 = 0.0d0 
              del332 = 0.0d0 
              del007 = 0.0d0
              del099 = 0.0d0
              del100 = 0.0d0
              del165 = 0.0d0
              del176 = 0.0d0
              del277 = 0.0d0
              del312 = 0.0d0
              del008 = 0.0d0
              del103 = 0.0d0
              del102 = 0.0d0
              del166 = 0.0d0
              del179 = 0.0d0
              del313 = 0.0d0
              del105 = 0.0d0
              del106 = 0.0d0
              del314 = 0.0d0
              del315 = 0.0d0          
!init error 08/10/29 dppp
              del104 = 0.0d0
              del110 = 0.0d0
              del115 = 0.0d0
              del127 = 0.0d0
              del131 = 0.0d0
              del135 = 0.0d0
!init error 08/10/29 dppp
              ram = z(k,kshel,kasp)

              DO L = 1, LGP
                 del = z(l,lshel,lasp)
                  eta = zetpq(l,k,lshel,lasp,kshel,kasp)
                 qpx =  yq(1,l,k) - px
                 qpy =  yq(2,l,k) - py
                 qpz =  yq(3,l,k) - pz
                 vq  = uv(l,k)
     
                 ROO2 = 1.0d0 / (ZET + ETA)
                 qp2  = qpx*qpx + qpy*qpy + qpz*qpz
                 T    = 0.5d0* ROO2*qp2

                IF (T. LE .TF(6)) THEN
                  
                   IT = IDINT((T+0.025D+00)*20.0D+00)
                   DT = 0.05D+00*IT - T
                   IT = IT + 1
                   DT4 = DT*0.25D+00
                   DT5 = DT*0.2D+00
                   DT6 = DT*0.166666666666666667D+00
                   DT2 = DT4 + DT4
                   DT3 = DT6 + DT6
                   F5   = (((((FDAT(12,IT)*DT6   + FDAT(11,IT))*DT5
     &               + FDAT(10,IT))*DT4  + FDAT(9,IT))*DT3
     &               + FDAT(8,IT))*DT2  + FDAT(7,IT))*DT
     &               + FDAT(6,IT)
 
                   EED  = (((((GA6*DT + GA5)*DT + GA4)*DT
     &               + GA3)*DT + GA2)*DT + GA1)*DT + GA0
                   EE   = EDAT(IT)*EED
 
                   T2   = T*2.0d0
                   F4   = (T2*F5 + EE)*RMINV(5)
                   F3   = (T2*F4 + EE)*RMINV(4)
                   F2   = (T2*F3 + EE)*RMINV(3)
                   F1   = (T2*F2 + EE)*RMINV(2)
                   F0   =  T2*F1 + EE
                   FPP =  UP * VQ * DSQRT(ROO2)
               
                   DM0 = FPP*F0
                   DM1 = FPP*F1 *  ROO2
                   DM2 = FPP*F2 *( ROO2)**2
                   DM3 = FPP*F3 *( ROO2)**3
                   DM4 = FPP*F4 *( ROO2)**4
                   DM5 = FPP*F5 *( ROO2)**5

                ELSE 

                   qinv = 1.0d0/qp2
                   dm0=  up*vq*dsqrt( pi2*qinv)*0.5d0
                   dm1 =        qinv *  dm0
                   dm2 =  3.0d0*qinv *  dm1
                   dm3 =  5.0d0*qinv *  dm2
                   dm4 =  7.0d0*qinv *  dm3
                   dm5 =  9.0d0*qinv *  dm4

                ENDIF
 
#ifdef PL_SIMD_ERI
                  val(1,l) = eta
                  val(2,l) = del

                  dmx(1,l) = dm0
                  dmx(2,l) = dm1
                  dmx(3,l) = dm2
                  dmx(4,l) = dm3
                  dmx(5,l) = dm4
                  dmx(6,l) = dm5
               enddo
               
               do l = 1, lgp
                  eta = val(1,l)
                  del = val(2,l)

                  dm0 = dmx(1,l)
                  dm1 = dmx(2,l)
                  dm2 = dmx(3,l)
                  dm3 = dmx(4,l)
                  dm4 = dmx(5,l)
                  dm5 = dmx(6,l)
#endif

!** KERNEL DPPP =====================================================
!           CALUCLATE & CONTRACT KERNELS FOR DPPP TYPE
!====================================================================
c
                del009 = del009 + dm2             
                del001 = del001 + dm2       *eta   
                del003 = del003 + dm2       *eta   
                del004 = del004 + dm2       *eta**2
                del008 = del008 + dm2       *eta**3
                del002 = del002 + dm2*del   *eta   
                del012 = del012 + dm2*del   *eta   
                del005 = del005 + dm2*del   *eta**2
                del007 = del007 + dm2*del   *eta**3
!================================================--
                del093 = del093 + dm4       *eta   
                del095 = del095 + dm4       *eta   
                del096 = del096 + dm4       *eta**2
                del099 = del099 + dm4       *eta**3
          
                del102 = del102 + dm4       *eta**4
                del104 = del104 + dm4       *eta**4
                del110 = del110 + dm4       *eta**4
                del115 = del115 + dm4       *eta**4
                del127 = del127 + dm4       *eta**4
                del131 = del131 + dm4       *eta**4
                del135 = del135 + dm4       *eta**4
                del106 = del106 + dm4       *eta**5
                del094 = del094 + dm4*del   *eta   
                del097 = del097 + dm4*del   *eta**2
                del100 = del100 + dm4*del   *eta**3
                del103 = del103 + dm4*del   *eta**4
                del105 = del105 + dm4*del   *eta**5
!========================================================      

                del167 = del167 + dm3             
                del206 = del206 + dm3             
                del238 = del238 + dm3             
                del258 = del258 + dm3             
                del163 = del163 + dm3       *eta   
                del170 = del170 + dm3       *eta   
                del164 = del164 + dm3       *eta**2
                del165 = del165 + dm3       *eta**3
                del166 = del166 + dm3       *eta**4
                del169 = del169 + dm3*del   *eta   
                del172 = del172 + dm3*del   *eta   
                del173 = del173 + dm3*del   *eta**2
                del176 = del176 + dm3*del   *eta**3
                del179 = del179 + dm3*del   *eta**4
!==============================================
                del268 = del268 + dm1             
                del270 = del270 + dm1       *eta   
                del273 = del273 + dm1       *eta   
                del277 = del277 + dm1       *eta**2
                del269 = del269 + dm1*del   *eta   
                del271 = del271 + dm1*del   *eta   
                del272 = del272 + dm1*del   *eta**2
!==============================================

                del310 = del310 + dm5       *eta   
                del311 = del311 + dm5       *eta**2
                del312 = del312 + dm5       *eta**3
                del313 = del313 + dm5       *eta**4
                del314 = del314 + dm5       *eta**5
                del315 = del315 + dm5       *eta**6
!==============================================
                del331 = del331 + dm0             
                del332 = del332 + dm0*del   *eta   

             enddo
             ram003 = ram003 + del003 * ram   
             ram004 = ram004 + del004 * ram   
             ram005 = ram005 + del005 * ram   
             ram012 = ram012 + del012 * ram   
             ram095 = ram095 + del095 * ram   
             ram096 = ram096 + del096 * ram   
             ram097 = ram097 + del097 * ram   
             ram164 = ram164 + del164 * ram   
             ram170 = ram170 + del170 * ram   
             ram172 = ram172 + del172 * ram   
             ram173 = ram173 + del173 * ram   
             ram270 = ram270 + del270 * ram   
             ram271 = ram271 + del271 * ram   
             ram276 = ram276 + del272 * ram   
             ram311 = ram311 + del311 * ram   
             ram332 = ram332 + del332 * ram   
       
             ram006 = ram006 + del004 * ram**2
             ram013 = ram013 + del005 * ram**2
             ram007 = ram007 + del007 * ram**2
             ram016 = ram016 + del008 * ram**2
             ram099 = ram099 + del099 * ram**2
             ram098 = ram098 + del096 * ram**2
             ram100 = ram100 + del100 * ram**2
             ram165 = ram165 + del165 * ram**2
             ram174 = ram174 + del164 * ram**2
             ram176 = ram176 + del176 * ram**2
             ram175 = ram175 + del173 * ram**2
             ram272 = ram272 + del272 * ram**2
             ram277 = ram277 + del277 * ram**2
             ram312 = ram312 + del312 * ram**2

             ram008 = ram008 + del008 * ram**3
             ram025 = ram025 + del007 * ram**3
             ram101 = ram101 + del099 * ram**3
             ram103 = ram103 + del103 * ram**3
             ram102 = ram102 + del102 * ram**3
             ram178 = ram178 + del176 * ram**3
             ram177 = ram177 + del165 * ram**3
             ram166 = ram166 + del166 * ram**3
             ram189 = ram189 + del179 * ram**3
             ram313 = ram313 + del313 * ram**3
       
             ram190 = ram190 + del166 * ram**4
             ram104 = ram104 + del102 * ram**4
             ram105 = ram105 + del105 * ram**4
             ram111 = ram111 + del106 * ram**4
             ram179 = ram179 + del179 * ram**4
             ram314 = ram314 + del314 * ram**4
       
             ram106 = ram106 + del106 * ram**5
             ram315 = ram315 + del315 * ram**5

          enddo

          dppp(  1)=dppp(  1)+del001              *zet   
          dppp(  2)=dppp(  2)+del002              *zet   
          dppp(  9)=dppp(  9)+del009              *zet**2
          dppp( 10)=dppp( 10)+del002              *zet**2
          dppp( 14)=dppp( 14)+del001       *bet   *zet   
          dppp( 17)=dppp( 17)+del009       *bet   *zet**2
          dppp( 18)=dppp( 18)+del001       *bet   *zet**2
          dppp( 19)=dppp( 19)+del002       *bet   *zet**2
          dppp( 26)=dppp( 26)+del001       *bet**2*zet**2
          dppp( 27)=dppp( 27)+del002       *bet**2*zet**2
          dppp( 93)=dppp( 93)+del093              *zet**2
          dppp( 94)=dppp( 94)+del094              *zet**2
          dppp(107)=dppp(107)+del093       *bet   *zet**2
          dppp(163)=dppp(163)+del163              *zet   
          dppp(167)=dppp(167)+del167              *zet**2
          dppp(168)=dppp(168)+del163              *zet**2
          dppp(169)=dppp(169)+del169              *zet**2
          dppp(180)=dppp(180)+del163       *bet   *zet**2
          dppp(181)=dppp(181)+del169       *bet   *zet**2
          dppp(191)=dppp(191)+del163       *bet**2*zet**2
          dppp(268)=dppp(268)+del268              *zet   
          dppp(269)=dppp(269)+del269              *zet   
          dppp(273)=dppp(273)+del273       *bet   *zet   
          dppp(274)=dppp(274)+del269       *bet   *zet   
          dppp(278)=dppp(278)+del268       *bet   *zet**2
          dppp(280)=dppp(280)+del268       *bet**2*zet**2
          dppp(281)=dppp(281)+del269       *bet**2*zet**2
          dppp(310)=dppp(310)+del310              *zet**2
          dppp(331)=dppp(331)+del331       *bet   *zet   
        
          bet034 =bet034 +del001       *zet   
          bet038 =bet038 +del001       *zet**2
          bet069 =bet069 +del001       *zet**2
          bet087 =bet087 +del001       *zet**3
          bet048 =bet048 +del001*bet   *zet**2
          bet076 =bet076 +del001*bet   *zet**3
          bet061 =bet061 +del001*bet**2*zet**2
          bet064 =bet064 +del001*bet**2*zet**3
          bet092 =bet092 +del001*bet**2*zet**4

          bet039 =bet039 +del002       *zet**2
          bet072 =bet072 +del002       *zet**3
          bet049 =bet049 +del002*bet   *zet**2
          bet057 =bet057 +del002*bet   *zet**3
          bet090 =bet090 +del002*bet   *zet**4
          bet065 =bet065 +del002*bet**2*zet**3
          bet085 =bet085 +del002*bet**2*zet**4
    
          bet037 =bet037 +del009       *zet**2
          bet046 =bet046 +del009       *zet**3
          bet088 =bet088 +del009       *zet**4
          bet056 =bet056 +del009*bet   *zet**3
          bet081 =bet081 +del009*bet   *zet**4
!=======================================================
          bet112 =bet112 +del093       *zet**2
          bet117 =bet117 +del093       *zet**3
          bet136 =bet136 +del093       *zet**4
          bet150 =bet150 +del093       *zet**5
          bet162 =bet162 +del093       *zet**6
          bet128 =bet128 +del093*bet   *zet**3
          bet144 =bet144 +del093*bet   *zet**4
          bet155 =bet155 +del093*bet   *zet**5
          bet161 =bet161 +del093*bet   *zet**6
    
          bet118 =bet118 +del094       *zet**3
          bet137 =bet137 +del094       *zet**4
          bet151 =bet151 +del094       *zet**5
          bet159 =bet159 +del094       *zet**6
    
          bet195 =bet195 +del163       *zet**2
          bet207 =bet207 +del163       *zet**3
          bet253 =bet253 +del163       *zet**4
    
          bet215 =bet215 +del163*bet   *zet**2
          bet219 =bet219 +del163*bet   *zet**3
          bet246 =bet246 +del163*bet   *zet**4
          bet267 =bet267 +del163*bet   *zet**5
    
          bet227 =bet227 +del163*bet**2*zet**3
          bet251 =bet251 +del163*bet**2*zet**4
          bet264 =bet264 +del163*bet**2*zet**5
    
          bet196 =bet196 +del169       *zet**2
          bet208 =bet208 +del169       *zet**3
          bet239 =bet239 +del169       *zet**4
          bet265 =bet265 +del169       *zet**5
          bet220 =bet220 +del169*bet   *zet**3
          bet247 =bet247 +del169*bet   *zet**4
          bet262 =bet262 +del169*bet   *zet**5
    
          bet206 =bet206 +del206       *zet**3
          bet238 =bet238 +del238       *zet**4
          bet258 =bet258 +del258       *zet**5
          bet285 =bet285 +del273       *zet   
          bet297 =bet297 +del273*bet**2*zet**2

          bet286 =bet286 +del269       *zet   
          bet304 =bet304 +del269       *zet**2
          bet293 =bet293 +del269*bet   *zet**2
          bet298 =bet298 +del269*bet**2*zet**2
          bet308 =bet308 +del269*bet**2*zet**3
    
          bet290 =bet290 +del268       *zet**2
          bet292 =bet292 +del268*bet   *zet**2
          bet306 =bet306 +del268*bet   *zet**3
          bet302 =bet302 +del268*bet**2*zet**3
    
          bet316 =bet316 +del310       *zet**3
          bet321 =bet321 +del310       *zet**4
          bet325 =bet325 +del310       *zet**5
          bet328 =bet328 +del310       *zet**6
          bet330 =bet330 +del310       *zet**7
    
          bet333 =bet333 +del331       *zet   
          bet335 =bet335 +del331*bet**2*zet**2
    
          dppp(  3)=dppp(  3)+ram003              *zet    
          dppp(  4)=dppp(  4)+ram004              *zet    
          dppp(  5)=dppp(  5)+ram005              *zet    
          dppp(  6)=dppp(  6)+ram006              *zet    
          dppp(  7)=dppp(  7)+ram007              *zet    
          dppp(  8)=dppp(  8)+ram008              *zet    
          dppp( 11)=dppp( 11)+ram003              *zet**2 
          dppp( 12)=dppp( 12)+ram012              *zet**2 
          dppp( 13)=dppp( 13)+ram013              *zet**2 
          dppp( 15)=dppp( 15)+ram004       *bet   *zet    
          dppp( 16)=dppp( 16)+ram016       *bet   *zet    
          dppp( 20)=dppp( 20)+ram003       *bet   *zet**2 
          dppp( 21)=dppp( 21)+ram012       *bet   *zet**2 
          dppp( 22)=dppp( 22)+ram005       *bet   *zet**2 
          dppp( 23)=dppp( 23)+ram006       *bet   *zet**2 
          dppp( 24)=dppp( 24)+ram013       *bet   *zet**2 
          dppp( 25)=dppp( 25)+ram025       *bet   *zet**2 
          dppp( 28)=dppp( 28)+ram003       *bet**2*zet**2 
          dppp( 29)=dppp( 29)+ram004       *bet**2*zet**2 
          dppp( 30)=dppp( 30)+ram005       *bet**2*zet**2 
          dppp( 31)=dppp( 31)+ram006       *bet**2*zet**2 
          dppp( 32)=dppp( 32)+ram007       *bet**2*zet**2 
          dppp( 33)=dppp( 33)+ram008       *bet**2*zet**2 
          dppp( 95)=dppp( 95)+ram095              *zet**2
          dppp( 96)=dppp( 96)+ram096              *zet**2
          dppp( 97)=dppp( 97)+ram097              *zet**2
          dppp( 98)=dppp( 98)+ram098              *zet**2
          dppp( 99)=dppp( 99)+ram099              *zet**2
          dppp(100)=dppp(100)+ram100              *zet**2
          dppp(101)=dppp(101)+ram101              *zet**2
          dppp(102)=dppp(102)+ram102              *zet**2
          dppp(103)=dppp(103)+ram103              *zet**2
          dppp(104)=dppp(104)+ram104              *zet**2
          dppp(105)=dppp(105)+ram105              *zet**2
          dppp(106)=dppp(106)+ram106              *zet**2
          dppp(108)=dppp(108)+ram096       *bet   *zet**2
          dppp(109)=dppp(109)+ram099       *bet   *zet**2
          dppp(110)=dppp(110)+ram102       *bet   *zet**2
          dppp(111)=dppp(111)+ram111       *bet   *zet**2
          dppp(164)=dppp(164)+ram164              *zet   
          dppp(165)=dppp(165)+ram165              *zet   
          dppp(166)=dppp(166)+ram166              *zet   
          dppp(170)=dppp(170)+ram170              *zet**2
          dppp(171)=dppp(171)+ram164              *zet**2
          dppp(172)=dppp(172)+ram172              *zet**2
          dppp(173)=dppp(173)+ram173              *zet**2
          dppp(174)=dppp(174)+ram174              *zet**2
          dppp(175)=dppp(175)+ram175              *zet**2
          dppp(176)=dppp(176)+ram176              *zet**2
          dppp(177)=dppp(177)+ram177              *zet**2
          dppp(178)=dppp(178)+ram178              *zet**2
          dppp(179)=dppp(179)+ram179              *zet**2
          dppp(182)=dppp(182)+ram170       *bet   *zet**2
          dppp(183)=dppp(183)+ram164       *bet   *zet**2
          dppp(184)=dppp(184)+ram173       *bet   *zet**2
          dppp(185)=dppp(185)+ram174       *bet   *zet**2
          dppp(186)=dppp(186)+ram165       *bet   *zet**2
          dppp(187)=dppp(187)+ram176       *bet   *zet**2
          dppp(188)=dppp(188)+ram177       *bet   *zet**2
          dppp(189)=dppp(189)+ram189       *bet   *zet**2
          dppp(190)=dppp(190)+ram190       *bet   *zet**2
          dppp(192)=dppp(192)+ram164       *bet**2*zet**2
          dppp(193)=dppp(193)+ram165       *bet**2*zet**2
          dppp(194)=dppp(194)+ram166       *bet**2*zet**2
          dppp(270)=dppp(270)+ram270              *zet   
          dppp(271)=dppp(271)+ram271              *zet   
          dppp(272)=dppp(272)+ram272              *zet   
          dppp(275)=dppp(275)+ram270       *bet   *zet   
          dppp(276)=dppp(276)+ram276       *bet   *zet   
          dppp(277)=dppp(277)+ram277       *bet   *zet   
          dppp(279)=dppp(279)+ram271       *bet   *zet**2
          dppp(282)=dppp(282)+ram270       *bet**2*zet**2
          dppp(283)=dppp(283)+ram271       *bet**2*zet**2
          dppp(284)=dppp(284)+ram272       *bet**2*zet**2
          dppp(311)=dppp(311)+ram311              *zet**2
          dppp(312)=dppp(312)+ram312              *zet**2
          dppp(313)=dppp(313)+ram313              *zet**2
          dppp(314)=dppp(314)+ram314              *zet**2
          dppp(315)=dppp(315)+ram315              *zet**2
          dppp(332)=dppp(332)+ram332       *bet   *zet   
    
          bet035 =bet035 +ram004       *zet   
          bet036 =bet036 +ram016       *zet   
          bet040 =bet040 +ram003       *zet**2
          bet041 =bet041 +ram012       *zet**2
          bet042 =bet042 +ram005       *zet**2
          bet043 =bet043 +ram006       *zet**2
          bet044 =bet044 +ram013       *zet**2
          bet045 =bet045 +ram025       *zet**2
          bet047 =bet047 +ram012       *zet**3
          bet050 =bet050 +ram003*bet   *zet**2
          bet051 =bet051 +ram004*bet   *zet**2
          bet052 =bet052 +ram005*bet   *zet**2
          bet053 =bet053 +ram006*bet   *zet**2
          bet054 =bet054 +ram007*bet   *zet**2
          bet055 =bet055 +ram008*bet   *zet**2
          bet058 =bet058 +ram003*bet   *zet**3
          bet059 =bet059 +ram012*bet   *zet**3
          bet060 =bet060 +ram013*bet   *zet**3
          bet062 =bet062 +ram004*bet**2*zet**2
          bet063 =bet063 +ram016*bet**2*zet**2
          bet066 =bet066 +ram003*bet**2*zet**3
          bet067 =bet067 +ram005*bet**2*zet**3
          bet068 =bet068 +ram006*bet**2*zet**3
          bet070 =bet070 +ram004       *zet**2
          bet073 =bet073 +ram003       *zet**3
          bet074 =bet074 +ram012       *zet**3
          bet075 =bet075 +ram013       *zet**3
          bet078 =bet078 +ram003*bet   *zet**3
          bet079 =bet079 +ram005*bet   *zet**3
          bet080 =bet080 +ram006*bet   *zet**3
          bet082 =bet082 +ram012*bet   *zet**4
          bet084 =bet084 +ram004*bet**2*zet**3
          bet086 =bet086 +ram003*bet**2*zet**4
          bet089 =bet089 +ram012       *zet**4
          bet091 =bet091 +ram003*bet   *zet**4
          bet113 =bet113 +ram096       *zet**2
          bet114 =bet114 +ram099       *zet**2
          bet115 =bet115 +ram102       *zet**2
          bet116 =bet116 +ram111       *zet**2
          bet119 =bet119 +ram095       *zet**3
          bet120 =bet120 +ram096       *zet**3
          bet121 =bet121 +ram097       *zet**3
          bet122 =bet122 +ram098       *zet**3
          bet123 =bet123 +ram099       *zet**3
          bet124 =bet124 +ram100       *zet**3
          bet125 =bet125 +ram101       *zet**3
          bet126 =bet126 +ram103       *zet**3
          bet127 =bet127 +ram104       *zet**3
          bet129 =bet129 +ram096*bet   *zet**3
          bet130 =bet130 +ram099*bet   *zet**3
          bet131 =bet131 +ram102*bet   *zet**3
          bet133 =bet133 +ram096       *zet**3
          bet134 =bet134 +ram099       *zet**3
          bet135 =bet135 +ram102       *zet**3
          bet138 =bet138 +ram095       *zet**4
          bet139 =bet139 +ram096       *zet**4
          bet140 =bet140 +ram097       *zet**4
          bet141 =bet141 +ram098       *zet**4
          bet142 =bet142 +ram100       *zet**4
          bet143 =bet143 +ram101       *zet**4
          bet145 =bet145 +ram096*bet   *zet**4
          bet146 =bet146 +ram099*bet   *zet**4
          bet148 =bet148 +ram096       *zet**4
          bet149 =bet149 +ram099       *zet**4
          bet152 =bet152 +ram095       *zet**5
          bet153 =bet153 +ram097       *zet**5
          bet154 =bet154 +ram098       *zet**5
          bet156 =bet156 +ram096*bet   *zet**5
          bet158 =bet158 +ram096       *zet**5
          bet160 =bet160 +ram095       *zet**6
          bet197 =bet197 +ram170       *zet**2
          bet198 =bet198 +ram164       *zet**2
          bet199 =bet199 +ram173       *zet**2
          bet200 =bet200 +ram174       *zet**2
          bet201 =bet201 +ram165       *zet**2
          bet202 =bet202 +ram176       *zet**2
          bet203 =bet203 +ram177       *zet**2
          bet204 =bet204 +ram189       *zet**2
          bet205 =bet205 +ram190       *zet**2
          bet209 =bet209 +ram170       *zet**3
          bet210 =bet210 +ram172       *zet**3
          bet211 =bet211 +ram173       *zet**3
          bet212 =bet212 +ram174       *zet**3
          bet213 =bet213 +ram175       *zet**3
          bet214 =bet214 +ram178       *zet**3
          bet216 =bet216 +ram164*bet   *zet**2
          bet217 =bet217 +ram165*bet   *zet**2
          bet218 =bet218 +ram166*bet   *zet**2
          bet221 =bet221 +ram170*bet   *zet**3
          bet222 =bet222 +ram164*bet   *zet**3
          bet223 =bet223 +ram173*bet   *zet**3
          bet224 =bet224 +ram174*bet   *zet**3
          bet225 =bet225 +ram176*bet   *zet**3
          bet226 =bet226 +ram177*bet   *zet**3
          bet228 =bet228 +ram164*bet**2*zet**3
          bet229 =bet229 +ram165*bet**2*zet**3
          bet232 =bet232 +ram170       *zet**3
          bet233 =bet233 +ram164       *zet**3
          bet234 =bet234 +ram173       *zet**3
          bet235 =bet235 +ram174       *zet**3
          bet236 =bet236 +ram176       *zet**3
          bet237 =bet237 +ram177       *zet**3
          bet240 =bet240 +ram170       *zet**4
          bet241 =bet241 +ram172       *zet**4
          bet242 =bet242 +ram175       *zet**4
          bet244 =bet244 +ram164*bet   *zet**3
          bet245 =bet245 +ram165*bet   *zet**3
          bet248 =bet248 +ram170*bet   *zet**4
          bet249 =bet249 +ram173*bet   *zet**4
          bet250 =bet250 +ram174*bet   *zet**4
          bet252 =bet252 +ram164*bet**2*zet**4
          bet255 =bet255 +ram170       *zet**4
          bet256 =bet256 +ram173       *zet**4
          bet257 =bet257 +ram174       *zet**4
          bet259 =bet259 +ram172       *zet**5
          bet261 =bet261 +ram164*bet   *zet**4
          bet263 =bet263 +ram170*bet   *zet**5
          bet266 =bet266 +ram170       *zet**5
          bet287 =bet287 +ram270       *zet   
          bet288 =bet288 +ram276       *zet   
          bet289 =bet289 +ram277       *zet   
          bet291 =bet291 +ram271       *zet**2
          bet294 =bet294 +ram270*bet   *zet**2
          bet295 =bet295 +ram271*bet   *zet**2
          bet296 =bet296 +ram272*bet   *zet**2
          bet299 =bet299 +ram270*bet**2*zet**2
          bet300 =bet300 +ram276*bet**2*zet**2
          bet301 =bet301 +ram277*bet**2*zet**2
          bet303 =bet303 +ram271*bet**2*zet**3
          bet305 =bet305 +ram270       *zet**2
          bet307 =bet307 +ram271*bet   *zet**3
          bet309 =bet309 +ram270*bet**2*zet**3
          bet317 =bet317 +ram311       *zet**3
          bet318 =bet318 +ram312       *zet**3
          bet319 =bet319 +ram313       *zet**3
          bet320 =bet320 +ram314       *zet**3
          bet322 =bet322 +ram311       *zet**4
          bet323 =bet323 +ram312       *zet**4
          bet324 =bet324 +ram313       *zet**4
          bet326 =bet326 +ram311       *zet**5
          bet327 =bet327 +ram312       *zet**5
          bet329 =bet329 +ram311       *zet**6
          bet334 =bet334 +ram332       *zet   
          bet336 =bet336 +ram332*bet**2*zet**2
       enddo
       dppp( 34)=dppp( 34)+bet034*alp   
       dppp( 37)=dppp( 37)+bet037*alp   
       dppp( 38)=dppp( 38)+bet038*alp   
       dppp( 39)=dppp( 39)+bet039*alp   
       dppp( 46)=dppp( 46)+bet046*alp   
       dppp( 48)=dppp( 48)+bet048*alp   
       dppp( 49)=dppp( 49)+bet049*alp   
       dppp( 56)=dppp( 56)+bet056*alp   
       dppp( 57)=dppp( 57)+bet057*alp   
       dppp( 61)=dppp( 61)+bet061*alp   
       dppp( 64)=dppp( 64)+bet064*alp   
       dppp( 65)=dppp( 65)+bet065*alp   
       dppp( 69)=dppp( 69)+bet038*alp**2
       dppp( 71)=dppp( 71)+bet046*alp**2
       dppp( 72)=dppp( 72)+bet072*alp**2
       dppp( 76)=dppp( 76)+bet076*alp**2
       dppp( 77)=dppp( 77)+bet057*alp**2
       dppp( 81)=dppp( 81)+bet081*alp**2
       dppp( 83)=dppp( 83)+bet064*alp**2
       dppp( 85)=dppp( 85)+bet085*alp**2
       dppp( 87)=dppp( 87)+bet087*alp**3
       dppp( 88)=dppp( 88)+bet088*alp**3
       dppp( 90)=dppp( 90)+bet090*alp**3
       dppp( 92)=dppp( 92)+bet092*alp**3
       dppp(112)=dppp(112)+bet112*alp   
       dppp(117)=dppp(117)+bet117*alp   
       dppp(118)=dppp(118)+bet118*alp   
       dppp(128)=dppp(128)+bet128*alp   
       dppp(132)=dppp(132)+bet117*alp**2
       dppp(136)=dppp(136)+bet136*alp**2
       dppp(137)=dppp(137)+bet137*alp**2
       dppp(144)=dppp(144)+bet144*alp**2
       dppp(147)=dppp(147)+bet136*alp**3
       dppp(150)=dppp(150)+bet150*alp**3
       dppp(151)=dppp(151)+bet151*alp**3
       dppp(155)=dppp(155)+bet155*alp**3
       dppp(157)=dppp(157)+bet150*alp**4
       dppp(159)=dppp(159)+bet159*alp**4
       dppp(161)=dppp(161)+bet161*alp**4
       dppp(162)=dppp(162)+bet162*alp**5
       dppp(195)=dppp(195)+bet195*alp   
       dppp(196)=dppp(196)+bet196*alp   
       dppp(206)=dppp(206)+bet206*alp   
       dppp(207)=dppp(207)+bet207*alp   
       dppp(208)=dppp(208)+bet208*alp   
       dppp(215)=dppp(215)+bet215*alp   
       dppp(219)=dppp(219)+bet219*alp   
       dppp(220)=dppp(220)+bet220*alp   
       dppp(227)=dppp(227)+bet227*alp   
       dppp(230)=dppp(230)+bet207*alp**2
       dppp(231)=dppp(231)+bet208*alp**2
       dppp(238)=dppp(238)+bet238*alp**2
       dppp(239)=dppp(239)+bet239*alp**2
       dppp(243)=dppp(243)+bet219*alp**2
       dppp(246)=dppp(246)+bet246*alp**2
       dppp(247)=dppp(247)+bet247*alp**2
       dppp(251)=dppp(251)+bet251*alp**2
       dppp(253)=dppp(253)+bet253*alp**3
       dppp(254)=dppp(254)+bet239*alp**3
       dppp(258)=dppp(258)+bet258*alp**3
       dppp(260)=dppp(260)+bet246*alp**3
       dppp(262)=dppp(262)+bet262*alp**3
       dppp(264)=dppp(264)+bet264*alp**3
       dppp(265)=dppp(265)+bet265*alp**4
       dppp(267)=dppp(267)+bet267*alp**4
       dppp(285)=dppp(285)+bet285*alp   
       dppp(286)=dppp(286)+bet286*alp   
       dppp(290)=dppp(290)+bet290*alp   
       dppp(292)=dppp(292)+bet292*alp   
       dppp(293)=dppp(293)+bet293*alp   
       dppp(297)=dppp(297)+bet297*alp   
       dppp(298)=dppp(298)+bet298*alp   
       dppp(302)=dppp(302)+bet302*alp   
       dppp(304)=dppp(304)+bet304*alp**2
       dppp(306)=dppp(306)+bet306*alp**2
       dppp(308)=dppp(308)+bet308*alp**2
       dppp(316)=dppp(316)+bet316*alp   
       dppp(321)=dppp(321)+bet321*alp**2
       dppp(325)=dppp(325)+bet325*alp**3
       dppp(328)=dppp(328)+bet328*alp**4
       dppp(330)=dppp(330)+bet330*alp**5
       dppp(333)=dppp(333)+bet333*alp   
       dppp(335)=dppp(335)+bet335*alp   

       dppp( 35)=dppp( 35)+bet035*alp   
       dppp( 36)=dppp( 36)+bet036*alp   
       dppp( 40)=dppp( 40)+bet040*alp   
       dppp( 41)=dppp( 41)+bet041*alp   
       dppp( 42)=dppp( 42)+bet042*alp   
       dppp( 43)=dppp( 43)+bet043*alp   
       dppp( 44)=dppp( 44)+bet044*alp   
       dppp( 45)=dppp( 45)+bet045*alp   
       dppp( 47)=dppp( 47)+bet047*alp   
       dppp( 50)=dppp( 50)+bet050*alp   
       dppp( 51)=dppp( 51)+bet051*alp   
       dppp( 52)=dppp( 52)+bet052*alp   
       dppp( 53)=dppp( 53)+bet053*alp   
       dppp( 54)=dppp( 54)+bet054*alp   
       dppp( 55)=dppp( 55)+bet055*alp   
       dppp( 58)=dppp( 58)+bet058*alp   
       dppp( 59)=dppp( 59)+bet059*alp   
       dppp( 60)=dppp( 60)+bet060*alp   
       dppp( 62)=dppp( 62)+bet062*alp   
       dppp( 63)=dppp( 63)+bet063*alp   
       dppp( 66)=dppp( 66)+bet066*alp   
       dppp( 67)=dppp( 67)+bet067*alp   
       dppp( 68)=dppp( 68)+bet068*alp   
       dppp( 70)=dppp( 70)+bet070*alp**2
       dppp( 73)=dppp( 73)+bet073*alp**2
       dppp( 74)=dppp( 74)+bet074*alp**2
       dppp( 75)=dppp( 75)+bet075*alp**2
       dppp( 78)=dppp( 78)+bet078*alp**2
       dppp( 79)=dppp( 79)+bet079*alp**2
       dppp( 80)=dppp( 80)+bet080*alp**2
       dppp( 82)=dppp( 82)+bet082*alp**2
       dppp( 84)=dppp( 84)+bet084*alp**2
       dppp( 86)=dppp( 86)+bet086*alp**2
       dppp( 89)=dppp( 89)+bet089*alp**3
       dppp( 91)=dppp( 91)+bet091*alp**3
       dppp(113)=dppp(113)+bet113*alp   
       dppp(114)=dppp(114)+bet114*alp   
       dppp(115)=dppp(115)+bet115*alp   
       dppp(116)=dppp(116)+bet116*alp   
       dppp(119)=dppp(119)+bet119*alp   
       dppp(120)=dppp(120)+bet120*alp   
       dppp(121)=dppp(121)+bet121*alp   
       dppp(122)=dppp(122)+bet122*alp   
       dppp(123)=dppp(123)+bet123*alp   
       dppp(124)=dppp(124)+bet124*alp   
       dppp(125)=dppp(125)+bet125*alp   
       dppp(126)=dppp(126)+bet126*alp   
       dppp(127)=dppp(127)+bet127*alp   
       dppp(129)=dppp(129)+bet129*alp   
       dppp(130)=dppp(130)+bet130*alp   
       dppp(131)=dppp(131)+bet131*alp   
       dppp(133)=dppp(133)+bet133*alp**2
       dppp(134)=dppp(134)+bet134*alp**2
       dppp(135)=dppp(135)+bet135*alp**2
       dppp(138)=dppp(138)+bet138*alp**2
       dppp(139)=dppp(139)+bet139*alp**2
       dppp(140)=dppp(140)+bet140*alp**2
       dppp(141)=dppp(141)+bet141*alp**2
       dppp(142)=dppp(142)+bet142*alp**2
       dppp(143)=dppp(143)+bet143*alp**2
       dppp(145)=dppp(145)+bet145*alp**2
       dppp(146)=dppp(146)+bet146*alp**2
       dppp(148)=dppp(148)+bet148*alp**3
       dppp(149)=dppp(149)+bet149*alp**3
       dppp(152)=dppp(152)+bet152*alp**3
       dppp(153)=dppp(153)+bet153*alp**3
       dppp(154)=dppp(154)+bet154*alp**3
       dppp(156)=dppp(156)+bet156*alp**3
       dppp(158)=dppp(158)+bet158*alp**4
       dppp(160)=dppp(160)+bet160*alp**4
       dppp(197)=dppp(197)+bet197*alp   
       dppp(198)=dppp(198)+bet198*alp   
       dppp(199)=dppp(199)+bet199*alp   
       dppp(200)=dppp(200)+bet200*alp   
       dppp(201)=dppp(201)+bet201*alp   
       dppp(202)=dppp(202)+bet202*alp   
       dppp(203)=dppp(203)+bet203*alp   
       dppp(204)=dppp(204)+bet204*alp   
       dppp(205)=dppp(205)+bet205*alp   
       dppp(209)=dppp(209)+bet209*alp   
       dppp(210)=dppp(210)+bet210*alp   
       dppp(211)=dppp(211)+bet211*alp   
       dppp(212)=dppp(212)+bet212*alp   
       dppp(213)=dppp(213)+bet213*alp   
       dppp(214)=dppp(214)+bet214*alp   
       dppp(216)=dppp(216)+bet216*alp   
       dppp(217)=dppp(217)+bet217*alp   
       dppp(218)=dppp(218)+bet218*alp   
       dppp(221)=dppp(221)+bet221*alp   
       dppp(222)=dppp(222)+bet222*alp   
       dppp(223)=dppp(223)+bet223*alp   
       dppp(224)=dppp(224)+bet224*alp   
       dppp(225)=dppp(225)+bet225*alp   
       dppp(226)=dppp(226)+bet226*alp   
       dppp(228)=dppp(228)+bet228*alp   
       dppp(229)=dppp(229)+bet229*alp   
       dppp(232)=dppp(232)+bet232*alp**2
       dppp(233)=dppp(233)+bet233*alp**2
       dppp(234)=dppp(234)+bet234*alp**2
       dppp(235)=dppp(235)+bet235*alp**2
       dppp(236)=dppp(236)+bet236*alp**2
       dppp(237)=dppp(237)+bet237*alp**2
       dppp(240)=dppp(240)+bet240*alp**2
       dppp(241)=dppp(241)+bet241*alp**2
       dppp(242)=dppp(242)+bet242*alp**2
       dppp(244)=dppp(244)+bet244*alp**2
       dppp(245)=dppp(245)+bet245*alp**2
       dppp(248)=dppp(248)+bet248*alp**2
       dppp(249)=dppp(249)+bet249*alp**2
       dppp(250)=dppp(250)+bet250*alp**2
       dppp(252)=dppp(252)+bet252*alp**2
       dppp(255)=dppp(255)+bet255*alp**3
       dppp(256)=dppp(256)+bet256*alp**3
       dppp(257)=dppp(257)+bet257*alp**3
       dppp(259)=dppp(259)+bet259*alp**3
       dppp(261)=dppp(261)+bet261*alp**3
       dppp(263)=dppp(263)+bet263*alp**3
       dppp(266)=dppp(266)+bet266*alp**4
       dppp(287)=dppp(287)+bet287*alp   
       dppp(288)=dppp(288)+bet288*alp   
       dppp(289)=dppp(289)+bet289*alp   
       dppp(291)=dppp(291)+bet291*alp   
       dppp(294)=dppp(294)+bet294*alp   
       dppp(295)=dppp(295)+bet295*alp   
       dppp(296)=dppp(296)+bet296*alp   
       dppp(299)=dppp(299)+bet299*alp   
       dppp(300)=dppp(300)+bet300*alp   
       dppp(301)=dppp(301)+bet301*alp   
       dppp(303)=dppp(303)+bet303*alp   
       dppp(305)=dppp(305)+bet305*alp**2
       dppp(307)=dppp(307)+bet307*alp**2
       dppp(309)=dppp(309)+bet309*alp**2
       dppp(317)=dppp(317)+bet317*alp   
       dppp(318)=dppp(318)+bet318*alp   
       dppp(319)=dppp(319)+bet319*alp   
       dppp(320)=dppp(320)+bet320*alp   
       dppp(322)=dppp(322)+bet322*alp**2
       dppp(323)=dppp(323)+bet323*alp**2
       dppp(324)=dppp(324)+bet324*alp**2
       dppp(326)=dppp(326)+bet326*alp**3
       dppp(327)=dppp(327)+bet327*alp**3
       dppp(329)=dppp(329)+bet329*alp**4
       dppp(334)=dppp(334)+bet334*alp   
       dppp(336)=dppp(336)+bet336*alp   
       enddo
#ifdef PL_OMP_ERI
!$omp end do
!$omp end parallel
#endif
       end
C** IKDPDS ===========================================================
C          CALUCATE KERNELS FOR DPPP TYPE
C====================================================================

      subroutine ikdpds(
     &           dpds ,
     &           fdat ,  edat  ,
     &           yp   ,  yq    ,
     &           uu   ,  uv    ,
     &           z    ,  zetpq , 
     &           nasp ,  mxsa  ,  mxprm,  ntprm,  
     &           iasp ,  jasp  ,  kasp ,  lasp ,
     &           ishel,  jshel ,  kshel,  lshel     )

      implicit double precision  (a-h,o-z)
      dimension fdat(31,2001)
      dimension edat(2001)
      dimension zetpq(mxprm,mxprm,mxsa,nasp,mxsa,nasp)
      dimension yp(3,mxprm,mxprm),yq(3,mxprm,mxprm)
      dimension uu(mxprm,mxprm),uv(mxprm,mxprm)
      dimension z(mxprm,mxsa,nasp)
      dimension ntprm(mxsa,nasp)
      dimension dpds(284)
 
      real(8),parameter::pi    = 3.14159265358979323846d+00
      real(8),parameter::tf(25) = (/
     &                33.0,37.0,41.0,43.0,46.0,49.0,
     &                51.0,54.0,56.0,58.0,61.0,63.0,
     &                66.0,68.0,70.0,72.0,74.0,80.0,
     &                85.0,90.0,95.0,95.0,98.0,99.0,
     &                99.0                           /)

      real(8),parameter::ga0 = 1.0d+00
      real(8),parameter::ga1 = 1.000 000 000 000 005 298d+00
      real(8),parameter::ga2 = 0.5d+00
      real(8),parameter::ga3 = 0.166 666 666 598 849 826d+00
      real(8),parameter::ga4 = 0.416 666 666 666 666 667d-01
      real(8),parameter::ga5 = 0.833 355 034 722 222 222d-02
      real(8),parameter::ga6 = 0.138 888 888 888 888 889d-02

      real(8),parameter::rminv(5) = (/
     &   1.0d+00,  0.333333333333333333d+00,  0.20d+00,
     &   0.14285714285714286d+00,  0.11111111111111111d+00 /)

#ifdef PL_SIMD_ERI
      dimension dmx(6,mxprm), val(2,mxprm)
#endif

      DO IJK=1,284
         DPDS(IJK)=0.0d0
      ENDDO

#ifdef PL_OMP_ERI
!$omp parallel reduction(+:dpds)
!$omp& default(private)
!$omp& shared(fdat,edat,
!$omp&        zetpq,ntprm,z,yp,yq,uu,uv,
!$omp&        ishel,jshel,kshel,lshel,iasp,jasp,kasp,lasp)
#endif
      igp = ntprm(ishel,iasp)
      jgp = ntprm(jshel,jasp)
      kgp = ntprm(kshel,kasp)
      lgp = ntprm(lshel,lasp)

      pi2 = 2.0d0*pi

#ifdef PL_OMP_ERI
!$omp do
#endif
      DO I = 1, IGP
         alp = z(i,ishel,iasp)
         bet029=0.0d0
         bet030=0.0d0
         bet031=0.0d0
         bet032=0.0d0
         bet033=0.0d0
         bet034=0.0d0
         bet035=0.0d0
         bet036=0.0d0
         bet037=0.0d0
         bet038=0.0d0
         bet039=0.0d0
         bet040=0.0d0
         bet041=0.0d0
         bet042=0.0d0
         bet043=0.0d0
         bet044=0.0d0
         bet045=0.0d0
         bet046=0.0d0
         bet047=0.0d0
         bet048=0.0d0
         bet049=0.0d0
         bet050=0.0d0
         bet051=0.0d0
         bet052=0.0d0
         bet053=0.0d0
         bet054=0.0d0
         bet055=0.0d0
         bet056=0.0d0
         bet057=0.0d0
         bet058=0.0d0
         bet060=0.0d0
         bet062=0.0d0
         bet064=0.0d0
         bet065=0.0d0
         bet066=0.0d0
         bet068=0.0d0
         bet069=0.0d0
         bet070=0.0d0
         bet072=0.0d0
         bet073=0.0d0
         bet074=0.0d0
         bet075=0.0d0
         bet076=0.0d0
         bet077=0.0d0
         bet078=0.0d0
         bet094=0.0d0
         bet095=0.0d0
         bet096=0.0d0
         bet097=0.0d0
         bet098=0.0d0
         bet099=0.0d0
         bet100=0.0d0
         bet101=0.0d0
         bet102=0.0d0
         bet103=0.0d0
         bet104=0.0d0
         bet105=0.0d0
         bet106=0.0d0
         bet107=0.0d0
         bet108=0.0d0
         bet109=0.0d0
         bet110=0.0d0
         bet114=0.0d0
         bet115=0.0d0
         bet115=0.0d0
         bet116=0.0d0
         bet117=0.0d0
         bet118=0.0d0
         bet119=0.0d0
         bet120=0.0d0
         bet121=0.0d0
         bet122=0.0d0
         bet125=0.0d0
         bet126=0.0d0
         bet127=0.0d0
         bet128=0.0d0
         bet129=0.0d0
         bet130=0.0d0
         bet132=0.0d0
         bet133=0.0d0
         bet134=0.0d0
         bet135=0.0d0
         bet165=0.0d0
         bet166=0.0d0
         bet167=0.0d0
         bet168=0.0d0
         bet169=0.0d0
         bet170=0.0d0
         bet171=0.0d0
         bet172=0.0d0
         bet173=0.0d0
         bet174=0.0d0
         bet175=0.0d0
         bet176=0.0d0
         bet177=0.0d0
         bet178=0.0d0
         bet179=0.0d0
         bet180=0.0d0
         bet181=0.0d0
         bet182=0.0d0
         bet183=0.0d0
         bet184=0.0d0
         bet185=0.0d0
         bet186=0.0d0
         bet187=0.0d0
         bet188=0.0d0
         bet189=0.0d0
         bet189=0.0d0
         bet190=0.0d0
         bet191=0.0d0
         bet192=0.0d0
         bet193=0.0d0
         bet194=0.0d0
         bet197=0.0d0
         bet199=0.0d0
         bet200=0.0d0
         bet201=0.0d0
         bet202=0.0d0
         bet203=0.0d0
         bet204=0.0d0
         bet207=0.0d0
         bet208=0.0d0
         bet209=0.0d0
         bet210=0.0d0
         bet211=0.0d0
         bet212=0.0d0
         bet213=0.0d0
         bet215=0.0d0
         bet216=0.0d0
         bet217=0.0d0
         bet218=0.0d0
         bet219=0.0d0
         bet220=0.0d0
         bet221=0.0d0
         bet222=0.0d0
         bet223=0.0d0
         bet239=0.0d0
         bet240=0.0d0
         bet241=0.0d0
         bet242=0.0d0
         bet243=0.0d0
         bet244=0.0d0
         bet245=0.0d0
         bet246=0.0d0
         bet247=0.0d0
         bet248=0.0d0
         bet249=0.0d0
         bet250=0.0d0
         bet251=0.0d0
         bet252=0.0d0
         bet253=0.0d0
         bet254=0.0d0
         bet255=0.0d0
         bet256=0.0d0
         bet257=0.0d0
         bet264=0.0d0
         bet265=0.0d0
         bet266=0.0d0
         bet267=0.0d0
         bet268=0.0d0
         bet269=0.0d0
         bet270=0.0d0
         bet271=0.0d0
         bet272=0.0d0
         bet273=0.0d0
         bet274=0.0d0
         bet275=0.0d0
         bet276=0.0d0
         bet277=0.0d0
         bet278=0.0d0
         bet281=0.0d0
         bet282=0.0d0
         bet283=0.0d0
         bet284=0.0d0
         DO J = 1, JGP
            bet = z(j,jshel,jasp)
            zet = zetpq(j,i,jshel,jasp,ishel,iasp)
            px  = yp (1,j,i)
            py  = yp (2,j,i)
            pz  = yp (3,j,i)
            up  = uu(j,i)
            del001=0.0d0
            del002=0.0d0
            del003=0.0d0
            del009=0.0d0
            del079=0.0d0
            del080=0.0d0
            del136=0.0d0
            del140=0.0d0
            del142=0.0d0
            del143=0.0d0
            del224=0.0d0
            del225=0.0d0
            del226=0.0d0
            del229=0.0d0
            del258=0.0d0
            del279=0.0d0
            del280=0.0d0
            ram004=0.0d0
            ram005=0.0d0
            ram006=0.0d0
            ram010=0.0d0
            ram011=0.0d0
            ram014=0.0d0
            ram022=0.0d0
            ram023=0.0d0
            ram081=0.0d0
            ram082=0.0d0
            ram083=0.0d0
            ram084=0.0d0
            ram085=0.0d0
            ram086=0.0d0
            ram087=0.0d0
            ram088=0.0d0
            ram093=0.0d0
            ram137=0.0d0
            ram138=0.0d0
            ram139=0.0d0
            ram144=0.0d0
            ram145=0.0d0
            ram147=0.0d0
            ram148=0.0d0
            ram149=0.0d0
            ram150=0.0d0
            ram151=0.0d0
            ram152=0.0d0
            ram153=0.0d0
            ram160=0.0d0
            ram227=0.0d0
            ram228=0.0d0
            ram231=0.0d0
            ram259=0.0d0
            ram260=0.0d0
            ram261=0.0d0
            ram262=0.0d0
            ram263=0.0d0
            DO K = 1, KGP
               ram = z(k,kshel,kasp)
               del004=0.0d0
               del005=0.0d0
               del010=0.0d0
               del011=0.0d0
               del081=0.0d0
               del082=0.0d0
               del083=0.0d0
               del137=0.0d0
               del144=0.0d0
               del145=0.0d0
               del147=0.0d0
               del148=0.0d0
               del227=0.0d0
               del228=0.0d0
               del231=0.0d0
               del259=0.0d0
               del006=0.0d0
               del014=0.0d0
               del023=0.0d0
               del084=0.0d0
               del085=0.0d0
               del138=0.0d0
               del150=0.0d0
               del151=0.0d0
               del260=0.0d0
               del086=0.0d0
               del087=0.0d0
               del153=0.0d0
               del139=0.0d0
               del160=0.0d0
               del261=0.0d0
               del088=0.0d0
               del093=0.0d0
               del262=0.0d0
               del263=0.0d0
               DO L = 1, LGP
                  del = z(l,lshel,lasp)
                  eta = zetpq(l,k,lshel,lasp,kshel,kasp)
                  qpx =  yq(1,l,k) - px
                  qpy =  yq(2,l,k) - py
                  qpz =  yq(3,l,k) - pz
                  vq  = uv(l,k)
            
                  ROO2 = 1.0d0 / (ZET + ETA)
                  qp2  = qpx*qpx + qpy*qpy + qpz*qpz
                  T    = 0.5d0* ROO2*qp2

                  IF (T. LE .TF(6)) THEN
                  
                     IT = IDINT((T+0.025D+00)*20.0D+00)
                     DT = 0.05D+00*IT - T
                     IT = IT + 1
                     DT4 = DT*0.25D+00
                     DT5 = DT*0.2D+00
                     DT6 = DT*0.166666666666666667D+00
                     DT2 = DT4 + DT4
                     DT3 = DT6 + DT6
                     F5   = (((((FDAT(12,IT)*DT6   + FDAT(11,IT))*DT5
     &                 + FDAT(10,IT))*DT4  + FDAT(9,IT))*DT3
     &                 + FDAT(8,IT))*DT2  + FDAT(7,IT))*DT
     &                 + FDAT(6,IT)
                    
                     EED  = (((((GA6*DT + GA5)*DT + GA4)*DT
     &                 + GA3)*DT + GA2)*DT + GA1)*DT + GA0
                     EE   = EDAT(IT)*EED
                    
                     T2   = T*2.0d0
                     F4   = (T2*F5 + EE)*RMINV(5)
                     F3   = (T2*F4 + EE)*RMINV(4)
                     F2   = (T2*F3 + EE)*RMINV(3)
                     F1   = (T2*F2 + EE)*RMINV(2)
                     F0   =  T2*F1 + EE
                     FPP =  UP * VQ * DSQRT(ROO2)
                    
                     DM0 = FPP*F0
                     DM1 = FPP*F1 *  ROO2
                     DM2 = FPP*F2 *( ROO2)**2
                     DM3 = FPP*F3 *( ROO2)**3
                     DM4 = FPP*F4 *( ROO2)**4
                     DM5 = FPP*F5 *( ROO2)**5

                  ELSE 

                     qinv = 1.0d0/qp2
                     dm0=  up*vq*dsqrt( pi2*qinv)*0.5d0
                     dm1 =        qinv *  dm0
                     dm2 =  3.0d0*qinv *  dm1
                     dm3 =  5.0d0*qinv *  dm2
                     dm4 =  7.0d0*qinv *  dm3
                     dm5 =  9.0d0*qinv *  dm4

                  ENDIF
 
#ifdef PL_SIMD_ERI
                  val(1,l) = eta
                  val(2,l) = del

                  dmx(1,l) = dm0
                  dmx(2,l) = dm1
                  dmx(3,l) = dm2
                  dmx(4,l) = dm3
                  dmx(5,l) = dm4
                  dmx(6,l) = dm5
               enddo
               
               do l = 1, lgp
                  eta = val(1,l)
                  del = val(2,l)

                  dm0 = dmx(1,l)
                  dm1 = dmx(2,l)
                  dm2 = dmx(3,l)
                  dm3 = dmx(4,l)
                  dm4 = dmx(5,l)
                  dm5 = dmx(6,l)
#endif

!** KERNEL DPDS =====================================================
!           CALUCLATE & CONTRACT KERNELS FOR DPDS TYPE
!====================================================================
!  
!ddd
                  del001 =del001 +dm2             
                  del010 =del010 +dm2       *eta   
                  del002 =del002 +dm2       *eta   
                  del004 =del004 +dm2       *eta**2
                  del014 =del014 +dm2       *eta**3
                  del003 =del003 +dm2*del   *eta   
                  del005 =del005 +dm2*del   *eta**2
                  del006 =del006 +dm2*del   *eta**3
                  del009 =del009 +dm2*del**2*eta   
                  del011 =del011 +dm2*del**2*eta**2
                  del023 =del023 +dm2*del**2*eta**3
!=======================================
                  del079 =del079 +dm4       *eta   
                  del081 =del081 +dm4       *eta   
                  del082 =del082 +dm4       *eta**2
                  del084 =del084 +dm4       *eta**3
                  del086 =del086 +dm4       *eta**4
                  del093 =del093 +dm4       *eta**5
                  del080 =del080 +dm4*del   *eta   
                  del083 =del083 +dm4*del   *eta**2
                  del085 =del085 +dm4*del   *eta**3
                  del087 =del087 +dm4*del   *eta**4
                  del088 =del088 +dm4*del   *eta**5
!=======================================
                  del144 =del144 +dm3
                  del140 =del140 +dm3
                  del136 =del136 +dm3       *eta   
                  del145 =del145 +dm3       *eta   
                  del137 =del137 +dm3       *eta**2
                  del138 =del138 +dm3       *eta**3
                  del139 =del139 +dm3       *eta**4
                  del142 =del142 +dm3*del   *eta   
                  del147 =del147 +dm3*del   *eta**2
                  del150 =del150 +dm3*del   *eta**3
                  del160 =del160 +dm3*del   *eta**4
                  del143 =del143 +dm3*del**2*eta   
                  del148 =del148 +dm3*del**2*eta**2
                  del151 =del151 +dm3*del**2*eta**3
                  del153 =del153 +dm3*del**2*eta**4
!=======================================
                  del224 =del224 +dm1             
                  del227 =del227 +dm1       *eta   
                  del229 =del229 +dm1       *eta   
                  del225 =del225 +dm1*del   *eta   
                  del231 =del231 +dm1*del   *eta**2
                  del226 =del226 +dm1*del**2*eta   
                  del228 =del228 +dm1*del**2*eta**2
!=======================================
                  del258 =del258 +dm5       *eta   
                  del259 =del259 +dm5       *eta**2
                  del260 =del260 +dm5       *eta**3
                  del261 =del261 +dm5       *eta**4
                  del262 =del262 +dm5       *eta**5
                  del263 =del263 +dm5       *eta**6
!=======================================
                  del279 =del279 +dm0             
                  del280 =del280 +dm0*del**2*eta   
               enddo
               ram004 =ram004 +del004*ram   
               ram005 =ram005 +del005*ram   
               ram010 =ram010 +del010*ram   
               ram011 =ram011 +del011*ram   
               ram081 =ram081 +del081*ram   
               ram082 =ram082 +del082*ram   
               ram083 =ram083 +del083*ram   
               ram137 =ram137 +del137*ram   
               ram144 =ram144 +del144*ram   
               ram145 =ram145 +del145*ram   
               ram147 =ram147 +del147*ram   
               ram148 =ram148 +del148*ram   
               ram227 =ram227 +del227*ram   
               ram228 =ram228 +del228*ram   
               ram231 =ram231 +del231*ram   
               ram259 =ram259 +del259*ram   
               ram006 =ram006 +del006*ram**2
               ram014 =ram014 +del014*ram**2
               ram022 =ram022 +del004*ram**2
               ram023 =ram023 +del023*ram**2
               ram084 =ram084 +del084*ram**2
               ram085 =ram085 +del085*ram**2
               ram138 =ram138 +del138*ram**2
               ram149 =ram149 +del137*ram**2
               ram150 =ram150 +del150*ram**2
               ram151 =ram151 +del151*ram**2
               ram260 =ram260 +del260*ram**2
               ram086 =ram086 +del086*ram**3
               ram087 =ram087 +del087*ram**3
               ram152 =ram152 +del138*ram**3
               ram153 =ram153 +del153*ram**3
               ram139 =ram139 +del139*ram**3
               ram160 =ram160 +del160*ram**3
               ram261 =ram261 +del261*ram**3
               ram088 =ram088 +del088*ram**4
               ram093 =ram093 +del093*ram**4
               ram262 =ram262 +del262*ram**4
               ram263 =ram263 +del263*ram**5
            enddo
            dpds(  1)=dpds(  1)+del001              *zet   
            dpds(  2)=dpds(  2)+del002              *zet   
            dpds(  3)=dpds(  3)+del003              *zet   
            dpds(  7)=dpds(  7)+del001              *zet**2
            dpds(  8)=dpds(  8)+del003              *zet**2
            dpds(  9)=dpds(  9)+del009              *zet**2
            dpds( 12)=dpds( 12)+del002       *bet   *zet   
            dpds( 15)=dpds( 15)+del001       *bet   *zet**2
            dpds( 16)=dpds( 16)+del002       *bet   *zet**2
            dpds( 17)=dpds( 17)+del003       *bet   *zet**2
            dpds( 18)=dpds( 18)+del009       *bet   *zet**2
            dpds( 24)=dpds( 24)+del002       *bet**2*zet**2
            dpds( 25)=dpds( 25)+del003       *bet**2*zet**2
            dpds( 79)=dpds( 79)+del079              *zet**2
            dpds( 80)=dpds( 80)+del080              *zet**2
            dpds( 89)=dpds( 89)+del079       *bet   *zet**2
            dpds(136)=dpds(136)+del136              *zet   
            dpds(140)=dpds(140)+del140              *zet**2
            dpds(141)=dpds(141)+del136              *zet**2
            dpds(142)=dpds(142)+del142              *zet**2
            dpds(143)=dpds(143)+del143              *zet**2
            dpds(154)=dpds(154)+del136       *bet   *zet**2
            dpds(155)=dpds(155)+del142       *bet   *zet**2
            dpds(161)=dpds(161)+del136       *bet**2*zet**2
            dpds(224)=dpds(224)+del224              *zet   
            dpds(225)=dpds(225)+del225              *zet   
            dpds(226)=dpds(226)+del226              *zet   
            dpds(229)=dpds(229)+del229       *bet   *zet   
            dpds(230)=dpds(230)+del225       *bet   *zet   
            dpds(232)=dpds(232)+del224       *bet   *zet**2
            dpds(233)=dpds(233)+del226       *bet   *zet**2
            dpds(234)=dpds(234)+del224       *bet**2*zet**2
            dpds(235)=dpds(235)+del225       *bet**2*zet**2
            dpds(236)=dpds(236)+del226       *bet**2*zet**2
            dpds(258)=dpds(258)+del258              *zet**2
            dpds(279)=dpds(279)+del279       *bet   *zet   
            dpds(280)=dpds(280)+del280       *bet   *zet   
      
            bet029 =bet029+del002       *zet   
            bet239 =bet239+del229       *zet   
            bet240 =bet240+del225       *zet   
            bet281 =bet281+del279       *zet   
            bet282 =bet282+del280       *zet   
      
            bet032 =bet032+del001       *zet**2
            bet033 =bet033+del002       *zet**2
            bet034 =bet034+del003       *zet**2
            bet035 =bet035+del009       *zet**2
            bet094 =bet094+del079       *zet**2
            bet165 =bet165+del136       *zet**2
            bet166 =bet166+del142       *zet**2
            bet242 =bet242+del224       *zet**2
            bet243 =bet243+del226       *zet**2
            bet254 =bet254+del225       *zet**2
      
            bet041 =bet041+del001       *zet**3
            bet062 =bet062+del003       *zet**3
            bet042 =bet042+del009       *zet**3
            bet074 =bet074+del002       *zet**3
            bet100 =bet100+del080       *zet**3
            bet099 =bet099+del079       *zet**3
            bet172 =bet172+del140       *zet**3
            bet173 =bet173+del136       *zet**3
            bet175 =bet175+del143       *zet**3
            bet174 =bet174+del142       *zet**3
            bet264 =bet264+del258       *zet**3
            
            bet075 =bet075+del001       *zet**4
            bet076 =bet076+del009       *zet**4
            bet115 =bet115+del079       *zet**4
            bet116 =bet116+del080       *zet**4
            bet200 =bet200+del140       *zet**4
            bet201 =bet201+del142       *zet**4
            bet202 =bet202+del143       *zet**4
            bet213 =bet213+del136       *zet**4
            bet269 =bet269+del258       *zet**4

            bet126 =bet126+del079       *zet**5
            bet127 =bet127+del080       *zet**5
            bet216 =bet216+del140       *zet**5
            bet217 =bet217+del143       *zet**5
            bet222 =bet222+del142       *zet**5
            bet273 =bet273+del258       *zet**5
      
            bet133 =bet133+del080       *zet**6
            bet135 =bet135+del079       *zet**6
            bet276 =bet276+del258       *zet**6
      
            bet278 =bet278+del258       *zet**7
      
            bet043 =bet043+del002*bet   *zet**2
            bet044 =bet044+del003*bet   *zet**2
            bet182 =bet182+del136*bet   *zet**2
            bet244 =bet244+del224*bet   *zet**2
            bet245 =bet245+del225*bet   *zet**2
            bet246 =bet246+del226*bet   *zet**2
      
            bet048 =bet048+del001*bet   *zet**3
            bet049 =bet049+del003*bet   *zet**3
            bet050 =bet050+del009*bet   *zet**3
            bet066 =bet066+del002*bet   *zet**3
            bet107 =bet107+del079*bet   *zet**3
            bet187 =bet187+del142*bet   *zet**3
            bet186 =bet186+del136*bet   *zet**3
            bet255 =bet255+del224*bet   *zet**3
            bet256 =bet256+del226*bet   *zet**3
      
            bet069 =bet069+del001*bet   *zet**4
            bet070 =bet070+del009*bet   *zet**4
            bet077 =bet077+del003*bet   *zet**4
            bet120 =bet120+del079*bet   *zet**4
            bet208 =bet208+del136*bet   *zet**4
            bet209 =bet209+del142*bet   *zet**4
            bet218 =bet218+del136*bet   *zet**4
      
            bet129 =bet129+del079*bet   *zet**5
            bet220 =bet220+del142*bet   *zet**5
            bet223 =bet223+del136*bet   *zet**5
      
            bet134 =bet134+del079*bet   *zet**6
      
            bet053 =bet053+del002*bet**2*zet**2
            bet249 =bet249+del229*bet**2*zet**2
            bet250 =bet250+del225*bet**2*zet**2
            bet283 =bet283+del279*bet**2*zet**2
            bet284 =bet284+del280*bet**2*zet**2
      
            bet056 =bet056+del002*bet**2*zet**3
            bet057 =bet057+del003*bet**2*zet**3
            bet192 =bet192+del136*bet**2*zet**3
            bet252 =bet252+del224*bet**2*zet**3
            bet253 =bet253+del226*bet**2*zet**3
            bet257 =bet257+del225*bet**2*zet**3
      
            bet073 =bet073+del003*bet**2*zet**4
            bet078 =bet078+del002*bet**2*zet**4
            bet211 =bet211+del136*bet**2*zet**4
      
            bet221 =bet221+del136*bet**2*zet**5
      
            dpds(  4)=dpds(  4)+ram004              *zet   
            dpds(  5)=dpds(  5)+ram005              *zet   
            dpds(  6)=dpds(  6)+ram006              *zet   
            dpds( 10)=dpds( 10)+ram010              *zet**2
            dpds( 11)=dpds( 11)+ram011              *zet**2
            dpds( 13)=dpds( 13)+ram004       *bet   *zet   
            dpds( 14)=dpds( 14)+ram014       *bet   *zet   
            dpds( 19)=dpds( 19)+ram010       *bet   *zet**2
            dpds( 20)=dpds( 20)+ram005       *bet   *zet**2
            dpds( 21)=dpds( 21)+ram011       *bet   *zet**2
            dpds( 22)=dpds( 22)+ram022       *bet   *zet**2
            dpds( 23)=dpds( 23)+ram023       *bet   *zet**2
            dpds( 26)=dpds( 26)+ram004       *bet**2*zet**2
            dpds( 27)=dpds( 27)+ram005       *bet**2*zet**2
            dpds( 28)=dpds( 28)+ram006       *bet**2*zet**2
            dpds( 81)=dpds( 81)+ram081              *zet**2
            dpds( 82)=dpds( 82)+ram082              *zet**2
            dpds( 83)=dpds( 83)+ram083              *zet**2
            dpds( 84)=dpds( 84)+ram084              *zet**2
            dpds( 85)=dpds( 85)+ram085              *zet**2
            dpds( 86)=dpds( 86)+ram086              *zet**2
            dpds( 87)=dpds( 87)+ram087              *zet**2
            dpds( 88)=dpds( 88)+ram088              *zet**2
            dpds( 90)=dpds( 90)+ram082       *bet   *zet**2
            dpds( 91)=dpds( 91)+ram084       *bet   *zet**2
            dpds( 92)=dpds( 92)+ram086       *bet   *zet**2
            dpds( 93)=dpds( 93)+ram093       *bet   *zet**2
            dpds(137)=dpds(137)+ram137              *zet   
            dpds(138)=dpds(138)+ram138              *zet   
            dpds(139)=dpds(139)+ram139              *zet   
            dpds(144)=dpds(144)+ram144              *zet**2
            dpds(145)=dpds(145)+ram145              *zet**2
            dpds(146)=dpds(146)+ram137              *zet**2
            dpds(147)=dpds(147)+ram147              *zet**2
            dpds(148)=dpds(148)+ram148              *zet**2
            dpds(149)=dpds(149)+ram149              *zet**2
            dpds(150)=dpds(150)+ram150              *zet**2
            dpds(151)=dpds(151)+ram151              *zet**2
            dpds(152)=dpds(152)+ram152              *zet**2
            dpds(153)=dpds(153)+ram153              *zet**2
            dpds(156)=dpds(156)+ram137       *bet   *zet**2
            dpds(157)=dpds(157)+ram147       *bet   *zet**2
            dpds(158)=dpds(158)+ram138       *bet   *zet**2
            dpds(159)=dpds(159)+ram150       *bet   *zet**2
            dpds(160)=dpds(160)+ram160       *bet   *zet**2
            dpds(162)=dpds(162)+ram137       *bet**2*zet**2
            dpds(163)=dpds(163)+ram138       *bet**2*zet**2
            dpds(164)=dpds(164)+ram139       *bet**2*zet**2
            dpds(227)=dpds(227)+ram227              *zet   
            dpds(228)=dpds(228)+ram228              *zet   
            dpds(231)=dpds(231)+ram231       *bet   *zet   
            dpds(237)=dpds(237)+ram227       *bet**2*zet**2
            dpds(238)=dpds(238)+ram228       *bet**2*zet**2
            dpds(259)=dpds(259)+ram259              *zet**2
            dpds(260)=dpds(260)+ram260              *zet**2
            dpds(261)=dpds(261)+ram261              *zet**2
            dpds(262)=dpds(262)+ram262              *zet**2
            dpds(263)=dpds(263)+ram263              *zet**2
      
            bet030 =bet030 +ram004        *zet    
            bet031 =bet031 +ram014        *zet    
            bet241 =bet241 +ram231        *zet    
      
            bet036 =bet036 +ram010        *zet**2 
            bet037 =bet037 +ram005        *zet**2 
            bet038 =bet038 +ram011        *zet**2 
            bet039 =bet039 +ram022        *zet**2 
            bet040 =bet040 +ram023        *zet**2 
            bet060 =bet060 +ram004        *zet**2 
            bet095 =bet095 +ram082        *zet**2 
            bet096 =bet096 +ram084        *zet**2 
            bet097 =bet097 +ram086        *zet**2 
            bet098 =bet098 +ram093        *zet**2 
            bet167 =bet167 +ram137        *zet**2 
            bet168 =bet168 +ram147        *zet**2 
            bet169 =bet169 +ram138        *zet**2 
            bet170 =bet170 +ram150        *zet**2 
            bet171 =bet171 +ram160        *zet**2 
      
            bet064 =bet064 +ram010        *zet**3 
            bet065 =bet065 +ram011        *zet**3 
            bet101 =bet101 +ram081        *zet**3 
            bet102 =bet102 +ram082        *zet**3 
            bet103 =bet103 +ram083        *zet**3 
            bet104 =bet104 +ram084        *zet**3 
            bet105 =bet105 +ram085        *zet**3 
            bet106 =bet106 +ram087        *zet**3 
            bet114 =bet114 +ram086        *zet**3 
            bet176 =bet176 +ram144        *zet**3 
            bet177 =bet177 +ram145        *zet**3 
            bet178 =bet178 +ram147        *zet**3 
            bet179 =bet179 +ram148        *zet**3 
            bet180 =bet180 +ram149        *zet**3 
            bet181 =bet181 +ram151        *zet**3 
            bet197 =bet197 +ram137        *zet**3 
            bet199 =bet199 +ram150        *zet**3 
            bet265 =bet265 +ram259        *zet**3 
            bet266 =bet266 +ram260        *zet**3 
            bet267 =bet267 +ram261        *zet**3 
            bet268 =bet268 +ram262        *zet**3 
      
            bet117 =bet117 +ram082        *zet**4 
            bet118 =bet118 +ram083        *zet**4 
            bet119 =bet119 +ram085        *zet**4 
            bet125 =bet125 +ram084        *zet**4 
            bet203 =bet203 +ram145        *zet**4 
            bet204 =bet204 +ram148        *zet**4 
            bet215 =bet215 +ram147        *zet**4 
            bet270 =bet270 +ram259        *zet**4 
            bet271 =bet271 +ram260        *zet**4 
            bet272 =bet272 +ram261        *zet**4 
      
            bet128 =bet128 +ram083        *zet**5 
            bet132 =bet132 +ram082        *zet**5 
            bet274 =bet274 +ram259        *zet**5 
            bet275 =bet275 +ram260        *zet**5 
      
            bet277 =bet277 +ram259        *zet**6 
      
            bet045 =bet045 +ram004 *bet   *zet**2 
            bet046 =bet046 +ram005 *bet   *zet**2 
            bet047 =bet047 +ram006 *bet   *zet**2 
            bet247 =bet247 +ram227 *bet   *zet**2 
            bet248 =bet248 +ram228 *bet   *zet**2 
            bet183 =bet183 +ram137 *bet   *zet**2 
            bet184 =bet184 +ram138 *bet   *zet**2 
            bet185 =bet185 +ram139 *bet   *zet**2 
      
            bet051 =bet051 +ram010 *bet   *zet**3 
            bet052 =bet052 +ram011 *bet   *zet**3 
            bet068 =bet068 +ram005 *bet   *zet**3 
            bet108 =bet108 +ram082 *bet   *zet**3 
            bet109 =bet109 +ram084 *bet   *zet**3 
            bet110 =bet110 +ram086 *bet   *zet**3 
            bet188 =bet188 +ram145 *bet   *zet**3 
            bet189 =bet189 +ram137 *bet   *zet**3 
            bet190 =bet190 +ram147 *bet   *zet**3 
            bet191 =bet191 +ram150 *bet   *zet**3 
            bet207 =bet207 +ram138 *bet   *zet**3 
      
            bet121 =bet121 +ram082 *bet   *zet**4 
            bet122 =bet122 +ram084 *bet   *zet**4 
            bet210 =bet210 +ram147 *bet   *zet**4 
            bet219 =bet219 +ram137 *bet   *zet**4 
      
            bet130 =bet130 +ram082 *bet   *zet**5 
      
            bet054 =bet054 +ram004 *bet**2*zet**2 
            bet055 =bet055 +ram014 *bet**2*zet**2 
            bet251 =bet251 +ram231 *bet**2*zet**2 
      
            bet058 =bet058 +ram005 *bet**2*zet**3 
            bet072 =bet072 +ram004 *bet**2*zet**3 
            bet193 =bet193 +ram137 *bet**2*zet**3 
            bet194 =bet194 +ram138 *bet**2*zet**3 
      
            bet212 =bet212 +ram137 *bet**2*zet**4 
         enddo
         dpds( 29)=dpds( 29)+bet029*alp   
         dpds( 32)=dpds( 32)+bet032*alp   
         dpds( 33)=dpds( 33)+bet033*alp   
         dpds( 34)=dpds( 34)+bet034*alp   
         dpds( 35)=dpds( 35)+bet035*alp   
         dpds( 41)=dpds( 41)+bet041*alp   
         dpds( 42)=dpds( 42)+bet042*alp   
         dpds( 43)=dpds( 43)+bet043*alp   
         dpds( 44)=dpds( 44)+bet044*alp   
         dpds( 48)=dpds( 48)+bet048*alp   
         dpds( 49)=dpds( 49)+bet049*alp   
         dpds( 50)=dpds( 50)+bet050*alp   
         dpds( 53)=dpds( 53)+bet053*alp   
         dpds( 56)=dpds( 56)+bet056*alp   
         dpds( 57)=dpds( 57)+bet057*alp   
         dpds( 59)=dpds( 59)+bet033*alp**2
         dpds( 61)=dpds( 61)+bet041*alp**2
         dpds( 62)=dpds( 62)+bet062*alp**2
         dpds( 63)=dpds( 63)+bet042*alp**2
         dpds( 66)=dpds( 66)+bet066*alp**2
         dpds( 67)=dpds( 67)+bet049*alp**2
         dpds( 69)=dpds( 69)+bet069*alp**2
         dpds( 70)=dpds( 70)+bet070*alp**2
         dpds( 71)=dpds( 71)+bet056*alp**2
         dpds( 73)=dpds( 73)+bet073*alp**2
         dpds( 74)=dpds( 74)+bet074*alp**3
         dpds( 75)=dpds( 75)+bet075*alp**3
         dpds( 76)=dpds( 76)+bet076*alp**3
         dpds( 77)=dpds( 77)+bet077*alp**3
         dpds( 78)=dpds( 78)+bet078*alp**3
         dpds( 94)=dpds( 94)+bet094*alp   
         dpds( 99)=dpds( 99)+bet099*alp   
         dpds(100)=dpds(100)+bet100*alp   
         dpds(107)=dpds(107)+bet107*alp   
         dpds(111)=dpds(111)+bet099*alp**2
         dpds(115)=dpds(115)+bet115*alp**2
         dpds(116)=dpds(116)+bet116*alp**2
         dpds(120)=dpds(120)+bet120*alp**2
         dpds(123)=dpds(123)+bet115*alp**3
         dpds(126)=dpds(126)+bet126*alp**3
         dpds(127)=dpds(127)+bet127*alp**3
         dpds(129)=dpds(129)+bet129*alp**3
         dpds(131)=dpds(131)+bet126*alp**4
         dpds(133)=dpds(133)+bet133*alp**4
         dpds(134)=dpds(134)+bet134*alp**4
         dpds(135)=dpds(135)+bet135*alp**5
         dpds(165)=dpds(165)+bet165*alp   
         dpds(166)=dpds(166)+bet166*alp   
         dpds(172)=dpds(172)+bet172*alp   
         dpds(173)=dpds(173)+bet173*alp   
         dpds(174)=dpds(174)+bet174*alp   
         dpds(175)=dpds(175)+bet175*alp   
         dpds(182)=dpds(182)+bet182*alp   
         dpds(186)=dpds(186)+bet186*alp   
         dpds(187)=dpds(187)+bet187*alp   
         dpds(192)=dpds(192)+bet192*alp   
         dpds(195)=dpds(195)+bet173*alp**2
         dpds(196)=dpds(196)+bet174*alp**2
         dpds(200)=dpds(200)+bet200*alp**2
         dpds(201)=dpds(201)+bet201*alp**2
         dpds(202)=dpds(202)+bet202*alp**2
         dpds(205)=dpds(205)+bet186*alp**2
         dpds(208)=dpds(208)+bet208*alp**2
         dpds(209)=dpds(209)+bet209*alp**2
         dpds(211)=dpds(211)+bet211*alp**2
         dpds(213)=dpds(213)+bet213*alp**3
         dpds(214)=dpds(214)+bet201*alp**3
         dpds(216)=dpds(216)+bet216*alp**3
         dpds(217)=dpds(217)+bet217*alp**3
         dpds(218)=dpds(218)+bet218*alp**3
         dpds(220)=dpds(220)+bet220*alp**3
         dpds(221)=dpds(221)+bet221*alp**3
         dpds(222)=dpds(222)+bet222*alp**4
         dpds(223)=dpds(223)+bet223*alp**4
         dpds(239)=dpds(239)+bet239*alp   
         dpds(240)=dpds(240)+bet240*alp   
         dpds(242)=dpds(242)+bet242*alp   
         dpds(243)=dpds(243)+bet243*alp   
         dpds(244)=dpds(244)+bet244*alp   
         dpds(245)=dpds(245)+bet245*alp   
         dpds(246)=dpds(246)+bet246*alp   
         dpds(249)=dpds(249)+bet249*alp   
         dpds(250)=dpds(250)+bet250*alp   
         dpds(252)=dpds(252)+bet252*alp   
         dpds(253)=dpds(253)+bet253*alp   
         dpds(254)=dpds(254)+bet254*alp**2
         dpds(255)=dpds(255)+bet255*alp**2
         dpds(256)=dpds(256)+bet256*alp**2
         dpds(257)=dpds(257)+bet257*alp**2
         dpds(269)=dpds(269)+bet269*alp**2
         dpds(264)=dpds(264)+bet264*alp   
         dpds(276)=dpds(276)+bet276*alp**4
         dpds(273)=dpds(273)+bet273*alp**3
         dpds(278)=dpds(278)+bet278*alp**5
         dpds(281)=dpds(281)+bet281*alp   
         dpds(282)=dpds(282)+bet282*alp   
         dpds(283)=dpds(283)+bet283*alp   
         dpds(284)=dpds(284)+bet284*alp   
   
         dpds( 30)=dpds( 30)+bet030*alp   
         dpds( 31)=dpds( 31)+bet031*alp   
         dpds( 36)=dpds( 36)+bet036*alp   
         dpds( 37)=dpds( 37)+bet037*alp   
         dpds( 38)=dpds( 38)+bet038*alp   
         dpds( 39)=dpds( 39)+bet039*alp   
         dpds( 40)=dpds( 40)+bet040*alp   
         dpds( 45)=dpds( 45)+bet045*alp   
         dpds( 46)=dpds( 46)+bet046*alp   
         dpds( 47)=dpds( 47)+bet047*alp   
         dpds( 51)=dpds( 51)+bet051*alp   
         dpds( 52)=dpds( 52)+bet052*alp   
         dpds( 54)=dpds( 54)+bet054*alp   
         dpds( 55)=dpds( 55)+bet055*alp   
         dpds( 58)=dpds( 58)+bet058*alp   
         dpds( 60)=dpds( 60)+bet060*alp**2
         dpds( 64)=dpds( 64)+bet064*alp**2
         dpds( 65)=dpds( 65)+bet065*alp**2
         dpds( 68)=dpds( 68)+bet068*alp**2
         dpds( 72)=dpds( 72)+bet072*alp**2
         dpds( 95)=dpds( 95)+bet095*alp   
         dpds( 96)=dpds( 96)+bet096*alp   
         dpds( 97)=dpds( 97)+bet097*alp   
         dpds( 98)=dpds( 98)+bet098*alp   
         dpds(101)=dpds(101)+bet101*alp   
         dpds(102)=dpds(102)+bet102*alp   
         dpds(103)=dpds(103)+bet103*alp   
         dpds(104)=dpds(104)+bet104*alp   
         dpds(105)=dpds(105)+bet105*alp   
         dpds(106)=dpds(106)+bet106*alp   
         dpds(108)=dpds(108)+bet108*alp   
         dpds(109)=dpds(109)+bet109*alp   
         dpds(110)=dpds(110)+bet110*alp   
         dpds(112)=dpds(112)+bet102*alp**2
         dpds(113)=dpds(113)+bet104*alp**2
         dpds(114)=dpds(114)+bet114*alp**2
         dpds(117)=dpds(117)+bet117*alp**2
         dpds(118)=dpds(118)+bet118*alp**2
         dpds(119)=dpds(119)+bet119*alp**2
         dpds(121)=dpds(121)+bet121*alp**2
         dpds(122)=dpds(122)+bet122*alp**2
         dpds(124)=dpds(124)+bet117*alp**3
         dpds(125)=dpds(125)+bet125*alp**3
         dpds(128)=dpds(128)+bet128*alp**3
         dpds(130)=dpds(130)+bet130*alp**3
         dpds(132)=dpds(132)+bet132*alp**4
         dpds(167)=dpds(167)+bet167*alp   
         dpds(168)=dpds(168)+bet168*alp   
         dpds(169)=dpds(169)+bet169*alp   
         dpds(170)=dpds(170)+bet170*alp   
         dpds(171)=dpds(171)+bet171*alp   
         dpds(176)=dpds(176)+bet176*alp   
         dpds(177)=dpds(177)+bet177*alp   
         dpds(178)=dpds(178)+bet178*alp   
         dpds(179)=dpds(179)+bet179*alp   
         dpds(180)=dpds(180)+bet180*alp   
         dpds(181)=dpds(181)+bet181*alp   
         dpds(183)=dpds(183)+bet183*alp   
         dpds(184)=dpds(184)+bet184*alp   
         dpds(185)=dpds(185)+bet185*alp   
         dpds(188)=dpds(188)+bet188*alp   
         dpds(189)=dpds(189)+bet189*alp   
         dpds(190)=dpds(190)+bet190*alp   
         dpds(191)=dpds(191)+bet191*alp   
         dpds(193)=dpds(193)+bet193*alp   
         dpds(194)=dpds(194)+bet194*alp   
         dpds(197)=dpds(197)+bet197*alp**2
         dpds(198)=dpds(198)+bet178*alp**2
         dpds(199)=dpds(199)+bet199*alp**2
         dpds(203)=dpds(203)+bet203*alp**2
         dpds(204)=dpds(204)+bet204*alp**2
         dpds(206)=dpds(206)+bet189*alp**2
         dpds(207)=dpds(207)+bet207*alp**2
         dpds(210)=dpds(210)+bet210*alp**2
         dpds(212)=dpds(212)+bet212*alp**2
         dpds(215)=dpds(215)+bet215*alp**3
         dpds(219)=dpds(219)+bet219*alp**3
         dpds(241)=dpds(241)+bet241*alp   
         dpds(247)=dpds(247)+bet247*alp   
         dpds(248)=dpds(248)+bet248*alp   
         dpds(251)=dpds(251)+bet251*alp   
         dpds(265)=dpds(265)+bet265*alp   
         dpds(266)=dpds(266)+bet266*alp   
         dpds(267)=dpds(267)+bet267*alp   
         dpds(268)=dpds(268)+bet268*alp   
         dpds(270)=dpds(270)+bet270*alp**2
         dpds(271)=dpds(271)+bet271*alp**2
         dpds(272)=dpds(272)+bet272*alp**2
         dpds(274)=dpds(274)+bet274*alp**3
         dpds(275)=dpds(275)+bet275*alp**3
         dpds(277)=dpds(277)+bet277*alp**4
      enddo
#ifdef PL_OMP_ERI
!$omp end do
!$omp end parallel
#endif
      end

C** IKDDPS ===========================================================
C          CALUCATE KERNELS FOR DDPS TYPE
C====================================================================

      subroutine ikddps(
     &           ddps ,
     &           fdat ,  edat  ,
     &           yp   ,  yq    ,
     &           uu   ,  uv    ,
     &           z    ,  zetpq , 
     &           nasp ,  mxsa  ,  mxprm,  ntprm,  
     &           iasp ,  jasp  ,  kasp ,  lasp ,
     &           ishel,  jshel ,  kshel,  lshel     )


      implicit double precision  (a-h,o-z)
      dimension fdat(31,2001)
      dimension edat(2001)
      dimension zetpq(mxprm,mxprm,mxsa,nasp,mxsa,nasp)
      dimension yp(3,mxprm,mxprm),yq(3,mxprm,mxprm)
      dimension uu(mxprm,mxprm),uv(mxprm,mxprm)
      dimension z(mxprm,mxsa,nasp)
      dimension ntprm(mxsa,nasp)
      dimension ddps(245)

      real(8),parameter::pi    = 3.14159265358979323846d+00
      real(8),parameter::tf(25) = (/
     &                33.0,37.0,41.0,43.0,46.0,49.0,
     &                51.0,54.0,56.0,58.0,61.0,63.0,
     &                66.0,68.0,70.0,72.0,74.0,80.0,
     &                85.0,90.0,95.0,95.0,98.0,99.0,
     &                99.0                           /)

      real(8),parameter::ga0 = 1.0d+00
      real(8),parameter::ga1 = 1.000 000 000 000 005 298d+00
      real(8),parameter::ga2 = 0.5d+00
      real(8),parameter::ga3 = 0.166 666 666 598 849 826d+00
      real(8),parameter::ga4 = 0.416 666 666 666 666 667d-01
      real(8),parameter::ga5 = 0.833 355 034 722 222 222d-02
      real(8),parameter::ga6 = 0.138 888 888 888 888 889d-02

      real(8),parameter::rminv(5) = (/
     &   1.0d+00,  0.333333333333333333d+00,  0.20d+00,
     &   0.14285714285714286d+00,  0.11111111111111111d+00 /)

#ifdef PL_SIMD_ERI
      dimension dmx(6,mxprm), val(2,mxprm)
#endif

      DO IJK=1,245
         DDPS(IJK)=0.0d0
      ENDDO

#ifdef PL_OMP_ERI
!$omp parallel reduction(+:ddps)
!$omp& default(private)
!$omp& shared(fdat,edat,
!$omp&        zetpq,ntprm,z,yp,yq,uu,uv,
!$omp&        ishel,jshel,kshel,lshel,iasp,jasp,kasp,lasp)
#endif
      igp = ntprm(ishel,iasp)
      jgp = ntprm(jshel,jasp)
      kgp = ntprm(kshel,kasp)
      lgp = ntprm(lshel,lasp)

      pi2 = 2.0d0*pi

#ifdef PL_OMP_ERI
!$omp do
#endif
      DO I = 1, IGP
         alp = z(i,ishel,iasp)
         bet018=0.0d0
         bet019=0.0d0
         bet020=0.0d0
         bet021=0.0d0
         bet022=0.0d0
         bet023=0.0d0
         bet024=0.0d0
         bet025=0.0d0
         bet026=0.0d0
         bet027=0.0d0
         bet028=0.0d0
         bet029=0.0d0
         bet030=0.0d0
         bet031=0.0d0
         bet032=0.0d0
         bet033=0.0d0
         bet034=0.0d0
         bet035=0.0d0
         bet038=0.0d0
         bet040=0.0d0
         bet041=0.0d0
         bet044=0.0d0
         bet045=0.0d0
         bet047=0.0d0
         bet049=0.0d0
         bet050=0.0d0
         bet051=0.0d0
         bet053=0.0d0
         bet055=0.0d0
         bet056=0.0d0
         bet057=0.0d0
         bet058=0.0d0
         bet059=0.0d0
         bet074=0.0d0
         bet075=0.0d0
         bet076=0.0d0
         bet077=0.0d0
         bet078=0.0d0
         bet079=0.0d0
         bet080=0.0d0
         bet081=0.0d0
         bet082=0.0d0
         bet083=0.0d0
         bet084=0.0d0
         bet085=0.0d0
         bet086=0.0d0
         bet087=0.0d0
         bet088=0.0d0
         bet089=0.0d0
         bet093=0.0d0
         bet094=0.0d0
         bet095=0.0d0
         bet096=0.0d0
         bet096=0.0d0
         bet097=0.0d0
         bet098=0.0d0
         bet099=0.0d0
         bet100=0.0d0
         bet101=0.0d0
         bet104=0.0d0
         bet105=0.0d0
         bet106=0.0d0
         bet107=0.0d0
         bet108=0.0d0
         bet109=0.0d0
         bet111=0.0d0
         bet112=0.0d0
         bet113=0.0d0
         bet114=0.0d0
         bet135=0.0d0
         bet136=0.0d0
         bet137=0.0d0
         bet138=0.0d0
         bet139=0.0d0
         bet140=0.0d0
         bet141=0.0d0
         bet142=0.0d0
         bet143=0.0d0
         bet144=0.0d0
         bet145=0.0d0
         bet146=0.0d0
         bet147=0.0d0
         bet148=0.0d0
         bet149=0.0d0
         bet149=0.0d0
         bet150=0.0d0
         bet151=0.0d0
         bet152=0.0d0
         bet153=0.0d0
         bet154=0.0d0
         bet155=0.0d0
         bet156=0.0d0
         bet160=0.0d0
         bet163=0.0d0
         bet165=0.0d0
         bet166=0.0d0
         bet169=0.0d0
         bet170=0.0d0
         bet171=0.0d0
         bet172=0.0d0
         bet173=0.0d0
         bet174=0.0d0
         bet177=0.0d0
         bet178=0.0d0
         bet180=0.0d0
         bet182=0.0d0
         bet183=0.0d0
         bet184=0.0d0
         bet186=0.0d0
         bet187=0.0d0
         bet188=0.0d0
         bet189=0.0d0
         bet199=0.0d0
         bet200=0.0d0
         bet201=0.0d0
         bet202=0.0d0
         bet203=0.0d0
         bet204=0.0d0
         bet205=0.0d0
         bet206=0.0d0
         bet207=0.0d0
         bet209=0.0d0
         bet210=0.0d0
         bet211=0.0d0
         bet213=0.0d0
         bet215=0.0d0
         bet216=0.0d0
         bet217=0.0d0
         bet218=0.0d0
         bet219=0.0d0
         bet226=0.0d0
         bet227=0.0d0
         bet228=0.0d0
         bet229=0.0d0
         bet230=0.0d0
         bet231=0.0d0
         bet232=0.0d0
         bet233=0.0d0
         bet234=0.0d0
         bet235=0.0d0
         bet236=0.0d0
         bet237=0.0d0
         bet238=0.0d0
         bet239=0.0d0
         bet240=0.0d0
         bet243=0.0d0
         bet244=0.0d0
         bet245=0.0d0
         DO J = 1, JGP
            bet = z(j,jshel,jasp)
            zet = zetpq(j,i,jshel,jasp,ishel,iasp)
            px  = yp (1,j,i)
            py  = yp (2,j,i)
            pz  = yp (3,j,i)
            up  = uu(j,i)
            del001 =0.0d0
            del002 =0.0d0
            del060 =0.0d0
            del061 =0.0d0
            del115 =0.0d0
            del120 =0.0d0
            del190 =0.0d0
            del192 =0.0d0
            del220 =0.0d0
            del241 =0.0d0
            ram003 =0.0d0
            ram004 =0.0d0
            ram005 =0.0d0
            ram009 =0.0d0
            ram062 =0.0d0
            ram063 =0.0d0
            ram064 =0.0d0
            ram065 =0.0d0
            ram066 =0.0d0
            ram067 =0.0d0
            ram068 =0.0d0
            ram073 =0.0d0
            ram116 =0.0d0
            ram117 =0.0d0
            ram118 =0.0d0
            ram122 =0.0d0
            ram123 =0.0d0
            ram130 =0.0d0
            ram191 =0.0d0
            ram195 =0.0d0
            ram221 =0.0d0
            ram222 =0.0d0
            ram223 =0.0d0
            ram224 =0.0d0
            ram225 =0.0d0
            DO K = 1, KGP
               ram = z(k,kshel,kasp)
               del003 = 0.0d0
               del004 = 0.0d0
               del062 = 0.0d0
               del063 = 0.0d0
               del116 = 0.0d0
               del122 = 0.0d0
               del191 = 0.0d0
               del195 = 0.0d0
               del221 = 0.0d0
               del005 = 0.0d0
               del009 = 0.0d0
               del064 = 0.0d0
               del065 = 0.0d0
               del117 = 0.0d0
               del123 = 0.0d0
               del222 = 0.0d0
               del066 = 0.0d0
               del067 = 0.0d0
               del118 = 0.0d0
               del130 = 0.0d0
               del223 = 0.0d0
               del068 = 0.0d0
               del073 = 0.0d0
               del224 = 0.0d0
               del225 = 0.0d0
               DO L = 1, LGP
                  del = z(l,lshel,lasp)
                  eta = zetpq(l,k,lshel,lasp,kshel,kasp)
                  qpx =  yq(1,l,k) - px
                  qpy =  yq(2,l,k) - py
                  qpz =  yq(3,l,k) - pz
                  vq  = uv(l,k)
      
                  ROO2 = 1.0d0 / (ZET + ETA)
                  qp2  = qpx*qpx + qpy*qpy + qpz*qpz
                  T    = 0.5d0* ROO2*qp2

                  IF (T. LE .TF(6)) THEN
                  
                     IT = IDINT((T+0.025D+00)*20.0D+00)
                     DT = 0.05D+00*IT - T
                     IT = IT + 1
                     DT4 = DT*0.25D+00
                     DT5 = DT*0.2D+00
                     DT6 = DT*0.166666666666666667D+00
                     DT2 = DT4 + DT4
                     DT3 = DT6 + DT6
                     F5   = (((((FDAT(12,IT)*DT6   + FDAT(11,IT))*DT5
     &                 + FDAT(10,IT))*DT4  + FDAT(9,IT))*DT3
     &                 + FDAT(8,IT))*DT2  + FDAT(7,IT))*DT
     &                 + FDAT(6,IT)
                    
                     EED  = (((((GA6*DT + GA5)*DT + GA4)*DT
     &                 + GA3)*DT + GA2)*DT + GA1)*DT + GA0
                     EE   = EDAT(IT)*EED
                    
                     T2   = T*2.0d0
                     F4   = (T2*F5 + EE)*RMINV(5)
                     F3   = (T2*F4 + EE)*RMINV(4)
                     F2   = (T2*F3 + EE)*RMINV(3)
                     F1   = (T2*F2 + EE)*RMINV(2)
                     F0   =  T2*F1 + EE
                     FPP =  UP * VQ * DSQRT(ROO2)
                    
                     DM0 = FPP*F0
                     DM1 = FPP*F1 *  ROO2
                     DM2 = FPP*F2 *( ROO2)**2
                     DM3 = FPP*F3 *( ROO2)**3
                     DM4 = FPP*F4 *( ROO2)**4
                     DM5 = FPP*F5 *( ROO2)**5

                  ELSE 

                     qinv = 1.0d0/qp2
                     dm0=  up*vq*dsqrt( pi2*qinv)*0.5d0
                     dm1 =        qinv *  dm0
                     dm2 =  3.0d0*qinv *  dm1
                     dm3 =  5.0d0*qinv *  dm2
                     dm4 =  7.0d0*qinv *  dm3
                     dm5 =  9.0d0*qinv *  dm4

                  ENDIF
 
#ifdef PL_SIMD_ERI
                  val(1,l) = eta
                  val(2,l) = del

                  dmx(1,l) = dm0
                  dmx(2,l) = dm1
                  dmx(3,l) = dm2
                  dmx(4,l) = dm3
                  dmx(5,l) = dm4
                  dmx(6,l) = dm5
               enddo
               
               do l = 1, lgp
                  eta = val(1,l)
                  del = val(2,l)

                  dm0 = dmx(1,l)
                  dm1 = dmx(2,l)
                  dm2 = dmx(3,l)
                  dm3 = dmx(4,l)
                  dm4 = dmx(5,l)
                  dm5 = dmx(6,l)
#endif

!** KERNEL DDPS =====================================================
!           CALUCLATE & CONTRACT KERNELS FOR DDPS TYPE
!====================================================================
                  del001 =del001 +dm2             
                  del003 =del003 +dm2       *eta   
                  del009 =del009 +dm2       *eta**2
                  del002 =del002 +dm2*del         
                  del004 =del004 +dm2*del   *eta   
                  del005 =del005 +dm2*del   *eta**2
!==============================================
                  del060 =del060 +dm4             
                  del062 =del062 +dm4       *eta   
                  del064 =del064 +dm4       *eta**2
                  del066 =del066 +dm4       *eta**3
                  del073 =del073 +dm4       *eta**4
                  del061 =del061 +dm4*del         
                  del063 =del063 +dm4*del   *eta   
                  del065 =del065 +dm4*del   *eta**2
                  del067 =del067 +dm4*del   *eta**3
                  del068 =del068 +dm4*del   *eta**4
!=========================================
                  del115 =del115 +dm3             
                  del116 =del116 +dm3       *eta   
                  del117 =del117 +dm3       *eta**2
                  del118 =del118 +dm3       *eta**3
                  del120 =del120 +dm3*del         
                  del122 =del122 +dm3*del   *eta   
                  del123 =del123 +dm3*del   *eta**2
                  del130 =del130 +dm3*del   *eta**3
!=============================================-
                  del190 =del190 +dm1             
                  del191 =del191 +dm1       *eta   
                  del192 =del192 +dm1*del         
                  del195 =del195 +dm1*del   *eta   
! ==========================================-
                  del220 =del220 +dm5             
                  del221 =del221 +dm5       *eta   
                  del222 =del222 +dm5       *eta**2
                  del223 =del223 +dm5       *eta**3
                  del224 =del224 +dm5       *eta**4
                  del225 =del225 +dm5       *eta**5
! ==========================================-
                  del241 =del241 +dm0*del         
               enddo

               ram003 = ram003 + del003 * ram   
               ram004 = ram004 + del004 * ram   
               ram062 = ram062 + del062 * ram   
               ram063 = ram063 + del063 * ram 
               ram116 = ram116 + del116 * ram   
               ram122 = ram122 + del122 * ram   
               ram191 = ram191 + del191 * ram   
               ram195 = ram195 + del195 * ram   
               ram221 = ram221 + del221 * ram   
               ram005 = ram005 + del005 * ram**2
               ram009 = ram009 + del009 * ram**2
               ram064 = ram064 + del064 * ram**2
               ram065 = ram065 + del065 * ram**2
               ram117 = ram117 + del117 * ram**2
               ram123 = ram123 + del123 * ram**2
               ram222 = ram222 + del222 * ram**2
               ram066 = ram066 + del066 * ram**3
               ram067 = ram067 + del067 * ram**3
               ram118 = ram118 + del118 * ram**3
               ram130 = ram130 + del130 * ram**3
               ram223 = ram223 + del223 * ram**3
               ram068 = ram068 + del068 * ram**4
               ram073 = ram073 + del073 * ram**4
               ram224 = ram224 + del224 * ram**4
               ram225 = ram225 + del225 * ram**5
            enddo
c===================================================================
            ddps(  3)=ddps(  3)+ram003              *zet**2
            ddps(  4)=ddps(  4)+ram004              *zet**2
            ddps(  5)=ddps(  5)+ram005              *zet**2
            ddps(  8)=ddps(  8)+ram003       *bet   *zet**2
            ddps(  9)=ddps(  9)+ram009       *bet   *zet**2
            ddps( 12)=ddps( 12)+ram004       *bet   *zet**3
            ddps( 15)=ddps( 15)+ram003       *bet**2*zet**3
            ddps( 16)=ddps( 16)+ram004       *bet**2*zet**3
            ddps( 17)=ddps( 17)+ram005       *bet**2*zet**3
            ddps( 62)=ddps( 62)+ram062              *zet**3
            ddps( 63)=ddps( 63)+ram063              *zet**3
            ddps( 64)=ddps( 64)+ram064              *zet**3
            ddps( 65)=ddps( 65)+ram065              *zet**3
            ddps( 66)=ddps( 66)+ram066              *zet**3
            ddps( 67)=ddps( 67)+ram067              *zet**3
            ddps( 68)=ddps( 68)+ram068              *zet**3
            ddps( 70)=ddps( 70)+ram062       *bet   *zet**3
            ddps( 71)=ddps( 71)+ram064       *bet   *zet**3
            ddps( 72)=ddps( 72)+ram066       *bet   *zet**3
            ddps( 73)=ddps( 73)+ram073       *bet   *zet**3
            ddps(116)=ddps(116)+ram116              *zet**2
            ddps(117)=ddps(117)+ram117              *zet**2
            ddps(118)=ddps(118)+ram118              *zet**2
            ddps(121)=ddps(121)+ram116              *zet**3
            ddps(122)=ddps(122)+ram122              *zet**3
            ddps(123)=ddps(123)+ram123              *zet**3
            ddps(126)=ddps(126)+ram116       *bet   *zet**3
            ddps(127)=ddps(127)+ram122       *bet   *zet**3
            ddps(128)=ddps(128)+ram117       *bet   *zet**3
            ddps(129)=ddps(129)+ram123       *bet   *zet**3
            ddps(130)=ddps(130)+ram130       *bet   *zet**3
            ddps(132)=ddps(132)+ram116       *bet**2*zet**3
            ddps(133)=ddps(133)+ram117       *bet**2*zet**3
            ddps(134)=ddps(134)+ram118       *bet**2*zet**3
            ddps(191)=ddps(191)+ram191              *zet   
            ddps(195)=ddps(195)+ram195       *bet   *zet**2
            ddps(197)=ddps(197)+ram191       *bet**2*zet**2
            ddps(221)=ddps(221)+ram221              *zet**3
            ddps(222)=ddps(222)+ram222              *zet**3
            ddps(223)=ddps(223)+ram223              *zet**3
            ddps(224)=ddps(224)+ram224              *zet**3
            ddps(225)=ddps(225)+ram225              *zet**3

            bet019=bet019+ram003       *zet**2
            bet020=bet020+ram009       *zet**2
            bet201=bet201+ram195       *zet**2
            bet209=bet209+ram191       *zet**2
      
            bet203=bet203+ram191*bet   *zet**2
      
            bet038=bet038+ram003       *zet**3
            bet023=bet023+ram004       *zet**3
            bet040=bet040+ram005       *zet**3
            bet075=bet075+ram062       *zet**3
            bet076=bet076+ram064       *zet**3
            bet077=bet077+ram066       *zet**3
            bet078=bet078+ram073       *zet**3
            bet137=bet137+ram116       *zet**3
            bet138=bet138+ram122       *zet**3
            bet139=bet139+ram117       *zet**3
            bet140=bet140+ram123       *zet**3
            bet141=bet141+ram130       *zet**3
            bet160=bet160+ram118       *zet**3
      
            bet026=bet026+ram003*bet   *zet**3
            bet027=bet027+ram004*bet   *zet**3
            bet028=bet028+ram005*bet   *zet**3
            bet044=bet044+ram009*bet   *zet**3
            bet146=bet146+ram116*bet   *zet**3
            bet147=bet147+ram117*bet   *zet**3
            bet148=bet148+ram118*bet   *zet**3
            bet213=bet213+ram195*bet   *zet**3
      
            bet031=bet031+ram003*bet**2*zet**3
            bet032=bet032+ram009*bet**2*zet**3
            bet207=bet207+ram195*bet**2*zet**3
            bet215=bet215+ram191*bet**2*zet**3
      
            bet053=bet053+ram004       *zet**4
            bet081=bet081+ram062       *zet**4
            bet082=bet082+ram063       *zet**4
            bet083=bet083+ram064       *zet**4
            bet084=bet084+ram065       *zet**4
            bet085=bet085+ram067       *zet**4
            bet093=bet093+ram066       *zet**4
            bet144=bet144+ram122       *zet**4
            bet163=bet163+ram116       *zet**4
            bet165=bet165+ram123       *zet**4
            bet177=bet177+ram117       *zet**4
            bet227=bet227+ram221       *zet**4
            bet228=bet228+ram222       *zet**4
            bet229=bet229+ram223       *zet**4
            bet230=bet230+ram224       *zet**4
       
            bet055=bet055+ram003*bet   *zet**4
            bet047=bet047+ram004*bet   *zet**4
            bet087=bet087+ram062*bet   *zet**4
            bet088=bet088+ram064*bet   *zet**4
            bet089=bet089+ram066*bet   *zet**4
            bet151=bet151+ram116*bet   *zet**4
            bet152=bet152+ram122*bet   *zet**4
            bet153=bet153+ram123*bet   *zet**4
            bet169=bet169+ram117*bet   *zet**4
       
            bet035=bet035+ram004*bet**2*zet**4
            bet049=bet049+ram003*bet**2*zet**4
            bet155=bet155+ram116*bet**2*zet**4
            bet156=bet156+ram117*bet**2*zet**4
       
            bet096=bet096+ram062       *zet**5
            bet097=bet097+ram063       *zet**5
            bet098=bet098+ram065       *zet**5
            bet104=bet104+ram064       *zet**5
            bet180=bet180+ram122       *zet**5
            bet186=bet186+ram116       *zet**5
            bet232=bet232+ram221       *zet**5
            bet233=bet233+ram222       *zet**5
            bet234=bet234+ram223       *zet**5
       
            bet100=bet100+ram062*bet   *zet**5
            bet101=bet101+ram064*bet   *zet**5
            bet172=bet172+ram122*bet   *zet**5
            bet182=bet182+ram116*bet   *zet**5
       
            bet174=bet174+ram116*bet**2*zet**5
       
            bet107=bet107+ram063       *zet**6
            bet111=bet111+ram062       *zet**6
            bet236=bet236+ram221       *zet**6
            bet237=bet237+ram222       *zet**6
      
            bet109=bet109+ram062*bet   *zet**6
      
            bet239=bet239+ram221       *zet**7
c===================================================================
            ddps(  1)=ddps(  1)+del001           *zet**2
            ddps(  2)=ddps(  2)+del002           *zet**2
            ddps(  6)=ddps(  6)+del002           *zet**3
            ddps(  7)=ddps(  7)+del001    *bet   *zet**2
            ddps( 10)=ddps( 10)+del001    *bet   *zet**3
            ddps( 11)=ddps( 11)+del002    *bet   *zet**3
            ddps( 13)=ddps( 13)+del001    *bet**2*zet**3
            ddps( 14)=ddps( 14)+del002    *bet**2*zet**3
            ddps( 60)=ddps( 60)+del060           *zet**3
            ddps( 61)=ddps( 61)+del061           *zet**3
            ddps( 69)=ddps( 69)+del060    *bet   *zet**3
            ddps(115)=ddps(115)+del115           *zet**2
            ddps(119)=ddps(119)+del115           *zet**3
            ddps(120)=ddps(120)+del120           *zet**3
            ddps(124)=ddps(124)+del115    *bet   *zet**3
            ddps(125)=ddps(125)+del120    *bet   *zet**3
            ddps(131)=ddps(131)+del115    *bet**2*zet**3
            ddps(190)=ddps(190)+del190           *zet   
            ddps(192)=ddps(192)+del192           *zet**2
            ddps(193)=ddps(193)+del190    *bet   *zet**2
            ddps(194)=ddps(194)+del192    *bet   *zet**2
            ddps(196)=ddps(196)+del190    *bet**2*zet**2
            ddps(198)=ddps(198)+del192    *bet**2*zet**3
            ddps(220)=ddps(220)+del220           *zet**3
            ddps(241)=ddps(241)+del241           *zet   
            ddps(242)=ddps(242)+del241    *bet**2*zet**2
      
            bet018 =bet018 +del001       *zet**2
            bet199 =bet199 +del190       *zet**2
            bet200 =bet200 +del192       *zet**2
            bet244 =bet244 +del241       *zet**2
      
            bet202 =bet202 +del190*bet   *zet**2
            bet243 =bet243 +del241*bet   *zet**2
      
            bet021 =bet021 +del001       *zet**3
            bet022 =bet022 +del002       *zet**3
            bet074 =bet074 +del060       *zet**3
            bet135 =bet135 +del115       *zet**3
            bet136 =bet136 +del120       *zet**3
            bet210 =bet210 +del192       *zet**3
            bet217 =bet217 +del190       *zet**3
      
            bet024 =bet024 +del001*bet   *zet**3
            bet025 =bet025 +del002*bet   *zet**3
            bet145 =bet145 +del115*bet   *zet**3
            bet211 =bet211 +del190*bet   *zet**3
            bet204 =bet204 +del192*bet   *zet**3
      
            bet030 =bet030 +del001*bet**2*zet**3
            bet205 =bet205 +del190*bet**2*zet**3
            bet206 =bet206 +del192*bet**2*zet**3
            bet245 =bet245 +del241*bet**2*zet**3
      
            bet041 =bet041 +del002       *zet**4
            bet051 =bet051 +del001       *zet**4
            bet079 =bet079 +del060       *zet**4
            bet080 =bet080 +del061       *zet**4
            bet142 =bet142 +del115       *zet**4
            bet143 =bet143 +del120       *zet**4
            bet226 =bet226 +del220       *zet**4
      
            bet029 =bet029 +del002*bet   *zet**4
            bet045 =bet045 +del001*bet   *zet**4
            bet086 =bet086 +del060*bet   *zet**4
            bet149 =bet149 +del115*bet   *zet**4
            bet150 =bet150 +del120*bet   *zet**4
            bet218 =bet218 +del192*bet   *zet**4
      
            bet033 =bet033 +del001*bet**2*zet**4
            bet034 =bet034 +del002*bet**2*zet**4
            bet154 =bet154 +del115*bet**2*zet**4
            bet216 =bet216 +del192*bet**2*zet**4
            bet219 =bet219 +del190*bet**2*zet**4
      
            bet058 =bet058 +del002       *zet**5
            bet094 =bet094 +del060       *zet**5
            bet095 =bet095 +del061       *zet**5
            bet166 =bet166 +del120       *zet**5
            bet178 =bet178 +del115       *zet**5
            bet231 =bet231 +del220       *zet**5
      
            bet056 =bet056 +del002*bet   *zet**5
            bet059 =bet059 +del001*bet   *zet**5
            bet099 =bet099 +del060*bet   *zet**5
            bet170 =bet170 +del115*bet   *zet**5
            bet171 =bet171 +del120*bet   *zet**5
      
            bet050 =bet050 +del002*bet**2*zet**5
            bet057 =bet057 +del001*bet**2*zet**5
            bet173 =bet173 +del115*bet**2*zet**5
      
            bet105 =bet105 +del060       *zet**6
            bet106 =bet106 +del061       *zet**6
            bet187 =bet187 +del120       *zet**6
            bet189 =bet189 +del115       *zet**6
            bet235 =bet235 +del220       *zet**6
      
            bet108 =bet108 +del060*bet   *zet**6
            bet183 =bet183 +del120*bet   *zet**6
            bet188 =bet188 +del115*bet   *zet**6
      
            bet184 =bet184 +del115*bet**2*zet**6
      
            bet112 =bet112 +del061       *zet**7
            bet114 =bet114 +del060       *zet**7
            bet238 =bet238 +del220       *zet**7
      
            bet113 =bet113 +del060*bet   *zet**7
      
            bet240 =bet240 +del220       *zet**8
         enddo
         ddps( 18)=ddps( 18)+bet018*alp   
         ddps( 19)=ddps( 19)+bet019*alp   
         ddps( 20)=ddps( 20)+bet020*alp   
         ddps( 21)=ddps( 21)+bet021*alp   
         ddps( 22)=ddps( 22)+bet022*alp   
         ddps( 23)=ddps( 23)+bet023*alp   
         ddps( 24)=ddps( 24)+bet024*alp   
         ddps( 25)=ddps( 25)+bet025*alp   
         ddps( 26)=ddps( 26)+bet026*alp   
         ddps( 27)=ddps( 27)+bet027*alp   
         ddps( 28)=ddps( 28)+bet028*alp   
         ddps( 29)=ddps( 29)+bet029*alp   
         ddps( 30)=ddps( 30)+bet030*alp   
         ddps( 31)=ddps( 31)+bet031*alp   
         ddps( 32)=ddps( 32)+bet032*alp   
         ddps( 33)=ddps( 33)+bet033*alp   
         ddps( 34)=ddps( 34)+bet034*alp   
         ddps( 35)=ddps( 35)+bet035*alp   
         ddps( 36)=ddps( 36)+bet021*alp**2
         ddps( 37)=ddps( 37)+bet022*alp**2
         ddps( 38)=ddps( 38)+bet038*alp**2
         ddps( 39)=ddps( 39)+bet023*alp**2
         ddps( 40)=ddps( 40)+bet040*alp**2
         ddps( 41)=ddps( 41)+bet041*alp**2
         ddps( 42)=ddps( 42)+bet024*alp**2
         ddps( 43)=ddps( 43)+bet026*alp**2
         ddps( 44)=ddps( 44)+bet044*alp**2
         ddps( 45)=ddps( 45)+bet045*alp**2
         ddps( 46)=ddps( 46)+bet029*alp**2
         ddps( 47)=ddps( 47)+bet047*alp**2
         ddps( 48)=ddps( 48)+bet033*alp**2
         ddps( 49)=ddps( 49)+bet049*alp**2
         ddps( 50)=ddps( 50)+bet050*alp**2
         ddps( 51)=ddps( 51)+bet051*alp**3
         ddps( 52)=ddps( 52)+bet041*alp**3
         ddps( 53)=ddps( 53)+bet053*alp**3
         ddps( 54)=ddps( 54)+bet045*alp**3
         ddps( 55)=ddps( 55)+bet055*alp**3
         ddps( 56)=ddps( 56)+bet056*alp**3
         ddps( 57)=ddps( 57)+bet057*alp**3
         ddps( 58)=ddps( 58)+bet058*alp**4
         ddps( 59)=ddps( 59)+bet059*alp**4
         ddps( 74)=ddps( 74)+bet074*alp   
         ddps( 75)=ddps( 75)+bet075*alp   
         ddps( 76)=ddps( 76)+bet076*alp   
         ddps( 77)=ddps( 77)+bet077*alp   
         ddps( 78)=ddps( 78)+bet078*alp   
         ddps( 79)=ddps( 79)+bet079*alp   
         ddps( 80)=ddps( 80)+bet080*alp   
         ddps( 81)=ddps( 81)+bet081*alp   
         ddps( 82)=ddps( 82)+bet082*alp   
         ddps( 83)=ddps( 83)+bet083*alp   
         ddps( 84)=ddps( 84)+bet084*alp   
         ddps( 85)=ddps( 85)+bet085*alp   
         ddps( 86)=ddps( 86)+bet086*alp   
         ddps( 87)=ddps( 87)+bet087*alp   
         ddps( 88)=ddps( 88)+bet088*alp   
         ddps( 89)=ddps( 89)+bet089*alp   
         ddps( 90)=ddps( 90)+bet079*alp**2
         ddps( 91)=ddps( 91)+bet081*alp**2
         ddps( 92)=ddps( 92)+bet083*alp**2
         ddps( 93)=ddps( 93)+bet093*alp**2
         ddps( 94)=ddps( 94)+bet094*alp**2
         ddps( 95)=ddps( 95)+bet095*alp**2
         ddps( 96)=ddps( 96)+bet096*alp**2
         ddps( 97)=ddps( 97)+bet097*alp**2
         ddps( 98)=ddps( 98)+bet098*alp**2
         ddps( 99)=ddps( 99)+bet099*alp**2
         ddps(100)=ddps(100)+bet100*alp**2
         ddps(101)=ddps(101)+bet101*alp**2
         ddps(102)=ddps(102)+bet094*alp**3
         ddps(103)=ddps(103)+bet096*alp**3
         ddps(104)=ddps(104)+bet104*alp**3
         ddps(105)=ddps(105)+bet105*alp**3
         ddps(106)=ddps(106)+bet106*alp**3
         ddps(107)=ddps(107)+bet107*alp**3
         ddps(108)=ddps(108)+bet108*alp**3
         ddps(109)=ddps(109)+bet109*alp**3
         ddps(110)=ddps(110)+bet105*alp**4
         ddps(111)=ddps(111)+bet111*alp**4
         ddps(112)=ddps(112)+bet112*alp**4
         ddps(113)=ddps(113)+bet113*alp**4
         ddps(114)=ddps(114)+bet114*alp**5
         ddps(135)=ddps(135)+bet135*alp   
         ddps(136)=ddps(136)+bet136*alp   
         ddps(137)=ddps(137)+bet137*alp   
         ddps(138)=ddps(138)+bet138*alp   
         ddps(139)=ddps(139)+bet139*alp   
         ddps(140)=ddps(140)+bet140*alp   
         ddps(141)=ddps(141)+bet141*alp   
         ddps(142)=ddps(142)+bet142*alp   
         ddps(143)=ddps(143)+bet143*alp   
         ddps(144)=ddps(144)+bet144*alp   
         ddps(145)=ddps(145)+bet145*alp   
         ddps(146)=ddps(146)+bet146*alp   
         ddps(147)=ddps(147)+bet147*alp   
         ddps(148)=ddps(148)+bet148*alp   
         ddps(149)=ddps(149)+bet149*alp   
         ddps(150)=ddps(150)+bet150*alp   
         ddps(151)=ddps(151)+bet151*alp   
         ddps(152)=ddps(152)+bet152*alp   
         ddps(153)=ddps(153)+bet153*alp   
         ddps(154)=ddps(154)+bet154*alp   
         ddps(155)=ddps(155)+bet155*alp   
         ddps(156)=ddps(156)+bet156*alp   
         ddps(157)=ddps(157)+bet135*alp**2
         ddps(158)=ddps(158)+bet137*alp**2
         ddps(159)=ddps(159)+bet139*alp**2
         ddps(160)=ddps(160)+bet160*alp**2
         ddps(161)=ddps(161)+bet142*alp**2
         ddps(162)=ddps(162)+bet143*alp**2
         ddps(163)=ddps(163)+bet163*alp**2
         ddps(164)=ddps(164)+bet144*alp**2
         ddps(165)=ddps(165)+bet165*alp**2
         ddps(166)=ddps(166)+bet166*alp**2
         ddps(167)=ddps(167)+bet149*alp**2
         ddps(168)=ddps(168)+bet151*alp**2
         ddps(169)=ddps(169)+bet169*alp**2
         ddps(170)=ddps(170)+bet170*alp**2
         ddps(171)=ddps(171)+bet171*alp**2
         ddps(172)=ddps(172)+bet172*alp**2
         ddps(173)=ddps(173)+bet173*alp**2
         ddps(174)=ddps(174)+bet174*alp**2
         ddps(175)=ddps(175)+bet142*alp**3
         ddps(176)=ddps(176)+bet163*alp**3
         ddps(177)=ddps(177)+bet177*alp**3
         ddps(178)=ddps(178)+bet178*alp**3
         ddps(179)=ddps(179)+bet166*alp**3
         ddps(180)=ddps(180)+bet180*alp**3
         ddps(181)=ddps(181)+bet170*alp**3
         ddps(182)=ddps(182)+bet182*alp**3
         ddps(183)=ddps(183)+bet183*alp**3
         ddps(184)=ddps(184)+bet184*alp**3
         ddps(185)=ddps(185)+bet178*alp**4
         ddps(186)=ddps(186)+bet186*alp**4
         ddps(187)=ddps(187)+bet187*alp**4
         ddps(188)=ddps(188)+bet188*alp**4
         ddps(189)=ddps(189)+bet189*alp**5
         ddps(199)=ddps(199)+bet199*alp   
         ddps(200)=ddps(200)+bet200*alp   
         ddps(201)=ddps(201)+bet201*alp   
         ddps(202)=ddps(202)+bet202*alp   
         ddps(203)=ddps(203)+bet203*alp   
         ddps(204)=ddps(204)+bet204*alp   
         ddps(205)=ddps(205)+bet205*alp   
         ddps(206)=ddps(206)+bet206*alp   
         ddps(207)=ddps(207)+bet207*alp   
         ddps(208)=ddps(208)+bet199*alp**2
         ddps(209)=ddps(209)+bet209*alp**2
         ddps(210)=ddps(210)+bet210*alp**2
         ddps(211)=ddps(211)+bet211*alp**2
         ddps(212)=ddps(212)+bet204*alp**2
         ddps(213)=ddps(213)+bet213*alp**2
         ddps(214)=ddps(214)+bet205*alp**2
         ddps(215)=ddps(215)+bet215*alp**2
         ddps(216)=ddps(216)+bet216*alp**2
         ddps(217)=ddps(217)+bet217*alp**3
         ddps(218)=ddps(218)+bet218*alp**3
         ddps(219)=ddps(219)+bet219*alp**3
         ddps(226)=ddps(226)+bet226*alp   
         ddps(227)=ddps(227)+bet227*alp   
         ddps(228)=ddps(228)+bet228*alp   
         ddps(229)=ddps(229)+bet229*alp   
         ddps(230)=ddps(230)+bet230*alp   
         ddps(231)=ddps(231)+bet231*alp**2
         ddps(232)=ddps(232)+bet232*alp**2
         ddps(233)=ddps(233)+bet233*alp**2
         ddps(234)=ddps(234)+bet234*alp**2
         ddps(235)=ddps(235)+bet235*alp**3
         ddps(236)=ddps(236)+bet236*alp**3
         ddps(237)=ddps(237)+bet237*alp**3
         ddps(238)=ddps(238)+bet238*alp**4
         ddps(239)=ddps(239)+bet239*alp**4
         ddps(240)=ddps(240)+bet240*alp**5
         ddps(243)=ddps(243)+bet243*alp   
         ddps(244)=ddps(244)+bet244*alp**2
         ddps(245)=ddps(245)+bet245*alp**2
      enddo
#ifdef PL_OMP_ERI
!$omp end do
!$omp end parallel
#endif

      end
