! -*- coding: utf-8 -*-
!
! Dual licensed under the MIT or GPL Version 3 license.
! See 'LICENSE.MIT' for the definition of the MIT license,
! and 'COPYING' for the definition of the GPL license.
!
subroutine os_csi_os3(rsrs,tables,atoms,fmt)
  use integral_tables_class
  use atoms_class
  use integral_fmt_class
  use pl_prof_class
  type(tables_type),intent(in)::tables
  real(8),intent(inout)::rsrs(tables%nshelp)
  type(atoms_type),intent(in)::atoms
  type(fmt_type),intent(in)::fmt

  real(8),allocatable::rsrsx(:)
  integer::nshellpairs

  nshellpairs = tables%nshelp
  
  rsrs(:) = 0.0d0

  allocate(rsrsx(tables%nshelp))
  rsrsx(:) = 0.0d0

!$omp parallel
  call pl_prof_region_begin('os_csi')
!$omp end parallel

!!$  write(6,*) 'csi_ssss'
!$omp parallel
  call pl_prof_region_begin('os_csi_ssss_os3')
  call os_csi_ssss_os3(&
       rsrs,&
       tables%ishp,tables%npsh,tables%npsh2pr,tables%nsh2atm,tables%nsh2ao,&
       atoms%xyz,tables%ovl,tables%bdz,tables%rzeth,&
       tables%nshell,tables%nshelp,tables%nprimp,atoms%natom,fmt)
  call pl_prof_region_end('os_csi_ssss_os3')
!$omp end parallel

!!$  write(6,*) 'csi_psps'
!$omp parallel
  call pl_prof_region_begin('os_csi_psps_os3')
  call os_csi_psps_os3(&
       rsrs,&
       tables%ishp,tables%npsh,tables%npsh2pr,tables%nsh2atm,tables%nsh2ao,&
       atoms%xyz,tables%ovl,tables%bdz,tables%rzeth,&
       tables%nshell,tables%nshelp,tables%nprimp,atoms%natom,fmt)
  call pl_prof_region_end('os_csi_psps_os3')
!$omp end parallel

!!$  write(6,*) 'csi_pppp'
!$omp parallel
  call pl_prof_region_begin('os_csi_pppp_os3')
  call os_csi_pppp_os3(&
       rsrs,&
       tables%ishp,tables%npsh,tables%npsh2pr,tables%nsh2atm,tables%nsh2ao,&
       atoms%xyz,tables%ovl,tables%bdz,tables%rzeth,&
       tables%nshell,tables%nshelp,tables%nprimp,atoms%natom,fmt)
  call pl_prof_region_end('os_csi_pppp_os3')
!$omp end parallel

!!$  write(6,*) 'csi_dsds'
!$omp parallel
  call pl_prof_region_begin('os_csi_dsds_os3')
  call os_csi_dsds_os3(&
       rsrs,&
       tables%ishp,tables%npsh,tables%npsh2pr,tables%nsh2atm,tables%nsh2ao,&
       atoms%xyz,tables%ovl,tables%bdz,tables%rzeth,&
       tables%nshell,tables%nshelp,tables%nprimp,atoms%natom,fmt)
  call pl_prof_region_end('os_csi_dsds_os3')
!$omp end parallel

!!$  write(6,*) 'csi_dpdp'
!$omp parallel
  call pl_prof_region_begin('os_csi_dpdp_os3')
  call os_csi_dpdp_os3(&
       rsrs,&
       tables%ishp,tables%npsh,tables%npsh2pr,tables%nsh2atm,tables%nsh2ao,&
       atoms%xyz,tables%ovl,tables%bdz,tables%rzeth,&
       tables%nshell,tables%nshelp,tables%nprimp,atoms%natom,fmt)
  call pl_prof_region_end('os_csi_dpdp_os3')
!$omp end parallel

!!$  write(6,*) 'csi_dddd'
!$omp parallel
  call pl_prof_region_begin('os_csi_dddd_os3')
  call os_csi_dddd_os3(&
       rsrs,&
       tables%ishp,tables%npsh,tables%npsh2pr,tables%nsh2atm,tables%nsh2ao,&
       atoms%xyz,tables%ovl,tables%bdz,tables%rzeth,&
       tables%nshell,tables%nshelp,tables%nprimp,atoms%natom,fmt)
  call pl_prof_region_end('os_csi_dddd_os3')
!$omp end parallel

!$omp parallel
  call pl_prof_region_end('os_csi')
!$omp end parallel
!!$  write(6,*) 'maxval2(rsrs)=',maxval(rsrs)

end subroutine os_csi_os3
!> Cauchy-Schwarz inequality: (ss|ss)
!
! Target class:
!    (ss|ss)
! Applying the horizontal recurrence relation:
! Classes to be formes by contraction of primitive integrals:
!    (ss|ss)
! Applying the vertical recurrence relation:
!    [ss|ss]
!
! References:
!  [1] S.Obara and A.Saika, J.Chem.Phys. 84, 3963 (1986).
!  [2] M.Head-Gordon and J.A.Pople, J.Chem.Phys. 89, 5777 (1988).
!
subroutine os_csi_ssss_os3(&
     rsrs,&
     ishp,npsh,npsh2pr,nsh2atm,nsh2ao,xyz,ss,xiozetaa,rzet,&
     nshell,nshellp,nprimp,natom,&
     fmt)
  use physical_constants_class
  use integral_fmt_class
  use omp_lib

  implicit real(8) (a-h,o-z)

  type(fmt_type),intent(in)::fmt
  integer,intent(in)::nshell,nshellp,nprimp,natom
  real(8),intent(inout)::rsrs(nshellp)
  integer,intent(in)::ishp(6),npsh(2,nshellp),npsh2pr(nshellp+1)
  integer,intent(in)::nsh2atm(nshell),nsh2ao(nshell)
  real(8),intent(in)::ss(nprimp),xiozetaa(nprimp),rzet(nprimp)
  real(8),intent(in)::xyz(3,natom)

  real(8)::ec(1)
!  integer::tid

!!$  rsrs(1:nshellp) = 0.0d0

  tid = omp_get_thread_num() + 1

  tosqrtpi = 2.d0/dsqrt(pi)

!$omp do schedule(dynamic)
!!$!$omp do
  do ij = 1, ishp(1)
     i = npsh(1,ij)
     j = npsh(2,ij)
     iatom = nsh2atm(i)
     jatom = nsh2atm(j)
     Ax = xyz(1,iatom)
     Ay = xyz(2,iatom)
     Az = xyz(3,iatom)
     Bx = xyz(1,jatom)
     By = xyz(2,jatom)
     Bz = xyz(3,jatom)
     ABx = Ax - Bx
     ABy = Ay - By
     ABz = Az - Bz
     CDx = ABx
     CDy = ABy
     CDz = ABz

        ! Initializing VRR variables
        ssss = 0.0d0

        do irs = npsh2pr(ij),npsh2pr(ij+1)-1
           xioza = xiozetaa(irs)
           PAx = -xioza*ABx
           PAy = -xioza*ABy
           PAz = -xioza*ABz
           Px = PAx + Ax
           Py = PAy + Ay
           Pz = PAz + Az
           Sp = ss(irs)
           ooz = rzet(irs)*2.0d0
           oo2zeta = 0.5d0 * ooz

           do itu = npsh2pr(ij),npsh2pr(ij+1)-1
              xiozb = xiozetaa(itu)
              QCx = -xiozb*ABx
              QCy = -xiozb*ABy
              QCz = -xiozb*ABz
              Qx = QCx + Ax
              Qy = QCy + Ay
              Qz = QCz + Az
              Sq = ss(itu)
              ooe = rzet(itu)*2.0d0

              QPx = Qx - Px
              QPy = Qy - Py
              QPz = Qz - Pz
              QP2 = QPx*QPx + QPy*QPy + QPz*QPz
              rho = 1.0d0/(ooz + ooe)
              t = rho * QP2

              if (t <= tf(1)) then
                 it = idint((t + 0.025d0)*20.0d0)
                 dt = 0.05d0*it - t
                 it = it + 1
                 dt4 = dt * 0.25d0
                 dt5 = dt * 0.2d0
                 dt6 = dt * 0.1666666666666666666d0
                 dt2 = dt4 + dt4
                 dt3 = dt6 + dt6
                 ft  = (((((fmt%fdat(7,it)*dt6 + fmt%fdat(6,it))*dt5&
                     + fmt%fdat(5,it))*dt4 + fmt%fdat(4,it))*dt3&
                     + fmt%fdat(3,it))*dt2 + fmt%fdat(2,it))*dt + fmt%fdat(1,it)
                 ssss0 = ft * dsqrt(rho) * tosqrtpi * Sp * Sq
              else
                 ft = 0.0d0
                 ssss0 = dsqrt(1.0d0/QP2) * Sp * Sq
              end if

              ssss = ssss + ssss0
           end do
        end do

        ec(1) = ssss
        rsrs(ij) = maxval(dabs(ec(:)))
  end do
!$omp end do
end subroutine os_csi_ssss_os3
!> Cauchy-Schwarz inequality: (ps|ps)
!
! Target class:
!    (ps|ps)
! Applying the horizontal recurrence relation:
! Classes to be formes by contraction of primitive integrals:
!    (ps|ps)
! Applying the vertical recurrence relation:
!    [ps|ps](0)    = VRR{ [ps|ss](0..1) [ss|ss](1) }
!    [ps|ss](0..1) = VRR{ [ss|ss](0..2) }
!    [ss|ss](0..2)
!
! References:
!  [1] S.Obara and A.Saika, J.Chem.Phys. 84, 3963 (1986).
!  [2] M.Head-Gordon and J.A.Pople, J.Chem.Phys. 89, 5777 (1988).
!
subroutine os_csi_psps_os3(&
     rsrs,&
     ishp,npsh,npsh2pr,nsh2atm,nsh2ao,xyz,ss,xiozetaa,rzet,&
     nshell,nshellp,nprimp,natom,&
     fmt)
  use physical_constants_class
  use integral_fmt_class
  use omp_lib

  implicit real(8) (a-h,o-z)

  type(fmt_type),intent(in)::fmt
  integer,intent(in)::nshell,nshellp,nprimp,natom
  real(8),intent(inout)::rsrs(nshellp)
  integer,intent(in)::ishp(6),npsh(2,nshellp),npsh2pr(nshellp+1)
  integer,intent(in)::nsh2atm(nshell),nsh2ao(nshell)
  real(8),intent(in)::ss(nprimp),xiozetaa(nprimp),rzet(nprimp)
  real(8),intent(in)::xyz(3,natom)

  real(8)::ec(3)
!  integer::tid

  tid = omp_get_thread_num() + 1

  tosqrtpi = 2.d0/dsqrt(pi)

!$omp do schedule(dynamic)
!!$!$omp do
  do ij = ishp(1)+1, ishp(2)
     i = npsh(1,ij)
     j = npsh(2,ij)
     iatom = nsh2atm(i)
     jatom = nsh2atm(j)
     Ax = xyz(1,iatom)
     Ay = xyz(2,iatom)
     Az = xyz(3,iatom)
     Bx = xyz(1,jatom)
     By = xyz(2,jatom)
     Bz = xyz(3,jatom)
     ABx = Ax - Bx
     ABy = Ay - By
     ABz = Az - Bz
     CDx = ABx
     CDy = ABy
     CDz = ABz

        ! Initializing VRR variables
        pxspxs = 0.0d0
        pyspys = 0.0d0
        pzspzs = 0.0d0

        do irs = npsh2pr(ij),npsh2pr(ij+1)-1
           xioza = xiozetaa(irs)
           PAx = -xioza*ABx
           PAy = -xioza*ABy
           PAz = -xioza*ABz
           Px = PAx + Ax
           Py = PAy + Ay
           Pz = PAz + Az
           Sp = ss(irs)
           ooz = rzet(irs)*2.0d0
           oo2zeta = 0.5d0 * ooz

           do itu = npsh2pr(ij),npsh2pr(ij+1)-1
              xiozb = xiozetaa(itu)
              QCx = -xiozb*ABx
              QCy = -xiozb*ABy
              QCz = -xiozb*ABz
              Qx = QCx + Ax
              Qy = QCy + Ay
              Qz = QCz + Az
              Sq = ss(itu)
              ooe = rzet(itu)*2.0d0

              QPx = Qx - Px
              QPy = Qy - Py
              QPz = Qz - Pz
              QP2 = QPx*QPx + QPy*QPy + QPz*QPz
              rho = 1.0d0/(ooz + ooe)
              t = rho * QP2

              if (t <= tf(3)) then
                 it = idint((t + 0.025d0)*20.0d0)
                 dt = 0.05d0*it - t
                 it = it + 1
                 dt4 = dt * 0.25d0
                 dt5 = dt * 0.2d0
                 dt6 = dt * 0.1666666666666666666d0
                 dt2 = dt4 + dt4
                 dt3 = dt6 + dt6
                 ft  = (((((fmt%fdat(9,it)*dt6 + fmt%fdat(8,it))*dt5&
                     + fmt%fdat(7,it))*dt4 + fmt%fdat(6,it))*dt3&
                     + fmt%fdat(5,it))*dt2 + fmt%fdat(4,it))*dt + fmt%fdat(3,it)
                 et  = ((((((ga6*dt + ga5)*dt + ga4)*dt&
                     + ga3)*dt + ga2)*dt + ga1)*dt + ga0) * fmt%edat(it)
                 t2 = 2.d0 * t
                 cn = dsqrt(rho) * tosqrtpi * Sp * Sq
                 etcn = et * cn
                 ssss2 = ft * cn
                 ssss1 = (ssss2 * t2 + etcn) * rminv(2)
                 ssss0 =  ssss1 * t2 + etcn
              else
                 tinv = 1.d0/t
                 ssss0 = dsqrt(rho*tinv) * Sp * Sq
                 ssss1 = ssss0 * tinv * 0.5d0
                 ssss2 = ssss1 * tinv * 1.5d0
              end if

              rhoozeta = rho*ooz
              oo2eta = 0.5d0 * ooe
              rhoo2ze = rhoozeta*oo2eta
              WPx = QPx * rhoozeta
              WPy = QPy * rhoozeta
              WPz = QPz * rhoozeta
              WQx = WPx - QPx
              WQy = WPy - QPy
              WQz = WPz - QPz

              ak_ssss1 = rhoo2ze*ssss1
              pzsss1 = ssss1*PAz + ssss2*WPz
              pysss1 = ssss1*PAy + ssss2*WPy
              pxsss1 = ssss1*PAx + ssss2*WPx
              pzsss0 = ssss0*PAz + ssss1*WPz
              pzspzs0 = pzsss0*QCz + pzsss1*WQz + ak_ssss1
              pzspzs = pzspzs + pzspzs0
              pysss0 = ssss0*PAy + ssss1*WPy
              pyspys0 = pysss0*QCy + pysss1*WQy + ak_ssss1
              pyspys = pyspys + pyspys0
              pxsss0 = ssss0*PAx + ssss1*WPx
              pxspxs0 = pxsss0*QCx + pxsss1*WQx + ak_ssss1
              pxspxs = pxspxs + pxspxs0
           end do
        end do

        ec(1) = pxspxs
        ec(2) = pyspys
        ec(3) = pzspzs
        rsrs(ij) = maxval(dabs(ec(:)))
  end do
!$omp end do
end subroutine os_csi_psps_os3
!> Cauchy-Schwarz inequality: (pp|pp)
!
! Target class:
!    (pp|pp)
! Applying the horizontal recurrence relation:
!    (pp|pp) = HRR{ (ds|pp) (ps|pp) }
!    (ds|pp) = HRR{ (ds|ds) (ds|ps) }
!    (ps|pp) = HRR{ (ps|ds) (ps|ps) }
! Classes to be formes by contraction of primitive integrals:
!    (ps|ps) (ps|ds) (ds|ps) (ds|ds)
! Applying the vertical recurrence relation:
!    [ds|ds](0)    = VRR{ [ds|ps](0..1) [ds|ss](0..1) [ps|ps](1) }
!    [ds|ps](0..1) = VRR{ [ds|ss](0..2) [ps|ss](1..2) }
!    [ds|ss](0..2) = VRR{ [ps|ss](0..3) [ss|ss](0..3) }
!    [ps|ds](0)    = VRR{ [ps|ps](0..1) [ps|ss](0..1) [ss|ps](1) }
!    [ps|ps](0..1) = VRR{ [ps|ss](0..2) [ss|ss](1..2) }
!    [ps|ss](0..3) = VRR{ [ss|ss](0..4) }
!    [ss|ps](1)    = VRR{ [ss|ss](1..2) }
!    [ss|ss](1..4)
!
! References:
!  [1] S.Obara and A.Saika, J.Chem.Phys. 84, 3963 (1986).
!  [2] M.Head-Gordon and J.A.Pople, J.Chem.Phys. 89, 5777 (1988).
!
subroutine os_csi_pppp_os3(&
     rsrs,&
     ishp,npsh,npsh2pr,nsh2atm,nsh2ao,xyz,ss,xiozetaa,rzet,&
     nshell,nshellp,nprimp,natom,&
     fmt)
  use physical_constants_class
  use integral_fmt_class
  use omp_lib

  implicit real(8) (a-h,o-z)

  type(fmt_type),intent(in)::fmt
  integer,intent(in)::nshell,nshellp,nprimp,natom
  real(8),intent(inout)::rsrs(nshellp)
  integer,intent(in)::ishp(6),npsh(2,nshellp),npsh2pr(nshellp+1)
  integer,intent(in)::nsh2atm(nshell),nsh2ao(nshell)
  real(8),intent(in)::ss(nprimp),xiozetaa(nprimp),rzet(nprimp)
  real(8),intent(in)::xyz(3,natom)

  real(8)::ec(9)
!  integer::tid

  tid = omp_get_thread_num() + 1

  tosqrtpi = 2.d0/dsqrt(pi)

!$omp do schedule(dynamic)
!!$!$omp do
  do ij = ishp(2)+1, ishp(3)
     i = npsh(1,ij)
     j = npsh(2,ij)
     iatom = nsh2atm(i)
     jatom = nsh2atm(j)
     Ax = xyz(1,iatom)
     Ay = xyz(2,iatom)
     Az = xyz(3,iatom)
     Bx = xyz(1,jatom)
     By = xyz(2,jatom)
     Bz = xyz(3,jatom)
     ABx = Ax - Bx
     ABy = Ay - By
     ABz = Az - Bz
     CDx = ABx
     CDy = ABy
     CDz = ABz

        ! Initializing VRR variables
        pxsdxxs = 0.0d0
        pxspxs = 0.0d0
        pxsdxys = 0.0d0
        pxsdxzs = 0.0d0
        pysdxys = 0.0d0
        pyspys = 0.0d0
        pysdyys = 0.0d0
        pysdyzs = 0.0d0
        pzsdxzs = 0.0d0
        pzsdyzs = 0.0d0
        pzspzs = 0.0d0
        pzsdzzs = 0.0d0
        dxxsdxxs = 0.0d0
        dxxspxs = 0.0d0
        dyyspys = 0.0d0
        dyysdyys = 0.0d0
        dzzspzs = 0.0d0
        dzzsdzzs = 0.0d0
        dxyspxs = 0.0d0
        dxysdxys = 0.0d0
        dxyspys = 0.0d0
        dxzspxs = 0.0d0
        dxzsdxzs = 0.0d0
        dxzspzs = 0.0d0
        dyzspys = 0.0d0
        dyzsdyzs = 0.0d0
        dyzspzs = 0.0d0

        do irs = npsh2pr(ij),npsh2pr(ij+1)-1
           xioza = xiozetaa(irs)
           PAx = -xioza*ABx
           PAy = -xioza*ABy
           PAz = -xioza*ABz
           Px = PAx + Ax
           Py = PAy + Ay
           Pz = PAz + Az
           Sp = ss(irs)
           ooz = rzet(irs)*2.0d0
           oo2zeta = 0.5d0 * ooz

           do itu = npsh2pr(ij),npsh2pr(ij+1)-1
              xiozb = xiozetaa(itu)
              QCx = -xiozb*ABx
              QCy = -xiozb*ABy
              QCz = -xiozb*ABz
              Qx = QCx + Ax
              Qy = QCy + Ay
              Qz = QCz + Az
              Sq = ss(itu)
              ooe = rzet(itu)*2.0d0

              QPx = Qx - Px
              QPy = Qy - Py
              QPz = Qz - Pz
              QP2 = QPx*QPx + QPy*QPy + QPz*QPz
              rho = 1.0d0/(ooz + ooe)
              t = rho * QP2

              if (t <= tf(5)) then
                 it = idint((t + 0.025d0)*20.0d0)
                 dt = 0.05d0*it - t
                 it = it + 1
                 dt4 = dt * 0.25d0
                 dt5 = dt * 0.2d0
                 dt6 = dt * 0.1666666666666666666d0
                 dt2 = dt4 + dt4
                 dt3 = dt6 + dt6
                 ft  = (((((fmt%fdat(11,it)*dt6 + fmt%fdat(10,it))*dt5&
                     + fmt%fdat(9,it))*dt4 + fmt%fdat(8,it))*dt3&
                     + fmt%fdat(7,it))*dt2 + fmt%fdat(6,it))*dt + fmt%fdat(5,it)
                 et  = ((((((ga6*dt + ga5)*dt + ga4)*dt&
                     + ga3)*dt + ga2)*dt + ga1)*dt + ga0) * fmt%edat(it)
                 t2 = 2.d0 * t
                 cn = dsqrt(rho) * tosqrtpi * Sp * Sq
                 etcn = et * cn
                 ssss4 = ft * cn
                 ssss3 = (ssss4 * t2 + etcn) * rminv(4)
                 ssss2 = (ssss3 * t2 + etcn) * rminv(3)
                 ssss1 = (ssss2 * t2 + etcn) * rminv(2)
                 ssss0 =  ssss1 * t2 + etcn
              else
                 tinv = 1.d0/t
                 ssss0 = dsqrt(rho*tinv) * Sp * Sq
                 ssss1 = ssss0 * tinv * 0.5d0
                 ssss2 = ssss1 * tinv * 1.5d0
                 ssss3 = ssss2 * tinv * 2.5d0
                 ssss4 = ssss3 * tinv * 3.5d0
              end if

              rhoozeta = rho*ooz
              rhooeta = rho*ooe
              oo2eta = 0.5d0 * ooe
              rhoo2ze = rhoozeta*oo2eta
              WPx = QPx * rhoozeta
              WPy = QPy * rhoozeta
              WPz = QPz * rhoozeta
              WQx = WPx - QPx
              WQy = WPy - QPy
              WQz = WPz - QPz

              pzsss3 = ssss3*PAz + ssss4*WPz
              pysss3 = ssss3*PAy + ssss4*WPy
              pxsss3 = ssss3*PAx + ssss4*WPx
              ai_ssss23 = oo2zeta*( ssss2 - rhoozeta*ssss3 )
              ak_ssss2 = rhoo2ze*ssss2
              pzsss2 = ssss2*PAz + ssss3*WPz
              ak_pzsss2 = rhoo2ze*pzsss2
              dzzsss2 = pzsss2*PAz + pzsss3*WPz + ai_ssss23
              pysss2 = ssss2*PAy + ssss3*WPy
              ak_pysss2 = rhoo2ze*pysss2
              dyzsss2 = pysss2*PAz + pysss3*WPz
              dyysss2 = pysss2*PAy + pysss3*WPy + ai_ssss23
              pxsss2 = ssss2*PAx + ssss3*WPx
              ak_pxsss2 = rhoo2ze*pxsss2
              dxzsss2 = pxsss2*PAz + pxsss3*WPz
              dxysss2 = pxsss2*PAy + pxsss3*WPy
              dxxsss2 = pxsss2*PAx + pxsss3*WPx + ai_ssss23
              ai_ssss12 = oo2zeta*( ssss1 - rhoozeta*ssss2 )
              ai_ssss01 = oo2zeta*( ssss0 - rhoozeta*ssss1 )
              ak_ssss1 = rhoo2ze*ssss1
              pzsss1 = ssss1*PAz + ssss2*WPz
              ak_pzsss1 = rhoo2ze*pzsss1
              dzzsss1 = pzsss1*PAz + pzsss2*WPz + ai_ssss12
              dzzspzs1 = dzzsss1*QCz + dzzsss2*WQz + ak_pzsss2*2.0d0
              pzspzs1 = pzsss1*QCz + pzsss2*WQz + ak_ssss2
              ak_pzspzs1 = rhoo2ze*pzspzs1
              pzspys1 = pzsss1*QCy + pzsss2*WQy
              pzspxs1 = pzsss1*QCx + pzsss2*WQx
              pysss1 = ssss1*PAy + ssss2*WPy
              ak_pysss1 = rhoo2ze*pysss1
              dyzsss1 = pysss1*PAz + pysss2*WPz
              dyzspys1 = dyzsss1*QCy + dyzsss2*WQy + ak_pzsss2
              dyysss1 = pysss1*PAy + pysss2*WPy + ai_ssss12
              dyyspys1 = dyysss1*QCy + dyysss2*WQy + ak_pysss2*2.0d0
              pyspys1 = pysss1*QCy + pysss2*WQy + ak_ssss2
              ak_pyspys1 = rhoo2ze*pyspys1
              pyspxs1 = pysss1*QCx + pysss2*WQx
              pxsss1 = ssss1*PAx + ssss2*WPx
              ak_pxsss1 = rhoo2ze*pxsss1
              dxzsss1 = pxsss1*PAz + pxsss2*WPz
              dxzspxs1 = dxzsss1*QCx + dxzsss2*WQx + ak_pzsss2
              dxysss1 = pxsss1*PAy + pxsss2*WPy
              dxyspxs1 = dxysss1*QCx + dxysss2*WQx + ak_pysss2
              dxxsss1 = pxsss1*PAx + pxsss2*WPx + ai_ssss12
              dxxspxs1 = dxxsss1*QCx + dxxsss2*WQx + ak_pxsss2*2.0d0
              pxspxs1 = pxsss1*QCx + pxsss2*WQx + ak_ssss2
              ak_pxspxs1 = rhoo2ze*pxspxs1
              pzsss0 = ssss0*PAz + ssss1*WPz
              dzzsss0 = pzsss0*PAz + pzsss1*WPz + ai_ssss01
              ck_dzzsss01 = oo2eta*( dzzsss0 - rhooeta*dzzsss1 )
              dzzspzs0 = dzzsss0*QCz + dzzsss1*WQz + ak_pzsss1*2.0d0
              dzzspzs = dzzspzs + dzzspzs0
              dzzsdzzs0 = dzzspzs0*QCz + dzzspzs1*WQz + ck_dzzsss01 + ak_pzspzs1*2.0d0
              dzzsdzzs = dzzsdzzs + dzzsdzzs0
              ck_pzsss01 = oo2eta*( pzsss0 - rhooeta*pzsss1 )
              pzspzs0 = pzsss0*QCz + pzsss1*WQz + ak_ssss1
              pzspzs = pzspzs + pzspzs0
              pzspys0 = pzsss0*QCy + pzsss1*WQy
              pzspxs0 = pzsss0*QCx + pzsss1*WQx
              pysss0 = ssss0*PAy + ssss1*WPy
              dyzsss0 = pysss0*PAz + pysss1*WPz
              dyzspzs0 = dyzsss0*QCz + dyzsss1*WQz + ak_pysss1
              dyzspzs = dyzspzs + dyzspzs0
              dyzspys0 = dyzsss0*QCy + dyzsss1*WQy + ak_pzsss1
              dyzspys = dyzspys + dyzspys0
              dyzsdyzs0 = dyzspys0*QCz + dyzspys1*WQz + ak_pyspys1
              dyzsdyzs = dyzsdyzs + dyzsdyzs0
              dyysss0 = pysss0*PAy + pysss1*WPy + ai_ssss01
              ck_dyysss01 = oo2eta*( dyysss0 - rhooeta*dyysss1 )
              dyyspys0 = dyysss0*QCy + dyysss1*WQy + ak_pysss1*2.0d0
              dyyspys = dyyspys + dyyspys0
              dyysdyys0 = dyyspys0*QCy + dyyspys1*WQy + ck_dyysss01 + ak_pyspys1*2.0d0
              dyysdyys = dyysdyys + dyysdyys0
              ck_pysss01 = oo2eta*( pysss0 - rhooeta*pysss1 )
              pyspys0 = pysss0*QCy + pysss1*WQy + ak_ssss1
              pyspys = pyspys + pyspys0
              pysdyzs0 = pyspys0*QCz + pyspys1*WQz
              pysdyzs = pysdyzs + pysdyzs0
              pyspxs0 = pysss0*QCx + pysss1*WQx
              pxsss0 = ssss0*PAx + ssss1*WPx
              dxzsss0 = pxsss0*PAz + pxsss1*WPz
              dxzspzs0 = dxzsss0*QCz + dxzsss1*WQz + ak_pxsss1
              dxzspzs = dxzspzs + dxzspzs0
              dxzspxs0 = dxzsss0*QCx + dxzsss1*WQx + ak_pzsss1
              dxzspxs = dxzspxs + dxzspxs0
              dxzsdxzs0 = dxzspxs0*QCz + dxzspxs1*WQz + ak_pxspxs1
              dxzsdxzs = dxzsdxzs + dxzsdxzs0
              dxysss0 = pxsss0*PAy + pxsss1*WPy
              dxyspys0 = dxysss0*QCy + dxysss1*WQy + ak_pxsss1
              dxyspys = dxyspys + dxyspys0
              dxyspxs0 = dxysss0*QCx + dxysss1*WQx + ak_pysss1
              dxyspxs = dxyspxs + dxyspxs0
              dxysdxys0 = dxyspxs0*QCy + dxyspxs1*WQy + ak_pxspxs1
              dxysdxys = dxysdxys + dxysdxys0
              dxxsss0 = pxsss0*PAx + pxsss1*WPx + ai_ssss01
              ck_dxxsss01 = oo2eta*( dxxsss0 - rhooeta*dxxsss1 )
              dxxspxs0 = dxxsss0*QCx + dxxsss1*WQx + ak_pxsss1*2.0d0
              dxxspxs = dxxspxs + dxxspxs0
              dxxsdxxs0 = dxxspxs0*QCx + dxxspxs1*WQx + ck_dxxsss01 + ak_pxspxs1*2.0d0
              dxxsdxxs = dxxsdxxs + dxxsdxxs0
              ck_pxsss01 = oo2eta*( pxsss0 - rhooeta*pxsss1 )
              pxspxs0 = pxsss0*QCx + pxsss1*WQx + ak_ssss1
              pxspxs = pxspxs + pxspxs0
              pxsdxzs0 = pxspxs0*QCz + pxspxs1*WQz
              pxsdxzs = pxsdxzs + pxsdxzs0
              pxsdxys0 = pxspxs0*QCy + pxspxs1*WQy
              pxsdxys = pxsdxys + pxsdxys0
              sspzs1 = ssss1*QCz + ssss2*WQz
              ak_sspzs1 = rhoo2ze*sspzs1
              pzsdzzs0 = pzspzs0*QCz + pzspzs1*WQz + ck_pzsss01 + ak_sspzs1
              pzsdzzs = pzsdzzs + pzsdzzs0
              sspys1 = ssss1*QCy + ssss2*WQy
              ak_sspys1 = rhoo2ze*sspys1
              pzsdyzs0 = pzspys0*QCz + pzspys1*WQz + ak_sspys1
              pzsdyzs = pzsdyzs + pzsdyzs0
              pysdyys0 = pyspys0*QCy + pyspys1*WQy + ck_pysss01 + ak_sspys1
              pysdyys = pysdyys + pysdyys0
              sspxs1 = ssss1*QCx + ssss2*WQx
              ak_sspxs1 = rhoo2ze*sspxs1
              pzsdxzs0 = pzspxs0*QCz + pzspxs1*WQz + ak_sspxs1
              pzsdxzs = pzsdxzs + pzsdxzs0
              pysdxys0 = pyspxs0*QCy + pyspxs1*WQy + ak_sspxs1
              pysdxys = pysdxys + pysdxys0
              pxsdxxs0 = pxspxs0*QCx + pxspxs1*WQx + ck_pxsss01 + ak_sspxs1
              pxsdxxs = pxsdxxs + pxsdxxs0
           end do
        end do

        dyzspzpy = dyzsdyzs + dyzspzs*CDy
        dyzspypz = dyzsdyzs + dyzspys*CDz
        dxzspzpx = dxzsdxzs + dxzspzs*CDx
        dxzspxpz = dxzsdxzs + dxzspxs*CDz
        dxyspypx = dxysdxys + dxyspys*CDx
        dxyspxpy = dxysdxys + dxyspxs*CDy
        dzzspzpz = dzzsdzzs + dzzspzs*CDz
        dyyspypy = dyysdyys + dyyspys*CDy
        dxxspxpx = dxxsdxxs + dxxspxs*CDx
        pzspzpz = pzsdzzs + pzspzs*CDz
        pzpzpzpz = dzzspzpz + pzspzpz*ABz
        pzspzpy = pzsdyzs + pzspzs*CDy
        pzpypzpy = dyzspzpy + pzspzpy*ABy
        pzspzpx = pzsdxzs + pzspzs*CDx
        pzpxpzpx = dxzspzpx + pzspzpx*ABx
        pyspypz = pysdyzs + pyspys*CDz
        pypzpypz = dyzspypz + pyspypz*ABz
        pyspypy = pysdyys + pyspys*CDy
        pypypypy = dyyspypy + pyspypy*ABy
        pyspypx = pysdxys + pyspys*CDx
        pypxpypx = dxyspypx + pyspypx*ABx
        pxspxpz = pxsdxzs + pxspxs*CDz
        pxpzpxpz = dxzspxpz + pxspxpz*ABz
        pxspxpy = pxsdxys + pxspxs*CDy
        pxpypxpy = dxyspxpy + pxspxpy*ABy
        pxspxpx = pxsdxxs + pxspxs*CDx
        pxpxpxpx = dxxspxpx + pxspxpx*ABx
        ec(1) = pxpxpxpx
        ec(2) = pxpypxpy
        ec(3) = pxpzpxpz
        ec(4) = pypxpypx
        ec(5) = pypypypy
        ec(6) = pypzpypz
        ec(7) = pzpxpzpx
        ec(8) = pzpypzpy
        ec(9) = pzpzpzpz
        rsrs(ij) = maxval(dabs(ec(:)))
  end do
!$omp end do
end subroutine os_csi_pppp_os3
!> Cauchy-Schwarz inequality: (ds|ds)
!
! Target class:
!    (ds|ds)
! Applying the horizontal recurrence relation:
! Classes to be formes by contraction of primitive integrals:
!    (ds|ds)
! Applying the vertical recurrence relation:
!    [ds|ds](0)    = VRR{ [ds|ps](0..1) [ds|ss](0..1) [ps|ps](1) }
!    [ds|ps](0..1) = VRR{ [ds|ss](0..2) [ps|ss](1..2) }
!    [ds|ss](0..2) = VRR{ [ps|ss](0..3) [ss|ss](0..3) }
!    [ps|ps](1)    = VRR{ [ps|ss](1..2) [ss|ss](2) }
!    [ps|ss](0..3) = VRR{ [ss|ss](0..4) }
!    [ss|ss](0..4)
!
! References:
!  [1] S.Obara and A.Saika, J.Chem.Phys. 84, 3963 (1986).
!  [2] M.Head-Gordon and J.A.Pople, J.Chem.Phys. 89, 5777 (1988).
!
subroutine os_csi_dsds_os3(&
     rsrs,&
     ishp,npsh,npsh2pr,nsh2atm,nsh2ao,xyz,ss,xiozetaa,rzet,&
     nshell,nshellp,nprimp,natom,&
     fmt)
  use physical_constants_class
  use integral_fmt_class
  use omp_lib

  implicit real(8) (a-h,o-z)

  type(fmt_type),intent(in)::fmt
  integer,intent(in)::nshell,nshellp,nprimp,natom
  real(8),intent(inout)::rsrs(nshellp)
  integer,intent(in)::ishp(6),npsh(2,nshellp),npsh2pr(nshellp+1)
  integer,intent(in)::nsh2atm(nshell),nsh2ao(nshell)
  real(8),intent(in)::ss(nprimp),xiozetaa(nprimp),rzet(nprimp)
  real(8),intent(in)::xyz(3,natom)

  real(8)::ec(6)
!  integer::tid

  tid = omp_get_thread_num() + 1

  tosqrtpi = 2.d0/dsqrt(pi)

!$omp do schedule(dynamic)
!!$!$omp do
  do ij = ishp(3)+1, ishp(4)
     i = npsh(1,ij)
     j = npsh(2,ij)
     iatom = nsh2atm(i)
     jatom = nsh2atm(j)
     Ax = xyz(1,iatom)
     Ay = xyz(2,iatom)
     Az = xyz(3,iatom)
     Bx = xyz(1,jatom)
     By = xyz(2,jatom)
     Bz = xyz(3,jatom)
     ABx = Ax - Bx
     ABy = Ay - By
     ABz = Az - Bz
     CDx = ABx
     CDy = ABy
     CDz = ABz

        ! Initializing VRR variables
        dxxsdxxs = 0.0d0
        dyysdyys = 0.0d0
        dzzsdzzs = 0.0d0
        dxysdxys = 0.0d0
        dxzsdxzs = 0.0d0
        dyzsdyzs = 0.0d0

        do irs = npsh2pr(ij),npsh2pr(ij+1)-1
           xioza = xiozetaa(irs)
           PAx = -xioza*ABx
           PAy = -xioza*ABy
           PAz = -xioza*ABz
           Px = PAx + Ax
           Py = PAy + Ay
           Pz = PAz + Az
           Sp = ss(irs)
           ooz = rzet(irs)*2.0d0
           oo2zeta = 0.5d0 * ooz

           do itu = npsh2pr(ij),npsh2pr(ij+1)-1
              xiozb = xiozetaa(itu)
              QCx = -xiozb*ABx
              QCy = -xiozb*ABy
              QCz = -xiozb*ABz
              Qx = QCx + Ax
              Qy = QCy + Ay
              Qz = QCz + Az
              Sq = ss(itu)
              ooe = rzet(itu)*2.0d0

              QPx = Qx - Px
              QPy = Qy - Py
              QPz = Qz - Pz
              QP2 = QPx*QPx + QPy*QPy + QPz*QPz
              rho = 1.0d0/(ooz + ooe)
              t = rho * QP2

              if (t <= tf(5)) then
                 it = idint((t + 0.025d0)*20.0d0)
                 dt = 0.05d0*it - t
                 it = it + 1
                 dt4 = dt * 0.25d0
                 dt5 = dt * 0.2d0
                 dt6 = dt * 0.1666666666666666666d0
                 dt2 = dt4 + dt4
                 dt3 = dt6 + dt6
                 ft  = (((((fmt%fdat(11,it)*dt6 + fmt%fdat(10,it))*dt5&
                     + fmt%fdat(9,it))*dt4 + fmt%fdat(8,it))*dt3&
                     + fmt%fdat(7,it))*dt2 + fmt%fdat(6,it))*dt + fmt%fdat(5,it)
                 et  = ((((((ga6*dt + ga5)*dt + ga4)*dt&
                     + ga3)*dt + ga2)*dt + ga1)*dt + ga0) * fmt%edat(it)
                 t2 = 2.d0 * t
                 cn = dsqrt(rho) * tosqrtpi * Sp * Sq
                 etcn = et * cn
                 ssss4 = ft * cn
                 ssss3 = (ssss4 * t2 + etcn) * rminv(4)
                 ssss2 = (ssss3 * t2 + etcn) * rminv(3)
                 ssss1 = (ssss2 * t2 + etcn) * rminv(2)
                 ssss0 =  ssss1 * t2 + etcn
              else
                 tinv = 1.d0/t
                 ssss0 = dsqrt(rho*tinv) * Sp * Sq
                 ssss1 = ssss0 * tinv * 0.5d0
                 ssss2 = ssss1 * tinv * 1.5d0
                 ssss3 = ssss2 * tinv * 2.5d0
                 ssss4 = ssss3 * tinv * 3.5d0
              end if

              rhoozeta = rho*ooz
              rhooeta = rho*ooe
              oo2eta = 0.5d0 * ooe
              rhoo2ze = rhoozeta*oo2eta
              WPx = QPx * rhoozeta
              WPy = QPy * rhoozeta
              WPz = QPz * rhoozeta
              WQx = WPx - QPx
              WQy = WPy - QPy
              WQz = WPz - QPz

              pzsss3 = ssss3*PAz + ssss4*WPz
              pysss3 = ssss3*PAy + ssss4*WPy
              pxsss3 = ssss3*PAx + ssss4*WPx
              ai_ssss23 = oo2zeta*( ssss2 - rhoozeta*ssss3 )
              ak_ssss2 = rhoo2ze*ssss2
              pzsss2 = ssss2*PAz + ssss3*WPz
              ak_pzsss2 = rhoo2ze*pzsss2
              dzzsss2 = pzsss2*PAz + pzsss3*WPz + ai_ssss23
              pysss2 = ssss2*PAy + ssss3*WPy
              ak_pysss2 = rhoo2ze*pysss2
              dyzsss2 = pysss2*PAz + pysss3*WPz
              dyysss2 = pysss2*PAy + pysss3*WPy + ai_ssss23
              pxsss2 = ssss2*PAx + ssss3*WPx
              ak_pxsss2 = rhoo2ze*pxsss2
              dxzsss2 = pxsss2*PAz + pxsss3*WPz
              dxysss2 = pxsss2*PAy + pxsss3*WPy
              dxxsss2 = pxsss2*PAx + pxsss3*WPx + ai_ssss23
              ai_ssss12 = oo2zeta*( ssss1 - rhoozeta*ssss2 )
              pzsss1 = ssss1*PAz + ssss2*WPz
              ak_pzsss1 = rhoo2ze*pzsss1
              dzzsss1 = pzsss1*PAz + pzsss2*WPz + ai_ssss12
              dzzspzs1 = dzzsss1*QCz + dzzsss2*WQz + ak_pzsss2*2.0d0
              pzspzs1 = pzsss1*QCz + pzsss2*WQz + ak_ssss2
              ak_pzspzs1 = rhoo2ze*pzspzs1
              pysss1 = ssss1*PAy + ssss2*WPy
              ak_pysss1 = rhoo2ze*pysss1
              dyzsss1 = pysss1*PAz + pysss2*WPz
              dyzspys1 = dyzsss1*QCy + dyzsss2*WQy + ak_pzsss2
              dyysss1 = pysss1*PAy + pysss2*WPy + ai_ssss12
              dyyspys1 = dyysss1*QCy + dyysss2*WQy + ak_pysss2*2.0d0
              pyspys1 = pysss1*QCy + pysss2*WQy + ak_ssss2
              ak_pyspys1 = rhoo2ze*pyspys1
              pxsss1 = ssss1*PAx + ssss2*WPx
              ak_pxsss1 = rhoo2ze*pxsss1
              dxzsss1 = pxsss1*PAz + pxsss2*WPz
              dxzspxs1 = dxzsss1*QCx + dxzsss2*WQx + ak_pzsss2
              dxysss1 = pxsss1*PAy + pxsss2*WPy
              dxyspxs1 = dxysss1*QCx + dxysss2*WQx + ak_pysss2
              dxxsss1 = pxsss1*PAx + pxsss2*WPx + ai_ssss12
              dxxspxs1 = dxxsss1*QCx + dxxsss2*WQx + ak_pxsss2*2.0d0
              pxspxs1 = pxsss1*QCx + pxsss2*WQx + ak_ssss2
              ak_pxspxs1 = rhoo2ze*pxspxs1
              ai_ssss01 = oo2zeta*( ssss0 - rhoozeta*ssss1 )
              pzsss0 = ssss0*PAz + ssss1*WPz
              dzzsss0 = pzsss0*PAz + pzsss1*WPz + ai_ssss01
              ck_dzzsss01 = oo2eta*( dzzsss0 - rhooeta*dzzsss1 )
              dzzspzs0 = dzzsss0*QCz + dzzsss1*WQz + ak_pzsss1*2.0d0
              dzzsdzzs0 = dzzspzs0*QCz + dzzspzs1*WQz + ck_dzzsss01 + ak_pzspzs1*2.0d0
              dzzsdzzs = dzzsdzzs + dzzsdzzs0
              pysss0 = ssss0*PAy + ssss1*WPy
              dyzsss0 = pysss0*PAz + pysss1*WPz
              dyzspys0 = dyzsss0*QCy + dyzsss1*WQy + ak_pzsss1
              dyzsdyzs0 = dyzspys0*QCz + dyzspys1*WQz + ak_pyspys1
              dyzsdyzs = dyzsdyzs + dyzsdyzs0
              dyysss0 = pysss0*PAy + pysss1*WPy + ai_ssss01
              ck_dyysss01 = oo2eta*( dyysss0 - rhooeta*dyysss1 )
              dyyspys0 = dyysss0*QCy + dyysss1*WQy + ak_pysss1*2.0d0
              dyysdyys0 = dyyspys0*QCy + dyyspys1*WQy + ck_dyysss01 + ak_pyspys1*2.0d0
              dyysdyys = dyysdyys + dyysdyys0
              pxsss0 = ssss0*PAx + ssss1*WPx
              dxzsss0 = pxsss0*PAz + pxsss1*WPz
              dxzspxs0 = dxzsss0*QCx + dxzsss1*WQx + ak_pzsss1
              dxzsdxzs0 = dxzspxs0*QCz + dxzspxs1*WQz + ak_pxspxs1
              dxzsdxzs = dxzsdxzs + dxzsdxzs0
              dxysss0 = pxsss0*PAy + pxsss1*WPy
              dxyspxs0 = dxysss0*QCx + dxysss1*WQx + ak_pysss1
              dxysdxys0 = dxyspxs0*QCy + dxyspxs1*WQy + ak_pxspxs1
              dxysdxys = dxysdxys + dxysdxys0
              dxxsss0 = pxsss0*PAx + pxsss1*WPx + ai_ssss01
              ck_dxxsss01 = oo2eta*( dxxsss0 - rhooeta*dxxsss1 )
              dxxspxs0 = dxxsss0*QCx + dxxsss1*WQx + ak_pxsss1*2.0d0
              dxxsdxxs0 = dxxspxs0*QCx + dxxspxs1*WQx + ck_dxxsss01 + ak_pxspxs1*2.0d0
              dxxsdxxs = dxxsdxxs + dxxsdxxs0
           end do
        end do

        ec(1) = dxxsdxxs
        ec(2) = dyysdyys
        ec(3) = dzzsdzzs
        ec(4) = dxysdxys*root3x2
        ec(5) = dxzsdxzs*root3x2
        ec(6) = dyzsdyzs*root3x2
        rsrs(ij) = maxval(dabs(ec(:)))
  end do
!$omp end do
end subroutine os_csi_dsds_os3
!> Cauchy-Schwarz inequality: (dp|dp)
!
! Target class:
!    (dp|dp)
! Applying the horizontal recurrence relation:
!    (dp|dp) = HRR{ (fs|dp) (ds|dp) }
!    (fs|dp) = HRR{ (fs|fs) (fs|ds) }
!    (ds|dp) = HRR{ (ds|fs) (ds|ds) }
! Classes to be formes by contraction of primitive integrals:
!    (ds|ds) (ds|fs) (fs|ds) (fs|fs)
! Applying the vertical recurrence relation:
!    [fs|fs](0)    = VRR{ [fs|ds](0..1) [fs|ps](0..1) [ds|ds](1) }
!    [fs|ds](0..1) = VRR{ [fs|ps](0..2) [fs|ss](0..2) [ds|ps](1..2) }
!    [fs|ps](0..2) = VRR{ [fs|ss](0..3) [ds|ss](1..3) }
!    [fs|ss](0..3) = VRR{ [ds|ss](0..4) [ps|ss](0..4) }
!    [ds|fs](0)    = VRR{ [ds|ds](0..1) [ds|ps](0..1) [ps|ds](1) }
!    [ds|ds](0..1) = VRR{ [ds|ps](0..2) [ds|ss](0..2) [ps|ps](1..2) }
!    [ds|ps](0..2) = VRR{ [ds|ss](0..3) [ps|ss](1..3) }
!    [ds|ss](0..4) = VRR{ [ps|ss](0..5) [ss|ss](0..5) }
!    [ps|ds](1)    = VRR{ [ps|ps](1..2) [ps|ss](1..2) [ss|ps](2) }
!    [ps|ps](1..2) = VRR{ [ps|ss](1..3) [ss|ss](2..3) }
!    [ps|ss](0..5) = VRR{ [ss|ss](0..6) }
!    [ss|ps](2)    = VRR{ [ss|ss](2..3) }
!    [ss|ss](2..6)
!
! References:
!  [1] S.Obara and A.Saika, J.Chem.Phys. 84, 3963 (1986).
!  [2] M.Head-Gordon and J.A.Pople, J.Chem.Phys. 89, 5777 (1988).
!
subroutine os_csi_dpdp_os3(&
     rsrs,&
     ishp,npsh,npsh2pr,nsh2atm,nsh2ao,xyz,ss,xiozetaa,rzet,&
     nshell,nshellp,nprimp,natom,&
     fmt)
  use physical_constants_class
  use integral_fmt_class
  use omp_lib

  implicit real(8) (a-h,o-z)

  type(fmt_type),intent(in)::fmt
  integer,intent(in)::nshell,nshellp,nprimp,natom
  real(8),intent(inout)::rsrs(nshellp)
  integer,intent(in)::ishp(6),npsh(2,nshellp),npsh2pr(nshellp+1)
  integer,intent(in)::nsh2atm(nshell),nsh2ao(nshell)
  real(8),intent(in)::ss(nprimp),xiozetaa(nprimp),rzet(nprimp)
  real(8),intent(in)::xyz(3,natom)

  real(8)::ec(18)
!  integer::tid

  tid = omp_get_thread_num() + 1

  tosqrtpi = 2.d0/dsqrt(pi)

!$omp do schedule(dynamic)
!!$!$omp do
  do ij = ishp(4)+1, ishp(5)
     i = npsh(1,ij)
     j = npsh(2,ij)
     iatom = nsh2atm(i)
     jatom = nsh2atm(j)
     Ax = xyz(1,iatom)
     Ay = xyz(2,iatom)
     Az = xyz(3,iatom)
     Bx = xyz(1,jatom)
     By = xyz(2,jatom)
     Bz = xyz(3,jatom)
     ABx = Ax - Bx
     ABy = Ay - By
     ABz = Az - Bz
     CDx = ABx
     CDy = ABy
     CDz = ABz

        ! Initializing VRR variables
        dxxsfxxxs = 0.0d0
        dxxsdxxs = 0.0d0
        dxxsfxxys = 0.0d0
        dxxsfxxzs = 0.0d0
        dyysfyyxs = 0.0d0
        dyysdyys = 0.0d0
        dyysfyyys = 0.0d0
        dyysfyyzs = 0.0d0
        dzzsfzzxs = 0.0d0
        dzzsdzzs = 0.0d0
        dzzsfzzys = 0.0d0
        dzzsfzzzs = 0.0d0
        dxysfxxys = 0.0d0
        dxysfyyxs = 0.0d0
        dxysdxys = 0.0d0
        dxysfxyzs = 0.0d0
        dxzsfxxzs = 0.0d0
        dxzsfzzxs = 0.0d0
        dxzsfxyzs = 0.0d0
        dxzsdxzs = 0.0d0
        dyzsfyyzs = 0.0d0
        dyzsfzzys = 0.0d0
        dyzsfxyzs = 0.0d0
        dyzsdyzs = 0.0d0
        fxxxsfxxxs = 0.0d0
        fxxxsdxxs = 0.0d0
        fyyysdyys = 0.0d0
        fyyysfyyys = 0.0d0
        fzzzsdzzs = 0.0d0
        fzzzsfzzzs = 0.0d0
        fxxysdxxs = 0.0d0
        fxxysfxxys = 0.0d0
        fxxysdxys = 0.0d0
        fxxzsdxxs = 0.0d0
        fxxzsfxxzs = 0.0d0
        fxxzsdxzs = 0.0d0
        fyyxsfyyxs = 0.0d0
        fyyxsdyys = 0.0d0
        fyyxsdxys = 0.0d0
        fyyzsdyys = 0.0d0
        fyyzsfyyzs = 0.0d0
        fyyzsdyzs = 0.0d0
        fzzxsfzzxs = 0.0d0
        fzzxsdzzs = 0.0d0
        fzzxsdxzs = 0.0d0
        fzzysdzzs = 0.0d0
        fzzysfzzys = 0.0d0
        fzzysdyzs = 0.0d0
        fxyzsdxys = 0.0d0
        fxyzsfxyzs = 0.0d0
        fxyzsdxzs = 0.0d0
        fxyzsdyzs = 0.0d0

        do irs = npsh2pr(ij),npsh2pr(ij+1)-1
           xioza = xiozetaa(irs)
           PAx = -xioza*ABx
           PAy = -xioza*ABy
           PAz = -xioza*ABz
           Px = PAx + Ax
           Py = PAy + Ay
           Pz = PAz + Az
           Sp = ss(irs)
           ooz = rzet(irs)*2.0d0
           oo2zeta = 0.5d0 * ooz

           do itu = npsh2pr(ij),npsh2pr(ij+1)-1
              xiozb = xiozetaa(itu)
              QCx = -xiozb*ABx
              QCy = -xiozb*ABy
              QCz = -xiozb*ABz
              Qx = QCx + Ax
              Qy = QCy + Ay
              Qz = QCz + Az
              Sq = ss(itu)
              ooe = rzet(itu)*2.0d0

              QPx = Qx - Px
              QPy = Qy - Py
              QPz = Qz - Pz
              QP2 = QPx*QPx + QPy*QPy + QPz*QPz
              rho = 1.0d0/(ooz + ooe)
              t = rho * QP2

              if (t <= tf(7)) then
                 it = idint((t + 0.025d0)*20.0d0)
                 dt = 0.05d0*it - t
                 it = it + 1
                 dt4 = dt * 0.25d0
                 dt5 = dt * 0.2d0
                 dt6 = dt * 0.1666666666666666666d0
                 dt2 = dt4 + dt4
                 dt3 = dt6 + dt6
                 ft  = (((((fmt%fdat(13,it)*dt6 + fmt%fdat(12,it))*dt5&
                     + fmt%fdat(11,it))*dt4 + fmt%fdat(10,it))*dt3&
                     + fmt%fdat(9,it))*dt2 + fmt%fdat(8,it))*dt + fmt%fdat(7,it)
                 et  = ((((((ga6*dt + ga5)*dt + ga4)*dt&
                     + ga3)*dt + ga2)*dt + ga1)*dt + ga0) * fmt%edat(it)
                 t2 = 2.d0 * t
                 cn = dsqrt(rho) * tosqrtpi * Sp * Sq
                 etcn = et * cn
                 ssss6 = ft * cn
                 ssss5 = (ssss6 * t2 + etcn) * rminv(6)
                 ssss4 = (ssss5 * t2 + etcn) * rminv(5)
                 ssss3 = (ssss4 * t2 + etcn) * rminv(4)
                 ssss2 = (ssss3 * t2 + etcn) * rminv(3)
                 ssss1 = (ssss2 * t2 + etcn) * rminv(2)
                 ssss0 =  ssss1 * t2 + etcn
              else
                 tinv = 1.d0/t
                 ssss0 = dsqrt(rho*tinv) * Sp * Sq
                 ssss1 = ssss0 * tinv * 0.5d0
                 ssss2 = ssss1 * tinv * 1.5d0
                 ssss3 = ssss2 * tinv * 2.5d0
                 ssss4 = ssss3 * tinv * 3.5d0
                 ssss5 = ssss4 * tinv * 4.5d0
                 ssss6 = ssss5 * tinv * 5.5d0
              end if

              rhoozeta = rho*ooz
              rhooeta = rho*ooe
              oo2eta = 0.5d0 * ooe
              rhoo2ze = rhoozeta*oo2eta
              WPx = QPx * rhoozeta
              WPy = QPy * rhoozeta
              WPz = QPz * rhoozeta
              WQx = WPx - QPx
              WQy = WPy - QPy
              WQz = WPz - QPz

              pzsss5 = ssss5*PAz + ssss6*WPz
              pysss5 = ssss5*PAy + ssss6*WPy
              pxsss5 = ssss5*PAx + ssss6*WPx
              ai_ssss45 = oo2zeta*( ssss4 - rhoozeta*ssss5 )
              pzsss4 = ssss4*PAz + ssss5*WPz
              dzzsss4 = pzsss4*PAz + pzsss5*WPz + ai_ssss45
              pysss4 = ssss4*PAy + ssss5*WPy
              dyysss4 = pysss4*PAy + pysss5*WPy + ai_ssss45
              pxsss4 = ssss4*PAx + ssss5*WPx
              dxysss4 = pxsss4*PAy + pxsss5*WPy
              dxxsss4 = pxsss4*PAx + pxsss5*WPx + ai_ssss45
              ai_ssss01 = oo2zeta*( ssss0 - rhoozeta*ssss1 )
              pzsss0 = ssss0*PAz + ssss1*WPz
              pysss0 = ssss0*PAy + ssss1*WPy
              pxsss0 = ssss0*PAx + ssss1*WPx
              ai_ssss34 = oo2zeta*( ssss3 - rhoozeta*ssss4 )
              ak_ssss3 = rhoo2ze*ssss3
              pzsss3 = ssss3*PAz + ssss4*WPz
              ai_pzsss34 = oo2zeta*( pzsss3 - rhoozeta*pzsss4 )
              ak_pzsss3 = rhoo2ze*pzsss3
              dzzsss3 = pzsss3*PAz + pzsss4*WPz + ai_ssss34
              ak_dzzsss3 = rhoo2ze*dzzsss3
              fzzysss3 = dzzsss3*PAy + dzzsss4*WPy
              fzzxsss3 = dzzsss3*PAx + dzzsss4*WPx
              fzzzsss3 = dzzsss3*PAz + dzzsss4*WPz + ai_pzsss34*2.0d0
              pysss3 = ssss3*PAy + ssss4*WPy
              ai_pysss34 = oo2zeta*( pysss3 - rhoozeta*pysss4 )
              ak_pysss3 = rhoo2ze*pysss3
              dyzsss3 = pysss3*PAz + pysss4*WPz
              ak_dyzsss3 = rhoo2ze*dyzsss3
              dyysss3 = pysss3*PAy + pysss4*WPy + ai_ssss34
              ak_dyysss3 = rhoo2ze*dyysss3
              fyyzsss3 = dyysss3*PAz + dyysss4*WPz
              fyyxsss3 = dyysss3*PAx + dyysss4*WPx
              fyyysss3 = dyysss3*PAy + dyysss4*WPy + ai_pysss34*2.0d0
              pxsss3 = ssss3*PAx + ssss4*WPx
              ai_pxsss34 = oo2zeta*( pxsss3 - rhoozeta*pxsss4 )
              ak_pxsss3 = rhoo2ze*pxsss3
              dxzsss3 = pxsss3*PAz + pxsss4*WPz
              ak_dxzsss3 = rhoo2ze*dxzsss3
              dxysss3 = pxsss3*PAy + pxsss4*WPy
              ak_dxysss3 = rhoo2ze*dxysss3
              fxyzsss3 = dxysss3*PAz + dxysss4*WPz
              dxxsss3 = pxsss3*PAx + pxsss4*WPx + ai_ssss34
              ak_dxxsss3 = rhoo2ze*dxxsss3
              fxxzsss3 = dxxsss3*PAz + dxxsss4*WPz
              fxxysss3 = dxxsss3*PAy + dxxsss4*WPy
              fxxxsss3 = dxxsss3*PAx + dxxsss4*WPx + ai_pxsss34*2.0d0
              ai_ssss23 = oo2zeta*( ssss2 - rhoozeta*ssss3 )
              ai_ssss12 = oo2zeta*( ssss1 - rhoozeta*ssss2 )
              ak_ssss2 = rhoo2ze*ssss2
              pzsss2 = ssss2*PAz + ssss3*WPz
              ai_pzsss23 = oo2zeta*( pzsss2 - rhoozeta*pzsss3 )
              ak_pzsss2 = rhoo2ze*pzsss2
              dzzsss2 = pzsss2*PAz + pzsss3*WPz + ai_ssss23
              ak_dzzsss2 = rhoo2ze*dzzsss2
              fzzysss2 = dzzsss2*PAy + dzzsss3*WPy
              fzzyspzs2 = fzzysss2*QCz + fzzysss3*WQz + ak_dyzsss3*2.0d0
              fzzxsss2 = dzzsss2*PAx + dzzsss3*WPx
              fzzxspzs2 = fzzxsss2*QCz + fzzxsss3*WQz + ak_dxzsss3*2.0d0
              fzzzsss2 = dzzsss2*PAz + dzzsss3*WPz + ai_pzsss23*2.0d0
              fzzzspzs2 = fzzzsss2*QCz + fzzzsss3*WQz + ak_dzzsss3*3.0d0
              dzzspzs2 = dzzsss2*QCz + dzzsss3*WQz + ak_pzsss3*2.0d0
              ak_dzzspzs2 = rhoo2ze*dzzspzs2
              pzspzs2 = pzsss2*QCz + pzsss3*WQz + ak_ssss3
              ak_pzspzs2 = rhoo2ze*pzspzs2
              pzspys2 = pzsss2*QCy + pzsss3*WQy
              ak_pzspys2 = rhoo2ze*pzspys2
              pzspxs2 = pzsss2*QCx + pzsss3*WQx
              ak_pzspxs2 = rhoo2ze*pzspxs2
              pysss2 = ssss2*PAy + ssss3*WPy
              ai_pysss23 = oo2zeta*( pysss2 - rhoozeta*pysss3 )
              ak_pysss2 = rhoo2ze*pysss2
              dyzsss2 = pysss2*PAz + pysss3*WPz
              ak_dyzsss2 = rhoo2ze*dyzsss2
              dyzspzs2 = dyzsss2*QCz + dyzsss3*WQz + ak_pysss3
              ak_dyzspzs2 = rhoo2ze*dyzspzs2
              dyzspys2 = dyzsss2*QCy + dyzsss3*WQy + ak_pzsss3
              ak_dyzspys2 = rhoo2ze*dyzspys2
              dyzspxs2 = dyzsss2*QCx + dyzsss3*WQx
              dyysss2 = pysss2*PAy + pysss3*WPy + ai_ssss23
              ak_dyysss2 = rhoo2ze*dyysss2
              fyyzsss2 = dyysss2*PAz + dyysss3*WPz
              fyyzspys2 = fyyzsss2*QCy + fyyzsss3*WQy + ak_dyzsss3*2.0d0
              fyyxsss2 = dyysss2*PAx + dyysss3*WPx
              fyyxspys2 = fyyxsss2*QCy + fyyxsss3*WQy + ak_dxysss3*2.0d0
              fyyysss2 = dyysss2*PAy + dyysss3*WPy + ai_pysss23*2.0d0
              fyyyspys2 = fyyysss2*QCy + fyyysss3*WQy + ak_dyysss3*3.0d0
              dyyspys2 = dyysss2*QCy + dyysss3*WQy + ak_pysss3*2.0d0
              ak_dyyspys2 = rhoo2ze*dyyspys2
              pyspzs2 = pysss2*QCz + pysss3*WQz
              ak_pyspzs2 = rhoo2ze*pyspzs2
              pyspys2 = pysss2*QCy + pysss3*WQy + ak_ssss3
              ak_pyspys2 = rhoo2ze*pyspys2
              pyspxs2 = pysss2*QCx + pysss3*WQx
              ak_pyspxs2 = rhoo2ze*pyspxs2
              pxsss2 = ssss2*PAx + ssss3*WPx
              ai_pxsss23 = oo2zeta*( pxsss2 - rhoozeta*pxsss3 )
              ak_pxsss2 = rhoo2ze*pxsss2
              dxzsss2 = pxsss2*PAz + pxsss3*WPz
              ak_dxzsss2 = rhoo2ze*dxzsss2
              dxzspzs2 = dxzsss2*QCz + dxzsss3*WQz + ak_pxsss3
              ak_dxzspzs2 = rhoo2ze*dxzspzs2
              dxzspxs2 = dxzsss2*QCx + dxzsss3*WQx + ak_pzsss3
              ak_dxzspxs2 = rhoo2ze*dxzspxs2
              dxysss2 = pxsss2*PAy + pxsss3*WPy
              ak_dxysss2 = rhoo2ze*dxysss2
              fxyzsss2 = dxysss2*PAz + dxysss3*WPz
              fxyzspxs2 = fxyzsss2*QCx + fxyzsss3*WQx + ak_dyzsss3
              dxyspys2 = dxysss2*QCy + dxysss3*WQy + ak_pxsss3
              ak_dxyspys2 = rhoo2ze*dxyspys2
              dxyspxs2 = dxysss2*QCx + dxysss3*WQx + ak_pysss3
              ak_dxyspxs2 = rhoo2ze*dxyspxs2
              dxxsss2 = pxsss2*PAx + pxsss3*WPx + ai_ssss23
              ak_dxxsss2 = rhoo2ze*dxxsss2
              fxxzsss2 = dxxsss2*PAz + dxxsss3*WPz
              fxxzspxs2 = fxxzsss2*QCx + fxxzsss3*WQx + ak_dxzsss3*2.0d0
              fxxysss2 = dxxsss2*PAy + dxxsss3*WPy
              fxxyspxs2 = fxxysss2*QCx + fxxysss3*WQx + ak_dxysss3*2.0d0
              fxxxsss2 = dxxsss2*PAx + dxxsss3*WPx + ai_pxsss23*2.0d0
              fxxxspxs2 = fxxxsss2*QCx + fxxxsss3*WQx + ak_dxxsss3*3.0d0
              dxxspxs2 = dxxsss2*QCx + dxxsss3*WQx + ak_pxsss3*2.0d0
              ak_dxxspxs2 = rhoo2ze*dxxspxs2
              pxspzs2 = pxsss2*QCz + pxsss3*WQz
              ak_pxspzs2 = rhoo2ze*pxspzs2
              pxspys2 = pxsss2*QCy + pxsss3*WQy
              ak_pxspys2 = rhoo2ze*pxspys2
              pxspxs2 = pxsss2*QCx + pxsss3*WQx + ak_ssss3
              ak_pxspxs2 = rhoo2ze*pxspxs2
              pzsss1 = ssss1*PAz + ssss2*WPz
              ai_pzsss12 = oo2zeta*( pzsss1 - rhoozeta*pzsss2 )
              ai_pzsss01 = oo2zeta*( pzsss0 - rhoozeta*pzsss1 )
              ak_pzsss1 = rhoo2ze*pzsss1
              dzzsss1 = pzsss1*PAz + pzsss2*WPz + ai_ssss12
              ak_dzzsss1 = rhoo2ze*dzzsss1
              fzzysss1 = dzzsss1*PAy + dzzsss2*WPy
              ck_fzzysss12 = oo2eta*( fzzysss1 - rhooeta*fzzysss2 )
              fzzyspzs1 = fzzysss1*QCz + fzzysss2*WQz + ak_dyzsss2*2.0d0
              fzzysdzzs1 = fzzyspzs1*QCz + fzzyspzs2*WQz + ck_fzzysss12 + ak_dyzspzs2*2.0d0
              fzzyspys1 = fzzysss1*QCy + fzzysss2*WQy + ak_dzzsss2
              fzzxsss1 = dzzsss1*PAx + dzzsss2*WPx
              ck_fzzxsss12 = oo2eta*( fzzxsss1 - rhooeta*fzzxsss2 )
              fzzxspzs1 = fzzxsss1*QCz + fzzxsss2*WQz + ak_dxzsss2*2.0d0
              fzzxsdzzs1 = fzzxspzs1*QCz + fzzxspzs2*WQz + ck_fzzxsss12 + ak_dxzspzs2*2.0d0
              fzzxspxs1 = fzzxsss1*QCx + fzzxsss2*WQx + ak_dzzsss2
              fzzzsss1 = dzzsss1*PAz + dzzsss2*WPz + ai_pzsss12*2.0d0
              ck_fzzzsss12 = oo2eta*( fzzzsss1 - rhooeta*fzzzsss2 )
              fzzzspzs1 = fzzzsss1*QCz + fzzzsss2*WQz + ak_dzzsss2*3.0d0
              fzzzsdzzs1 = fzzzspzs1*QCz + fzzzspzs2*WQz + ck_fzzzsss12 + ak_dzzspzs2*3.0d0
              ck_dzzsss12 = oo2eta*( dzzsss1 - rhooeta*dzzsss2 )
              dzzspzs1 = dzzsss1*QCz + dzzsss2*WQz + ak_pzsss2*2.0d0
              ak_dzzspzs1 = rhoo2ze*dzzspzs1
              dzzsdzzs1 = dzzspzs1*QCz + dzzspzs2*WQz + ck_dzzsss12 + ak_pzspzs2*2.0d0
              ak_dzzsdzzs1 = rhoo2ze*dzzsdzzs1
              dzzsss0 = pzsss0*PAz + pzsss1*WPz + ai_ssss01
              fzzysss0 = dzzsss0*PAy + dzzsss1*WPy
              ck_fzzysss01 = oo2eta*( fzzysss0 - rhooeta*fzzysss1 )
              fzzyspys0 = fzzysss0*QCy + fzzysss1*WQy + ak_dzzsss1
              fzzxsss0 = dzzsss0*PAx + dzzsss1*WPx
              ck_fzzxsss01 = oo2eta*( fzzxsss0 - rhooeta*fzzxsss1 )
              fzzxspxs0 = fzzxsss0*QCx + fzzxsss1*WQx + ak_dzzsss1
              fzzzsss0 = dzzsss0*PAz + dzzsss1*WPz + ai_pzsss01*2.0d0
              ck_fzzzsss01 = oo2eta*( fzzzsss0 - rhooeta*fzzzsss1 )
              fzzzspzs0 = fzzzsss0*QCz + fzzzsss1*WQz + ak_dzzsss1*3.0d0
              ck_fzzzspzs01 = oo2eta*( fzzzspzs0 - rhooeta*fzzzspzs1 )
              fzzzsdzzs0 = fzzzspzs0*QCz + fzzzspzs1*WQz + ck_fzzzsss01 + ak_dzzspzs1*3.0d0
              fzzzsdzzs = fzzzsdzzs + fzzzsdzzs0
              fzzzsfzzzs0 = fzzzsdzzs0*QCz + fzzzsdzzs1*WQz + ck_fzzzspzs01*2.0d0 + ak_dzzsdzzs1*3.0d0
              fzzzsfzzzs = fzzzsfzzzs + fzzzsfzzzs0
              ck_dzzsss01 = oo2eta*( dzzsss0 - rhooeta*dzzsss1 )
              dzzspzs0 = dzzsss0*QCz + dzzsss1*WQz + ak_pzsss1*2.0d0
              ck_dzzspzs01 = oo2eta*( dzzspzs0 - rhooeta*dzzspzs1 )
              ck_pzsss12 = oo2eta*( pzsss1 - rhooeta*pzsss2 )
              pzspzs1 = pzsss1*QCz + pzsss2*WQz + ak_ssss2
              ak_pzspzs1 = rhoo2ze*pzspzs1
              dzzsdzzs0 = dzzspzs0*QCz + dzzspzs1*WQz + ck_dzzsss01 + ak_pzspzs1*2.0d0
              dzzsdzzs = dzzsdzzs + dzzsdzzs0
              dzzsfzzys0 = dzzsdzzs0*QCy + dzzsdzzs1*WQy
              dzzsfzzys = dzzsfzzys + dzzsfzzys0
              dzzsfzzxs0 = dzzsdzzs0*QCx + dzzsdzzs1*WQx
              dzzsfzzxs = dzzsfzzxs + dzzsfzzxs0
              pzspys1 = pzsss1*QCy + pzsss2*WQy
              ak_pzspys1 = rhoo2ze*pzspys1
              pzspxs1 = pzsss1*QCx + pzsss2*WQx
              ak_pzspxs1 = rhoo2ze*pzspxs1
              pysss1 = ssss1*PAy + ssss2*WPy
              ai_pysss12 = oo2zeta*( pysss1 - rhoozeta*pysss2 )
              ai_pysss01 = oo2zeta*( pysss0 - rhoozeta*pysss1 )
              ak_pysss1 = rhoo2ze*pysss1
              dyzsss1 = pysss1*PAz + pysss2*WPz
              ak_dyzsss1 = rhoo2ze*dyzsss1
              fzzyspzs0 = fzzysss0*QCz + fzzysss1*WQz + ak_dyzsss1*2.0d0
              ck_dyzsss12 = oo2eta*( dyzsss1 - rhooeta*dyzsss2 )
              dyzspzs1 = dyzsss1*QCz + dyzsss2*WQz + ak_pysss2
              ak_dyzspzs1 = rhoo2ze*dyzspzs1
              fzzysdzzs0 = fzzyspzs0*QCz + fzzyspzs1*WQz + ck_fzzysss01 + ak_dyzspzs1*2.0d0
              fzzysdzzs = fzzysdzzs + fzzysdzzs0
              fzzysfzzys0 = fzzysdzzs0*QCy + fzzysdzzs1*WQy + ak_dzzsdzzs1
              fzzysfzzys = fzzysfzzys + fzzysfzzys0
              dyzsdzzs1 = dyzspzs1*QCz + dyzspzs2*WQz + ck_dyzsss12 + ak_pyspzs2
              dyzspys1 = dyzsss1*QCy + dyzsss2*WQy + ak_pzsss2
              ak_dyzspys1 = rhoo2ze*dyzspys1
              fzzysdyzs0 = fzzyspys0*QCz + fzzyspys1*WQz + ak_dyzspys1*2.0d0
              fzzysdyzs = fzzysdyzs + fzzysdyzs0
              dyzsdyys1 = dyzspys1*QCy + dyzspys2*WQy + ck_dyzsss12 + ak_pzspys2
              dyzspxs1 = dyzsss1*QCx + dyzsss2*WQx
              dyzsdxys1 = dyzspxs1*QCy + dyzspxs2*WQy + ak_pzspxs2
              dyysss1 = pysss1*PAy + pysss2*WPy + ai_ssss12
              ak_dyysss1 = rhoo2ze*dyysss1
              fyyzsss1 = dyysss1*PAz + dyysss2*WPz
              ck_fyyzsss12 = oo2eta*( fyyzsss1 - rhooeta*fyyzsss2 )
              fyyzspys1 = fyyzsss1*QCy + fyyzsss2*WQy + ak_dyzsss2*2.0d0
              fyyzsdyys1 = fyyzspys1*QCy + fyyzspys2*WQy + ck_fyyzsss12 + ak_dyzspys2*2.0d0
              fyyxsss1 = dyysss1*PAx + dyysss2*WPx
              ck_fyyxsss12 = oo2eta*( fyyxsss1 - rhooeta*fyyxsss2 )
              fyyxspys1 = fyyxsss1*QCy + fyyxsss2*WQy + ak_dxysss2*2.0d0
              fyyxsdyys1 = fyyxspys1*QCy + fyyxspys2*WQy + ck_fyyxsss12 + ak_dxyspys2*2.0d0
              fyyxspxs1 = fyyxsss1*QCx + fyyxsss2*WQx + ak_dyysss2
              fyyysss1 = dyysss1*PAy + dyysss2*WPy + ai_pysss12*2.0d0
              ck_fyyysss12 = oo2eta*( fyyysss1 - rhooeta*fyyysss2 )
              fyyyspys1 = fyyysss1*QCy + fyyysss2*WQy + ak_dyysss2*3.0d0
              fyyysdyys1 = fyyyspys1*QCy + fyyyspys2*WQy + ck_fyyysss12 + ak_dyyspys2*3.0d0
              ck_dyysss12 = oo2eta*( dyysss1 - rhooeta*dyysss2 )
              dyyspys1 = dyysss1*QCy + dyysss2*WQy + ak_pysss2*2.0d0
              ak_dyyspys1 = rhoo2ze*dyyspys1
              dyysdyys1 = dyyspys1*QCy + dyyspys2*WQy + ck_dyysss12 + ak_pyspys2*2.0d0
              ak_dyysdyys1 = rhoo2ze*dyysdyys1
              dyzsss0 = pysss0*PAz + pysss1*WPz
              ck_dyzsss01 = oo2eta*( dyzsss0 - rhooeta*dyzsss1 )
              dyzspzs0 = dyzsss0*QCz + dyzsss1*WQz + ak_pysss1
              dyzspys0 = dyzsss0*QCy + dyzsss1*WQy + ak_pzsss1
              dyzsdyys0 = dyzspys0*QCy + dyzspys1*WQy + ck_dyzsss01 + ak_pzspys1
              dyzspxs0 = dyzsss0*QCx + dyzsss1*WQx
              dyzsdxys0 = dyzspxs0*QCy + dyzspxs1*WQy + ak_pzspxs1
              dyysss0 = pysss0*PAy + pysss1*WPy + ai_ssss01
              fyyzsss0 = dyysss0*PAz + dyysss1*WPz
              ck_fyyzsss01 = oo2eta*( fyyzsss0 - rhooeta*fyyzsss1 )
              fyyzspys0 = fyyzsss0*QCy + fyyzsss1*WQy + ak_dyzsss1*2.0d0
              fyyzsdyzs0 = fyyzspys0*QCz + fyyzspys1*WQz + ak_dyyspys1
              fyyzsdyzs = fyyzsdyzs + fyyzsdyzs0
              fyyzsdyys0 = fyyzspys0*QCy + fyyzspys1*WQy + ck_fyyzsss01 + ak_dyzspys1*2.0d0
              fyyzsdyys = fyyzsdyys + fyyzsdyys0
              fyyzsfyyzs0 = fyyzsdyys0*QCz + fyyzsdyys1*WQz + ak_dyysdyys1
              fyyzsfyyzs = fyyzsfyyzs + fyyzsfyyzs0
              fyyxsss0 = dyysss0*PAx + dyysss1*WPx
              ck_fyyxsss01 = oo2eta*( fyyxsss0 - rhooeta*fyyxsss1 )
              fyyxspxs0 = fyyxsss0*QCx + fyyxsss1*WQx + ak_dyysss1
              fyyysss0 = dyysss0*PAy + dyysss1*WPy + ai_pysss01*2.0d0
              ck_fyyysss01 = oo2eta*( fyyysss0 - rhooeta*fyyysss1 )
              fyyyspys0 = fyyysss0*QCy + fyyysss1*WQy + ak_dyysss1*3.0d0
              ck_fyyyspys01 = oo2eta*( fyyyspys0 - rhooeta*fyyyspys1 )
              fyyysdyys0 = fyyyspys0*QCy + fyyyspys1*WQy + ck_fyyysss01 + ak_dyyspys1*3.0d0
              fyyysdyys = fyyysdyys + fyyysdyys0
              fyyysfyyys0 = fyyysdyys0*QCy + fyyysdyys1*WQy + ck_fyyyspys01*2.0d0 + ak_dyysdyys1*3.0d0
              fyyysfyyys = fyyysfyyys + fyyysfyyys0
              ck_dyysss01 = oo2eta*( dyysss0 - rhooeta*dyysss1 )
              dyyspys0 = dyysss0*QCy + dyysss1*WQy + ak_pysss1*2.0d0
              ck_dyyspys01 = oo2eta*( dyyspys0 - rhooeta*dyyspys1 )
              ck_pysss12 = oo2eta*( pysss1 - rhooeta*pysss2 )
              pyspzs1 = pysss1*QCz + pysss2*WQz
              ak_pyspzs1 = rhoo2ze*pyspzs1
              dyzsdzzs0 = dyzspzs0*QCz + dyzspzs1*WQz + ck_dyzsss01 + ak_pyspzs1
              pyspys1 = pysss1*QCy + pysss2*WQy + ak_ssss2
              ak_pyspys1 = rhoo2ze*pyspys1
              dyzsdyzs0 = dyzspys0*QCz + dyzspys1*WQz + ak_pyspys1
              dyzsdyzs = dyzsdyzs + dyzsdyzs0
              dyysdyys0 = dyyspys0*QCy + dyyspys1*WQy + ck_dyysss01 + ak_pyspys1*2.0d0
              dyysdyys = dyysdyys + dyysdyys0
              dyysfyyzs0 = dyysdyys0*QCz + dyysdyys1*WQz
              dyysfyyzs = dyysfyyzs + dyysfyyzs0
              dyysfyyxs0 = dyysdyys0*QCx + dyysdyys1*WQx
              dyysfyyxs = dyysfyyxs + dyysfyyxs0
              pyspxs1 = pysss1*QCx + pysss2*WQx
              ak_pyspxs1 = rhoo2ze*pyspxs1
              pxsss1 = ssss1*PAx + ssss2*WPx
              ai_pxsss12 = oo2zeta*( pxsss1 - rhoozeta*pxsss2 )
              ai_pxsss01 = oo2zeta*( pxsss0 - rhoozeta*pxsss1 )
              ak_pxsss1 = rhoo2ze*pxsss1
              dxzsss1 = pxsss1*PAz + pxsss2*WPz
              ak_dxzsss1 = rhoo2ze*dxzsss1
              fzzxspzs0 = fzzxsss0*QCz + fzzxsss1*WQz + ak_dxzsss1*2.0d0
              ck_dxzsss12 = oo2eta*( dxzsss1 - rhooeta*dxzsss2 )
              dxzspzs1 = dxzsss1*QCz + dxzsss2*WQz + ak_pxsss2
              ak_dxzspzs1 = rhoo2ze*dxzspzs1
              fzzxsdzzs0 = fzzxspzs0*QCz + fzzxspzs1*WQz + ck_fzzxsss01 + ak_dxzspzs1*2.0d0
              fzzxsdzzs = fzzxsdzzs + fzzxsdzzs0
              fzzxsfzzxs0 = fzzxsdzzs0*QCx + fzzxsdzzs1*WQx + ak_dzzsdzzs1
              fzzxsfzzxs = fzzxsfzzxs + fzzxsfzzxs0
              dxzsdzzs1 = dxzspzs1*QCz + dxzspzs2*WQz + ck_dxzsss12 + ak_pxspzs2
              dxzspxs1 = dxzsss1*QCx + dxzsss2*WQx + ak_pzsss2
              ak_dxzspxs1 = rhoo2ze*dxzspxs1
              fzzxsdxzs0 = fzzxspxs0*QCz + fzzxspxs1*WQz + ak_dxzspxs1*2.0d0
              fzzxsdxzs = fzzxsdxzs + fzzxsdxzs0
              dxzsdxys1 = dxzspxs1*QCy + dxzspxs2*WQy
              dxzsdxxs1 = dxzspxs1*QCx + dxzspxs2*WQx + ck_dxzsss12 + ak_pzspxs2
              dxysss1 = pxsss1*PAy + pxsss2*WPy
              ak_dxysss1 = rhoo2ze*dxysss1
              fyyxspys0 = fyyxsss0*QCy + fyyxsss1*WQy + ak_dxysss1*2.0d0
              fxyzsss1 = dxysss1*PAz + dxysss2*WPz
              fxyzspys1 = fxyzsss1*QCy + fxyzsss2*WQy + ak_dxzsss2
              fxyzspxs1 = fxyzsss1*QCx + fxyzsss2*WQx + ak_dyzsss2
              fxyzsdxys1 = fxyzspxs1*QCy + fxyzspxs2*WQy + ak_dxzspxs2
              ck_dxysss12 = oo2eta*( dxysss1 - rhooeta*dxysss2 )
              dxyspys1 = dxysss1*QCy + dxysss2*WQy + ak_pxsss2
              ak_dxyspys1 = rhoo2ze*dxyspys1
              fyyxsdyys0 = fyyxspys0*QCy + fyyxspys1*WQy + ck_fyyxsss01 + ak_dxyspys1*2.0d0
              fyyxsdyys = fyyxsdyys + fyyxsdyys0
              fyyxsfyyxs0 = fyyxsdyys0*QCx + fyyxsdyys1*WQx + ak_dyysdyys1
              fyyxsfyyxs = fyyxsfyyxs + fyyxsfyyxs0
              dxysdyys1 = dxyspys1*QCy + dxyspys2*WQy + ck_dxysss12 + ak_pxspys2
              dxyspxs1 = dxysss1*QCx + dxysss2*WQx + ak_pysss2
              ak_dxyspxs1 = rhoo2ze*dxyspxs1
              fyyxsdxys0 = fyyxspxs0*QCy + fyyxspxs1*WQy + ak_dxyspxs1*2.0d0
              fyyxsdxys = fyyxsdxys + fyyxsdxys0
              dxysdxys1 = dxyspxs1*QCy + dxyspxs2*WQy + ak_pxspxs2
              ak_dxysdxys1 = rhoo2ze*dxysdxys1
              dxysdxxs1 = dxyspxs1*QCx + dxyspxs2*WQx + ck_dxysss12 + ak_pyspxs2
              dxxsss1 = pxsss1*PAx + pxsss2*WPx + ai_ssss12
              ak_dxxsss1 = rhoo2ze*dxxsss1
              fxxzsss1 = dxxsss1*PAz + dxxsss2*WPz
              ck_fxxzsss12 = oo2eta*( fxxzsss1 - rhooeta*fxxzsss2 )
              fxxzspxs1 = fxxzsss1*QCx + fxxzsss2*WQx + ak_dxzsss2*2.0d0
              fxxzsdxxs1 = fxxzspxs1*QCx + fxxzspxs2*WQx + ck_fxxzsss12 + ak_dxzspxs2*2.0d0
              fxxysss1 = dxxsss1*PAy + dxxsss2*WPy
              ck_fxxysss12 = oo2eta*( fxxysss1 - rhooeta*fxxysss2 )
              fxxyspxs1 = fxxysss1*QCx + fxxysss2*WQx + ak_dxysss2*2.0d0
              fxxysdxxs1 = fxxyspxs1*QCx + fxxyspxs2*WQx + ck_fxxysss12 + ak_dxyspxs2*2.0d0
              fxxxsss1 = dxxsss1*PAx + dxxsss2*WPx + ai_pxsss12*2.0d0
              ck_fxxxsss12 = oo2eta*( fxxxsss1 - rhooeta*fxxxsss2 )
              fxxxspxs1 = fxxxsss1*QCx + fxxxsss2*WQx + ak_dxxsss2*3.0d0
              fxxxsdxxs1 = fxxxspxs1*QCx + fxxxspxs2*WQx + ck_fxxxsss12 + ak_dxxspxs2*3.0d0
              ck_dxxsss12 = oo2eta*( dxxsss1 - rhooeta*dxxsss2 )
              dxxspxs1 = dxxsss1*QCx + dxxsss2*WQx + ak_pxsss2*2.0d0
              ak_dxxspxs1 = rhoo2ze*dxxspxs1
              dxxsdxxs1 = dxxspxs1*QCx + dxxspxs2*WQx + ck_dxxsss12 + ak_pxspxs2*2.0d0
              ak_dxxsdxxs1 = rhoo2ze*dxxsdxxs1
              dxzsss0 = pxsss0*PAz + pxsss1*WPz
              ck_dxzsss01 = oo2eta*( dxzsss0 - rhooeta*dxzsss1 )
              dxzspzs0 = dxzsss0*QCz + dxzsss1*WQz + ak_pxsss1
              dxzspxs0 = dxzsss0*QCx + dxzsss1*WQx + ak_pzsss1
              dxzsdxys0 = dxzspxs0*QCy + dxzspxs1*WQy
              dxzsdxxs0 = dxzspxs0*QCx + dxzspxs1*WQx + ck_dxzsss01 + ak_pzspxs1
              dxysss0 = pxsss0*PAy + pxsss1*WPy
              fxyzsss0 = dxysss0*PAz + dxysss1*WPz
              fxyzspys0 = fxyzsss0*QCy + fxyzsss1*WQy + ak_dxzsss1
              fxyzsdyzs0 = fxyzspys0*QCz + fxyzspys1*WQz + ak_dxyspys1
              fxyzsdyzs = fxyzsdyzs + fxyzsdyzs0
              fxyzspxs0 = fxyzsss0*QCx + fxyzsss1*WQx + ak_dyzsss1
              fxyzsdxzs0 = fxyzspxs0*QCz + fxyzspxs1*WQz + ak_dxyspxs1
              fxyzsdxzs = fxyzsdxzs + fxyzsdxzs0
              fxyzsdxys0 = fxyzspxs0*QCy + fxyzspxs1*WQy + ak_dxzspxs1
              fxyzsdxys = fxyzsdxys + fxyzsdxys0
              fxyzsfxyzs0 = fxyzsdxys0*QCz + fxyzsdxys1*WQz + ak_dxysdxys1
              fxyzsfxyzs = fxyzsfxyzs + fxyzsfxyzs0
              ck_dxysss01 = oo2eta*( dxysss0 - rhooeta*dxysss1 )
              dxyspys0 = dxysss0*QCy + dxysss1*WQy + ak_pxsss1
              dxyspxs0 = dxysss0*QCx + dxysss1*WQx + ak_pysss1
              dxysdxxs0 = dxyspxs0*QCx + dxyspxs1*WQx + ck_dxysss01 + ak_pyspxs1
              dxxsss0 = pxsss0*PAx + pxsss1*WPx + ai_ssss01
              fxxzsss0 = dxxsss0*PAz + dxxsss1*WPz
              ck_fxxzsss01 = oo2eta*( fxxzsss0 - rhooeta*fxxzsss1 )
              fxxzspxs0 = fxxzsss0*QCx + fxxzsss1*WQx + ak_dxzsss1*2.0d0
              fxxzsdxzs0 = fxxzspxs0*QCz + fxxzspxs1*WQz + ak_dxxspxs1
              fxxzsdxzs = fxxzsdxzs + fxxzsdxzs0
              fxxzsdxxs0 = fxxzspxs0*QCx + fxxzspxs1*WQx + ck_fxxzsss01 + ak_dxzspxs1*2.0d0
              fxxzsdxxs = fxxzsdxxs + fxxzsdxxs0
              fxxzsfxxzs0 = fxxzsdxxs0*QCz + fxxzsdxxs1*WQz + ak_dxxsdxxs1
              fxxzsfxxzs = fxxzsfxxzs + fxxzsfxxzs0
              fxxysss0 = dxxsss0*PAy + dxxsss1*WPy
              ck_fxxysss01 = oo2eta*( fxxysss0 - rhooeta*fxxysss1 )
              fxxyspxs0 = fxxysss0*QCx + fxxysss1*WQx + ak_dxysss1*2.0d0
              fxxysdxys0 = fxxyspxs0*QCy + fxxyspxs1*WQy + ak_dxxspxs1
              fxxysdxys = fxxysdxys + fxxysdxys0
              fxxysdxxs0 = fxxyspxs0*QCx + fxxyspxs1*WQx + ck_fxxysss01 + ak_dxyspxs1*2.0d0
              fxxysdxxs = fxxysdxxs + fxxysdxxs0
              fxxysfxxys0 = fxxysdxxs0*QCy + fxxysdxxs1*WQy + ak_dxxsdxxs1
              fxxysfxxys = fxxysfxxys + fxxysfxxys0
              fxxxsss0 = dxxsss0*PAx + dxxsss1*WPx + ai_pxsss01*2.0d0
              ck_fxxxsss01 = oo2eta*( fxxxsss0 - rhooeta*fxxxsss1 )
              fxxxspxs0 = fxxxsss0*QCx + fxxxsss1*WQx + ak_dxxsss1*3.0d0
              ck_fxxxspxs01 = oo2eta*( fxxxspxs0 - rhooeta*fxxxspxs1 )
              fxxxsdxxs0 = fxxxspxs0*QCx + fxxxspxs1*WQx + ck_fxxxsss01 + ak_dxxspxs1*3.0d0
              fxxxsdxxs = fxxxsdxxs + fxxxsdxxs0
              fxxxsfxxxs0 = fxxxsdxxs0*QCx + fxxxsdxxs1*WQx + ck_fxxxspxs01*2.0d0 + ak_dxxsdxxs1*3.0d0
              fxxxsfxxxs = fxxxsfxxxs + fxxxsfxxxs0
              ck_dxxsss01 = oo2eta*( dxxsss0 - rhooeta*dxxsss1 )
              dxxspxs0 = dxxsss0*QCx + dxxsss1*WQx + ak_pxsss1*2.0d0
              ck_dxxspxs01 = oo2eta*( dxxspxs0 - rhooeta*dxxspxs1 )
              ck_pxsss12 = oo2eta*( pxsss1 - rhooeta*pxsss2 )
              pxspzs1 = pxsss1*QCz + pxsss2*WQz
              ak_pxspzs1 = rhoo2ze*pxspzs1
              dxzsdzzs0 = dxzspzs0*QCz + dxzspzs1*WQz + ck_dxzsss01 + ak_pxspzs1
              pxspys1 = pxsss1*QCy + pxsss2*WQy
              ak_pxspys1 = rhoo2ze*pxspys1
              dxysdyys0 = dxyspys0*QCy + dxyspys1*WQy + ck_dxysss01 + ak_pxspys1
              pxspxs1 = pxsss1*QCx + pxsss2*WQx + ak_ssss2
              ak_pxspxs1 = rhoo2ze*pxspxs1
              dxzsdxzs0 = dxzspxs0*QCz + dxzspxs1*WQz + ak_pxspxs1
              dxzsdxzs = dxzsdxzs + dxzsdxzs0
              dxysdxys0 = dxyspxs0*QCy + dxyspxs1*WQy + ak_pxspxs1
              dxysdxys = dxysdxys + dxysdxys0
              dxysfxyzs0 = dxysdxys0*QCz + dxysdxys1*WQz
              dxysfxyzs = dxysfxyzs + dxysfxyzs0
              dxxsdxxs0 = dxxspxs0*QCx + dxxspxs1*WQx + ck_dxxsss01 + ak_pxspxs1*2.0d0
              dxxsdxxs = dxxsdxxs + dxxsdxxs0
              dxxsfxxzs0 = dxxsdxxs0*QCz + dxxsdxxs1*WQz
              dxxsfxxzs = dxxsfxxzs + dxxsfxxzs0
              dxxsfxxys0 = dxxsdxxs0*QCy + dxxsdxxs1*WQy
              dxxsfxxys = dxxsfxxys + dxxsfxxys0
              pxsdxys1 = pxspxs1*QCy + pxspxs2*WQy
              ak_pxsdxys1 = rhoo2ze*pxsdxys1
              dxzsfxyzs0 = dxzsdxys0*QCz + dxzsdxys1*WQz + ak_pxsdxys1
              dxzsfxyzs = dxzsfxyzs + dxzsfxyzs0
              sspzs2 = ssss2*QCz + ssss3*WQz
              ak_sspzs2 = rhoo2ze*sspzs2
              pzsdzzs1 = pzspzs1*QCz + pzspzs2*WQz + ck_pzsss12 + ak_sspzs2
              ak_pzsdzzs1 = rhoo2ze*pzsdzzs1
              dyzsfzzys0 = dyzsdzzs0*QCy + dyzsdzzs1*WQy + ak_pzsdzzs1
              dyzsfzzys = dyzsfzzys + dyzsfzzys0
              dxzsfzzxs0 = dxzsdzzs0*QCx + dxzsdzzs1*WQx + ak_pzsdzzs1
              dxzsfzzxs = dxzsfzzxs + dxzsfzzxs0
              dzzsfzzzs0 = dzzsdzzs0*QCz + dzzsdzzs1*WQz + ck_dzzspzs01*2.0d0 + ak_pzsdzzs1*2.0d0
              dzzsfzzzs = dzzsfzzzs + dzzsfzzzs0
              sspys2 = ssss2*QCy + ssss3*WQy
              ak_sspys2 = rhoo2ze*sspys2
              pysdyys1 = pyspys1*QCy + pyspys2*WQy + ck_pysss12 + ak_sspys2
              ak_pysdyys1 = rhoo2ze*pysdyys1
              dyzsfyyzs0 = dyzsdyys0*QCz + dyzsdyys1*WQz + ak_pysdyys1
              dyzsfyyzs = dyzsfyyzs + dyzsfyyzs0
              dxysfyyxs0 = dxysdyys0*QCx + dxysdyys1*WQx + ak_pysdyys1
              dxysfyyxs = dxysfyyxs + dxysfyyxs0
              dyysfyyys0 = dyysdyys0*QCy + dyysdyys1*WQy + ck_dyyspys01*2.0d0 + ak_pysdyys1*2.0d0
              dyysfyyys = dyysfyyys + dyysfyyys0
              sspxs2 = ssss2*QCx + ssss3*WQx
              ak_sspxs2 = rhoo2ze*sspxs2
              pysdxys1 = pyspxs1*QCy + pyspxs2*WQy + ak_sspxs2
              ak_pysdxys1 = rhoo2ze*pysdxys1
              dyzsfxyzs0 = dyzsdxys0*QCz + dyzsdxys1*WQz + ak_pysdxys1
              dyzsfxyzs = dyzsfxyzs + dyzsfxyzs0
              pxsdxxs1 = pxspxs1*QCx + pxspxs2*WQx + ck_pxsss12 + ak_sspxs2
              ak_pxsdxxs1 = rhoo2ze*pxsdxxs1
              dxzsfxxzs0 = dxzsdxxs0*QCz + dxzsdxxs1*WQz + ak_pxsdxxs1
              dxzsfxxzs = dxzsfxxzs + dxzsfxxzs0
              dxysfxxys0 = dxysdxxs0*QCy + dxysdxxs1*WQy + ak_pxsdxxs1
              dxysfxxys = dxysfxxys + dxysfxxys0
              dxxsfxxxs0 = dxxsdxxs0*QCx + dxxsdxxs1*WQx + ck_dxxspxs01*2.0d0 + ak_pxsdxxs1*2.0d0
              dxxsfxxxs = dxxsfxxxs + dxxsfxxxs0
           end do
        end do

        fxyzsdyzpx = fxyzsfxyzs + fxyzsdyzs*CDx
        fxyzsdxzpy = fxyzsfxyzs + fxyzsdxzs*CDy
        fxyzsdxypz = fxyzsfxyzs + fxyzsdxys*CDz
        fzzysdyzpz = fzzysfzzys + fzzysdyzs*CDz
        fzzysdzzpy = fzzysfzzys + fzzysdzzs*CDy
        fzzxsdxzpz = fzzxsfzzxs + fzzxsdxzs*CDz
        fzzxsdzzpx = fzzxsfzzxs + fzzxsdzzs*CDx
        fyyzsdyzpy = fyyzsfyyzs + fyyzsdyzs*CDy
        fyyzsdyypz = fyyzsfyyzs + fyyzsdyys*CDz
        fyyxsdxypy = fyyxsfyyxs + fyyxsdxys*CDy
        fyyxsdyypx = fyyxsfyyxs + fyyxsdyys*CDx
        fxxzsdxzpx = fxxzsfxxzs + fxxzsdxzs*CDx
        fxxzsdxxpz = fxxzsfxxzs + fxxzsdxxs*CDz
        fxxysdxypx = fxxysfxxys + fxxysdxys*CDx
        fxxysdxxpy = fxxysfxxys + fxxysdxxs*CDy
        fzzzsdzzpz = fzzzsfzzzs + fzzzsdzzs*CDz
        fyyysdyypy = fyyysfyyys + fyyysdyys*CDy
        fxxxsdxxpx = fxxxsfxxxs + fxxxsdxxs*CDx
        dyzsdyzpx = dyzsfxyzs + dyzsdyzs*CDx
        dyzpxdyzpx = fxyzsdyzpx + dyzsdyzpx*ABx
        dyzsdyzpz = dyzsfzzys + dyzsdyzs*CDz
        dyzpzdyzpz = fzzysdyzpz + dyzsdyzpz*ABz
        dyzsdyzpy = dyzsfyyzs + dyzsdyzs*CDy
        dyzpydyzpy = fyyzsdyzpy + dyzsdyzpy*ABy
        dxzsdxzpy = dxzsfxyzs + dxzsdxzs*CDy
        dxzpydxzpy = fxyzsdxzpy + dxzsdxzpy*ABy
        dxzsdxzpz = dxzsfzzxs + dxzsdxzs*CDz
        dxzpzdxzpz = fzzxsdxzpz + dxzsdxzpz*ABz
        dxzsdxzpx = dxzsfxxzs + dxzsdxzs*CDx
        dxzpxdxzpx = fxxzsdxzpx + dxzsdxzpx*ABx
        dxysdxypz = dxysfxyzs + dxysdxys*CDz
        dxypzdxypz = fxyzsdxypz + dxysdxypz*ABz
        dxysdxypy = dxysfyyxs + dxysdxys*CDy
        dxypydxypy = fyyxsdxypy + dxysdxypy*ABy
        dxysdxypx = dxysfxxys + dxysdxys*CDx
        dxypxdxypx = fxxysdxypx + dxysdxypx*ABx
        dzzsdzzpz = dzzsfzzzs + dzzsdzzs*CDz
        dzzpzdzzpz = fzzzsdzzpz + dzzsdzzpz*ABz
        dzzsdzzpy = dzzsfzzys + dzzsdzzs*CDy
        dzzpydzzpy = fzzysdzzpy + dzzsdzzpy*ABy
        dzzsdzzpx = dzzsfzzxs + dzzsdzzs*CDx
        dzzpxdzzpx = fzzxsdzzpx + dzzsdzzpx*ABx
        dyysdyypz = dyysfyyzs + dyysdyys*CDz
        dyypzdyypz = fyyzsdyypz + dyysdyypz*ABz
        dyysdyypy = dyysfyyys + dyysdyys*CDy
        dyypydyypy = fyyysdyypy + dyysdyypy*ABy
        dyysdyypx = dyysfyyxs + dyysdyys*CDx
        dyypxdyypx = fyyxsdyypx + dyysdyypx*ABx
        dxxsdxxpz = dxxsfxxzs + dxxsdxxs*CDz
        dxxpzdxxpz = fxxzsdxxpz + dxxsdxxpz*ABz
        dxxsdxxpy = dxxsfxxys + dxxsdxxs*CDy
        dxxpydxxpy = fxxysdxxpy + dxxsdxxpy*ABy
        dxxsdxxpx = dxxsfxxxs + dxxsdxxs*CDx
        dxxpxdxxpx = fxxxsdxxpx + dxxsdxxpx*ABx
        ec(1) = dxxpxdxxpx
        ec(2) = dxxpydxxpy
        ec(3) = dxxpzdxxpz
        ec(4) = dyypxdyypx
        ec(5) = dyypydyypy
        ec(6) = dyypzdyypz
        ec(7) = dzzpxdzzpx
        ec(8) = dzzpydzzpy
        ec(9) = dzzpzdzzpz
        ec(10) = dxypxdxypx*root3x2
        ec(11) = dxypydxypy*root3x2
        ec(12) = dxypzdxypz*root3x2
        ec(13) = dxzpxdxzpx*root3x2
        ec(14) = dxzpydxzpy*root3x2
        ec(15) = dxzpzdxzpz*root3x2
        ec(16) = dyzpxdyzpx*root3x2
        ec(17) = dyzpydyzpy*root3x2
        ec(18) = dyzpzdyzpz*root3x2
        rsrs(ij) = maxval(dabs(ec(:)))
  end do
!$omp end do
end subroutine os_csi_dpdp_os3
!> Cauchy-Schwarz inequality: (dd|dd)
!
! Target class:
!    (dd|dd)
! Applying the horizontal recurrence relation:
!    (dd|dd) = HRR{ (fp|dd) (dp|dd) }
!    (fp|dd) = HRR{ (gs|dd) (fs|dd) }
!    (gs|dd) = HRR{ (gs|fp) (gs|dp) }
!    (gs|fp) = HRR{ (gs|gs) (gs|fs) }
!    (gs|dp) = HRR{ (gs|fs) (gs|ds) }
!    (dp|dd) = HRR{ (fs|dd) (ds|dd) }
!    (fs|dd) = HRR{ (fs|fp) (fs|dp) }
!    (fs|fp) = HRR{ (fs|gs) (fs|fs) }
!    (fs|dp) = HRR{ (fs|fs) (fs|ds) }
!    (ds|dd) = HRR{ (ds|fp) (ds|dp) }
!    (ds|fp) = HRR{ (ds|gs) (ds|fs) }
!    (ds|dp) = HRR{ (ds|fs) (ds|ds) }
! Classes to be formes by contraction of primitive integrals:
!    (ds|ds) (ds|fs) (ds|gs) (fs|ds) (fs|fs) (fs|gs) (gs|ds) (gs|fs) (gs|gs)
! Applying the vertical recurrence relation:
!    [gs|gs](0)    = VRR{ [gs|fs](0..1) [gs|ds](0..1) [fs|fs](1) }
!    [gs|fs](0..1) = VRR{ [gs|ds](0..2) [gs|ps](0..2) [fs|ds](1..2) }
!    [gs|ds](0..2) = VRR{ [gs|ps](0..3) [gs|ss](0..3) [fs|ps](1..3) }
!    [gs|ps](0..3) = VRR{ [gs|ss](0..4) [fs|ss](1..4) }
!    [gs|ss](0..4) = VRR{ [fs|ss](0..5) [ds|ss](0..5) }
!    [fs|gs](0)    = VRR{ [fs|fs](0..1) [fs|ds](0..1) [ds|fs](1) }
!    [fs|fs](0..1) = VRR{ [fs|ds](0..2) [fs|ps](0..2) [ds|ds](1..2) }
!    [fs|ds](0..2) = VRR{ [fs|ps](0..3) [fs|ss](0..3) [ds|ps](1..3) }
!    [fs|ps](0..3) = VRR{ [fs|ss](0..4) [ds|ss](1..4) }
!    [fs|ss](0..5) = VRR{ [ds|ss](0..6) [ps|ss](0..6) }
!    [ds|gs](0)    = VRR{ [ds|fs](0..1) [ds|ds](0..1) [ps|fs](1) }
!    [ds|fs](0..1) = VRR{ [ds|ds](0..2) [ds|ps](0..2) [ps|ds](1..2) }
!    [ds|ds](0..2) = VRR{ [ds|ps](0..3) [ds|ss](0..3) [ps|ps](1..3) }
!    [ds|ps](0..3) = VRR{ [ds|ss](0..4) [ps|ss](1..4) }
!    [ds|ss](0..6) = VRR{ [ps|ss](0..7) [ss|ss](0..7) }
!    [ps|fs](1)    = VRR{ [ps|ds](1..2) [ps|ps](1..2) [ss|ds](2) }
!    [ps|ds](1..2) = VRR{ [ps|ps](1..3) [ps|ss](1..3) [ss|ps](2..3) }
!    [ps|ps](1..3) = VRR{ [ps|ss](1..4) [ss|ss](2..4) }
!    [ps|ss](0..7) = VRR{ [ss|ss](0..8) }
!    [ss|ds](2)    = VRR{ [ss|ps](2..3) [ss|ss](2..3) }
!    [ss|ps](2..3) = VRR{ [ss|ss](2..4) }
!    [ss|ss](2..8)
!
! References:
!  [1] S.Obara and A.Saika, J.Chem.Phys. 84, 3963 (1986).
!  [2] M.Head-Gordon and J.A.Pople, J.Chem.Phys. 89, 5777 (1988).
!
subroutine os_csi_dddd_os3(&
     rsrs,&
     ishp,npsh,npsh2pr,nsh2atm,nsh2ao,xyz,ss,xiozetaa,rzet,&
     nshell,nshellp,nprimp,natom,&
     fmt)
  use physical_constants_class
  use integral_fmt_class
  use omp_lib

  implicit real(8) (a-h,o-z)

  type(fmt_type),intent(in)::fmt
  integer,intent(in)::nshell,nshellp,nprimp,natom
  real(8),intent(inout)::rsrs(nshellp)
  integer,intent(in)::ishp(6),npsh(2,nshellp),npsh2pr(nshellp+1)
  integer,intent(in)::nsh2atm(nshell),nsh2ao(nshell)
  real(8),intent(in)::ss(nprimp),xiozetaa(nprimp),rzet(nprimp)
  real(8),intent(in)::xyz(3,natom)

  real(8)::ec(36)
!  integer::tid

  tid = omp_get_thread_num() + 1

  tosqrtpi = 2.d0/dsqrt(pi)

!$omp do schedule(dynamic)
!!$!$omp do
  do ij = ishp(5)+1, ishp(6)
     i = npsh(1,ij)
     j = npsh(2,ij)
     iatom = nsh2atm(i)
     jatom = nsh2atm(j)
     Ax = xyz(1,iatom)
     Ay = xyz(2,iatom)
     Az = xyz(3,iatom)
     Bx = xyz(1,jatom)
     By = xyz(2,jatom)
     Bz = xyz(3,jatom)
     ABx = Ax - Bx
     ABy = Ay - By
     ABz = Az - Bz
     CDx = ABx
     CDy = ABy
     CDz = ABz

        ! Initializing VRR variables
        dxxsfxxxs = 0.0d0
        dxxsdxxs = 0.0d0
        dxxsfxxys = 0.0d0
        dxxsfxxzs = 0.0d0
        dyysfyyxs = 0.0d0
        dyysdyys = 0.0d0
        dyysfyyys = 0.0d0
        dyysfyyzs = 0.0d0
        dzzsfzzxs = 0.0d0
        dzzsdzzs = 0.0d0
        dzzsfzzys = 0.0d0
        dzzsfzzzs = 0.0d0
        dxysfxxys = 0.0d0
        dxysfyyxs = 0.0d0
        dxysdxys = 0.0d0
        dxysfxyzs = 0.0d0
        dxzsfxxzs = 0.0d0
        dxzsfzzxs = 0.0d0
        dxzsfxyzs = 0.0d0
        dxzsdxzs = 0.0d0
        dyzsfyyzs = 0.0d0
        dyzsfzzys = 0.0d0
        dyzsfxyzs = 0.0d0
        dyzsdyzs = 0.0d0
        dxxsgxxxxs = 0.0d0
        dxxsgxxxys = 0.0d0
        dxxsgxxxzs = 0.0d0
        dxxsgxxyys = 0.0d0
        dxxsgxxyzs = 0.0d0
        dxxsgxxzzs = 0.0d0
        dyysgyyyxs = 0.0d0
        dyysgyyyys = 0.0d0
        dyysgyyyzs = 0.0d0
        dyysgxxyys = 0.0d0
        dyysgyyxzs = 0.0d0
        dyysgyyzzs = 0.0d0
        dzzsgzzzxs = 0.0d0
        dzzsgzzzys = 0.0d0
        dzzsgzzzzs = 0.0d0
        dzzsgxxzzs = 0.0d0
        dzzsgyyzzs = 0.0d0
        dzzsgzzxys = 0.0d0
        dxysgxxxys = 0.0d0
        dxysgyyyxs = 0.0d0
        dxysgxxyys = 0.0d0
        dxysgxxyzs = 0.0d0
        dxysgyyxzs = 0.0d0
        dxysgzzxys = 0.0d0
        dxzsgxxxzs = 0.0d0
        dxzsgzzzxs = 0.0d0
        dxzsgxxyzs = 0.0d0
        dxzsgxxzzs = 0.0d0
        dxzsgyyxzs = 0.0d0
        dxzsgzzxys = 0.0d0
        dyzsgyyyzs = 0.0d0
        dyzsgzzzys = 0.0d0
        dyzsgxxyzs = 0.0d0
        dyzsgyyxzs = 0.0d0
        dyzsgyyzzs = 0.0d0
        dyzsgzzxys = 0.0d0
        fxxxsfxxxs = 0.0d0
        fxxxsdxxs = 0.0d0
        fxxxsfxxys = 0.0d0
        fxxxsfxxzs = 0.0d0
        fyyysfyyxs = 0.0d0
        fyyysdyys = 0.0d0
        fyyysfyyys = 0.0d0
        fyyysfyyzs = 0.0d0
        fzzzsfzzxs = 0.0d0
        fzzzsdzzs = 0.0d0
        fzzzsfzzys = 0.0d0
        fzzzsfzzzs = 0.0d0
        fxxysfxxxs = 0.0d0
        fxxysdxxs = 0.0d0
        fxxysfxxys = 0.0d0
        fxxysfxxzs = 0.0d0
        fxxysfyyxs = 0.0d0
        fxxysdxys = 0.0d0
        fxxysfxyzs = 0.0d0
        fxxzsfxxxs = 0.0d0
        fxxzsdxxs = 0.0d0
        fxxzsfxxys = 0.0d0
        fxxzsfxxzs = 0.0d0
        fxxzsfzzxs = 0.0d0
        fxxzsfxyzs = 0.0d0
        fxxzsdxzs = 0.0d0
        fyyxsfxxys = 0.0d0
        fyyxsfyyxs = 0.0d0
        fyyxsdyys = 0.0d0
        fyyxsfyyys = 0.0d0
        fyyxsfyyzs = 0.0d0
        fyyxsdxys = 0.0d0
        fyyxsfxyzs = 0.0d0
        fyyzsfyyxs = 0.0d0
        fyyzsdyys = 0.0d0
        fyyzsfyyys = 0.0d0
        fyyzsfyyzs = 0.0d0
        fyyzsfzzys = 0.0d0
        fyyzsfxyzs = 0.0d0
        fyyzsdyzs = 0.0d0
        fzzxsfxxzs = 0.0d0
        fzzxsfzzxs = 0.0d0
        fzzxsdzzs = 0.0d0
        fzzxsfzzys = 0.0d0
        fzzxsfzzzs = 0.0d0
        fzzxsfxyzs = 0.0d0
        fzzxsdxzs = 0.0d0
        fzzysfyyzs = 0.0d0
        fzzysfzzxs = 0.0d0
        fzzysdzzs = 0.0d0
        fzzysfzzys = 0.0d0
        fzzysfzzzs = 0.0d0
        fzzysfxyzs = 0.0d0
        fzzysdyzs = 0.0d0
        fxyzsfxxys = 0.0d0
        fxyzsfxxzs = 0.0d0
        fxyzsfyyxs = 0.0d0
        fxyzsfyyzs = 0.0d0
        fxyzsfzzxs = 0.0d0
        fxyzsfzzys = 0.0d0
        fxyzsdxys = 0.0d0
        fxyzsfxyzs = 0.0d0
        fxyzsdxzs = 0.0d0
        fxyzsdyzs = 0.0d0
        fxxxsgxxxxs = 0.0d0
        fxxxsgxxxys = 0.0d0
        fxxxsgxxxzs = 0.0d0
        fyyysgyyyxs = 0.0d0
        fyyysgyyyys = 0.0d0
        fyyysgyyyzs = 0.0d0
        fzzzsgzzzxs = 0.0d0
        fzzzsgzzzys = 0.0d0
        fzzzsgzzzzs = 0.0d0
        fxxysgxxxys = 0.0d0
        fxxysgxxyys = 0.0d0
        fxxysgxxyzs = 0.0d0
        fxxzsgxxxzs = 0.0d0
        fxxzsgxxyzs = 0.0d0
        fxxzsgxxzzs = 0.0d0
        fyyxsgyyyxs = 0.0d0
        fyyxsgxxyys = 0.0d0
        fyyxsgyyxzs = 0.0d0
        fyyzsgyyyzs = 0.0d0
        fyyzsgyyxzs = 0.0d0
        fyyzsgyyzzs = 0.0d0
        fzzxsgzzzxs = 0.0d0
        fzzxsgxxzzs = 0.0d0
        fzzxsgzzxys = 0.0d0
        fzzysgzzzys = 0.0d0
        fzzysgyyzzs = 0.0d0
        fzzysgzzxys = 0.0d0
        fxyzsgxxyzs = 0.0d0
        fxyzsgyyxzs = 0.0d0
        fxyzsgzzxys = 0.0d0
        gxxxxsfxxxs = 0.0d0
        gxxxxsdxxs = 0.0d0
        gyyyysdyys = 0.0d0
        gyyyysfyyys = 0.0d0
        gzzzzsdzzs = 0.0d0
        gzzzzsfzzzs = 0.0d0
        gxxxysfxxxs = 0.0d0
        gxxxysdxxs = 0.0d0
        gxxxysfxxys = 0.0d0
        gxxxysdxys = 0.0d0
        gxxxzsfxxxs = 0.0d0
        gxxxzsdxxs = 0.0d0
        gxxxzsfxxzs = 0.0d0
        gxxxzsdxzs = 0.0d0
        gyyyxsfyyxs = 0.0d0
        gyyyxsdyys = 0.0d0
        gyyyxsfyyys = 0.0d0
        gyyyxsdxys = 0.0d0
        gyyyzsdyys = 0.0d0
        gyyyzsfyyys = 0.0d0
        gyyyzsfyyzs = 0.0d0
        gyyyzsdyzs = 0.0d0
        gzzzxsfzzxs = 0.0d0
        gzzzxsdzzs = 0.0d0
        gzzzxsfzzzs = 0.0d0
        gzzzxsdxzs = 0.0d0
        gzzzysdzzs = 0.0d0
        gzzzysfzzys = 0.0d0
        gzzzysfzzzs = 0.0d0
        gzzzysdyzs = 0.0d0
        gxxyysdxxs = 0.0d0
        gxxyysfxxys = 0.0d0
        gxxyysfyyxs = 0.0d0
        gxxyysdyys = 0.0d0
        gxxyysdxys = 0.0d0
        gxxzzsdxxs = 0.0d0
        gxxzzsfxxzs = 0.0d0
        gxxzzsfzzxs = 0.0d0
        gxxzzsdzzs = 0.0d0
        gxxzzsdxzs = 0.0d0
        gyyzzsdyys = 0.0d0
        gyyzzsfyyzs = 0.0d0
        gyyzzsdzzs = 0.0d0
        gyyzzsfzzys = 0.0d0
        gyyzzsdyzs = 0.0d0
        gxxyzsdxxs = 0.0d0
        gxxyzsfxxys = 0.0d0
        gxxyzsfxxzs = 0.0d0
        gxxyzsdxys = 0.0d0
        gxxyzsfxyzs = 0.0d0
        gxxyzsdxzs = 0.0d0
        gxxyzsdyzs = 0.0d0
        gyyxzsfyyxs = 0.0d0
        gyyxzsdyys = 0.0d0
        gyyxzsfyyzs = 0.0d0
        gyyxzsdxys = 0.0d0
        gyyxzsfxyzs = 0.0d0
        gyyxzsdxzs = 0.0d0
        gyyxzsdyzs = 0.0d0
        gzzxysfzzxs = 0.0d0
        gzzxysdzzs = 0.0d0
        gzzxysfzzys = 0.0d0
        gzzxysdxys = 0.0d0
        gzzxysfxyzs = 0.0d0
        gzzxysdxzs = 0.0d0
        gzzxysdyzs = 0.0d0
        gxxxxsgxxxxs = 0.0d0
        gyyyysgyyyys = 0.0d0
        gzzzzsgzzzzs = 0.0d0
        gxxxysgxxxys = 0.0d0
        gxxxzsgxxxzs = 0.0d0
        gyyyxsgyyyxs = 0.0d0
        gyyyzsgyyyzs = 0.0d0
        gzzzxsgzzzxs = 0.0d0
        gzzzysgzzzys = 0.0d0
        gxxyysgxxyys = 0.0d0
        gxxzzsgxxzzs = 0.0d0
        gyyzzsgyyzzs = 0.0d0
        gxxyzsgxxyzs = 0.0d0
        gyyxzsgyyxzs = 0.0d0
        gzzxysgzzxys = 0.0d0

        do irs = npsh2pr(ij),npsh2pr(ij+1)-1
           xioza = xiozetaa(irs)
           PAx = -xioza*ABx
           PAy = -xioza*ABy
           PAz = -xioza*ABz
           Px = PAx + Ax
           Py = PAy + Ay
           Pz = PAz + Az
           Sp = ss(irs)
           ooz = rzet(irs)*2.0d0
           oo2zeta = 0.5d0 * ooz

           do itu = npsh2pr(ij),npsh2pr(ij+1)-1
              xiozb = xiozetaa(itu)
              QCx = -xiozb*ABx
              QCy = -xiozb*ABy
              QCz = -xiozb*ABz
              Qx = QCx + Ax
              Qy = QCy + Ay
              Qz = QCz + Az
              Sq = ss(itu)
              ooe = rzet(itu)*2.0d0

              QPx = Qx - Px
              QPy = Qy - Py
              QPz = Qz - Pz
              QP2 = QPx*QPx + QPy*QPy + QPz*QPz
              rho = 1.0d0/(ooz + ooe)
              t = rho * QP2

              if (t <= tf(9)) then
                 it = idint((t + 0.025d0)*20.0d0)
                 dt = 0.05d0*it - t
                 it = it + 1
                 dt4 = dt * 0.25d0
                 dt5 = dt * 0.2d0
                 dt6 = dt * 0.1666666666666666666d0
                 dt2 = dt4 + dt4
                 dt3 = dt6 + dt6
                 ft  = (((((fmt%fdat(15,it)*dt6 + fmt%fdat(14,it))*dt5&
                     + fmt%fdat(13,it))*dt4 + fmt%fdat(12,it))*dt3&
                     + fmt%fdat(11,it))*dt2 + fmt%fdat(10,it))*dt + fmt%fdat(9,it)
                 et  = ((((((ga6*dt + ga5)*dt + ga4)*dt&
                     + ga3)*dt + ga2)*dt + ga1)*dt + ga0) * fmt%edat(it)
                 t2 = 2.d0 * t
                 cn = dsqrt(rho) * tosqrtpi * Sp * Sq
                 etcn = et * cn
                 ssss8 = ft * cn
                 ssss7 = (ssss8 * t2 + etcn) * rminv(8)
                 ssss6 = (ssss7 * t2 + etcn) * rminv(7)
                 ssss5 = (ssss6 * t2 + etcn) * rminv(6)
                 ssss4 = (ssss5 * t2 + etcn) * rminv(5)
                 ssss3 = (ssss4 * t2 + etcn) * rminv(4)
                 ssss2 = (ssss3 * t2 + etcn) * rminv(3)
                 ssss1 = (ssss2 * t2 + etcn) * rminv(2)
                 ssss0 =  ssss1 * t2 + etcn
              else
                 tinv = 1.d0/t
                 ssss0 = dsqrt(rho*tinv) * Sp * Sq
                 ssss1 = ssss0 * tinv * 0.5d0
                 ssss2 = ssss1 * tinv * 1.5d0
                 ssss3 = ssss2 * tinv * 2.5d0
                 ssss4 = ssss3 * tinv * 3.5d0
                 ssss5 = ssss4 * tinv * 4.5d0
                 ssss6 = ssss5 * tinv * 5.5d0
                 ssss7 = ssss6 * tinv * 6.5d0
                 ssss8 = ssss7 * tinv * 7.5d0
              end if

              rhoozeta = rho*ooz
              rhooeta = rho*ooe
              oo2eta = 0.5d0 * ooe
              rhoo2ze = rhoozeta*oo2eta
              WPx = QPx * rhoozeta
              WPy = QPy * rhoozeta
              WPz = QPz * rhoozeta
              WQx = WPx - QPx
              WQy = WPy - QPy
              WQz = WPz - QPz

              pzsss7 = ssss7*PAz + ssss8*WPz
              pysss7 = ssss7*PAy + ssss8*WPy
              pxsss7 = ssss7*PAx + ssss8*WPx
              ai_ssss67 = oo2zeta*( ssss6 - rhoozeta*ssss7 )
              pzsss6 = ssss6*PAz + ssss7*WPz
              dzzsss6 = pzsss6*PAz + pzsss7*WPz + ai_ssss67
              pysss6 = ssss6*PAy + ssss7*WPy
              dyysss6 = pysss6*PAy + pysss7*WPy + ai_ssss67
              pxsss6 = ssss6*PAx + ssss7*WPx
              dxxsss6 = pxsss6*PAx + pxsss7*WPx + ai_ssss67
              ai_ssss56 = oo2zeta*( ssss5 - rhoozeta*ssss6 )
              pzsss5 = ssss5*PAz + ssss6*WPz
              ai_pzsss56 = oo2zeta*( pzsss5 - rhoozeta*pzsss6 )
              dzzsss5 = pzsss5*PAz + pzsss6*WPz + ai_ssss56
              fzzxsss5 = dzzsss5*PAx + dzzsss6*WPx
              fzzzsss5 = dzzsss5*PAz + dzzsss6*WPz + ai_pzsss56*2.0d0
              pysss5 = ssss5*PAy + ssss6*WPy
              ai_pysss56 = oo2zeta*( pysss5 - rhoozeta*pysss6 )
              dyysss5 = pysss5*PAy + pysss6*WPy + ai_ssss56
              fyyzsss5 = dyysss5*PAz + dyysss6*WPz
              fyyxsss5 = dyysss5*PAx + dyysss6*WPx
              fyyysss5 = dyysss5*PAy + dyysss6*WPy + ai_pysss56*2.0d0
              pxsss5 = ssss5*PAx + ssss6*WPx
              ai_pxsss56 = oo2zeta*( pxsss5 - rhoozeta*pxsss6 )
              dxysss5 = pxsss5*PAy + pxsss6*WPy
              dxxsss5 = pxsss5*PAx + pxsss6*WPx + ai_ssss56
              fxxzsss5 = dxxsss5*PAz + dxxsss6*WPz
              fxxysss5 = dxxsss5*PAy + dxxsss6*WPy
              fxxxsss5 = dxxsss5*PAx + dxxsss6*WPx + ai_pxsss56*2.0d0
              ai_ssss01 = oo2zeta*( ssss0 - rhoozeta*ssss1 )
              pzsss0 = ssss0*PAz + ssss1*WPz
              pysss0 = ssss0*PAy + ssss1*WPy
              pxsss0 = ssss0*PAx + ssss1*WPx
              ai_ssss45 = oo2zeta*( ssss4 - rhoozeta*ssss5 )
              ak_ssss4 = rhoo2ze*ssss4
              pzsss4 = ssss4*PAz + ssss5*WPz
              ai_pzsss45 = oo2zeta*( pzsss4 - rhoozeta*pzsss5 )
              ak_pzsss4 = rhoo2ze*pzsss4
              dzzsss4 = pzsss4*PAz + pzsss5*WPz + ai_ssss45
              ai_dzzsss45 = oo2zeta*( dzzsss4 - rhoozeta*dzzsss5 )
              ak_dzzsss4 = rhoo2ze*dzzsss4
              fzzysss4 = dzzsss4*PAy + dzzsss5*WPy
              ak_fzzysss4 = rhoo2ze*fzzysss4
              fzzxsss4 = dzzsss4*PAx + dzzsss5*WPx
              ak_fzzxsss4 = rhoo2ze*fzzxsss4
              gzzxysss4 = fzzxsss4*PAy + fzzxsss5*WPy
              fzzzsss4 = dzzsss4*PAz + dzzsss5*WPz + ai_pzsss45*2.0d0
              ak_fzzzsss4 = rhoo2ze*fzzzsss4
              gzzzysss4 = fzzzsss4*PAy + fzzzsss5*WPy
              gzzzxsss4 = fzzzsss4*PAx + fzzzsss5*WPx
              gzzzzsss4 = fzzzsss4*PAz + fzzzsss5*WPz + ai_dzzsss45*3.0d0
              pysss4 = ssss4*PAy + ssss5*WPy
              ai_pysss45 = oo2zeta*( pysss4 - rhoozeta*pysss5 )
              ak_pysss4 = rhoo2ze*pysss4
              dyzsss4 = pysss4*PAz + pysss5*WPz
              ak_dyzsss4 = rhoo2ze*dyzsss4
              dyysss4 = pysss4*PAy + pysss5*WPy + ai_ssss45
              ai_dyysss45 = oo2zeta*( dyysss4 - rhoozeta*dyysss5 )
              ak_dyysss4 = rhoo2ze*dyysss4
              fyyzsss4 = dyysss4*PAz + dyysss5*WPz
              ak_fyyzsss4 = rhoo2ze*fyyzsss4
              gyyzzsss4 = fyyzsss4*PAz + fyyzsss5*WPz + ai_dyysss45
              fyyxsss4 = dyysss4*PAx + dyysss5*WPx
              ak_fyyxsss4 = rhoo2ze*fyyxsss4
              gyyxzsss4 = fyyxsss4*PAz + fyyxsss5*WPz
              fyyysss4 = dyysss4*PAy + dyysss5*WPy + ai_pysss45*2.0d0
              ak_fyyysss4 = rhoo2ze*fyyysss4
              gyyyzsss4 = fyyysss4*PAz + fyyysss5*WPz
              gyyyxsss4 = fyyysss4*PAx + fyyysss5*WPx
              gyyyysss4 = fyyysss4*PAy + fyyysss5*WPy + ai_dyysss45*3.0d0
              pxsss4 = ssss4*PAx + ssss5*WPx
              ai_pxsss45 = oo2zeta*( pxsss4 - rhoozeta*pxsss5 )
              ak_pxsss4 = rhoo2ze*pxsss4
              dxzsss4 = pxsss4*PAz + pxsss5*WPz
              ak_dxzsss4 = rhoo2ze*dxzsss4
              dxysss4 = pxsss4*PAy + pxsss5*WPy
              ak_dxysss4 = rhoo2ze*dxysss4
              fxyzsss4 = dxysss4*PAz + dxysss5*WPz
              ak_fxyzsss4 = rhoo2ze*fxyzsss4
              dxxsss4 = pxsss4*PAx + pxsss5*WPx + ai_ssss45
              ai_dxxsss45 = oo2zeta*( dxxsss4 - rhoozeta*dxxsss5 )
              ak_dxxsss4 = rhoo2ze*dxxsss4
              fxxzsss4 = dxxsss4*PAz + dxxsss5*WPz
              ak_fxxzsss4 = rhoo2ze*fxxzsss4
              gxxzzsss4 = fxxzsss4*PAz + fxxzsss5*WPz + ai_dxxsss45
              fxxysss4 = dxxsss4*PAy + dxxsss5*WPy
              ak_fxxysss4 = rhoo2ze*fxxysss4
              gxxyzsss4 = fxxysss4*PAz + fxxysss5*WPz
              gxxyysss4 = fxxysss4*PAy + fxxysss5*WPy + ai_dxxsss45
              fxxxsss4 = dxxsss4*PAx + dxxsss5*WPx + ai_pxsss45*2.0d0
              ak_fxxxsss4 = rhoo2ze*fxxxsss4
              gxxxzsss4 = fxxxsss4*PAz + fxxxsss5*WPz
              gxxxysss4 = fxxxsss4*PAy + fxxxsss5*WPy
              gxxxxsss4 = fxxxsss4*PAx + fxxxsss5*WPx + ai_dxxsss45*3.0d0
              ai_ssss34 = oo2zeta*( ssss3 - rhoozeta*ssss4 )
              ak_ssss3 = rhoo2ze*ssss3
              pzsss3 = ssss3*PAz + ssss4*WPz
              ai_pzsss34 = oo2zeta*( pzsss3 - rhoozeta*pzsss4 )
              ak_pzsss3 = rhoo2ze*pzsss3
              dzzsss3 = pzsss3*PAz + pzsss4*WPz + ai_ssss34
              ai_dzzsss34 = oo2zeta*( dzzsss3 - rhoozeta*dzzsss4 )
              ak_dzzsss3 = rhoo2ze*dzzsss3
              fzzysss3 = dzzsss3*PAy + dzzsss4*WPy
              ak_fzzysss3 = rhoo2ze*fzzysss3
              fzzyspzs3 = fzzysss3*QCz + fzzysss4*WQz + ak_dyzsss4*2.0d0
              ak_fzzyspzs3 = rhoo2ze*fzzyspzs3
              fzzyspys3 = fzzysss3*QCy + fzzysss4*WQy + ak_dzzsss4
              ak_fzzyspys3 = rhoo2ze*fzzyspys3
              fzzxsss3 = dzzsss3*PAx + dzzsss4*WPx
              ak_fzzxsss3 = rhoo2ze*fzzxsss3
              gzzxysss3 = fzzxsss3*PAy + fzzxsss4*WPy
              gzzxyspzs3 = gzzxysss3*QCz + gzzxysss4*WQz + ak_fxyzsss4*2.0d0
              fzzxspzs3 = fzzxsss3*QCz + fzzxsss4*WQz + ak_dxzsss4*2.0d0
              ak_fzzxspzs3 = rhoo2ze*fzzxspzs3
              fzzxspxs3 = fzzxsss3*QCx + fzzxsss4*WQx + ak_dzzsss4
              ak_fzzxspxs3 = rhoo2ze*fzzxspxs3
              fzzzsss3 = dzzsss3*PAz + dzzsss4*WPz + ai_pzsss34*2.0d0
              ak_fzzzsss3 = rhoo2ze*fzzzsss3
              gzzzysss3 = fzzzsss3*PAy + fzzzsss4*WPy
              gzzzyspzs3 = gzzzysss3*QCz + gzzzysss4*WQz + ak_fzzysss4*3.0d0
              gzzzxsss3 = fzzzsss3*PAx + fzzzsss4*WPx
              gzzzxspzs3 = gzzzxsss3*QCz + gzzzxsss4*WQz + ak_fzzxsss4*3.0d0
              gzzzzsss3 = fzzzsss3*PAz + fzzzsss4*WPz + ai_dzzsss34*3.0d0
              gzzzzspzs3 = gzzzzsss3*QCz + gzzzzsss4*WQz + ak_fzzzsss4*4.0d0
              fzzzspzs3 = fzzzsss3*QCz + fzzzsss4*WQz + ak_dzzsss4*3.0d0
              ak_fzzzspzs3 = rhoo2ze*fzzzspzs3
              dzzspzs3 = dzzsss3*QCz + dzzsss4*WQz + ak_pzsss4*2.0d0
              ak_dzzspzs3 = rhoo2ze*dzzspzs3
              dzzspys3 = dzzsss3*QCy + dzzsss4*WQy
              ak_dzzspys3 = rhoo2ze*dzzspys3
              dzzspxs3 = dzzsss3*QCx + dzzsss4*WQx
              ak_dzzspxs3 = rhoo2ze*dzzspxs3
              pzspzs3 = pzsss3*QCz + pzsss4*WQz + ak_ssss4
              ak_pzspzs3 = rhoo2ze*pzspzs3
              pzspys3 = pzsss3*QCy + pzsss4*WQy
              ak_pzspys3 = rhoo2ze*pzspys3
              pzspxs3 = pzsss3*QCx + pzsss4*WQx
              ak_pzspxs3 = rhoo2ze*pzspxs3
              pysss3 = ssss3*PAy + ssss4*WPy
              ai_pysss34 = oo2zeta*( pysss3 - rhoozeta*pysss4 )
              ak_pysss3 = rhoo2ze*pysss3
              dyzsss3 = pysss3*PAz + pysss4*WPz
              ak_dyzsss3 = rhoo2ze*dyzsss3
              dyzspzs3 = dyzsss3*QCz + dyzsss4*WQz + ak_pysss4
              ak_dyzspzs3 = rhoo2ze*dyzspzs3
              dyzspys3 = dyzsss3*QCy + dyzsss4*WQy + ak_pzsss4
              ak_dyzspys3 = rhoo2ze*dyzspys3
              dyzspxs3 = dyzsss3*QCx + dyzsss4*WQx
              ak_dyzspxs3 = rhoo2ze*dyzspxs3
              dyysss3 = pysss3*PAy + pysss4*WPy + ai_ssss34
              ai_dyysss34 = oo2zeta*( dyysss3 - rhoozeta*dyysss4 )
              ak_dyysss3 = rhoo2ze*dyysss3
              fyyzsss3 = dyysss3*PAz + dyysss4*WPz
              ak_fyyzsss3 = rhoo2ze*fyyzsss3
              gyyzzsss3 = fyyzsss3*PAz + fyyzsss4*WPz + ai_dyysss34
              gyyzzspys3 = gyyzzsss3*QCy + gyyzzsss4*WQy + ak_fzzysss4*2.0d0
              fyyzspys3 = fyyzsss3*QCy + fyyzsss4*WQy + ak_dyzsss4*2.0d0
              ak_fyyzspys3 = rhoo2ze*fyyzspys3
              fyyxsss3 = dyysss3*PAx + dyysss4*WPx
              ak_fyyxsss3 = rhoo2ze*fyyxsss3
              gyyxzsss3 = fyyxsss3*PAz + fyyxsss4*WPz
              gyyxzspys3 = gyyxzsss3*QCy + gyyxzsss4*WQy + ak_fxyzsss4*2.0d0
              fyyxspys3 = fyyxsss3*QCy + fyyxsss4*WQy + ak_dxysss4*2.0d0
              ak_fyyxspys3 = rhoo2ze*fyyxspys3
              fyyxspxs3 = fyyxsss3*QCx + fyyxsss4*WQx + ak_dyysss4
              ak_fyyxspxs3 = rhoo2ze*fyyxspxs3
              fyyysss3 = dyysss3*PAy + dyysss4*WPy + ai_pysss34*2.0d0
              ak_fyyysss3 = rhoo2ze*fyyysss3
              gyyyzsss3 = fyyysss3*PAz + fyyysss4*WPz
              gyyyzspys3 = gyyyzsss3*QCy + gyyyzsss4*WQy + ak_fyyzsss4*3.0d0
              gyyyxsss3 = fyyysss3*PAx + fyyysss4*WPx
              gyyyxspys3 = gyyyxsss3*QCy + gyyyxsss4*WQy + ak_fyyxsss4*3.0d0
              gyyyysss3 = fyyysss3*PAy + fyyysss4*WPy + ai_dyysss34*3.0d0
              gyyyyspys3 = gyyyysss3*QCy + gyyyysss4*WQy + ak_fyyysss4*4.0d0
              fyyyspys3 = fyyysss3*QCy + fyyysss4*WQy + ak_dyysss4*3.0d0
              ak_fyyyspys3 = rhoo2ze*fyyyspys3
              dyyspys3 = dyysss3*QCy + dyysss4*WQy + ak_pysss4*2.0d0
              ak_dyyspys3 = rhoo2ze*dyyspys3
              dyyspxs3 = dyysss3*QCx + dyysss4*WQx
              ak_dyyspxs3 = rhoo2ze*dyyspxs3
              pyspzs3 = pysss3*QCz + pysss4*WQz
              ak_pyspzs3 = rhoo2ze*pyspzs3
              pyspys3 = pysss3*QCy + pysss4*WQy + ak_ssss4
              ak_pyspys3 = rhoo2ze*pyspys3
              pyspxs3 = pysss3*QCx + pysss4*WQx
              ak_pyspxs3 = rhoo2ze*pyspxs3
              pxsss3 = ssss3*PAx + ssss4*WPx
              ai_pxsss34 = oo2zeta*( pxsss3 - rhoozeta*pxsss4 )
              ak_pxsss3 = rhoo2ze*pxsss3
              dxzsss3 = pxsss3*PAz + pxsss4*WPz
              ak_dxzsss3 = rhoo2ze*dxzsss3
              dxzspzs3 = dxzsss3*QCz + dxzsss4*WQz + ak_pxsss4
              ak_dxzspzs3 = rhoo2ze*dxzspzs3
              dxzspys3 = dxzsss3*QCy + dxzsss4*WQy
              ak_dxzspys3 = rhoo2ze*dxzspys3
              dxzspxs3 = dxzsss3*QCx + dxzsss4*WQx + ak_pzsss4
              ak_dxzspxs3 = rhoo2ze*dxzspxs3
              dxysss3 = pxsss3*PAy + pxsss4*WPy
              ak_dxysss3 = rhoo2ze*dxysss3
              fxyzsss3 = dxysss3*PAz + dxysss4*WPz
              ak_fxyzsss3 = rhoo2ze*fxyzsss3
              fxyzspzs3 = fxyzsss3*QCz + fxyzsss4*WQz + ak_dxysss4
              ak_fxyzspzs3 = rhoo2ze*fxyzspzs3
              fxyzspys3 = fxyzsss3*QCy + fxyzsss4*WQy + ak_dxzsss4
              ak_fxyzspys3 = rhoo2ze*fxyzspys3
              fxyzspxs3 = fxyzsss3*QCx + fxyzsss4*WQx + ak_dyzsss4
              ak_fxyzspxs3 = rhoo2ze*fxyzspxs3
              dxyspzs3 = dxysss3*QCz + dxysss4*WQz
              ak_dxyspzs3 = rhoo2ze*dxyspzs3
              dxyspys3 = dxysss3*QCy + dxysss4*WQy + ak_pxsss4
              ak_dxyspys3 = rhoo2ze*dxyspys3
              dxyspxs3 = dxysss3*QCx + dxysss4*WQx + ak_pysss4
              ak_dxyspxs3 = rhoo2ze*dxyspxs3
              dxxsss3 = pxsss3*PAx + pxsss4*WPx + ai_ssss34
              ai_dxxsss34 = oo2zeta*( dxxsss3 - rhoozeta*dxxsss4 )
              ak_dxxsss3 = rhoo2ze*dxxsss3
              fxxzsss3 = dxxsss3*PAz + dxxsss4*WPz
              ak_fxxzsss3 = rhoo2ze*fxxzsss3
              gxxzzsss3 = fxxzsss3*PAz + fxxzsss4*WPz + ai_dxxsss34
              gxxzzspxs3 = gxxzzsss3*QCx + gxxzzsss4*WQx + ak_fzzxsss4*2.0d0
              fxxzspxs3 = fxxzsss3*QCx + fxxzsss4*WQx + ak_dxzsss4*2.0d0
              ak_fxxzspxs3 = rhoo2ze*fxxzspxs3
              fxxysss3 = dxxsss3*PAy + dxxsss4*WPy
              ak_fxxysss3 = rhoo2ze*fxxysss3
              gxxyzsss3 = fxxysss3*PAz + fxxysss4*WPz
              gxxyzspxs3 = gxxyzsss3*QCx + gxxyzsss4*WQx + ak_fxyzsss4*2.0d0
              gxxyysss3 = fxxysss3*PAy + fxxysss4*WPy + ai_dxxsss34
              gxxyyspxs3 = gxxyysss3*QCx + gxxyysss4*WQx + ak_fyyxsss4*2.0d0
              fxxyspxs3 = fxxysss3*QCx + fxxysss4*WQx + ak_dxysss4*2.0d0
              ak_fxxyspxs3 = rhoo2ze*fxxyspxs3
              fxxxsss3 = dxxsss3*PAx + dxxsss4*WPx + ai_pxsss34*2.0d0
              ak_fxxxsss3 = rhoo2ze*fxxxsss3
              gxxxzsss3 = fxxxsss3*PAz + fxxxsss4*WPz
              gxxxzspxs3 = gxxxzsss3*QCx + gxxxzsss4*WQx + ak_fxxzsss4*3.0d0
              gxxxysss3 = fxxxsss3*PAy + fxxxsss4*WPy
              gxxxyspxs3 = gxxxysss3*QCx + gxxxysss4*WQx + ak_fxxysss4*3.0d0
              gxxxxsss3 = fxxxsss3*PAx + fxxxsss4*WPx + ai_dxxsss34*3.0d0
              gxxxxspxs3 = gxxxxsss3*QCx + gxxxxsss4*WQx + ak_fxxxsss4*4.0d0
              fxxxspxs3 = fxxxsss3*QCx + fxxxsss4*WQx + ak_dxxsss4*3.0d0
              ak_fxxxspxs3 = rhoo2ze*fxxxspxs3
              dxxspxs3 = dxxsss3*QCx + dxxsss4*WQx + ak_pxsss4*2.0d0
              ak_dxxspxs3 = rhoo2ze*dxxspxs3
              pxspzs3 = pxsss3*QCz + pxsss4*WQz
              ak_pxspzs3 = rhoo2ze*pxspzs3
              pxspys3 = pxsss3*QCy + pxsss4*WQy
              ak_pxspys3 = rhoo2ze*pxspys3
              pxspxs3 = pxsss3*QCx + pxsss4*WQx + ak_ssss4
              ak_pxspxs3 = rhoo2ze*pxspxs3
              sspzs3 = ssss3*QCz + ssss4*WQz
              ak_sspzs3 = rhoo2ze*sspzs3
              sspys3 = ssss3*QCy + ssss4*WQy
              ak_sspys3 = rhoo2ze*sspys3
              sspxs3 = ssss3*QCx + ssss4*WQx
              ak_sspxs3 = rhoo2ze*sspxs3
              ai_ssss23 = oo2zeta*( ssss2 - rhoozeta*ssss3 )
              ai_ssss12 = oo2zeta*( ssss1 - rhoozeta*ssss2 )
              ak_ssss2 = rhoo2ze*ssss2
              pzsss2 = ssss2*PAz + ssss3*WPz
              ai_pzsss23 = oo2zeta*( pzsss2 - rhoozeta*pzsss3 )
              ak_pzsss2 = rhoo2ze*pzsss2
              dzzsss2 = pzsss2*PAz + pzsss3*WPz + ai_ssss23
              ai_dzzsss23 = oo2zeta*( dzzsss2 - rhoozeta*dzzsss3 )
              ak_dzzsss2 = rhoo2ze*dzzsss2
              fzzysss2 = dzzsss2*PAy + dzzsss3*WPy
              ak_fzzysss2 = rhoo2ze*fzzysss2
              ck_fzzysss23 = oo2eta*( fzzysss2 - rhooeta*fzzysss3 )
              fzzyspzs2 = fzzysss2*QCz + fzzysss3*WQz + ak_dyzsss3*2.0d0
              ak_fzzyspzs2 = rhoo2ze*fzzyspzs2
              fzzysdzzs2 = fzzyspzs2*QCz + fzzyspzs3*WQz + ck_fzzysss23 + ak_dyzspzs3*2.0d0
              ak_fzzysdzzs2 = rhoo2ze*fzzysdzzs2
              fzzyspys2 = fzzysss2*QCy + fzzysss3*WQy + ak_dzzsss3
              ak_fzzyspys2 = rhoo2ze*fzzyspys2
              fzzysdyys2 = fzzyspys2*QCy + fzzyspys3*WQy + ck_fzzysss23 + ak_dzzspys3
              fzzyspxs2 = fzzysss2*QCx + fzzysss3*WQx
              fzzxsss2 = dzzsss2*PAx + dzzsss3*WPx
              ak_fzzxsss2 = rhoo2ze*fzzxsss2
              gzzxysss2 = fzzxsss2*PAy + fzzxsss3*WPy
              ck_gzzxysss23 = oo2eta*( gzzxysss2 - rhooeta*gzzxysss3 )
              gzzxyspzs2 = gzzxysss2*QCz + gzzxysss3*WQz + ak_fxyzsss3*2.0d0
              gzzxysdzzs2 = gzzxyspzs2*QCz + gzzxyspzs3*WQz + ck_gzzxysss23 + ak_fxyzspzs3*2.0d0
              gzzxyspxs2 = gzzxysss2*QCx + gzzxysss3*WQx + ak_fzzysss3
              ck_fzzxsss23 = oo2eta*( fzzxsss2 - rhooeta*fzzxsss3 )
              fzzxspzs2 = fzzxsss2*QCz + fzzxsss3*WQz + ak_dxzsss3*2.0d0
              ak_fzzxspzs2 = rhoo2ze*fzzxspzs2
              fzzxsdzzs2 = fzzxspzs2*QCz + fzzxspzs3*WQz + ck_fzzxsss23 + ak_dxzspzs3*2.0d0
              ak_fzzxsdzzs2 = rhoo2ze*fzzxsdzzs2
              fzzxspxs2 = fzzxsss2*QCx + fzzxsss3*WQx + ak_dzzsss3
              ak_fzzxspxs2 = rhoo2ze*fzzxspxs2
              fzzxsdxxs2 = fzzxspxs2*QCx + fzzxspxs3*WQx + ck_fzzxsss23 + ak_dzzspxs3
              fzzzsss2 = dzzsss2*PAz + dzzsss3*WPz + ai_pzsss23*2.0d0
              ak_fzzzsss2 = rhoo2ze*fzzzsss2
              gzzzysss2 = fzzzsss2*PAy + fzzzsss3*WPy
              ck_gzzzysss23 = oo2eta*( gzzzysss2 - rhooeta*gzzzysss3 )
              gzzzyspzs2 = gzzzysss2*QCz + gzzzysss3*WQz + ak_fzzysss3*3.0d0
              gzzzysdzzs2 = gzzzyspzs2*QCz + gzzzyspzs3*WQz + ck_gzzzysss23 + ak_fzzyspzs3*3.0d0
              gzzzxsss2 = fzzzsss2*PAx + fzzzsss3*WPx
              ck_gzzzxsss23 = oo2eta*( gzzzxsss2 - rhooeta*gzzzxsss3 )
              gzzzxspzs2 = gzzzxsss2*QCz + gzzzxsss3*WQz + ak_fzzxsss3*3.0d0
              gzzzxsdzzs2 = gzzzxspzs2*QCz + gzzzxspzs3*WQz + ck_gzzzxsss23 + ak_fzzxspzs3*3.0d0
              gzzzzsss2 = fzzzsss2*PAz + fzzzsss3*WPz + ai_dzzsss23*3.0d0
              ck_gzzzzsss23 = oo2eta*( gzzzzsss2 - rhooeta*gzzzzsss3 )
              gzzzzspzs2 = gzzzzsss2*QCz + gzzzzsss3*WQz + ak_fzzzsss3*4.0d0
              gzzzzsdzzs2 = gzzzzspzs2*QCz + gzzzzspzs3*WQz + ck_gzzzzsss23 + ak_fzzzspzs3*4.0d0
              ck_fzzzsss23 = oo2eta*( fzzzsss2 - rhooeta*fzzzsss3 )
              fzzzspzs2 = fzzzsss2*QCz + fzzzsss3*WQz + ak_dzzsss3*3.0d0
              ak_fzzzspzs2 = rhoo2ze*fzzzspzs2
              fzzzsdzzs2 = fzzzspzs2*QCz + fzzzspzs3*WQz + ck_fzzzsss23 + ak_dzzspzs3*3.0d0
              ak_fzzzsdzzs2 = rhoo2ze*fzzzsdzzs2
              ck_dzzsss23 = oo2eta*( dzzsss2 - rhooeta*dzzsss3 )
              dzzspzs2 = dzzsss2*QCz + dzzsss3*WQz + ak_pzsss3*2.0d0
              ak_dzzspzs2 = rhoo2ze*dzzspzs2
              dzzsdzzs2 = dzzspzs2*QCz + dzzspzs3*WQz + ck_dzzsss23 + ak_pzspzs3*2.0d0
              ak_dzzsdzzs2 = rhoo2ze*dzzsdzzs2
              dzzspys2 = dzzsss2*QCy + dzzsss3*WQy
              ak_dzzspys2 = rhoo2ze*dzzspys2
              dzzsdyys2 = dzzspys2*QCy + dzzspys3*WQy + ck_dzzsss23
              dzzspxs2 = dzzsss2*QCx + dzzsss3*WQx
              ak_dzzspxs2 = rhoo2ze*dzzspxs2
              dzzsdxxs2 = dzzspxs2*QCx + dzzspxs3*WQx + ck_dzzsss23
              ck_pzsss23 = oo2eta*( pzsss2 - rhooeta*pzsss3 )
              pzspzs2 = pzsss2*QCz + pzsss3*WQz + ak_ssss3
              ak_pzspzs2 = rhoo2ze*pzspzs2
              pzsdzzs2 = pzspzs2*QCz + pzspzs3*WQz + ck_pzsss23 + ak_sspzs3
              ak_pzsdzzs2 = rhoo2ze*pzsdzzs2
              pzspys2 = pzsss2*QCy + pzsss3*WQy
              ak_pzspys2 = rhoo2ze*pzspys2
              pzsdyys2 = pzspys2*QCy + pzspys3*WQy + ck_pzsss23
              ak_pzsdyys2 = rhoo2ze*pzsdyys2
              pzspxs2 = pzsss2*QCx + pzsss3*WQx
              ak_pzspxs2 = rhoo2ze*pzspxs2
              pzsdxxs2 = pzspxs2*QCx + pzspxs3*WQx + ck_pzsss23
              ak_pzsdxxs2 = rhoo2ze*pzsdxxs2
              pysss2 = ssss2*PAy + ssss3*WPy
              ai_pysss23 = oo2zeta*( pysss2 - rhoozeta*pysss3 )
              ak_pysss2 = rhoo2ze*pysss2
              dyzsss2 = pysss2*PAz + pysss3*WPz
              ak_dyzsss2 = rhoo2ze*dyzsss2
              ck_dyzsss23 = oo2eta*( dyzsss2 - rhooeta*dyzsss3 )
              dyzspzs2 = dyzsss2*QCz + dyzsss3*WQz + ak_pysss3
              ak_dyzspzs2 = rhoo2ze*dyzspzs2
              dyzsdzzs2 = dyzspzs2*QCz + dyzspzs3*WQz + ck_dyzsss23 + ak_pyspzs3
              ak_dyzsdzzs2 = rhoo2ze*dyzsdzzs2
              dyzspys2 = dyzsss2*QCy + dyzsss3*WQy + ak_pzsss3
              ak_dyzspys2 = rhoo2ze*dyzspys2
              dyzsdyys2 = dyzspys2*QCy + dyzspys3*WQy + ck_dyzsss23 + ak_pzspys3
              ak_dyzsdyys2 = rhoo2ze*dyzsdyys2
              dyzspxs2 = dyzsss2*QCx + dyzsss3*WQx
              ak_dyzspxs2 = rhoo2ze*dyzspxs2
              dyzsdxxs2 = dyzspxs2*QCx + dyzspxs3*WQx + ck_dyzsss23
              dyysss2 = pysss2*PAy + pysss3*WPy + ai_ssss23
              ai_dyysss23 = oo2zeta*( dyysss2 - rhoozeta*dyysss3 )
              ak_dyysss2 = rhoo2ze*dyysss2
              fyyzsss2 = dyysss2*PAz + dyysss3*WPz
              ak_fyyzsss2 = rhoo2ze*fyyzsss2
              gyyzzsss2 = fyyzsss2*PAz + fyyzsss3*WPz + ai_dyysss23
              ck_gyyzzsss23 = oo2eta*( gyyzzsss2 - rhooeta*gyyzzsss3 )
              gyyzzspzs2 = gyyzzsss2*QCz + gyyzzsss3*WQz + ak_fyyzsss3*2.0d0
              gyyzzspys2 = gyyzzsss2*QCy + gyyzzsss3*WQy + ak_fzzysss3*2.0d0
              gyyzzsdyys2 = gyyzzspys2*QCy + gyyzzspys3*WQy + ck_gyyzzsss23 + ak_fzzyspys3*2.0d0
              ck_fyyzsss23 = oo2eta*( fyyzsss2 - rhooeta*fyyzsss3 )
              fyyzspzs2 = fyyzsss2*QCz + fyyzsss3*WQz + ak_dyysss3
              ak_fyyzspzs2 = rhoo2ze*fyyzspzs2
              fyyzspys2 = fyyzsss2*QCy + fyyzsss3*WQy + ak_dyzsss3*2.0d0
              ak_fyyzspys2 = rhoo2ze*fyyzspys2
              fyyzsdyys2 = fyyzspys2*QCy + fyyzspys3*WQy + ck_fyyzsss23 + ak_dyzspys3*2.0d0
              ak_fyyzsdyys2 = rhoo2ze*fyyzsdyys2
              fyyzspxs2 = fyyzsss2*QCx + fyyzsss3*WQx
              fyyxsss2 = dyysss2*PAx + dyysss3*WPx
              ak_fyyxsss2 = rhoo2ze*fyyxsss2
              gyyxzsss2 = fyyxsss2*PAz + fyyxsss3*WPz
              ck_gyyxzsss23 = oo2eta*( gyyxzsss2 - rhooeta*gyyxzsss3 )
              gyyxzspys2 = gyyxzsss2*QCy + gyyxzsss3*WQy + ak_fxyzsss3*2.0d0
              gyyxzsdyys2 = gyyxzspys2*QCy + gyyxzspys3*WQy + ck_gyyxzsss23 + ak_fxyzspys3*2.0d0
              gyyxzspxs2 = gyyxzsss2*QCx + gyyxzsss3*WQx + ak_fyyzsss3
              ck_fyyxsss23 = oo2eta*( fyyxsss2 - rhooeta*fyyxsss3 )
              fyyxspys2 = fyyxsss2*QCy + fyyxsss3*WQy + ak_dxysss3*2.0d0
              ak_fyyxspys2 = rhoo2ze*fyyxspys2
              fyyxsdyys2 = fyyxspys2*QCy + fyyxspys3*WQy + ck_fyyxsss23 + ak_dxyspys3*2.0d0
              ak_fyyxsdyys2 = rhoo2ze*fyyxsdyys2
              fyyxspxs2 = fyyxsss2*QCx + fyyxsss3*WQx + ak_dyysss3
              ak_fyyxspxs2 = rhoo2ze*fyyxspxs2
              fyyxsdxxs2 = fyyxspxs2*QCx + fyyxspxs3*WQx + ck_fyyxsss23 + ak_dyyspxs3
              fyyysss2 = dyysss2*PAy + dyysss3*WPy + ai_pysss23*2.0d0
              ak_fyyysss2 = rhoo2ze*fyyysss2
              gyyyzsss2 = fyyysss2*PAz + fyyysss3*WPz
              ck_gyyyzsss23 = oo2eta*( gyyyzsss2 - rhooeta*gyyyzsss3 )
              gyyyzspys2 = gyyyzsss2*QCy + gyyyzsss3*WQy + ak_fyyzsss3*3.0d0
              gyyyzsdyys2 = gyyyzspys2*QCy + gyyyzspys3*WQy + ck_gyyyzsss23 + ak_fyyzspys3*3.0d0
              gyyyxsss2 = fyyysss2*PAx + fyyysss3*WPx
              ck_gyyyxsss23 = oo2eta*( gyyyxsss2 - rhooeta*gyyyxsss3 )
              gyyyxspys2 = gyyyxsss2*QCy + gyyyxsss3*WQy + ak_fyyxsss3*3.0d0
              gyyyxsdyys2 = gyyyxspys2*QCy + gyyyxspys3*WQy + ck_gyyyxsss23 + ak_fyyxspys3*3.0d0
              gyyyysss2 = fyyysss2*PAy + fyyysss3*WPy + ai_dyysss23*3.0d0
              ck_gyyyysss23 = oo2eta*( gyyyysss2 - rhooeta*gyyyysss3 )
              gyyyyspys2 = gyyyysss2*QCy + gyyyysss3*WQy + ak_fyyysss3*4.0d0
              gyyyysdyys2 = gyyyyspys2*QCy + gyyyyspys3*WQy + ck_gyyyysss23 + ak_fyyyspys3*4.0d0
              ck_fyyysss23 = oo2eta*( fyyysss2 - rhooeta*fyyysss3 )
              fyyyspys2 = fyyysss2*QCy + fyyysss3*WQy + ak_dyysss3*3.0d0
              ak_fyyyspys2 = rhoo2ze*fyyyspys2
              fyyysdyys2 = fyyyspys2*QCy + fyyyspys3*WQy + ck_fyyysss23 + ak_dyyspys3*3.0d0
              ak_fyyysdyys2 = rhoo2ze*fyyysdyys2
              ck_dyysss23 = oo2eta*( dyysss2 - rhooeta*dyysss3 )
              dyyspzs2 = dyysss2*QCz + dyysss3*WQz
              ak_dyyspzs2 = rhoo2ze*dyyspzs2
              dyyspys2 = dyysss2*QCy + dyysss3*WQy + ak_pysss3*2.0d0
              ak_dyyspys2 = rhoo2ze*dyyspys2
              dyysdyys2 = dyyspys2*QCy + dyyspys3*WQy + ck_dyysss23 + ak_pyspys3*2.0d0
              ak_dyysdyys2 = rhoo2ze*dyysdyys2
              dyyspxs2 = dyysss2*QCx + dyysss3*WQx
              ak_dyyspxs2 = rhoo2ze*dyyspxs2
              dyysdxxs2 = dyyspxs2*QCx + dyyspxs3*WQx + ck_dyysss23
              ck_pysss23 = oo2eta*( pysss2 - rhooeta*pysss3 )
              pyspzs2 = pysss2*QCz + pysss3*WQz
              ak_pyspzs2 = rhoo2ze*pyspzs2
              pysdzzs2 = pyspzs2*QCz + pyspzs3*WQz + ck_pysss23
              ak_pysdzzs2 = rhoo2ze*pysdzzs2
              pyspys2 = pysss2*QCy + pysss3*WQy + ak_ssss3
              ak_pyspys2 = rhoo2ze*pyspys2
              pysdyys2 = pyspys2*QCy + pyspys3*WQy + ck_pysss23 + ak_sspys3
              ak_pysdyys2 = rhoo2ze*pysdyys2
              pyspxs2 = pysss2*QCx + pysss3*WQx
              ak_pyspxs2 = rhoo2ze*pyspxs2
              pysdxxs2 = pyspxs2*QCx + pyspxs3*WQx + ck_pysss23
              ak_pysdxxs2 = rhoo2ze*pysdxxs2
              pxsss2 = ssss2*PAx + ssss3*WPx
              ai_pxsss23 = oo2zeta*( pxsss2 - rhoozeta*pxsss3 )
              ak_pxsss2 = rhoo2ze*pxsss2
              dxzsss2 = pxsss2*PAz + pxsss3*WPz
              ak_dxzsss2 = rhoo2ze*dxzsss2
              ck_dxzsss23 = oo2eta*( dxzsss2 - rhooeta*dxzsss3 )
              dxzspzs2 = dxzsss2*QCz + dxzsss3*WQz + ak_pxsss3
              ak_dxzspzs2 = rhoo2ze*dxzspzs2
              dxzsdzzs2 = dxzspzs2*QCz + dxzspzs3*WQz + ck_dxzsss23 + ak_pxspzs3
              ak_dxzsdzzs2 = rhoo2ze*dxzsdzzs2
              dxzspys2 = dxzsss2*QCy + dxzsss3*WQy
              ak_dxzspys2 = rhoo2ze*dxzspys2
              dxzsdyys2 = dxzspys2*QCy + dxzspys3*WQy + ck_dxzsss23
              dxzspxs2 = dxzsss2*QCx + dxzsss3*WQx + ak_pzsss3
              ak_dxzspxs2 = rhoo2ze*dxzspxs2
              dxzsdxxs2 = dxzspxs2*QCx + dxzspxs3*WQx + ck_dxzsss23 + ak_pzspxs3
              ak_dxzsdxxs2 = rhoo2ze*dxzsdxxs2
              dxysss2 = pxsss2*PAy + pxsss3*WPy
              ak_dxysss2 = rhoo2ze*dxysss2
              fxyzsss2 = dxysss2*PAz + dxysss3*WPz
              ak_fxyzsss2 = rhoo2ze*fxyzsss2
              ck_fxyzsss23 = oo2eta*( fxyzsss2 - rhooeta*fxyzsss3 )
              fxyzspzs2 = fxyzsss2*QCz + fxyzsss3*WQz + ak_dxysss3
              ak_fxyzspzs2 = rhoo2ze*fxyzspzs2
              fxyzsdzzs2 = fxyzspzs2*QCz + fxyzspzs3*WQz + ck_fxyzsss23 + ak_dxyspzs3
              fxyzspys2 = fxyzsss2*QCy + fxyzsss3*WQy + ak_dxzsss3
              ak_fxyzspys2 = rhoo2ze*fxyzspys2
              fxyzsdyys2 = fxyzspys2*QCy + fxyzspys3*WQy + ck_fxyzsss23 + ak_dxzspys3
              fxyzspxs2 = fxyzsss2*QCx + fxyzsss3*WQx + ak_dyzsss3
              ak_fxyzspxs2 = rhoo2ze*fxyzspxs2
              fxyzsdxxs2 = fxyzspxs2*QCx + fxyzspxs3*WQx + ck_fxyzsss23 + ak_dyzspxs3
              ck_dxysss23 = oo2eta*( dxysss2 - rhooeta*dxysss3 )
              dxyspzs2 = dxysss2*QCz + dxysss3*WQz
              ak_dxyspzs2 = rhoo2ze*dxyspzs2
              dxysdzzs2 = dxyspzs2*QCz + dxyspzs3*WQz + ck_dxysss23
              dxyspys2 = dxysss2*QCy + dxysss3*WQy + ak_pxsss3
              ak_dxyspys2 = rhoo2ze*dxyspys2
              dxysdyys2 = dxyspys2*QCy + dxyspys3*WQy + ck_dxysss23 + ak_pxspys3
              ak_dxysdyys2 = rhoo2ze*dxysdyys2
              dxyspxs2 = dxysss2*QCx + dxysss3*WQx + ak_pysss3
              ak_dxyspxs2 = rhoo2ze*dxyspxs2
              dxysdxxs2 = dxyspxs2*QCx + dxyspxs3*WQx + ck_dxysss23 + ak_pyspxs3
              ak_dxysdxxs2 = rhoo2ze*dxysdxxs2
              dxxsss2 = pxsss2*PAx + pxsss3*WPx + ai_ssss23
              ai_dxxsss23 = oo2zeta*( dxxsss2 - rhoozeta*dxxsss3 )
              ak_dxxsss2 = rhoo2ze*dxxsss2
              fxxzsss2 = dxxsss2*PAz + dxxsss3*WPz
              ak_fxxzsss2 = rhoo2ze*fxxzsss2
              gxxzzsss2 = fxxzsss2*PAz + fxxzsss3*WPz + ai_dxxsss23
              ck_gxxzzsss23 = oo2eta*( gxxzzsss2 - rhooeta*gxxzzsss3 )
              gxxzzspzs2 = gxxzzsss2*QCz + gxxzzsss3*WQz + ak_fxxzsss3*2.0d0
              gxxzzspxs2 = gxxzzsss2*QCx + gxxzzsss3*WQx + ak_fzzxsss3*2.0d0
              gxxzzsdxxs2 = gxxzzspxs2*QCx + gxxzzspxs3*WQx + ck_gxxzzsss23 + ak_fzzxspxs3*2.0d0
              ck_fxxzsss23 = oo2eta*( fxxzsss2 - rhooeta*fxxzsss3 )
              fxxzspzs2 = fxxzsss2*QCz + fxxzsss3*WQz + ak_dxxsss3
              ak_fxxzspzs2 = rhoo2ze*fxxzspzs2
              fxxzspxs2 = fxxzsss2*QCx + fxxzsss3*WQx + ak_dxzsss3*2.0d0
              ak_fxxzspxs2 = rhoo2ze*fxxzspxs2
              fxxzsdxxs2 = fxxzspxs2*QCx + fxxzspxs3*WQx + ck_fxxzsss23 + ak_dxzspxs3*2.0d0
              ak_fxxzsdxxs2 = rhoo2ze*fxxzsdxxs2
              fxxysss2 = dxxsss2*PAy + dxxsss3*WPy
              ak_fxxysss2 = rhoo2ze*fxxysss2
              gxxyzsss2 = fxxysss2*PAz + fxxysss3*WPz
              ck_gxxyzsss23 = oo2eta*( gxxyzsss2 - rhooeta*gxxyzsss3 )
              gxxyzspxs2 = gxxyzsss2*QCx + gxxyzsss3*WQx + ak_fxyzsss3*2.0d0
              gxxyzsdxxs2 = gxxyzspxs2*QCx + gxxyzspxs3*WQx + ck_gxxyzsss23 + ak_fxyzspxs3*2.0d0
              gxxyysss2 = fxxysss2*PAy + fxxysss3*WPy + ai_dxxsss23
              ck_gxxyysss23 = oo2eta*( gxxyysss2 - rhooeta*gxxyysss3 )
              gxxyyspys2 = gxxyysss2*QCy + gxxyysss3*WQy + ak_fxxysss3*2.0d0
              gxxyyspxs2 = gxxyysss2*QCx + gxxyysss3*WQx + ak_fyyxsss3*2.0d0
              gxxyysdxxs2 = gxxyyspxs2*QCx + gxxyyspxs3*WQx + ck_gxxyysss23 + ak_fyyxspxs3*2.0d0
              ck_fxxysss23 = oo2eta*( fxxysss2 - rhooeta*fxxysss3 )
              fxxyspys2 = fxxysss2*QCy + fxxysss3*WQy + ak_dxxsss3
              ak_fxxyspys2 = rhoo2ze*fxxyspys2
              fxxyspxs2 = fxxysss2*QCx + fxxysss3*WQx + ak_dxysss3*2.0d0
              ak_fxxyspxs2 = rhoo2ze*fxxyspxs2
              fxxysdxxs2 = fxxyspxs2*QCx + fxxyspxs3*WQx + ck_fxxysss23 + ak_dxyspxs3*2.0d0
              ak_fxxysdxxs2 = rhoo2ze*fxxysdxxs2
              fxxxsss2 = dxxsss2*PAx + dxxsss3*WPx + ai_pxsss23*2.0d0
              ak_fxxxsss2 = rhoo2ze*fxxxsss2
              gxxxzsss2 = fxxxsss2*PAz + fxxxsss3*WPz
              ck_gxxxzsss23 = oo2eta*( gxxxzsss2 - rhooeta*gxxxzsss3 )
              gxxxzspxs2 = gxxxzsss2*QCx + gxxxzsss3*WQx + ak_fxxzsss3*3.0d0
              gxxxzsdxxs2 = gxxxzspxs2*QCx + gxxxzspxs3*WQx + ck_gxxxzsss23 + ak_fxxzspxs3*3.0d0
              gxxxysss2 = fxxxsss2*PAy + fxxxsss3*WPy
              ck_gxxxysss23 = oo2eta*( gxxxysss2 - rhooeta*gxxxysss3 )
              gxxxyspxs2 = gxxxysss2*QCx + gxxxysss3*WQx + ak_fxxysss3*3.0d0
              gxxxysdxxs2 = gxxxyspxs2*QCx + gxxxyspxs3*WQx + ck_gxxxysss23 + ak_fxxyspxs3*3.0d0
              gxxxxsss2 = fxxxsss2*PAx + fxxxsss3*WPx + ai_dxxsss23*3.0d0
              ck_gxxxxsss23 = oo2eta*( gxxxxsss2 - rhooeta*gxxxxsss3 )
              gxxxxspxs2 = gxxxxsss2*QCx + gxxxxsss3*WQx + ak_fxxxsss3*4.0d0
              gxxxxsdxxs2 = gxxxxspxs2*QCx + gxxxxspxs3*WQx + ck_gxxxxsss23 + ak_fxxxspxs3*4.0d0
              ck_fxxxsss23 = oo2eta*( fxxxsss2 - rhooeta*fxxxsss3 )
              fxxxspxs2 = fxxxsss2*QCx + fxxxsss3*WQx + ak_dxxsss3*3.0d0
              ak_fxxxspxs2 = rhoo2ze*fxxxspxs2
              fxxxsdxxs2 = fxxxspxs2*QCx + fxxxspxs3*WQx + ck_fxxxsss23 + ak_dxxspxs3*3.0d0
              ak_fxxxsdxxs2 = rhoo2ze*fxxxsdxxs2
              ck_dxxsss23 = oo2eta*( dxxsss2 - rhooeta*dxxsss3 )
              dxxspzs2 = dxxsss2*QCz + dxxsss3*WQz
              ak_dxxspzs2 = rhoo2ze*dxxspzs2
              dxxspys2 = dxxsss2*QCy + dxxsss3*WQy
              ak_dxxspys2 = rhoo2ze*dxxspys2
              dxxspxs2 = dxxsss2*QCx + dxxsss3*WQx + ak_pxsss3*2.0d0
              ak_dxxspxs2 = rhoo2ze*dxxspxs2
              dxxsdxxs2 = dxxspxs2*QCx + dxxspxs3*WQx + ck_dxxsss23 + ak_pxspxs3*2.0d0
              ak_dxxsdxxs2 = rhoo2ze*dxxsdxxs2
              ck_pxsss23 = oo2eta*( pxsss2 - rhooeta*pxsss3 )
              pxspzs2 = pxsss2*QCz + pxsss3*WQz
              ak_pxspzs2 = rhoo2ze*pxspzs2
              pxsdzzs2 = pxspzs2*QCz + pxspzs3*WQz + ck_pxsss23
              ak_pxsdzzs2 = rhoo2ze*pxsdzzs2
              pxspys2 = pxsss2*QCy + pxsss3*WQy
              ak_pxspys2 = rhoo2ze*pxspys2
              pxsdyys2 = pxspys2*QCy + pxspys3*WQy + ck_pxsss23
              ak_pxsdyys2 = rhoo2ze*pxsdyys2
              pxspxs2 = pxsss2*QCx + pxsss3*WQx + ak_ssss3
              ak_pxspxs2 = rhoo2ze*pxspxs2
              pxsdxxs2 = pxspxs2*QCx + pxspxs3*WQx + ck_pxsss23 + ak_sspxs3
              ak_pxsdxxs2 = rhoo2ze*pxsdxxs2
              pzsss1 = ssss1*PAz + ssss2*WPz
              ai_pzsss12 = oo2zeta*( pzsss1 - rhoozeta*pzsss2 )
              ai_pzsss01 = oo2zeta*( pzsss0 - rhoozeta*pzsss1 )
              ak_pzsss1 = rhoo2ze*pzsss1
              dzzsss1 = pzsss1*PAz + pzsss2*WPz + ai_ssss12
              ai_dzzsss12 = oo2zeta*( dzzsss1 - rhoozeta*dzzsss2 )
              ak_dzzsss1 = rhoo2ze*dzzsss1
              fzzysss1 = dzzsss1*PAy + dzzsss2*WPy
              ak_fzzysss1 = rhoo2ze*fzzysss1
              ck_fzzysss12 = oo2eta*( fzzysss1 - rhooeta*fzzysss2 )
              fzzyspzs1 = fzzysss1*QCz + fzzysss2*WQz + ak_dyzsss2*2.0d0
              ak_fzzyspzs1 = rhoo2ze*fzzyspzs1
              ck_fzzyspzs12 = oo2eta*( fzzyspzs1 - rhooeta*fzzyspzs2 )
              fzzysdzzs1 = fzzyspzs1*QCz + fzzyspzs2*WQz + ck_fzzysss12 + ak_dyzspzs2*2.0d0
              ak_fzzysdzzs1 = rhoo2ze*fzzysdzzs1
              fzzysfzzxs1 = fzzysdzzs1*QCx + fzzysdzzs2*WQx
              fzzysfzzzs1 = fzzysdzzs1*QCz + fzzysdzzs2*WQz + ck_fzzyspzs12*2.0d0 + ak_dyzsdzzs2*2.0d0
              fzzyspys1 = fzzysss1*QCy + fzzysss2*WQy + ak_dzzsss2
              ak_fzzyspys1 = rhoo2ze*fzzyspys1
              fzzysdyys1 = fzzyspys1*QCy + fzzyspys2*WQy + ck_fzzysss12 + ak_dzzspys2
              fzzysfyyzs1 = fzzysdyys1*QCz + fzzysdyys2*WQz + ak_dyzsdyys2*2.0d0
              fzzyspxs1 = fzzysss1*QCx + fzzysss2*WQx
              fzzysdxys1 = fzzyspxs1*QCy + fzzyspxs2*WQy + ak_dzzspxs2
              fzzxsss1 = dzzsss1*PAx + dzzsss2*WPx
              ak_fzzxsss1 = rhoo2ze*fzzxsss1
              gzzxysss1 = fzzxsss1*PAy + fzzxsss2*WPy
              ck_gzzxysss12 = oo2eta*( gzzxysss1 - rhooeta*gzzxysss2 )
              gzzxyspzs1 = gzzxysss1*QCz + gzzxysss2*WQz + ak_fxyzsss2*2.0d0
              gzzxysdzzs1 = gzzxyspzs1*QCz + gzzxyspzs2*WQz + ck_gzzxysss12 + ak_fxyzspzs2*2.0d0
              gzzxysfzzxs1 = gzzxysdzzs1*QCx + gzzxysdzzs2*WQx + ak_fzzysdzzs2
              gzzxyspys1 = gzzxysss1*QCy + gzzxysss2*WQy + ak_fzzxsss2
              gzzxyspxs1 = gzzxysss1*QCx + gzzxysss2*WQx + ak_fzzysss2
              gzzxysdxys1 = gzzxyspxs1*QCy + gzzxyspxs2*WQy + ak_fzzxspxs2
              ck_fzzxsss12 = oo2eta*( fzzxsss1 - rhooeta*fzzxsss2 )
              fzzxspzs1 = fzzxsss1*QCz + fzzxsss2*WQz + ak_dxzsss2*2.0d0
              ak_fzzxspzs1 = rhoo2ze*fzzxspzs1
              ck_fzzxspzs12 = oo2eta*( fzzxspzs1 - rhooeta*fzzxspzs2 )
              fzzxsdzzs1 = fzzxspzs1*QCz + fzzxspzs2*WQz + ck_fzzxsss12 + ak_dxzspzs2*2.0d0
              ak_fzzxsdzzs1 = rhoo2ze*fzzxsdzzs1
              fzzxsfzzxs1 = fzzxsdzzs1*QCx + fzzxsdzzs2*WQx + ak_dzzsdzzs2
              ak_fzzxsfzzxs1 = rhoo2ze*fzzxsfzzxs1
              fzzxsfzzzs1 = fzzxsdzzs1*QCz + fzzxsdzzs2*WQz + ck_fzzxspzs12*2.0d0 + ak_dxzsdzzs2*2.0d0
              fzzxspxs1 = fzzxsss1*QCx + fzzxsss2*WQx + ak_dzzsss2
              ak_fzzxspxs1 = rhoo2ze*fzzxspxs1
              fzzxsdxys1 = fzzxspxs1*QCy + fzzxspxs2*WQy
              fzzxsdxxs1 = fzzxspxs1*QCx + fzzxspxs2*WQx + ck_fzzxsss12 + ak_dzzspxs2
              fzzxsfxxzs1 = fzzxsdxxs1*QCz + fzzxsdxxs2*WQz + ak_dxzsdxxs2*2.0d0
              fzzzsss1 = dzzsss1*PAz + dzzsss2*WPz + ai_pzsss12*2.0d0
              ak_fzzzsss1 = rhoo2ze*fzzzsss1
              gzzzysss1 = fzzzsss1*PAy + fzzzsss2*WPy
              ck_gzzzysss12 = oo2eta*( gzzzysss1 - rhooeta*gzzzysss2 )
              gzzzyspzs1 = gzzzysss1*QCz + gzzzysss2*WQz + ak_fzzysss2*3.0d0
              ck_gzzzyspzs12 = oo2eta*( gzzzyspzs1 - rhooeta*gzzzyspzs2 )
              gzzzysdzzs1 = gzzzyspzs1*QCz + gzzzyspzs2*WQz + ck_gzzzysss12 + ak_fzzyspzs2*3.0d0
              gzzzysfzzzs1 = gzzzysdzzs1*QCz + gzzzysdzzs2*WQz + ck_gzzzyspzs12*2.0d0 + ak_fzzysdzzs2*3.0d0
              gzzzyspys1 = gzzzysss1*QCy + gzzzysss2*WQy + ak_fzzzsss2
              gzzzxsss1 = fzzzsss1*PAx + fzzzsss2*WPx
              ck_gzzzxsss12 = oo2eta*( gzzzxsss1 - rhooeta*gzzzxsss2 )
              gzzzxspzs1 = gzzzxsss1*QCz + gzzzxsss2*WQz + ak_fzzxsss2*3.0d0
              ck_gzzzxspzs12 = oo2eta*( gzzzxspzs1 - rhooeta*gzzzxspzs2 )
              gzzzxsdzzs1 = gzzzxspzs1*QCz + gzzzxspzs2*WQz + ck_gzzzxsss12 + ak_fzzxspzs2*3.0d0
              gzzzxsfzzzs1 = gzzzxsdzzs1*QCz + gzzzxsdzzs2*WQz + ck_gzzzxspzs12*2.0d0 + ak_fzzxsdzzs2*3.0d0
              gzzzxspxs1 = gzzzxsss1*QCx + gzzzxsss2*WQx + ak_fzzzsss2
              gzzzzsss1 = fzzzsss1*PAz + fzzzsss2*WPz + ai_dzzsss12*3.0d0
              ck_gzzzzsss12 = oo2eta*( gzzzzsss1 - rhooeta*gzzzzsss2 )
              gzzzzspzs1 = gzzzzsss1*QCz + gzzzzsss2*WQz + ak_fzzzsss2*4.0d0
              ck_gzzzzspzs12 = oo2eta*( gzzzzspzs1 - rhooeta*gzzzzspzs2 )
              gzzzzsdzzs1 = gzzzzspzs1*QCz + gzzzzspzs2*WQz + ck_gzzzzsss12 + ak_fzzzspzs2*4.0d0
              gzzzzsfzzzs1 = gzzzzsdzzs1*QCz + gzzzzsdzzs2*WQz + ck_gzzzzspzs12*2.0d0 + ak_fzzzsdzzs2*4.0d0
              ck_fzzzsss12 = oo2eta*( fzzzsss1 - rhooeta*fzzzsss2 )
              fzzzspzs1 = fzzzsss1*QCz + fzzzsss2*WQz + ak_dzzsss2*3.0d0
              ak_fzzzspzs1 = rhoo2ze*fzzzspzs1
              ck_fzzzspzs12 = oo2eta*( fzzzspzs1 - rhooeta*fzzzspzs2 )
              fzzzsdzzs1 = fzzzspzs1*QCz + fzzzspzs2*WQz + ck_fzzzsss12 + ak_dzzspzs2*3.0d0
              ak_fzzzsdzzs1 = rhoo2ze*fzzzsdzzs1
              fzzzsfzzzs1 = fzzzsdzzs1*QCz + fzzzsdzzs2*WQz + ck_fzzzspzs12*2.0d0 + ak_dzzsdzzs2*3.0d0
              ak_fzzzsfzzzs1 = rhoo2ze*fzzzsfzzzs1
              ck_dzzsss12 = oo2eta*( dzzsss1 - rhooeta*dzzsss2 )
              dzzspzs1 = dzzsss1*QCz + dzzsss2*WQz + ak_pzsss2*2.0d0
              ak_dzzspzs1 = rhoo2ze*dzzspzs1
              ck_dzzspzs12 = oo2eta*( dzzspzs1 - rhooeta*dzzspzs2 )
              dzzsdzzs1 = dzzspzs1*QCz + dzzspzs2*WQz + ck_dzzsss12 + ak_pzspzs2*2.0d0
              ak_dzzsdzzs1 = rhoo2ze*dzzsdzzs1
              dzzsfzzxs1 = dzzsdzzs1*QCx + dzzsdzzs2*WQx
              ak_dzzsfzzxs1 = rhoo2ze*dzzsfzzxs1
              dzzsfzzzs1 = dzzsdzzs1*QCz + dzzsdzzs2*WQz + ck_dzzspzs12*2.0d0 + ak_pzsdzzs2*2.0d0
              ak_dzzsfzzzs1 = rhoo2ze*dzzsfzzzs1
              dzzspys1 = dzzsss1*QCy + dzzsss2*WQy
              ak_dzzspys1 = rhoo2ze*dzzspys1
              dzzsdyys1 = dzzspys1*QCy + dzzspys2*WQy + ck_dzzsss12
              dzzsfyyzs1 = dzzsdyys1*QCz + dzzsdyys2*WQz + ak_pzsdyys2*2.0d0
              dzzspxs1 = dzzsss1*QCx + dzzsss2*WQx
              ak_dzzspxs1 = rhoo2ze*dzzspxs1
              dzzsdxxs1 = dzzspxs1*QCx + dzzspxs2*WQx + ck_dzzsss12
              dzzsfxxzs1 = dzzsdxxs1*QCz + dzzsdxxs2*WQz + ak_pzsdxxs2*2.0d0
              dzzsss0 = pzsss0*PAz + pzsss1*WPz + ai_ssss01
              ai_dzzsss01 = oo2zeta*( dzzsss0 - rhoozeta*dzzsss1 )
              fzzysss0 = dzzsss0*PAy + dzzsss1*WPy
              ck_fzzysss01 = oo2eta*( fzzysss0 - rhooeta*fzzysss1 )
              fzzyspys0 = fzzysss0*QCy + fzzysss1*WQy + ak_dzzsss1
              fzzysdyys0 = fzzyspys0*QCy + fzzyspys1*WQy + ck_fzzysss01 + ak_dzzspys1
              ck_fzzysdyys01 = oo2eta*( fzzysdyys0 - rhooeta*fzzysdyys1 )
              fzzyspxs0 = fzzysss0*QCx + fzzysss1*WQx
              fzzysdxys0 = fzzyspxs0*QCy + fzzyspxs1*WQy + ak_dzzspxs1
              fzzxsss0 = dzzsss0*PAx + dzzsss1*WPx
              gzzxysss0 = fzzxsss0*PAy + fzzxsss1*WPy
              ck_gzzxysss01 = oo2eta*( gzzxysss0 - rhooeta*gzzxysss1 )
              gzzxyspys0 = gzzxysss0*QCy + gzzxysss1*WQy + ak_fzzxsss1
              gzzxyspxs0 = gzzxysss0*QCx + gzzxysss1*WQx + ak_fzzysss1
              gzzxysdxys0 = gzzxyspxs0*QCy + gzzxyspxs1*WQy + ak_fzzxspxs1
              gzzxysdxys = gzzxysdxys + gzzxysdxys0
              ck_fzzxsss01 = oo2eta*( fzzxsss0 - rhooeta*fzzxsss1 )
              fzzxspxs0 = fzzxsss0*QCx + fzzxsss1*WQx + ak_dzzsss1
              fzzxsdxys0 = fzzxspxs0*QCy + fzzxspxs1*WQy
              fzzxsdxxs0 = fzzxspxs0*QCx + fzzxspxs1*WQx + ck_fzzxsss01 + ak_dzzspxs1
              ck_fzzxsdxxs01 = oo2eta*( fzzxsdxxs0 - rhooeta*fzzxsdxxs1 )
              fzzzsss0 = dzzsss0*PAz + dzzsss1*WPz + ai_pzsss01*2.0d0
              gzzzysss0 = fzzzsss0*PAy + fzzzsss1*WPy
              ck_gzzzysss01 = oo2eta*( gzzzysss0 - rhooeta*gzzzysss1 )
              gzzzyspzs0 = gzzzysss0*QCz + gzzzysss1*WQz + ak_fzzysss1*3.0d0
              ck_gzzzyspzs01 = oo2eta*( gzzzyspzs0 - rhooeta*gzzzyspzs1 )
              gzzzysdzzs0 = gzzzyspzs0*QCz + gzzzyspzs1*WQz + ck_gzzzysss01 + ak_fzzyspzs1*3.0d0
              gzzzysdzzs = gzzzysdzzs + gzzzysdzzs0
              gzzzysfzzys0 = gzzzysdzzs0*QCy + gzzzysdzzs1*WQy + ak_fzzzsdzzs1
              gzzzysfzzys = gzzzysfzzys + gzzzysfzzys0
              gzzzysfzzzs0 = gzzzysdzzs0*QCz + gzzzysdzzs1*WQz + ck_gzzzyspzs01*2.0d0 + ak_fzzysdzzs1*3.0d0
              gzzzysfzzzs = gzzzysfzzzs + gzzzysfzzzs0
              gzzzysgzzzys0 = gzzzysfzzzs0*QCy + gzzzysfzzzs1*WQy + ak_fzzzsfzzzs1
              gzzzysgzzzys = gzzzysgzzzys + gzzzysgzzzys0
              gzzzyspys0 = gzzzysss0*QCy + gzzzysss1*WQy + ak_fzzzsss1
              gzzzysdyzs0 = gzzzyspys0*QCz + gzzzyspys1*WQz + ak_fzzyspys1*3.0d0
              gzzzysdyzs = gzzzysdyzs + gzzzysdyzs0
              gzzzxsss0 = fzzzsss0*PAx + fzzzsss1*WPx
              ck_gzzzxsss01 = oo2eta*( gzzzxsss0 - rhooeta*gzzzxsss1 )
              gzzzxspzs0 = gzzzxsss0*QCz + gzzzxsss1*WQz + ak_fzzxsss1*3.0d0
              ck_gzzzxspzs01 = oo2eta*( gzzzxspzs0 - rhooeta*gzzzxspzs1 )
              gzzzxsdzzs0 = gzzzxspzs0*QCz + gzzzxspzs1*WQz + ck_gzzzxsss01 + ak_fzzxspzs1*3.0d0
              gzzzxsdzzs = gzzzxsdzzs + gzzzxsdzzs0
              gzzzxsfzzxs0 = gzzzxsdzzs0*QCx + gzzzxsdzzs1*WQx + ak_fzzzsdzzs1
              gzzzxsfzzxs = gzzzxsfzzxs + gzzzxsfzzxs0
              gzzzxsfzzzs0 = gzzzxsdzzs0*QCz + gzzzxsdzzs1*WQz + ck_gzzzxspzs01*2.0d0 + ak_fzzxsdzzs1*3.0d0
              gzzzxsfzzzs = gzzzxsfzzzs + gzzzxsfzzzs0
              gzzzxsgzzzxs0 = gzzzxsfzzzs0*QCx + gzzzxsfzzzs1*WQx + ak_fzzzsfzzzs1
              gzzzxsgzzzxs = gzzzxsgzzzxs + gzzzxsgzzzxs0
              gzzzxspxs0 = gzzzxsss0*QCx + gzzzxsss1*WQx + ak_fzzzsss1
              gzzzxsdxzs0 = gzzzxspxs0*QCz + gzzzxspxs1*WQz + ak_fzzxspxs1*3.0d0
              gzzzxsdxzs = gzzzxsdxzs + gzzzxsdxzs0
              gzzzzsss0 = fzzzsss0*PAz + fzzzsss1*WPz + ai_dzzsss01*3.0d0
              ck_gzzzzsss01 = oo2eta*( gzzzzsss0 - rhooeta*gzzzzsss1 )
              gzzzzspzs0 = gzzzzsss0*QCz + gzzzzsss1*WQz + ak_fzzzsss1*4.0d0
              ck_gzzzzspzs01 = oo2eta*( gzzzzspzs0 - rhooeta*gzzzzspzs1 )
              gzzzzsdzzs0 = gzzzzspzs0*QCz + gzzzzspzs1*WQz + ck_gzzzzsss01 + ak_fzzzspzs1*4.0d0
              gzzzzsdzzs = gzzzzsdzzs + gzzzzsdzzs0
              ck_gzzzzsdzzs01 = oo2eta*( gzzzzsdzzs0 - rhooeta*gzzzzsdzzs1 )
              gzzzzsfzzzs0 = gzzzzsdzzs0*QCz + gzzzzsdzzs1*WQz + ck_gzzzzspzs01*2.0d0 + ak_fzzzsdzzs1*4.0d0
              gzzzzsfzzzs = gzzzzsfzzzs + gzzzzsfzzzs0
              gzzzzsgzzzzs0 = gzzzzsfzzzs0*QCz + gzzzzsfzzzs1*WQz + ck_gzzzzsdzzs01*3.0d0 + ak_fzzzsfzzzs1*4.0d0
              gzzzzsgzzzzs = gzzzzsgzzzzs + gzzzzsgzzzzs0
              ck_fzzzsss01 = oo2eta*( fzzzsss0 - rhooeta*fzzzsss1 )
              fzzzspzs0 = fzzzsss0*QCz + fzzzsss1*WQz + ak_dzzsss1*3.0d0
              ck_fzzzspzs01 = oo2eta*( fzzzspzs0 - rhooeta*fzzzspzs1 )
              fzzzsdzzs0 = fzzzspzs0*QCz + fzzzspzs1*WQz + ck_fzzzsss01 + ak_dzzspzs1*3.0d0
              fzzzsdzzs = fzzzsdzzs + fzzzsdzzs0
              ck_fzzzsdzzs01 = oo2eta*( fzzzsdzzs0 - rhooeta*fzzzsdzzs1 )
              fzzzsfzzys0 = fzzzsdzzs0*QCy + fzzzsdzzs1*WQy
              fzzzsfzzys = fzzzsfzzys + fzzzsfzzys0
              fzzzsfzzxs0 = fzzzsdzzs0*QCx + fzzzsdzzs1*WQx
              fzzzsfzzxs = fzzzsfzzxs + fzzzsfzzxs0
              fzzzsfzzzs0 = fzzzsdzzs0*QCz + fzzzsdzzs1*WQz + ck_fzzzspzs01*2.0d0 + ak_dzzsdzzs1*3.0d0
              fzzzsfzzzs = fzzzsfzzzs + fzzzsfzzzs0
              fzzzsgzzzys0 = fzzzsfzzzs0*QCy + fzzzsfzzzs1*WQy
              fzzzsgzzzys = fzzzsgzzzys + fzzzsgzzzys0
              fzzzsgzzzxs0 = fzzzsfzzzs0*QCx + fzzzsfzzzs1*WQx
              fzzzsgzzzxs = fzzzsgzzzxs + fzzzsgzzzxs0
              fzzzsgzzzzs0 = fzzzsfzzzs0*QCz + fzzzsfzzzs1*WQz + ck_fzzzsdzzs01*3.0d0 + ak_dzzsfzzzs1*3.0d0
              fzzzsgzzzzs = fzzzsgzzzzs + fzzzsgzzzzs0
              ck_dzzsss01 = oo2eta*( dzzsss0 - rhooeta*dzzsss1 )
              dzzspzs0 = dzzsss0*QCz + dzzsss1*WQz + ak_pzsss1*2.0d0
              ck_dzzspzs01 = oo2eta*( dzzspzs0 - rhooeta*dzzspzs1 )
              dzzspys0 = dzzsss0*QCy + dzzsss1*WQy
              dzzsdyys0 = dzzspys0*QCy + dzzspys1*WQy + ck_dzzsss01
              ck_dzzsdyys01 = oo2eta*( dzzsdyys0 - rhooeta*dzzsdyys1 )
              dzzspxs0 = dzzsss0*QCx + dzzsss1*WQx
              dzzsdxxs0 = dzzspxs0*QCx + dzzspxs1*WQx + ck_dzzsss01
              ck_dzzsdxxs01 = oo2eta*( dzzsdxxs0 - rhooeta*dzzsdxxs1 )
              ck_pzsss12 = oo2eta*( pzsss1 - rhooeta*pzsss2 )
              pzspzs1 = pzsss1*QCz + pzsss2*WQz + ak_ssss2
              ak_pzspzs1 = rhoo2ze*pzspzs1
              dzzsdzzs0 = dzzspzs0*QCz + dzzspzs1*WQz + ck_dzzsss01 + ak_pzspzs1*2.0d0
              dzzsdzzs = dzzsdzzs + dzzsdzzs0
              ck_dzzsdzzs01 = oo2eta*( dzzsdzzs0 - rhooeta*dzzsdzzs1 )
              dzzsfzzys0 = dzzsdzzs0*QCy + dzzsdzzs1*WQy
              dzzsfzzys = dzzsfzzys + dzzsfzzys0
              dzzsfzzxs0 = dzzsdzzs0*QCx + dzzsdzzs1*WQx
              dzzsfzzxs = dzzsfzzxs + dzzsfzzxs0
              dzzsgzzxys0 = dzzsfzzxs0*QCy + dzzsfzzxs1*WQy
              dzzsgzzxys = dzzsgzzxys + dzzsgzzxys0
              ck_pzspzs12 = oo2eta*( pzspzs1 - rhooeta*pzspzs2 )
              pzspys1 = pzsss1*QCy + pzsss2*WQy
              ak_pzspys1 = rhoo2ze*pzspys1
              pzsdyys1 = pzspys1*QCy + pzspys2*WQy + ck_pzsss12
              ak_pzsdyys1 = rhoo2ze*pzsdyys1
              dzzsfyyzs0 = dzzsdyys0*QCz + dzzsdyys1*WQz + ak_pzsdyys1*2.0d0
              pzspxs1 = pzsss1*QCx + pzsss2*WQx
              ak_pzspxs1 = rhoo2ze*pzspxs1
              pzsdxxs1 = pzspxs1*QCx + pzspxs2*WQx + ck_pzsss12
              ak_pzsdxxs1 = rhoo2ze*pzsdxxs1
              dzzsfxxzs0 = dzzsdxxs0*QCz + dzzsdxxs1*WQz + ak_pzsdxxs1*2.0d0
              pysss1 = ssss1*PAy + ssss2*WPy
              ai_pysss12 = oo2zeta*( pysss1 - rhoozeta*pysss2 )
              ai_pysss01 = oo2zeta*( pysss0 - rhoozeta*pysss1 )
              ak_pysss1 = rhoo2ze*pysss1
              dyzsss1 = pysss1*PAz + pysss2*WPz
              ak_dyzsss1 = rhoo2ze*dyzsss1
              fzzyspzs0 = fzzysss0*QCz + fzzysss1*WQz + ak_dyzsss1*2.0d0
              ck_fzzyspzs01 = oo2eta*( fzzyspzs0 - rhooeta*fzzyspzs1 )
              ck_dyzsss12 = oo2eta*( dyzsss1 - rhooeta*dyzsss2 )
              dyzspzs1 = dyzsss1*QCz + dyzsss2*WQz + ak_pysss2
              ak_dyzspzs1 = rhoo2ze*dyzspzs1
              fzzysdzzs0 = fzzyspzs0*QCz + fzzyspzs1*WQz + ck_fzzysss01 + ak_dyzspzs1*2.0d0
              fzzysdzzs = fzzysdzzs + fzzysdzzs0
              fzzysfzzys0 = fzzysdzzs0*QCy + fzzysdzzs1*WQy + ak_dzzsdzzs1
              fzzysfzzys = fzzysfzzys + fzzysfzzys0
              fzzysfzzxs0 = fzzysdzzs0*QCx + fzzysdzzs1*WQx
              fzzysfzzxs = fzzysfzzxs + fzzysfzzxs0
              fzzysgzzxys0 = fzzysfzzxs0*QCy + fzzysfzzxs1*WQy + ak_dzzsfzzxs1
              fzzysgzzxys = fzzysgzzxys + fzzysgzzxys0
              ck_dyzspzs12 = oo2eta*( dyzspzs1 - rhooeta*dyzspzs2 )
              dyzsdzzs1 = dyzspzs1*QCz + dyzspzs2*WQz + ck_dyzsss12 + ak_pyspzs2
              ak_dyzsdzzs1 = rhoo2ze*dyzsdzzs1
              fzzysfzzzs0 = fzzysdzzs0*QCz + fzzysdzzs1*WQz + ck_fzzyspzs01*2.0d0 + ak_dyzsdzzs1*2.0d0
              fzzysfzzzs = fzzysfzzzs + fzzysfzzzs0
              fzzysgzzzys0 = fzzysfzzzs0*QCy + fzzysfzzzs1*WQy + ak_dzzsfzzzs1
              fzzysgzzzys = fzzysgzzzys + fzzysgzzzys0
              dyzsfzzxs1 = dyzsdzzs1*QCx + dyzsdzzs2*WQx
              dyzsfzzzs1 = dyzsdzzs1*QCz + dyzsdzzs2*WQz + ck_dyzspzs12*2.0d0 + ak_pysdzzs2
              dyzspys1 = dyzsss1*QCy + dyzsss2*WQy + ak_pzsss2
              ak_dyzspys1 = rhoo2ze*dyzspys1
              fzzysdyzs0 = fzzyspys0*QCz + fzzyspys1*WQz + ak_dyzspys1*2.0d0
              fzzysdyzs = fzzysdyzs + fzzysdyzs0
              ck_dyzspys12 = oo2eta*( dyzspys1 - rhooeta*dyzspys2 )
              dyzsdyys1 = dyzspys1*QCy + dyzspys2*WQy + ck_dyzsss12 + ak_pzspys2
              ak_dyzsdyys1 = rhoo2ze*dyzsdyys1
              fzzysfyyzs0 = fzzysdyys0*QCz + fzzysdyys1*WQz + ak_dyzsdyys1*2.0d0
              fzzysfyyzs = fzzysfyyzs + fzzysfyyzs0
              dyzsfyyzs1 = dyzsdyys1*QCz + dyzsdyys2*WQz + ak_pysdyys2
              ak_dyzsfyyzs1 = rhoo2ze*dyzsfyyzs1
              fzzysgyyzzs0 = fzzysfyyzs0*QCz + fzzysfyyzs1*WQz + ck_fzzysdyys01 + ak_dyzsfyyzs1*2.0d0
              fzzysgyyzzs = fzzysgyyzzs + fzzysgyyzzs0
              dyzsfyyxs1 = dyzsdyys1*QCx + dyzsdyys2*WQx
              dyzsfyyys1 = dyzsdyys1*QCy + dyzsdyys2*WQy + ck_dyzspys12*2.0d0 + ak_pzsdyys2
              dyzspxs1 = dyzsss1*QCx + dyzsss2*WQx
              ak_dyzspxs1 = rhoo2ze*dyzspxs1
              dyzsdxys1 = dyzspxs1*QCy + dyzspxs2*WQy + ak_pzspxs2
              ak_dyzsdxys1 = rhoo2ze*dyzsdxys1
              fzzysfxyzs0 = fzzysdxys0*QCz + fzzysdxys1*WQz + ak_dyzsdxys1*2.0d0
              fzzysfxyzs = fzzysfxyzs + fzzysfxyzs0
              dyzsdxxs1 = dyzspxs1*QCx + dyzspxs2*WQx + ck_dyzsss12
              dyzsfxxys1 = dyzsdxxs1*QCy + dyzsdxxs2*WQy + ak_pzsdxxs2
              dyysss1 = pysss1*PAy + pysss2*WPy + ai_ssss12
              ai_dyysss12 = oo2zeta*( dyysss1 - rhoozeta*dyysss2 )
              ak_dyysss1 = rhoo2ze*dyysss1
              fyyzsss1 = dyysss1*PAz + dyysss2*WPz
              ak_fyyzsss1 = rhoo2ze*fyyzsss1
              gyyzzsss1 = fyyzsss1*PAz + fyyzsss2*WPz + ai_dyysss12
              ck_gyyzzsss12 = oo2eta*( gyyzzsss1 - rhooeta*gyyzzsss2 )
              gyyzzspzs1 = gyyzzsss1*QCz + gyyzzsss2*WQz + ak_fyyzsss2*2.0d0
              gyyzzsdzzs1 = gyyzzspzs1*QCz + gyyzzspzs2*WQz + ck_gyyzzsss12 + ak_fyyzspzs2*2.0d0
              gyyzzspys1 = gyyzzsss1*QCy + gyyzzsss2*WQy + ak_fzzysss2*2.0d0
              gyyzzsdyys1 = gyyzzspys1*QCy + gyyzzspys2*WQy + ck_gyyzzsss12 + ak_fzzyspys2*2.0d0
              gyyzzsfyyzs1 = gyyzzsdyys1*QCz + gyyzzsdyys2*WQz + ak_fyyzsdyys2*2.0d0
              ck_fyyzsss12 = oo2eta*( fyyzsss1 - rhooeta*fyyzsss2 )
              fyyzspzs1 = fyyzsss1*QCz + fyyzsss2*WQz + ak_dyysss2
              ak_fyyzspzs1 = rhoo2ze*fyyzspzs1
              fyyzsdzzs1 = fyyzspzs1*QCz + fyyzspzs2*WQz + ck_fyyzsss12 + ak_dyyspzs2
              fyyzspys1 = fyyzsss1*QCy + fyyzsss2*WQy + ak_dyzsss2*2.0d0
              ak_fyyzspys1 = rhoo2ze*fyyzspys1
              ck_fyyzspys12 = oo2eta*( fyyzspys1 - rhooeta*fyyzspys2 )
              fyyzsdyys1 = fyyzspys1*QCy + fyyzspys2*WQy + ck_fyyzsss12 + ak_dyzspys2*2.0d0
              ak_fyyzsdyys1 = rhoo2ze*fyyzsdyys1
              fyyzsfyyzs1 = fyyzsdyys1*QCz + fyyzsdyys2*WQz + ak_dyysdyys2
              ak_fyyzsfyyzs1 = rhoo2ze*fyyzsfyyzs1
              fyyzsfyyxs1 = fyyzsdyys1*QCx + fyyzsdyys2*WQx
              fyyzsfyyys1 = fyyzsdyys1*QCy + fyyzsdyys2*WQy + ck_fyyzspys12*2.0d0 + ak_dyzsdyys2*2.0d0
              fyyzspxs1 = fyyzsss1*QCx + fyyzsss2*WQx
              fyyzsdxys1 = fyyzspxs1*QCy + fyyzspxs2*WQy + ak_dyzspxs2*2.0d0
              fyyxsss1 = dyysss1*PAx + dyysss2*WPx
              ak_fyyxsss1 = rhoo2ze*fyyxsss1
              gyyxzsss1 = fyyxsss1*PAz + fyyxsss2*WPz
              ck_gyyxzsss12 = oo2eta*( gyyxzsss1 - rhooeta*gyyxzsss2 )
              gyyxzspys1 = gyyxzsss1*QCy + gyyxzsss2*WQy + ak_fxyzsss2*2.0d0
              gyyxzsdyys1 = gyyxzspys1*QCy + gyyxzspys2*WQy + ck_gyyxzsss12 + ak_fxyzspys2*2.0d0
              gyyxzsfyyxs1 = gyyxzsdyys1*QCx + gyyxzsdyys2*WQx + ak_fyyzsdyys2
              gyyxzspxs1 = gyyxzsss1*QCx + gyyxzsss2*WQx + ak_fyyzsss2
              gyyxzsdxys1 = gyyxzspxs1*QCy + gyyxzspxs2*WQy + ak_fxyzspxs2*2.0d0
              ck_fyyxsss12 = oo2eta*( fyyxsss1 - rhooeta*fyyxsss2 )
              fyyxspys1 = fyyxsss1*QCy + fyyxsss2*WQy + ak_dxysss2*2.0d0
              ak_fyyxspys1 = rhoo2ze*fyyxspys1
              ck_fyyxspys12 = oo2eta*( fyyxspys1 - rhooeta*fyyxspys2 )
              fyyxsdyys1 = fyyxspys1*QCy + fyyxspys2*WQy + ck_fyyxsss12 + ak_dxyspys2*2.0d0
              ak_fyyxsdyys1 = rhoo2ze*fyyxsdyys1
              fyyxsfyyxs1 = fyyxsdyys1*QCx + fyyxsdyys2*WQx + ak_dyysdyys2
              ak_fyyxsfyyxs1 = rhoo2ze*fyyxsfyyxs1
              fyyxsfyyys1 = fyyxsdyys1*QCy + fyyxsdyys2*WQy + ck_fyyxspys12*2.0d0 + ak_dxysdyys2*2.0d0
              fyyxspxs1 = fyyxsss1*QCx + fyyxsss2*WQx + ak_dyysss2
              ak_fyyxspxs1 = rhoo2ze*fyyxspxs1
              fyyxsdxys1 = fyyxspxs1*QCy + fyyxspxs2*WQy + ak_dxyspxs2*2.0d0
              ak_fyyxsdxys1 = rhoo2ze*fyyxsdxys1
              fyyxsdxxs1 = fyyxspxs1*QCx + fyyxspxs2*WQx + ck_fyyxsss12 + ak_dyyspxs2
              fyyxsfxxys1 = fyyxsdxxs1*QCy + fyyxsdxxs2*WQy + ak_dxysdxxs2*2.0d0
              fyyysss1 = dyysss1*PAy + dyysss2*WPy + ai_pysss12*2.0d0
              ak_fyyysss1 = rhoo2ze*fyyysss1
              gyyyzsss1 = fyyysss1*PAz + fyyysss2*WPz
              ck_gyyyzsss12 = oo2eta*( gyyyzsss1 - rhooeta*gyyyzsss2 )
              gyyyzspys1 = gyyyzsss1*QCy + gyyyzsss2*WQy + ak_fyyzsss2*3.0d0
              ck_gyyyzspys12 = oo2eta*( gyyyzspys1 - rhooeta*gyyyzspys2 )
              gyyyzsdyys1 = gyyyzspys1*QCy + gyyyzspys2*WQy + ck_gyyyzsss12 + ak_fyyzspys2*3.0d0
              gyyyzsfyyys1 = gyyyzsdyys1*QCy + gyyyzsdyys2*WQy + ck_gyyyzspys12*2.0d0 + ak_fyyzsdyys2*3.0d0
              gyyyxsss1 = fyyysss1*PAx + fyyysss2*WPx
              ck_gyyyxsss12 = oo2eta*( gyyyxsss1 - rhooeta*gyyyxsss2 )
              gyyyxspys1 = gyyyxsss1*QCy + gyyyxsss2*WQy + ak_fyyxsss2*3.0d0
              ck_gyyyxspys12 = oo2eta*( gyyyxspys1 - rhooeta*gyyyxspys2 )
              gyyyxsdyys1 = gyyyxspys1*QCy + gyyyxspys2*WQy + ck_gyyyxsss12 + ak_fyyxspys2*3.0d0
              gyyyxsfyyys1 = gyyyxsdyys1*QCy + gyyyxsdyys2*WQy + ck_gyyyxspys12*2.0d0 + ak_fyyxsdyys2*3.0d0
              gyyyxspxs1 = gyyyxsss1*QCx + gyyyxsss2*WQx + ak_fyyysss2
              gyyyysss1 = fyyysss1*PAy + fyyysss2*WPy + ai_dyysss12*3.0d0
              ck_gyyyysss12 = oo2eta*( gyyyysss1 - rhooeta*gyyyysss2 )
              gyyyyspys1 = gyyyysss1*QCy + gyyyysss2*WQy + ak_fyyysss2*4.0d0
              ck_gyyyyspys12 = oo2eta*( gyyyyspys1 - rhooeta*gyyyyspys2 )
              gyyyysdyys1 = gyyyyspys1*QCy + gyyyyspys2*WQy + ck_gyyyysss12 + ak_fyyyspys2*4.0d0
              gyyyysfyyys1 = gyyyysdyys1*QCy + gyyyysdyys2*WQy + ck_gyyyyspys12*2.0d0 + ak_fyyysdyys2*4.0d0
              ck_fyyysss12 = oo2eta*( fyyysss1 - rhooeta*fyyysss2 )
              fyyyspys1 = fyyysss1*QCy + fyyysss2*WQy + ak_dyysss2*3.0d0
              ak_fyyyspys1 = rhoo2ze*fyyyspys1
              ck_fyyyspys12 = oo2eta*( fyyyspys1 - rhooeta*fyyyspys2 )
              fyyysdyys1 = fyyyspys1*QCy + fyyyspys2*WQy + ck_fyyysss12 + ak_dyyspys2*3.0d0
              ak_fyyysdyys1 = rhoo2ze*fyyysdyys1
              fyyysfyyys1 = fyyysdyys1*QCy + fyyysdyys2*WQy + ck_fyyyspys12*2.0d0 + ak_dyysdyys2*3.0d0
              ak_fyyysfyyys1 = rhoo2ze*fyyysfyyys1
              ck_dyysss12 = oo2eta*( dyysss1 - rhooeta*dyysss2 )
              dyyspzs1 = dyysss1*QCz + dyysss2*WQz
              ak_dyyspzs1 = rhoo2ze*dyyspzs1
              dyyspys1 = dyysss1*QCy + dyysss2*WQy + ak_pysss2*2.0d0
              ak_dyyspys1 = rhoo2ze*dyyspys1
              ck_dyyspys12 = oo2eta*( dyyspys1 - rhooeta*dyyspys2 )
              dyysdyys1 = dyyspys1*QCy + dyyspys2*WQy + ck_dyysss12 + ak_pyspys2*2.0d0
              ak_dyysdyys1 = rhoo2ze*dyysdyys1
              dyysfyyzs1 = dyysdyys1*QCz + dyysdyys2*WQz
              ak_dyysfyyzs1 = rhoo2ze*dyysfyyzs1
              dyysfyyxs1 = dyysdyys1*QCx + dyysdyys2*WQx
              ak_dyysfyyxs1 = rhoo2ze*dyysfyyxs1
              dyysfyyys1 = dyysdyys1*QCy + dyysdyys2*WQy + ck_dyyspys12*2.0d0 + ak_pysdyys2*2.0d0
              ak_dyysfyyys1 = rhoo2ze*dyysfyyys1
              dyyspxs1 = dyysss1*QCx + dyysss2*WQx
              ak_dyyspxs1 = rhoo2ze*dyyspxs1
              dyysdxys1 = dyyspxs1*QCy + dyyspxs2*WQy + ak_pyspxs2*2.0d0
              ak_dyysdxys1 = rhoo2ze*dyysdxys1
              dyysdxxs1 = dyyspxs1*QCx + dyyspxs2*WQx + ck_dyysss12
              dyysfxxys1 = dyysdxxs1*QCy + dyysdxxs2*WQy + ak_pysdxxs2*2.0d0
              dyzsss0 = pysss0*PAz + pysss1*WPz
              ck_dyzsss01 = oo2eta*( dyzsss0 - rhooeta*dyzsss1 )
              dyzspzs0 = dyzsss0*QCz + dyzsss1*WQz + ak_pysss1
              ck_dyzspzs01 = oo2eta*( dyzspzs0 - rhooeta*dyzspzs1 )
              dyzspys0 = dyzsss0*QCy + dyzsss1*WQy + ak_pzsss1
              ck_dyzspys01 = oo2eta*( dyzspys0 - rhooeta*dyzspys1 )
              dyzsdyys0 = dyzspys0*QCy + dyzspys1*WQy + ck_dyzsss01 + ak_pzspys1
              ck_dyzsdyys01 = oo2eta*( dyzsdyys0 - rhooeta*dyzsdyys1 )
              dyzsfyyxs0 = dyzsdyys0*QCx + dyzsdyys1*WQx
              dyzsfyyys0 = dyzsdyys0*QCy + dyzsdyys1*WQy + ck_dyzspys01*2.0d0 + ak_pzsdyys1
              dyzspxs0 = dyzsss0*QCx + dyzsss1*WQx
              dyzsdxys0 = dyzspxs0*QCy + dyzspxs1*WQy + ak_pzspxs1
              dyzsdxxs0 = dyzspxs0*QCx + dyzspxs1*WQx + ck_dyzsss01
              dyzsfxxys0 = dyzsdxxs0*QCy + dyzsdxxs1*WQy + ak_pzsdxxs1
              dyysss0 = pysss0*PAy + pysss1*WPy + ai_ssss01
              ai_dyysss01 = oo2zeta*( dyysss0 - rhoozeta*dyysss1 )
              fyyzsss0 = dyysss0*PAz + dyysss1*WPz
              gyyzzsss0 = fyyzsss0*PAz + fyyzsss1*WPz + ai_dyysss01
              ck_gyyzzsss01 = oo2eta*( gyyzzsss0 - rhooeta*gyyzzsss1 )
              gyyzzspzs0 = gyyzzsss0*QCz + gyyzzsss1*WQz + ak_fyyzsss1*2.0d0
              gyyzzsdzzs0 = gyyzzspzs0*QCz + gyyzzspzs1*WQz + ck_gyyzzsss01 + ak_fyyzspzs1*2.0d0
              gyyzzsdzzs = gyyzzsdzzs + gyyzzsdzzs0
              gyyzzsfzzys0 = gyyzzsdzzs0*QCy + gyyzzsdzzs1*WQy + ak_fzzysdzzs1*2.0d0
              gyyzzsfzzys = gyyzzsfzzys + gyyzzsfzzys0
              gyyzzspys0 = gyyzzsss0*QCy + gyyzzsss1*WQy + ak_fzzysss1*2.0d0
              gyyzzsdyzs0 = gyyzzspys0*QCz + gyyzzspys1*WQz + ak_fyyzspys1*2.0d0
              gyyzzsdyzs = gyyzzsdyzs + gyyzzsdyzs0
              gyyzzsdyys0 = gyyzzspys0*QCy + gyyzzspys1*WQy + ck_gyyzzsss01 + ak_fzzyspys1*2.0d0
              gyyzzsdyys = gyyzzsdyys + gyyzzsdyys0
              ck_gyyzzsdyys01 = oo2eta*( gyyzzsdyys0 - rhooeta*gyyzzsdyys1 )
              gyyzzsfyyzs0 = gyyzzsdyys0*QCz + gyyzzsdyys1*WQz + ak_fyyzsdyys1*2.0d0
              gyyzzsfyyzs = gyyzzsfyyzs + gyyzzsfyyzs0
              gyyzzsgyyzzs0 = gyyzzsfyyzs0*QCz + gyyzzsfyyzs1*WQz + ck_gyyzzsdyys01 + ak_fyyzsfyyzs1*2.0d0
              gyyzzsgyyzzs = gyyzzsgyyzzs + gyyzzsgyyzzs0
              ck_fyyzsss01 = oo2eta*( fyyzsss0 - rhooeta*fyyzsss1 )
              fyyzspzs0 = fyyzsss0*QCz + fyyzsss1*WQz + ak_dyysss1
              fyyzsdzzs0 = fyyzspzs0*QCz + fyyzspzs1*WQz + ck_fyyzsss01 + ak_dyyspzs1
              fyyzsfzzys0 = fyyzsdzzs0*QCy + fyyzsdzzs1*WQy + ak_dyzsdzzs1*2.0d0
              fyyzsfzzys = fyyzsfzzys + fyyzsfzzys0
              fyyzspys0 = fyyzsss0*QCy + fyyzsss1*WQy + ak_dyzsss1*2.0d0
              ck_fyyzspys01 = oo2eta*( fyyzspys0 - rhooeta*fyyzspys1 )
              fyyzsdyzs0 = fyyzspys0*QCz + fyyzspys1*WQz + ak_dyyspys1
              fyyzsdyzs = fyyzsdyzs + fyyzsdyzs0
              fyyzsdyys0 = fyyzspys0*QCy + fyyzspys1*WQy + ck_fyyzsss01 + ak_dyzspys1*2.0d0
              fyyzsdyys = fyyzsdyys + fyyzsdyys0
              ck_fyyzsdyys01 = oo2eta*( fyyzsdyys0 - rhooeta*fyyzsdyys1 )
              fyyzsfyyzs0 = fyyzsdyys0*QCz + fyyzsdyys1*WQz + ak_dyysdyys1
              fyyzsfyyzs = fyyzsfyyzs + fyyzsfyyzs0
              fyyzsgyyzzs0 = fyyzsfyyzs0*QCz + fyyzsfyyzs1*WQz + ck_fyyzsdyys01 + ak_dyysfyyzs1
              fyyzsgyyzzs = fyyzsgyyzzs + fyyzsgyyzzs0
              fyyzsfyyxs0 = fyyzsdyys0*QCx + fyyzsdyys1*WQx
              fyyzsfyyxs = fyyzsfyyxs + fyyzsfyyxs0
              fyyzsgyyxzs0 = fyyzsfyyxs0*QCz + fyyzsfyyxs1*WQz + ak_dyysfyyxs1
              fyyzsgyyxzs = fyyzsgyyxzs + fyyzsgyyxzs0
              fyyzsfyyys0 = fyyzsdyys0*QCy + fyyzsdyys1*WQy + ck_fyyzspys01*2.0d0 + ak_dyzsdyys1*2.0d0
              fyyzsfyyys = fyyzsfyyys + fyyzsfyyys0
              fyyzsgyyyzs0 = fyyzsfyyys0*QCz + fyyzsfyyys1*WQz + ak_dyysfyyys1
              fyyzsgyyyzs = fyyzsgyyyzs + fyyzsgyyyzs0
              fyyzspxs0 = fyyzsss0*QCx + fyyzsss1*WQx
              fyyzsdxys0 = fyyzspxs0*QCy + fyyzspxs1*WQy + ak_dyzspxs1*2.0d0
              fyyzsfxyzs0 = fyyzsdxys0*QCz + fyyzsdxys1*WQz + ak_dyysdxys1
              fyyzsfxyzs = fyyzsfxyzs + fyyzsfxyzs0
              fyyxsss0 = dyysss0*PAx + dyysss1*WPx
              gyyxzsss0 = fyyxsss0*PAz + fyyxsss1*WPz
              ck_gyyxzsss01 = oo2eta*( gyyxzsss0 - rhooeta*gyyxzsss1 )
              gyyxzspxs0 = gyyxzsss0*QCx + gyyxzsss1*WQx + ak_fyyzsss1
              gyyxzsdxzs0 = gyyxzspxs0*QCz + gyyxzspxs1*WQz + ak_fyyxspxs1
              gyyxzsdxzs = gyyxzsdxzs + gyyxzsdxzs0
              ck_fyyxsss01 = oo2eta*( fyyxsss0 - rhooeta*fyyxsss1 )
              fyyxspxs0 = fyyxsss0*QCx + fyyxsss1*WQx + ak_dyysss1
              fyyxsdxxs0 = fyyxspxs0*QCx + fyyxspxs1*WQx + ck_fyyxsss01 + ak_dyyspxs1
              ck_fyyxsdxxs01 = oo2eta*( fyyxsdxxs0 - rhooeta*fyyxsdxxs1 )
              fyyysss0 = dyysss0*PAy + dyysss1*WPy + ai_pysss01*2.0d0
              gyyyzsss0 = fyyysss0*PAz + fyyysss1*WPz
              ck_gyyyzsss01 = oo2eta*( gyyyzsss0 - rhooeta*gyyyzsss1 )
              gyyyzspys0 = gyyyzsss0*QCy + gyyyzsss1*WQy + ak_fyyzsss1*3.0d0
              ck_gyyyzspys01 = oo2eta*( gyyyzspys0 - rhooeta*gyyyzspys1 )
              gyyyzsdyzs0 = gyyyzspys0*QCz + gyyyzspys1*WQz + ak_fyyyspys1
              gyyyzsdyzs = gyyyzsdyzs + gyyyzsdyzs0
              gyyyzsdyys0 = gyyyzspys0*QCy + gyyyzspys1*WQy + ck_gyyyzsss01 + ak_fyyzspys1*3.0d0
              gyyyzsdyys = gyyyzsdyys + gyyyzsdyys0
              gyyyzsfyyzs0 = gyyyzsdyys0*QCz + gyyyzsdyys1*WQz + ak_fyyysdyys1
              gyyyzsfyyzs = gyyyzsfyyzs + gyyyzsfyyzs0
              gyyyzsfyyys0 = gyyyzsdyys0*QCy + gyyyzsdyys1*WQy + ck_gyyyzspys01*2.0d0 + ak_fyyzsdyys1*3.0d0
              gyyyzsfyyys = gyyyzsfyyys + gyyyzsfyyys0
              gyyyzsgyyyzs0 = gyyyzsfyyys0*QCz + gyyyzsfyyys1*WQz + ak_fyyysfyyys1
              gyyyzsgyyyzs = gyyyzsgyyyzs + gyyyzsgyyyzs0
              gyyyxsss0 = fyyysss0*PAx + fyyysss1*WPx
              ck_gyyyxsss01 = oo2eta*( gyyyxsss0 - rhooeta*gyyyxsss1 )
              gyyyxspys0 = gyyyxsss0*QCy + gyyyxsss1*WQy + ak_fyyxsss1*3.0d0
              ck_gyyyxspys01 = oo2eta*( gyyyxspys0 - rhooeta*gyyyxspys1 )
              gyyyxsdyys0 = gyyyxspys0*QCy + gyyyxspys1*WQy + ck_gyyyxsss01 + ak_fyyxspys1*3.0d0
              gyyyxsdyys = gyyyxsdyys + gyyyxsdyys0
              gyyyxsfyyxs0 = gyyyxsdyys0*QCx + gyyyxsdyys1*WQx + ak_fyyysdyys1
              gyyyxsfyyxs = gyyyxsfyyxs + gyyyxsfyyxs0
              gyyyxsfyyys0 = gyyyxsdyys0*QCy + gyyyxsdyys1*WQy + ck_gyyyxspys01*2.0d0 + ak_fyyxsdyys1*3.0d0
              gyyyxsfyyys = gyyyxsfyyys + gyyyxsfyyys0
              gyyyxsgyyyxs0 = gyyyxsfyyys0*QCx + gyyyxsfyyys1*WQx + ak_fyyysfyyys1
              gyyyxsgyyyxs = gyyyxsgyyyxs + gyyyxsgyyyxs0
              gyyyxspxs0 = gyyyxsss0*QCx + gyyyxsss1*WQx + ak_fyyysss1
              gyyyxsdxys0 = gyyyxspxs0*QCy + gyyyxspxs1*WQy + ak_fyyxspxs1*3.0d0
              gyyyxsdxys = gyyyxsdxys + gyyyxsdxys0
              gyyyysss0 = fyyysss0*PAy + fyyysss1*WPy + ai_dyysss01*3.0d0
              ck_gyyyysss01 = oo2eta*( gyyyysss0 - rhooeta*gyyyysss1 )
              gyyyyspys0 = gyyyysss0*QCy + gyyyysss1*WQy + ak_fyyysss1*4.0d0
              ck_gyyyyspys01 = oo2eta*( gyyyyspys0 - rhooeta*gyyyyspys1 )
              gyyyysdyys0 = gyyyyspys0*QCy + gyyyyspys1*WQy + ck_gyyyysss01 + ak_fyyyspys1*4.0d0
              gyyyysdyys = gyyyysdyys + gyyyysdyys0
              ck_gyyyysdyys01 = oo2eta*( gyyyysdyys0 - rhooeta*gyyyysdyys1 )
              gyyyysfyyys0 = gyyyysdyys0*QCy + gyyyysdyys1*WQy + ck_gyyyyspys01*2.0d0 + ak_fyyysdyys1*4.0d0
              gyyyysfyyys = gyyyysfyyys + gyyyysfyyys0
              gyyyysgyyyys0 = gyyyysfyyys0*QCy + gyyyysfyyys1*WQy + ck_gyyyysdyys01*3.0d0 + ak_fyyysfyyys1*4.0d0
              gyyyysgyyyys = gyyyysgyyyys + gyyyysgyyyys0
              ck_fyyysss01 = oo2eta*( fyyysss0 - rhooeta*fyyysss1 )
              fyyyspys0 = fyyysss0*QCy + fyyysss1*WQy + ak_dyysss1*3.0d0
              ck_fyyyspys01 = oo2eta*( fyyyspys0 - rhooeta*fyyyspys1 )
              fyyysdyys0 = fyyyspys0*QCy + fyyyspys1*WQy + ck_fyyysss01 + ak_dyyspys1*3.0d0
              fyyysdyys = fyyysdyys + fyyysdyys0
              ck_fyyysdyys01 = oo2eta*( fyyysdyys0 - rhooeta*fyyysdyys1 )
              fyyysfyyzs0 = fyyysdyys0*QCz + fyyysdyys1*WQz
              fyyysfyyzs = fyyysfyyzs + fyyysfyyzs0
              fyyysfyyxs0 = fyyysdyys0*QCx + fyyysdyys1*WQx
              fyyysfyyxs = fyyysfyyxs + fyyysfyyxs0
              fyyysfyyys0 = fyyysdyys0*QCy + fyyysdyys1*WQy + ck_fyyyspys01*2.0d0 + ak_dyysdyys1*3.0d0
              fyyysfyyys = fyyysfyyys + fyyysfyyys0
              fyyysgyyyzs0 = fyyysfyyys0*QCz + fyyysfyyys1*WQz
              fyyysgyyyzs = fyyysgyyyzs + fyyysgyyyzs0
              fyyysgyyyxs0 = fyyysfyyys0*QCx + fyyysfyyys1*WQx
              fyyysgyyyxs = fyyysgyyyxs + fyyysgyyyxs0
              fyyysgyyyys0 = fyyysfyyys0*QCy + fyyysfyyys1*WQy + ck_fyyysdyys01*3.0d0 + ak_dyysfyyys1*3.0d0
              fyyysgyyyys = fyyysgyyyys + fyyysgyyyys0
              ck_dyysss01 = oo2eta*( dyysss0 - rhooeta*dyysss1 )
              dyyspys0 = dyysss0*QCy + dyysss1*WQy + ak_pysss1*2.0d0
              ck_dyyspys01 = oo2eta*( dyyspys0 - rhooeta*dyyspys1 )
              dyyspxs0 = dyysss0*QCx + dyysss1*WQx
              dyysdxxs0 = dyyspxs0*QCx + dyyspxs1*WQx + ck_dyysss01
              ck_dyysdxxs01 = oo2eta*( dyysdxxs0 - rhooeta*dyysdxxs1 )
              ck_pysss12 = oo2eta*( pysss1 - rhooeta*pysss2 )
              pyspzs1 = pysss1*QCz + pysss2*WQz
              ak_pyspzs1 = rhoo2ze*pyspzs1
              dyzsdzzs0 = dyzspzs0*QCz + dyzspzs1*WQz + ck_dyzsss01 + ak_pyspzs1
              dyzsfzzxs0 = dyzsdzzs0*QCx + dyzsdzzs1*WQx
              pysdzzs1 = pyspzs1*QCz + pyspzs2*WQz + ck_pysss12
              ak_pysdzzs1 = rhoo2ze*pysdzzs1
              dyzsfzzzs0 = dyzsdzzs0*QCz + dyzsdzzs1*WQz + ck_dyzspzs01*2.0d0 + ak_pysdzzs1
              pyspys1 = pysss1*QCy + pysss2*WQy + ak_ssss2
              ak_pyspys1 = rhoo2ze*pyspys1
              dyzsdyzs0 = dyzspys0*QCz + dyzspys1*WQz + ak_pyspys1
              dyzsdyzs = dyzsdyzs + dyzsdyzs0
              dyysdyys0 = dyyspys0*QCy + dyyspys1*WQy + ck_dyysss01 + ak_pyspys1*2.0d0
              dyysdyys = dyysdyys + dyysdyys0
              ck_dyysdyys01 = oo2eta*( dyysdyys0 - rhooeta*dyysdyys1 )
              dyysfyyzs0 = dyysdyys0*QCz + dyysdyys1*WQz
              dyysfyyzs = dyysfyyzs + dyysfyyzs0
              dyysgyyzzs0 = dyysfyyzs0*QCz + dyysfyyzs1*WQz + ck_dyysdyys01
              dyysgyyzzs = dyysgyyzzs + dyysgyyzzs0
              dyysfyyxs0 = dyysdyys0*QCx + dyysdyys1*WQx
              dyysfyyxs = dyysfyyxs + dyysfyyxs0
              dyysgyyxzs0 = dyysfyyxs0*QCz + dyysfyyxs1*WQz
              dyysgyyxzs = dyysgyyxzs + dyysgyyxzs0
              ck_pyspys12 = oo2eta*( pyspys1 - rhooeta*pyspys2 )
              pyspxs1 = pysss1*QCx + pysss2*WQx
              ak_pyspxs1 = rhoo2ze*pyspxs1
              pysdxxs1 = pyspxs1*QCx + pyspxs2*WQx + ck_pysss12
              ak_pysdxxs1 = rhoo2ze*pysdxxs1
              dyysfxxys0 = dyysdxxs0*QCy + dyysdxxs1*WQy + ak_pysdxxs1*2.0d0
              pxsss1 = ssss1*PAx + ssss2*WPx
              ai_pxsss12 = oo2zeta*( pxsss1 - rhoozeta*pxsss2 )
              ai_pxsss01 = oo2zeta*( pxsss0 - rhoozeta*pxsss1 )
              ak_pxsss1 = rhoo2ze*pxsss1
              dxzsss1 = pxsss1*PAz + pxsss2*WPz
              ak_dxzsss1 = rhoo2ze*dxzsss1
              fzzxspzs0 = fzzxsss0*QCz + fzzxsss1*WQz + ak_dxzsss1*2.0d0
              ck_fzzxspzs01 = oo2eta*( fzzxspzs0 - rhooeta*fzzxspzs1 )
              ck_dxzsss12 = oo2eta*( dxzsss1 - rhooeta*dxzsss2 )
              dxzspzs1 = dxzsss1*QCz + dxzsss2*WQz + ak_pxsss2
              ak_dxzspzs1 = rhoo2ze*dxzspzs1
              fzzxsdzzs0 = fzzxspzs0*QCz + fzzxspzs1*WQz + ck_fzzxsss01 + ak_dxzspzs1*2.0d0
              fzzxsdzzs = fzzxsdzzs + fzzxsdzzs0
              fzzxsfzzys0 = fzzxsdzzs0*QCy + fzzxsdzzs1*WQy
              fzzxsfzzys = fzzxsfzzys + fzzxsfzzys0
              fzzxsfzzxs0 = fzzxsdzzs0*QCx + fzzxsdzzs1*WQx + ak_dzzsdzzs1
              fzzxsfzzxs = fzzxsfzzxs + fzzxsfzzxs0
              fzzxsgzzxys0 = fzzxsfzzxs0*QCy + fzzxsfzzxs1*WQy
              fzzxsgzzxys = fzzxsgzzxys + fzzxsgzzxys0
              ck_dxzspzs12 = oo2eta*( dxzspzs1 - rhooeta*dxzspzs2 )
              dxzsdzzs1 = dxzspzs1*QCz + dxzspzs2*WQz + ck_dxzsss12 + ak_pxspzs2
              ak_dxzsdzzs1 = rhoo2ze*dxzsdzzs1
              fzzxsfzzzs0 = fzzxsdzzs0*QCz + fzzxsdzzs1*WQz + ck_fzzxspzs01*2.0d0 + ak_dxzsdzzs1*2.0d0
              fzzxsfzzzs = fzzxsfzzzs + fzzxsfzzzs0
              fzzxsgzzzxs0 = fzzxsfzzzs0*QCx + fzzxsfzzzs1*WQx + ak_dzzsfzzzs1
              fzzxsgzzzxs = fzzxsgzzzxs + fzzxsgzzzxs0
              dxzsfzzxs1 = dxzsdzzs1*QCx + dxzsdzzs2*WQx + ak_pzsdzzs2
              ak_dxzsfzzxs1 = rhoo2ze*dxzsfzzxs1
              dxzsfzzzs1 = dxzsdzzs1*QCz + dxzsdzzs2*WQz + ck_dxzspzs12*2.0d0 + ak_pxsdzzs2
              dxzspys1 = dxzsss1*QCy + dxzsss2*WQy
              ak_dxzspys1 = rhoo2ze*dxzspys1
              dxzsdyys1 = dxzspys1*QCy + dxzspys2*WQy + ck_dxzsss12
              dxzsfyyxs1 = dxzsdyys1*QCx + dxzsdyys2*WQx + ak_pzsdyys2
              dxzspxs1 = dxzsss1*QCx + dxzsss2*WQx + ak_pzsss2
              ak_dxzspxs1 = rhoo2ze*dxzspxs1
              fzzxsdxzs0 = fzzxspxs0*QCz + fzzxspxs1*WQz + ak_dxzspxs1*2.0d0
              fzzxsdxzs = fzzxsdxzs + fzzxsdxzs0
              ck_dxzspxs12 = oo2eta*( dxzspxs1 - rhooeta*dxzspxs2 )
              dxzsdxys1 = dxzspxs1*QCy + dxzspxs2*WQy
              ak_dxzsdxys1 = rhoo2ze*dxzsdxys1
              fzzxsfxyzs0 = fzzxsdxys0*QCz + fzzxsdxys1*WQz + ak_dxzsdxys1*2.0d0
              fzzxsfxyzs = fzzxsfxyzs + fzzxsfxyzs0
              dxzsdxxs1 = dxzspxs1*QCx + dxzspxs2*WQx + ck_dxzsss12 + ak_pzspxs2
              ak_dxzsdxxs1 = rhoo2ze*dxzsdxxs1
              fzzxsfxxzs0 = fzzxsdxxs0*QCz + fzzxsdxxs1*WQz + ak_dxzsdxxs1*2.0d0
              fzzxsfxxzs = fzzxsfxxzs + fzzxsfxxzs0
              dxzsfxxzs1 = dxzsdxxs1*QCz + dxzsdxxs2*WQz + ak_pxsdxxs2
              ak_dxzsfxxzs1 = rhoo2ze*dxzsfxxzs1
              fzzxsgxxzzs0 = fzzxsfxxzs0*QCz + fzzxsfxxzs1*WQz + ck_fzzxsdxxs01 + ak_dxzsfxxzs1*2.0d0
              fzzxsgxxzzs = fzzxsgxxzzs + fzzxsgxxzzs0
              dxzsfxxys1 = dxzsdxxs1*QCy + dxzsdxxs2*WQy
              dxzsfxxxs1 = dxzsdxxs1*QCx + dxzsdxxs2*WQx + ck_dxzspxs12*2.0d0 + ak_pzsdxxs2
              dxysss1 = pxsss1*PAy + pxsss2*WPy
              ak_dxysss1 = rhoo2ze*dxysss1
              fyyxspys0 = fyyxsss0*QCy + fyyxsss1*WQy + ak_dxysss1*2.0d0
              ck_fyyxspys01 = oo2eta*( fyyxspys0 - rhooeta*fyyxspys1 )
              fxyzsss1 = dxysss1*PAz + dxysss2*WPz
              ak_fxyzsss1 = rhoo2ze*fxyzsss1
              gzzxyspzs0 = gzzxysss0*QCz + gzzxysss1*WQz + ak_fxyzsss1*2.0d0
              gyyxzspys0 = gyyxzsss0*QCy + gyyxzsss1*WQy + ak_fxyzsss1*2.0d0
              gyyxzsdyzs0 = gyyxzspys0*QCz + gyyxzspys1*WQz + ak_fyyxspys1
              gyyxzsdyzs = gyyxzsdyzs + gyyxzsdyzs0
              ck_fxyzsss12 = oo2eta*( fxyzsss1 - rhooeta*fxyzsss2 )
              fxyzspzs1 = fxyzsss1*QCz + fxyzsss2*WQz + ak_dxysss2
              ak_fxyzspzs1 = rhoo2ze*fxyzspzs1
              gzzxysdzzs0 = gzzxyspzs0*QCz + gzzxyspzs1*WQz + ck_gzzxysss01 + ak_fxyzspzs1*2.0d0
              gzzxysdzzs = gzzxysdzzs + gzzxysdzzs0
              gzzxysfzzys0 = gzzxysdzzs0*QCy + gzzxysdzzs1*WQy + ak_fzzxsdzzs1
              gzzxysfzzys = gzzxysfzzys + gzzxysfzzys0
              gzzxysfzzxs0 = gzzxysdzzs0*QCx + gzzxysdzzs1*WQx + ak_fzzysdzzs1
              gzzxysfzzxs = gzzxysfzzxs + gzzxysfzzxs0
              gzzxysgzzxys0 = gzzxysfzzxs0*QCy + gzzxysfzzxs1*WQy + ak_fzzxsfzzxs1
              gzzxysgzzxys = gzzxysgzzxys + gzzxysgzzxys0
              fxyzsdzzs1 = fxyzspzs1*QCz + fxyzspzs2*WQz + ck_fxyzsss12 + ak_dxyspzs2
              fxyzsfzzxs1 = fxyzsdzzs1*QCx + fxyzsdzzs2*WQx + ak_dyzsdzzs2
              fxyzspys1 = fxyzsss1*QCy + fxyzsss2*WQy + ak_dxzsss2
              ak_fxyzspys1 = rhoo2ze*fxyzspys1
              gzzxysdyzs0 = gzzxyspys0*QCz + gzzxyspys1*WQz + ak_fxyzspys1*2.0d0
              gzzxysdyzs = gzzxysdyzs + gzzxysdyzs0
              gyyxzsdyys0 = gyyxzspys0*QCy + gyyxzspys1*WQy + ck_gyyxzsss01 + ak_fxyzspys1*2.0d0
              gyyxzsdyys = gyyxzsdyys + gyyxzsdyys0
              gyyxzsfyyzs0 = gyyxzsdyys0*QCz + gyyxzsdyys1*WQz + ak_fyyxsdyys1
              gyyxzsfyyzs = gyyxzsfyyzs + gyyxzsfyyzs0
              gyyxzsfyyxs0 = gyyxzsdyys0*QCx + gyyxzsdyys1*WQx + ak_fyyzsdyys1
              gyyxzsfyyxs = gyyxzsfyyxs + gyyxzsfyyxs0
              gyyxzsgyyxzs0 = gyyxzsfyyxs0*QCz + gyyxzsfyyxs1*WQz + ak_fyyxsfyyxs1
              gyyxzsgyyxzs = gyyxzsgyyxzs + gyyxzsgyyxzs0
              fxyzsdyys1 = fxyzspys1*QCy + fxyzspys2*WQy + ck_fxyzsss12 + ak_dxzspys2
              fxyzsfyyxs1 = fxyzsdyys1*QCx + fxyzsdyys2*WQx + ak_dyzsdyys2
              fxyzspxs1 = fxyzsss1*QCx + fxyzsss2*WQx + ak_dyzsss2
              ak_fxyzspxs1 = rhoo2ze*fxyzspxs1
              gzzxysdxzs0 = gzzxyspxs0*QCz + gzzxyspxs1*WQz + ak_fxyzspxs1*2.0d0
              gzzxysdxzs = gzzxysdxzs + gzzxysdxzs0
              gyyxzsdxys0 = gyyxzspxs0*QCy + gyyxzspxs1*WQy + ak_fxyzspxs1*2.0d0
              gyyxzsdxys = gyyxzsdxys + gyyxzsdxys0
              gyyxzsfxyzs0 = gyyxzsdxys0*QCz + gyyxzsdxys1*WQz + ak_fyyxsdxys1
              gyyxzsfxyzs = gyyxzsfxyzs + gyyxzsfxyzs0
              fxyzsdxys1 = fxyzspxs1*QCy + fxyzspxs2*WQy + ak_dxzspxs2
              ak_fxyzsdxys1 = rhoo2ze*fxyzsdxys1
              gzzxysfxyzs0 = gzzxysdxys0*QCz + gzzxysdxys1*WQz + ak_fxyzsdxys1*2.0d0
              gzzxysfxyzs = gzzxysfxyzs + gzzxysfxyzs0
              fxyzsdxxs1 = fxyzspxs1*QCx + fxyzspxs2*WQx + ck_fxyzsss12 + ak_dyzspxs2
              fxyzsfxxys1 = fxyzsdxxs1*QCy + fxyzsdxxs2*WQy + ak_dxzsdxxs2
              ck_dxysss12 = oo2eta*( dxysss1 - rhooeta*dxysss2 )
              dxyspzs1 = dxysss1*QCz + dxysss2*WQz
              ak_dxyspzs1 = rhoo2ze*dxyspzs1
              dxysdzzs1 = dxyspzs1*QCz + dxyspzs2*WQz + ck_dxysss12
              dxysfzzxs1 = dxysdzzs1*QCx + dxysdzzs2*WQx + ak_pysdzzs2
              dxyspys1 = dxysss1*QCy + dxysss2*WQy + ak_pxsss2
              ak_dxyspys1 = rhoo2ze*dxyspys1
              fyyxsdyys0 = fyyxspys0*QCy + fyyxspys1*WQy + ck_fyyxsss01 + ak_dxyspys1*2.0d0
              fyyxsdyys = fyyxsdyys + fyyxsdyys0
              fyyxsfyyzs0 = fyyxsdyys0*QCz + fyyxsdyys1*WQz
              fyyxsfyyzs = fyyxsfyyzs + fyyxsfyyzs0
              fyyxsfyyxs0 = fyyxsdyys0*QCx + fyyxsdyys1*WQx + ak_dyysdyys1
              fyyxsfyyxs = fyyxsfyyxs + fyyxsfyyxs0
              fyyxsgyyxzs0 = fyyxsfyyxs0*QCz + fyyxsfyyxs1*WQz
              fyyxsgyyxzs = fyyxsgyyxzs + fyyxsgyyxzs0
              ck_dxyspys12 = oo2eta*( dxyspys1 - rhooeta*dxyspys2 )
              dxysdyys1 = dxyspys1*QCy + dxyspys2*WQy + ck_dxysss12 + ak_pxspys2
              ak_dxysdyys1 = rhoo2ze*dxysdyys1
              fyyxsfyyys0 = fyyxsdyys0*QCy + fyyxsdyys1*WQy + ck_fyyxspys01*2.0d0 + ak_dxysdyys1*2.0d0
              fyyxsfyyys = fyyxsfyyys + fyyxsfyyys0
              fyyxsgyyyxs0 = fyyxsfyyys0*QCx + fyyxsfyyys1*WQx + ak_dyysfyyys1
              fyyxsgyyyxs = fyyxsgyyyxs + fyyxsgyyyxs0
              dxysfyyxs1 = dxysdyys1*QCx + dxysdyys2*WQx + ak_pysdyys2
              ak_dxysfyyxs1 = rhoo2ze*dxysfyyxs1
              dxysfyyys1 = dxysdyys1*QCy + dxysdyys2*WQy + ck_dxyspys12*2.0d0 + ak_pxsdyys2
              dxyspxs1 = dxysss1*QCx + dxysss2*WQx + ak_pysss2
              ak_dxyspxs1 = rhoo2ze*dxyspxs1
              fyyxsdxys0 = fyyxspxs0*QCy + fyyxspxs1*WQy + ak_dxyspxs1*2.0d0
              fyyxsdxys = fyyxsdxys + fyyxsdxys0
              fyyxsfxyzs0 = fyyxsdxys0*QCz + fyyxsdxys1*WQz
              fyyxsfxyzs = fyyxsfxyzs + fyyxsfxyzs0
              ck_dxyspxs12 = oo2eta*( dxyspxs1 - rhooeta*dxyspxs2 )
              dxysdxys1 = dxyspxs1*QCy + dxyspxs2*WQy + ak_pxspxs2
              ak_dxysdxys1 = rhoo2ze*dxysdxys1
              dxysdxxs1 = dxyspxs1*QCx + dxyspxs2*WQx + ck_dxysss12 + ak_pyspxs2
              ak_dxysdxxs1 = rhoo2ze*dxysdxxs1
              fyyxsfxxys0 = fyyxsdxxs0*QCy + fyyxsdxxs1*WQy + ak_dxysdxxs1*2.0d0
              fyyxsfxxys = fyyxsfxxys + fyyxsfxxys0
              dxysfxxys1 = dxysdxxs1*QCy + dxysdxxs2*WQy + ak_pxsdxxs2
              ak_dxysfxxys1 = rhoo2ze*dxysfxxys1
              fyyxsgxxyys0 = fyyxsfxxys0*QCy + fyyxsfxxys1*WQy + ck_fyyxsdxxs01 + ak_dxysfxxys1*2.0d0
              fyyxsgxxyys = fyyxsgxxyys + fyyxsgxxyys0
              dxysfxxxs1 = dxysdxxs1*QCx + dxysdxxs2*WQx + ck_dxyspxs12*2.0d0 + ak_pysdxxs2
              dxxsss1 = pxsss1*PAx + pxsss2*WPx + ai_ssss12
              ai_dxxsss12 = oo2zeta*( dxxsss1 - rhoozeta*dxxsss2 )
              ak_dxxsss1 = rhoo2ze*dxxsss1
              fxxzsss1 = dxxsss1*PAz + dxxsss2*WPz
              ak_fxxzsss1 = rhoo2ze*fxxzsss1
              gxxzzsss1 = fxxzsss1*PAz + fxxzsss2*WPz + ai_dxxsss12
              ck_gxxzzsss12 = oo2eta*( gxxzzsss1 - rhooeta*gxxzzsss2 )
              gxxzzspzs1 = gxxzzsss1*QCz + gxxzzsss2*WQz + ak_fxxzsss2*2.0d0
              gxxzzsdzzs1 = gxxzzspzs1*QCz + gxxzzspzs2*WQz + ck_gxxzzsss12 + ak_fxxzspzs2*2.0d0
              gxxzzspxs1 = gxxzzsss1*QCx + gxxzzsss2*WQx + ak_fzzxsss2*2.0d0
              gxxzzsdxxs1 = gxxzzspxs1*QCx + gxxzzspxs2*WQx + ck_gxxzzsss12 + ak_fzzxspxs2*2.0d0
              gxxzzsfxxzs1 = gxxzzsdxxs1*QCz + gxxzzsdxxs2*WQz + ak_fxxzsdxxs2*2.0d0
              ck_fxxzsss12 = oo2eta*( fxxzsss1 - rhooeta*fxxzsss2 )
              fxxzspzs1 = fxxzsss1*QCz + fxxzsss2*WQz + ak_dxxsss2
              ak_fxxzspzs1 = rhoo2ze*fxxzspzs1
              fxxzsdzzs1 = fxxzspzs1*QCz + fxxzspzs2*WQz + ck_fxxzsss12 + ak_dxxspzs2
              fxxzspxs1 = fxxzsss1*QCx + fxxzsss2*WQx + ak_dxzsss2*2.0d0
              ak_fxxzspxs1 = rhoo2ze*fxxzspxs1
              ck_fxxzspxs12 = oo2eta*( fxxzspxs1 - rhooeta*fxxzspxs2 )
              fxxzsdxys1 = fxxzspxs1*QCy + fxxzspxs2*WQy
              fxxzsdxxs1 = fxxzspxs1*QCx + fxxzspxs2*WQx + ck_fxxzsss12 + ak_dxzspxs2*2.0d0
              ak_fxxzsdxxs1 = rhoo2ze*fxxzsdxxs1
              fxxzsfxxzs1 = fxxzsdxxs1*QCz + fxxzsdxxs2*WQz + ak_dxxsdxxs2
              ak_fxxzsfxxzs1 = rhoo2ze*fxxzsfxxzs1
              fxxzsfxxys1 = fxxzsdxxs1*QCy + fxxzsdxxs2*WQy
              fxxzsfxxxs1 = fxxzsdxxs1*QCx + fxxzsdxxs2*WQx + ck_fxxzspxs12*2.0d0 + ak_dxzsdxxs2*2.0d0
              fxxysss1 = dxxsss1*PAy + dxxsss2*WPy
              ak_fxxysss1 = rhoo2ze*fxxysss1
              gxxyzsss1 = fxxysss1*PAz + fxxysss2*WPz
              ck_gxxyzsss12 = oo2eta*( gxxyzsss1 - rhooeta*gxxyzsss2 )
              gxxyzspys1 = gxxyzsss1*QCy + gxxyzsss2*WQy + ak_fxxzsss2
              gxxyzspxs1 = gxxyzsss1*QCx + gxxyzsss2*WQx + ak_fxyzsss2*2.0d0
              gxxyzsdxys1 = gxxyzspxs1*QCy + gxxyzspxs2*WQy + ak_fxxzspxs2
              gxxyzsdxxs1 = gxxyzspxs1*QCx + gxxyzspxs2*WQx + ck_gxxyzsss12 + ak_fxyzspxs2*2.0d0
              gxxyzsfxxys1 = gxxyzsdxxs1*QCy + gxxyzsdxxs2*WQy + ak_fxxzsdxxs2
              gxxyysss1 = fxxysss1*PAy + fxxysss2*WPy + ai_dxxsss12
              ck_gxxyysss12 = oo2eta*( gxxyysss1 - rhooeta*gxxyysss2 )
              gxxyyspys1 = gxxyysss1*QCy + gxxyysss2*WQy + ak_fxxysss2*2.0d0
              gxxyysdyys1 = gxxyyspys1*QCy + gxxyyspys2*WQy + ck_gxxyysss12 + ak_fxxyspys2*2.0d0
              gxxyyspxs1 = gxxyysss1*QCx + gxxyysss2*WQx + ak_fyyxsss2*2.0d0
              gxxyysdxxs1 = gxxyyspxs1*QCx + gxxyyspxs2*WQx + ck_gxxyysss12 + ak_fyyxspxs2*2.0d0
              gxxyysfxxys1 = gxxyysdxxs1*QCy + gxxyysdxxs2*WQy + ak_fxxysdxxs2*2.0d0
              ck_fxxysss12 = oo2eta*( fxxysss1 - rhooeta*fxxysss2 )
              fxxyspys1 = fxxysss1*QCy + fxxysss2*WQy + ak_dxxsss2
              ak_fxxyspys1 = rhoo2ze*fxxyspys1
              fxxysdyys1 = fxxyspys1*QCy + fxxyspys2*WQy + ck_fxxysss12 + ak_dxxspys2
              fxxyspxs1 = fxxysss1*QCx + fxxysss2*WQx + ak_dxysss2*2.0d0
              ak_fxxyspxs1 = rhoo2ze*fxxyspxs1
              ck_fxxyspxs12 = oo2eta*( fxxyspxs1 - rhooeta*fxxyspxs2 )
              fxxysdxys1 = fxxyspxs1*QCy + fxxyspxs2*WQy + ak_dxxspxs2
              ak_fxxysdxys1 = rhoo2ze*fxxysdxys1
              fxxysdxxs1 = fxxyspxs1*QCx + fxxyspxs2*WQx + ck_fxxysss12 + ak_dxyspxs2*2.0d0
              ak_fxxysdxxs1 = rhoo2ze*fxxysdxxs1
              fxxysfxxys1 = fxxysdxxs1*QCy + fxxysdxxs2*WQy + ak_dxxsdxxs2
              ak_fxxysfxxys1 = rhoo2ze*fxxysfxxys1
              fxxysfxxxs1 = fxxysdxxs1*QCx + fxxysdxxs2*WQx + ck_fxxyspxs12*2.0d0 + ak_dxysdxxs2*2.0d0
              fxxxsss1 = dxxsss1*PAx + dxxsss2*WPx + ai_pxsss12*2.0d0
              ak_fxxxsss1 = rhoo2ze*fxxxsss1
              gxxxzsss1 = fxxxsss1*PAz + fxxxsss2*WPz
              ck_gxxxzsss12 = oo2eta*( gxxxzsss1 - rhooeta*gxxxzsss2 )
              gxxxzspxs1 = gxxxzsss1*QCx + gxxxzsss2*WQx + ak_fxxzsss2*3.0d0
              ck_gxxxzspxs12 = oo2eta*( gxxxzspxs1 - rhooeta*gxxxzspxs2 )
              gxxxzsdxxs1 = gxxxzspxs1*QCx + gxxxzspxs2*WQx + ck_gxxxzsss12 + ak_fxxzspxs2*3.0d0
              gxxxzsfxxxs1 = gxxxzsdxxs1*QCx + gxxxzsdxxs2*WQx + ck_gxxxzspxs12*2.0d0 + ak_fxxzsdxxs2*3.0d0
              gxxxysss1 = fxxxsss1*PAy + fxxxsss2*WPy
              ck_gxxxysss12 = oo2eta*( gxxxysss1 - rhooeta*gxxxysss2 )
              gxxxyspxs1 = gxxxysss1*QCx + gxxxysss2*WQx + ak_fxxysss2*3.0d0
              ck_gxxxyspxs12 = oo2eta*( gxxxyspxs1 - rhooeta*gxxxyspxs2 )
              gxxxysdxxs1 = gxxxyspxs1*QCx + gxxxyspxs2*WQx + ck_gxxxysss12 + ak_fxxyspxs2*3.0d0
              gxxxysfxxxs1 = gxxxysdxxs1*QCx + gxxxysdxxs2*WQx + ck_gxxxyspxs12*2.0d0 + ak_fxxysdxxs2*3.0d0
              gxxxxsss1 = fxxxsss1*PAx + fxxxsss2*WPx + ai_dxxsss12*3.0d0
              ck_gxxxxsss12 = oo2eta*( gxxxxsss1 - rhooeta*gxxxxsss2 )
              gxxxxspxs1 = gxxxxsss1*QCx + gxxxxsss2*WQx + ak_fxxxsss2*4.0d0
              ck_gxxxxspxs12 = oo2eta*( gxxxxspxs1 - rhooeta*gxxxxspxs2 )
              gxxxxsdxxs1 = gxxxxspxs1*QCx + gxxxxspxs2*WQx + ck_gxxxxsss12 + ak_fxxxspxs2*4.0d0
              gxxxxsfxxxs1 = gxxxxsdxxs1*QCx + gxxxxsdxxs2*WQx + ck_gxxxxspxs12*2.0d0 + ak_fxxxsdxxs2*4.0d0
              ck_fxxxsss12 = oo2eta*( fxxxsss1 - rhooeta*fxxxsss2 )
              fxxxspxs1 = fxxxsss1*QCx + fxxxsss2*WQx + ak_dxxsss2*3.0d0
              ak_fxxxspxs1 = rhoo2ze*fxxxspxs1
              ck_fxxxspxs12 = oo2eta*( fxxxspxs1 - rhooeta*fxxxspxs2 )
              fxxxsdxxs1 = fxxxspxs1*QCx + fxxxspxs2*WQx + ck_fxxxsss12 + ak_dxxspxs2*3.0d0
              ak_fxxxsdxxs1 = rhoo2ze*fxxxsdxxs1
              fxxxsfxxxs1 = fxxxsdxxs1*QCx + fxxxsdxxs2*WQx + ck_fxxxspxs12*2.0d0 + ak_dxxsdxxs2*3.0d0
              ak_fxxxsfxxxs1 = rhoo2ze*fxxxsfxxxs1
              ck_dxxsss12 = oo2eta*( dxxsss1 - rhooeta*dxxsss2 )
              dxxspzs1 = dxxsss1*QCz + dxxsss2*WQz
              ak_dxxspzs1 = rhoo2ze*dxxspzs1
              dxxspys1 = dxxsss1*QCy + dxxsss2*WQy
              ak_dxxspys1 = rhoo2ze*dxxspys1
              dxxspxs1 = dxxsss1*QCx + dxxsss2*WQx + ak_pxsss2*2.0d0
              ak_dxxspxs1 = rhoo2ze*dxxspxs1
              ck_dxxspxs12 = oo2eta*( dxxspxs1 - rhooeta*dxxspxs2 )
              dxxsdxys1 = dxxspxs1*QCy + dxxspxs2*WQy
              ak_dxxsdxys1 = rhoo2ze*dxxsdxys1
              dxxsdxxs1 = dxxspxs1*QCx + dxxspxs2*WQx + ck_dxxsss12 + ak_pxspxs2*2.0d0
              ak_dxxsdxxs1 = rhoo2ze*dxxsdxxs1
              dxxsfxxzs1 = dxxsdxxs1*QCz + dxxsdxxs2*WQz
              ak_dxxsfxxzs1 = rhoo2ze*dxxsfxxzs1
              dxxsfxxys1 = dxxsdxxs1*QCy + dxxsdxxs2*WQy
              ak_dxxsfxxys1 = rhoo2ze*dxxsfxxys1
              dxxsfxxxs1 = dxxsdxxs1*QCx + dxxsdxxs2*WQx + ck_dxxspxs12*2.0d0 + ak_pxsdxxs2*2.0d0
              ak_dxxsfxxxs1 = rhoo2ze*dxxsfxxxs1
              dxzsss0 = pxsss0*PAz + pxsss1*WPz
              ck_dxzsss01 = oo2eta*( dxzsss0 - rhooeta*dxzsss1 )
              dxzspzs0 = dxzsss0*QCz + dxzsss1*WQz + ak_pxsss1
              ck_dxzspzs01 = oo2eta*( dxzspzs0 - rhooeta*dxzspzs1 )
              dxzspys0 = dxzsss0*QCy + dxzsss1*WQy
              dxzsdyys0 = dxzspys0*QCy + dxzspys1*WQy + ck_dxzsss01
              dxzsfyyxs0 = dxzsdyys0*QCx + dxzsdyys1*WQx + ak_pzsdyys1
              dxzspxs0 = dxzsss0*QCx + dxzsss1*WQx + ak_pzsss1
              ck_dxzspxs01 = oo2eta*( dxzspxs0 - rhooeta*dxzspxs1 )
              dxzsdxys0 = dxzspxs0*QCy + dxzspxs1*WQy
              dxzsdxxs0 = dxzspxs0*QCx + dxzspxs1*WQx + ck_dxzsss01 + ak_pzspxs1
              ck_dxzsdxxs01 = oo2eta*( dxzsdxxs0 - rhooeta*dxzsdxxs1 )
              dxzsfxxys0 = dxzsdxxs0*QCy + dxzsdxxs1*WQy
              dxzsfxxxs0 = dxzsdxxs0*QCx + dxzsdxxs1*WQx + ck_dxzspxs01*2.0d0 + ak_pzsdxxs1
              dxysss0 = pxsss0*PAy + pxsss1*WPy
              fxyzsss0 = dxysss0*PAz + dxysss1*WPz
              ck_fxyzsss01 = oo2eta*( fxyzsss0 - rhooeta*fxyzsss1 )
              fxyzspzs0 = fxyzsss0*QCz + fxyzsss1*WQz + ak_dxysss1
              fxyzsdzzs0 = fxyzspzs0*QCz + fxyzspzs1*WQz + ck_fxyzsss01 + ak_dxyspzs1
              fxyzsfzzys0 = fxyzsdzzs0*QCy + fxyzsdzzs1*WQy + ak_dxzsdzzs1
              fxyzsfzzys = fxyzsfzzys + fxyzsfzzys0
              fxyzsfzzxs0 = fxyzsdzzs0*QCx + fxyzsdzzs1*WQx + ak_dyzsdzzs1
              fxyzsfzzxs = fxyzsfzzxs + fxyzsfzzxs0
              fxyzsgzzxys0 = fxyzsfzzxs0*QCy + fxyzsfzzxs1*WQy + ak_dxzsfzzxs1
              fxyzsgzzxys = fxyzsgzzxys + fxyzsgzzxys0
              fxyzspys0 = fxyzsss0*QCy + fxyzsss1*WQy + ak_dxzsss1
              fxyzsdyzs0 = fxyzspys0*QCz + fxyzspys1*WQz + ak_dxyspys1
              fxyzsdyzs = fxyzsdyzs + fxyzsdyzs0
              fxyzsdyys0 = fxyzspys0*QCy + fxyzspys1*WQy + ck_fxyzsss01 + ak_dxzspys1
              fxyzsfyyzs0 = fxyzsdyys0*QCz + fxyzsdyys1*WQz + ak_dxysdyys1
              fxyzsfyyzs = fxyzsfyyzs + fxyzsfyyzs0
              fxyzsfyyxs0 = fxyzsdyys0*QCx + fxyzsdyys1*WQx + ak_dyzsdyys1
              fxyzsfyyxs = fxyzsfyyxs + fxyzsfyyxs0
              fxyzsgyyxzs0 = fxyzsfyyxs0*QCz + fxyzsfyyxs1*WQz + ak_dxysfyyxs1
              fxyzsgyyxzs = fxyzsgyyxzs + fxyzsgyyxzs0
              fxyzspxs0 = fxyzsss0*QCx + fxyzsss1*WQx + ak_dyzsss1
              fxyzsdxzs0 = fxyzspxs0*QCz + fxyzspxs1*WQz + ak_dxyspxs1
              fxyzsdxzs = fxyzsdxzs + fxyzsdxzs0
              fxyzsdxys0 = fxyzspxs0*QCy + fxyzspxs1*WQy + ak_dxzspxs1
              fxyzsdxys = fxyzsdxys + fxyzsdxys0
              fxyzsfxyzs0 = fxyzsdxys0*QCz + fxyzsdxys1*WQz + ak_dxysdxys1
              fxyzsfxyzs = fxyzsfxyzs + fxyzsfxyzs0
              fxyzsdxxs0 = fxyzspxs0*QCx + fxyzspxs1*WQx + ck_fxyzsss01 + ak_dyzspxs1
              fxyzsfxxzs0 = fxyzsdxxs0*QCz + fxyzsdxxs1*WQz + ak_dxysdxxs1
              fxyzsfxxzs = fxyzsfxxzs + fxyzsfxxzs0
              fxyzsfxxys0 = fxyzsdxxs0*QCy + fxyzsdxxs1*WQy + ak_dxzsdxxs1
              fxyzsfxxys = fxyzsfxxys + fxyzsfxxys0
              fxyzsgxxyzs0 = fxyzsfxxys0*QCz + fxyzsfxxys1*WQz + ak_dxysfxxys1
              fxyzsgxxyzs = fxyzsgxxyzs + fxyzsgxxyzs0
              ck_dxysss01 = oo2eta*( dxysss0 - rhooeta*dxysss1 )
              dxyspzs0 = dxysss0*QCz + dxysss1*WQz
              dxysdzzs0 = dxyspzs0*QCz + dxyspzs1*WQz + ck_dxysss01
              dxysfzzxs0 = dxysdzzs0*QCx + dxysdzzs1*WQx + ak_pysdzzs1
              dxyspys0 = dxysss0*QCy + dxysss1*WQy + ak_pxsss1
              ck_dxyspys01 = oo2eta*( dxyspys0 - rhooeta*dxyspys1 )
              dxyspxs0 = dxysss0*QCx + dxysss1*WQx + ak_pysss1
              ck_dxyspxs01 = oo2eta*( dxyspxs0 - rhooeta*dxyspxs1 )
              dxysdxxs0 = dxyspxs0*QCx + dxyspxs1*WQx + ck_dxysss01 + ak_pyspxs1
              ck_dxysdxxs01 = oo2eta*( dxysdxxs0 - rhooeta*dxysdxxs1 )
              dxysfxxxs0 = dxysdxxs0*QCx + dxysdxxs1*WQx + ck_dxyspxs01*2.0d0 + ak_pysdxxs1
              dxxsss0 = pxsss0*PAx + pxsss1*WPx + ai_ssss01
              ai_dxxsss01 = oo2zeta*( dxxsss0 - rhoozeta*dxxsss1 )
              fxxzsss0 = dxxsss0*PAz + dxxsss1*WPz
              gxxzzsss0 = fxxzsss0*PAz + fxxzsss1*WPz + ai_dxxsss01
              ck_gxxzzsss01 = oo2eta*( gxxzzsss0 - rhooeta*gxxzzsss1 )
              gxxzzspzs0 = gxxzzsss0*QCz + gxxzzsss1*WQz + ak_fxxzsss1*2.0d0
              gxxzzsdzzs0 = gxxzzspzs0*QCz + gxxzzspzs1*WQz + ck_gxxzzsss01 + ak_fxxzspzs1*2.0d0
              gxxzzsdzzs = gxxzzsdzzs + gxxzzsdzzs0
              gxxzzsfzzxs0 = gxxzzsdzzs0*QCx + gxxzzsdzzs1*WQx + ak_fzzxsdzzs1*2.0d0
              gxxzzsfzzxs = gxxzzsfzzxs + gxxzzsfzzxs0
              gxxzzspxs0 = gxxzzsss0*QCx + gxxzzsss1*WQx + ak_fzzxsss1*2.0d0
              gxxzzsdxzs0 = gxxzzspxs0*QCz + gxxzzspxs1*WQz + ak_fxxzspxs1*2.0d0
              gxxzzsdxzs = gxxzzsdxzs + gxxzzsdxzs0
              gxxzzsdxxs0 = gxxzzspxs0*QCx + gxxzzspxs1*WQx + ck_gxxzzsss01 + ak_fzzxspxs1*2.0d0
              gxxzzsdxxs = gxxzzsdxxs + gxxzzsdxxs0
              ck_gxxzzsdxxs01 = oo2eta*( gxxzzsdxxs0 - rhooeta*gxxzzsdxxs1 )
              gxxzzsfxxzs0 = gxxzzsdxxs0*QCz + gxxzzsdxxs1*WQz + ak_fxxzsdxxs1*2.0d0
              gxxzzsfxxzs = gxxzzsfxxzs + gxxzzsfxxzs0
              gxxzzsgxxzzs0 = gxxzzsfxxzs0*QCz + gxxzzsfxxzs1*WQz + ck_gxxzzsdxxs01 + ak_fxxzsfxxzs1*2.0d0
              gxxzzsgxxzzs = gxxzzsgxxzzs + gxxzzsgxxzzs0
              ck_fxxzsss01 = oo2eta*( fxxzsss0 - rhooeta*fxxzsss1 )
              fxxzspzs0 = fxxzsss0*QCz + fxxzsss1*WQz + ak_dxxsss1
              fxxzsdzzs0 = fxxzspzs0*QCz + fxxzspzs1*WQz + ck_fxxzsss01 + ak_dxxspzs1
              fxxzsfzzxs0 = fxxzsdzzs0*QCx + fxxzsdzzs1*WQx + ak_dxzsdzzs1*2.0d0
              fxxzsfzzxs = fxxzsfzzxs + fxxzsfzzxs0
              fxxzspxs0 = fxxzsss0*QCx + fxxzsss1*WQx + ak_dxzsss1*2.0d0
              ck_fxxzspxs01 = oo2eta*( fxxzspxs0 - rhooeta*fxxzspxs1 )
              fxxzsdxzs0 = fxxzspxs0*QCz + fxxzspxs1*WQz + ak_dxxspxs1
              fxxzsdxzs = fxxzsdxzs + fxxzsdxzs0
              fxxzsdxys0 = fxxzspxs0*QCy + fxxzspxs1*WQy
              fxxzsfxyzs0 = fxxzsdxys0*QCz + fxxzsdxys1*WQz + ak_dxxsdxys1
              fxxzsfxyzs = fxxzsfxyzs + fxxzsfxyzs0
              fxxzsdxxs0 = fxxzspxs0*QCx + fxxzspxs1*WQx + ck_fxxzsss01 + ak_dxzspxs1*2.0d0
              fxxzsdxxs = fxxzsdxxs + fxxzsdxxs0
              ck_fxxzsdxxs01 = oo2eta*( fxxzsdxxs0 - rhooeta*fxxzsdxxs1 )
              fxxzsfxxzs0 = fxxzsdxxs0*QCz + fxxzsdxxs1*WQz + ak_dxxsdxxs1
              fxxzsfxxzs = fxxzsfxxzs + fxxzsfxxzs0
              fxxzsgxxzzs0 = fxxzsfxxzs0*QCz + fxxzsfxxzs1*WQz + ck_fxxzsdxxs01 + ak_dxxsfxxzs1
              fxxzsgxxzzs = fxxzsgxxzzs + fxxzsgxxzzs0
              fxxzsfxxys0 = fxxzsdxxs0*QCy + fxxzsdxxs1*WQy
              fxxzsfxxys = fxxzsfxxys + fxxzsfxxys0
              fxxzsgxxyzs0 = fxxzsfxxys0*QCz + fxxzsfxxys1*WQz + ak_dxxsfxxys1
              fxxzsgxxyzs = fxxzsgxxyzs + fxxzsgxxyzs0
              fxxzsfxxxs0 = fxxzsdxxs0*QCx + fxxzsdxxs1*WQx + ck_fxxzspxs01*2.0d0 + ak_dxzsdxxs1*2.0d0
              fxxzsfxxxs = fxxzsfxxxs + fxxzsfxxxs0
              fxxzsgxxxzs0 = fxxzsfxxxs0*QCz + fxxzsfxxxs1*WQz + ak_dxxsfxxxs1
              fxxzsgxxxzs = fxxzsgxxxzs + fxxzsgxxxzs0
              fxxysss0 = dxxsss0*PAy + dxxsss1*WPy
              gxxyzsss0 = fxxysss0*PAz + fxxysss1*WPz
              ck_gxxyzsss01 = oo2eta*( gxxyzsss0 - rhooeta*gxxyzsss1 )
              gxxyzspys0 = gxxyzsss0*QCy + gxxyzsss1*WQy + ak_fxxzsss1
              gxxyzsdyzs0 = gxxyzspys0*QCz + gxxyzspys1*WQz + ak_fxxyspys1
              gxxyzsdyzs = gxxyzsdyzs + gxxyzsdyzs0
              gxxyzspxs0 = gxxyzsss0*QCx + gxxyzsss1*WQx + ak_fxyzsss1*2.0d0
              gxxyzsdxzs0 = gxxyzspxs0*QCz + gxxyzspxs1*WQz + ak_fxxyspxs1
              gxxyzsdxzs = gxxyzsdxzs + gxxyzsdxzs0
              gxxyzsdxys0 = gxxyzspxs0*QCy + gxxyzspxs1*WQy + ak_fxxzspxs1
              gxxyzsdxys = gxxyzsdxys + gxxyzsdxys0
              gxxyzsfxyzs0 = gxxyzsdxys0*QCz + gxxyzsdxys1*WQz + ak_fxxysdxys1
              gxxyzsfxyzs = gxxyzsfxyzs + gxxyzsfxyzs0
              gxxyzsdxxs0 = gxxyzspxs0*QCx + gxxyzspxs1*WQx + ck_gxxyzsss01 + ak_fxyzspxs1*2.0d0
              gxxyzsdxxs = gxxyzsdxxs + gxxyzsdxxs0
              gxxyzsfxxzs0 = gxxyzsdxxs0*QCz + gxxyzsdxxs1*WQz + ak_fxxysdxxs1
              gxxyzsfxxzs = gxxyzsfxxzs + gxxyzsfxxzs0
              gxxyzsfxxys0 = gxxyzsdxxs0*QCy + gxxyzsdxxs1*WQy + ak_fxxzsdxxs1
              gxxyzsfxxys = gxxyzsfxxys + gxxyzsfxxys0
              gxxyzsgxxyzs0 = gxxyzsfxxys0*QCz + gxxyzsfxxys1*WQz + ak_fxxysfxxys1
              gxxyzsgxxyzs = gxxyzsgxxyzs + gxxyzsgxxyzs0
              gxxyysss0 = fxxysss0*PAy + fxxysss1*WPy + ai_dxxsss01
              ck_gxxyysss01 = oo2eta*( gxxyysss0 - rhooeta*gxxyysss1 )
              gxxyyspys0 = gxxyysss0*QCy + gxxyysss1*WQy + ak_fxxysss1*2.0d0
              gxxyysdyys0 = gxxyyspys0*QCy + gxxyyspys1*WQy + ck_gxxyysss01 + ak_fxxyspys1*2.0d0
              gxxyysdyys = gxxyysdyys + gxxyysdyys0
              gxxyysfyyxs0 = gxxyysdyys0*QCx + gxxyysdyys1*WQx + ak_fyyxsdyys1*2.0d0
              gxxyysfyyxs = gxxyysfyyxs + gxxyysfyyxs0
              gxxyyspxs0 = gxxyysss0*QCx + gxxyysss1*WQx + ak_fyyxsss1*2.0d0
              gxxyysdxys0 = gxxyyspxs0*QCy + gxxyyspxs1*WQy + ak_fxxyspxs1*2.0d0
              gxxyysdxys = gxxyysdxys + gxxyysdxys0
              gxxyysdxxs0 = gxxyyspxs0*QCx + gxxyyspxs1*WQx + ck_gxxyysss01 + ak_fyyxspxs1*2.0d0
              gxxyysdxxs = gxxyysdxxs + gxxyysdxxs0
              ck_gxxyysdxxs01 = oo2eta*( gxxyysdxxs0 - rhooeta*gxxyysdxxs1 )
              gxxyysfxxys0 = gxxyysdxxs0*QCy + gxxyysdxxs1*WQy + ak_fxxysdxxs1*2.0d0
              gxxyysfxxys = gxxyysfxxys + gxxyysfxxys0
              gxxyysgxxyys0 = gxxyysfxxys0*QCy + gxxyysfxxys1*WQy + ck_gxxyysdxxs01 + ak_fxxysfxxys1*2.0d0
              gxxyysgxxyys = gxxyysgxxyys + gxxyysgxxyys0
              ck_fxxysss01 = oo2eta*( fxxysss0 - rhooeta*fxxysss1 )
              fxxyspys0 = fxxysss0*QCy + fxxysss1*WQy + ak_dxxsss1
              fxxysdyys0 = fxxyspys0*QCy + fxxyspys1*WQy + ck_fxxysss01 + ak_dxxspys1
              fxxysfyyxs0 = fxxysdyys0*QCx + fxxysdyys1*WQx + ak_dxysdyys1*2.0d0
              fxxysfyyxs = fxxysfyyxs + fxxysfyyxs0
              fxxyspxs0 = fxxysss0*QCx + fxxysss1*WQx + ak_dxysss1*2.0d0
              ck_fxxyspxs01 = oo2eta*( fxxyspxs0 - rhooeta*fxxyspxs1 )
              fxxysdxys0 = fxxyspxs0*QCy + fxxyspxs1*WQy + ak_dxxspxs1
              fxxysdxys = fxxysdxys + fxxysdxys0
              fxxysfxyzs0 = fxxysdxys0*QCz + fxxysdxys1*WQz
              fxxysfxyzs = fxxysfxyzs + fxxysfxyzs0
              fxxysdxxs0 = fxxyspxs0*QCx + fxxyspxs1*WQx + ck_fxxysss01 + ak_dxyspxs1*2.0d0
              fxxysdxxs = fxxysdxxs + fxxysdxxs0
              ck_fxxysdxxs01 = oo2eta*( fxxysdxxs0 - rhooeta*fxxysdxxs1 )
              fxxysfxxzs0 = fxxysdxxs0*QCz + fxxysdxxs1*WQz
              fxxysfxxzs = fxxysfxxzs + fxxysfxxzs0
              fxxysfxxys0 = fxxysdxxs0*QCy + fxxysdxxs1*WQy + ak_dxxsdxxs1
              fxxysfxxys = fxxysfxxys + fxxysfxxys0
              fxxysgxxyzs0 = fxxysfxxys0*QCz + fxxysfxxys1*WQz
              fxxysgxxyzs = fxxysgxxyzs + fxxysgxxyzs0
              fxxysgxxyys0 = fxxysfxxys0*QCy + fxxysfxxys1*WQy + ck_fxxysdxxs01 + ak_dxxsfxxys1
              fxxysgxxyys = fxxysgxxyys + fxxysgxxyys0
              fxxysfxxxs0 = fxxysdxxs0*QCx + fxxysdxxs1*WQx + ck_fxxyspxs01*2.0d0 + ak_dxysdxxs1*2.0d0
              fxxysfxxxs = fxxysfxxxs + fxxysfxxxs0
              fxxysgxxxys0 = fxxysfxxxs0*QCy + fxxysfxxxs1*WQy + ak_dxxsfxxxs1
              fxxysgxxxys = fxxysgxxxys + fxxysgxxxys0
              fxxxsss0 = dxxsss0*PAx + dxxsss1*WPx + ai_pxsss01*2.0d0
              gxxxzsss0 = fxxxsss0*PAz + fxxxsss1*WPz
              ck_gxxxzsss01 = oo2eta*( gxxxzsss0 - rhooeta*gxxxzsss1 )
              gxxxzspxs0 = gxxxzsss0*QCx + gxxxzsss1*WQx + ak_fxxzsss1*3.0d0
              ck_gxxxzspxs01 = oo2eta*( gxxxzspxs0 - rhooeta*gxxxzspxs1 )
              gxxxzsdxzs0 = gxxxzspxs0*QCz + gxxxzspxs1*WQz + ak_fxxxspxs1
              gxxxzsdxzs = gxxxzsdxzs + gxxxzsdxzs0
              gxxxzsdxxs0 = gxxxzspxs0*QCx + gxxxzspxs1*WQx + ck_gxxxzsss01 + ak_fxxzspxs1*3.0d0
              gxxxzsdxxs = gxxxzsdxxs + gxxxzsdxxs0
              gxxxzsfxxzs0 = gxxxzsdxxs0*QCz + gxxxzsdxxs1*WQz + ak_fxxxsdxxs1
              gxxxzsfxxzs = gxxxzsfxxzs + gxxxzsfxxzs0
              gxxxzsfxxxs0 = gxxxzsdxxs0*QCx + gxxxzsdxxs1*WQx + ck_gxxxzspxs01*2.0d0 + ak_fxxzsdxxs1*3.0d0
              gxxxzsfxxxs = gxxxzsfxxxs + gxxxzsfxxxs0
              gxxxzsgxxxzs0 = gxxxzsfxxxs0*QCz + gxxxzsfxxxs1*WQz + ak_fxxxsfxxxs1
              gxxxzsgxxxzs = gxxxzsgxxxzs + gxxxzsgxxxzs0
              gxxxysss0 = fxxxsss0*PAy + fxxxsss1*WPy
              ck_gxxxysss01 = oo2eta*( gxxxysss0 - rhooeta*gxxxysss1 )
              gxxxyspxs0 = gxxxysss0*QCx + gxxxysss1*WQx + ak_fxxysss1*3.0d0
              ck_gxxxyspxs01 = oo2eta*( gxxxyspxs0 - rhooeta*gxxxyspxs1 )
              gxxxysdxys0 = gxxxyspxs0*QCy + gxxxyspxs1*WQy + ak_fxxxspxs1
              gxxxysdxys = gxxxysdxys + gxxxysdxys0
              gxxxysdxxs0 = gxxxyspxs0*QCx + gxxxyspxs1*WQx + ck_gxxxysss01 + ak_fxxyspxs1*3.0d0
              gxxxysdxxs = gxxxysdxxs + gxxxysdxxs0
              gxxxysfxxys0 = gxxxysdxxs0*QCy + gxxxysdxxs1*WQy + ak_fxxxsdxxs1
              gxxxysfxxys = gxxxysfxxys + gxxxysfxxys0
              gxxxysfxxxs0 = gxxxysdxxs0*QCx + gxxxysdxxs1*WQx + ck_gxxxyspxs01*2.0d0 + ak_fxxysdxxs1*3.0d0
              gxxxysfxxxs = gxxxysfxxxs + gxxxysfxxxs0
              gxxxysgxxxys0 = gxxxysfxxxs0*QCy + gxxxysfxxxs1*WQy + ak_fxxxsfxxxs1
              gxxxysgxxxys = gxxxysgxxxys + gxxxysgxxxys0
              gxxxxsss0 = fxxxsss0*PAx + fxxxsss1*WPx + ai_dxxsss01*3.0d0
              ck_gxxxxsss01 = oo2eta*( gxxxxsss0 - rhooeta*gxxxxsss1 )
              gxxxxspxs0 = gxxxxsss0*QCx + gxxxxsss1*WQx + ak_fxxxsss1*4.0d0
              ck_gxxxxspxs01 = oo2eta*( gxxxxspxs0 - rhooeta*gxxxxspxs1 )
              gxxxxsdxxs0 = gxxxxspxs0*QCx + gxxxxspxs1*WQx + ck_gxxxxsss01 + ak_fxxxspxs1*4.0d0
              gxxxxsdxxs = gxxxxsdxxs + gxxxxsdxxs0
              ck_gxxxxsdxxs01 = oo2eta*( gxxxxsdxxs0 - rhooeta*gxxxxsdxxs1 )
              gxxxxsfxxxs0 = gxxxxsdxxs0*QCx + gxxxxsdxxs1*WQx + ck_gxxxxspxs01*2.0d0 + ak_fxxxsdxxs1*4.0d0
              gxxxxsfxxxs = gxxxxsfxxxs + gxxxxsfxxxs0
              gxxxxsgxxxxs0 = gxxxxsfxxxs0*QCx + gxxxxsfxxxs1*WQx + ck_gxxxxsdxxs01*3.0d0 + ak_fxxxsfxxxs1*4.0d0
              gxxxxsgxxxxs = gxxxxsgxxxxs + gxxxxsgxxxxs0
              ck_fxxxsss01 = oo2eta*( fxxxsss0 - rhooeta*fxxxsss1 )
              fxxxspxs0 = fxxxsss0*QCx + fxxxsss1*WQx + ak_dxxsss1*3.0d0
              ck_fxxxspxs01 = oo2eta*( fxxxspxs0 - rhooeta*fxxxspxs1 )
              fxxxsdxxs0 = fxxxspxs0*QCx + fxxxspxs1*WQx + ck_fxxxsss01 + ak_dxxspxs1*3.0d0
              fxxxsdxxs = fxxxsdxxs + fxxxsdxxs0
              ck_fxxxsdxxs01 = oo2eta*( fxxxsdxxs0 - rhooeta*fxxxsdxxs1 )
              fxxxsfxxzs0 = fxxxsdxxs0*QCz + fxxxsdxxs1*WQz
              fxxxsfxxzs = fxxxsfxxzs + fxxxsfxxzs0
              fxxxsfxxys0 = fxxxsdxxs0*QCy + fxxxsdxxs1*WQy
              fxxxsfxxys = fxxxsfxxys + fxxxsfxxys0
              fxxxsfxxxs0 = fxxxsdxxs0*QCx + fxxxsdxxs1*WQx + ck_fxxxspxs01*2.0d0 + ak_dxxsdxxs1*3.0d0
              fxxxsfxxxs = fxxxsfxxxs + fxxxsfxxxs0
              fxxxsgxxxzs0 = fxxxsfxxxs0*QCz + fxxxsfxxxs1*WQz
              fxxxsgxxxzs = fxxxsgxxxzs + fxxxsgxxxzs0
              fxxxsgxxxys0 = fxxxsfxxxs0*QCy + fxxxsfxxxs1*WQy
              fxxxsgxxxys = fxxxsgxxxys + fxxxsgxxxys0
              fxxxsgxxxxs0 = fxxxsfxxxs0*QCx + fxxxsfxxxs1*WQx + ck_fxxxsdxxs01*3.0d0 + ak_dxxsfxxxs1*3.0d0
              fxxxsgxxxxs = fxxxsgxxxxs + fxxxsgxxxxs0
              ck_dxxsss01 = oo2eta*( dxxsss0 - rhooeta*dxxsss1 )
              dxxspxs0 = dxxsss0*QCx + dxxsss1*WQx + ak_pxsss1*2.0d0
              ck_dxxspxs01 = oo2eta*( dxxspxs0 - rhooeta*dxxspxs1 )
              ck_pxsss12 = oo2eta*( pxsss1 - rhooeta*pxsss2 )
              pxspzs1 = pxsss1*QCz + pxsss2*WQz
              ak_pxspzs1 = rhoo2ze*pxspzs1
              dxzsdzzs0 = dxzspzs0*QCz + dxzspzs1*WQz + ck_dxzsss01 + ak_pxspzs1
              pxsdzzs1 = pxspzs1*QCz + pxspzs2*WQz + ck_pxsss12
              ak_pxsdzzs1 = rhoo2ze*pxsdzzs1
              dxzsfzzzs0 = dxzsdzzs0*QCz + dxzsdzzs1*WQz + ck_dxzspzs01*2.0d0 + ak_pxsdzzs1
              pxspys1 = pxsss1*QCy + pxsss2*WQy
              ak_pxspys1 = rhoo2ze*pxspys1
              dxysdyys0 = dxyspys0*QCy + dxyspys1*WQy + ck_dxysss01 + ak_pxspys1
              pxsdyys1 = pxspys1*QCy + pxspys2*WQy + ck_pxsss12
              ak_pxsdyys1 = rhoo2ze*pxsdyys1
              dxysfyyys0 = dxysdyys0*QCy + dxysdyys1*WQy + ck_dxyspys01*2.0d0 + ak_pxsdyys1
              pxspxs1 = pxsss1*QCx + pxsss2*WQx + ak_ssss2
              ak_pxspxs1 = rhoo2ze*pxspxs1
              dxzsdxzs0 = dxzspxs0*QCz + dxzspxs1*WQz + ak_pxspxs1
              dxzsdxzs = dxzsdxzs + dxzsdxzs0
              dxysdxys0 = dxyspxs0*QCy + dxyspxs1*WQy + ak_pxspxs1
              dxysdxys = dxysdxys + dxysdxys0
              dxysfxyzs0 = dxysdxys0*QCz + dxysdxys1*WQz
              dxysfxyzs = dxysfxyzs + dxysfxyzs0
              dxxsdxxs0 = dxxspxs0*QCx + dxxspxs1*WQx + ck_dxxsss01 + ak_pxspxs1*2.0d0
              dxxsdxxs = dxxsdxxs + dxxsdxxs0
              ck_dxxsdxxs01 = oo2eta*( dxxsdxxs0 - rhooeta*dxxsdxxs1 )
              dxxsfxxzs0 = dxxsdxxs0*QCz + dxxsdxxs1*WQz
              dxxsfxxzs = dxxsfxxzs + dxxsfxxzs0
              dxxsgxxzzs0 = dxxsfxxzs0*QCz + dxxsfxxzs1*WQz + ck_dxxsdxxs01
              dxxsgxxzzs = dxxsgxxzzs + dxxsgxxzzs0
              dxxsfxxys0 = dxxsdxxs0*QCy + dxxsdxxs1*WQy
              dxxsfxxys = dxxsfxxys + dxxsfxxys0
              dxxsgxxyzs0 = dxxsfxxys0*QCz + dxxsfxxys1*WQz
              dxxsgxxyzs = dxxsgxxyzs + dxxsgxxyzs0
              dxxsgxxyys0 = dxxsfxxys0*QCy + dxxsfxxys1*WQy + ck_dxxsdxxs01
              dxxsgxxyys = dxxsgxxyys + dxxsgxxyys0
              ck_pxspxs12 = oo2eta*( pxspxs1 - rhooeta*pxspxs2 )
              pxsdxys1 = pxspxs1*QCy + pxspxs2*WQy
              ak_pxsdxys1 = rhoo2ze*pxsdxys1
              dxzsfxyzs0 = dxzsdxys0*QCz + dxzsdxys1*WQz + ak_pxsdxys1
              dxzsfxyzs = dxzsfxyzs + dxzsfxyzs0
              ck_ssss23 = oo2eta*( ssss2 - rhooeta*ssss3 )
              sspzs2 = ssss2*QCz + ssss3*WQz
              ak_sspzs2 = rhoo2ze*sspzs2
              pzsdzzs1 = pzspzs1*QCz + pzspzs2*WQz + ck_pzsss12 + ak_sspzs2
              ak_pzsdzzs1 = rhoo2ze*pzsdzzs1
              dyzsfzzys0 = dyzsdzzs0*QCy + dyzsdzzs1*WQy + ak_pzsdzzs1
              dyzsfzzys = dyzsfzzys + dyzsfzzys0
              dxzsfzzxs0 = dxzsdzzs0*QCx + dxzsdzzs1*WQx + ak_pzsdzzs1
              dxzsfzzxs = dxzsfzzxs + dxzsfzzxs0
              dxzsgzzxys0 = dxzsfzzxs0*QCy + dxzsfzzxs1*WQy
              dxzsgzzxys = dxzsgzzxys + dxzsgzzxys0
              dzzsfzzzs0 = dzzsdzzs0*QCz + dzzsdzzs1*WQz + ck_dzzspzs01*2.0d0 + ak_pzsdzzs1*2.0d0
              dzzsfzzzs = dzzsfzzzs + dzzsfzzzs0
              dzzsgzzzys0 = dzzsfzzzs0*QCy + dzzsfzzzs1*WQy
              dzzsgzzzys = dzzsgzzzys + dzzsgzzzys0
              dzzsgzzzxs0 = dzzsfzzzs0*QCx + dzzsfzzzs1*WQx
              dzzsgzzzxs = dzzsgzzzxs + dzzsgzzzxs0
              pzsfzzxs1 = pzsdzzs1*QCx + pzsdzzs2*WQx
              ak_pzsfzzxs1 = rhoo2ze*pzsfzzxs1
              dyzsgzzxys0 = dyzsfzzxs0*QCy + dyzsfzzxs1*WQy + ak_pzsfzzxs1
              dyzsgzzxys = dyzsgzzxys + dyzsgzzxys0
              ssdzzs2 = sspzs2*QCz + sspzs3*WQz + ck_ssss23
              ak_ssdzzs2 = rhoo2ze*ssdzzs2
              pzsfzzzs1 = pzsdzzs1*QCz + pzsdzzs2*WQz + ck_pzspzs12*2.0d0 + ak_ssdzzs2
              ak_pzsfzzzs1 = rhoo2ze*pzsfzzzs1
              dyzsgzzzys0 = dyzsfzzzs0*QCy + dyzsfzzzs1*WQy + ak_pzsfzzzs1
              dyzsgzzzys = dyzsgzzzys + dyzsgzzzys0
              dxzsgzzzxs0 = dxzsfzzzs0*QCx + dxzsfzzzs1*WQx + ak_pzsfzzzs1
              dxzsgzzzxs = dxzsgzzzxs + dxzsgzzzxs0
              dzzsgzzzzs0 = dzzsfzzzs0*QCz + dzzsfzzzs1*WQz + ck_dzzsdzzs01*3.0d0 + ak_pzsfzzzs1*2.0d0
              dzzsgzzzzs = dzzsgzzzzs + dzzsgzzzzs0
              pxsfzzxs1 = pxsdzzs1*QCx + pxsdzzs2*WQx + ak_ssdzzs2
              ak_pxsfzzxs1 = rhoo2ze*pxsfzzxs1
              dxysgzzxys0 = dxysfzzxs0*QCy + dxysfzzxs1*WQy + ak_pxsfzzxs1
              dxysgzzxys = dxysgzzxys + dxysgzzxys0
              sspys2 = ssss2*QCy + ssss3*WQy
              ak_sspys2 = rhoo2ze*sspys2
              pysdyys1 = pyspys1*QCy + pyspys2*WQy + ck_pysss12 + ak_sspys2
              ak_pysdyys1 = rhoo2ze*pysdyys1
              dyzsfyyzs0 = dyzsdyys0*QCz + dyzsdyys1*WQz + ak_pysdyys1
              dyzsfyyzs = dyzsfyyzs + dyzsfyyzs0
              dxysfyyxs0 = dxysdyys0*QCx + dxysdyys1*WQx + ak_pysdyys1
              dxysfyyxs = dxysfyyxs + dxysfyyxs0
              dxysgyyxzs0 = dxysfyyxs0*QCz + dxysfyyxs1*WQz
              dxysgyyxzs = dxysgyyxzs + dxysgyyxzs0
              dyysfyyys0 = dyysdyys0*QCy + dyysdyys1*WQy + ck_dyyspys01*2.0d0 + ak_pysdyys1*2.0d0
              dyysfyyys = dyysfyyys + dyysfyyys0
              dyysgyyyzs0 = dyysfyyys0*QCz + dyysfyyys1*WQz
              dyysgyyyzs = dyysgyyyzs + dyysgyyyzs0
              dyysgyyyxs0 = dyysfyyys0*QCx + dyysfyyys1*WQx
              dyysgyyyxs = dyysgyyyxs + dyysgyyyxs0
              pysfyyzs1 = pysdyys1*QCz + pysdyys2*WQz
              ak_pysfyyzs1 = rhoo2ze*pysfyyzs1
              dyzsgyyzzs0 = dyzsfyyzs0*QCz + dyzsfyyzs1*WQz + ck_dyzsdyys01 + ak_pysfyyzs1
              dyzsgyyzzs = dyzsgyyzzs + dyzsgyyzzs0
              pysfyyxs1 = pysdyys1*QCx + pysdyys2*WQx
              ak_pysfyyxs1 = rhoo2ze*pysfyyxs1
              dyzsgyyxzs0 = dyzsfyyxs0*QCz + dyzsfyyxs1*WQz + ak_pysfyyxs1
              dyzsgyyxzs = dyzsgyyxzs + dyzsgyyxzs0
              ssdyys2 = sspys2*QCy + sspys3*WQy + ck_ssss23
              ak_ssdyys2 = rhoo2ze*ssdyys2
              pzsfyyzs1 = pzsdyys1*QCz + pzsdyys2*WQz + ak_ssdyys2
              ak_pzsfyyzs1 = rhoo2ze*pzsfyyzs1
              dzzsgyyzzs0 = dzzsfyyzs0*QCz + dzzsfyyzs1*WQz + ck_dzzsdyys01 + ak_pzsfyyzs1*2.0d0
              dzzsgyyzzs = dzzsgyyzzs + dzzsgyyzzs0
              pysfyyys1 = pysdyys1*QCy + pysdyys2*WQy + ck_pyspys12*2.0d0 + ak_ssdyys2
              ak_pysfyyys1 = rhoo2ze*pysfyyys1
              dyzsgyyyzs0 = dyzsfyyys0*QCz + dyzsfyyys1*WQz + ak_pysfyyys1
              dyzsgyyyzs = dyzsgyyyzs + dyzsgyyyzs0
              dxysgyyyxs0 = dxysfyyys0*QCx + dxysfyyys1*WQx + ak_pysfyyys1
              dxysgyyyxs = dxysgyyyxs + dxysgyyyxs0
              dyysgyyyys0 = dyysfyyys0*QCy + dyysfyyys1*WQy + ck_dyysdyys01*3.0d0 + ak_pysfyyys1*2.0d0
              dyysgyyyys = dyysgyyyys + dyysgyyyys0
              pxsfyyxs1 = pxsdyys1*QCx + pxsdyys2*WQx + ak_ssdyys2
              ak_pxsfyyxs1 = rhoo2ze*pxsfyyxs1
              dxzsgyyxzs0 = dxzsfyyxs0*QCz + dxzsfyyxs1*WQz + ak_pxsfyyxs1
              dxzsgyyxzs = dxzsgyyxzs + dxzsgyyxzs0
              sspxs2 = ssss2*QCx + ssss3*WQx
              ak_sspxs2 = rhoo2ze*sspxs2
              pysdxys1 = pyspxs1*QCy + pyspxs2*WQy + ak_sspxs2
              ak_pysdxys1 = rhoo2ze*pysdxys1
              dyzsfxyzs0 = dyzsdxys0*QCz + dyzsdxys1*WQz + ak_pysdxys1
              dyzsfxyzs = dyzsfxyzs + dyzsfxyzs0
              pxsdxxs1 = pxspxs1*QCx + pxspxs2*WQx + ck_pxsss12 + ak_sspxs2
              ak_pxsdxxs1 = rhoo2ze*pxsdxxs1
              dxzsfxxzs0 = dxzsdxxs0*QCz + dxzsdxxs1*WQz + ak_pxsdxxs1
              dxzsfxxzs = dxzsfxxzs + dxzsfxxzs0
              dxysfxxys0 = dxysdxxs0*QCy + dxysdxxs1*WQy + ak_pxsdxxs1
              dxysfxxys = dxysfxxys + dxysfxxys0
              dxysgxxyzs0 = dxysfxxys0*QCz + dxysfxxys1*WQz
              dxysgxxyzs = dxysgxxyzs + dxysgxxyzs0
              dxxsfxxxs0 = dxxsdxxs0*QCx + dxxsdxxs1*WQx + ck_dxxspxs01*2.0d0 + ak_pxsdxxs1*2.0d0
              dxxsfxxxs = dxxsfxxxs + dxxsfxxxs0
              dxxsgxxxzs0 = dxxsfxxxs0*QCz + dxxsfxxxs1*WQz
              dxxsgxxxzs = dxxsgxxxzs + dxxsgxxxzs0
              dxxsgxxxys0 = dxxsfxxxs0*QCy + dxxsfxxxs1*WQy
              dxxsgxxxys = dxxsgxxxys + dxxsgxxxys0
              pxsfxxzs1 = pxsdxxs1*QCz + pxsdxxs2*WQz
              ak_pxsfxxzs1 = rhoo2ze*pxsfxxzs1
              dxzsgxxzzs0 = dxzsfxxzs0*QCz + dxzsfxxzs1*WQz + ck_dxzsdxxs01 + ak_pxsfxxzs1
              dxzsgxxzzs = dxzsgxxzzs + dxzsgxxzzs0
              pxsfxxys1 = pxsdxxs1*QCy + pxsdxxs2*WQy
              ak_pxsfxxys1 = rhoo2ze*pxsfxxys1
              dxzsgxxyzs0 = dxzsfxxys0*QCz + dxzsfxxys1*WQz + ak_pxsfxxys1
              dxzsgxxyzs = dxzsgxxyzs + dxzsgxxyzs0
              dxysgxxyys0 = dxysfxxys0*QCy + dxysfxxys1*WQy + ck_dxysdxxs01 + ak_pxsfxxys1
              dxysgxxyys = dxysgxxyys + dxysgxxyys0
              ssdxxs2 = sspxs2*QCx + sspxs3*WQx + ck_ssss23
              ak_ssdxxs2 = rhoo2ze*ssdxxs2
              pzsfxxzs1 = pzsdxxs1*QCz + pzsdxxs2*WQz + ak_ssdxxs2
              ak_pzsfxxzs1 = rhoo2ze*pzsfxxzs1
              dzzsgxxzzs0 = dzzsfxxzs0*QCz + dzzsfxxzs1*WQz + ck_dzzsdxxs01 + ak_pzsfxxzs1*2.0d0
              dzzsgxxzzs = dzzsgxxzzs + dzzsgxxzzs0
              pysfxxys1 = pysdxxs1*QCy + pysdxxs2*WQy + ak_ssdxxs2
              ak_pysfxxys1 = rhoo2ze*pysfxxys1
              dyzsgxxyzs0 = dyzsfxxys0*QCz + dyzsfxxys1*WQz + ak_pysfxxys1
              dyzsgxxyzs = dyzsgxxyzs + dyzsgxxyzs0
              dyysgxxyys0 = dyysfxxys0*QCy + dyysfxxys1*WQy + ck_dyysdxxs01 + ak_pysfxxys1*2.0d0
              dyysgxxyys = dyysgxxyys + dyysgxxyys0
              pxsfxxxs1 = pxsdxxs1*QCx + pxsdxxs2*WQx + ck_pxspxs12*2.0d0 + ak_ssdxxs2
              ak_pxsfxxxs1 = rhoo2ze*pxsfxxxs1
              dxzsgxxxzs0 = dxzsfxxxs0*QCz + dxzsfxxxs1*WQz + ak_pxsfxxxs1
              dxzsgxxxzs = dxzsgxxxzs + dxzsgxxxzs0
              dxysgxxxys0 = dxysfxxxs0*QCy + dxysfxxxs1*WQy + ak_pxsfxxxs1
              dxysgxxxys = dxysgxxxys + dxysgxxxys0
              dxxsgxxxxs0 = dxxsfxxxs0*QCx + dxxsfxxxs1*WQx + ck_dxxsdxxs01*3.0d0 + ak_pxsfxxxs1*2.0d0
              dxxsgxxxxs = dxxsgxxxxs + dxxsgxxxxs0
           end do
        end do

        gzzxysfxyzpz = gzzxysgzzxys + gzzxysfxyzs*CDz
        gzzxysdyzpx = gzzxysfxyzs + gzzxysdyzs*CDx
        gzzxysdxzpy = gzzxysfxyzs + gzzxysdxzs*CDy
        gzzxysdxypz = gzzxysfxyzs + gzzxysdxys*CDz
        gzzxysdxydzz = gzzxysfxyzpz + gzzxysdxypz*CDz
        gzzxysfzzypx = gzzxysgzzxys + gzzxysfzzys*CDx
        gzzxysdyzdxz = gzzxysfzzypx + gzzxysdyzpx*CDz
        gzzxysfzzxpy = gzzxysgzzxys + gzzxysfzzxs*CDy
        gzzxysdxzdyz = gzzxysfzzxpy + gzzxysdxzpy*CDz
        gzzxysdzzpx = gzzxysfzzxs + gzzxysdzzs*CDx
        gzzxysdzzdxy = gzzxysfzzypx + gzzxysdzzpx*CDy
        gyyxzsfxyzpy = gyyxzsgyyxzs + gyyxzsfxyzs*CDy
        gyyxzsdyzpx = gyyxzsfxyzs + gyyxzsdyzs*CDx
        gyyxzsdxzpy = gyyxzsfxyzs + gyyxzsdxzs*CDy
        gyyxzsdxzdyy = gyyxzsfxyzpy + gyyxzsdxzpy*CDy
        gyyxzsfyyzpx = gyyxzsgyyxzs + gyyxzsfyyzs*CDx
        gyyxzsdyzdxy = gyyxzsfyyzpx + gyyxzsdyzpx*CDy
        gyyxzsdxypy = gyyxzsfyyxs + gyyxzsdxys*CDy
        gyyxzsdxydyz = gyyxzsfxyzpy + gyyxzsdxypy*CDz
        gyyxzsdyypx = gyyxzsfyyxs + gyyxzsdyys*CDx
        gyyxzsdyydxz = gyyxzsfyyzpx + gyyxzsdyypx*CDz
        gxxyzsfxyzpx = gxxyzsgxxyzs + gxxyzsfxyzs*CDx
        gxxyzsdyzpx = gxxyzsfxyzs + gxxyzsdyzs*CDx
        gxxyzsdyzdxx = gxxyzsfxyzpx + gxxyzsdyzpx*CDx
        gxxyzsfxxzpy = gxxyzsgxxyzs + gxxyzsfxxzs*CDy
        gxxyzsdxzpx = gxxyzsfxxzs + gxxyzsdxzs*CDx
        gxxyzsdxzdxy = gxxyzsfxyzpx + gxxyzsdxzpx*CDy
        gxxyzsdxypx = gxxyzsfxxys + gxxyzsdxys*CDx
        gxxyzsdxydxz = gxxyzsfxyzpx + gxxyzsdxypx*CDz
        gxxyzsdxxpy = gxxyzsfxxys + gxxyzsdxxs*CDy
        gxxyzsdxxdyz = gxxyzsfxxzpy + gxxyzsdxxpy*CDz
        gyyzzsfzzypy = gyyzzsgyyzzs + gyyzzsfzzys*CDy
        gyyzzsdzzpy = gyyzzsfzzys + gyyzzsdzzs*CDy
        gyyzzsdzzdyy = gyyzzsfzzypy + gyyzzsdzzpy*CDy
        gyyzzsfyyzpz = gyyzzsgyyzzs + gyyzzsfyyzs*CDz
        gyyzzsdyzpy = gyyzzsfyyzs + gyyzzsdyzs*CDy
        gyyzzsdyzdyz = gyyzzsfzzypy + gyyzzsdyzpy*CDz
        gyyzzsdyypz = gyyzzsfyyzs + gyyzzsdyys*CDz
        gyyzzsdyydzz = gyyzzsfyyzpz + gyyzzsdyypz*CDz
        gxxzzsfzzxpx = gxxzzsgxxzzs + gxxzzsfzzxs*CDx
        gxxzzsdzzpx = gxxzzsfzzxs + gxxzzsdzzs*CDx
        gxxzzsdzzdxx = gxxzzsfzzxpx + gxxzzsdzzpx*CDx
        gxxzzsfxxzpz = gxxzzsgxxzzs + gxxzzsfxxzs*CDz
        gxxzzsdxzpx = gxxzzsfxxzs + gxxzzsdxzs*CDx
        gxxzzsdxzdxz = gxxzzsfzzxpx + gxxzzsdxzpx*CDz
        gxxzzsdxxpz = gxxzzsfxxzs + gxxzzsdxxs*CDz
        gxxzzsdxxdzz = gxxzzsfxxzpz + gxxzzsdxxpz*CDz
        gxxyysfyyxpx = gxxyysgxxyys + gxxyysfyyxs*CDx
        gxxyysdyypx = gxxyysfyyxs + gxxyysdyys*CDx
        gxxyysdyydxx = gxxyysfyyxpx + gxxyysdyypx*CDx
        gxxyysfxxypy = gxxyysgxxyys + gxxyysfxxys*CDy
        gxxyysdxypx = gxxyysfxxys + gxxyysdxys*CDx
        gxxyysdxydxy = gxxyysfyyxpx + gxxyysdxypx*CDy
        gxxyysdxxpy = gxxyysfxxys + gxxyysdxxs*CDy
        gxxyysdxxdyy = gxxyysfxxypy + gxxyysdxxpy*CDy
        gzzzysfzzzpy = gzzzysgzzzys + gzzzysfzzzs*CDy
        gzzzysfzzypz = gzzzysgzzzys + gzzzysfzzys*CDz
        gzzzysdyzpz = gzzzysfzzys + gzzzysdyzs*CDz
        gzzzysdyzdzz = gzzzysfzzypz + gzzzysdyzpz*CDz
        gzzzysdzzpy = gzzzysfzzys + gzzzysdzzs*CDy
        gzzzysdzzdyz = gzzzysfzzzpy + gzzzysdzzpy*CDz
        gzzzxsfzzzpx = gzzzxsgzzzxs + gzzzxsfzzzs*CDx
        gzzzxsfzzxpz = gzzzxsgzzzxs + gzzzxsfzzxs*CDz
        gzzzxsdxzpz = gzzzxsfzzxs + gzzzxsdxzs*CDz
        gzzzxsdxzdzz = gzzzxsfzzxpz + gzzzxsdxzpz*CDz
        gzzzxsdzzpx = gzzzxsfzzxs + gzzzxsdzzs*CDx
        gzzzxsdzzdxz = gzzzxsfzzzpx + gzzzxsdzzpx*CDz
        gyyyzsfyyzpy = gyyyzsgyyyzs + gyyyzsfyyzs*CDy
        gyyyzsdyzpy = gyyyzsfyyzs + gyyyzsdyzs*CDy
        gyyyzsdyzdyy = gyyyzsfyyzpy + gyyyzsdyzpy*CDy
        gyyyzsdyypy = gyyyzsfyyys + gyyyzsdyys*CDy
        gyyyzsdyydyz = gyyyzsfyyzpy + gyyyzsdyypy*CDz
        gyyyxsfyyypx = gyyyxsgyyyxs + gyyyxsfyyys*CDx
        gyyyxsfyyxpy = gyyyxsgyyyxs + gyyyxsfyyxs*CDy
        gyyyxsdxypy = gyyyxsfyyxs + gyyyxsdxys*CDy
        gyyyxsdxydyy = gyyyxsfyyxpy + gyyyxsdxypy*CDy
        gyyyxsdyypx = gyyyxsfyyxs + gyyyxsdyys*CDx
        gyyyxsdyydxy = gyyyxsfyyypx + gyyyxsdyypx*CDy
        gxxxzsfxxzpx = gxxxzsgxxxzs + gxxxzsfxxzs*CDx
        gxxxzsdxzpx = gxxxzsfxxzs + gxxxzsdxzs*CDx
        gxxxzsdxzdxx = gxxxzsfxxzpx + gxxxzsdxzpx*CDx
        gxxxzsdxxpx = gxxxzsfxxxs + gxxxzsdxxs*CDx
        gxxxzsdxxdxz = gxxxzsfxxzpx + gxxxzsdxxpx*CDz
        gxxxysfxxypx = gxxxysgxxxys + gxxxysfxxys*CDx
        gxxxysdxypx = gxxxysfxxys + gxxxysdxys*CDx
        gxxxysdxydxx = gxxxysfxxypx + gxxxysdxypx*CDx
        gxxxysdxxpx = gxxxysfxxxs + gxxxysdxxs*CDx
        gxxxysdxxdxy = gxxxysfxxypx + gxxxysdxxpx*CDy
        gzzzzsfzzzpz = gzzzzsgzzzzs + gzzzzsfzzzs*CDz
        gzzzzsdzzpz = gzzzzsfzzzs + gzzzzsdzzs*CDz
        gzzzzsdzzdzz = gzzzzsfzzzpz + gzzzzsdzzpz*CDz
        gyyyysfyyypy = gyyyysgyyyys + gyyyysfyyys*CDy
        gyyyysdyypy = gyyyysfyyys + gyyyysdyys*CDy
        gyyyysdyydyy = gyyyysfyyypy + gyyyysdyypy*CDy
        gxxxxsfxxxpx = gxxxxsgxxxxs + gxxxxsfxxxs*CDx
        gxxxxsdxxpx = gxxxxsfxxxs + gxxxxsdxxs*CDx
        gxxxxsdxxdxx = gxxxxsfxxxpx + gxxxxsdxxpx*CDx
        fxyzsfxyzpz = fxyzsgzzxys + fxyzsfxyzs*CDz
        fxyzsfxyzpy = fxyzsgyyxzs + fxyzsfxyzs*CDy
        fxyzsfxyzpx = fxyzsgxxyzs + fxyzsfxyzs*CDx
        fxyzsdyzpx = fxyzsfxyzs + fxyzsdyzs*CDx
        fxyzsdyzdxx = fxyzsfxyzpx + fxyzsdyzpx*CDx
        fxyzpxdyzdxx = gxxyzsdyzdxx + fxyzsdyzdxx*ABx
        fxyzsdxzpy = fxyzsfxyzs + fxyzsdxzs*CDy
        fxyzsdxzdyy = fxyzsfxyzpy + fxyzsdxzpy*CDy
        fxyzpydxzdyy = gyyxzsdxzdyy + fxyzsdxzdyy*ABy
        fxyzsdxypz = fxyzsfxyzs + fxyzsdxys*CDz
        fxyzsdxydzz = fxyzsfxyzpz + fxyzsdxypz*CDz
        fxyzpzdxydzz = gzzxysdxydzz + fxyzsdxydzz*ABz
        fxyzsfzzypx = fxyzsgzzxys + fxyzsfzzys*CDx
        fxyzsdyzdxz = fxyzsfzzypx + fxyzsdyzpx*CDz
        fxyzsfzzxpy = fxyzsgzzxys + fxyzsfzzxs*CDy
        fxyzsdxzdyz = fxyzsfzzxpy + fxyzsdxzpy*CDz
        fxyzsfyyzpx = fxyzsgyyxzs + fxyzsfyyzs*CDx
        fxyzsdyzdxy = fxyzsfyyzpx + fxyzsdyzpx*CDy
        fxyzsdxypy = fxyzsfyyxs + fxyzsdxys*CDy
        fxyzsdxydyz = fxyzsfxyzpy + fxyzsdxypy*CDz
        fxyzpydxydyz = gyyxzsdxydyz + fxyzsdxydyz*ABy
        fxyzsdxzpx = fxyzsfxxzs + fxyzsdxzs*CDx
        fxyzsdxzdxy = fxyzsfxyzpx + fxyzsdxzpx*CDy
        fxyzpxdxzdxy = gxxyzsdxzdxy + fxyzsdxzdxy*ABx
        fxyzsdxypx = fxyzsfxxys + fxyzsdxys*CDx
        fxyzsdxydxz = fxyzsfxyzpx + fxyzsdxypx*CDz
        fxyzpxdxydxz = gxxyzsdxydxz + fxyzsdxydxz*ABx
        fzzysdyzpx = fzzysfxyzs + fzzysdyzs*CDx
        fzzysfzzzpy = fzzysgzzzys + fzzysfzzzs*CDy
        fzzysfzzypz = fzzysgzzzys + fzzysfzzys*CDz
        fzzysfzzypy = fzzysgyyzzs + fzzysfzzys*CDy
        fzzysfzzypx = fzzysgzzxys + fzzysfzzys*CDx
        fzzysdyzdxz = fzzysfzzypx + fzzysdyzpx*CDz
        fzzypxdyzdxz = gzzxysdyzdxz + fzzysdyzdxz*ABx
        fzzysdyzpz = fzzysfzzys + fzzysdyzs*CDz
        fzzysdyzdzz = fzzysfzzypz + fzzysdyzpz*CDz
        fzzypzdyzdzz = gzzzysdyzdzz + fzzysdyzdzz*ABz
        fzzysdzzpy = fzzysfzzys + fzzysdzzs*CDy
        fzzysdzzdyz = fzzysfzzzpy + fzzysdzzpy*CDz
        fzzysdzzdyy = fzzysfzzypy + fzzysdzzpy*CDy
        fzzypydzzdyy = gyyzzsdzzdyy + fzzysdzzdyy*ABy
        fzzysdzzpx = fzzysfzzxs + fzzysdzzs*CDx
        fzzysdzzdxy = fzzysfzzypx + fzzysdzzpx*CDy
        fzzypxdzzdxy = gzzxysdzzdxy + fzzysdzzdxy*ABx
        fzzysdyzpy = fzzysfyyzs + fzzysdyzs*CDy
        fzzysdyzdyz = fzzysfzzypy + fzzysdyzpy*CDz
        fzzypydyzdyz = gyyzzsdyzdyz + fzzysdyzdyz*ABy
        fzzxsdxzpy = fzzxsfxyzs + fzzxsdxzs*CDy
        fzzxsfzzzpx = fzzxsgzzzxs + fzzxsfzzzs*CDx
        fzzxsfzzypx = fzzxsgzzxys + fzzxsfzzys*CDx
        fzzxsfzzxpz = fzzxsgzzzxs + fzzxsfzzxs*CDz
        fzzxsfzzxpy = fzzxsgzzxys + fzzxsfzzxs*CDy
        fzzxsdxzdyz = fzzxsfzzxpy + fzzxsdxzpy*CDz
        fzzxpydxzdyz = gzzxysdxzdyz + fzzxsdxzdyz*ABy
        fzzxsfzzxpx = fzzxsgxxzzs + fzzxsfzzxs*CDx
        fzzxsdxzpz = fzzxsfzzxs + fzzxsdxzs*CDz
        fzzxsdxzdzz = fzzxsfzzxpz + fzzxsdxzpz*CDz
        fzzxpzdxzdzz = gzzzxsdxzdzz + fzzxsdxzdzz*ABz
        fzzxsdzzpx = fzzxsfzzxs + fzzxsdzzs*CDx
        fzzxsdzzdxz = fzzxsfzzzpx + fzzxsdzzpx*CDz
        fzzxsdzzdxy = fzzxsfzzypx + fzzxsdzzpx*CDy
        fzzxsdzzdxx = fzzxsfzzxpx + fzzxsdzzpx*CDx
        fzzxpxdzzdxx = gxxzzsdzzdxx + fzzxsdzzdxx*ABx
        fzzxsdxzpx = fzzxsfxxzs + fzzxsdxzs*CDx
        fzzxsdxzdxz = fzzxsfzzxpx + fzzxsdxzpx*CDz
        fzzxpxdxzdxz = gxxzzsdxzdxz + fzzxsdxzdxz*ABx
        fyyzsdyzpx = fyyzsfxyzs + fyyzsdyzs*CDx
        fyyzsfzzypy = fyyzsgyyzzs + fyyzsfzzys*CDy
        fyyzsfyyzpz = fyyzsgyyzzs + fyyzsfyyzs*CDz
        fyyzsfyyzpy = fyyzsgyyyzs + fyyzsfyyzs*CDy
        fyyzsfyyzpx = fyyzsgyyxzs + fyyzsfyyzs*CDx
        fyyzsdyzdxy = fyyzsfyyzpx + fyyzsdyzpx*CDy
        fyyzpxdyzdxy = gyyxzsdyzdxy + fyyzsdyzdxy*ABx
        fyyzsdyzpy = fyyzsfyyzs + fyyzsdyzs*CDy
        fyyzsdyzdyz = fyyzsfzzypy + fyyzsdyzpy*CDz
        fyyzsdyzdyy = fyyzsfyyzpy + fyyzsdyzpy*CDy
        fyyzpydyzdyy = gyyyzsdyzdyy + fyyzsdyzdyy*ABy
        fyyzsdyypz = fyyzsfyyzs + fyyzsdyys*CDz
        fyyzsdyydzz = fyyzsfyyzpz + fyyzsdyypz*CDz
        fyyzpzdyydzz = gyyzzsdyydzz + fyyzsdyydzz*ABz
        fyyzsdyypy = fyyzsfyyys + fyyzsdyys*CDy
        fyyzsdyydyz = fyyzsfyyzpy + fyyzsdyypy*CDz
        fyyzpydyydyz = gyyyzsdyydyz + fyyzsdyydyz*ABy
        fyyzsdyypx = fyyzsfyyxs + fyyzsdyys*CDx
        fyyzsdyydxz = fyyzsfyyzpx + fyyzsdyypx*CDz
        fyyzpxdyydxz = gyyxzsdyydxz + fyyzsdyydxz*ABx
        fyyxsfxyzpy = fyyxsgyyxzs + fyyxsfxyzs*CDy
        fyyxsfyyzpx = fyyxsgyyxzs + fyyxsfyyzs*CDx
        fyyxsfyyypx = fyyxsgyyyxs + fyyxsfyyys*CDx
        fyyxsfyyxpy = fyyxsgyyyxs + fyyxsfyyxs*CDy
        fyyxsfyyxpx = fyyxsgxxyys + fyyxsfyyxs*CDx
        fyyxsdxypy = fyyxsfyyxs + fyyxsdxys*CDy
        fyyxsdxydyz = fyyxsfxyzpy + fyyxsdxypy*CDz
        fyyxsdxydyy = fyyxsfyyxpy + fyyxsdxypy*CDy
        fyyxpydxydyy = gyyyxsdxydyy + fyyxsdxydyy*ABy
        fyyxsdyypx = fyyxsfyyxs + fyyxsdyys*CDx
        fyyxsdyydxz = fyyxsfyyzpx + fyyxsdyypx*CDz
        fyyxsdyydxy = fyyxsfyyypx + fyyxsdyypx*CDy
        fyyxsdyydxx = fyyxsfyyxpx + fyyxsdyypx*CDx
        fyyxpxdyydxx = gxxyysdyydxx + fyyxsdyydxx*ABx
        fyyxsdxypx = fyyxsfxxys + fyyxsdxys*CDx
        fyyxsdxydxy = fyyxsfyyxpx + fyyxsdxypx*CDy
        fyyxpxdxydxy = gxxyysdxydxy + fyyxsdxydxy*ABx
        fxxzsfxyzpx = fxxzsgxxyzs + fxxzsfxyzs*CDx
        fxxzsfzzxpx = fxxzsgxxzzs + fxxzsfzzxs*CDx
        fxxzsfxxzpz = fxxzsgxxzzs + fxxzsfxxzs*CDz
        fxxzsfxxzpy = fxxzsgxxyzs + fxxzsfxxzs*CDy
        fxxzsfxxzpx = fxxzsgxxxzs + fxxzsfxxzs*CDx
        fxxzsdxzpx = fxxzsfxxzs + fxxzsdxzs*CDx
        fxxzsdxzdxz = fxxzsfzzxpx + fxxzsdxzpx*CDz
        fxxzsdxzdxy = fxxzsfxyzpx + fxxzsdxzpx*CDy
        fxxzsdxzdxx = fxxzsfxxzpx + fxxzsdxzpx*CDx
        fxxzpxdxzdxx = gxxxzsdxzdxx + fxxzsdxzdxx*ABx
        fxxzsdxxpz = fxxzsfxxzs + fxxzsdxxs*CDz
        fxxzsdxxdzz = fxxzsfxxzpz + fxxzsdxxpz*CDz
        fxxzpzdxxdzz = gxxzzsdxxdzz + fxxzsdxxdzz*ABz
        fxxzsdxxpy = fxxzsfxxys + fxxzsdxxs*CDy
        fxxzsdxxdyz = fxxzsfxxzpy + fxxzsdxxpy*CDz
        fxxzpydxxdyz = gxxyzsdxxdyz + fxxzsdxxdyz*ABy
        fxxzsdxxpx = fxxzsfxxxs + fxxzsdxxs*CDx
        fxxzsdxxdxz = fxxzsfxxzpx + fxxzsdxxpx*CDz
        fxxzpxdxxdxz = gxxxzsdxxdxz + fxxzsdxxdxz*ABx
        fxxysfxyzpx = fxxysgxxyzs + fxxysfxyzs*CDx
        fxxysfyyxpx = fxxysgxxyys + fxxysfyyxs*CDx
        fxxysfxxzpy = fxxysgxxyzs + fxxysfxxzs*CDy
        fxxysfxxypy = fxxysgxxyys + fxxysfxxys*CDy
        fxxysfxxypx = fxxysgxxxys + fxxysfxxys*CDx
        fxxysdxypx = fxxysfxxys + fxxysdxys*CDx
        fxxysdxydxz = fxxysfxyzpx + fxxysdxypx*CDz
        fxxysdxydxy = fxxysfyyxpx + fxxysdxypx*CDy
        fxxysdxydxx = fxxysfxxypx + fxxysdxypx*CDx
        fxxypxdxydxx = gxxxysdxydxx + fxxysdxydxx*ABx
        fxxysdxxpy = fxxysfxxys + fxxysdxxs*CDy
        fxxysdxxdyz = fxxysfxxzpy + fxxysdxxpy*CDz
        fxxysdxxdyy = fxxysfxxypy + fxxysdxxpy*CDy
        fxxypydxxdyy = gxxyysdxxdyy + fxxysdxxdyy*ABy
        fxxysdxxpx = fxxysfxxxs + fxxysdxxs*CDx
        fxxysdxxdxy = fxxysfxxypx + fxxysdxxpx*CDy
        fxxypxdxxdxy = gxxxysdxxdxy + fxxysdxxdxy*ABx
        fzzzsfzzzpz = fzzzsgzzzzs + fzzzsfzzzs*CDz
        fzzzsfzzzpy = fzzzsgzzzys + fzzzsfzzzs*CDy
        fzzzsfzzzpx = fzzzsgzzzxs + fzzzsfzzzs*CDx
        fzzzsdzzpz = fzzzsfzzzs + fzzzsdzzs*CDz
        fzzzsdzzdzz = fzzzsfzzzpz + fzzzsdzzpz*CDz
        fzzzpzdzzdzz = gzzzzsdzzdzz + fzzzsdzzdzz*ABz
        fzzzsdzzpy = fzzzsfzzys + fzzzsdzzs*CDy
        fzzzsdzzdyz = fzzzsfzzzpy + fzzzsdzzpy*CDz
        fzzzpydzzdyz = gzzzysdzzdyz + fzzzsdzzdyz*ABy
        fzzzsdzzpx = fzzzsfzzxs + fzzzsdzzs*CDx
        fzzzsdzzdxz = fzzzsfzzzpx + fzzzsdzzpx*CDz
        fzzzpxdzzdxz = gzzzxsdzzdxz + fzzzsdzzdxz*ABx
        fyyysfyyzpy = fyyysgyyyzs + fyyysfyyzs*CDy
        fyyysfyyypy = fyyysgyyyys + fyyysfyyys*CDy
        fyyysfyyypx = fyyysgyyyxs + fyyysfyyys*CDx
        fyyysdyypy = fyyysfyyys + fyyysdyys*CDy
        fyyysdyydyz = fyyysfyyzpy + fyyysdyypy*CDz
        fyyysdyydyy = fyyysfyyypy + fyyysdyypy*CDy
        fyyypydyydyy = gyyyysdyydyy + fyyysdyydyy*ABy
        fyyysdyypx = fyyysfyyxs + fyyysdyys*CDx
        fyyysdyydxy = fyyysfyyypx + fyyysdyypx*CDy
        fyyypxdyydxy = gyyyxsdyydxy + fyyysdyydxy*ABx
        fxxxsfxxzpx = fxxxsgxxxzs + fxxxsfxxzs*CDx
        fxxxsfxxypx = fxxxsgxxxys + fxxxsfxxys*CDx
        fxxxsfxxxpx = fxxxsgxxxxs + fxxxsfxxxs*CDx
        fxxxsdxxpx = fxxxsfxxxs + fxxxsdxxs*CDx
        fxxxsdxxdxz = fxxxsfxxzpx + fxxxsdxxpx*CDz
        fxxxsdxxdxy = fxxxsfxxypx + fxxxsdxxpx*CDy
        fxxxsdxxdxx = fxxxsfxxxpx + fxxxsdxxpx*CDx
        fxxxpxdxxdxx = gxxxxsdxxdxx + fxxxsdxxdxx*ABx
        dyzsfxyzpx = dyzsgxxyzs + dyzsfxyzs*CDx
        dyzsdyzpx = dyzsfxyzs + dyzsdyzs*CDx
        dyzsdyzdxx = dyzsfxyzpx + dyzsdyzpx*CDx
        dyzpxdyzdxx = fxyzsdyzdxx + dyzsdyzdxx*ABx
        dyzdxxdyzdxx = fxyzpxdyzdxx + dyzpxdyzdxx*ABx
        dyzsfzzypz = dyzsgzzzys + dyzsfzzys*CDz
        dyzsfzzypy = dyzsgyyzzs + dyzsfzzys*CDy
        dyzsfzzypx = dyzsgzzxys + dyzsfzzys*CDx
        dyzsdyzdxz = dyzsfzzypx + dyzsdyzpx*CDz
        dyzpxdyzdxz = fxyzsdyzdxz + dyzsdyzdxz*ABx
        dyzdxzdyzdxz = fzzypxdyzdxz + dyzpxdyzdxz*ABz
        dyzsdyzpz = dyzsfzzys + dyzsdyzs*CDz
        dyzsdyzdzz = dyzsfzzypz + dyzsdyzpz*CDz
        dyzpzdyzdzz = fzzysdyzdzz + dyzsdyzdzz*ABz
        dyzdzzdyzdzz = fzzypzdyzdzz + dyzpzdyzdzz*ABz
        dyzsfyyzpy = dyzsgyyyzs + dyzsfyyzs*CDy
        dyzsfyyzpx = dyzsgyyxzs + dyzsfyyzs*CDx
        dyzsdyzdxy = dyzsfyyzpx + dyzsdyzpx*CDy
        dyzpxdyzdxy = fxyzsdyzdxy + dyzsdyzdxy*ABx
        dyzdxydyzdxy = fyyzpxdyzdxy + dyzpxdyzdxy*ABy
        dyzsdyzpy = dyzsfyyzs + dyzsdyzs*CDy
        dyzsdyzdyz = dyzsfzzypy + dyzsdyzpy*CDz
        dyzpydyzdyz = fyyzsdyzdyz + dyzsdyzdyz*ABy
        dyzdyzdyzdyz = fzzypydyzdyz + dyzpydyzdyz*ABz
        dyzsdyzdyy = dyzsfyyzpy + dyzsdyzpy*CDy
        dyzpydyzdyy = fyyzsdyzdyy + dyzsdyzdyy*ABy
        dyzdyydyzdyy = fyyzpydyzdyy + dyzpydyzdyy*ABy
        dxzsfxyzpy = dxzsgyyxzs + dxzsfxyzs*CDy
        dxzsfxyzpx = dxzsgxxyzs + dxzsfxyzs*CDx
        dxzsdxzpy = dxzsfxyzs + dxzsdxzs*CDy
        dxzsdxzdyy = dxzsfxyzpy + dxzsdxzpy*CDy
        dxzpydxzdyy = fxyzsdxzdyy + dxzsdxzdyy*ABy
        dxzdyydxzdyy = fxyzpydxzdyy + dxzpydxzdyy*ABy
        dxzsfzzxpz = dxzsgzzzxs + dxzsfzzxs*CDz
        dxzsfzzxpy = dxzsgzzxys + dxzsfzzxs*CDy
        dxzsdxzdyz = dxzsfzzxpy + dxzsdxzpy*CDz
        dxzpydxzdyz = fxyzsdxzdyz + dxzsdxzdyz*ABy
        dxzdyzdxzdyz = fzzxpydxzdyz + dxzpydxzdyz*ABz
        dxzsfzzxpx = dxzsgxxzzs + dxzsfzzxs*CDx
        dxzsdxzpz = dxzsfzzxs + dxzsdxzs*CDz
        dxzsdxzdzz = dxzsfzzxpz + dxzsdxzpz*CDz
        dxzpzdxzdzz = fzzxsdxzdzz + dxzsdxzdzz*ABz
        dxzdzzdxzdzz = fzzxpzdxzdzz + dxzpzdxzdzz*ABz
        dxzsfxxzpx = dxzsgxxxzs + dxzsfxxzs*CDx
        dxzsdxzpx = dxzsfxxzs + dxzsdxzs*CDx
        dxzsdxzdxz = dxzsfzzxpx + dxzsdxzpx*CDz
        dxzpxdxzdxz = fxxzsdxzdxz + dxzsdxzdxz*ABx
        dxzdxzdxzdxz = fzzxpxdxzdxz + dxzpxdxzdxz*ABz
        dxzsdxzdxy = dxzsfxyzpx + dxzsdxzpx*CDy
        dxzpxdxzdxy = fxxzsdxzdxy + dxzsdxzdxy*ABx
        dxzdxydxzdxy = fxyzpxdxzdxy + dxzpxdxzdxy*ABy
        dxzsdxzdxx = dxzsfxxzpx + dxzsdxzpx*CDx
        dxzpxdxzdxx = fxxzsdxzdxx + dxzsdxzdxx*ABx
        dxzdxxdxzdxx = fxxzpxdxzdxx + dxzpxdxzdxx*ABx
        dxysfxyzpz = dxysgzzxys + dxysfxyzs*CDz
        dxysfxyzpy = dxysgyyxzs + dxysfxyzs*CDy
        dxysfxyzpx = dxysgxxyzs + dxysfxyzs*CDx
        dxysdxypz = dxysfxyzs + dxysdxys*CDz
        dxysdxydzz = dxysfxyzpz + dxysdxypz*CDz
        dxypzdxydzz = fxyzsdxydzz + dxysdxydzz*ABz
        dxydzzdxydzz = fxyzpzdxydzz + dxypzdxydzz*ABz
        dxysfyyxpy = dxysgyyyxs + dxysfyyxs*CDy
        dxysfyyxpx = dxysgxxyys + dxysfyyxs*CDx
        dxysdxypy = dxysfyyxs + dxysdxys*CDy
        dxysdxydyz = dxysfxyzpy + dxysdxypy*CDz
        dxypydxydyz = fyyxsdxydyz + dxysdxydyz*ABy
        dxydyzdxydyz = fxyzpydxydyz + dxypydxydyz*ABz
        dxysdxydyy = dxysfyyxpy + dxysdxypy*CDy
        dxypydxydyy = fyyxsdxydyy + dxysdxydyy*ABy
        dxydyydxydyy = fyyxpydxydyy + dxypydxydyy*ABy
        dxysfxxypx = dxysgxxxys + dxysfxxys*CDx
        dxysdxypx = dxysfxxys + dxysdxys*CDx
        dxysdxydxz = dxysfxyzpx + dxysdxypx*CDz
        dxypxdxydxz = fxxysdxydxz + dxysdxydxz*ABx
        dxydxzdxydxz = fxyzpxdxydxz + dxypxdxydxz*ABz
        dxysdxydxy = dxysfyyxpx + dxysdxypx*CDy
        dxypxdxydxy = fxxysdxydxy + dxysdxydxy*ABx
        dxydxydxydxy = fyyxpxdxydxy + dxypxdxydxy*ABy
        dxysdxydxx = dxysfxxypx + dxysdxypx*CDx
        dxypxdxydxx = fxxysdxydxx + dxysdxydxx*ABx
        dxydxxdxydxx = fxxypxdxydxx + dxypxdxydxx*ABx
        dzzsfzzzpz = dzzsgzzzzs + dzzsfzzzs*CDz
        dzzsfzzzpy = dzzsgzzzys + dzzsfzzzs*CDy
        dzzsfzzzpx = dzzsgzzzxs + dzzsfzzzs*CDx
        dzzsfzzypy = dzzsgyyzzs + dzzsfzzys*CDy
        dzzsfzzypx = dzzsgzzxys + dzzsfzzys*CDx
        dzzsdzzpz = dzzsfzzzs + dzzsdzzs*CDz
        dzzsdzzdzz = dzzsfzzzpz + dzzsdzzpz*CDz
        dzzpzdzzdzz = fzzzsdzzdzz + dzzsdzzdzz*ABz
        dzzdzzdzzdzz = fzzzpzdzzdzz + dzzpzdzzdzz*ABz
        dzzsdzzpy = dzzsfzzys + dzzsdzzs*CDy
        dzzsdzzdyz = dzzsfzzzpy + dzzsdzzpy*CDz
        dzzpydzzdyz = fzzysdzzdyz + dzzsdzzdyz*ABy
        dzzdyzdzzdyz = fzzzpydzzdyz + dzzpydzzdyz*ABz
        dzzsdzzdyy = dzzsfzzypy + dzzsdzzpy*CDy
        dzzpydzzdyy = fzzysdzzdyy + dzzsdzzdyy*ABy
        dzzdyydzzdyy = fzzypydzzdyy + dzzpydzzdyy*ABy
        dzzsfzzxpx = dzzsgxxzzs + dzzsfzzxs*CDx
        dzzsdzzpx = dzzsfzzxs + dzzsdzzs*CDx
        dzzsdzzdxz = dzzsfzzzpx + dzzsdzzpx*CDz
        dzzpxdzzdxz = fzzxsdzzdxz + dzzsdzzdxz*ABx
        dzzdxzdzzdxz = fzzzpxdzzdxz + dzzpxdzzdxz*ABz
        dzzsdzzdxy = dzzsfzzypx + dzzsdzzpx*CDy
        dzzpxdzzdxy = fzzxsdzzdxy + dzzsdzzdxy*ABx
        dzzdxydzzdxy = fzzypxdzzdxy + dzzpxdzzdxy*ABy
        dzzsdzzdxx = dzzsfzzxpx + dzzsdzzpx*CDx
        dzzpxdzzdxx = fzzxsdzzdxx + dzzsdzzdxx*ABx
        dzzdxxdzzdxx = fzzxpxdzzdxx + dzzpxdzzdxx*ABx
        dyysfyyzpz = dyysgyyzzs + dyysfyyzs*CDz
        dyysfyyzpy = dyysgyyyzs + dyysfyyzs*CDy
        dyysfyyzpx = dyysgyyxzs + dyysfyyzs*CDx
        dyysfyyypy = dyysgyyyys + dyysfyyys*CDy
        dyysfyyypx = dyysgyyyxs + dyysfyyys*CDx
        dyysdyypz = dyysfyyzs + dyysdyys*CDz
        dyysdyydzz = dyysfyyzpz + dyysdyypz*CDz
        dyypzdyydzz = fyyzsdyydzz + dyysdyydzz*ABz
        dyydzzdyydzz = fyyzpzdyydzz + dyypzdyydzz*ABz
        dyysdyypy = dyysfyyys + dyysdyys*CDy
        dyysdyydyz = dyysfyyzpy + dyysdyypy*CDz
        dyypydyydyz = fyyysdyydyz + dyysdyydyz*ABy
        dyydyzdyydyz = fyyzpydyydyz + dyypydyydyz*ABz
        dyysdyydyy = dyysfyyypy + dyysdyypy*CDy
        dyypydyydyy = fyyysdyydyy + dyysdyydyy*ABy
        dyydyydyydyy = fyyypydyydyy + dyypydyydyy*ABy
        dyysfyyxpx = dyysgxxyys + dyysfyyxs*CDx
        dyysdyypx = dyysfyyxs + dyysdyys*CDx
        dyysdyydxz = dyysfyyzpx + dyysdyypx*CDz
        dyypxdyydxz = fyyxsdyydxz + dyysdyydxz*ABx
        dyydxzdyydxz = fyyzpxdyydxz + dyypxdyydxz*ABz
        dyysdyydxy = dyysfyyypx + dyysdyypx*CDy
        dyypxdyydxy = fyyxsdyydxy + dyysdyydxy*ABx
        dyydxydyydxy = fyyypxdyydxy + dyypxdyydxy*ABy
        dyysdyydxx = dyysfyyxpx + dyysdyypx*CDx
        dyypxdyydxx = fyyxsdyydxx + dyysdyydxx*ABx
        dyydxxdyydxx = fyyxpxdyydxx + dyypxdyydxx*ABx
        dxxsfxxzpz = dxxsgxxzzs + dxxsfxxzs*CDz
        dxxsfxxzpy = dxxsgxxyzs + dxxsfxxzs*CDy
        dxxsfxxzpx = dxxsgxxxzs + dxxsfxxzs*CDx
        dxxsfxxypy = dxxsgxxyys + dxxsfxxys*CDy
        dxxsfxxypx = dxxsgxxxys + dxxsfxxys*CDx
        dxxsdxxpz = dxxsfxxzs + dxxsdxxs*CDz
        dxxsdxxdzz = dxxsfxxzpz + dxxsdxxpz*CDz
        dxxpzdxxdzz = fxxzsdxxdzz + dxxsdxxdzz*ABz
        dxxdzzdxxdzz = fxxzpzdxxdzz + dxxpzdxxdzz*ABz
        dxxsdxxpy = dxxsfxxys + dxxsdxxs*CDy
        dxxsdxxdyz = dxxsfxxzpy + dxxsdxxpy*CDz
        dxxpydxxdyz = fxxysdxxdyz + dxxsdxxdyz*ABy
        dxxdyzdxxdyz = fxxzpydxxdyz + dxxpydxxdyz*ABz
        dxxsdxxdyy = dxxsfxxypy + dxxsdxxpy*CDy
        dxxpydxxdyy = fxxysdxxdyy + dxxsdxxdyy*ABy
        dxxdyydxxdyy = fxxypydxxdyy + dxxpydxxdyy*ABy
        dxxsfxxxpx = dxxsgxxxxs + dxxsfxxxs*CDx
        dxxsdxxpx = dxxsfxxxs + dxxsdxxs*CDx
        dxxsdxxdxz = dxxsfxxzpx + dxxsdxxpx*CDz
        dxxpxdxxdxz = fxxxsdxxdxz + dxxsdxxdxz*ABx
        dxxdxzdxxdxz = fxxzpxdxxdxz + dxxpxdxxdxz*ABz
        dxxsdxxdxy = dxxsfxxypx + dxxsdxxpx*CDy
        dxxpxdxxdxy = fxxxsdxxdxy + dxxsdxxdxy*ABx
        dxxdxydxxdxy = fxxypxdxxdxy + dxxpxdxxdxy*ABy
        dxxsdxxdxx = dxxsfxxxpx + dxxsdxxpx*CDx
        dxxpxdxxdxx = fxxxsdxxdxx + dxxsdxxdxx*ABx
        dxxdxxdxxdxx = fxxxpxdxxdxx + dxxpxdxxdxx*ABx
        ec(1) = dxxdxxdxxdxx
        ec(2) = dxxdyydxxdyy
        ec(3) = dxxdzzdxxdzz
        ec(4) = dxxdxydxxdxy*root3x2
        ec(5) = dxxdxzdxxdxz*root3x2
        ec(6) = dxxdyzdxxdyz*root3x2
        ec(7) = dyydxxdyydxx
        ec(8) = dyydyydyydyy
        ec(9) = dyydzzdyydzz
        ec(10) = dyydxydyydxy*root3x2
        ec(11) = dyydxzdyydxz*root3x2
        ec(12) = dyydyzdyydyz*root3x2
        ec(13) = dzzdxxdzzdxx
        ec(14) = dzzdyydzzdyy
        ec(15) = dzzdzzdzzdzz
        ec(16) = dzzdxydzzdxy*root3x2
        ec(17) = dzzdxzdzzdxz*root3x2
        ec(18) = dzzdyzdzzdyz*root3x2
        ec(19) = dxydxxdxydxx*root3x2
        ec(20) = dxydyydxydyy*root3x2
        ec(21) = dxydzzdxydzz*root3x2
        ec(22) = dxydxydxydxy*root3x4
        ec(23) = dxydxzdxydxz*root3x4
        ec(24) = dxydyzdxydyz*root3x4
        ec(25) = dxzdxxdxzdxx*root3x2
        ec(26) = dxzdyydxzdyy*root3x2
        ec(27) = dxzdzzdxzdzz*root3x2
        ec(28) = dxzdxydxzdxy*root3x4
        ec(29) = dxzdxzdxzdxz*root3x4
        ec(30) = dxzdyzdxzdyz*root3x4
        ec(31) = dyzdxxdyzdxx*root3x2
        ec(32) = dyzdyydyzdyy*root3x2
        ec(33) = dyzdzzdyzdzz*root3x2
        ec(34) = dyzdxydyzdxy*root3x4
        ec(35) = dyzdxzdyzdxz*root3x4
        ec(36) = dyzdyzdyzdyz*root3x4
        rsrs(ij) = maxval(dabs(ec(:)))
  end do
!$omp end do
end subroutine os_csi_dddd_os3
